@@ -12,6 +12,7 @@
 #include "../universe/ShipDesign.h"
 #include "../universe/System.h"
 #include "../universe/Tech.h"
+#include "../universe/Enums.h"
 #include "../Empire/Empire.h"
 
 #include "../util/Directories.h"
@@ -21,11 +22,16 @@
 #include "../util/OrderSet.h"
 
 #include <boost/timer.hpp>
-#include <boost/lexical_cast.hpp>
+#include <boost/python/str.hpp>
 
+
+#include <map>
 #include <stdexcept>
 #include <string>
-#include <map>
+
+
+using boost::python::object;
+using boost::python::str;
 
 //////////////////////////////////
 //          AI Base             //
@@ -74,7 +80,7 @@ namespace AIInterface {
         if (it != players.end())
             return it->second.name;
         else {
-            DebugLogger() << "AIInterface::PlayerName(" << boost::lexical_cast<std::string>(player_id) << ") - passed an invalid player_id";
+            DebugLogger() << "AIInterface::PlayerName(" << std::to_string(player_id) << ") - passed an invalid player_id";
             throw std::invalid_argument("AIInterface::PlayerName : given invalid player_id");
         }
     }
@@ -86,19 +92,17 @@ namespace AIInterface {
     { return AIClientApp::GetApp()->EmpireID(); }
 
     int PlayerEmpireID(int player_id) {
-        const std::map<int, PlayerInfo>& players = AIClientApp::GetApp()->Players();
-        for (std::map<int, PlayerInfo>::const_iterator it = players.begin(); it != players.end(); ++it) {
-            if (it->first == player_id)
-                return it->second.empire_id;
+        for (std::map<int, PlayerInfo>::value_type& entry : AIClientApp::GetApp()->Players()) {
+            if (entry.first == player_id)
+                return entry.second.empire_id;
         }
         return ALL_EMPIRES; // default invalid value
     }
 
     std::vector<int>  AllEmpireIDs() {
-        const std::map<int, PlayerInfo>& players = AIClientApp::GetApp()->Players();
         std::vector<int> empire_ids;
-        for (std::map<int, PlayerInfo>::const_iterator it = players.begin(); it != players.end(); ++it)
-            empire_ids.push_back(it->second.empire_id);
+        for (std::map<int, PlayerInfo>::value_type& entry : AIClientApp::GetApp()->Players())
+            empire_ids.push_back(entry.second.empire_id);
         return empire_ids;
     }
 
@@ -111,15 +115,14 @@ namespace AIInterface {
     int EmpirePlayerID(int empire_id) {
         int player_id = AIClientApp::GetApp()->EmpirePlayerID(empire_id);
         if (-1 == player_id)
-            DebugLogger() << "AIInterface::EmpirePlayerID(" << boost::lexical_cast<std::string>(empire_id) << ") - passed an invalid empire_id";
+            DebugLogger() << "AIInterface::EmpirePlayerID(" << std::to_string(empire_id) << ") - passed an invalid empire_id";
         return player_id;
     }
 
     std::vector<int> AllPlayerIDs() {
-        const std::map<int, PlayerInfo>& players = AIClientApp::GetApp()->Players();
         std::vector<int> player_ids;
-        for (std::map<int, PlayerInfo>::const_iterator it = players.begin(); it != players.end(); ++it)
-            player_ids.push_back(it->first);
+        for (std::map<int, PlayerInfo>::value_type& entry : AIClientApp::GetApp()->Players())
+            player_ids.push_back(entry.first);
         return player_ids;
     }
 
@@ -143,8 +146,17 @@ namespace AIInterface {
     int CurrentTurn()
     { return AIClientApp::GetApp()->CurrentTurn(); }
 
-    std::string GetAIConfigStr()
-    { return GetOptionsDB().Get<std::string>("ai-config"); }
+    object GetOptionsDBOptionStr(std::string const &option)
+    { return GetOptionsDB().OptionExists(option) ? str(GetOptionsDB().Get<std::string>(option)) : str(); }
+
+    object GetOptionsDBOptionInt(std::string const &option)
+    { return GetOptionsDB().OptionExists(option) ? object(GetOptionsDB().Get<int>(option)) : object(); }
+
+    object GetOptionsDBOptionBool(std::string const &option)
+    { return GetOptionsDB().OptionExists(option) ? object(GetOptionsDB().Get<bool>(option)) : object(); }
+
+    object GetOptionsDBOptionDouble(std::string const &option)
+    { return GetOptionsDB().OptionExists(option) ? object(GetOptionsDB().Get<double>(option)) : object(); }
 
     std::string GetAIDir()
     { return (GetResourceDir() / GetOptionsDB().Get<std::string>("ai-path")).string(); }
@@ -168,7 +180,7 @@ namespace AIInterface {
     }
 
     void UpdateMeterEstimates(bool pretend_to_own_unowned_planets) {
-        std::vector<TemporaryPtr<Planet> > unowned_planets;
+        std::vector<std::shared_ptr<Planet>> unowned_planets;
         int player_id = -1;
         Universe& universe = AIClientApp::GetApp()->GetUniverse();
         if (pretend_to_own_unowned_planets) {
@@ -187,10 +199,8 @@ namespace AIInterface {
             // (aren't owned by anyone).  Add this the current player's
             // ownership to all, while remembering which planets this is done
             // to.
-            std::vector<TemporaryPtr<Planet> > all_planets = universe.Objects().FindObjects<Planet>();
             universe.InhibitUniverseObjectSignals(true);
-            for (std::vector<TemporaryPtr<Planet> >::iterator it = all_planets.begin(); it != all_planets.end(); ++it) {
-                 TemporaryPtr<Planet> planet = *it;
+            for (std::shared_ptr<Planet> planet : universe.Objects().FindObjects<Planet>()) {
                  if (planet->Unowned()) {
                      unowned_planets.push_back(planet);
                      planet->SetOwner(player_id);
@@ -203,16 +213,15 @@ namespace AIInterface {
 
         if (pretend_to_own_unowned_planets) {
             // remove temporary ownership added above
-            for (std::vector<TemporaryPtr<Planet> >::iterator it = unowned_planets.begin(); it != unowned_planets.end(); ++it)
-                (*it)->SetOwner(ALL_EMPIRES);
+            for (std::shared_ptr<Planet> planet : unowned_planets)
+                planet->SetOwner(ALL_EMPIRES);
             universe.InhibitUniverseObjectSignals(false);
         }
     }
 
     void UpdateResourcePools() {
-        EmpireManager& manager = AIClientApp::GetApp()->Empires();
-        for (EmpireManager::iterator it = manager.begin(); it != manager.end(); ++it)
-            it->second->UpdateResourcePools();
+        for (std::map<int, Empire*>::value_type& entry : AIClientApp::GetApp()->Empires())
+            entry.second->UpdateResourcePools();
     }
 
     void UpdateResearchQueue() {
@@ -236,7 +245,7 @@ namespace AIInterface {
     }
 
     int IssueFleetMoveOrder(int fleet_id, int destination_id) {
-        TemporaryPtr<const Fleet> fleet = GetFleet(fleet_id);
+        std::shared_ptr<const Fleet> fleet = GetFleet(fleet_id);
         if (!fleet) {
             ErrorLogger() << "IssueFleetMoveOrder : passed an invalid fleet_id";
             return 0;
@@ -267,7 +276,7 @@ namespace AIInterface {
         }
 
         int empire_id = AIClientApp::GetApp()->EmpireID();
-        TemporaryPtr<const UniverseObject> obj = GetUniverseObject(object_id);
+        std::shared_ptr<const UniverseObject> obj = GetUniverseObject(object_id);
 
         if (!obj) {
             ErrorLogger() << "IssueRenameOrder : passed an invalid object_id";
@@ -292,8 +301,8 @@ namespace AIInterface {
             int empire_id = AIClientApp::GetApp()->EmpireID();
 
             // make sure all objects exist and are owned just by this player
-            for (std::vector<int>::const_iterator it = object_ids.begin(); it != object_ids.end(); ++it) {
-                TemporaryPtr<const UniverseObject> obj = GetUniverseObject(*it);
+            for (int object_id : object_ids) {
+                std::shared_ptr<const UniverseObject> obj = GetUniverseObject(object_id);
 
                 if (!obj) {
                     ErrorLogger() << "IssueScrapOrder : passed an invalid object_id";
@@ -305,7 +314,7 @@ namespace AIInterface {
                     return 0;
                 }
 
-                AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(new ScrapOrder(empire_id, *it)));
+                AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(new ScrapOrder(empire_id, object_id)));
             }
 
             return 1;
@@ -329,11 +338,11 @@ namespace AIInterface {
         }
 
         int empire_id = AIClientApp::GetApp()->EmpireID();
-        TemporaryPtr<const Ship> ship = TemporaryPtr<Ship>();
+        std::shared_ptr<const Ship> ship;
 
         // make sure all ships exist and are owned just by this player
-        for (std::vector<int>::const_iterator it = ship_ids.begin(); it != ship_ids.end(); ++it) {
-            ship = GetShip(*it);
+        for (int ship_id : ship_ids) {
+            ship = GetShip(ship_id);
             if (!ship) {
                 ErrorLogger() << "IssueNewFleetOrder : passed an invalid ship_id";
                 return 0;
@@ -353,7 +362,7 @@ namespace AIInterface {
 
         std::vector<int>::const_iterator it = ship_ids.begin();
         for (++it; it != ship_ids.end(); ++it) {
-            TemporaryPtr<const Ship> ship2 = GetShip(*it);
+            std::shared_ptr<const Ship> ship2 = GetShip(*it);
             if (ship2->SystemID() != system_id) {
                 ErrorLogger() << "IssueNewFleetOrder : passed ship_ids of ships at different locations";
                 return 0;
@@ -376,7 +385,7 @@ namespace AIInterface {
     int IssueFleetTransferOrder(int ship_id, int new_fleet_id) {
         int empire_id = AIClientApp::GetApp()->EmpireID();
 
-        TemporaryPtr<const Ship> ship = GetShip(ship_id);
+        std::shared_ptr<const Ship> ship = GetShip(ship_id);
         if (!ship) {
             ErrorLogger() << "IssueFleetTransferOrder : passed an invalid ship_id " << ship_id;
             return 0;
@@ -391,7 +400,7 @@ namespace AIInterface {
             return 0;
         }
 
-        TemporaryPtr<const Fleet> fleet = GetFleet(new_fleet_id);
+        std::shared_ptr<const Fleet> fleet = GetFleet(new_fleet_id);
         if (!fleet) {
             ErrorLogger() << "IssueFleetTransferOrder : passed an invalid new_fleet_id " << new_fleet_id;
             return 0;
@@ -422,14 +431,14 @@ namespace AIInterface {
         int empire_id = AIClientApp::GetApp()->EmpireID();
 
         // make sure ship_id is a ship...
-        TemporaryPtr<const Ship> ship = GetShip(ship_id);
+        std::shared_ptr<const Ship> ship = GetShip(ship_id);
         if (!ship) {
             ErrorLogger() << "IssueColonizeOrder : passed an invalid ship_id";
             return 0;
         }
 
         // get fleet of ship
-        TemporaryPtr<const Fleet> fleet = GetFleet(ship->FleetID());
+        std::shared_ptr<const Fleet> fleet = GetFleet(ship->FleetID());
         if (!fleet) {
             ErrorLogger() << "IssueColonizeOrder : ship with passed ship_id has invalid fleet_id";
             return 0;
@@ -446,7 +455,7 @@ namespace AIInterface {
         }
 
         // verify that planet exists and is un-occupied.
-        TemporaryPtr<const Planet> planet = GetPlanet(planet_id);
+        std::shared_ptr<const Planet> planet = GetPlanet(planet_id);
         if (!planet) {
             ErrorLogger() << "IssueColonizeOrder : no planet with passed planet_id";
             return 0;
@@ -475,14 +484,14 @@ namespace AIInterface {
         int empire_id = AIClientApp::GetApp()->EmpireID();
 
         // make sure ship_id is a ship...
-        TemporaryPtr<const Ship> ship = GetShip(ship_id);
+        std::shared_ptr<const Ship> ship = GetShip(ship_id);
         if (!ship) {
             ErrorLogger() << "IssueInvadeOrder : passed an invalid ship_id";
             return 0;
         }
 
         // get fleet of ship
-        TemporaryPtr<const Fleet> fleet = GetFleet(ship->FleetID());
+        std::shared_ptr<const Fleet> fleet = GetFleet(ship->FleetID());
         if (!fleet) {
             ErrorLogger() << "IssueInvadeOrder : ship with passed ship_id has invalid fleet_id";
             return 0;
@@ -499,7 +508,7 @@ namespace AIInterface {
         }
 
         // verify that planet exists and is occupied by another empire
-        TemporaryPtr<const Planet> planet = GetPlanet(planet_id);
+        std::shared_ptr<const Planet> planet = GetPlanet(planet_id);
         if (!planet) {
             ErrorLogger() << "IssueInvadeOrder : no planet with passed planet_id";
             return 0;
@@ -545,7 +554,7 @@ namespace AIInterface {
         int empire_id = AIClientApp::GetApp()->EmpireID();
 
         // make sure ship_id is a ship...
-        TemporaryPtr<const Ship> ship = GetShip(ship_id);
+        std::shared_ptr<const Ship> ship = GetShip(ship_id);
         if (!ship) {
             ErrorLogger() << "IssueBombardOrder : passed an invalid ship_id";
             return 0;
@@ -561,7 +570,7 @@ namespace AIInterface {
 
 
         // verify that planet exists and is occupied by another empire
-        TemporaryPtr<const Planet> planet = GetPlanet(planet_id);
+        std::shared_ptr<const Planet> planet = GetPlanet(planet_id);
         if (!planet) {
             ErrorLogger() << "IssueBombardOrder : no planet with passed planet_id";
             return 0;
@@ -603,7 +612,7 @@ namespace AIInterface {
     int IssueAggressionOrder(int object_id, bool aggressive) {
         int empire_id = AIClientApp::GetApp()->EmpireID();
 
-        TemporaryPtr<const Fleet> fleet = GetFleet(object_id);
+        std::shared_ptr<const Fleet> fleet = GetFleet(object_id);
         if (!fleet) {
             ErrorLogger() << "IssueAggressionOrder : no fleet with passed id";
             return 0;
@@ -632,7 +641,7 @@ namespace AIInterface {
             return 0;
         }
 
-        TemporaryPtr<UniverseObject> obj = GetUniverseObject(object_id);
+        std::shared_ptr<UniverseObject> obj = GetUniverseObject(object_id);
         if (!obj) {
             ErrorLogger() << "IssueGiveObjectToEmpireOrder : passed invalid object id";
             return 0;
@@ -648,7 +657,7 @@ namespace AIInterface {
             return 0;
         }
 
-        TemporaryPtr<System> system = GetSystem(obj->SystemID());
+        std::shared_ptr<System> system = GetSystem(obj->SystemID());
         if (!system) {
             ErrorLogger() << "IssueGiveObjectToEmpireOrder : couldn't get system of object";
             return 0;
@@ -656,12 +665,9 @@ namespace AIInterface {
 
         // can only give to empires with something present to receive the gift
         bool recipient_has_something_here = false;
-        std::vector<TemporaryPtr<const UniverseObject> > system_objects =
+        std::vector<std::shared_ptr<const UniverseObject>> system_objects =
             Objects().FindObjects<const UniverseObject>(system->ObjectIDs());
-        for (std::vector<TemporaryPtr<const UniverseObject> >::const_iterator it = system_objects.begin();
-                it != system_objects.end(); ++it)
-        {
-            TemporaryPtr<const UniverseObject> obj = *it;
+        for (std::shared_ptr<const UniverseObject> obj : system_objects) {
             if (obj->Owner() == recipient_id) {
                 recipient_has_something_here = true;
                 break;
@@ -681,7 +687,7 @@ namespace AIInterface {
     int IssueChangeFocusOrder(int planet_id, const std::string& focus) {
         int empire_id = AIClientApp::GetApp()->EmpireID();
 
-        TemporaryPtr<const Planet> planet = GetPlanet(planet_id);
+        std::shared_ptr<const Planet> planet = GetPlanet(planet_id);
         if (!planet) {
             ErrorLogger() << "IssueChangeFocusOrder : no planet with passed planet_id "<<planet_id;
             return 0;
