@@ -4,6 +4,19 @@
 #include <string>
 #include <vector>
 
+#ifdef FREEORION_MACOSX
+// Bugfix for https://github.com/freeorion/freeorion/issues/1228
+
+// The problem on OSX is that the boost/python/str.hpp redefines toupper() and
+// similar functions if they are not already defined.
+
+// This includes iostream before the boost/python/str.hpp to fix this issue.
+// If the subsequent #include <boost/python/str.hpp> is removed then so can this workaround.
+#include <iostream>
+#endif
+
+#include <boost/python/str.hpp>
+
 struct CombatData;
 class Empire;
 class Tech;
@@ -91,14 +104,14 @@ public:
      * level.
      *
      * @param aggr The new aggression level.  The value should be one of
-     *      ::Aggression.
+     *      Aggression.
      */
     void SetAggression(int aggr);
 
 protected:
     /** @brief The current aggressiveness of this AI
      *
-     * The value should be one of ::Aggression.
+     * The value should be one of Aggression.
      */
     int m_aggression;
 };
@@ -210,7 +223,7 @@ namespace AIInterface {
      *
      * @param name The identifying name of the requested ::Tech.
      *
-     * @return A pointer to the ::Tech matching @a name or NULL if no ::Tech
+     * @return A pointer to the ::Tech matching @a name or nullptr if no ::Tech
      *      with that name was found.
      */
     const Tech* GetTech(const std::string& name);
@@ -221,12 +234,33 @@ namespace AIInterface {
      */
     int CurrentTurn();
 
-    /** @brief Return the value of the ::OptionsDB `ai-config` key
+    /** @brief Return the OptionsDB option @a option
      *
-     * @return An UTF-8 encoded and NUL terminated string containing the path
-     *      to an optional ai configuration file or an empty string if not set.
+     * @return Return the OptionsDB option @a option or None if not set.
+     * @ throw boost::bad_any_cast if option exists but is not a string
      */
-    std::string GetAIConfigStr();
+    boost::python::object GetOptionsDBOptionStr(std::string const &option);
+
+    /** @brief Return the OptionsDB option @a option
+     *
+     * @return Return the OptionsDB option @a option or None if not set.
+     * @ throw boost::bad_any_cast if option exists but is not an int
+     */
+    boost::python::object GetOptionsDBOptionInt(std::string const &option);
+
+    /** @brief Return the OptionsDB option @a option
+     *
+     * @return Return the OptionsDB option @a option or None if not set.
+     * @ throw boost::bad_any_cast if option exists but is not bool
+     */
+    boost::python::object GetOptionsDBOptionBool(std::string const &option);
+
+    /** @brief Return the OptionsDB option @a option
+     *
+     * @return Return the OptionsDB option @a option or None if not set.
+     * @ throw boost::bad_any_cast if option exists but is not a double
+     */
+    boost::python::object GetOptionsDBOptionDouble(std::string const &option);
 
     /** @brief Return the canonical AI directory path
      *
