@@ -5,6 +5,7 @@
 #include "../util/Export.h"
 #include "../universe/Meter.h"
 #include "../universe/ShipDesign.h"
+#include "../universe/UniverseObject.h"
 
 #include <GG/Clr.h>
 
@@ -14,8 +15,9 @@
 struct ItemSpec;
 class ShipDesign;
 class SitRepEntry;
-extern const int INVALID_GAME_TURN;
-extern const int INVALID_OBJECT_ID;
+FO_COMMON_API extern const int INVALID_GAME_TURN;
+FO_COMMON_API extern const int INVALID_OBJECT_ID;
+FO_COMMON_API extern const int ALL_EMPIRES;
 
 class Alignment {
 public:
@@ -44,19 +46,22 @@ struct FO_COMMON_API ResearchQueue {
         Element() :
             name(),
             empire_id(ALL_EMPIRES),
-            allocated_rp(0.0),
-            turns_left(0)
+            allocated_rp(0.0f),
+            turns_left(0),
+            paused(false)
         {}
-        Element(const std::string& name_, int empire_id_, float spending_, int turns_left_) :
+        Element(const std::string& name_, int empire_id_, float spending_, int turns_left_, bool paused_ = false) :
             name(name_),
             empire_id(empire_id_),
             allocated_rp(spending_),
-            turns_left(turns_left_)
+            turns_left(turns_left_),
+            paused(paused_)
         {}
         std::string name;
         int         empire_id;
         float       allocated_rp;
         int         turns_left;
+        bool        paused;
     private:
         friend class boost::serialization::access;
         template <class Archive>
@@ -80,6 +85,8 @@ struct FO_COMMON_API ResearchQueue {
 
     /** \name Accessors */ //@{
     bool                        InQueue(const std::string& tech_name) const;///< Returns true iff \a tech is in this queue.
+    bool                        Paused(const std::string& tech_name) const; ///< Returns true iff \a tech is in this queue and paused.
+    bool                        Paused(int idx) const;                      ///< Returns true iff there are at least \a idx - 1 items in the queue and item with index \a idx is paused
     int                         ProjectsInProgress() const;                 ///< Returns the number of research projects currently (perhaps partially) funded.
     float                       TotalRPsSpent() const;                      ///< Returns the number of RPs currently spent on the projects in this queue.
     int                         EmpireID() const { return m_empire_id; }
@@ -108,8 +115,8 @@ struct FO_COMMON_API ResearchQueue {
     void            Update(float RPs, const std::map<std::string, float>& research_progress);
 
     // STL container-like interface
-    void            push_back(const std::string& tech_name);
-    void            insert(iterator it, const std::string& tech_name);
+    void            push_back(const std::string& tech_name, bool paused = false);
+    void            insert(iterator it, const std::string& tech_name, bool paused = false);
     void            erase(iterator it);
 
     iterator        begin();
@@ -138,8 +145,9 @@ private:
 
 struct FO_COMMON_API ProductionQueue {
     /** The type that specifies a single production item (BuildType and name string). */
-    struct ProductionItem {
-        ProductionItem(); ///< default ctor.
+    struct FO_COMMON_API ProductionItem {
+        ProductionItem();
+
         ProductionItem(BuildType build_type_, std::string name_);   ///< basic ctor for BuildTypes that use std::string to identify specific items (BuildingTypes)
         ProductionItem(BuildType build_type_, int design_id_);      ///< basic ctor for BuildTypes that use int to indentify the design of the item (ShipDesigns)
 
@@ -147,8 +155,10 @@ struct FO_COMMON_API ProductionQueue {
 
         bool operator<(const ProductionItem& rhs) const;
 
-        std::map<std::string, std::map<int, float> >    CompletionSpecialConsumption(int location_id) const;// for each special name, what object ids have those special capacities reduced by what amount
-        std::map<MeterType, std::map<int, float> >      CompletionMeterConsumption(int location_id) const;  // for each meter type, what object ids have those meters reduced by what amount
+        std::map<std::string, std::map<int, float>> CompletionSpecialConsumption(int location_id) const;// for each special name, what object ids have those special capacities reduced by what amount
+        std::map<MeterType, std::map<int, float>>   CompletionMeterConsumption(int location_id) const;  // for each meter type, what object ids have those meters reduced by what amount
+
+        std::string Dump() const;
 
         BuildType   build_type;
         // only one of these may be valid, depending on BuildType
@@ -162,11 +172,14 @@ struct FO_COMMON_API ProductionQueue {
     };
 
     /** The type of a single element in the production queue. */
-    struct Element {
-        Element(); ///< default ctor.
-        Element(ProductionItem item_, int empire_id_, int ordered_, int remaining_, int location_);
-        Element(BuildType build_type, std::string name, int empire_id_, int ordered_, int remaining_, int location_);
-        Element(BuildType build_type, int design_id, int empire_id_, int ordered_, int remaining_, int location_);
+    struct FO_COMMON_API Element {
+        Element();
+
+        Element(ProductionItem item_, int empire_id_, int ordered_, int remaining_, int location_, bool paused_ = false);
+
+        Element(BuildType build_type, std::string name, int empire_id_, int ordered_, int remaining_, int location_, bool paused_ = false);
+
+        Element(BuildType build_type, int design_id, int empire_id_, int ordered_, int remaining_, int location_, bool paused_ = false);
 
         ProductionItem  item;
         int             empire_id;
@@ -175,12 +188,15 @@ struct FO_COMMON_API ProductionQueue {
         int             remaining;                  ///< how many left to produce
         int             location;                   ///< the ID of the UniverseObject at which this item is being produced
         float           allocated_pp;               ///< PP allocated to this ProductionQueue Element by Empire production update
-        float           progress;                   ///< PP that has been spent on this production element (will increase by allocation during next turn processing)
+        float           progress;                   ///< fraction of this item that is complete.
         float           progress_memory;            ///< updated by server turn processing; aides in allowing blocksize changes to be undone in same turn w/o progress loss
         int             blocksize_memory;           ///< used along with progress_memory
         int             turns_left_to_next_item;
         int             turns_left_to_completion;
         int             rally_point_id;
+        bool            paused;
+
+        std::string Dump() const;
 
     private:
         friend class boost::serialization::access;
@@ -196,7 +212,7 @@ struct FO_COMMON_API ProductionQueue {
     typedef QueueType::const_iterator const_iterator;
 
     /** \name Structors */ //@{
-    ProductionQueue(int empire_id); ///< Basic ctor.
+    ProductionQueue(int empire_id);
     //@}
 
     /** \name Accessors */ //@{
@@ -206,7 +222,7 @@ struct FO_COMMON_API ProductionQueue {
 
     /** Returns map from sets of object ids that can share resources to amount
       * of PP available in those groups of objects */
-    std::map<std::set<int>, float>         AvailablePP(const boost::shared_ptr<ResourcePool>& industry_pool) const;
+    std::map<std::set<int>, float> AvailablePP(const std::shared_ptr<ResourcePool>& industry_pool) const;
 
     /** Returns map from sets of object ids that can share resources to amount
       * of PP allocated to production queue elements that have build locations
@@ -214,7 +230,7 @@ struct FO_COMMON_API ProductionQueue {
     const std::map<std::set<int>, float>&  AllocatedPP() const;
 
     /** Returns sets of object ids that have more available than allocated PP */
-    std::set<std::set<int> >                ObjectsWithWastedPP(const boost::shared_ptr<ResourcePool>& industry_pool) const;
+    std::set<std::set<int>> ObjectsWithWastedPP(const std::shared_ptr<ResourcePool>& industry_pool) const;
 
 
     // STL container-like interface
@@ -278,10 +294,7 @@ public:
     friend class EmpireManager;
 
     /** \name Iterator Types */ //@{
-    typedef std::set<std::string>::const_iterator       TechItr;
-    typedef std::set<std::string>::const_iterator       BuildingTypeItr;
     typedef std::set<int>::const_iterator               SystemIDItr;
-    typedef std::list<int>::const_iterator              ShipDesignItr;
     typedef std::vector<SitRepEntry>::const_iterator    SitRepItr;
     //@}
 
@@ -291,24 +304,39 @@ public:
     //@}
 
     /** \name Accessors */ //@{
-    const std::string&      Name() const;                                           ///< Returns the Empire's name
-    const std::string&      PlayerName() const;                                     ///< Returns the Empire's player's name
+    const std::string&      Name() const;                               ///< Returns the Empire's name
+    const std::string&      PlayerName() const;                         ///< Returns the Empire's player's name
 
-    int                     EmpireID() const;                                       ///< Returns the Empire's unique numeric ID
+    int                     EmpireID() const;                           ///< Returns the Empire's unique numeric ID
 
-    const GG::Clr&          Color() const;                                          ///< Returns the Empire's color
+    const GG::Clr&          Color() const;                              ///< Returns the Empire's color
 
-    int                     CapitalID() const;                                      ///< Returns the numeric ID of the empire's capital
-    int                     StockpileID(ResourceType res = INVALID_RESOURCE_TYPE) const;///< Returns the numeric ID of the empire's stockpile location for the resource of type \a res
+    int                     CapitalID() const;                          ///< Returns the numeric ID of the empire's capital
+    int                     StockpileID(ResourceType res) const;        ///< Returns the numeric ID of the empire's stockpile location for the resource of type \a res
+
+    /** Return an object id that is owned by the empire or INVALID_OBJECT_ID. */
+    int                     SourceID() const;
+    /** Return an object that is owned by the empire or null.*/
+    std::shared_ptr<const UniverseObject> Source() const;
 
     std::string             Dump() const;
 
-    const std::set<std::string>&    AvailableTechs() const;                         ///< Returns the set of all available techs.
-    const std::set<std::string>&    AvailableBuildingTypes() const;                 ///< Returns the set of all available building types.
-    std::set<int>                   AvailableShipDesigns() const;                   ///< Returns the set of ship design ids of this empire that the empire can actually build
-    const std::set<int>&            ShipDesigns() const;                            ///< Returns the set of all ship design ids of this empire
-    const std::set<std::string>&    AvailableShipParts() const;                     ///< Returns the set of ship part names this empire that the empire can currently build
-    const std::set<std::string>&    AvailableShipHulls() const;                     ///< Returns the set of ship hull names that that the empire can currently build
+    /** Returns the set of Tech names available to this empire. */
+    const std::set<std::string>& AvailableTechs() const;
+
+    /** Returns the set of BuildingType names availble to this empire. */
+    const std::set<std::string>& AvailableBuildingTypes() const;
+
+    /** Returns the set of ShipDesign IDs available for this empire to build. */
+    std::set<int> AvailableShipDesigns() const;
+
+    /** Returns an ordered vector of ShipDesign IDs available for this empire to
+        build. */
+    const std::vector<int>& OrderedShipDesigns() const;
+
+    const std::set<int>&            ShipDesigns() const;                ///< Returns the set of all ship design ids of this empire
+    const std::set<std::string>&    AvailableShipParts() const;         ///< Returns the set of ship part names this empire that the empire can currently build
+    const std::set<std::string>&    AvailableShipHulls() const;         ///< Returns the set of ship hull names that that the empire can currently build
 
     const std::string&              TopPriorityEnqueuedTech() const;
     const std::string&              MostExpensiveEnqueuedTech() const;
@@ -378,21 +406,18 @@ public:
     const bool                              UnrestrictedLaneTravel(int start_system_id, int dest_system_id) const;
 
     const std::set<int>&                    ExploredSystems() const;    ///< returns set of ids of systems that this empire has explored
-    const std::map<int, std::set<int> >     KnownStarlanes() const;     ///< returns map from system id (start) to set of system ids (endpoints) of all starlanes known to this empire
-    const std::map<int, std::set<int> >     VisibleStarlanes() const;   ///< returns map from system id (start) to set of system ids (endpoints) of all starlanes visible to this empire this turn
+    const std::map<int, std::set<int>>      KnownStarlanes() const;     ///< returns map from system id (start) to set of system ids (endpoints) of all starlanes known to this empire
+    const std::map<int, std::set<int>>      VisibleStarlanes() const;   ///< returns map from system id (start) to set of system ids (endpoints) of all starlanes visible to this empire this turn
 
-    TechItr                 TechBegin() const;                  ///< starting iterator for techs this empire has researched
-    TechItr                 TechEnd() const;                    ///< end iterator for techs
-    BuildingTypeItr         AvailableBuildingTypeBegin() const; ///< starting iterator for building types this empire can produce
-    BuildingTypeItr         AvailableBuildingTypeEnd() const;   ///< end iterator for building types
-    ShipDesignItr           ShipDesignBegin() const;            ///< starting iterator for ship designs this empire has on file.  individual designs may or may not be producible for this empire
-    ShipDesignItr           ShipDesignEnd() const;              ///< end iterator for ship designs
     SitRepItr               SitRepBegin() const;                ///< starting iterator for sitrep entries for this empire
     SitRepItr               SitRepEnd() const;                  ///< end iterator for sitreps
 
     float                   ProductionPoints() const;           ///< Returns the number of production points available to the empire (this is available industry)
 
-    const boost::shared_ptr<ResourcePool>   GetResourcePool(ResourceType resource_type) const;  ///< Returns ResourcePool for \a resource_type or 0 if no such ResourcePool exists
+    /** Returns ResourcePool for \a resource_type or 0 if no such ResourcePool
+        exists. */
+    const std::shared_ptr<ResourcePool> GetResourcePool(ResourceType resource_type) const;
+
     float                   ResourceStockpile(ResourceType type) const;         ///< returns current stockpiled amount of resource \a type
     float                   ResourceOutput(ResourceType type) const;            ///< returns amount of resource \a type being generated by ResourceCenters
     float                   ResourceAvailable(ResourceType type) const;         ///< returns amount of resource \a type immediately available.  This = production + stockpile
@@ -420,25 +445,31 @@ public:
     /** Removes tech with \a name from the research queue, if it is in the
       * research queue already. */
     void        RemoveTechFromQueue(const std::string& name);
+
+    void        PauseResearch(const std::string& name);
+    void        ResumeResearch(const std::string& name);
+
     /** Sets research progress of tech with \a name to \a progress. */
     void        SetTechResearchProgress(const std::string& name, float progress);
     /** Adds the indicated build to the production queue, placing it before
       * position \a pos.  If \a pos < 0 or queue.size() <= pos, the build is
       * placed at the end of the queue. */
-    void        PlaceBuildInQueue(BuildType build_type, const std::string& name, int number, int location, int pos = -1);
+    void        PlaceProductionOnQueue(BuildType build_type, const std::string& name, int number, int location, int pos = -1);
     /** Adds the indicated build to the production queue, placing it before
       * position \a pos.  If \a pos < 0 or queue.size() <= pos, the build is
       * placed at the end of the queue. */
-    void        PlaceBuildInQueue(BuildType build_type, int design_id, int number, int location, int pos = -1);
+    void        PlaceProductionOnQueue(BuildType build_type, int design_id, int number, int location, int pos = -1);
     /** Adds the indicated build to the production queue, placing it before
       * position \a pos.  If \a pos < 0 or queue.size() <= pos, the build is
       * placed at the end of the queue. */
-    void        PlaceBuildInQueue(const ProductionQueue::ProductionItem& item, int number, int location, int pos = -1);
-    void        SetBuildQuantity(int index, int quantity);      ///< Changes the remaining number to build for queue item \a index to \a quantity
-    void        SetBuildQuantityAndBlocksize(int index, int quantity, int blocksize);   ///< Changes the remaining number and blocksize to build for queue item \a index to \a quantity and \a blocksize 
-    void        SetBuildRallyPoint(int index, int rally_point_id = INVALID_OBJECT_ID);  ///< Sets the rally point for ships produced by this build, to which they are automatically ordered to move after they are produced.
-    void        MoveBuildWithinQueue(int index, int new_index); ///< Moves \a tech from the production queue, if it is in the production queue already.
-    void        RemoveBuildFromQueue(int index);                ///< Removes the build at position \a index in the production queue, if such an index exists.
+    void        PlaceProductionOnQueue(const ProductionQueue::ProductionItem& item, int number, int location, int pos = -1);
+    void        SetProductionQuantity(int index, int quantity);      ///< Changes the remaining number to build for queue item \a index to \a quantity
+    void        SetProductionQuantityAndBlocksize(int index, int quantity, int blocksize);   ///< Changes the remaining number and blocksize to build for queue item \a index to \a quantity and \a blocksize 
+    void        SetProductionRallyPoint(int index, int rally_point_id = INVALID_OBJECT_ID);  ///< Sets the rally point for ships produced by this build, to which they are automatically ordered to move after they are produced.
+    void        MoveProductionWithinQueue(int index, int new_index); ///< Moves \a tech from the production queue, if it is in the production queue already.
+    void        RemoveProductionFromQueue(int index);                ///< Removes the build at position \a index in the production queue, if such an index exists.
+    void        PauseProduction(int index);
+    void        ResumeProduction(int index);
 
     void        AddTech(const std::string& name);           ///< Inserts the given Tech into the Empire's list of available technologies.
     void        UnlockItem(const ItemSpec& item);           ///< Adds a given producible item (Building, Ship Hull, Ship part) to the list of available items.
@@ -551,10 +582,13 @@ public:
     void                        UpdateOwnedObjectCounters();
 
     int                         TotalShipsOwned() const;
+    int                         TotalShipPartsOwned() const;    ///< Total number of parts for all owned ships in this empire
     int                         TotalBuildingsOwned() const;
 
     std::map<std::string, int>& SpeciesShipsOwned()     { return m_species_ships_owned; }
     std::map<int, int>&         ShipDesignsOwned()      { return m_ship_designs_owned; }
+    std::map<std::string, int>&       ShipPartTypesOwned()        { return m_ship_part_types_owned; }
+    std::map<ShipPartClass, int>&     ShipPartClassOwned()        { return m_ship_part_class_owned; }
     std::map<std::string, int>& SpeciesColoniesOwned()  { return m_species_colonies_owned; }
     int&                        OutpostsOwned()         { return m_outposts_owned; }
     std::map<std::string, int>& BuildingTypesOwned()    { return m_building_types_owned; }
@@ -599,6 +633,10 @@ private:
     GG::Clr                         m_color;                    ///< Empire's color
     int                             m_capital_id;               ///< the ID of the empire's capital planet
 
+    /** The source id is the id of any object owned by the empire.  It is
+        mutable so that Source() can be const and still cache its result. */
+    mutable int                     m_source_id;
+
     bool                            m_eliminated;               ///< Whether the empire has lost
     std::set<std::string>           m_victories;                ///< The ways that the empire has won, if any
 
@@ -616,17 +654,22 @@ private:
     std::set<std::string>           m_available_hull_types;     ///< list of acquired ship HullType.  These are string names referencing HullType objects
     std::set<int>                   m_explored_systems;         ///< systems you've explored
     std::set<int>                   m_ship_designs;             ///< The Empire's ship designs sorted by id
-    std::list<int>                  m_ship_designs_ordered;     ///<The Empire's ship designs in preferred order
+
+    /** The Empire's ship designs in preferred order. */
+    std::vector<int> m_ship_designs_ordered;
 
     std::vector<SitRepEntry>        m_sitrep_entries;           ///< The Empire's sitrep entries
 
-    std::map<ResourceType, boost::shared_ptr<ResourcePool> >    m_resource_pools;
+    std::map<ResourceType, std::shared_ptr<ResourcePool>> m_resource_pools;
+
     PopulationPool                                              m_population_pool;
 
     std::map<std::string, int>      m_ship_names_used;          ///< map from name to number of times used
 
     std::map<std::string, int>      m_species_ships_owned;      ///< how many ships of each species does this empire currently own?
     std::map<int, int>              m_ship_designs_owned;       ///< how many ships of each design does this empire currently own?
+    std::map<std::string, int>      m_ship_part_types_owned;    ///< how many ship parts are currently owned, indexed by PartType
+    std::map<ShipPartClass, int>    m_ship_part_class_owned;    ///< how many ship parts are currently owned, indexed by ShipPartClass
     std::map<std::string, int>      m_species_colonies_owned;   ///< how many colonies of each species does this empire currently own?
     int                             m_outposts_owned;           ///< how many uncolonized outposts does this empire currently own?
     std::map<std::string, int>      m_building_types_owned;     ///< how many buildings does this empire currently own?
@@ -652,8 +695,8 @@ private:
     // cached calculation results, returned by reference
     std::map<int, float>            m_supply_system_ranges;         ///< number of starlane jumps away from each system (by id) supply can be conveyed.  This is the number due to a system's contents conveying supply and is computed and set by UpdateSystemSupplyRanges
     std::set<int>                   m_supply_unobstructed_systems;  ///< ids of system that don't block supply from flowing
-    std::map<int, std::set<int> >   m_available_system_exit_lanes;  ///< for each system known to this empire, the set of available/non-blockaded exit lanes for fleet travel
-    std::map<int, std::set<int> >   m_pending_system_exit_lanes;    ///< pending updates to m_available_system_exit_lanes
+    std::map<int, std::set<int>>    m_available_system_exit_lanes;  ///< for each system known to this empire, the set of available/non-blockaded exit lanes for fleet travel
+    std::map<int, std::set<int>>    m_pending_system_exit_lanes;    ///< pending updates to m_available_system_exit_lanes
 
     friend class boost::serialization::access;
     Empire();
