@@ -1,9 +1,10 @@
 #ifndef _EmpireManager_h_
 #define _EmpireManager_h_
 
-#include "../universe/Enums.h"
+#include "../universe/EnumsFwd.h"
 #include "Diplomacy.h"
 #include "../util/Export.h"
+#include "../util/Serialize.h"
 
 #include <GG/Clr.h>
 
@@ -16,6 +17,7 @@
 #include <vector>
 
 class Empire;
+class UniverseObject;
 
 struct FO_COMMON_API DiplomaticStatusUpdateInfo {
     DiplomaticStatusUpdateInfo();
@@ -35,13 +37,16 @@ public:
     typedef std::map<int, Empire*>::const_iterator const_iterator;
 
     /** \name Structors */ //@{
-    virtual ~EmpireManager(); ///< virtual dtor
+    virtual ~EmpireManager();
+
     const EmpireManager& operator=(EmpireManager& rhs); ///< assignment operator (move semantics)
     //@}
 
     /** \name Accessors */ //@{
     /** Returns the empire whose ID is \a ID, or 0 if none exists. */
     const Empire*       GetEmpire(int id) const;
+    /** Return the empire source or nullptr if the empire or source doesn't exist.*/
+    std::shared_ptr<const UniverseObject> GetSource(int id) const;
     const std::string&  GetEmpireName(int id) const;
 
     const_iterator      begin() const;
@@ -50,8 +55,10 @@ public:
     int                 NumEmpires() const;
 
     DiplomaticStatus            GetDiplomaticStatus(int empire1, int empire2) const;
-    bool                        DiplomaticMessageAvailable(int empire1, int empire2) const;
-    const DiplomaticMessage&    GetDiplomaticMessage(int empire1, int empire2) const;
+    std::set<int>               GetEmpireIDsWithDiplomaticStatusWithEmpire(int empire_id,
+                                                                           DiplomaticStatus diplo_status) const;
+    bool                        DiplomaticMessageAvailable(int sender_id, int recipient_id) const;
+    const DiplomaticMessage&    GetDiplomaticMessage(int sender_id, int recipient_id) const;
 
     std::string         Dump() const;
     //@}
@@ -68,7 +75,7 @@ public:
     void        SetDiplomaticStatus(int empire1, int empire2, DiplomaticStatus status);
     void        HandleDiplomaticMessage(const DiplomaticMessage& message);
     void        SetDiplomaticMessage(const DiplomaticMessage& message);
-    void        RemoveDiplomaticMessage(int empire1, int empire2);
+    void        RemoveDiplomaticMessage(int sender_id, int recipient_id);
 
     void        ResetDiplomacy();
 
@@ -77,7 +84,7 @@ public:
       * caller's responsibility to make sure that universe updates planet
       * ownership. */
     Empire*     CreateEmpire(int empire_id, const std::string& name, const std::string& player_name,
-                                 const GG::Clr& color);
+                             const GG::Clr& color);
 
     /** Removes and deletes all empires from the manager. */
     void        Clear();
@@ -91,6 +98,8 @@ public:
 private:
     EmpireManager();
 
+    std::string DumpDiplomacy() const;
+
     /** Adds the given empire to the manager. */
     void        InsertEmpire(Empire* empire);
     void        GetDiplomaticMessagesToSerialize(std::map<std::pair<int, int>, DiplomaticMessage>& messages,
@@ -108,6 +117,11 @@ private:
     void serialize(Archive& ar, const unsigned int version);
 };
 
+extern template FO_COMMON_API void EmpireManager::serialize<freeorion_bin_oarchive>(freeorion_bin_oarchive&, const unsigned int);
+extern template FO_COMMON_API void EmpireManager::serialize<freeorion_bin_iarchive>(freeorion_bin_iarchive&, const unsigned int);
+extern template FO_COMMON_API void EmpireManager::serialize<freeorion_xml_oarchive>(freeorion_xml_oarchive&, const unsigned int);
+extern template FO_COMMON_API void EmpireManager::serialize<freeorion_xml_iarchive>(freeorion_xml_iarchive&, const unsigned int);
+
 /** The colors that are available for use for empires in the game. */
 FO_COMMON_API const std::vector<GG::Clr>& EmpireColors();
 
