@@ -61,12 +61,12 @@ public:
     //@}
 
     /** \name Structors */ ///@{
-    Button(const std::string& str, const boost::shared_ptr<Font>& font, Clr color,
-           Clr text_color = CLR_BLACK, Flags<WndFlag> flags = INTERACTIVE); ///< ctor
+    Button(const std::string& str, const std::shared_ptr<Font>& font, Clr color,
+           Clr text_color = CLR_BLACK, Flags<WndFlag> flags = INTERACTIVE);
     //@}
 
     /** \name Accessors */ ///@{
-    virtual Pt       MinUsableSize() const;
+    Pt MinUsableSize() const override;
 
     /** Returns button state \see ButtonState */
     ButtonState       State() const;
@@ -83,11 +83,12 @@ public:
     //@}
 
     /** \name Mutators */ ///@{
-    virtual void   Show(bool children = true);
-    virtual void   Render();
-    virtual void   SizeMove(const Pt& ul, const Pt& lr);
+    void Show(bool children = true) override;
+    void Render() override;
+    void SizeMove(const Pt& ul, const Pt& lr) override;
 
-    virtual void   SetColor(Clr c); ///< Sets the control's color; does not affect the text color
+    /** Sets the control's color; does not affect the text color. */
+    void SetColor(Clr c) override;
 
     /** Sets button state programmatically \see ButtonState */
     void           SetState(ButtonState state);
@@ -100,20 +101,24 @@ public:
 
 protected:
     /** \name Mutators */ ///@{
-    virtual void   LButtonDown(const Pt& pt, Flags<ModKey> mod_keys);
-    virtual void   LDrag(const Pt& pt, const Pt& move, Flags<ModKey> mod_keys);
-    virtual void   LButtonUp(const Pt& pt, Flags<ModKey> mod_keys);
-    virtual void   LClick(const Pt& pt, Flags<ModKey> mod_keys);
-    virtual void   RButtonDown(const Pt& pt, Flags<ModKey> mod_keys);
-    virtual void   RDrag(const Pt& pt, const Pt& move, Flags<ModKey> mod_keys);
-    virtual void   RButtonUp(const Pt& pt, Flags<ModKey> mod_keys);
-    virtual void   RClick(const Pt& pt, Flags<ModKey> mod_keys);
-    virtual void   MouseHere(const Pt& pt, Flags<ModKey> mod_keys);
-    virtual void   MouseLeave();
+    void LButtonDown(const Pt& pt, Flags<ModKey> mod_keys) override;
+    void LDrag(const Pt& pt, const Pt& move, Flags<ModKey> mod_keys) override;
+    void LButtonUp(const Pt& pt, Flags<ModKey> mod_keys) override;
+    void LClick(const Pt& pt, Flags<ModKey> mod_keys) override;
+    void RButtonDown(const Pt& pt, Flags<ModKey> mod_keys) override;
+    void RDrag(const Pt& pt, const Pt& move, Flags<ModKey> mod_keys) override;
+    void RButtonUp(const Pt& pt, Flags<ModKey> mod_keys) override;
+    void RClick(const Pt& pt, Flags<ModKey> mod_keys) override;
+    void MouseEnter(const Pt& pt, Flags<ModKey> mod_keys) override;
+    void MouseHere(const Pt& pt, Flags<ModKey> mod_keys) override;
+    void MouseLeave() override;
 
-    virtual void   RenderUnpressed();   ///< Draws the button unpressed.  If an unpressed graphic has been supplied, it is used.
-    virtual void   RenderPressed();     ///< Draws the button pressed.  If an pressed graphic has been supplied, it is used.
-    virtual void   RenderRollover();    ///< Draws the button rolled-over.  If an rollover graphic has been supplied, it is used.
+    /** Draws the button unpressed.  If an unpressed graphic has been supplied, it is used. */
+    virtual void RenderUnpressed();
+    /** Draws the button pressed.  If an pressed graphic has been supplied, it is used. */
+    virtual void RenderPressed();
+    /** Draws the button rolled-over.  If an rollover graphic has been supplied, it is used. */
+    virtual void RenderRollover();
     //@}
 
     TextControl*   m_label;             ///< Label used to display text
@@ -128,6 +133,89 @@ private:
 };
 
 
+class StateButtonRepresenter;
+
+
+/** \brief This is a basic state button control.
+
+    This class is for checkboxes and radio buttons, etc.  The button/checkbox
+    area is determined from the text height and format; the button height and
+    width will be the text height, and the the button will be positioned to
+    the left of the text and vertically the same as the text, unless the text
+    is centered, in which case the button and text will be centered, and the
+    button will appear above or below the text.  Whenever there is not room to
+    place the button and the text in the proper orientation because the entire
+    control's size is too small, the button and text are positioned in their
+    default spots (button on left, text on right, centered vertically). */
+class GG_API StateButton : public Control
+{
+public:
+    /// the states of being for a GG::Button
+    GG_CLASS_ENUM(ButtonState,
+        BN_PRESSED,    ///< The button is being pressed by the user, and the cursor is over the button
+        BN_UNPRESSED,  ///< The button is unpressed
+        BN_ROLLOVER    ///< The button has the cursor over it, but is unpressed
+    )
+
+    /** \name Signal Types */ ///@{
+    /** Emitted when the StateButton is checked or unchecked; the checked or
+        unchecked status is indicated by the bool parameter */
+    typedef boost::signals2::signal<void (bool)> CheckedSignalType;
+    //@}
+
+    /** \name Structors */ ///@{
+    StateButton(const std::string& str, const std::shared_ptr<Font>& font, Flags<TextFormat> format,
+                Clr color, std::shared_ptr<StateButtonRepresenter> representer, Clr text_color = CLR_BLACK); ///< Ctor
+    //@}
+
+    /** \name Accessors */ ///@{
+    Pt MinUsableSize() const override;
+
+    /** Returns button state \see ButtonState */
+    ButtonState       State() const;
+
+    const std::string& Text() const;        ///< Returns the label to be used as the button label
+
+    bool             Checked() const;       ///< Returns true if button is checked
+
+    TextControl* GetLabel() const;
+
+    mutable CheckedSignalType CheckedSignal; ///< The checked signal object for this StaticButton
+    //@}
+
+    /** \name Mutators */ ///@{
+    void Show(bool children = true) override;
+    void Render() override;
+    void SizeMove(const Pt& ul, const Pt& lr) override;
+
+    void             Reset();                 ///< Unchecks button
+    void             SetCheck(bool b = true); ///< (Un)checks button
+    void             SetTextColor(Clr c); ///< Sets the color of the box label text
+    //@}
+
+protected:
+    /** \name Mutators */ ///@{
+    void LButtonDown(const Pt& pt, Flags<ModKey> mod_keys) override;
+    void LDrag(const Pt& pt, const Pt& move, Flags<ModKey> mod_keys) override;
+    void LButtonUp(const Pt& pt, Flags<ModKey> mod_keys) override;
+    void LClick(const Pt& pt, Flags<ModKey> mod_keys) override;
+    void MouseHere(const Pt& pt, Flags<ModKey> mod_keys) override;
+    void MouseLeave() override;
+
+    /** Sets button state programmatically \see ButtonState */
+    void         SetState(ButtonState next_state);
+    //@}
+
+private:
+    std::shared_ptr<StateButtonRepresenter> m_representer;
+
+    TextControl*      m_label;       ///< Label used to display text
+
+    ButtonState       m_state;       ///< Button is always in exactly one of the ButtonState states above
+    bool              m_checked;     ///< true when this button in a checked, active state
+};
+
+
 /** \brief Visual representation of a state button.
 
     The StateButtonRepresenter is a stub interface to implement a visual
@@ -144,6 +232,16 @@ public:
      */
     virtual void Render(const StateButton& button) const;
 
+    /** \brief Respong to a button state change.
+
+        This method is called whenever a Button chanes its state.
+
+        \param button  The button that changed its state and is in the most
+                       recent state.
+        \param previous_state  The previous button state.
+     */
+    virtual void OnChanged(const StateButton& button, StateButton::ButtonState previous_state) const;
+
     /** \brief Respond to a state button change.
 
         This method is called whenever a StateButton changes its
@@ -183,7 +281,7 @@ class GG_API BeveledCheckBoxRepresenter: public StateButtonRepresenter
 public:
     explicit BeveledCheckBoxRepresenter(Clr int_color = CLR_ZERO);
 
-    virtual void Render(const StateButton& button) const;
+    void Render(const StateButton& button) const override;
 
 private:
     Clr m_int_color;
@@ -196,7 +294,7 @@ class GG_API BeveledRadioRepresenter: public StateButtonRepresenter
 public:
     explicit BeveledRadioRepresenter(Clr int_color = CLR_ZERO);
 
-    virtual void Render(const StateButton& button) const;
+    void Render(const StateButton& button) const override;
 
 private:
     Clr m_int_color;
@@ -207,74 +305,9 @@ private:
 class GG_API BeveledTabRepresenter: public StateButtonRepresenter
 {
 public:
-    virtual void Render(const StateButton& button) const;
+    void Render(const StateButton& button) const override;
 
-    virtual Pt MinUsableSize(const StateButton& button) const;
-};
-
-
-/** \brief This is a basic state button control.
-
-    This class is for checkboxes and radio buttons, etc.  The button/checkbox
-    area is determined from the text height and format; the button height and
-    width will be the text height, and the the button will be positioned to
-    the left of the text and vertically the same as the text, unless the text
-    is centered, in which case the button and text will be centered, and the
-    button will appear above or below the text.  Whenever there is not room to
-    place the button and the text in the proper orientation because the entire
-    control's size is too small, the button and text are positioned in their
-    default spots (button on left, text on right, centered vertically). */
-class GG_API StateButton : public Control
-{
-public:
-    /** \name Signal Types */ ///@{
-    /** Emitted when the StateButton is checked or unchecked; the checked or
-        unchecked status is indicated by the bool parameter */
-    typedef boost::signals2::signal<void (bool)> CheckedSignalType;
-    //@}
-
-    /** \name Structors */ ///@{
-    StateButton(const std::string& str, const boost::shared_ptr<Font>& font, Flags<TextFormat> format,
-                Clr color, boost::shared_ptr<StateButtonRepresenter> representer, Clr text_color = CLR_BLACK); ///< Ctor
-    //@}
-
-    /** \name Accessors */ ///@{
-    virtual Pt       MinUsableSize() const;
-
-    const std::string& Text() const;        ///< Returns the label to be used as the button label
-
-    bool             Checked() const;       ///< Returns true if button is checked
-    bool             IsMouseover() const;   ///< Returns true if mouse pointer is over this button
-
-    TextControl* GetLabel() const;
-
-    mutable CheckedSignalType CheckedSignal; ///< The checked signal object for this StaticButton
-    //@}
-
-    /** \name Mutators */ ///@{
-    virtual void     Show(bool children = true);
-    virtual void     Render();
-    virtual void     SizeMove(const Pt& ul, const Pt& lr);
-
-    void             Reset();                 ///< Unchecks button
-    void             SetCheck(bool b = true); ///< (Un)checks button
-    virtual void     SetColor(Clr c);         ///< Sets the color of the button; does not affect text color
-    void             SetTextColor(Clr c); ///< Sets the color of the box label text
-    //@}
-
-protected:
-    /** \name Mutators */ ///@{
-    virtual void LClick(const Pt& pt, Flags<ModKey> mod_keys);
-    virtual void MouseEnter(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void MouseLeave();
-    //@}
-
-private:
-    boost::shared_ptr<StateButtonRepresenter> m_representer;
-    TextControl*      m_label;       ///< Label used to display text
-
-    bool              m_checked;     ///< true when this button in a checked, active state
-    bool              m_mouseover;   ///< true if the pointer is currently over the control
+    Pt MinUsableSize(const StateButton& button) const override;
 };
 
 
@@ -297,11 +330,11 @@ public:
     //@}
 
     /** \name Structors */ ///@{
-    RadioButtonGroup(Orientation orientation); ///< ctor
+    RadioButtonGroup(Orientation orientation);
     //@}
 
     /** \name Accessors */ ///@{
-    virtual Pt       MinUsableSize() const;
+    Pt MinUsableSize() const override;
 
     /** Returns the orientation of the buttons in the group */
     Orientation      GetOrientation() const;
@@ -336,7 +369,7 @@ public:
     //@}
 
     /** \name Mutators */ ///@{
-    virtual void Render();
+    void Render() override;
 
     /** Checks the index-th button, and unchecks all others.  If there is no
         index-th button, they are all unchecked, and the currently-checked
@@ -394,9 +427,10 @@ protected:
         RadioButtonGroup. */
     struct GG_API ButtonSlot
     {
-        ButtonSlot();
         ButtonSlot(StateButton* button_);
-        StateButton*                button;
+
+        StateButton* button;
+
         boost::signals2::connection connection;
     };
 
@@ -405,17 +439,6 @@ protected:
     //@}
 
 private:
-    class ButtonClickedFunctor // for catching button-click signals from the contained buttons
-    {
-    public:
-        ButtonClickedFunctor(RadioButtonGroup* group, StateButton* button, std::size_t index);
-        void operator()(bool checked);
-    private:
-        RadioButtonGroup* m_group;
-        StateButton*      m_button;
-        std::size_t       m_index;
-    };
-
     void ConnectSignals();
     void SetCheckImpl(std::size_t index, bool signal);
     void Reconnect();
@@ -426,8 +449,6 @@ private:
     bool                    m_expand_buttons;
     bool                    m_expand_buttons_proportionally;
     bool                    m_render_outline;
-
-    friend class ButtonClickedFunctor;
 };
 
 } // namespace GG
