@@ -34,6 +34,9 @@
 #include <GG/ListBox.h>
 #include <GG/GLClientAndServerBuffer.h>
 
+#include <memory>
+
+
 class ModalListPicker;
 
 namespace GG {
@@ -50,7 +53,16 @@ namespace GG {
     DropDownList.  Though you can still set the alignment, etc. of individual
     rows, as in ListBox, the currently-selected row will have the same
     alignment, etc. when displayed in the control in its unopened state.  Note
-    that this may look quite ugly. */
+    that this may look quite ugly.
+
+    On selection DropDownList emits one of two signals, SelChangedSignal or
+    SelChangedWhileDroppedSignal.  SelChangedWhileDroppedSignal is emitted
+    when the selection changes while running a ModalEventPump to display the
+    drop down list and handle its events.
+
+    SelChangedSignal will also be emitted when the drop down list closes if
+    the selected item changed.
+*/
 class GG_API DropDownList : public Control
 {
 public:
@@ -64,13 +76,16 @@ public:
     /** emitted when a new item is selected; will be end() when no item is
       * selected */
     typedef boost::signals2::signal<void (iterator)>   SelChangedSignalType;
+
+    /** Signal \a true when drop down opens and false when it closes.*/
+    typedef boost::signals2::signal<void (bool)>       DropDownOpenedSignalType;
     //@}
 
     /** \name Structors */ ///@{
     /** basic ctor.  DropDownList retains ownership of \a lb, if it is non-null. */
     DropDownList(size_t num_shown_elements, Clr color);
 
-    ~DropDownList(); ///< dtor
+    ~DropDownList();
     //@}
 
     /** \name Accessors */ ///@{
@@ -91,6 +106,7 @@ public:
     Clr             InteriorColor() const;          ///< returns the color painted into the client area of the control
 
     Y               DropHeight() const; ///< returns the height of the drop-down list
+    bool            Dropped() const;                ///< Return true if the drop down list is open.
 
     /** Returns the style flags of the list \see GG::ListBoxStyle */
     Flags<ListBoxStyle> Style() const;
@@ -107,16 +123,34 @@ public:
     Alignment       ColAlignment(std::size_t n) const; ///< returns the alignment of column \a n; must be LIST_LEFT, LIST_CENTER, or LIST_RIGHT; not range-checked
     Alignment       RowAlignment(iterator it) const;   ///< returns the alignment of row \a n; must be LIST_TOP, LIST_VCENTER, or LIST_BOTTOM; not range-checked
 
-    virtual Pt      ClientUpperLeft() const;
-    virtual Pt      ClientLowerRight() const;
+    Pt ClientUpperLeft() const override;
+    Pt ClientLowerRight() const override;
 
-    mutable SelChangedSignalType SelChangedSignal; ///< the selection change signal object for this DropDownList
+    /** Return the width of the displayed row.  Override this function if the displayed row is a
+        different width than the client width.*/
+    virtual GG::X DisplayedRowWidth() const;
+
+    /** Return the width of the dropped row.  Override this function if the dropped row is a
+        different width than the client width.*/
+    virtual GG::X DroppedRowWidth() const;
+
+    /** The selection change signal while not running the modal drop down box.
+        This will also signal an event when the drop list closes if the selection changed.
+     */
+    mutable SelChangedSignalType SelChangedSignal;
+    /** The selection change signal while running the modal drop down box.*/
+    mutable SelChangedSignalType SelChangedWhileDroppedSignal;
+
+    DropDownOpenedSignalType DropDownOpenedSignal;
     //@}
 
     /** \name Mutators */ ///@{
-    virtual void    Render();
-    virtual void    SizeMove(const Pt& ul, const Pt& lr); ///< resizes the control, ensuring the proper height is maintained based on the list's row height
-    virtual void    SetColor(Clr c);
+    void PreRender() override;
+    void Render() override;
+    /** Resizes the control, ensuring the proper height is maintained based on
+        the list's row height. */
+    void SizeMove(const Pt& ul, const Pt& lr) override;
+    void SetColor(Clr c) override;
 
     /** Insertion sorts \a row into a sorted list, or inserts into an unsorted
         list before \a it; returns index of insertion point.  This Row becomes
@@ -170,30 +204,40 @@ public:
         to an empty ListBox */
     void            UnLockColWidths();
 
+    /** Set ListBox to stop managing column widths and alignment.  The number of columns must be
+        set with SetColWidth(), but widths of individual rows columns or the header will not be
+        managed by ListBox. */
+    void            ManuallyManageColProps();
+
     void            SetColAlignment(std::size_t n, Alignment align); ///< sets the alignment of column \a n to \a align; not range-checked
     void            SetRowAlignment(iterator it, Alignment align);   ///< sets the alignment of the Row at row index \a n to \a align; not range-checked
+
+    /** Sets the stretch of column \a n to \a stretch; not range-checked */
+    void            SetColStretch(std::size_t n, double stretch);
+
+    /** Sets whether to normalize rows when inserted (true) or leave them as
+      * they are. */
+    void            NormalizeRowsOnInsert(bool enable = true);
     //@}
 
 protected:
     /** \name Mutators */ ///@{
-    virtual void    LClick(const Pt& pt, Flags<ModKey> mod_keys);
-    virtual void    KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_keys);
-    virtual void    MouseWheel(const Pt& pt, int move, Flags<ModKey> mod_keys);
+    void LClick(const Pt& pt, Flags<ModKey> mod_keys) override;
+    void KeyPress(Key key, std::uint32_t key_code_point, Flags<ModKey> mod_keys) override;
+    void MouseWheel(const Pt& pt, int move, Flags<ModKey> mod_keys) override;
 
     ListBox*        LB();                ///< returns the ListBox used to render the selected row and the popup list
 
-    virtual void    InitBuffer();
-    virtual void    RenderDisplayedRow();
+    virtual void InitBuffer();
+    virtual void RenderDisplayedRow();
 
     GL2DVertexBuffer    m_buffer;
     //@}
 
 private:
-    void            SelectImpl(iterator it, bool signal);
     const ListBox*  LB() const;
 
-    ModalListPicker*    m_modal_picker;
-    size_t              m_num_shown_elements;
+    const std::shared_ptr<ModalListPicker> m_modal_picker;
 };
 
 } // namespace GG
