@@ -32,6 +32,8 @@
 #include <GG/Font.h>
 #include <GG/WndEvent.h>
 
+#include <chrono>
+
 
 namespace boost { namespace archive {
     class xml_oarchive;
@@ -148,14 +150,14 @@ public:
 
     /** The type of iterator returned by non-const accel_begin() and
         accel_end(). */
-    typedef std::set<std::pair<Key, Flags<ModKey> > >::iterator accel_iterator;
+    typedef std::set<std::pair<Key, Flags<ModKey>>>::iterator accel_iterator;
 
     /** The type of iterator returned by const accel_begin() and
         accel_end(). */
-    typedef std::set<std::pair<Key, Flags<ModKey> > >::const_iterator const_accel_iterator;
+    typedef std::set<std::pair<Key, Flags<ModKey>>>::const_iterator const_accel_iterator;
 
     /** \name Structors */ ///@{
-    virtual ~GUI(); ///< virtual dtor
+    virtual ~GUI();
     //@}
 
     /** \name Accessors */ ///@{
@@ -197,24 +199,27 @@ public:
         this function to exist, but the notion of what a "word" is is so
         application-specific that it was placed here so that users can
         customize this behavior. */
-    virtual std::set<std::pair<CPSize, CPSize> >    FindWords(const std::string& str) const;
-    virtual std::set<std::pair<StrSize, StrSize> >  FindWordsStringIndices(const std::string& str) const;
+    virtual std::set<std::pair<CPSize, CPSize>>     FindWords(const std::string& str) const;
+    virtual std::set<std::pair<StrSize, StrSize>>   FindWordsStringIndices(const std::string& str) const;
     /** Returns true if \a word is a word that appears in \a str */
     virtual bool                                    ContainsWord(const std::string& str, const std::string& word) const;
 
-    const boost::shared_ptr<StyleFactory>& GetStyleFactory() const; ///< returns the currently-installed style factory
+    /** Returns the currently-installed style factory. */
+    const std::shared_ptr<StyleFactory>&    GetStyleFactory() const;
 
-    bool                                   RenderCursor() const; ///< returns true iff the GUI is responsible for rendering the cursor
-    const boost::shared_ptr<Cursor>&       GetCursor() const; ///< returns the currently-installed cursor
+    bool                                    RenderCursor() const; ///< returns true iff the GUI is responsible for rendering the cursor
+
+    /* Returns the currently-installed cursor. */
+    const std::shared_ptr<Cursor>&          GetCursor() const;
 
     /** Returns an iterator to one past the first defined keyboard accelerator. */
-    const_accel_iterator accel_begin() const;
+    const_accel_iterator                    accel_begin() const;
 
     /** Returns an iterator to one past the last defined keyboard accelerator. */
-    const_accel_iterator accel_end() const;
+    const_accel_iterator                    accel_end() const;
 
     /** Returns the signal that is emitted when the requested keyboard accelerator is invoked. */
-    AcceleratorSignalType& AcceleratorSignal(Key key, Flags<ModKey> mod_keys = MOD_KEY_NONE) const;
+    AcceleratorSignalType&                  AcceleratorSignal(Key key, Flags<ModKey> mod_keys = MOD_KEY_NONE) const;
 
     /** Returns true iff keyboard accelerator signals fire while modal windows are open. */
     bool ModalAcceleratorSignalsEnabled() const;
@@ -236,10 +241,16 @@ public:
         function should only be called from custom EventPump event
         handlers. */
     virtual void    HandleSystemEvents() = 0;
-    void            HandleGGEvent(EventType event, Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_keys, const Pt& pos, const Pt& rel, const std::string* text = NULL); ///< event handler for GG events
+
+    /** Event handler for GG events. */
+    void HandleGGEvent(EventType event, Key key, std::uint32_t key_code_point, Flags<ModKey> mod_keys, const Pt& pos, const Pt& rel, const std::string* text = nullptr);
+
     void            ClearEventState();
 
     void            SetFocusWnd(Wnd* wnd);          ///< sets the input focus window to \a wnd
+    /** Suspend the GUI thread for \p us microseconds.  Singlethreaded GUI subclasses may do
+        nothing here, or may pause for \p us microseconds. */
+    virtual void    Wait(std::chrono::microseconds us);
     virtual void    Wait(unsigned int ms);          ///< suspends the GUI thread for \a ms milliseconds.  Singlethreaded GUI subclasses may do nothing here, or may pause for \a ms milliseconds.
     void            Register(Wnd* wnd);             ///< adds \a wnd into the z-list.  Registering a null pointer or registering the same window multiple times is a no-op.
     void            RegisterModal(Wnd* wnd);        ///< adds \a wnd onto the modal windows "stack"
@@ -248,8 +259,9 @@ public:
     void            MoveUp(Wnd* wnd);               ///< moves \a wnd to the top of the z-list
     void            MoveDown(Wnd* wnd);             ///< moves \a wnd to the bottom of the z-list
 
-    virtual boost::shared_ptr<ModalEventPump>
-                   CreateModalEventPump(bool& done); ///< creates a new ModalEventPump that will terminate when \a done is set to true
+    /** Creates a new ModalEventPump that will terminate when \a done is set to
+        true. */
+    virtual std::shared_ptr<ModalEventPump> CreateModalEventPump(bool& done);
 
     /** Adds \a wnd to the set of current drag-and-drop Wnds, to be rendered
         \a offset pixels from the cursor position. \a originating_wnd
@@ -301,29 +313,29 @@ public:
 
     /** Returns a shared_ptr to the desired font, supporting all printable
         ASCII characters. */
-    boost::shared_ptr<Font>    GetFont(const std::string& font_filename, unsigned int pts);
+    std::shared_ptr<Font> GetFont(const std::string& font_filename, unsigned int pts);
 
     /** Returns a shared_ptr to the desired font, supporting all printable
         ASCII characters, from the in-memory contents \a file_contents. */
-    boost::shared_ptr<Font>    GetFont(const std::string& font_filename, unsigned int pts,
-                                       const std::vector<unsigned char>& file_contents);
+    std::shared_ptr<Font> GetFont(const std::string& font_filename, unsigned int pts,
+                                  const std::vector<unsigned char>& file_contents);
 
     /** Returns a shared_ptr to the desired font, supporting all the
         characters in the UnicodeCharsets in the range [first, last). */
     template <class CharSetIter>
-    boost::shared_ptr<Font>    GetFont(const std::string& font_filename, unsigned int pts,
-                                       CharSetIter first, CharSetIter last);
+    std::shared_ptr<Font> GetFont(const std::string& font_filename, unsigned int pts,
+                                  CharSetIter first, CharSetIter last);
 
     /** Returns a shared_ptr to the desired font, supporting all the
         characters in the UnicodeCharsets in the range [first, last), from the
         in-memory contents \a file_contents. */
     template <class CharSetIter>
-    boost::shared_ptr<Font>    GetFont(const std::string& font_filename, unsigned int pts,
-                                       const std::vector<unsigned char>& file_contents,
-                                       CharSetIter first, CharSetIter last);
+    std::shared_ptr<Font> GetFont(const std::string& font_filename, unsigned int pts,
+                                  const std::vector<unsigned char>& file_contents,
+                                  CharSetIter first, CharSetIter last);
 
     /** Returns a shared_ptr to existing font \a font in a new size, \a pts. */
-    boost::shared_ptr<Font>    GetFont(const boost::shared_ptr<Font>& font, unsigned int pts);
+    std::shared_ptr<Font> GetFont(const std::shared_ptr<Font>& font, unsigned int pts);
 
     /** Removes the desired font from the managed pool; since shared_ptr's are
         used, the font may be deleted much later. */
@@ -332,18 +344,18 @@ public:
     /** Adds an already-constructed texture to the managed pool \warning
         calling code <b>must not</b> delete \a texture; the texture pool will
         do that. */
-    boost::shared_ptr<Texture> StoreTexture(Texture* texture, const std::string& texture_name);
+    std::shared_ptr<Texture> StoreTexture(Texture* texture, const std::string& texture_name);
 
     /** Adds an already-constructed texture to the managed pool. */
-    boost::shared_ptr<Texture> StoreTexture(const boost::shared_ptr<Texture> &texture, const std::string& texture_name);
+    std::shared_ptr<Texture> StoreTexture(const std::shared_ptr<Texture> &texture, const std::string& texture_name);
 
     /** Loads the requested texture from file \a name; mipmap textures are
       * generated if \a mipmap is true. */
-    boost::shared_ptr<Texture> GetTexture(const std::string& name, bool mipmap = false);
+    std::shared_ptr<Texture> GetTexture(const std::string& name, bool mipmap = false);
 
     /** Loads the requested texture from file \a name; mipmap textures are
       * generated if \a mipmap is true. */
-    boost::shared_ptr<Texture> GetTexture(const boost::filesystem::path& path, bool mipmap = false);
+    std::shared_ptr<Texture> GetTexture(const boost::filesystem::path& path, bool mipmap = false);
 
     /** Removes the desired texture from the managed pool; since shared_ptr's
       * are used, the texture may be deleted much later. */
@@ -353,10 +365,13 @@ public:
       * are used, the texture may be deleted much later. */
     void                       FreeTexture(const boost::filesystem::path& path);
 
-    void SetStyleFactory(const boost::shared_ptr<StyleFactory>& factory); ///< sets the currently-installed style factory
+    /** Sets the currently-installed style factory. */
+    void SetStyleFactory(const std::shared_ptr<StyleFactory>& factory);
 
     void RenderCursor(bool render); ///< set this to true iff the GUI should render the cursor
-    void SetCursor(const boost::shared_ptr<Cursor>& cursor); ///< sets the currently-installed cursor
+
+    /** Sets the currently-installed cursor. */
+    void SetCursor(const std::shared_ptr<Cursor>& cursor);
 
     virtual bool SetClipboardText(const std::string& text); ///< sets text stored in clipboard
     bool CopyFocusWndText();                                ///< copies current focus Wnd as text to clipboard
@@ -376,6 +391,11 @@ public:
     //@}
 
     static GUI*  GetGUI();                  ///< allows any GG code access to GUI framework by calling GUI::GetGUI()
+
+    /** If \p wnd is visible recursively call PreRenderWindow() on all \p wnd's children and then
+        call \p wnd->PreRender().  The order guarantees that when wnd->PreRender() is called all
+        of \p wnd's children have already been prerendered.*/
+    static void  PreRenderWindow(Wnd* wnd);
     static void  RenderWindow(Wnd* wnd);    ///< renders a window (if it is visible) and all its visible descendents recursively
     virtual void RenderDragDropWnds();      ///< renders Wnds currently being drag-dropped
 
@@ -390,6 +410,12 @@ public:
       * focus. bool parameter is true when gaining focus, and false otherwise.*/
      boost::signals2::signal<void (bool)>   FocusChangedSignal;
 
+    /** Emitted whenever the window manager requests the window close. */
+    boost::signals2::signal<void ()>    WindowClosingSignal;
+
+    /** Emitted whenever the app is requested to close. */
+    boost::signals2::signal<void ()>    AppQuittingSignal;
+
     /** \name Exceptions */ ///@{
     /** The base class for GUI exceptions. */
     GG_ABSTRACT_EXCEPTION(Exception);
@@ -408,7 +434,7 @@ public:
     virtual std::vector<std::string> GetSupportedResolutions() const = 0;
 
     /** Returns the default resolution of the display */
-    virtual Pt GetDefaultResolution(int display_id) = 0;
+    virtual Pt GetDefaultResolution(int display_id) const = 0;
 
 protected:
     /** \name Structors */ ///@{
@@ -417,6 +443,8 @@ protected:
 
     /** \name Mutators */ ///@{
     void           ProcessBrowseInfo();    ///< determines the current browse info mode, if any
+    /** Allow all windows in the z-list to update data before rendering. */
+    virtual void   PreRender();
     virtual void   RenderBegin() = 0;      ///< clears the backbuffer, etc.
     virtual void   Render();               ///< renders the windows in the z-list
     virtual void   RenderEnd() = 0;        ///< swaps buffers, etc.
@@ -437,7 +465,7 @@ private:
     Wnd*           CheckedGetWindowUnder(const Pt& pt, Flags<ModKey> mod_keys);
 
     static GUI*                       s_gui;
-    static boost::shared_ptr<GUIImpl> s_impl;
+    static std::shared_ptr<GUIImpl> s_impl;
 
     friend class EventPumpBase; ///< allows EventPumpBase types to drive GUI
     friend struct GUIImpl;
@@ -465,14 +493,14 @@ bool GUI::OrCombiner::operator()(InIt first, InIt last) const
 }
 
 template <class CharSetIter>
-boost::shared_ptr<Font> GUI::GetFont(const std::string& font_filename, unsigned int pts,
-                                     CharSetIter first, CharSetIter last)
+std::shared_ptr<Font> GUI::GetFont(const std::string& font_filename, unsigned int pts,
+                                   CharSetIter first, CharSetIter last)
 { return GetFontManager().GetFont(font_filename, pts, first, last); }
 
 template <class CharSetIter>
-boost::shared_ptr<Font> GUI::GetFont(const std::string& font_filename, unsigned int pts,
-                                     const std::vector<unsigned char>& file_contents,
-                                     CharSetIter first, CharSetIter last)
+std::shared_ptr<Font> GUI::GetFont(const std::string& font_filename, unsigned int pts,
+                                   const std::vector<unsigned char>& file_contents,
+                                   CharSetIter first, CharSetIter last)
 { return GetFontManager().GetFont(font_filename, pts, file_contents, first, last); }
 
 } // namespace GG
