@@ -30,6 +30,7 @@
 
 #include <GG/Base.h>
 #include <GG/StrongTypedef.h>
+#include <boost/functional/hash.hpp>
 
 
 namespace GG {
@@ -56,7 +57,8 @@ extern GG_API const Y Y1;
 struct GG_API Pt
 {
     /** \name Structors */ ///@{
-    Pt();               ///< Default ctor.
+    Pt();
+
     Pt(X x_, Y y_);     ///< Ctor that creates a Pt ( \a _x , \a y ).
     Pt(X_d x_, Y y_);   ///< Ctor that creates a Pt ( \a _x , \a y ).
     Pt(X x_, Y_d y_);   ///< Ctor that creates a Pt ( \a _x , \a y ).
@@ -92,7 +94,8 @@ GG_API std::ostream& operator<<(std::ostream& os, const Pt& pt);
 struct GG_API Rect
 {
     /** \name Structors */ ///@{
-    Rect();                                ///< default ctor
+    Rect();
+
     Rect(const Pt& pt1, const Pt& pt2);    ///< ctor that constructs a Rect from two corners; any two opposing corners will do
     Rect(X x1, Y y1, X x2, Y y2);  ///< ctor that constructs a Rect from its left, upper, right, and bottom boundaries
     //@}
@@ -148,6 +151,23 @@ GG_API inline Rect operator-(const Pt& pt, const Rect& rect) { return rect - pt;
 
 GG_API std::ostream& operator<<(std::ostream& os, const Rect& rect); ///< Rect stream-output operator for debug output
 
+    // Hash functions
+    // Replace with C++11 equilvalent when converted to C++11
+    GG_API inline std::size_t hash_value(X const& x) { return boost::hash<int>()(Value(x)); }
+    GG_API inline std::size_t hash_value(Y const& y) { return boost::hash<int>()(Value(y)); }
+    GG_API inline std::size_t hash_value(Pt const& pt) {
+        std::size_t seed(0);
+        boost::hash_combine(seed, pt.x);
+        boost::hash_combine(seed, pt.y);
+        return seed;
+    }
+    GG_API inline std::size_t hash_value(Rect const& r) {
+        std::size_t seed(0);
+        boost::hash_combine(seed, r.ul);
+        boost::hash_combine(seed, r.lr);
+        return seed;
+    }
+
 } // namepace GG
 
 #endif
