@@ -88,42 +88,49 @@ public:
     /** \name Structors */ ///@{
     explicit SDLGUI(int w = 1024, int h = 768, bool calc_FPS = false, const std::string& app_name = "GG",
                     int x = SDL_WINDOWPOS_UNDEFINED, int y = SDL_WINDOWPOS_UNDEFINED, bool fullscreen = false,
-                    bool fake_mode_change = false); ///< ctor
+                    bool fake_mode_change = false);
+
     virtual ~SDLGUI();
     //@}
 
     /** \name Accessors */ ///@{
-    virtual X               AppWidth() const;
-    virtual Y               AppHeight() const;
-    virtual unsigned int    Ticks() const;
-    virtual bool            Fullscreen() const;
-    virtual bool            FakeModeChange() const;
-    virtual std::string     ClipboardText() const;
+    X AppWidth() const override;
+    Y AppHeight() const override;
+    unsigned int Ticks() const override;
+    std::string ClipboardText() const override;
+    virtual std::vector<std::string> GetSupportedResolutions() const override;
+    virtual Pt GetDefaultResolution (int display_id) const override;
+
+    virtual bool Fullscreen() const final;
+    virtual bool FakeModeChange() const final;
     //@}
 
     /** \name Mutators */ ///@{
+    void Exit(int code) override;
+    bool SetClipboardText(const std::string& text) override;
+
+    void Enter2DMode() override;
+    void Exit2DMode() override;
+
+    void HandleSystemEvents() override;
+
+    void RenderBegin() override;
+    void RenderEnd() override;
+
+    void Run() override;
+
     void            operator()();      ///< external interface to Run()
-    virtual void    Exit(int code);
 
     void            SetWindowTitle(const std::string& title);
     void            SetVideoMode(X width, Y height, bool fullscreen, bool fake_mode_change);
-    virtual bool    SetClipboardText(const std::string& text);
     //@}
 
     static SDLGUI*  GetGUI();                             ///< allows any code to access the gui framework by calling SDLGUI::GetGUI()
     GG::Key         GGKeyFromSDLKey(const SDL_Keysym& key); ///< gives the GGKey equivalent of key
 
-    virtual void    Enter2DMode();
-    virtual void    Exit2DMode();
-
-    // \override
-    virtual std::vector<std::string> GetSupportedResolutions() const;
-
-    // \override
-    virtual Pt      GetDefaultResolution (int display_id);
     static  Pt      GetDefaultResolutionStatic(int display_id);
     static int      NumVideoDisplaysStatic();
-    virtual bool    FramebuffersAvailable() const;
+    bool            FramebuffersAvailable() const;
 
     /** Returns the largest possible width if all displays are aligned horizontally.
         Ideally it reports actual desktop width using all displays.*/
@@ -135,21 +142,25 @@ protected:
     void SetAppSize(const GG::Pt& size);
 
     // these are called at the beginning of the gui's execution
-    virtual void    SDLInit();        ///< initializes SDL, FE, and SDL OpenGL functionality
-    virtual void    GLInit();         ///< allows user to specify OpenGL initialization code; called at the end of SDLInit()
-    virtual void    Initialize() = 0; ///< provides one-time gui initialization
+    /** Initializes SDL, FE, and SDL OpenGL functionality. */
+    void SDLInit();
 
-    virtual void    HandleSystemEvents();
-    virtual void    HandleNonGGEvent(const SDL_Event& event); ///< event handler for all SDL events that are not GG-related
+    /** Allows user to specify OpenGL initialization code;
+        called at the end of SDLInit(). */
+    void GLInit();
 
-    virtual void    RenderBegin();
-    virtual void    RenderEnd();
+    /** Provides one-time gui initialization. */
+    virtual void Initialize() = 0;
+
+    /** event handler for all SDL events that are not GG-related. */
+    void HandleNonGGEvent(const SDL_Event& event);
 
     // these are called at the end of the gui's execution
-    virtual void    FinalCleanup();   ///< provides one-time gui cleanup
-    virtual void    SDLQuit();        ///< cleans up SDL and (if used) FE
+    /** Provides one-time gui cleanup. */
+    void FinalCleanup();
 
-    virtual void    Run();
+    /** Cleans up SDL and (if used) FE. */
+    void SDLQuit();
 
     void            ResetFramebuffer(); ///< Resizes or deletes the framebuffer for fake fullscreen.
 
@@ -174,7 +185,11 @@ private:
     SDL_Window*     m_window; ///< The sdl window
     SDL_GLContext   m_gl_context; //< The OpenGL context
     bool            m_done; //< Set true true when we should exit.
-    boost::scoped_ptr<Framebuffer>  m_framebuffer; //< virtual screen for fake fullscreen. Null if m_fake_mode_change == false
+
+    /** Virtual screen for fake fullscreen.  Equals nullptr ifi
+        m_fake_mode_change == false. */
+    std::unique_ptr<Framebuffer> m_framebuffer;
+
     std::map<SDL_Keycode, Key>      m_key_map; //< a mapping from sdl keycodes to GiGi keys
 };
 
