@@ -81,14 +81,14 @@ public:
     /** \name Structors */ ///@{
     /** Ctor that does not required height. Height is determined from the font
         and point size used.*/
-    Spin(T value, T step, T min, T max, bool edits, const boost::shared_ptr<Font>& font, Clr color,
+    Spin(T value, T step, T min, T max, bool edits, const std::shared_ptr<Font>& font, Clr color,
          Clr text_color = CLR_BLACK);
 
-    ~Spin(); // dtor
+    ~Spin();
     //@}
 
     /** \name Accessors */ ///@{
-    virtual Pt MinUsableSize() const;
+    Pt MinUsableSize() const override;
 
     T      Value() const;              ///< returns the current value of the control's text
     T      StepSize() const;           ///< returns the step size of the control
@@ -107,12 +107,13 @@ public:
     //@}
 
     /** \name Mutators */ ///@{
-    virtual void Render();
+    void Render() override;
 
-    virtual void SizeMove(const Pt& ul, const Pt& lr);
+    void SizeMove(const Pt& ul, const Pt& lr) override;
 
-    virtual void Disable(bool b = true);
-    virtual void SetColor(Clr c);
+    void Disable(bool b = true) override;
+
+    void SetColor(Clr c) override;
 
     void Incr();  ///< increments the value of the control's text by StepSize(), up to at most MaxValue()
     void Decr();  ///< decrements the value of the control's text by StepSize(), down to at least MinValue()
@@ -147,15 +148,16 @@ protected:
     //@}
 
     /** \name Mutators */ ///@{
-    virtual void KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_keys);
-    virtual void MouseWheel(const Pt& pt, int move, Flags<ModKey> mod_keys);
+    void KeyPress(Key key, std::uint32_t key_code_point, Flags<ModKey> mod_keys) override;
 
-    virtual bool EventFilter(Wnd* w, const WndEvent& event);
+    void MouseWheel(const Pt& pt, int move, Flags<ModKey> mod_keys) override;
+
+    bool EventFilter(Wnd* w, const WndEvent& event) override;
     //@}
 
 private:
     void ConnectSignals();
-    void Init(const boost::shared_ptr<Font>& font, Clr color, Clr text_color);
+    void Init(const std::shared_ptr<Font>& font, Clr color, Clr text_color);
     void ValueUpdated(const std::string& val_text);
     void IncrImpl(bool signal);
     void DecrImpl(bool signal);
@@ -180,7 +182,7 @@ private:
 
 // template implementations
 template<class T>
-Spin<T>::Spin(T value, T step, T min, T max, bool edits, const boost::shared_ptr<Font>& font, Clr color,
+Spin<T>::Spin(T value, T step, T min, T max, bool edits, const std::shared_ptr<Font>& font, Clr color,
               Clr text_color/* = CLR_BLACK*/) :
     Control(X0, Y0, X1, font->Height() + 2 * PIXEL_MARGIN, INTERACTIVE),
     m_value(value),
@@ -188,9 +190,9 @@ Spin<T>::Spin(T value, T step, T min, T max, bool edits, const boost::shared_ptr
     m_min_value(min),
     m_max_value(max),
     m_editable(edits),
-    m_edit(0),
-    m_up_button(0),
-    m_down_button(0),
+    m_edit(nullptr),
+    m_up_button(nullptr),
+    m_down_button(nullptr),
     m_button_width(15)
 {
     Init(font, color, text_color);
@@ -367,7 +369,7 @@ Edit* Spin<T>::GetEdit() const
 { return m_edit; }
 
 template<class T>
-void Spin<T>::KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_keys)
+void Spin<T>::KeyPress(Key key, std::uint32_t key_code_point, Flags<ModKey> mod_keys)
 {
     switch (key) {
     case GGK_HOME:
@@ -427,12 +429,12 @@ void Spin<T>::ConnectSignals()
 }
 
 template<class T>
-void Spin<T>::Init(const boost::shared_ptr<Font>& font, Clr color, Clr text_color)
+void Spin<T>::Init(const std::shared_ptr<Font>& font, Clr color, Clr text_color)
 {
-    boost::shared_ptr<StyleFactory> style = GetStyleFactory();
+    std::shared_ptr<StyleFactory> style = GetStyleFactory();
     Control::SetColor(color);
     m_edit = style->NewSpinEdit(boost::lexical_cast<std::string>(m_value), font, CLR_ZERO, text_color, CLR_ZERO);
-    boost::shared_ptr<Font> small_font = GUI::GetGUI()->GetFont(font, static_cast<int>(font->PointSize() * 0.75));
+    std::shared_ptr<Font> small_font = GUI::GetGUI()->GetFont(font, static_cast<int>(font->PointSize() * 0.75));
     m_up_button = style->NewSpinIncrButton(small_font, color);
     m_down_button = style->NewSpinDecrButton(small_font, color);
     m_edit->InstallEventFilter(this);
