@@ -42,21 +42,14 @@ class WndEvent;
 class GG_API OverlayWnd : public Wnd
 {
 public:
-    /** \name Signal Types */ ///@{
-    /** Emitted when the currently-selected Wnd has changed; the new selected
-        Wnd's index in the group is provided (this may be NO_WND if no Wnd is
-        currently selected). */
-    typedef boost::signals2::signal<void (std::size_t)> TabChangedSignalType;
-    //@}
-
     /** \name Structors */ ///@{
-    /** Basic ctor. */
     OverlayWnd(X x, Y y, X w, Y h, Flags<WndFlag> flags = NO_WND_FLAGS);
+
     ~OverlayWnd();
     //@}
 
     /** \name Accessors */ ///@{
-    virtual Pt MinUsableSize() const;
+    Pt MinUsableSize() const override;
 
     /** Returns true iff NumWnds() == 0. */
     bool Empty() const;
@@ -124,12 +117,11 @@ public:
     //@}
 
     /** \name Structors */ ///@{
-    /** Basic ctor. */
-    TabWnd(X x, Y y, X w, Y h, const boost::shared_ptr<Font>& font, Clr color, Clr text_color = CLR_BLACK);
+    TabWnd(X x, Y y, X w, Y h, const std::shared_ptr<Font>& font, Clr color, Clr text_color = CLR_BLACK);
     //@}
 
     /** \name Accessors */ ///@{
-    virtual Pt      MinUsableSize() const;
+    Pt MinUsableSize() const override;
 
     /** Returns true iff NumWnds() == 0. */
     bool            Empty() const;
@@ -212,13 +204,12 @@ public:
     //@}
 
     /** \name Structors */ ///@{
-    /** Basic ctor. */
-    TabBar(const boost::shared_ptr<Font>& font, Clr color, Clr text_color = CLR_BLACK,
+    TabBar(const std::shared_ptr<Font>& font, Clr color, Clr text_color = CLR_BLACK,
            Flags<WndFlag> flags = INTERACTIVE);
     //@}
 
     /** \name Accessors */ ///@{
-    virtual Pt      MinUsableSize() const;
+    Pt MinUsableSize() const override;
 
     /** Returns true iff NumWnds() == 0. */
     bool            Empty() const;
@@ -236,10 +227,11 @@ public:
     //@}
 
     /** \name Mutators */ ///@{
-    virtual void    MouseWheel(const Pt& pt, int move, Flags<ModKey> mod_keys);
-    virtual void    SizeMove(const Pt& ul, const Pt& lr);
-    virtual void    DoLayout();
-    virtual void    Render();
+    void MouseWheel(const Pt& pt, int move, Flags<ModKey> mod_keys) override;
+    void SizeMove(const Pt& ul, const Pt& lr) override;
+    void Render() override;
+
+    virtual void DoLayout();
 
     /** Adds a tab called \a name to the sequence of tabs in this TabBar.  \a
         name can be used later to remove the tab (\a name is not checked for
@@ -277,7 +269,7 @@ protected:
     //@}
 
     /** \name Mutators */ ///@{
-    virtual bool    EventFilter(Wnd* w, const WndEvent& event);
+    bool EventFilter(Wnd* w, const WndEvent& event) override;
 
     /** Brings the currently-selected tab button to the top within the tab
         button group. */
@@ -294,7 +286,7 @@ private:
 
     RadioButtonGroup*         m_tabs;
     std::vector<StateButton*> m_tab_buttons;
-    boost::shared_ptr<Font>   m_font;
+    std::shared_ptr<Font> m_font;
     Button*                   m_left_button;
     Button*                   m_right_button;
     Layout*                   m_left_right_button_layout;
