@@ -194,6 +194,37 @@ extern GG_API const WndFlag NO_WND_FLAGS;
     current.  Note the use of the phrase "client-area children".  This refers
     to children entirely within the client area of the window.
 
+    <h3>Frame Timing</h3>
+
+    <br>Each frame is a cycle culminating in rendering the screen.  The
+    following things happen, in order:
+      + events from the GUI and the data sources are handled
+      + PreRender() is called to update layout if required.
+      + Render() is called.
+
+    Wnd does not distinguish between events from the GUI and events from the
+    data sources.
+
+    PreRender() and Render() are only called if the window is visible.
+
+    PreRender() defers all layout changes until after all events have been
+    handled.  This improves performance by preventing event handling causing
+    multiple layout actions between calls to Render().  PreRender() allows Wnd
+    to perform any required expensive updates only once per frame.
+
+    PreRender() prevents bugs caused by early events in a frame changing the
+    layout so that later events trigger on a layout different from the layout
+    visible to the player.  This is a change from the original GG behavior.
+    Core parts of GG will continue to immediately update layout until
+    peripheral parts are updated to expect deferred updates.
+
+    The default implementation of PreRender() only clears the flag set by RequirePreRender().
+    For a class to defer its layout to the prerender phase it needs to override PreRender() and
+    implement its layout changes in PreRender().
+
+    Legacy GG did not have a prerender phase and all layout changes were immediate.  Any Wnd class
+    that does not override PreRender() will update layout immediately when executing mutating functions.
+
     <h3>Browse Info</h3>
 
     <br>Browse info is a non-interactive informational window that pops up
@@ -240,7 +271,7 @@ public:
 
         /** The BrowseInfoWnd used to display the browse info for this
             mode. */
-        boost::shared_ptr<BrowseInfoWnd> wnd;
+        std::shared_ptr<BrowseInfoWnd> wnd;
 
         /** The text to display in the BrowseInfoWnd shown for this mode. */
         std::string                      text;
@@ -272,7 +303,7 @@ public:
     };
 
     /** \name Structors */ ///@{
-    virtual ~Wnd(); ///< Virtual dtor.
+    virtual ~Wnd();
     //@}
 
     /** \name Accessors */ ///@{
@@ -311,6 +342,9 @@ public:
     /** Returns true iff this Wnd will be rendered if it is registered. */
     bool Visible() const;
 
+    /** Returns true if this Wnd will be pre-rendered. */
+    bool PreRenderRequired() const;
+
     /** Returns the name of this Wnd.  This name is not used by GG in any way;
         it only exists for user convenience. */
      const std::string& Name() const;
@@ -345,10 +379,6 @@ public:
     X Width() const;  ///< Returns width of window.
     Y Height() const; ///< Returns height of window.
 
-    /** Returns the position of this window in the z-order (root (non-child)
-        windows only). */
-    int ZOrder() const;
-
     /** Returns a \a Pt packed with width in \a x and height in \a y. */
     Pt Size() const;
 
@@ -427,10 +457,13 @@ public:
 
     /** Returns the currently-installed style factory if none exists, or the
         GUI-wide one otherwise. */
-    const boost::shared_ptr<StyleFactory>& GetStyleFactory() const;
+    const std::shared_ptr<StyleFactory>& GetStyleFactory() const;
 
     /** Returns the region under point \a pt. */
     virtual WndRegion WindowRegion(const Pt& pt) const;
+
+    /** Adjusts \p ul and \p lr to correct for minsize and maxsize.*/
+    void ClampRectWithMinAndMaxSize(Pt& ul, Pt& lr) const;
     //@}
 
     /** \name Mutators */ ///@{
@@ -508,10 +541,10 @@ public:
     void Resize(const Pt& sz);
 
     /** Sets the minimum allowable size of window \a pt. */
-    void SetMinSize(const Pt& sz);
+    virtual void SetMinSize(const Pt& sz);
 
     /** Sets the maximum allowable size of window \a pt. */
-    void SetMaxSize(const Pt& sz);
+    virtual void SetMaxSize(const Pt& sz);
 
     /** Places \a wnd in child ptr list, sets's child's \a m_parent member to
         \a this. */
@@ -583,6 +616,21 @@ public:
         If no layout exists for the window, this has no effect. */
     void SetLayoutCellMargin(unsigned int margin);
 
+    /** Update Wnd prior to Render().
+
+        PreRender() is called before Render() if RequirePreRender() was called.
+        The default PreRender() resets the flag from RequirePreRender().
+        Wnd::PreRender() should be called in any overrides to reset
+        RequirePreRender().
+
+        In the GUI processing loop the PreRender() of child windows whose
+        RequirePreRender() flag is set will have been called before their
+        parent PreRender(). */
+    virtual void PreRender();
+
+    /** Require that PreRender() be called to update layout before the next Render(). */
+    virtual void RequirePreRender();
+
     /** Draws this Wnd.  Note that Wnds being dragged for a drag-and-drop
         operation are rendered twice -- once in-place as normal, once in the
         location of the drag operation, attached to the cursor.  Such Wnds may
@@ -597,7 +645,7 @@ public:
     virtual bool Run();
 
     /** Ends the current execution of Run(), if any. */
-    void EndRun();
+    virtual void EndRun();
 
     /** Sets the time cutoff (in milliseconds) for a browse info mode.  If \a
         mode is not less than the current number of modes, extra modes will be
@@ -609,7 +657,7 @@ public:
     /** Sets the Wnd that is used to show browse info about this Wnd in the
         browse info mode \a mode.  \throw std::out_of_range May throw
         std::out_of_range if \a mode is not a valid browse mode. */
-    void SetBrowseInfoWnd(const boost::shared_ptr<BrowseInfoWnd>& wnd, std::size_t mode = 0);
+    void SetBrowseInfoWnd(const std::shared_ptr<BrowseInfoWnd>& wnd, std::size_t mode = 0);
 
     /** Removes the Wnd that is used to show browse info about this Wnd in the
         browse info mode \a mode (but does nothing to the mode itself).
@@ -633,7 +681,7 @@ public:
     void SetBrowseModes(const std::vector<BrowseInfoMode>& modes);
 
     /** Sets the currently-installed style factory. */
-    void SetStyleFactory(const boost::shared_ptr<StyleFactory>& factory);
+    void SetStyleFactory(const std::shared_ptr<StyleFactory>& factory);
     //@}
 
 
@@ -648,11 +696,11 @@ public:
     /** Returns the single BrowseInfoWnd to place in the browse modes during
         Wnd construction.  This returns a TextBoxBrowseInfoWnd with a default
         parameterization. */
-    static const boost::shared_ptr<BrowseInfoWnd>& DefaultBrowseInfoWnd();
+    static const std::shared_ptr<BrowseInfoWnd>& DefaultBrowseInfoWnd();
 
     /** Sets the single BrowseInfoWnd to place in the browse modes during Wnd
         construction. */
-    static void SetDefaultBrowseInfoWnd(const boost::shared_ptr<BrowseInfoWnd>& browse_info_wnd);
+    static void SetDefaultBrowseInfoWnd(const std::shared_ptr<BrowseInfoWnd>& browse_info_wnd);
 
     /** \name Exceptions */ ///@{
     /** The base class for Wnd exceptions. */
@@ -692,7 +740,7 @@ protected:
     };
 
     /** \name Structors */ ///@{
-    Wnd(); ///< Default ctor.
+    Wnd();
 
     /** Ctor that allows a size and position to be specified, as well as
         creation flags. */
@@ -831,14 +879,14 @@ protected:
         KeyPress(), not KeyRelease(); in fact, by default no Wnd class does
         anything at all on a KeyRelease event.  \note \a key_code_point will
         be zero if Unicode support is unavailable. */
-    virtual void KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_keys);
+    virtual void KeyPress(Key key, std::uint32_t key_code_point, Flags<ModKey> mod_keys);
 
     /** Respond to up-keystrokes (focus window only).  A window may receive
         KeyRelease() messages passed up to it from its children.  For
         instance, Control-derived classes pass KeyRelease() messages to their
         Parent() windows by default.  \note \a key_code_point will be zero if
         Unicode support is unavailable. */
-    virtual void KeyRelease(Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_keys);
+    virtual void KeyRelease(Key key, std::uint32_t key_code_point, Flags<ModKey> mod_keys);
 
     /** Respond to text input regardless of the method. Focus window only.
         A window may receive TextInput() messages passed up to it from its
@@ -900,8 +948,8 @@ private:
     Wnd*              m_parent;        ///< Ptr to this window's parent; may be 0
     std::string       m_name;          ///< A user-significant name for this Wnd
     std::list<Wnd*>   m_children;      ///< List of ptrs to child windows kept in order of decreasing area
-    int               m_zorder;        ///< Where this window is in the z-order (root (non-child) windows only)
     bool              m_visible;
+    bool              m_needs_prerender; ///< Indicates if Wnd needs a PreRender();
     std::string       m_drag_drop_data_type; ///< The type of drag-and-drop data this Wnd represents, if any. If empty/blank, indicates that this Wnd cannot be drag-dropped.
     ChildClippingMode m_child_clipping_mode;
     bool              m_non_client_child;
@@ -920,8 +968,8 @@ private:
     std::vector<BrowseInfoMode>
                       m_browse_modes;      ///< The browse info modes for this window
 
-    boost::shared_ptr<StyleFactory>
-                      m_style_factory;     ///< The style factory to use when creating dialogs or child controls
+    /** The style factory to use when creating dialogs or child controls. */
+    std::shared_ptr<StyleFactory> m_style_factory;
 
     /** Flags supplied at window creation for clickability, dragability,
         resizability, etc. */
@@ -933,12 +981,12 @@ private:
 
     /** The default BrowseInfoWmd to set for the first (and only) value in
         m_browse_mode_times during Wnd contruction */
-    static boost::shared_ptr<BrowseInfoWnd> s_default_browse_info_wnd;
+    static std::shared_ptr<BrowseInfoWnd> s_default_browse_info_wnd;
 
-    friend class GUI;   ///< GUI needs access to \a m_zorder, m_children, etc.
+    friend class GUI;   ///< GUI needs access to \a m_children, etc.
     friend struct GUIImpl;
     friend class Timer; ///< Timer needs to be able to call HandleEvent
-    friend class ZList; ///< ZList needs access to \a m_zorder in order to order windows
+    friend class ZList; ///< ZList needs access to \a m_children in order to order windows
 };
 
 } // namespace GG
