@@ -66,7 +66,7 @@ public:
         indicates whether multiple file selections are allowed.  \throw
         GG::FileDlg::BadInitialDirectory Throws when \a directory is
         invalid. */
-    FileDlg(const std::string& directory, const std::string& filename, bool save, bool multi, const boost::shared_ptr<Font>& font,
+    FileDlg(const std::string& directory, const std::string& filename, bool save, bool multi, const std::shared_ptr<Font>& font,
             Clr color, Clr border_color, Clr text_color = CLR_BLACK);
     //@}
 
@@ -86,8 +86,8 @@ public:
     //@}
 
     /** \name Mutators */ ///@{
-    virtual void Render();
-    virtual void KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_keys);
+    void Render() override;
+    void KeyPress(Key key, std::uint32_t key_code_point, Flags<ModKey> mod_keys) override;
 
     /** Set this to true if this FileDlg should select directories instead of
         files.  Note that this will only have an effect in file-open mode. */
@@ -112,7 +112,7 @@ public:
         that is either a Targa or a PNG file.  Note that an empty filter is
         considered to match all files, so ("All Files", "") is perfectly
         correct. */
-    void SetFileFilters(const std::vector<std::pair<std::string, std::string> >& filters);
+    void SetFileFilters(const std::vector<std::pair<std::string, std::string>>& filters);
     //@}
 
     /** Returns the current directory (the one that will be used by default on
@@ -157,11 +157,10 @@ private:
     Clr              m_color;
     Clr              m_border_color;
     Clr              m_text_color;
-    boost::shared_ptr<Font>
-                     m_font;
+    std::shared_ptr<Font> m_font;
 
     bool             m_save;
-    std::vector<std::pair<std::string, std::string> > 
+    std::vector<std::pair<std::string, std::string>>
                      m_file_filters;
     std::set<std::string>
                      m_result;
