@@ -29,8 +29,6 @@
 #include <GG/StyleFactory.h>
 #include <GG/WndEvent.h>
 
-#include <boost/lexical_cast.hpp>
-
 
 using namespace GG;
 
@@ -51,7 +49,7 @@ namespace {
 ////////////////////////////////////////////////
 // GG::Button
 ////////////////////////////////////////////////
-Button::Button(const std::string& str, const boost::shared_ptr<Font>& font, Clr color,
+Button::Button(const std::string& str, const std::shared_ptr<Font>& font, Clr color,
                Clr text_color/* = CLR_BLACK*/, Flags<WndFlag> flags/* = INTERACTIVE*/) :
     Control(X0, Y0, X1, Y1, flags),
     m_label(new TextControl(X0, Y0, X1, Y1, str, font, text_color, FORMAT_NONE, NO_WND_FLAGS)),
@@ -181,6 +179,12 @@ void Button::RClick(const Pt& pt, Flags<ModKey> mod_keys)
     }
 }
 
+void Button::MouseEnter(const Pt& pt, Flags<ModKey> mod_keys)
+{
+    if (!Disabled())
+        m_state = BN_ROLLOVER;
+}
+
 void Button::MouseHere(const Pt& pt, Flags<ModKey> mod_keys)
 {
     if (!Disabled())
@@ -255,7 +259,108 @@ void Button::RenderDefault()
 
 
 ////////////////////////////////////////////////
-// GG::BeveledCheckBoxRepresenter
+// GG::StateButton
+////////////////////////////////////////////////
+StateButton::StateButton(const std::string& str, const std::shared_ptr<Font>& font, Flags<TextFormat> format,
+                         Clr color, std::shared_ptr<StateButtonRepresenter> representer, Clr text_color/* = CLR_BLACK*/) :
+    Control(X0, Y0, X1, Y1, INTERACTIVE),
+    m_representer(representer),
+    m_label(new TextControl(X0, Y0, X1, Y1, str, font, text_color, format, NO_WND_FLAGS)),
+    m_state(BN_UNPRESSED),
+    m_checked(false)
+{
+    m_color = color;
+    AttachChild(m_label);
+    m_label->Hide();
+
+    if (INSTRUMENT_ALL_SIGNALS)
+        Connect(CheckedSignal, &CheckedEcho);
+}
+
+Pt StateButton::MinUsableSize() const
+{
+    if (m_representer)
+        return m_representer->MinUsableSize(*this);
+
+    return Pt();
+}
+
+StateButton::ButtonState StateButton::State() const
+{ return m_state; }
+
+const std::string& StateButton::Text() const
+{ return m_label->Text(); }
+
+bool StateButton::Checked() const
+{ return m_checked; }
+
+void StateButton::Render()
+{
+    if (m_representer)
+        m_representer->Render(*this);
+}
+
+void StateButton::Show(bool children/* = true*/)
+{ Wnd::Show(false); }
+
+void StateButton::LButtonDown(const Pt& pt, Flags<ModKey> mod_keys)
+{ SetState(BN_PRESSED); }
+
+void StateButton::LDrag(const Pt& pt, const Pt& move, Flags<ModKey> mod_keys)
+{
+    SetState(BN_PRESSED);
+    Wnd::LDrag(pt, move, mod_keys);
+}
+
+void StateButton::LButtonUp(const Pt& pt, Flags<ModKey> mod_keys)
+{ SetState(BN_UNPRESSED); }
+
+void StateButton::LClick(const Pt& pt, Flags<ModKey> mod_keys)
+{
+    if (!Disabled()) {
+        SetCheck(!m_checked);
+        if (m_representer)
+            m_representer->OnChecked(m_checked);
+        CheckedSignal(m_checked);
+    }
+}
+
+void StateButton::MouseHere(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys)
+{ SetState(BN_ROLLOVER); }
+
+void StateButton::MouseLeave()
+{ SetState(BN_UNPRESSED); }
+
+void StateButton::SetState(ButtonState next_state) {
+    if (!Disabled() && next_state != m_state) {
+        ButtonState prev_state = m_state;
+        m_state = next_state;
+        if (m_representer)
+            m_representer->OnChanged(*this, prev_state);
+    }
+}
+
+void StateButton::SizeMove(const Pt& ul, const Pt& lr)
+{
+    Control::SizeMove(ul, lr);
+    m_label->Resize(Size());
+}
+
+void StateButton::Reset()
+{ SetCheck(false); }
+
+void StateButton::SetCheck(bool b/* = true*/)
+{ m_checked = b; }
+
+void StateButton::SetTextColor(Clr c)
+{ m_label->SetTextColor(c); }
+
+TextControl* StateButton::GetLabel() const
+{ return m_label; }
+
+
+////////////////////////////////////////////////
+// GG::StateButtonRepresenter
 ////////////////////////////////////////////////
 void StateButtonRepresenter::Render(const GG::StateButton& button) const
 {}
@@ -285,7 +390,7 @@ void StateButtonRepresenter::DoLayout(const GG::StateButton& button, Pt& button_
     }
     if (format & FORMAT_BOTTOM) {      // put button at bottom, text just above
         bn_y = (h - BN_H);
-        text_ul.y = h - (BN_H * (1 + SPACING)) - (static_cast<int>(button.GetLabel()->GetLineData().size() - 1) * button.GetLabel()->GetFont()->Lineskip() + button.GetLabel()->GetFont()->Height()) + 0.5;
+        text_ul.y = h - (BN_H * (1 + SPACING)) - (std::max(0, static_cast<int>(button.GetLabel()->GetLineData().size() - 1)) * button.GetLabel()->GetFont()->Lineskip() + button.GetLabel()->GetFont()->Height()) + 0.5;
     }
 
     if (format & FORMAT_CENTER) {      // center button horizontally
@@ -312,6 +417,9 @@ void StateButtonRepresenter::DoLayout(const GG::StateButton& button, Pt& button_
     button_lr = button_ul + Pt(BN_W, BN_H);
 }
 
+void StateButtonRepresenter::OnChanged(const StateButton& button, StateButton::ButtonState previous_state) const
+{}
+
 void StateButtonRepresenter::OnChecked(bool checked) const
 {}
 
@@ -431,112 +539,10 @@ void BeveledTabRepresenter::Render(const StateButton& button) const
 }
 
 
-////////////////////////////////////////////////
-// GG::StateButton
-////////////////////////////////////////////////
-StateButton::StateButton(const std::string& str, const boost::shared_ptr<Font>& font, Flags<TextFormat> format,
-                         Clr color, boost::shared_ptr<StateButtonRepresenter> representer, Clr text_color/* = CLR_BLACK*/) :
-    Control(X0, Y0, X1, Y1, INTERACTIVE),
-    m_representer(representer),
-    m_label(new TextControl(X0, Y0, X1, Y1, str, font, text_color, format, NO_WND_FLAGS)),
-    m_checked(false),
-    m_mouseover(false)
-{
-    m_color = color;
-    AttachChild(m_label);
-    m_label->Hide();
-
-    if (INSTRUMENT_ALL_SIGNALS)
-        Connect(CheckedSignal, &CheckedEcho);
-}
-
-Pt StateButton::MinUsableSize() const
-{
-    if (m_representer)
-        return m_representer->MinUsableSize(*this);
-
-    return Pt();
-}
-
-const std::string& StateButton::Text() const
-{ return m_label->Text(); }
-
-bool StateButton::Checked() const
-{ return m_checked; }
-
-bool StateButton::IsMouseover() const
-{ return m_mouseover; }
-
-void StateButton::Render()
-{
-    if (m_representer)
-        m_representer->Render(*this);
-}
-
-void StateButton::Show(bool children/* = true*/)
-{ Wnd::Show(false); }
-
-void StateButton::LClick(const Pt& pt, Flags<ModKey> mod_keys)
-{
-    if (!Disabled()) {
-        SetCheck(!m_checked);
-        if (m_representer)
-            m_representer->OnChecked(m_checked);
-        CheckedSignal(m_checked);
-    }
-}
-
-void StateButton::MouseEnter(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys)
-{ m_mouseover = true; }
-
-void StateButton::MouseLeave()
-{ m_mouseover = false; }
-
-void StateButton::SizeMove(const Pt& ul, const Pt& lr)
-{
-    Control::SizeMove(ul, lr);
-    m_label->Resize(Size());
-}
-
-void StateButton::Reset()
-{ SetCheck(false); }
-
-void StateButton::SetCheck(bool b/* = true*/)
-{ m_checked = b; }
-
-void StateButton::SetColor(Clr c)
-{ Control::SetColor(c); }
-
-void StateButton::SetTextColor(Clr c)
-{ m_label->SetTextColor(c); }
-
-TextControl* StateButton::GetLabel() const
-{ return m_label; }
-
-
 ////////////////////////////////////////////////
 // GG::RadioButtonGroup
 ////////////////////////////////////////////////
-// ButtonClickedFunctor
-RadioButtonGroup::ButtonClickedFunctor::ButtonClickedFunctor(RadioButtonGroup* group, StateButton* button, std::size_t index) :
-    m_group(group),
-    m_button(button),
-    m_index(index)
-{}
-
-void RadioButtonGroup::ButtonClickedFunctor::operator()(bool checked)
-{
-    if (checked)
-        m_group->SetCheckImpl(m_index, true);
-    else
-        m_button->SetCheck(true);
-}
-
 // ButtonSlot
-RadioButtonGroup::ButtonSlot::ButtonSlot() :
-    button(0)
-{}
-
 RadioButtonGroup::ButtonSlot::ButtonSlot(StateButton* button_) :
     button(button_)
 {}
@@ -562,8 +568,8 @@ RadioButtonGroup::RadioButtonGroup(Orientation orientation) :
 Pt RadioButtonGroup::MinUsableSize() const
 {
     Pt retval;
-    for (std::size_t i = 0; i < m_button_slots.size(); ++i) {
-        Pt min_usable_size = m_button_slots[i].button->MinUsableSize();
+    for (const ButtonSlot& button_slot : m_button_slots) {
+        Pt min_usable_size = button_slot.button->MinUsableSize();
         if (m_orientation == VERTICAL) {
             retval.x = std::max(retval.x, min_usable_size.x);
             retval.y += min_usable_size.y;
@@ -741,8 +747,8 @@ void RadioButtonGroup::ExpandButtons(bool expand)
             RemoveButton(button);
         }
         m_expand_buttons = expand;
-        for (std::size_t i = 0; i < buttons.size(); ++i) {
-            AddButton(buttons[i]);
+        for (StateButton* button : buttons) {
+            AddButton(button);
         }
         SetCheck(old_checked_button);
     }
@@ -759,8 +765,8 @@ void RadioButtonGroup::ExpandButtonsProportionally(bool proportional)
             RemoveButton(button);
         }
         m_expand_buttons_proportionally = proportional;
-        for (std::size_t i = 0; i < buttons.size(); ++i) {
-            AddButton(buttons[i]);
+        for (StateButton* button : buttons) {
+            AddButton(button);
         }
         SetCheck(old_checked_button);
     }
@@ -775,9 +781,12 @@ const std::vector<RadioButtonGroup::ButtonSlot>& RadioButtonGroup::ButtonSlots()
 void RadioButtonGroup::ConnectSignals()
 {
     for (std::size_t i = 0; i < m_button_slots.size(); ++i) {
-        m_button_slots[i].connection =
-            Connect(m_button_slots[i].button->CheckedSignal,
-                    ButtonClickedFunctor(this, m_button_slots[i].button, i));
+        m_button_slots[i].connection = Connect(m_button_slots[i].button->CheckedSignal, [this, i](bool checked) {
+            if (checked)
+                this->SetCheckImpl(i, true);
+            else
+                this->m_button_slots[i].button->SetCheck(true);
+        });
     }
     SetCheck(m_checked_button);
 }
@@ -796,8 +805,8 @@ void RadioButtonGroup::SetCheckImpl(std::size_t index, bool signal)
 
 void RadioButtonGroup::Reconnect()
 {
-    for (std::size_t i = 0; i < m_button_slots.size(); ++i) {
-        m_button_slots[i].connection.disconnect();
+    for (ButtonSlot& button_slot : m_button_slots) {
+        button_slot.connection.disconnect();
     }
     ConnectSignals();
 }
