@@ -49,7 +49,7 @@ namespace {
         const CPSize m_value;
     };
 
-    Y HeightFromFont(const boost::shared_ptr<Font>& font, unsigned int pixel_margin)
+    Y HeightFromFont(const std::shared_ptr<Font>& font, unsigned int pixel_margin)
     {  return font->Height() + 2 * static_cast<int>(pixel_margin); }
 }
 
@@ -59,7 +59,7 @@ namespace {
 // static(s)
 const int Edit::PIXEL_MARGIN = 5;
 
-Edit::Edit(const std::string& str, const boost::shared_ptr<Font>& font, Clr color,
+Edit::Edit(const std::string& str, const std::shared_ptr<Font>& font, Clr color,
            Clr text_color/* = CLR_BLACK*/, Clr interior/* = CLR_ZERO*/) :
     TextControl(X0, Y0, X1, HeightFromFont(font, PIXEL_MARGIN), str, font, text_color, FORMAT_LEFT | FORMAT_IGNORETAGS, INTERACTIVE | REPEAT_KEY_PRESS),
     m_cursor_pos(CP0, CP0),
@@ -122,17 +122,15 @@ void Edit::Render()
 
     BeginScissorClipping(Pt(client_ul.x - 1, client_ul.y), client_lr);
 
-    if (GetLineData().empty())
-        return;
-    const std::vector<Font::LineData::CharData>& char_data = GetLineData()[0].char_data;
-
     X first_char_offset = FirstCharOffset();
     Y text_y_pos(ul.y + ((lr.y - ul.y) - GetFont()->Height()) / 2.0 + 0.5);
     CPSize last_visible_char = LastVisibleChar();
     const StrSize INDEX_0 = StringIndexOf(0, m_first_char_shown, GetLineData());
     const StrSize INDEX_END = StringIndexOf(0, last_visible_char, GetLineData());
 
-    if (MultiSelected()) {
+    if (!GetLineData().empty() && MultiSelected()) {
+        const std::vector<Font::LineData::CharData>& char_data = GetLineData()[0].char_data;
+
         // if one or more chars are selected, hilite, then draw the range in
         // the selected-text color
         CPSize low_cursor_pos  = std::min(CPSize(char_data.size()), std::max(CP0, std::min(m_cursor_pos.first, m_cursor_pos.second)));
@@ -226,7 +224,7 @@ void Edit::SetText(const std::string& str)
     // make sure the change in text did not make the cursor or view position invalid
     if (Text().empty() || GetLineData().empty() || GetLineData()[0].char_data.size() < m_cursor_pos.first) {
         m_first_char_shown = CP0;
-        m_cursor_pos = std::make_pair(CP0, CP0);
+        m_cursor_pos = {CP0, CP0};
     }
 
     m_recently_edited = true;
@@ -293,12 +291,12 @@ CPSize Edit::CharIndexOf(X x) const
 }
 
 X Edit::FirstCharOffset() const
-{ return (m_first_char_shown ? GetLineData()[0].char_data[Value(m_first_char_shown - 1)].extent : X0); }
+{ return (!GetLineData().empty() && m_first_char_shown ? GetLineData()[0].char_data[Value(m_first_char_shown - 1)].extent : X0); }
 
 X Edit::ScreenPosOfChar(CPSize idx) const
 {
     X first_char_offset = FirstCharOffset();
-    return ClientUpperLeft().x + ((idx ? GetLineData()[0].char_data[Value(idx - 1)].extent : X0) - first_char_offset);
+    return ClientUpperLeft().x + ((!GetLineData().empty() && idx ? GetLineData()[0].char_data[Value(idx - 1)].extent : X0) - first_char_offset);
 }
 
 CPSize Edit::LastVisibleChar() const
@@ -338,6 +336,10 @@ bool Edit::InDoubleButtonDownMode() const
 std::pair<CPSize, CPSize> Edit::DoubleButtonDownCursorPos() const
 { return m_double_click_cursor_pos; }
 
+std::vector<GG::Font::LineData>::size_type Edit::NumLines() const {
+    return std::max(std::vector<GG::Font::LineData>::size_type(0), GetLineData().size() - 1);
+}
+
 void Edit::LButtonDown(const Pt& pt, Flags<ModKey> mod_keys)
 {
     if (Disabled())
@@ -346,7 +348,8 @@ void Edit::LButtonDown(const Pt& pt, Flags<ModKey> mod_keys)
     X click_xpos = ScreenToClient(pt).x; // x coord of click within text space
     CPSize idx = CharIndexOf(click_xpos);
     //std::cout << "Edit::LButtonDown got idx: " << idx << std::endl;
-    m_cursor_pos.first = m_cursor_pos.second = idx;
+    m_cursor_pos = {idx, idx};
+
     std::pair<CPSize, CPSize> word_indices = GetDoubleButtonDownWordIndices(idx);
     //std::cout << "Edit::LButtonDown got word indices: " << word_indices.first << ", " << word_indices.second << std::endl;
     if (word_indices.first != word_indices.second)
@@ -400,7 +403,7 @@ void Edit::LButtonUp(const Pt& pt, Flags<ModKey> mod_keys)
 void Edit::LClick(const Pt& pt, Flags<ModKey> mod_keys)
 { ClearDoubleButtonDownMode(); }
 
-void Edit::KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_keys)
+void Edit::KeyPress(Key key, std::uint32_t key_code_point, Flags<ModKey> mod_keys)
 {
     if (Disabled()) {
         TextControl::KeyPress(key, key_code_point, mod_keys);
@@ -449,9 +452,11 @@ void Edit::KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_k
         } else if (0 < m_cursor_pos.second) {
             if (!ctrl_down) {
                 --m_cursor_pos.second;
-                X extent = GetLineData()[0].char_data[Value(m_cursor_pos.second)].extent;
-                while (0 < m_cursor_pos.second && extent == GetLineData()[0].char_data[Value(m_cursor_pos.second - 1)].extent)
-                    --m_cursor_pos.second;
+                if (!GetLineData().empty()) {
+                    X extent = GetLineData()[0].char_data[Value(m_cursor_pos.second)].extent;
+                    while (0 < m_cursor_pos.second && extent == GetLineData()[0].char_data[Value(m_cursor_pos.second - 1)].extent)
+                        --m_cursor_pos.second;
+                }
             } else {
                 m_cursor_pos.second = NextWordEdgeFrom(Text(), m_cursor_pos.second, false);
             }
@@ -469,9 +474,11 @@ void Edit::KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_k
 
         } else if (m_cursor_pos.second < Length()) {
             if (!ctrl_down) {
-                X extent = GetLineData()[0].char_data[Value(m_cursor_pos.second)].extent;
-                while (m_cursor_pos.second < Length() && extent == GetLineData()[0].char_data[Value(m_cursor_pos.second)].extent)
-                    ++m_cursor_pos.second;
+                if (!GetLineData().empty()) {
+                    X extent = GetLineData()[0].char_data[Value(m_cursor_pos.second)].extent;
+                    while (m_cursor_pos.second < Length() && extent == GetLineData()[0].char_data[Value(m_cursor_pos.second)].extent)
+                        ++m_cursor_pos.second;
+                }
             } else {
                 m_cursor_pos.second = NextWordEdgeFrom(Text(), m_cursor_pos.second, true);
             }
@@ -565,8 +572,8 @@ std::pair<CPSize, CPSize> Edit::GetDoubleButtonDownDragWordIndices(CPSize char_i
 {
     std::pair<CPSize, CPSize> retval(char_index, char_index);
 
-    std::set<std::pair<CPSize, CPSize> > words = GUI::GetGUI()->FindWords(Text());
-    std::set<std::pair<CPSize, CPSize> >::const_iterator it =
+    std::set<std::pair<CPSize, CPSize>> words = GUI::GetGUI()->FindWords(Text());
+    std::set<std::pair<CPSize, CPSize>>::const_iterator it =
         std::find_if(words.begin(), words.end(), InRange(char_index));
 
     if (it != words.end())
@@ -624,14 +631,14 @@ void Edit::AdjustView()
 ////////////////////////////////////////////////////////////
 // Free Functions
 ////////////////////////////////////////////////////////////
-void GG::GetTranslatedCodePoint(Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_keys,
+void GG::GetTranslatedCodePoint(Key key, std::uint32_t key_code_point, Flags<ModKey> mod_keys,
                                 std::string& translated_code_point)
 {
     // only process it if it's a valid code point or a known printable
     // key, and no significant modifiers are in use
     if (key_code_point) {
         try {
-            boost::uint32_t chars[] = { key_code_point };
+            std::uint32_t chars[] = { key_code_point };
             utf8::utf32to8(chars, chars + 1, std::back_inserter(translated_code_point));
         } catch (const utf8::invalid_code_point&) {
             translated_code_point.clear();
@@ -646,7 +653,7 @@ void GG::GetTranslatedCodePoint(Key key, boost::uint32_t key_code_point, Flags<M
 }
 
 CPSize GG::NextWordEdgeFrom(const std::string& text, CPSize from_position, bool search_right) {
-    std::set<std::pair<CPSize, CPSize> > words = GUI::GetGUI()->FindWords(text);
+    std::set<std::pair<CPSize, CPSize>> words = GUI::GetGUI()->FindWords(text);
     CPSize retval = CP0;
 
     if (!search_right) {
@@ -654,33 +661,31 @@ CPSize GG::NextWordEdgeFrom(const std::string& text, CPSize from_position, bool
 
         // start with the leftmost word, traverse words to the right
         // until past the reference point
-        for (std::set<std::pair<CPSize, CPSize> >::const_iterator it = words.begin();
-            it != words.end(); ++it)
-        {
-            if (it->first > from_position) {
+        for (const std::pair<CPSize, CPSize>& word_range : words) {
+            if (word_range.first > from_position) {
                 // found word is after of the position. can stop
                 // searching and use whatever the last found word's position was
                 break;
 
-            } else if (it->first < from_position && it->second >= from_position) {
+            } else if (word_range.first < from_position && word_range.second >= from_position) {
                 // found word starting before and ending at/after the position. can
                 // stop searching and use the start of the found word.
-                retval = it->first;
+                retval = word_range.first;
                 break;
 
-            } else if (it->second < from_position) {
+            } else if (word_range.second < from_position) {
                 // found word ending before the position. can use the start
                 // or end of the found word...
-                if (it->second < from_position - 1) {
+                if (word_range.second < from_position - 1) {
                     // there is a gap between the end of the word and the search
                     // reference position. use one past the end of the word
-                    retval = it->second + 1;
+                    retval = word_range.second + 1;
                     // don't break, as there might be later words that are closer to
                     // the search reference position
                 } else {
                     // the end of the word is immediately before the search
                     // reference position. use the start of the word.
-                    retval = it->first;
+                    retval = word_range.first;
                     // can stop searching since the word is right next to the
                     // search reference position
                     break;
@@ -696,7 +701,7 @@ CPSize GG::NextWordEdgeFrom(const std::string& text, CPSize from_position, bool
 
         // start and the rightmost end, traverse the words leftwards
         // until past the reference point
-        for (std::set<std::pair<CPSize, CPSize> >::const_reverse_iterator rit = words.rbegin();
+        for (std::set<std::pair<CPSize, CPSize>>::const_reverse_iterator rit = words.rbegin();
             rit != words.rend(); ++rit)
         {
             if (rit->second < from_position) {
