@@ -41,17 +41,17 @@
 #  pragma GCC diagnostic push
 #  pragma GCC diagnostic ignored "-Wunused-local-typedefs"
 # endif
-# include "GIL/extension/io/png_io.hpp"
+# include "gilext/io/png_io.hpp"
 # if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ > 7)
 #  pragma GCC diagnostic pop
 # endif
 #endif
 
+#include <boost/algorithm/string/predicate.hpp>
 #include <boost/format.hpp>
-#include <boost/thread.hpp>
-#include <boost/algorithm/string.hpp>
 #include <boost/xpressive/xpressive.hpp>
 
+#include <thread>
 #include <cassert>
 #include <iostream>
 #include <fstream>
@@ -151,8 +151,11 @@ struct GG::GUIImpl
     void HandleMouseDrag(         unsigned int mouse_button, const GG::Pt& pos, int curr_ticks);
     void HandleMouseButtonRelease(unsigned int mouse_button, const GG::Pt& pos, int curr_ticks);
     void HandleIdle(             Flags<ModKey> mod_keys, const GG::Pt& pos, int curr_ticks);
-    void HandleKeyPress(         Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_keys, int curr_ticks);
-    void HandleKeyRelease(       Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_keys, int curr_ticks);
+
+    void HandleKeyPress(Key key, std::uint32_t key_code_point, Flags<ModKey> mod_keys, int curr_ticks);
+
+    void HandleKeyRelease(Key key, std::uint32_t key_code_point, Flags<ModKey> mod_keys, int curr_ticks);
+
     void HandleTextInput(        const std::string* text);
     void HandleMouseMove(        Flags<ModKey> mod_keys, const GG::Pt& pos, const Pt& rel, int curr_ticks);
     void HandleMouseWheel(       Flags<ModKey> mod_keys, const GG::Pt& pos, const Pt& rel, int curr_ticks);
@@ -164,7 +167,7 @@ struct GG::GUIImpl
 
     ZList        m_zlist;                 // object that keeps the GUI windows in the correct depth ordering
     Wnd*         m_focus_wnd;             // GUI window that currently has the input focus (this is the base level focus window, used when no modal windows are active)
-    std::list<std::pair<Wnd*, Wnd*> >
+    std::list<std::pair<Wnd*, Wnd*>>
                  m_modal_wnds;            // modal GUI windows, and the window with focus for that modality (only the one in back is active, simulating a stack but allowing traversal of the list)
     bool         m_allow_modal_accelerator_signals; // iff true: keyboard accelerator signals will be output while modal window(s) is open
 
@@ -176,8 +179,9 @@ struct GG::GUIImpl
     int          m_key_press_repeat_delay;          // see note above GUI class definition
     int          m_key_press_repeat_interval;
     int          m_last_key_press_repeat_time;      // last time of a simulated key press message
-    std::pair<Key, boost::uint32_t>
-                 m_last_pressed_key_code_point;
+
+    std::pair<Key, std::uint32_t> m_last_pressed_key_code_point;
+
     int          m_prev_key_press_time;             // the time of the most recent key press
 
     int          m_mouse_button_down_repeat_delay;      // see note above GUI class definition
@@ -202,8 +206,9 @@ struct GG::GUIImpl
     Pt           m_wnd_resize_offset;     // offset from the cursor of either the upper-left or lower-right corner of the GUI window currently being resized
     WndRegion    m_wnd_region;            // window region currently being dragged or clicked; for non-frame windows, this will always be WR_NONE
 
-    boost::shared_ptr<BrowseInfoWnd>
-                 m_browse_info_wnd;       // the current browse info window, if any
+    /** The current browse info window, if any. */
+    std::shared_ptr<BrowseInfoWnd> m_browse_info_wnd;
+
     int          m_browse_info_mode;      // the current browse info mode (only valid if browse_info_wnd is non-null)
     Wnd*         m_browse_target;         // the current browse info target
 
@@ -213,11 +218,11 @@ struct GG::GUIImpl
     std::map<const Wnd*, bool>
                  m_drag_drop_wnds_acceptable; // the Wnds being dragged and dropped, and whether they are acceptable for dropping over their current target.
 
-    std::set<std::pair<Key, Flags<ModKey> > >
+    std::set<std::pair<Key, Flags<ModKey>>>
                  m_accelerators;          // the keyboard accelerators
 
-    std::map<std::pair<Key, Flags<ModKey> >, boost::shared_ptr<GUI::AcceleratorSignalType> >
-                 m_accelerator_sigs;      // the signals emitted by the keyboard accelerators
+    /** The signals emitted by the keyboard accelerators. */
+    std::map<std::pair<Key, Flags<ModKey>>, std::shared_ptr<GUI::AcceleratorSignalType>> m_accelerator_sigs;
 
     bool         m_mouse_lr_swap;         // treat left and right mouse events as each other
     std::map<Key, Key>
@@ -234,9 +239,9 @@ struct GG::GUIImpl
     int          m_double_click_start_time;// the time from which we started measuring double_click_time, in ms
     int          m_double_click_time;     // time elapsed since last click, in ms
 
-    boost::shared_ptr<StyleFactory> m_style_factory;
+    std::shared_ptr<StyleFactory> m_style_factory;
     bool                            m_render_cursor;
-    boost::shared_ptr<Cursor>       m_cursor;
+    std::shared_ptr<Cursor> m_cursor;
 
     std::set<Timer*>  m_timers;
 
@@ -247,7 +252,7 @@ struct GG::GUIImpl
 };
 
 GUIImpl::GUIImpl() :
-    m_focus_wnd(0),
+    m_focus_wnd(nullptr),
     m_allow_modal_accelerator_signals(false),
     m_mouse_pos(X(-1000), Y(-1000)),
     m_mouse_rel(X(0), Y(0)),
@@ -255,7 +260,7 @@ GUIImpl::GUIImpl() :
     m_key_press_repeat_delay(250),
     m_key_press_repeat_interval(66),
     m_last_key_press_repeat_time(0),
-    m_last_pressed_key_code_point(std::make_pair(GGK_UNKNOWN, 0u)),
+    m_last_pressed_key_code_point{GGK_UNKNOWN, 0u},
     m_prev_key_press_time(-1),
     m_mouse_button_down_repeat_delay(250),
     m_mouse_button_down_repeat_interval(66),
@@ -264,35 +269,35 @@ GUIImpl::GUIImpl() :
     m_min_drag_time(250),
     m_min_drag_distance(5),
     m_prev_mouse_button_press_time(-1),
-    m_prev_wnd_under_cursor(0),
+    m_prev_wnd_under_cursor(nullptr),
     m_prev_wnd_under_cursor_time(-1),
-    m_curr_wnd_under_cursor(0),
+    m_curr_wnd_under_cursor(nullptr),
     m_drag_wnds(),
     m_curr_drag_wnd_dragged(false),
-    m_curr_drag_wnd(0),
-    m_curr_drag_drop_here_wnd(0),
+    m_curr_drag_wnd(nullptr),
+    m_curr_drag_drop_here_wnd(nullptr),
     m_wnd_region(WR_NONE),
     m_browse_info_mode(0),
-    m_browse_target(0),
-    m_drag_drop_originating_wnd(0),
+    m_browse_target(nullptr),
+    m_drag_drop_originating_wnd(nullptr),
     m_mouse_lr_swap(false),
     m_delta_t(0),
     m_rendering_drag_drop_wnds(false),
     m_FPS(-1.0),
     m_calc_FPS(false),
     m_max_FPS(0.0),
-    m_double_click_wnd(0),
+    m_double_click_wnd(nullptr),
     m_double_click_button(0),
     m_double_click_start_time(-1),
     m_double_click_time(-1),
     m_style_factory(new StyleFactory()),
     m_render_cursor(false),
     m_cursor(),
-    m_save_as_png_wnd(0),
+    m_save_as_png_wnd(nullptr),
     m_clipboard_text()
 {
+    m_drag_wnds[0] = m_drag_wnds[1] = m_drag_wnds[2] = nullptr;
     m_mouse_button_state[0] = m_mouse_button_state[1] = m_mouse_button_state[2] = false;
-    m_drag_wnds[0] = m_drag_wnds[1] = m_drag_wnds[2] = 0;
 }
 
 void GUIImpl::HandleMouseButtonPress(unsigned int mouse_button, const Pt& pos, int curr_ticks)
@@ -303,7 +308,7 @@ void GUIImpl::HandleMouseButtonPress(unsigned int mouse_button, const Pt& pos, i
     m_wnd_drag_offset = Pt();
     m_prev_mouse_button_press_time = 0;
     m_browse_info_wnd.reset();
-    m_browse_target = 0;
+    m_browse_target = nullptr;
     m_prev_wnd_under_cursor_time = curr_ticks;
     m_prev_mouse_button_press_time = curr_ticks;
     m_prev_mouse_button_press_pos = pos;
@@ -316,7 +321,7 @@ void GUIImpl::HandleMouseButtonPress(unsigned int mouse_button, const Pt& pos, i
     }
 
     // if this window is not a disabled Control window, it becomes the focus window
-    Control* control = 0;
+    Control* control = nullptr;
     if (m_drag_wnds[mouse_button] && (!(control = dynamic_cast<Control*>(m_drag_wnds[mouse_button])) || !control->Disabled()))
         GUI::s_gui->SetFocusWnd(m_drag_wnds[mouse_button]);
 
@@ -393,9 +398,9 @@ void GUIImpl::HandleMouseDrag(unsigned int mouse_button, const Pt& pos, int curr
         }
 
         // notify wnd under cursor of presence of drag-and-drop wnd(s)
-        if (m_curr_drag_wnd_dragged &&
-            dragged_wnd->DragDropDataType() != "" &&
-            mouse_button == 0 ||
+        if ((m_curr_drag_wnd_dragged &&
+             (dragged_wnd->DragDropDataType() != "") &&
+             (mouse_button == 0)) ||
             !m_drag_drop_wnds.empty())
         {
             std::set<Wnd*> ignores;
@@ -472,7 +477,7 @@ void GUIImpl::HandleMouseButtonRelease(unsigned int mouse_button, const GG::Pt&
     m_last_mouse_button_down_repeat_time = 0;
     m_prev_wnd_drag_position = Pt();
     m_browse_info_wnd.reset();
-    m_browse_target = 0;
+    m_browse_target = nullptr;
     m_prev_wnd_under_cursor_time = curr_ticks;
 
     Wnd* click_wnd = m_drag_wnds[mouse_button];
@@ -483,10 +488,10 @@ void GUIImpl::HandleMouseButtonRelease(unsigned int mouse_button, const GG::Pt&
 
     bool in_drag_drop =
         !m_drag_drop_wnds.empty() ||
-        m_curr_drag_wnd_dragged && click_wnd && click_wnd->DragDropDataType() != "" && mouse_button == 0;
+        (m_curr_drag_wnd_dragged && click_wnd && (click_wnd->DragDropDataType() != "") && (mouse_button == 0));
 
     m_mouse_button_state[mouse_button] = false;
-    m_drag_wnds[mouse_button] = 0; // if the mouse button is released, stop the tracking the drag window
+    m_drag_wnds[mouse_button] = nullptr; // if the mouse button is released, stop the tracking the drag window
     m_wnd_region = WR_NONE;        // and clear this, just in case
 
     if (!in_drag_drop && click_wnd && m_curr_wnd_under_cursor == click_wnd) {
@@ -498,7 +503,7 @@ void GUIImpl::HandleMouseButtonRelease(unsigned int mouse_button, const GG::Pt&
             // within the time limit, so it's a double-click, not a click
             m_double_click_button == mouse_button)
         {
-            m_double_click_wnd = 0;
+            m_double_click_wnd = nullptr;
             m_double_click_start_time = -1;
             m_double_click_time = -1;
             click_wnd->HandleEvent(WndEvent(ButtonEvent(WndEvent::LDoubleClick, mouse_button), pos, m_mod_keys));
@@ -506,7 +511,7 @@ void GUIImpl::HandleMouseButtonRelease(unsigned int mouse_button, const GG::Pt&
         } else {
             // just a single click
             if (m_double_click_time > 0) {
-                m_double_click_wnd = 0;
+                m_double_click_wnd = nullptr;
                 m_double_click_start_time = -1;
                 m_double_click_time = -1;
             } else {
@@ -520,7 +525,7 @@ void GUIImpl::HandleMouseButtonRelease(unsigned int mouse_button, const GG::Pt&
 
     } else {
         // drag-dropping
-        m_double_click_wnd = 0;
+        m_double_click_wnd = nullptr;
         m_double_click_time = -1;
         if (click_wnd)
             click_wnd->HandleEvent(WndEvent(ButtonEvent(WndEvent::LButtonUp, mouse_button), pos, m_mod_keys));
@@ -539,7 +544,7 @@ void GUIImpl::HandleMouseButtonRelease(unsigned int mouse_button, const GG::Pt&
                     // prep / handle end of drag-drop
                     m_drag_drop_originating_wnd = click_wnd->Parent();
                     m_curr_wnd_under_cursor->HandleEvent(WndEvent(WndEvent::DragDropLeave));
-                    m_curr_drag_drop_here_wnd = 0;
+                    m_curr_drag_drop_here_wnd = nullptr;
 
                     // put dragged Wnd into container depending on whether it is accepted by the drop target
                     std::vector<Wnd*> accepted_wnds;
@@ -570,18 +575,16 @@ void GUIImpl::HandleMouseButtonRelease(unsigned int mouse_button, const GG::Pt&
                 // prep / handle end of drag-drop
                 m_drag_drop_originating_wnd = click_wnd->Parent();
                 m_curr_wnd_under_cursor->HandleEvent(WndEvent(WndEvent::DragDropLeave));
-                m_curr_drag_drop_here_wnd = 0;
+                m_curr_drag_drop_here_wnd = nullptr;
 
                 // put dragged Wnds into containers depending on whether they were accepted by the drop target
                 std::vector<Wnd*> accepted_wnds;
                 std::vector<const Wnd*> unaccepted_wnds;
-                for (std::map<const Wnd*, bool>::iterator it = m_drag_drop_wnds_acceptable.begin();
-                     it != m_drag_drop_wnds_acceptable.end(); ++it)
-                {
-                    if (it->second)
-                        accepted_wnds.push_back(const_cast<Wnd*>(it->first));
+                for (std::map<const Wnd*, bool>::value_type& drop_wnd : m_drag_drop_wnds_acceptable) {
+                    if (drop_wnd.second)
+                        accepted_wnds.push_back(const_cast<Wnd*>(drop_wnd.first));
                     else
-                        unaccepted_wnds.push_back(it->first);
+                        unaccepted_wnds.push_back(drop_wnd.first);
                 }
                 // if dragged Wnds came from somehwere, inform originating
                  // Wnd its children are or are not being dragged away
@@ -596,12 +599,12 @@ void GUIImpl::HandleMouseButtonRelease(unsigned int mouse_button, const GG::Pt&
             }
         }
     }
-    m_drag_drop_originating_wnd = 0;
+    m_drag_drop_originating_wnd = nullptr;
     m_drag_drop_wnds.clear();
     m_drag_drop_wnds_acceptable.clear();
     m_prev_wnd_under_cursor = m_curr_wnd_under_cursor; // update this for the next time around
     m_curr_drag_wnd_dragged = false;
-    m_curr_drag_wnd = 0;
+    m_curr_drag_wnd = nullptr;
 }
 
 void GUIImpl::HandleIdle(Flags<ModKey> mod_keys, const GG::Pt& pos, int curr_ticks)
@@ -646,13 +649,13 @@ void GUIImpl::HandleIdle(Flags<ModKey> mod_keys, const GG::Pt& pos, int curr_tic
     }
 }
 
-void GUIImpl::HandleKeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_keys, int curr_ticks)
+void GUIImpl::HandleKeyPress(Key key, std::uint32_t key_code_point, Flags<ModKey> mod_keys, int curr_ticks)
 {
     key = KeyMappedKey(key, m_key_map);
     m_browse_info_wnd.reset();
     m_browse_info_mode = -1;
-    m_browse_target = 0;
-    m_last_pressed_key_code_point = std::make_pair(key, key_code_point);
+    m_browse_target = nullptr;
+    m_last_pressed_key_code_point = {key, key_code_point};
     m_last_key_press_repeat_time = 0;
     m_prev_key_press_time = curr_ticks;
 
@@ -664,7 +667,7 @@ void GUIImpl::HandleKeyPress(Key key, boost::uint32_t key_code_point, Flags<ModK
         // capslock, or which side of the keyboard's CTRL, SHIFT, etc.
         // was pressed, but the accelerators don't
         Flags<ModKey> massaged_mods = MassagedAccelModKeys(mod_keys);
-        if (m_accelerators.find(std::make_pair(key, massaged_mods))
+        if (m_accelerators.find({key, massaged_mods})
             != m_accelerators.end())
         {
             processed = GUI::s_gui->AcceleratorSignal(key, massaged_mods)();
@@ -675,14 +678,14 @@ void GUIImpl::HandleKeyPress(Key key, boost::uint32_t key_code_point, Flags<ModK
             WndEvent::KeyPress, key, key_code_point, mod_keys));
 }
 
-void GUIImpl::HandleKeyRelease(Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_keys, int curr_ticks)
+void GUIImpl::HandleKeyRelease(Key key, std::uint32_t key_code_point, Flags<ModKey> mod_keys, int curr_ticks)
 {
     key = KeyMappedKey(key, m_key_map);
     m_last_key_press_repeat_time = 0;
     m_last_pressed_key_code_point.first = GGK_UNKNOWN;
     m_browse_info_wnd.reset();
     m_browse_info_mode = -1;
-    m_browse_target = 0;
+    m_browse_target = nullptr;
     if (GUI::s_gui->FocusWnd())
         GUI::s_gui->FocusWnd()->HandleEvent(WndEvent(
             WndEvent::KeyRelease, key, key_code_point, mod_keys));
@@ -691,7 +694,7 @@ void GUIImpl::HandleKeyRelease(Key key, boost::uint32_t key_code_point, Flags<Mo
 void GUIImpl::HandleTextInput(const std::string* text) {
     m_browse_info_wnd.reset();
     m_browse_info_mode = -1;
-    m_browse_target = 0;
+    m_browse_target = nullptr;
     if (GUI::s_gui->FocusWnd())
         GUI::s_gui->FocusWnd()->HandleEvent(WndEvent(WndEvent::TextInput, text));
 }
@@ -720,7 +723,7 @@ void GUIImpl::HandleMouseMove(Flags<ModKey> mod_keys, const GG::Pt& pos, const P
     }
     if (m_prev_wnd_under_cursor != m_curr_wnd_under_cursor) {
         m_browse_info_wnd.reset();
-        m_browse_target = 0;
+        m_browse_target = nullptr;
         m_prev_wnd_under_cursor_time = curr_ticks;
     }
     m_prev_wnd_under_cursor = m_curr_wnd_under_cursor; // update this for the next time around
@@ -730,7 +733,7 @@ void GUIImpl::HandleMouseWheel(Flags<ModKey> mod_keys, const GG::Pt& pos, const
 {
     m_curr_wnd_under_cursor = GUI::s_gui->CheckedGetWindowUnder(pos, mod_keys);
     m_browse_info_wnd.reset();
-    m_browse_target = 0;
+    m_browse_target = nullptr;
     m_prev_wnd_under_cursor_time = curr_ticks;
     // don't send out 0-movement wheel messages
     if (m_curr_wnd_under_cursor && rel.y)
@@ -747,44 +750,44 @@ void GUIImpl::HandleMouseEnter(Flags< ModKey > mod_keys, const GG::Pt& pos, Wnd*
 
 void GUIImpl::ClearState()
 {
-    m_focus_wnd = 0;
+    m_focus_wnd = nullptr;
     m_mouse_pos = GG::Pt(X(-1000), Y(-1000));
     m_mouse_rel = GG::Pt(X(0), Y(0));
     m_mod_keys = Flags<ModKey>();
     m_last_mouse_button_down_repeat_time = 0;
     m_last_key_press_repeat_time = 0;
-    m_last_pressed_key_code_point = std::make_pair(GGK_UNKNOWN, 0u);
+    m_last_pressed_key_code_point = {GGK_UNKNOWN, 0u};
 
     m_prev_wnd_drag_position = Pt();
     m_browse_info_wnd.reset();
-    m_browse_target = 0;
+    m_browse_target = nullptr;
 
     m_prev_mouse_button_press_time = -1;
-    m_prev_wnd_under_cursor = 0;
+    m_prev_wnd_under_cursor = nullptr;
     m_prev_wnd_under_cursor_time = -1;
-    m_curr_wnd_under_cursor = 0;
+    m_curr_wnd_under_cursor = nullptr;
 
     m_mouse_button_state[0] = m_mouse_button_state[1] = m_mouse_button_state[2] = false;
-    m_drag_wnds[0] = m_drag_wnds[1] = m_drag_wnds[2] = 0;
+    m_drag_wnds[0] = m_drag_wnds[1] = m_drag_wnds[2] = nullptr;
 
     m_curr_drag_wnd_dragged = false;
-    m_curr_drag_wnd = 0;
-    m_curr_drag_drop_here_wnd = 0;
+    m_curr_drag_wnd = nullptr;
+    m_curr_drag_drop_here_wnd = nullptr;
     m_wnd_region = WR_NONE;
-    m_browse_target = 0;
-    m_drag_drop_originating_wnd = 0;
-    m_curr_wnd_under_cursor = 0;
+    m_browse_target = nullptr;
+    m_drag_drop_originating_wnd = nullptr;
+    m_curr_wnd_under_cursor = nullptr;
 
     m_delta_t = 0;
 
-    m_double_click_wnd = 0;
+    m_double_click_wnd = nullptr;
     m_double_click_start_time = -1;
     m_double_click_time = -1;
 }
 
 // static member(s)
-GUI*                       GUI::s_gui = 0;
-boost::shared_ptr<GUIImpl> GUI::s_impl;
+GUI*                       GUI::s_gui = nullptr;
+std::shared_ptr<GUIImpl> GUI::s_impl;
 
 // member functions
 GUI::GUI(const std::string& app_name)
@@ -978,9 +981,9 @@ bool GUI::MouseLRSwapped() const
 const std::map<Key, Key>& GUI::KeyMap() const
 { return s_impl->m_key_map; }
 
-std::set<std::pair<CPSize, CPSize> > GUI::FindWords(const std::string& str) const
+std::set<std::pair<CPSize, CPSize>> GUI::FindWords(const std::string& str) const
 {
-    std::set<std::pair<CPSize, CPSize> > retval;
+    std::set<std::pair<CPSize, CPSize>> retval;
     utf8_wchar_iterator first(str.begin(), str.begin(), str.end());
     utf8_wchar_iterator last(str.end(), str.begin(), str.end());
     word_regex_iterator it(first, last, DEFAULT_WORD_REGEX);
@@ -993,9 +996,9 @@ std::set<std::pair<CPSize, CPSize> > GUI::FindWords(const std::string& str) cons
     return retval;
 }
 
-std::set<std::pair<StrSize, StrSize> > GUI::FindWordsStringIndices(const std::string& str) const
+std::set<std::pair<StrSize, StrSize>> GUI::FindWordsStringIndices(const std::string& str) const
 {
-    std::set<std::pair<StrSize, StrSize> > retval;
+    std::set<std::pair<StrSize, StrSize>> retval;
 
     utf8_wchar_iterator first(str.begin(), str.begin(), str.end());
     utf8_wchar_iterator last(str.end(), str.begin(), str.end());
@@ -1013,7 +1016,7 @@ std::set<std::pair<StrSize, StrSize> > GUI::FindWordsStringIndices(const std::st
         std::advance(word_pos_it, match_result.length());
         StrSize end_idx(std::distance(str.begin(), word_pos_it.base()));
 
-        retval.insert(std::make_pair(start_idx, end_idx));
+        retval.insert({start_idx, end_idx});
     }
     return retval;
 }
@@ -1046,13 +1049,13 @@ bool GUI::ContainsWord(const std::string& str, const std::string& word) const
     return false;
 }
 
-const boost::shared_ptr<StyleFactory>& GUI::GetStyleFactory() const
+const std::shared_ptr<StyleFactory>& GUI::GetStyleFactory() const
 { return s_impl->m_style_factory; }
 
 bool GUI::RenderCursor() const
 { return s_impl->m_render_cursor; }
 
-const boost::shared_ptr<Cursor>& GUI::GetCursor() const
+const std::shared_ptr<Cursor>& GUI::GetCursor() const
 { return s_impl->m_cursor; }
 
 GUI::const_accel_iterator GUI::accel_begin() const
@@ -1063,7 +1066,7 @@ GUI::const_accel_iterator GUI::accel_end() const
 
 GUI::AcceleratorSignalType& GUI::AcceleratorSignal(Key key, Flags<ModKey> mod_keys/* = MOD_KEY_NONE*/) const
 {
-    boost::shared_ptr<AcceleratorSignalType>& sig_ptr = s_impl->m_accelerator_sigs[std::make_pair(key, mod_keys)];
+    std::shared_ptr<AcceleratorSignalType>& sig_ptr = s_impl->m_accelerator_sigs[{key, mod_keys}];
     if (!sig_ptr)
         sig_ptr.reset(new AcceleratorSignalType());
     if (INSTRUMENT_ALL_SIGNALS)
@@ -1086,7 +1089,7 @@ void GUI::SaveWndAsPNG(const Wnd* wnd, const std::string& filename) const
 void GUI::operator()()
 { Run(); }
 
-void GUI::HandleGGEvent(EventType event, Key key, boost::uint32_t key_code_point,
+void GUI::HandleGGEvent(EventType event, Key key, std::uint32_t key_code_point,
                         Flags<ModKey> mod_keys, const Pt& pos, const Pt& rel, const std::string* text)
 {
     s_impl->m_mod_keys = mod_keys;
@@ -1100,7 +1103,7 @@ void GUI::HandleGGEvent(EventType event, Key key, boost::uint32_t key_code_point
         if (s_impl->m_double_click_time >= s_impl->m_double_click_interval) {
             s_impl->m_double_click_start_time = -1;
             s_impl->m_double_click_time = -1;
-            s_impl->m_double_click_wnd = 0;
+            s_impl->m_double_click_wnd = nullptr;
         }
     }
 
@@ -1195,7 +1198,10 @@ bool GUI::SetNextFocusWndInCycle()
 }
 
 void GUI::Wait(unsigned int ms)
-{ boost::this_thread::sleep_for(boost::chrono::milliseconds(ms)); }
+{ std::this_thread::sleep_for(std::chrono::milliseconds(ms)); }
+
+void GUI::Wait(std::chrono::microseconds us)
+{ std::this_thread::sleep_for(us); }
 
 void GUI::Register(Wnd* wnd)
 { if (wnd) s_impl->m_zlist.Add(wnd); }
@@ -1203,7 +1209,7 @@ void GUI::Register(Wnd* wnd)
 void GUI::RegisterModal(Wnd* wnd)
 {
     if (wnd && wnd->Modal()) {
-        s_impl->m_modal_wnds.push_back(std::make_pair(wnd, wnd));
+        s_impl->m_modal_wnds.push_back({wnd, wnd});
         wnd->HandleEvent(WndEvent(WndEvent::GainingFocus));
     }
 }
@@ -1225,41 +1231,41 @@ void GUI::WndDying(Wnd* wnd)
 
     Remove(wnd);
     if (MatchesOrContains(wnd, s_impl->m_focus_wnd))
-        s_impl->m_focus_wnd = 0;
-    for (std::list<std::pair<Wnd*, Wnd*> >::iterator it = s_impl->m_modal_wnds.begin(); it != s_impl->m_modal_wnds.end(); ++it) {
-        if (MatchesOrContains(wnd, it->second)) {
-            if (MatchesOrContains(wnd, it->first)) {
-                it->second = 0;
+        s_impl->m_focus_wnd = nullptr;
+    for (std::pair<Wnd*, Wnd*>& modal_wnd : s_impl->m_modal_wnds) {
+        if (MatchesOrContains(wnd, modal_wnd.second)) {
+            if (MatchesOrContains(wnd, modal_wnd.first)) {
+                modal_wnd.second = nullptr;
             } else { // if the modal window for the removed window's focus level is available, revert focus to the modal window
-                if ((it->second = it->first))
-                    it->first->HandleEvent(WndEvent(WndEvent::GainingFocus));
+                if ((modal_wnd.second = modal_wnd.first))
+                    modal_wnd.first->HandleEvent(WndEvent(WndEvent::GainingFocus));
             }
         }
     }
     if (MatchesOrContains(wnd, s_impl->m_prev_wnd_under_cursor))
-        s_impl->m_prev_wnd_under_cursor = 0;
+        s_impl->m_prev_wnd_under_cursor = nullptr;
     if (MatchesOrContains(wnd, s_impl->m_curr_wnd_under_cursor))
-        s_impl->m_curr_wnd_under_cursor = 0;
+        s_impl->m_curr_wnd_under_cursor = nullptr;
     if (MatchesOrContains(wnd, s_impl->m_drag_wnds[0])) {
-        s_impl->m_drag_wnds[0] = 0;
+        s_impl->m_drag_wnds[0] = nullptr;
         s_impl->m_wnd_region = WR_NONE;
     }
     if (MatchesOrContains(wnd, s_impl->m_drag_wnds[1])) {
-        s_impl->m_drag_wnds[1] = 0;
+        s_impl->m_drag_wnds[1] = nullptr;
         s_impl->m_wnd_region = WR_NONE;
     }
     if (MatchesOrContains(wnd, s_impl->m_drag_wnds[2])) {
-        s_impl->m_drag_wnds[2] = 0;
+        s_impl->m_drag_wnds[2] = nullptr;
         s_impl->m_wnd_region = WR_NONE;
     }
     if (MatchesOrContains(wnd, s_impl->m_curr_drag_drop_here_wnd))
-        s_impl->m_curr_drag_drop_here_wnd = 0;
+        s_impl->m_curr_drag_drop_here_wnd = nullptr;
     if (MatchesOrContains(wnd, s_impl->m_drag_drop_originating_wnd))
-        s_impl->m_drag_drop_originating_wnd = 0;
+        s_impl->m_drag_drop_originating_wnd = nullptr;
     s_impl->m_drag_drop_wnds.erase(wnd);
     s_impl->m_drag_drop_wnds_acceptable.erase(wnd);
     if (MatchesOrContains(wnd, s_impl->m_double_click_wnd)) {
-        s_impl->m_double_click_wnd = 0;
+        s_impl->m_double_click_wnd = nullptr;
         s_impl->m_double_click_start_time = -1;
         s_impl->m_double_click_time = -1;
     }
@@ -1285,8 +1291,8 @@ void GUI::MoveUp(Wnd* wnd)
 void GUI::MoveDown(Wnd* wnd)
 { if (wnd) s_impl->m_zlist.MoveDown(wnd); }
 
-boost::shared_ptr<ModalEventPump> GUI::CreateModalEventPump(bool& done)
-{ return boost::shared_ptr<ModalEventPump>(new ModalEventPump(done)); }
+std::shared_ptr<ModalEventPump> GUI::CreateModalEventPump(bool& done)
+{ return std::make_shared<ModalEventPump>(done); }
 
 void GUI::RegisterDragDropWnd(Wnd* wnd, const Pt& offset, Wnd* originating_wnd)
 {
@@ -1360,13 +1366,13 @@ GUI::accel_iterator GUI::accel_end()
 void GUI::SetAccelerator(Key key, Flags<ModKey> mod_keys/* = MOD_KEY_NONE*/)
 {
     mod_keys = MassagedAccelModKeys(mod_keys);
-    s_impl->m_accelerators.insert(std::make_pair(key, mod_keys));
+    s_impl->m_accelerators.insert({key, mod_keys});
 }
 
 void GUI::RemoveAccelerator(Key key, Flags<ModKey> mod_keys/* = MOD_KEY_NONE*/)
 {
     mod_keys = MassagedAccelModKeys(mod_keys);
-    s_impl->m_accelerators.erase(std::make_pair(key, mod_keys));
+    s_impl->m_accelerators.erase({key, mod_keys});
 }
 
 void GUI::RemoveAccelerator(accel_iterator it)
@@ -1381,16 +1387,16 @@ void GUI::SetMouseLRSwapped(bool swapped/* = true*/)
 void GUI::SetKeyMap(const std::map<Key, Key>& key_map)
 { s_impl->m_key_map = key_map; }
 
-boost::shared_ptr<Font> GUI::GetFont(const std::string& font_filename, unsigned int pts)
+std::shared_ptr<Font> GUI::GetFont(const std::string& font_filename, unsigned int pts)
 { return GetFontManager().GetFont(font_filename, pts); }
 
-boost::shared_ptr<Font> GUI::GetFont(const std::string& font_filename, unsigned int pts,
-                                     const std::vector<unsigned char>& file_contents)
+std::shared_ptr<Font> GUI::GetFont(const std::string& font_filename, unsigned int pts,
+                                   const std::vector<unsigned char>& file_contents)
 { return GetFontManager().GetFont(font_filename, pts, file_contents); }
 
-boost::shared_ptr<Font> GUI::GetFont(const boost::shared_ptr<Font>& font, unsigned int pts)
+std::shared_ptr<Font> GUI::GetFont(const std::shared_ptr<Font>& font, unsigned int pts)
 {
-    boost::shared_ptr<Font> retval;
+    std::shared_ptr<Font> retval;
     if (font->FontName() == StyleFactory::DefaultFontName()) {
         retval = GetStyleFactory()->DefaultFont(pts);
     } else {
@@ -1404,19 +1410,19 @@ boost::shared_ptr<Font> GUI::GetFont(const boost::shared_ptr<Font>& font, unsign
 void GUI::FreeFont(const std::string& font_filename, unsigned int pts)
 { GetFontManager().FreeFont(font_filename, pts); }
 
-boost::shared_ptr<Texture> GUI::StoreTexture(Texture* texture, const std::string& texture_name)
+std::shared_ptr<Texture> GUI::StoreTexture(Texture* texture, const std::string& texture_name)
 { return GetTextureManager().StoreTexture(texture, texture_name); }
 
-boost::shared_ptr<Texture> GUI::StoreTexture(const boost::shared_ptr<Texture>& texture, const std::string& texture_name)
+std::shared_ptr<Texture> GUI::StoreTexture(const std::shared_ptr<Texture>& texture, const std::string& texture_name)
 { return GetTextureManager().StoreTexture(texture, texture_name); }
 
-boost::shared_ptr<Texture> GUI::GetTexture(const boost::filesystem::path& path, bool mipmap/* = false*/)
+std::shared_ptr<Texture> GUI::GetTexture(const boost::filesystem::path& path, bool mipmap/* = false*/)
 { return GetTextureManager().GetTexture(path, mipmap); }
 
 void GUI::FreeTexture(const boost::filesystem::path& path)
 { GetTextureManager().FreeTexture(path); }
 
-void GUI::SetStyleFactory(const boost::shared_ptr<StyleFactory>& factory)
+void GUI::SetStyleFactory(const std::shared_ptr<StyleFactory>& factory)
 {
     s_impl->m_style_factory = factory;
     if (!s_impl->m_style_factory)
@@ -1426,7 +1432,7 @@ void GUI::SetStyleFactory(const boost::shared_ptr<StyleFactory>& factory)
 void GUI::RenderCursor(bool render)
 { s_impl->m_render_cursor = render; }
 
-void GUI::SetCursor(const boost::shared_ptr<Cursor>& cursor)
+void GUI::SetCursor(const std::shared_ptr<Cursor>& cursor)
 { s_impl->m_cursor = cursor; }
 
 std::string GUI::ClipboardText() const
@@ -1550,6 +1556,19 @@ bool GUI::FocusWndDeselect()
 GUI* GUI::GetGUI()
 { return s_gui; }
 
+void GUI::PreRenderWindow(Wnd* wnd)
+{
+    if (!wnd || !wnd->Visible())
+        return;
+
+    for (Wnd* child_wnd : wnd->m_children) {
+        PreRenderWindow(child_wnd);
+    }
+
+    if (wnd->PreRenderRequired())
+        wnd->PreRender();
+}
+
 void GUI::RenderWindow(Wnd* wnd)
 {
     if (!wnd)
@@ -1564,9 +1583,9 @@ void GUI::RenderWindow(Wnd* wnd)
             bool clip = clip_mode != Wnd::DontClip;
             if (clip)
                 wnd->BeginClipping();
-            for (std::list<Wnd*>::iterator it = wnd->m_children.begin(); it != wnd->m_children.end(); ++it) {
-                if ((*it) && (*it)->Visible())
-                    RenderWindow(*it);
+            for (Wnd* child_wnd : wnd->m_children) {
+                if (child_wnd && child_wnd->Visible())
+                    RenderWindow(child_wnd);
             }
             if (clip)
                 wnd->EndClipping();
@@ -1598,7 +1617,7 @@ void GUI::RenderWindow(Wnd* wnd)
 
     if (wnd == s_impl->m_save_as_png_wnd) {
         WriteWndToPNG(s_impl->m_save_as_png_wnd, s_impl->m_save_as_png_filename);
-        s_impl->m_save_as_png_wnd = 0;
+        s_impl->m_save_as_png_wnd = nullptr;
         s_impl->m_save_as_png_filename.clear();
     }
 }
@@ -1607,17 +1626,17 @@ void GUI::RenderDragDropWnds()
 {
     // render drag-and-drop windows in arbitrary order (sorted by pointer value)
     s_impl->m_rendering_drag_drop_wnds = true;
-    for (std::map<Wnd*, Pt>::const_iterator it = s_impl->m_drag_drop_wnds.begin(); it != s_impl->m_drag_drop_wnds.end(); ++it) {
-        bool old_visible = it->first->Visible();
+    for (const std::map<Wnd*, Pt>::value_type& drop_wnd : s_impl->m_drag_drop_wnds) {
+        bool old_visible = drop_wnd.first->Visible();
         if (!old_visible)
-            it->first->Show();
-        Pt parent_offset = it->first->Parent() ? it->first->Parent()->ClientUpperLeft() : Pt();
-        Pt old_pos = it->first->UpperLeft() - parent_offset;
-        it->first->MoveTo(s_impl->m_mouse_pos - parent_offset - it->second);
-        RenderWindow(it->first);
-        it->first->MoveTo(old_pos);
+            drop_wnd.first->Show();
+        Pt parent_offset = drop_wnd.first->Parent() ? drop_wnd.first->Parent()->ClientUpperLeft() : Pt();
+        Pt old_pos = drop_wnd.first->UpperLeft() - parent_offset;
+        drop_wnd.first->MoveTo(s_impl->m_mouse_pos - parent_offset - drop_wnd.second);
+        RenderWindow(drop_wnd.first);
+        drop_wnd.first->MoveTo(old_pos);
         if (!old_visible)
-            it->first->Hide();
+            drop_wnd.first->Hide();
     }
     s_impl->m_rendering_drag_drop_wnds = false;
 }
@@ -1636,12 +1655,35 @@ void GUI::ProcessBrowseInfo()
     }
 }
 
+void GUI::PreRender()
+{
+    // pre-render normal windows back-to-front
+    for (ZList::reverse_iterator it = s_impl->m_zlist.rbegin(); it != s_impl->m_zlist.rend(); ++it) {
+        PreRenderWindow(*it);
+    }
+
+    // pre-render modal windows back-to-front (on top of non-modal Wnds rendered above)
+    for (const std::pair<Wnd*, Wnd*>& modal_wnd : s_impl->m_modal_wnds) {
+        PreRenderWindow(modal_wnd.first);
+    }
+
+    // pre-render the active browse info window, if any
+    if (s_impl->m_browse_info_wnd && s_impl->m_curr_wnd_under_cursor) {
+        assert(s_impl->m_browse_target);
+        PreRenderWindow(s_impl->m_browse_info_wnd.get());
+    }
+
+    for (const std::map<Wnd*, Pt>::value_type& drag_drop_wnd : s_impl->m_drag_drop_wnds) {
+        PreRenderWindow(drag_drop_wnd.first);
+    }
+}
+
 void GUI::Render()
 {
     // update timers
     int ticks = Ticks();
-    for (std::set<Timer*>::iterator it = s_impl->m_timers.begin(); it != s_impl->m_timers.end(); ++it) {
-        (*it)->Update(ticks);
+    for (Timer* timer : s_impl->m_timers) {
+        timer->Update(ticks);
     }
 
     Enter2DMode();
@@ -1652,11 +1694,9 @@ void GUI::Render()
     }
 
     // render modal windows back-to-front (on top of non-modal Wnds rendered above)
-    for (std::list<std::pair<Wnd*, Wnd*> >::iterator it = s_impl->m_modal_wnds.begin();
-         it != s_impl->m_modal_wnds.end(); ++it)
-    {
-        if (it->first)
-            RenderWindow(it->first);
+    for (const std::pair<Wnd*, Wnd*>& modal_wnd : s_impl->m_modal_wnds) {
+        if (modal_wnd.first)
+            RenderWindow(modal_wnd.first);
     }
 
     // render the active browse info window, if any
@@ -1664,7 +1704,7 @@ void GUI::Render()
         if (!s_impl->m_curr_wnd_under_cursor) {
             s_impl->m_browse_info_wnd.reset();
             s_impl->m_browse_info_mode = -1;
-            s_impl->m_browse_target = 0;
+            s_impl->m_browse_target = nullptr;
             s_impl->m_prev_wnd_under_cursor_time = Ticks();
         } else {
             assert(s_impl->m_browse_target);
@@ -1709,7 +1749,7 @@ bool GUI::ProcessBrowseInfoImpl(Wnd* wnd)
 
 Wnd* GUI::ModalWindow() const
 {
-    Wnd* retval = 0;
+    Wnd* retval = nullptr;
     if (!s_impl->m_modal_wnds.empty())
         retval = s_impl->m_modal_wnds.back().first;
     return retval;
@@ -1727,7 +1767,7 @@ Wnd* GUI::CheckedGetWindowUnder(const Pt& pt, Flags<ModKey> mod_keys)
 
     if (s_impl->m_curr_drag_drop_here_wnd && !unregistered_drag_drop && !registered_drag_drop) {
         s_impl->m_curr_drag_drop_here_wnd->HandleEvent(WndEvent(WndEvent::DragDropLeave));
-        s_impl->m_curr_drag_drop_here_wnd = 0;
+        s_impl->m_curr_drag_drop_here_wnd = nullptr;
     }
 
     if (w == s_impl->m_curr_wnd_under_cursor)
@@ -1738,14 +1778,13 @@ Wnd* GUI::CheckedGetWindowUnder(const Pt& pt, Flags<ModKey> mod_keys)
         if (unregistered_drag_drop) {
             s_impl->m_curr_wnd_under_cursor->HandleEvent(WndEvent(WndEvent::DragDropLeave));
             s_impl->m_drag_drop_wnds_acceptable[dragged_wnd] = false;
-            s_impl->m_curr_drag_drop_here_wnd = 0;
+            s_impl->m_curr_drag_drop_here_wnd = nullptr;
 
         } else if (registered_drag_drop) {
             s_impl->m_curr_wnd_under_cursor->HandleEvent(WndEvent(WndEvent::DragDropLeave));
-            for (std::map<const Wnd*, bool>::iterator it = s_impl->m_drag_drop_wnds_acceptable.begin();
-                 it != s_impl->m_drag_drop_wnds_acceptable.end(); ++it)
-            { it->second = false; }
-            s_impl->m_curr_drag_drop_here_wnd = 0;
+            for (std::map<const Wnd*, bool>::value_type& acceptable_wnd : s_impl->m_drag_drop_wnds_acceptable)
+            { acceptable_wnd.second = false; }
+            s_impl->m_curr_drag_drop_here_wnd = nullptr;
 
         } else {
             s_impl->m_curr_wnd_under_cursor->HandleEvent(WndEvent(WndEvent::MouseLeave));
