@@ -53,12 +53,7 @@ namespace {
 // GG::MenuItem
 ////////////////////////////////////////////////
 MenuItem::MenuItem() :
-    SelectedIDSignal(new SelectedIDSignalType()),
-    SelectedSignal(new SelectedSignalType()),
-    item_ID(0),
-    disabled(false),
-    checked(false),
-    separator(false)
+    MenuItem("", 0, false, false)
 {}
 
 MenuItem::MenuItem(bool separator) :
@@ -85,40 +80,6 @@ MenuItem::MenuItem(const std::string& str, int id, bool disable, bool check) :
     }
 }
 
-MenuItem::MenuItem(const std::string& str, int id, bool disable, bool check, const SelectedIDSlotType& slot) :
-    SelectedIDSignal(new SelectedIDSignalType()),
-    SelectedSignal(new SelectedSignalType()),
-    label(str),
-    item_ID(id),
-    disabled(disable),
-    checked(check),
-    separator(false)
-{
-    SelectedIDSignal->connect(slot);
-
-    if (INSTRUMENT_ALL_SIGNALS) {
-        Connect(*SelectedIDSignal, MenuSignalEcho("MenuItem::SelectedIDSignal"));
-        Connect(*SelectedSignal, MenuSignalEcho("MenuItem::SelectedSignal"));
-    }
-}
-
-MenuItem::MenuItem(const std::string& str, int id, bool disable, bool check, const SelectedSlotType& slot) :
-    SelectedIDSignal(new SelectedIDSignalType()),
-    SelectedSignal(new SelectedSignalType()),
-    label(str),
-    item_ID(id),
-    disabled(disable),
-    checked(check),
-    separator(false)
-{
-    SelectedSignal->connect(slot);
-
-    if (INSTRUMENT_ALL_SIGNALS) {
-        Connect(*SelectedIDSignal, MenuSignalEcho("MenuItem::SelectedIDSignal"));
-        Connect(*SelectedSignal, MenuSignalEcho("MenuItem::SelectedSignal"));
-    }
-}
-
 MenuItem::~MenuItem()
 {}
 
@@ -128,7 +89,7 @@ MenuItem::~MenuItem()
 ////////////////////////////////////////////////
 const std::size_t MenuBar::INVALID_CARET = std::numeric_limits<std::size_t>::max();
 
-MenuBar::MenuBar(X x, Y y, X w, const boost::shared_ptr<Font>& font, Clr text_color/* = CLR_WHITE*/,
+MenuBar::MenuBar(X x, Y y, X w, const std::shared_ptr<Font>& font, Clr text_color/* = CLR_WHITE*/,
                  Clr color/* = CLR_BLACK*/, Clr interior/* = CLR_SHADOW*/) :
     Control(x, y, w, font->Lineskip()),
     m_font(font),
@@ -147,7 +108,7 @@ MenuBar::MenuBar(X x, Y y, X w, const boost::shared_ptr<Font>& font, Clr text_co
         Connect(BrowsedSignal, MenuSignalEcho("MenuBar::BrowsedSignal"));
 }
 
-MenuBar::MenuBar(X x, Y y, X w, const boost::shared_ptr<Font>& font, const MenuItem& m,
+MenuBar::MenuBar(X x, Y y, X w, const std::shared_ptr<Font>& font, const MenuItem& m,
                  Clr text_color/* = CLR_WHITE*/, Clr color/* = CLR_BLACK*/, Clr interior/* = CLR_SHADOW*/) :
     Control(x, y, w, font->Lineskip()),
     m_font(font),
@@ -170,8 +131,8 @@ MenuBar::MenuBar(X x, Y y, X w, const boost::shared_ptr<Font>& font, const MenuI
 Pt MenuBar::MinUsableSize() const
 {
     Pt retval;
-    for (std::size_t i = 0; i < m_menu_labels.size(); ++i) {
-        Pt min_size = m_menu_labels[i]->MinUsableSize();
+    for (TextControl* menu_label : m_menu_labels) {
+        Pt min_size = menu_label->MinUsableSize();
         retval.y = std::max(retval.y, min_size.y);
         retval.x += min_size.x;
     }
@@ -184,8 +145,8 @@ const MenuItem& MenuBar::AllMenus() const
 bool MenuBar::ContainsMenu(const std::string& str) const
 {
     bool retval = false;
-    for (std::vector<MenuItem>::const_iterator it = m_menu_data.next_level.begin(); it != m_menu_data.next_level.end(); ++it) {
-        if (it->label == str) {
+    for (const MenuItem& submenu : m_menu_data.next_level) {
+        if (submenu.label == str) {
             retval = true;
             break;
         }
@@ -198,12 +159,11 @@ std::size_t MenuBar::NumMenus() const
 
 const MenuItem& MenuBar::GetMenu(const std::string& str) const
 {
-    std::vector<MenuItem>::const_iterator it = m_menu_data.next_level.begin();
-    for (; it != m_menu_data.next_level.end(); ++it) {
-        if (it->label == str)
-            break;
+    for (const MenuItem& submenu : m_menu_data.next_level) {
+        if (submenu.label == str)
+            return submenu;
     }
-    return *it;
+    return m_menu_data.next_level.front();
 }
 
 const MenuItem& MenuBar::GetMenu(std::size_t n) const
@@ -252,7 +212,6 @@ void MenuBar::LButtonDown(const Pt& pt, Flags<ModKey> mod_keys)
                     (*m_menu_data.next_level[i].SelectedIDSignal)(m_menu_data.next_level[i].item_ID);
                     (*m_menu_data.next_level[i].SelectedSignal)();
                 } else {
-                    MenuItem popup_data;
                     PopupMenu menu(m_menu_labels[i]->Left(), m_menu_labels[i]->Bottom(), m_font, m_menu_data.next_level[i], m_text_color, m_border_color, m_int_color);
                     menu.SetHiliteColor(m_hilite_color);
                     menu.SetSelectedTextColor(m_sel_text_color);
@@ -291,12 +250,11 @@ MenuItem& MenuBar::AllMenus()
 
 MenuItem& MenuBar::GetMenu(const std::string& str)
 {
-    std::vector<MenuItem>::iterator it = m_menu_data.next_level.begin();
-    for (; it != m_menu_data.next_level.end(); ++it) {
-        if (it->label == str)
-            break;
+    for (MenuItem& submenu : m_menu_data.next_level) {
+        if (submenu.label == str)
+            return submenu;
     }
-    return *it;
+    return m_menu_data.next_level.front();
 }
 
 MenuItem& MenuBar::GetMenu(int n)
@@ -323,7 +281,7 @@ void MenuBar::SetHiliteColor(Clr clr)
 void MenuBar::SetSelectedTextColor(Clr clr)
 { m_sel_text_color = clr; }
 
-const boost::shared_ptr<Font>& MenuBar::GetFont() const
+const std::shared_ptr<Font>& MenuBar::GetFont() const
 { return m_font; }
 
 const std::vector<TextControl*>& MenuBar::MenuLabels() const
@@ -392,7 +350,7 @@ namespace {
 
 const std::size_t PopupMenu::INVALID_CARET = std::numeric_limits<std::size_t>::max();
 
-PopupMenu::PopupMenu(X x, Y y, const boost::shared_ptr<Font>& font, const MenuItem& m, Clr text_color/* = CLR_WHITE*/,
+PopupMenu::PopupMenu(X x, Y y, const std::shared_ptr<Font>& font, const MenuItem& m, Clr text_color/* = CLR_WHITE*/,
                      Clr border_color/* = CLR_BLACK*/, Clr interior_color/* = CLR_SHADOW*/, Clr hilite_color/* = CLR_GRAY*/) :
     Wnd(X0, Y0, GUI::GetGUI()->AppWidth() - 1, GUI::GetGUI()->AppHeight() - 1, INTERACTIVE | MODAL),
     m_font(font),
@@ -405,7 +363,7 @@ PopupMenu::PopupMenu(X x, Y y, const boost::shared_ptr<Font>& font, const MenuIt
     m_open_levels(),
     m_caret(1, INVALID_CARET),
     m_origin(x, y),
-    m_item_selected(0)
+    m_item_selected(nullptr)
 {
     m_open_levels.resize(1);
 
@@ -463,9 +421,11 @@ void PopupMenu::Render()
                 if (menu.next_level[j].next_level.size() || menu.next_level[j].checked)
                     needs_indicator = true;
             }
-            std::vector<Font::LineData> lines;
             Flags<TextFormat> fmt = FORMAT_LEFT | FORMAT_TOP;
-            Pt menu_sz = m_font->DetermineLines(str, fmt, X0, lines); // get dimensions of text in menu
+            std::vector<std::shared_ptr<Font::TextElement>> text_elements
+                = m_font->ExpensiveParseFromTextToTextElements(str, fmt);
+            std::vector<Font::LineData> lines = m_font->DetermineLines(str, fmt, X0, text_elements);
+            Pt menu_sz = m_font->TextExtent(lines); // get dimensions of text in menu
             menu_sz.x += 2 * HORIZONTAL_MARGIN;
             if (needs_indicator)
                 menu_sz.x += CHECK_WIDTH + 2 * HORIZONTAL_MARGIN; // make room for the little arrow
@@ -519,7 +479,13 @@ void PopupMenu::Render()
                 glColor3ub(clr.r, clr.g, clr.b);
 
                 if (!menu.next_level[j].separator) {
-                    m_font->RenderText(line_rect.ul, line_rect.lr, menu.next_level[j].label, fmt);
+                    // TODO cache line data v expensive calculation
+                    std::vector<std::shared_ptr<Font::TextElement>> text_elements =
+                        m_font->ExpensiveParseFromTextToTextElements(menu.next_level[j].label, fmt);
+                    std::vector<Font::LineData> lines =
+                        m_font->DetermineLines(menu.next_level[j].label, fmt, X0, text_elements);
+
+                    m_font->RenderText(line_rect.ul, line_rect.lr, menu.next_level[j].label, fmt, lines);
 
                 } else {
                     Line(line_rect.ul.x + HORIZONTAL_MARGIN, line_rect.ul.y + INDICATOR_HEIGHT/2 + INDICATOR_VERTICAL_MARGIN,
@@ -553,9 +519,9 @@ void PopupMenu::LButtonUp(const Pt& pt, Flags<ModKey> mod_keys)
 {
     if (m_caret[0] != INVALID_CARET) {
         MenuItem* menu_ptr = &m_menu_data;
-        for (std::size_t i = 0; i < m_caret.size(); ++i) {
-            if (m_caret[i] != INVALID_CARET) {
-                menu_ptr = &menu_ptr->next_level[m_caret[i]];
+        for (std::size_t caret : m_caret) {
+            if (caret != INVALID_CARET) {
+                menu_ptr = &menu_ptr->next_level[caret];
             }
         }
         if (!menu_ptr->disabled && !menu_ptr->separator) {
@@ -607,8 +573,11 @@ void PopupMenu::LDrag(const Pt& pt, const Pt& move, Flags<ModKey> mod_keys)
     int update_ID = 0;
     if (m_caret[0] != INVALID_CARET) {
         MenuItem* menu_ptr = &m_menu_data;
-        for (std::size_t i = 0; i < m_caret.size(); ++i)
-            menu_ptr = &menu_ptr->next_level[m_caret[i]];
+        for (std::size_t caret : m_caret) {
+            if (caret == INVALID_CARET)
+                break;
+            menu_ptr = &menu_ptr->next_level[caret];
+        }
         update_ID = menu_ptr->item_ID;
     }
     BrowsedSignal(update_ID);
@@ -648,7 +617,7 @@ void PopupMenu::SetHiliteColor(Clr clr)
 void PopupMenu::SetSelectedTextColor(Clr clr)
 { m_sel_text_color = clr; }
 
-const boost::shared_ptr<Font>& PopupMenu::GetFont() const
+const std::shared_ptr<Font>& PopupMenu::GetFont() const
 { return m_font; }
 
 const MenuItem& PopupMenu::MenuData() const
