@@ -31,8 +31,6 @@
 #include <GG/WndEvent.h>
 #include <GG/utf8/checked.h>
 
-#include <boost/assign/list_of.hpp>
-
 
 using namespace GG;
 
@@ -102,7 +100,7 @@ const std::size_t MultiEdit::ALL_LINES = std::numeric_limits<std::size_t>::max()
 const unsigned int MultiEdit::SCROLL_WIDTH = 14;
 const unsigned int MultiEdit::BORDER_THICK = 2;
 
-MultiEdit::MultiEdit(const std::string& str, const boost::shared_ptr<Font>& font, Clr color,
+MultiEdit::MultiEdit(const std::string& str, const std::shared_ptr<Font>& font, Clr color,
                      Flags<MultiEditStyle> style/* = MULTI_LINEWRAP*/, Clr text_color/* = CLR_BLACK*/, Clr interior/* = CLR_ZERO*/) :
     Edit(str, font, color, text_color, interior),
     m_style(style),
@@ -111,8 +109,8 @@ MultiEdit::MultiEdit(const std::string& str, const boost::shared_ptr<Font>& font
     m_first_col_shown(0),
     m_first_row_shown(0),
     m_max_lines_history(ALL_LINES),
-    m_vscroll(0),
-    m_hscroll(0),
+    m_vscroll(nullptr),
+    m_hscroll(nullptr),
     m_vscroll_wheel_scroll_increment(0),
     m_hscroll_wheel_scroll_increment(0),
     m_preserve_text_position_on_next_set_text(false),
@@ -169,7 +167,7 @@ void MultiEdit::Render()
     Font::RenderState state(text_color_to_use);
     std::size_t first_visible_row = FirstVisibleRow();
     std::size_t last_visible_row = LastVisibleRow();
-    Flags<TextFormat> text_format = TextFormat() & ~(FORMAT_TOP | FORMAT_BOTTOM) | FORMAT_VCENTER;
+    Flags<TextFormat> text_format = (TextFormat() & ~(FORMAT_TOP | FORMAT_BOTTOM)) | FORMAT_VCENTER;
     const std::vector<Font::LineData>& lines = GetLineData();
     GetFont()->ProcessTagsBefore(lines, state, first_visible_row, CP0);
 
@@ -254,7 +252,7 @@ void MultiEdit::SizeMove(const Pt& ul, const Pt& lr)
     // resizing this control doesn't reset the scroll position to the top.
     // just calling PreserveTextPositionOnNextSetText() before the SetText
     // call doesn't work as that leaves the scrollbars unadjusted for the resize
-    GG::Pt initial_scroll_pos = ScrollPosition();
+    Pt initial_scroll_pos = ScrollPosition();
 
     Edit::SizeMove(ul, lower_right);
 
@@ -267,13 +265,13 @@ void MultiEdit::SizeMove(const Pt& ul, const Pt& lr)
 void MultiEdit::SelectAll()
 {
     m_cursor_begin = std::pair<std::size_t, CPSize>(0, CP0);
-    m_cursor_end = std::pair<std::size_t, CPSize>(
+    m_cursor_end = GetLineData().empty() ? m_cursor_begin : std::pair<std::size_t, CPSize>(
         GetLineData().size() - 1,
         CPSize(GetLineData()[GetLineData().size() - 1].char_data.size()));
 
     CPSize begin_cursor_pos = CharIndexOf(m_cursor_begin.first, m_cursor_begin.second);
     CPSize end_cursor_pos = CharIndexOf(m_cursor_end.first, m_cursor_end.second);
-    this->m_cursor_pos = std::make_pair(begin_cursor_pos, end_cursor_pos);
+    this->m_cursor_pos = {begin_cursor_pos, end_cursor_pos};
 }
 
 void MultiEdit::DeselectAll()
@@ -282,7 +280,7 @@ void MultiEdit::DeselectAll()
     m_cursor_end = m_cursor_begin;
 
     CPSize cursor_pos = CharIndexOf(m_cursor_begin.first, m_cursor_begin.second);
-    this->m_cursor_pos = std::make_pair(cursor_pos, cursor_pos);
+    this->m_cursor_pos = {cursor_pos, cursor_pos};
 }
 
 void MultiEdit::SetText(const std::string& str)
@@ -302,8 +300,9 @@ void MultiEdit::SetText(const std::string& str)
         if (m_max_lines_history == ALL_LINES) {
             TextControl::SetText(str);
         } else {
-            std::vector<Font::LineData> lines;
-            GetFont()->DetermineLines(str, format, cl_sz.x, lines);
+            std::vector<std::shared_ptr<Font::TextElement>> text_elements
+                = GetFont()->ExpensiveParseFromTextToTextElements(str, format);
+            std::vector<Font::LineData> lines = GetFont()->DetermineLines(str, format, cl_sz.x, text_elements);
             if (m_max_lines_history < lines.size()) {
                 std::size_t first_line = 0;
                 std::size_t last_line = m_max_lines_history - 1;
@@ -346,7 +345,10 @@ void MultiEdit::SetText(const std::string& str)
         }
 
         // make sure the change in text did not make the cursor position invalid
-        if (GetLineData().size() <= m_cursor_end.first) {
+        if (GetLineData().empty()) {
+            m_cursor_end.first = 0;
+            m_cursor_end.second = CP0;
+        } else if (GetLineData().size() <= m_cursor_end.first) {
             m_cursor_end.first = GetLineData().size() - 1;
             m_cursor_end.second = CPSize(GetLineData()[m_cursor_end.first].char_data.size());
         } else if (GetLineData()[m_cursor_end.first].char_data.size() < m_cursor_end.second) {
@@ -355,9 +357,9 @@ void MultiEdit::SetText(const std::string& str)
         m_cursor_begin = m_cursor_end; // eliminate any hiliting
 
         CPSize cursor_pos = CharIndexOf(m_cursor_end.first, m_cursor_end.second);
-        this->m_cursor_pos = std::make_pair(cursor_pos, cursor_pos);
+        this->m_cursor_pos = {cursor_pos, cursor_pos};
 
-        m_contents_sz = GetFont()->TextExtent(Text(), GetLineData());
+        m_contents_sz = GetFont()->TextExtent(GetLineData());
 
         AdjustScrolls();
         AdjustView();
@@ -406,9 +408,9 @@ void MultiEdit::SetScrollPosition(Pt pt)
     if (m_hscroll) {
         std::pair<int, int> range = m_hscroll->ScrollRange();
         if (pt.x < range.first)
-            pt.x = GG::X(range.first);
+            pt.x = X(range.first);
         if (pt.x > range.second)
-            pt.x = GG::X(range.second);
+            pt.x = X(range.second);
         std::pair<int, int> posn_range = m_hscroll->PosnRange();
         if (pt.x != posn_range.first) {
             m_hscroll->ScrollTo(Value(pt.x));
@@ -418,9 +420,9 @@ void MultiEdit::SetScrollPosition(Pt pt)
     if (m_vscroll) {
         std::pair<int, int> range = m_vscroll->ScrollRange();
         if (pt.y < range.first)
-            pt.y = GG::Y(range.first);
+            pt.y = Y(range.first);
         if (pt.y > range.second)
-            pt.y = GG::Y(range.second);
+            pt.y = Y(range.second);
         std::pair<int, int> posn_range = m_vscroll->PosnRange();
         if (pt.y != posn_range.first) {
             m_vscroll->ScrollTo(Value(pt.y));
@@ -452,6 +454,9 @@ Y MultiEdit::BottomMargin() const
 
 std::pair<std::size_t, CPSize> MultiEdit::CharAt(const Pt& pt) const
 {
+    if (GetLineData().empty())
+        return std::pair<std::size_t, CPSize>(0, CP0);
+
     std::pair<std::size_t, CPSize> retval;
 
     size_t row = RowAt(pt.y);
@@ -468,12 +473,12 @@ std::pair<std::size_t, CPSize> MultiEdit::CharAt(const Pt& pt) const
 
 std::pair<std::size_t, CPSize> MultiEdit::CharAt(CPSize idx) const
 {
-    std::pair<std::size_t, CPSize> retval(0, CP0);
-    if (idx > Text().size())
-        return retval;
-
     const std::vector<Font::LineData>& lines = GetLineData();
 
+    std::pair<std::size_t, CPSize> retval(0, CP0);
+    if (lines.empty() || idx > Text().size())
+        return retval;
+
     retval = LinePositionOf(idx, lines);
 
     if (retval.second == INVALID_CP_SIZE) {
@@ -486,11 +491,11 @@ std::pair<std::size_t, CPSize> MultiEdit::CharAt(CPSize idx) const
 
 Pt MultiEdit::ScrollPosition() const
 {
-    Pt retval(GG::X0, GG::Y0);
+    Pt retval(X0, Y0);
     if (m_hscroll)
-        retval.x = GG::X(m_hscroll->PosnRange().first);
+        retval.x = X(m_hscroll->PosnRange().first);
     if (m_vscroll)
-        retval.y = GG::Y(m_vscroll->PosnRange().first);
+        retval.y = Y(m_vscroll->PosnRange().first);
     return retval;
 }
 
@@ -521,8 +526,8 @@ CPSize MultiEdit::CharIndexOf(std::size_t row, CPSize char_idx,
     // "rewind" the first position to encompass all tag text that is
     // associated with that position
     CPSize retval = line.char_data[Value(char_idx)].code_point_index;
-    for (std::size_t i = 0; i < line.char_data[Value(char_idx)].tags.size(); ++i)
-        retval -= line.char_data[Value(char_idx)].tags[i]->CodePointSize();
+    for (const std::shared_ptr<Font::FormattingTag>& tag : line.char_data[Value(char_idx)].tags)
+        retval -= tag->CodePointSize();
 
     return retval;
 }
@@ -538,7 +543,7 @@ X MultiEdit::RowStartX(std::size_t row) const
     else if (m_style & MULTI_CENTER)
         retval -= excess_width / 2;
 
-    if (!GetLineData()[row].Empty()) {
+    if (!GetLineData().empty() && !GetLineData()[row].Empty()) {
         X line_width = GetLineData()[row].char_data.back().extent;
         if (GetLineData()[row].justification == ALIGN_LEFT) {
             retval += (m_vscroll && m_hscroll ? RightMargin() : X0);
@@ -553,7 +558,7 @@ X MultiEdit::RowStartX(std::size_t row) const
 }
 
 X MultiEdit::CharXOffset(std::size_t row, CPSize idx) const
-{ return (0 < idx ? GetLineData()[row].char_data[Value(idx - 1)].extent : X0); }
+{ return (0 < idx && !GetLineData().empty() ? GetLineData()[row].char_data[Value(idx - 1)].extent : X0); }
 
 std::size_t MultiEdit::RowAt(Y y) const
 {
@@ -563,7 +568,7 @@ std::size_t MultiEdit::RowAt(Y y) const
     if ((format & FORMAT_TOP) || m_contents_sz.y - ClientSize().y < 0) {
         retval = Value(y / GetFont()->Lineskip());
     } else { // FORMAT_BOTTOM
-        retval = GetLineData().size() - 1 -
+        retval = NumLines() -
             Value((ClientSize().y + (m_vscroll && m_hscroll ? BottomMargin() : Y0) - y - 1) / GetFont()->Lineskip());
     }
     return retval;
@@ -571,6 +576,8 @@ std::size_t MultiEdit::RowAt(Y y) const
 
 CPSize MultiEdit::CharAt(std::size_t row, X x) const
 {
+    if (GetLineData().empty())
+        return CP0;
     // out of range of rows?
     if (row >= GetLineData().size()) {
         return CPSize(GetLineData().back().char_data.size());
@@ -614,17 +621,17 @@ CPSize MultiEdit::CharAt(std::size_t row, X x) const
 }
 
 std::size_t MultiEdit::FirstVisibleRow() const
-{ return std::min(RowAt(Y0), GetLineData().size() - 1); }
+{ return std::min(RowAt(Y0), NumLines()); }
 
 std::size_t MultiEdit::LastVisibleRow() const
-{ return std::min(RowAt(ClientSize().y), GetLineData().size() - 1); }
+{ return std::min(RowAt(ClientSize().y), NumLines()); }
 
 std::size_t MultiEdit::FirstFullyVisibleRow() const
 {
     std::size_t retval = RowAt(Y0);
     if (m_first_row_shown % GetFont()->Lineskip())
         ++retval;
-    return std::min(retval, GetLineData().size() - 1);
+    return std::min(retval, NumLines());
 }
 
 std::size_t MultiEdit::LastFullyVisibleRow() const
@@ -632,11 +639,13 @@ std::size_t MultiEdit::LastFullyVisibleRow() const
     std::size_t retval = RowAt(ClientSize().y);
     if ((m_first_row_shown + ClientSize().y + BottomMargin()) % GetFont()->Lineskip())
         --retval;
-    return std::min(retval, GetLineData().size() - 1);
+    return std::min(retval, NumLines());
 }
 
 CPSize MultiEdit::FirstVisibleChar(std::size_t row) const
 {
+    if (GetLineData().empty())
+        return CP0;
     if (GetLineData()[row].Empty())
         return CharAt(row, X0);
     else
@@ -645,6 +654,8 @@ CPSize MultiEdit::FirstVisibleChar(std::size_t row) const
 
 CPSize MultiEdit::LastVisibleChar(std::size_t row) const
 {
+    if (GetLineData().empty())
+        return CP0;
     if (GetLineData()[row].Empty())
         return CharAt(row, ClientSize().x);
     else
@@ -687,15 +698,17 @@ namespace {
 
 std::pair<CPSize, CPSize> MultiEdit::GetDoubleButtonDownWordIndices(CPSize char_index)
 {
+    //std::cout << "GetDoubleButtonDownWordIndices index: " << Value(char_index) << std::endl;
     unsigned int ticks = GUI::GetGUI()->Ticks();
     if (ticks - this->m_last_button_down_time <= GUI::GetGUI()->DoubleClickInterval())
         m_in_double_click_mode = true;
     this->m_last_button_down_time = ticks;
     this->m_double_click_cursor_pos = std::pair<CPSize, CPSize>(CP0, CP0);
     if (m_in_double_click_mode) {
-        std::set<std::pair<CPSize, CPSize> > words;// =
+        //std::cout << "GetDoubleButtonDownWordIndices in double click mode!" << std::endl;
+        std::set<std::pair<CPSize, CPSize>> words =
             GUI::GetGUI()->FindWords(Text());
-        std::set<std::pair<CPSize, CPSize> >::const_iterator it =
+        std::set<std::pair<CPSize, CPSize>>::const_iterator it =
             std::find_if(words.begin(), words.end(), InRange(char_index));
         if (it != words.end())
             this->m_double_click_cursor_pos = *it;
@@ -714,10 +727,18 @@ void MultiEdit::LButtonDown(const Pt& pt, Flags<ModKey> mod_keys)
     m_cursor_begin = m_cursor_end = click_pos;
     //std::cout << "click pos: " << click_pos.first << " / " << click_pos.second << std::endl;
 
-    CPSize begin_cursor_pos = CharIndexOf(m_cursor_begin.first, m_cursor_begin.second);
-    CPSize end_cursor_pos = CharIndexOf(m_cursor_end.first, m_cursor_end.second);
-    this->m_cursor_pos = std::make_pair(begin_cursor_pos, end_cursor_pos);
+    CPSize idx = CharIndexOf(click_pos.first, click_pos.second);
+    this->m_cursor_pos = {idx, idx};
     //std::cout << "cursor pos: " << this->m_cursor_pos.first << std::endl;
+
+    // double-click-drag whole-word selection disabled due to the following code
+    // resulting in weird highlighting glitches, possibly due to inconsistency
+    // between this->m_cursor_pos and  m_cursor_pos and m_cursor_end
+
+    //std::pair<CPSize, CPSize> word_indices = GetDoubleButtonDownWordIndices(idx);
+    ////std::cout << "Edit::LButtonDown got word indices: " << word_indices.first << ", " << word_indices.second << std::endl;
+    //if (word_indices.first != word_indices.second)
+    //    this->m_cursor_pos = word_indices;
 }
 
 void MultiEdit::LDrag(const Pt& pt, const Pt& move, Flags<ModKey> mod_keys)
@@ -768,7 +789,7 @@ void MultiEdit::LDrag(const Pt& pt, const Pt& move, Flags<ModKey> mod_keys)
 
     //std::cout << "MultiEdit::LDrag cursor covers code points: " << begin_cursor_pos << " to " << end_cursor_pos << std::endl << std::flush;
 
-    this->m_cursor_pos = std::make_pair(begin_cursor_pos, end_cursor_pos);
+    this->m_cursor_pos = {begin_cursor_pos, end_cursor_pos};
 
     // if dragging past the currently visible text, adjust
     // the view so more text can be selected
@@ -787,15 +808,17 @@ void MultiEdit::MouseWheel(const Pt& pt, int move, Flags<ModKey> mod_keys)
     SignalScroll(*m_vscroll, true);
 }
 
-void MultiEdit::KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_keys)
+void MultiEdit::KeyPress(Key key, std::uint32_t key_code_point, Flags<ModKey> mod_keys)
 {
     if (Disabled()) {
         TextControl::KeyPress(key, key_code_point, mod_keys);
         return;
     }
 
-    if (m_style & MULTI_READ_ONLY)
+    if (m_style & MULTI_READ_ONLY) {
+        ForwardEventToParent();
         return;
+    }
 
     bool shift_down = mod_keys & (MOD_KEY_LSHIFT | MOD_KEY_RSHIFT);
     bool ctrl_down = mod_keys & (MOD_KEY_CTRL | MOD_KEY_RCTRL);
@@ -830,7 +853,10 @@ void MultiEdit::KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey>
         ++m_cursor_end.first;
         m_cursor_end.second = CP0;
         // the cursor might be off the bottom if the bottom row was just chopped off to satisfy m_max_lines_history
-        if (GetLineData().size() <= m_cursor_end.first) {
+        if (GetLineData().empty()) {
+            m_cursor_end.first = 0;
+            m_cursor_end.second = CP0;
+        } else if (GetLineData().size() <= m_cursor_end.first) {
             m_cursor_end.first = GetLineData().size() - 1;
             m_cursor_end.second = CPSize(GetLineData()[m_cursor_end.first].char_data.size());
             if (LineEndsWithEndlineCharacter(GetLineData(), m_cursor_end.first, Text()))
@@ -847,10 +873,15 @@ void MultiEdit::KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey>
         } else if (0 < m_cursor_end.second) {
             --m_cursor_end.second;
         } else if (0 < m_cursor_end.first) {
-            --m_cursor_end.first;
-            m_cursor_end.second = CPSize(GetLineData()[m_cursor_end.first].char_data.size());
-            if (LineEndsWithEndlineCharacter(GetLineData(), m_cursor_end.first, Text()))
-                --m_cursor_end.second;
+            if (GetLineData().empty()) {
+                m_cursor_end.first = 0;
+                m_cursor_end.second = CP0;
+            } else {
+                --m_cursor_end.first;
+                m_cursor_end.second = CPSize(GetLineData()[m_cursor_end.first].char_data.size());
+                if (LineEndsWithEndlineCharacter(GetLineData(), m_cursor_end.first, Text()))
+                    --m_cursor_end.second;
+            }
         }
         if (!shift_down)
             m_cursor_begin = m_cursor_end;
@@ -860,11 +891,12 @@ void MultiEdit::KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey>
     case GGK_RIGHT: {
         if (MultiSelected() && !shift_down) {
             m_cursor_begin = m_cursor_end = HighCursorPos();
-        } else if (m_cursor_end.second <
+        } else if (!GetLineData().empty() && m_cursor_end.second <
                     GetLineData()[m_cursor_end.first].char_data.size() -
-                    (LineEndsWithEndlineCharacter(GetLineData(), m_cursor_end.first, Text()) ? 1 : 0)) {
+                    (LineEndsWithEndlineCharacter(GetLineData(), m_cursor_end.first, Text()) ? 1 : 0))
+        {
             ++m_cursor_end.second;
-        } else if (m_cursor_end.first < GetLineData().size() - 1) {
+        } else if (!GetLineData().empty() && m_cursor_end.first < GetLineData().size() - 1) {
             ++m_cursor_end.first;
             m_cursor_end.second = CP0;
         }
@@ -890,7 +922,7 @@ void MultiEdit::KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey>
     case GGK_DOWN: {
         if (MultiSelected() && !shift_down) {
             m_cursor_begin = m_cursor_end = HighCursorPos();
-        } else if (m_cursor_end.first < GetLineData().size() - 1) {
+        } else if (!GetLineData().empty() && m_cursor_end.first < GetLineData().size() - 1) {
             X row_start = RowStartX(m_cursor_end.first);
             X char_offset = CharXOffset(m_cursor_end.first, m_cursor_end.second);
             ++m_cursor_end.first;
@@ -937,9 +969,14 @@ void MultiEdit::KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey>
             m_vscroll->ScrollPageIncr();
             SignalScroll(*m_vscroll, true);
             std::size_t rows_moved = m_vscroll->PageSize() / Value(GetFont()->Lineskip());
-            m_cursor_end.first = std::min(m_cursor_end.first + rows_moved, GetLineData().size() - 1);
-            if (GetLineData()[m_cursor_end.first].char_data.size() < m_cursor_end.second)
-                m_cursor_end.second = CPSize(GetLineData()[m_cursor_end.first].char_data.size());
+            if (GetLineData().empty()) {
+                m_cursor_end.first = 0;
+                m_cursor_end.second = CP0;
+            } else {
+                m_cursor_end.first = std::min(m_cursor_end.first + rows_moved, NumLines());
+                if (GetLineData()[m_cursor_end.first].char_data.size() < m_cursor_end.second)
+                    m_cursor_end.second = CPSize(GetLineData()[m_cursor_end.first].char_data.size());
+            }
             m_cursor_begin = m_cursor_end;
         }
         break;
@@ -953,7 +990,7 @@ void MultiEdit::KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey>
             m_cursor_end.second = --m_cursor_begin.second;
             Erase(m_cursor_begin.first, m_cursor_begin.second, CP1);
             emit_signal = true;
-        } else if (0 < m_cursor_begin.first) {
+        } else if (!GetLineData().empty() && 0 < m_cursor_begin.first) {
             m_cursor_end.first = --m_cursor_begin.first;
             m_cursor_begin.second = CPSize(GetLineData()[m_cursor_begin.first].char_data.size());
             if (LineEndsWithEndlineCharacter(GetLineData(), m_cursor_begin.first, Text()))
@@ -969,7 +1006,7 @@ void MultiEdit::KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey>
         if (MultiSelected()) {
             ClearSelected();
             emit_signal = true;
-        } else if (m_cursor_begin.second < GetLineData()[m_cursor_begin.first].char_data.size()) {
+        } else if (!GetLineData().empty() && m_cursor_begin.second < GetLineData()[m_cursor_begin.first].char_data.size()) {
             // cursor is not at the end of its current line
             Erase(m_cursor_begin.first, m_cursor_begin.second, CP1);
             emit_signal = true;
@@ -995,7 +1032,7 @@ void MultiEdit::KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey>
 
     CPSize begin_cursor_pos = CharIndexOf(m_cursor_begin.first, m_cursor_begin.second);
     CPSize end_cursor_pos = CharIndexOf(m_cursor_end.first, m_cursor_end.second);
-    this->m_cursor_pos = std::make_pair(begin_cursor_pos, end_cursor_pos);
+    this->m_cursor_pos = {begin_cursor_pos, end_cursor_pos};
 
     AdjustView();
     if (emit_signal)
@@ -1020,7 +1057,7 @@ void MultiEdit::RecreateScrolls()
 {
     delete m_vscroll;
     delete m_hscroll;
-    m_vscroll = m_hscroll = 0;
+    m_vscroll = m_hscroll = nullptr;
     AdjustScrolls();
 }
 
@@ -1072,7 +1109,7 @@ void MultiEdit::ClearSelected()
 
     CPSize cursor_pos = CharIndexOf(m_cursor_end.first, m_cursor_end.second);
     //std::cout << "got cursor pos: " << cursor_pos << std::endl;
-    this->m_cursor_pos = std::make_pair(cursor_pos, cursor_pos);
+    this->m_cursor_pos = {cursor_pos, cursor_pos};
 }
 
 void MultiEdit::AdjustView()
@@ -1199,7 +1236,7 @@ void MultiEdit::AdjustScrolls()
 
     const int GAP = PIXEL_MARGIN - 2; // the space between the client area and the border
 
-    boost::shared_ptr<StyleFactory> style = GetStyleFactory();
+    std::shared_ptr<StyleFactory> style = GetStyleFactory();
 
     Y vscroll_min = (m_style & MULTI_TERMINAL_STYLE) ? cl_sz.y - m_contents_sz.y : Y0;
     if (cl_sz.y - m_contents_sz.y > 0 ) {
@@ -1218,7 +1255,7 @@ void MultiEdit::AdjustScrolls()
     if (m_vscroll) { // if scroll already exists...
         if (!need_vert) { // remove scroll
             DeleteChild(m_vscroll);
-            m_vscroll = 0;
+            m_vscroll = nullptr;
         } else { // ensure vertical scroll has the right logical and physical dimensions
             unsigned int line_size = (m_vscroll_wheel_scroll_increment != 0
                                         ? m_vscroll_wheel_scroll_increment
@@ -1254,7 +1291,7 @@ void MultiEdit::AdjustScrolls()
     if (m_hscroll) { // if scroll already exists...
         if (!need_horz) { // remove scroll
             DeleteChild(m_hscroll);
-            m_hscroll = 0;
+            m_hscroll = nullptr;
         } else { // ensure horizontal scroll has the right logical and physical dimensions
             unsigned int line_size = (m_hscroll_wheel_scroll_increment != 0
                                         ? m_hscroll_wheel_scroll_increment
@@ -1356,7 +1393,10 @@ void MultiEdit::AcceptPastedText(const std::string& text)
 
         // the cursor might be off the bottom if the bottom row was just
         // chopped off to satisfy m_max_lines_history
-        if (GetLineData().size() - 1 < m_cursor_begin.first) {
+        if (GetLineData().empty()) {
+            m_cursor_begin.first = 0;
+            m_cursor_begin.second = CP0;
+        } else if (GetLineData().size() - 1 < m_cursor_begin.first) {
             m_cursor_begin.first = GetLineData().size() - 1;
             m_cursor_begin.second = CPSize(GetLineData()[m_cursor_begin.first].char_data.size());
         }
@@ -1367,7 +1407,7 @@ void MultiEdit::AcceptPastedText(const std::string& text)
 
     CPSize begin_cursor_pos = CharIndexOf(m_cursor_begin.first, m_cursor_begin.second);
     CPSize end_cursor_pos = CharIndexOf(m_cursor_end.first, m_cursor_end.second);
-    this->m_cursor_pos = std::make_pair(begin_cursor_pos, end_cursor_pos);
+    this->m_cursor_pos = {begin_cursor_pos, end_cursor_pos};
 
     //std::cout << "after converting cursor begin/end to cursor pos, cursor pos: " << m_cursor_pos.first << " - " << m_cursor_pos.second << std::endl;
 
