@@ -323,7 +323,7 @@ namespace {
         target.w = width;
         target.h = height;
         target.format = 0; // DOn't care
-        target.driverdata = 0;
+        target.driverdata = nullptr;
         target.refresh_rate = 0;
         SDL_DisplayMode closest;
         SDL_GetClosestDisplayMode(display_id, &target, &closest);
@@ -364,7 +364,7 @@ namespace {
             std::exception(), exit_code(exit_code_)
         {}
 
-        virtual const char* what() const throw()
+        const char* what() const noexcept override
         { return  exit_code ? "Quitting SDLGUI with error." : "Quitting SDLGUI normally."; }
 
         int exit_code;
@@ -376,7 +376,7 @@ namespace {
             m_status(status)
         {}
 
-        virtual const char* what() const throw() {
+        const char* what() const noexcept override {
             switch (m_status) {
                 case GL_FRAMEBUFFER_UNSUPPORTED_EXT:
                     return "The requested framebuffer format was unsupported";
@@ -415,7 +415,7 @@ namespace GG {
             glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
             glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_FALSE);
             glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, width, height, 0,
-                         GL_RGBA, GL_UNSIGNED_BYTE, 0);
+                         GL_RGBA, GL_UNSIGNED_BYTE, nullptr);
             glBindTexture(GL_TEXTURE_2D, 0);
 
             // create a renderbuffer object to store depth and stencil info
@@ -486,9 +486,10 @@ SDLGUI::SDLGUI(int w/* = 1024*/, int h/* = 768*/, bool calc_FPS/* = false*/, con
     m_fullscreen(fullscreen),
     m_fake_mode_change(fake_mode_change),
     m_display_id(0),
-    m_window(NULL),
+    m_window(nullptr),
+    m_gl_context(nullptr),
     m_done(false),
-    m_framebuffer(NULL),
+    m_framebuffer(nullptr),
     m_key_map()
 {
     SDLInit();
@@ -651,7 +652,7 @@ void SDLGUI::SDLInit()
         }
 
         SDL_ShowSimpleMessageBox(
-            SDL_MESSAGEBOX_ERROR, "OpenGL initialization error", msg.c_str(), 0);
+            SDL_MESSAGEBOX_ERROR, "OpenGL initialization error", msg.c_str(), nullptr);
         std::cerr << msg << std::endl;
         Exit(1);
     }
@@ -693,7 +694,7 @@ void SDLGUI::HandleSystemEvents()
         bool send_to_gg = false;
         EventType gg_event = MOUSEMOVE;
         Key key = GGK_UNKNOWN;
-        boost::uint32_t key_code_point = 0;
+        std::uint32_t key_code_point = 0;
         Flags<ModKey> mod_keys = GetSDLModKeys();
         // In GiGi some events contain mouse position info,
         // where the corresponding sdl event does not.
@@ -781,8 +782,15 @@ void SDLGUI::HandleSystemEvents()
                 case SDL_WINDOWEVENT_MOVED:
                     WindowMovedSignal(X(event.window.data1), Y(event.window.data2));
                     break;
+                case SDL_WINDOWEVENT_MINIMIZED:
+                    SDL_EnableScreenSaver();
+                    break;
+                case SDL_WINDOWEVENT_MAXIMIZED:
+                case SDL_WINDOWEVENT_RESTORED:
+                    SDL_DisableScreenSaver();
+                    break;
                 case SDL_WINDOWEVENT_CLOSE:
-
+                    WindowClosingSignal();
                     break;
             }
             break;
@@ -799,7 +807,7 @@ void SDLGUI::HandleNonGGEvent(const SDL_Event& event)
 {
     switch (event.type) {
     case SDL_QUIT:
-        Exit(0);
+        AppQuittingSignal();
         break;
     }
 }
@@ -848,12 +856,12 @@ void SDLGUI::FinalCleanup()
 {
     if (m_gl_context){
         SDL_GL_DeleteContext(m_gl_context);
-        m_gl_context = 0;
+        m_gl_context = nullptr;
     }
 
     if (m_window) {
         SDL_DestroyWindow(m_window);
-        m_window = 0;
+        m_window = nullptr;
     }
 }
 
@@ -899,7 +907,7 @@ std::vector<std::string> SDLGUI::GetSupportedResolutions() const
             if (SDL_GetDisplayMode(m_display_id, i, &mode) != 0) {
                 SDL_Log("SDL_GetDisplayMode failed: %s", SDL_GetError());
             } else {
-                mode_vec.push_back(boost::io::str(boost::format("%1% x %2% @ %3%") % mode.w % mode.h % SDL_BITSPERPIXEL(mode.format)));
+                mode_vec.push_back(boost::io::str(boost::format("%1% x %2%") % mode.w % mode.h));
             }
         }
     }
@@ -917,7 +925,7 @@ void SDLGUI::SDLMinimalInit()
     }
 }
 
-Pt SDLGUI::GetDefaultResolution(int display_id)
+Pt SDLGUI::GetDefaultResolution(int display_id) const
 { return GetDefaultResolutionStatic(display_id); }
 
 Pt SDLGUI::GetDefaultResolutionStatic(int display_id)
@@ -977,7 +985,7 @@ void SDLGUI::RelayTextInput(const SDL_TextInputEvent& text, GG::Pt mouse_pos)
 
 void SDLGUI::ResetFramebuffer()
 {
-    m_framebuffer.reset(NULL);
+    m_framebuffer.reset();
     if (m_fake_mode_change && m_fullscreen) {
         try {
             m_framebuffer.reset(new Framebuffer(Pt(m_app_width, m_app_height)));
