@@ -61,16 +61,16 @@ Scroll::Scroll(Orientation orientation, Clr color, Clr interior) :
     m_range_max(99),
     m_line_sz(5),
     m_page_sz(25),
-    m_tab(0),
-    m_incr(0),
-    m_decr(0),
+    m_tab(nullptr),
+    m_incr(nullptr),
+    m_decr(nullptr),
     m_initial_depressed_region(SBR_NONE),
     m_depressed_region(SBR_NONE),
     m_dragging_tab(false),
     m_tab_dragged(false)
 {
     Control::SetColor(color);
-    boost::shared_ptr<StyleFactory> style = GetStyleFactory();
+    std::shared_ptr<StyleFactory> style = GetStyleFactory();
     if (m_orientation == VERTICAL) {
         m_decr = style->NewScrollUpButton(color);
         m_incr = style->NewScrollDownButton(color);
@@ -428,7 +428,7 @@ void Scroll::UpdatePosn()
 void Scroll::MoveTabToPosn()
 {
     int start_tabspace = 0; // the tab's lowest posible extent
-    if(m_decr)
+    if (m_decr)
         start_tabspace = (m_orientation == VERTICAL ?
                           Value(m_decr->Size().y) :
                           Value(m_decr->Size().x));
