@@ -27,8 +27,6 @@
 #include <GG/ClrConstants.h>
 #include <GG/DrawUtil.h>
 
-#include <boost/assign/list_of.hpp>
-
 
 using namespace GG;
 
@@ -71,7 +69,7 @@ namespace {
 ////////////////////////////////////////////////
 // GG::StaticGraphic
 ////////////////////////////////////////////////
-StaticGraphic::StaticGraphic(const boost::shared_ptr<Texture>& texture, Flags<GraphicStyle> style/* = GRAPHIC_NONE*/,
+StaticGraphic::StaticGraphic(const std::shared_ptr<Texture>& texture, Flags<GraphicStyle> style/* = GRAPHIC_NONE*/,
                              Flags<WndFlag> flags/* = 0*/) :
     Control(X0, Y0, X1, Y1, flags),
     m_style(style)
@@ -165,6 +163,12 @@ void StaticGraphic::SetStyle(Flags<GraphicStyle> style)
     ValidateStyle();
 }
 
+void StaticGraphic::SetTexture(const std::shared_ptr<Texture>& texture)
+{ SetTexture(SubTexture(texture, X0, Y0, texture->DefaultWidth(), texture->DefaultHeight())); }
+
+void StaticGraphic::SetTexture(const SubTexture& subtexture)
+{ m_graphic = subtexture; }
+
 void StaticGraphic::Init(const SubTexture& subtexture)
 {
     ValidateStyle();  // correct any disagreements in the style flags
