@@ -41,7 +41,7 @@ namespace {
         std::string m_name;
     };
 
-    Y TabHeightFromFont(const boost::shared_ptr<Font>& font)
+    Y TabHeightFromFont(const std::shared_ptr<Font>& font)
     { return font->Lineskip() + 10; }
 }
 
@@ -58,16 +58,16 @@ OverlayWnd::OverlayWnd(X x, Y y, X w, Y h, Flags<WndFlag> flags) :
 
 OverlayWnd::~OverlayWnd()
 {
-    for (std::size_t i = 0; i < m_wnds.size(); ++i) {
-        delete m_wnds[i];
+    for (Wnd* wnd : m_wnds) {
+        delete wnd;
     }
 }
 
 Pt OverlayWnd::MinUsableSize() const
 {
     Pt retval;
-    for (std::size_t i = 0; i < m_wnds.size(); ++i) {
-        Pt min_usable_size = m_wnds[i]->MinUsableSize();
+    for (Wnd* wnd : m_wnds) {
+        Pt min_usable_size = wnd->MinUsableSize();
         retval.x = std::max(retval.x, min_usable_size.x);
         retval.y = std::max(retval.y, min_usable_size.y);
     }
@@ -81,7 +81,7 @@ std::size_t OverlayWnd::NumWnds() const
 { return m_wnds.size(); }
 
 Wnd* OverlayWnd::CurrentWnd() const
-{ return m_current_wnd_index == NO_WND ? 0 : m_wnds[m_current_wnd_index]; }
+{ return m_current_wnd_index == NO_WND ? nullptr : m_wnds[m_current_wnd_index]; }
 
 std::size_t OverlayWnd::CurrentWndIndex() const
 { return m_current_wnd_index; }
@@ -105,7 +105,7 @@ void OverlayWnd::InsertWnd(std::size_t index, Wnd* wnd)
 
 Wnd* OverlayWnd::RemoveWnd(std::size_t index)
 {
-    Wnd* retval = 0;
+    Wnd* retval = nullptr;
     if (index < m_wnds.size()) {
         std::vector<Wnd*>::iterator it = m_wnds.begin() + index;
         retval = *it;
@@ -118,7 +118,7 @@ Wnd* OverlayWnd::RemoveWnd(std::size_t index)
 
 Wnd* OverlayWnd::RemoveWnd(Wnd* wnd)
 {
-    Wnd* retval = 0;
+    Wnd* retval = nullptr;
     std::vector<Wnd*>::iterator it = std::find(m_wnds.begin(), m_wnds.end(), wnd);
     if (it != m_wnds.end()) {
         if (it - m_wnds.begin() == static_cast<std::ptrdiff_t>(m_current_wnd_index))
@@ -163,7 +163,7 @@ void OverlayWnd::SetCurrentWnd(std::size_t index)
 // static(s)
 const std::size_t TabWnd::NO_WND = std::numeric_limits<std::size_t>::max();
 
-TabWnd::TabWnd(X x, Y y, X w, Y h, const boost::shared_ptr<Font>& font, Clr color,
+TabWnd::TabWnd(X x, Y y, X w, Y h, const std::shared_ptr<Font>& font, Clr color,
                Clr text_color/* = CLR_BLACK*/) :
     Wnd(x, y, w, h, INTERACTIVE),
     m_tab_bar(GetStyleFactory()->NewTabBar(font, color, text_color)),
@@ -265,13 +265,13 @@ void TabWnd::TabChanged(std::size_t index, bool signal)
 const std::size_t TabBar::NO_TAB = TabWnd::NO_WND;
 const X TabBar::BUTTON_WIDTH(10);
 
-TabBar::TabBar(const boost::shared_ptr<Font>& font, Clr color, Clr text_color/* = CLR_BLACK*/,
+TabBar::TabBar(const std::shared_ptr<Font>& font, Clr color, Clr text_color/* = CLR_BLACK*/,
                Flags<WndFlag> flags/* = INTERACTIVE*/) :
     Control(X0, Y0, X1, TabHeightFromFont(font), flags),
-    m_tabs(0),
+    m_tabs(nullptr),
     m_font(font),
-    m_left_button(0),
-    m_right_button(0),
+    m_left_button(nullptr),
+    m_right_button(nullptr),
     m_left_right_button_layout(new Layout(X0, Y0, X1, TabHeightFromFont(font), 1, 3)),
     m_text_color(text_color),
     m_first_tab_shown(0)
@@ -280,7 +280,7 @@ TabBar::TabBar(const boost::shared_ptr<Font>& font, Clr color, Clr text_color/*
 
     SetChildClippingMode(ClipToClient);
 
-    boost::shared_ptr<StyleFactory> style_factory = GetStyleFactory();
+    std::shared_ptr<StyleFactory> style_factory = GetStyleFactory();
 
     m_tabs = style_factory->NewRadioButtonGroup(HORIZONTAL);
     m_tabs->ExpandButtons(true);
@@ -316,8 +316,8 @@ TabBar::TabBar(const boost::shared_ptr<Font>& font, Clr color, Clr text_color/*
 Pt TabBar::MinUsableSize() const
 {
     Y y(0);
-    for (std::size_t i = 0; i < m_tab_buttons.size(); ++i) {
-        Y button_min_y = m_tab_buttons[i]->MinUsableSize().y;
+    for (StateButton* button : m_tab_buttons) {
+        Y button_min_y = button->MinUsableSize().y;
         if (y < button_min_y)
             y = button_min_y;
     }
@@ -376,7 +376,7 @@ std::size_t TabBar::AddTab(const std::string& name)
 void TabBar::InsertTab(std::size_t index, const std::string& name)
 {
     assert(index <= m_tab_buttons.size());
-    boost::shared_ptr<StyleFactory> style_factory = GetStyleFactory();
+    std::shared_ptr<StyleFactory> style_factory = GetStyleFactory();
     StateButton* button = style_factory->NewTabBarTab(name,
                                                       m_font, FORMAT_CENTER, Color(),
                                                       m_text_color);
