@@ -27,7 +27,6 @@
 #include <GG/DrawUtil.h>
 #include <GG/utf8/checked.h>
 
-#include <boost/assign/list_of.hpp>
 
 using namespace GG;
 
@@ -38,7 +37,7 @@ namespace {
 ////////////////////////////////////////////////
 // GG::TextControl
 ////////////////////////////////////////////////
-TextControl::TextControl(X x, Y y, X w, Y h, const std::string& str, const boost::shared_ptr<Font>& font, Clr color/* = CLR_BLACK*/,
+TextControl::TextControl(X x, Y y, X w, Y h, const std::string& str, const std::shared_ptr<Font>& font, Clr color/* = CLR_BLACK*/,
                          Flags<TextFormat> format/* = FORMAT_NONE*/, Flags<WndFlag> flags/* = NO_WND_FLAGS*/) :
     Control(x, y, w, h, flags),
     m_format(format),
@@ -47,7 +46,7 @@ TextControl::TextControl(X x, Y y, X w, Y h, const std::string& str, const boost
     m_set_min_size(false),
     m_code_points(0),
     m_font(font),
-    m_render_cache(0),
+    m_render_cache(nullptr),
     m_cached_minusable_size_width(X0),
     m_cached_minusable_size(Pt())
 {
@@ -55,10 +54,72 @@ TextControl::TextControl(X x, Y y, X w, Y h, const std::string& str, const boost
     SetText(str);
 }
 
+TextControl::TextControl(X x, Y y, X w, Y h, const std::string& str,
+                         const std::vector<std::shared_ptr<Font::TextElement>>& text_elements,
+                         const std::shared_ptr<Font>& font,
+                         Clr color /*= CLR_BLACK*/, Flags<TextFormat> format /*= FORMAT_NONE*/,
+                         Flags<WndFlag> flags /*= NO_WND_FLAGS*/) :
+    Control(x, y, w, h, flags),
+    m_text(),
+    m_format(format),
+    m_text_color(color),
+    m_clip_text(false),
+    m_set_min_size(false),
+    m_text_elements(),
+    m_code_points(0),
+    m_font(font),
+    m_render_cache(nullptr),
+    m_cached_minusable_size_width(X0),
+    m_cached_minusable_size(Pt())
+{
+    ValidateFormat();
+    SetText(str, text_elements);
+}
+
+TextControl::TextControl(const TextControl& that) :
+    Control(that.Left(), that.Top(), that.Width(), that.Height()),
+    m_text(that.m_text),
+    m_format(that.m_format),
+    m_text_color(that.m_text_color),
+    m_clip_text(that.m_clip_text),
+    m_set_min_size(that.m_set_min_size),
+    m_text_elements(that.m_text_elements),
+    m_code_points(that.m_code_points),
+    m_font(that.m_font),
+    m_render_cache(that.m_render_cache),
+    m_cached_minusable_size_width(that.m_cached_minusable_size_width),
+    m_cached_minusable_size(that.m_cached_minusable_size)
+{
+    for (std::shared_ptr<Font::TextElement> elem : m_text_elements) {
+        elem->Bind(m_text);
+    }
+}
+
 TextControl::~TextControl()
 {
     delete m_render_cache;
-    m_render_cache = 0;
+    m_render_cache = nullptr;
+}
+
+TextControl& TextControl::operator=(const TextControl& that)
+{
+    m_text = that.m_text;
+    m_format = that.m_format;
+    m_text_color = that.m_text_color;
+    m_clip_text = that.m_clip_text;
+    m_set_min_size = that.m_set_min_size;
+    m_text_elements = that.m_text_elements;
+    m_code_points = that.m_code_points;
+    m_font = that.m_font;
+    m_render_cache = that.m_render_cache;
+    m_cached_minusable_size_width = that.m_cached_minusable_size_width;
+    m_cached_minusable_size = that.m_cached_minusable_size;
+
+    for (std::shared_ptr<Font::TextElement> elem : m_text_elements) {
+        elem->Bind(m_text);
+    }
+
+    return *this;
 }
 
 Pt TextControl::MinUsableSize() const
@@ -66,14 +127,19 @@ Pt TextControl::MinUsableSize() const
 
 Pt TextControl::MinUsableSize(X width) const
 {
+    // If the requested width is within one space width of the cached width
+    // don't recalculate the size
     X min_delta = m_font->SpaceWidth();
     X abs_delta_w = X(std::abs(Value(m_cached_minusable_size_width - width)));
-    if ( m_cached_minusable_size_width != X0 &&  abs_delta_w < min_delta)
+    if (m_cached_minusable_size_width != X0 &&  abs_delta_w < min_delta)
         return m_cached_minusable_size;
 
-    std::vector<Font::LineData> dummy_line_data;
+    // Calculate and cache the minimum usable size when m_cached_minusable_size is equal to width.
+    // Create dummy line data with line breaks added so that lines are not wider than width.
     Flags<TextFormat> dummy_format(m_format);
-    m_cached_minusable_size = m_font->DetermineLines(m_text, dummy_format, width, dummy_line_data)
+    std::vector<Font::LineData> dummy_line_data =
+        m_font->DetermineLines(m_text, dummy_format, width, m_text_elements);
+    m_cached_minusable_size = m_font->TextExtent(dummy_line_data)
         + (ClientUpperLeft() - UpperLeft()) + (LowerRight() - ClientLowerRight());
     m_cached_minusable_size_width = width;
     return m_cached_minusable_size;
@@ -91,11 +157,11 @@ std::string TextControl::Text(CPSize from, CPSize to) const
 
     //std::cout << "low: " << low << "  high: " << high << std::endl << std::flush;
 
-    std::pair<std::size_t, CPSize> low_pos = LinePositionOf(low, GetLineData());
-    std::pair<std::size_t, CPSize> high_pos = LinePositionOf(high, GetLineData());
+    std::pair<std::size_t, CPSize> low_pos = LinePositionOf(low, m_line_data);
+    std::pair<std::size_t, CPSize> high_pos = LinePositionOf(high, m_line_data);
 
-    StrSize low_string_idx = StringIndexOf(low_pos.first, low_pos.second, GetLineData());
-    StrSize high_string_idx = StringIndexOf(high_pos.first, high_pos.second, GetLineData());
+    StrSize low_string_idx = StringIndexOf(low_pos.first, low_pos.second, m_line_data);
+    StrSize high_string_idx = StringIndexOf(high_pos.first, high_pos.second, m_line_data);
 
     std::string::const_iterator low_it = m_text.begin() + Value(low_string_idx);
     std::string::const_iterator high_it = m_text.begin() + Value(high_string_idx);
@@ -143,7 +209,7 @@ void TextControl::Render()
     Clr clr_to_use = Disabled() ? DisabledColor(TextColor()) : TextColor();
     glColor(clr_to_use);
     if (m_font) {
-        if (m_render_cache == 0) {
+        if (!m_render_cache) {
             RefreshCache();
         }
         if (m_clip_text)
@@ -162,7 +228,7 @@ void TextControl::RefreshCache() {
     PurgeCache();
     m_render_cache = new Font::RenderCache();
     if (m_font)
-        m_font->PreRenderText(Pt(X0, Y0), Size(), m_text, m_format, *m_render_cache, &m_line_data);
+        m_font->PreRenderText(Pt(X0, Y0), Size(), m_text, m_format, *m_render_cache, m_line_data);
 }
 
 void TextControl::PurgeCache()
@@ -170,7 +236,7 @@ void TextControl::PurgeCache()
     if (m_render_cache)
         delete m_render_cache;
 
-    m_render_cache = 0;
+    m_render_cache = nullptr;
 }
 
 void TextControl::SetText(const std::string& str)
@@ -182,13 +248,49 @@ void TextControl::SetText(const std::string& str)
     if (!m_font)
         return;
 
-    m_code_points = CPSize(utf8::distance(str.begin(), str.end()));
-    m_text_elements.clear();
-    Pt text_sz =
-        m_font->DetermineLines(m_text, m_format, ClientSize().x, m_line_data, m_text_elements);
+    m_text_elements = m_font->ExpensiveParseFromTextToTextElements(m_text, m_format);
+    RecomputeLineData();
+}
+
+void TextControl::SetText(const std::string& str,
+                          const std::vector<std::shared_ptr<Font::TextElement>>& text_elements)
+{
+    if (!utf8::is_valid(str.begin(), str.end()))
+        return;
+
+    std::size_t expected_length(0);
+    for (std::shared_ptr<Font::TextElement> elem : text_elements) {
+        expected_length += elem->text.size();
+    }
+
+    if (expected_length > str.size())
+        return;
+
+    m_text = str;
+
+    m_text_elements = text_elements;
+    for (std::shared_ptr<Font::TextElement> elem : m_text_elements) {
+        elem->Bind(m_text);
+    }
+
+    RecomputeLineData();
+}
+
+void TextControl::ChangeTemplatedText(const std::string& new_text, size_t targ_offset) {
+    m_font->ChangeTemplatedText(m_text, m_text_elements, new_text, targ_offset);
+    RecomputeLineData();
+}
+
+void TextControl::RecomputeLineData() {
+    if (!m_font)
+        return;
+
+    m_code_points = CPSize(utf8::distance(m_text.begin(), m_text.end()));
+
+    m_line_data = m_font->DetermineLines(m_text, m_format, ClientSize().x, m_text_elements);
+    Pt text_sz = m_font->TextExtent(m_line_data);
     m_text_ul = Pt();
     m_text_lr = text_sz;
-    AdjustMinimumSize();
     PurgeCache();
     if (m_format & FORMAT_NOWRAP) {
         Resize(text_sz);
@@ -199,10 +301,10 @@ void TextControl::SetText(const std::string& str)
     m_cached_minusable_size_width = X0;
 }
 
-const boost::shared_ptr<Font>& TextControl::GetFont() const
+const std::shared_ptr<Font>& TextControl::GetFont() const
 { return m_font; }
 
-void TextControl::SetFont(boost::shared_ptr<Font> font)
+void TextControl::SetFont(std::shared_ptr<Font> font)
 {
     m_font = font;
     SetText(m_text);
@@ -232,17 +334,12 @@ void TextControl::SizeMove(const Pt& ul, const Pt& lr)
     }
 
     if (redo_determine_lines) {
-        Pt text_sz;
-        if (m_text_elements.empty()) {
-            text_sz =
-                m_font->DetermineLines(m_text, m_format, client_width, m_line_data, m_text_elements);
-        } else {
-            text_sz =
-                m_font->DetermineLines(m_text, m_format, client_width, m_text_elements, m_line_data);
-        }
+        if (m_text_elements.empty())
+            m_text_elements = m_font->ExpensiveParseFromTextToTextElements(m_text, m_format);
+        m_line_data = m_font->DetermineLines(m_text, m_format, client_width, m_text_elements);
+        Pt text_sz = m_font->TextExtent(m_line_data);
         m_text_ul = Pt();
         m_text_lr = text_sz;
-        AdjustMinimumSize();
         PurgeCache();
     }
     RecomputeTextBounds();
@@ -294,21 +391,21 @@ void TextControl::Clear()
 void TextControl::Insert(CPSize pos, char c)
 {
     std::size_t line;
-    boost::tie(line, pos) = LinePositionOf(pos, m_line_data);
+    std::tie(line, pos) = LinePositionOf(pos, m_line_data);
     Insert(line, pos, c);
 }
 
 void TextControl::Insert(CPSize pos, const std::string& s)
 {
     std::size_t line;
-    boost::tie(line, pos) = LinePositionOf(pos, m_line_data);
+    std::tie(line, pos) = LinePositionOf(pos, m_line_data);
     Insert(line, pos, s);
 }
 
 void TextControl::Erase(CPSize pos, CPSize num/* = CP1*/)
 {
     std::size_t line;
-    boost::tie(line, pos) = LinePositionOf(pos, m_line_data);
+    std::tie(line, pos) = LinePositionOf(pos, m_line_data);
     Erase(line, pos, num);
 }
 
