@@ -29,29 +29,15 @@
 #include <GG/DrawUtil.h>
 #include <GG/utf8/checked.h>
 
-#if GG_USE_DEVIL_IMAGE_LOAD_LIBRARY
-# include <IL/il.h>
-# include <IL/ilu.h>
-#else
-# include <boost/filesystem/operations.hpp>
-# if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ > 7)
-#  pragma GCC diagnostic push
-#  pragma GCC diagnostic ignored "-Wunused-local-typedefs"
-# endif
-# include "GIL/extension/dynamic_image/any_image.hpp"
-# if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ > 7)
-#  pragma GCC diagnostic pop
-# endif
-# if GG_HAVE_LIBJPEG
-#  include "GIL/extension/io/jpeg_dynamic_io.hpp"
-# endif
-# if GG_HAVE_LIBPNG
-#  include "GIL/extension/io/png_dynamic_io.hpp"
-# endif
-# if GG_HAVE_LIBTIFF
-#  include "GIL/extension/io/tiff_dynamic_io.hpp"
-# endif
-# include <boost/algorithm/string/case_conv.hpp>
+#include <boost/filesystem/operations.hpp>
+#include <boost/gil/extension/dynamic_image/any_image.hpp>
+#if GG_HAVE_LIBTIFF
+# include <boost/gil/extension/io/tiff_dynamic_io.hpp>
+#endif
+#include <boost/algorithm/string/case_conv.hpp>
+
+#if GG_HAVE_LIBPNG
+# include "gilext/io/png_dynamic_io.hpp"
 #endif
 
 #include <iostream>
@@ -70,57 +56,6 @@ namespace {
             value *= 2;
         return value;
     }
-
-#if GG_USE_DEVIL_IMAGE_LOAD_LIBRARY
-    const bool VERBOSE_DEVIL_ERROR_REPORTING = true;
-
-    void CheckILErrors(const std::string& function_call)
-    {
-        ILuint error;
-        while ((error = ilGetError()) != IL_NO_ERROR) {
-            if (VERBOSE_DEVIL_ERROR_REPORTING) {
-                std::cerr << "IL call \"" << function_call << "\" failed with IL error \"" << iluErrorString(error)
-                          << "\" (code " << error << ")\n";
-            }
-        }
-    }
-
-    std::string ILenumToString(ILenum il_enum)
-    {
-#define ENUM_CASE(x) if (il_enum == x) return #x
-        ENUM_CASE(IL_COLOR_INDEX);
-        ENUM_CASE(IL_RGB);
-        ENUM_CASE(IL_RGBA);
-        ENUM_CASE(IL_BGR);
-        ENUM_CASE(IL_BGRA);
-        ENUM_CASE(IL_LUMINANCE);
-        ENUM_CASE(IL_LUMINANCE_ALPHA);
-        ENUM_CASE(IL_BYTE);
-        ENUM_CASE(IL_UNSIGNED_BYTE);
-        ENUM_CASE(IL_SHORT);
-        ENUM_CASE(IL_UNSIGNED_SHORT);
-        ENUM_CASE(IL_INT);
-        ENUM_CASE(IL_UNSIGNED_INT);
-        ENUM_CASE(IL_FLOAT);
-        ENUM_CASE(IL_DOUBLE);
-#undef ENUM_CASE
-        return "UNKNOWN";
-    }
-
-    bool g_il_initialized = false;
-    void InitDevIL()
-    {
-        if (!g_il_initialized) {
-            // ensure we're starting with an empty error stack
-            while (ilGetError() != IL_NO_ERROR) ;
-            ilInit();
-            CheckILErrors("ilInit()");
-            iluInit();
-            CheckILErrors("iluInit()");
-            g_il_initialized = true;
-        }
-    }
-#endif
 }
 
 ///////////////////////////////////////
@@ -279,55 +214,6 @@ void Texture::Load(const boost::filesystem::path& path, bool mipmap/* = false*/)
     std::string filename = path.generic_string();
 #endif
 
-
-#if GG_USE_DEVIL_IMAGE_LOAD_LIBRARY
-
-    InitDevIL();
-
-    ILuint id, error;
-    ilGenImages(1, &id);
-    CheckILErrors("ilGenImages(1, &id)");
-    ilBindImage(id);
-    CheckILErrors("ilBindImage(id)");
-    ilLoadImage(const_cast<char*>(filename.c_str()));
-    if ((error = ilGetError()) != IL_NO_ERROR) {
-        std::string call = "ilLoadImage(\"" + filename + "\")";
-        if (VERBOSE_DEVIL_ERROR_REPORTING) {
-            std::cerr << "IL call \"" << call << "\" failed with IL error \"" << iluErrorString(error)
-                      << "\" (code " << error << ")\n";
-        }
-        CheckILErrors(call);
-        throw BadFile("Could not load temporary DevIL image from file \"" + filename + "\"");
-    }
-
-    m_path = path;
-    m_default_width = X(ilGetInteger(IL_IMAGE_WIDTH));
-    CheckILErrors("ilGetInteger(IL_IMAGE_WIDTH)");
-    m_default_height = Y(ilGetInteger(IL_IMAGE_HEIGHT));
-    CheckILErrors("ilGetInteger(IL_IMAGE_HEIGHT)");
-    m_bytes_pp = ilGetInteger(IL_IMAGE_BYTES_PER_PIXEL);
-    CheckILErrors("ilGetInteger(IL_IMAGE_BYTES_PER_PIXEL)");
-    m_format = ilGetInteger(IL_IMAGE_FORMAT);
-    CheckILErrors("ilGetInteger(IL_IMAGE_FORMAT)");
-    if (m_format == IL_COLOR_INDEX) {
-        m_format = IL_RGBA;
-        m_type = IL_UNSIGNED_BYTE;
-        m_bytes_pp = 4;
-        ilConvertImage(m_format, m_type);
-        CheckILErrors("ilConvertImage(IL_RGBA, IL_UNSIGNED_BYTE)");
-    } else {
-        m_type = ilGetInteger(IL_IMAGE_TYPE);
-        CheckILErrors("ilGetInteger(IL_IMAGE_TYPE)");
-    }
-    ILubyte* image_data = ilGetData();
-    CheckILErrors("ilGetData()");
-    Init(m_default_width, m_default_height, image_data, m_format, m_type, m_bytes_pp, mipmap);
-
-    ilDeleteImages(1, &id);
-    CheckILErrors("ilDeleteImages(1, &id)");
-
-#else
-
     BOOST_STATIC_ASSERT((sizeof(gil::gray8_pixel_t) == 1));
     BOOST_STATIC_ASSERT((sizeof(gil::gray_alpha8_pixel_t) == 2));
     BOOST_STATIC_ASSERT((sizeof(gil::rgb8_pixel_t) == 3));
@@ -352,11 +238,6 @@ void Texture::Load(const boost::filesystem::path& path, bool mipmap/* = false*/)
     try {
         // First attempt -- try just to read the file in one of the default
         // formats above.
-#if GG_HAVE_LIBJPEG
-        if (extension == ".jpg" || extension == ".jpe" || extension == ".jpeg")
-            gil::jpeg_read_image(filename, image);
-        else
-#endif
 #if GG_HAVE_LIBPNG
         if (extension == ".png")
             gil::png_read_image(path, image);
@@ -371,13 +252,6 @@ void Texture::Load(const boost::filesystem::path& path, bool mipmap/* = false*/)
     } catch (const std::ios_base::failure &) {
         // Second attempt -- If *_read_image() throws, see if we can convert
         // the image to RGBA.  This is needed for color-indexed images.
-#if GG_HAVE_LIBJPEG
-        if (extension == ".jpg" || extension == ".jpe" || extension == ".jpeg") {
-            gil::rgba8_image_t rgba_image;
-            gil::jpeg_read_and_convert_image(filename, rgba_image);
-            image.move_in(rgba_image);
-        }
-#endif
 #if GG_HAVE_LIBPNG
         if (extension == ".png") {
             gil::rgba8_image_t rgba_image;
@@ -406,7 +280,7 @@ void Texture::Load(const boost::filesystem::path& path, bool mipmap/* = false*/)
             const_view(image._dynamic_cast<image_prefix ## _image_t>())); \
     }
 
-    const unsigned char* image_data = 0;
+    const unsigned char* image_data = nullptr;
 
     IF_IMAGE_TYPE_IS(gil::gray8)
     else IF_IMAGE_TYPE_IS(gil::gray_alpha8)
@@ -425,8 +299,6 @@ void Texture::Load(const boost::filesystem::path& path, bool mipmap/* = false*/)
 
     assert(image_data);
     Init(m_default_width, m_default_height, image_data, m_format, m_type, m_bytes_pp, mipmap);
-
-#endif
 }
 
 void Texture::Init(X x, Y y, X width, Y height, X image_width, const unsigned char* image,
@@ -537,7 +409,7 @@ void Texture::InitFromRawData(X width, Y height, const unsigned char* image, GLe
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, m_wrap_s);
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, m_wrap_t);
 
-    glTexImage2D(GL_PROXY_TEXTURE_2D, 0, format, Value(GL_texture_width), Value(GL_texture_height), 0, format, type, 0);
+    glTexImage2D(GL_PROXY_TEXTURE_2D, 0, format, Value(GL_texture_width), Value(GL_texture_height), 0, format, type, nullptr);
     GLint checked_format;
     glGetTexLevelParameteriv(GL_PROXY_TEXTURE_2D, 0, GL_TEXTURE_INTERNAL_FORMAT, &checked_format);
     if (!checked_format)
@@ -584,7 +456,7 @@ void Texture::InitFromRawData(X width, Y height, const unsigned char* image, GLe
 
 unsigned char* Texture::GetRawBytes()
 {
-    unsigned char* retval = 0;
+    unsigned char* retval = nullptr;
     glPushClientAttrib(GL_CLIENT_PIXEL_STORE_BIT);
     glPixelStorei(GL_PACK_SWAP_BYTES, false);
     glPixelStorei(GL_PACK_LSB_FIRST, false);
@@ -611,7 +483,7 @@ SubTexture::SubTexture() :
     m_tex_coords()
 {}
 
-SubTexture::SubTexture(const boost::shared_ptr<const Texture>& texture, X x1, Y y1, X x2, Y y2) :
+SubTexture::SubTexture(const std::shared_ptr<const Texture>& texture, X x1, Y y1, X x2, Y y2) :
     m_texture(texture),
     m_width(x2 - x1),
     m_height(y2 - y1),
@@ -626,7 +498,7 @@ SubTexture::SubTexture(const boost::shared_ptr<const Texture>& texture, X x1, Y
     m_tex_coords[3] = Value(y2 * 1.0 / texture->Height());
 }
 
-SubTexture::SubTexture(const boost::shared_ptr<const Texture>& texture) :
+SubTexture::SubTexture(const std::shared_ptr<const Texture>& texture) :
     m_texture(texture),
     m_width(GG::X1),
     m_height(GG::Y1),
@@ -694,18 +566,21 @@ bool TextureManager::s_il_initialized = false;
 TextureManager::TextureManager()
 {}
 
-boost::shared_ptr<Texture> TextureManager::StoreTexture(Texture* texture, const std::string& texture_name)
+const std::map<std::string, std::shared_ptr<Texture>>& TextureManager::Textures() const
+{ return m_textures; }
+
+std::shared_ptr<Texture> TextureManager::StoreTexture(Texture* texture, const std::string& texture_name)
 {
-    boost::shared_ptr<Texture> temp(texture);
+    std::shared_ptr<Texture> temp(texture);
     return StoreTexture(temp, texture_name);
 }
 
-boost::shared_ptr<Texture> TextureManager::StoreTexture(const boost::shared_ptr<Texture>& texture, const std::string& texture_name)
+std::shared_ptr<Texture> TextureManager::StoreTexture(const std::shared_ptr<Texture>& texture, const std::string& texture_name)
 { return (m_textures[texture_name] = texture); }
 
-boost::shared_ptr<Texture> TextureManager::GetTexture(const boost::filesystem::path& path, bool mipmap/* = false*/)
+std::shared_ptr<Texture> TextureManager::GetTexture(const boost::filesystem::path& path, bool mipmap/* = false*/)
 {
-    std::map<std::string, boost::shared_ptr<Texture> >::iterator it = m_textures.find(path.generic_string());
+    std::map<std::string, std::shared_ptr<Texture>>::iterator it = m_textures.find(path.generic_string());
     if (it == m_textures.end()) { // if no such texture was found, attempt to load it now, using name as the filename
         //std::cout << "TextureManager::GetTexture storing new texture under name: " << path.generic_string();
         return (m_textures[path.generic_string()] = LoadTexture(path, mipmap));
@@ -719,14 +594,14 @@ void TextureManager::FreeTexture(const boost::filesystem::path& path)
 
 void TextureManager::FreeTexture(const std::string& name)
 {
-    std::map<std::string, boost::shared_ptr<Texture> >::iterator it = m_textures.find(name);
+    std::map<std::string, std::shared_ptr<Texture>>::iterator it = m_textures.find(name);
     if (it != m_textures.end())
         m_textures.erase(it);
 }
 
-boost::shared_ptr<Texture> TextureManager::LoadTexture(const boost::filesystem::path& path, bool mipmap)
+std::shared_ptr<Texture> TextureManager::LoadTexture(const boost::filesystem::path& path, bool mipmap)
 {
-    boost::shared_ptr<Texture> temp(new Texture());
+    auto temp = std::make_shared<Texture>();
     temp->Load(path, mipmap);
     return (m_textures[path.generic_string()] = temp);
 }
