@@ -42,7 +42,10 @@ namespace {
     using namespace boost::multi_index;
     struct GridLayoutWnd
     {
-        GridLayoutWnd() : wnd(0) {}
+        GridLayoutWnd() :
+            wnd(nullptr)
+        {}
+
         GridLayoutWnd(Wnd* wnd_, const Pt& ul_, const Pt& lr_) : wnd(wnd_), ul(ul_), lr(lr_) {}
         Wnd* wnd;
         Pt ul;
@@ -80,7 +83,7 @@ namespace {
     typedef multi_index_container<
         GridLayoutWnd,
         indexed_by<
-            ordered_unique<tag<Pointer>,            member<GridLayoutWnd, Wnd*, &GridLayoutWnd::wnd> >,
+            ordered_unique<tag<Pointer>,            member<GridLayoutWnd, Wnd*, &GridLayoutWnd::wnd>>,
             ordered_non_unique<tag<LayoutLeft>,     member<GridLayoutWnd, Pt,   &GridLayoutWnd::ul>, IsLeft>,
             ordered_non_unique<tag<LayoutTop>,      member<GridLayoutWnd, Pt,   &GridLayoutWnd::ul>, IsTop>,
             ordered_non_unique<tag<LayoutRight>,    member<GridLayoutWnd, Pt,   &GridLayoutWnd::lr>, IsRight>,
@@ -108,30 +111,6 @@ namespace {
     const int DEFAULT_LAYOUT_BORDER_MARGIN = 0;
     const int DEFAULT_LAYOUT_CELL_MARGIN = 5;
 
-    struct WndSizeFunctor
-    {
-        std::string operator()(const Wnd* wnd)
-        {
-            if (!wnd)
-                return "";
-            std::stringstream stream;
-            stream << "(" << wnd->Width() << ", " << wnd->Height() << ")";
-            return stream.str();
-        }
-    };
-
-    struct WndClientSizeFunctor
-    {
-        std::string operator()(const Wnd* wnd)
-        {
-            if (!wnd)
-                return "";
-            std::stringstream stream;
-            stream << "(" << wnd->ClientWidth() << ", " << wnd->ClientHeight() << ")";
-            return stream.str();
-        }
-    };
-
     struct ForwardToParentException {};
 }
 
@@ -172,18 +151,18 @@ namespace {
 ///////////////////////////////////////
 // static(s)
 unsigned int Wnd::s_default_browse_time = 1500;
-boost::shared_ptr<BrowseInfoWnd> Wnd::s_default_browse_info_wnd;
+std::shared_ptr<BrowseInfoWnd> Wnd::s_default_browse_info_wnd;
 
 Wnd::Wnd() :
     m_done(false),
-    m_parent(0),
-    m_zorder(0),
+    m_parent(nullptr),
     m_visible(true),
+    m_needs_prerender(false),
     m_child_clipping_mode(DontClip),
     m_non_client_child(false),
     m_max_size(X(1 << 30), Y(1 << 30)),
-    m_layout(0),
-    m_containing_layout(0),
+    m_layout(nullptr),
+    m_containing_layout(nullptr),
     m_flags()
 {
     m_browse_modes.resize(1);
@@ -193,16 +172,15 @@ Wnd::Wnd() :
 
 Wnd::Wnd(X x, Y y, X w, Y h, Flags<WndFlag> flags/* = INTERACTIVE | DRAGABLE*/) :
     m_done(false),
-    m_parent(0),
-    m_zorder(0),
+    m_parent(nullptr),
     m_visible(true),
     m_child_clipping_mode(DontClip),
     m_non_client_child(false),
     m_upperleft(x, y),
     m_lowerright(x + w, y + h),
     m_max_size(X(1 << 30), Y(1 << 30)),
-    m_layout(0),
-    m_containing_layout(0),
+    m_layout(nullptr),
+    m_containing_layout(nullptr),
     m_flags(flags)
 {
     ValidateFlags();
@@ -214,15 +192,15 @@ Wnd::Wnd(X x, Y y, X w, Y h, Flags<WndFlag> flags/* = INTERACTIVE | DRAGABLE*/)
 Wnd::~Wnd()
 {
     // remove this-references from Wnds that this Wnd filters
-    for (std::set<Wnd*>::iterator it1 = m_filtering.begin(); it1 != m_filtering.end(); ++it1) {
-        std::vector<Wnd*>::iterator it2 = std::find((*it1)->m_filters.begin(), (*it1)->m_filters.end(), this);
-        if (it2 != (*it1)->m_filters.end())
-            (*it1)->m_filters.erase(it2);
+    for (Wnd* filtering_wnd : m_filtering) {
+        std::vector<Wnd*>::iterator it2 = std::find(filtering_wnd->m_filters.begin(), filtering_wnd->m_filters.end(), this);
+        if (it2 != filtering_wnd->m_filters.end())
+            filtering_wnd->m_filters.erase(it2);
     }
 
     // remove this-references from Wnds that filter this Wnd
-    for (std::vector<Wnd*>::iterator it1 = m_filters.begin(); it1 != m_filters.end(); ++it1) {
-        (*it1)->m_filtering.erase(this);
+    for (Wnd* filter_wnd : m_filters) {
+        filter_wnd->m_filtering.erase(this);
     }
 
     if (Wnd* parent = Parent())
@@ -263,6 +241,9 @@ bool Wnd::NonClientChild() const
 bool Wnd::Visible() const
 { return m_visible; }
 
+bool Wnd::PreRenderRequired() const
+{ return m_needs_prerender || (m_layout && m_layout->m_needs_prerender); }
+
 const std::string& Wnd::Name() const
 { return m_name; }
 
@@ -317,9 +298,6 @@ X Wnd::Width() const
 Y Wnd::Height() const
 { return m_lowerright.y - m_upperleft.y; }
 
-int Wnd::ZOrder() const
-{ return m_zorder; }
-
 Pt Wnd::Size() const
 { return Pt(m_lowerright.x - m_upperleft.x, m_lowerright.y - m_upperleft.y); }
 
@@ -386,7 +364,7 @@ const std::vector<Wnd::BrowseInfoMode>& Wnd::BrowseModes() const
 const std::string& Wnd::BrowseInfoText(std::size_t mode) const
 { return m_browse_modes.at(mode).text; }
 
-const boost::shared_ptr<StyleFactory>& Wnd::GetStyleFactory() const
+const std::shared_ptr<StyleFactory>& Wnd::GetStyleFactory() const
 { return m_style_factory ? m_style_factory : GUI::GetGUI()->GetStyleFactory(); }
 
 WndRegion Wnd::WindowRegion(const Pt& pt) const
@@ -416,6 +394,39 @@ WndRegion Wnd::WindowRegion(const Pt& pt) const
     return (Resizable() ? WndRegion(x_pos + 3 * y_pos) : WR_NONE);
 }
 
+void Wnd::ClampRectWithMinAndMaxSize(Pt& ul, Pt& lr) const
+{
+    Pt min_sz = MinSize();
+    Pt max_sz = MaxSize();
+    if (m_layout) {
+        Pt layout_min_sz = m_layout->MinSize() + (Size() - ClientSize());
+        min_sz.x = std::max(min_sz.x, layout_min_sz.x);
+        min_sz.y = std::max(min_sz.y, layout_min_sz.y);
+    }
+    if (lr.x - ul.x < min_sz.x) {
+        if (ul.x != m_upperleft.x)
+            ul.x = lr.x - min_sz.x;
+        else
+            lr.x = ul.x + min_sz.x;
+    } else if (max_sz.x < lr.x - ul.x) {
+        if (lr.x != m_lowerright.x)
+            lr.x = ul.x + max_sz.x;
+        else
+            ul.x = lr.x - max_sz.x;
+    }
+    if (lr.y - ul.y < min_sz.y) {
+        if (ul.y != m_upperleft.y)
+            ul.y = lr.y - min_sz.y;
+        else
+            lr.y = ul.y + min_sz.y;
+    } else if (max_sz.y < lr.y - ul.y) {
+        if (lr.y != m_lowerright.y)
+            lr.y = ul.y + max_sz.y;
+        else
+            ul.y = lr.y - max_sz.y;
+    }
+}
+
 void Wnd::SetDragDropDataType(const std::string& data_type)
 { m_drag_drop_data_type = data_type; }
 
@@ -427,8 +438,8 @@ void Wnd::AcceptDrops(const Pt& pt, const std::vector<Wnd*>& wnds, Flags<ModKey>
     if (!Interactive() && Parent())
         ForwardEventToParent();
     // if dropped Wnds were accepted, but no handler or parent exists to handle them, delete them
-    for (std::vector<Wnd*>::const_iterator it = wnds.begin(); it != wnds.end(); ++it)
-        delete *it;
+    for (Wnd* wnd : wnds)
+        delete wnd;
 }
 
 void Wnd::CancellingChildDragDrop(const std::vector<const Wnd*>& wnds)
@@ -436,8 +447,8 @@ void Wnd::CancellingChildDragDrop(const std::vector<const Wnd*>& wnds)
 
 void Wnd::ChildrenDraggedAway(const std::vector<Wnd*>& wnds, const Wnd* destination)
 {
-    for (std::vector<Wnd*>::const_iterator it = wnds.begin(); it != wnds.end(); ++it) {
-        DetachChild(*it);
+    for (Wnd* wnd : wnds) {
+        DetachChild(wnd);
     }
 }
 
@@ -448,9 +459,8 @@ void Wnd::Hide(bool children/* = true*/)
 {
     m_visible = false;
     if (children) {
-        std::list<Wnd*>::iterator it = m_children.begin();
-        for (; it != m_children.end(); ++it)
-            (*it)->Hide(children);
+        for (Wnd* child : m_children)
+            child->Hide(children);
     }
 }
 
@@ -458,9 +468,8 @@ void Wnd::Show(bool children/* = true*/)
 {
     m_visible = true;
     if (children) {
-        std::list<Wnd*>::iterator it = m_children.begin();
-        for (; it != m_children.end(); ++it)
-            (*it)->Show(children);
+        for (Wnd* child : m_children)
+            child->Show(children);
     }
 }
 
@@ -483,38 +492,10 @@ void Wnd::SizeMove(const Pt& ul_, const Pt& lr_)
 {
     Pt ul = ul_, lr = lr_;
     Pt original_sz = Size();
-    bool resized = (original_sz.x != (lr.x - ul.x)) || (original_sz.y != (lr.y - ul.y));
-    if (resized) {
-        Pt min_sz = MinSize();
-        Pt max_sz = MaxSize();
-        if (m_layout) {
-            Pt layout_min_sz = m_layout->MinSize() + (Size() - ClientSize());
-            min_sz.x = std::max(min_sz.x, layout_min_sz.x);
-            min_sz.y = std::max(min_sz.y, layout_min_sz.y);
-        }
-        if (lr.x - ul.x < min_sz.x) {
-            if (ul.x != m_upperleft.x)
-                ul.x = lr.x - min_sz.x;
-            else if (lr.x != m_lowerright.x)
-                lr.x = ul.x + min_sz.x;
-        } else if (max_sz.x < lr.x - ul.x) {
-            if (lr.x != m_lowerright.x)
-                lr.x = ul.x + max_sz.x;
-            else
-                ul.x = lr.x - max_sz.x;
-        }
-        if (lr.y - ul.y < min_sz.y) {
-            if (ul.y != m_upperleft.y)
-                ul.y = lr.y - min_sz.y;
-            else if (lr.y != m_lowerright.y)
-                lr.y = ul.y + min_sz.y;
-        } else if (max_sz.y < lr.y - ul.y) {
-            if (lr.y != m_lowerright.y)
-                lr.y = ul.y + max_sz.y;
-            else
-                ul.y = lr.y - max_sz.y;
-        }
-    }
+    bool resized = (original_sz != (lr - ul));
+    if (resized)
+        ClampRectWithMinAndMaxSize(ul, lr);
+
     m_upperleft = ul;
     m_lowerright = lr;
     if (resized) {
@@ -522,7 +503,7 @@ void Wnd::SizeMove(const Pt& ul_, const Pt& lr_)
         if (m_layout && size_changed)
             m_layout->Resize(ClientSize());
         if (m_containing_layout && size_changed && !dynamic_cast<Layout*>(this))
-            m_containing_layout->ChildSizeOrMinSizeOrMaxSizeChanged();
+            m_containing_layout->ChildSizeOrMinSizeChanged();
     }
 }
 
@@ -535,8 +516,9 @@ void Wnd::SetMinSize(const Pt& sz)
     m_min_size = sz;
     if (Width() < m_min_size.x || Height() < m_min_size.y)
         Resize(Pt(std::max(Width(), m_min_size.x), std::max(Height(), m_min_size.y)));
+    // The previous Resize() will call ChildSizeOrMinSizeChanged() itself if needed
     else if (m_containing_layout && min_size_changed && !dynamic_cast<Layout*>(this))
-        m_containing_layout->ChildSizeOrMinSizeOrMaxSizeChanged();
+        m_containing_layout->ChildSizeOrMinSizeChanged();
 }
 
 void Wnd::SetMaxSize(const Pt& sz)
@@ -589,12 +571,12 @@ void Wnd::DetachChild(Wnd* wnd)
     if (it == m_children.end())
         return;
     m_children.erase(it);
-    wnd->SetParent(0);
+    wnd->SetParent(nullptr);
     if (wnd == m_layout)
-        m_layout = 0;
+        m_layout = nullptr;
     if (Layout* this_as_layout = dynamic_cast<Layout*>(this)) {
         this_as_layout->Remove(wnd);
-        wnd->m_containing_layout = 0;
+        wnd->m_containing_layout = nullptr;
     }
 }
 
@@ -623,7 +605,7 @@ void Wnd::DeleteChild(Wnd* wnd)
 
 void Wnd::DeleteChildren()
 {
-    m_layout = 0;
+    m_layout = nullptr;
     for (std::list<Wnd*>::iterator it = m_children.begin(); it != m_children.end();) {
         Wnd* wnd = *it++;
         delete wnd;
@@ -656,11 +638,11 @@ void Wnd::HorizontalLayout()
 
     std::multiset<Wnd*, WndHorizontalLess> wnds;
     Pt client_sz = ClientSize();
-    for (std::list<Wnd*>::const_iterator it = m_children.begin(); it != m_children.end(); ++it) {
-        Pt wnd_ul = (*it)->RelativeUpperLeft(), wnd_lr = (*it)->RelativeLowerRight();
+    for (Wnd* child : m_children) {
+        Pt wnd_ul = child->RelativeUpperLeft(), wnd_lr = child->RelativeLowerRight();
         if (wnd_ul.x < 0 || wnd_ul.y < 0 || client_sz.x < wnd_lr.x || client_sz.y < wnd_lr.y)
             continue;
-        wnds.insert(*it);
+        wnds.insert(child);
     }
 
     m_layout = new Layout(X0, Y0, ClientSize().x, ClientSize().y,
@@ -669,8 +651,8 @@ void Wnd::HorizontalLayout()
     AttachChild(m_layout);
 
     int i = 0;
-    for (std::multiset<Wnd*, WndHorizontalLess>::iterator it = wnds.begin(); it != wnds.end(); ++it) {
-        m_layout->Add(*it, 0, i++);
+    for (const std::multiset<Wnd*, WndHorizontalLess>::value_type& wnd : wnds) {
+        m_layout->Add(wnd, 0, i++);
     }
 }
 
@@ -680,11 +662,11 @@ void Wnd::VerticalLayout()
 
     std::multiset<Wnd*, WndVerticalLess> wnds;
     Pt client_sz = ClientSize();
-    for (std::list<Wnd*>::const_iterator it = m_children.begin(); it != m_children.end(); ++it) {
-        Pt wnd_ul = (*it)->RelativeUpperLeft(), wnd_lr = (*it)->RelativeLowerRight();
+    for (Wnd* child : m_children) {
+        Pt wnd_ul = child->RelativeUpperLeft(), wnd_lr = child->RelativeLowerRight();
         if (wnd_ul.x < 0 || wnd_ul.y < 0 || client_sz.x < wnd_lr.x || client_sz.y < wnd_lr.y)
             continue;
-        wnds.insert(*it);
+        wnds.insert(child);
     }
 
     m_layout = new Layout(X0, Y0, ClientSize().x, ClientSize().y,
@@ -693,8 +675,8 @@ void Wnd::VerticalLayout()
     AttachChild(m_layout);
 
     int i = 0;
-    for (std::multiset<Wnd*, WndVerticalLess>::iterator it = wnds.begin(); it != wnds.end(); ++it) {
-        m_layout->Add(*it, i++, 0);
+    for (const std::multiset<Wnd*, WndVerticalLess>::value_type& wnd : wnds) {
+        m_layout->Add(wnd, i++, 0);
     }
 }
 
@@ -790,11 +772,11 @@ void Wnd::GridLayout()
     // create an actual layout with a more reasonable number of cells from the pixel-grid layout
     std::set<X> unique_lefts;
     std::set<Y> unique_tops;
-    for (LeftIter it = grid_layout.get<LayoutLeft>().begin(); it != grid_layout.get<LayoutLeft>().end(); ++it) {
-        unique_lefts.insert(it->ul.x);
+    for (const GridLayoutWnd& layout_wnd : grid_layout.get<LayoutLeft>()) {
+        unique_lefts.insert(layout_wnd.ul.x);
     }
-    for (TopIter it = grid_layout.get<LayoutTop>().begin(); it != grid_layout.get<LayoutTop>().end(); ++it) {
-        unique_tops.insert(it->ul.y);
+    for (const GridLayoutWnd& layout_wnd : grid_layout.get<LayoutTop>()) {
+        unique_tops.insert(layout_wnd.ul.y);
     }
 
     if (unique_lefts.empty() || unique_tops.empty())
@@ -806,10 +788,10 @@ void Wnd::GridLayout()
     AttachChild(m_layout);
 
     // populate this new layout with the child windows, based on their placements in the pixel-grid layout
-    for (PointerIter it = grid_layout.get<Pointer>().begin(); it != grid_layout.get<Pointer>().end(); ++it) {
-        Wnd* wnd = it->wnd;
-        Pt ul = it->ul;
-        Pt lr = it->lr;
+    for (const GridLayoutWnd& layout_wnd : grid_layout.get<Pointer>()) {
+        Wnd* wnd = layout_wnd.wnd;
+        Pt ul = layout_wnd.ul;
+        Pt lr = layout_wnd.lr;
         int left = std::distance(unique_lefts.begin(), unique_lefts.find(ul.x));
         int top = std::distance(unique_tops.begin(), unique_tops.find(ul.y));
         int right = std::distance(unique_lefts.begin(), unique_lefts.lower_bound(lr.x));
@@ -826,12 +808,12 @@ void Wnd::SetLayout(Layout* layout)
     std::list<Wnd*> children = m_children;
     DetachChildren();
     Pt client_sz = ClientSize();
-    for (std::list<Wnd*>::const_iterator it = children.begin(); it != children.end(); ++it) {
-        Pt wnd_ul = (*it)->RelativeUpperLeft(), wnd_lr = (*it)->RelativeLowerRight();
+    for (Wnd* wnd : children) {
+        Pt wnd_ul = wnd->RelativeUpperLeft(), wnd_lr = wnd->RelativeLowerRight();
         if (wnd_ul.x < 0 || wnd_ul.y < 0 || client_sz.x < wnd_lr.x || client_sz.y < wnd_lr.y)
-            AttachChild(*it);
+            AttachChild(wnd);
         else
-            delete *it;
+            delete wnd;
     }
     AttachChild(layout);
     m_layout = layout;
@@ -843,12 +825,12 @@ void Wnd::RemoveLayout()
     if (m_layout) {
         std::list<Wnd*> layout_children = m_layout->Children();
         m_layout->DetachAndResetChildren();
-        for (std::list<Wnd*>::iterator it = layout_children.begin(); it != layout_children.end(); ++it) {
-            AttachChild(*it);
+        for (Wnd* wnd : layout_children) {
+            AttachChild(wnd);
         }
         // prevent DeleteChild from recursively calling RemoveLayout
         GG::Layout* temp_layout = m_layout;
-        m_layout = 0;
+        m_layout = nullptr;
         DeleteChild(temp_layout);
     }
 }
@@ -872,6 +854,16 @@ void Wnd::SetLayoutCellMargin(unsigned int margin)
         m_layout->SetCellMargin(margin);
 }
 
+void Wnd::PreRender()
+{
+    m_needs_prerender = false;
+    if (m_layout && m_layout->m_needs_prerender)
+        m_layout->PreRender();
+}
+
+void Wnd::RequirePreRender()
+{ m_needs_prerender = true; }
+
 void Wnd::Render() {}
 
 bool Wnd::Run()
@@ -882,7 +874,7 @@ bool Wnd::Run()
         gui->RegisterModal(this);
         ModalInit();
         m_done = false;
-        boost::shared_ptr<ModalEventPump> pump = gui->CreateModalEventPump(m_done);
+        std::shared_ptr<ModalEventPump> pump = gui->CreateModalEventPump(m_done);
         (*pump)();
         gui->Remove(this);
         retval = true;
@@ -912,7 +904,7 @@ void Wnd::SetBrowseModeTime(unsigned int time, std::size_t mode/* = 0*/)
     m_browse_modes[mode].time = time;
 }
 
-void Wnd::SetBrowseInfoWnd(const boost::shared_ptr<BrowseInfoWnd>& wnd, std::size_t mode/* = 0*/)
+void Wnd::SetBrowseInfoWnd(const std::shared_ptr<BrowseInfoWnd>& wnd, std::size_t mode/* = 0*/)
 { m_browse_modes.at(mode).wnd = wnd; }
 
 void Wnd::ClearBrowseInfoWnd(std::size_t mode/* = 0*/)
@@ -924,7 +916,7 @@ void Wnd::SetBrowseText(const std::string& text, std::size_t mode/* = 0*/)
 void Wnd::SetBrowseModes(const std::vector<BrowseInfoMode>& modes)
 { m_browse_modes = modes; }
 
-void Wnd::SetStyleFactory(const boost::shared_ptr<StyleFactory>& factory)
+void Wnd::SetStyleFactory(const std::shared_ptr<StyleFactory>& factory)
 { m_style_factory = factory; }
 
 unsigned int Wnd::DefaultBrowseTime()
@@ -933,10 +925,10 @@ unsigned int Wnd::DefaultBrowseTime()
 void Wnd::SetDefaultBrowseTime(unsigned int time)
 { s_default_browse_time = time; }
 
-const boost::shared_ptr<BrowseInfoWnd>& Wnd::DefaultBrowseInfoWnd()
+const std::shared_ptr<BrowseInfoWnd>& Wnd::DefaultBrowseInfoWnd()
 { return s_default_browse_info_wnd; }
 
-void Wnd::SetDefaultBrowseInfoWnd(const boost::shared_ptr<BrowseInfoWnd>& browse_info_wnd)
+void Wnd::SetDefaultBrowseInfoWnd(const std::shared_ptr<BrowseInfoWnd>& browse_info_wnd)
 { s_default_browse_info_wnd = browse_info_wnd; }
 
 Wnd::DragDropRenderingState Wnd::GetDragDropRenderingState() const
@@ -1036,10 +1028,10 @@ void Wnd::CheckDrops(const Pt& pt, std::map<const Wnd*, bool>& drop_wnds_accepta
 void Wnd::DragDropLeave()
 { if (!Interactive()) ForwardEventToParent(); }
 
-void Wnd::KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_keys)
+void Wnd::KeyPress(Key key, std::uint32_t key_code_point, Flags<ModKey> mod_keys)
 { if (!Interactive()) ForwardEventToParent(); }
 
-void Wnd::KeyRelease(Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_keys)
+void Wnd::KeyRelease(Key key, std::uint32_t key_code_point, Flags<ModKey> mod_keys)
 { if (!Interactive()) ForwardEventToParent(); }
 
 void Wnd::TextInput(const std::string* text)
