@@ -83,8 +83,8 @@ WndEvent::WndEvent(EventType type, const Pt& pt, Flags<ModKey> mod_keys) :
     m_mod_keys(mod_keys),
     m_wheel_move(0),
     m_ticks(0),
-    m_timer(0),
-    m_text(0)
+    m_timer(nullptr),
+    m_text(nullptr)
 {}
 
 WndEvent::WndEvent(EventType type, const Pt& pt, const Pt& move, Flags<ModKey> mod_keys) :
@@ -96,8 +96,8 @@ WndEvent::WndEvent(EventType type, const Pt& pt, const Pt& move, Flags<ModKey> m
     m_drag_move(move),
     m_wheel_move(0),
     m_ticks(0),
-    m_timer(0),
-    m_text(0)
+    m_timer(nullptr),
+    m_text(nullptr)
 {}
 
 WndEvent::WndEvent(EventType type, const Pt& pt, int move, Flags<ModKey> mod_keys) :
@@ -108,8 +108,8 @@ WndEvent::WndEvent(EventType type, const Pt& pt, int move, Flags<ModKey> mod_key
     m_mod_keys(mod_keys),
     m_wheel_move(move),
     m_ticks(0),
-    m_timer(0),
-    m_text(0)
+    m_timer(nullptr),
+    m_text(nullptr)
 {}
 
 WndEvent::WndEvent(EventType type, const Pt& pt, const std::map<Wnd*, Pt>& drag_drop_wnds, Flags<ModKey> mod_keys) :
@@ -121,12 +121,12 @@ WndEvent::WndEvent(EventType type, const Pt& pt, const std::map<Wnd*, Pt>& drag_
     m_wheel_move(0),
     m_drag_drop_wnds(drag_drop_wnds),
     m_ticks(0),
-    m_timer(0),
-    m_text(0)
+    m_timer(nullptr),
+    m_text(nullptr)
 {
     // initialize storage for acceptable Wnds
-    for (std::map<Wnd*, Pt>::const_iterator it = m_drag_drop_wnds.begin(); it != m_drag_drop_wnds.end(); ++it)
-    { m_acceptable_drop_wnds[it->first] = false; }
+    for (const std::map<Wnd*, Pt>::value_type& drag_drop_wnd : m_drag_drop_wnds)
+    { m_acceptable_drop_wnds[drag_drop_wnd.first] = false; }
 }
 
 WndEvent::WndEvent(EventType type, const Pt& pt, const std::vector<Wnd*>& drag_drop_wnds, Flags<ModKey> mod_keys) :
@@ -137,8 +137,8 @@ WndEvent::WndEvent(EventType type, const Pt& pt, const std::vector<Wnd*>& drag_d
     m_mod_keys(mod_keys),
     m_wheel_move(0),
     m_ticks(0),
-    m_timer(0),
-    m_text(0),
+    m_timer(nullptr),
+    m_text(nullptr),
     m_dropped_wnds(drag_drop_wnds)
 {}
 
@@ -151,23 +151,23 @@ WndEvent::WndEvent(EventType type, const Pt& pt, Wnd* drag_wnd, Flags<ModKey> mo
     m_wheel_move(0),
     m_drag_drop_wnds(),
     m_ticks(0),
-    m_timer(0),
-    m_text(0)
+    m_timer(nullptr),
+    m_text(nullptr)
 {
     // initialize storage for single dragged Wnd
     m_drag_drop_wnds[drag_wnd] = pt;
     m_acceptable_drop_wnds[drag_wnd] = false;
 }
 
-WndEvent::WndEvent(EventType type, Key key, boost::uint32_t code_point, Flags<ModKey> mod_keys) :
+WndEvent::WndEvent(EventType type, Key key, std::uint32_t code_point, Flags<ModKey> mod_keys) :
     m_type(type),
     m_key(key),
     m_key_code_point(code_point),
     m_mod_keys(mod_keys),
     m_wheel_move(0),
     m_ticks(0),
-    m_timer(0),
-    m_text(0)
+    m_timer(nullptr),
+    m_text(nullptr)
 {}
 
 WndEvent::WndEvent(EventType type, unsigned int ticks, Timer* timer) :
@@ -178,7 +178,7 @@ WndEvent::WndEvent(EventType type, unsigned int ticks, Timer* timer) :
     m_wheel_move(0),
     m_ticks(ticks),
     m_timer(timer),
-    m_text(0)
+    m_text(nullptr)
 {}
 
 WndEvent::WndEvent (WndEvent::EventType type, const std::string* text):
@@ -188,7 +188,7 @@ WndEvent::WndEvent (WndEvent::EventType type, const std::string* text):
     m_mod_keys(),
     m_wheel_move(0),
     m_ticks(0),
-    m_timer(0),
+    m_timer(nullptr),
     m_text(text)
 {}
 
@@ -199,8 +199,8 @@ WndEvent::WndEvent(EventType type) :
     m_mod_keys(), 
     m_wheel_move(0),
     m_ticks(0),
-    m_timer(0),
-    m_text(0)
+    m_timer(nullptr),
+    m_text(nullptr)
 {}
 
 WndEvent::EventType WndEvent::Type() const
@@ -212,7 +212,7 @@ const Pt& WndEvent::Point() const
 Key WndEvent::GetKey() const
 { return m_key; }
 
-boost::uint32_t WndEvent::KeyCodePoint() const
+std::uint32_t WndEvent::KeyCodePoint() const
 { return m_key_code_point; }
 
 Flags<ModKey> WndEvent::ModKeys() const
@@ -242,3 +242,38 @@ Timer* WndEvent::GetTimer() const
 const std::string* WndEvent::GetText() const
 { return m_text; }
 
+std::string EventTypeName(const WndEvent& event) {
+    switch(event.Type()) {
+    case WndEvent::LButtonDown:     return "(LButtonDown)";
+    case WndEvent::LDrag:           return "(LDrag)";
+    case WndEvent::LButtonUp:       return "(LButtonUp)";
+    case WndEvent::LClick:          return "(LClick)";
+    case WndEvent::LDoubleClick:    return "(LDoubleClick)";
+    case WndEvent::MButtonDown:     return "(MButtonDown)";
+    case WndEvent::MDrag:           return "(MDrag)";
+    case WndEvent::MButtonUp:       return "(MButtonUp)";
+    case WndEvent::MClick:          return "(MClick)";
+    case WndEvent::MDoubleClick:    return "(MDoubleClick)";
+    case WndEvent::RButtonDown:     return "(RButtonDown)";
+    case WndEvent::RDrag:           return "(RDrag)";
+    case WndEvent::RButtonUp:       return "(RButtonUp)";
+    case WndEvent::RClick:          return "(RClick)";
+    case WndEvent::RDoubleClick:    return "(RDoubleClick)";
+    case WndEvent::MouseEnter:      return "(MouseEnter)";
+    case WndEvent::MouseHere:       return "(MouseHere)";
+    case WndEvent::MouseLeave:      return "(MouseLeave)";
+    case WndEvent::MouseWheel:      return "(MouseWheel)";
+    case WndEvent::DragDropEnter:   return "(DragDropEnter)";
+    case WndEvent::DragDropHere:    return "(DragDropHere)";
+    case WndEvent::CheckDrops:      return "(CheckDrops)";
+    case WndEvent::DragDropLeave:   return "(DragDropLeave)";
+    case WndEvent::DragDroppedOn:   return "(DragDroppedOn)";
+    case WndEvent::KeyPress:        return "(KeyPress)";
+    case WndEvent::KeyRelease:      return "(KeyRelease)";
+    case WndEvent::TextInput:       return "(TextInput)";
+    case WndEvent::GainingFocus:    return "(GainingFocus)";
+    case WndEvent::LosingFocus:     return "(LosingFocus)";
+    case WndEvent::TimerFiring:     return "(TimerFiring)";
+    default:                            return "(Unknown Event Type)";
+    }
+}
