@@ -162,14 +162,14 @@ HueSaturationPicker::HueSaturationPicker(X x, Y y, X w, Y h) :
     const int SAMPLES = 100;
     const double INCREMENT = 1.0 / (SAMPLES + 1);
     const double VALUE = 1.0;
-    m_vertices.resize(SAMPLES, std::vector<std::pair<double, double> >(2 * (SAMPLES + 1)));
+    m_vertices.resize(SAMPLES, std::vector<std::pair<double, double>>(2 * (SAMPLES + 1)));
     m_colors.resize(SAMPLES, std::vector<Clr>(2 * (SAMPLES + 1)));
     for (int col = 0; col < SAMPLES; ++col) {
         for (int row = 0; row < SAMPLES + 1; ++row) {
-            m_vertices[col][2 * row] = std::make_pair(col * INCREMENT, row * INCREMENT);
-            m_vertices[col][2 * row + 1] = std::make_pair((col + 1) * INCREMENT, row * INCREMENT);
-            m_colors[col][2 * row] = Convert(HSVClr(col * INCREMENT, 1.0 - row * INCREMENT, VALUE));
-            m_colors[col][2 * row + 1] = Convert(HSVClr((col + 1) * INCREMENT, 1.0 - row * INCREMENT, VALUE));
+            m_vertices[col][2 * row] =      {col * INCREMENT, row * INCREMENT};
+            m_vertices[col][2 * row + 1] =  {(col + 1) * INCREMENT, row * INCREMENT};
+            m_colors[col][2 * row] =        Convert(HSVClr(col * INCREMENT, 1.0 - row * INCREMENT, VALUE));
+            m_colors[col][2 * row + 1] =    Convert(HSVClr((col + 1) * INCREMENT, 1.0 - row * INCREMENT, VALUE));
         }
     }
 }
@@ -383,7 +383,7 @@ void ValuePicker::SetValueFromPt(Pt pt)
 
 // ColorDlg::ColorButton
 ColorDlg::ColorButton::ColorButton(const Clr& color) :
-    Button("", boost::shared_ptr<Font>(), color),
+    Button("", nullptr, color),
     m_represented_color(CLR_BLACK)
 {}
 
@@ -486,41 +486,31 @@ void ColorDlg::ColorDisplay::Render()
 }
 
 
-// ColorDlg::ColorButtonClickFunctor
-ColorDlg::ColorButtonClickFunctor::ColorButtonClickFunctor(std::size_t id, ColorDlg* picker_) :
-    button_id(id),
-    picker(picker_)
-{}
-
-void ColorDlg::ColorButtonClickFunctor::operator()()
-{ picker->ColorButtonClicked(button_id); }
-
-
 // ColorDlg
 
 // static(s)
 std::vector<Clr> ColorDlg::s_custom_colors;
 const std::size_t ColorDlg::INVALID_COLOR_BUTTON = std::numeric_limits<std::size_t>::max();
 
-ColorDlg::ColorDlg(X x, Y y, Clr original_color, const boost::shared_ptr<Font>& font,
+ColorDlg::ColorDlg(X x, Y y, Clr original_color, const std::shared_ptr<Font>& font,
                    Clr dialog_color, Clr border_color, Clr text_color/* = CLR_BLACK*/) :
     Wnd(x, y, X(315), Y(300), INTERACTIVE | DRAGABLE | MODAL),
     m_original_color(original_color),
     m_original_color_specified(true),
     m_color_was_picked(false),
-    m_hue_saturation_picker(0),
-    m_value_picker(0),
-    m_pickers_layout(0),
-    m_new_color_square(0),
-    m_old_color_square(0),
-    m_new_color_square_text(0),
-    m_old_color_square_text(0),
-    m_color_squares_layout(0),
-    m_color_buttons_layout(0),
+    m_hue_saturation_picker(nullptr),
+    m_value_picker(nullptr),
+    m_pickers_layout(nullptr),
+    m_new_color_square(nullptr),
+    m_old_color_square(nullptr),
+    m_new_color_square_text(nullptr),
+    m_old_color_square_text(nullptr),
+    m_color_squares_layout(nullptr),
+    m_color_buttons_layout(nullptr),
     m_current_color_button(INVALID_COLOR_BUTTON),
-    m_ok(0),
-    m_cancel(0),
-    m_sliders_ok_cancel_layout(0),
+    m_ok(nullptr),
+    m_cancel(nullptr),
+    m_sliders_ok_cancel_layout(nullptr),
     m_color(dialog_color),
     m_border_color(border_color),
     m_text_color(text_color)
@@ -543,7 +533,7 @@ void ColorDlg::Render()
     }
 }
 
-void ColorDlg::KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_keys)
+void ColorDlg::KeyPress(Key key, std::uint32_t key_code_point, Flags<ModKey> mod_keys)
 {
     if (key == GGK_RETURN || key == GGK_KP_ENTER)
         OkClicked();
@@ -551,35 +541,20 @@ void ColorDlg::KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey> m
         CancelClicked();
 }
 
-void ColorDlg::Init(const boost::shared_ptr<Font>& font)
+void ColorDlg::Init(const std::shared_ptr<Font>& font)
 {
     m_current_color = m_original_color_specified ? Convert(m_original_color) : Convert(CLR_BLACK);
     Clr color = Convert(m_current_color);
 
-    boost::shared_ptr<StyleFactory> style = GetStyleFactory();
+    std::shared_ptr<StyleFactory> style = GetStyleFactory();
 
     const int COLOR_BUTTON_ROWS = 4;
     const int COLOR_BUTTON_COLS = 5;
     if (s_custom_colors.empty()) {
-        s_custom_colors.push_back(GG::CLR_WHITE);
-        s_custom_colors.push_back(GG::CLR_LIGHT_GRAY);
-        s_custom_colors.push_back(GG::CLR_GRAY);
-        s_custom_colors.push_back(GG::CLR_DARK_GRAY);
-        s_custom_colors.push_back(GG::CLR_BLACK);
-        s_custom_colors.push_back(GG::CLR_PINK);
-        s_custom_colors.push_back(GG::CLR_RED);
-        s_custom_colors.push_back(GG::CLR_DARK_RED);
-        s_custom_colors.push_back(GG::CLR_MAGENTA);
-        s_custom_colors.push_back(GG::CLR_PURPLE);
-        s_custom_colors.push_back(GG::CLR_BLUE);
-        s_custom_colors.push_back(GG::CLR_DARK_BLUE);
-        s_custom_colors.push_back(GG::CLR_TEAL);
-        s_custom_colors.push_back(GG::CLR_CYAN);
-        s_custom_colors.push_back(GG::CLR_GREEN);
-        s_custom_colors.push_back(GG::CLR_DARK_GREEN);
-        s_custom_colors.push_back(GG::CLR_OLIVE);
-        s_custom_colors.push_back(GG::CLR_YELLOW);
-        s_custom_colors.push_back(GG::CLR_ORANGE);
+        s_custom_colors = { GG::CLR_WHITE,      GG::CLR_LIGHT_GRAY, GG::CLR_GRAY,       GG::CLR_DARK_GRAY,  GG::CLR_BLACK,
+                            GG::CLR_PINK,       GG::CLR_RED,        GG::CLR_DARK_RED,   GG::CLR_MAGENTA,    GG::CLR_PURPLE,
+                            GG::CLR_BLUE,       GG::CLR_DARK_BLUE,  GG::CLR_TEAL,       GG::CLR_CYAN,       GG::CLR_GREEN,
+                            GG::CLR_DARK_GREEN, GG::CLR_OLIVE,      GG::CLR_YELLOW,     GG::CLR_ORANGE};
 
         for (unsigned int i = s_custom_colors.size(); i < COLOR_BUTTON_ROWS * COLOR_BUTTON_COLS; ++i) {
             s_custom_colors.push_back(CLR_GRAY);
@@ -626,7 +601,6 @@ void ColorDlg::Init(const boost::shared_ptr<Font>& font)
         }
     }
 
-    using boost::lexical_cast;
     m_sliders_ok_cancel_layout = new Layout(m_pickers_layout->Right() + 5, Y0, X(150), Y((25 + 5) * 8 - 5),
                                             9, 3, 0, 5);
     m_sliders_ok_cancel_layout->SetMinimumColumnWidth(0, X(15));
@@ -634,7 +608,7 @@ void ColorDlg::Init(const boost::shared_ptr<Font>& font)
     m_sliders_ok_cancel_layout->SetColumnStretch(2, 1);
     m_slider_labels.push_back(style->NewTextControl(style->Translate("R:"), font, m_text_color, FORMAT_RIGHT));
     m_sliders_ok_cancel_layout->Add(m_slider_labels.back(), 0, 0);
-    m_slider_values.push_back(style->NewTextControl(lexical_cast<std::string>(static_cast<int>(color.r)),
+    m_slider_values.push_back(style->NewTextControl(std::to_string(color.r),
                                                     font, m_text_color, FORMAT_LEFT));
     m_sliders_ok_cancel_layout->Add(m_slider_values.back(), 0, 1);
     m_sliders.push_back(style->NewIntSlider(0, 255, HORIZONTAL, m_color, 10));
@@ -643,7 +617,7 @@ void ColorDlg::Init(const boost::shared_ptr<Font>& font)
 
     m_slider_labels.push_back(style->NewTextControl(style->Translate("G:"), font, m_text_color, FORMAT_RIGHT));
     m_sliders_ok_cancel_layout->Add(m_slider_labels.back(), 1, 0);
-    m_slider_values.push_back(style->NewTextControl(lexical_cast<std::string>(static_cast<int>(color.g)),
+    m_slider_values.push_back(style->NewTextControl(std::to_string(color.g),
                                                     font, m_text_color, FORMAT_LEFT));
     m_sliders_ok_cancel_layout->Add(m_slider_values.back(), 1, 1);
     m_sliders.push_back(style->NewIntSlider(0, 255, HORIZONTAL, m_color, 10));
@@ -652,7 +626,7 @@ void ColorDlg::Init(const boost::shared_ptr<Font>& font)
 
     m_slider_labels.push_back(style->NewTextControl(style->Translate("B:"), font, m_text_color, FORMAT_RIGHT));
     m_sliders_ok_cancel_layout->Add(m_slider_labels.back(), 2, 0);
-    m_slider_values.push_back(style->NewTextControl(lexical_cast<std::string>(static_cast<int>(color.b)),
+    m_slider_values.push_back(style->NewTextControl(std::to_string(color.b),
                                                     font, m_text_color, FORMAT_LEFT));
     m_sliders_ok_cancel_layout->Add(m_slider_values.back(), 2, 1);
     m_sliders.push_back(style->NewIntSlider(0, 255, HORIZONTAL, m_color, 10));
@@ -661,7 +635,7 @@ void ColorDlg::Init(const boost::shared_ptr<Font>& font)
 
     m_slider_labels.push_back(style->NewTextControl(style->Translate("A:"), font, m_text_color, FORMAT_RIGHT));
     m_sliders_ok_cancel_layout->Add(m_slider_labels.back(), 3, 0);
-    m_slider_values.push_back(style->NewTextControl(lexical_cast<std::string>(static_cast<int>(color.a)),
+    m_slider_values.push_back(style->NewTextControl(std::to_string(color.a),
                                                     font, m_text_color, FORMAT_LEFT));
     m_sliders_ok_cancel_layout->Add(m_slider_values.back(), 3, 1);
     m_sliders.push_back(style->NewIntSlider(0, 255, HORIZONTAL, m_color, 10));
@@ -670,7 +644,7 @@ void ColorDlg::Init(const boost::shared_ptr<Font>& font)
 
     m_slider_labels.push_back(style->NewTextControl(style->Translate("H:"), font, m_text_color, FORMAT_RIGHT));
     m_sliders_ok_cancel_layout->Add(m_slider_labels.back(), 4, 0);
-    m_slider_values.push_back(style->NewTextControl(lexical_cast<std::string>(static_cast<int>(m_current_color.h * 359)),
+    m_slider_values.push_back(style->NewTextControl(std::to_string(static_cast<int>(m_current_color.h * 359)),
                                                     font, m_text_color, FORMAT_LEFT));
     m_sliders_ok_cancel_layout->Add(m_slider_values.back(), 4, 1);
     m_sliders.push_back(style->NewIntSlider(0, 359, HORIZONTAL, m_color, 10));
@@ -679,7 +653,7 @@ void ColorDlg::Init(const boost::shared_ptr<Font>& font)
 
     m_slider_labels.push_back(style->NewTextControl(style->Translate("S:"), font, m_text_color, FORMAT_RIGHT));
     m_sliders_ok_cancel_layout->Add(m_slider_labels.back(), 5, 0);
-    m_slider_values.push_back(style->NewTextControl(lexical_cast<std::string>(static_cast<int>(m_current_color.s * 255)),
+    m_slider_values.push_back(style->NewTextControl(std::to_string(static_cast<int>(m_current_color.s * 255)),
                                                     font, m_text_color, FORMAT_LEFT));
     m_sliders_ok_cancel_layout->Add(m_slider_values.back(), 5, 1);
     m_sliders.push_back(style->NewIntSlider(0, 255, HORIZONTAL, m_color, 10));
@@ -688,7 +662,7 @@ void ColorDlg::Init(const boost::shared_ptr<Font>& font)
 
     m_slider_labels.push_back(style->NewTextControl(style->Translate("V:"), font, m_text_color, FORMAT_RIGHT));
     m_sliders_ok_cancel_layout->Add(m_slider_labels.back(), 6, 0);
-    m_slider_values.push_back(style->NewTextControl(lexical_cast<std::string>(static_cast<int>(m_current_color.v * 255)),
+    m_slider_values.push_back(style->NewTextControl(std::to_string(static_cast<int>(m_current_color.v * 255)),
                                                     font, m_text_color, FORMAT_LEFT));
     m_sliders_ok_cancel_layout->Add(m_slider_values.back(), 6, 1);
     m_sliders.push_back(style->NewIntSlider(0, 255, HORIZONTAL, m_color, 10));
@@ -718,7 +692,7 @@ void ColorDlg::Init(const boost::shared_ptr<Font>& font)
 void ColorDlg::ConnectSignals()
 {
     for (std::size_t i = 0; i < m_color_buttons.size(); ++i) {
-        Connect(m_color_buttons[i]->LeftClickedSignal, ColorButtonClickFunctor(i, this));
+        Connect(m_color_buttons[i]->LeftClickedSignal, [this, i](){ this->ColorButtonClicked(i); });
     }
     Connect(m_sliders[R]->SlidSignal, &ColorDlg::RedSliderChanged, this);
     Connect(m_sliders[G]->SlidSignal, &ColorDlg::GreenSliderChanged, this);
