@@ -36,15 +36,14 @@
 
 #include <GG/dialogs/ThreeButtonDlg.h>
 
-// HACK! MSVC #defines int64_t to be __int64, which breaks the code in boost's cstdint.hpp
-#ifdef int64_t
-#undef int64_t
-#endif
-
 #include <boost/cast.hpp>
 #include <boost/format.hpp>
 #include <boost/algorithm/string/predicate.hpp>
 #include <boost/filesystem/operations.hpp>
+// boost::spirit::classic pulls in windows.h which in turn defines macro
+// versions of min and max.  Defining NOMINMAX disables the creation of those
+// macros
+#define NOMINMAX
 #include <boost/spirit/include/classic.hpp>
 #include <boost/spirit/include/classic_dynamic.hpp>
 #include <boost/system/system_error.hpp>
@@ -90,27 +89,43 @@ namespace {
 
     struct FrontStringBegin
     {
-        FrontStringBegin(const boost::shared_ptr<std::vector<std::string> >& strings) : m_strings(strings) {}
+        FrontStringBegin(const std::shared_ptr<std::vector<std::string>>& strings) :
+            m_strings(strings)
+        {}
+
         const char* operator()() const {return m_strings->front().c_str();}
-        boost::shared_ptr<std::vector<std::string> > m_strings;
+
+        std::shared_ptr<std::vector<std::string>> m_strings;
     };
     struct FrontStringEnd
     {
-        FrontStringEnd(const boost::shared_ptr<std::vector<std::string> >& strings) : m_strings(strings) {}
+        FrontStringEnd(const std::shared_ptr<std::vector<std::string>>& strings) :
+            m_strings(strings)
+        {}
+
         const char* operator()() const {return m_strings->front().c_str() + m_strings->front().size();}
-        boost::shared_ptr<std::vector<std::string> > m_strings;
+
+        std::shared_ptr<std::vector<std::string>> m_strings;
     };
     struct IndexedStringBegin
     {
-        IndexedStringBegin(const boost::shared_ptr<std::vector<std::string> >& strings) : m_strings(strings) {}
+        IndexedStringBegin(const std::shared_ptr<std::vector<std::string>>& strings) :
+            m_strings(strings)
+        {}
+
         const char* operator()() const {return (*m_strings)[Index::value].c_str();}
-        boost::shared_ptr<std::vector<std::string> > m_strings;
+
+        std::shared_ptr<std::vector<std::string>> m_strings;
     };
     struct IndexedStringEnd
     {
-        IndexedStringEnd(const boost::shared_ptr<std::vector<std::string> >& strings) : m_strings(strings) {}
+        IndexedStringEnd(const std::shared_ptr<std::vector<std::string>>& strings) :
+            m_strings(strings)
+        {}
+
         const char* operator()() const {return (*m_strings)[Index::value].c_str() + (*m_strings)[Index::value].size();}
-        boost::shared_ptr<std::vector<std::string> > m_strings;
+
+        std::shared_ptr<std::vector<std::string>> m_strings;
     };
 
     bool WindowsRoot(const std::string& root_name)
@@ -132,7 +147,7 @@ const Y FileDlg::DEFAULT_HEIGHT(450);
 
 
 FileDlg::FileDlg(const std::string& directory, const std::string& filename, bool save, bool multi,
-                 const boost::shared_ptr<Font>& font, Clr color, Clr border_color, Clr text_color/* = CLR_BLACK*/) : 
+                 const std::shared_ptr<Font>& font, Clr color, Clr border_color, Clr text_color/* = CLR_BLACK*/) :
     Wnd((GUI::GetGUI()->AppWidth() - DEFAULT_WIDTH) / 2,
         (GUI::GetGUI()->AppHeight() - DEFAULT_HEIGHT) / 2,
         DEFAULT_WIDTH, DEFAULT_HEIGHT, INTERACTIVE | DRAGABLE | MODAL),
@@ -146,14 +161,14 @@ FileDlg::FileDlg(const std::string& directory, const std::string& filename, bool
     m_in_win32_drive_selection(false),
     m_save_str(GetStyleFactory()->Translate("Save")),
     m_open_str(GetStyleFactory()->Translate("Open")),
-    m_curr_dir_text(0),
-    m_files_list(0),
-    m_files_edit(0),
-    m_filter_list(0),
-    m_ok_button(0),
-    m_cancel_button(0),
-    m_files_label(0),
-    m_file_types_label(0)
+    m_curr_dir_text(nullptr),
+    m_files_list(nullptr),
+    m_files_edit(nullptr),
+    m_filter_list(nullptr),
+    m_ok_button(nullptr),
+    m_cancel_button(nullptr),
+    m_files_label(nullptr),
+    m_file_types_label(nullptr)
 {
     CreateChildren(multi);
     Init(directory);
@@ -185,7 +200,7 @@ void FileDlg::Render()
     }
 }
 
-void FileDlg::KeyPress(Key key, boost::uint32_t key_code_point, Flags<ModKey> mod_keys)
+void FileDlg::KeyPress(Key key, std::uint32_t key_code_point, Flags<ModKey> mod_keys)
 {
     if (key == GGK_RETURN || key == GGK_KP_ENTER)
         OkHandler(false);
@@ -206,7 +221,7 @@ void FileDlg::SelectDirectories(bool directories)
 void FileDlg::AppendMissingSaveExtension(bool append)
 { m_append_missing_save_extension = append; }
 
-void FileDlg::SetFileFilters(const std::vector<std::pair<std::string, std::string> >& filters)
+void FileDlg::SetFileFilters(const std::vector<std::pair<std::string, std::string>>& filters)
 {
     m_file_filters = filters;
     PopulateFilters();
@@ -232,7 +247,7 @@ void FileDlg::CreateChildren(bool multi)
     if (m_save)
         multi = false;
 
-    boost::shared_ptr<StyleFactory> style = GetStyleFactory();
+    std::shared_ptr<StyleFactory> style = GetStyleFactory();
 
     m_files_edit = style->NewEdit("", m_font, m_border_color, m_text_color);
     m_filter_list = style->NewDropDownList(3, m_border_color);
@@ -265,8 +280,8 @@ void FileDlg::DoLayout()
 
     // determine the space needed to display both text labels in the chosen font; use this to expand the edit as far as
     // possible
-    X labels_width = std::max(m_font->TextExtent(m_files_label->Text()).x, 
-                              m_font->TextExtent(m_file_types_label->Text()).x) + H_SPACING;
+    X labels_width = std::max((m_files_label->MinUsableSize()).x,
+                              (m_file_types_label->MinUsableSize()).x) + H_SPACING;
 
     m_files_label->MoveTo(Pt(X0, Height() - (button_height + V_SPACING) * 2));
     m_files_label->Resize(Pt(labels_width - H_SPACING / 2, button_height));
@@ -342,7 +357,7 @@ void FileDlg::OkHandler(bool double_click)
     parse(m_files_edit->Text().c_str(), (+anychar_p)[append(files)], space_p);
     std::sort(files.begin(), files.end());
 
-    boost::shared_ptr<StyleFactory> style = GetStyleFactory();
+    std::shared_ptr<StyleFactory> style = GetStyleFactory();
 
     if (m_save) { // file save case
         if (m_ok_button->Text() != m_save_str) {
@@ -379,7 +394,7 @@ void FileDlg::OkHandler(bool double_click)
             // check to see if file already exists; if so, ask if it's ok to overwrite
             if (fs::exists(p)) {
                 std::string msg_str = boost::str(boost::format(style->Translate("%1% exists.\nOk to overwrite it?")) % save_file);
-                boost::shared_ptr<ThreeButtonDlg> dlg(
+                std::shared_ptr<ThreeButtonDlg> dlg(
                     style->NewThreeButtonDlg(X(300), Y(125), msg_str, m_font, m_color, m_border_color, m_color, m_text_color,
                                              2, style->Translate("Ok"), style->Translate("Cancel")));
                 dlg->Run();
@@ -390,21 +405,20 @@ void FileDlg::OkHandler(bool double_click)
         if (files.empty() || (m_select_directories && double_click)) {
             OpenDirectory();
         } else { // ensure the file(s) are valid before returning them
-            for (std::vector<std::string>::iterator it = files.begin(); it != files.end(); ++it) {
+            for (const std::string& file_name : files) {
 #if defined(_WIN32)
                 // convert UTF-8 file name to UTF-16
-                const std::string& file_name = *it;
                 boost::filesystem::path::string_type file_name_native;
                 utf8::utf8to16(file_name.begin(), file_name.end(), std::back_inserter(file_name_native));
                 fs::path p = s_working_dir / fs::path(file_name_native);
 #else
-                fs::path p = s_working_dir / fs::path(*it);
+                fs::path p = s_working_dir / fs::path(file_name);
 #endif
                 if (fs::exists(p)) {
                     bool p_is_directory = fs::is_directory(p);
                     if (!m_select_directories && p_is_directory) {
-                        std::string msg_str = boost::str(boost::format(style->Translate("\"%1%\"\nis a directory.")) % (*it));
-                        boost::shared_ptr<ThreeButtonDlg> dlg(
+                        std::string msg_str = boost::str(boost::format(style->Translate("\"%1%\"\nis a directory.")) % file_name);
+                        std::shared_ptr<ThreeButtonDlg> dlg(
                             style->NewThreeButtonDlg(X(300), Y(125), msg_str, m_font, m_color, m_border_color, m_color,
                                                      m_text_color, 1, style->Translate("Ok")));
                         dlg->Run();
@@ -422,8 +436,8 @@ void FileDlg::OkHandler(bool double_click)
 #endif
                     results_valid = true; // indicate validity only if at least one good file was found
                 } else {
-                    std::string msg_str = boost::str(boost::format(style->Translate("File \"%1%\"\ndoes not exist.")) % (*it));
-                    boost::shared_ptr<ThreeButtonDlg> dlg(
+                    std::string msg_str = boost::str(boost::format(style->Translate("File \"%1%\"\ndoes not exist.")) % file_name);
+                    std::shared_ptr<ThreeButtonDlg> dlg(
                         style->NewThreeButtonDlg(X(300), Y(125), msg_str, m_font, m_color, m_border_color, m_color,
                                                  m_text_color, 1, style->Translate("Ok")));
                     dlg->Run();
@@ -447,8 +461,8 @@ void FileDlg::FileSetChanged(const ListBox::SelectionSet& files)
 {
     std::string all_files;
     bool dir_selected = false;
-    for (ListBox::SelectionSet::const_iterator it = files.begin(); it != files.end(); ++it) {
-        std::string filename = boost::polymorphic_downcast<TextControl*>((***it)[0])->Text();
+    for (const ListBox::SelectionSet::value_type& file : files) {
+        std::string filename = !(**file).empty() ? boost::polymorphic_downcast<TextControl*>((**file).at(0))->Text() : "";
         if (filename[0] != '[') {
             if (!all_files.empty())
                 all_files += " ";
@@ -502,9 +516,9 @@ void FileDlg::PopulateFilters()
         m_file_types_label->Disable();
         m_filter_list->Disable();
     } else {
-        for (std::size_t i = 0; i < m_file_filters.size(); ++i) {
+        for (std::pair<std::string, std::string>& file_filter : m_file_filters) {
             ListBox::Row* row = new ListBox::Row();
-            row->push_back(GetStyleFactory()->NewTextControl(m_file_filters[i].first, m_font, m_text_color, FORMAT_NOWRAP));
+            row->push_back(GetStyleFactory()->NewTextControl(file_filter.first, m_font, m_text_color, FORMAT_NOWRAP));
             m_filter_list->Insert(row);
         }
         m_filter_list->Select(m_filter_list->begin());
@@ -521,7 +535,7 @@ void FileDlg::UpdateList()
     rule<> wildcard = anychar_p;
 
     // define file filters based on the filter strings in the filter drop list
-    std::vector<rule<> > file_filters;
+    std::vector<rule<>> file_filters;
 
     DropDownList::iterator it = m_filter_list->CurrentItem();
     if (it != m_filter_list->end()) {
@@ -529,7 +543,7 @@ void FileDlg::UpdateList()
         parse(m_file_filters[std::distance(m_filter_list->begin(), it)].second.c_str(), *(!ch_p(',') >> (+(anychar_p - ','))[append(filter_specs)]), space_p);
         file_filters.resize(filter_specs.size());
         for (std::size_t i = 0; i < filter_specs.size(); ++i) {
-            boost::shared_ptr<std::vector<std::string> > non_wildcards(new std::vector<std::string>); // the parts of the filter spec that are not wildcards
+            auto non_wildcards = std::make_shared<std::vector<std::string>>(); // the parts of the filter spec that are not wildcards
             parse(filter_specs[i].c_str(), *(*ch_p('*') >> (+(anychar_p - '*'))[append(*non_wildcards)]));
 
             if (non_wildcards->empty()) {
@@ -596,7 +610,7 @@ void FileDlg::UpdateList()
                     std::string row_text = "[" + it->path().filename().string() + "]";
 #endif
                     row->push_back(GetStyleFactory()->NewTextControl(row_text, m_font, m_text_color, FORMAT_NOWRAP));
-                    sorted_rows.insert(std::make_pair(row_text, row));
+                    sorted_rows.insert({row_text, row});
                 }
             } catch (const fs::filesystem_error&) {
             }
@@ -604,9 +618,8 @@ void FileDlg::UpdateList()
 
         std::vector<ListBox::Row*> rows;
         rows.reserve(sorted_rows.size());
-        for (std::multimap<std::string, ListBox::Row*>::const_iterator it = sorted_rows.begin();
-             it != sorted_rows.end(); ++it)
-        { rows.push_back(it->second); }
+        for (std::multimap<std::string, ListBox::Row*>::value_type& row : sorted_rows)
+        { rows.push_back(row.second); }
         m_files_list->Insert(rows, false);
 
         if (!m_select_directories) {
@@ -622,14 +635,14 @@ void FileDlg::UpdateList()
                         if (meets_filters) {
                             ListBox::Row* row = new ListBox::Row();
                             row->push_back(GetStyleFactory()->NewTextControl(it->path().filename().string(), m_font, m_text_color, FORMAT_NOWRAP));
-                            sorted_rows.insert(std::make_pair(it->path().filename().string(), row));
+                            sorted_rows.insert({it->path().filename().string(), row});
                         }
                     }
                 } catch (const fs::filesystem_error&) {
                 }
             }
-            for (std::multimap<std::string, ListBox::Row*>::const_iterator it = sorted_rows.begin(); it != sorted_rows.end(); ++it) {
-                m_files_list->Insert(it->second);
+            for (const std::multimap<std::string, ListBox::Row*>::value_type& row : sorted_rows) {
+                m_files_list->Insert(row.second);
             }
         }
     } else {
@@ -657,7 +670,8 @@ void FileDlg::UpdateDirectoryText()
 #else
     std::string str = s_working_dir.string();
 #endif
-    while (m_font->TextExtent(str).x > Width() - 2 * H_SPACING) {
+    m_curr_dir_text->SetText(str);
+    while (m_curr_dir_text->Width() > Width() - 2 * H_SPACING) {
         std::string::size_type slash_idx = str.find('/', 1);
         std::string::size_type backslash_idx = str.find('\\', 1);
         if (slash_idx != std::string::npos) {
@@ -669,14 +683,14 @@ void FileDlg::UpdateDirectoryText()
         } else {
             break;
         }
+        m_curr_dir_text->SetText(str);
     }
-    *m_curr_dir_text << str;
     DoLayout();
 }
 
 void FileDlg::OpenDirectory()
 {
-    boost::shared_ptr<StyleFactory> style = GetStyleFactory();
+    std::shared_ptr<StyleFactory> style = GetStyleFactory();
 
     // see if there is a directory selected; if so open the directory.
     // if more than one is selected, take the first one
@@ -685,7 +699,7 @@ void FileDlg::OpenDirectory()
         return;
 
     std::string directory;
-    directory = boost::polymorphic_downcast<TextControl*>((***sels.begin())[0])->Text();
+    directory = !(***sels.begin()).empty() ? boost::polymorphic_downcast<TextControl*>((***sels.begin()).at(0))->Text() : "";
 
     if (directory.size() < 2 || directory[0] != '[')
         return;
@@ -738,7 +752,7 @@ void FileDlg::OpenDirectory()
                     m_curr_dir_text->SetText("");
                     DoLayout();
                     UpdateList();
-                    boost::shared_ptr<ThreeButtonDlg> dlg(
+                    std::shared_ptr<ThreeButtonDlg> dlg(
                         GetStyleFactory()->NewThreeButtonDlg(X(175), Y(75),
                                                              style->Translate("Device is not ready."),
                                                              m_font, m_color,
