@@ -5,7 +5,7 @@
 
 AccordionPanel::AccordionPanel(GG::X w, GG::Y h, bool is_button_on_left /*= false*/) :
     GG::Control(GG::X0, GG::Y0, w, h, GG::INTERACTIVE),
-    m_expand_button(0),
+    m_expand_button(nullptr),
     m_collapsed(true),
     m_is_left(is_button_on_left),
     m_interior_color(ClientUI::WndColor()),
@@ -91,6 +91,9 @@ void AccordionPanel::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
 }
 
 void AccordionPanel::SetCollapsed(bool collapsed) {
+    if (collapsed == m_collapsed)
+        return;
+
     boost::filesystem::path button_texture_dir = ClientUI::ArtDir() / "icons" / "buttons";
 
     m_collapsed = collapsed;
@@ -115,5 +118,6 @@ void AccordionPanel::DoLayout() {
     GG::Pt expand_button_ul(m_is_left ? GG::X(-(EXPAND_BUTTON_SIZE + m_border_margin))
                             : (Width() + GG::X(-(EXPAND_BUTTON_SIZE + m_border_margin))), GG::Y0);
     GG::Pt expand_button_lr = expand_button_ul + GG::Pt(GG::X(EXPAND_BUTTON_SIZE), GG::Y(EXPAND_BUTTON_SIZE));
+    Wnd::MoveChildUp(m_expand_button);
     m_expand_button->SizeMove(expand_button_ul, expand_button_lr);
 }
