@@ -19,12 +19,16 @@
 #include "../universe/ShipDesign.h"
 #include "../universe/Effect.h"
 #include "../universe/Condition.h"
+#include "../universe/Enums.h"
 #include "../client/human/HumanClientApp.h"
 
 #include <GG/DrawUtil.h>
 #include <GG/Layout.h>
 #include <GG/StaticGraphic.h>
 
+#include <iterator>
+
+
 namespace {
     const std::string PROD_PEDIA_WND_NAME = "production.pedia";
     const std::string PROD_SELECTOR_WND_NAME = "production.selector";
@@ -46,24 +50,24 @@ namespace {
             m_item(item),
             m_empire_id(empire_id),
             m_location_id(location_id),
-            m_icon(0),
-            m_name(0),
-            m_cost(0),
-            m_time(0),
-            m_desc(0)
+            m_icon(nullptr),
+            m_name(nullptr),
+            m_cost(nullptr),
+            m_time(nullptr),
+            m_desc(nullptr)
         {
             SetChildClippingMode(ClipToClient);
         }
 
         /** Renders panel background and border. */
-        virtual void    Render() {
+        void Render() override {
             if (!m_initialized)
                 Init();
             GG::Clr background_clr = this->Disabled() ? ClientUI::WndColor() : ClientUI::CtrlColor();
             GG::FlatRectangle(UpperLeft(), LowerRight(), background_clr, ClientUI::WndOuterBorderColor(), 1u);
         }
 
-        virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
+        void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override {
             const GG::Pt old_size = Size();
             GG::Control::SizeMove(ul, lr);
             //std::cout << "ProductionItemPanel::SizeMove new size: (" << Value(Width()) << ", " << Value(Height()) << ")" << std::endl;
@@ -107,7 +111,7 @@ namespace {
 
             const Empire* empire = GetEmpire(m_empire_id);
 
-            boost::shared_ptr<GG::Texture>          texture;
+            std::shared_ptr<GG::Texture> texture;
             std::string                             name_text;
             std::string                             cost_text;
             std::string                             time_text;
@@ -138,7 +142,7 @@ namespace {
             if (empire) {
                 std::pair<double, int> cost_time = empire->ProductionCostAndTime(m_item, m_location_id);
                 cost_text = DoubleToString(cost_time.first, 3, false);
-                time_text = boost::lexical_cast<std::string>(cost_time.second);
+                time_text = std::to_string(cost_time.second);
             }
 
             m_icon = new GG::StaticGraphic(texture, GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE);
@@ -168,10 +172,10 @@ namespace {
         GG::Label*          m_desc;
     };
 
-    TemporaryPtr<const UniverseObject> GetSourceObjectForEmpire(int empire_id) {
+    std::shared_ptr<const UniverseObject> GetSourceObjectForEmpire(int empire_id) {
         // get a source object, which is owned by the empire,
         // preferably the capital
-        TemporaryPtr<const UniverseObject> source;
+        std::shared_ptr<const UniverseObject> source;
         if (empire_id == ALL_EMPIRES)
             return source;
         const Empire* empire = GetEmpire(empire_id);
@@ -182,10 +186,9 @@ namespace {
             return source;
 
         // not a valid source?!  scan through all objects to find one owned by this empire
-        const ObjectMap& objects = GetUniverse().Objects();
-        for (ObjectMap::const_iterator<> obj_it = objects.const_begin(); obj_it != objects.const_end(); ++obj_it) {
-            if (obj_it->OwnedBy(empire_id)) {
-                source = *obj_it;
+        for (std::shared_ptr<const UniverseObject> obj : GetUniverse().Objects()) {
+            if (obj->OwnedBy(empire_id)) {
+                source = obj;
                 break;
             }
         }
@@ -204,7 +207,7 @@ namespace {
             enqueue_conditions.push_back(const_cast<Condition::ConditionBase*>(building_type->EnqueueLocation()));
             enqueue_conditions.push_back(const_cast<Condition::ConditionBase*>(building_type->Location()));
         }
-        TemporaryPtr<const UniverseObject> source = GetSourceObjectForEmpire(empire_id);
+        std::shared_ptr<const UniverseObject> source = GetSourceObjectForEmpire(empire_id);
         if (only_failed_conditions)
             return ConditionFailedDescription(enqueue_conditions, GetUniverseObject(candidate_object_id), source);
         else
@@ -215,11 +218,11 @@ namespace {
                                              int empire_id, bool only_failed_conditions)
     {
         std::vector<Condition::ConditionBase*> location_conditions;
-        boost::shared_ptr<Condition::ConditionBase> can_prod_ship_cond(new Condition::CanProduceShips());
+        auto can_prod_ship_cond = std::make_shared<Condition::CanProduceShips>();
         location_conditions.push_back(can_prod_ship_cond.get());
-        boost::shared_ptr<Condition::ConditionBase> ship_avail_cond(new Condition::OwnerHasShipDesignAvailable(ship_design_id));
+        auto ship_avail_cond = std::make_shared<Condition::OwnerHasShipDesignAvailable>(ship_design_id);
         location_conditions.push_back(ship_avail_cond.get());
-        boost::shared_ptr<Condition::ConditionBase> can_colonize_cond;
+        std::shared_ptr<Condition::ConditionBase> can_colonize_cond;
         if (const ShipDesign* ship_design = GetShipDesign(ship_design_id)) {
             if (ship_design->CanColonize()) {
                 can_colonize_cond.reset(new Condition::CanColonize());
@@ -227,15 +230,12 @@ namespace {
             }
             if (const HullType* hull_type = ship_design->GetHull())
                 location_conditions.push_back(const_cast<Condition::ConditionBase*>(hull_type->Location()));
-            std::vector<std::string> parts = ship_design->Parts();
-            for (std::vector<std::string>::const_iterator it = parts.begin();
-                 it != parts.end(); ++it)
-            {
-                if (const PartType* part_type = GetPartType(*it))
+            for (const std::string& part_name : ship_design->Parts()) {
+                if (const PartType* part_type = GetPartType(part_name))
                     location_conditions.push_back(const_cast<Condition::ConditionBase*>(part_type->Location()));
             }
         }
-        TemporaryPtr<const UniverseObject> source = GetSourceObjectForEmpire(empire_id);
+        std::shared_ptr<const UniverseObject> source = GetSourceObjectForEmpire(empire_id);
 
         if (only_failed_conditions)
             return ConditionFailedDescription(location_conditions, GetUniverseObject(candidate_object_id), source);
@@ -243,16 +243,14 @@ namespace {
             return ConditionDescription(location_conditions, GetUniverseObject(candidate_object_id), source);
     }
 
-    boost::shared_ptr<GG::BrowseInfoWnd> ProductionItemRowBrowseWnd(const ProductionQueue::ProductionItem& item,
-                                                                    int candidate_object_id, int empire_id)
+    std::shared_ptr<GG::BrowseInfoWnd> ProductionItemRowBrowseWnd(const ProductionQueue::ProductionItem& item,
+                                                                  int candidate_object_id, int empire_id)
     {
         // production item is a building
         if (item.build_type == BT_BUILDING) {
             const BuildingType* building_type = GetBuildingType(item.name);
-            if (!building_type) {
-                boost::shared_ptr<GG::BrowseInfoWnd> browse_wnd;
-                return browse_wnd;
-            }
+            if (!building_type)
+                return nullptr;
 
             // create title, description, production time and cost
             const std::string& title = UserString(item.name);
@@ -261,28 +259,26 @@ namespace {
             int production_time = building_type->ProductionTime(empire_id, candidate_object_id);
 
             main_text += "\n\n" + UserString("PRODUCTION_WND_TOOLTIP_PROD_COST") + ": " + DoubleToString(total_cost, 3, false);
-            main_text += "\n" + UserString("PRODUCTION_WND_TOOLTIP_PROD_TIME") + ": " + boost::lexical_cast<std::string>(production_time);
+            main_text += "\n" + UserString("PRODUCTION_WND_TOOLTIP_PROD_TIME") + ": " + std::to_string(production_time);
 
             // show build conditions
             const std::string& enqueue_and_location_condition_failed_text = EnqueueAndLocationConditionDescription(item.name, candidate_object_id, empire_id, true);
             if (!enqueue_and_location_condition_failed_text.empty())
-                if (TemporaryPtr<UniverseObject> location = GetUniverseObject(candidate_object_id)) {
+                if (std::shared_ptr<UniverseObject> location = GetUniverseObject(candidate_object_id)) {
                     std::string failed_cond_loc = boost::io::str(FlexibleFormat(UserString("PRODUCTION_WND_TOOLTIP_FAILED_COND")) % location->Name());
                     main_text += "\n\n" + failed_cond_loc + ":\n" + enqueue_and_location_condition_failed_text;
             }
 
             // create tooltip
-            boost::shared_ptr<GG::BrowseInfoWnd> browse_wnd(new IconTextBrowseWnd(
-                ClientUI::BuildingIcon(item.name), title, main_text));
-            return browse_wnd;
+            return std::make_shared<IconTextBrowseWnd>(
+                ClientUI::BuildingIcon(item.name), title, main_text);
         }
 
         // production item is a ship
         if (item.build_type == BT_SHIP) {
             const ShipDesign* design = GetShipDesign(item.design_id);
             if (!design) {
-                boost::shared_ptr<GG::BrowseInfoWnd> browse_wnd;
-                return browse_wnd;
+                return nullptr;
             }
 
             // create title, description, production time and cost, hull type
@@ -292,27 +288,26 @@ namespace {
             int production_time = design->ProductionTime(empire_id, candidate_object_id);
 
             main_text += "\n\n" + UserString("PRODUCTION_WND_TOOLTIP_PROD_COST") + ": " + DoubleToString(total_cost, 3, false);
-            main_text += "\n" + UserString("PRODUCTION_WND_TOOLTIP_PROD_TIME") + ": " + boost::lexical_cast<std::string>(production_time);
+            main_text += "\n" + UserString("PRODUCTION_WND_TOOLTIP_PROD_TIME") + ": " + std::to_string(production_time);
             main_text += "\n\n" + UserString("ENC_SHIP_HULL") + ": " + UserString(design->Hull());
 
             // load ship parts, stack ship parts that are used multiple times
             std::string ship_parts_formatted;
             std::map<std::string, int> ship_part_names;
-            const std::vector<std::string>& parts = design->Parts();
 
-            for (std::vector<std::string>::const_iterator it = parts.begin(); it != parts.end(); ++it) {
-                if (ship_part_names.find(*it) != ship_part_names.end())
-                    ship_part_names[*it]++;
+            for (const std::string& part_name : design->Parts()) {
+                if (ship_part_names.find(part_name) != ship_part_names.end())
+                    ship_part_names[part_name]++;
                 else
-                    ship_part_names.insert(std::pair<std::string, int>(*it, 1));
+                    ship_part_names.insert(std::pair<std::string, int>(part_name, 1));
             }
 
-            for (std::map<std::string, int>::const_iterator it = ship_part_names.begin(); it != ship_part_names.end(); ++it) {
-                if (!UserStringExists(it->first)) continue;
-                if (ship_part_names[it->first] == 1)
-                    ship_parts_formatted += (UserString(it->first) + ", ");
+            for (const std::map<std::string, int>::value_type& part_name_count : ship_part_names) {
+                if (!UserStringExists(part_name_count.first)) continue;
+                if (ship_part_names[part_name_count.first] == 1)
+                    ship_parts_formatted += (UserString(part_name_count.first) + ", ");
                 else
-                    ship_parts_formatted += (UserString(it->first) + " x" + boost::lexical_cast<std::string>(it->second) + ", ");
+                    ship_parts_formatted += (UserString(part_name_count.first) + " x" + std::to_string(part_name_count.second) + ", ");
             }
 
             main_text += "\n" + UserString("PRODUCTION_WND_TOOLTIP_PARTS") + ": " + ship_parts_formatted.substr(0, ship_parts_formatted.length() - 2);
@@ -320,20 +315,19 @@ namespace {
             // show build conditions
             const std::string& location_condition_failed_text = LocationConditionDescription(item.design_id, candidate_object_id, empire_id, true);
             if (!location_condition_failed_text.empty())
-                if (TemporaryPtr<UniverseObject> location = GetUniverseObject(candidate_object_id)) {
+                if (std::shared_ptr<UniverseObject> location = GetUniverseObject(candidate_object_id)) {
                     std::string failed_cond_loc = boost::io::str(FlexibleFormat(UserString("PRODUCTION_WND_TOOLTIP_FAILED_COND")) % location->Name());
                     main_text += ("\n\n" + failed_cond_loc + ":\n" + location_condition_failed_text);
                 }
 
             // create tooltip
-            boost::shared_ptr<GG::BrowseInfoWnd> browse_wnd(new IconTextBrowseWnd(
-                ClientUI::ShipDesignIcon(item.design_id), title, main_text));
-            return browse_wnd;
+            return std::make_shared<IconTextBrowseWnd>(
+                ClientUI::ShipDesignIcon(item.design_id), title, main_text);
         }
 
         // other production item (?)
         else {
-            return boost::shared_ptr<GG::BrowseInfoWnd>();
+            return nullptr;
         }
     }
 
@@ -348,13 +342,13 @@ namespace {
             m_item(item),
             m_empire_id(empire_id),
             m_location_id(location_id),
-            m_panel(0)
+            m_panel(nullptr)
         {
             SetName("ProductionItemRow");
             SetChildClippingMode(ClipToClient);
 
             if (m_item.build_type == BT_SHIP) {
-                SetDragDropDataType(boost::lexical_cast<std::string>(m_item.design_id));
+                SetDragDropDataType(std::to_string(m_item.design_id));
             } else {
                 SetDragDropDataType(m_item.name);
             }
@@ -381,7 +375,7 @@ namespace {
         int Location() const
         { return m_location_id; }
 
-        void SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
+        void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override {
             const GG::Pt old_size = Size();
             GG::ListBox::Row::SizeMove(ul, lr);
             //std::cout << "ProductionItemRow::SizeMove size: (" << Value(Width()) << ", " << Value(Height()) << ")" << std::endl;
@@ -414,15 +408,15 @@ namespace {
             SetVScrollWheelIncrement(Value(ListRowHeight())*3);
         }
 
-        virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
+        void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override {
             const GG::Pt old_size = Size();
             CUIListBox::SizeMove(ul, lr);
             //std::cout << "BuildableItemsListBox::SizeMove size: (" << Value(Width()) << ", " << Value(Height()) << ")" << std::endl;
             if (old_size != Size()) {
                 const GG::Pt row_size = ListRowSize();
                 //std::cout << "BuildableItemsListBox::SizeMove list row size: (" << Value(row_size.x) << ", " << Value(row_size.y) << ")" << std::endl;
-                for (GG::ListBox::iterator it = begin(); it != end(); ++it)
-                    (*it)->Resize(row_size);
+                for (GG::ListBox::Row* row : *this)
+                    row->Resize(row_size);
             }
         }
 
@@ -432,20 +426,6 @@ namespace {
         static GG::Y    ListRowHeight()
         { return GG::Y(ClientUI::Pts() * 3/2); }
     };
-
-    struct ToggleBuildTypeFunctor {
-        ToggleBuildTypeFunctor(BuildDesignatorWnd* designator_wnd, BuildType type) : m_designator_wnd(designator_wnd), m_build_type(type) {}
-        void operator()() {m_designator_wnd->ToggleType(m_build_type);}
-        BuildDesignatorWnd* const m_designator_wnd;
-        const BuildType m_build_type;
-    };
-
-    struct ToggleAvailabilityFunctor {
-        ToggleAvailabilityFunctor(BuildDesignatorWnd* designator_wnd, bool available) : m_designator_wnd(designator_wnd), m_available(available) {}
-        void operator()() {m_designator_wnd->ToggleAvailabilitly(m_available);}
-        BuildDesignatorWnd* const m_designator_wnd;
-        const bool m_available; // true: toggle whether to show available techs; false: toggle whether to show unavailable techs
-    };
 }
 
 //////////////////////////////////////////////////
@@ -466,7 +446,7 @@ public:
     //@}
 
     /** \name Mutators */ //@{
-    virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr);
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
 
     /** Sets build location for this selector, which may be used to filter
       * items in the list or enable / disable them at some point in the
@@ -522,8 +502,8 @@ private:
     /** respond to the user right-clicking a producible item in the build selector */
     void    BuildItemRightClicked(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys);
 
-    std::map<BuildType, CUIButton*>         m_build_type_buttons;
-    std::vector<CUIButton*>                 m_availability_buttons;
+    std::map<BuildType, CUIStateButton*>    m_build_type_buttons;
+    std::vector<CUIStateButton*>            m_availability_buttons;
 
     std::set<BuildType>                     m_build_types_shown;
     std::pair<bool, bool>                   m_availabilities_shown; //!< .first -> available items; .second -> unavailable items
@@ -550,15 +530,15 @@ BuildDesignatorWnd::BuildSelector::BuildSelector(const std::string& config_name)
     m_empire_id(ALL_EMPIRES)
 {
     // create build type toggle buttons (ship, building, all)
-    m_build_type_buttons[BT_BUILDING] = new CUIButton(UserString("PRODUCTION_WND_CATEGORY_BT_BUILDING"), ClientUI::ButtonHiliteColor(), ClientUI::ButtonHiliteBorderColor());
+    m_build_type_buttons[BT_BUILDING] = new CUIStateButton(UserString("PRODUCTION_WND_CATEGORY_BT_BUILDING"), GG::FORMAT_CENTER, std::make_shared<CUILabelButtonRepresenter>());
     AttachChild(m_build_type_buttons[BT_BUILDING]);
-    m_build_type_buttons[BT_SHIP] = new CUIButton(UserString("PRODUCTION_WND_CATEGORY_BT_SHIP"), ClientUI::ButtonHiliteColor(), ClientUI::ButtonHiliteBorderColor());
+    m_build_type_buttons[BT_SHIP] = new CUIStateButton(UserString("PRODUCTION_WND_CATEGORY_BT_SHIP"), GG::FORMAT_CENTER, std::make_shared<CUILabelButtonRepresenter>());
     AttachChild(m_build_type_buttons[BT_SHIP]);
 
     // create availability toggle buttons (available, not available)
-    m_availability_buttons.push_back(new CUIButton(UserString("PRODUCTION_WND_AVAILABILITY_AVAILABLE"), ClientUI::ButtonHiliteColor(), ClientUI::ButtonHiliteBorderColor()));
+    m_availability_buttons.push_back(new CUIStateButton(UserString("PRODUCTION_WND_AVAILABILITY_AVAILABLE"), GG::FORMAT_CENTER, std::make_shared<CUILabelButtonRepresenter>()));
     AttachChild(m_availability_buttons.back());
-    m_availability_buttons.push_back(new CUIButton(UserString("PRODUCTION_WND_AVAILABILITY_UNAVAILABLE"), ClientUI::ButtonHiliteColor(), ClientUI::ButtonHiliteBorderColor()));
+    m_availability_buttons.push_back(new CUIStateButton(UserString("PRODUCTION_WND_AVAILABILITY_UNAVAILABLE"), GG::FORMAT_CENTER, std::make_shared<CUILabelButtonRepresenter>()));
     AttachChild(m_availability_buttons.back());
 
     // selectable list of buildable items
@@ -571,7 +551,7 @@ BuildDesignatorWnd::BuildSelector::BuildSelector(const std::string& config_name)
                 &BuildDesignatorWnd::BuildSelector::BuildItemRightClicked, this);
 
     //GG::ListBox::Row* header = new GG::ListBox::Row();
-    //boost::shared_ptr<GG::Font> font = ClientUI::GetFont();
+    //std::shared_ptr<GG::Font> font = ClientUI::GetFont();
     //GG::Clr clr = ClientUI::TextColor();
     //header->push_back("item", font, clr);
     //header->push_back("PP/turn", font, clr);
@@ -659,7 +639,7 @@ void BuildDesignatorWnd::BuildSelector::SetEmpireID(int empire_id/* = ALL_EMPIRE
 
 void BuildDesignatorWnd::BuildSelector::Refresh() {
     ScopedTimer timer("BuildDesignatorWnd::BuildSelector::Refresh()");
-    if (TemporaryPtr<const UniverseObject> prod_loc = GetUniverseObject(this->m_production_location))
+    if (std::shared_ptr<const UniverseObject> prod_loc = GetUniverseObject(this->m_production_location))
         this->SetName(boost::io::str(FlexibleFormat(UserString("PRODUCTION_WND_BUILD_ITEMS_TITLE_LOCATION")) % prod_loc->Name()));
     else
         this->SetName(UserString("PRODUCTION_WND_BUILD_ITEMS_TITLE"));
@@ -792,31 +772,17 @@ void BuildDesignatorWnd::BuildSelector::PopulateList() {
     if (!empire)
         return;
 
-    // keep track of first shown row and the scroll position, in order to
-    // restore the first shown row, or the scroll position if the first row
-    // shown isn't in the repopulated list.
-    GG::ListBox::iterator initial_first_row_shown_it = m_buildable_items->FirstRowShown();
-    std::string initial_first_row_name;
-    if (initial_first_row_shown_it != m_buildable_items->end())
-        initial_first_row_name = (*initial_first_row_shown_it)->DragDropDataType();
-    std::size_t initial_scroll_pos = std::distance(m_buildable_items->begin(), initial_first_row_shown_it);
-
+    // Capture the list scroll state
+    // Try to preserve the same queue context with completely new queue items
+    std::size_t initial_offset_from_begin = std::distance(m_buildable_items->begin(), m_buildable_items->FirstRowShown());
+    std::size_t initial_offset_to_end = std::distance(m_buildable_items->FirstRowShown(), m_buildable_items->end());
+    bool initial_last_visible_row_is_end(m_buildable_items->LastVisibleRow() == m_buildable_items->end());
 
     m_buildable_items->Clear(); // the list of items to be populated
 
-
-    boost::shared_ptr<GG::Font> default_font = ClientUI::GetFont();
+    std::shared_ptr<GG::Font> default_font = ClientUI::GetFont();
     const GG::Pt row_size = m_buildable_items->ListRowSize();
 
-    // may be set while populating - used to reselect previously selected row
-    // after populating
-    //GG::ListBox::iterator row_to_select_it = m_buildable_items->end();
-    // may be set while populating - used to rescroll the list after populating
-    GG::ListBox::iterator new_first_row_it = m_buildable_items->end();
-    // counter that keeps track of how many rows have been added so far
-    int i = 0;
-
-
     // populate list with building types
     //DebugLogger() << "BuildDesignatorWnd::BuildSelector::PopulateList() : Adding Buildings ";
     if (m_build_types_shown.find(BT_BUILDING) != m_build_types_shown.end()) {
@@ -824,8 +790,8 @@ void BuildDesignatorWnd::BuildSelector::PopulateList() {
         // craete and insert rows...
         std::vector<GG::ListBox::Row*> rows;
         rows.reserve(std::distance(manager.begin(), manager.end()));
-        for (BuildingTypeManager::iterator it = manager.begin(); it != manager.end(); ++it, ++i) {
-            const std::string name = it->first;
+        for (const std::map<std::string, BuildingType*>::value_type& entry : manager) {
+            const std::string name = entry.first;
             if (!BuildableItemVisible(BT_BUILDING, name))
                 continue;
             ProductionItemRow* item_row = new ProductionItemRow(row_size.x, row_size.y,
@@ -835,12 +801,8 @@ void BuildDesignatorWnd::BuildSelector::PopulateList() {
         }
         m_buildable_items->Insert(rows, false);
         // resize inserted rows and record first row to show
-        for (BuildableItemsListBox::iterator it = m_buildable_items->begin();
-            it != m_buildable_items->end(); ++it)
-        {
-            if ((*it)->DragDropDataType() == initial_first_row_name)
-                new_first_row_it = it;
-            (*it)->Resize(row_size);
+        for (GG::ListBox::Row* row : *m_buildable_items) {
+            row->Resize(row_size);
         }
     }
 
@@ -850,17 +812,15 @@ void BuildDesignatorWnd::BuildSelector::PopulateList() {
         // get ids of designs to show... for specific empire, or for all empires
         std::vector<int> design_ids;
         if (empire)
-            for (Empire::ShipDesignItr it = empire->ShipDesignBegin(); it != empire->ShipDesignEnd(); ++it)
-                design_ids.push_back(*it);
+            design_ids = empire->OrderedShipDesigns();
         else
             for (Universe::ship_design_iterator it = GetUniverse().beginShipDesigns(); it != GetUniverse().endShipDesigns(); ++it)
                 design_ids.push_back(it->first);
 
-        // craete and insert rows...
+        // create and insert rows...
         std::vector<GG::ListBox::Row*> rows;
         rows.reserve(design_ids.size());
-        for (std::vector<int>::const_iterator it = design_ids.begin(); it != design_ids.end(); ++it, ++i) {
-            int ship_design_id = *it;
+        for (int ship_design_id : design_ids) {
             if (!BuildableItemVisible(BT_SHIP, ship_design_id))
                 continue;
             const ShipDesign* ship_design = GetShipDesign(ship_design_id);
@@ -873,24 +833,28 @@ void BuildDesignatorWnd::BuildSelector::PopulateList() {
         }
         m_buildable_items->Insert(rows, false);
         // resize inserted rows and record first row to show
-        for (BuildableItemsListBox::iterator it = m_buildable_items->begin();
-            it != m_buildable_items->end(); ++it)
-        {
-            if ((*it)->DragDropDataType() == initial_first_row_name)
-                new_first_row_it = it;
-            (*it)->Resize(row_size);
+        for (GG::ListBox::Row* row : *m_buildable_items) {
+            row->Resize(row_size);
         }
     }
 
-    if (new_first_row_it != m_buildable_items->end()) {
-        m_buildable_items->SetFirstRowShown(new_first_row_it);
-    } else {
-        if (!m_buildable_items->Empty())
-            m_buildable_items->BringRowIntoView(--m_buildable_items->end());
-        if (initial_scroll_pos < m_buildable_items->NumRows())
-            m_buildable_items->BringRowIntoView(boost::next(m_buildable_items->begin(), initial_scroll_pos));
-    }
-    //DebugLogger() << "Done";
+    // Restore the list scroll state
+    // If we were at the top stay at the top
+    if (initial_offset_from_begin == 0)
+        m_buildable_items->SetFirstRowShown(m_buildable_items->begin());
+
+    // If we were not at the bottom then keep the same first row position
+    else if (!initial_last_visible_row_is_end && initial_offset_from_begin < m_buildable_items->NumRows())
+        m_buildable_items->SetFirstRowShown(
+            std::next(m_buildable_items->begin(), initial_offset_from_begin));
+
+    // otherwise keep the same relative position from the bottom to
+    // preserve the end of list dead space
+    else if (initial_offset_to_end < m_buildable_items->NumRows())
+        m_buildable_items->SetFirstRowShown(
+            std::next(m_buildable_items->begin(), m_buildable_items->NumRows() -  initial_offset_to_end));
+    else
+        m_buildable_items->SetFirstRowShown(m_buildable_items->begin());
 }
 
 void BuildDesignatorWnd::BuildSelector::BuildItemLeftClicked(GG::ListBox::iterator it,
@@ -970,8 +934,7 @@ void BuildDesignatorWnd::BuildSelector::BuildItemRightClicked(GG::ListBox::itera
     std::string popup_label = boost::io::str(FlexibleFormat(UserString("ENC_LOOKUP")) % item_name);
     menu_contents.next_level.push_back(GG::MenuItem(popup_label, 3, false, false));
 
-    GG::PopupMenu popup(pt.x, pt.y, ClientUI::GetFont(), menu_contents, ClientUI::TextColor(),
-                        ClientUI::WndOuterBorderColor(), ClientUI::WndColor(), ClientUI::EditHiliteColor());
+    CUIPopupMenu popup(pt.x, pt.y, menu_contents);
 
     if (popup.Run()) {
         switch (popup.MenuID()) {
@@ -1001,9 +964,9 @@ const std::string BuildDesignatorWnd::PRODUCTION_ITEM_DROP_TYPE = "Production It
 
 BuildDesignatorWnd::BuildDesignatorWnd(GG::X w, GG::Y h) :
     Wnd(GG::X0, GG::Y0, w, h, GG::INTERACTIVE | GG::ONTOP),
-    m_enc_detail_panel(0),
-    m_build_selector(0),
-    m_side_panel(0)
+    m_enc_detail_panel(nullptr),
+    m_build_selector(nullptr),
+    m_side_panel(nullptr)
 {
     m_enc_detail_panel = new EncyclopediaDetailPanel(GG::ONTOP | GG::INTERACTIVE | GG::DRAGABLE | GG::RESIZABLE | CLOSABLE | PINABLE, PROD_PEDIA_WND_NAME);
     GG::Connect(m_enc_detail_panel->ClosingSignal, boost::bind(&BuildDesignatorWnd::HidePedia, this));     // Wnd is manually closed by user
@@ -1021,16 +984,16 @@ BuildDesignatorWnd::BuildDesignatorWnd(GG::X w, GG::Y h) :
     GG::Connect(m_build_selector->ShowPediaSignal,        &BuildDesignatorWnd::ShowPedia,          this);
     GG::Connect(m_build_selector->RequestBuildItemSignal, &BuildDesignatorWnd::BuildItemRequested, this);
 
-    GG::Connect(m_side_panel->PlanetSelectedSignal, PlanetSelectedSignal);
-    GG::Connect(m_side_panel->SystemSelectedSignal, SystemSelectedSignal);
+    GG::Connect(SidePanel::PlanetSelectedSignal, PlanetSelectedSignal);
+    GG::Connect(SidePanel::SystemSelectedSignal, SystemSelectedSignal);
 
     // connect build type button clicks to update display
-    GG::Connect(m_build_selector->m_build_type_buttons[BT_BUILDING]->LeftClickedSignal, ToggleBuildTypeFunctor(this, BT_BUILDING));
-    GG::Connect(m_build_selector->m_build_type_buttons[BT_SHIP]->LeftClickedSignal,     ToggleBuildTypeFunctor(this, BT_SHIP));
+    GG::Connect(m_build_selector->m_build_type_buttons[BT_BUILDING]->CheckedSignal, boost::bind(&BuildDesignatorWnd::ToggleType, this, BT_BUILDING, true));
+    GG::Connect(m_build_selector->m_build_type_buttons[BT_SHIP]->CheckedSignal,     boost::bind(&BuildDesignatorWnd::ToggleType, this, BT_SHIP, true));
 
     // connect availability button clicks to update display
-    GG::Connect(m_build_selector->m_availability_buttons.at(0)->LeftClickedSignal, ToggleAvailabilityFunctor(this, true));    // available items
-    GG::Connect(m_build_selector->m_availability_buttons.at(1)->LeftClickedSignal, ToggleAvailabilityFunctor(this, false));   // UNavailable items
+    GG::Connect(m_build_selector->m_availability_buttons.at(0)->CheckedSignal, boost::bind(&BuildDesignatorWnd::ToggleAvailabilitly, this, true, true));    // available items
+    GG::Connect(m_build_selector->m_availability_buttons.at(1)->CheckedSignal, boost::bind(&BuildDesignatorWnd::ToggleAvailabilitly, this, false, true));   // UNavailable items
 
     AttachChild(m_enc_detail_panel);
     AttachChild(m_build_selector);
@@ -1082,13 +1045,13 @@ void BuildDesignatorWnd::CenterOnBuild(int queue_idx, bool open) {
     const ProductionQueue& queue = empire->GetProductionQueue();
     if (0 <= queue_idx && queue_idx < static_cast<int>(queue.size())) {
         int location_id = queue[queue_idx].location;
-        if (TemporaryPtr<const UniverseObject> build_location = objects.Object(location_id)) {
+        if (std::shared_ptr<const UniverseObject> build_location = objects.Object(location_id)) {
             // centre map on system of build location
             int system_id = build_location->SystemID();
             MapWnd* map = ClientUI::GetClientUI()->GetMapWnd();
             map->CenterOnObject(system_id);
             if (open) {
-                SelectSystem(system_id);
+                HumanClientApp::GetApp()->GetClientUI().GetMapWnd()->SelectSystem(system_id);
                 SelectPlanet(location_id);
             }
         }
@@ -1110,7 +1073,7 @@ void BuildDesignatorWnd::SetBuild(int queue_idx) {
             assert(building_type);
             m_build_selector->DisplayBuildingTypeSignal(building_type);
         } else if (buildType == BT_SHIP) {
-            const ShipDesign* design = GetShipDesign(boost::lexical_cast<int>(queue[queue_idx].item.design_id));
+            const ShipDesign* design = GetShipDesign(queue[queue_idx].item.design_id);
             assert(design);
             m_build_selector->DisplayShipDesignSignal(design);
         }
@@ -1363,13 +1326,16 @@ void BuildDesignatorWnd::SelectDefaultPlanet() {
     // couldn't reselect stored default, so need to find a reasonable other
     // planet to select.  attempt to find one owned by this client's player
 
-    TemporaryPtr<const System> sys = GetSystem(system_id);   // only checking visible objects for this clients empire (and not the latest known objects) as an empire shouldn't be able to use a planet or system it can't currently see as a production location
+    // only checking visible objects for this clients empire (and not the
+    // latest known objects) as an empire shouldn't be able to use a planet or
+    // system it can't currently see as a production location.
+    std::shared_ptr<const System> sys = GetSystem(system_id);
     if (!sys) {
         ErrorLogger() << "BuildDesignatorWnd::SelectDefaultPlanet couldn't get system with id " << system_id;
         return;
     }
 
-    std::vector<TemporaryPtr<const Planet> > planets = Objects().FindObjects<const Planet>(sys->PlanetIDs());
+    std::vector<std::shared_ptr<const Planet>> planets = Objects().FindObjects<const Planet>(sys->PlanetIDs());
 
     if (planets.empty()) {
         this->SelectPlanet(INVALID_OBJECT_ID);
@@ -1381,8 +1347,7 @@ void BuildDesignatorWnd::SelectDefaultPlanet() {
     int best_planet_id = INVALID_OBJECT_ID; // id of selected planet
     double best_planet_pop = -99999.9;                      // arbitrary negative number, so any planet's pop will be better
 
-    for (std::vector<TemporaryPtr<const Planet> >::const_iterator it = planets.begin(); it != planets.end(); ++it) {
-        TemporaryPtr<const Planet> planet = *it;
+    for (std::shared_ptr<const Planet> planet : planets) {
         int planet_id = planet->ID();
         if (!m_side_panel->PlanetSelectable(planet_id))
             continue;
