@@ -1,7 +1,7 @@
 #ifndef _BuildDesignatorWnd_h_
 #define _BuildDesignatorWnd_h_
 
-#include "../universe/Enums.h"
+#include "../universe/EnumsFwd.h"
 #include "../Empire/Empire.h"
 
 #include <GG/Wnd.h>
@@ -19,19 +19,21 @@ public:
     //@}
 
     /** \name Accessors */ //@{
+    bool InWindow(const GG::Pt& pt) const override;
+
+    bool InClient(const GG::Pt& pt) const override;
+
     /** returns set of BulldType shown in this selector */
     const std::set<BuildType>&      GetBuildTypesShown() const;
 
     /** .first -> available items; .second -> unavailable items */
     const std::pair<bool, bool>&    GetAvailabilitiesShown() const;
 
-    virtual bool    InWindow(const GG::Pt& pt) const;
-    virtual bool    InClient(const GG::Pt& pt) const;
     int             SelectedPlanetID() const;
     //@}
 
     /** \name Mutators */ //@{
-    virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr);
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
 
     /** Centres map wnd on location of item on queue with index \a queue_idx
       * and displays info about that item in encyclopedia window
@@ -102,16 +104,16 @@ public:
 
     /** emitted when the indicated build is indicated by the user */
     mutable boost::signals2::signal<void (const ProductionQueue::ProductionItem&, int, int, int)>
-                                            AddBuildToQueueSignal;
+                                                        AddBuildToQueueSignal;
     /** emitted when the quantity of items in a single build queue item is
       * changed by the user */
-    mutable boost::signals2::signal<void (int, int)>  BuildQuantityChangedSignal;
+    mutable boost::signals2::signal<void (int, int)>    BuildQuantityChangedSignal;
     /** emitted when the user selects a system from within this Wnd (but not
       * when this Wnd's system is set programatically) */
-    mutable boost::signals2::signal<void (int)>       SystemSelectedSignal;
+    mutable boost::signals2::signal<void (int)>         SystemSelectedSignal;
     /** emitted when the user changes the planet selection from within this
       * Wnd (but not when this Wnd's selected planet is set programatically) */
-    mutable boost::signals2::signal<void (int)>       PlanetSelectedSignal;
+    mutable boost::signals2::signal<void (int)>         PlanetSelectedSignal;
 
 private:
     class BuildSelector;
