@@ -11,6 +11,7 @@
 #include "../universe/Building.h"
 #include "../universe/Effect.h"
 #include "../universe/Planet.h"
+#include "../universe/Enums.h"
 #include "../Empire/Empire.h"
 #include "../client/human/HumanClientApp.h"
 #include "CUIControls.h"
@@ -34,10 +35,9 @@ namespace {
         const ClientApp* app = ClientApp::GetApp();
         if (!app)
             return retval;
-        const OrderSet& orders = app->Orders();
-        for (OrderSet::const_iterator it = orders.begin(); it != orders.end(); ++it) {
-            if (boost::shared_ptr<ScrapOrder> order = boost::dynamic_pointer_cast<ScrapOrder>(it->second)) {
-                retval[order->ObjectID()] = it->first;
+        for (const std::map<int, OrderPtr>::value_type& entry : app->Orders()) {
+            if (std::shared_ptr<ScrapOrder> order = std::dynamic_pointer_cast<ScrapOrder>(entry.second)) {
+                retval[order->ObjectID()] = entry.first;
             }
         }
         return retval;
@@ -63,21 +63,21 @@ BuildingsPanel::BuildingsPanel(GG::X w, int columns, int planet_id) :
     GG::Connect(m_expand_button->LeftClickedSignal, &BuildingsPanel::ExpandCollapseButtonPressed, this);
 
     // get owner, connect its production queue changed signal to update this panel
-    TemporaryPtr<const UniverseObject> planet = GetUniverseObject(m_planet_id);
+    std::shared_ptr<const UniverseObject> planet = GetUniverseObject(m_planet_id);
     if (planet) {
         if (const Empire* empire = GetEmpire(planet->Owner())) {
             const ProductionQueue& queue = empire->GetProductionQueue();
-            GG::Connect(queue.ProductionQueueChangedSignal, &BuildingsPanel::Refresh, this);
+            GG::Connect(queue.ProductionQueueChangedSignal, &BuildingsPanel::RequirePreRender, this);
         }
     }
 
-    Refresh();
+    RequirePreRender();
 }
 
 BuildingsPanel::~BuildingsPanel() {
     // delete building indicators
-    for (std::vector<BuildingIndicator*>::iterator it = m_building_indicators.begin(); it != m_building_indicators.end(); ++it)
-        delete *it;
+    for (BuildingIndicator* indicator : m_building_indicators)
+        delete indicator;
     m_building_indicators.clear();
 }
 
@@ -92,18 +92,17 @@ void BuildingsPanel::Update() {
     //std::cout << "BuildingsPanel::Update" << std::endl;
 
     // remove old indicators
-    for (std::vector<BuildingIndicator*>::iterator it = m_building_indicators.begin(); it != m_building_indicators.end(); ++it) {
-        DetachChild(*it);
-        delete (*it);
+    for (BuildingIndicator* indicator : m_building_indicators) {
+        DetachChild(indicator);
+        delete (indicator);
     }
     m_building_indicators.clear();
 
-    TemporaryPtr<const Planet> planet = GetPlanet(m_planet_id);
+    std::shared_ptr<const Planet> planet = GetPlanet(m_planet_id);
     if (!planet) {
         ErrorLogger() << "BuildingsPanel::Update couldn't get planet with id " << m_planet_id;
         return;
     }
-    const std::set<int>& buildings = planet->BuildingIDs();
     int system_id = planet->SystemID();
 
     const int indicator_size = static_cast<int>(Value(Width() * 1.0 / m_columns));
@@ -113,16 +112,14 @@ void BuildingsPanel::Update() {
     const std::set<int>& this_client_stale_object_info = GetUniverse().EmpireStaleKnowledgeObjectIDs(this_client_empire_id);
 
     // get existing / finished buildings and use them to create building indicators
-    for (std::set<int>::const_iterator it = buildings.begin(); it != buildings.end(); ++it) {
-        int object_id = *it;
-
+    for (int object_id : planet->BuildingIDs()) {
         // skip known destroyed and stale info objects
         if (this_client_known_destroyed_objects.find(object_id) != this_client_known_destroyed_objects.end())
             continue;
         if (this_client_stale_object_info.find(object_id) != this_client_stale_object_info.end())
             continue;
 
-        TemporaryPtr<const Building> building = GetBuilding(object_id);
+        std::shared_ptr<const Building> building = GetBuilding(object_id);
         if (!building) {
             ErrorLogger() << "BuildingsPanel::Update couldn't get building with id: " << object_id << " on planet " << planet->Name();
             continue;
@@ -142,22 +139,20 @@ void BuildingsPanel::Update() {
     if (!empire)
         return;
 
-    const ProductionQueue& queue = empire->GetProductionQueue();
-
-    int queue_index = 0;
-    for (ProductionQueue::const_iterator queue_it = queue.begin(); queue_it != queue.end(); ++queue_it, ++queue_index) {
-        const ProductionQueue::Element elem = *queue_it;
-
+    int queue_index = -1;
+    for (const ProductionQueue::Element& elem : empire->GetProductionQueue()) {
+        ++queue_index;
+        //std::cout << "queue index: " << queue_index << " elem: " << elem.Dump() << std::endl;
         if (elem.item.build_type != BT_BUILDING) continue;  // don't show in-progress ships in BuildingsPanel...
         if (elem.location != m_planet_id) continue;         // don't show buildings located elsewhere
 
         double total_cost;
         int total_turns;
         double turn_spending = elem.allocated_pp;
-        boost::tie(total_cost, total_turns) = empire->ProductionCostAndTime(elem);
+        std::tie(total_cost, total_turns) = empire->ProductionCostAndTime(elem);
 
         double progress = std::max(0.0f, empire->ProductionStatus(queue_index));
-        double turns_completed = total_turns * std::min<double>(1.0, progress / total_cost);
+        double turns_completed = progress / std::max(total_cost, 1.0);
         BuildingIndicator* ind = new BuildingIndicator(GG::X(indicator_size), elem.item.name,
                                                        turns_completed, total_turns, total_cost, turn_spending);
 
@@ -165,21 +160,30 @@ void BuildingsPanel::Update() {
     }
 }
 
+void BuildingsPanel::PreRender() {
+    AccordionPanel::PreRender();
+    RefreshImpl();
+}
+
 void BuildingsPanel::Refresh() {
+    RequirePreRender();
+}
+
+void BuildingsPanel::RefreshImpl() {
     Update();
     DoLayout();
 }
 
 void BuildingsPanel::EnableOrderIssuing(bool enable/* = true*/) {
-    for (std::vector<BuildingIndicator*>::iterator it = m_building_indicators.begin();
-         it != m_building_indicators.end(); ++it)
-    { (*it)->EnableOrderIssuing(enable); }
+    for (BuildingIndicator* indicator : m_building_indicators)
+    { indicator->EnableOrderIssuing(enable); }
 }
 
 void BuildingsPanel::ExpandCollapseButtonPressed()
 { ExpandCollapse(!s_expanded_map[m_planet_id]); }
 
 void BuildingsPanel::DoLayout() {
+    auto old_size = Size();
     AccordionPanel::DoLayout();
 
     int row = 0;
@@ -192,9 +196,7 @@ void BuildingsPanel::DoLayout() {
     // update size of panel and position and visibility of widgets
     if (!s_expanded_map[m_planet_id]) {
         int n = 0;
-        for (std::vector<BuildingIndicator*>::iterator it = m_building_indicators.begin(); it != m_building_indicators.end(); ++it) {
-            BuildingIndicator* ind = *it;
-
+        for (BuildingIndicator* ind : m_building_indicators) {
             const GG::Pt ul = GG::Pt(MeterIconSize().x * n, GG::Y0);
             const GG::Pt lr = ul + MeterIconSize();
 
@@ -209,9 +211,7 @@ void BuildingsPanel::DoLayout() {
 
         height = m_expand_button->Height();
     } else {
-        for (std::vector<BuildingIndicator*>::iterator it = m_building_indicators.begin(); it != m_building_indicators.end(); ++it) {
-            BuildingIndicator* ind = *it;
-
+        for (BuildingIndicator* ind : m_building_indicators) {
             const GG::Pt ul = GG::Pt(GG::X(padding * (column + 1) + indicator_size * column), GG::Y(padding * (row + 1) + indicator_size * row));
             const GG::Pt lr = ul + GG::Pt(GG::X(indicator_size), GG::Y(indicator_size));
 
@@ -244,11 +244,10 @@ void BuildingsPanel::DoLayout() {
 
     Resize(GG::Pt(Width(), height));
 
-    Wnd::MoveChildUp(m_expand_button);
-
-    m_expand_button->MoveTo(GG::Pt(Width() - m_expand_button->Width(), GG::Y0));
-
     SetCollapsed(!s_expanded_map[m_planet_id]);
+
+    if (old_size != Size() && Parent())
+        Parent()->RequirePreRender();
 }
 
 std::map<int, bool> BuildingsPanel::s_expanded_map;
@@ -260,42 +259,49 @@ ScanlineRenderer BuildingIndicator::s_scanline_shader;
 
 BuildingIndicator::BuildingIndicator(GG::X w, int building_id) :
     GG::Wnd(GG::X0, GG::Y0, w, GG::Y(Value(w)), GG::INTERACTIVE),
-    m_graphic(0),
-    m_scrap_indicator(0),
-    m_progress_bar(0),
+    m_graphic(nullptr),
+    m_scrap_indicator(nullptr),
+    m_progress_bar(nullptr),
     m_building_id(building_id),
     m_order_issuing_enabled(true)
 {
-    if (TemporaryPtr<const Building> building = GetBuilding(m_building_id))
-        GG::Connect(building->StateChangedSignal,   &BuildingIndicator::Refresh,     this);
+    if (std::shared_ptr<const Building> building = GetBuilding(m_building_id))
+        GG::Connect(building->StateChangedSignal,   &BuildingIndicator::RequirePreRender,     this);
     Refresh();
 }
 
 BuildingIndicator::BuildingIndicator(GG::X w, const std::string& building_type,
                                      double turns_completed, double total_turns, double total_cost, double turn_spending) :
     GG::Wnd(GG::X0, GG::Y0, w, GG::Y(Value(w)), GG::INTERACTIVE),
-    m_graphic(0),
-    m_scrap_indicator(0),
-    m_progress_bar(0),
+    m_graphic(nullptr),
+    m_scrap_indicator(nullptr),
+    m_progress_bar(nullptr),
     m_building_id(INVALID_OBJECT_ID),
     m_order_issuing_enabled(true)
 {
-    boost::shared_ptr<GG::Texture> texture = ClientUI::BuildingIcon(building_type);
+    std::shared_ptr<GG::Texture> texture = ClientUI::BuildingIcon(building_type);
 
     const BuildingType* type = GetBuildingType(building_type);
     const std::string& desc = type ? type->Description() : "";
 
-    SetBrowseInfoWnd(boost::shared_ptr<GG::BrowseInfoWnd>(
-        new IconTextBrowseWnd(texture, UserString(building_type), UserString(desc))));
+    SetBrowseInfoWnd(std::make_shared<IconTextBrowseWnd>(
+        texture, UserString(building_type), UserString(desc)));
 
     m_graphic = new GG::StaticGraphic(texture, GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE);
     AttachChild(m_graphic);
 
-    m_progress_bar = new MultiTurnProgressBar(total_turns, turns_completed, total_cost, turn_spending, GG::LightColor(ClientUI::TechWndProgressBarBackgroundColor()),
-                                              ClientUI::TechWndProgressBarColor(), GG::LightColor(ClientUI::ResearchableTechFillColor()));
+    float next_progress = turn_spending / std::max(1.0, total_cost);
+
+    m_progress_bar = new MultiTurnProgressBar(total_turns,
+                                              turns_completed,
+                                              next_progress,
+                                              GG::LightColor(ClientUI::TechWndProgressBarBackgroundColor()),
+                                              ClientUI::TechWndProgressBarColor(),
+                                              GG::LightColor(ClientUI::ResearchableTechFillColor()));
+
     AttachChild(m_progress_bar);
 
-    Refresh();
+    RequirePreRender();
 }
 
 void BuildingIndicator::Render() {
@@ -338,10 +344,15 @@ void BuildingIndicator::Render() {
     s_scanline_shader.StopUsing();
 }
 
+void BuildingIndicator::PreRender() {
+    GG::Wnd::PreRender();
+    Refresh();
+}
+
 void BuildingIndicator::Refresh() {
     SetBrowseModeTime(GetOptionsDB().Get<int>("UI.tooltip-delay"));
 
-    TemporaryPtr<const Building> building = GetBuilding(m_building_id);
+    std::shared_ptr<const Building> building = GetBuilding(m_building_id);
     if (!building)
         return;
 
@@ -349,15 +360,15 @@ void BuildingIndicator::Refresh() {
 
     if (m_graphic) {
         delete m_graphic;
-        m_graphic = 0;
+        m_graphic = nullptr;
     }
     if (m_scrap_indicator) {
         delete m_scrap_indicator;
-        m_scrap_indicator = 0;
+        m_scrap_indicator = nullptr;
     }
 
     if (const BuildingType* type = GetBuildingType(building->BuildingTypeName())) {
-        boost::shared_ptr<GG::Texture> texture = ClientUI::BuildingIcon(type->Name());
+        std::shared_ptr<GG::Texture> texture = ClientUI::BuildingIcon(type->Name());
         m_graphic = new GG::StaticGraphic(texture, GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE);
         AttachChild(m_graphic);
 
@@ -367,12 +378,12 @@ void BuildingIndicator::Refresh() {
         if (GetOptionsDB().Get<bool>("UI.dump-effects-descriptions") && !type->Effects().empty())
             desc += "\n" + Dump(type->Effects());
 
-        SetBrowseInfoWnd(boost::shared_ptr<GG::BrowseInfoWnd>(
-            new IconTextBrowseWnd(texture, UserString(type->Name()), desc)));
+        SetBrowseInfoWnd(std::make_shared<IconTextBrowseWnd>(
+            texture, UserString(type->Name()), desc));
     }
 
     if (building && building->OrderedScrapped()) {
-        boost::shared_ptr<GG::Texture> scrap_texture = ClientUI::GetTexture(ClientUI::ArtDir() / "misc" / "scrapped.png", true);
+        std::shared_ptr<GG::Texture> scrap_texture = ClientUI::GetTexture(ClientUI::ArtDir() / "misc" / "scrapped.png", true);
         m_scrap_indicator = new GG::StaticGraphic(scrap_texture, GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE);
         AttachChild(m_scrap_indicator);
     }
@@ -397,7 +408,7 @@ void BuildingIndicator::RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys)
     // queued production item, and that the owner of the building is this
     // client's player's empire
     int empire_id = HumanClientApp::GetApp()->EmpireID();
-    TemporaryPtr<Building> building = GetBuilding(m_building_id);
+    std::shared_ptr<Building> building = GetBuilding(m_building_id);
     if (!building)
         return;
 
@@ -425,8 +436,7 @@ void BuildingIndicator::RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys)
     }
 
 
-    GG::PopupMenu popup(pt.x, pt.y, ClientUI::GetFont(), menu_contents, ClientUI::TextColor(),
-                        ClientUI::WndOuterBorderColor(), ClientUI::WndColor(), ClientUI::EditHiliteColor());
+    CUIPopupMenu popup(pt.x, pt.y, menu_contents);
     if (popup.Run()) {
         switch (popup.MenuID()) {
         case 3: { // scrap building
