@@ -21,14 +21,14 @@ public:
     //@}
 
     /** \name Mutators */ //@{
-    /** expands or collapses panel to show details or just summary info */
-    void ExpandCollapse(bool expanded);
+    void PreRender() override;
 
-    /** updates indicators with values of associated object.  Does not do layout and resizing. */
-    void Update();
     /** updates, redoes layout, resizes indicator */
     void Refresh();
 
+    /** expands or collapses panel to show details or just summary info */
+    void ExpandCollapse(bool expanded);
+
     /** Enables, or disables if \a enable is false, issuing orders via this panel. */
     void EnableOrderIssuing(bool enable = true);
     //@}
@@ -38,7 +38,11 @@ public:
 protected:
     /** \name Mutators */ //@{
     /** resizes panel and positions widgets */
-    virtual void DoLayout();
+    void DoLayout() override;
+
+    /** updates indicators with values of associated object.  Does not do layout and resizing. */
+    void Update();
+    void RefreshImpl();
     //@}
 
 private:
@@ -68,12 +72,15 @@ public:
                       double turns_completed, double total_turns, double total_cost, double turn_spending);
 
     /** \name Mutators */ //@{
-    virtual void    Render();
-    void            Refresh();
+    void PreRender() override;
 
-    virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr);
-    virtual void    MouseWheel(const GG::Pt& pt, int move, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
+    void Render() override;
+
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
+
+    void MouseWheel(const GG::Pt& pt, int move, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
 
     /** Enables, or disables if \a enable is false, issuing orders via this BuildingIndicator. */
     void            EnableOrderIssuing(bool enable = true);
@@ -82,6 +89,7 @@ public:
     mutable boost::signals2::signal<void (int)> RightClickedSignal;
 
 private:
+    void            Refresh();
     void            DoLayout();
 
     static ScanlineRenderer s_scanline_shader;
