@@ -35,10 +35,18 @@ public:
     /** \name Structors */ //@{
     CUILabel(const std::string& str,
              GG::Flags<GG::TextFormat> format = GG::FORMAT_NONE,
-             GG::Flags<GG::WndFlag> flags = GG::NO_WND_FLAGS);
+             GG::Flags<GG::WndFlag> flags = GG::NO_WND_FLAGS,
+             GG::X x = GG::X0, GG::Y y = GG::Y0, GG::X w = GG::X1, GG::Y h = GG::Y1);
+
+    CUILabel(const std::string& str,
+             const std::vector<std::shared_ptr<GG::Font::TextElement>>& text_elements,
+             GG::Flags<GG::TextFormat> format = GG::FORMAT_NONE,
+             GG::Flags<GG::WndFlag> flags = GG::NO_WND_FLAGS,
+             GG::X x = GG::X0, GG::Y y = GG::Y0, GG::X w = GG::X1, GG::Y h = GG::Y1);
     //@}
+
     /** \name Mutators */ //@{
-    virtual void RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
+    void RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
     //@}
 };
 
@@ -46,35 +54,29 @@ public:
 class CUIButton : public GG::Button {
 public:
     /** \name Structors */ //@{
-    CUIButton(const std::string& str); ///< basic ctor
-
-    CUIButton(const std::string& str, GG::Clr background, GG::Clr border);
+    CUIButton(const std::string& str);
 
     CUIButton(const GG::SubTexture& unpressed, const GG::SubTexture& pressed, const GG::SubTexture& rollover);
     //@}
 
     /** \name Accessors */ //@{
-    virtual GG::Pt MinUsableSize() const;
+    GG::Pt MinUsableSize() const override;
 
-    virtual bool InWindow(const GG::Pt& pt) const;
+    bool InWindow(const GG::Pt& pt) const override;
     //@}
 
     /** \name Mutators */ //@{
-    virtual void MouseHere(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    void         SetCheck(bool b = true);
+    void MouseHere(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
     //@}
 
 protected:
     /** \name Mutators control */ //@{
-    virtual void RenderPressed();
-    virtual void RenderRollover();
-    virtual void RenderUnpressed();
-    //@}
+    void RenderPressed() override;
 
-private:
-    GG::Clr m_border_color;
-    int     m_border_thick;
-    bool    m_checked;     ///< true when this button in a checked, active state
+    void RenderRollover() override;
+
+    void RenderUnpressed() override;
+    //@}
 };
 
 class SettableInWindowCUIButton : public CUIButton {
@@ -84,7 +86,7 @@ public:
     //@}
 
     /** \name Accessors */ //@{
-    virtual bool    InWindow(const GG::Pt& pt) const;
+    bool InWindow(const GG::Pt& pt) const override;
     //@}
 
 private:
@@ -96,22 +98,24 @@ class CUIArrowButton : public GG::Button {
 public:
     /** \name Structors */ //@{
     CUIArrowButton(ShapeOrientation orientation, bool fill_background,
-                   GG::Flags<GG::WndFlag> flags = GG::INTERACTIVE); ///< basic ctor
+                   GG::Flags<GG::WndFlag> flags = GG::INTERACTIVE);
     //@}
 
     /** \name Accessors */ //@{
-    virtual bool    InWindow(const GG::Pt& pt) const;
+    bool InWindow(const GG::Pt& pt) const override;
     //@}
 
     /** \name Mutators */ //@{
-    virtual void    MouseHere(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
+    void MouseHere(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
     //@}
 
 protected:
     /** \name Mutators control */ //@{
-    virtual void RenderPressed();
-    virtual void RenderRollover();
-    virtual void RenderUnpressed();
+    void RenderPressed() override;
+
+    void RenderRollover() override;
+
+    void RenderUnpressed() override;
     //@}
 
 private:
@@ -122,34 +126,82 @@ private:
 /** \brief A FreeOrion styled check box state button. */
 class CUICheckBoxRepresenter : public GG::StateButtonRepresenter {
 public:
-    virtual void Render(const GG::StateButton& button) const;
+    void Render(const GG::StateButton& button) const override;
 
-    virtual void OnChecked(bool checked) const;
+    void OnChanged(const GG::StateButton& button, GG::StateButton::ButtonState prev_state) const override;
+
+    void OnChecked(bool checked) const override;
 };
 
 /** \brief A FreeOrion styled radio state button. */
 class CUIRadioRepresenter : public GG::StateButtonRepresenter {
 public:
-    virtual void Render(const GG::StateButton& button) const;
+    void Render(const GG::StateButton& button) const override;
 
-    virtual void OnChecked(bool checked) const;
+    void OnChanged(const GG::StateButton& button, GG::StateButton::ButtonState prev_state) const override;
+
+    void OnChecked(bool checked) const override;
 };
 
 /** \brief A FreeOrion styled TabBar tab. */
 class CUITabRepresenter : public GG::StateButtonRepresenter {
 public:
-    virtual void Render(const GG::StateButton& button) const;
+    void Render(const GG::StateButton& button) const override;
 
-    virtual void OnChecked(bool checked) const;
+    void OnChanged(const GG::StateButton& button, GG::StateButton::ButtonState prev_state) const override;
 
-    virtual GG::Pt MinUsableSize(const GG::StateButton& button) const;
+    void OnChecked(bool checked) const override;
+
+    GG::Pt MinUsableSize(const GG::StateButton& button) const override;
 };
 
+/** @brief A FreeOrion styled label toggle button. */
+class CUILabelButtonRepresenter : public GG::StateButtonRepresenter {
+public:
+    void Render(const GG::StateButton& button) const override;
+
+    void OnChanged(const GG::StateButton& button, GG::StateButton::ButtonState prev_state) const override;
+
+    void OnChecked(bool checked) const override;
+};
+
+
+/** @brief A FreeOrion styled icon toggle button.
+ *
+ * Renders a SubTexture depending on the checked state of the CUIToggleButton.
+ * When the @a button has single texture and is unchecked, it renders the same
+ * icon recolored towards grey.  If a second SubTexture is assigned to the
+ * @a button, it will render the appropriate SubTexture.  When a mouse pointer
+ * hovers over this button, the opposing SubTexture and color are used and
+ * highlighted.
+ */
+class CUIIconButtonRepresenter : public GG::StateButtonRepresenter {
+public:
+    CUIIconButtonRepresenter(std::shared_ptr<GG::SubTexture> icon,
+                             const GG::Clr& highlight_clr);
+
+    CUIIconButtonRepresenter(std::shared_ptr<GG::SubTexture> unchecked_icon,
+                             const GG::Clr& unchecked_clr,
+                             std::shared_ptr<GG::SubTexture> checked_icon,
+                             const GG::Clr& checked_clr);
+
+    void Render(const GG::StateButton& button) const override;
+
+    void OnChecked(bool checked) const override;
+
+private:
+    std::shared_ptr<GG::SubTexture> m_unchecked_icon;
+    std::shared_ptr<GG::SubTexture> m_checked_icon;
+    GG::Clr                             m_unchecked_color;
+    GG::Clr                             m_checked_color;
+};
+
+
 /** a FreeOrion StateButton control */
 class CUIStateButton : public GG::StateButton {
 public:
     /** \name Structors */ //@{
-    CUIStateButton(const std::string& str, GG::Flags<GG::TextFormat> format, boost::shared_ptr<GG::StateButtonRepresenter> representer); ///< ctor
+    CUIStateButton(const std::string& str, GG::Flags<GG::TextFormat> format, std::shared_ptr<GG::StateButtonRepresenter> representer);
     //@}
 };
 
@@ -157,13 +209,12 @@ public:
 class CUITabBar : public GG::TabBar {
 public:
     /** \name Structors */ ///@{
-    /** Basic ctor. */
-    CUITabBar(const boost::shared_ptr<GG::Font>& font, GG::Clr color,
+    CUITabBar(const std::shared_ptr<GG::Font>& font, GG::Clr color,
               GG::Clr text_color);
     //@}
 
 private:
-    virtual void DistinguishCurrentTab(const std::vector<GG::StateButton*>& tab_buttons);
+    void DistinguishCurrentTab(const std::vector<GG::StateButton*>& tab_buttons) override;
 };
 
 /** a FreeOrion Scroll control */
@@ -172,14 +223,22 @@ public:
     /** represents the tab button for a CUIScroll */
     class ScrollTab : public GG::Button {
     public:
-        ScrollTab(GG::Orientation orientation, int scroll_width, GG::Clr color, GG::Clr border_color); ///< basic ctor
-        virtual void SetColor(GG::Clr c);
-        virtual void Render();
-        virtual void LButtonDown(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-        virtual void LButtonUp(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-        virtual void LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-        virtual void MouseEnter(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-        virtual void MouseLeave();
+        ScrollTab(GG::Orientation orientation, int scroll_width, GG::Clr color, GG::Clr border_color);
+
+        void SetColor(GG::Clr c) override;
+
+        void Render() override;
+
+        void LButtonDown(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+        void LButtonUp(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+        void LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+        void MouseEnter(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+        void MouseLeave() override;
+
     private:
         GG::Clr m_border_color;
         GG::Orientation m_orientation;
@@ -188,12 +247,13 @@ public:
     };
 
     /** \name Structors */ //@{
-    CUIScroll(GG::Orientation orientation); ///< basic ctor
+    CUIScroll(GG::Orientation orientation);
     //@}
 
     /** \name Mutators */ //@{
-    virtual void Render();
-    virtual void SizeMove(const GG::Pt& ul, const GG::Pt& lr);
+    void Render() override;
+
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
     //@}
 
 protected:
@@ -204,11 +264,11 @@ protected:
 class CUIListBox : public GG::ListBox {
 public:
     /** \name Structors */ //@{
-    CUIListBox(void); ///< basic ctor
+    CUIListBox(void);
     //@}
 
     /** \name Mutators */ //@{
-    virtual void Render();
+    void Render() override;
     //@}
 };
 
@@ -216,21 +276,29 @@ public:
 class CUIDropDownList : public GG::DropDownList {
 public:
     /** \name Structors */ //@{
-    explicit CUIDropDownList(size_t num_shown_elements); ///< basic ctor
+    explicit CUIDropDownList(size_t num_shown_elements);
     //@}
 
+    /** Return the width of the dropped row which excludes the DropArrow. */
+    GG::X DroppedRowWidth() const override;
+
+    GG::Pt ClientLowerRight() const override;
+
     /** \name Mutators */ //@{
-    virtual void    Render();
-    virtual void    LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    MouseEnter(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    MouseLeave();
+    void Render() override;
+
+    void LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void MouseEnter(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void MouseLeave() override;
 
     void            DisableDropArrow();  ///< disables rendering of the small downward-facing arrow on the right of the control
     void            EnableDropArrow();   ///< enables rendering of the small downward-facing arrow on the right of the control
     //@}
 
 private:
-    virtual void    InitBuffer();
+    void InitBuffer() override;
 
     bool    m_render_drop_arrow;
     bool    m_mouse_here;
@@ -240,15 +308,19 @@ private:
 class CUIEdit : public GG::Edit {
 public:
     /** \name Structors */ //@{
-    CUIEdit(const std::string& str); ///< basic ctor
+    CUIEdit(const std::string& str);
     //@}
 
     /** \name Mutators */ //@{
-    virtual void RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void KeyPress(GG::Key key, boost::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys);
-    virtual void GainingFocus();
-    virtual void LosingFocus();
-    virtual void Render();
+    void RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void KeyPress(GG::Key key, std::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void GainingFocus() override;
+
+    void LosingFocus() override;
+
+    void Render() override;
     //@}
 
     mutable boost::signals2::signal<void ()> GainingFocusSignal;
@@ -259,12 +331,13 @@ public:
 class CUIMultiEdit : public GG::MultiEdit {
 public:
     /** \name Structors */ //@{
-    CUIMultiEdit(const std::string& str, GG::Flags<GG::MultiEditStyle> style = GG::MULTI_LINEWRAP); ///< basic ctor
+    CUIMultiEdit(const std::string& str, GG::Flags<GG::MultiEditStyle> style = GG::MULTI_LINEWRAP);
     //@}
 
     /** \name Mutators */ //@{
-    virtual void    Render();
-    virtual void    RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
+    void Render() override;
+
+    void RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
     //@}
 };
 
@@ -272,33 +345,44 @@ public:
 class CUILinkTextMultiEdit : public CUIMultiEdit, public TextLinker {
 public:
     /** \name Structors */ //@{
-    CUILinkTextMultiEdit(const std::string& str, GG::Flags<GG::MultiEditStyle> style = GG::MULTI_LINEWRAP); ///< basic ctor
+    CUILinkTextMultiEdit(const std::string& str, GG::Flags<GG::MultiEditStyle> style = GG::MULTI_LINEWRAP);
     //@}
+
     /** \name Accessors */ //@{
-    virtual const std::vector<GG::Font::LineData>&  GetLineData() const;
-    virtual const boost::shared_ptr<GG::Font>&      GetFont() const;
-    virtual GG::Pt                                  TextUpperLeft() const;
-    virtual GG::Pt                                  TextLowerRight() const;
-    virtual const std::string&                      RawText() const;
+    const std::vector<GG::Font::LineData>& GetLineData() const override;
+
+    const std::shared_ptr<GG::Font>& GetFont() const override;
+
+    GG::Pt TextUpperLeft() const override;
+
+    GG::Pt TextLowerRight() const override;
+
+    const std::string& RawText() const override;
     //@}
 
     /** \name Mutators */ //@{
-    virtual void    Render();
-    virtual void    LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    MouseHere(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    MouseLeave();
-    virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr);   // needed primarily so the SetText call will take a RawText
+    void Render() override;
+
+    void LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void MouseHere(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void MouseLeave() override;
+
+    /** Needed primarily so the SetText call will take a RawText. */
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
 
     /** sets the text to \a str; may resize the window.  If the window was
         constructed to fit the size of the text (i.e. if the second ctor type
         was used), calls to this function cause the window to be resized to
         whatever space the newly rendered text occupies. */
-    virtual void    SetText(const std::string& str);
+    void SetText(const std::string& str) override;
     //@}
 
 private:
-    virtual void    SetLinkedText(const std::string& str);
+    void SetLinkedText(const std::string& str) override;
 
     bool            m_already_setting_text_so_dont_link;
     std::string     m_raw_text;
@@ -316,33 +400,52 @@ struct CUISimpleDropDownListRow : public GG::ListBox::Row {
   * is an icon image, not text.
   * The icon may have one or two numerical values.  If one, just that number is
   * displayed.  If two, the first number is displayed followed by the second in
-  * brackets "()" */
+  * brackets "()"
+
+  * Sizing StatisticIcon correctly in the constructor saves time because resizing the value string
+  * is processor intensive.
+  */
 class StatisticIcon : public GG::Control {
 public:
     /** \name Structors */ //@{
-    StatisticIcon(const boost::shared_ptr<GG::Texture> texture);///< initialized with no value (just an icon)
+    StatisticIcon(const std::shared_ptr<GG::Texture> texture,
+                  GG::X x = GG::X0, GG::Y y = GG::Y0,
+                  GG::X w = GG::X1, GG::Y h = GG::Y1); ///< initialized with no value (just an icon)
 
-    StatisticIcon(const boost::shared_ptr<GG::Texture> texture,
-                  double value, int digits, bool showsign);     ///< initializes with one value
+    StatisticIcon(const std::shared_ptr<GG::Texture> texture,
+                  double value, int digits, bool showsign,
+                  GG::X x = GG::X0, GG::Y y = GG::Y0,
+                  GG::X w = GG::X1, GG::Y h = GG::Y1); ///< initializes with one value
 
-    StatisticIcon(const boost::shared_ptr<GG::Texture> texture,
+    StatisticIcon(const std::shared_ptr<GG::Texture> texture,
                   double value0, double value1, int digits0, int digits1,
-                  bool showsign0, bool showsign1);              ///< initializes with two values
+                  bool showsign0, bool showsign1,
+                  GG::X x = GG::X0, GG::Y y = GG::Y0,
+                  GG::X w = GG::X1, GG::Y h = GG::Y1); ///< initializes with two values
     //@}
 
     /** \name Accessors */ //@{
     double          GetValue(int index = 0) const;
+    GG::Pt          MinUsableSize() const override;
     //@}
 
     /** \name Mutators */ //@{
-    virtual void    Render() {}
+    void PreRender() override;
 
-    virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr);
-    virtual void    LButtonDown(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    RButtonDown(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    MouseWheel(const GG::Pt& pt, int move, GG::Flags<GG::ModKey> mod_keys);
+    void Render() override
+    {}
+
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
+
+    void LButtonDown(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void RButtonDown(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void MouseWheel(const GG::Pt& pt, int move, GG::Flags<GG::ModKey> mod_keys) override;
 
     void            SetValue(double value, int index = 0);  ///< sets displayed \a value with \a index
     //@}
@@ -352,7 +455,6 @@ public:
 
 private:
     void            DoLayout();
-    void            Refresh();
     GG::Clr         ValueColor(int index) const;        ///< returns colour in which to draw value
 
     int                 m_num_values;
@@ -372,11 +474,11 @@ public:
     //@}
 
     /** \name Accessors */ //@{
-    virtual bool    InWindow(const GG::Pt& pt) const;
+    bool InWindow(const GG::Pt& pt) const override;
     //@}
 
     /** \name Mutators */ //@{
-    void            Render();
+    void Render() override;
     //@}
 private:
 };
@@ -385,8 +487,7 @@ private:
 class SpeciesSelector : public CUIDropDownList {
 public:
     /** \name Structors */ //@{
-    SpeciesSelector(GG::X w, GG::Y h);                                                  ///< populates with all species in SpeciesManager
-    SpeciesSelector(GG::X w, GG::Y h, const std::vector<std::string>& species_names);   ///< populates with the species in \a species_names
+    SpeciesSelector(GG::X w, GG::Y h);                          ///< populates with all species in SpeciesManager
     //@}
 
     /** \name Accessors */ //@{
@@ -396,7 +497,6 @@ public:
 
     /** \name Mutators */ //@{
     void SelectSpecies(const std::string& species_name);
-    void SetSpecies(const std::vector<std::string>& species_names);         ///< sets the species that can be selected
     //@}
 
     mutable boost::signals2::signal<void (const std::string&)> SpeciesChangedSignal;
@@ -431,14 +531,18 @@ class ColorSelector : public GG::Control {
 public:
     /** \name Structors */ //@{
     ColorSelector(GG::Clr color, GG::Clr default_color);
+
     virtual ~ColorSelector();
     //@}
 
     /** \name Mutators */ //@{
-    virtual void Render();
-    virtual void LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void SizeMove(const GG::Pt& ul, const GG::Pt& lr);
+    void Render() override;
+
+    void LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
     //@}
 
     mutable boost::signals2::signal<void (const GG::Clr&)> ColorChangedSignal;
@@ -454,7 +558,7 @@ class FileDlg : public GG::FileDlg {
 public:
     /** \name Structors */ //@{
     FileDlg(const std::string& directory, const std::string& filename, bool save, bool multi,
-            const std::vector<std::pair<std::string, std::string> >& types);
+            const std::vector<std::pair<std::string, std::string>>& types);
     //@}
 };
 
@@ -467,11 +571,11 @@ public:
     //@}
 
     /** \name Accessors */ //@{
-    virtual GG::Pt  MinUsableSize() const;
+    GG::Pt MinUsableSize() const override;
     //@}
 
     /** \name Mutators */ //@{
-    virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr);
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
 
     void            SetTotalPointsCost(float total_points, float total_cost);
     void            SetLocalPointsCost(float local_points, float local_cost, const std::string& location_name);
@@ -505,34 +609,51 @@ private:
 /** Displays progress that is divided over mulitple turns, as in the Research and Production screens. */
 class MultiTurnProgressBar : public GG::Control {
 public:
-    /** \name Structors */ //@{
-    /** ctor */
-    MultiTurnProgressBar(int total_turns, double turns_completed, double total_cost, double turn_spending,
-                         const GG::Clr& bar_color, const GG::Clr& background, const GG::Clr& outline_color);
+    /** @name Structors */ //@{
+    /** Ctor
+    * @param[in] num_segments Number of segments in the bar
+    * @param[in] percent_completed Percent(0.0-1.0) of bar to fill,
+    * @param[in] percent_predicted Percent(0.0-1.0) of bar to fill after
+    *               @p percent_completed.  Not to exceed 1.0 after addition
+    *               to @p percent_completed
+    * @param[in] bar_color Color for @p percent_completed, adjusted to lighter
+    *               color for @p percent_predicted
+    * @param[in] bg_color Color for background of this control
+    * @param[in] outline_color Color for the border.  Adjusted to darker color
+    *               for segment lines within @p percent_completed portion.
+    *               Adjusted to lighter color for segment lines after
+    *               @p percent_completed portion
+    */
+    MultiTurnProgressBar(int num_segments, float percent_completed,
+                         float percent_predicted, const GG::Clr& bar_color,
+                         const GG::Clr& bg_color, const GG::Clr& outline_color);
     //@}
 
-    /** \name Mutators */ //@{
-    virtual void Render();
+    /** @name Mutators */ //@{
+    void Render() override;
     //@}
 
 private:
-    int     m_total_turns;
-    double  m_turns_completed;
-    double  m_total_cost;
-    double  m_turn_spending;
-    GG::Clr m_bar_color;
-    GG::Clr m_background;
-    GG::Clr m_outline_color;
+    int m_num_segments;
+    float m_perc_completed;
+    float m_perc_predicted;
+    GG::Clr m_clr_bar;
+    GG::Clr m_clr_bg;
+    GG::Clr m_clr_outline;
 };
 
 /** Displays current rendering frames per second. */
 class FPSIndicator : public GG::Label {
 public:
     FPSIndicator();
-    virtual void Render();
+
+    void Render() override;
+
+    void PreRender() override;
 private:
     void UpdateEnabled();
     bool m_enabled;
+    int m_displayed_FPS;
 };
 
 /** Functions like a StaticGraphic, except can have multiple textures rendered
@@ -545,23 +666,24 @@ public:
       * order they are specified in \a textures with GraphicStyles specified in the same-indexed value of \a styles.
       * if \a styles is not specified or contains fewer entres than \a textures, entries in \a textures without 
       * associated styles use the style GRAPHIC_CENTER. */
-    MultiTextureStaticGraphic(const std::vector<boost::shared_ptr<GG::Texture> >& textures,
-                              const std::vector<GG::Flags<GG::GraphicStyle> >& styles = std::vector<GG::Flags<GG::GraphicStyle> >());
+    MultiTextureStaticGraphic(const std::vector<std::shared_ptr<GG::Texture>>& textures,
+                              const std::vector<GG::Flags<GG::GraphicStyle>>& styles = std::vector<GG::Flags<GG::GraphicStyle>>());
 
     /** creates a MultiTextureStaticGraphic from multiple pre-existing SubTextures which are rendered back-to-front in the
       * order they are specified in \a subtextures with GraphicStyles specified in the same-indexed value of \a styles.
       * if \a styles is not specified or contains fewer entres than \a subtextures, entries in \a subtextures without 
       * associated styles use the style GRAPHIC_CENTER. */
     MultiTextureStaticGraphic(const std::vector<GG::SubTexture>& subtextures,
-                              const std::vector<GG::Flags<GG::GraphicStyle> >& styles = std::vector<GG::Flags<GG::GraphicStyle> >());
+                              const std::vector<GG::Flags<GG::GraphicStyle>>& styles = std::vector<GG::Flags<GG::GraphicStyle>>());
     //@}
 
     /** \name Mutators */ ///@{
-    virtual void    Render();       ///< renders textures in order specified in constructor, back-to-front
+    /** Renders textures in order specified in constructor, back-to-front. */
+    void Render() override;
     //@}
 
 protected:
-    MultiTextureStaticGraphic();    ///< default ctor
+    MultiTextureStaticGraphic();
 
     /** Returns the area in which the graphic is actually rendered, in
         UpperLeft()-relative coordinates.  This may not be the entire area of
@@ -573,7 +695,7 @@ private:
     void            ValidateStyles();      ///< ensures that the style flags are consistent
 
     std::vector<GG::SubTexture>                 m_graphics;
-    std::vector<GG::Flags<GG::GraphicStyle> >   m_styles;   ///< position of texture wrt the window area
+    std::vector<GG::Flags<GG::GraphicStyle>>    m_styles;   ///< position of texture wrt the window area
 };
 
 /** Functions like a StaticGraphic, except can be rotated with a fixed phase
@@ -581,14 +703,15 @@ private:
 class RotatingGraphic : public GG::StaticGraphic {
 public:
     /** \name Structors */ ///@{
-    RotatingGraphic(const boost::shared_ptr<GG::Texture>& texture, GG::Flags<GG::GraphicStyle> style = GG::GRAPHIC_NONE,
+    RotatingGraphic(const std::shared_ptr<GG::Texture>& texture, GG::Flags<GG::GraphicStyle> style = GG::GRAPHIC_NONE,
                     GG::Flags<GG::WndFlag> flags = GG::NO_WND_FLAGS);
     //@}
 
     /** \name Mutators */ ///@{
+    void Render() override;
+
     void            SetRPM(float rpm)               { m_rpm = std::max(-3600.0f, std::min(3600.0f, rpm)); }
     void            SetPhaseOffset(float degrees)   { m_phase_offset = degrees; }
-    virtual void    Render();
     //@}
 
 private:
@@ -596,40 +719,26 @@ private:
     float   m_phase_offset;
 };
 
-/** \brief Represents a CUIToggleButton
- * 
- * Renders a SubTexture depending on the checked state of the CUIToggleButton.
- * When the \a button has single texture and is unchecked, it renders the same icon recolored towards grey.
- * If a second SubTexture is assigned to the \a button, it will render the appropriate SubTexture.
- * When a mouse pointer hovers over this button, the opposing SubTexture and color are used and highlighted
- */
-class CUIToggleRepresenter : public GG::StateButtonRepresenter {
-public:
-    /** \name Structors */ //@{
-    CUIToggleRepresenter(boost::shared_ptr<GG::SubTexture> icon, const GG::Clr& highlight_clr); ///< ctor
-    CUIToggleRepresenter(boost::shared_ptr<GG::SubTexture> unchecked_icon, const GG::Clr& unchecked_clr,
-                         boost::shared_ptr<GG::SubTexture> checked_icon, const GG::Clr& checked_clr); ///< ctor
-    //@}
-
-    /** \name Mutators */ //@{
-    virtual void                        OnChecked(bool checked) const;
-    virtual void                        Render(const GG::StateButton& button) const;
-    //@}
-
-private:
-    boost::shared_ptr<GG::SubTexture>   m_unchecked_icon;
-    boost::shared_ptr<GG::SubTexture>   m_checked_icon;
-    GG::Clr                             m_unchecked_color;
-    GG::Clr                             m_checked_color;
-};
-
 /** Renders scanlines over its area. */
 class ScanlineControl : public GG::Control {
 public:
-    ScanlineControl(GG::X x, GG::Y y, GG::X w, GG::Y h, bool square = false);
-    virtual void Render();
+    ScanlineControl(GG::X x, GG::Y y, GG::X w, GG::Y h, bool square = false, GG::Clr clr = GG::CLR_BLACK);
+
+    /** Changes the color used to draw the scanlines. */
+    void Render() override;
+
+    void SetColor(GG::Clr clr) override
+    { m_color = clr; };
+
 private:
     bool m_square;
+    GG::Clr m_color;
+};
+
+/** Consistently rendered popup menu */
+class CUIPopupMenu : public GG::PopupMenu {
+public:
+    CUIPopupMenu(GG::X x, GG::Y y, const GG::MenuItem& menu);
 };
 
 #endif // _CUIControls_h_
