@@ -79,7 +79,7 @@ void BufferStoreCircleArcVertices(GG::GL2DVertexBuffer& buffer, const GG::Pt& ul
         SLICES = num_slices;
     const double   HORZ_THETA = (2 * PI) / SLICES;
 
-    static std::map<int, std::vector<double> > unit_circle_coords;
+    static std::map<int, std::vector<double>> unit_circle_coords;
     std::vector<double>& unit_vertices = unit_circle_coords[SLICES];
     bool calc_vertices = unit_vertices.size() == 0;
     if (calc_vertices) {
@@ -117,7 +117,7 @@ void BufferStoreCircleArcVertices(GG::GL2DVertexBuffer& buffer, const GG::Pt& ul
 
     } else {    // (not a fan) store a list of complete lines / triangles
         // if storing a filled_shape, the first point in each triangle should be the centre of the arc
-        std::pair<GLfloat, GLfloat> first_point = std::make_pair(static_cast<GLfloat>(center_x), static_cast<GLfloat>(center_y));
+        std::pair<GLfloat, GLfloat> first_point = {static_cast<GLfloat>(center_x), static_cast<GLfloat>(center_y)};
         // (not used for non-filled-shape)
 
         // angles in between theta1 and theta2, if any
@@ -182,6 +182,33 @@ GG::Clr OpaqueColor(const GG::Clr& color)
     return retval;
 }
 
+void BufferStoreRectangle(GG::GL2DVertexBuffer& buffer,
+                          const GG::Rect& area,
+                          const GG::Rect& border_thickness)
+{
+        GG::X inner_x1(area.ul.x + border_thickness.ul.x);
+        GG::Y inner_y1(area.ul.y + border_thickness.ul.y);
+        GG::X inner_x2(area.lr.x - border_thickness.lr.x);
+        GG::Y inner_y2(area.lr.y - border_thickness.lr.y);
+
+        buffer.reserve(14);
+        buffer.store(inner_x2, inner_y1);
+        buffer.store(area.lr.x, area.ul.y);
+        buffer.store(inner_x1, inner_y1);
+        buffer.store(area.ul.x, area.ul.y);
+        buffer.store(inner_x1, inner_y2);
+        buffer.store(area.ul.x, area.lr.y);
+        buffer.store(inner_x2, inner_y2);
+        buffer.store(area.lr.x, area.lr.y);
+        buffer.store(inner_x2, inner_y1);
+        buffer.store(area.lr.x, area.ul.y);
+
+        buffer.store(inner_x2, inner_y1);
+        buffer.store(inner_x1, inner_y1);
+        buffer.store(inner_x1, inner_y2);
+        buffer.store(inner_x2, inner_y2);
+}
+
 void AngledCornerRectangle(const GG::Pt& ul, const GG::Pt& lr, GG::Clr color, GG::Clr border, int angle_offset, int thick,
                            bool upper_left_angled/* = true*/, bool lower_right_angled/* = true*/, bool draw_bottom/* = true*/)
 {
@@ -246,7 +273,7 @@ bool InAngledCornerRect(const GG::Pt& pt, const GG::Pt& ul, const GG::Pt& lr, in
                         bool upper_left_angled/* = true*/, bool lower_right_angled)
 {
     bool retval = false;
-    if (retval = (ul <= pt && pt < lr)) {
+    if ((retval = ((ul <= pt) && (pt < lr)))) {
         GG::Pt dist_from_ul = pt - ul;
         GG::Pt dist_from_lr = lr - pt;
         bool inside_upper_left_corner = upper_left_angled ? (angle_offset < Value(dist_from_ul.x) + Value(dist_from_ul.y)) : true;
@@ -386,11 +413,12 @@ namespace {
     const double TWO_PI = 2.0 * 3.14159;
 }
 
-class ScanlineRenderer::ScanlineRendererImpl {
+class ScanlineRenderer::Impl {
 public:
-    ScanlineRendererImpl() :
-        m_scanline_shader(), m_failed_init(false)
-    {}
+    Impl() :
+        m_scanline_shader(),
+        m_failed_init(false)
+    { m_color = GG::CLR_BLACK; }
 
     void StartUsing() {
         if (m_failed_init)
@@ -399,9 +427,12 @@ public:
         if (!m_scanline_shader) {
             boost::filesystem::path shader_path = GetRootDataDir() / "default" / "shaders" / "scanlines.frag";
             std::string shader_text;
-            ReadFile(shader_path, shader_text);
-            m_scanline_shader = boost::shared_ptr<ShaderProgram>(
-                ShaderProgram::shaderProgramFactory("", shader_text));
+            if (!ReadFile(shader_path, shader_text)) {
+                ErrorLogger() << "ScanlineRenderer failed to read shader at path " << shader_path.string();
+                m_failed_init = true;
+                return;
+            }
+            m_scanline_shader = ShaderProgram::shaderProgramFactory("", shader_text);
 
             if (!m_scanline_shader) {
                 ErrorLogger() << "ScanlineRenderer failed to initialize shader.";
@@ -413,6 +444,11 @@ public:
         float fog_scanline_spacing = static_cast<float>(GetOptionsDB().Get<double>("UI.system-fog-of-war-spacing"));
         m_scanline_shader->Use();
         m_scanline_shader->Bind("scanline_spacing", fog_scanline_spacing);
+        m_scanline_shader->Bind("line_color", m_color.r * (1.f / 255.f), m_color.g * (1.f / 255.f), m_color.b * (1.f / 255.f), m_color.a * (1.f / 255.f));
+    }
+
+    void SetColor(GG::Clr clr) {
+        m_color = clr;
     }
 
     void StopUsing()
@@ -430,27 +466,30 @@ public:
         StopUsing();
     }
 
-    boost::shared_ptr<ShaderProgram> m_scanline_shader;
+    std::unique_ptr<ShaderProgram> m_scanline_shader;
     bool m_failed_init;
+    GG::Clr m_color;
 };
 
 
 ScanlineRenderer::ScanlineRenderer() :
-    pimpl(new ScanlineRendererImpl())
+    m_impl(new Impl())
 {}
 
 // This destructor is required here because ~ScanlineRendererImpl is declared here.
-ScanlineRenderer::~ScanlineRenderer()
-{}
+ScanlineRenderer::~ScanlineRenderer() = default;
 
 void ScanlineRenderer::RenderCircle(const GG::Pt& ul, const GG::Pt& lr)
-{ pimpl->RenderCircle(ul, lr); }
+{ m_impl->RenderCircle(ul, lr); }
 
 void ScanlineRenderer::RenderRectangle(const GG::Pt& ul, const GG::Pt& lr)
-{ pimpl->RenderRectangle(ul, lr); }
+{ m_impl->RenderRectangle(ul, lr); }
 
 void ScanlineRenderer::StartUsing()
-{ pimpl->StartUsing(); }
+{ m_impl->StartUsing(); }
+
+void ScanlineRenderer::SetColor(GG::Clr clr)
+{ m_impl->SetColor(clr); }
 
 void ScanlineRenderer::StopUsing()
-{ pimpl->StopUsing(); }
+{ m_impl->StopUsing(); }
