@@ -4,7 +4,9 @@
 #include <GG/Clr.h>
 #include <GG/PtRect.h>
 #include <GG/GLClientAndServerBuffer.h>
-#include <boost/scoped_ptr.hpp>
+
+#include <memory>
+
 
 /** adjusts the intensity of the color up or down by \a amount units per color
   * channel; leaves alpha unchanged if \a jointly_capped is true then, if the
@@ -23,6 +25,16 @@ void AdjustBrightness(GG::Clr& color, double amount, bool jointly_capped=false);
 /** returns fully opaque (max alpha channel) version of the color */
 GG::Clr OpaqueColor(const GG::Clr& color);
 
+/** Stores verticies in CCW order that outline a rectangle
+ * @param[out] buffer Buffer to store verticies
+ * @param[in] area Outside corners of rectangle
+ * @param[in] border_thickness Thickness of border for each corner, drawn
+ *              towards the interior of @p area.
+ */
+void BufferStoreRectangle(GG::GL2DVertexBuffer& buffer,
+                          const GG::Rect& area,
+                          const GG::Rect& border_thickness);
+
 /** Using immediate GL calls, renders a rectangle whose upper left and lower
   * right corners are angled.  If \a upper_left_angled == false, the upper left
   * corner is drawn as a normal corner */
@@ -108,6 +120,9 @@ public:
     /** Draw scanlines in the square area bounded by \p ul and \p lr.*/
     void RenderRectangle(const GG::Pt& ul, const GG::Pt& lr);
 
+    /** Changes the color used to draw the scanlines. Set color before calling StartUsing()*/
+    void SetColor(GG::Clr clr);
+
     /** Start using ScanlineRenderer to draw arbitrary shapes with the scanline
         shader program.*/
     void StartUsing();
@@ -117,9 +132,9 @@ public:
     void StopUsing();
 
 private:
-    class ScanlineRendererImpl;
-    // TODO use C++11 unique_ptr
-    boost::scoped_ptr<ScanlineRendererImpl> const pimpl;
+    struct Impl;
+
+    std::unique_ptr<Impl> const m_impl;
 };
 
 #endif // _CUIDrawUtil_h_
