@@ -42,7 +42,7 @@ namespace {
 // CUI_MinRestoreButton
 ////////////////////////////////////////////////
 CUI_MinRestoreButton::CUI_MinRestoreButton() :
-    GG::Button("", boost::shared_ptr<GG::Font>(), ClientUI::WndInnerBorderColor()),
+    GG::Button("", nullptr, ClientUI::WndInnerBorderColor()),
     m_mode(MIN_BUTTON)
 { GG::Connect(LeftClickedSignal, &CUI_MinRestoreButton::Toggle, this); }
 
@@ -78,7 +78,7 @@ void CUI_MinRestoreButton::Toggle() {
 // CUI_PinButton
 ////////////////////////////////////////////////
 CUI_PinButton::CUI_PinButton() :
-    GG::Button("", boost::shared_ptr<GG::Font>(), ClientUI::WndInnerBorderColor())
+    GG::Button("", nullptr, ClientUI::WndInnerBorderColor())
 {
     GG::Connect(LeftClickedSignal, &PlayCloseSound, -1);
     SetUnpressedGraphic(GG::SubTexture(ClientUI::GetTexture( ClientUI::ArtDir() / "icons" / "buttons" / "pin.png"   )));
@@ -143,9 +143,9 @@ CUIWnd::CUIWnd(const std::string& wnd_name,
     m_mouse_in_resize_tab(false),
     m_config_save(true),
     m_config_name(AddWindowOptions(config_name, x, y, w, h, visible, false, false)),
-    m_close_button(0),
-    m_minimize_button(0),
-    m_pin_button(0),
+    m_close_button(nullptr),
+    m_minimize_button(nullptr),
+    m_pin_button(nullptr),
     m_vertex_buffer(),
     m_buffer_indices()
 {
@@ -169,9 +169,9 @@ CUIWnd::CUIWnd(const std::string& wnd_name, GG::Flags<GG::WndFlag> flags, const
     m_mouse_in_resize_tab(false),
     m_config_save(true),
     m_config_name(AddWindowOptions(config_name, INVALID_POS, INVALID_POS, 1, 1, visible, false, false)),
-    m_close_button(0),
-    m_minimize_button(0),
-    m_pin_button(0),
+    m_close_button(nullptr),
+    m_minimize_button(nullptr),
+    m_pin_button(nullptr),
     m_vertex_buffer(),
     m_buffer_indices()
 { Init(wnd_name); }
@@ -326,7 +326,7 @@ void CUIWnd::Render() {
     GG::Pt lr = LowerRight();
     GG::BeginScissorClipping(ul, lr);
     glColor(ClientUI::TextColor());
-    boost::shared_ptr<GG::Font> font = ClientUI::GetTitleFont();
+    std::shared_ptr<GG::Font> font = ClientUI::GetTitleFont();
     font->RenderText(GG::Pt(ul.x + BORDER_LEFT, ul.y + TITLE_OFFSET), Name());
     GG::EndScissorClipping();
 }
@@ -825,13 +825,13 @@ void CUIWnd::InvalidateUnusedOptions() {
     // instantiated so they fall back on defaults when they are re-constructed.
     std::set<std::string> window_options;
     db.FindOptions(window_options, prefix);
-    for (std::set<std::string>::iterator it = window_options.begin(); it != window_options.end(); ++it) {
+    for (const std::string& option : window_options) {
         // If the ".left" option is registered, the rest are implied to be
         // there.
-        if (it->rfind(suffix_exist) == it->length() - suffix_exist.length() &&
-            db.OptionExists(*it))
+        if (option.rfind(suffix_exist) == option.length() - suffix_exist.length() &&
+            db.OptionExists(option))
         {
-            std::string name = it->substr(prefix.length(), it->length() - prefix.length() - suffix_exist.length());
+            std::string name = option.substr(prefix.length(), option.length() - prefix.length() - suffix_exist.length());
             // If the ".initialized" option isn't present under this name,
             // remove the options.
             if (window_options.find(prefix + name + suffix_used) == window_options.end()) {
@@ -888,7 +888,7 @@ CUIEditWnd::CUIEditWnd(GG::X w, const std::string& prompt_text, const std::strin
 void CUIEditWnd::ModalInit()
 { GG::GUI::GetGUI()->SetFocusWnd(m_edit); }
 
-void CUIEditWnd::KeyPress(GG::Key key, boost::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys) {
+void CUIEditWnd::KeyPress(GG::Key key, std::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys) {
     switch (key) {
     case GG::GGK_RETURN: if (!m_ok_bn->Disabled()) OkClicked(); break;
     case GG::GGK_ESCAPE: CloseClicked(); break;
