@@ -32,9 +32,9 @@ class CensusRowPanel : public GG::Control {
 public:
     CensusRowPanel(GG::X w, GG::Y h, const std::string& name, double census_val, bool show_icon) :
         GG::Control(GG::X0, GG::Y0, w, h, GG::NO_WND_FLAGS),
-        m_icon(0),
-        m_name(0),
-        m_census_val(0)
+        m_icon(nullptr),
+        m_name(nullptr),
+        m_census_val(nullptr)
     {
         m_show_icon = show_icon;
         SetChildClippingMode(ClipToClient);
@@ -57,7 +57,7 @@ public:
         DoLayout();
     }
 
-    virtual void    Render() {
+    void Render() override {
         GG::Pt ul = UpperLeft();
 
         glPushMatrix();
@@ -120,11 +120,11 @@ private:
 CensusBrowseWnd::CensusBrowseWnd(const std::string& title_text, const std::map<std::string, float>& population_counts,
                                  const std::map<std::string, float>& tag_counts) :
     GG::BrowseInfoWnd(GG::X0, GG::Y0, BrowseTextWidth(), GG::Y1),
-    m_title_text(0),
-    m_species_text(0),
-    m_list(0),
-    m_tags_text(0),
-    m_tags_list(0),
+    m_title_text(nullptr),
+    m_species_text(nullptr),
+    m_list(nullptr),
+    m_tags_text(nullptr),
+    m_tags_list(nullptr),
     m_offset(GG::X0, ICON_BROWSE_ICON_HEIGHT/2)
 {
     const GG::Y ROW_HEIGHT(MeterIconSize().y);
@@ -155,9 +155,8 @@ CensusBrowseWnd::CensusBrowseWnd(const std::string& title_text, const std::map<s
 
     // put into multimap to sort by population, ascending
     std::multimap<float, std::string> counts_species;
-    for (std::map<std::string, float>::const_iterator it = population_counts.begin();
-         it != population_counts.end(); ++it)
-    { counts_species.insert(std::make_pair(it->second, it->first)); }
+    for (const std::map<std::string, float>::value_type& entry : population_counts)
+    { counts_species.insert({entry.second, entry.first}); }
 
     // add species rows
     for (std::multimap<float, std::string>::const_reverse_iterator it = counts_species.rbegin();
@@ -200,13 +199,12 @@ CensusBrowseWnd::CensusBrowseWnd(const std::string& title_text, const std::map<s
     std::vector<std::string> tag_order;
     std::copy(std::istream_iterator<std::string>(tag_stream),
         std::istream_iterator<std::string>(),
-        std::back_inserter<std::vector<std::string> >(tag_order));
+        std::back_inserter<std::vector<std::string>>(tag_order));
 
     // add tags/characteristics rows
-    //for (std::map<std::string, float>::const_iterator it = tag_counts.begin(); it != tag_counts.end(); ++it) {
-    for (std::vector<std::string>::iterator it = tag_order.begin(); it != tag_order.end(); ++it) {
-        //DebugLogger() << "Census checking for tag '"<< *it <<"'";
-        std::map<std::string, float>::const_iterator it2 = tag_counts.find(*it);
+    for (const std::string& tag_ord : tag_order) {
+        //DebugLogger() << "Census checking for tag '"<< tag_ord <<"'";
+        std::map<std::string, float>::const_iterator it2 = tag_counts.find(tag_ord);
         if (it2 != tag_counts.end()) {
             GG::ListBox::Row* row = new GG::ListBox::Row(m_list->Width(), ROW_HEIGHT, "Census Characteristics Row");
             row->push_back(new CensusRowPanel(m_tags_list->Width(), ROW_HEIGHT, it2->first, it2->second, false));
