@@ -1,6 +1,7 @@
 #include "CombatLogWnd.h"
 
-#include <GG/Layout.h>
+#include <GG/GUI.h>
+#include <GG/DeferredLayout.h>
 #include <GG/Scroll.h>
 #include <GG/ScrollPanel.h>
 
@@ -8,25 +9,26 @@
 
 #include "../../client/human/HumanClientApp.h"
 #include "../../combat/CombatLogManager.h"
-#include "../../combat/CombatEvents.h"
 #include "../../universe/System.h"
+#include "../../universe/UniverseObject.h"
+#include "../../universe/Enums.h"
 #include "../../util/AppInterface.h"
 #include "../../util/i18n.h"
 #include "../../util/Logger.h"
-#include "../../universe/UniverseObject.h"
+#include "../../util/VarText.h"
 #include "../AccordionPanel.h"
 #include "../../Empire/Empire.h"
 
-class CombatLogWnd::CombatLogWndImpl {
+class CombatLogWnd::Impl {
 public:
-    CombatLogWndImpl(CombatLogWnd& _log);//GG::X w, GG::Y h);
+    Impl(CombatLogWnd& _log);
 
     /** \name Accessors */ ///@{
     GG::Pt MinUsableSize() const;
     //@}
 
     /** \name Mutators */ //@{
-    void SetFont(boost::shared_ptr<GG::Font> font);
+    void SetFont(std::shared_ptr<GG::Font> font);
     /// Set which log to show
     void SetLog(int log_id);
     /** Add a row at the end of the combat report*/
@@ -46,30 +48,30 @@ public:
         GG::X w, int viewing_empire_id, std::vector<GG::Wnd *>& new_logs,
         ConstCombatEventPtr& event, std::string& details);
 
-    //Returns either a simple LinkText for a simple log or a CombatLogAccordionPanel for a complex log
+    // Returns either a simple LinkText for a simple log or a CombatLogAccordionPanel for a complex log
     std::vector<GG::Wnd*> MakeCombatLogPanel(
         GG::X w, int viewing_empire_id, ConstCombatEventPtr event);
 
     // public interface
     CombatLogWnd& m_wnd;
 
-    ///default flags for a text link log segment
+    // default flags for a text link log segment
     GG::Flags<GG::TextFormat> m_text_format_flags;
-    boost::shared_ptr<GG::Font> m_font;
+    std::shared_ptr<GG::Font> m_font;
 
 };
 
 namespace {
-    typedef boost::shared_ptr<LinkText> LinkTextPtr;
+    typedef std::shared_ptr<LinkText> LinkTextPtr;
 
     const std::string EMPTY_STRING;
 
     std::map<int, int> CountByOwner(const std::set<int>& owners, const std::set<int>& objects) {
         std::map<int, int> objects_per_owner;
-        for (std::set<int>::const_iterator it = owners.begin(); it != owners.end(); ++it)
-            objects_per_owner[*it] = 0;
-        for (std::set<int>::const_iterator it = objects.begin(); it != objects.end(); ++it) {
-            TemporaryPtr<const UniverseObject> object = Objects().Object(*it);
+        for (int owner_id : owners)
+            objects_per_owner[owner_id] = 0;
+        for (int obj_id : objects) {
+            std::shared_ptr<const UniverseObject> object = Objects().Object(obj_id);
             if (object && (
                     object->ObjectType() == OBJ_SHIP || (
                         object->GetMeter(METER_POPULATION) &&
@@ -87,11 +89,13 @@ namespace {
     std::string CountsToText(const std::map<int, int>& count_per_empire, const std::string& delimiter = ", ") {
         std::stringstream ss;
         for (std::map<int,int>::const_iterator it = count_per_empire.begin(); it != count_per_empire.end(); ) {
-            std::string owner_string = UserString("NEUTRAL");
+            std::string owner_string;
             if (const Empire* owner = GetEmpire(it->first))
                 owner_string = GG::RgbaTag(owner->Color()) + "<" + VarText::EMPIRE_ID_TAG + " "
-                    + boost::lexical_cast<std::string>(owner->EmpireID()) + ">" + owner->Name()
+                    + std::to_string(owner->EmpireID()) + ">" + owner->Name()
                     + "</" + VarText::EMPIRE_ID_TAG + ">" + "</rgba>";
+            else
+                owner_string = GG::RgbaTag(ClientUI::DefaultLinkColor()) + UserString("NEUTRAL")  + "</rgba>";
             ss << owner_string << ": " << it->second;
             ++it;
             if (it != count_per_empire.end())
@@ -104,7 +108,7 @@ namespace {
     /// and expanding into specifics.
     class CombatLogAccordionPanel : public AccordionPanel {
         public:
-        CombatLogAccordionPanel(GG::X w, CombatLogWnd::CombatLogWndImpl &log_,
+        CombatLogAccordionPanel(GG::X w, CombatLogWnd::Impl &log_,
                                 int viewing_empire_id_, ConstCombatEventPtr event_);
         ~CombatLogAccordionPanel();
 
@@ -112,18 +116,18 @@ namespace {
         /** toggles panel expanded or collapsed */
         void ToggleExpansion();
 
-        CombatLogWnd::CombatLogWndImpl & log;
+        CombatLogWnd::Impl & log;
         int viewing_empire_id;
         ConstCombatEventPtr event;
         LinkText* title;
         std::vector<GG::Wnd *> details;
 
-        //distance between expansion symbol and text
+        // distance between expansion symbol and text
         static const unsigned int BORDER_MARGIN = 5;
 
     };
 
-    CombatLogAccordionPanel::CombatLogAccordionPanel(GG::X w, CombatLogWnd::CombatLogWndImpl& log_,
+    CombatLogAccordionPanel::CombatLogAccordionPanel(GG::X w, CombatLogWnd::Impl& log_,
                                                      int viewing_empire_id_, ConstCombatEventPtr event_) :
         AccordionPanel(w, GG::Y(ClientUI::Pts()), true),
         log(log_),
@@ -135,29 +139,29 @@ namespace {
         AccordionPanel::SetInteriorColor(ClientUI::CtrlColor());
 
         GG::Connect(m_expand_button->LeftClickedSignal, &CombatLogAccordionPanel::ToggleExpansion, this);
-        GG::Connect(this->ExpandCollapseSignal, &CombatLogWnd::CombatLogWndImpl::HandleWndChanged, &log);
+        GG::Connect(this->ExpandCollapseSignal, &CombatLogWnd::Impl::HandleWndChanged, &log);
 
         SetBorderMargin(BORDER_MARGIN);
 
-        SetLayout(new GG::Layout(UpperLeft().x, UpperLeft().y, Width(), Height(), 1, 1));
+        SetLayout(new GG::DeferredLayout(UpperLeft().x, UpperLeft().y, Width(), Height(), 1, 1));
         GetLayout()->Add(title, 0, 0, 1, 1);
         SetCollapsed(true);
+        RequirePreRender();
     }
 
     CombatLogAccordionPanel::~CombatLogAccordionPanel() {
         if (!IsCollapsed() && !details.empty()) {
-            for (std::vector<GG::Wnd *>::iterator it = details.begin(); it != details.end(); ++it) {
-                delete *it;
+            for (GG::Wnd* wnd : details) {
+                delete wnd;
             }
         }
     }
 
     void CombatLogAccordionPanel::ToggleExpansion() {
-        DebugLogger() << "Expand/Collapse of detailed combat log.";
         bool new_collapsed = !IsCollapsed();
         if (new_collapsed) {
-            for (std::vector<GG::Wnd*>::iterator it = details.begin(); it != details.end(); ++it) {
-                GetLayout()->Remove(*it);
+            for (GG::Wnd* wnd : details) {
+                GetLayout()->Remove(wnd);
             }
         } else {
             if (details.empty()) {
@@ -165,8 +169,8 @@ namespace {
                 log.PopulateWithFlatLogs(Width(), viewing_empire_id, details, event, detail_text);
             }
 
-            for (std::vector<GG::Wnd*>::iterator it = details.begin(); it != details.end(); ++it) {
-                GetLayout()->Add(*it, GetLayout()->Rows(), 0);
+            for (GG::Wnd* wnd : details) {
+                GetLayout()->Add(wnd, GetLayout()->Rows(), 0);
             }
         }
         SetCollapsed(new_collapsed);
@@ -174,22 +178,18 @@ namespace {
 
 }
 
-CombatLogWnd::CombatLogWndImpl::CombatLogWndImpl(CombatLogWnd& _wnd) :
+CombatLogWnd::Impl::Impl(CombatLogWnd& _wnd) :
     m_wnd(_wnd),
     m_text_format_flags(GG::FORMAT_WORDBREAK| GG::FORMAT_LEFT | GG::FORMAT_TOP),
     m_font(ClientUI::GetFont())
 { }
 
-GG::Pt CombatLogWnd::CombatLogWndImpl::MinUsableSize() const {
+GG::Pt CombatLogWnd::Impl::MinUsableSize() const {
     return GG::Pt(m_font->SpaceWidth()*20, m_font->Lineskip()*10);
 }
 
-void CombatLogWnd::CombatLogWndImpl::HandleWndChanged() {
-    GG::Pt size = m_wnd.Size();
-    m_wnd.Resize(size + GG::Pt(2*m_font->SpaceWidth(), GG::Y0));
-    m_wnd.Resize(size);
-    m_wnd.WndChangedSignal();
-}
+void CombatLogWnd::Impl::HandleWndChanged()
+{ m_wnd.RequirePreRender(); }
 
 
 namespace {
@@ -197,7 +197,7 @@ namespace {
     template <typename T>
     T const* FindParentOfType(GG::Wnd const* parent) {
         GG::Wnd const * iwnd = parent;
-        T const* type_T = 0;
+        T const* type_T = nullptr;
         while (iwnd && !(type_T = dynamic_cast<const T*>(iwnd))){
             iwnd = iwnd->Parent();
         }
@@ -228,13 +228,13 @@ namespace {
 
         LazyScrollerLinkText(
             GG::Wnd & parent, GG::X x, GG::Y y, const std::string& str,
-            const boost::shared_ptr<GG::Font>& font, GG::Clr color = GG::CLR_BLACK) :
+            const std::shared_ptr<GG::Font>& font, GG::Clr color = GG::CLR_BLACK) :
             LinkText(x, y, UserString("ELLIPSIS"), font, color),
             m_text( new std::string(str)),
             m_signals()
         {
 
-            //Register for signals that might bring the text into view
+            // Register for signals that might bring the text into view
 
             if (CombatLogWnd const* log = FindParentOfType<CombatLogWnd>(&parent)) {
                 m_signals.push_back(
@@ -248,8 +248,8 @@ namespace {
 
             }
 
-            //Parent doesn't contain any of the expected parents so just
-            //show the text.
+            // Parent doesn't contain any of the expected parents so just
+            // show the text.
             if (m_signals.empty()) {
                 SetText(str);
                 m_text.reset();
@@ -260,7 +260,7 @@ namespace {
 
         void HandleMaybeVisible() {
 
-            //Assumes the log is the second tab.
+            // Assumes the log is the second tab.
             GG::OverlayWnd const* tab = FindParentOfType<const GG::OverlayWnd>(Parent());
             if (tab && (tab->CurrentWndIndex() != 1))
                 return;
@@ -271,9 +271,8 @@ namespace {
                                  || scroll_panel->InClient(LowerRight())
                                  || scroll_panel->InClient(GG::Pt(Right(), Top()))
                                  || scroll_panel->InClient(GG::Pt(Left(), Bottom())))) {
-                for (std::vector<boost::signals2::connection>::iterator sig_it = m_signals.begin();
-                     sig_it != m_signals.end(); ++sig_it){
-                    sig_it->disconnect();
+                for (boost::signals2::connection& signal : m_signals) {
+                    signal.disconnect();
                 }
                 m_signals.clear();
 
@@ -288,19 +287,19 @@ namespace {
             HandleMaybeVisible();
         }
 
-        virtual void SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
+        void SizeMove(const GG::Pt& ul, const GG::Pt& lr)  override {
             LinkText::SizeMove(ul, lr);
             if (! m_signals.empty())
                 HandleMaybeVisible();
         }
 
-        boost::scoped_ptr<std::string> m_text;
+        std::unique_ptr<std::string> m_text;
         std::vector<boost::signals2::connection> m_signals;
     };
 
 }
 
-LinkText * CombatLogWnd::CombatLogWndImpl::DecorateLinkText(std::string const & text) {
+LinkText * CombatLogWnd::Impl::DecorateLinkText(std::string const & text) {
     LazyScrollerLinkText * links = new LazyScrollerLinkText(m_wnd, GG::X0, GG::Y0, text, m_font, GG::CLR_WHITE);
 
     links->SetTextFormat(m_text_format_flags);
@@ -313,41 +312,39 @@ LinkText * CombatLogWnd::CombatLogWndImpl::DecorateLinkText(std::string const &
     links->LinkClickedSignal.connect(m_wnd.LinkClickedSignal);
     links->LinkDoubleClickedSignal.connect(m_wnd.LinkDoubleClickedSignal);
     links->LinkRightClickedSignal.connect(m_wnd.LinkRightClickedSignal);
-    GG::Connect(links->ChangedSignal,           &CombatLogWnd::CombatLogWndImpl::HandleWndChanged,         this);
+    GG::Connect(links->ChangedSignal, &CombatLogWnd::Impl::HandleWndChanged, this);
 
     return links;
 }
 
 /** Fill \p new_logs with pointers to the flat log contents of \p
     event using the pre-calculated \p details.*/
-void CombatLogWnd::CombatLogWndImpl::PopulateWithFlatLogs(GG::X w, int viewing_empire_id, std::vector<GG::Wnd*> &new_logs,
-                                                          ConstCombatEventPtr& event, std::string& details)
+void CombatLogWnd::Impl::PopulateWithFlatLogs(GG::X w, int viewing_empire_id, std::vector<GG::Wnd*> &new_logs,
+                                              ConstCombatEventPtr& event, std::string& details)
 {
     if (!details.empty()) {
         new_logs.push_back(DecorateLinkText(details));
     }
 
     if (!event->AreSubEventsEmpty(viewing_empire_id)) {
-        std::vector<ConstCombatEventPtr> sub_events = event->SubEvents(viewing_empire_id);
-        for (std::vector<ConstCombatEventPtr>::iterator sub_event_it = sub_events.begin();
-             sub_event_it != sub_events.end(); ++sub_event_it) {
+        for (ConstCombatEventPtr sub_event : event->SubEvents(viewing_empire_id)) {
             std::vector<GG::Wnd*> flat_logs =
-                MakeCombatLogPanel(w, viewing_empire_id, *sub_event_it);
+                MakeCombatLogPanel(w, viewing_empire_id, sub_event);
             new_logs.insert(new_logs.end(), flat_logs.begin(), flat_logs.end());
         }
     }
 }
 
 
-//Returns either a simple LinkText for a simple log or a CombatLogAccordionPanel for a complex log
-std::vector<GG::Wnd*> CombatLogWnd::CombatLogWndImpl::MakeCombatLogPanel(GG::X w, int viewing_empire_id,
-                                                                         ConstCombatEventPtr event)
+// Returns either a simple LinkText for a simple log or a CombatLogAccordionPanel for a complex log
+std::vector<GG::Wnd*> CombatLogWnd::Impl::MakeCombatLogPanel(GG::X w, int viewing_empire_id,
+                                                             ConstCombatEventPtr event)
 {
     std::vector<GG::Wnd *> new_logs;
 
-    //Create and accordion log if there are detail or sub events and
-    //the log isn't explicitly flatten.  Otherwise, flatten the log,
-    //details and sub events.
+    // Create an accordion log if there are detail or sub events and
+    // the log isn't explicitly flattened.  Otherwise, flatten the log,
+    // details and sub events.
 
     if (!event->FlattenSubEvents() && !event->AreSubEventsEmpty(viewing_empire_id) ) {
         new_logs.push_back(new CombatLogAccordionPanel(w, *this, viewing_empire_id, event));
@@ -370,63 +367,64 @@ std::vector<GG::Wnd*> CombatLogWnd::CombatLogWndImpl::MakeCombatLogPanel(GG::X w
 }
 
 
-void CombatLogWnd::CombatLogWndImpl::AddRow(GG::Wnd* wnd) {
+void CombatLogWnd::Impl::AddRow(GG::Wnd* wnd) {
     if (GG::Layout* layout = m_wnd.GetLayout())
         layout->Add(wnd, layout->Rows(), 0);
 }
 
-void CombatLogWnd::CombatLogWndImpl::SetFont(boost::shared_ptr<GG::Font> font)
+void CombatLogWnd::Impl::SetFont(std::shared_ptr<GG::Font> font)
 { m_font = font; }
 
-void CombatLogWnd::CombatLogWndImpl::SetLog(int log_id) {
-    if (!CombatLogAvailable(log_id)) {
+void CombatLogWnd::Impl::SetLog(int log_id) {
+    boost::optional<const CombatLog&> log = GetCombatLog(log_id);
+    if (!log) {
         ErrorLogger() << "Couldn't find combat log with id: " << log_id;
         return;
     }
 
+    bool verbose_logging = GetOptionsDB().Get<bool>("verbose-logging") ||
+                           GetOptionsDB().Get<bool>("verbose-combat-logging");
+
     m_wnd.DeleteChildren();
-    GG::Layout* layout = new GG::Layout(m_wnd.UpperLeft().x, m_wnd.UpperLeft().y
-                                        , m_wnd.Width(), m_wnd.Height()
-                                        , 1, 1 ///< numrows, numcols
-                                        , 0, 0 ///< wnd margin, cell margin
-                                       );
+    GG::Layout* layout = new GG::DeferredLayout(m_wnd.RelativeUpperLeft().x, m_wnd.RelativeUpperLeft().y,
+                                                m_wnd.Width(), m_wnd.Height(),
+                                                1, 1, ///< numrows, numcols
+                                                0, 0 ///< wnd margin, cell margin
+                                               );
     m_wnd.SetLayout(layout);
 
-    const CombatLog& log = GetCombatLog(log_id);
     int client_empire_id = HumanClientApp::GetApp()->EmpireID();
 
     // Write Header text
-    DebugLogger() << "Setting log with " << log.combat_events.size() << " events";
+    if (verbose_logging)
+        DebugLogger() << "Showing combat log #" << log_id << " with " << log->combat_events.size() << " events";
 
-    TemporaryPtr<const System> system = GetSystem(log.system_id);
+    std::shared_ptr<const System> system = GetSystem(log->system_id);
     const std::string& sys_name = (system ? system->PublicName(client_empire_id) : UserString("ERROR"));
 
     AddRow(DecorateLinkText(str(FlexibleFormat(UserString("ENC_COMBAT_LOG_DESCRIPTION_STR"))
-                                % LinkTaggedIDText(VarText::SYSTEM_ID_TAG, log.system_id, sys_name)
-                                % log.turn) + "\n"
+                                % LinkTaggedIDText(VarText::SYSTEM_ID_TAG, log->system_id, sys_name)
+                                % log->turn) + "\n"
                            ));
     AddRow(DecorateLinkText(UserString("COMBAT_INITIAL_FORCES")));
-    AddRow(DecorateLinkText(CountsToText(CountByOwner(log.empire_ids, log.object_ids))));
+    AddRow(DecorateLinkText(CountsToText(CountByOwner(log->empire_ids, log->object_ids))));
 
     std::stringstream summary_text;
     summary_text << std::endl << UserString("COMBAT_SUMMARY_DESTROYED")
-                 << std::endl << CountsToText(CountByOwner(log.empire_ids, log.destroyed_object_ids));
+                 << std::endl << CountsToText(CountByOwner(log->empire_ids, log->destroyed_object_ids));
     AddRow(DecorateLinkText(summary_text.str()));
 
     // Write Logs
-    for (std::vector<CombatEventPtr>::const_iterator it = log.combat_events.begin();
-         it != log.combat_events.end(); ++it) {
-        DebugLogger() << "event debug info: " << it->get()->DebugString();
+    for (CombatEventPtr event : log->combat_events) {
+        if (verbose_logging)
+            DebugLogger() << "event debug info: " << event->DebugString();
 
-        std::vector<GG::Wnd *> flat_logs =
-            MakeCombatLogPanel(m_font->SpaceWidth()*10, client_empire_id, *it);
-        for (std::vector<GG::Wnd *>::iterator log_it = flat_logs.begin();
-             log_it != flat_logs.end(); ++log_it) {
-            AddRow(*log_it);
+        for (GG::Wnd* wnd : MakeCombatLogPanel(m_font->SpaceWidth()*10, client_empire_id, event)) {
+            AddRow(wnd);
         }
     }
 
-    //Add a dummy row that the layout manager can use to add space.
+    // Add a dummy row that the layout manager can use to add space.
     AddRow(DecorateLinkText(""));
     layout->SetRowStretch(layout->Rows() - 1, 1);
 
@@ -437,20 +435,20 @@ void CombatLogWnd::CombatLogWndImpl::SetLog(int log_id) {
 // Forward request to private implementation
 CombatLogWnd::CombatLogWnd(GG::X w, GG::Y h) :
     GG::Wnd(GG::X0, GG::Y0, w, h, GG::NO_WND_FLAGS),
-    pimpl(new CombatLogWndImpl(*this))
+    m_impl(new Impl(*this))
 {
     SetName("CombatLogWnd");
 }
 
-//This virtual destructor must exist to ensure that the pimpl is destroyed.
+// This virtual destructor must exist to ensure that the m_impl is destroyed.
 CombatLogWnd::~CombatLogWnd()
 {}
 
-void CombatLogWnd::SetFont(boost::shared_ptr<GG::Font> font)
-{ pimpl->SetFont(font); }
+void CombatLogWnd::SetFont(std::shared_ptr<GG::Font> font)
+{ m_impl->SetFont(font); }
 
 void CombatLogWnd::SetLog(int log_id)
-{ pimpl->SetLog(log_id); }
+{ m_impl->SetLog(log_id); }
 
 GG::Pt CombatLogWnd::ClientUpperLeft() const
 { return UpperLeft() + GG::Pt(GG::X(MARGIN), GG::Y(MARGIN)); }
@@ -459,7 +457,28 @@ GG::Pt CombatLogWnd::ClientLowerRight() const
 { return LowerRight() - GG::Pt(GG::X(MARGIN), GG::Y(MARGIN)); }
 
 GG::Pt CombatLogWnd::MinUsableSize() const
-{ return pimpl->MinUsableSize(); }
+{ return m_impl->MinUsableSize(); }
 
 void CombatLogWnd::HandleMadeVisible()
-{ return pimpl->HandleWndChanged(); }
+{ return m_impl->HandleWndChanged(); }
+
+void CombatLogWnd::PreRender() {
+    GG::Wnd::PreRender();
+
+    /* Workaround
+
+     Problem: CombatLogWnd is incorrectly initialized with a width of 30 which causes the combat
+     accordion windows to incorrectly size themselves as if the title text were 30 wide.
+
+    This fix forces the combat accordion window to correctly resize itself.
+
+    TODO: Fix intial size of CombatReport from (30,15) to its actual first displayed size.*/
+    GG::Pt size = Size();
+    Resize(size + GG::Pt(2 * m_impl->m_font->SpaceWidth(), GG::Y0));
+    GG::GUI::PreRenderWindow(this);
+    Resize(size);
+    GG::GUI::PreRenderWindow(this);
+    /* End workaround. */
+
+    WndChangedSignal();
+}
