@@ -1,5 +1,4 @@
 #include "CombatReportData.h"
-#include "../../combat/CombatEvents.h"
 #include "../../combat/CombatLogManager.h"
 #include "../../universe/UniverseObject.h"
 #include "../../util/AppInterface.h"
@@ -74,8 +73,8 @@ std::string CombatSummary::SideName() const {
 
 unsigned int CombatSummary::DestroyedUnits() const {
     unsigned count = 0;
-    for (UnitSummaries::const_iterator it = unit_summaries.begin(); it != unit_summaries.end(); ++it) {
-        if ((*it)->current_health <= 0.0f && (*it)->max_health > 0.0f) {
+    for (ParticipantSummaryPtr summary : unit_summaries) {
+        if (summary->current_health <= 0.0f && summary->max_health > 0.0f) {
             ++count;
         }
     }
@@ -83,7 +82,7 @@ unsigned int CombatSummary::DestroyedUnits() const {
 }
 
 void CombatSummary::AddUnit(int unit_id, const CombatParticipantState& state) {
-    unit_summaries.push_back(boost::make_shared<ParticipantSummary>(unit_id, SafeEmpireId(empire), state));
+    unit_summaries.push_back(std::make_shared<ParticipantSummary>(unit_id, SafeEmpireId(empire), state));
 
     total_current_health += std::max(unit_summaries.back()->current_health, 0.0f);
     total_max_health += std::max(unit_summaries.back()->max_health, 0.0f);
