@@ -78,9 +78,15 @@ namespace {
                      UserStringNop("OPTIONS_DB_UI_COMBAT_SUMMARY_GRAPH_HEIGHT_PROPORTIONAL"),
                      false);
 
-        db.Add<StreamableColor>("UI.combat.summary.dead-color", UserStringNop("OPTIONS_DB_UI_COMBAT_SUMMARY_DEAD_COLOR"), StreamableColor(GG::Clr(128, 0, 0, 255)));
-        db.Add<StreamableColor>("UI.combat.summary.wound-color", UserStringNop("OPTIONS_DB_UI_COMBAT_SUMMARY_WOUND_COLOR"), StreamableColor(GG::CLR_RED));
-        db.Add<StreamableColor>("UI.combat.summary.health-color", UserStringNop("OPTIONS_DB_UI_COMBAT_SUMMARY_HEALTH_COLOR"), StreamableColor(GG::CLR_GREEN));
+        db.Add<GG::Clr>("UI.combat.summary.dead-color",
+                        UserStringNop("OPTIONS_DB_UI_COMBAT_SUMMARY_DEAD_COLOR"),
+                        GG::Clr(128, 0, 0, 255));
+        db.Add<GG::Clr>("UI.combat.summary.wound-color",
+                        UserStringNop("OPTIONS_DB_UI_COMBAT_SUMMARY_WOUND_COLOR"),
+                        GG::CLR_RED);
+        db.Add<GG::Clr>("UI.combat.summary.health-color",
+                        UserStringNop("OPTIONS_DB_UI_COMBAT_SUMMARY_HEALTH_COLOR"),
+                        GG::CLR_GREEN);
     }
     bool temp_bool = RegisterOptions(&AddOptions);
 
@@ -113,15 +119,15 @@ public:
         // We want to measure health on a single scale that shows as much as possible
         // while fitting the data of both sides.
         // Therefore we make the side with more health fill the window
-        for (std::map<int, CombatSummary>::const_iterator it = combat_summaries.begin(); it != combat_summaries.end(); ++it ) {
-            m_max_total_max_health = std::max( it->second.total_max_health, m_max_total_max_health );
-            m_max_units_on_a_side = std::max( static_cast<int>(it->second.unit_summaries.size()), m_max_units_on_a_side );
-            m_sum_of_max_max_healths += it->second.max_max_health;
+        for (const std::map<int, CombatSummary>::value_type& combat_summary : combat_summaries) {
+            m_max_total_max_health = std::max(combat_summary.second.total_max_health, m_max_total_max_health);
+            m_max_units_on_a_side = std::max(static_cast<int>(combat_summary.second.unit_summaries.size()), m_max_units_on_a_side);
+            m_sum_of_max_max_healths += combat_summary.second.max_max_health;
 
             if (0.f > m_min_of_max_max_healths) {
-                m_min_of_max_max_healths = it->second.total_max_health;
+                m_min_of_max_max_healths = combat_summary.second.total_max_health;
             } else {
-                m_min_of_max_max_healths = std::min(it->second.max_max_health, m_min_of_max_max_healths);
+                m_min_of_max_max_healths = std::min(combat_summary.second.max_max_health, m_min_of_max_max_healths);
             }
         }
 
@@ -263,22 +269,22 @@ public:
         m_sizer(sizer),
         m_hovered(false)
     {
-        TemporaryPtr<UniverseObject> object =  Objects().Object(participant.object_id);
+        std::shared_ptr<UniverseObject> object = Objects().Object(participant.object_id);
         if (object) {
             SetBrowseText(object->PublicName(ClientApp::GetApp()->EmpireID()) + " " +
                           DoubleToString(participant.current_health, 3, false) + "/" +
                           DoubleToString(participant.max_health, 3, false)
             );
         }
-        SetBrowseModeTime(300);
+        SetBrowseModeTime(GetOptionsDB().Get<int>("UI.tooltip-delay"));
 
         OptionsDB& options = GetOptionsDB();
-        m_dead_color = options.Get<StreamableColor>("UI.combat.summary.dead-color").ToClr();
-        m_wound_color = options.Get<StreamableColor>("UI.combat.summary.wound-color").ToClr();
-        m_health_color = options.Get<StreamableColor>("UI.combat.summary.health-color").ToClr();
+        m_dead_color = options.Get<GG::Clr>("UI.combat.summary.dead-color");
+        m_wound_color = options.Get<GG::Clr>("UI.combat.summary.wound-color");
+        m_health_color = options.Get<GG::Clr>("UI.combat.summary.health-color");
     }
 
-    virtual void Render() {
+    void Render() override {
         GG::Clr base_color = Alive() ? m_wound_color : m_dead_color;
 
         // Always draw the red background, health will cover it
@@ -303,10 +309,10 @@ public:
         }
     }
 
-    virtual void MouseEnter(const GG::Pt& pt, GG::Flags< GG::ModKey > mod_keys)
+    void MouseEnter(const GG::Pt& pt, GG::Flags< GG::ModKey > mod_keys) override
     { m_hovered = true; }
 
-    virtual void MouseLeave()
+    void MouseLeave() override
     { m_hovered = false; }
 
     /// Resizes the bar to have a width and height based on the
@@ -340,7 +346,7 @@ public:
     SideBar(const CombatSummary& combat_summary, const BarSizer& sizer):
         GG::Wnd(GG::X0, GG::Y0, GG::X1, GG::Y1, GG::INTERACTIVE),
         m_side_summary(combat_summary),
-        m_x_axis_label(0),
+        m_x_axis_label(nullptr),
         m_sizer(sizer)
     {
 
@@ -365,11 +371,9 @@ public:
     }
 
     void MakeBars() {
-        for (CombatSummary::UnitSummaries::const_iterator it = m_side_summary.unit_summaries.begin();
-                it != m_side_summary.unit_summaries.end();
-                ++it) {
-            if ((*it)->max_health > 0) {
-                m_participant_bars.push_back(new ParticipantBar(**it, m_sizer));
+        for (CombatSummary::ParticipantSummaryPtr unit : m_side_summary.unit_summaries) {
+            if (unit->max_health > 0) {
+                m_participant_bars.push_back(new ParticipantBar(*unit, m_sizer));
                 AttachChild(m_participant_bars.back());
             }
         }
@@ -386,11 +390,7 @@ public:
         alive_ll.y += ClientHeight();
         GG::Pt dead_lr = ClientSize();
 
-        for (std::vector<ParticipantBar*>::const_iterator it = m_participant_bars.begin();
-                it != m_participant_bars.end();
-                ++it)
-        {
-            ParticipantBar* bar = *it;
+        for (ParticipantBar* bar : m_participant_bars) {
             bar->DoLayout(m_y_axis_label->MinUsableSize().x);
             if (bar->Alive()) {
                 bar->MoveBottomTo(alive_ll);
@@ -482,7 +482,7 @@ public:
         glEnable(GL_TEXTURE_2D);
     }
 
-    virtual void Render() {
+    void Render() override {
         // Draw the axes outside th3e client area
         GG::Pt begin(ClientUpperLeft().x - AXIS_WIDTH/2, ClientLowerRight().y + AXIS_HEIGHT/2);
         GG::Pt x_end(ClientLowerRight().x, begin.y);
@@ -491,7 +491,7 @@ public:
         DrawArrow(begin, y_end);
     }
 
-    virtual void SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override {
         GG::Wnd::SizeMove(ul, lr);
         DoLayout();
     }
@@ -501,10 +501,10 @@ public:
         return ul;
     }
 
-    virtual GG::Pt ClientUpperLeft() const
+    GG::Pt ClientUpperLeft() const override
     { return  GG::Wnd::UpperLeft() + RelativeClientUpperLeft(); }
 
-    virtual GG::Pt ClientLowerRight() const {
+    GG::Pt ClientLowerRight() const override {
         // The axes are considered to be outside the client area.
         GG::Pt lr = GG::Wnd::ClientLowerRight();
         lr.y -= AXIS_HEIGHT;
@@ -522,9 +522,8 @@ private:
 
     float MaxMaxHealth() {
         float max_health = -1.0f;
-        for (CombatSummary::UnitSummaries::const_iterator it = m_side_summary.unit_summaries.begin();
-             it != m_side_summary.unit_summaries.end(); ++it)
-        { max_health = std::max(max_health, (*it)->max_health); }
+        for (CombatSummary::ParticipantSummaryPtr participant : m_side_summary.unit_summaries)
+        { max_health = std::max(max_health, participant->max_health); }
         return max_health;
     }
 };
@@ -535,7 +534,7 @@ class OptionsBar : public GG::Wnd {
 public:
     boost::signals2::signal<void ()> ChangedSignal;
 
-    OptionsBar(boost::scoped_ptr<BarSizer>& sizer) :
+    OptionsBar(std::unique_ptr<BarSizer>& sizer) :
         GG::Wnd(),
         m_sizer(sizer)
     {
@@ -563,17 +562,16 @@ public:
     }
 
     virtual ~OptionsBar() {
-        for (std::vector<ToggleData*>::iterator it = m_toggles.begin(); it != m_toggles.end(); ++it)
-        { delete *it; }
+        for (ToggleData* data : m_toggles)
+        { delete data; }
         m_toggles.clear();
     }
 
-    virtual GG::Pt MinUsableSize() const {
+    GG::Pt MinUsableSize() const override {
         GG::Pt min_size(GG::X0, GG::Y0);
 
-        for (std::vector<ToggleData*>::const_iterator it = m_toggles.begin();
-             it != m_toggles.end(); ++it)
-        { min_size.x += (*it)->button->Width() + OPTION_BUTTON_PADDING; }
+        for (ToggleData* data : m_toggles)
+        { min_size.x += data->button->Width() + OPTION_BUTTON_PADDING; }
 
         min_size.y = OPTION_BAR_HEIGHT;
 
@@ -581,20 +579,26 @@ public:
     }
 
     void DoLayout() {
-        boost::shared_ptr<GG::Font> cui_font = ClientUI::GetFont();
+        std::shared_ptr<GG::Font> cui_font = ClientUI::GetFont();
 
         GG::Pt pos(GG::X(0), GG::Y(0));
-        for (std::vector<ToggleData*>::iterator it = m_toggles.begin(); it != m_toggles.end(); ++it) {
-            ToggleData& toggle = **it;
-            toggle.button->Resize(GG::Pt(cui_font->TextExtent(toggle.button->Text(), GG::FORMAT_LEFT).x + OPTION_BUTTON_PADDING,
-                                         OPTION_BUTTON_HEIGHT));
-            toggle.button->MoveTo(pos);
-            pos.x += toggle.button->Width() + OPTION_BUTTON_PADDING;
+        for (ToggleData* data : m_toggles) {
+            if (!data->button->Children().empty()) {
+                //Assume first child of the button is the label
+                if (const GG::TextControl* label = dynamic_cast<GG::TextControl const*>(data->button->Children().front()))
+                {
+                    data->button->Resize(
+                        GG::Pt(label->MinUsableSize().x + OPTION_BUTTON_PADDING,
+                               OPTION_BUTTON_HEIGHT));
+                }
+            }
+            data->button->MoveTo(pos);
+            pos.x += data->button->Width() + OPTION_BUTTON_PADDING;
         }
     }
 
 private:
-    boost::scoped_ptr<BarSizer>& m_sizer;
+    std::unique_ptr<BarSizer>& m_sizer;
     std::vector<ToggleData*> m_toggles;
 
     struct ToggleData : public boost::signals2::trackable {
@@ -606,7 +610,7 @@ private:
         std::string tip_true;
         std::string tip_false;
         std::string option_key;
-        boost::scoped_ptr<BarSizer>* sizer;
+        std::unique_ptr<BarSizer>* sizer;
         OptionsBar* parent;
         GG::Button* button;
 
@@ -627,7 +631,7 @@ private:
         ToggleData(const std::string& label_true, const std::string& label_false,
                    const std::string& tip_true, const std::string& tip_false,
                    std::string option_key,
-                   boost::scoped_ptr<BarSizer>* sizer, OptionsBar* parent) :
+                   std::unique_ptr<BarSizer>* sizer, OptionsBar* parent) :
             label_true(label_true),
             label_false(label_false),
             tip_true(tip_true),
@@ -635,7 +639,7 @@ private:
             option_key(option_key),
             sizer(sizer),
             parent(parent),
-            button(0)
+            button(nullptr)
         {
             button = new CUIButton("-");
             parent->AttachChild(button);
@@ -648,8 +652,8 @@ private:
 
 GraphicalSummaryWnd::GraphicalSummaryWnd() :
     GG::Wnd(GG::X0, GG::Y0, GG::X1, GG::Y1, GG::NO_WND_FLAGS),
-    m_sizer(0),
-    m_options_bar(0)
+    m_sizer(nullptr),
+    m_options_bar(nullptr)
 {}
 
 GG::Pt GraphicalSummaryWnd::MinUsableSize() const {
@@ -688,8 +692,7 @@ void GraphicalSummaryWnd::DoLayout() {
     m_sizer->SetAvailableSize(space_for_bars);
     m_options_bar->DoLayout();
 
-    for (std::vector<SideBar*>::iterator it = m_side_boxes.begin(); it != m_side_boxes.end(); ++it ) {
-        SideBar* box = *it;
+    for (SideBar* box : m_side_boxes) {
         box->MoveTo(ul);
         box->DoLayout();
         ul.y += box->Height() + SIDE_BOX_MARGIN;
@@ -710,15 +713,14 @@ void GraphicalSummaryWnd::HandleButtonChanged() {
 
 void GraphicalSummaryWnd::MakeSummaries(int log_id) {
     m_summaries.clear();
-    if (!CombatLogAvailable(log_id)) {
+    boost::optional<const CombatLog&> log = GetCombatLog(log_id);
+    if (!log) {
         ErrorLogger() << "CombatReportWnd::CombatReportPrivate::MakeSummaries: Could not find log: " << log_id;
     } else {
-        const CombatLog& log = GetCombatLog(log_id);
-        for (std::set<int>::const_iterator it = log.object_ids.begin(); it != log.object_ids.end(); ++it) {
-            int object_id = *it;
+        for (int object_id : log->object_ids) {
             if (object_id < 0)
                 continue;   // fighters and invalid objects
-            TemporaryPtr<UniverseObject> object = GetUniverseObject(object_id);
+            std::shared_ptr<UniverseObject> object = GetUniverseObject(object_id);
             if (!object) {
                 ErrorLogger() << "GraphicalSummaryWnd::MakeSummaries couldn't find object with id: " << object_id;
                 continue;
@@ -728,19 +730,19 @@ void GraphicalSummaryWnd::MakeSummaries(int log_id) {
             if (m_summaries.find(owner_id) == m_summaries.end())
                 m_summaries[owner_id] = CombatSummary(owner_id);
 
-            std::map<int, CombatParticipantState>::const_iterator map_it = log.participant_states.find(object_id);
-            if (map_it != log.participant_states.end()) {
+            std::map<int, CombatParticipantState>::const_iterator map_it = log->participant_states.find(object_id);
+            if (map_it != log->participant_states.end()) {
                 m_summaries[owner_id].AddUnit(object_id, map_it->second);
             } else {
                 ErrorLogger() << "Participant state missing from log. Object id: " << object_id << " log id: " << log_id;
             }
         }
 
-        for (std::map<int, CombatSummary>::iterator it = m_summaries.begin(); it != m_summaries.end(); ++it) {
-            DebugLogger() << "MakeSummaries: empire " << it->first
-                          << " total health: " << it->second.total_current_health
-                          << " max health: " << it->second.total_max_health
-                          << " units: " << it->second.unit_summaries.size();
+        for (std::map<int, CombatSummary>::value_type& summary : m_summaries) {
+            DebugLogger() << "MakeSummaries: empire " << summary.first
+                          << " total health: " << summary.second.total_current_health
+                          << " max health: " << summary.second.total_max_health
+                          << " units: " << summary.second.unit_summaries.size();
         }
     }
 
@@ -748,9 +750,8 @@ void GraphicalSummaryWnd::MakeSummaries(int log_id) {
 }
 
 void GraphicalSummaryWnd::DeleteSideBars() {
-    for (std::vector<SideBar*>::iterator it = m_side_boxes.begin();
-         it != m_side_boxes.end(); ++it)
-    { DeleteChild(*it); }
+    for (SideBar* box : m_side_boxes)
+    { DeleteChild(box); }
     m_side_boxes.clear();
 }
 
@@ -759,10 +760,10 @@ void GraphicalSummaryWnd::GenerateGraph() {
 
     m_sizer.reset(new BarSizer(m_summaries, ClientSize()));
 
-    for (std::map<int, CombatSummary>::iterator it = m_summaries.begin(); it != m_summaries.end(); ++it) {
-        if (it->second.total_max_health > EPSILON) {
-            it->second.Sort();
-            SideBar* box = new SideBar(it->second, *m_sizer);
+    for (std::map<int, CombatSummary>::value_type& summary : m_summaries) {
+        if (summary.second.total_max_health > EPSILON) {
+            summary.second.Sort();
+            SideBar* box = new SideBar(summary.second, *m_sizer);
             m_side_boxes.push_back(box);
             AttachChild(box);
         }
