@@ -18,7 +18,9 @@ public:
 
     /// Get the minimum size of this window required to show all of its
     /// children
-    virtual GG::Pt MinUsableSize() const;
+    GG::Pt MinUsableSize() const override;
+
+    void Render() override;
 
     /// Sets the log to show
     void SetLog(int log_id);
@@ -26,12 +28,12 @@ public:
     /// Position and sizes things appropriately
     void DoLayout();
 
-    virtual void Render();
-
 private:
     std::vector<SideBar*>        m_side_boxes;
     std::map<int, CombatSummary> m_summaries;
-    boost::scoped_ptr<BarSizer>  m_sizer;
+
+    std::unique_ptr<BarSizer> m_sizer;
+
     OptionsBar*                  m_options_bar; // Is a child window->GG handles memory
 
     void HandleButtonChanged();
