@@ -11,7 +11,8 @@
 #include "../client/human/HumanClientApp.h"
 #include "../universe/Fleet.h"
 #include "../universe/Ship.h"
-#include "../universe/System.h" 
+#include "../universe/System.h"
+#include "../universe/Enums.h"
 #include "../Empire/Empire.h"
 
 #include <GG/DrawUtil.h>
@@ -67,20 +68,20 @@ namespace {
 // FleetButton           //
 ///////////////////////////
 FleetButton::FleetButton(const std::vector<int>& fleet_IDs, SizeType size_type) :
-    GG::Button("", boost::shared_ptr<GG::Font>(), GG::CLR_ZERO),
+    GG::Button("", nullptr, GG::CLR_ZERO),
     m_fleets(),
     m_icons(),
-    m_selection_indicator(0),
-    m_scanline_control(0),
+    m_selection_indicator(nullptr),
+    m_scanline_control(nullptr),
     m_selected(false)
 { Init(fleet_IDs, size_type); }
 
 FleetButton::FleetButton(int fleet_id, SizeType size_type) :
-    GG::Button("", boost::shared_ptr<GG::Font>(), GG::CLR_ZERO),
+    GG::Button("", nullptr, GG::CLR_ZERO),
     m_fleets(),
     m_icons(),
-    m_selection_indicator(0),
-    m_scanline_control(0),
+    m_selection_indicator(nullptr),
+    m_scanline_control(nullptr),
     m_selected(false)
 {
     std::vector<int> fleet_IDs;
@@ -102,14 +103,14 @@ void FleetButton::Init(const std::vector<int>& fleet_IDs, SizeType size_type) {
     //std::cout << "FleetButton::Init" << std::endl;
 
     // get fleets
-    std::vector<TemporaryPtr<const Fleet> > fleets;
-    for (std::vector<int>::const_iterator it = fleet_IDs.begin(); it != fleet_IDs.end(); ++it) {
-        TemporaryPtr<const Fleet> fleet = GetFleet(*it);
+    std::vector<std::shared_ptr<const Fleet>> fleets;
+    for (int fleet_id : fleet_IDs) {
+        std::shared_ptr<const Fleet> fleet = GetFleet(fleet_id);
         if (!fleet) {
-            ErrorLogger() << "FleetButton::FleetButton couldn't get fleet with id " << *it;
+            ErrorLogger() << "FleetButton::FleetButton couldn't get fleet with id " << fleet_id;
             continue;
         }
-        m_fleets.push_back(*it);
+        m_fleets.push_back(fleet_id);
         fleets.push_back(fleet);
     }
 
@@ -124,8 +125,7 @@ void FleetButton::Init(const std::vector<int>& fleet_IDs, SizeType size_type) {
     } else {
         owner_id = (*fleets.begin())->Owner();
         // use ALL_EMPIRES if there are multiple owners (including no owner and an owner)
-        for (std::vector<TemporaryPtr<const Fleet> >::const_iterator it = fleets.begin(); it != fleets.end(); ++it) {
-            TemporaryPtr<const Fleet> fleet = *it;
+        for (std::shared_ptr<const Fleet> fleet : fleets) {
             if (fleet->Owner() != owner_id) {
                 owner_id = ALL_EMPIRES;
                 multiple_owners = true;
@@ -142,12 +142,9 @@ void FleetButton::Init(const std::vector<int>& fleet_IDs, SizeType size_type) {
         // all ships owned by now empire
         bool monsters = true;
         // find if any ship in fleets in button is not a monster
-        for (std::vector<TemporaryPtr<const Fleet> >::const_iterator it = fleets.begin(); it != fleets.end(); ++it) {
-            TemporaryPtr<const Fleet> fleet = *it;
-            for (std::set<int>::const_iterator ship_it = fleet->ShipIDs().begin();
-                 ship_it != fleet->ShipIDs().end(); ++ship_it)
-            {
-                if (TemporaryPtr<const Ship> ship = GetShip(*ship_it)) {
+        for (std::shared_ptr<const Fleet> fleet : fleets) {
+            for (int ship_id : fleet->ShipIDs()) {
+                if (std::shared_ptr<const Ship> ship = GetShip(ship_id)) {
                     if (!ship->IsMonster()) {
                         monsters = false;
                         break;
@@ -172,12 +169,12 @@ void FleetButton::Init(const std::vector<int>& fleet_IDs, SizeType size_type) {
     // determine direction button should be rotated to orient along a starlane
     GLfloat pointing_angle = 0.0f;
 
-    TemporaryPtr<const Fleet> first_fleet;
+    std::shared_ptr<const Fleet> first_fleet;
     if (!m_fleets.empty())
         first_fleet = *fleets.begin();
     if (first_fleet && first_fleet->SystemID() == INVALID_OBJECT_ID && first_fleet->NextSystemID() != INVALID_OBJECT_ID) {
         int next_sys_id = first_fleet->NextSystemID();
-        if (TemporaryPtr<const UniverseObject> obj = GetUniverseObject(next_sys_id)) {
+        if (std::shared_ptr<const UniverseObject> obj = GetUniverseObject(next_sys_id)) {
             // fleet is not in a system and has a valid next destination, so can orient it in that direction
             // fleet icons might not appear on the screen in the exact place corresponding to their
             // actual universe position, but if they're moving along a starlane, this code will assume
@@ -199,14 +196,11 @@ void FleetButton::Init(const std::vector<int>& fleet_IDs, SizeType size_type) {
 
     // select icon(s) for fleet(s)
     int num_ships = 0;
-    for (std::vector<TemporaryPtr<const Fleet> >::const_iterator it = fleets.begin(); it != fleets.end(); ++it) {
-        TemporaryPtr<const Fleet> fleet = *it;
+    for (std::shared_ptr<const Fleet> fleet : fleets) {
         if (fleet)
             num_ships += fleet->NumShips();
     }
-    boost::shared_ptr<GG::Texture> size_texture = FleetSizeIcon(num_ships, size_type);
-    std::vector<boost::shared_ptr<GG::Texture> > head_textures = FleetHeadIcons(fleets, size_type);
-
+    std::shared_ptr<GG::Texture> size_texture = FleetSizeIcon(num_ships, size_type);
 
     // add RotatingGraphics for all icons needed
     if (size_texture) {
@@ -219,16 +213,14 @@ void FleetButton::Init(const std::vector<int>& fleet_IDs, SizeType size_type) {
         AttachChild(icon);
     }
 
-    for (std::vector<boost::shared_ptr<GG::Texture> >::const_iterator it = head_textures.begin();
-         it != head_textures.end(); ++it)
-    {
-        RotatingGraphic* icon = new RotatingGraphic(*it, GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE);
+    for (std::shared_ptr<GG::Texture> texture : FleetHeadIcons(fleets, size_type)) {
+        RotatingGraphic* icon = new RotatingGraphic(texture, GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE);
         icon->SetPhaseOffset(pointing_angle);
         icon->SetRPM(0.0f);
         icon->SetColor(this->Color());
         m_icons.push_back(icon);
-        if (Width() < (*it)->DefaultWidth())
-            Resize(GG::Pt((*it)->DefaultWidth(), (*it)->DefaultHeight()));
+        if (Width() < texture->DefaultWidth())
+            Resize(GG::Pt(texture->DefaultWidth(), texture->DefaultHeight()));
         AttachChild(icon);
     }
 
@@ -244,15 +236,16 @@ void FleetButton::Init(const std::vector<int>& fleet_IDs, SizeType size_type) {
         return;
 
     bool at_least_one_fleet_visible = false;
-    for (std::vector<int>::const_iterator it = m_fleets.begin(); it != m_fleets.end(); ++it) {
-        if (GetUniverse().GetObjectVisibilityByEmpire(*it, empire_id) >= VIS_BASIC_VISIBILITY) {
+    for (int fleet_id : m_fleets) {
+        if (GetUniverse().GetObjectVisibilityByEmpire(fleet_id, empire_id) >= VIS_BASIC_VISIBILITY) {
             at_least_one_fleet_visible = true;
             break;
         }
     }
 
-    // Create scanline renderer control
-    m_scanline_control = new ScanlineControl(GG::X0, GG::Y0, Width(), Height());
+    // Create scanline renderer control, use opposite color of fleet btn
+    GG::Clr opposite_clr(255 - Color().r, 255 - Color().g, 255 - Color().b, 64);
+    m_scanline_control = new ScanlineControl(GG::X0, GG::Y0, Width(), Height(), false, opposite_clr);
 
     if (!at_least_one_fleet_visible)
         AttachChild(m_scanline_control);
@@ -296,8 +289,7 @@ void FleetButton::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
 
 void FleetButton::LayoutIcons() {
     GG::Pt middle = GG::Pt(Width() / 2, Height() / 2);
-    for (std::vector<RotatingGraphic*>::iterator it = m_icons.begin(); it != m_icons.end(); ++it) {
-        RotatingGraphic* graphic = *it;
+    for (RotatingGraphic* graphic : m_icons) {
         GG::SubTexture subtexture = graphic->GetTexture();
         GG::Pt subtexture_sz = GG::Pt(subtexture.Width(), subtexture.Height());
         //std::cout << "FleetButton::LayoutIcons repositioning icon: sz: " << subtexture_sz << "  tex: " << subtexture.GetTexture()->Filename() << std::endl;
@@ -367,24 +359,23 @@ void FleetButton::PlayFleetButtonOpenSound()
 /////////////////////
 // Free Functions
 /////////////////////
-std::vector<boost::shared_ptr<GG::Texture> > FleetHeadIcons(TemporaryPtr<const Fleet> fleet, FleetButton::SizeType size_type) {
-    std::vector<TemporaryPtr<const Fleet> > fleets(1U, fleet);
+std::vector<std::shared_ptr<GG::Texture>> FleetHeadIcons(std::shared_ptr<const Fleet> fleet, FleetButton::SizeType size_type) {
+    std::vector<std::shared_ptr<const Fleet>> fleets(1U, fleet);
     return FleetHeadIcons(fleets, size_type);
 }
 
-std::vector<boost::shared_ptr<GG::Texture> > FleetHeadIcons(const std::vector< TemporaryPtr<const Fleet> >& fleets, FleetButton::SizeType size_type) {
+std::vector<std::shared_ptr<GG::Texture>> FleetHeadIcons(const std::vector<std::shared_ptr<const Fleet>>& fleets, FleetButton::SizeType size_type) {
     if (size_type == FleetButton::FLEET_BUTTON_NONE || size_type == FleetButton::FLEET_BUTTON_TINY)
-        return std::vector<boost::shared_ptr<GG::Texture> >();
+        return std::vector<std::shared_ptr<GG::Texture>>();
 
     // get file name prefix for appropriate size of icon
     std::string size_prefix = FleetIconSizePrefix(size_type);
     if (size_prefix.empty())
-        return std::vector<boost::shared_ptr<GG::Texture> >();
+        return std::vector<std::shared_ptr<GG::Texture>>();
 
     // the set of fleets is treated like a fleet that contains all the ships
     bool hasColonyShips = false; bool hasOutpostShips = false; bool hasTroopShips = false; bool hasMonsters = false; bool hasArmedShips = false;
-    for (std::vector< TemporaryPtr<const Fleet> >::const_iterator fleet_it = fleets.begin(); fleet_it != fleets.end(); ++fleet_it) {
-        const TemporaryPtr<const Fleet> fleet = *fleet_it;
+    for (std::shared_ptr<const Fleet> fleet : fleets) {
         if (!fleet)
             continue;
 
@@ -392,7 +383,7 @@ std::vector<boost::shared_ptr<GG::Texture> > FleetHeadIcons(const std::vector< T
         hasOutpostShips = hasOutpostShips || fleet->HasOutpostShips();
         hasTroopShips   = hasTroopShips   || fleet->HasTroopShips();
         hasMonsters     = hasMonsters     || fleet->HasMonsters();
-        hasArmedShips   = hasArmedShips   || fleet->HasArmedShips();
+        hasArmedShips   = hasArmedShips   || fleet->HasArmedShips() || fleet->HasFighterShips();
     }
 
     // get file name main part depending on type of fleet
@@ -409,10 +400,10 @@ std::vector<boost::shared_ptr<GG::Texture> > FleetHeadIcons(const std::vector< T
     }
     if (main_filenames.empty()) { main_filenames.push_back("head-scout.png"); }
 
-    std::vector<boost::shared_ptr<GG::Texture> > result;
-    for (std::vector<std::string>::const_iterator it = main_filenames.begin(); it != main_filenames.end(); ++it) {
-        boost::shared_ptr<GG::Texture> texture_temp = ClientUI::GetTexture(
-            ClientUI::ArtDir() / "icons" / "fleet" / (size_prefix + *it), false);
+    std::vector<std::shared_ptr<GG::Texture>> result;
+    for (const std::string& name : main_filenames) {
+        std::shared_ptr<GG::Texture> texture_temp = ClientUI::GetTexture(
+            ClientUI::ArtDir() / "icons" / "fleet" / (size_prefix + name), false);
         glBindTexture(GL_TEXTURE_2D, texture_temp->OpenGLId());
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
@@ -424,18 +415,18 @@ std::vector<boost::shared_ptr<GG::Texture> > FleetHeadIcons(const std::vector< T
     return result;
 }
 
-boost::shared_ptr<GG::Texture> FleetSizeIcon(TemporaryPtr<const Fleet> fleet, FleetButton::SizeType size_type) {
+std::shared_ptr<GG::Texture> FleetSizeIcon(std::shared_ptr<const Fleet> fleet, FleetButton::SizeType size_type) {
     if (!fleet)
         return FleetSizeIcon(1u, size_type);
     return FleetSizeIcon(fleet->NumShips(), size_type);
 }
 
-boost::shared_ptr<GG::Texture> FleetSizeIcon(unsigned int fleet_size, FleetButton::SizeType size_type) {
+std::shared_ptr<GG::Texture> FleetSizeIcon(unsigned int fleet_size, FleetButton::SizeType size_type) {
     if (fleet_size < 1u)
         fleet_size = 1u; // because there's no zero-ship icon, and the one-ship icon is (as of this writing) blank, so is fitting for zero ships
 
     if (size_type == FleetButton::FLEET_BUTTON_NONE)
-        return boost::shared_ptr<GG::Texture>();
+        return nullptr;
 
     if (size_type == FleetButton::FLEET_BUTTON_TINY) {
         if (fleet_size > 1u)
@@ -447,9 +438,9 @@ boost::shared_ptr<GG::Texture> FleetSizeIcon(unsigned int fleet_size, FleetButto
     std::string size_prefix = FleetIconSizePrefix(size_type);
 
     if (size_prefix.empty())
-        return boost::shared_ptr<GG::Texture>();
+        return nullptr;
 
-    boost::shared_ptr<GG::Texture> texture_temp = ClientUI::GetClientUI()->GetModuloTexture(
+    std::shared_ptr<GG::Texture> texture_temp = ClientUI::GetClientUI()->GetModuloTexture(
         ClientUI::ArtDir() / "icons" / "fleet", (size_prefix + "tail-"), FleetSizeIconNumber(fleet_size), false);
     glBindTexture(GL_TEXTURE_2D, texture_temp->OpenGLId());
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
@@ -459,8 +450,8 @@ boost::shared_ptr<GG::Texture> FleetSizeIcon(unsigned int fleet_size, FleetButto
     return texture_temp;
 }
 
-boost::shared_ptr<GG::Texture> FleetSelectionIndicatorIcon() {
-    static boost::shared_ptr<GG::Texture> retval;
+std::shared_ptr<GG::Texture> FleetSelectionIndicatorIcon() {
+    static std::shared_ptr<GG::Texture> retval;
     if (!retval) {
         retval = ClientUI::GetClientUI()->GetTexture(ClientUI::ArtDir() / "icons" / "fleet" / "fleet_selection.png", false);
         glBindTexture(GL_TEXTURE_2D, retval->OpenGLId());
