@@ -1,10 +1,13 @@
 #ifndef _FleetButton_h_
 #define _FleetButton_h_
 
+
 #include <GG/Button.h>
 
+#include <memory>
+
+
 class Fleet;
-template <class T> class TemporaryPtr;
 class RotatingGraphic;
 class ScanlineControl;
 
@@ -26,15 +29,19 @@ public:
     //@}
 
     /** \name Accessors */ //@{
-    virtual bool            InWindow(const GG::Pt& pt) const;       ///< returns true if \a pt is within or over the button
+    /** Returns true if \a pt is within or over the button. */
+    bool InWindow(const GG::Pt& pt) const override;
+
     const std::vector<int>& Fleets() const {return m_fleets;}       ///< returns the fleets represented by this control
     bool                    Selected() const {return m_selected;}   ///< returns whether this button has been marked selected
     //@}
 
     /** \name Mutators */ //@{
-    virtual void            MouseHere(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
+    void MouseHere(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
+
     void                    SetSelected(bool selected = true);      ///< sets selection status of button.  if selected = true, marks button as selected.  otherwise marks button as not selected
-    virtual void            SizeMove(const GG::Pt& ul, const GG::Pt& lr);
     //@}
 
     static void             PlayFleetButtonOpenSound();
@@ -42,9 +49,11 @@ public:
 
 protected:
     /** \name Mutators */ //@{
-    virtual void            RenderUnpressed();
-    virtual void            RenderPressed();
-    virtual void            RenderRollover();
+    void RenderUnpressed() override;
+
+    void RenderPressed() override;
+
+    void RenderRollover() override;
     //@}
 
 private:
@@ -59,18 +68,18 @@ private:
 };
 
 /* returns head icon for passed fleet at passed icon size */
-std::vector<boost::shared_ptr<GG::Texture> > FleetHeadIcons(TemporaryPtr<const Fleet>, FleetButton::SizeType size_type);
+std::vector<std::shared_ptr<GG::Texture>> FleetHeadIcons(std::shared_ptr<const Fleet>, FleetButton::SizeType size_type);
 
 /* returns head icon for passed fleets at passed icon size */
-std::vector<boost::shared_ptr<GG::Texture> > FleetHeadIcons(const std::vector<TemporaryPtr<const Fleet> >& fleets, FleetButton::SizeType size_type);
+std::vector<std::shared_ptr<GG::Texture>> FleetHeadIcons(const std::vector<std::shared_ptr<const Fleet>>& fleets, FleetButton::SizeType size_type);
 
 /* returns size icon for passed fleet at passed icon size */
-boost::shared_ptr<GG::Texture> FleetSizeIcon(TemporaryPtr<const Fleet> fleet, FleetButton::SizeType size_type);
+std::shared_ptr<GG::Texture> FleetSizeIcon(std::shared_ptr<const Fleet> fleet, FleetButton::SizeType size_type);
 
 /* returns head icon for passed fleet size at passed icon size */
-boost::shared_ptr<GG::Texture> FleetSizeIcon(unsigned int fleet_size, FleetButton::SizeType size_type);
+std::shared_ptr<GG::Texture> FleetSizeIcon(unsigned int fleet_size, FleetButton::SizeType size_type);
 
 /* returns icon for indication fleet icon selection */
-boost::shared_ptr<GG::Texture> FleetSelectionIndicatorIcon();
+std::shared_ptr<GG::Texture> FleetSelectionIndicatorIcon();
 
 #endif // _FleetButton_h_
