@@ -35,7 +35,7 @@ public:
     iterator        begin() const;
     iterator        end() const;
     FleetWnd*       ActiveFleetWnd() const;
-    FleetWnd*       WndForFleet(TemporaryPtr<const Fleet> fleet) const;
+    FleetWnd* WndForFleet(std::shared_ptr<const Fleet> fleet) const;
     int             SelectedShipID() const;     // if a single ship is selected in the active fleetwnd, returns that ship's ID.  Otherwise, returns INVALID_OBJECT_ID
     std::set<int>   SelectedShipIDs() const;    // returns the ids of all selected ships in the active fleetwnd
     //@}
@@ -45,10 +45,6 @@ public:
                                 int selected_fleet_id = INVALID_OBJECT_ID,
                                 GG::Flags<GG::WndFlag> flags = GG::INTERACTIVE | GG::DRAGABLE | GG::ONTOP | CLOSABLE | GG::RESIZABLE);
 
-    FleetWnd*       NewFleetWnd(int system_id, int empire_id,
-                                int selected_fleet_id = INVALID_OBJECT_ID,
-                                GG::Flags<GG::WndFlag> flags = GG::INTERACTIVE | GG::DRAGABLE | GG::ONTOP | CLOSABLE | GG::RESIZABLE);
-
     void            CullEmptyWnds();
     void            SetActiveFleetWnd(FleetWnd* fleet_wnd);
     bool            CloseAll();
@@ -94,7 +90,8 @@ class FleetWnd : public MapWndPopup {
 public:
     /** \name Structors */ //@{
     FleetWnd();
-    ~FleetWnd(); ///< dtor
+
+    ~FleetWnd();
     //@}
 
     //! \name Accessors //@{
@@ -105,15 +102,18 @@ public:
     std::set<int>           SelectedFleetIDs() const;           ///< returns IDs of selected fleets in this FleetWnd
     std::set<int>           SelectedShipIDs() const;            ///< returns IDs of selected ships in this FleetWnd
     NewFleetAggression      GetNewFleetAggression() const;      ///< returns this FleetWnd's setting for new fleet aggression (auto, aggressive, or passive)
-    virtual GG::Rect        CalculatePosition() const;
+
+    GG::Rect CalculatePosition() const override;
     //@}
 
     //! \name Mutators //@{
+    void PreRender() override;
+
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
+
     void                    SelectFleet(int fleet_id);                          ///< deselects any selected fleets, and selects the indicated fleet, bringing it into the fleet detail window
     void                    SetSelectedFleets(const std::set<int>& fleet_ids);  ///< deselects any selected fleets, and selects the fleets with the indicated ids
     void                    SetSelectedShips(const std::set<int>& ship_ids);    ///< deselected any selected ships, and selects the ships with the indicated ids if they are in the selected fleet.
-    virtual void            SizeMove(const GG::Pt& ul, const GG::Pt& lr);
-    void                    Refresh();                          ///< regenerates contents
 
     /** Enables, or disables if \a enable is false, issuing orders via this FleetWnd. */
     void                    EnableOrderIssuing(bool enable = true);
@@ -127,8 +127,10 @@ public:
 
 protected:
     //! \name Mutators //@{
-    virtual void    CloseClicked();
-    virtual void    LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
+    void CloseClicked() override;
+
+    void LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
     void            DoLayout();
     //@}
 
@@ -138,14 +140,10 @@ private:
              int selected_fleet_id = INVALID_OBJECT_ID,
              GG::Flags<GG::WndFlag> flags = GG::INTERACTIVE | GG::DRAGABLE | GG::ONTOP | CLOSABLE | GG::RESIZABLE,
              const std::string& config_name = "");
-
-    FleetWnd(int system_id, int empire_id, bool order_issuing_enabled,
-             int selected_fleet_id = INVALID_OBJECT_ID,
-             GG::Flags<GG::WndFlag> flags = GG::INTERACTIVE | GG::DRAGABLE | GG::ONTOP | CLOSABLE | GG::RESIZABLE,
-             const std::string& config_name = "");
     //@}
 
     void            Init(int selected_fleet_id);
+    void            Refresh();                          ///< regenerates contents
     void            RefreshStateChangedSignals();
 
     void            AddFleet(int fleet_id);     ///< adds a new fleet row to this FleetWnd's ListBox of FleetRows and updates internal fleets bookkeeping
@@ -162,7 +160,7 @@ private:
 
     void            ShipSelectionChanged(const GG::ListBox::SelectionSet& rows);
 
-    void            UniverseObjectDeleted(TemporaryPtr<const UniverseObject> obj);
+    void UniverseObjectDeleted(std::shared_ptr<const UniverseObject> obj);
 
     void            SystemChangedSlot();                    ///< responds to StateChangedSignal emitted by the system this FleetWnd is showing the contents of
     void            SetStatIconValues();          ///< sets values for multi-fleet aggregate stat icons at top of FleetWnd
@@ -180,7 +178,7 @@ private:
     FleetDataPanel*     m_new_fleet_drop_target;
     FleetDetailPanel*   m_fleet_detail_panel;
 
-    std::vector<std::pair<MeterType, StatisticIcon*> >    m_stat_icons;   /// statistic icons and associated meter types for multi-fleet aggregate
+    std::vector<std::pair<MeterType, StatisticIcon*>> m_stat_icons; /// statistic icons and associated meter types for multi-fleet aggregate
 
     friend class FleetUIManager;
 };
@@ -195,17 +193,19 @@ public:
 
     //! \name Accessors //@{
     /** Excludes border from the client area. */
-    virtual GG::Pt  ClientUpperLeft() const;
+    GG::Pt ClientUpperLeft() const override;
     /** Excludes border from the client area. */
-    virtual GG::Pt  ClientLowerRight() const;
+    GG::Pt ClientLowerRight() const override;
     //@}
 
     //! \name Mutators //@{
     /** Renders black panel background, border with color depending on the
       *current state and a background for the ship's name text. */
-    virtual void    Render();
+    void Render() override;
+
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
+
     void            Select(bool b);
-    virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr);
     //@}
 
 private:
@@ -229,11 +229,11 @@ private:
     GG::Label*                  m_ship_name_text;
     GG::Label*                  m_design_name_text;
 
-    std::vector<std::pair<MeterType, StatisticIcon*> >    m_stat_icons;   // statistic icons and associated meter types
+    std::vector<std::pair<MeterType, StatisticIcon*>> m_stat_icons;   // statistic icons and associated meter types
 
     bool                        m_selected;
-    boost::signals2::connection  m_ship_connection;
-    boost::signals2::connection  m_fleet_connection;
+    boost::signals2::connection m_ship_connection;
+    boost::signals2::connection m_fleet_connection;
 };
 
 #endif // _FleetWnd_h_
