@@ -4,12 +4,14 @@
 #include "CUISpin.h"
 #include "Sound.h"
 #include "../universe/Universe.h"
+#include "../universe/Enums.h"
 #include "../client/human/HumanClientApp.h"
 #include "../util/i18n.h"
 #include "../util/Logger.h"
 #include "../util/MultiplayerCommon.h"
 #include "../util/OptionsDB.h"
 #include "../util/Directories.h"
+#include "../util/AppInterface.h"
 
 #include <boost/filesystem/fstream.hpp>
 
@@ -31,7 +33,6 @@ namespace {
     }
     const GG::Pt PREVIEW_SZ(GG::X(248), GG::Y(186));
     const bool ALLOW_NO_STARLANES = false;
-    const int MAX_AI_PLAYERS = 40;
 
     // persistant between-executions galaxy setup settings, mainly so I don't have to redo these settings to what I want every time I run FO to test something
     void AddOptions(OptionsDB& db) {
@@ -42,14 +43,14 @@ namespace {
         db.Add("GameSetup.planet-density",      UserStringNop("OPTIONS_DB_GAMESETUP_PLANET_DENSITY"),          GALAXY_SETUP_MEDIUM,RangedValidator<GalaxySetupOption>(GALAXY_SETUP_LOW, GALAXY_SETUP_RANDOM));
         db.Add("GameSetup.starlane-frequency",  UserStringNop("OPTIONS_DB_GAMESETUP_STARLANE_FREQUENCY"),      GALAXY_SETUP_MEDIUM,RangedValidator<GalaxySetupOption>(ALLOW_NO_STARLANES ? GALAXY_SETUP_NONE : GALAXY_SETUP_LOW, GALAXY_SETUP_RANDOM));
         db.Add("GameSetup.specials-frequency",  UserStringNop("OPTIONS_DB_GAMESETUP_SPECIALS_FREQUENCY"),      GALAXY_SETUP_MEDIUM,RangedValidator<GalaxySetupOption>(GALAXY_SETUP_NONE, GALAXY_SETUP_RANDOM));
-        db.Add("GameSetup.monster-frequency",   UserStringNop("OPTIONS_DB_GAMESETUP_MONSTER_FREQUENCY"),       GALAXY_SETUP_LOW,   RangedValidator<GalaxySetupOption>(GALAXY_SETUP_NONE, GALAXY_SETUP_RANDOM));
+        db.Add("GameSetup.monster-frequency",   UserStringNop("OPTIONS_DB_GAMESETUP_MONSTER_FREQUENCY"),       GALAXY_SETUP_MEDIUM,RangedValidator<GalaxySetupOption>(GALAXY_SETUP_NONE, GALAXY_SETUP_RANDOM));
         db.Add("GameSetup.native-frequency",    UserStringNop("OPTIONS_DB_GAMESETUP_NATIVE_FREQUENCY"),        GALAXY_SETUP_MEDIUM,RangedValidator<GalaxySetupOption>(GALAXY_SETUP_NONE, GALAXY_SETUP_RANDOM));
         db.Add("GameSetup.empire-name",         UserStringNop("OPTIONS_DB_GAMESETUP_EMPIRE_NAME"),             std::string(""),    Validator<std::string>());
         db.Add("GameSetup.player-name",         UserStringNop("OPTIONS_DB_GAMESETUP_PLAYER_NAME"),             std::string(""),    Validator<std::string>());
         db.Add("GameSetup.empire-color",        UserStringNop("OPTIONS_DB_GAMESETUP_EMPIRE_COLOR"),            9,                  RangedValidator<int>(0, 100));
         db.Add("GameSetup.starting-species",    UserStringNop("OPTIONS_DB_GAMESETUP_STARTING_SPECIES_NAME"),   std::string("SP_HUMAN"),    Validator<std::string>());
-        db.Add("GameSetup.ai-players",          UserStringNop("OPTIONS_DB_GAMESETUP_NUM_AI_PLAYERS"),          6,                  RangedValidator<int>(0, MAX_AI_PLAYERS));
-        db.Add("GameSetup.ai-aggression",       UserStringNop("OPTIONS_DB_GAMESETUP_AI_MAX_AGGRESSION"),       BEGINNER,           RangedValidator<Aggression>(BEGINNER, MANIACAL));
+        db.Add("GameSetup.ai-players",          UserStringNop("OPTIONS_DB_GAMESETUP_NUM_AI_PLAYERS"),          6,                  RangedValidator<int>(0, IApp::MAX_AI_PLAYERS()));
+        db.Add("GameSetup.ai-aggression",       UserStringNop("OPTIONS_DB_GAMESETUP_AI_MAX_AGGRESSION"),       MANIACAL,           RangedValidator<Aggression>(BEGINNER, MANIACAL));
     }
     bool temp_bool = RegisterOptions(&AddOptions);
 }
@@ -63,27 +64,27 @@ const GG::X GalaxySetupPanel::DefaultWidth() {
 
 GalaxySetupPanel::GalaxySetupPanel(GG::X w, GG::Y h) :
     GG::Control(GG::X0, GG::Y0, w, h, GG::NO_WND_FLAGS),
-    m_seed_label(0),
-    m_seed_edit(0),
-    m_random(0),
-    m_stars_label(0),
-    m_stars_spin(0),
-    m_galaxy_shapes_label(0),
-    m_galaxy_shapes_list(0),
-    m_galaxy_ages_label(0),
-    m_galaxy_ages_list(0),
-    m_starlane_freq_label(0),
-    m_starlane_freq_list(0),
-    m_planet_density_label(0),
-    m_planet_density_list(0),
-    m_specials_freq_label(0),
-    m_specials_freq_list(0),
-    m_monster_freq_label(0),
-    m_monster_freq_list(0),
-    m_native_freq_label(0),
-    m_native_freq_list(0),
-    m_ai_aggression_label(0),
-    m_ai_aggression_list(0)
+    m_seed_label(nullptr),
+    m_seed_edit(nullptr),
+    m_random(nullptr),
+    m_stars_label(nullptr),
+    m_stars_spin(nullptr),
+    m_galaxy_shapes_label(nullptr),
+    m_galaxy_shapes_list(nullptr),
+    m_galaxy_ages_label(nullptr),
+    m_galaxy_ages_list(nullptr),
+    m_starlane_freq_label(nullptr),
+    m_starlane_freq_list(nullptr),
+    m_planet_density_label(nullptr),
+    m_planet_density_list(nullptr),
+    m_specials_freq_label(nullptr),
+    m_specials_freq_list(nullptr),
+    m_monster_freq_label(nullptr),
+    m_monster_freq_list(nullptr),
+    m_native_freq_label(nullptr),
+    m_native_freq_list(nullptr),
+    m_ai_aggression_label(nullptr),
+    m_ai_aggression_list(nullptr)
 {
     Sound::TempUISoundDisabler sound_disabler;
 
@@ -211,8 +212,8 @@ GalaxySetupPanel::GalaxySetupPanel(GG::X w, GG::Y h) :
     m_textures[SPIRAL_4] =    ClientUI::GetTexture(ClientUI::ArtDir() / "gp_spiral4.png");
     m_textures[CLUSTER] =     ClientUI::GetTexture(ClientUI::ArtDir() / "gp_cluster.png");
     m_textures[ELLIPTICAL] =  ClientUI::GetTexture(ClientUI::ArtDir() / "gp_elliptical.png");
-    m_textures[DISC] =        ClientUI::GetTexture(ClientUI::ArtDir() / "gp_elliptical.png");
-    m_textures[BOX] =         ClientUI::GetTexture(ClientUI::ArtDir() / "gp_irregular.png");
+    m_textures[DISC] =        ClientUI::GetTexture(ClientUI::ArtDir() / "gp_disc.png");
+    m_textures[BOX] =         ClientUI::GetTexture(ClientUI::ArtDir() / "gp_box.png");
     m_textures[IRREGULAR] =   ClientUI::GetTexture(ClientUI::ArtDir() / "gp_irregular.png");
     m_textures[RING] =        ClientUI::GetTexture(ClientUI::ArtDir() / "gp_ring.png");
     m_textures[RANDOM] =      ClientUI::GetTexture(ClientUI::ArtDir() / "gp_random.png");
@@ -332,7 +333,7 @@ GalaxySetupOption GalaxySetupPanel::GetNativeFrequency() const
 Aggression GalaxySetupPanel::GetAIAggression() const
 { return Aggression(m_ai_aggression_list->CurrentItemIndex()); }
 
-boost::shared_ptr<GG::Texture> GalaxySetupPanel::PreviewImage() const
+std::shared_ptr<GG::Texture> GalaxySetupPanel::PreviewImage() const
 { return m_textures[GetShape()]; }
 
 void GalaxySetupPanel::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
@@ -433,8 +434,8 @@ void GalaxySetupPanel::DoLayout() {
 }
 
 void GalaxySetupPanel::Disable(bool b/* = true*/) {
-    for (std::list<GG::Wnd*>::const_iterator it = Children().begin(); it != Children().end(); ++it)
-        static_cast<GG::Control*>(*it)->Disable(b);
+    for (GG::Wnd* child : Children())
+        static_cast<GG::Control*>(child)->Disable(b);
 }
 
 void GalaxySetupPanel::SetFromSetupData(const GalaxySetupData& setup_data) {
@@ -486,20 +487,20 @@ GalaxySetupWnd::GalaxySetupWnd() :
     CUIWnd(UserString("GSETUP_WINDOW_TITLE"),
            GG::INTERACTIVE | GG::MODAL),
     m_ended_with_ok(false),
-    m_galaxy_setup_panel(0),
-    m_player_name_label(0),
-    m_player_name_edit(0),
-    m_empire_name_label(0),
-    m_empire_name_edit(0),
-    m_empire_color_label(0),
-    m_empire_color_selector(0),
-    m_starting_secies_selector(0),
-    m_starting_species_label(0),
-    m_number_ais_label(0),
-    m_number_ais_spin(0),
-    m_preview_image(0),
-    m_ok(0),
-    m_cancel(0)
+    m_galaxy_setup_panel(nullptr),
+    m_player_name_label(nullptr),
+    m_player_name_edit(nullptr),
+    m_empire_name_label(nullptr),
+    m_empire_name_edit(nullptr),
+    m_empire_color_label(nullptr),
+    m_empire_color_selector(nullptr),
+    m_starting_secies_selector(nullptr),
+    m_starting_species_label(nullptr),
+    m_number_ais_label(nullptr),
+    m_number_ais_spin(nullptr),
+    m_preview_image(nullptr),
+    m_ok(nullptr),
+    m_cancel(nullptr)
 {
     Sound::TempUISoundDisabler sound_disabler;
 
@@ -542,11 +543,9 @@ GalaxySetupWnd::GalaxySetupWnd() :
         // if no previously-stored species selection, need to pick a default
         std::vector<std::string> selector_avail_species = m_starting_secies_selector->AvailableSpeciesNames();
         if (!selector_avail_species.empty()) {
-            for (std::vector<std::string>::const_iterator it = selector_avail_species.begin();
-                 it != selector_avail_species.end(); ++it)
-            {
+            for (const std::string& species_name : selector_avail_species) {
                 // special case: see if humans are available.
-                if ("SP_HUMAN" == *it) {
+                if ("SP_HUMAN" == species_name) {
                     default_starting_species = "SP_HUMAN";
                     break;
                 }
@@ -562,11 +561,11 @@ GalaxySetupWnd::GalaxySetupWnd() :
     m_number_ais_label = new CUILabel(UserString("GSETUP_NUMBER_AIS"), GG::FORMAT_RIGHT, GG::INTERACTIVE);
     m_number_ais_label->SetBrowseModeTime(GetOptionsDB().Get<int>("UI.tooltip-delay"));
     m_number_ais_label->SetBrowseText(UserString(GetOptionsDB().GetDescription("GameSetup.ai-players")));
-    m_number_ais_spin = new CUISpin<int>(GetOptionsDB().Get<int>("GameSetup.ai-players"), 1, 0, MAX_AI_PLAYERS, true);
+    m_number_ais_spin = new CUISpin<int>(GetOptionsDB().Get<int>("GameSetup.ai-players"), 1, 0, IApp::MAX_AI_PLAYERS(), true);
 
     // create a temporary texture and static graphic
-    boost::shared_ptr<GG::Texture> temp_tex(new GG::Texture());
-    m_preview_image =  new GG::StaticGraphic(temp_tex, GG::GRAPHIC_FITGRAPHIC); // create a blank graphic
+    static auto temp_tex = std::make_shared<GG::Texture>();
+    m_preview_image =  new GG::StaticGraphic(temp_tex, GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE); // create a blank graphic
 
     m_ok = new CUIButton(UserString("OK"));
     m_cancel = new CUIButton(UserString("CANCEL"));
@@ -619,7 +618,7 @@ void GalaxySetupWnd::Render() {
                       GG::CLR_BLACK, ClientUI::WndInnerBorderColor(), 1);
 }
 
-void GalaxySetupWnd::KeyPress(GG::Key key, boost::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys) {
+void GalaxySetupWnd::KeyPress(GG::Key key, std::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys) {
     if (!m_ok->Disabled() && (key == GG::GGK_RETURN || key == GG::GGK_KP_ENTER)) // Same behaviour as if "OK" was pressed
         OkClicked();
     else if (key == GG::GGK_ESCAPE) // Same behaviour as if "Cancel" was pressed
@@ -700,14 +699,9 @@ void GalaxySetupWnd::DoLayout() {
     m_cancel->SizeMove(button_ul, button_lr);
 }
 
-void GalaxySetupWnd::PreviewImageChanged(boost::shared_ptr<GG::Texture> new_image) {
-    if (m_preview_image) {
-        DeleteChild(m_preview_image);
-        m_preview_image = 0;
-    }
-    m_preview_image = new GG::StaticGraphic(new_image, GG::GRAPHIC_FITGRAPHIC);
-    AttachChild(m_preview_image);
-
+void GalaxySetupWnd::PreviewImageChanged(std::shared_ptr<GG::Texture> new_image) {
+    if (m_preview_image)
+        m_preview_image->SetTexture(new_image);
     DoLayout();
 }
 
