@@ -33,20 +33,23 @@ public:
     GalaxySetupOption               GetNativeFrequency() const;     //!< Returns the frequency of natives
     Aggression                      GetAIAggression() const;        //!< Returns the  maximum AI aggression level 
 
-    boost::shared_ptr<GG::Texture>  PreviewImage() const;           //!< Returns the current preview image texture
+    /** Returns the current preview image texture. */
+    std::shared_ptr<GG::Texture> PreviewImage() const;
 
     /** the settings changed signal object for this GalaxySetupPanel */
     mutable boost::signals2::signal<void ()>  SettingsChangedSignal;
 
     /** the image changed signal object for this GalaxySetupPanel */
-    mutable boost::signals2::signal<void (boost::shared_ptr<GG::Texture>)>
-                                    ImageChangedSignal;
+    mutable boost::signals2::signal<void (std::shared_ptr<GG::Texture>)> ImageChangedSignal;
     //!@}
 
     /** \name Mutators*/ //!@{
-    virtual void SizeMove(const GG::Pt& ul, const GG::Pt& lr);
-    virtual void Render() {}
-    virtual void Disable(bool b = true);
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
+
+    void Render() override
+    {}
+
+    void Disable(bool b = true) override;
 
     void SetFromSetupData(const GalaxySetupData& setup_data); ///< sets the controls from a GalaxySetupData
     void GetSetupData(GalaxySetupData& setup_data) const;     ///< fills values in \a setup_data from the panel's current state
@@ -81,8 +84,9 @@ private:
     GG::DropDownList*   m_native_freq_list;     //!< The frequency of natives
     GG::Label*          m_ai_aggression_label;
     GG::DropDownList*   m_ai_aggression_list;   //!< The max aggression choices for AI opponents
-    
-    std::vector<boost::shared_ptr<GG::Texture> > m_textures; //!< textures for galaxy previews
+
+    /** Textures for galaxy previews. */
+    std::vector<std::shared_ptr<GG::Texture>> m_textures;
 };
 
 //! This class is the Galaxy Setup window.  It is a modal window
@@ -90,7 +94,7 @@ private:
 class GalaxySetupWnd : public CUIWnd {
 public:
     /** \name Structors*/ //!@{
-    GalaxySetupWnd();   //!< default ctor
+    GalaxySetupWnd();
     //!@}
 
     /** \name Accessors*/ //!@{
@@ -106,17 +110,19 @@ public:
     //!@}
 
     /** \name Mutators*/ //!@{
-    virtual void Render();    //!< drawing code
-    virtual void KeyPress (GG::Key key, boost::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys);
-    virtual void SizeMove(const GG::Pt& ul, const GG::Pt& lr);
+    void Render() override;
+
+    void KeyPress(GG::Key key, std::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
     //!@}
 
 protected:
-    virtual GG::Rect CalculatePosition() const;
+    GG::Rect CalculatePosition() const override;
 
 private:
     void DoLayout();
-    void PreviewImageChanged(boost::shared_ptr<GG::Texture> new_image);
+    void PreviewImageChanged(std::shared_ptr<GG::Texture> new_image);
     void EmpireNameChanged(const std::string& name);
     void PlayerNameChanged(const std::string& name);
     void OkClicked();
