@@ -21,28 +21,18 @@ GraphControl::GraphControl() :
     m_x_scale_ticks(),
     m_y_scale_ticks()
 {
-    std::vector<std::pair<double, double> > test_data;
-    test_data.push_back(std::make_pair(0.0, 1.0));
-    test_data.push_back(std::make_pair(1.0, 3.2));
-    test_data.push_back(std::make_pair(4.2, -1));
-    test_data.push_back(std::make_pair(0, 0));
-    test_data.push_back(std::make_pair(-1, 1));
-    m_data.push_back(std::make_pair(test_data, GG::CLR_CYAN));
+    std::vector<std::pair<double, double>> test_data = {{0.0, 1.0}, {1.0, 3.2}, {4.2, -1}, {0, 0}, {-1, 1}};
+    m_data.push_back({test_data, GG::CLR_CYAN});
 
-    test_data.clear();
-    test_data.push_back(std::make_pair(1.0, 1.0));
-    test_data.push_back(std::make_pair(2.0, 3.2));
-    test_data.push_back(std::make_pair(3.2, -1));
-    test_data.push_back(std::make_pair(4, 0));
-    test_data.push_back(std::make_pair(5, 1));
-    m_data.push_back(std::make_pair(test_data, GG::CLR_YELLOW));
+    test_data = {{1.0, 1.0}, {2.0, 3.2}, {3.2, -1}, {4, 0}, {5, 1}};
+    m_data.push_back({test_data, GG::CLR_YELLOW});
 
     AutoSetRange();
 }
 
-void GraphControl::AddSeries(const std::vector<std::pair<double, double> >& data, const GG::Clr& clr) {
+void GraphControl::AddSeries(const std::vector<std::pair<double, double>>& data, const GG::Clr& clr) {
     if (!data.empty()) {
-        m_data.push_back(std::make_pair(data, clr));
+        m_data.push_back({data, clr});
         DoLayout();
     }
 }
@@ -101,21 +91,16 @@ void GraphControl::AutoSetRange() {
     double x_max = -99999999.9;
     double y_max = -99999999.9;
 
-    for (std::vector<std::pair<std::vector<std::pair<double, double> >, GG::Clr> >::const_iterator
-         it = m_data.begin(); it != m_data.end(); ++it)
-    {
-        const std::vector<std::pair<double, double> >& curve_pts = it->first;
-        for (std::vector<std::pair<double, double> >::const_iterator curve_it = curve_pts.begin();
-             curve_it != curve_pts.end(); ++curve_it)
-        {
-            if (curve_it->first < x_min)
-                x_min = curve_it->first;
-            if (curve_it->first > x_max)
-                x_max = curve_it->first;
-            if (curve_it->second < y_min)
-                y_min = curve_it->second;
-            if (curve_it->second > y_max)
-                y_max = curve_it->second;
+    for (const std::pair<std::vector<std::pair<double, double>>, GG::Clr>& curve : m_data) {
+        for (const std::pair<double, double>& curve_pt : curve.first) {
+            if (curve_pt.first < x_min)
+                x_min = curve_pt.first;
+            if (curve_pt.first > x_max)
+                x_max = curve_pt.first;
+            if (curve_pt.second < y_min)
+                y_min = curve_pt.second;
+            if (curve_pt.second > y_max)
+                y_max = curve_pt.second;
         }
     }
 
@@ -204,21 +189,19 @@ void GraphControl::DoLayout() {
     m_colour_buf.clear();
 
 
-    for (std::vector<std::pair<std::vector<std::pair<double, double> >, GG::Clr> >::const_iterator
-         it = m_data.begin(); it != m_data.end(); ++it)
-    {
-        const std::vector<std::pair<double, double> >& curve_pts = it->first;
+    for (const std::pair<std::vector<std::pair<double, double>>, GG::Clr>& curve : m_data) {
+        const std::vector<std::pair<double, double>>& curve_pts = curve.first;
         if (curve_pts.empty())
             continue;
 
-        GG::Clr curve_colour = it->second;
+        GG::Clr curve_colour = curve.second;
         double curve_x, curve_y;
         float screen_x, screen_y;
 
         // pairs of n and n+1 point, starting with first-second, to the second-last-last
-        std::vector<std::pair<double, double> >::const_iterator curve_it = curve_pts.begin();
+        std::vector<std::pair<double, double>>::const_iterator curve_it = curve_pts.begin();
         for (; curve_it != curve_pts.end(); ++curve_it) {
-            std::vector<std::pair<double, double> >::const_iterator next_it = curve_it;
+            std::vector<std::pair<double, double>>::const_iterator next_it = curve_it;
             ++next_it;
             if (next_it == curve_pts.end())
                 break;
