@@ -16,24 +16,11 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <GG/DrawUtil.h>
-#include <GG/MultiEdit.h>
-#include <GG/WndEvent.h>
-#include <GG/Layout.h>
-#include <GG/GUI.h>
-
-#include <algorithm>
-#include <vector>
-#include <set>
-
 #include "Hotkeys.h"
 
 #include "../util/OptionsDB.h"
-#include "../util/i18n.h"
 #include "../util/Logger.h"
 
-#include <sstream>
-#include <cctype>
 
 /// A helper class that stores both a connection and the
 /// conditions in which it should be on.
@@ -41,14 +28,14 @@ struct HotkeyManager::ConditionalConnection {
     /// Block or unblocks the connection based on condition.
     void UpdateConnection() {
         if (connection.connected()) {
-            if (!condition || condition->IsActive())
+            if (!condition || condition())
                 blocker.unblock();
             else
                 blocker.block();
         }
     };
 
-    ConditionalConnection(const boost::signals2::connection& conn, HotkeyCondition* cond) :
+    ConditionalConnection(const boost::signals2::connection& conn, std::function<bool()> cond) :
         condition(cond),
         connection(conn),
         blocker(connection)
@@ -57,7 +44,7 @@ struct HotkeyManager::ConditionalConnection {
     }
 
     /// The condition. If null, always on.
-    boost::shared_ptr<HotkeyCondition> condition;
+    std::function<bool()> condition;
 
     boost::signals2::connection connection;
     boost::signals2::shared_connection_block blocker;
@@ -66,12 +53,12 @@ struct HotkeyManager::ConditionalConnection {
 /////////////////////////////////////////////////////////
 // Hotkey
 /////////////////////////////////////////////////////////
-std::map<std::string, Hotkey>* Hotkey::s_hotkeys = 0;
+std::map<std::string, Hotkey>* Hotkey::s_hotkeys = nullptr;
 
 void Hotkey::AddHotkey(const std::string& name, const std::string& description, GG::Key key, GG::Flags<GG::ModKey> mod) {
     if (!s_hotkeys)
         s_hotkeys = new std::map<std::string, Hotkey>;
-    s_hotkeys->insert(std::make_pair(name, Hotkey(name, description, key, mod)));
+    s_hotkeys->insert({name, Hotkey(name, description, key, mod)});
 }
 
 std::string Hotkey::HotkeyToString(GG::Key key, GG::Flags<GG::ModKey> mod) {
@@ -89,9 +76,8 @@ std::string Hotkey::HotkeyToString(GG::Key key, GG::Flags<GG::ModKey> mod) {
 std::set<std::string> Hotkey::DefinedHotkeys() {
     std::set<std::string> retval;
     if (s_hotkeys) {
-        for (std::map<std::string, Hotkey>::iterator i = s_hotkeys->begin();
-             i != s_hotkeys->end(); ++i)
-        { retval.insert(i->first); }
+        for (const std::map<std::string, Hotkey>::value_type& entry : *s_hotkeys)
+        { retval.insert(entry.first); }
     }
     return retval;
 }
@@ -99,9 +85,9 @@ std::set<std::string> Hotkey::DefinedHotkeys() {
 std::string Hotkey::ToString() const
 { return HotkeyToString(m_key, m_mod_keys); }
 
-std::pair<GG::Key, GG::Flags<GG::ModKey> > Hotkey::HotkeyFromString(const std::string& str) {
+std::pair<GG::Key, GG::Flags<GG::ModKey>> Hotkey::HotkeyFromString(const std::string& str) {
     if (str.empty())
-        return std::make_pair(GG::GGK_NONE, GG::Flags<GG::ModKey>());
+        return {GG::GGK_NONE, GG::Flags<GG::ModKey>()};
 
     // Strip whitespace
     std::string copy = str;
@@ -131,7 +117,7 @@ std::pair<GG::Key, GG::Flags<GG::ModKey> > Hotkey::HotkeyFromString(const std::s
             }
         } catch (...) {
             ErrorLogger() << "Unable make flag from string: " << str;
-            return std::make_pair(GG::GGK_NONE, GG::Flags<GG::ModKey>());
+            return {GG::GGK_NONE, GG::Flags<GG::ModKey>()};
         }
     }
 
@@ -139,11 +125,11 @@ std::pair<GG::Key, GG::Flags<GG::ModKey> > Hotkey::HotkeyFromString(const std::s
     std::istringstream s(v);
     GG::Key key;
     s >> key;
-    return std::pair<GG::Key, GG::Flags<GG::ModKey> >(key, mod);
+    return std::pair<GG::Key, GG::Flags<GG::ModKey>>(key, mod);
 }
 
 void Hotkey::SetFromString(const std::string& str) {
-    std::pair<GG::Key, GG::Flags<GG::ModKey> > km = HotkeyFromString(str);
+    std::pair<GG::Key, GG::Flags<GG::ModKey>> km = HotkeyFromString(str);
     m_key = km.first;
     m_mod_keys = km.second;
 }
@@ -151,10 +137,8 @@ void Hotkey::SetFromString(const std::string& str) {
 void Hotkey::AddOptions(OptionsDB& db) {
     if (!s_hotkeys)
         return;
-    for (std::map<std::string, Hotkey>::const_iterator i = s_hotkeys->begin();
-         i != s_hotkeys->end(); ++i)
-    {
-        const Hotkey& hotkey = i->second;
+    for (const std::map<std::string, Hotkey>::value_type& entry : *s_hotkeys) {
+        const Hotkey& hotkey = entry.second;
         std::string n = "UI.hotkeys.";
         n += hotkey.m_name;
         db.Add(n, hotkey.GetDescription(),
@@ -203,10 +187,8 @@ std::string Hotkey::PrettyPrint() const
 { return PrettyPrint(m_key, m_mod_keys); }
 
 void Hotkey::ReadFromOptions(OptionsDB& db) {
-    for (std::map<std::string, Hotkey>::iterator i = s_hotkeys->begin();
-         i != s_hotkeys->end(); ++i)
-    {
-        Hotkey& hotkey = i->second;
+    for (std::map<std::string, Hotkey>::value_type& entry : *s_hotkeys) {
+        Hotkey& hotkey = entry.second;
 
         std::string options_db_name = "UI.hotkeys." + hotkey.m_name;
         if (!db.OptionExists(options_db_name)) {
@@ -215,7 +197,7 @@ void Hotkey::ReadFromOptions(OptionsDB& db) {
         }
         std::string option_string = db.Get<std::string>(options_db_name);
 
-        std::pair<GG::Key, GG::Flags<GG::ModKey> > key_modkey_pair = std::make_pair(GG::GGK_NONE, GG::MOD_KEY_NONE);
+        std::pair<GG::Key, GG::Flags<GG::ModKey>> key_modkey_pair = {GG::GGK_NONE, GG::MOD_KEY_NONE};
         try {
             key_modkey_pair = HotkeyFromString(option_string);
         } catch (...) {
@@ -275,13 +257,11 @@ Hotkey& Hotkey::PrivateNamedHotkey(const std::string& name) {
     return i->second;
 }
 
-std::map<std::string, std::set<std::string> > Hotkey::ClassifyHotkeys() {
-    std::map<std::string, std::set<std::string> > ret;
+std::map<std::string, std::set<std::string>> Hotkey::ClassifyHotkeys() {
+    std::map<std::string, std::set<std::string>> ret;
     if (s_hotkeys) {
-        for (std::map<std::string, Hotkey>::iterator i = s_hotkeys->begin();
-             i != s_hotkeys->end(); ++i)
-        {
-            const std::string& hk_name = i->first;
+        for (const std::map<std::string, Hotkey>::value_type& hotkey : *s_hotkeys) {
+            const std::string& hk_name = hotkey.first;
             std::string section = "HOTKEYS_GENERAL";
             size_t j = hk_name.find('.');
             if (j != std::string::npos) {
@@ -335,116 +315,55 @@ void Hotkey::ClearHotkey(const Hotkey& old_hotkey)
 //////////////////////////////////////////////////////////////////////
 // InvisibleWindowCondition
 //////////////////////////////////////////////////////////////////////
-InvisibleWindowCondition::InvisibleWindowCondition(const GG::Wnd* w1, const GG::Wnd* w2,
-                                                   const GG::Wnd* w3, const GG::Wnd* w4)
-{
-    m_blacklist.push_back(w1);
-    if (w2)
-        m_blacklist.push_back(w2);
-    if (w3)
-        m_blacklist.push_back(w3);
-    if (w4)
-        m_blacklist.push_back(w4);
-}
-
-InvisibleWindowCondition::InvisibleWindowCondition(const std::list<const GG::Wnd*>& bl) :
+InvisibleWindowCondition::InvisibleWindowCondition(std::initializer_list<const GG::Wnd*> bl) :
     m_blacklist(bl)
 {}
 
-bool InvisibleWindowCondition::IsActive() const {
-    for (std::list<const GG::Wnd*>::const_iterator i = m_blacklist.begin();
-         i != m_blacklist.end(); ++i)
-    {
-        if ((*i)->Visible())
+bool InvisibleWindowCondition::operator()() const {
+    for (const GG::Wnd* wnd : m_blacklist) {
+        if (wnd->Visible())
             return false;
     }
     return true;
 }
 
+
 //////////////////////////////////////////////////////////////////////
 // OrCondition
 //////////////////////////////////////////////////////////////////////
-OrCondition::OrCondition(HotkeyCondition* c1, HotkeyCondition* c2,
-                         HotkeyCondition* c3, HotkeyCondition* c4,
-                         HotkeyCondition* c5, HotkeyCondition* c6,
-                         HotkeyCondition* c7, HotkeyCondition* c8)
-{
-    m_conditions.push_back(c1);
-    m_conditions.push_back(c2);
-    if (c3)
-        m_conditions.push_back(c3);
-    if (c4)
-        m_conditions.push_back(c4);
-    if (c5)
-        m_conditions.push_back(c5);
-    if (c6)
-        m_conditions.push_back(c6);
-    if (c7)
-        m_conditions.push_back(c7);
-    if (c8)
-        m_conditions.push_back(c8);
-}
+OrCondition::OrCondition(std::initializer_list<std::function<bool()>> conditions) :
+    m_conditions(conditions)
+{}
 
-bool OrCondition::IsActive() const {
-    for (std::list<HotkeyCondition*>::const_iterator i = m_conditions.begin();
-         i != m_conditions.end(); ++i)
-    {
-        if ((*i)->IsActive())
+bool OrCondition::operator()() const {
+    for (auto cond : m_conditions) {
+        if (cond())
             return true;
     }
     return false;
 }
 
-OrCondition::~OrCondition() {
-    for (std::list<HotkeyCondition*>::iterator i = m_conditions.begin();
-         i != m_conditions.end(); ++i)
-    { delete *i; }
-}
 
 //////////////////////////////////////////////////////////////////////
 // AndCondition
 //////////////////////////////////////////////////////////////////////
-AndCondition::AndCondition(HotkeyCondition* c1, HotkeyCondition* c2,
-                           HotkeyCondition* c3, HotkeyCondition* c4,
-                           HotkeyCondition* c5, HotkeyCondition* c6,
-                           HotkeyCondition* c7, HotkeyCondition* c8)
-{
-    m_conditions.push_back(c1);
-    m_conditions.push_back(c2);
-    if (c3)
-        m_conditions.push_back(c3);
-    if (c4)
-        m_conditions.push_back(c4);
-    if (c5)
-        m_conditions.push_back(c5);
-    if (c6)
-        m_conditions.push_back(c6);
-    if (c7)
-        m_conditions.push_back(c7);
-    if (c8)
-        m_conditions.push_back(c8);
-}
+AndCondition::AndCondition(std::initializer_list<std::function<bool()>> conditions) :
+    m_conditions(conditions)
+{}
 
-bool AndCondition::IsActive() const {
-    for (std::list<HotkeyCondition*>::const_iterator i = m_conditions.begin();
-         i != m_conditions.end(); ++i)
-    {
-        if (!(*i)->IsActive())
+bool AndCondition::operator()() const {
+    for (auto cond : m_conditions) {
+        if (!cond())
             return false;
     }
     return true;
 }
 
-AndCondition::~AndCondition() {
-    for (std::list<HotkeyCondition*>::iterator i = m_conditions.begin();
-         i != m_conditions.end(); ++i)
-    { delete *i; }
-}
 
 //////////////////////////////////////////////////////////////////////
 // HotkeyManager
 //////////////////////////////////////////////////////////////////////
-HotkeyManager* HotkeyManager::s_singleton = 0;
+HotkeyManager* HotkeyManager::s_singleton = nullptr;
 
 HotkeyManager::HotkeyManager()
 {}
@@ -459,9 +378,8 @@ HotkeyManager* HotkeyManager::GetManager() {
 }
 
 void HotkeyManager::RebuildShortcuts() {
-    for (std::set<boost::signals2::connection>::iterator i = m_internal_connections.begin();
-         i != m_internal_connections.end(); ++i)
-    { i->disconnect(); }
+    for (const boost::signals2::connection& con : m_internal_connections)
+    { con.disconnect(); }
     m_internal_connections.clear();
 
     /// @todo Disable the shortcuts that we've enabled so far ? Is it
@@ -470,9 +388,8 @@ void HotkeyManager::RebuildShortcuts() {
 
     // Now, build up again all the shortcuts
     GG::GUI* gui = GG::GUI::GetGUI();
-    for (Connections::iterator i = m_connections.begin(); i != m_connections.end(); ++i)
-    {
-        const Hotkey& hk = Hotkey::NamedHotkey(i->first);
+    for (Connections::value_type& entry : m_connections) {
+        const Hotkey& hk = Hotkey::NamedHotkey(entry.first);
 
         gui->SetAccelerator(hk.m_key, hk.m_mod_keys);
 
@@ -484,7 +401,7 @@ void HotkeyManager::RebuildShortcuts() {
 
 void HotkeyManager::AddConditionalConnection(const std::string& name,
                                              const boost::signals2::connection& conn,
-                                             HotkeyCondition* cond)
+                                             std::function<bool()> cond)
 {
     ConditionalConnectionList& list = m_connections[name];
     list.push_back(ConditionalConnection(conn, cond));
