@@ -18,14 +18,13 @@
 #ifndef _Hotkey_h_
 #define _Hotkey_h_
 
-#include <GG/DrawUtil.h>
-#include <GG/MultiEdit.h>
 #include <GG/Wnd.h>
-#include <GG/WndEvent.h>
-#include <GG/Layout.h>
 #include <GG/GUI.h>
 #include <boost/signals2/shared_connection_block.hpp>
 
+#include <functional>
+#include <initializer_list>
+
 
 class OptionsDB;
 
@@ -63,7 +62,7 @@ public:
     /// Returns the names of all defined hotkeys, classified by
     /// "sections" (ie "namespace"), converted into user string by
     /// naming it HOTKEYS_uppercase)
-    static std::map<std::string, std::set<std::string> > ClassifyHotkeys();
+    static std::map<std::string, std::set<std::string>> ClassifyHotkeys();
 
     /// Returns the Hotkey of the given name, or raises an exception
     /// if there is no such hotkey.
@@ -88,7 +87,7 @@ public:
     std::string ToString() const;
 
     /// Converts a string back to the pair key/modifier
-    static std::pair<GG::Key, GG::Flags<GG::ModKey> > HotkeyFromString(const std::string& str);
+    static std::pair<GG::Key, GG::Flags<GG::ModKey>> HotkeyFromString(const std::string& str);
 
     void SetFromString(const std::string& str);
 
@@ -118,122 +117,81 @@ public:
     GG::Flags<GG::ModKey>   m_mod_keys_default;
 };
 
-/// A simple functor returning a boolean value. The children will be
-/// used to check simply if a condition is fullfilled as a prelude to
-/// activate a signal.
-///
-/// @todo Write functions that make it easier to create complex
-/// or-stuff ?
-class HotkeyCondition {
-protected:
-    friend class HotkeyManager;
-    HotkeyCondition() {}
-
-public:
-    virtual bool IsActive() const = 0;
-    virtual ~HotkeyCondition() {}
-};
 
 /// On only when no modal Wnds are open
-class NoModalWndsOpenCondition : public HotkeyCondition {
-public:
-    NoModalWndsOpenCondition() {};
-
-    virtual bool IsActive() const {
-        return !GG::GUI::GetGUI()->ModalWndsOpen();
-    };
+inline bool NoModalWndsOpenCondition() {
+    return !GG::GUI::GetGUI()->ModalWndsOpen();
 };
 
 /// On when the given window is visible
-class VisibleWindowCondition : public HotkeyCondition {
-protected:
-    const GG::Wnd* target;
-
+class VisibleWindowCondition {
 public:
     VisibleWindowCondition(const GG::Wnd* tg) :
         target(tg)
     {}
-    virtual bool IsActive() const {
-        if (!target)
-            return false;
-        return target->Visible();
+
+    bool operator()() const {
+        return target && target->Visible();
     };
+
+private:
+    const GG::Wnd* target;
 };
 
 /// On when the given windows are invisible
-class InvisibleWindowCondition : public HotkeyCondition {
-protected:
-    std::list<const GG::Wnd*> m_blacklist;
-
+class InvisibleWindowCondition {
 public:
-    InvisibleWindowCondition(const GG::Wnd* w1,     const GG::Wnd* w2 = 0,
-                             const GG::Wnd* w3 = 0, const GG::Wnd* w4 = 0);
-    InvisibleWindowCondition(const std::list<const GG::Wnd*>& bl);
+    InvisibleWindowCondition(std::initializer_list<const GG::Wnd*> bl);
 
-    virtual bool IsActive() const;
+    bool operator()() const;
+
+private:
+    std::list<const GG::Wnd*> m_blacklist;
 };
 
 /// On when the given window is visible
-class FocusWindowCondition : public HotkeyCondition {
-protected:
-    const GG::Wnd* target;
-
+class FocusWindowCondition {
 public:
     FocusWindowCondition(const GG::Wnd* tg) :
         target(tg)
     {}
-    virtual bool IsActive() const {
-        if (!target)
-            return false;
+
+    bool operator()() const {
         const GG::Wnd* foc = GG::GUI::GetGUI()->FocusWnd();
-        return target == foc;
+        return target && target == foc;
     };
+
+private:
+    const GG::Wnd* target;
 };
 
 template<class W>
-class FocusWindowIsA : public HotkeyCondition {
+class FocusWindowIsA {
 public:
-    FocusWindowIsA() {};
-
-    virtual bool IsActive() const {
+    bool operator()() const {
         const GG::Wnd* foc = GG::GUI::GetGUI()->FocusWnd();
-        //std::cout << "Focus: " << foc << std::endl;
-        if (dynamic_cast<const W*>(foc))
-            return true;
-        return false;
+        return (nullptr != dynamic_cast<const W*>(foc));
     };
 };
 
-class OrCondition : public HotkeyCondition {
-protected:
-    std::list<HotkeyCondition*> m_conditions;
+class OrCondition {
 public:
-    OrCondition(HotkeyCondition* c1, HotkeyCondition* c2,
-                HotkeyCondition* c3 = 0,
-                HotkeyCondition* c4 = 0,
-                HotkeyCondition* c5 = 0,
-                HotkeyCondition* c6 = 0,
-                HotkeyCondition* c7 = 0,
-                HotkeyCondition* c8 = 0);
+    OrCondition(std::initializer_list<std::function<bool()>> conditions);
 
-    virtual ~OrCondition();
-    virtual bool IsActive() const;
+    bool operator()() const;
+
+private:
+    std::list<std::function<bool()>> m_conditions;
 };
 
-class AndCondition : public HotkeyCondition {
-protected:
-    std::list<HotkeyCondition*> m_conditions;
+class AndCondition {
 public:
-    AndCondition(HotkeyCondition* c1, HotkeyCondition* c2,
-                 HotkeyCondition* c3 = 0,
-                 HotkeyCondition* c4 = 0,
-                 HotkeyCondition* c5 = 0,
-                 HotkeyCondition* c6 = 0,
-                 HotkeyCondition* c7 = 0,
-                 HotkeyCondition* c8 = 0);
+    AndCondition(std::initializer_list<std::function<bool()>> conditions);
 
-    virtual ~AndCondition();
-    virtual bool IsActive() const;
+    bool operator()() const;
+
+private:
+    std::list<std::function<bool()>> m_conditions;
 };
 
 class HotkeyManager {
@@ -250,11 +208,7 @@ public:
     static HotkeyManager* GetManager();
 
     /// Connects a named shortcut to the target slot in the target instance.
-    template <class T, class R>
-    void Connect(T* instance, R (T::*member)(), const std::string& name, HotkeyCondition* cond = 0)
-    { AddConditionalConnection(name, GG::Connect(NamedSignal(name), member, instance), cond); };
-
-    void Connect(boost::function<bool()> func, const std::string& name, HotkeyCondition* cond = 0)
+    void Connect(std::function<bool()> func, const std::string& name, std::function<bool()> cond = nullptr)
     { AddConditionalConnection(name, GG::Connect(NamedSignal(name), func), cond); };
 
 
@@ -271,7 +225,7 @@ private:
     /// Add the given conditional connection.
     void AddConditionalConnection(const std::string& name,
                                   const boost::signals2::connection& conn,
-                                  HotkeyCondition* cond);
+                                  std::function<bool()> cond);
 
     struct ConditionalConnection;
     typedef std::list<ConditionalConnection> ConditionalConnectionList;
