@@ -16,7 +16,6 @@
 #include <GG/Button.h>
 #include <GG/Clr.h>
 #include <GG/DrawUtil.h>
-#include <GG/utf8/checked.h>
 
 #include <boost/filesystem/operations.hpp>
 
@@ -55,8 +54,11 @@ InGameMenu::InGameMenu():
     if (!HumanClientApp::GetApp()->CanSaveNow()) {
         m_save_btn->Disable();
         m_save_btn->SetBrowseModeTime(GetOptionsDB().Get<int>("UI.tooltip-delay"));
-        m_save_btn->SetBrowseInfoWnd(boost::shared_ptr<GG::BrowseInfoWnd>(
-            new TextBrowseWnd(UserString("BUTTON_DISABLED"), UserString("SAVE_DISABLED_BROWSE_TEXT"), GG::X(400))));
+        m_save_btn->SetBrowseInfoWnd(std::make_shared<TextBrowseWnd>(
+            UserString("BUTTON_DISABLED"),
+            UserString("SAVE_DISABLED_BROWSE_TEXT"),
+            GG::X(400)
+        ));
     }
 
     ResetDefaultPosition();
@@ -124,8 +126,8 @@ void InGameMenu::DoLayout() {
     m_done_btn->SizeMove(button_ul, button_lr);
 }
 
-void InGameMenu::KeyPress (GG::Key key, boost::uint32_t key_code_point,
-                           GG::Flags<GG::ModKey> mod_keys)
+void InGameMenu::KeyPress(GG::Key key, std::uint32_t key_code_point,
+                          GG::Flags<GG::ModKey> mod_keys)
 {
     // Same behaviour as if "done" was pressed
     if (key == GG::GGK_RETURN || key == GG::GGK_KP_ENTER || key == GG::GGK_ESCAPE )
@@ -152,15 +154,11 @@ void InGameMenu::Save() {
 
         // When saving in multiplayer, you cannot see the old saves or
         // browse directories, only give a save file name.
-        if (app->SinglePlayerGame()) {
-            DebugLogger() << "... running save file dialog";
-            SaveFileDialog dlg(SAVE_GAME_EXTENSION);
-            dlg.Run();
-            filename = dlg.Result();
-        } else {
-            /// Multiplayer save. Talk to the server.
-            filename = app->SelectSaveFile();
-        }
+        DebugLogger() << "... running save file dialog";
+        SaveFileDialog dlg(app->SinglePlayerGame() ? SAVE_GAME_EXTENSION : MP_SAVE_FILE_EXTENSION);
+        dlg.Run();
+        filename = dlg.Result();
+
         if (!filename.empty()) {
             if (!app->CanSaveNow()) {
                 ErrorLogger() << "InGameMenu::Save aborting; Client app can't save now";
@@ -190,7 +188,7 @@ void InGameMenu::Options() {
 }
 
 void InGameMenu::Exit() {
-    HumanClientApp::GetApp()->EndGame();
+    HumanClientApp::GetApp()->ResetToIntro();
     CloseClicked();
 }
 
