@@ -6,13 +6,15 @@
 #include "About.h"
 #include "ClientUI.h"
 #include "CUIControls.h"
+#include "OptionsWnd.h"
+#include "EncyclopediaDetailPanel.h"
+#include "../network/Message.h"
 #include "../util/Directories.h"
 #include "../util/i18n.h"
-#include "../network/Message.h"
-#include "OptionsWnd.h"
 #include "../util/OptionsDB.h"
 #include "../util/Serialize.h"
 #include "../util/Version.h"
+#include "../util/XMLDoc.h"
 
 #include <GG/GUI.h>
 #include <GG/DrawUtil.h>
@@ -27,7 +29,7 @@
 
 namespace {
     const GG::X MAIN_MENU_WIDTH(200);
-    const GG::Y MAIN_MENU_HEIGHT(380);
+    const GG::Y MAIN_MENU_HEIGHT(450);
 
     void Options(OptionsDB& db) {
         db.AddFlag("force-external-server",             UserStringNop("OPTIONS_DB_FORCE_EXTERNAL_SERVER"),     false);
@@ -49,15 +51,20 @@ public:
     CreditsWnd(GG::X x, GG::Y y, GG::X w, GG::Y h, const XMLElement &credits, int cx, int cy, int cw, int ch, int co);
     ~CreditsWnd();
 
-    virtual void    Render();
-    virtual void    LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys)
+    void Render() override;
+
+    void LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override
     { OnExit(); }
-    virtual void    KeyPress(GG::Key key, boost::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys)
-    {
+    void MouseWheel(const GG::Pt& pt, int move, GG::Flags<GG::ModKey> mod_keys) override
+    { m_scroll_offset -= move * 2000; }
+
+    void KeyPress(GG::Key key, std::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys) override {
         if (key == GG::GGK_ESCAPE)
             OnExit();
     }
 
+    /** Handle app resizing by closing the credits window. */
+    void WindowResizedSlot(GG::X x, GG::Y y);
 
 private:
     void            DrawCredits(GG::X x1, GG::Y y1, GG::X x2, GG::Y y2, int transparency);
@@ -66,10 +73,11 @@ private:
     XMLElement                  m_credits;
     int                         m_cx, m_cy, m_cw, m_ch, m_co;
     int                         m_start_time;
+    int                         m_scroll_offset = 0;
     int                         m_render;
     int                         m_display_list_id;
     int                         m_credits_height;
-    boost::shared_ptr<GG::Font> m_font;
+    std::shared_ptr<GG::Font>   m_font;
 };
 
 CreditsWnd::CreditsWnd(GG::X x, GG::Y y, GG::X w, GG::Y h, const XMLElement &credits, int cx, int cy, int cw, int ch, int co) :
@@ -86,6 +94,9 @@ CreditsWnd::CreditsWnd(GG::X x, GG::Y y, GG::X w, GG::Y h, const XMLElement &cre
     m_credits_height(0)
 {
     m_font = ClientUI::GetFont(static_cast<int>(ClientUI::Pts()*1.3));
+
+    Connect(GG::GUI::GetGUI()->WindowResizedSignal,
+            boost::bind(&CreditsWnd::WindowResizedSlot, this, _1, _2));
 }
 
 CreditsWnd::~CreditsWnd() {
@@ -112,28 +123,54 @@ void CreditsWnd::DrawCredits(GG::X x1, GG::Y y1, GG::X x2, GG::Y y2, int transpa
     glColor(GG::Clr(transparency, transparency, transparency, 255));
 
     std::string credit;
-    for (int i = 0; i < m_credits.NumChildren(); i++) {
-        if (0 == m_credits.Child(i).Tag().compare("GROUP")) {
-            XMLElement group = m_credits.Child(i);
-            for (int j = 0; j < group.NumChildren(); j++) {
-                if (0 == group.Child(j).Tag().compare("PERSON")) {
-                    XMLElement person = group.Child(j);
-                    credit = "";
-                    if (person.ContainsAttribute("name"))
-                        credit += person.Attribute("name");
-                    if (person.ContainsAttribute("nick") && person.Attribute("nick").length() > 0) {
-                        credit += " <rgba 153 153 153 " + boost::lexical_cast<std::string>(transparency) +">(";
-                        credit += person.Attribute("nick");
+    for (const XMLElement& group : m_credits.children) {
+        if (0 == group.Tag().compare("GROUP")) {
+            for (const XMLElement& item : group.children) {
+                credit = "";
+
+                if (0 == item.Tag().compare("PERSON")) {    
+                    if (item.attributes.count("name"))
+                        credit += item.attributes.at("name");
+                    if (item.attributes.count("nick") && item.attributes.at("nick").length() > 0) {
+                        credit += " <rgba 153 153 153 " + std::to_string(transparency) +">(";
+                        credit += item.attributes.at("nick");
                         credit += ")</rgba>";
                     }
-                    if (person.ContainsAttribute("task") && person.Attribute("task").length() > 0) {
-                        credit += " - <rgba 204 204 204 " + boost::lexical_cast<std::string>(transparency) +">";
-                        credit += person.Attribute("task");
+                    if (item.attributes.count("task") && item.attributes.at("task").length() > 0) {
+                        credit += " - <rgba 204 204 204 " + std::to_string(transparency) +">";
+                        credit += item.attributes.at("task");
                         credit += "</rgba>";
                     }
-                    m_font->RenderText(GG::Pt(x1, y1 + offset), GG::Pt(x2, y2), credit, format, 0);
-                    offset += m_font->TextExtent(credit, format).y + 2;
                 }
+
+                if (0 == item.Tag().compare("RESOURCE")) {
+                    if (item.attributes.count("author"))
+                        credit += item.attributes.at("author");
+                    if (item.attributes.count("title")) {
+                        credit += "<rgba 153 153 153 " + std::to_string(transparency) + "> - ";
+                        credit += item.attributes.at("title");
+                        credit += "</rgba>\n";
+                    }
+                    if (item.attributes.count("license"))
+                        credit += UserString("INTRO_CREDITS_LICENSE") + " " + item.attributes.at("license");
+                    if (item.attributes.count("source")) {
+                        credit += "<rgba 153 153 153 " + std::to_string(transparency) + "> - ";
+                        credit += item.attributes.at("source");
+                        credit += "</rgba>\n";
+                    }
+                    if (item.attributes.count("notes") && item.attributes.at("notes").length() > 0) {
+                        credit += "<rgba 204 204 204 " + std::to_string(transparency) + ">(";
+                        credit += item.attributes.at("notes");
+                        credit += ")</rgba>";
+                    }
+                }
+
+                std::vector<std::shared_ptr<GG::Font::TextElement>> text_elements =
+                    m_font->ExpensiveParseFromTextToTextElements(credit, format);
+                std::vector<GG::Font::LineData> lines =
+                    m_font->DetermineLines(credit, format, x2 - x1, text_elements);
+                m_font->RenderText(GG::Pt(x1, y1 + offset), GG::Pt(x2, y2), credit, format, lines);
+                offset += m_font->TextExtent(lines).y + 2;
             }
             offset += m_font->Lineskip() + 2;
         }
@@ -154,7 +191,7 @@ void CreditsWnd::Render() {
         glEndList();
     }
     //time passed
-    int ticks_delta = GG::GUI::GetGUI()->Ticks() - m_start_time;
+    int ticks_delta = GG::GUI::GetGUI()->Ticks() - m_start_time + m_scroll_offset;
 
     //draw background
     GG::FlatRectangle(ul, lr, GG::FloatClr(0.0f, 0.0f, 0.0f, 0.5f), GG::CLR_ZERO, 0);
@@ -186,25 +223,28 @@ void CreditsWnd::Render() {
         OnExit();
 }
 
+void CreditsWnd::WindowResizedSlot(GG::X x, GG::Y y)
+{ OnExit(); }
 
 /////////////////////////////////
 // IntroScreen
 /////////////////////////////////
 IntroScreen::IntroScreen() :
     GG::Wnd(GG::X0, GG::Y0, GG::GUI::GetGUI()->AppWidth(), GG::GUI::GetGUI()->AppHeight(), GG::NO_WND_FLAGS),
-    m_single_player(0),
-    m_quick_start(0),
-    m_multi_player(0),
-    m_load_game(0),
-    m_options(0),
-    m_about(0),
-    m_website(0),
-    m_credits(0),
-    m_exit_game(0),
-    m_menu(0),
-    m_splash(0),
-    m_logo(0),
-    m_version(0)
+    m_single_player(nullptr),
+    m_quick_start(nullptr),
+    m_multi_player(nullptr),
+    m_load_game(nullptr),
+    m_options(nullptr),
+    m_pedia(nullptr),
+    m_about(nullptr),
+    m_website(nullptr),
+    m_credits(nullptr),
+    m_exit_game(nullptr),
+    m_menu(nullptr),
+    m_splash(nullptr),
+    m_logo(nullptr),
+    m_version(nullptr)
 {
     m_menu = new CUIWnd(UserString("INTRO_WINDOW_TITLE"), GG::X1, GG::Y1,
                         MAIN_MENU_WIDTH, MAIN_MENU_HEIGHT, GG::ONTOP | GG::INTERACTIVE);
@@ -227,6 +267,7 @@ IntroScreen::IntroScreen() :
     m_multi_player =  new CUIButton(UserString("INTRO_BTN_MULTI_PLAYER"));
     m_load_game =     new CUIButton(UserString("INTRO_BTN_LOAD_GAME"));
     m_options =       new CUIButton(UserString("INTRO_BTN_OPTIONS"));
+    m_pedia =         new CUIButton(UserString("INTRO_BTN_PEDIA"));
     m_about =         new CUIButton(UserString("INTRO_BTN_ABOUT"));
     m_website =       new CUIButton(UserString("INTRO_BTN_WEBSITE"));
     m_credits =       new CUIButton(UserString("INTRO_BTN_CREDITS"));
@@ -238,6 +279,7 @@ IntroScreen::IntroScreen() :
     m_menu->AttachChild(m_multi_player);
     m_menu->AttachChild(m_load_game);
     m_menu->AttachChild(m_options);
+    m_menu->AttachChild(m_pedia);
     m_menu->AttachChild(m_about);
     m_menu->AttachChild(m_website);
     m_menu->AttachChild(m_credits);
@@ -249,6 +291,7 @@ IntroScreen::IntroScreen() :
     GG::Connect(m_multi_player->LeftClickedSignal,  &IntroScreen::OnMultiPlayer,    this);
     GG::Connect(m_load_game->LeftClickedSignal,     &IntroScreen::OnLoadGame,       this);
     GG::Connect(m_options->LeftClickedSignal,       &IntroScreen::OnOptions,        this);
+    GG::Connect(m_pedia->LeftClickedSignal,         &IntroScreen::OnPedia,          this);
     GG::Connect(m_about->LeftClickedSignal,         &IntroScreen::OnAbout,          this);
     GG::Connect(m_website->LeftClickedSignal,       &IntroScreen::OnWebsite,        this);
     GG::Connect(m_credits->LeftClickedSignal,       &IntroScreen::OnCredits,        this);
@@ -283,6 +326,20 @@ void IntroScreen::OnOptions() {
     options_wnd.Run();
 }
 
+void IntroScreen::OnPedia() {
+    static const std::string INTRO_PEDIA_WND_NAME = "introscreen.pedia";
+    EncyclopediaDetailPanel enc_panel(GG::MODAL | GG::INTERACTIVE | GG::DRAGABLE |
+                                      GG::RESIZABLE | CLOSABLE | PINABLE, INTRO_PEDIA_WND_NAME);
+    enc_panel.SizeMove(GG::Pt(GG::X(100), GG::Y(100)), Size() - GG::Pt(GG::X(100), GG::Y(100)));
+    enc_panel.ClearItems();
+    enc_panel.SetIndex();
+    enc_panel.ValidatePosition();
+
+    GG::Connect(enc_panel.ClosingSignal, &EncyclopediaDetailPanel::EndRun, &enc_panel);
+
+    enc_panel.Run();
+}
+
 void IntroScreen::OnAbout() {
     About about_wnd;
     about_wnd.Run();
@@ -321,7 +378,7 @@ void IntroScreen::OnExitGame() {
     GG::GUI::GetGUI()->Exit(0);
 }
 
-void IntroScreen::KeyPress(GG::Key key, boost::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys) {
+void IntroScreen::KeyPress(GG::Key key, std::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys) {
     if (key == GG::GGK_ESCAPE)
         OnExitGame();
 }
@@ -353,6 +410,7 @@ void IntroScreen::DoLayout() {
     button_width = std::max(button_width, m_multi_player->MinUsableSize().x);
     button_width = std::max(button_width, m_load_game->MinUsableSize().x);
     button_width = std::max(button_width, m_options->MinUsableSize().x);
+    button_width = std::max(button_width, m_pedia->MinUsableSize().x);
     button_width = std::max(button_width, m_about->MinUsableSize().x);
     button_width = std::max(button_width, m_website->MinUsableSize().x);
     button_width = std::max(button_width, m_credits->MinUsableSize().x);
@@ -362,8 +420,8 @@ void IntroScreen::DoLayout() {
     //calculate  necessary button height
     button_cell_height = std::max(MIN_BUTTON_HEIGHT, m_exit_game->MinUsableSize().y);
     //culate window width and height
-    mainmenu_width  =        button_width  + H_MAINMENU_MARGIN;
-    mainmenu_height = 9.75 * button_cell_height + V_MAINMENU_MARGIN; // 8 rows + 0.75 before exit button
+    mainmenu_width  =         button_width  + H_MAINMENU_MARGIN;
+    mainmenu_height = 10.75 * button_cell_height + V_MAINMENU_MARGIN; // 10 rows + 0.75 before exit button
 
     // place buttons
     GG::Pt button_ul(GG::X(15), GG::Y(12));
@@ -386,6 +444,9 @@ void IntroScreen::DoLayout() {
     m_options->SizeMove(button_ul, button_lr);
     button_ul.y += GG::Y(button_cell_height);
     button_lr.y += GG::Y(button_cell_height);
+    m_pedia->SizeMove(button_ul, button_lr);
+    button_ul.y += GG::Y(button_cell_height);
+    button_lr.y += GG::Y(button_cell_height);
     m_about->SizeMove(button_ul, button_lr);
     button_ul.y += GG::Y(button_cell_height);
     button_lr.y += GG::Y(button_cell_height);
