@@ -1,6 +1,7 @@
 #include "LinkText.h"
 
 #include "ClientUI.h"
+#include "CUIControls.h"
 #include "../Empire/Empire.h"
 #include "../universe/UniverseObject.h"
 #include "../util/AppInterface.h"
@@ -10,7 +11,6 @@
 
 #include <GG/DrawUtil.h>
 #include <GG/WndEvent.h>
-#include <GG/Menu.h>
 #include <GG/GUI.h>
 
 namespace {
@@ -18,44 +18,12 @@ namespace {
 
     // closing format tag
     static const std::string LINK_FORMAT_CLOSE = "</rgba>";
-
-    static bool link_tags_registered = false;
-    void RegisterLinkTags() {
-        if (link_tags_registered)
-            return;
-        link_tags_registered = true;
-
-        // need to register the tags that link text uses so GG::Font will know how to (not) render them
-        GG::Font::RegisterKnownTag(VarText::PLANET_ID_TAG);
-        GG::Font::RegisterKnownTag(VarText::SYSTEM_ID_TAG);
-        GG::Font::RegisterKnownTag(VarText::SHIP_ID_TAG);
-        GG::Font::RegisterKnownTag(VarText::FLEET_ID_TAG);
-        GG::Font::RegisterKnownTag(VarText::BUILDING_ID_TAG);
-        GG::Font::RegisterKnownTag(VarText::FIELD_ID_TAG);
-
-        GG::Font::RegisterKnownTag(VarText::COMBAT_ID_TAG);
-
-        GG::Font::RegisterKnownTag(VarText::EMPIRE_ID_TAG);
-        GG::Font::RegisterKnownTag(VarText::DESIGN_ID_TAG);
-        GG::Font::RegisterKnownTag(VarText::PREDEFINED_DESIGN_TAG);
-
-        GG::Font::RegisterKnownTag(VarText::TECH_TAG);
-        GG::Font::RegisterKnownTag(VarText::BUILDING_TYPE_TAG);
-        GG::Font::RegisterKnownTag(VarText::SPECIAL_TAG);
-        GG::Font::RegisterKnownTag(VarText::SHIP_HULL_TAG);
-        GG::Font::RegisterKnownTag(VarText::SHIP_PART_TAG);
-        GG::Font::RegisterKnownTag(VarText::SPECIES_TAG);
-        GG::Font::RegisterKnownTag(VarText::FIELD_TYPE_TAG);
-
-        GG::Font::RegisterKnownTag(TextLinker::ENCYCLOPEDIA_TAG);
-        GG::Font::RegisterKnownTag(TextLinker::GRAPH_TAG);
-    }
 }
 
 ///////////////////////////////////////
 // LinkText
 ///////////////////////////////////////
-LinkText::LinkText(GG::X x, GG::Y y, GG::X w, const std::string& str, const boost::shared_ptr<GG::Font>& font,
+LinkText::LinkText(GG::X x, GG::Y y, GG::X w, const std::string& str, const std::shared_ptr<GG::Font>& font,
                    GG::Flags<GG::TextFormat> format/* = GG::FORMAT_NONE*/, GG::Clr color/* = GG::CLR_BLACK*/) :
     GG::TextControl(x, y, w, GG::Y1, str, font, color, format, GG::INTERACTIVE),
     TextLinker(),
@@ -66,7 +34,7 @@ LinkText::LinkText(GG::X x, GG::Y y, GG::X w, const std::string& str, const boos
     MarkLinks();
 }
 
-LinkText::LinkText(GG::X x, GG::Y y, const std::string& str, const boost::shared_ptr<GG::Font>& font, 
+LinkText::LinkText(GG::X x, GG::Y y, const std::string& str, const std::shared_ptr<GG::Font>& font,
                    GG::Clr color/* = GG::CLR_BLACK*/) :
     GG::TextControl(x, y, GG::X1, GG::Y1, str, font, color, GG::FORMAT_NOWRAP, GG::INTERACTIVE),
     TextLinker(),
@@ -100,7 +68,7 @@ GG::Pt LinkText::TextLowerRight() const
 const std::vector<GG::Font::LineData>& LinkText::GetLineData() const
 { return GG::TextControl::GetLineData(); }
 
-const boost::shared_ptr<GG::Font>& LinkText::GetFont() const
+const std::shared_ptr<GG::Font>& LinkText::GetFont() const
 { return GG::TextControl::GetFont(); }
 
 const std::string& LinkText::RawText() const
@@ -118,8 +86,7 @@ void LinkText::RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) {
     }
     menu_contents.next_level.push_back(GG::MenuItem(UserString("HOTKEY_COPY"),  2, false, false));
 
-    GG::PopupMenu popup(pt.x, pt.y, ClientUI::GetFont(), menu_contents, ClientUI::TextColor(),
-                        ClientUI::WndOuterBorderColor(), ClientUI::WndColor(), ClientUI::EditHiliteColor());
+    CUIPopupMenu popup(pt.x, pt.y, menu_contents);
     if (popup.Run()) {
         switch (popup.MenuID()) {
         case 9: {
@@ -177,10 +144,10 @@ int LinkDecorator::CastStringToInt(const std::string& str) {
 
 std::string ColorByOwner::Decorate(const std::string& object_id_str, const std::string& content) const {
     GG::Clr color = ClientUI::DefaultLinkColor();
-    const Empire* empire = 0;
+    const Empire* empire = nullptr;
     // get object indicated by object_id, and then get object's owner, if any
     int object_id = CastStringToInt(object_id_str);
-    TemporaryPtr<const UniverseObject> object = Objects().Object(object_id);
+    std::shared_ptr<const UniverseObject> object = Objects().Object(object_id);
     if (object && !object->Unowned())
         empire = GetEmpire(object->Owner());
     if (empire)
@@ -207,6 +174,7 @@ struct TextLinker::Link {
 // static(s)
 const std::string TextLinker::ENCYCLOPEDIA_TAG("encyclopedia");
 const std::string TextLinker::GRAPH_TAG("graph");
+const std::string TextLinker::URL_TAG("url");
 
 TextLinker::TextLinker() :
     m_links(),
@@ -219,7 +187,7 @@ TextLinker::~TextLinker()
 {}
 
 void TextLinker::SetDecorator(const std::string& link_type, LinkDecorator* decorator) {
-    m_decorators[link_type] = boost::shared_ptr<LinkDecorator>(decorator);
+    m_decorators[link_type] = std::shared_ptr<LinkDecorator>(decorator);
     MarkLinks();
 }
 
@@ -253,9 +221,9 @@ void TextLinker::Render_() {
     FlatRectangle(bounds.ul, bounds.lr, GG::CLR_ZERO, GG::CLR_YELLOW, 1);
 
     // draw red box around individual linkified bits of text within block
-    for (unsigned int i = 0; i < m_links.size(); ++i) {
-        for (unsigned int j = 0; j < m_links[i].rects.size(); ++j) {
-            GG::Rect r = TextUpperLeft() + m_links[i].rects[j];
+    for (const Link& link : m_links) {
+        for (const GG::Rect& rect : link.rects) {
+            GG::Rect r = TextUpperLeft() + rect;
             FlatRectangle(r.ul, r.lr, GG::CLR_ZERO, GG::CLR_RED, 1);
         }
     }
@@ -324,8 +292,8 @@ const std::vector<GG::Font::LineData>& TextLinker::GetLineData() const {
     return retval;
 }
 
-const boost::shared_ptr<GG::Font>& TextLinker::GetFont() const {
-    static boost::shared_ptr<GG::Font> retval;
+const std::shared_ptr<GG::Font>& TextLinker::GetFont() const {
+    static std::shared_ptr<GG::Font> retval;
     if (!retval)
         retval = ClientUI::GetFont();
     return retval;
@@ -348,19 +316,12 @@ const std::string& TextLinker::RawText() const {
 void TextLinker::FindLinks() {
     m_links.clear();
 
-    const std::vector<GG::Font::LineData>& line_data = GetLineData();
-
     GG::Y y_posn(0); // y-coordinate of the top of the current line
     Link link;
 
-    for (std::vector<GG::Font::LineData>::const_iterator it = line_data.begin(); 
-         it != line_data.end(); ++it)
-    {
-        const GG::Font::LineData& curr_line = *it;
-
-        for (unsigned int i = 0; i < curr_line.char_data.size(); ++i) {
-            for (unsigned int j = 0; j < curr_line.char_data[i].tags.size(); ++j) {
-                const boost::shared_ptr<GG::Font::FormattingTag>& tag = curr_line.char_data[i].tags[j];
+    for (const GG::Font::LineData& curr_line : GetLineData()) {
+        for (const GG::Font::LineData::CharData& curr_char : curr_line.char_data) {
+            for (const std::shared_ptr<GG::Font::FormattingTag>& tag : curr_char.tags) {
                 if (tag->tag_name == VarText::PLANET_ID_TAG ||
                     tag->tag_name == VarText::SYSTEM_ID_TAG ||
                     tag->tag_name == VarText::SHIP_ID_TAG ||
@@ -378,19 +339,24 @@ void TextLinker::FindLinks() {
                     tag->tag_name == VarText::SHIP_PART_TAG ||
                     tag->tag_name == VarText::SPECIES_TAG ||
                     tag->tag_name == VarText::FIELD_TYPE_TAG ||
+                    tag->tag_name == VarText::METER_TYPE_TAG ||
                     tag->tag_name == TextLinker::ENCYCLOPEDIA_TAG ||
-                    tag->tag_name == TextLinker::GRAPH_TAG)
+                    tag->tag_name == TextLinker::GRAPH_TAG ||
+                    tag->tag_name == TextLinker::URL_TAG)
                 {
                     link.type = tag->tag_name;
                     if (tag->close_tag) {
-                        link.text_posn.second = Value(curr_line.char_data[i].string_index);
+                        link.text_posn.second = Value(curr_char.string_index);
                         m_links.push_back(link);
                         link = Link();
                     } else {
-                        link.data = tag->params[0];
-                        link.text_posn.first = Value(curr_line.char_data[i].string_index);
-                        for (unsigned int k = 0; k < curr_line.char_data[i].tags.size(); ++k) {
-                            link.text_posn.first -= Value(curr_line.char_data[i].tags[k]->StringSize());
+                        if (!tag->params.empty())
+                            link.data = tag->params[0];
+                        else
+                            link.data.clear();
+                        link.text_posn.first = Value(curr_char.string_index);
+                        for (std::shared_ptr<GG::Font::FormattingTag> itag : curr_char.tags) {
+                            link.text_posn.first -= Value(itag->StringSize());
                         }
                     }
                     // Before decoration, the real positions are the same as the raw ones
@@ -404,8 +370,7 @@ void TextLinker::FindLinks() {
 }
 
 void TextLinker::LocateLinks() {
-    const std::vector<GG::Font::LineData>& line_data = GetLineData();
-    const boost::shared_ptr<GG::Font>& font = GetFont();
+    const std::shared_ptr<GG::Font>& font = GetFont();
 
     if (m_links.empty())
         return;
@@ -417,11 +382,7 @@ void TextLinker::LocateLinks() {
     std::vector<Link>::iterator current_link = m_links.begin();
     bool inside_link = false;
 
-    for (std::vector<GG::Font::LineData>::const_iterator it = line_data.begin(); 
-         it != line_data.end(); ++it, y_posn += font->Lineskip())
-    {
-        const GG::Font::LineData& curr_line = *it;
-
+    for (const GG::Font::LineData& curr_line : GetLineData()) {
         // if the last line ended without the current tag ending
         if (inside_link)
             current_link->rects.push_back(GG::Rect(GG::X0, y_posn, GG::X0, y_posn + font->Height()));
@@ -449,6 +410,8 @@ void TextLinker::LocateLinks() {
         // if a line is ending without the current tag ending
         if (inside_link)
             current_link->rects.back().lr.x = curr_line.char_data.empty() ? GG::X0 : curr_line.char_data.back().extent;
+
+        y_posn += font->Lineskip();
     }
 }
 
@@ -466,8 +429,7 @@ int TextLinker::GetLinkUnderPt(const GG::Pt& pt) {
     for (unsigned int i = 0; i < links.size(); ++i) {
         const Link& link = links[i];
 
-        for (unsigned int j = 0; j < link.rects.size(); ++j) {
-            GG::Rect link_rect = link.rects[j];
+        for (const GG::Rect& link_rect : link.rects) {
             GG::Rect r = tex_ul + link_rect;
             if (r.Contains(pt))
                 return i;
@@ -532,7 +494,44 @@ std::string LinkTaggedText(const std::string& tag, const std::string& stringtabl
 { return "<" + tag + " " + stringtable_entry + ">" + UserString(stringtable_entry) + "</" + tag + ">"; }
 
 std::string LinkTaggedIDText(const std::string& tag, int id, const std::string& text)
-{ return "<" + tag + " " + boost::lexical_cast<std::string>(id) + ">" + text + "</" + tag + ">"; }
+{ return "<" + tag + " " + std::to_string(id) + ">" + text + "</" + tag + ">"; }
 
 std::string LinkTaggedPresetText(const std::string& tag, const std::string& stringtable_entry, const std::string& display_text)
 { return "<" + tag + " " + stringtable_entry + ">" + display_text + "</" + tag + ">"; }
+
+namespace {
+    static bool link_tags_registered = false;
+}
+
+void RegisterLinkTags() {
+    if (link_tags_registered)
+        return;
+    link_tags_registered = true;
+
+    // need to register the tags that link text uses so GG::Font will know how to (not) render them
+    GG::Font::RegisterKnownTag(VarText::PLANET_ID_TAG);
+    GG::Font::RegisterKnownTag(VarText::SYSTEM_ID_TAG);
+    GG::Font::RegisterKnownTag(VarText::SHIP_ID_TAG);
+    GG::Font::RegisterKnownTag(VarText::FLEET_ID_TAG);
+    GG::Font::RegisterKnownTag(VarText::BUILDING_ID_TAG);
+    GG::Font::RegisterKnownTag(VarText::FIELD_ID_TAG);
+
+    GG::Font::RegisterKnownTag(VarText::COMBAT_ID_TAG);
+
+    GG::Font::RegisterKnownTag(VarText::EMPIRE_ID_TAG);
+    GG::Font::RegisterKnownTag(VarText::DESIGN_ID_TAG);
+    GG::Font::RegisterKnownTag(VarText::PREDEFINED_DESIGN_TAG);
+
+    GG::Font::RegisterKnownTag(VarText::TECH_TAG);
+    GG::Font::RegisterKnownTag(VarText::BUILDING_TYPE_TAG);
+    GG::Font::RegisterKnownTag(VarText::SPECIAL_TAG);
+    GG::Font::RegisterKnownTag(VarText::SHIP_HULL_TAG);
+    GG::Font::RegisterKnownTag(VarText::SHIP_PART_TAG);
+    GG::Font::RegisterKnownTag(VarText::SPECIES_TAG);
+    GG::Font::RegisterKnownTag(VarText::FIELD_TYPE_TAG);
+    GG::Font::RegisterKnownTag(VarText::METER_TYPE_TAG);
+
+    GG::Font::RegisterKnownTag(TextLinker::ENCYCLOPEDIA_TAG);
+    GG::Font::RegisterKnownTag(TextLinker::GRAPH_TAG);
+    GG::Font::RegisterKnownTag(TextLinker::URL_TAG);
+}
