@@ -10,6 +10,7 @@ class LinkDecorator {
 public:
     /** \name Structors */ //@{
     LinkDecorator() {};
+    virtual ~LinkDecorator() {};
     //@}
 
     /// Gets called for each link of the type this decorator is assigned to.
@@ -36,11 +37,11 @@ protected:
 };
 
 // Should be unique_ptr, but we don't have c++11
-typedef boost::shared_ptr<LinkDecorator> LinkDecoratorPtr;
+typedef std::shared_ptr<LinkDecorator> LinkDecoratorPtr;
 
 class ColorByOwner: public LinkDecorator {
 public:
-    virtual std::string Decorate(const std::string& object_id_str, const std::string& content) const;
+    std::string Decorate(const std::string& object_id_str, const std::string& content) const override;
 };
 
 class TextLinker {
@@ -62,6 +63,7 @@ public:
 
     static const std::string ENCYCLOPEDIA_TAG;
     static const std::string GRAPH_TAG;
+    static const std::string URL_TAG;
 
 protected:
     void        Render_();
@@ -72,7 +74,9 @@ protected:
     void        MouseLeave_();
 
     virtual const std::vector<GG::Font::LineData>&  GetLineData() const = 0;
-    virtual const boost::shared_ptr<GG::Font>&      GetFont() const = 0;
+
+    virtual const std::shared_ptr<GG::Font>& GetFont() const = 0;
+
     virtual GG::Pt                                  TextUpperLeft() const = 0;
     virtual GG::Pt                                  TextLowerRight() const = 0;
     virtual void                                    SetLinkedText(const std::string& str) = 0;
@@ -123,40 +127,48 @@ private:
 class LinkText : public GG::TextControl, public TextLinker {
 public:
     /** \name Structors */ //@{
-    LinkText(GG::X x, GG::Y y, GG::X w, const std::string& str, const boost::shared_ptr<GG::Font>& font, GG::Flags<GG::TextFormat> format = GG::FORMAT_NONE, GG::Clr color = GG::CLR_BLACK); ///< ctor taking a font directly
+    LinkText(GG::X x, GG::Y y, GG::X w, const std::string& str, const std::shared_ptr<GG::Font>& font, GG::Flags<GG::TextFormat> format = GG::FORMAT_NONE, GG::Clr color = GG::CLR_BLACK); ///< ctor taking a font directly
 
     /** ctor that does not require window size.
         Window size is determined from the string and font; the window will be large enough to fit the text as rendered,
         and no larger.  \see DynamicText::DynamicText() */
-    LinkText(GG::X x, GG::Y y, const std::string& str, const boost::shared_ptr<GG::Font>& font, GG::Clr color = GG::CLR_BLACK);
+    LinkText(GG::X x, GG::Y y, const std::string& str, const std::shared_ptr<GG::Font>& font, GG::Clr color = GG::CLR_BLACK);
     //@}
 
     /** \name Accessors */ //@{
-    virtual GG::Pt  TextUpperLeft() const;
-    virtual GG::Pt  TextLowerRight() const;
+    GG::Pt TextUpperLeft() const override;
 
-    virtual const std::vector<GG::Font::LineData>&  GetLineData() const;
-    virtual const boost::shared_ptr<GG::Font>&      GetFont() const;
-    virtual const std::string&                      RawText() const;          ///< returns text displayed before link formatting is added
+    GG::Pt TextLowerRight() const override;
+
+    const std::vector<GG::Font::LineData>& GetLineData() const override;
+
+    const std::shared_ptr<GG::Font>& GetFont() const override;
+
+    /** Returns text displayed before link formatting is added. */
+    const std::string& RawText() const override;
     //@}
 
     /** \name Mutators */ //@{
-    virtual void    Render();
-    virtual void    LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    MouseHere(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    MouseLeave();
-    virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr);
+    void Render() override;
+
+    void LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void MouseHere(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+    void MouseLeave() override;
+
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
 
     /** sets the text to \a str; may resize the window.  If the window was
         constructed to fit the size of the text (i.e. if the second ctor type
         was used), calls to this function cause the window to be resized to
         whatever space the newly rendered text occupies. */
-    virtual void    SetText(const std::string& str);
+    void SetText(const std::string& str) override;
     //@}
 
 private:
-    virtual void    SetLinkedText(const std::string& str);
+    void SetLinkedText(const std::string& str) override;
 
     std::string     m_raw_text;
 };
@@ -170,4 +182,10 @@ std::string LinkTaggedIDText(const std::string& tag, int id, const std::string&
 
 /// Helper for generating a link string with preset display text (not to be looked up in stringtable)
 std::string LinkTaggedPresetText(const std::string& tag, const std::string& stringtable_entry, const std::string& display_text);
+
+/// Free function to register link tags that TextLinker knows of.  This allows GG::Font to remove
+/// them so that they will not be rendered.  Must be called at least once before text with embedded
+/// XML tags is handled by GG::Font
+void RegisterLinkTags();
+
 #endif // _LinkText_h_
