@@ -1,19 +1,21 @@
 #ifndef _MapWnd_h_
 #define _MapWnd_h_
 
-#include <vector>
 #include <GG/GGFwd.h>
 #include <GG/GLClientAndServerBuffer.h>
 
 #include "CUIWnd.h"
 #include "CUISlider.h"
-#include "../universe/Enums.h"
+#include "../universe/EnumsFwd.h"
 #include "../universe/Fleet.h"
 #include "FleetButton.h"
 
-#include <boost/unordered_set.hpp>
 #include <boost/unordered_map.hpp>
 
+#include <unordered_set>
+#include <vector>
+
+
 class FleetWnd;
 class MapWndPopup;
 class DesignWnd;
@@ -40,6 +42,12 @@ class ShaderProgram;
  * positioning fleet buttons that are moving along the starlane. */
 struct LaneEndpoints {
     LaneEndpoints();
+    LaneEndpoints(float x1, float y1, float x2, float y2) :
+        X1(x1),
+        Y1(y1),
+        X2(x2),
+        Y2(y2)
+    {}
     float X1, Y1, X2, Y2;
 };
 
@@ -48,12 +56,13 @@ struct LaneEndpoints {
 class MapWnd : public GG::Wnd {
 public:
     //! \name Structors //!@{
-    MapWnd();    //!< default ctor
-    ~MapWnd();   //!< default dtor
+    MapWnd();
+
+    ~MapWnd();
     //!@}
 
     //! \name Accessors //!@{
-    virtual GG::Pt              ClientUpperLeft() const;
+    GG::Pt ClientUpperLeft() const override;
 
     double                      ZoomFactor() const;
     int                         SystemIconSize() const;
@@ -105,15 +114,25 @@ public:
     //!@}
 
     //! \name Mutators //!@{
-    virtual void    Render();
-    virtual void    LButtonDown(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    LDrag(const GG::Pt& pt, const GG::Pt& move, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    LButtonUp(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    MouseWheel(const GG::Pt& pt, int move, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    KeyPress(GG::Key key, boost::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    KeyRelease(GG::Key key, boost::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys);
+    void PreRender() override;
+
+    void Render() override;
+
+    void LButtonDown(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void LDrag(const GG::Pt& pt, const GG::Pt& move, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void LButtonUp(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void MouseWheel(const GG::Pt& pt, int move, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void KeyPress(GG::Key key, std::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void KeyRelease(GG::Key key, std::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys) override;
 
     void            DoLayout();
 
@@ -136,7 +155,9 @@ public:
 
     void            CenterOnMapCoord(double x, double y);                   //!< centers the map on map position (x, y)
     void            CenterOnObject(int id);                                 //!< centers the map on object with id \a id
-    void            CenterOnObject(TemporaryPtr<const UniverseObject> obj);              //!< centers the map on object \a id
+
+    /** Centers the map on object \a id. */
+    void CenterOnObject(std::shared_ptr<const UniverseObject> obj);
 
     void            ShowPlanet(int planet_id);                              //!< brings up encyclopedia panel and displays info about the planet
     void            ShowCombatLog(int log_id);                              //!< brings up encyclopedia panel and displays info about the combat
@@ -149,13 +170,17 @@ public:
     void            ShowSpecies(const std::string& species_name);           //!< brings up encyclopedia panel and displays info about the species with name \a species_name
     void            ShowFieldType(const std::string& field_type_name);      //!< brings up encyclopedia panel and displays info about the field type with name \a field_type_name
     void            ShowEmpire(int empire_id);                              //!< brings up encyclopedia panel and displays info about the empire with id \a empire_id
+    void            ShowMeterTypeArticle(const std::string& meter_string);  //!< brings up encyclopedia panel and displays info about the MeterType @a meter_type
     void            ShowEncyclopediaEntry(const std::string& str);          //!< brings up encyclopedia panel and displays info about the specified string \a str
 
     void            SelectSystem(int systemID);                             //!< programatically selects systems on map, sidepanel, and production screen.  catches signals from these when the user changes the selected system
     void            ReselectLastSystem();                                   //!< re-selects the most recently selected system, if a valid one exists
     void            SelectPlanet(int planetID);                             //!< programatically selects planets on sidepanels.  catches signals from production wnd or sidepanel for when the user changes the selected planet
     void            SelectFleet(int fleetID);                               //!< programatically selects fleets by ID
-    void            SelectFleet(TemporaryPtr<Fleet> fleet);                              //!< programatically selects fleets
+
+    /** Programatically selects fleets. */
+    void SelectFleet(std::shared_ptr<Fleet> fleet);
+
     void            ReselectLastFleet();                                    //!< re-selects the most recent selected fleet, if a valid one exists
 
     void            RemoveFleet(int fleet_id); //!< removes specified fleet.
@@ -184,8 +209,6 @@ public:
     bool            IsFleetExploring(const int fleet_id);
     void            DispatchFleetsExploring();                      //!< called at each turn begin and when a fleet start/stop exploring to redispatch everyone.
 
-protected:
-    virtual bool    EventFilter(GG::Wnd* w, const GG::WndEvent& event);
 
 private:
     void            RefreshTradeResourceIndicator();
@@ -235,8 +258,8 @@ private:
         type_fleet_buttons and record the fleet buttons in \p m_fleet_buttons.*/
     template <typename K>
     void            CreateFleetButtonsOfType(
-        boost::unordered_map<K, boost::unordered_set<FleetButton*> >& type_fleet_buttons,
-        const boost::unordered_map<std::pair<K, int>, std::vector<int> > &fleets_map,
+        boost::unordered_map<K, std::unordered_set<FleetButton*>>& type_fleet_buttons,
+        const boost::unordered_map<std::pair<K, int>, std::vector<int>> &fleets_map,
         const FleetButton::SizeType& fleet_button_size);
 
     /** Delete all fleet buttons.*/
@@ -245,17 +268,27 @@ private:
     void            RefreshFleetButtonSelectionIndicators();    //!< marks (only) selected fleets' buttons as selected
 
     /** Connect all \p fleets StateChangedSignal to RefreshFleetButtons. */
-    void            AddFleetsStateChangedSignal(const std::vector<TemporaryPtr<Fleet> >& fleets);
+    void AddFleetsStateChangedSignal(const std::vector<std::shared_ptr<Fleet>>& fleets);
+
     /** Disconnect all \p fleets StateChangedSignal from RefreshFleetButtons. */
-    void            RemoveFleetsStateChangedSignal(const std::vector<TemporaryPtr<Fleet> >& fleets);
-    /** Handle FleetsInsertedSignal by connecting signals and refreshing fleet buttons. */
-    void            FleetsInsertedSignalHandler(const std::vector<TemporaryPtr<Fleet> >& fleets);
-    /** Handle FleetsRemovedSignal by disconnecting signals and refreshing fleet buttons. */
-    void            FleetsRemovedSignalHandler(const std::vector<TemporaryPtr<Fleet> >& fleets);
+    void RemoveFleetsStateChangedSignal(const std::vector<std::shared_ptr<Fleet>>& fleets);
+
+    /** Handle FleetsInsertedSignal by connecting signals and refreshing fleet
+        buttons. */
+    void FleetsInsertedSignalHandler(const std::vector<std::shared_ptr<Fleet>>& fleets);
+
+    /** Handle FleetsRemovedSignal by disconnecting signals and refreshing fleet
+        buttons. */
+    void FleetsRemovedSignalHandler(const std::vector<std::shared_ptr<Fleet>>& fleets);
 
 
     void            DoFleetButtonsLayout();                     //!< does layout of fleet buttons
-    std::pair<double, double>   MovingFleetMapPositionOnLane(TemporaryPtr<const Fleet> fleet) const; //!< returns position on map where a moving fleet should be displayed.  This is different from the fleet's actual universe position due to the squishing of fleets moving along a lane into the space between the system circles at the ends of the lane
+
+    /** Returns position on map where a moving fleet should be displayed.  This
+        is different from the fleet's actual universe position due to the
+        squishing of fleets moving along a lane into the space between the
+        system circles at the ends of the lane. */
+    std::pair<double, double> MovingFleetMapPositionOnLane(std::shared_ptr<const Fleet> fleet) const;
 
     void            DoSystemIconsLayout();                      //!< does layout of system icons
     void            DoFieldIconsLayout();                       //!< does layout of field icons
@@ -277,10 +310,6 @@ private:
     void            ClearScaleCircleRenderingBuffer();
     void            ClearStarfieldRenderingBuffers();
 
-    /* Takes X and Y coordinates of a pair of systems and moves these points inwards along the vector
-     * between them by the radius of a system on screen (at zoom 1.0) and return result */ 
-    LaneEndpoints   StarlaneEndPointsFromSystemPositions(double X1, double Y1, double X2, double Y2);
-
     void            RenderStarfields();                         //!< renders the background starfiends
     void            RenderGalaxyGas();                          //!< renders gassy substance to make shape of galaxy
     void            RenderSystemOverlays();                     //!< renders textures "overlayed" on systems by effects
@@ -336,7 +365,7 @@ private:
     void            ShipRightClicked(int fleet_id);
     void            ShipsRightClicked(const std::vector<int>& fleet_ids);
 
-    void            UniverseObjectDeleted(TemporaryPtr<const UniverseObject> obj);
+    void UniverseObjectDeleted(std::shared_ptr<const UniverseObject> obj);
 
     bool            ReturnToMap();
 
@@ -395,6 +424,8 @@ private:
     bool            ZoomToHomeSystem();
     bool            ZoomToPrevOwnedSystem();
     bool            ZoomToNextOwnedSystem();
+    bool            ZoomToPrevSystem();
+    bool            ZoomToNextSystem();
     bool            ZoomToPrevIdleFleet();
     bool            ZoomToNextIdleFleet();
     bool            ZoomToPrevFleet();
@@ -430,21 +461,30 @@ private:
 
     std::map<std::pair<int, int>, LaneEndpoints>    m_starlane_endpoints;                   //!< map from starlane start and end system IDs (stored in pair in increasing order) to the universe coordiates at which to draw the starlane ends
 
-    boost::unordered_map<int, boost::unordered_set<FleetButton*> >          m_stationary_fleet_buttons;             //!< icons representing fleets at a system that are not departing, indexed by system
-    boost::unordered_map<int, boost::unordered_set<FleetButton*> >          m_departing_fleet_buttons;              //!< icons representing fleets at a system that are departing, indexed by system
-    boost::unordered_map<std::pair<double, double>,  boost::unordered_set<FleetButton*> > m_moving_fleet_buttons;   //!< icons representing fleets not at a system
-    boost::unordered_map<int, FleetButton*>                     m_fleet_buttons;                        //!< fleet icons, index by fleet
+    /** Icons representing fleets at a system that are not departing, indexed
+        by system. */
+    boost::unordered_map<int, std::unordered_set<FleetButton*>> m_stationary_fleet_buttons;
 
-    boost::unordered_map<int, boost::signals2::connection>               m_fleet_state_change_signals;
-    boost::unordered_map<int, std::vector<boost::signals2::connection> > m_system_fleet_insert_remove_signals;
+    /** Icons representing fleets at a system that are departing, indexed by
+        system. */
+    boost::unordered_map<int, std::unordered_set<FleetButton*>> m_departing_fleet_buttons;
 
-    std::map<int, MovementLineData>                 m_fleet_lines;                          //!< lines used for moving fleets in the main map
-    std::map<int, MovementLineData>                 m_projected_fleet_lines;                //!< lines that show the projected path of the active fleet in the FleetWnd
+    /** Icons representing fleets not at a system. */
+    boost::unordered_map<std::pair<double, double>,
+                         std::unordered_set<FleetButton*>>              m_moving_fleet_buttons;
 
-    std::pair<int, int>                 m_line_between_systems;                             //!< set when map should render line connecting 2 systems
+    boost::unordered_map<int, FleetButton*>                             m_fleet_buttons;                        //!< fleet icons, index by fleet
 
-    std::map<boost::shared_ptr<GG::Texture>, GG::GL2DVertexBuffer>  m_star_core_quad_vertices;
-    std::map<boost::shared_ptr<GG::Texture>, GG::GL2DVertexBuffer>  m_star_halo_quad_vertices;
+    boost::unordered_map<int, boost::signals2::connection>              m_fleet_state_change_signals;
+    boost::unordered_map<int, std::vector<boost::signals2::connection>> m_system_fleet_insert_remove_signals;
+
+    std::map<int, MovementLineData>     m_fleet_lines;                  //!< lines used for moving fleets in the main map
+    std::map<int, MovementLineData>     m_projected_fleet_lines;        //!< lines that show the projected path of the active fleet in the FleetWnd
+
+    std::pair<int, int>                 m_line_between_systems;         //!< set when map should render line connecting 2 systems
+
+    std::map<std::shared_ptr<GG::Texture>, GG::GL2DVertexBuffer> m_star_core_quad_vertices;
+    std::map<std::shared_ptr<GG::Texture>, GG::GL2DVertexBuffer> m_star_halo_quad_vertices;
     GG::GL2DVertexBuffer                m_galaxy_gas_quad_vertices;
     GG::GLTexCoordBuffer                m_galaxy_gas_texture_coords;
     GG::GLTexCoordBuffer                m_star_texture_coords;
@@ -455,7 +495,9 @@ private:
     GG::GL2DVertexBuffer                m_RC_starlane_vertices;
     GG::GLRGBAColorBuffer               m_RC_starlane_colors;
 
-    std::map<boost::shared_ptr<GG::Texture>, std::pair<GG::GL2DVertexBuffer, GG::GL2DVertexBuffer> >    m_field_vertices;   //!< first buffer is visible fields, second buffer is not visible (scanlined) fields for each texture
+    /** First buffer is visible fields, second buffer is not visible (scanlined)
+        fields for each texture. */
+    std::map<std::shared_ptr<GG::Texture>, std::pair<GG::GL2DVertexBuffer, GG::GL2DVertexBuffer>> m_field_vertices;
     GG::GL2DVertexBuffer                m_field_scanline_circles;
     GG::GLTexCoordBuffer                m_field_texture_coords;
 
@@ -463,7 +505,7 @@ private:
     GG::GLRGBAColorBuffer               m_visibility_radii_colors;
     GG::GL2DVertexBuffer                m_visibility_radii_border_vertices;
     GG::GLRGBAColorBuffer               m_visibility_radii_border_colors;
-    std::vector<std::pair<std::pair<std::size_t, std::size_t>, std::pair<std::size_t, std::size_t> > >
+    std::vector<std::pair<std::pair<std::size_t, std::size_t>, std::pair<std::size_t, std::size_t>>>
                                         m_radii_radii_vertices_indices_runs;
 
     GG::GL2DVertexBuffer                m_scale_circle_vertices;
@@ -515,9 +557,13 @@ class MapWndPopup : public CUIWnd {
 public:
     MapWndPopup(const std::string& t, GG::X default_x, GG::Y default_y, GG::X default_w, GG::Y default_h,
                 GG::Flags<GG::WndFlag> flags, const std::string& config_name = "");
+
     MapWndPopup(const std::string& t, GG::Flags<GG::WndFlag> flags, const std::string& config_name = "");
+
     virtual ~MapWndPopup();
-    void    CloseClicked();
+
+    void CloseClicked() override;
+
     void    Close();
 };
 
