@@ -4,8 +4,8 @@
 #include <GG/GGFwd.h>
 #include <GG/BrowseInfoWnd.h>
 
-#include "../universe/Enums.h"
-#include <boost/tuple/tuple.hpp>
+#include "../universe/EnumsFwd.h"
+#include <tuple>
 
 class UniverseObject;
 
@@ -13,7 +13,7 @@ namespace DualMeter {
 
     /** Return the triplet of {Current, Projected, Target} meter value for the pair of meters \p
         actual_meter_type and \p target_meter_type associated with \p obj. */
-    boost::tuple<float, float, float> CurrentProjectedTarget(
+    std::tuple<float, float, float> CurrentProjectedTarget(
         const UniverseObject& obj, const MeterType& actual_meter_type, const MeterType& target_meter_type);
 }
 
@@ -21,15 +21,18 @@ namespace DualMeter {
   * shows the current turn's current meter value and the predicted current meter value for next turn. */
 class MeterBrowseWnd : public GG::BrowseInfoWnd {
 public:
-    MeterBrowseWnd(int object_id, MeterType primary_meter_type, MeterType secondary_meter_type = INVALID_METER_TYPE);
+    MeterBrowseWnd(int object_id, MeterType primary_meter_type, MeterType secondary_meter_type);
+    MeterBrowseWnd(int object_id, MeterType primary_meter_type);
 
-    virtual bool    WndHasBrowseInfo(const Wnd* wnd, std::size_t mode) const;
-    virtual void    Render();
+    bool WndHasBrowseInfo(const Wnd* wnd, std::size_t mode) const override;
+
+    void Render() override;
 
 protected:
+    void UpdateImpl(std::size_t mode, const Wnd* target) override;
+
     void            Initialize();
 
-    virtual void    UpdateImpl(std::size_t mode, const Wnd* target);
     void            UpdateSummary();
     void            UpdateEffectLabelsAndValues(GG::Y& top);
 
@@ -48,7 +51,7 @@ protected:
 
     GG::Label*              m_meter_title;
 
-    std::vector<std::pair<GG::Label*, GG::Label*> >
+    std::vector<std::pair<GG::Label*, GG::Label*>>
                             m_effect_labels_and_values;
 
     GG::Y                   m_row_height;
@@ -62,12 +65,31 @@ public:
     ShipDamageBrowseWnd(int object_id, MeterType primary_meter_type);
 
 private:
+    void UpdateImpl(std::size_t mode, const Wnd* target) override;
+
     void            Initialize();
 
-    virtual void    UpdateImpl(std::size_t mode, const Wnd* target);
     void            UpdateSummary();
     void            UpdateEffectLabelsAndValues(GG::Y& top);
 
 };
 
+class ShipFightersBrowseWnd : public MeterBrowseWnd {
+public:
+    ShipFightersBrowseWnd(int object_id, MeterType primary_meter_type, bool show_all_bouts = false);
+
+private:
+    void UpdateImpl(std::size_t mode, const Wnd* target) override;
+
+    void            Initialize();
+
+    void            UpdateSummary();
+    void            UpdateEffectLabelsAndValues(GG::Y& top);
+
+    GG::ListBox*    m_bay_list;
+    GG::ListBox*    m_hangar_list;
+    bool            m_show_all_bouts;
+
+};
+
 #endif
