@@ -2,8 +2,8 @@
 #define _MilitaryPanel_h_
 
 #include "AccordionPanel.h"
-#include "../universe/Enums.h"
-#include "../universe/TemporaryPtr.h"
+#include "../universe/EnumsFwd.h"
+
 
 class MultiIconValueIndicator;
 class MultiMeterStatusBar;
@@ -19,10 +19,13 @@ public:
     //@}
 
     /** \name Accessors */ //@{
+    bool EventFilter(GG::Wnd* w, const GG::WndEvent& event) override;
+
     int PlanetID() const { return m_planet_id; }
     //@}
 
     /** \name Mutators */ //@{
+    void PreRender() override;
     /** expands or collapses panel to show details or just summary info */
     void ExpandCollapse(bool expanded);
 
@@ -35,7 +38,7 @@ public:
 protected:
     /** \name Mutators */ //@{
     /** resizes panel and positions widgets */
-    virtual void DoLayout();
+    void DoLayout() override;
     //@}
 
 private:
@@ -46,10 +49,10 @@ private:
     int m_planet_id;
 
     /** returns the Planet object with id m_planet_id */
-    TemporaryPtr<const Planet> GetPlanet() const;
+    std::shared_ptr<const Planet> GetPlanet() const;
 
     /** Icons for the associated meter type. */
-    std::vector<std::pair<MeterType, StatisticIcon*> > m_meter_stats;
+    std::vector<std::pair<MeterType, StatisticIcon*>> m_meter_stats;
 
     /** textually / numerically indicates military capabilities */
     MultiIconValueIndicator* m_multi_icon_value_indicator;
