@@ -6,6 +6,7 @@
 #include "../util/Logger.h"
 #include "../universe/PopCenter.h"
 #include "../universe/UniverseObject.h"
+#include "../universe/Enums.h"
 #include "../client/human/HumanClientApp.h"
 #include "ClientUI.h"
 #include "CUIControls.h"
@@ -21,53 +22,39 @@ namespace {
     { return GG::Y(IconSpacing()*2); }
 }
 
+MultiIconValueIndicator::MultiIconValueIndicator(GG::X w) :
+    MultiIconValueIndicator(w, {}, {})
+{}
+
 MultiIconValueIndicator::MultiIconValueIndicator(GG::X w, int object_id,
-                                                 const std::vector<std::pair<MeterType, MeterType> >& meter_types) :
-    GG::Wnd(GG::X0, GG::Y0, w, GG::Y1, GG::INTERACTIVE),
-    m_icons(),
-    m_meter_types(meter_types),
-    m_object_ids()
-{
-    m_object_ids.push_back(object_id);
-    Init();
-}
+                                                 const std::vector<std::pair<MeterType, MeterType>>& meter_types) :
+    MultiIconValueIndicator(w, std::vector<int>{object_id}, meter_types)
+{}
 
 MultiIconValueIndicator::MultiIconValueIndicator(GG::X w, const std::vector<int>& object_ids,
-                                                 const std::vector<std::pair<MeterType, MeterType> >& meter_types) :
+                                                 const std::vector<std::pair<MeterType, MeterType>>& meter_types) :
     GG::Wnd(GG::X0, GG::Y0, w, GG::Y1, GG::INTERACTIVE),
     m_icons(),
     m_meter_types(meter_types),
     m_object_ids(object_ids)
-{ Init(); }
-
-MultiIconValueIndicator::MultiIconValueIndicator(GG::X w) :
-    GG::Wnd(GG::X0, GG::Y0, w, GG::Y1, GG::INTERACTIVE),
-    m_icons(),
-    m_meter_types(),
-    m_object_ids()
-{ Init(); }
-
-MultiIconValueIndicator::~MultiIconValueIndicator()
-{}  // nothing needs deleting, as all contained indicators are childs and auto deleted
-
-void MultiIconValueIndicator::Init() {
+{
     SetName("MultiIconValueIndicator");
 
     GG::X x(EDGE_PAD);
-    for (std::vector<std::pair<MeterType, MeterType> >::const_iterator it = m_meter_types.begin(); it != m_meter_types.end(); ++it) {
-        const MeterType PRIMARY_METER_TYPE = it->first;
+    for (const std::pair<MeterType, MeterType>& meter_type : m_meter_types) {
+        const MeterType PRIMARY_METER_TYPE = meter_type.first;
         // get icon texture.
-        boost::shared_ptr<GG::Texture> texture = ClientUI::MeterIcon(PRIMARY_METER_TYPE);
+        std::shared_ptr<GG::Texture> texture = ClientUI::MeterIcon(PRIMARY_METER_TYPE);
 
         // special case for population meter for an indicator showing only a
         // single popcenter: icon is species icon, rather than generic pop icon
         if (PRIMARY_METER_TYPE == METER_POPULATION && m_object_ids.size() == 1) {
-            if (TemporaryPtr<const UniverseObject> obj = GetUniverseObject(*m_object_ids.begin()))
-                if (TemporaryPtr<const PopCenter> pc = boost::dynamic_pointer_cast<const PopCenter>(obj))
-                    texture = ClientUI::SpeciesIcon(pc->SpeciesName());
+	    if (std::shared_ptr<const PopCenter> pc = GetPopCenter(*m_object_ids.begin()))
+		texture = ClientUI::SpeciesIcon(pc->SpeciesName());
         }
 
-        m_icons.push_back(new StatisticIcon(texture, 0.0, 3, false));
+        m_icons.push_back(new StatisticIcon(texture, 0.0, 3, false,
+                                            GG::X0, GG::Y0, IconWidth(), IconHeight()));
         GG::Pt icon_ul(x, GG::Y(EDGE_PAD));
         GG::Pt icon_lr = icon_ul + GG::Pt(IconWidth(), IconHeight() + ClientUI::Pts()*3/2);
         m_icons.back()->SizeMove(icon_ul, icon_lr);
@@ -103,9 +90,8 @@ void MultiIconValueIndicator::Update() {
     for (std::size_t i = 0; i < m_icons.size(); ++i) {
         assert(m_icons[i]);
         double sum = 0.0;
-        for (std::size_t j = 0; j < m_object_ids.size(); ++j) {
-            int object_id = m_object_ids[j];
-            TemporaryPtr<const UniverseObject> obj = GetUniverseObject(object_id);
+        for (int object_id : m_object_ids) {
+            std::shared_ptr<const UniverseObject> obj = GetUniverseObject(object_id);
             if (!obj) {
                 ErrorLogger() << "MultiIconValueIndicator::Update couldn't get object with id " << object_id;
                 continue;
@@ -118,7 +104,7 @@ void MultiIconValueIndicator::Update() {
     }
 }
 
-void MultiIconValueIndicator::SetToolTip(MeterType meter_type, const boost::shared_ptr<GG::BrowseInfoWnd>& browse_wnd) {
+void MultiIconValueIndicator::SetToolTip(MeterType meter_type, const std::shared_ptr<GG::BrowseInfoWnd>& browse_wnd) {
     for (unsigned int i = 0; i < m_icons.size(); ++i)
         if (m_meter_types.at(i).first == meter_type)
             m_icons.at(i)->SetBrowseInfoWnd(browse_wnd);
@@ -135,42 +121,62 @@ bool MultiIconValueIndicator::EventFilter(GG::Wnd* w, const GG::WndEvent& event)
         return false;
     const GG::Pt& pt = event.Point();
 
-    if (m_object_ids.size() != 1)
-        return false;
-    TemporaryPtr<const UniverseObject> obj = GetUniverseObject(*m_object_ids.begin());
-    if (!obj)
-        return false;
-
-    TemporaryPtr<const PopCenter> pc = boost::dynamic_pointer_cast<const PopCenter>(obj);
-    if (!pc)
-        return false;
-
-    const std::string& species_name = pc->SpeciesName();
-    if (species_name.empty())
-        return false;
-
+    MeterType meter_type = INVALID_METER_TYPE;
     for (unsigned int i = 0; i < m_icons.size(); ++i) {
-        if (m_icons.at(i) != w)
-            continue;
-        MeterType meter_type = m_meter_types.at(i).first;
-        if (meter_type != METER_POPULATION)
-            continue;
-
-        GG::MenuItem menu_contents;
-
-        std::string popup_label = boost::io::str(FlexibleFormat(UserString("ENC_LOOKUP")) % UserString(species_name));
-        menu_contents.next_level.push_back(GG::MenuItem(popup_label, 1, false, false));
-        GG::PopupMenu popup(pt.x, pt.y, ClientUI::GetFont(), menu_contents, ClientUI::TextColor(),
-                            ClientUI::WndOuterBorderColor(), ClientUI::WndColor(), ClientUI::EditHiliteColor());
-
-        if (!popup.Run() || popup.MenuID() != 1) {
+        try {
+            if (m_icons.at(i) == w) {
+                meter_type = m_meter_types.at(i).first;
+                break;
+            }
+        } catch(std::out_of_range &e) {
+            ErrorLogger() << e.what();
             return false;
-            break;
         }
+    }
+    if (meter_type == INVALID_METER_TYPE)
+        return false;
 
-        ClientUI::GetClientUI()->ZoomToSpecies(species_name);
-        return true;
+    std::string meter_string = boost::lexical_cast<std::string>(meter_type);
+    std::string meter_title;
+    if (UserStringExists(meter_string))
+        meter_title = UserString(meter_string);
+
+    GG::MenuItem menu_contents;
+    std::string species_name;
+
+    std::shared_ptr<const PopCenter> pc = GetPopCenter(*m_object_ids.begin());
+    if (meter_type == METER_POPULATION && pc && m_object_ids.size() == 1) {
+	species_name = pc->SpeciesName();
+	if (!species_name.empty()) {
+	    std::string species_label = boost::io::str(FlexibleFormat(UserString("ENC_LOOKUP")) % UserString(species_name));
+	    menu_contents.next_level.push_back(GG::MenuItem(species_label, 1, false, false));
+	}
     }
 
-    return false;
+    if (!meter_title.empty()) {
+        std::string popup_label = boost::io::str(FlexibleFormat(UserString("ENC_LOOKUP")) %
+                                                                meter_title);
+        menu_contents.next_level.push_back(GG::MenuItem(popup_label, 2, false, false));
+    }
+
+    CUIPopupMenu popup(pt.x, pt.y, menu_contents);
+
+    bool retval = false;
+
+    if (popup.Run()) {
+        switch (popup.MenuID()) {
+            case 1: {
+                retval = ClientUI::GetClientUI()->ZoomToSpecies(species_name);
+                break;
+            }
+            case 2: {
+                retval = ClientUI::GetClientUI()->ZoomToMeterTypeArticle(meter_string);
+                break;
+            }
+            default:
+                break;
+        }
+    }
+
+    return retval;
 }
