@@ -9,6 +9,7 @@
 #include "../util/Logger.h"
 #include "../universe/Meter.h"
 #include "../universe/UniverseObject.h"
+#include "../universe/Enums.h"
 #include "../client/human/HumanClientApp.h"
 #include "ClientUI.h"
 
@@ -72,7 +73,7 @@ namespace {
     const double    MULTI_METER_STATUS_BAR_DISPLAYED_METER_RANGE_INCREMENT = 100.0;
 }
 
-MultiMeterStatusBar::MultiMeterStatusBar(GG::X w, int object_id, const std::vector<std::pair<MeterType, MeterType> >& meter_types) :
+MultiMeterStatusBar::MultiMeterStatusBar(GG::X w, int object_id, const std::vector<std::pair<MeterType, MeterType>>& meter_types) :
     GG::Wnd(GG::X0, GG::Y0, w, GG::Y1, GG::INTERACTIVE),
     m_bar_shading_texture(ClientUI::GetTexture(ClientUI::ArtDir() / "misc" / "meter_bar_shading.png")),
     m_meter_types(meter_types),
@@ -109,8 +110,9 @@ void MultiMeterStatusBar::Render() {
         y += BAR_HEIGHT + BAR_PAD;
     }
 
-    // Find the largest value to be displayed to determine the scale factor
-    double largest_value = 0;
+    // Find the largest value to be displayed to determine the scale factor.  Must be greater than
+    // zero so that there is at least one segment drawn.
+    double largest_value = 0.1;
     for (unsigned int i = 0; i < m_initial_values.size(); ++i) {
         if ((m_initial_values[i] != Meter::INVALID_VALUE) && (m_initial_values[i] > largest_value))
             largest_value = m_initial_values[i];
@@ -126,12 +128,12 @@ void MultiMeterStatusBar::Render() {
         num_full_increments * MULTI_METER_STATUS_BAR_DISPLAYED_METER_RANGE_INCREMENT;
 
     // lines for 20, 40, 60, 80 etc.
-    int num_segments = num_full_increments * 5;
     GG::GL2DVertexBuffer bar_verts;
+    unsigned int num_segments = num_full_increments * 5;
     bar_verts.reserve(num_segments - 1);
-    for (int ii_div_line = 1; ii_div_line <= (num_segments -1); ++ii_div_line) {
-        bar_verts.store(BAR_LEFT + ii_div_line*BAR_MAX_LENGTH/num_segments, TOP);
-        bar_verts.store(BAR_LEFT + ii_div_line*BAR_MAX_LENGTH/num_segments, y - BAR_PAD);
+    for (unsigned int ii_div_line = 1; ii_div_line <= (num_segments - 1); ++ii_div_line) {
+        bar_verts.store(Value(BAR_LEFT) + ii_div_line*Value(BAR_MAX_LENGTH)/num_segments, TOP);
+        bar_verts.store(Value(BAR_LEFT) + ii_div_line*Value(BAR_MAX_LENGTH)/num_segments, y - BAR_PAD);
     }
     bar_verts.activate();
 
@@ -201,7 +203,7 @@ void MultiMeterStatusBar::Update() {
     m_projected_values.clear(); // projected current value of .first MeterTypes for the start of next turn
     m_target_max_values.clear();// current values of the .second MeterTypes in m_meter_types
 
-    TemporaryPtr<const UniverseObject> obj = GetUniverseObject(m_object_id);
+    std::shared_ptr<const UniverseObject> obj = GetUniverseObject(m_object_id);
     if (!obj) {
         ErrorLogger() << "MultiMeterStatusBar couldn't get object with id " << m_object_id;
         return;
@@ -209,19 +211,18 @@ void MultiMeterStatusBar::Update() {
 
     int num_bars = 0;   // count number of valid bars' data added
 
-    for (std::vector<std::pair<MeterType, MeterType> >::const_iterator meter_pairs_it = m_meter_types.begin(); meter_pairs_it != m_meter_types.end(); ++meter_pairs_it) {
-        const std::pair<MeterType, MeterType>& meter_types_pair = *meter_pairs_it;
+    for (const std::pair<MeterType, MeterType>& meter_types_pair : m_meter_types) {
         const Meter* actual_meter = obj->GetMeter(meter_types_pair.first);
         const Meter* target_max_meter = obj->GetMeter(meter_types_pair.second);
 
-        boost::tuple<float, float, float> current_projected_target = DualMeter::CurrentProjectedTarget(
+        std::tuple<float, float, float> current_projected_target = DualMeter::CurrentProjectedTarget(
             *obj, meter_types_pair.first, meter_types_pair.second);
 
         if (actual_meter || target_max_meter) {
             ++num_bars;
-            m_initial_values.push_back(boost::get<0>(current_projected_target));
-            m_projected_values.push_back(boost::get<1>(current_projected_target));
-            m_target_max_values.push_back(boost::get<2>(current_projected_target));
+            m_initial_values.push_back(std::get<0>(current_projected_target));
+            m_projected_values.push_back(std::get<1>(current_projected_target));
+            m_target_max_values.push_back(std::get<2>(current_projected_target));
             m_bar_colours.push_back(MeterColor(meter_types_pair.first));
         }
     }
