@@ -9,6 +9,7 @@
 #include "../util/MultiplayerCommon.h"
 
 class Message;
+struct PlayerLabelRow;
 
 
 /** multiplayer lobby window */
@@ -19,14 +20,17 @@ public:
     //@}
 
     /** \name Accessors */ //@{
+    GG::Pt MinUsableSize() const override;
+
     bool            LoadGameSelected() const;
-    virtual GG::Pt  MinUsableSize() const;
     //@}
 
     /** \name Mutators */ //@{
-    virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr);
-    virtual void    Render();
-    virtual void    KeyPress(GG::Key key, boost::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys);
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
+
+    void Render() override;
+
+    void KeyPress(GG::Key key, std::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys) override;
 
     void            ChatMessage(int player_id, const std::string& msg);
     void            LobbyUpdate(const MultiplayerLobbyData& lobby_data);
@@ -34,21 +38,30 @@ public:
     //@}
 
 protected:
-    virtual GG::Rect CalculatePosition() const;
+    struct PlayerLabelRow : GG::ListBox::Row {
+        PlayerLabelRow(GG::X width = GG::X(600));
+
+        /** Set text of control at @p column to @p str */
+        void SetText(size_t column, const std::string& str);
+
+        void Render() override;
+    };
+
+    GG::Rect CalculatePosition() const override;
 
 private:
     void            DoLayout();
     void            NewLoadClicked(std::size_t idx);
     void            GalaxySetupPanelChanged();
     void            SaveGameBrowse();
-    void            PreviewImageChanged(boost::shared_ptr<GG::Texture> new_image);
+    void PreviewImageChanged(std::shared_ptr<GG::Texture> new_image);
     void            PlayerDataChangedLocally();
     bool            PopulatePlayerList();   ///< repopulate list with rows built from current m_lobby_data.  returns true iff something in the lobby data was changed during population and an update should be sent back to the server
     void            SendUpdate();
     bool            PlayerDataAcceptable() const;
     bool            CanStart() const;
     bool            ThisClientIsHost() const;
-    void            StartGameClicked();
+    void            ReadyClicked();
     void            CancelClicked();
 
     MultiplayerLobbyData    m_lobby_data;   ///< a copy of the most recently received lobby update
@@ -60,13 +73,9 @@ private:
     GG::Label*              m_save_file_text;
     GG::Button*             m_browse_saves_btn;
     GG::StaticGraphic*      m_preview_image;
-    GG::Label*              m_players_lb_player_type_label;
-    GG::Label*              m_players_lb_player_name_column_label;
-    GG::Label*              m_players_lb_empire_name_column_label;
-    GG::Label*              m_players_lb_empire_colour_column_label;
-    GG::Label*              m_players_lb_species_or_original_player_label;
     GG::ListBox*            m_players_lb;
-    GG::Button*             m_start_game_bn;
+    PlayerLabelRow*         m_players_lb_headers;
+    GG::Button*             m_ready_bn;
     GG::Button*             m_cancel_bn;
     GG::Label*              m_start_conditions_text;
 };
