@@ -5,6 +5,7 @@
 #include "../Empire/Empire.h"
 #include "../Empire/EmpireManager.h"
 #include "../network/Message.h"
+#include "../network/ClientNetworking.h"
 #include "../util/i18n.h"
 #include "../util/Logger.h"
 #include "../util/OptionsDB.h"
@@ -12,6 +13,7 @@
 #include "../universe/Ship.h"
 #include "../universe/Planet.h"
 #include "../universe/System.h"
+#include "../universe/Enums.h"
 
 #include <GG/DrawUtil.h>
 
@@ -20,72 +22,76 @@
 namespace {
     const int           DATA_PANEL_BORDER = 1;
 
-    boost::shared_ptr<GG::Texture> AIIcon() {
-        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "ai.png");
+    std::shared_ptr<GG::Texture> AIIcon() {
+        static std::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "ai.png");
         return retval;
     }
-    boost::shared_ptr<GG::Texture> HumanIcon() {
-        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "human.png");
+    std::shared_ptr<GG::Texture> HumanIcon() {
+        static std::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "human.png");
         return retval;
     }
-    boost::shared_ptr<GG::Texture> ObserverIcon() {
-        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "observer.png");
+    std::shared_ptr<GG::Texture> ObserverIcon() {
+        static std::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "observer.png");
         return retval;
     }
-    boost::shared_ptr<GG::Texture> ModeratorIcon() {
-        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "moderator.png");
+    std::shared_ptr<GG::Texture> ModeratorIcon() {
+        static std::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "moderator.png");
         return retval;
     }
-    boost::shared_ptr<GG::Texture> HostIcon() {
-        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "host.png");
+    std::shared_ptr<GG::Texture> HostIcon() {
+        static std::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "host.png");
         return retval;
     }
-    boost::shared_ptr<GG::Texture> PlayingIcon() {
-        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "playing.png");
+    std::shared_ptr<GG::Texture> PlayingIcon() {
+        static std::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "not_ready.png");
         return retval;
     }
-    boost::shared_ptr<GG::Texture> WaitingIcon() {
-        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "waiting.png");
+    std::shared_ptr<GG::Texture> WaitingIcon() {
+        static std::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "ready.png");
         return retval;
     }
-    boost::shared_ptr<GG::Texture> CombatIcon() {
-        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "combat.png");
+    std::shared_ptr<GG::Texture> CombatIcon() {
+        static std::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "combat.png");
         return retval;
     }
-    boost::shared_ptr<GG::Texture> WarIcon() {
-        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "war.png");
+    std::shared_ptr<GG::Texture> WarIcon() {
+        static std::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "war.png");
         return retval;
     }
-    boost::shared_ptr<GG::Texture> PeaceIcon() {
-        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "peace.png");
+    std::shared_ptr<GG::Texture> PeaceIcon() {
+        static std::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "peace.png");
         return retval;
     }
-    boost::shared_ptr<GG::Texture> UnknownIcon() {
-        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "unknown.png");
+    std::shared_ptr<GG::Texture> AlliedIcon() {
+        static std::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "allied.png");
         return retval;
     }
-    boost::shared_ptr<GG::Texture> ShipIcon() {
-        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "sitrep" / "fleet_arrived.png");
+    std::shared_ptr<GG::Texture> UnknownIcon() {
+        static std::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "unknown.png");
         return retval;
     }
-    boost::shared_ptr<GG::Texture> ProductionIcon() {
-        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "meter" / "industry.png");
+    std::shared_ptr<GG::Texture> ShipIcon() {
+        static std::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "sitrep" / "fleet_arrived.png");
         return retval;
     }
-    boost::shared_ptr<GG::Texture> ResearchIcon() {
-        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "meter" / "research.png");
+    std::shared_ptr<GG::Texture> ProductionIcon() {
+        static std::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "meter" / "industry.png");
         return retval;
     }
-    boost::shared_ptr<GG::Texture> DetectionIcon() {
-        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "meter" / "detection.png");
+    std::shared_ptr<GG::Texture> ResearchIcon() {
+        static std::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "meter" / "research.png");
         return retval;
     }
-    boost::shared_ptr<GG::Texture> WonIcon() {
-        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "sitrep" / "victory.png");
+    std::shared_ptr<GG::Texture> DetectionIcon() {
+        static std::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "meter" / "detection.png");
         return retval;
     }
-    boost::shared_ptr<GG::Texture> LostIcon() {
-        static boost::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "sitrep" / "empire_eliminated.png");
+    std::shared_ptr<GG::Texture> WonIcon() {
+        static std::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "sitrep" / "victory.png");
+        return retval;
+    }
+    std::shared_ptr<GG::Texture> LostIcon() {
+        static std::shared_ptr<GG::Texture> retval = ClientUI::GetTexture(ClientUI::ArtDir() / "icons" / "sitrep" / "empire_eliminated.png");
         return retval;
     }
 
@@ -100,15 +106,15 @@ namespace {
         PlayerDataPanel(GG::X w, GG::Y h, int player_id) :
             Control(GG::X0, GG::Y0, w, h, GG::NO_WND_FLAGS),
             m_player_id(player_id),
-            //m_player_name_text(0),
-            m_empire_name_text(0),
-            m_empire_ship_text(0),
-            m_empire_planet_text(0),
-            m_empire_production_text(0),
-            m_empire_research_text(0),
-            m_empire_detection_text(0),
+            //m_player_name_text(nullptr),
+            m_empire_name_text(nullptr),
+            m_empire_ship_text(nullptr),
+            m_empire_planet_text(nullptr),
+            m_empire_production_text(nullptr),
+            m_empire_research_text(nullptr),
+            m_empire_detection_text(nullptr),
             m_diplo_status(INVALID_DIPLOMATIC_STATUS),
-            m_player_status(Message::WAITING),
+            m_player_status(Message::PLAYING_TURN),
             m_player_type(Networking::INVALID_CLIENT_TYPE),
             m_host(false),
             m_win_status(NEITHER),
@@ -137,15 +143,15 @@ namespace {
         }
 
         /** Excludes border from the client area. */
-        virtual GG::Pt  ClientUpperLeft() const
+        GG::Pt ClientUpperLeft() const override
         { return UpperLeft() + GG::Pt(GG::X(DATA_PANEL_BORDER), GG::Y(DATA_PANEL_BORDER)); }
 
         /** Excludes border from the client area. */
-        virtual GG::Pt  ClientLowerRight() const
+        GG::Pt ClientLowerRight() const override
         { return LowerRight() - GG::Pt(GG::X(DATA_PANEL_BORDER), GG::Y(DATA_PANEL_BORDER)); }
 
         /** Renders panel background, border with color depending on the current state. */
-        virtual void    Render() {
+        void Render() override {
             const GG::Clr& background_colour = ClientUI::WndColor();
             const GG::Clr& unselected_colour = ClientUI::WndOuterBorderColor();
             const GG::Clr& selected_colour = ClientUI::WndInnerBorderColor();
@@ -179,6 +185,7 @@ namespace {
                 switch (m_diplo_status) {
                 case DIPLO_WAR:                 WarIcon()->OrthoBlit(    UpperLeft() + m_diplo_status_icon_ul, UpperLeft() + m_diplo_status_icon_ul + ICON_SIZE); break;
                 case DIPLO_PEACE:               PeaceIcon()->OrthoBlit(  UpperLeft() + m_diplo_status_icon_ul, UpperLeft() + m_diplo_status_icon_ul + ICON_SIZE); break;
+                case DIPLO_ALLIED:              AlliedIcon()->OrthoBlit( UpperLeft() + m_diplo_status_icon_ul, UpperLeft() + m_diplo_status_icon_ul + ICON_SIZE); break;
                 case INVALID_DIPLOMATIC_STATUS: UnknownIcon()->OrthoBlit(UpperLeft() + m_diplo_status_icon_ul, UpperLeft() + m_diplo_status_icon_ul + ICON_SIZE); break;
                 default:    break;
                 }
@@ -216,7 +223,7 @@ namespace {
         void            Select(bool b)
         { m_selected = b; }
 
-        virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
+        void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override {
             const GG::Pt old_size = Size();
             GG::Control::SizeMove(ul, lr);
             if (old_size != Size())
@@ -273,8 +280,7 @@ namespace {
             const std::set<int>& this_client_known_destroyed_objects = GetUniverse().EmpireKnownDestroyedObjectIDs(HumanClientApp::GetApp()->EmpireID());
             const std::set<int>& this_client_stale_object_info       = GetUniverse().EmpireStaleKnowledgeObjectIDs(HumanClientApp::GetApp()->EmpireID());
 
-            for (ObjectMap::const_iterator<Ship> ship_it = objects.const_begin<Ship>(); ship_it != objects.const_end<Ship>(); ++ship_it) {
-                TemporaryPtr<const Ship> ship = *ship_it;
+            for (std::shared_ptr<const Ship> ship : objects.FindObjects<Ship>()) {
                 if (empire) {
                     if (ship->Owner() == empire->EmpireID()
                         && this_client_known_destroyed_objects.find(ship->ID()) == this_client_known_destroyed_objects.end()
@@ -284,8 +290,7 @@ namespace {
                 }
             }
 
-            for (ObjectMap::const_iterator<Planet> planet_it = objects.const_begin<Planet>(); planet_it != objects.const_end<Planet>(); ++planet_it) {
-                TemporaryPtr<const Planet> planet = *planet_it;
+            for (std::shared_ptr<const Planet> planet : objects.FindObjects<Planet>()) {
                 if (empire) {
                     if (planet->Owner() == empire->EmpireID()) {
                         empires_planet_count      += 1;
@@ -452,7 +457,7 @@ namespace {
         PlayerRow(GG::X w, GG::Y h, int player_id) :
             GG::ListBox::Row(w, h, "", GG::ALIGN_NONE, 0),
             m_player_id(player_id),
-            m_panel(0)
+            m_panel(nullptr)
         {
             SetName("PlayerRow");
             SetChildClippingMode(ClipToClient);
@@ -476,7 +481,7 @@ namespace {
 
         /** This function overridden because otherwise, rows don't expand
           * larger than their initial size when resizing the list. */
-        void    SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
+        void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override {
             const GG::Pt old_size = Size();
             GG::ListBox::Row::SizeMove(ul, lr);
             //std::cout << "PlayerRow::SizeMove size: (" << Value(Width()) << ", " << Value(Height()) << ")" << std::endl;
@@ -505,15 +510,15 @@ public:
         LockColWidths();
     }
 
-    virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override {
         const GG::Pt old_size = Size();
         CUIListBox::SizeMove(ul, lr);
         //std::cout << "PlayerListBox::SizeMove size: (" << Value(Width()) << ", " << Value(Height()) << ")" << std::endl;
         if (old_size != Size()) {
             const GG::Pt row_size = ListRowSize();
             //std::cout << "PlayerListBox::SizeMove list row size: (" << Value(row_size.x) << ", " << Value(row_size.y) << ")" << std::endl;
-            for (GG::ListBox::iterator it = begin(); it != end(); ++it)
-                (*it)->Resize(row_size);
+            for (GG::ListBox::Row* row : *this)
+                row->Resize(row_size);
         }
     }
 
@@ -532,7 +537,7 @@ PlayerListWnd::PlayerListWnd(const std::string& config_name) :
     CUIWnd(UserString("PLAYERS_LIST_PANEL_TITLE"),
            GG::INTERACTIVE | GG::DRAGABLE | GG::ONTOP | GG::RESIZABLE | CLOSABLE | PINABLE,
            config_name),
-    m_player_list(0)
+    m_player_list(nullptr)
 {
     m_player_list = new PlayerListBox();
     m_player_list->SetHiliteColor(GG::CLR_ZERO);
@@ -564,8 +569,7 @@ std::set<int> PlayerListWnd::SelectedPlayerIDs() const {
 }
 
 void PlayerListWnd::HandlePlayerStatusUpdate(Message::PlayerStatus player_status, int about_player_id) {
-    for (CUIListBox::iterator row_it = m_player_list->begin(); row_it != m_player_list->end(); ++row_it) {
-        CUIListBox::Row* row = *row_it;
+    for (CUIListBox::Row* row : *m_player_list) {
         if (PlayerRow* player_row = dynamic_cast<PlayerRow*>(row)) {
             if (about_player_id == Networking::INVALID_PLAYER_ID) {
                 player_row->SetStatus(player_status);
@@ -578,8 +582,7 @@ void PlayerListWnd::HandlePlayerStatusUpdate(Message::PlayerStatus player_status
 }
 
 void PlayerListWnd::Update() {
-    for (CUIListBox::iterator row_it = m_player_list->begin(); row_it != m_player_list->end(); ++row_it) {
-        CUIListBox::Row* row = *row_it;
+    for (CUIListBox::Row* row : *m_player_list) {
         if (PlayerRow* player_row = dynamic_cast<PlayerRow*>(row))
             player_row->Update();
     }
@@ -599,10 +602,8 @@ void PlayerListWnd::Refresh() {
 
     const GG::Pt row_size = m_player_list->ListRowSize();
 
-    for (std::map<int, PlayerInfo>::const_iterator player_it = players.begin();
-         player_it != players.end(); ++player_it)
-    {
-        int player_id = player_it->first;
+    for (const std::map<int, PlayerInfo>::value_type& player : players) {
+        int player_id = player.first;
         PlayerRow* player_row = new PlayerRow(row_size.x, row_size.y, player_id);
         m_player_list->Insert(player_row);
         player_row->Resize(row_size);
@@ -673,7 +674,7 @@ void PlayerListWnd::PlayerSelectionChanged(const GG::ListBox::SelectionSet& rows
             ErrorLogger() << "PlayerListWnd::PlayerSelectionChanged got empty row";
             continue;
         }
-        GG::Control* control = (*row)[0];
+        GG::Control* control = !row->empty() ? row->at(0) : nullptr;
         if (!control) {
             ErrorLogger() << "PlayerListWnd::PlayerSelectionChanged couldn't get control from row";
             continue;
@@ -722,48 +723,93 @@ void PlayerListWnd::PlayerRightClicked(GG::ListBox::iterator it, const GG::Pt& p
 
     if (!GetEmpire(clicked_empire_id)) {
         ErrorLogger() << "PlayerListWnd::PlayerRightClicked tried to look up empire id "
-                               << clicked_empire_id << " for player " << clicked_player_id
-                               << " but couldn't find such an empire";
+                      << clicked_empire_id << " for player " << clicked_player_id
+                      << " but couldn't find such an empire";
         return;
     }
 
     GG::MenuItem menu_contents;
-    if (app->GetClientType() == Networking::CLIENT_TYPE_HUMAN_PLAYER) {
+    if (app->GetClientType() == Networking::CLIENT_TYPE_HUMAN_PLAYER &&
+        client_empire_id != ALL_EMPIRES &&
+        clicked_empire_id != ALL_EMPIRES)
+    {
         // get diplomatic status between client and clicked empires
         DiplomaticStatus diplo_status = Empires().GetDiplomaticStatus(clicked_empire_id, client_empire_id);
         if (diplo_status == INVALID_DIPLOMATIC_STATUS && clicked_player_id != client_player_id) {
             ErrorLogger() << "PlayerListWnd::PlayerRightClicked found invalid diplomatic status between client and clicked empires.";
             return;
         }
-        DiplomaticMessage existing_message = Empires().GetDiplomaticMessage(clicked_empire_id, client_empire_id);
+        const DiplomaticMessage& existing_message_from_clicked_empire_to_this_player =
+            Empires().GetDiplomaticMessage(clicked_empire_id, client_empire_id);
+        const DiplomaticMessage& existing_message_from_this_player_to_clicked_empire =
+            Empires().GetDiplomaticMessage(client_empire_id, clicked_empire_id);
 
-        // create popup menu with diplomacy options in it
-        if ( client_empire_id != ALL_EMPIRES) {
-            if (diplo_status == DIPLO_WAR) {
-                if (existing_message.GetType() == DiplomaticMessage::PEACE_PROPOSAL) {
-                    // who sent message?
-                    if (existing_message.SenderEmpireID() == client_empire_id)
-                        menu_contents.next_level.push_back(GG::MenuItem(UserString("PEACE_PROPOSAL_CANEL"), 4, false, false));
-                    else if (existing_message.SenderEmpireID() == clicked_empire_id)
-                        menu_contents.next_level.push_back(GG::MenuItem(UserString("PEACE_ACCEPT"),         3, false, false));
+        // decide what diplomatic commands to show in popup menu
+        bool show_peace_propose = false;
+        bool show_peace_cancel = false;
+        bool show_peace_reject = false;
+        bool show_peace_accept = false;
+        bool show_allies_end = false;
+        bool show_allies_propose = false;
+        bool show_allies_cancel = false;
+        bool show_allies_reject = false;
+        bool show_allies_accept = false;
+        bool show_declare_war = false;
 
-                } else if (existing_message.GetType() == DiplomaticMessage::INVALID_DIPLOMATIC_MESSAGE_TYPE) {
-                    menu_contents.next_level.push_back(GG::MenuItem(UserString("PEACE_PROPOSAL"),           2, false, false));
-                }
+        if (existing_message_from_this_player_to_clicked_empire.GetType() == DiplomaticMessage::PEACE_PROPOSAL)
+            show_peace_cancel = true;
+        else if (existing_message_from_this_player_to_clicked_empire.GetType() == DiplomaticMessage::ALLIES_PROPOSAL)
+            show_allies_cancel = true;
 
-            } else if (diplo_status == DIPLO_PEACE) {
-                if (existing_message.GetType() == DiplomaticMessage::INVALID_DIPLOMATIC_MESSAGE_TYPE)
-                    menu_contents.next_level.push_back(GG::MenuItem(UserString("WAR_DECLARATION"),          1, false, false));
-            }
+        if (existing_message_from_clicked_empire_to_this_player.GetType() == DiplomaticMessage::PEACE_PROPOSAL) {
+            show_peace_accept = true;
+            show_peace_reject = true;
+        } else if (existing_message_from_clicked_empire_to_this_player.GetType() == DiplomaticMessage::ALLIES_PROPOSAL) {
+            show_allies_accept = true;
+            show_allies_reject = true;
         }
+
+        if (diplo_status == DIPLO_WAR) {
+            if (!show_peace_accept)
+                show_peace_propose = true;
+
+        } else if (diplo_status == DIPLO_PEACE) {
+            if (!show_allies_accept)
+                show_allies_propose = true;
+            show_declare_war = true;
+
+        } else if (diplo_status == DIPLO_ALLIED) {
+            show_allies_end = true;
+        }
+
+
+        if (show_peace_propose)
+            menu_contents.next_level.push_back(GG::MenuItem(UserString("PEACE_PROPOSAL"),           2, false, false));
+        if (show_peace_cancel)
+            menu_contents.next_level.push_back(GG::MenuItem(UserString("PEACE_PROPOSAL_CANCEL"),    4, false, false));
+        if (show_peace_accept)
+            menu_contents.next_level.push_back(GG::MenuItem(UserString("PEACE_ACCEPT"),             3, false, false));
+        if (show_peace_reject)
+            menu_contents.next_level.push_back(GG::MenuItem(UserString("PEACE_REJECT"),             9, false, false));
+        if (show_allies_end)
+            menu_contents.next_level.push_back(GG::MenuItem(UserString("END_ALLIANCE_DECLARATION"), 8, false, false));
+        if (show_allies_propose)
+            menu_contents.next_level.push_back(GG::MenuItem(UserString("ALLIES_PROPOSAL"),          6, false, false));
+        if (show_allies_accept)
+            menu_contents.next_level.push_back(GG::MenuItem(UserString("ALLIES_ACCEPT"),            7, false, false));
+        if (show_allies_cancel)
+            menu_contents.next_level.push_back(GG::MenuItem(UserString("ALLIES_PROPOSAL_CANCEL"),   4, false, false));
+        if (show_allies_reject)
+            menu_contents.next_level.push_back(GG::MenuItem(UserString("ALLIES_REJECT"),            9, false, false));
+        if (show_declare_war)
+            menu_contents.next_level.push_back(GG::MenuItem(UserString("WAR_DECLARATION"),          1, false, false));
     }
 
     menu_contents.next_level.push_back(GG::MenuItem(str(FlexibleFormat(UserString("ENC_LOOKUP")) % GetEmpire(clicked_empire_id)->Name()), 5, false, false));
 
     ClientNetworking& net = HumanClientApp::GetApp()->Networking();
 
-    GG::PopupMenu popup(pt.x, pt.y, ClientUI::GetFont(), menu_contents, ClientUI::TextColor(),
-                        ClientUI::WndOuterBorderColor(), ClientUI::WndColor(), ClientUI::EditHiliteColor());
+    CUIPopupMenu popup(pt.x, pt.y, menu_contents);
     if (popup.Run()) {
         switch (popup.MenuID()) {
         case 1: {   // WAR_DECLARATION
@@ -778,14 +824,37 @@ void PlayerListWnd::PlayerRightClicked(GG::ListBox::iterator it, const GG::Pt& p
         }
         case 3: {   // PEACE_ACCEPT
             net.SendMessage(DiplomacyMessage(client_player_id, clicked_player_id,
-                                             AcceptDiplomaticMessage(client_empire_id, clicked_empire_id)));
+                                             AcceptPeaceDiplomaticMessage(client_empire_id, clicked_empire_id)));
             break;
         }
-        case 4: {   // PEACE_PROPOSAL_CANEL
+
+        case 6: {   // ALLIES_PROPOSAL
+            net.SendMessage(DiplomacyMessage(client_player_id, clicked_player_id,
+                                             AlliesProposalDiplomaticMessage(client_empire_id, clicked_empire_id)));
+            break;
+        }
+        case 7: {   // ALLIES_ACCEPT
+            net.SendMessage(DiplomacyMessage(client_player_id, clicked_player_id,
+                                             AcceptAlliesDiplomaticMessage(client_empire_id, clicked_empire_id)));
+            break;
+        }
+        case 8: {   // END_ALLIANCE_DECLARATION
+            net.SendMessage(DiplomacyMessage(client_player_id, clicked_player_id,
+                                             EndAllianceDiplomaticMessage(client_empire_id, clicked_empire_id)));
+            break;
+        }
+
+        case 4: {   // PROPOSAL_CANCEL
             net.SendMessage(DiplomacyMessage(client_player_id, clicked_player_id,
                                              CancelDiplomaticMessage(client_empire_id, clicked_empire_id)));
             break;
         }
+        case 9: {   // PROPOSAL_REJECT
+            net.SendMessage(DiplomacyMessage(client_player_id, clicked_player_id,
+                                             RejectProposalDiplomaticMessage(client_empire_id, clicked_empire_id)));
+            break;
+        }
+
         case 5: { // Pedia lookup
             ClientUI::GetClientUI()->ZoomToEmpire(clicked_empire_id);
             break;
