@@ -2,8 +2,10 @@
 #define _PopulationPanel_h_
 
 #include "AccordionPanel.h"
-#include "../universe/Enums.h"
-#include "../universe/TemporaryPtr.h"
+#include "../universe/EnumsFwd.h"
+
+#include <memory>
+
 
 class MultiIconValueIndicator;
 class MultiMeterStatusBar;
@@ -23,11 +25,13 @@ public:
     //@}
 
     /** \name Mutators */ //@{
+    void PreRender() override;
+
+    bool EventFilter(GG::Wnd* w, const GG::WndEvent& event) override;
+
     /** expands or collapses panel to show details or just summary info */
     void ExpandCollapse(bool expanded);
 
-    bool EventFilter(GG::Wnd* w, const GG::WndEvent& event);
-
     /** updates indicators with values of associated object.  Does not do layout and resizing. */
     void Update();
     /** updates, redoes layout, resizes indicator */
@@ -37,7 +41,7 @@ public:
 protected:
     /** \name Mutators */ //@{
     /** resizes panel and positions widgets */
-    virtual void DoLayout();
+    void DoLayout() override;
     //@}
 
 private:
@@ -48,10 +52,10 @@ private:
     int m_popcenter_id;
 
     /** returns the PopCenter object with id m_popcenter_id */
-    TemporaryPtr<const PopCenter> GetPopCenter() const;
+    std::shared_ptr<const PopCenter> GetPopCenter() const;
 
     /** Icons for the associated meter type. */
-    std::vector<std::pair<MeterType, StatisticIcon*> > m_meter_stats;
+    std::vector<std::pair<MeterType, StatisticIcon*>> m_meter_stats;
 
     /** textually / numerically indicates population */
     MultiIconValueIndicator* m_multi_icon_value_indicator;
