@@ -15,13 +15,16 @@
 #include "../util/ScopedTimer.h"
 #include "../universe/Building.h"
 #include "../universe/ShipDesign.h"
+#include "../universe/Enums.h"
 
 #include <GG/DrawUtil.h>
+#include <GG/Layout.h>
 #include <GG/StaticGraphic.h>
 
 #include <boost/cast.hpp>
 
 #include <cmath>
+#include <iterator>
 
 
 namespace {
@@ -58,7 +61,8 @@ namespace {
             Resize(GG::Pt(nwidth, text->Height()));
         }
 
-        void Render() {}
+        void Render() override
+        {}
     };
 
     //////////////
@@ -69,20 +73,13 @@ namespace {
         QuantRow(int quantity, int designID, GG::X nwidth, GG::Y h,
                  bool inProgress, bool amBlockType) :
             GG::ListBox::Row(),
-            width(0),
             m_quant(quantity)
         {
             QuantLabel* newLabel = new QuantLabel(m_quant, designID, nwidth, h, inProgress, amBlockType);
-            width = newLabel->Width();
-            height = newLabel->Height();
             push_back(newLabel);
-            Resize(GG::Pt(nwidth, height-GG::Y0));//might subtract more; assessing aesthetics
-            //OffsetMove(GG::Pt(GG::X0, GG::Y(-2))); // didn't appear to have an effect
+            Resize(GG::Pt(nwidth, newLabel->Height()-GG::Y0));//might subtract more; assessing aesthetics
         }
 
-        GG::X width;
-        GG::Y height;
-
         int Quant() const { return m_quant; }
 
     private:
@@ -127,16 +124,15 @@ namespace {
             else
                 myQuantSet.insert(quantity);
 
-            for (std::set<int>::iterator it = myQuantSet.begin(); it != myQuantSet.end(); ++it)
-            {
-                QuantRow* row =  new QuantRow(*it, build.item.design_id, nwidth, h, inProgress, amBlockType);
+            for (int quantity : myQuantSet) {
+                QuantRow* row =  new QuantRow(quantity, build.item.design_id, nwidth, h, inProgress, amBlockType);
                 GG::DropDownList::iterator latest_it = Insert(row);
 
                 if (amBlockType) {
-                    if (build.blocksize == *it)
+                    if (build.blocksize == quantity)
                         Select(latest_it);
                 } else {
-                    if (build.remaining == *it)
+                    if (build.remaining == quantity)
                         Select(latest_it);
                 }
             }
@@ -161,7 +157,7 @@ namespace {
         }
 
     private:
-        const ProductionQueue::Element m_build;
+        const ProductionQueue::Element elem;
         int     quantity;
         int     prevQuant;
         int     blocksize;
@@ -170,17 +166,17 @@ namespace {
         bool    amOn;
         GG::Y   h;
 
-        void LosingFocus() {
+        void LosingFocus() override {
             amOn = false;
             DropDownList::LosingFocus();
         }
 
-        void LButtonDown(const GG::Pt&  pt, GG::Flags<GG::ModKey> mod_keys) {
+        void LButtonDown(const GG::Pt&  pt, GG::Flags<GG::ModKey> mod_keys) override {
             amOn = !amOn;
             DropDownList::LButtonDown(pt, mod_keys);
         }
 
-        void LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) {
+        void LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override {
             if (this->Disabled())
                 return;
 
@@ -195,22 +191,28 @@ namespace {
     //////////////////////////////////////////////////
     class QueueProductionItemPanel : public GG::Control {
     public:
-        QueueProductionItemPanel(GG::X w, const ProductionQueue::Element& build, double turn_cost, double total_cost,
-                                 int turns, int number, int turns_completed, double partially_complete_turn);
+        QueueProductionItemPanel(GG::X x, GG::Y y, GG::X w,
+                                 const ProductionQueue::Element& build, double turn_cost, double total_cost,
+                                 int turns, int number, double completed_progress);
+
+        void PreRender() override;
+        void Render() override;
+        void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
 
-        virtual void    Render();
         void            UpdateQueue();
         void            ItemQuantityChanged(int quant, int blocksize);
         void            ItemBlocksizeChanged(int quant, int blocksize);
-        virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr);
+
+        static GG::Y    DefaultHeight();
 
         mutable boost::signals2::signal<void(int,int)>    PanelUpdateQuantSignal;
 
     private:
         void            Draw(GG::Clr clr, bool fill);
         void            DoLayout();
+        void            Init();
 
-        const ProductionQueue::Element  m_build;
+        const ProductionQueue::Element  elem;
         GG::Label*                      m_name_text;
         GG::Label*                      m_location_text;
         GG::Label*                      m_PPs_and_turns_text;
@@ -221,15 +223,17 @@ namespace {
         QuantitySelector*               m_block_size_selector;
         bool                            m_in_progress;
         int                             m_total_turns;
-        int                             m_turns_completed;
-        double                          m_partially_complete_turn;
+        double                          m_turn_spending;
+        double                          m_total_cost;
+        double                          m_completed_progress;
         bool                            m_order_issuing_enabled;
+        bool                            m_initialized;
     };
 
     /////////////////////////////
     // ProductionItemBrowseWnd //
     /////////////////////////////
-    boost::shared_ptr<GG::BrowseInfoWnd> ProductionItemBrowseWnd(const ProductionQueue::Element& elem) {
+    std::shared_ptr<GG::BrowseInfoWnd> ProductionItemBrowseWnd(const ProductionQueue::Element& elem) {
         //const Empire* empire = GetEmpire(elem.empire_id);
 
         std::string main_text;
@@ -238,16 +242,14 @@ namespace {
         //int min_turns = 1;
         float total_cost = 0.0f;
         float max_allocation = 0.0f;
-        boost::shared_ptr<GG::Texture> icon;
+        std::shared_ptr<GG::Texture> icon;
         //bool available = false;
         bool location_ok = false;
 
         if (elem.item.build_type == BT_BUILDING) {
             const BuildingType* building_type = GetBuildingType(elem.item.name);
-            if (!building_type) {
-                boost::shared_ptr<GG::BrowseInfoWnd> browse_wnd;
-                return browse_wnd;
-            }
+            if (!building_type)
+                return nullptr;
             main_text += UserString("OBJ_BUILDING") + "\n";
 
             item_name = UserString(elem.item.name);
@@ -260,10 +262,8 @@ namespace {
 
         } else if (elem.item.build_type == BT_SHIP) {
             const ShipDesign* design = GetShipDesign(elem.item.design_id);
-            if (!design) {
-                boost::shared_ptr<GG::BrowseInfoWnd> browse_wnd;
-                return browse_wnd;
-            }
+            if (!design)
+                return nullptr;
             main_text += UserString("OBJ_SHIP") + "\n";
 
             item_name = design->Name(true);
@@ -275,12 +275,12 @@ namespace {
             icon = ClientUI::ShipDesignIcon(elem.item.design_id);
         }
 
-        if (TemporaryPtr<UniverseObject> rally_object = GetUniverseObject(elem.rally_point_id)) {
+        if (std::shared_ptr<UniverseObject> rally_object = GetUniverseObject(elem.rally_point_id)) {
             main_text += boost::io::str(FlexibleFormat(UserString("PRODUCTION_QUEUE_RALLIED_TO"))
                                         % rally_object->Name()) + "\n";
         }
 
-        if (TemporaryPtr<UniverseObject> location = GetUniverseObject(elem.location))
+        if (std::shared_ptr<UniverseObject> location = GetUniverseObject(elem.location))
             main_text += boost::io::str(FlexibleFormat(UserString("PRODUCTION_QUEUE_ENQUEUED_ITEM_LOCATION"))
                                         % location->Name()) + "\n";
 
@@ -294,7 +294,7 @@ namespace {
 
         // %1% / %2%  +  %3% / %4% PP/turn
         main_text += boost::io::str(FlexibleFormat(UserString("PRODUCTION_WND_PROGRESS"))
-                        % DoubleToString(progress, 3, false)
+                        % DoubleToString(progress*100.0f, 3, false)
                         % DoubleToString(total_cost, 3, false)
                         % DoubleToString(allocation, 3, false)
                         % DoubleToString(max_allocation, 3, false)) + "\n";
@@ -306,122 +306,139 @@ namespace {
 
         std::string title_text;
         if (elem.blocksize > 1)
-            title_text = boost::lexical_cast<std::string>(elem.blocksize) + "x ";
+            title_text = std::to_string(elem.blocksize) + "x ";
         title_text += item_name;
 
-        boost::shared_ptr<GG::BrowseInfoWnd> browse_wnd(new IconTextBrowseWnd(icon, title_text, main_text));
-        return browse_wnd;
+        return std::make_shared<IconTextBrowseWnd>(icon, title_text, main_text);
     }
 
     //////////////////////////////////////////////////
     // QueueRow
     //////////////////////////////////////////////////
+    const int MARGIN = 2;
+    const GG::X X_MARGIN = GG::X(MARGIN);
+    const GG::Y Y_MARGIN = GG::Y(MARGIN);
+
     struct QueueRow : GG::ListBox::Row {
         QueueRow(GG::X w, const ProductionQueue::Element& elem, int queue_index_) :
-            GG::ListBox::Row(w, GG::Y1, BuildDesignatorWnd::PRODUCTION_ITEM_DROP_TYPE),
+            GG::ListBox::Row(w, QueueProductionItemPanel::DefaultHeight(),
+                             BuildDesignatorWnd::PRODUCTION_ITEM_DROP_TYPE),
+            panel(nullptr),
             queue_index(queue_index_),
-            m_build(elem)
-        {
-            //std::cout << "QueueRow(" << w << ", ...)" << std::endl;
+            elem(elem)
+        { RequirePreRender(); }
 
+        void Init() {
             const Empire* empire = GetEmpire(HumanClientApp::GetApp()->EmpireID());
             float total_cost(1.0f);
             int minimum_turns(1);
             if (empire)
-                boost::tie(total_cost, minimum_turns) = empire->ProductionCostAndTime(elem);
+                std::tie(total_cost, minimum_turns) = empire->ProductionCostAndTime(elem);
             total_cost *= elem.blocksize;
-            float per_turn_cost = total_cost / std::max(1, minimum_turns);
-            float progress = empire ? empire->ProductionStatus(queue_index) : 0.0f;
-            if (progress == -1.0f)
-                progress = 0.0f;
 
-            m_panel = new QueueProductionItemPanel(w, elem, elem.allocated_pp, total_cost, minimum_turns, elem.remaining,
-                                                   static_cast<int>(progress / std::max(1e-6f, per_turn_cost)),
-                                                   std::fmod(progress, per_turn_cost) / std::max(1e-6f, per_turn_cost));
-            Resize(m_panel->Size());
-            push_back(m_panel);
+            float pp_accumulated = empire ? empire->ProductionStatus(queue_index) : 0.0f; // returns as PP
+            if (pp_accumulated == -1.0f)
+                pp_accumulated = 0.0f;
+
+            panel = new QueueProductionItemPanel(GG::X0, GG::Y0, ClientWidth() - MARGIN - MARGIN,
+                                                 elem, elem.allocated_pp, total_cost, minimum_turns, elem.remaining,
+                                                 pp_accumulated);
+            push_back(panel);
+
+            // Since this is only called during PreRender force panel to PreRender()
+            GG::GUI::PreRenderWindow(panel);
+            GetLayout()->PreRender();
 
             SetDragDropDataType(BuildDesignatorWnd::PRODUCTION_ITEM_DROP_TYPE);
 
             SetBrowseModeTime(GetOptionsDB().Get<int>("UI.tooltip-delay"));
             SetBrowseInfoWnd(ProductionItemBrowseWnd(elem));
 
-            GG::Connect(m_panel->PanelUpdateQuantSignal,  &QueueRow::RowQuantChanged, this);
+            GG::Connect(panel->PanelUpdateQuantSignal,  &QueueRow::RowQuantChanged, this);
+
         }
 
-        virtual void Disable(bool b) {
+        void PreRender() override {
+            GG::ListBox::Row::PreRender();
+
+            if (!panel)
+                Init();
+        }
+
+        void Disable(bool b) override {
             GG::ListBox::Row::Disable(b);
 
-            for (std::vector<GG::Control*>::iterator it = this->m_cells.begin();
-                 it != this->m_cells.end(); ++it)
-            {
-                GG::Control* ctrl = *it;
+            for (GG::Control* ctrl : m_cells) {
                 if (ctrl)
                     ctrl->Disable(this->Disabled());
             }
         }
 
-        virtual void SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
-            const GG::Pt old_size = Size();
-            GG::ListBox::Row::SizeMove(ul, lr);
-            if (!empty() && old_size != Size() && m_panel) {
-                //std::cout << "QueueRow resized to: " << Size() << std::endl;
-                m_panel->Resize(Size());
-            }
-        }
-
         void RowQuantChanged(int quantity, int blocksize) {
             if (this->Disabled())
                 return;
             RowQuantChangedSignal(queue_index, quantity, blocksize);
         }
 
-        QueueProductionItemPanel*                           m_panel;
+        QueueProductionItemPanel*                           panel;
         const int                                           queue_index;
-        const ProductionQueue::Element                      m_build;
+        const ProductionQueue::Element                      elem;
         mutable boost::signals2::signal<void (int,int,int)> RowQuantChangedSignal;
     };
 
     //////////////////////////////////////////////////
     // QueueProductionItemPanel implementation
     //////////////////////////////////////////////////
-    const int MARGIN = 2;
-
-    QueueProductionItemPanel::QueueProductionItemPanel(GG::X w, const ProductionQueue::Element& build,
+    QueueProductionItemPanel::QueueProductionItemPanel(GG::X x, GG::Y y, GG::X w, const ProductionQueue::Element& build,
                                                        double turn_spending, double total_cost, int turns, int number,
-                                                       int turns_completed, double partially_complete_turn) :
-        GG::Control(GG::X0, GG::Y0, w, GG::Y(10), GG::NO_WND_FLAGS),
-        m_build(build),
-        m_name_text(0),
-        m_location_text(0),
-        m_PPs_and_turns_text(0),
-        m_turns_remaining_until_next_complete_text(0),
-        m_icon(0),
-        m_progress_bar(0),
-        m_quantity_selector(0),
-        m_block_size_selector(0),
+                                                       double completed_progress) :
+        GG::Control(x, y, w, DefaultHeight(), GG::NO_WND_FLAGS),
+        elem(build),
+        m_name_text(nullptr),
+        m_location_text(nullptr),
+        m_PPs_and_turns_text(nullptr),
+        m_turns_remaining_until_next_complete_text(nullptr),
+        m_icon(nullptr),
+        m_progress_bar(nullptr),
+        m_quantity_selector(nullptr),
+        m_block_size_selector(nullptr),
         m_in_progress(build.allocated_pp || build.turns_left_to_next_item == 1),
         m_total_turns(turns),
-        m_turns_completed(turns_completed),
-        m_partially_complete_turn(partially_complete_turn)
+        m_turn_spending(turn_spending),
+        m_total_cost(total_cost),
+        m_completed_progress(completed_progress),
+        m_initialized(false)
     {
+        SetChildClippingMode(ClipToClient);
+        RequirePreRender();
+    }
+
+    GG::Y QueueProductionItemPanel::DefaultHeight() {
+        const int FONT_PTS = ClientUI::Pts();
+        const GG::Y HEIGHT = Y_MARGIN + FONT_PTS + Y_MARGIN + FONT_PTS + Y_MARGIN + FONT_PTS + Y_MARGIN + 6;
+        return HEIGHT;
+    }
+
+    void QueueProductionItemPanel::Init() {
+        m_initialized = true;
+
         GG::Clr clr = m_in_progress
             ? GG::LightColor(ClientUI::ResearchableTechTextAndBorderColor())
             : ClientUI::ResearchableTechTextAndBorderColor();
 
         // get graphic and player-visible name text for item
-        boost::shared_ptr<GG::Texture> graphic;
+        std::shared_ptr<GG::Texture> graphic;
         std::string name_text;
-        if (build.item.build_type == BT_BUILDING) {
-            graphic = ClientUI::BuildingIcon(build.item.name);
-            name_text = UserString(build.item.name);
-        } else if (build.item.build_type == BT_SHIP) {
-            graphic = ClientUI::ShipDesignIcon(build.item.design_id);
-            const ShipDesign* design = GetShipDesign(build.item.design_id);
+        if (elem.item.build_type == BT_BUILDING) {
+            graphic = ClientUI::BuildingIcon(elem.item.name);
+            name_text = UserString(elem.item.name);
+        } else if (elem.item.build_type == BT_SHIP) {
+            graphic = ClientUI::ShipDesignIcon(elem.item.design_id);
+            const ShipDesign* design = GetShipDesign(elem.item.design_id);
             if (design)
                 name_text = design->Name();
             else
-                ErrorLogger() << "QueueProductionItemPanel unable to get design with id: " << build.item.design_id;
+                ErrorLogger() << "QueueProductionItemPanel unable to get design with id: " << elem.item.design_id;
         } else {
             graphic = ClientUI::GetTexture(""); // get "missing texture" texture by supply intentionally bad path
             name_text = UserString("FW_UNKNOWN_DESIGN_NAME");
@@ -430,24 +447,22 @@ namespace {
         // get location indicator text
         std::string location_text;
         bool system_selected = false;
-        if (TemporaryPtr<const UniverseObject> location = GetUniverseObject(build.location)) {
+        if (std::shared_ptr<const UniverseObject> location = GetUniverseObject(elem.location)) {
             system_selected = (location->SystemID() != -1 && location ->SystemID() == SidePanel::SystemID());
             if (GetOptionsDB().Get<bool>("UI.show-production-location-on-queue"))
                 location_text = boost::io::str(FlexibleFormat(UserString("PRODUCTION_QUEUE_ITEM_LOCATION")) % location->Name()) + " ";
         }
 
-
         const int FONT_PTS = ClientUI::Pts();
 
-
-        m_icon = 0;
+        m_icon = nullptr;
         if (graphic)
             m_icon = new GG::StaticGraphic(graphic, GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE);
 
-        if (m_build.item.build_type == BT_SHIP) {
-            m_quantity_selector = new QuantitySelector(m_build, GG::X1, GG::Y(MARGIN), GG::Y(FONT_PTS-2*MARGIN),
+        if (elem.item.build_type == BT_SHIP) {
+            m_quantity_selector = new QuantitySelector(elem, GG::X1, GG::Y(MARGIN), GG::Y(FONT_PTS-2*MARGIN),
                                                        m_in_progress, GG::X(FONT_PTS*2.5), false);
-            m_block_size_selector = new QuantitySelector(m_build, GG::X1, GG::Y(MARGIN), GG::Y(FONT_PTS-2*MARGIN),
+            m_block_size_selector = new QuantitySelector(elem, GG::X1, GG::Y(MARGIN), GG::Y(FONT_PTS-2*MARGIN),
                                                          m_in_progress, GG::X(FONT_PTS*2.5), true);
         }
 
@@ -466,22 +481,33 @@ namespace {
             m_location_text->SetTextColor(location_clr);
         }
 
-        m_progress_bar = new MultiTurnProgressBar(turns, turns_completed + partially_complete_turn, total_cost, turn_spending,
+        double perc_complete = 1.0;
+        double next_progress = 0.0;
+        if (m_total_cost > 0.0) {
+            perc_complete = m_completed_progress / m_total_cost;
+            next_progress = m_turn_spending / std::max(m_turn_spending, m_total_cost);
+        }
+
+        GG::Clr outline_color = ClientUI::ResearchableTechFillColor();
+        if (m_in_progress)
+            outline_color = GG::LightColor(outline_color);
+
+        m_progress_bar = new MultiTurnProgressBar(m_total_turns,
+                                                  perc_complete,
+                                                  next_progress,
                                                   GG::LightColor(ClientUI::TechWndProgressBarBackgroundColor()),
                                                   ClientUI::TechWndProgressBarColor(),
-                                                  m_in_progress
-                                                    ? ClientUI::ResearchableTechFillColor()
-                                                    : GG::LightColor(ClientUI::ResearchableTechFillColor()));
+                                                  outline_color);
 
-        double max_spending_per_turn = total_cost / m_total_turns;
+        double max_spending_per_turn = m_total_cost / m_total_turns;
         std::string turn_spending_text = boost::io::str(FlexibleFormat(UserString("PRODUCTION_TURN_COST_STR"))
-            % DoubleToString(turn_spending, 3, false)
+            % DoubleToString(m_turn_spending, 3, false)
             % DoubleToString(max_spending_per_turn, 3, false));
         m_PPs_and_turns_text = new CUILabel(turn_spending_text, GG::FORMAT_LEFT);
         m_PPs_and_turns_text->SetTextColor(clr);
 
 
-        int turns_left = build.turns_left_to_next_item;
+        int turns_left = elem.turns_left_to_next_item;
         std::string turns_left_text = turns_left < 0
             ? UserString("PRODUCTION_TURNS_LEFT_NEVER")
             : str(FlexibleFormat(UserString("PRODUCTION_TURNS_LEFT_STR")) % turns_left);
@@ -505,8 +531,17 @@ namespace {
         if (m_block_size_selector)
             GG::Connect(m_block_size_selector->QuantChangedSignal,  &QueueProductionItemPanel::ItemBlocksizeChanged, this);
 
-        const GG::Y HEIGHT = GG::Y(MARGIN) + FONT_PTS + MARGIN + FONT_PTS + MARGIN + FONT_PTS + MARGIN + 6;
-        Resize(GG::Pt(w, HEIGHT));
+        // Since this is only called during PreRender force quantity indicators to PreRender()
+        GG::GUI::PreRenderWindow(m_quantity_selector);
+        GG::GUI::PreRenderWindow(m_block_size_selector);
+    }
+
+    void QueueProductionItemPanel::PreRender() {
+        if (!m_initialized)
+            Init();
+
+        GG::Wnd::PreRender();
+        DoLayout();
     }
 
     void QueueProductionItemPanel::DoLayout() {
@@ -532,14 +567,13 @@ namespace {
         }
         if (m_block_size_selector) {
             m_block_size_selector->MoveTo(GG::Pt(left, GG::Y(MARGIN)));
-            if (m_quantity_selector) {
-                left += m_quantity_selector->Width();
-            }
+            left += m_block_size_selector->Width();
         }
 
-        const GG::X NAME_WIDTH = Width() - left - MARGIN;
+        const GG::X NAME_WIDTH = Width() - left - MARGIN - 3;
         m_name_text->MoveTo(GG::Pt(left, top));
         m_name_text->Resize(GG::Pt(NAME_WIDTH, GG::Y(FONT_PTS + 2*MARGIN)));
+        m_name_text->SetChildClippingMode(ClipToClient);
 
         m_location_text->MoveTo(GG::Pt(left, top));
         m_location_text->Resize(GG::Pt(NAME_WIDTH, GG::Y(FONT_PTS + 2*MARGIN)));
@@ -562,10 +596,10 @@ namespace {
     }
 
     void QueueProductionItemPanel::ItemQuantityChanged(int quant, int blocksize)
-    { PanelUpdateQuantSignal(quant,m_build.blocksize); }
+    { PanelUpdateQuantSignal(quant,elem.blocksize); }
 
     void QueueProductionItemPanel::ItemBlocksizeChanged(int quant, int blocksize) // made separate funcion in case wna to do extra checking
-    { PanelUpdateQuantSignal(m_build.remaining, blocksize); }
+    { PanelUpdateQuantSignal(elem.remaining, blocksize); }
 
     void QueueProductionItemPanel::Render() {
         GG::Clr fill = m_in_progress
@@ -589,14 +623,15 @@ namespace {
     void QueueProductionItemPanel::Draw(GG::Clr clr, bool fill) {
         const int CORNER_RADIUS = 7;
         glColor(clr);
-        PartlyRoundedRect(UpperLeft(), LowerRight(), CORNER_RADIUS, true, false, true, false, fill);
+        GG::Pt LINE_WIDTH(GG::X(3), GG::Y0);
+        PartlyRoundedRect(UpperLeft(), LowerRight() - LINE_WIDTH, CORNER_RADIUS, true, false, true, false, fill);
     }
 
     void QueueProductionItemPanel::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
-            const GG::Pt old_size = Size();
-            GG::Control::SizeMove(ul, lr);
-            if (Size() != old_size)
-                DoLayout();
+        const GG::Pt old_size = Size();
+        GG::Control::SizeMove(ul, lr);
+        if (Size() != old_size)
+            RequirePreRender();
     }
 
     class ProdQueueListBox : public QueueListBox {
@@ -605,12 +640,13 @@ namespace {
             QueueListBox(BuildDesignatorWnd::PRODUCTION_ITEM_DROP_TYPE,  UserString("PRODUCTION_QUEUE_PROMPT"))
         {}
 
-        boost::signals2::signal<void (GG::ListBox::iterator, int)>   QueueItemRalliedToSignal;
-        boost::signals2::signal<void ()>                             ShowPediaSignal;
+        boost::signals2::signal<void (GG::ListBox::iterator, int)>  QueueItemRalliedToSignal;
+        boost::signals2::signal<void ()>                            ShowPediaSignal;
+        boost::signals2::signal<void (GG::ListBox::iterator, bool)> QueueItemPausedSignal;
 
     protected:
-        void ItemRightClickedImpl(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys) {
-            // mostly duplicated equivalent in QueueListBox, but with an extra command...
+        void ItemRightClickedImpl(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys) override {
+            // mostly duplicated equivalent in QueueListBox, but with extra commands...
 
             GG::MenuItem menu_contents;
             menu_contents.next_level.push_back(GG::MenuItem(UserString("MOVE_UP_QUEUE_ITEM"),   1, false, false));
@@ -619,33 +655,40 @@ namespace {
 
             // inspect clicked item: was it a ship?
             GG::ListBox::Row* row = *it;
-            QueueRow* queue_row = row ? dynamic_cast<QueueRow*>(row) : 0;
-            BuildType build_type = queue_row ? queue_row->m_build.item.build_type : INVALID_BUILD_TYPE;
+            QueueRow* queue_row = row ? dynamic_cast<QueueRow*>(row) : nullptr;
+            BuildType build_type = queue_row ? queue_row->elem.item.build_type : INVALID_BUILD_TYPE;
             if (build_type == BT_SHIP) {
-                if (TemporaryPtr<const System> system = GetSystem(SidePanel::SystemID())) {
+                // for ships, add a set rally point command
+                if (std::shared_ptr<const System> system = GetSystem(SidePanel::SystemID())) {
                     std::string rally_prompt = boost::io::str(FlexibleFormat(UserString("RALLY_QUEUE_ITEM")) % system->PublicName(HumanClientApp::GetApp()->EmpireID()));
-                    menu_contents.next_level.push_back(GG::MenuItem(rally_prompt,                   4, false, false));
+                    menu_contents.next_level.push_back(GG::MenuItem(rally_prompt,               4, false, false));
                 }
             }
+
+            // pause / resume commands
+            if (queue_row && queue_row->elem.paused) {
+                menu_contents.next_level.push_back(GG::MenuItem(UserString("RESUME"),           7, false, false));
+            } else {
+                menu_contents.next_level.push_back(GG::MenuItem(UserString("PAUSE"),            8, false, false));
+            }
+
             // pedia lookup
             std::string item_name = "";
             if (build_type == BT_BUILDING) {
-                item_name = queue_row->m_build.item.name;
+                item_name = queue_row->elem.item.name;
             } else if (build_type == BT_SHIP) {
-                item_name = GetShipDesign(queue_row->m_build.item.design_id)->Name(false);
+                item_name = GetShipDesign(queue_row->elem.item.design_id)->Name(false);
             } else {
                 ErrorLogger() << "Invalid build type (" << build_type << ") for row item";
                 return;
             }
-
             if (UserStringExists(item_name))
                 item_name = UserString(item_name);
-
             std::string popup_label = boost::io::str(FlexibleFormat(UserString("ENC_LOOKUP")) % item_name);
             menu_contents.next_level.push_back(GG::MenuItem(popup_label, 5, false, false));
 
-            GG::PopupMenu popup(pt.x, pt.y, ClientUI::GetFont(), menu_contents, ClientUI::TextColor(),
-                                ClientUI::WndOuterBorderColor(), ClientUI::WndColor(), ClientUI::EditHiliteColor());
+
+            CUIPopupMenu popup(pt.x, pt.y, menu_contents);
 
             if (popup.Run()) {
                 switch (popup.MenuID()) {
@@ -673,6 +716,16 @@ namespace {
                     this->LeftClickedSignal(it, pt, modkeys);
                     break;
                 }
+
+                case 7: { // resume
+                    this->QueueItemPausedSignal(it, false);
+                    break;
+                }
+                case 8: { // pause
+                    this->QueueItemPausedSignal(it, true);
+                    break;
+                }
+
                 default:
                     break;
                 }
@@ -690,14 +743,14 @@ public:
     ProductionQueueWnd(GG::X x, GG::Y y, GG::X w, GG::Y h) :
         CUIWnd("", x, y, w, h, GG::INTERACTIVE | GG::RESIZABLE | GG::DRAGABLE | GG::ONTOP | PINABLE,
                "production.ProductionQueueWnd"),
-        m_queue_lb(0)
+        m_queue_lb(nullptr)
     {
         Init(HumanClientApp::GetApp()->EmpireID());
     }
     //@}
 
     /** \name Mutators */ //@{
-    virtual void        SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override {
         GG::Pt sz = Size();
         CUIWnd::SizeMove(ul, lr);
         if (Size() != sz)
@@ -717,7 +770,7 @@ public:
 private:
     void DoLayout() {
         m_queue_lb->SizeMove(GG::Pt(GG::X0, GG::Y0),
-                                GG::Pt(ClientWidth(), ClientHeight() - GG::Y(CUIWnd::INNER_BORDER_ANGLE_OFFSET)));
+                             GG::Pt(ClientWidth(), ClientHeight() - GG::Y(CUIWnd::INNER_BORDER_ANGLE_OFFSET)));
     }
 
     void Init(int empire_id) {
@@ -739,9 +792,9 @@ private:
 //////////////////////////////////////////////////
 ProductionWnd::ProductionWnd(GG::X w, GG::Y h) :
     GG::Wnd(GG::X0, GG::Y0, w, h, GG::INTERACTIVE | GG::ONTOP),
-    m_production_info_panel(0),
-    m_queue_wnd(0),
-    m_build_designator_wnd(0),
+    m_production_info_panel(nullptr),
+    m_queue_wnd(nullptr),
+    m_build_designator_wnd(nullptr),
     m_order_issuing_enabled(false),
     m_empire_shown_id(ALL_EMPIRES)
 {
@@ -768,6 +821,7 @@ ProductionWnd::ProductionWnd(GG::X w, GG::Y h) :
     GG::Connect(m_queue_wnd->GetQueueListBox()->DoubleClickedSignal,        &ProductionWnd::QueueItemDoubleClickedSlot, this);
     GG::Connect(m_queue_wnd->GetQueueListBox()->QueueItemRalliedToSignal,   &ProductionWnd::QueueItemRallied, this);
     GG::Connect(m_queue_wnd->GetQueueListBox()->ShowPediaSignal,            &ProductionWnd::ShowPedia, this);
+    GG::Connect(m_queue_wnd->GetQueueListBox()->QueueItemPausedSignal,      &ProductionWnd::QueueItemPaused, this);
 
     AttachChild(m_production_info_panel);
     AttachChild(m_queue_wnd);
@@ -889,8 +943,10 @@ bool ProductionWnd::PediaVisible()
 void ProductionWnd::CenterOnBuild(int queue_idx, bool open)
 { m_build_designator_wnd->CenterOnBuild(queue_idx, open); }
 
-void ProductionWnd::SelectPlanet(int planet_id)
-{ m_build_designator_wnd->SelectPlanet(planet_id); }
+void ProductionWnd::SelectPlanet(int planet_id) {
+    m_build_designator_wnd->SelectPlanet(planet_id);
+    UpdateInfoPanel();
+}
 
 void ProductionWnd::SelectDefaultPlanet()
 { m_build_designator_wnd->SelectDefaultPlanet(); }
@@ -933,28 +989,43 @@ void ProductionWnd::UpdateQueue() {
     ScopedTimer timer("ProductionWnd::UpdateQueue");
 
     m_queue_wnd->SetEmpire(m_empire_shown_id);
-
     QueueListBox* queue_lb = m_queue_wnd->GetQueueListBox();
-    std::size_t first_visible_queue_row = std::distance(queue_lb->begin(), queue_lb->FirstRowShown());
+
+    // Capture the list scroll state
+    // Try to preserve the same queue context with completely new queue items
+    std::size_t initial_offset_from_begin = std::distance(queue_lb->begin(), queue_lb->FirstRowShown());
+    std::size_t initial_offset_to_end = std::distance(queue_lb->FirstRowShown(), queue_lb->end());
+    bool initial_last_visible_row_is_end(queue_lb->LastVisibleRow() == queue_lb->end());
+
     queue_lb->Clear();
 
     const Empire* empire = GetEmpire(m_empire_shown_id);
     if (!empire)
         return;
 
-    const ProductionQueue& queue = empire->GetProductionQueue();
-
     int i = 0;
-    for (ProductionQueue::const_iterator it = queue.begin(); it != queue.end(); ++it, ++i) {
-        QueueRow* row = new QueueRow(queue_lb->RowWidth(), *it, i);
+    for (const ProductionQueue::Element& elem : empire->GetProductionQueue()) {
+        QueueRow* row = new QueueRow(queue_lb->RowWidth(), elem, i);
         GG::Connect(row->RowQuantChangedSignal,     &ProductionWnd::ChangeBuildQuantityBlockSlot, this);
         queue_lb->Insert(row);
+        ++i;
     }
 
-    if (!queue_lb->Empty())
-        queue_lb->BringRowIntoView(--queue_lb->end());
-    if (first_visible_queue_row < queue_lb->NumRows())
-        queue_lb->BringRowIntoView(boost::next(queue_lb->begin(), first_visible_queue_row));
+    // Restore the list scroll state
+    // If we were at the top stay at the top
+    if (initial_offset_from_begin == 0)
+        queue_lb->SetFirstRowShown(queue_lb->begin());
+
+    // If we were not at the bottom then keep the same first row position
+    else if (!initial_last_visible_row_is_end && initial_offset_from_begin < queue_lb->NumRows())
+        queue_lb->SetFirstRowShown(std::next(queue_lb->begin(), initial_offset_from_begin));
+
+    // otherwise keep the same relative position from the bottom to
+    // preserve the end of list dead space
+    else if (initial_offset_to_end < queue_lb->NumRows())
+        queue_lb->SetFirstRowShown(std::next(queue_lb->begin(), queue_lb->NumRows() - initial_offset_to_end));
+    else
+        queue_lb->SetFirstRowShown(queue_lb->begin());
 }
 
 void ProductionWnd::UpdateInfoPanel() {
@@ -974,7 +1045,7 @@ void ProductionWnd::UpdateInfoPanel() {
 
     // find if there is a local location
     int prod_loc_id = this->SelectedPlanetID();
-    TemporaryPtr<UniverseObject> loc_obj = GetUniverseObject(prod_loc_id);
+    std::shared_ptr<UniverseObject> loc_obj = GetUniverseObject(prod_loc_id);
     if (loc_obj) {
         // extract available and allocated PP at production location
         std::map<std::set<int>, float> available_pp = queue.AvailablePP(empire->GetResourcePool(RE_INDUSTRY));
@@ -982,19 +1053,16 @@ void ProductionWnd::UpdateInfoPanel() {
 
         float available_pp_at_loc = 0.0f, allocated_pp_at_loc = 0.0f;   // for the resource sharing group containing the selected production location
 
-        for (std::map<std::set<int>, float>::const_iterator map_it = available_pp.begin();
-             map_it != available_pp.end(); ++map_it)
-        {
-            if (map_it->first.find(prod_loc_id) != map_it->first.end()) {
-                available_pp_at_loc = map_it->second;
+        for (const std::map<std::set<int>, float>::value_type& map : available_pp) {
+            if (map.first.find(prod_loc_id) != map.first.end()) {
+                available_pp_at_loc = map.second;
                 break;
             }
         }
-                for (std::map<std::set<int>, float>::const_iterator map_it = allocated_pp.begin();
-             map_it != allocated_pp.end(); ++map_it)
-        {
-            if (map_it->first.find(prod_loc_id) != map_it->first.end()) {
-                allocated_pp_at_loc = map_it->second;
+
+        for (const std::map<std::set<int>, float>::value_type& map : allocated_pp) {
+            if (map.first.find(prod_loc_id) != map.first.end()) {
+                allocated_pp_at_loc = map.second;
                 break;
             }
         }
@@ -1101,6 +1169,21 @@ void ProductionWnd::QueueItemRallied(GG::ListBox::iterator it, int object_id) {
     empire->UpdateProductionQueue();
 }
 
+void ProductionWnd::QueueItemPaused(GG::ListBox::iterator it, bool pause) {
+    if (!m_order_issuing_enabled)
+        return;
+    int client_empire_id = HumanClientApp::GetApp()->EmpireID();
+    Empire* empire = GetEmpire(client_empire_id);
+    if (!empire)
+        return;
+
+    HumanClientApp::GetApp()->Orders().IssueOrder(
+        OrderPtr(new ProductionQueueOrder(client_empire_id, std::distance(m_queue_wnd->GetQueueListBox()->begin(), it),
+                                          pause, -1.0f)));
+
+    empire->UpdateProductionQueue();
+}
+
 void ProductionWnd::EnableOrderIssuing(bool enable/* = true*/) {
     m_order_issuing_enabled = enable;
     m_queue_wnd->GetQueueListBox()->EnableOrderIssuing(m_order_issuing_enabled);
