@@ -2,7 +2,7 @@
 #define _ProductionWnd_h_
 
 #include "CUIWnd.h"
-#include "../universe/Enums.h"
+#include "../universe/EnumsFwd.h"
 #include "../Empire/Empire.h"
 
 #include <GG/ListBox.h>
@@ -16,6 +16,7 @@ class ProductionWnd : public GG::Wnd {
 public:
     /** \name Structors */ //@{
     ProductionWnd(GG::X w, GG::Y h);
+
     virtual ~ProductionWnd();
     //@}
 
@@ -25,11 +26,13 @@ public:
     //@}
 
     /** \name Mutators */ //@{
-    virtual bool    InWindow(const GG::Pt& pt) const;
-    virtual bool    InClient(const GG::Pt& pt) const;
-    virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr);
+    bool InWindow(const GG::Pt& pt) const override;
 
-    void            Render();
+    bool InClient(const GG::Pt& pt) const override;
+
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
+
+    void Render() override;
 
     void            SetEmpireShown(int empire_id);
 
@@ -122,6 +125,7 @@ private:
     void    QueueItemClickedSlot(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys);
     void    QueueItemDoubleClickedSlot(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys);
     void    QueueItemRallied(GG::ListBox::iterator it, int object_id);
+    void    QueueItemPaused(GG::ListBox::iterator it, bool pause);
 
     ProductionInfoPanel*    m_production_info_panel;
     ProductionQueueWnd*     m_queue_wnd;
