@@ -14,7 +14,7 @@
 struct PromptRow : GG::ListBox::Row {
     PromptRow(GG::X w, std::string prompt_str) :
         GG::ListBox::Row(w, GG::Y(20), ""),
-        m_prompt(0)
+        m_prompt(nullptr)
     {
         //std::cout << "PromptRow(" << w << ", ...)" << std::endl;
 
@@ -27,7 +27,7 @@ struct PromptRow : GG::ListBox::Row {
         push_back(m_prompt);
     }
 
-    virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override {
         const GG::Pt old_size = Size();
         GG::ListBox::Row::SizeMove(ul, lr);
         if (!empty() && old_size != Size() && m_prompt)
@@ -58,32 +58,17 @@ QueueListBox::QueueListBox(const std::string& drop_type_str, const std::string&
     GG::Connect(ClearedSignal,                      &QueueListBox::ShowPromptSlot,              this);
     GG::Connect(GG::ListBox::RightClickedSignal,    &QueueListBox::ItemRightClicked,            this);
 
-    // preinitialize listbox/row column widths, because what
-    // ListBox::Insert does on default is not suitable for this case
     SetNumCols(1);
-    SetColWidth(0, GG::X0);
-    LockColWidths();
+    ManuallyManageColProps();
     NormalizeRowsOnInsert(false);
 
     ShowPromptSlot();
 }
 
 GG::X QueueListBox::RowWidth() const
-{ return Width() - ClientUI::ScrollWidth() - 5; }
+{ return ClientWidth(); }
 
-void QueueListBox::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
-    const GG::Pt old_size = Size();
-    CUIListBox::SizeMove(ul, lr);
-
-    if (old_size != Size()) {
-        for (GG::ListBox::iterator it = begin(); it != end(); ++it) {
-            GG::Pt old_row_sz = (*it)->Size();
-            (*it)->Resize(GG::Pt(Width() - ClientUI::ScrollWidth() - 5, old_row_sz.y));
-        }
-    }
-}
-
-void QueueListBox::KeyPress(GG::Key key, boost::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys)
+void QueueListBox::KeyPress(GG::Key key, std::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys)
 {
     if (Disabled()) {
         CUIListBox::KeyPress(key, key_code_point, mod_keys);
@@ -149,7 +134,7 @@ void QueueListBox::Render() {
         if (m_drop_point == end())
             ul.y = lr.y;
         if (!row->empty()) {
-            GG::Control* panel = (*row)[0];
+            GG::Control* panel =  row->at(0);
             ul.x = panel->Left();
             lr.x = panel->Right();
         }
@@ -157,6 +142,16 @@ void QueueListBox::Render() {
     }
 }
 
+void QueueListBox::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
+    const GG::Pt old_size = Size();
+    CUIListBox::SizeMove(ul, lr);
+    if (old_size != Size() && !Empty()) {
+        const GG::Pt row_size(RowWidth(), (*begin())->Height());
+        for (GG::ListBox::Row* row : *this)
+            row->Resize(row_size);
+    }
+}
+
 void QueueListBox::DragDropHere(const GG::Pt& pt, std::map<const GG::Wnd*, bool>& drop_wnds_acceptable,
                                  GG::Flags<GG::ModKey> mod_keys)
 {
@@ -181,8 +176,8 @@ void QueueListBox::DragDropLeave() {
 
 void QueueListBox::EnableOrderIssuing(bool enable/* = true*/) {
     m_order_issuing_enabled = enable;
-    for (GG::ListBox::iterator it = begin(); it != end(); ++it)
-        (*it)->Disable(!enable);
+    for (GG::ListBox::Row* row : *this)
+        row->Disable(!enable);
 }
 
 bool QueueListBox::DisplayingValidQueueItems()
@@ -202,8 +197,7 @@ void QueueListBox::ItemRightClickedImpl(GG::ListBox::iterator it, const GG::Pt&
     menu_contents.next_level.push_back(GG::MenuItem(UserString("MOVE_DOWN_QUEUE_ITEM"), 2, false, false));
     menu_contents.next_level.push_back(GG::MenuItem(UserString("DELETE_QUEUE_ITEM"),    3, false, false));
 
-    GG::PopupMenu popup(pt.x, pt.y, ClientUI::GetFont(), menu_contents, ClientUI::TextColor(),
-                        ClientUI::WndOuterBorderColor(), ClientUI::WndColor(), ClientUI::EditHiliteColor());
+    CUIPopupMenu popup(pt.x, pt.y, menu_contents);
 
     if (popup.Run()) {
         switch (popup.MenuID()) {
