@@ -6,6 +6,7 @@
 #include "TechTreeWnd.h"
 #include "../Empire/Empire.h"
 #include "../universe/Tech.h"
+#include "../universe/Enums.h"
 #include "../util/i18n.h"
 #include "../util/Order.h"
 #include "../util/OptionsDB.h"
@@ -13,11 +14,14 @@
 #include "../client/human/HumanClientApp.h"
 
 #include <GG/DrawUtil.h>
+#include <GG/Layout.h>
 #include <GG/StaticGraphic.h>
 
 #include <boost/cast.hpp>
 
 #include <cmath>
+#include <iterator>
+
 
 namespace {
     const float OUTER_LINE_THICKNESS = 2.0f;
@@ -34,9 +38,14 @@ namespace {
     //////////////////////////////////////////////////
     class QueueTechPanel : public GG::Control {
     public:
-        QueueTechPanel(GG::X w, const std::string& tech_name, double allocated_rp,
-                       int turns_left, double turns_completed, int empire_id);
-        virtual void Render();
+        QueueTechPanel(GG::X x, GG::Y y, GG::X w, const std::string& tech_name, double allocated_rp,
+                       int turns_left, double turns_completed, int empire_id, bool paused = false);
+
+        void Render() override;
+
+        void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
+
+        static GG::Y DefaultHeight();
 
     private:
         void Draw(GG::Clr clr, bool fill);
@@ -50,6 +59,7 @@ namespace {
         bool                    m_in_progress;
         int                     m_total_turns;
         int                     m_empire_id;
+        bool                    m_paused;
     };
 
     //////////////////////////////////////////////////
@@ -57,64 +67,90 @@ namespace {
     //////////////////////////////////////////////////
     struct QueueRow : GG::ListBox::Row {
         QueueRow(GG::X w, const ResearchQueue::Element& queue_element) :
-            GG::ListBox::Row(),
-            tech_name(queue_element.name)
+            GG::ListBox::Row(w, QueueTechPanel::DefaultHeight(), "RESEARCH_QUEUE_ROW"),
+            elem(queue_element),
+            panel(nullptr)
         {
-            const Empire* empire = GetEmpire(queue_element.empire_id);
+            RequirePreRender();
+            Resize(GG::Pt(w, QueueTechPanel::DefaultHeight()));
+        }
 
-            const Tech* tech = GetTech(tech_name);
-            double per_turn_cost = tech ? tech->PerTurnCost(queue_element.empire_id) : 1;
+        void Init() {
+            const Empire* empire = GetEmpire(elem.empire_id);
+
+            const Tech* tech = GetTech(elem.name);
+            double per_turn_cost = tech ? tech->PerTurnCost(elem.empire_id) : 1;
             double progress = 0.0;
-            if (empire && empire->TechResearched(tech_name))
-                progress = tech ? tech->ResearchCost(queue_element.empire_id) : 0.0;
+            if (empire && empire->TechResearched(elem.name))
+                progress = tech ? tech->ResearchCost(elem.empire_id) : 0.0;
             else if (empire)
-                progress = empire->ResearchProgress(tech_name);
+                progress = empire->ResearchProgress(elem.name);
 
-            GG::Control* panel = new QueueTechPanel(w, tech_name, queue_element.allocated_rp,
-                                                    queue_element.turns_left, progress / per_turn_cost,
-                                                    queue_element.empire_id);
-            Resize(panel->Size());
+            panel = new QueueTechPanel(GG::X(GetLayout()->BorderMargin()), GG::Y(GetLayout()->BorderMargin()),
+                                       ClientWidth(), elem.name, elem.allocated_rp,
+                                       elem.turns_left, progress / per_turn_cost,
+                                       elem.empire_id);
             push_back(panel);
 
             SetDragDropDataType("RESEARCH_QUEUE_ROW");
 
             SetBrowseModeTime(GetOptionsDB().Get<int>("UI.tooltip-delay"));
-            SetBrowseInfoWnd(TechPanelRowBrowseWnd(queue_element.name, queue_element.empire_id));
+            SetBrowseInfoWnd(TechPanelRowBrowseWnd(elem.name, elem.empire_id));
         }
 
-        std::string tech_name;
+        void PreRender() override {
+            GG::ListBox::Row::PreRender();
+
+            if (!panel)
+                Init();
+
+            GG::Pt border(GG::X(2 * GetLayout()->BorderMargin()), GG::Y(2 * GetLayout()->BorderMargin()));
+            panel->Resize(Size() - border);
+            GG::ListBox::Row::Resize(panel->Size() + border);
+        }
+
+        void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override {
+            if (panel) {
+                GG::Pt border(GG::X(2 * GetLayout()->BorderMargin()), GG::Y(2 * GetLayout()->BorderMargin()));
+                panel->Resize(lr - ul - border);
+            }
+            GG::ListBox::Row::SizeMove(ul, lr);
+        }
+
+        const ResearchQueue::Element elem;
+        GG::Control* panel;
     };
 
     //////////////////////////////////////////////////
     // QueueTechPanel implementation
     //////////////////////////////////////////////////
-    QueueTechPanel::QueueTechPanel(GG::X w, const std::string& tech_name, double turn_spending,
-                                   int turns_left, double turns_completed, int empire_id) :
-        GG::Control(GG::X0, GG::Y0, w, GG::Y(10), GG::NO_WND_FLAGS),
+    const int MARGIN = 2;
+
+    QueueTechPanel::QueueTechPanel(GG::X x, GG::Y y, GG::X w, const std::string& tech_name, double turn_spending,
+                                   int turns_left, double turns_completed, int empire_id, bool paused) :
+        GG::Control(x, y, w, DefaultHeight(), GG::NO_WND_FLAGS),
         m_tech_name(tech_name),
         m_in_progress(turn_spending),
         m_total_turns(1),
-        m_empire_id(empire_id)
+        m_empire_id(empire_id),
+        m_paused(paused)
     {
-        const int MARGIN = 2;
+        SetChildClippingMode(ClipToClient);
 
         const int FONT_PTS = ClientUI::Pts();
         const GG::Y METER_HEIGHT(FONT_PTS);
 
-        const GG::Y HEIGHT = MARGIN + FONT_PTS + MARGIN + METER_HEIGHT + MARGIN + FONT_PTS + MARGIN + 6;
+        // 9 pixels accounts for border thickness so the sharp-cornered icon doesn't with the rounded panel corner
+        const int GRAPHIC_SIZE = std::max(Value(DefaultHeight() - 9), 1);
 
-        const int GRAPHIC_SIZE = Value(HEIGHT - 9);    // 9 pixels accounts for border thickness so the sharp-cornered icon doesn't with the rounded panel corner
-
-        const GG::X NAME_WIDTH = w - GRAPHIC_SIZE - 2*MARGIN - 3;
-        const GG::X METER_WIDTH = w - GRAPHIC_SIZE - 4*MARGIN - 3;
-        const GG::X TURNS_AND_COST_WIDTH = NAME_WIDTH/2 - MARGIN;
+        const GG::X NAME_WIDTH  = std::max(Width() - GRAPHIC_SIZE - 4*MARGIN - 3, GG::X(1));
+        const GG::X METER_WIDTH = std::max(Width() - GRAPHIC_SIZE - 4*MARGIN - 3, GG::X(1));
+        const GG::X TURNS_AND_COST_WIDTH = std::max(NAME_WIDTH/2 - MARGIN, GG::X(1));
 
         const Tech* tech = GetTech(m_tech_name);
         if (tech)
             m_total_turns = tech->ResearchTime(m_empire_id);
 
-        Resize(GG::Pt(w, HEIGHT));
-
         GG::Clr clr = m_in_progress
                         ? GG::LightColor(ClientUI::ResearchableTechTextAndBorderColor())
                         : ClientUI::ResearchableTechTextAndBorderColor();
@@ -122,25 +158,40 @@ namespace {
         GG::Y top(MARGIN);
         GG::X left(MARGIN);
 
-
         m_icon = new GG::StaticGraphic(ClientUI::TechIcon(m_tech_name), GG::GRAPHIC_FITGRAPHIC);
         m_icon->MoveTo(GG::Pt(left, top));
         m_icon->Resize(GG::Pt(GG::X(GRAPHIC_SIZE), GG::Y(GRAPHIC_SIZE)));
         m_icon->SetColor(tech ? ClientUI::CategoryColor(tech->Category()) : GG::Clr());
         left += m_icon->Width() + MARGIN;
 
-        m_name_text = new CUILabel(UserString(m_tech_name), GG::FORMAT_TOP | GG::FORMAT_LEFT);
+        m_name_text = new CUILabel(m_paused ? UserString("PAUSED") : UserString(m_tech_name), GG::FORMAT_TOP | GG::FORMAT_LEFT);
         m_name_text->MoveTo(GG::Pt(left, top));
         m_name_text->Resize(GG::Pt(NAME_WIDTH, GG::Y(FONT_PTS + 2*MARGIN)));
         m_name_text->SetTextColor(clr);
         m_name_text->ClipText(true);
+        m_name_text->SetChildClippingMode(ClipToClient);
         top += m_name_text->Height();
 
-        m_progress_bar = new MultiTurnProgressBar(tech ? tech->ResearchTime(m_empire_id) : 1,
-                                                  turns_completed, tech ? tech->ResearchCost(m_empire_id) : 0.0, turn_spending,
+        int total_time = 0;
+        float perc_complete = 0.0f;
+        float next_progress = 0.0f;
+        if (tech) {
+            total_time = tech->ResearchTime(m_empire_id);
+            perc_complete = total_time > 0 ? turns_completed / total_time : 0.0f;
+            float total_cost = tech->ResearchCost(m_empire_id);
+            next_progress = turn_spending / std::max(1.0f, total_cost);
+        }
+        GG::Clr outline_color = ClientUI::ResearchableTechFillColor();
+        if (m_in_progress)
+            outline_color = GG::LightColor(outline_color);
+
+        m_progress_bar = new MultiTurnProgressBar(total_time,
+                                                  perc_complete,
+                                                  next_progress,
                                                   GG::LightColor(ClientUI::TechWndProgressBarBackgroundColor()),
                                                   ClientUI::TechWndProgressBarColor(),
-                                                  m_in_progress ? ClientUI::ResearchableTechFillColor() : GG::LightColor(ClientUI::ResearchableTechFillColor()) );
+                                                  outline_color);
+
         m_progress_bar->MoveTo(GG::Pt(left, top));
         m_progress_bar->Resize(GG::Pt(METER_WIDTH, METER_HEIGHT));
         top += m_progress_bar->Height() + MARGIN;
@@ -155,6 +206,8 @@ namespace {
         m_RPs_and_turns_text->MoveTo(GG::Pt(left, top));
         m_RPs_and_turns_text->Resize(GG::Pt(TURNS_AND_COST_WIDTH, GG::Y(FONT_PTS + MARGIN)));
         m_RPs_and_turns_text->SetTextColor(clr);
+        m_RPs_and_turns_text->ClipText(true);
+        m_RPs_and_turns_text->SetChildClippingMode(ClipToClient);
 
         left += TURNS_AND_COST_WIDTH;
 
@@ -165,6 +218,7 @@ namespace {
         m_turns_remaining_text->Resize(GG::Pt(TURNS_AND_COST_WIDTH, GG::Y(FONT_PTS + MARGIN)));
         m_turns_remaining_text->SetTextColor(clr);
         m_turns_remaining_text->ClipText(true);
+        m_turns_remaining_text->SetChildClippingMode(ClipToClient);
 
         AttachChild(m_name_text);
         AttachChild(m_RPs_and_turns_text);
@@ -191,7 +245,42 @@ namespace {
     void QueueTechPanel::Draw(GG::Clr clr, bool fill) {
         const int CORNER_RADIUS = 7;
         glColor(clr);
-        PartlyRoundedRect(UpperLeft(), LowerRight(), CORNER_RADIUS, true, false, true, false, fill);
+        GG::Pt LINE_WIDTH(GG::X(3), GG::Y0);
+        PartlyRoundedRect(UpperLeft(), LowerRight() - LINE_WIDTH, CORNER_RADIUS, true, false, true, false, fill);
+    }
+
+    void QueueTechPanel::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
+        GG::Pt old_size(Size());
+        GG::Control::SizeMove(ul, lr);
+        if (Size() != old_size) {
+            const int FONT_PTS = ClientUI::Pts();
+            const GG::Y METER_HEIGHT(FONT_PTS);
+
+            // 9 pixels accounts for border thickness so the sharp-cornered icon doesn't with the rounded panel corner
+            const int GRAPHIC_SIZE = std::max(Value(DefaultHeight() - 9), 1);
+
+            const GG::X NAME_WIDTH  = std::max(Width() - GRAPHIC_SIZE - 4*MARGIN - 3, GG::X(1));
+            const GG::X METER_WIDTH = std::max(Width() - GRAPHIC_SIZE - 4*MARGIN - 3, GG::X(1));
+            const GG::X TURNS_AND_COST_WIDTH = std::max(NAME_WIDTH/2 - MARGIN, GG::X(1));
+
+            m_name_text->Resize(GG::Pt(NAME_WIDTH, GG::Y(FONT_PTS + 2*MARGIN)));
+            m_progress_bar->Resize(GG::Pt(METER_WIDTH, METER_HEIGHT));
+            m_RPs_and_turns_text->Resize(GG::Pt(TURNS_AND_COST_WIDTH, GG::Y(FONT_PTS + MARGIN)));
+            m_turns_remaining_text->Resize(GG::Pt(TURNS_AND_COST_WIDTH, GG::Y(FONT_PTS + MARGIN)));
+            m_turns_remaining_text->MoveTo(
+                m_RPs_and_turns_text->RelativeUpperLeft() + GG::Pt(TURNS_AND_COST_WIDTH, GG::Y0));
+        }
+    }
+
+    GG::Y QueueTechPanel::DefaultHeight() {
+        const int MARGIN = 2;
+
+        const int FONT_PTS = ClientUI::Pts();
+        const GG::Y METER_HEIGHT(FONT_PTS);
+
+        const GG::Y HEIGHT = MARGIN + FONT_PTS + MARGIN + METER_HEIGHT + MARGIN + FONT_PTS + MARGIN + 6;
+
+        return HEIGHT;
     }
 }
 
@@ -204,11 +293,11 @@ public:
         QueueListBox(drop_type_str, prompt_str)
     {}
 
-    boost::signals2::signal<void (GG::ListBox::iterator, int)>   QueueItemRalliedToSignal;
-    boost::signals2::signal<void ()>                             ShowPediaSignal;
+    boost::signals2::signal<void ()>                            ShowPediaSignal;
+    boost::signals2::signal<void (GG::ListBox::iterator, bool)> QueueItemPausedSignal;
 
 protected:
-    void ItemRightClickedImpl(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys) {
+    void ItemRightClickedImpl(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys) override {
         // mostly duplicated equivalent in QueueListBox, but with an extra command...
 
         GG::MenuItem menu_contents;
@@ -217,23 +306,31 @@ protected:
         menu_contents.next_level.push_back(GG::MenuItem(UserString("DELETE_QUEUE_ITEM"),    3, false, false));
 
         GG::ListBox::Row* row = *it;
-        QueueRow* queue_row = row ? dynamic_cast<QueueRow*>(row) : 0;
+        QueueRow* queue_row = row ? dynamic_cast<QueueRow*>(row) : nullptr;
+        if (!queue_row)
+            return;
+
+        // pause / resume commands
+        if (queue_row->elem.paused) {
+            menu_contents.next_level.push_back(GG::MenuItem(UserString("RESUME"),           7, false, false));
+        } else {
+            menu_contents.next_level.push_back(GG::MenuItem(UserString("PAUSE"),            8, false, false));
+        }
 
         // pedia lookup
-        std::string tech_name = queue_row->tech_name;
-        if (tech_name == "") {
+        if (queue_row->elem.name.empty()) {
             ErrorLogger() << "Empty tech name referenced during right click";
             return;
         }
 
-        if (UserStringExists(tech_name))
-            tech_name = UserString(tech_name);
+        std::string tech_name;
+        if (UserStringExists(queue_row->elem.name))
+            tech_name = UserString(queue_row->elem.name);
 
         std::string popup_label = boost::io::str(FlexibleFormat(UserString("ENC_LOOKUP")) % tech_name);
         menu_contents.next_level.push_back(GG::MenuItem(popup_label, 4, false, false));
 
-        GG::PopupMenu popup(pt.x, pt.y, ClientUI::GetFont(), menu_contents, ClientUI::TextColor(),
-                            ClientUI::WndOuterBorderColor(), ClientUI::WndColor(), ClientUI::EditHiliteColor());
+        CUIPopupMenu popup(pt.x, pt.y, menu_contents);
 
         if (popup.Run()) {
             switch (popup.MenuID()) {
@@ -256,6 +353,17 @@ protected:
                 ShowPediaSignal();
                 break;
             }
+
+
+            case 7: { // resume
+                this->QueueItemPausedSignal(it, false);
+                break;
+            }
+            case 8: { // pause
+                this->QueueItemPausedSignal(it, true);
+                break;
+            }
+
             default:
                 break;
             }
@@ -272,14 +380,14 @@ public:
     ResearchQueueWnd(GG::X x, GG::Y y, GG::X w, GG::Y h) :
         CUIWnd("", x, y, w, h, GG::INTERACTIVE | GG::RESIZABLE | GG::DRAGABLE | GG::ONTOP | PINABLE,
                "research.ResearchQueueWnd"),
-        m_queue_lb(0)
+        m_queue_lb(nullptr)
     {
         Init(HumanClientApp::GetApp()->EmpireID());
     }
     //@}
 
     /** \name Mutators */ //@{
-    virtual void        SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override {
         GG::Pt sz = Size();
         CUIWnd::SizeMove(ul, lr);
         if (Size() != sz)
@@ -320,11 +428,11 @@ private:
 //////////////////////////////////////////////////
 // ResearchWnd                                  //
 //////////////////////////////////////////////////
-ResearchWnd::ResearchWnd(GG::X w, GG::Y h) :
+ResearchWnd::ResearchWnd(GG::X w, GG::Y h, bool initially_hidden /*= true*/) :
     GG::Wnd(GG::X0, GG::Y0, w, h, GG::INTERACTIVE | GG::ONTOP),
-    m_research_info_panel(0),
-    m_queue_wnd(0),
-    m_tech_tree_wnd(0),
+    m_research_info_panel(nullptr),
+    m_queue_wnd(nullptr),
+    m_tech_tree_wnd(nullptr),
     m_enabled(false),
     m_empire_shown_id(ALL_EMPIRES)
 {
@@ -334,13 +442,14 @@ ResearchWnd::ResearchWnd(GG::X w, GG::Y h) :
     m_research_info_panel = new ProductionInfoPanel(UserString("RESEARCH_WND_TITLE"), UserString("RESEARCH_INFO_RP"),
                                                     GG::X0, GG::Y0, GG::X(queue_width), GG::Y(100), "research.InfoPanel");
     m_queue_wnd = new ResearchQueueWnd(GG::X0, GG::Y(100), queue_width, GG::Y(ClientSize().y - 100));
-    m_tech_tree_wnd = new TechTreeWnd(tech_tree_wnd_size.x, tech_tree_wnd_size.y);
+    m_tech_tree_wnd = new TechTreeWnd(tech_tree_wnd_size.x, tech_tree_wnd_size.y, initially_hidden);
 
     GG::Connect(m_queue_wnd->GetQueueListBox()->QueueItemMovedSignal,   &ResearchWnd::QueueItemMoved,               this);
     GG::Connect(m_queue_wnd->GetQueueListBox()->QueueItemDeletedSignal, &ResearchWnd::DeleteQueueItem,              this);
     GG::Connect(m_queue_wnd->GetQueueListBox()->LeftClickedSignal,      &ResearchWnd::QueueItemClickedSlot,         this);
     GG::Connect(m_queue_wnd->GetQueueListBox()->DoubleClickedSignal,    &ResearchWnd::QueueItemDoubleClickedSlot,   this);
     GG::Connect(m_queue_wnd->GetQueueListBox()->ShowPediaSignal,        &ResearchWnd::ShowPedia,                    this);
+    GG::Connect(m_queue_wnd->GetQueueListBox()->QueueItemPausedSignal,  &ResearchWnd::QueueItemPaused,              this);
     GG::Connect(m_tech_tree_wnd->AddTechsToQueueSignal,                 &ResearchWnd::AddTechsToQueueSlot,          this);
 
     AttachChild(m_research_info_panel);
@@ -422,12 +531,11 @@ void ResearchWnd::TogglePedia()
 bool ResearchWnd::PediaVisible()
 { return m_tech_tree_wnd->PediaVisible(); }
 
-
 void ResearchWnd::QueueItemMoved(GG::ListBox::Row* row, std::size_t position) {
     if (QueueRow* queue_row = boost::polymorphic_downcast<QueueRow*>(row)) {
         int empire_id = HumanClientApp::GetApp()->EmpireID();
         HumanClientApp::GetApp()->Orders().IssueOrder(
-            OrderPtr(new ResearchQueueOrder(empire_id, queue_row->tech_name, position)));
+            OrderPtr(new ResearchQueueOrder(empire_id, queue_row->elem.name, static_cast<int>(position))));
         if (Empire* empire = GetEmpire(empire_id))
             empire->UpdateResearchQueue();
     }
@@ -466,11 +574,7 @@ void ResearchWnd::UpdateQueue() {
     if (!empire)
         return;
 
-    const ResearchQueue& queue = empire->GetResearchQueue();
-
-
-    for (ResearchQueue::const_iterator it = queue.begin(); it != queue.end(); ++it) {
-        const ResearchQueue::Element& elem = *it;
+    for (const ResearchQueue::Element& elem : empire->GetResearchQueue()) {
         QueueRow* row = new QueueRow(queue_lb->RowWidth(), elem);
         queue_lb->Insert(row);
     }
@@ -478,7 +582,7 @@ void ResearchWnd::UpdateQueue() {
     if (!queue_lb->Empty())
         queue_lb->BringRowIntoView(--queue_lb->end());
     if (first_visible_queue_row < queue_lb->NumRows())
-        queue_lb->BringRowIntoView(boost::next(queue_lb->begin(), first_visible_queue_row));
+        queue_lb->BringRowIntoView(std::next(queue_lb->begin(), first_visible_queue_row));
 }
 
 void ResearchWnd::UpdateInfoPanel() {
@@ -513,8 +617,7 @@ void ResearchWnd::AddTechsToQueueSlot(const std::vector<std::string>& tech_vec,
         return;
     const ResearchQueue& queue = empire->GetResearchQueue();
     OrderSet& orders = HumanClientApp::GetApp()->Orders();
-    for (std::vector<std::string>::const_iterator it = tech_vec.begin(); it != tech_vec.end(); ++it) {
-        const std::string& tech_name = *it;
+    for (const std::string& tech_name : tech_vec) {
         if (empire->TechResearched(tech_name))
             continue;
         // AddTechsToQueueSlot is currently used for (i) adding a tech and any not-yet-queued prereqs to the
@@ -549,7 +652,7 @@ void ResearchWnd::DeleteQueueItem(GG::ListBox::iterator it) {
     int empire_id = HumanClientApp::GetApp()->EmpireID();
     OrderSet& orders = HumanClientApp::GetApp()->Orders();
     if (QueueRow* queue_row = boost::polymorphic_downcast<QueueRow*>(*it))
-        orders.IssueOrder(OrderPtr(new ResearchQueueOrder(empire_id, queue_row->tech_name)));
+        orders.IssueOrder(OrderPtr(new ResearchQueueOrder(empire_id, queue_row->elem.name)));
     if (Empire* empire = GetEmpire(empire_id))
         empire->UpdateResearchQueue();
 }
@@ -559,16 +662,33 @@ void ResearchWnd::QueueItemClickedSlot(GG::ListBox::iterator it, const GG::Pt& p
         QueueRow* queue_row = boost::polymorphic_downcast<QueueRow*>(*it);
         if (!queue_row)
             return;
-        ShowTech(queue_row->tech_name);
+        ShowTech(queue_row->elem.name);
     }
 }
 
 void ResearchWnd::QueueItemDoubleClickedSlot(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys) {
-    if (m_queue_wnd->GetQueueListBox()->DisplayingValidQueueItems()) {
+    if (m_queue_wnd->GetQueueListBox()->DisplayingValidQueueItems())
         DeleteQueueItem(it);
-    }
 }
 
+void ResearchWnd::QueueItemPaused(GG::ListBox::iterator it, bool pause) {
+    if (!m_enabled)
+        return;
+    int client_empire_id = HumanClientApp::GetApp()->EmpireID();
+    Empire* empire = GetEmpire(client_empire_id);
+    if (!empire)
+        return;
+
+    // todo: reject action if shown queue is not this client's empire's queue
+
+    if (QueueRow* queue_row = boost::polymorphic_downcast<QueueRow*>(*it))
+        HumanClientApp::GetApp()->Orders().IssueOrder(
+            OrderPtr(new ResearchQueueOrder(client_empire_id, queue_row->elem.name, pause, -1.0f)));
+
+    empire->UpdateResearchQueue();
+}
+
+
 void ResearchWnd::EnableOrderIssuing(bool enable/* = true*/) {
     m_enabled = enable;
     m_queue_wnd->GetQueueListBox()->EnableOrderIssuing(m_enabled);
