@@ -6,8 +6,8 @@
 #include "../util/Logger.h"
 #include "../universe/Effect.h"
 #include "../universe/ResourceCenter.h"
-#include "../universe/TemporaryPtr.h"
 #include "../universe/UniverseObject.h"
+#include "../universe/Enums.h"
 #include "../client/human/HumanClientApp.h"
 #include "ClientUI.h"
 #include "CUIControls.h"
@@ -15,6 +15,7 @@
 #include "MultiIconValueIndicator.h"
 #include "MultiMeterStatusBar.h"
 
+
 namespace {
     const int       EDGE_PAD(3);
 
@@ -34,29 +35,48 @@ ResourcePanel::ResourcePanel(GG::X w, int object_id) :
     AccordionPanel(w, GG::Y(ClientUI::Pts()*2)),
     m_rescenter_id(object_id),
     m_meter_stats(),
-    m_multi_icon_value_indicator(0),
-    m_multi_meter_status_bar(0)
+    m_multi_icon_value_indicator(nullptr),
+    m_multi_meter_status_bar(nullptr)
 {
     SetName("ResourcePanel");
 
-    TemporaryPtr<const ResourceCenter> res = GetResCenter();
+    std::shared_ptr<const ResourceCenter> res = GetResCenter();
     if (!res)
         throw std::invalid_argument("Attempted to construct a ResourcePanel with an UniverseObject that is not a ResourceCenter");
 
     GG::Connect(m_expand_button->LeftClickedSignal, &ResourcePanel::ExpandCollapseButtonPressed, this);
 
+    const auto obj = GetUniverseObject(m_rescenter_id);
+    if (!obj) {
+        ErrorLogger() << "Invalid object id " << m_rescenter_id;
+        return;
+    }
+
     // small meter indicators - for use when panel is collapsed
-    m_meter_stats.push_back(std::make_pair(METER_INDUSTRY, new StatisticIcon(ClientUI::MeterIcon(METER_INDUSTRY), 0, 3, false)));
-    m_meter_stats.push_back(std::make_pair(METER_RESEARCH, new StatisticIcon(ClientUI::MeterIcon(METER_RESEARCH), 0, 3, false)));
-    m_meter_stats.push_back(std::make_pair(METER_TRADE, new StatisticIcon(ClientUI::MeterIcon(METER_TRADE), 0, 3, false)));
-    m_meter_stats.push_back(std::make_pair(METER_SUPPLY, new StatisticIcon(ClientUI::MeterIcon(METER_SUPPLY), 0, 3, false)));
+    m_meter_stats.push_back(
+        std::make_pair(METER_INDUSTRY, new StatisticIcon(ClientUI::MeterIcon(METER_INDUSTRY),
+                                                         obj->InitialMeterValue(METER_INDUSTRY), 3, false,
+                                                         GG::X0, GG::Y0, MeterIconSize().x, MeterIconSize().y)));
+    m_meter_stats.push_back(
+        std::make_pair(METER_RESEARCH, new StatisticIcon(ClientUI::MeterIcon(METER_RESEARCH),
+                                                         obj->InitialMeterValue(METER_RESEARCH), 3, false,
+                                                         GG::X0, GG::Y0, MeterIconSize().x, MeterIconSize().y)));
+    m_meter_stats.push_back(
+        std::make_pair(METER_TRADE, new StatisticIcon(ClientUI::MeterIcon(METER_TRADE),
+                                                      obj->InitialMeterValue(METER_TRADE), 3, false,
+                                                      GG::X0, GG::Y0, MeterIconSize().x, MeterIconSize().y)));
+    m_meter_stats.push_back(
+        std::make_pair(METER_SUPPLY, new StatisticIcon(ClientUI::MeterIcon(METER_SUPPLY),
+                                                       obj->InitialMeterValue(METER_SUPPLY), 3, false,
+                                                       GG::X0, GG::Y0, MeterIconSize().x, MeterIconSize().y)));
 
     // meter and production indicators
-    std::vector<std::pair<MeterType, MeterType> > meters;
+    std::vector<std::pair<MeterType, MeterType>> meters;
 
-    for (std::vector<std::pair<MeterType, StatisticIcon*> >::iterator it = m_meter_stats.begin(); it != m_meter_stats.end(); ++it) {
-        AttachChild(it->second);
-        meters.push_back(std::make_pair(it->first, AssociatedMeterType(it->first)));
+    for (std::pair<MeterType, StatisticIcon*>& meter_stat : m_meter_stats) {
+        meter_stat.second->InstallEventFilter(this);
+        AttachChild(meter_stat.second);
+        meters.push_back(std::make_pair(meter_stat.first, AssociatedMeterType(meter_stat.first)));
     }
 
     // attach and show meter bars and large resource indicators
@@ -76,8 +96,8 @@ ResourcePanel::~ResourcePanel() {
     delete m_multi_icon_value_indicator;
     delete m_multi_meter_status_bar;
 
-    for (std::vector<std::pair<MeterType, StatisticIcon*> >::iterator it = m_meter_stats.begin(); it != m_meter_stats.end(); ++it) {
-        delete it->second;
+    for (std::pair<MeterType, StatisticIcon*>& meter_stat : m_meter_stats) {
+        delete meter_stat.second;
     }
 }
 
@@ -85,9 +105,57 @@ void ResourcePanel::ExpandCollapse(bool expanded) {
     if (expanded == s_expanded_map[m_rescenter_id]) return; // nothing to do
     s_expanded_map[m_rescenter_id] = expanded;
 
-    DoLayout();
+    RequirePreRender();
 }
 
+bool ResourcePanel::EventFilter(GG::Wnd* w, const GG::WndEvent& event) {
+    if (event.Type() != GG::WndEvent::RClick)
+        return false;
+    const GG::Pt& pt = event.Point();
+
+    MeterType meter_type = INVALID_METER_TYPE;
+    for (const std::pair<MeterType, StatisticIcon*>& meter_stat : m_meter_stats) {
+        if (meter_stat.second == w) {
+            meter_type = meter_stat.first;
+            break;
+        }
+    }
+
+    if (meter_type == INVALID_METER_TYPE)
+        return false;
+
+    std::string meter_string = boost::lexical_cast<std::string>(meter_type);
+    std::string meter_title;
+    if (UserStringExists(meter_string))
+        meter_title = UserString(meter_string);
+    if (meter_title.empty())
+        return false;
+
+    GG::MenuItem menu_contents;
+
+    std::string popup_label = boost::io::str(FlexibleFormat(UserString("ENC_LOOKUP")) % meter_title);
+    menu_contents.next_level.push_back(GG::MenuItem(popup_label, 2, false, false));
+
+    CUIPopupMenu popup(pt.x, pt.y, menu_contents);
+
+    bool retval = false;
+
+    if (popup.Run()) {
+        switch (popup.MenuID()) {
+            case 2: {
+                retval = ClientUI::GetClientUI()->ZoomToMeterTypeArticle(meter_string);
+                break;
+            }
+            default:
+                break;
+        }
+    }
+
+    return retval;
+}
+
+
+
 namespace {
     bool sortByMeterValue(std::pair<MeterType, StatisticIcon*> left, std::pair<MeterType, StatisticIcon*> right) {
         if (left.second->GetValue() == right.second->GetValue()) {
@@ -106,12 +174,12 @@ namespace {
 
 void ResourcePanel::Update() {
     // remove any old browse wnds
-    for (std::vector<std::pair<MeterType, StatisticIcon*> >::iterator it = m_meter_stats.begin(); it != m_meter_stats.end(); ++it) {
-        it->second->ClearBrowseInfoWnd();
-        m_multi_icon_value_indicator->ClearToolTip(it->first);
+    for (std::pair<MeterType, StatisticIcon*>& meter_stat : m_meter_stats) {
+        meter_stat.second->ClearBrowseInfoWnd();
+        m_multi_icon_value_indicator->ClearToolTip(meter_stat.first);
     }
 
-    TemporaryPtr<const UniverseObject> obj = GetUniverseObject(m_rescenter_id);
+    std::shared_ptr<const UniverseObject> obj = GetUniverseObject(m_rescenter_id);
     if (!obj) {
         ErrorLogger() << "BuildingPanel::Update couldn't get object with id " << m_rescenter_id;
         return;
@@ -122,20 +190,28 @@ void ResourcePanel::Update() {
     m_multi_icon_value_indicator->Update();
 
     // tooltips
-    boost::shared_ptr<GG::BrowseInfoWnd> browse_wnd;
+    std::shared_ptr<GG::BrowseInfoWnd> browse_wnd;
 
-    for (std::vector<std::pair<MeterType, StatisticIcon*> >::iterator it = m_meter_stats.begin(); it != m_meter_stats.end(); ++it) {
-        it->second->SetValue(obj->InitialMeterValue(it->first));
+    for (std::pair<MeterType, StatisticIcon*>& meter_stat : m_meter_stats) {
+        meter_stat.second->SetValue(obj->InitialMeterValue(meter_stat.first));
 
-        browse_wnd = boost::shared_ptr<GG::BrowseInfoWnd>(new MeterBrowseWnd(m_rescenter_id, it->first, AssociatedMeterType(it->first)));
-        it->second->SetBrowseInfoWnd(browse_wnd);
-        m_multi_icon_value_indicator->SetToolTip(it->first, browse_wnd);
+        browse_wnd = std::make_shared<MeterBrowseWnd>(m_rescenter_id, meter_stat.first, AssociatedMeterType(meter_stat.first));
+        meter_stat.second->SetBrowseInfoWnd(browse_wnd);
+        m_multi_icon_value_indicator->SetToolTip(meter_stat.first, browse_wnd);
     }
 
     std::sort(m_meter_stats.begin(), m_meter_stats.end(), sortByMeterValue);
 }
 
 void ResourcePanel::Refresh() {
+    for (auto& meter_stat : m_meter_stats)
+        meter_stat.second->RequirePreRender();
+
+    RequirePreRender();
+}
+
+void ResourcePanel::PreRender() {
+    AccordionPanel::PreRender();
     Update();
     DoLayout();
 }
@@ -146,8 +222,8 @@ void ResourcePanel::ExpandCollapseButtonPressed()
 void ResourcePanel::DoLayout() {
     AccordionPanel::DoLayout();
 
-    for (std::vector<std::pair<MeterType, StatisticIcon*> >::iterator it = m_meter_stats.begin(); it != m_meter_stats.end(); ++it) {
-        DetachChild(it->second);
+    for (std::pair<MeterType, StatisticIcon*>& meter_stat : m_meter_stats) {
+        DetachChild(meter_stat.second);
     }
 
     // detach / hide meter bars and large resource indicators
@@ -158,16 +234,19 @@ void ResourcePanel::DoLayout() {
     if (!s_expanded_map[m_rescenter_id]) {
         // position and reattach icons to be shown
         int n = 0;
-        for (std::vector<std::pair<MeterType, StatisticIcon*> >::iterator it = m_meter_stats.begin(); it != m_meter_stats.end(); ++it) {
-            GG::X x = MeterIconSize().x*n*7/2;
+        GG::X stride = MeterIconSize().x * 7/2;
+        for (std::pair<MeterType, StatisticIcon*>& meter_stat : m_meter_stats) {
+            GG::X x = n * stride;
 
-            if (x > Width() - m_expand_button->Width() - MeterIconSize().x*5/2) break;  // ensure icon doesn't extend past right edge of panel
-
-            StatisticIcon* icon = it->second;
-            AttachChild(icon);
+            StatisticIcon* icon = meter_stat.second;
             GG::Pt icon_ul(x, GG::Y0);
             GG::Pt icon_lr = icon_ul + MeterIconSize();
             icon->SizeMove(icon_ul, icon_lr);
+
+            if (x + icon->MinUsableSize().x >= ClientWidth())
+                break;
+
+            AttachChild(icon);
             icon->Show();
 
             n++;
@@ -191,21 +270,14 @@ void ResourcePanel::DoLayout() {
         Resize(GG::Pt(Width(), m_multi_meter_status_bar->Bottom() + EDGE_PAD - top));
     }
 
-    m_expand_button->MoveTo(GG::Pt(Width() - m_expand_button->Width(), GG::Y0));
-
     SetCollapsed(!s_expanded_map[m_rescenter_id]);
 }
 
-TemporaryPtr<const ResourceCenter> ResourcePanel::GetResCenter() const {
-    TemporaryPtr<const UniverseObject> obj = GetUniverseObject(m_rescenter_id);
-    if (!obj) {
-        ErrorLogger() << "ResourcePanel tried to get an object with an invalid m_rescenter_id";
-        return TemporaryPtr<const ResourceCenter>();
-    }
-    TemporaryPtr<const ResourceCenter> res = boost::dynamic_pointer_cast<const ResourceCenter>(obj);
+std::shared_ptr<const ResourceCenter> ResourcePanel::GetResCenter() const {
+    std::shared_ptr<const ResourceCenter> res = GetResourceCenter(m_rescenter_id);
     if (!res) {
-        ErrorLogger() << "ResourcePanel failed casting an object pointer to a ResourceCenter pointer";
-        return TemporaryPtr<const ResourceCenter>();
+        ErrorLogger() << "ResourcePanel tried to get an object with an invalid m_rescenter_id";
+        return nullptr;
     }
     return res;
 }
