@@ -2,8 +2,10 @@
 #define _ResourcePanel_h_
 
 #include "AccordionPanel.h"
-#include "../universe/Enums.h"
-#include "../universe/TemporaryPtr.h"
+#include "../universe/EnumsFwd.h"
+
+#include <memory>
+
 
 class MultiIconValueIndicator;
 class MultiMeterStatusBar;
@@ -23,6 +25,10 @@ public:
     //@}
 
     /** \name Mutators */ //@{
+    void PreRender() override;
+
+    bool EventFilter(GG::Wnd* w, const GG::WndEvent& event) override;
+
     /** expands or collapses panel to show details or just summary info */
     void ExpandCollapse(bool expanded);
 
@@ -35,7 +41,7 @@ public:
 protected:
     /** \name Mutators */ //@{
     /** resizes panel and positions widgets */
-    virtual void DoLayout();
+    void DoLayout() override;
     //@}
 
 private:
@@ -46,10 +52,10 @@ private:
     int m_rescenter_id;
 
     /** returns the ResourceCenter object with id m_rescenter_id */
-    TemporaryPtr<const ResourceCenter> GetResCenter() const;
+    std::shared_ptr<const ResourceCenter> GetResCenter() const;
 
     /** Icons for the associated meter type. */
-    std::vector<std::pair<MeterType, StatisticIcon*> > m_meter_stats;
+    std::vector<std::pair<MeterType, StatisticIcon*>> m_meter_stats;
 
     /** textually / numerically indicates resource production and construction meter */
     MultiIconValueIndicator* m_multi_icon_value_indicator;
