@@ -25,25 +25,25 @@ public:
     /// Constructor for local browsing
     /// @param extension The extension to enforce on the file name
     /// @param load If set to true, only allow choosing existing files
-    /// @param relative Return a relative file name
     SaveFileDialog(const std::string& extension, bool load = false);
 
     /// Contruct for getting the previews from the server
     SaveFileDialog(bool load = false);
-    ~SaveFileDialog(); //!< dtor
+
+    ~SaveFileDialog();
     //@}
 
     /** \name Mutators */ //@{
-    virtual void ModalInit(); //< Called when dialog is shown. Overrides
-    virtual void KeyPress(GG::Key key, boost::uint32_t key_code_point,
-                          GG::Flags<GG::ModKey> mod_keys);
+    void ModalInit() override;
+
+    void KeyPress(GG::Key key, std::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys) override;
     //@}
 
     /// Get the chosen save files full path
     std::string Result() const;
 
 protected:
-    virtual GG::Rect CalculatePosition() const;
+    GG::Rect CalculatePosition() const override;
 
 private:
     void Init();
