@@ -13,6 +13,7 @@
 #include "../util/OptionsDB.h"
 #include "../util/Directories.h"
 #include "../client/human/HumanClientApp.h"
+#include "../network/ClientNetworking.h"
 #include "ClientUI.h"
 #include "CUIControls.h"
 #include "Sound.h"
@@ -24,8 +25,8 @@ namespace {
 
     bool NameOK(const std::string& name)
     {
-        for (unsigned int i = 0; i < name.size(); ++i) {
-            if (!std::isalnum(name[i]) && name[i] != '_' && name[i] != '-')
+        for (const std::string::value_type& character : name) {
+            if (!std::isalnum(character) && character != '_' && character != '-')
                 return false;
         }
         return !name.empty();
@@ -41,23 +42,23 @@ namespace {
 ServerConnectWnd::ServerConnectWnd() :
     CUIWnd(UserString("SCONNECT_WINDOW_TITLE"),
            GG::INTERACTIVE | GG::MODAL),
-    m_host_or_join_radio_group(0),
-    m_LAN_game_label(0),
-    m_servers_lb(0),
-    m_find_LAN_servers_bn(0),
-    m_internet_game_label(0),
-    m_IP_address_edit(0),
-    m_player_name_edit(0),
-    m_ok_bn(0),
-    m_cancel_bn(0)
+    m_host_or_join_radio_group(nullptr),
+    m_LAN_game_label(nullptr),
+    m_servers_lb(nullptr),
+    m_find_LAN_servers_bn(nullptr),
+    m_internet_game_label(nullptr),
+    m_IP_address_edit(nullptr),
+    m_player_name_edit(nullptr),
+    m_ok_bn(nullptr),
+    m_cancel_bn(nullptr)
 {
     Sound::TempUISoundDisabler sound_disabler;
 
     GG::Label* player_name_label = new CUILabel(UserString("PLAYER_NAME_LABEL"), GG::FORMAT_LEFT);
     m_player_name_edit = new CUIEdit(GetOptionsDB().Get<std::string>("multiplayersetup.player-name"));
     m_host_or_join_radio_group = new GG::RadioButtonGroup(GG::VERTICAL);
-    m_host_or_join_radio_group->AddButton(new CUIStateButton(UserString("HOST_GAME_BN"), GG::FORMAT_LEFT, boost::make_shared<CUIRadioRepresenter>()));
-    m_host_or_join_radio_group->AddButton(new CUIStateButton(UserString("JOIN_GAME_BN"), GG::FORMAT_LEFT, boost::make_shared<CUIRadioRepresenter>()));
+    m_host_or_join_radio_group->AddButton(new CUIStateButton(UserString("HOST_GAME_BN"), GG::FORMAT_LEFT, std::make_shared<CUIRadioRepresenter>()));
+    m_host_or_join_radio_group->AddButton(new CUIStateButton(UserString("JOIN_GAME_BN"), GG::FORMAT_LEFT, std::make_shared<CUIRadioRepresenter>()));
     m_LAN_game_label = new CUILabel(UserString("LAN_GAME_LABEL"), GG::FORMAT_LEFT | GG::FORMAT_NOWRAP);
     m_servers_lb = new CUIListBox();
     m_servers_lb->SetStyle(GG::LIST_NOSORT | GG::LIST_SINGLESEL);
@@ -99,7 +100,6 @@ ServerConnectWnd::ServerConnectWnd() :
 
     ResetDefaultPosition();
 
-    m_LAN_servers = HumanClientApp::GetApp()->Networking().DiscoverLANServers();
     Init();
 }
 
@@ -113,7 +113,7 @@ GG::Rect ServerConnectWnd::CalculatePosition() const {
     return GG::Rect(new_ul, new_ul + new_sz);
 }
 
-void ServerConnectWnd::KeyPress(GG::Key key, boost::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys) {
+void ServerConnectWnd::KeyPress(GG::Key key, std::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys) {
     if (!m_ok_bn->Disabled() && (key == GG::GGK_RETURN || key == GG::GGK_KP_ENTER)) { // Same behaviour as if "OK" was pressed
         OkClicked();
     } else if (key == GG::GGK_ESCAPE) { // Same behaviour as if "Cancel" was pressed
@@ -146,16 +146,16 @@ void ServerConnectWnd::Init() {
 void ServerConnectWnd::PopulateServerList()
 {
     m_servers_lb->Clear();
-    for (ClientNetworking::ServerList::iterator it = m_LAN_servers.begin(); it != m_LAN_servers.end(); ++it) {
+    const auto server_names = HumanClientApp::GetApp()->Networking().DiscoverLANServerNames();
+    for (const auto& server : server_names) {
         GG::ListBox::Row* row = new GG::ListBox::Row;
-        row->push_back(new CUILabel(it->second));
+        row->push_back(new CUILabel(server));
         m_servers_lb->Insert(row);
     }
 }
 
 void ServerConnectWnd::RefreshServerList()
 {
-    m_LAN_servers = HumanClientApp::GetApp()->Networking().DiscoverLANServers();
     PopulateServerList();
 }
 
@@ -197,10 +197,9 @@ void ServerConnectWnd::OkClicked()
         m_result.second = "HOST GAME SELECTED";
     } else {
         m_result.second = *m_IP_address_edit;
-        if (m_result.second == "") {
+        if (m_result.second == "" && !(***m_servers_lb->Selections().begin()).empty()) {
             m_result.second =
-                boost::polymorphic_downcast<GG::Label*>(
-                    (***m_servers_lb->Selections().begin())[0])->Text();
+                boost::polymorphic_downcast<GG::Label*>((***m_servers_lb->Selections().begin()).at(0))->Text() ;
         }
     }
     CUIWnd::CloseClicked();
