@@ -6,7 +6,6 @@
 #include <GG/ListBox.h>
 
 #include "CUIWnd.h"
-#include "../network/ClientNetworking.h"
 
 
 /** server connections window */
@@ -19,8 +18,9 @@ public:
 
     //! \name Mutators
     //!@{
-    virtual void ModalInit ();
-    virtual void KeyPress (GG::Key key, boost::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys);
+    void ModalInit() override;
+
+    void KeyPress(GG::Key key, std::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys) override;
     //!@}
 
     /** \name Accessors */ //@{
@@ -29,7 +29,7 @@ public:
     //@}
 
 protected:
-    virtual GG::Rect CalculatePosition() const;
+    GG::Rect CalculatePosition() const override;
 
 private:
     void Init();
@@ -54,8 +54,6 @@ private:
     GG::Edit*                           m_player_name_edit;
     GG::Button*                         m_ok_bn;
     GG::Button*                         m_cancel_bn;
-
-    ClientNetworking::ServerList        m_LAN_servers;
 };
 
 #endif // _ServerConnectWnd_h_
