@@ -12,13 +12,18 @@
 #include "../universe/ShipDesign.h"
 
 #include <GG/DrawUtil.h>
+#include <GG/Layout.h>
 
 #include <boost/lexical_cast.hpp>
 
+#include <iterator>
+
 
 namespace {
-    GG::X ICON_RIGHT_MARGIN(3);
-    GG::Y ITEM_VERTICAL_PADDING(2);
+    const int sitrep_row_margin(1);
+    const int sitrep_edge_to_outline_spacing(2);
+    const int sitrep_edge_to_content_spacing(sitrep_edge_to_outline_spacing + 1 + 2);
+    const int sitrep_spacing(2);
 
     /** Adds options related to SitRepPanel to Options DB. */
     void AddOptions(OptionsDB& db) {
@@ -28,11 +33,11 @@ namespace {
     }
     bool temp_bool = RegisterOptions(&AddOptions);
 
-    GG::X GetIconSize()
-    { return GG::X(GetOptionsDB().Get<int>("UI.sitrep-icon-size")); }
+    int GetIconSize()
+    { return GetOptionsDB().Get<int>("UI.sitrep-icon-size"); }
 
     std::map<std::string, std::string> label_display_map;
-    std::map<int, std::set<std::string> > snoozed_sitreps;
+    std::map<int, std::set<std::string>> snoozed_sitreps;
     std::set<std::string> permanently_snoozed_sitreps;
 
     void SnoozeSitRepForNTurns(std::string sitrep_text, int start_turn, int num_turns) {
@@ -77,7 +82,8 @@ namespace {
                 ClientUI::GetClientUI()->ZoomToShipPart(data);
             } else if (link_type == VarText::SPECIES_TAG) {
                 ClientUI::GetClientUI()->ZoomToSpecies(data);
-
+            } else if (link_type == VarText::METER_TYPE_TAG) {
+                ClientUI::GetClientUI()->ZoomToMeterTypeArticle(data);
             } else if (link_type == TextLinker::ENCYCLOPEDIA_TAG) {
                 ClientUI::GetClientUI()->ZoomToEncyclopediaEntry(data);
             }
@@ -92,7 +98,7 @@ namespace {
         std::vector<std::string> sitrep_order;
         std::copy(std::istream_iterator<std::string>(template_stream),
                   std::istream_iterator<std::string>(),
-                  std::back_inserter<std::vector<std::string> >(sitrep_order));
+                  std::back_inserter<std::vector<std::string>>(sitrep_order));
         return sitrep_order;
     }
 
@@ -125,29 +131,24 @@ namespace {
         std::set<std::string> template_set;
 
         // get templates for each empire
-        for (EmpireManager::iterator it = Empires().begin();
-             it != Empires().end(); ++it)
-        {
-            std::set<std::string> empire_strings = EmpireSitRepTemplateStrings(it->first);
+        for (std::map<int, Empire*>::value_type& entry : Empires()) {
+            std::set<std::string> empire_strings = EmpireSitRepTemplateStrings(entry.first);
             template_set.insert(empire_strings.begin(), empire_strings.end());
         }
 
         std::vector<std::string> retval;
 
         // only use those ordered templates actually in the current set of sitrep templates
-        std::vector<std::string> ordered_templates = OrderedSitrepTemplateStrings();
-        for (std::vector<std::string>::iterator it = ordered_templates.begin(); 
-             it!=ordered_templates.end(); ++it)
-        {
-            if ( (template_set.find(*it) != template_set.end()) && 
-                 (std::find(retval.begin(), retval.end(), *it) == retval.end()) )
-            { retval.push_back(*it); }
+        for (const std::string& templ : OrderedSitrepTemplateStrings()) {
+            if ( (template_set.find(templ) != template_set.end()) &&
+                 (std::find(retval.begin(), retval.end(), templ) == retval.end()) )
+            { retval.push_back(templ); }
         }
 
         //now add the current templates that did not have a specified order
-        for (std::set<std::string>::iterator it = template_set.begin(); it!= template_set.end(); ++it)
-            if (std::find(retval.begin(), retval.end(), *it) == retval.end())
-                retval.push_back(*it);
+        for (const std::string& templ : template_set)
+            if (std::find(retval.begin(), retval.end(), templ) == retval.end())
+                retval.push_back(templ);
 
         return retval;
     }
@@ -182,7 +183,7 @@ namespace {
 
     class ColorEmpire : public LinkDecorator {
     public:
-        virtual std::string Decorate(const std::string& target, const std::string& content) const {
+        std::string Decorate(const std::string& target, const std::string& content) const override {
             GG::Clr color = ClientUI::DefaultLinkColor();
             int id = CastStringToInt(target);
             Empire* empire = GetEmpire(id);
@@ -192,96 +193,124 @@ namespace {
         }
     };
 
+    //////////////////////////////////
+    // SitRepLinkText
+    //////////////////////////////////
+    class SitRepLinkText : public LinkText {
+    public:
+        SitRepLinkText(GG::X x, GG::Y y, GG::X w, const std::string& str, const std::shared_ptr<GG::Font>& font,
+                       GG::Flags<GG::TextFormat> format = GG::FORMAT_NONE, GG::Clr color = GG::CLR_BLACK) :
+            LinkText(x, y, w, str, font, format, color) {}
+
+        mutable boost::signals2::signal<void(const GG::Pt&, GG::Flags<GG::ModKey>)> RightClickedSignal;
+
+    protected:
+        void RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override {
+            if (GetLinkUnderPt(pt) != -1) {
+                LinkText::RClick(pt, mod_keys);
+            } else {
+                RightClickedSignal(pt, mod_keys);
+            }
+        }
+    };
+
     //////////////////////////////////
     // SitRepDataPanel
     //////////////////////////////////
     class SitRepDataPanel : public GG::Control {
     public:
-        SitRepDataPanel(GG::X w, GG::Y h, const SitRepEntry& sitrep) :
-            Control(GG::X0, GG::Y0, w, h, GG::NO_WND_FLAGS),
+        SitRepDataPanel(GG::X left, GG::Y top, GG::X w, GG::Y h, const SitRepEntry& sitrep) :
+            Control(left, top, w, h, GG::NO_WND_FLAGS),
             m_initialized(false),
             m_sitrep_entry(sitrep),
-            m_icon(0),
-            m_link_text(0)
+            m_icon(nullptr),
+            m_link_text(nullptr)
         {
             SetChildClippingMode(ClipToClient);
-        }
-
-        virtual void        Render() {
-            if (!m_initialized)
-                Init();
-            GG::Clr background_clr = this->Disabled() ? ClientUI::WndColor() : ClientUI::CtrlColor();
-            GG::FlatRectangle(UpperLeft(), LowerRight() - GG::Pt(GG::X0, GG::Y(2)), background_clr, ClientUI::WndOuterBorderColor(), 1u);
-        }
-
-        virtual void        SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
             Init();
-            const GG::Pt old_size = Size();
-            GG::Control::SizeMove(ul, lr);
-            if (old_size != Size()) {
-                DoLayout();
-            }
-            if (m_link_text) {
-                // establish sitrep panel's size; use icon or text size for panel height, whichever is larger
-                // DoLayout reflowed the text.
-                int text_size = Value((m_link_text->TextLowerRight() - m_link_text->TextUpperLeft()).y);
-                int icon_size = Value(GetIconSize());
-                int max_panel_size = std::max(text_size, icon_size);
-                max_panel_size += Value(2*ITEM_VERTICAL_PADDING);
+            DoLayout(GG::Pt(left, top), w);
+        }
 
-                GG::Pt panel_size = GG::Pt(GG::X(lr.x - ul.x), GG::Y(max_panel_size));
-                GG::Control::SizeMove(ul, ul + panel_size);
-                DoLayout();
-            }
+        void Render() override {
+            GG::Clr background_clr = this->Disabled() ? ClientUI::WndColor() : ClientUI::CtrlColor();
+            GG::Pt spacer = GG::Pt(GG::X(sitrep_edge_to_outline_spacing), GG::Y(sitrep_edge_to_outline_spacing));
+            GG::FlatRectangle(UpperLeft() + spacer, LowerRight() - spacer,
+                              background_clr, ClientUI::WndOuterBorderColor(), 1u);
+        }
+
+        void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override {
+            if (ul != ClientUpperLeft() || (lr.x - ul.x) != Width())
+                DoLayout(ul, lr.x - ul.x);
         }
 
         const SitRepEntry&  GetSitRepEntry() const { return m_sitrep_entry; }
 
+        mutable boost::signals2::signal<void(const GG::Pt&, GG::Flags<GG::ModKey>)> RightClickedSignal;
+
+    protected:
+        void RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod) override
+        { RightClickedSignal(pt, mod); }
+
     private:
-        void            DoLayout() {
-            if (!m_initialized)
-                return;
-            GG::Y ICON_HEIGHT(Value(GetIconSize()));
+        void            DoLayout(const GG::Pt& ul, const GG::X& width) {
+            // Resize the text
+            GG::Pt spacer = GG::Pt(GG::X(sitrep_edge_to_content_spacing), GG::Y(sitrep_edge_to_content_spacing));
+            GG::X icon_left(ul.x + spacer.x);
+            int icon_dim = GetIconSize();
+            GG::X text_left(icon_left + GG::X(icon_dim) + sitrep_spacing);
+            GG::X text_width(width - text_left - spacer.x);
+            m_link_text->SizeMove(GG::Pt(text_left, GG::Y0), GG::Pt(text_left, GG::Y0) + GG::Pt(text_width, GG::Y1));
 
-            GG::X left(GG::X0);
-            GG::Y bottom(ClientHeight() - ITEM_VERTICAL_PADDING);
+            // Choose height so that text always fits
+            GG::Y text_height = m_link_text->MinUsableSize().y;
+            GG::Y panel_height = std::max(text_height, GG::Y(icon_dim)) + 2 * spacer.y;
 
-            m_icon->SizeMove(GG::Pt(left, bottom/2 - ICON_HEIGHT/2), GG::Pt(left + GetIconSize(), bottom/2 + ICON_HEIGHT/2));
-            left += GetIconSize() + ICON_RIGHT_MARGIN;
+            // Move the elements into place.
+            GG::Y icon_top(ul.y + panel_height/2 - GG::Y(icon_dim)/2);
+            m_icon->MoveTo(GG::Pt(icon_left, icon_top));
 
-            m_link_text->SizeMove(GG::Pt(left, GG::Y0), GG::Pt(ClientWidth() - 3, bottom));
+            GG::Y text_top(ul.y + panel_height/2 - GG::Y(text_height)/2);
+            GG::Pt text_ul(text_left, text_top);
+            m_link_text->SizeMove(text_ul, text_ul + m_link_text->MinUsableSize());
+
+            //Resize control to fit
+            GG::Pt new_size = GG::Pt(width, panel_height);
+            GG::Control::SizeMove(ul, ul + new_size);
         }
 
         void            Init() {
-            if (m_initialized)
-                return;
             m_initialized = true;
 
+            int icon_dim = GetIconSize();
             std::string icon_texture = (m_sitrep_entry.GetIcon().empty() ?
                 "/icons/sitrep/generic.png" : m_sitrep_entry.GetIcon());
-            boost::shared_ptr<GG::Texture> icon = ClientUI::GetTexture(
+            std::shared_ptr<GG::Texture> icon = ClientUI::GetTexture(
                 ClientUI::ArtDir() / icon_texture, true);
             m_icon = new GG::StaticGraphic(icon, GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE);
             AttachChild(m_icon);
+            m_icon->Resize(GG::Pt(GG::X(icon_dim), GG::Y(icon_dim)));
 
-            m_link_text = new LinkText(GG::X0, GG::Y0, GG::X1, m_sitrep_entry.GetText() + " ", 
-                                       ClientUI::GetFont(),
-                                       GG::FORMAT_LEFT | GG::FORMAT_VCENTER | GG::FORMAT_WORDBREAK, ClientUI::TextColor());
+            GG::Pt spacer = GG::Pt(GG::X(sitrep_edge_to_content_spacing), GG::Y(sitrep_edge_to_content_spacing));
+            GG::X icon_left(spacer.x);
+            GG::X text_left(icon_left + GG::X(icon_dim) + sitrep_spacing);
+            GG::X text_width(ClientWidth() - text_left - spacer.x);
+            m_link_text = new SitRepLinkText(GG::X0, GG::Y0, text_width, m_sitrep_entry.GetText() + " ",
+                                             ClientUI::GetFont(),
+                                             GG::FORMAT_LEFT | GG::FORMAT_VCENTER | GG::FORMAT_WORDBREAK, ClientUI::TextColor());
             m_link_text->SetDecorator(VarText::EMPIRE_ID_TAG, new ColorEmpire());
             AttachChild(m_link_text);
 
             GG::Connect(m_link_text->LinkClickedSignal,       &HandleLinkClick);
             GG::Connect(m_link_text->LinkDoubleClickedSignal, &HandleLinkClick);
             GG::Connect(m_link_text->LinkRightClickedSignal,  &HandleLinkClick);
-
-            DoLayout();
+            GG::Connect(m_link_text->RightClickedSignal,      &SitRepDataPanel::RClick,     this);
         }
 
         bool                m_initialized;
 
-        SitRepEntry         m_sitrep_entry;
+        const SitRepEntry&  m_sitrep_entry;
         GG::StaticGraphic*  m_icon;
-        LinkText*           m_link_text;
+        SitRepLinkText*     m_link_text;
     };
 
     ////////////////////////////////////////////////
@@ -292,29 +321,48 @@ namespace {
     public:
         SitRepRow(GG::X w, GG::Y h, const SitRepEntry& sitrep) :
             GG::ListBox::Row(w, h, ""),
-            m_panel(0)
+            m_panel(nullptr),
+            m_sitrep(sitrep)
         {
             SetName("SitRepRow");
+            SetMargin(sitrep_row_margin);
             SetChildClippingMode(ClipToClient);
-            m_panel = new SitRepDataPanel(w, h, sitrep);
-            push_back(m_panel);
+            SetMinSize(GG::Pt(GG::X(2 * GetIconSize() + 2 * sitrep_edge_to_content_spacing),
+                              GG::Y(std::max(GetIconSize(), ClientUI::Pts() * 2) + 2 * sitrep_edge_to_content_spacing)));
+            RequirePreRender();
         }
 
-        void SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
-            const GG::Pt old_size = Size();
-            GG::Pt new_size = lr - ul;
-            new_size.x -= 9; // Avoid allowing the scrollbar to hide the very rightmost pixels.
-            if (!empty() && m_panel && old_size != new_size) {
-                m_panel->Resize(new_size);
-                new_size = m_panel->Size();
-            }
-            GG::ListBox::Row::SizeMove(ul, ul + new_size);
+        void PreRender() override {
+            GG::ListBox::Row::PreRender();
+
+            if (!m_panel)
+                Init();
+
+            // Resize to fit panel after text reflows
+            GG::Pt border(GG::X(2 * GetLayout()->BorderMargin()), GG::Y(2 * GetLayout()->BorderMargin()));
+            m_panel->Resize(Size() - border);
+            GG::ListBox::Row::Resize(m_panel->Size() + border);
+        }
+
+        void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override {
+            if (!m_panel || (Size() != (lr - ul)))
+                RequirePreRender();
+            GG::ListBox::Row::SizeMove(ul, lr);
+        }
+
+        void            Init() {
+            m_panel = new SitRepDataPanel(GG::X(GetLayout()->BorderMargin()), GG::Y(GetLayout()->BorderMargin()),
+                                          ClientWidth() - GG::X(2 * GetLayout()->BorderMargin()),
+                                          ClientHeight() - GG::Y(2 * GetLayout()->BorderMargin()), m_sitrep);
+            push_back(m_panel);
+            GG::Connect(m_panel->RightClickedSignal, &SitRepRow::RClick, this);
         }
 
         const SitRepEntry&  GetSitRepEntry() const { return m_panel->GetSitRepEntry(); }
 
     private:
         SitRepDataPanel*    m_panel;
+        const SitRepEntry   m_sitrep;
     };
 }
 
@@ -322,11 +370,11 @@ SitRepPanel::SitRepPanel(const std::string& config_name) :
     CUIWnd(UserString("SITREP_PANEL_TITLE"),
            GG::ONTOP | GG::INTERACTIVE | GG::DRAGABLE | GG::RESIZABLE | CLOSABLE | PINABLE,
            config_name),
-    m_sitreps_lb(0),
-    m_prev_turn_button(0),
-    m_next_turn_button(0),
-    m_last_turn_button(0),
-    m_filter_button(0),
+    m_sitreps_lb(nullptr),
+    m_prev_turn_button(nullptr),
+    m_next_turn_button(nullptr),
+    m_last_turn_button(nullptr),
+    m_filter_button(nullptr),
     m_showing_turn(INVALID_GAME_TURN),
     m_hidden_sitrep_templates(HiddenSitRepTemplateStringsFromOptions())
 {
@@ -336,6 +384,8 @@ SitRepPanel::SitRepPanel(const std::string& config_name) :
     m_sitreps_lb = new CUIListBox();
     m_sitreps_lb->SetStyle(GG::LIST_NOSORT | GG::LIST_NOSEL);
     m_sitreps_lb->SetVScrollWheelIncrement(ClientUI::Pts()*4.5);
+    m_sitreps_lb->ManuallyManageColProps();
+    m_sitreps_lb->NormalizeRowsOnInsert(false);
     AttachChild(m_sitreps_lb);
 
     m_prev_turn_button = new CUIButton(UserString("BACK"));
@@ -351,7 +401,7 @@ SitRepPanel::SitRepPanel(const std::string& config_name) :
     GG::Connect(m_next_turn_button->LeftClickedSignal,  &SitRepPanel::NextClicked,          this);
     GG::Connect(m_last_turn_button->LeftClickedSignal,  &SitRepPanel::LastClicked,          this);
     GG::Connect(m_filter_button->LeftClickedSignal,     &SitRepPanel::FilterClicked,        this);
-    GG::Connect(m_sitreps_lb->DoubleClickedSignal,      &SitRepPanel::DismissSitRep,        this);
+    GG::Connect(m_sitreps_lb->DoubleClickedSignal,      &SitRepPanel::IgnoreSitRep,         this);
     GG::Connect(m_sitreps_lb->RightClickedSignal,       &SitRepPanel::DismissalMenu,        this);
 
     DoLayout();
@@ -379,7 +429,7 @@ void SitRepPanel::DoLayout() {
     SetMinSize(GG::Pt(6*BUTTON_WIDTH, 6*BUTTON_HEIGHT));
 }
 
-void SitRepPanel::KeyPress(GG::Key key, boost::uint32_t key_code_point,
+void SitRepPanel::KeyPress(GG::Key key, std::uint32_t key_code_point,
                            GG::Flags<GG::ModKey> mod_keys)
 {
     switch (key) {
@@ -403,17 +453,15 @@ void SitRepPanel::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
     if (old_size != GG::Wnd::Size()) {
         DoLayout();
         Update();
-        if (!m_sitreps_lb->Empty()) {
-            m_sitreps_lb->BringRowIntoView(--m_sitreps_lb->end());
-            m_sitreps_lb->BringRowIntoView(boost::next(m_sitreps_lb->begin(), first_visible_queue_row));
-        }
+        if (!m_sitreps_lb->Empty())
+            m_sitreps_lb->SetFirstRowShown(std::next(m_sitreps_lb->begin(), first_visible_queue_row));
     }
 }
 
 namespace {
     /* Sort empire's sitreps for each turn */
-    std::map<int, std::list<SitRepEntry> > GetSitRepsSortedByTurn(int empire_id, std::set<std::string> hidden_sitrep_templates) {
-        std::map<int, std::list<SitRepEntry> > turns;
+    std::map<int, std::list<SitRepEntry>> GetSitRepsSortedByTurn(int empire_id, std::set<std::string> hidden_sitrep_templates) {
+        std::map<int, std::list<SitRepEntry>> turns;
         bool verbose_sitrep = GetOptionsDB().Get<bool>("verbose-sitrep");
         std::set<Empire*> sr_empires;
         Empire* empire = GetEmpire(empire_id);
@@ -422,12 +470,11 @@ namespace {
         } else {
             // Moderator mode, sort sitreps from all empires
             EmpireManager& empires = Empires();
-            for (EmpireManager::iterator it = empires.begin(); it != empires.end(); ++it)
-                sr_empires.insert(it->second);
+            for (std::map<int, Empire*>::value_type& entry : empires)
+                sr_empires.insert(entry.second);
         }
-        for (std::set<Empire*>::iterator it = sr_empires.begin(); it != sr_empires.end(); ++it)
-        {
-            for (Empire::SitRepItr sitrep_it = (*it)->SitRepBegin(); sitrep_it != (*it)->SitRepEnd(); ++sitrep_it) {
+        for (Empire* empire : sr_empires) {
+            for (Empire::SitRepItr sitrep_it = empire->SitRepBegin(); sitrep_it != empire->SitRepEnd(); ++sitrep_it) {
                 if (!verbose_sitrep) {
                     if (!sitrep_it->Validate())
                         continue;
@@ -436,7 +483,7 @@ namespace {
                     continue;
                 if (permanently_snoozed_sitreps.find(sitrep_it->GetText()) != permanently_snoozed_sitreps.end())
                     continue;
-                std::map< int, std::set< std::string > >::iterator sitrep_set_it = snoozed_sitreps.find(sitrep_it->GetTurn());
+                std::map< int, std::set<std::string>>::iterator sitrep_set_it = snoozed_sitreps.find(sitrep_it->GetTurn());
                 if (sitrep_set_it != snoozed_sitreps.end() && sitrep_set_it->second.find(sitrep_it->GetText()) != sitrep_set_it->second.end())
                     continue;
                 turns[sitrep_it->GetTurn()].push_back(*sitrep_it);
@@ -446,7 +493,7 @@ namespace {
     }
 }
 
-int SitRepPanel::GetNextNonEmptySitrepsTurn(const std::map<int, std::list<SitRepEntry> >& turns, int turn, bool forward) const {
+int SitRepPanel::GetNextNonEmptySitrepsTurn(const std::map<int, std::list<SitRepEntry>>& turns, int turn, bool forward) const {
     // All sitreps filtered out ?
     if (turns.size() == 0)
         return INVALID_GAME_TURN;
@@ -461,7 +508,7 @@ int SitRepPanel::GetNextNonEmptySitrepsTurn(const std::map<int, std::list<SitRep
         return (--turns.end())->first;
 
     // Search a suitable turn
-    std::map<int, std::list<SitRepEntry> >::const_iterator it = turns.find(turn);
+    std::map<int, std::list<SitRepEntry>>::const_iterator it = turns.find(turn);
     if (it != turns.end()) {
         int step = forward ? 1 : -1;
 
@@ -482,12 +529,12 @@ void SitRepPanel::CloseClicked()
 { ClosingSignal(); }
 
 void SitRepPanel::PrevClicked() {
-    std::map<int, std::list<SitRepEntry> > turns = GetSitRepsSortedByTurn(HumanClientApp::GetApp()->EmpireID(), m_hidden_sitrep_templates);
+    std::map<int, std::list<SitRepEntry>> turns = GetSitRepsSortedByTurn(HumanClientApp::GetApp()->EmpireID(), m_hidden_sitrep_templates);
     ShowSitRepsForTurn(GetNextNonEmptySitrepsTurn(turns, m_showing_turn, false));
 }
 
 void SitRepPanel::NextClicked() {
-    std::map<int, std::list<SitRepEntry> > turns = GetSitRepsSortedByTurn(HumanClientApp::GetApp()->EmpireID(), m_hidden_sitrep_templates);
+    std::map<int, std::list<SitRepEntry>> turns = GetSitRepsSortedByTurn(HumanClientApp::GetApp()->EmpireID(), m_hidden_sitrep_templates);
     ShowSitRepsForTurn(GetNextNonEmptySitrepsTurn(turns, m_showing_turn, true));
 }
 
@@ -496,35 +543,31 @@ void SitRepPanel::LastClicked() {
 }
 
 void SitRepPanel::FilterClicked() {
-    std::vector<std::string> all_templates = AllSitRepTemplateStrings();
-
     std::map<int, std::string> menu_index_templates;
     std::map<int, bool> menu_index_checked;
     int index = 1;
     bool all_checked = true;
     int ALL_INDEX = 9999;
 
+    std::vector<std::string> all_templates = AllSitRepTemplateStrings();
+
     GG::MenuItem menu_contents;
-    for (std::vector<std::string>::const_iterator it = all_templates.begin();
-         it != all_templates.end(); ++it, ++index)
-    {
-        menu_index_templates[index] = *it;
+    for (const std::string& templ : all_templates) {
+        menu_index_templates[index] = templ;
         bool check = true;
-        if (m_hidden_sitrep_templates.find(*it) != m_hidden_sitrep_templates.end()) {
+        if (m_hidden_sitrep_templates.find(templ) != m_hidden_sitrep_templates.end()) {
             check = false;
             all_checked = false;
         }
         menu_index_checked[index] = check;
-        const std::string& menu_label =  label_display_map[*it];
+        const std::string& menu_label =  label_display_map[templ];
         menu_contents.next_level.push_back(GG::MenuItem(menu_label, index, false, check));
+        ++index;
     }
     menu_contents.next_level.push_back(GG::MenuItem((all_checked ? UserString("NONE") : UserString("ALL")),
                                        ALL_INDEX, false, false));
 
-    GG::PopupMenu popup(m_filter_button->Left(), m_filter_button->Bottom(),
-                        ClientUI::GetFont(), menu_contents, ClientUI::TextColor(),
-                        ClientUI::WndOuterBorderColor(), ClientUI::WndColor(),
-                        ClientUI::EditHiliteColor());
+    CUIPopupMenu popup(m_filter_button->Left(), m_filter_button->Bottom(), menu_contents);
     if (!popup.Run())
         return;
     int selected_menu_item = popup.MenuID();
@@ -535,9 +578,9 @@ void SitRepPanel::FilterClicked() {
         // select / deselect all templates
         if (all_checked) {
             // deselect all
-            for (std::vector<std::string>::const_iterator it = all_templates.begin();
-                 it != all_templates.end(); ++it, ++index)
-            { m_hidden_sitrep_templates.insert(*it); }
+            for (const std::string& templ : all_templates) {
+                m_hidden_sitrep_templates.insert(templ);
+            }
         } else {
             // select all
             m_hidden_sitrep_templates.clear();
@@ -558,7 +601,7 @@ void SitRepPanel::FilterClicked() {
     Update();
 }
 
-void SitRepPanel::DismissSitRep(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& mod) {
+void SitRepPanel::IgnoreSitRep(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& mod) {
     SitRepRow* sitrep_row = dynamic_cast<SitRepRow*>(*it);
     if (!sitrep_row) {
         return;
@@ -573,26 +616,62 @@ void SitRepPanel::DismissSitRep(GG::ListBox::iterator it, const GG::Pt& pt, cons
 
 void SitRepPanel::DismissalMenu(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& mod) {
 
-    GG::MenuItem menu_contents;
-    std::string sitrep_text;
+    GG::MenuItem menu_contents, submenu_ignore, submenu_block, separator_item;
+    std::string sitrep_text, sitrep_template;
+    std::string entry_margin("  ");
+    separator_item.separator = true;
     int start_turn = 0;
-    SitRepRow* sitrep_row(0);
+    SitRepRow* sitrep_row = nullptr;
     if (it != m_sitreps_lb->end()) 
         sitrep_row = dynamic_cast<SitRepRow*>(*it);
+    submenu_ignore.label = entry_margin + UserString("SITREP_IGNORE_MENU");
     if (sitrep_row) {
-        sitrep_text = sitrep_row->GetSitRepEntry().GetText();
-        start_turn = sitrep_row->GetSitRepEntry().GetTurn();
+        const SitRepEntry& sitrep_entry = sitrep_row->GetSitRepEntry();
+        sitrep_text = sitrep_entry.GetText();
+        start_turn = sitrep_entry.GetTurn();
         if (start_turn > 0) {
-            menu_contents.next_level.push_back(GG::MenuItem(UserString("SITREP_SNOOZE_5_TURNS"),      1, false, false));
-            menu_contents.next_level.push_back(GG::MenuItem(UserString("SITREP_SNOOZE_10_TURNS"),     2, false, false));
-            menu_contents.next_level.push_back(GG::MenuItem(UserString("SITREP_SNOOZE_INDEFINITE"),   3, false, false));
+            submenu_ignore.next_level.push_back(GG::MenuItem(entry_margin + UserString("SITREP_SNOOZE_5_TURNS"),      1,
+                                                             false, false));
+            submenu_ignore.next_level.push_back(GG::MenuItem(entry_margin + UserString("SITREP_SNOOZE_10_TURNS"),     2,
+                                                             false, false));
+            submenu_ignore.next_level.push_back(GG::MenuItem(entry_margin + UserString("SITREP_SNOOZE_INDEFINITE"),   3,
+                                                             false, false));
+            submenu_ignore.next_level.push_back(separator_item);
         }
     }
-    menu_contents.next_level.push_back(GG::MenuItem(UserString("SITREP_SNOOZE_CLEAR_ALL"),        4, false, false));
-    menu_contents.next_level.push_back(GG::MenuItem(UserString("SITREP_SNOOZE_CLEAR_INDEFINITE"), 5, false, false));
+    submenu_ignore.next_level.push_back(GG::MenuItem(entry_margin + UserString("SITREP_SNOOZE_CLEAR_ALL"),            4,
+                                                     false, false));
+    submenu_ignore.next_level.push_back(GG::MenuItem(entry_margin + UserString("SITREP_SNOOZE_CLEAR_INDEFINITE"),     5,
+                                                     false, false));
+    menu_contents.next_level.push_back(submenu_ignore);
 
-    GG::PopupMenu popup(pt.x, pt.y, ClientUI::GetFont(), menu_contents, ClientUI::TextColor(),
-                        ClientUI::WndInnerBorderColor(), ClientUI::WndColor(), ClientUI::EditHiliteColor());
+    submenu_block.label = entry_margin + UserString("SITREP_BLOCK_MENU");
+    if (sitrep_row) {
+        const SitRepEntry& sitrep_entry = sitrep_row->GetSitRepEntry();
+        sitrep_template = sitrep_entry.GetLabelString();
+        std::string sitrep_label = sitrep_entry.GetStringtableLookupFlag() ? UserString(sitrep_template) : sitrep_template;
+
+        if (!sitrep_label.empty() && !sitrep_template.empty()) {
+            submenu_block.next_level.push_back(GG::MenuItem(entry_margin + str(FlexibleFormat(UserString("SITREP_HIDE_TEMPLATE"))
+                                                                               % sitrep_label),                       7,
+                                                            false, false));
+        }
+    }
+    if (m_hidden_sitrep_templates.size() > 0) {
+        if (sitrep_row)
+            submenu_block.next_level.push_back(separator_item);
+        submenu_block.next_level.push_back(GG::MenuItem(entry_margin + UserString("SITREP_SHOWALL_TEMPLATES"),        8,
+                                                        false, false));
+    }
+    menu_contents.next_level.push_back(submenu_block);
+
+    menu_contents.next_level.push_back(GG::MenuItem(entry_margin + UserString("HOTKEY_COPY"),                         9,
+                                                    false, false));
+    menu_contents.next_level.push_back(GG::MenuItem(entry_margin + UserString("POPUP_MENU_PEDIA_PREFIX") +
+                                                    UserString("SITREP_IGNORE_BLOCK_TITLE"),                         10,
+                                                    false, false));
+
+    CUIPopupMenu popup(pt.x, pt.y, menu_contents);
     if (!popup.Run())
         return;
     int selected_menu_item = popup.MenuID();
@@ -619,6 +698,28 @@ void SitRepPanel::DismissalMenu(GG::ListBox::iterator it, const GG::Pt& pt, cons
         permanently_snoozed_sitreps.clear();
         break;
     }
+    case 7: { // Add template for the row entry to hidden templates
+        m_hidden_sitrep_templates.insert(sitrep_template);
+        SetHiddenSitRepTemplateStringsInOptions(m_hidden_sitrep_templates);
+        Update();
+        break;
+    }
+    case 8: { // Show all hidden templates
+        m_hidden_sitrep_templates.clear();
+        SetHiddenSitRepTemplateStringsInOptions(m_hidden_sitrep_templates);
+        Update();
+        break;
+    }
+    case 9: { // Copy text of sitrep
+        if (sitrep_text.empty())
+            break;
+        GG::GUI::GetGUI()->SetClipboardText(GG::Font::StripTags(sitrep_text));
+        break;
+    }
+    case 10: { // Display help article
+        ClientUI::GetClientUI()->ZoomToEncyclopediaEntry("SITREP_IGNORE_BLOCK_TITLE");
+        break;
+    }
 
     default:
         break;
@@ -631,6 +732,10 @@ void SitRepPanel::Update() {
 
     std::size_t first_visible_row = std::distance(m_sitreps_lb->begin(), m_sitreps_lb->FirstRowShown());
     m_sitreps_lb->Clear();
+    m_sitreps_lb->SetStyle(GG::LIST_NOSORT | GG::LIST_NOSEL);
+    m_sitreps_lb->SetVScrollWheelIncrement(ClientUI::Pts()*4.5);
+    m_sitreps_lb->ManuallyManageColProps();
+    m_sitreps_lb->NormalizeRowsOnInsert(false);
 
     // Get back to sane default
     if (m_showing_turn == INVALID_GAME_TURN)
@@ -640,7 +745,7 @@ void SitRepPanel::Update() {
     // if this client is an observer or moderator.
     // todo: double check that no-empire players are actually moderator or
     //       observers, instead of just passing the client empire id.
-    std::map<int, std::list<SitRepEntry> > turns = GetSitRepsSortedByTurn(HumanClientApp::GetApp()->EmpireID(), m_hidden_sitrep_templates);
+    std::map<int, std::list<SitRepEntry>> turns = GetSitRepsSortedByTurn(HumanClientApp::GetApp()->EmpireID(), m_hidden_sitrep_templates);
 
     std::list<SitRepEntry> currentTurnSitreps;
     if (turns.find(m_showing_turn) != turns.end())
@@ -657,15 +762,12 @@ void SitRepPanel::Update() {
 
     // order sitreps for display
     std::vector<SitRepEntry> orderedSitreps;
-    std::vector<std::string> ordered_templates = OrderedSitrepTemplateStrings();
-    for (std::vector<std::string>::const_iterator template_it = ordered_templates.begin();
-         template_it != ordered_templates.end(); ++template_it)
-    {
+    for (const std::string& templ : OrderedSitrepTemplateStrings()) {
         for (std::list<SitRepEntry>::iterator sitrep_it = currentTurnSitreps.begin();
              sitrep_it != currentTurnSitreps.end(); ++sitrep_it)
         {
-            if ((sitrep_it->GetLabelString().empty() ? sitrep_it->GetTemplateString() : sitrep_it->GetLabelString()) == *template_it) {
-                //DebugLogger() << "saving into orderedSitreps -  sitrep of template "<<*template_it<<" with full string "<< sitrep_it->GetText();
+            if ((sitrep_it->GetLabelString().empty() ? sitrep_it->GetTemplateString() : sitrep_it->GetLabelString()) == templ) {
+                //DebugLogger() << "saving into orderedSitreps -  sitrep of template " << templ << " with full string "<< sitrep_it->GetText();
                 orderedSitreps.push_back(*sitrep_it);
                 //DebugLogger()<< "deleting above sitrep from currentTurnSitreps";
                 sitrep_it = --currentTurnSitreps.erase(sitrep_it);
@@ -674,18 +776,16 @@ void SitRepPanel::Update() {
     }
 
     // copy remaining unordered sitreps
-    for (std::list<SitRepEntry>::iterator sitrep_it = currentTurnSitreps.begin();
-         sitrep_it != currentTurnSitreps.end(); ++sitrep_it)
-    { orderedSitreps.push_back(*sitrep_it); }
+    for (const SitRepEntry& sitrep : currentTurnSitreps)
+    { orderedSitreps.push_back(sitrep); }
 
     // create UI rows for all sitrps
-    GG::X width = m_sitreps_lb->Width() - ClientUI::ScrollWidth();
-    for (std::vector<SitRepEntry>::iterator sitrep_it = orderedSitreps.begin();
-         sitrep_it != orderedSitreps.end(); ++sitrep_it)
-    { m_sitreps_lb->Insert(new SitRepRow(width, GG::Y(ClientUI::Pts()*2), *sitrep_it)); }
+    GG::X width = m_sitreps_lb->ClientWidth();
+    for (const SitRepEntry& sitrep : orderedSitreps)
+    { m_sitreps_lb->Insert(new SitRepRow(width, GG::Y(ClientUI::Pts()*2), sitrep)); }
 
     if (m_sitreps_lb->NumRows() > first_visible_row) {
-        m_sitreps_lb->SetFirstRowShown(boost::next(m_sitreps_lb->begin(), first_visible_row));
+        m_sitreps_lb->SetFirstRowShown(std::next(m_sitreps_lb->begin(), first_visible_row));
     } else if (!m_sitreps_lb->Empty()) {
         m_sitreps_lb->BringRowIntoView(--m_sitreps_lb->end());
     }
@@ -725,7 +825,7 @@ void SitRepPanel::SetHiddenSitRepTemplates(const std::set<std::string>& template
 }
 
 int SitRepPanel::NumVisibleSitrepsThisTurn() const {
-    std::map<int, std::list<SitRepEntry> > turns = GetSitRepsSortedByTurn(HumanClientApp::GetApp()->EmpireID(), m_hidden_sitrep_templates);
+    std::map<int, std::list<SitRepEntry>> turns = GetSitRepsSortedByTurn(HumanClientApp::GetApp()->EmpireID(), m_hidden_sitrep_templates);
     return turns[CurrentTurn()].size();
 }
 
