@@ -11,7 +11,7 @@ class SitRepEntry;
 class SitRepPanel : public CUIWnd {
 public:
     /** \name Structors */ //@{
-    SitRepPanel(const std::string& config_name = ""); ///< basic ctor
+    SitRepPanel(const std::string& config_name = "");
     //@}
 
     /** \name Accessors */ //@{
@@ -20,8 +20,9 @@ public:
     //@}
 
     /** \name Mutators */ //@{
-    virtual void    KeyPress (GG::Key key, boost::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr);
+    void KeyPress(GG::Key key, std::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
 
     void            Update(); ///< loads all the relevant SitReps into the window
 
@@ -32,16 +33,17 @@ public:
     mutable boost::signals2::signal<void ()> ClosingSignal;
 
 private:
-    virtual void    CloseClicked();
+    void CloseClicked() override;
+
     void            PrevClicked();
     void            NextClicked();
     void            LastClicked();
     void            FilterClicked();
-    void            DismissSitRep(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& mod);
+    void            IgnoreSitRep(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& mod);
     void            DismissalMenu(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& mod);
     void            DoLayout();
 
-    int             GetNextNonEmptySitrepsTurn(const std::map<int, std::list<SitRepEntry> >& turns,
+    int             GetNextNonEmptySitrepsTurn(const std::map<int, std::list<SitRepEntry>>& turns,
                                                int turn, bool forward) const;   ///< Return next turn with sitreps
 
     GG::ListBox*            m_sitreps_lb;
