@@ -11,17 +11,16 @@
 #include <AL/al.h>
 #endif
 
-#include <map>
-
-#include <boost/scoped_array.hpp>
-
 #include <vorbis/vorbisfile.h>
 
-class Sound::SoundImpl
+#include <map>
+
+class Sound::Impl
 {
 public:
-    SoundImpl();  ///< ctor.
-    ~SoundImpl(); ///< dotr.
+    Impl();
+
+    ~Impl();
 
     /** Enables the sound system.  Throws Sound::InitializationFailureException on failure. */
     void Enable();
@@ -39,6 +38,12 @@ public:
         loops + 1 times otherwise. */
     void PlayMusic(const boost::filesystem::path& path, int loops = 0);
 
+    /** Pauses music play, to be continued from the same position */
+    void PauseMusic();
+
+    /** Resumes music play */
+    void ResumeMusic();
+
     /** Stops playing music. */
     void StopMusic();
 
@@ -104,10 +109,10 @@ namespace {
         ALenum m_openal_error;
         int endian = 0; /// 0 for little-endian (x86), 1 for big-endian (ppc)
         int bitStream, bytes, bytes_new;
-        boost::scoped_array<char> array(new char[buffer_size]);
+        std::unique_ptr<char[]> array(new char[buffer_size]);
         bytes = 0;
 
-        if (alcGetCurrentContext() != 0) {
+        if (alcGetCurrentContext()) {
             /* First, let's fill up the buffer. We need the loop, as ov_read treats (buffer_size - bytes) to read as a suggestion only */
             do {
                 bytes_new = ov_read(ogg_file, &array[bytes],(buffer_size - bytes), endian, 2, 1, &bitStream);
@@ -141,10 +146,10 @@ namespace {
 // TempUISoundDisabler
 ////////////////////////////////////////////////////////////
 Sound::TempUISoundDisabler::TempUISoundDisabler()
-{ Sound::GetSound().pimpl->IncDisabledCount(); }
+{ Sound::GetSound().m_impl->IncDisabledCount(); }
 
 Sound::TempUISoundDisabler::~TempUISoundDisabler()
-{ Sound::GetSound().pimpl->DecDisabledCount(); }
+{ Sound::GetSound().m_impl->DecDisabledCount(); }
 
 ////////////////////////////////////////////////////////////
 // Sound
@@ -156,49 +161,50 @@ Sound& Sound::GetSound()
 }
 
 Sound::Sound() :
-    pimpl(new SoundImpl)
+    m_impl(new Impl)
 {}
 
-// Require here because SoundImpl is defined in this file.
-Sound::~Sound() {}
+// Require here because Sound::Impl is defined in this file.
+Sound::~Sound() = default;
 
 void Sound::Enable()
-{ pimpl->Enable(); }
+{ m_impl->Enable(); }
 
 void Sound::Disable()
-{ pimpl->Disable(); }
+{ m_impl->Disable(); }
 
 void Sound::PlayMusic(const boost::filesystem::path& path, int loops /* = 0*/)
-{ pimpl->PlayMusic(path, loops); }
+{ m_impl->PlayMusic(path, loops); }
+
+void Sound::PauseMusic()
+{ m_impl->PauseMusic(); }
+
+void Sound::ResumeMusic()
+{ m_impl->ResumeMusic(); }
 
 void Sound::StopMusic()
-{ pimpl->StopMusic(); }
+{ m_impl->StopMusic(); }
 
 void Sound::PlaySound(const boost::filesystem::path& path, bool is_ui_sound/* = false*/)
-{ pimpl->PlaySound(path, is_ui_sound); }
+{ m_impl->PlaySound(path, is_ui_sound); }
 
 void Sound::FreeSound(const boost::filesystem::path& path)
-{ pimpl->FreeSound(path); }
+{ m_impl->FreeSound(path); }
 
 void Sound::FreeAllSounds()
-{ pimpl->FreeAllSounds(); }
+{ m_impl->FreeAllSounds(); }
 
 void Sound::SetMusicVolume(int vol)
-{ pimpl->SetMusicVolume(vol); }
+{ m_impl->SetMusicVolume(vol); }
 
 void Sound::SetUISoundsVolume(int vol)
-{ pimpl->SetUISoundsVolume(vol); }
+{ m_impl->SetUISoundsVolume(vol); }
 
 void Sound::DoFrame()
-{ pimpl->DoFrame(); }
+{ m_impl->DoFrame(); }
 
 
-
-////////////////////////////////////////////////////////////
-// SoundImpl
-////////////////////////////////////////////////////////////
-
-Sound::SoundImpl::SoundImpl() :
+Sound::Impl::Impl() :
     m_sources(),
     m_music_loops(0),
     m_music_name(),
@@ -222,11 +228,11 @@ Sound::SoundImpl::SoundImpl() :
     }
 }
 
-Sound::SoundImpl::~SoundImpl() {
+Sound::Impl::~Impl() {
     Disable();
 }
 
-void Sound::SoundImpl::Enable() {
+void Sound::Impl::Enable() {
     if (m_initialized)
         return;
 
@@ -237,7 +243,7 @@ void Sound::SoundImpl::Enable() {
 
     InitOpenAL();
     if (!m_initialized) {
-        ErrorLogger() "Unable to initialize audio.  Sound effects and music are disabled.";
+        ErrorLogger() << "Unable to initialize audio.  Sound effects and music are disabled.";
         // TODO: Let InitOpenAL throw the OpenAL error message which
         // might be more useful.
         throw InitializationFailureException("ERROR_SOUND_INITIALIZATION_FAILED");
@@ -246,7 +252,7 @@ void Sound::SoundImpl::Enable() {
     DebugLogger() << "Audio " << (m_initialized ? "enabled." : "disabled.");
 }
 
-void Sound::SoundImpl::Disable() {
+void Sound::Impl::Disable() {
     if (!m_initialized)
         return;
     StopMusic();
@@ -265,24 +271,34 @@ void Sound::SoundImpl::Disable() {
 /// Initialize OpenAl and return true on success.
 // TODO rewrite with std::unique_ptr and custom deleter to remove long
 // chain of "destructor" code.
-void Sound::SoundImpl::InitOpenAL() {
+void Sound::Impl::InitOpenAL() {
     ALCcontext *context;
     ALCdevice *device;
     ALenum error_code;
+    ALboolean status;
 
-    device = alcOpenDevice(0);    /* currently only select the default output device - usually a NULL-terminated
-                                   * string desctribing a device can be passed here (of type ALchar*) */
-    if (device == 0) {
+    device = alcOpenDevice(nullptr);
+
+    if (!device) {
         ErrorLogger() << "Unable to initialise default OpenAL device.";
         m_initialized = false;
         return;
     }
 
-    context = alcCreateContext(device, 0);   // instead of 0 we can pass a ALCint* pointing to a set of
-    // attributes (ALC_FREQUENCY, ALC_REFRESH and ALC_SYNC)
+    context = alcCreateContext(device, nullptr);
+    if (!context) {
+        error_code = alGetError();
+        ErrorLogger() << "Unable to create OpenAL context: " << alGetString(error_code) << "\n";
+        alcCloseDevice(device);
+        m_initialized = false;
+        return;
+    }
 
-    if (!((context != 0) && (alcMakeContextCurrent(context) == AL_TRUE) && ((error_code = alGetError()) == AL_NO_ERROR) )) {
-        ErrorLogger() << "Unable to create OpenAL context : " << alGetString(error_code) << "\n";
+    status = alcMakeContextCurrent(context);
+    error_code = alGetError();
+    if (status != AL_TRUE || error_code != AL_NO_ERROR) {
+        ErrorLogger() << "Unable to make OpenAL context current: " << alGetString(error_code) << "\n";
+        alcDestroyContext(context);
         alcCloseDevice(device);
         m_initialized = false;
         return;
@@ -292,7 +308,7 @@ void Sound::SoundImpl::InitOpenAL() {
     error_code = alGetError();
     if (error_code != AL_NO_ERROR) {
         ErrorLogger() << "Unable to create OpenAL listener: " << alGetString(error_code) << "\n" << "Disabling OpenAL sound system!\n";
-        alcMakeContextCurrent(0);
+        alcMakeContextCurrent(nullptr);
         alcDestroyContext(context);
         alcCloseDevice(device);
         m_initialized = false;
@@ -303,7 +319,7 @@ void Sound::SoundImpl::InitOpenAL() {
     error_code = alGetError();
     if (error_code != AL_NO_ERROR) {
         ErrorLogger() << "Unable to create OpenAL sources: " << alGetString(error_code) << "\n" << "Disabling OpenAL sound system!\n";
-        alcMakeContextCurrent(0);
+        alcMakeContextCurrent(nullptr);
         alcDestroyContext(context);
         alcCloseDevice(device);
         m_initialized = false;
@@ -315,7 +331,7 @@ void Sound::SoundImpl::InitOpenAL() {
     if (error_code != AL_NO_ERROR) {
         ErrorLogger() << "Unable to create OpenAL buffers: " << alGetString(error_code) << "\n" << "Disabling OpenAL sound system!\n";
         alDeleteSources(NUM_SOURCES, m_sources);
-        alcMakeContextCurrent(0);
+        alcMakeContextCurrent(nullptr);
         alcDestroyContext(context);
         alcCloseDevice(device);
         m_initialized = false;
@@ -329,7 +345,7 @@ void Sound::SoundImpl::InitOpenAL() {
             ErrorLogger() << "Unable to set OpenAL source to relative: " << alGetString(error_code) << "\n" << "Disabling OpenAL sound system!\n";
             alDeleteBuffers(NUM_MUSIC_BUFFERS, m_music_buffers);
             alDeleteSources(NUM_SOURCES, m_sources);
-            alcMakeContextCurrent(0);
+            alcMakeContextCurrent(nullptr);
             alcDestroyContext(context);
             alcCloseDevice(device);
             m_initialized = false;
@@ -347,34 +363,35 @@ void Sound::SoundImpl::InitOpenAL() {
     m_initialized = true;
 }
 
-void Sound::SoundImpl::ShutdownOpenAL() {
+void Sound::Impl::ShutdownOpenAL() {
+    ALCcontext* context = alcGetCurrentContext();
 
-    if (alcGetCurrentContext() != 0) {
+    if (context) {
         alDeleteSources(NUM_SOURCES, m_sources); // Automatically stops currently playing sources
 
         alDeleteBuffers(NUM_MUSIC_BUFFERS, m_music_buffers);
-        for (std::map<std::string, ALuint>::iterator itr = m_buffers.begin(); itr != m_buffers.end(); ++itr)
-            alDeleteBuffers(1, &itr->second );
-    }
+        for (std::map<std::string, ALuint>::value_type& buffer : m_buffers)
+            alDeleteBuffers(1, &(buffer.second));
 
-    ALCcontext* context = alcGetCurrentContext();
-    if (context != 0) {
         ALCdevice* device = alcGetContextsDevice(context);
-        alcMakeContextCurrent(0);
+
+        alcMakeContextCurrent(nullptr);
+
         alcDestroyContext(context);
-        if (device != 0)
+
+        if (device)
             alcCloseDevice(device);
     }
 }
 
 
-void Sound::SoundImpl::PlayMusic(const boost::filesystem::path& path, int loops /* = 0*/) {
+void Sound::Impl::PlayMusic(const boost::filesystem::path& path, int loops /* = 0*/) {
     if (!m_initialized)
         return;
 
     ALenum m_openal_error;
     std::string filename = PathString(path);
-    FILE* m_f = 0;
+    FILE* m_f = nullptr;
     vorbis_info* vorbis_info;
     m_music_loops = 0;
 
@@ -392,17 +409,17 @@ void Sound::SoundImpl::PlayMusic(const boost::filesystem::path& path, int loops
     };
 #endif
 
-    if (alcGetCurrentContext() != 0)
+    if (alcGetCurrentContext())
     {
         if (m_music_name.size() > 0)
             StopMusic();
        
-        if ((m_f = fopen(filename.c_str(), "rb")) != 0) // make sure we CAN open it
+        if ((m_f = fopen(filename.c_str(), "rb")) != nullptr) // make sure we CAN open it
         {
 #ifdef FREEORION_WIN32
-            if (!(ov_test_callbacks(m_f, &m_ogg_file, 0, 0, callbacks))) // check if it's a proper ogg
+            if (!(ov_test_callbacks(m_f, &m_ogg_file, nullptr, 0, callbacks))) // check if it's a proper ogg
 #else
-            if (!(ov_test(m_f, &m_ogg_file, 0, 0))) // check if it's a proper ogg
+            if (!(ov_test(m_f, &m_ogg_file, nullptr, 0))) // check if it's a proper ogg
 #endif
             {
                 ov_test_open(&m_ogg_file); // it is, now fully open the file
@@ -449,11 +466,30 @@ void Sound::SoundImpl::PlayMusic(const boost::filesystem::path& path, int loops
         ErrorLogger() << "PlayMusic: OpenAL ERROR: " << alGetString(m_openal_error);
 }
 
-void Sound::SoundImpl::StopMusic() {
+void Sound::Impl::PauseMusic() {
     if (!m_initialized)
         return;
 
-    if (alcGetCurrentContext() != 0) {
+    if (alcGetCurrentContext()) {
+        alSourcePause(m_sources[0]);
+    }
+}
+
+void Sound::Impl::ResumeMusic() {
+    if (!m_initialized)
+        return;
+
+    if (alcGetCurrentContext()) {
+        alSourcePlay(m_sources[0]);
+    }
+}
+
+
+void Sound::Impl::StopMusic() {
+    if (!m_initialized)
+        return;
+
+    if (alcGetCurrentContext()) {
         alSourceStop(m_sources[0]);
         if (m_music_name.size() > 0) {
             m_music_name.clear();  // do this to avoid music being re-started by other functions
@@ -463,7 +499,7 @@ void Sound::SoundImpl::StopMusic() {
     }
 }
 
-void Sound::SoundImpl::PlaySound(const boost::filesystem::path& path, bool is_ui_sound/* = false*/) {
+void Sound::Impl::PlaySound(const boost::filesystem::path& path, bool is_ui_sound/* = false*/) {
     if (!m_initialized || !GetOptionsDB().Get<bool>("UI.sound.enabled") || (is_ui_sound && UISoundsTemporarilyDisabled()))
         return;
 
@@ -471,7 +507,7 @@ void Sound::SoundImpl::PlaySound(const boost::filesystem::path& path, bool is_ui
     ALuint current_buffer;
     ALenum source_state;
     ALsizei ogg_freq;
-    FILE *file = 0;
+    FILE *file = nullptr;
     int m_i;
     bool found_buffer = false;
     bool found_source = false;
@@ -490,21 +526,21 @@ void Sound::SoundImpl::PlaySound(const boost::filesystem::path& path, bool is_ui
     };
 #endif
 
-    if (alcGetCurrentContext() != 0) {
+    if (alcGetCurrentContext()) {
         /* First check if the sound data of the file we want to play is already buffered somewhere */
         std::map<std::string, ALuint>::iterator it = m_buffers.find(filename);
         if (it != m_buffers.end()) {
             current_buffer = it->second;
             found_buffer = true;
         } else {
-            if ((file = fopen(filename.c_str(), "rb")) != 0) { // make sure we CAN open it
+            if ((file = fopen(filename.c_str(), "rb")) != nullptr) { // make sure we CAN open it
                 OggVorbis_File ogg_file;
                 vorbis_info *vorbis_info;
                 ALenum ogg_format;
 #ifdef FREEORION_WIN32
-                if (!(ov_test_callbacks(file, &ogg_file, 0, 0, callbacks))) // check if it's a proper ogg
+                if (!(ov_test_callbacks(file, &ogg_file, nullptr, 0, callbacks))) // check if it's a proper ogg
 #else
-                if (!(ov_test(file, &ogg_file, 0, 0))) // check if it's a proper ogg
+                if (!(ov_test(file, &ogg_file, nullptr, 0))) // check if it's a proper ogg
 #endif
                 {
                     ov_test_open(&ogg_file); // it is, now fully open the file
@@ -563,7 +599,7 @@ void Sound::SoundImpl::PlaySound(const boost::filesystem::path& path, bool is_ui
     }
 }
 
-void Sound::SoundImpl::FreeSound(const boost::filesystem::path& path) {
+void Sound::Impl::FreeSound(const boost::filesystem::path& path) {
     if (!m_initialized)
         return;
 
@@ -582,7 +618,7 @@ void Sound::SoundImpl::FreeSound(const boost::filesystem::path& path) {
     }
 }
 
-void Sound::SoundImpl::FreeAllSounds() {
+void Sound::Impl::FreeAllSounds() {
     if (!m_initialized)
         return;
 
@@ -604,7 +640,7 @@ void Sound::SoundImpl::FreeAllSounds() {
     }
 }
 
-void Sound::SoundImpl::SetMusicVolume(int vol) {
+void Sound::Impl::SetMusicVolume(int vol) {
     if (!m_initialized)
         return;
 
@@ -613,7 +649,7 @@ void Sound::SoundImpl::SetMusicVolume(int vol) {
     /* normalize value, then apply to all sound sources */
     vol = std::max(0, std::min(vol, 255));
     GetOptionsDB().Set<int>("UI.sound.music-volume", vol);
-    if (alcGetCurrentContext() != 0)
+    if (alcGetCurrentContext())
     {
         alSourcef(m_sources[0], AL_GAIN, ((ALfloat) vol)/255.0);
         /* it is highly unlikely that we'll get an error here but better safe than sorry */
@@ -623,7 +659,7 @@ void Sound::SoundImpl::SetMusicVolume(int vol) {
     }
 }
 
-void Sound::SoundImpl::SetUISoundsVolume(int vol) {
+void Sound::Impl::SetUISoundsVolume(int vol) {
     if (!m_initialized)
         return;
 
@@ -632,7 +668,7 @@ void Sound::SoundImpl::SetUISoundsVolume(int vol) {
     /* normalize value, then apply to all sound sources */
     vol = std::max(0, std::min(vol, 255));
     GetOptionsDB().Set<int>("UI.sound.volume", vol);
-    if (alcGetCurrentContext() != 0) {
+    if (alcGetCurrentContext()) {
         for (int it = 1; it < NUM_SOURCES; ++it)
             alSourcef(m_sources[it], AL_GAIN, ((ALfloat) vol)/255.0);
         /* it is highly unlikely that we'll get an error here but better safe than sorry */
@@ -642,14 +678,14 @@ void Sound::SoundImpl::SetUISoundsVolume(int vol) {
     }
 }
 
-void Sound::SoundImpl::DoFrame() {
+void Sound::Impl::DoFrame() {
     if (!m_initialized)
         return;
 
     ALint    state;
     int      num_buffers_processed;
 
-    if ((alcGetCurrentContext() != 0) && (m_music_name.size() > 0)) {
+    if (alcGetCurrentContext() && (m_music_name.size() > 0)) {
         alGetSourcei(m_sources[0], AL_BUFFERS_PROCESSED, &num_buffers_processed);
         while (num_buffers_processed > 0) {
             ALuint buffer_name_yay;
@@ -668,11 +704,11 @@ void Sound::SoundImpl::DoFrame() {
     }
 } 
 
-bool Sound::SoundImpl::UISoundsTemporarilyDisabled() const
+bool Sound::Impl::UISoundsTemporarilyDisabled() const
 { return m_temporary_disable_count > 0; }
 
-void Sound::SoundImpl::IncDisabledCount()
+void Sound::Impl::IncDisabledCount()
 { ++m_temporary_disable_count; }
 
-void Sound::SoundImpl::DecDisabledCount()
+void Sound::Impl::DecDisabledCount()
 { --m_temporary_disable_count; }
