@@ -1,7 +1,5 @@
 #include "SpecialsPanel.h"
 
-#include <GG/Menu.h>
-
 #include "../util/i18n.h"
 #include "../util/Logger.h"
 #include "../util/OptionsDB.h"
@@ -31,8 +29,8 @@ SpecialsPanel::SpecialsPanel(GG::X w, int object_id) :
 
 bool SpecialsPanel::InWindow(const GG::Pt& pt) const {
     bool retval = false;
-    for (std::map<std::string, StatisticIcon*>::const_iterator it = m_icons.begin(); it != m_icons.end(); ++it) {
-        if (it->second->InWindow(pt)) {
+    for (const std::map<std::string, StatisticIcon*>::value_type& entry : m_icons) {
+        if (entry.second->InWindow(pt)) {
             retval = true;
             break;
         }
@@ -57,39 +55,39 @@ void SpecialsPanel::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
 
 void SpecialsPanel::Update() {
     //std::cout << "SpecialsPanel::Update" << std::endl;
-    for (std::map<std::string, StatisticIcon*>::iterator it = m_icons.begin(); it != m_icons.end(); ++it)
-        DeleteChild(it->second);
+    for (std::map<std::string, StatisticIcon*>::value_type& entry : m_icons)
+        DeleteChild(entry.second);
     m_icons.clear();
 
 
     // get specials to display
-    TemporaryPtr<const UniverseObject> obj = GetUniverseObject(m_object_id);
+    std::shared_ptr<const UniverseObject> obj = GetUniverseObject(m_object_id);
     if (!obj) {
         ErrorLogger() << "SpecialsPanel::Update couldn't get object with id " << m_object_id;
         return;
     }
-    const std::map<std::string, std::pair<int, float> >& specials = obj->Specials();
-
 
     // get specials and use them to create specials icons
     // for specials with a nonzero
-    for (std::map<std::string, std::pair<int, float> >::const_iterator it = specials.begin(); it != specials.end(); ++it) {
-        const Special* special = GetSpecial(it->first);
-        StatisticIcon* graphic = 0;
-        if (it->second.second > 0.0f)
-            graphic = new StatisticIcon(ClientUI::SpecialIcon(special->Name()), it->second.second, 2, false);
+    for (const std::map<std::string, std::pair<int, float>>::value_type& entry : obj->Specials()) {
+        const Special* special = GetSpecial(entry.first);
+        StatisticIcon* graphic = nullptr;
+        if (entry.second.second > 0.0f)
+            graphic = new StatisticIcon(ClientUI::SpecialIcon(special->Name()), entry.second.second, 2, false,
+                                        GG::X0, GG::Y0, SPECIAL_ICON_WIDTH, SPECIAL_ICON_HEIGHT);
         else
-            graphic = new StatisticIcon(ClientUI::SpecialIcon(special->Name()));
+            graphic = new StatisticIcon(ClientUI::SpecialIcon(special->Name()),
+                                        GG::X0, GG::Y0, SPECIAL_ICON_WIDTH, SPECIAL_ICON_HEIGHT);
 
         graphic->SetBrowseModeTime(GetOptionsDB().Get<int>("UI.tooltip-delay"));
 
         std::string desc = special->Description();
 
-        if (it->second.second > 0.0f)
-            desc += "\n" + boost::io::str(FlexibleFormat(UserString("SPECIAL_CAPACITY")) % DoubleToString(it->second.second, 2, false));
+        if (entry.second.second > 0.0f)
+            desc += "\n" + boost::io::str(FlexibleFormat(UserString("SPECIAL_CAPACITY")) % DoubleToString(entry.second.second, 2, false));
 
-        if (it->second.first > 0)
-            desc += "\n" + boost::io::str(FlexibleFormat(UserString("ADDED_ON_TURN")) % boost::lexical_cast<std::string>(it->second.first));
+        if (entry.second.first > 0)
+            desc += "\n" + boost::io::str(FlexibleFormat(UserString("ADDED_ON_TURN")) % entry.second.first);
         else
             desc += "\n" + UserString("ADDED_ON_INITIAL_TURN");
 
@@ -97,11 +95,9 @@ void SpecialsPanel::Update() {
             desc += "\n" + Dump(special->Effects());
         }
 
-        graphic->SetBrowseInfoWnd(boost::shared_ptr<GG::BrowseInfoWnd>(
-            new IconTextBrowseWnd(ClientUI::SpecialIcon(special->Name()),
-                                  UserString(special->Name()),
-                                  desc)));
-        m_icons[it->first] = graphic;
+        graphic->SetBrowseInfoWnd(std::make_shared<IconTextBrowseWnd>(
+            ClientUI::SpecialIcon(special->Name()), UserString(special->Name()), desc));
+        m_icons[entry.first] = graphic;
 
         graphic->InstallEventFilter(this);
     }
@@ -110,8 +106,8 @@ void SpecialsPanel::Update() {
     GG::X x(EDGE_PAD);
     GG::Y y(EDGE_PAD);
 
-    for (std::map<std::string, StatisticIcon*>::iterator it = m_icons.begin(); it != m_icons.end(); ++it) {
-        StatisticIcon* icon = it->second;
+    for (std::map<std::string, StatisticIcon*>::value_type& entry : m_icons) {
+        StatisticIcon* icon = entry.second;
         icon->SizeMove(GG::Pt(x, y), GG::Pt(x,y) + GG::Pt(SPECIAL_ICON_WIDTH, SPECIAL_ICON_HEIGHT));
         AttachChild(icon);
 
@@ -135,25 +131,23 @@ bool SpecialsPanel::EventFilter(GG::Wnd* w, const GG::WndEvent& event) {
         return false;
     const GG::Pt& pt = event.Point();
 
-    for (std::map<std::string, StatisticIcon*>::const_iterator it = m_icons.begin();
-            it != m_icons.end(); ++it)
+    for (const std::map<std::string, StatisticIcon*>::value_type& entry : m_icons)
     {
-        if (it->second != w)
+        if (entry.second != w)
             continue;
 
-        std::string popup_label = boost::io::str(FlexibleFormat(UserString("ENC_LOOKUP")) % UserString(it->first));
+        std::string popup_label = boost::io::str(FlexibleFormat(UserString("ENC_LOOKUP")) % UserString(entry.first));
 
         GG::MenuItem menu_contents;
         menu_contents.next_level.push_back(GG::MenuItem(popup_label, 1, false, false));
-        GG::PopupMenu popup(pt.x, pt.y, ClientUI::GetFont(), menu_contents, ClientUI::TextColor(),
-                            ClientUI::WndOuterBorderColor(), ClientUI::WndColor(), ClientUI::EditHiliteColor());
+        CUIPopupMenu popup(pt.x, pt.y, menu_contents);
 
         if (!popup.Run() || popup.MenuID() != 1) {
             return false;
             break;
         }
 
-        ClientUI::GetClientUI()->ZoomToSpecial(it->first);
+        ClientUI::GetClientUI()->ZoomToSpecial(entry.first);
         return true;
     }
     return false;
