@@ -9,19 +9,23 @@ class StatisticIcon;
 class SpecialsPanel : public GG::Wnd {
 public:
     /** \name Structors */ //@{
-    SpecialsPanel(GG::X w, int object_id);   ///< basic ctor
+    SpecialsPanel(GG::X w, int object_id);
     //@}
 
     /** \name Accessors */ //@{
-    bool                    InWindow(const GG::Pt& pt) const;
+    bool InWindow(const GG::Pt& pt) const override;
+
     int                     ObjectID() const { return m_object_id; }
     //@}
 
     /** \name Mutators */ //@{
-    virtual void            Render();
-    virtual void            MouseWheel(const GG::Pt& pt, int move, GG::Flags<GG::ModKey> mod_keys);
-    virtual void            SizeMove(const GG::Pt& ul, const GG::Pt& lr);
-    virtual bool            EventFilter(GG::Wnd* w, const GG::WndEvent& event);
+    void Render() override;
+
+    void MouseWheel(const GG::Pt& pt, int move, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
+
+    bool EventFilter(GG::Wnd* w, const GG::WndEvent& event) override;
 
     void                    Update();          ///< regenerates indicators according specials on object
     //@}
