@@ -13,6 +13,7 @@
 #include "../universe/Planet.h"
 #include "../universe/Building.h"
 #include "../universe/Species.h"
+#include "../universe/Enums.h"
 #include "../client/human/HumanClientApp.h"
 #include "../util/i18n.h"
 #include "../util/Logger.h"
@@ -25,7 +26,6 @@
 #include <GG/utf8/checked.h>
 
 #include <boost/format.hpp>
-#include <boost/lexical_cast.hpp>
 
 namespace {
     bool PlaySounds()                   { return GetOptionsDB().Get<bool>("UI.sound.enabled"); }
@@ -60,7 +60,7 @@ namespace {
             std::string retval;
             int start = 0;
 
-            for (std::set<int>::iterator it = owner_empire_ids.begin(); it != owner_empire_ids.end(); ++it) {
+            for (int empire_id : owner_empire_ids) {
                 int current_length = piece_length;
                 if (extra > 0) { // Use left over letters as long as we have them
                     ++current_length;
@@ -71,7 +71,7 @@ namespace {
                 utf8::utf32to8(name.begin() + start, name.begin() + start + current_length, std::back_inserter(piece));
 
                 GG::Clr empire_clr = ClientUI::TextColor();
-                const Empire* empire = GetEmpire(*it);
+                const Empire* empire = GetEmpire(empire_id);
                 if (empire)
                     empire_clr = empire->Color();
 
@@ -98,7 +98,7 @@ OwnerColoredSystemName::OwnerColoredSystemName(int system_id, int font_size, boo
 
     int client_empire_id = HumanClientApp::GetApp()->EmpireID();
 
-    TemporaryPtr<const System> system = GetSystem(system_id);
+    std::shared_ptr<const System> system = GetSystem(system_id);
     if (!system)
         return;
 
@@ -117,12 +117,9 @@ OwnerColoredSystemName::OwnerColoredSystemName(int system_id, int font_size, boo
     bool capital = false, homeworld = false, has_shipyard = false, has_neutrals = false, has_player_planet = false;
 
     std::set<int> owner_empire_ids;
-    std::vector<TemporaryPtr<const Planet> > system_planets = Objects().FindObjects<const Planet>(system->PlanetIDs());
+    std::vector<std::shared_ptr<const Planet>> system_planets = Objects().FindObjects<const Planet>(system->PlanetIDs());
 
-    for (std::vector<TemporaryPtr<const Planet> >::const_iterator planet_it = system_planets.begin();
-         planet_it != system_planets.end(); ++planet_it)
-    {
-        TemporaryPtr<const Planet> planet = *planet_it;
+    for (std::shared_ptr<const Planet> planet : system_planets) {
         int planet_id = planet->ID();
 
         if (known_destroyed_object_ids.find(planet_id) != known_destroyed_object_ids.end())
@@ -130,10 +127,8 @@ OwnerColoredSystemName::OwnerColoredSystemName(int system_id, int font_size, boo
 
         // is planet a capital?
         if (!capital) {
-            for (EmpireManager::const_iterator empire_it = empire_manager.begin();
-                 empire_it != empire_manager.end(); ++empire_it)
-            {
-                if (empire_it->second->CapitalID() == planet_id) {
+            for (const std::map<int, Empire*>::value_type& entry : empire_manager) {
+                if (entry.second->CapitalID() == planet_id) {
                     capital = true;
                     break;
                 }
@@ -142,10 +137,8 @@ OwnerColoredSystemName::OwnerColoredSystemName(int system_id, int font_size, boo
 
         // is planet a homeworld? (for any species)
         if (!homeworld) {
-            for (SpeciesManager::iterator species_it = species_manager.begin();
-                 species_it != species_manager.end(); ++species_it)
-            {
-                if (const Species* species = species_it->second) {
+            for (const std::map<std::string, Species*>::value_type& entry : species_manager) {
+                if (const Species* species = entry.second) {
                     const std::set<int>& homeworld_ids = species->Homeworlds();
                     if (homeworld_ids.find(planet_id) != homeworld_ids.end()) {
                         homeworld = true;
@@ -157,19 +150,13 @@ OwnerColoredSystemName::OwnerColoredSystemName(int system_id, int font_size, boo
 
         // does planet contain a shipyard?
         if (!has_shipyard) {
-            std::vector<TemporaryPtr<const Building> > buildings = Objects().FindObjects<const Building>(planet->BuildingIDs());
-            for (std::vector<TemporaryPtr<const Building> >::const_iterator building_it = buildings.begin();
-                 building_it != buildings.end(); ++building_it)
-            {
-                TemporaryPtr<const Building> building = *building_it;
+            for (std::shared_ptr<const Building> building : Objects().FindObjects<const Building>(planet->BuildingIDs())) {
                 int building_id = building->ID();
 
                 if (known_destroyed_object_ids.find(building_id) != known_destroyed_object_ids.end())
                     continue;
 
-                std::list<std::string> shipyards_list;
-                UserStringList("FUNCTIONAL_SHIPYARD_BUILDING_LIST", shipyards_list);
-                if (std::find(shipyards_list.begin(), shipyards_list.end(), building->BuildingTypeName()) != shipyards_list.end()) {
+                if (building->HasTag(TAG_SHIPYARD)) {
                     has_shipyard = true;
                     break;
                 }
@@ -202,7 +189,7 @@ OwnerColoredSystemName::OwnerColoredSystemName(int system_id, int font_size, boo
         wrapped_system_name = "<i>" + wrapped_system_name + "</i>";
     if (has_shipyard)
         wrapped_system_name = "<u>" + wrapped_system_name + "</u>";
-    boost::shared_ptr<GG::Font> font;
+    std::shared_ptr<GG::Font> font;
     if (capital)
         font = ClientUI::GetBoldFont(font_size);
     else
@@ -216,32 +203,50 @@ OwnerColoredSystemName::OwnerColoredSystemName(int system_id, int font_size, boo
         text_color = ClientUI::TextColor();
     }
 
-    GG::Control* text = new GG::TextControl(GG::X0, GG::Y0, GG::X1, GG::Y1, "<s>" + wrapped_system_name + "</s>", font, text_color);
+    if (GetOptionsDB().Get<bool>("UI.show-id-after-names")) {
+        wrapped_system_name = wrapped_system_name + " (" + std::to_string(system_id) + ")";
+    }
+
+    GG::TextControl* text = new GG::TextControl(
+        GG::X0, GG::Y0, GG::X1, GG::Y1, "<s>" + wrapped_system_name + "</s>", font, text_color);
     AttachChild(text);
-    Resize(GG::Pt(text->Width(), text->Height()));
+    GG::Pt text_size(text->TextLowerRight() - text->TextUpperLeft());
+    text->SizeMove(GG::Pt(GG::X0, GG::Y0), text_size);
+    Resize(text_size);
 }
 
 void OwnerColoredSystemName::Render()
 {}
 
+void OwnerColoredSystemName::SizeMove(const GG::Pt& ul, const GG::Pt& lr)
+{
+    GG::Control::SizeMove(ul, lr);
+
+    // Center text
+    if (!Children().empty())
+        if (GG::TextControl* text = dynamic_cast<GG::TextControl*>(*Children().begin())) {
+            text->MoveTo(GG::Pt((Width() - text->Width()) / 2, (Height() - text->Height()) / 2));
+        }
+}
+
 ////////////////////////////////////////////////
 // SystemIcon
 ////////////////////////////////////////////////
 SystemIcon::SystemIcon(GG::X x, GG::Y y, GG::X w, int system_id) :
     GG::Control(x, y, w, GG::Y(Value(w)), GG::INTERACTIVE),
     m_system_id(system_id),
-    m_tiny_graphic(0),
-    m_selection_indicator(0),
-    m_tiny_selection_indicator(0),
-    m_mouseover_indicator(0),
-    m_mouseover_unexplored_indicator(0),
-    m_tiny_mouseover_indicator(0),
+    m_tiny_graphic(nullptr),
+    m_selection_indicator(nullptr),
+    m_tiny_selection_indicator(nullptr),
+    m_mouseover_indicator(nullptr),
+    m_mouseover_unexplored_indicator(nullptr),
+    m_tiny_mouseover_indicator(nullptr),
     m_selected(false),
-    m_colored_name(0),
+    m_colored_name(nullptr),
     m_showing_name(false)
 {
     ClientUI* ui = ClientUI::GetClientUI();
-    if (TemporaryPtr<const System> system = GetSystem(m_system_id)) {
+    if (std::shared_ptr<const System> system = GetSystem(m_system_id)) {
         StarType star_type = system->GetStarType();
         m_disc_texture = ui->GetModuloTexture(ClientUI::ArtDir() / "stars",
                                               ClientUI::StarTypeFilePrefixes()[star_type],
@@ -264,7 +269,7 @@ SystemIcon::SystemIcon(GG::X x, GG::Y y, GG::X w, int system_id) :
     m_tiny_graphic->Hide();
 
     // selection indicator graphic
-    const boost::shared_ptr<GG::Texture>& texture = ClientUI::GetTexture(ClientUI::ArtDir() / "misc" / "system_selection" / "system_selection2.png", true);
+    const std::shared_ptr<GG::Texture>& texture = ClientUI::GetTexture(ClientUI::ArtDir() / "misc" / "system_selection" / "system_selection2.png", true);
 
     GG::X texture_width = texture->DefaultWidth();
     GG::Y texture_height = texture->DefaultHeight();
@@ -274,7 +279,7 @@ SystemIcon::SystemIcon(GG::X x, GG::Y y, GG::X w, int system_id) :
     m_selection_indicator->Resize(GG::Pt(texture_width, texture_height));
 
     // tiny selection indicator graphic
-    const boost::shared_ptr<GG::Texture>& tiny_texture = ClientUI::GetTexture(ClientUI::ArtDir() / "misc" / "system_selection_tiny" / "system_selection_tiny2.png", true);
+    const std::shared_ptr<GG::Texture>& tiny_texture = ClientUI::GetTexture(ClientUI::ArtDir() / "misc" / "system_selection_tiny" / "system_selection_tiny2.png", true);
     texture_width = tiny_texture->DefaultWidth();
     texture_height = tiny_texture->DefaultHeight();
     m_tiny_selection_indicator = new RotatingGraphic(tiny_texture, GG::GRAPHIC_NONE);
@@ -283,21 +288,21 @@ SystemIcon::SystemIcon(GG::X x, GG::Y y, GG::X w, int system_id) :
     m_tiny_selection_indicator->Resize(GG::Pt(texture_width, texture_height));
 
     // mouseover indicator graphic
-    const boost::shared_ptr<GG::Texture> mouseover_texture = ClientUI::GetTexture(ClientUI::ArtDir() / "misc" / "system_mouseover.png");
+    const std::shared_ptr<GG::Texture> mouseover_texture = ClientUI::GetTexture(ClientUI::ArtDir() / "misc" / "system_mouseover.png");
     texture_width = mouseover_texture->DefaultWidth();
     texture_height = mouseover_texture->DefaultHeight();
     m_mouseover_indicator = new GG::StaticGraphic(mouseover_texture, GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE);
     m_mouseover_indicator->Resize(GG::Pt(texture_width, texture_height));
 
     // unexplored mouseover indicator graphic
-    boost::shared_ptr<GG::Texture> unexplored_mouseover_texture = ClientUI::GetTexture(ClientUI::ArtDir() / "misc" / "system_mouseover_unexplored.png");
+    std::shared_ptr<GG::Texture> unexplored_mouseover_texture = ClientUI::GetTexture(ClientUI::ArtDir() / "misc" / "system_mouseover_unexplored.png");
     texture_width = unexplored_mouseover_texture->DefaultWidth();
     texture_height = unexplored_mouseover_texture->DefaultHeight();
     m_mouseover_unexplored_indicator = new GG::StaticGraphic(unexplored_mouseover_texture, GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE);
     m_mouseover_unexplored_indicator->Resize(GG::Pt(texture_width, texture_height));
 
     // tiny mouseover indicator graphic
-    boost::shared_ptr<GG::Texture> tiny_mouseover_texture = ClientUI::GetTexture(ClientUI::ArtDir() / "misc" / "system_mouseover_tiny.png");
+    std::shared_ptr<GG::Texture> tiny_mouseover_texture = ClientUI::GetTexture(ClientUI::ArtDir() / "misc" / "system_mouseover_tiny.png");
     texture_width = tiny_mouseover_texture->DefaultWidth();
     texture_height = tiny_mouseover_texture->DefaultHeight();
     m_tiny_mouseover_indicator = new GG::StaticGraphic(tiny_mouseover_texture, GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE);
@@ -318,13 +323,13 @@ SystemIcon::~SystemIcon() {
 int SystemIcon::SystemID() const
 { return m_system_id; }
 
-const boost::shared_ptr<GG::Texture>& SystemIcon::DiscTexture() const
+const std::shared_ptr<GG::Texture>& SystemIcon::DiscTexture() const
 { return m_disc_texture; }
 
-const boost::shared_ptr<GG::Texture>& SystemIcon::HaloTexture() const
+const std::shared_ptr<GG::Texture>& SystemIcon::HaloTexture() const
 { return m_halo_texture; }
 
-const boost::shared_ptr<GG::Texture>& SystemIcon::TinyTexture() const
+const std::shared_ptr<GG::Texture>& SystemIcon::TinyTexture() const
 { return m_tiny_texture; }
 
 GG::Pt SystemIcon::NthFleetButtonUpperLeft(unsigned int button_number, bool moving) const {
@@ -341,7 +346,7 @@ GG::Pt SystemIcon::NthFleetButtonUpperLeft(unsigned int button_number, bool movi
     FleetButton::SizeType fb_size_type = ClientUI::GetClientUI()->GetMapWnd()->FleetButtonSizeType();
     GG::Pt button_size = GG::Pt();
     double FB_RADIUS = 0.0;
-    const boost::shared_ptr<GG::Texture>& icon = FleetSizeIcon(1u, fb_size_type);
+    const std::shared_ptr<GG::Texture>& icon = FleetSizeIcon(1u, fb_size_type);
     if (icon) {
         button_size = GG::Pt(icon->DefaultWidth(), icon->DefaultHeight());
         FB_RADIUS = static_cast<double>(Value(button_size.x) + Value(button_size.y))/4.0;   // average half of two side lengths to get radius
@@ -633,7 +638,7 @@ void SystemIcon::Refresh() {
     std::string name;
     m_system_connection.disconnect();
 
-    TemporaryPtr<const System> system = GetSystem(m_system_id);
+    std::shared_ptr<const System> system = GetSystem(m_system_id);
     if (system) {
         name = system->Name();
         m_system_connection = GG::Connect(system->StateChangedSignal,   &SystemIcon::Refresh,   this,   boost::signals2::at_front);
@@ -643,7 +648,8 @@ void SystemIcon::Refresh() {
 
 
     // remove existing system name control
-    delete m_colored_name;  m_colored_name = 0;
+    delete m_colored_name;
+    m_colored_name = nullptr;
 
     // create new system name control
     if (!name.empty()) {
@@ -685,8 +691,8 @@ void SystemIcon::HideName() {
 
 void SystemIcon::PositionSystemName() {
     if (m_colored_name) {
-        GG::X name_left = ( Width()  - m_colored_name->Width()           )/2;
-        GG::Y name_top =  ( Height() + GG::Y(EnclosingCircleDiameter()*2))/2;
+        GG::X name_left = ( Width()  - m_colored_name->Width()                                      )/2;
+        GG::Y name_top =  ( Height() + GG::Y(EnclosingCircleDiameter()*2) - m_colored_name->Height())/2;
         m_colored_name->MoveTo(GG::Pt(name_left, name_top));
     }
 }
