@@ -4,6 +4,14 @@
 #include <GG/GGFwd.h>
 #include <GG/Control.h>
 
+/** @content_tag{CTRL_SHIPYARD} Building is to be treated as a shipyard when formatting containing objects
+ * 
+ * For objects containing a building with this tag:
+ * * Planets have an underlined name in the sidepanel
+ * * Systems have their name underlined on the map and sidepanel
+ */
+const std::string TAG_SHIPYARD = "CTRL_SHIPYARD";
+
 class FleetButton;
 class RotatingGraphic;
 
@@ -15,7 +23,10 @@ class RotatingGraphic;
 class OwnerColoredSystemName : public GG::Control {
 public:
     OwnerColoredSystemName(int system_id, int font_size, bool blank_unexplored_and_none);
-    virtual void Render();
+
+    void Render() override;
+
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
 };
 
 /** A control that allows interaction with a star system.  This class allows
@@ -25,38 +36,55 @@ public:
 class SystemIcon : public GG::Control {
 public:
     //! \name Structors //!@{
-    SystemIcon(GG::X x, GG::Y y, GG::X w, int system_id);       //!< construct from a universe ID at specified size and position
-    ~SystemIcon();                                              //!< dtor
+    /** Construct from a universe ID at specified size and position. */
+    SystemIcon(GG::X x, GG::Y y, GG::X w, int system_id);
+
+    ~SystemIcon();
     //!@}
 
     //! \name Accessors //!@{
+    /** Checks to see if point lies inside in-system fleet buttons before
+        checking parent InWindow method. */
+    bool InWindow(const GG::Pt& pt) const override;
+
     int             SystemID() const;                           //!< returns ID of system this icon represents
 
-    const boost::shared_ptr<GG::Texture>& DiscTexture() const;      //!< returns the solid star disc texture
-    const boost::shared_ptr<GG::Texture>& HaloTexture() const;      //!< returns the transparent star halo texture
-    const boost::shared_ptr<GG::Texture>& TinyTexture() const;      //!< returns the alternate texture shown when icon very small
+    /** Returns the solid star disc texture. */
+    const std::shared_ptr<GG::Texture>& DiscTexture() const;
+
+    /** Returns the transparent star halo texture. */
+    const std::shared_ptr<GG::Texture>& HaloTexture() const;
+
+    /** Returns the alternate texture shown when icon very small. */
+    const std::shared_ptr<GG::Texture>& TinyTexture() const;
 
-    virtual bool    InWindow(const GG::Pt& pt) const;       //!< Overrides GG::Wnd::InWindow. Checks to see if point lies inside in-system fleet buttons before checking main InWindow method.
     GG::Pt          NthFleetButtonUpperLeft(unsigned int button_number, bool moving) const; //!< returns upper left point of moving or stationary fleetbutton number \a button_number
     int             EnclosingCircleDiameter() const;        //!< returns diameter of circle enclosing icon around which other icons can be placed and within which the mouse is over the icon
     //!@}
 
     //! \name Mutators //!@{
-    virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr);
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
+
+    void Render() override;
+
+    void LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void LDoubleClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void RDoubleClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void MouseEnter(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override;
+
+    void MouseLeave() override;
+
+    void MouseWheel(const GG::Pt& pt, int move, GG::Flags<GG::ModKey> mod_keys) override;
 
-    virtual void    Render();
     void            RenderDisc();
     void            RenderHalo(double scale_factor);
     void            RenderOverlay(double zoom_factor);
 
-    virtual void    LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    LDoubleClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    RDoubleClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    MouseEnter(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys);
-    virtual void    MouseLeave();
-    virtual void    MouseWheel(const GG::Pt& pt, int move, GG::Flags<GG::ModKey> mod_keys);
-
     void            SetSelected(bool selected = true);   //!< shows/hides the system selection indicator over this system
 
     void            Refresh();                      //!< Resets system name text and calls RefreshFleetButtons().  Should be called after an icon is attached to the map
@@ -76,10 +104,19 @@ private:
     void            PositionSystemName();
 
     int                             m_system_id;                //!< the System associated with this SystemIcon
-    boost::shared_ptr<GG::Texture>  m_disc_texture;             //!< solid star disc texture
-    boost::shared_ptr<GG::Texture>  m_halo_texture;             //!< transparent star halo texture
-    boost::shared_ptr<GG::Texture>  m_tiny_texture;             //!< alternate texture shown when icon very small
-    boost::shared_ptr<GG::Texture>  m_overlay_texture;          //!< extra texture drawn over / behind system
+
+    /** Solid star disc texture. */
+    std::shared_ptr<GG::Texture> m_disc_texture;
+
+    /** Transparent star halo texture. */
+    std::shared_ptr<GG::Texture> m_halo_texture;
+
+    /** Alternate texture shown when icon very small. */
+    std::shared_ptr<GG::Texture> m_tiny_texture;
+
+    /** Extra texture drawn over / behind system. */
+    std::shared_ptr<GG::Texture> m_overlay_texture;
+
     double                          m_overlay_size;             //!< size of extra texture in universe units
     GG::StaticGraphic*              m_tiny_graphic;             //!< non-scaled texture shown when zoomed far enough out
     RotatingGraphic*                m_selection_indicator;      //!< shown to indicate system is selected in sidepanel
