@@ -4,6 +4,7 @@
 #include "../util/Logger.h"
 #include "../universe/ResourceCenter.h"
 #include "../universe/System.h"
+#include "../universe/Enums.h"
 #include "../Empire/Empire.h"
 #include "../client/human/HumanClientApp.h"
 #include "CUIControls.h"
@@ -12,7 +13,7 @@ namespace {
     /** Returns how much of specified \a resource_type is being consumed by the
       * empire with id \a empire_id at the location of the specified
       * object \a obj. */
-    double ObjectResourceConsumption(TemporaryPtr<const UniverseObject> obj, ResourceType resource_type, int empire_id = ALL_EMPIRES) {
+    double ObjectResourceConsumption(std::shared_ptr<const UniverseObject> obj, ResourceType resource_type, int empire_id = ALL_EMPIRES) {
         if (!obj) {
             ErrorLogger() << "ObjectResourceConsumption passed a null object";
             return 0.0;
@@ -23,7 +24,7 @@ namespace {
         }
 
 
-        const Empire* empire = 0;
+        const Empire* empire = nullptr;
 
         if (empire_id != ALL_EMPIRES) {
             empire = GetEmpire(empire_id);
@@ -40,28 +41,26 @@ namespace {
         }
 
 
-        //TemporaryPtr<const PopCenter> pc = 0;
+        //std::shared_ptr<const PopCenter> pc;
         double prod_queue_allocation_sum = 0.0;
-        TemporaryPtr<const Building> building;
+        std::shared_ptr<const Building> building;
 
         switch (resource_type) {
         case RE_INDUSTRY:
             // PP (equal to mineral and industry) cost of objects on production queue at this object's location
             if (empire) {
                 // add allocated PP for all production items at this location for this empire
-                const ProductionQueue& queue = empire->GetProductionQueue();
-                for (ProductionQueue::const_iterator queue_it = queue.begin(); queue_it != queue.end(); ++queue_it)
-                    if (queue_it->location == obj->ID())
-                        prod_queue_allocation_sum += queue_it->allocated_pp;
+                for (const ProductionQueue::Element& elem : empire->GetProductionQueue())
+                    if (elem.location == obj->ID())
+                        prod_queue_allocation_sum += elem.allocated_pp;
 
             } else {
                 // add allocated PP for all production items at this location for all empires
-                for (EmpireManager::const_iterator it = Empires().begin(); it != Empires().end(); ++it) {
-                    empire = it->second;
-                    const ProductionQueue& queue = empire->GetProductionQueue();
-                    for (ProductionQueue::const_iterator queue_it = queue.begin(); queue_it != queue.end(); ++queue_it)
-                        if (queue_it->location == obj->ID())
-                            prod_queue_allocation_sum += queue_it->allocated_pp;
+                for (std::map<int, Empire*>::value_type& entry : Empires()) {
+                    empire = entry.second;
+                    for (const ProductionQueue::Element& elem : empire->GetProductionQueue())
+                        if (elem.location == obj->ID())
+                            prod_queue_allocation_sum += elem.allocated_pp;
                 }
             }
             return prod_queue_allocation_sum;
@@ -93,9 +92,9 @@ SystemResourceSummaryBrowseWnd::SystemResourceSummaryBrowseWnd(ResourceType reso
     m_resource_type(resource_type),
     m_system_id(system_id),
     m_empire_id(empire_id),
-    m_production_label(0),
-    m_allocation_label(0),
-    m_import_export_label(0),
+    m_production_label(nullptr),
+    m_allocation_label(nullptr),
+    m_import_export_label(nullptr),
     row_height(1),
     production_label_top(0),
     allocation_label_top(0),
@@ -172,13 +171,13 @@ void SystemResourceSummaryBrowseWnd::Initialize() {
 void SystemResourceSummaryBrowseWnd::UpdateProduction(GG::Y& top) {
     // adds pairs of labels for ResourceCenter name and production of resource starting at vertical position \a top
     // and updates \a top to the vertical position after the last entry
-    for (unsigned int i = 0; i < m_production_labels_and_amounts.size(); ++i) {
-        DeleteChild(m_production_labels_and_amounts[i].first);
-        DeleteChild(m_production_labels_and_amounts[i].second);
+    for (const std::pair<GG::Label*, GG::Label*>& label_pair : m_production_labels_and_amounts) {
+        DeleteChild(label_pair.first);
+        DeleteChild(label_pair.second);
     }
     m_production_labels_and_amounts.clear();
 
-    TemporaryPtr<const System> system = GetSystem(m_system_id);
+    std::shared_ptr<const System> system = GetSystem(m_system_id);
     if (!system || m_resource_type == INVALID_RESOURCE_TYPE)
         return;
 
@@ -187,19 +186,15 @@ void SystemResourceSummaryBrowseWnd::UpdateProduction(GG::Y& top) {
 
 
     // add label-value pair for each resource-producing object in system to indicate amount of resource produced
-    std::vector<TemporaryPtr<const UniverseObject> > objects =
+    std::vector<std::shared_ptr<const UniverseObject>> objects =
         Objects().FindObjects<const UniverseObject>(system->ContainedObjectIDs());
 
-    for (std::vector<TemporaryPtr<const UniverseObject> >::const_iterator it = objects.begin();
-         it != objects.end(); ++it)
-    {
-        TemporaryPtr<const UniverseObject> obj = *it;
-
+    for (std::shared_ptr<const UniverseObject> obj : objects) {
         // display information only for the requested player
         if (m_empire_id != ALL_EMPIRES && !obj->OwnedBy(m_empire_id))
             continue;   // if m_empire_id == -1, display resource production for all empires.  otherwise, skip this resource production if it's not owned by the requested player
 
-        TemporaryPtr<const ResourceCenter> rc = boost::dynamic_pointer_cast<const ResourceCenter>(obj);
+        std::shared_ptr<const ResourceCenter> rc = std::dynamic_pointer_cast<const ResourceCenter>(obj);
         if (!rc) continue;
 
         std::string name = obj->Name();
@@ -266,13 +261,13 @@ void SystemResourceSummaryBrowseWnd::UpdateProduction(GG::Y& top) {
 void SystemResourceSummaryBrowseWnd::UpdateAllocation(GG::Y& top) {
     // adds pairs of labels for allocation of resources in system, starting at vertical position \a top and
     // updates \a top to be the vertical position after the last entry
-    for (unsigned int i = 0; i < m_allocation_labels_and_amounts.size(); ++i) {
-        DeleteChild(m_allocation_labels_and_amounts[i].first);
-        DeleteChild(m_allocation_labels_and_amounts[i].second);
+    for (const std::pair<GG::Label*, GG::Label*>& label_pair : m_allocation_labels_and_amounts) {
+        DeleteChild(label_pair.first);
+        DeleteChild(label_pair.second);
     }
     m_allocation_labels_and_amounts.clear();
 
-    TemporaryPtr<const System> system = GetSystem(m_system_id);
+    std::shared_ptr<const System> system = GetSystem(m_system_id);
     if (!system || m_resource_type == INVALID_RESOURCE_TYPE)
         return;
 
@@ -281,14 +276,7 @@ void SystemResourceSummaryBrowseWnd::UpdateAllocation(GG::Y& top) {
 
 
     // add label-value pair for each resource-consuming object in system to indicate amount of resource consumed
-    std::vector<TemporaryPtr<const UniverseObject> > objects =
-        Objects().FindObjects<const UniverseObject>(system->ContainedObjectIDs());
-
-    for (std::vector<TemporaryPtr<const UniverseObject> >::const_iterator it = objects.begin();
-         it != objects.end(); ++it)
-    {
-        TemporaryPtr<const UniverseObject> obj = *it;
-
+    for (std::shared_ptr<const UniverseObject> obj : Objects().FindObjects<const UniverseObject>(system->ContainedObjectIDs())) {
         // display information only for the requested player
         if (m_empire_id != ALL_EMPIRES && !obj->OwnedBy(m_empire_id))
             continue;   // if m_empire_id == ALL_EMPIRES, display resource production for all empires.  otherwise, skip this resource production if it's not owned by the requested player
@@ -378,7 +366,7 @@ void SystemResourceSummaryBrowseWnd::UpdateAllocation(GG::Y& top) {
 void SystemResourceSummaryBrowseWnd::UpdateImportExport(GG::Y& top) {
     m_import_export_label->SetText(UserString("IMPORT_EXPORT_TOOLTIP"));
 
-    const Empire* empire = 0;
+    const Empire* empire = nullptr;
 
     // check for early exit cases...
     bool abort = false;
@@ -452,21 +440,21 @@ void SystemResourceSummaryBrowseWnd::Clear() {
     DeleteChild(m_allocation_label);
     DeleteChild(m_import_export_label);
 
-    for (std::vector<std::pair<GG::Label*, GG::Label*> >::iterator it = m_production_labels_and_amounts.begin(); it != m_production_labels_and_amounts.end(); ++it) {
-        DeleteChild(it->first);
-        DeleteChild(it->second);
+    for (const std::pair<GG::Label*, GG::Label*>& label_pair : m_production_labels_and_amounts) {
+        DeleteChild(label_pair.first);
+        DeleteChild(label_pair.second);
     }
     m_production_labels_and_amounts.clear();
 
-    for (std::vector<std::pair<GG::Label*, GG::Label*> >::iterator it = m_allocation_labels_and_amounts.begin(); it != m_allocation_labels_and_amounts.end(); ++it) {
-        DeleteChild(it->first);
-        DeleteChild(it->second);
+    for (const std::pair<GG::Label*, GG::Label*>& label_pair : m_allocation_labels_and_amounts) {
+        DeleteChild(label_pair.first);
+        DeleteChild(label_pair.second);
     }
     m_allocation_labels_and_amounts.clear();
 
-    for (std::vector<std::pair<GG::Label*, GG::Label*> >::iterator it = m_import_export_labels_and_amounts.begin(); it != m_import_export_labels_and_amounts.end(); ++it) {
-        DeleteChild(it->first);
-        DeleteChild(it->second);
+    for (const std::pair<GG::Label*, GG::Label*>& label_pair : m_import_export_labels_and_amounts) {
+        DeleteChild(label_pair.first);
+        DeleteChild(label_pair.second);
     }
     m_import_export_labels_and_amounts.clear();
 }
