@@ -2,13 +2,13 @@
 #define _TechTreeWnd_h_
 
 #include <GG/Wnd.h>
-#include "../universe/Enums.h"
+#include "../universe/EnumsFwd.h"
 
 class Tech;
 class EncyclopediaDetailPanel;
 
 /** Returns a browse wnd for tech panels */
-boost::shared_ptr<GG::BrowseInfoWnd> TechPanelRowBrowseWnd(const std::string& tech_name, int empire_id);
+std::shared_ptr<GG::BrowseInfoWnd> TechPanelRowBrowseWnd(const std::string& tech_name, int empire_id);
 
 /** Contains the tech graph layout, some controls to control what is visible in
   * the tech layout, the tech navigator, and the tech detail window. */
@@ -20,7 +20,12 @@ public:
     typedef boost::signals2::signal<void (const std::vector<std::string>&, int)>    QueueAddTechsSignalType;
 
     /** \name Structors */ //@{
-    TechTreeWnd(GG::X w, GG::Y h);
+    /** TechTreeWnd contructor is usually called before client has
+        access to techs.  Attempting to show the tech tree takes a long
+        time and generates errors.  If \p initially_hidden is true then the
+        tech categories are not parsed until the first time Show() is
+        called, speeding up the constructor and preventing spurious errors.*/
+    TechTreeWnd(GG::X w, GG::Y h, bool initially_hidden = true);
     ~TechTreeWnd();
     //@}
 
@@ -32,7 +37,10 @@ public:
     //@}
 
     //! \name Mutators //@{
-    virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr);
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
+
+    void Show(bool children = true) override;
+
     void            Update();
     void            Clear();
     void            Reset();
@@ -44,8 +52,7 @@ public:
     void            HideAllCategories();
     void            ToggleAllCategories();
 
-    void            ShowStatus(const TechStatus status);
-    void            HideStatus(const TechStatus status);
+    void            SetTechStatus(const TechStatus status, const bool state);
 
     void            ToggleViewType(bool show_list_view);
     void            ShowTreeView();
@@ -60,7 +67,6 @@ public:
     void            TogglePedia();
     //@}
 
-    mutable TechSignalType          TechBrowsedSignal;
     mutable TechSignalType          TechSelectedSignal;
     mutable QueueAddTechsSignalType AddTechsToQueueSignal;
 
@@ -69,11 +75,8 @@ private:
     class LayoutPanel;
     class TechListBox;
 
-    void    TechBrowsedSlot(const std::string& tech_name);
     void    TechLeftClickedSlot(const std::string& tech_name,
                                 const GG::Flags<GG::ModKey>& modkeys);
-    void    TechRightClickedSlot(const std::string& tech_name,
-                                 const GG::Flags<GG::ModKey>& modkeys);
     void    TechDoubleClickedSlot(const std::string& tech_name,
                                   const GG::Flags<GG::ModKey>& modkeys);
     void    TechPediaDisplaySlot(const std::string& tech_name);
@@ -84,6 +87,11 @@ private:
     EncyclopediaDetailPanel*    m_enc_detail_panel;
     LayoutPanel*                m_layout_panel;
     TechListBox*                m_tech_list;
+
+    /// If m_init_flag is true tech categories are not parsed until the
+    /// first time Show() is called.  TechTreeWnd is constructed before the
+    /// tech categories are available to be parsed.
+    bool                        m_init_flag;
 };
 
 #endif // _TechTreeWnd_h_
