@@ -7,7 +7,9 @@
 #include "../../util/Directories.h"
 #include "../../util/Serialize.h"
 #include "../../util/i18n.h"
+#include "../util/AppInterface.h"
 #include "../../network/Message.h"
+#include "../../network/ClientNetworking.h"
 #include "../util/Random.h"
 
 #include "../../universe/System.h"
@@ -21,12 +23,54 @@
 #include <boost/lexical_cast.hpp>
 #include <boost/filesystem/fstream.hpp>
 
+#include <thread>
+#include <chrono>
+
 class CombatLogManager;
 CombatLogManager&   GetCombatLogManager();
 
+
+namespace {
+
+    /** AddTraitBypassOption creates a set of options for debugging of
+        the form:
+        AI.config.trait.\<trait name\>.force  -- If true use the following options to bypass the trait
+        AI.config.trait.\<trait name\>.all    -- If present use this value for all of the AIs not individually set
+        AI.config.trait.\<trait name\>.AI_1   -- Use for AI_1
+        ...
+        AI.config.trait.\<trait name\>.AI_40  -- Use for AI_40
+     */
+    template <typename T>
+    void AddTraitBypassOption(OptionsDB& db, std::string const & root, std::string ROOT,
+                                 T def, ValidatorBase const & validator) {
+        std::string option_root = "AI.config.trait." + root +".";
+        std::string user_string_root = "OPTIONS_DB_AI_CONFIG_TRAIT_"+ROOT;
+        db.Add<bool>(option_root + "force", UserStringNop(user_string_root + "_FORCE"), false, Validator<bool>());
+        db.Add<T>(option_root + "all", UserStringNop(user_string_root + "_FORCE_VALUE"), def, validator);
+
+        for (int ii = 1; ii <= IApp::MAX_AI_PLAYERS(); ++ii) {
+            std::stringstream ss;
+            ss << option_root << "AI_" << boost::lexical_cast<int>(ii);
+            db.Add<T>(ss.str(), UserStringNop(user_string_root + "_FORCE_VALUE"), def, validator);
+        }
+    }
+
+    void AddOptions(OptionsDB& db) {
+        // Create options to allow bypassing the traits of the AI
+        // character and forcing them all to one value for testing
+        // purposes.
+
+        const int max_aggression = 5;
+        const int no_value = -1;
+        AddTraitBypassOption<int>(db, "aggression", "AGGRESSION", no_value, RangedValidator<int>(0, max_aggression));
+        AddTraitBypassOption<int>(db, "empire-id", "EMPIREID", no_value, RangedValidator<int>(0, IApp::MAX_AI_PLAYERS()));
+    }
+    bool temp_bool = RegisterOptions(&AddOptions);
+
+}
+
 // static member(s)
 AIClientApp::AIClientApp(const std::vector<std::string>& args) :
-    m_AI(0),
     m_player_name(""),
     m_max_aggression(0)
 {
@@ -48,10 +92,9 @@ AIClientApp::AIClientApp(const std::vector<std::string>& args) :
 }
 
 AIClientApp::~AIClientApp() {
-    if (Networking().Connected())
-        Networking().DisconnectFromServer();
+    Networking().DisconnectFromServer();
 
-    DebugLogger() << "Shutting down " + PlayerName() + " ai client.";
+    DebugLogger() << "AIClientApp exited cleanly for ai client " << PlayerName();
 }
 
 void AIClientApp::operator()()
@@ -59,7 +102,9 @@ void AIClientApp::operator()()
 
 void AIClientApp::Exit(int code) {
     DebugLogger() << "Initiating Exit (code " << code << " - " << (code ? "error" : "normal") << " termination)";
-    exit(code);
+    if (code)
+        exit(code);
+    throw NormalExitException();
 }
 
 int AIClientApp::EffectsProcessingThreads() const
@@ -84,24 +129,23 @@ void AIClientApp::Run() {
         while (1) {
             try {
 
-                if (!Networking().Connected())
+                if (!Networking().IsRxConnected())
                     break;
-                if (Networking().MessageAvailable()) {
-                    Message msg;
-                    Networking().GetMessage(msg);
-                    HandleMessage(msg);
-                } else {
-                    boost::this_thread::sleep_for(boost::chrono::milliseconds(250));
-                }
+                if (const auto msg = Networking().GetMessage())
+                    HandleMessage(*msg);
+                else
+                    std::this_thread::sleep_for(std::chrono::milliseconds(10));
 
             } catch (boost::python::error_already_set) {
                 /* If the python interpreter is still running then keep
                    going, otherwise exit.*/
                 m_AI->HandleErrorAlreadySet();
-                if (m_AI->IsPythonRunning())
+                if (!m_AI->IsPythonRunning())
                     throw;
             }
         }
+    } catch (const NormalExitException&) {
+        // intentionally empty.
     } catch (boost::python::error_already_set) {
         HandlePythonAICrash();
     }
@@ -110,25 +154,8 @@ void AIClientApp::Run() {
 }
 
 void AIClientApp::ConnectToServer() {
-    // connect
-    const int MAX_TRIES = 10;
-    int tries = 0;
-    volatile bool connected = false;
-    while (tries < MAX_TRIES) {
-        DebugLogger() << "Attempting to contact server";
-        connected = Networking().ConnectToLocalHostServer();
-        if (!connected) {
-            std::cerr << "FreeOrion AI client server contact attempt " << tries + 1 << " failed." << std::endl;
-            ErrorLogger() << "Server contact attempt " << tries + 1 << " failed";
-        } else {
-            break;
-        }
-        ++tries;
-    }
-    if (!connected) {
-        ErrorLogger() << "AIClientApp::Run : Failed to connect to localhost server after " << MAX_TRIES << " tries.  Exiting.";
+    if (!Networking().ConnectToLocalHostServer())
         Exit(1);
-    }
 }
 
 void AIClientApp::StartPythonAI() {
@@ -148,7 +175,6 @@ void AIClientApp::HandlePythonAICrash() {
     ErrorLogger() << err_msg.str() << " id = " << PlayerID();
     Networking().SendMessage(
         ErrorMessage(PlayerID(), str(FlexibleFormat(UserString("ERROR_PYTHON_AI_CRASHED")) % PlayerName()) , true));
-    boost::this_thread::sleep_for(boost::chrono::milliseconds(500));
 }
 
 
@@ -172,7 +198,7 @@ void AIClientApp::HandleMessage(const Message& msg) {
                 ErrorLogger() << "AIClientApp::HandleMessage for HOST_ID : Couldn't parese message text: " << text;
             }
         }
-        m_networking.SetHostPlayerID(host_id);
+        m_networking->SetHostPlayerID(host_id);
         break;
     }
 
@@ -180,7 +206,7 @@ void AIClientApp::HandleMessage(const Message& msg) {
         if (msg.SendingPlayer() == Networking::INVALID_PLAYER_ID) {
             if (PlayerID() == Networking::INVALID_PLAYER_ID) {
                 DebugLogger() << "AIClientApp::HandleMessage : Received JOIN_GAME acknowledgement";
-                m_networking.SetPlayerID(msg.ReceivingPlayer());
+                m_networking->SetPlayerID(msg.ReceivingPlayer());
             } else {
                 ErrorLogger() << "AIClientApp::HandleMessage : Received erroneous JOIN_GAME acknowledgement when already in a game";
             }
@@ -199,12 +225,12 @@ void AIClientApp::HandleMessage(const Message& msg) {
             std::string save_state_string;
             m_player_status.clear();
 
-            ExtractMessageData(msg,                     single_player_game,     m_empire_id,
-                               m_current_turn,          m_empires,              m_universe,
-                               GetSpeciesManager(),     GetCombatLogManager(),  GetSupplyManager(),
-                               m_player_info,           m_orders,               loaded_game_data,
-                               ui_data_available,       ui_data,                state_string_available,
-                               save_state_string,       m_galaxy_setup_data);
+            ExtractGameStartMessageData(msg,                     single_player_game,     m_empire_id,
+                                        m_current_turn,          m_empires,              m_universe,
+                                        GetSpeciesManager(),     GetCombatLogManager(),  GetSupplyManager(),
+                                        m_player_info,           m_orders,               loaded_game_data,
+                                        ui_data_available,       ui_data,                state_string_available,
+                                        save_state_string,       m_galaxy_setup_data);
 
             DebugLogger() << "Extracted GameStart message for turn: " << m_current_turn << " with empire: " << m_empire_id;
 
@@ -289,9 +315,9 @@ void AIClientApp::HandleMessage(const Message& msg) {
     case Message::TURN_UPDATE: {
         if (msg.SendingPlayer() == Networking::INVALID_PLAYER_ID) {
             //DebugLogger() << "AIClientApp::HandleMessage : extracting turn update message data";
-            ExtractMessageData(msg,                     m_empire_id,        m_current_turn,
-                               m_empires,               m_universe,         GetSpeciesManager(),
-                               GetCombatLogManager(),   GetSupplyManager(), m_player_info);
+            ExtractTurnUpdateMessageData(msg,                     m_empire_id,        m_current_turn,
+                                         m_empires,               m_universe,         GetSpeciesManager(),
+                                         GetCombatLogManager(),   GetSupplyManager(), m_player_info);
             //DebugLogger() << "AIClientApp::HandleMessage : generating orders";
             GetUniverse().InitializeSystemGraph(m_empire_id);
             m_AI->GenerateOrders();
@@ -302,7 +328,7 @@ void AIClientApp::HandleMessage(const Message& msg) {
 
     case Message::TURN_PARTIAL_UPDATE:
         if (msg.SendingPlayer() == Networking::INVALID_PLAYER_ID)
-            ExtractMessageData(msg, m_empire_id, m_universe);
+            ExtractTurnPartialUpdateMessageData(msg, m_empire_id, m_universe);
         break;
 
     case Message::TURN_PROGRESS:
@@ -311,6 +337,8 @@ void AIClientApp::HandleMessage(const Message& msg) {
 
     case Message::END_GAME: {
         DebugLogger() << "Message::END_GAME : Exiting";
+        DebugLogger() << "Acknowledge server shutdown message.";
+        Networking().SendMessage(AIEndGameAcknowledgeMessage(Networking().PlayerID()));
         Exit(0);
         break;
     }
@@ -321,14 +349,14 @@ void AIClientApp::HandleMessage(const Message& msg) {
 
     case Message::DIPLOMACY: {
         DiplomaticMessage diplo_message;
-        ExtractMessageData(msg, diplo_message);
+        ExtractDiplomacyMessageData(msg, diplo_message);
         m_AI->HandleDiplomaticMessage(diplo_message);
         break;
     }
 
     case Message::DIPLOMATIC_STATUS: {
         DiplomaticStatusUpdateInfo diplo_update;
-        ExtractMessageData(msg, diplo_update);
+        ExtractDiplomaticStatusMessageData(msg, diplo_update);
         m_AI->HandleDiplomaticStatusUpdate(diplo_update);
         break;
     }
