@@ -2,8 +2,10 @@
 #define _AIClientApp_h_
 
 #include "../ClientApp.h"
+
+#include <memory>
 #include <vector>
-#include <boost/scoped_ptr.hpp>
+
 
 class AIBase;
 class PythonAI;
@@ -18,14 +20,15 @@ public:
 
     /** \name Mutators */ //@{
     void                operator()();   ///< external interface to Run()
-    void                Wait(int ms);   ///< put the main thread to sleep for \a ms milliseconds
     void                Exit(int code); ///< does basic clean-up, then calls exit(); callable from anywhere in user code via GetApp()
     void                SetPlayerName(const std::string& player_name) { m_player_name = player_name; }
     //@}
 
     /** \name Accessors */ //@{
-    const std::string&  PlayerName() const { return m_player_name; }
-    virtual int         EffectsProcessingThreads() const;
+    int EffectsProcessingThreads() const override;
+
+    const std::string& PlayerName() const
+    { return m_player_name; }
     //@}
 
     static AIClientApp* GetApp();       ///< returns a AIClientApp pointer to the singleton instance of the app
@@ -39,7 +42,9 @@ private:
     void                HandleMessage(const Message& msg);
 
 
-    boost::scoped_ptr<PythonAI> m_AI;     ///< implementation of AI logic
+    /** Implementation of AI logic. */
+    std::unique_ptr<PythonAI> m_AI;
+
     std::string         m_player_name;
     int                 m_max_aggression;
 };
