@@ -4,6 +4,7 @@
 #include "../../util/OptionsDB.h"
 #include "../../util/Directories.h"
 #include "../../util/Logger.h"
+#include "../../util/XMLDoc.h"
 
 #include <GG/utf8/checked.h>
 
@@ -92,7 +93,7 @@ int wmain(int argc, wchar_t* argv[], wchar_t* envp[]) {
         return 1;
     }
 #endif
-
+    DebugLogger() << "AI client main exited cleanly.";
     return 0;
 }
 
