@@ -3,12 +3,14 @@
 
 #include "../Empire/EmpireManager.h"
 #include "../Empire/Supply.h"
-#include "../network/ClientNetworking.h"
+#include "../network/Message.h"
 #include "../universe/Universe.h"
 #include "../util/OrderSet.h"
 #include "../util/AppInterface.h"
 #include "../util/MultiplayerCommon.h"
 
+class ClientNetworking;
+
 /** \brief Abstract base class for the application framework classes
  *
  * The static functions are designed to give both types of client (which are
@@ -39,7 +41,7 @@ public:
      *
      * @return The number representing the current game turn.
      */
-    int CurrentTurn() const;
+    int CurrentTurn() const override;
 
     /** @brief Return the player identfier of the player controlling the empire
      *      @a empire_id
@@ -62,7 +64,7 @@ public:
 
     /** @brief Return the player statuses in game
      *
-     * @return Return a map containing ::PlayerStatus instances as value and
+     * @return Return a map containing PlayerStatus instances as value and
      *      their player identifier as key.
      *
      * @{ */
@@ -76,7 +78,7 @@ public:
      *      the known universe of this client.
      *
      * @{ */
-    Universe& GetUniverse();
+    Universe& GetUniverse() override;
     const Universe& GetUniverse() const;
     /** @} */
 
@@ -86,7 +88,7 @@ public:
      *
      * @{ */
     GalaxySetupData& GetGalaxySetupData();
-    const GalaxySetupData& GetGalaxySetupData() const;
+    const GalaxySetupData& GetGalaxySetupData() const override;
     /** @} */
 
     /** @brief Return the OrderSet of this client
@@ -120,7 +122,7 @@ public:
      * @return the networking client type of the empire represented by @a
      *      empire_id parameter.
      */
-    Networking::ClientType GetEmpireClientType(int empire_id) const;
+    Networking::ClientType GetEmpireClientType(int empire_id) const override;
 
     /** @brief Return the Networking::ClientType of the player @a player_id
      *
@@ -129,7 +131,7 @@ public:
      * @return the networking client type of the player represented by @a
      *      player_id parameter.
      */
-    Networking::ClientType GetPlayerClientType(int player_id) const;
+    Networking::ClientType GetPlayerClientType(int player_id) const override;
 
     /** @brief Return the ::UniverseObject associated with @a object_id
      *
@@ -137,9 +139,9 @@ public:
      *
      * @return A pointer to the ::UniverseObject associated with @a object_id.
      *      When there is no matching object or the client does not know of the
-     *      object existence the pointer may be NULL.
+     *      object existence the pointer may be nullptr.
      */
-    TemporaryPtr<UniverseObject>GetUniverseObject(int object_id);
+    std::shared_ptr<UniverseObject> GetUniverseObject(int object_id) override;
 
     /** @brief Return the for this client visible name of @a object
      *
@@ -148,7 +150,7 @@ public:
      * @return The name of the @a object.  Depdending on Visibility it may not
      *      match with the actual object name.
      */
-    std::string GetVisibleObjectName(TemporaryPtr<const UniverseObject> object);
+    std::string GetVisibleObjectName(std::shared_ptr<const UniverseObject> object) override;
 
     /** @brief Send the OrderSet to the server and start a new turn */
     virtual void StartTurn();
@@ -158,7 +160,7 @@ public:
      * @return The EmpireManager instance in charge of maintaining the Empire
      *      object instances.
      */
-    EmpireManager& Empires();
+    EmpireManager& Empires() override;
 
     /** @brief Return the Empire identified by @a empire_id
      *
@@ -166,11 +168,11 @@ public:
      *
      * @return A pointer to the Empire instance represented by @a empire_id.
      *      If there is no Empire with this @a empire_id or if the Empire is
-     *      not yet known to this client a NULL pointer is returned.
+     *      not yet known to this client a nullptr is returned.
      */
-    Empire* GetEmpire(int empire_id);
+    Empire* GetEmpire(int empire_id) override;
 
-    SupplyManager& GetSupplyManager();
+    SupplyManager& GetSupplyManager() override;
 
     /** @brief Return all Objects known to @a empire_id
      *
@@ -179,19 +181,19 @@ public:
      * @return A map containing all Objects known to the ::Empire identified by
      *      @a empire_id.  If there is no ::Empire an empty map is returned.
      */
-    ObjectMap& EmpireKnownObjects(int empire_id);
+    ObjectMap& EmpireKnownObjects(int empire_id) override;
 
     /** @brief Return the UniverseObject @a object_id if known by @a empire_id
      *
-     * @param object An UniverseObject identifier.
+     * @param object_id An UniverseObject identifier.
      * @param empire_id An empire identifier.
      *
      * @return A pointer to the UniverseObject, that is identified by
      *      @a object_id and known by the ::Empire identified by @a empire_id.
      *      If there is no such object or if that object is not known to the
-     *      empire a NULL pointer is returned.
+     *      empire a nullptr is returned.
      */
-    TemporaryPtr<UniverseObject> EmpireKnownObject(int object_id, int empire_id);
+    std::shared_ptr<UniverseObject> EmpireKnownObject(int object_id, int empire_id) override;
 
     /** @brief Set the identifier of the ::Empire controlled by this client to
      *      @a empire_id
@@ -206,6 +208,10 @@ public:
      */
     void SetCurrentTurn(int turn);
 
+    /** @brief Handle the turn update.
+     */
+    virtual void HandleTurnUpdate() {}
+
     /** @brief Set whether the current game is a single player game or not
      *
      * @param single_player Set true if the game is a single player game,
@@ -229,7 +235,7 @@ public:
      * @return A new, unique universe object identifier.  If no new idenfifier
      *      can be create INVALID_OBJECT_ID is returned.
      */
-    int GetNewObjectID();
+    int GetNewObjectID() override;
 
     /** @brief Return a new ship design identifier
      *
@@ -239,7 +245,7 @@ public:
      * @return A new, unique ship design identifier.  If no new idenfifier
      *      can be create INVALID_OBJECT_ID is returned.
      */
-    int GetNewDesignID();
+    int GetNewDesignID() override;
 
     /** @brief Return the singleton instance of this Application
      *
@@ -253,7 +259,7 @@ protected:
     EmpireManager               m_empires;
     SupplyManager               m_supply_manager;
     OrderSet                    m_orders;
-    ClientNetworking            m_networking;
+    std::shared_ptr<ClientNetworking> m_networking;
     int                         m_empire_id;
     int                         m_current_turn;
     /** Indexed by player id, contains info about all players in the game */
