@@ -15,12 +15,14 @@ struct Disconnection : boost::statechart::event<Disconnection> {};
 /** The base class for all state machine events that are based on Messages. */
 struct MessageEventBase
 {
-    MessageEventBase(Message& message); ///< Basic ctor.
+    MessageEventBase(Message& message);
+
     Message m_message;
 };
 
 // Define Boost.Preprocessor list of all Message events
 #define MESSAGE_EVENTS                         \
+    (DispatchCombatLogs)                       \
     (Error)                                    \
     (HostMPGame)                               \
     (HostSPGame)                               \
