@@ -1,5 +1,3 @@
-#include <GL/glew.h>
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -18,6 +16,7 @@
 #include "../../UI/Sound.h"
 #include "../../network/Message.h"
 #include "../../network/Networking.h"
+#include "../../network/ClientNetworking.h"
 #include "../../util/i18n.h"
 #include "../../util/MultiplayerCommon.h"
 #include "../../util/OptionsDB.h"
@@ -29,24 +28,30 @@
 #include "../../util/Version.h"
 #include "../../universe/Planet.h"
 #include "../../universe/Species.h"
+#include "../../universe/Enums.h"
 #include "../../Empire/Empire.h"
 #include "../../combat/CombatLogManager.h"
 #include "../../parse/Parse.h"
 
 #include <GG/BrowseInfoWnd.h>
+#include <GG/dialogs/ThreeButtonDlg.h>
 #include <GG/Cursor.h>
 #include <GG/utf8/checked.h>
 
+#include <boost/algorithm/string/trim.hpp>
 #include <boost/lexical_cast.hpp>
 #include <boost/filesystem/operations.hpp>
 #include <boost/filesystem/fstream.hpp>
 #include <boost/format.hpp>
 #include <boost/serialization/vector.hpp>
-#include <boost/date_time/posix_time/posix_time.hpp>
 #include <boost/algorithm/string.hpp>
 
+#include <chrono>
+#include <thread>
+
 #include <sstream>
 
+
 namespace fs = boost::filesystem;
 
 #ifdef ENABLE_CRASH_BACKTRACE
@@ -83,8 +88,6 @@ void SigHandler(int sig) {
 #endif //ENABLE_CRASH_BACKTRACE
 
 namespace {
-    const unsigned int  SERVER_CONNECT_TIMEOUT = 4500; // in ms
-
     const bool          INSTRUMENT_MESSAGE_HANDLING = false;
 
     // command-line options
@@ -188,12 +191,12 @@ HumanClientApp::HumanClientApp(int width, int height, bool calculate_fps, const
                                int x, int y, bool fullscreen, bool fake_mode_change) :
     ClientApp(),
     SDLGUI(width, height, calculate_fps, name, x, y, fullscreen, fake_mode_change),
-    m_fsm(0),
     m_single_player_game(true),
     m_game_started(false),
     m_connected(false),
     m_auto_turns(0),
-    m_have_window_focus(true)
+    m_have_window_focus(true),
+    m_save_game_in_progress(false)
 {
 #ifdef ENABLE_CRASH_BACKTRACE
     signal(SIGSEGV, SigHandler);
@@ -201,22 +204,21 @@ HumanClientApp::HumanClientApp(int width, int height, bool calculate_fps, const
 #ifdef FREEORION_MACOSX
     SDL_SetHint(SDL_HINT_MAC_CTRL_CLICK_EMULATE_RIGHT_CLICK, "1");
 #endif
-    m_fsm = new HumanClientFSM(*this);
+    m_fsm.reset(new HumanClientFSM(*this));
 
     const std::string HUMAN_CLIENT_LOG_FILENAME((GetUserDataDir() / "freeorion.log").string());
 
     InitLogger(HUMAN_CLIENT_LOG_FILENAME, "Client");
 
     try {
-        DebugLogger() << "GL Version String: " << GetGLVersionString();
+        InfoLogger() << "GL Version String: " << GetGLVersionString();
     } catch (...) {
         ErrorLogger() << "Unable to get GL Version String?";
     }
 
     LogDependencyVersions();
 
-    boost::shared_ptr<GG::StyleFactory> style(new CUIStyle());
-    SetStyleFactory(style);
+    SetStyleFactory(std::make_shared<CUIStyle>());
 
     SetMinDragTime(0);
 
@@ -234,20 +236,21 @@ HumanClientApp::HumanClientApp(int width, int height, bool calculate_fps, const
         inform_user_sound_failed = true;
     }
 
-    m_ui = boost::shared_ptr<ClientUI>(new ClientUI());
+    m_ui.reset(new ClientUI());
 
     EnableFPS();
     UpdateFPSLimit();
-    GG::Connect(GetOptionsDB().OptionChangedSignal("show-fps"), &HumanClientApp::UpdateFPSLimit, this);
+    GG::Connect(GetOptionsDB().OptionChangedSignal("show-fps"),  &HumanClientApp::UpdateFPSLimit, this);
+    GG::Connect(GetOptionsDB().OptionChangedSignal("max-fps"),   &HumanClientApp::UpdateFPSLimit, this);
 
-    boost::shared_ptr<GG::BrowseInfoWnd> default_browse_info_wnd(
+    std::shared_ptr<GG::BrowseInfoWnd> default_browse_info_wnd(
         new GG::TextBoxBrowseInfoWnd(GG::X(400), ClientUI::GetFont(),
                                      GG::Clr(0, 0, 0, 200), ClientUI::WndOuterBorderColor(), ClientUI::TextColor(),
                                      GG::FORMAT_LEFT | GG::FORMAT_WORDBREAK, 1));
     GG::Wnd::SetDefaultBrowseInfoWnd(default_browse_info_wnd);
 
-    boost::shared_ptr<GG::Texture> cursor_texture = m_ui->GetTexture(ClientUI::ArtDir() / "cursors" / "default_cursor.png");
-    SetCursor(boost::shared_ptr<GG::TextureCursor>(new GG::TextureCursor(cursor_texture, GG::Pt(GG::X(6), GG::Y(3)))));
+    std::shared_ptr<GG::Texture> cursor_texture = m_ui->GetTexture(ClientUI::ArtDir() / "cursors" / "default_cursor.png");
+    SetCursor(std::make_shared<GG::TextureCursor>(cursor_texture, GG::Pt(GG::X(6), GG::Y(3))));
     RenderCursor(true);
 
     EnableKeyPressRepeat(GetOptionsDB().Get<int>("UI.keypress-repeat-delay"),
@@ -259,36 +262,30 @@ HumanClientApp::HumanClientApp(int width, int height, bool calculate_fps, const
     GG::Connect(WindowResizedSignal,    &HumanClientApp::HandleWindowResize,    this);
     GG::Connect(FocusChangedSignal,     &HumanClientApp::HandleFocusChange,     this);
     GG::Connect(WindowMovedSignal,      &HumanClientApp::HandleWindowMove,      this);
-    /* TODO: Wire these signals if theyare needed
-    GG::Connect(WindowClosingSignal,    &HumanClientApp::HandleWindowClosing,   this);
-    GG::Connect(WindowClosedSignal,     &HumanClientApp::HandleWindowClose,     this);
-    */
+    GG::Connect(WindowClosingSignal,    &HumanClientApp::HandleAppQuitting,     this);
+    GG::Connect(AppQuittingSignal,      &HumanClientApp::HandleAppQuitting,     this);
 
     SetStringtableDependentOptionDefaults();
     SetGLVersionDependentOptionDefaults();
 
     this->SetMouseLRSwapped(GetOptionsDB().Get<bool>("UI.swap-mouse-lr"));
 
-    std::map<std::string, std::map<int, int> > named_key_maps;
+    std::map<std::string, std::map<int, int>> named_key_maps;
     parse::keymaps(named_key_maps);
     if (GetOptionsDB().Get<bool>("verbose-logging")) {
         DebugLogger() << "Keymaps:";
-        for (std::map<std::string, std::map<int, int> >::const_iterator km_it = named_key_maps.begin();
-             km_it != named_key_maps.end(); ++km_it)
-        {
-            DebugLogger() << "Keymap name = \"" << km_it->first << "\"";
-            const std::map<int, int>& key_map = km_it->second;
-            for (std::map<int, int>::const_iterator keys_it = key_map.begin(); keys_it != key_map.end(); ++keys_it)
-                DebugLogger() << "    " << char(keys_it->first) << " : " << char(keys_it->second);
+        for (std::map<std::string, std::map<int, int>>::value_type& km : named_key_maps) {
+            DebugLogger() << "Keymap name = \"" << km.first << "\"";
+            for (std::map<int, int>::value_type& keys : km.second)
+                DebugLogger() << "    " << char(keys.first) << " : " << char(keys.second);
         }
     }
-    std::map<std::string, std::map<int, int> >::const_iterator km_it = named_key_maps.find("TEST");
+    std::map<std::string, std::map<int, int>>::const_iterator km_it = named_key_maps.find("TEST");
     if (km_it != named_key_maps.end()) {
         const std::map<int, int> int_key_map = km_it->second;
         std::map<GG::Key, GG::Key> key_map;
-        for (std::map<int, int>::const_iterator key_int_it = int_key_map.begin();
-             key_int_it != int_key_map.end(); ++key_int_it)
-        { key_map[GG::Key(key_int_it->first)] = GG::Key(key_int_it->second); }
+        for (const std::map<int, int>::value_type& int_key : int_key_map)
+        { key_map[GG::Key(int_key.first)] = GG::Key(int_key.second); }
         this->SetKeyMap(key_map);
     }
 
@@ -304,6 +301,9 @@ HumanClientApp::HumanClientApp(int width, int height, bool calculate_fps, const
     if (inform_user_sound_failed)
         ClientUI::MessageBox(UserString("ERROR_SOUND_INITIALIZATION_FAILED"), false);
 
+    // Register LinkText tags with GG::Font
+    RegisterLinkTags();
+
     m_fsm->initiate();
 }
 
@@ -311,21 +311,20 @@ void HumanClientApp::ConnectKeyboardAcceleratorSignals() {
     // Add global hotkeys
     HotkeyManager *hkm = HotkeyManager::GetManager();
 
-    hkm->Connect(boost::bind(&HumanClientApp::ExitGame, this),          "exit",
-                 new NoModalWndsOpenCondition());
-    hkm->Connect(boost::bind(&HumanClientApp::QuitGame, this),          "quit",
-                 new NoModalWndsOpenCondition());
+    hkm->Connect(boost::bind(&HumanClientApp::HandleHotkeyExitApp, this),          "exit",
+                 NoModalWndsOpenCondition);
+    hkm->Connect(boost::bind(&HumanClientApp::HandleHotkeyResetGame, this),        "quit",
+                 NoModalWndsOpenCondition);
     hkm->Connect(boost::bind(&HumanClientApp::ToggleFullscreen, this), "fullscreen",
-                 new NoModalWndsOpenCondition());
+                 NoModalWndsOpenCondition);
 
     hkm->RebuildShortcuts();
 }
 
 HumanClientApp::~HumanClientApp() {
-    if (m_networking.Connected())
-        m_networking.DisconnectFromServer();
+    m_networking->DisconnectFromServer();
     m_server_process.RequestTermination();
-    delete m_fsm;
+    DebugLogger() << "HumanClientApp exited cleanly.";
 }
 
 bool HumanClientApp::SinglePlayerGame() const
@@ -337,15 +336,13 @@ bool HumanClientApp::CanSaveNow() const {
         return false;
 
     // can't save while AIs are playing their turns...
-    for (std::map<int, PlayerInfo>::const_iterator player_it = this->m_player_info.begin();
-         player_it != this->m_player_info.end(); ++player_it)
-    {
-        const PlayerInfo& info = player_it->second;
+    for (const std::map<int, PlayerInfo>::value_type& entry : m_player_info) {
+        const PlayerInfo& info = entry.second;
         if (info.client_type != Networking::CLIENT_TYPE_AI_PLAYER)
             continue;   // only care about AIs
 
         std::map<int, Message::PlayerStatus>::const_iterator
-            status_it = this->m_player_status.find(player_it->first);
+            status_it = m_player_status.find(entry.first);
 
         if (status_it == this->m_player_status.end()) {
             return false;  // missing status for AI; can't assume it's ready
@@ -417,16 +414,8 @@ void HumanClientApp::StartServer() {
 
 void HumanClientApp::FreeServer() {
     m_server_process.Free();
-    m_networking.SetPlayerID(Networking::INVALID_PLAYER_ID);
-    m_networking.SetHostPlayerID(Networking::INVALID_PLAYER_ID);
-    SetEmpireID(ALL_EMPIRES);
-}
-
-void HumanClientApp::KillServer() {
-    DebugLogger() << "HumanClientApp::KillServer()";
-    m_server_process.Kill();
-    m_networking.SetPlayerID(Networking::INVALID_PLAYER_ID);
-    m_networking.SetHostPlayerID(Networking::INVALID_PLAYER_ID);
+    m_networking->SetPlayerID(Networking::INVALID_PLAYER_ID);
+    m_networking->SetHostPlayerID(Networking::INVALID_PLAYER_ID);
     SetEmpireID(ALL_EMPIRES);
 }
 
@@ -449,18 +438,14 @@ void HumanClientApp::NewSinglePlayerGame(bool quickstart) {
         ended_with_ok = galaxy_wnd.EndedWithOk();
     }
 
-    bool failed = false;
-    unsigned int start_time = Ticks();
-    while (!m_networking.ConnectToLocalHostServer(boost::posix_time::seconds(2))) {
-        if (SERVER_CONNECT_TIMEOUT < Ticks() - start_time) {
-            ClientUI::MessageBox(UserString("ERR_CONNECT_TIMED_OUT"), true);
-            failed = true;
-            break;
-        }
+    m_connected = m_networking->ConnectToLocalHostServer();
+    if (!m_connected) {
+        ResetToIntro();
+        ClientUI::MessageBox(UserString("ERR_CONNECT_TIMED_OUT"), true);
+        return;
     }
-    m_connected = !failed;
-    if (!failed && (quickstart || ended_with_ok)) {
-        DebugLogger() << "HumanClientApp::NewSinglePlayerGame : Connected to server";
+
+    if (quickstart || ended_with_ok) {
 
         SinglePlayerSetupData setup_data;
         setup_data.m_new_game = true;
@@ -504,13 +489,15 @@ void HumanClientApp::NewSinglePlayerGame(bool quickstart) {
         if (human_player_setup_data.m_starting_species_name == "1")
             human_player_setup_data.m_starting_species_name = "SP_HUMAN";   // kludge / bug workaround for bug with options storage and retreival.  Empty-string options are stored, but read in as "true" boolean, and converted to string equal to "1"
 
-        if (!GetSpecies(human_player_setup_data.m_starting_species_name)) {
+        if (human_player_setup_data.m_starting_species_name != "RANDOM" &&
+            !GetSpecies(human_player_setup_data.m_starting_species_name))
+        {
             const SpeciesManager& sm = GetSpeciesManager();
             if (sm.NumPlayableSpecies() < 1)
                 human_player_setup_data.m_starting_species_name.clear();
             else
                 human_player_setup_data.m_starting_species_name = sm.playable_begin()->first;
-            }
+        }
 
         human_player_setup_data.m_save_game_empire_id = ALL_EMPIRES; // not used for new games
         human_player_setup_data.m_client_type = Networking::CLIENT_TYPE_HUMAN_PLAYER;
@@ -524,7 +511,7 @@ void HumanClientApp::NewSinglePlayerGame(bool quickstart) {
         for (int ai_i = 1; ai_i <= num_AIs; ++ai_i) {
             PlayerSetupData ai_setup_data;
 
-            ai_setup_data.m_player_name = "AI_" + boost::lexical_cast<std::string>(ai_i);
+            ai_setup_data.m_player_name = "AI_" + std::to_string(ai_i);
             ai_setup_data.m_empire_name.clear();                // leave blank, to be set by server in Universe::GenerateEmpires
             ai_setup_data.m_empire_color = GG::CLR_ZERO;        // to be set by server
             ai_setup_data.m_starting_species_name.clear();      // leave blank, to be set by server
@@ -534,26 +521,16 @@ void HumanClientApp::NewSinglePlayerGame(bool quickstart) {
             setup_data.m_players.push_back(ai_setup_data);
         }
 
-        m_networking.SendMessage(HostSPGameMessage(setup_data));
+        m_networking->SendMessage(HostSPGameMessage(setup_data));
         m_fsm->process_event(HostSPGameRequested());
     } else {
-        DebugLogger() << "HumanClientApp::NewSinglePlayerGame killing server due to canceled game or server connection failure";
-        if (m_networking.Connected()) {
-            DebugLogger() << "HumanClientApp::NewSinglePlayerGame Sending server shutdown message.";
-            m_networking.SendMessage(ShutdownServerMessage(m_networking.PlayerID()));
-            boost::this_thread::sleep_for(boost::chrono::seconds(1));
-            m_networking.DisconnectFromServer();
-            if (!m_networking.Connected())
-                DebugLogger() << "HumanClientApp::NewSinglePlayerGame Disconnected from server.";
-            else
-                ErrorLogger() << "HumanClientApp::NewSinglePlayerGame Unexpectedly still connected to server...?";
-        }
-        KillServer();
+        ErrorLogger() << "HumanClientApp::NewSinglePlayerGame failed to start new game, killing server.";
+        ResetToIntro();
     }
 }
 
 void HumanClientApp::MultiPlayerGame() {
-    if (m_networking.Connected()) {
+    if (m_networking->IsConnected()) {
         ErrorLogger() << "HumanClientApp::MultiPlayerGame aborting because already connected to a server";
         return;
     }
@@ -582,24 +559,22 @@ void HumanClientApp::MultiPlayerGame() {
         server_name = GetOptionsDB().Get<std::string>("external-server-address");
     }
 
-    unsigned int start_time = Ticks();
-    while (!m_networking.ConnectToServer(server_name, boost::posix_time::seconds(2))) {
-        if (SERVER_CONNECT_TIMEOUT < Ticks() - start_time) {
-            ClientUI::MessageBox(UserString("ERR_CONNECT_TIMED_OUT"), true);
-            if (server_connect_wnd.Result().second == "HOST GAME SELECTED")
-                KillServer();
-            return;
-        }
+
+    m_connected = m_networking->ConnectToServer(server_name);
+    if (!m_connected) {
+        ClientUI::MessageBox(UserString("ERR_CONNECT_TIMED_OUT"), true);
+        if (server_connect_wnd.Result().second == "HOST GAME SELECTED")
+            ResetToIntro();
+        return;
     }
 
     if (server_connect_wnd.Result().second == "HOST GAME SELECTED") {
-        m_networking.SendMessage(HostMPGameMessage(server_connect_wnd.Result().first));
+        m_networking->SendMessage(HostMPGameMessage(server_connect_wnd.Result().first));
         m_fsm->process_event(HostMPGameRequested());
     } else {
-        m_networking.SendMessage(JoinGameMessage(server_connect_wnd.Result().first, Networking::CLIENT_TYPE_HUMAN_PLAYER));
+        m_networking->SendMessage(JoinGameMessage(server_connect_wnd.Result().first, Networking::CLIENT_TYPE_HUMAN_PLAYER));
         m_fsm->process_event(JoinMPGameRequested());
     }
-    m_connected = true;
 }
 
 void HumanClientApp::StartMultiPlayerGameFromLobby()
@@ -609,11 +584,18 @@ void HumanClientApp::CancelMultiplayerGameFromLobby()
 { m_fsm->process_event(CancelMPGameClicked()); }
 
 void HumanClientApp::SaveGame(const std::string& filename) {
+    m_save_game_in_progress = true;
     Message response_msg;
-    m_networking.SendMessage(HostSaveGameInitiateMessage(PlayerID(), filename));
+    m_networking->SendMessage(HostSaveGameInitiateMessage(PlayerID(), filename));
     DebugLogger() << "HumanClientApp::SaveGame sent save initiate message to server...";
 }
 
+void HumanClientApp::SaveGameCompleted() {
+    DebugLogger() << "HumanClientApp::SaveGameCompleted by server.";
+    m_save_game_in_progress = false;
+    SaveGameCompletedSignal();
+}
+
 void HumanClientApp::LoadSinglePlayerGame(std::string filename/* = ""*/) {
     DebugLogger() << "HumanClientApp::LoadSinglePlayerGame";
 
@@ -643,9 +625,9 @@ void HumanClientApp::LoadSinglePlayerGame(std::string filename/* = ""*/) {
 
     // end any currently-playing game before loading new one
     if (m_game_started) {
-        EndGame();
+        ResetToIntro();
         // delay to make sure old game is fully cleaned up before attempting to start a new one
-        boost::this_thread::sleep_for(boost::chrono::seconds(3));
+        std::this_thread::sleep_for(std::chrono::seconds(3));
     } else {
         DebugLogger() << "HumanClientApp::LoadSinglePlayerGame() not already in a game, so don't need to end it";
     }
@@ -660,21 +642,15 @@ void HumanClientApp::LoadSinglePlayerGame(std::string filename/* = ""*/) {
     }
 
     DebugLogger() << "HumanClientApp::LoadSinglePlayerGame() Connecting to server";
-    unsigned int start_time = Ticks();
-    while (!m_networking.ConnectToLocalHostServer()) {
-        if (SERVER_CONNECT_TIMEOUT < Ticks() - start_time) {
-            ErrorLogger() << "HumanClientApp::LoadSinglePlayerGame() server connecting timed out";
-            ClientUI::MessageBox(UserString("ERR_CONNECT_TIMED_OUT"), true);
-            KillServer();
-            return;
-        }
+    m_connected = m_networking->ConnectToLocalHostServer();
+    if (!m_connected) {
+        ResetToIntro();
+        ClientUI::MessageBox(UserString("ERR_CONNECT_TIMED_OUT"), true);
+        return;
     }
 
-    DebugLogger() << "HumanClientApp::LoadSinglePlayerGame() Connected to server";
-
-    m_connected = true;
-    m_networking.SetPlayerID(Networking::INVALID_PLAYER_ID);
-    m_networking.SetHostPlayerID(Networking::INVALID_PLAYER_ID);
+    m_networking->SetPlayerID(Networking::INVALID_PLAYER_ID);
+    m_networking->SetHostPlayerID(Networking::INVALID_PLAYER_ID);
     SetEmpireID(ALL_EMPIRES);
 
     SinglePlayerSetupData setup_data;
@@ -684,7 +660,7 @@ void HumanClientApp::LoadSinglePlayerGame(std::string filename/* = ""*/) {
     // leving setup_data.m_players empty : not specified when loading a game, as server will generate from save file
 
 
-    m_networking.SendMessage(HostSPGameMessage(setup_data));
+    m_networking->SendMessage(HostSPGameMessage(setup_data));
     m_fsm->process_event(HostSPGameRequested());
 }
 
@@ -693,32 +669,27 @@ void HumanClientApp::RequestSavePreviews(const std::string& directory, PreviewIn
     DebugLogger() << "HumanClientApp::RequestSavePreviews directory: " << directory << " valid UTF-8: " << utf8::is_valid(directory.begin(), directory.end());
 
     std::string  generic_directory = directory;//PathString(fs::path(directory));
-    if (!m_networking.Connected()) {
+    if (!m_networking->IsConnected()) {
         DebugLogger() << "HumanClientApp::RequestSavePreviews: No game running. Start a server for savegame queries.";
 
         m_single_player_game = true;
         StartServer();
 
         DebugLogger() << "HumanClientApp::RequestSavePreviews Connecting to server";
-        unsigned int start_time = Ticks();
-        while (!m_networking.ConnectToLocalHostServer()) {
-            if (SERVER_CONNECT_TIMEOUT < Ticks() - start_time) {
-                ErrorLogger() << "HumanClientApp::LoadSinglePlayerGame() server connecting timed out";
-                ClientUI::MessageBox(UserString("ERR_CONNECT_TIMED_OUT"), true);
-                KillServer();
-                return;
-            }
+        m_connected = m_networking->ConnectToLocalHostServer();
+        if (!m_connected) {
+            ResetToIntro();
+            ClientUI::MessageBox(UserString("ERR_CONNECT_TIMED_OUT"), true);
+            return;
         }
-        m_connected = true;
     }
     DebugLogger() << "HumanClientApp::RequestSavePreviews Requesting previews for " << generic_directory;
-    Message response;
-    m_networking.SendSynchronousMessage(RequestSavePreviewsMessage(PlayerID(), generic_directory), response);
-    if (response.Type() == Message::DISPATCH_SAVE_PREVIEWS){
-        ExtractMessageData(response, previews);
+    const auto response = m_networking->SendSynchronousMessage(RequestSavePreviewsMessage(PlayerID(), generic_directory));
+    if (response && response->Type() == Message::DISPATCH_SAVE_PREVIEWS) {
+        ExtractDispatchSavePreviewsMessageData(*response, previews);
         DebugLogger() << "HumanClientApp::RequestSavePreviews Got " << previews.previews.size() << " previews.";
-    }else{
-        ErrorLogger() << "HumanClientApp::RequestSavePreviews: Wrong response type from server: " << EnumToString(response.Type());
+    } else {
+        ErrorLogger() << "HumanClientApp::RequestSavePreviews: Wrong response type from server: " << response->Type();
     }
 }
 
@@ -736,7 +707,7 @@ std::pair<int, int> HumanClientApp::GetWindowLeftTop() {
     if (std::abs(top) < 10)
         top = 0;
 
-    return std::make_pair(left, top);
+    return {left, top};
 }
 
 std::pair<int, int> HumanClientApp::GetWindowWidthHeight() {
@@ -746,14 +717,14 @@ std::pair<int, int> HumanClientApp::GetWindowWidthHeight() {
     if (!fullscreen) {
         width = GetOptionsDB().Get<int>("app-width-windowed");
         height = GetOptionsDB().Get<int>("app-height-windowed");
-        return std::make_pair(width, height);
+        return {width, height};
     }
 
     bool reset_fullscreen = GetOptionsDB().Get<bool>("reset-fullscreen-size");
     if (!reset_fullscreen) {
         width = GetOptionsDB().Get<int>("app-width");
         height = GetOptionsDB().Get<int>("app-height");
-        return std::make_pair(width, height);
+        return {width, height};
     }
 
     GetOptionsDB().Set<bool>("reset-fullscreen-size", false);
@@ -761,7 +732,7 @@ std::pair<int, int> HumanClientApp::GetWindowWidthHeight() {
     GetOptionsDB().Set("app-width", Value(default_resolution.x));
     GetOptionsDB().Set("app-height", Value(default_resolution.y));
     GetOptionsDB().Commit();
-    return std::make_pair(Value(default_resolution.x), Value(default_resolution.y));
+    return {Value(default_resolution.x), Value(default_resolution.y)};
 }
 
 void HumanClientApp::Reinitialize() {
@@ -801,8 +772,7 @@ float HumanClientApp::GLVersion() const
 void HumanClientApp::StartTurn() {
     DebugLogger() << "HumanClientApp::StartTurn";
 
-    const Empire *empire = GetEmpire(EmpireID());
-    if (empire) {
+    if (const Empire* empire = GetEmpire(EmpireID())) {
         double RP = empire->ResourceOutput(RE_RESEARCH);
         double PP = empire->ResourceOutput(RE_INDUSTRY);
         int turn_number = CurrentTurn();
@@ -825,17 +795,15 @@ void HumanClientApp::HandleSystemEvents() {
     } catch (const utf8::invalid_utf8& e) {
         ErrorLogger() << "UTF-8 error handling system event: " << e.what();
     }
-    if (m_connected && !m_networking.Connected()) {
+    if (m_connected && !m_networking->IsConnected()) {
         m_connected = false;
         DisconnectedFromServer();
-    } else if (m_networking.MessageAvailable()) {
-        Message msg;
-        m_networking.GetMessage(msg);
+    } else if (auto msg = Networking().GetMessage()) {
         try {
-            HandleMessage(msg);
+            HandleMessage(*msg);
         } catch (const std::exception& e) {
             ErrorLogger() << "exception handing message: " << e.what();
-            ErrorLogger() << "message type: " << msg.Type() << " and text: " << msg.Text();
+            ErrorLogger() << "message type: " << msg->Type() << " and text: " << msg->Text();
         }
     }
 }
@@ -869,6 +837,8 @@ void HumanClientApp::HandleMessage(Message& msg) {
     case Message::DIPLOMACY:                m_fsm->process_event(Diplomacy(msg));               break;
     case Message::DIPLOMATIC_STATUS:        m_fsm->process_event(DiplomaticStatusUpdate(msg));  break;
     case Message::END_GAME:                 m_fsm->process_event(::EndGame(msg));               break;
+
+    case Message::DISPATCH_COMBAT_LOGS:     m_fsm->process_event(DispatchCombatLogs(msg));       break;
     default:
         ErrorLogger() << "HumanClientApp::HandleMessage : Received an unknown message type \"" << msg.Type() << "\".";
     }
@@ -879,7 +849,22 @@ void HumanClientApp::HandleSaveGameDataRequest() {
         std::cerr << "HumanClientApp::HandleSaveGameDataRequest(" << Message::SAVE_GAME_DATA_REQUEST << ")\n";
     SaveGameUIData ui_data;
     m_ui->GetSaveGameUIData(ui_data);
-    m_networking.SendMessage(ClientSaveDataMessage(PlayerID(), Orders(), ui_data));
+    m_networking->SendMessage(ClientSaveDataMessage(PlayerID(), Orders(), ui_data));
+}
+
+void HumanClientApp::UpdateCombatLogs(const Message& msg){
+    DebugLogger() << "HCL Update Combat Logs";
+
+    // Unpack the combat logs from the message
+    std::vector<std::pair<int, CombatLog>> logs;
+    ExtractDispatchCombatLogsMessageData(msg, logs);
+
+    // Update the combat log manager with the completed logs.
+    for (std::vector<std::pair<int, CombatLog>>::const_iterator it = logs.begin();
+         it != logs.end(); ++it)
+    {
+        GetCombatLogManager().CompleteLog(it->first, it->second);
+    }
 }
 
 void HumanClientApp::HandleWindowMove(GG::X w, GG::Y h) {
@@ -919,15 +904,6 @@ void HumanClientApp::HandleWindowResize(GG::X w, GG::Y h) {
     GetOptionsDB().Commit();
 }
 
-void HumanClientApp::HandleWindowClosing()
-{ DebugLogger() << "HumanClientApp::HandleWindowClosing()"; }
-
-void HumanClientApp::HandleWindowClose() {
-    DebugLogger() << "HumanClientApp::HandleWindowClose()";
-    EndGame();
-    Exit(0);
-}
-
 void HumanClientApp::HandleFocusChange(bool gained_focus) {
     DebugLogger() << "HumanClientApp::HandleFocusChange("
                   << (gained_focus ? "Gained Focus" : "Lost Focus")
@@ -935,31 +911,44 @@ void HumanClientApp::HandleFocusChange(bool gained_focus) {
 
     m_have_window_focus = gained_focus;
 
-    // limit rendering frequency when defocused to limit CPU use
+    // limit rendering frequency when defocused to limit CPU use, and disable sound
     if (!m_have_window_focus) {
         if (GetOptionsDB().Get<bool>("limit-fps-no-focus"))
             this->SetMaxFPS(GetOptionsDB().Get<double>("max-fps-no_focus"));
         else
             this->SetMaxFPS(0.0);
-    } else {
+
+        if (GetOptionsDB().Get<bool>("UI.sound.music-enabled"))
+            Sound::GetSound().PauseMusic();
+    }
+    else {
         if (GetOptionsDB().Get<bool>("limit-fps"))
             this->SetMaxFPS(GetOptionsDB().Get<double>("max-fps"));
         else
             this->SetMaxFPS(0.0);
+
+        if (GetOptionsDB().Get<bool>("UI.sound.music-enabled"))
+            Sound::GetSound().ResumeMusic();
     }
 
     CancelDragDrop();
     ClearEventState();
 }
 
-bool HumanClientApp::QuitGame() {
-    EndGame();
+void HumanClientApp::HandleAppQuitting() {
+    DebugLogger() << "HumanClientApp::HandleAppQuitting()";
+    ExitApp();
+}
+
+bool HumanClientApp::HandleHotkeyResetGame() {
+    DebugLogger() << "HumanClientApp::HandleHotkeyResetGame()";
+    ResetToIntro();
     return true;
 }
 
-bool HumanClientApp::ExitGame() {
-    QuitGame();
-    Exit(0);
+bool HumanClientApp::HandleHotkeyExitApp() {
+    DebugLogger() << "HumanClientApp::HandleHotkeyExitApp()";
+    HandleAppQuitting();
     // Not reached, but required for HotkeyManager::Connect()
     return true;
 }
@@ -977,6 +966,17 @@ void HumanClientApp::StartGame() {
 
     if (MapWnd* map_wnd = ClientUI::GetClientUI()->GetMapWnd())
         map_wnd->ResetEmpireShown();
+
+    UpdateCombatLogManager();
+}
+
+void HumanClientApp::HandleTurnUpdate()
+{ UpdateCombatLogManager(); }
+
+void HumanClientApp::UpdateCombatLogManager() {
+    boost::optional<std::vector<int>> incomplete_ids = GetCombatLogManager().IncompleteLogIDs();
+    if (incomplete_ids)
+        m_networking->SendMessage(RequestCombatLogsMessage(PlayerID(), *incomplete_ids));
 }
 
 namespace {
@@ -999,25 +999,22 @@ namespace {
                 { continue; }
 
                 std::time_t t = last_write_time(file_path);
-                files_by_write_time.insert(std::make_pair(t, file_path));
+                files_by_write_time.insert({t, file_path});
             }
 
             //DebugLogger() << "files by write time:";
-            //for (std::multimap<std::time_t, path>::const_iterator it = files_by_write_time.begin();
-            //     it != files_by_write_time.end(); ++it)
-            //{ DebugLogger() << it->first << " : " << it->second.filename(); }
+            //for (std::multimap<std::time_t, path>::value_type& entry : files_by_write_time)
+            //{ DebugLogger() << entry.first << " : " << entry.second.filename(); }
 
             int num_to_delete = files_by_write_time.size() - files_limit + 1;   // +1 because will add a new file after deleting, bringing number back up to limit
             if (num_to_delete <= 0)
                 return; // don't need to delete anything.
 
             int num_deleted = 0;
-            for (std::multimap<std::time_t, path>::const_iterator it = files_by_write_time.begin();
-                 it != files_by_write_time.end(); ++it)
-            {
+            for (std::multimap<std::time_t, path>::value_type& entry : files_by_write_time) {
                 if (num_deleted >= num_to_delete)
                     break;
-                remove(it->second);
+                remove(entry.second);
                 ++num_deleted;
             }
         } catch (...) {
@@ -1108,40 +1105,15 @@ void HumanClientApp::Autosave() {
     }
 }
 
-std::string HumanClientApp::SelectLoadFile(){
+std::string HumanClientApp::SelectLoadFile() {
     SaveFileDialog sfd(true);
     sfd.Run();
     return sfd.Result();
 }
 
-std::string HumanClientApp::SelectSaveFile() {
-    SaveFileDialog sfd(false);
-    sfd.Run();
-    return sfd.Result();
-}
-
-void HumanClientApp::EndGame(bool suppress_FSM_reset) {
-    DebugLogger() << "HumanClientApp::EndGame";
-    m_game_started = false;
-
-    if (m_networking.Connected()) {
-        DebugLogger() << "HumanClientApp::EndGame Sending server shutdown message.";
-        m_networking.SendMessage(ShutdownServerMessage(m_networking.PlayerID()));
-        boost::this_thread::sleep_for(boost::chrono::seconds(1));
-        m_networking.DisconnectFromServer();
-        if (!m_networking.Connected())
-            DebugLogger() << "HumanClientApp::EndGame Disconnected from server.";
-        else
-            ErrorLogger() << "HumanClientApp::EndGame Unexpectedly still connected to server...?";
-    }
-
-    if (!m_server_process.Empty()) {
-        DebugLogger() << "HumanClientApp::EndGame Terminated server process.";
-        m_server_process.RequestTermination();
-    }
-
-    m_networking.SetPlayerID(Networking::INVALID_PLAYER_ID);
-    m_networking.SetHostPlayerID(Networking::INVALID_PLAYER_ID);
+void HumanClientApp::ResetClientData() {
+    m_networking->SetPlayerID(Networking::INVALID_PLAYER_ID);
+    m_networking->SetHostPlayerID(Networking::INVALID_PLAYER_ID);
     SetEmpireID(ALL_EMPIRES);
     m_ui->GetMapWnd()->Sanitize();
 
@@ -1149,9 +1121,47 @@ void HumanClientApp::EndGame(bool suppress_FSM_reset) {
     m_empires.Clear();
     m_orders.Reset();
     GetCombatLogManager().Clear();
+}
 
-    if (!suppress_FSM_reset)
-        m_fsm->process_event(ResetToIntroMenu());
+namespace {
+    // Ask the player if they want to wait for the save game to complete
+    // The dialog automatically closes if the save completes while the user is waiting
+    class SaveGamePendingDialog : public GG::ThreeButtonDlg {
+        public:
+        SaveGamePendingDialog(bool reset, boost::signals2::signal<void()>& save_completed_signal) :
+            GG::ThreeButtonDlg(
+                GG::X(320), GG::Y(200), UserString("SAVE_GAME_IN_PROGRESS"),
+                ClientUI::GetFont(ClientUI::Pts()+2),
+                ClientUI::WndColor(), ClientUI::WndOuterBorderColor(),
+                ClientUI::CtrlColor(), ClientUI::TextColor(), 1,
+                (reset ? UserString("ABORT_SAVE_AND_RESET") : UserString("ABORT_SAVE_AND_EXIT")))
+        { GG::Connect(save_completed_signal, &SaveGamePendingDialog::SaveCompletedHandler, this); }
+
+        void SaveCompletedHandler() {
+            DebugLogger() << "SaveGamePendingDialog::SaveCompletedHandler save game completed handled.";
+            m_done = true;
+        }
+    };
+}
+
+void HumanClientApp::ResetToIntro()
+{ ResetOrExitApp(true); }
+
+void HumanClientApp::ExitApp()
+{ ResetOrExitApp(false); }
+
+void HumanClientApp::ResetOrExitApp(bool reset) {
+    DebugLogger() << (reset ? "HumanClientApp::ResetToIntro" : "HumanClientApp::ExitApp");
+
+    m_game_started = false;
+
+    if (m_save_game_in_progress) {
+        DebugLogger() << "save game in progress. Checking with player.";
+        auto dlg = SaveGamePendingDialog(reset, this->SaveGameCompletedSignal);
+        dlg.Run();
+    }
+
+    m_fsm->process_event(StartQuittingGame(reset, m_server_process));
 }
 
 void HumanClientApp::InitAutoTurns(int auto_turns) {
