@@ -12,11 +12,12 @@
 
 #include <GG/utf8/checked.h>
 
-#include <boost/lexical_cast.hpp>
 #include <boost/format.hpp>
 #include <boost/filesystem/operations.hpp>
 #include <boost/filesystem/fstream.hpp>
 
+#include <thread>
+#include <chrono>
 #include <iostream>
 
 #if defined(FREEORION_LINUX)
@@ -216,15 +217,15 @@ int mainConfigOptionsSetup(const std::vector<std::string>& args) {
 #ifndef FREEORION_CHMAIN_KEEP_STACKTRACE
     } catch (const std::invalid_argument& e) {
         std::cerr << "main() caught exception(std::invalid_argument): " << e.what() << std::endl;
-        boost::this_thread::sleep_for(boost::chrono::seconds(3));
+        std::this_thread::sleep_for(std::chrono::seconds(3));
         return 1;
     } catch (const std::runtime_error& e) {
         std::cerr << "main() caught exception(std::runtime_error): " << e.what() << std::endl;
-        boost::this_thread::sleep_for(boost::chrono::seconds(3));
+        std::this_thread::sleep_for(std::chrono::seconds(3));
         return 1;
     } catch (const std::exception& e) {
         std::cerr << "main() caught exception(std::exception): " << e.what() << std::endl;
-        boost::this_thread::sleep_for(boost::chrono::seconds(3));
+        std::this_thread::sleep_for(std::chrono::seconds(3));
         return 1;
     }
 #endif
@@ -237,7 +238,6 @@ int mainSetupAndRun() {
     try {
         RegisterOptions(&HumanClientApp::AddWindowSizeOptionsAfterMainStart);
 
-        int colour_depth = GetOptionsDB().Get<int>("color-depth");
         bool fullscreen = GetOptionsDB().Get<bool>("fullscreen");
         bool fake_mode_change = GetOptionsDB().Get<bool>("fake-mode-change");
 
@@ -246,14 +246,12 @@ int mainSetupAndRun() {
         std::pair<int, int> left_top = HumanClientApp::GetWindowLeftTop();
         int left(left_top.first), top(left_top.second);
 
-        int fullscreen_monitor_id = GetOptionsDB().Get<int>("fullscreen-monitor-id");
-
 #ifdef FREEORION_WIN32
 #  ifdef IDI_ICON1
         // set window icon to embedded application icon
         HWND hwnd;
         window->getCustomAttribute("WINDOW", &hwnd);
-        HINSTANCE hInst = (HINSTANCE)GetModuleHandle(NULL);
+        HINSTANCE hInst = (HINSTANCE)GetModuleHandle(nullptr);
         SetClassLong (hwnd, GCL_HICON,
             (LONG)LoadIcon (hInst, MAKEINTRESOURCE (IDI_ICON1)));
 #  endif
@@ -285,7 +283,6 @@ int mainSetupAndRun() {
 
     } catch (const HumanClientApp::CleanQuit&) {
         // do nothing
-        std::cout << "mainSetupAndRun caught CleanQuit" << std::endl;
     }
 #ifndef FREEORION_CHMAIN_KEEP_STACKTRACE
     catch (const std::invalid_argument& e) {
@@ -307,6 +304,7 @@ int mainSetupAndRun() {
     }
 #endif
 
+    DebugLogger() << "Human client main exited cleanly.";
     return 0;
 }
 
