@@ -5,14 +5,17 @@
 #include "../util/Serialize.ipp"
 #include "../util/Serialize.h"
 #include "../universe/Universe.h"
-
-#include <sstream>
+#include "../universe/Enums.h"
 
 #include "../util/i18n.h"
+#include "../util/Logger.h"
+#include "../util/AppInterface.h"
 #include "../util/VarText.h"
 #include "../UI/LinkText.h"
 #include "../Empire/Empire.h"
-#include "../client/human/HumanClientApp.h"
+
+#include <sstream>
+
 
 namespace {
     //TODO: Move this code into a common non UI linked location, so that
@@ -38,7 +41,7 @@ namespace {
 
     std::string WrapWithTagAndId(const std::string& meat, const std::string& tag, int id) {
         std::stringstream ss;
-        ss << "<" << tag << " " << boost::lexical_cast<std::string>(id) << ">" << meat << "</" << tag << ">";
+        ss << "<" << tag << " " << std::to_string(id) << ">" << meat << "</" << tag << ">";
         return ss.str();
     }
 
@@ -65,9 +68,6 @@ namespace {
         // Hardcoded default color becauses not linked with UI code.
         const Empire* empire = GetEmpire(empire_id);
         GG::Clr c = (empire ? empire->Color() : GG::Clr(80,255,128,255));
-        if (!empire) {
-            ErrorLogger() << "EmpireColorWrappedText expected an empire id and received " << empire_id;
-        }
         return WrapColorTag(text, c);
     }
 
@@ -76,7 +76,7 @@ namespace {
     /// Returns UserString("ENC_COMBAT_UNKNOWN_OBJECT") if object_id is not found.
     std::string PublicNameLink(int empire_id, int object_id) {
 
-        TemporaryPtr<const UniverseObject> object = GetUniverseObject(object_id);
+        std::shared_ptr<const UniverseObject> object = GetUniverseObject(object_id);
         if (object) {
             const std::string& name = object->PublicName(empire_id);
             const std::string& tag = LinkTag(object->ObjectType());
@@ -163,7 +163,7 @@ BoutEvent::BoutEvent():
 BoutEvent::BoutEvent(int _bout):
     bout(_bout), events() {}
 
-void BoutEvent::AddEvent(CombatEventPtr &event)
+void BoutEvent::AddEvent(const CombatEventPtr& event)
 { events.push_back(event); }
 
 std::string BoutEvent::DebugString() const {
@@ -178,9 +178,8 @@ std::string BoutEvent::CombatLogDescription(int viewing_empire_id) const {
 
 std::vector<ConstCombatEventPtr> BoutEvent::SubEvents(int viewing_empire_id) const {
     std::vector<ConstCombatEventPtr> all_events;
-    for (std::vector<CombatEventPtr>::const_iterator it = events.begin();
-         it != events.end(); ++it) {
-        all_events.push_back(*it);
+    for (CombatEventPtr event : events) {
+        all_events.push_back(event);
     }
     return all_events;
 }
@@ -215,7 +214,7 @@ void BoutEvent::serialize<freeorion_xml_iarchive>(freeorion_xml_iarchive& ar, co
 SimultaneousEvents::SimultaneousEvents() :
     events() {}
 
-void SimultaneousEvents::AddEvent(CombatEventPtr &event)
+void SimultaneousEvents::AddEvent(const CombatEventPtr& event)
 { events.push_back(event); }
 
 std::string SimultaneousEvents::DebugString() const {
@@ -235,11 +234,10 @@ std::vector<ConstCombatEventPtr> SimultaneousEvents::SubEvents(int viewing_empir
     typedef std::multimap<int, ConstCombatEventPtr>::iterator iterator;
     typedef std::pair<iterator, iterator> range;
 
-    for (std::vector<CombatEventPtr>::const_iterator it = events.begin();
-         it != events.end(); ++it) {
-        boost::optional<int> maybe_faction = (*it)->PrincipalFaction(viewing_empire_id);
+    for (CombatEventPtr event : events) {
+        boost::optional<int> maybe_faction = event->PrincipalFaction(viewing_empire_id);
         int faction = maybe_faction.get_value_or(ALL_EMPIRES);
-        empire_to_event.insert(std::make_pair(faction, *it));
+        empire_to_event.insert(std::make_pair(faction, event));
     }
 
     std::vector<ConstCombatEventPtr> ordered_events;
@@ -257,10 +255,9 @@ std::vector<ConstCombatEventPtr> SimultaneousEvents::SubEvents(int viewing_empir
         ordered_events.push_back(it->second);
     }
 
-    for (iterator it = empire_to_event.begin();
-         it != empire_to_event.end(); ++it) {
-        if (it->first != viewing_empire_id && it->first != ALL_EMPIRES)
-            ordered_events.push_back(it->second);
+    for (std::multimap<int, ConstCombatEventPtr>::value_type& entry : empire_to_event) {
+        if (entry.first != viewing_empire_id && entry.first != ALL_EMPIRES)
+            ordered_events.push_back(entry.second);
     }
 
     return ordered_events;
@@ -307,20 +304,16 @@ std::string InitialStealthEvent::DebugString() const {
     if (target_empire_id_to_invisble_obj_id.size() > 4) {
         ss << target_empire_id_to_invisble_obj_id.size() << " events.";
     } else {
-        for (StealthInvisbleMap::const_iterator attack_empire = target_empire_id_to_invisble_obj_id.begin();
-             attack_empire != target_empire_id_to_invisble_obj_id.end(); ++attack_empire) {
-            ss << " Attacking Empire: " << EmpireLink(attack_empire->first) << "\n";
-            for (std::map<int, std::set<std::pair<int, Visibility> > >::const_iterator
-                     target_empire = attack_empire->second.begin();
-                 target_empire != attack_empire->second.end(); ++target_empire) {
-                ss << " Target Empire: " << EmpireLink(target_empire->first) << " Targets: ";
+        for (const StealthInvisbleMap::value_type& attack_empire : target_empire_id_to_invisble_obj_id) {
+            ss << " Attacking Empire: " << EmpireLink(attack_empire.first) << "\n";
+            for (const std::map<int, std::set<std::pair<int, Visibility>>>::value_type& target_empire : attack_empire.second) {
+                ss << " Target Empire: " << EmpireLink(target_empire.first) << " Targets: ";
 
-                if (target_empire->second.size() > 4) {
-                    ss << target_empire->second.size() << " attackers.";
+                if (target_empire.second.size() > 4) {
+                    ss << target_empire.second.size() << " attackers.";
                 } else {
-                    for (std::set<std::pair<int, Visibility> >::const_iterator attacker_it = target_empire->second.begin();
-                         attacker_it != target_empire->second.end(); ++attacker_it) {
-                        ss << FighterOrPublicNameLink(ALL_EMPIRES, attacker_it->first, target_empire->first);
+                    for (const std::pair<int, Visibility>& attacker : target_empire.second) {
+                        ss << FighterOrPublicNameLink(ALL_EMPIRES, attacker.first, target_empire.first);
                     }
                 }
                 ss << "\n";
@@ -335,27 +328,25 @@ std::string InitialStealthEvent::CombatLogDescription(int viewing_empire_id) con
     std::string desc = "";
 
     //Viewing empire stealth first
-    for (StealthInvisbleMap::const_iterator attack_empire = target_empire_id_to_invisble_obj_id.begin();
-         attack_empire != target_empire_id_to_invisble_obj_id.end(); ++attack_empire) {
-        if (attack_empire->first == viewing_empire_id)
+    for (const StealthInvisbleMap::value_type& attack_empire : target_empire_id_to_invisble_obj_id) {
+        if (attack_empire.first == viewing_empire_id)
             continue;
 
-        std::map<int, std::set<std::pair<int, Visibility> > >::const_iterator target_empire
-            = attack_empire->second.find(viewing_empire_id);
-        if (target_empire != attack_empire->second.end()
+        std::map<int, std::set<std::pair<int, Visibility>>>::const_iterator target_empire
+            = attack_empire.second.find(viewing_empire_id);
+        if (target_empire != attack_empire.second.end()
             && !target_empire->second.empty()) {
 
             std::vector<std::string> cloaked_attackers;
-            for (std::set<std::pair<int, Visibility> >::const_iterator attacker_it = target_empire->second.begin();
-                 attacker_it != target_empire->second.end(); ++attacker_it) {
-                 std::string attacker_link = FighterOrPublicNameLink(viewing_empire_id, attacker_it->first, viewing_empire_id);
+            for (const std::set<std::pair<int, Visibility>>::value_type& attacker : target_empire->second) {
+                 std::string attacker_link = FighterOrPublicNameLink(viewing_empire_id, attacker.first, viewing_empire_id);
                 // It doesn't matter if targets of viewing empire have no_visibility or basic_visibility
                  cloaked_attackers.push_back(attacker_link);
             }
 
             if (!cloaked_attackers.empty()) {
                 desc += "\n"; //< Add \n at start of the report and between each empire
-                std::vector<std::string> attacker_empire_link(1, EmpireLink(attack_empire->first));
+                std::vector<std::string> attacker_empire_link(1, EmpireLink(attack_empire.first));
                 desc += FlexibleFormatList(attacker_empire_link, cloaked_attackers
                                            , UserString("ENC_COMBAT_INITIAL_STEALTH_LIST")).str();
             }
@@ -367,18 +358,15 @@ std::string InitialStealthEvent::CombatLogDescription(int viewing_empire_id) con
         = target_empire_id_to_invisble_obj_id.find(viewing_empire_id);
     if (attack_empire != target_empire_id_to_invisble_obj_id.end()
         && !attack_empire->second.empty()) {
-        for (std::map<int, std::set<std::pair<int, Visibility> > >::const_iterator
-                 target_empire = attack_empire->second.begin();
-             target_empire != attack_empire->second.end(); ++target_empire) {
-            if (target_empire->first == viewing_empire_id)
+        for (const std::map<int, std::set<std::pair<int, Visibility>>>::value_type& target_empire : attack_empire->second) {
+            if (target_empire.first == viewing_empire_id)
                 continue;
 
             std::vector<std::string> cloaked_attackers;
-            for (std::set<std::pair<int, Visibility> >::const_iterator attacker_it = target_empire->second.begin();
-            attacker_it != target_empire->second.end(); ++attacker_it) {
-                 std::string attacker_link = FighterOrPublicNameLink(viewing_empire_id, attacker_it->first, viewing_empire_id);
+            for (const std::pair<int, Visibility>& attacker : target_empire.second) {
+                 std::string attacker_link = FighterOrPublicNameLink(viewing_empire_id, attacker.first, viewing_empire_id);
                 // Don't even report on targets with no_visibility it is supposed to be a surprise
-                if (attacker_it->second >= VIS_BASIC_VISIBILITY ) {
+                if (attacker.second >= VIS_BASIC_VISIBILITY ) {
                     cloaked_attackers.push_back(attacker_link);
                 }
             }
@@ -468,14 +456,13 @@ std::string StealthChangeEvent::StealthChangeEventDetail::CombatLogDescription(i
                % attacker_link
                % target_link
                % empire_link);
-    return "";
 }
 
 
 void StealthChangeEvent::AddEvent(int attacker_id_, int target_id_, int attacker_empire_
                              , int target_empire_, Visibility new_visibility_) {
     events[target_empire_].push_back(
-        boost::make_shared<StealthChangeEventDetail>(
+        std::make_shared<StealthChangeEventDetail>(
             attacker_id_, target_id_, attacker_empire_, target_empire_, new_visibility_));
 }
 
@@ -485,16 +472,14 @@ std::string StealthChangeEvent::DebugString() const {
     if (events.size() > 4) {
         ss << events.size() << " empires.";
     } else {
-        for (std::map<int, std::vector<StealthChangeEventDetailPtr> >::const_iterator target_it = events.begin();
-             target_it != events.end(); ++target_it) {
-            ss << "Target Empire: " << EmpireLink(target_it->first) << "\n";
+        for (const std::map<int, std::vector<StealthChangeEventDetailPtr>>::value_type& target : events) {
+            ss << "Target Empire: " << EmpireLink(target.first) << "\n";
 
-            if (target_it->second.size() > 4) {
-                ss << target_it->second.size() << " events.";
+            if (target.second.size() > 4) {
+                ss << target.second.size() << " events.";
             } else {
-                for (std::vector<StealthChangeEventDetailPtr>::const_iterator event_it = target_it->second.begin();
-                     event_it != target_it->second.end(); ++event_it){
-                    ss << (*event_it)->DebugString();
+                for (const StealthChangeEventDetailPtr event : target.second) {
+                    ss << event->DebugString();
                 }
             }
         }
@@ -507,19 +492,16 @@ std::string StealthChangeEvent::CombatLogDescription(int viewing_empire_id) cons
         return "";
 
     std::string desc = "";
-    for (std::map<int, std::vector<StealthChangeEventDetailPtr> >::const_iterator target_it = events.begin();
-         target_it != events.end(); ++target_it) {
+    for (const std::map<int, std::vector<StealthChangeEventDetailPtr>>::value_type& target : events) {
         std::vector<std::string> uncloaked_attackers;
-        for (std::vector<StealthChangeEventDetailPtr>::const_iterator event_it = target_it->second.begin();
-             event_it != target_it->second.end(); ++event_it){
-            const StealthChangeEvent::StealthChangeEventDetail & e(**event_it);
-            uncloaked_attackers.push_back( FighterOrPublicNameLink(viewing_empire_id, e.attacker_id, e.attacker_empire_id));
+        for (const StealthChangeEventDetailPtr event : target.second) {
+            uncloaked_attackers.push_back(FighterOrPublicNameLink(viewing_empire_id, event->attacker_id, event->attacker_empire_id));
         }
 
         if (!uncloaked_attackers.empty()) {
             if (!desc.empty())
                 desc += "\n";
-            std::vector<std::string> target_empire_link(1, EmpireLink(target_it->first));
+            std::vector<std::string> target_empire_link(1, EmpireLink(target.first));
 
             desc += FlexibleFormatList(target_empire_link, uncloaked_attackers
                                        , UserString("ENC_COMBAT_STEALTH_DECLOAK_ATTACK_MANY_EVENTS")
@@ -535,12 +517,10 @@ bool StealthChangeEvent::AreSubEventsEmpty(int viewing_empire_id) const {
 }
 std::vector<ConstCombatEventPtr> StealthChangeEvent::SubEvents(int viewing_empire_id) const {
     std::vector<ConstCombatEventPtr> all_events;
-    for (std::map<int, std::vector<StealthChangeEventDetailPtr> >::const_iterator target_it = events.begin();
-         target_it != events.end(); ++target_it) {
+    for (const std::map<int, std::vector<StealthChangeEventDetailPtr>>::value_type& target : events) {
 
-        for (std::vector<StealthChangeEventDetailPtr>::const_iterator event_it = target_it->second.begin();
-             event_it != target_it->second.end(); ++event_it){
-            all_events.push_back(boost::dynamic_pointer_cast<CombatEvent>(*event_it));
+        for (const StealthChangeEventDetailPtr event : target.second){
+            all_events.push_back(std::dynamic_pointer_cast<CombatEvent>(event));
         }
     }
     return all_events;
@@ -608,18 +588,20 @@ WeaponFireEvent::WeaponFireEvent() :
 {}
 
 WeaponFireEvent::WeaponFireEvent(
-    int bout_, int round_, int attacker_id_, int target_id_, const std::string &weapon_name_
-    , float power_, float shield_, float damage_, int attacker_owner_id_) :
+    int bout_, int round_, int attacker_id_, int target_id_, const std::string& weapon_name_,
+    const std::tuple<float, float, float>& power_shield_damage,
+    int attacker_owner_id_, int target_owner_id_) :
     bout(bout_),
     round(round_),
     attacker_id(attacker_id_),
     target_id( target_id_),
     weapon_name(weapon_name_),
-    power(power_),
-    shield(shield_),
-    damage(damage_),
-    attacker_owner_id(attacker_owner_id_)
-{ }
+    power(),
+    shield(),
+    damage(),
+    attacker_owner_id(attacker_owner_id_),
+    target_owner_id(target_owner_id_)
+{ std::tie(power, shield, damage) = power_shield_damage; }
 
 std::string WeaponFireEvent::DebugString() const {
     std::stringstream ss;
@@ -631,7 +613,7 @@ std::string WeaponFireEvent::DebugString() const {
 
 std::string WeaponFireEvent::CombatLogDescription(int viewing_empire_id) const {
     std::string attacker_link = FighterOrPublicNameLink(viewing_empire_id, attacker_id, attacker_owner_id);
-    std::string target_link = PublicNameLink(viewing_empire_id, target_id);
+    std::string target_link = FighterOrPublicNameLink(viewing_empire_id, target_id, target_owner_id);
 
     const std::string& template_str = UserString("ENC_COMBAT_ATTACK_STR");
 
@@ -646,11 +628,18 @@ std::string WeaponFireEvent::CombatLogDescription(int viewing_empire_id) const {
 std::string WeaponFireEvent::CombatLogDetails(int viewing_empire_id) const {
     const std::string& template_str = UserString("ENC_COMBAT_ATTACK_DETAILS");
 
-    return str(FlexibleFormat(template_str)
-               % ShipPartLink(weapon_name)
-               % power
-               % shield
-               % damage);
+    if (shield >= 0)
+        return str(FlexibleFormat(template_str)
+                   % ShipPartLink(weapon_name)
+                   % power
+                   % shield
+                   % damage);
+    else
+        return str(FlexibleFormat(template_str)
+                   % ShipPartLink(weapon_name)
+                   % power
+                   % UserString("ENC_COMBAT_SHIELD_PIERCED")
+                   % damage);
 }
 
 boost::optional<int> WeaponFireEvent::PrincipalFaction(int viewing_empire_id) const {
@@ -669,6 +658,7 @@ void WeaponFireEvent::serialize(Archive& ar, const unsigned int version) {
        & BOOST_SERIALIZATION_NVP(power)
        & BOOST_SERIALIZATION_NVP(shield)
        & BOOST_SERIALIZATION_NVP(damage)
+       & BOOST_SERIALIZATION_NVP(target_owner_id)
        & BOOST_SERIALIZATION_NVP(attacker_owner_id);
 
     if (version < 3) {
@@ -715,7 +705,7 @@ std::string IncapacitationEvent::DebugString() const {
 
 
 std::string IncapacitationEvent::CombatLogDescription(int viewing_empire_id) const {
-    TemporaryPtr<const UniverseObject> object = GetUniverseObject(object_id);
+    std::shared_ptr<const UniverseObject> object = GetUniverseObject(object_id);
     std::string template_str, object_str;
     int owner_id = object_owner_id;
 
@@ -740,7 +730,9 @@ std::string IncapacitationEvent::CombatLogDescription(int viewing_empire_id) con
     if (const Empire* owner = GetEmpire(owner_id))
         owner_string += owner->Name() + " ";
 
-    return str(FlexibleFormat(template_str) % owner_string % object_str);
+    std::string object_link = FighterOrPublicNameLink(viewing_empire_id, object_id, object_owner_id);
+
+    return str(FlexibleFormat(template_str) % owner_string % object_link);
 }
 
 boost::optional<int> IncapacitationEvent::PrincipalFaction(int viewing_empire_id) const {
@@ -770,73 +762,106 @@ void IncapacitationEvent::serialize<freeorion_xml_oarchive>(freeorion_xml_oarchi
 template
 void IncapacitationEvent::serialize<freeorion_xml_iarchive>(freeorion_xml_iarchive& ar, const unsigned int version);
 
+
 //////////////////////////////////////////
-///////// FighterAttackedEvent ////////
+///////// FightersAttackFightersEvent ////
 //////////////////////////////////////////
-FighterAttackedEvent::FighterAttackedEvent() :
+
+FightersAttackFightersEvent::FightersAttackFightersEvent() :
     bout(-1),
-    round(-1),
-    attacker_owner_empire_id(ALL_EMPIRES),
-    attacked_by_object_id(INVALID_OBJECT_ID),
-    attacked_owner_id(ALL_EMPIRES)
+    events()
 {}
 
-FighterAttackedEvent::FighterAttackedEvent(int bout_, int round_, int attacked_by_object_id_,
-                                                 int attacker_owner_empire_id_, int attacked_owner_id_) :
+FightersAttackFightersEvent::FightersAttackFightersEvent(int bout_) :
     bout(bout_),
-    round(round_),
-    attacker_owner_empire_id(attacker_owner_empire_id_),
-    attacked_by_object_id(attacked_by_object_id_),
-    attacked_owner_id(attacked_owner_id_)
+    events()
 {}
 
-std::string FighterAttackedEvent::DebugString() const {
+void FightersAttackFightersEvent::AddEvent(int attacker_empire_, int target_empire_)
+{ events[{attacker_empire_, target_empire_}] += 1; }
+
+std::string FightersAttackFightersEvent::DebugString() const {
     std::stringstream ss;
-    ss << "rnd: " << round << " : "
-       << attacked_by_object_id << " -> (Fighter of Empire " << attacked_owner_id << ")"
-       << " by object owned by empire " << attacker_owner_empire_id
-       << " at bout " << bout << " round " << round;
+    ss << "FightersAttackFightersEvent: ";
+    for (const auto& index_and_event: events) {
+        ss << index_and_event.second << " repeated fighters from empire " << index_and_event.first.first
+           << " attacking fighters from empire " << index_and_event.first.second << ", ";
+    }
     return ss.str();
 }
 
-std::string FighterAttackedEvent::CombatLogDescription(int viewing_empire_id) const {
-    std::string attacked_by = FighterOrPublicNameLink(viewing_empire_id, attacked_by_object_id, attacker_owner_empire_id);
-    std::string empire_coloured_attacked_fighter = EmpireColorWrappedText(attacked_owner_id, UserString("OBJ_FIGHTER"));
+std::string FightersAttackFightersEvent::CombatLogDescription(int viewing_empire_id) const {
+    if (events.empty())
+        return "";
 
-    const std::string& template_str = UserString("ENC_COMBAT_ATTACK_SIMPLE_STR");
+    const auto& events_to_show = events;
+    auto num_events_remaining = events.size();
+    std::stringstream ss;
 
-    return str(FlexibleFormat(template_str)
-                                % attacked_by
-                                % empire_coloured_attacked_fighter);
+    // Use show_events_for_empire to show events in this order: viewing empire, ALL_EMPIRES and
+    // then the remainder.
+    auto show_events_for_empire =
+        [&ss, &num_events_remaining, &events_to_show, &viewing_empire_id]
+        (boost::optional<int> show_attacker) {
+            int attacker_empire;
+            int target_empire;
+            for (const auto& index_and_event : events_to_show) {
+                std::tie(attacker_empire, target_empire) = index_and_event.first;
+
+                // Skip if this is not the particular attacker requested
+                if (show_attacker && *show_attacker != attacker_empire)
+                    continue;
+
+                // Skip if no particular attacker was requested and this empire is the viewing
+                // empire or ALL_EMPIRES
+                if (!show_attacker && (attacker_empire == viewing_empire_id || attacker_empire == ALL_EMPIRES))
+                    continue;
+
+                auto count = std::to_string(index_and_event.second);
+                const auto&& attacker_link = FighterOrPublicNameLink(
+                    viewing_empire_id, INVALID_OBJECT_ID, attacker_empire);
+                const auto&& target_link = FighterOrPublicNameLink(
+                    viewing_empire_id, INVALID_OBJECT_ID, target_empire);
+                const std::string& template_str = UserString("ENC_COMBAT_ATTACK_REPEATED_STR");
+
+                ss << str(FlexibleFormat(template_str) % count % attacker_link % target_link);
+                if (--num_events_remaining > 0)
+                    ss << "\n";
+            }
+        };
+
+    // Sort the events by viewing empire, then ALL_EMPIRES and then other empires.
+    show_events_for_empire(viewing_empire_id);
+    show_events_for_empire(ALL_EMPIRES);
+    show_events_for_empire(boost::none);
+
+    return ss.str();
 }
 
-boost::optional<int> FighterAttackedEvent::PrincipalFaction(int viewing_empire_id) const {
-    return attacker_owner_empire_id;
-}
 
 template <class Archive>
-void FighterAttackedEvent::serialize (Archive& ar, const unsigned int version) {
+void FightersAttackFightersEvent::serialize(Archive& ar, const unsigned int version) {
     ar & BOOST_SERIALIZATION_BASE_OBJECT_NVP(CombatEvent);
     ar & BOOST_SERIALIZATION_NVP(bout)
-       & BOOST_SERIALIZATION_NVP(round)
-       & BOOST_SERIALIZATION_NVP(attacker_owner_empire_id)
-       & BOOST_SERIALIZATION_NVP(attacked_by_object_id)
-       & BOOST_SERIALIZATION_NVP(attacked_owner_id);
+       & BOOST_SERIALIZATION_NVP(events);
 }
 
-BOOST_CLASS_EXPORT(FighterAttackedEvent)
+BOOST_CLASS_VERSION(FightersAttackFightersEvent, 4)
+BOOST_CLASS_EXPORT(FightersAttackFightersEvent)
 
 template
-void FighterAttackedEvent::serialize<freeorion_bin_oarchive>(freeorion_bin_oarchive& ar, const unsigned int version);
+void FightersAttackFightersEvent::serialize<freeorion_bin_oarchive>(freeorion_bin_oarchive& ar, const unsigned int version);
 
 template
-void FighterAttackedEvent::serialize<freeorion_bin_iarchive>(freeorion_bin_iarchive& ar, const unsigned int version);
+void FightersAttackFightersEvent::serialize<freeorion_bin_iarchive>(freeorion_bin_iarchive& ar, const unsigned int version);
 
 template
-void FighterAttackedEvent::serialize<freeorion_xml_oarchive>(freeorion_xml_oarchive& ar, const unsigned int version);
+void FightersAttackFightersEvent::serialize<freeorion_xml_oarchive>(freeorion_xml_oarchive& ar, const unsigned int version);
 
 template
-void FighterAttackedEvent::serialize<freeorion_xml_iarchive>(freeorion_xml_iarchive& ar, const unsigned int version);
+void FightersAttackFightersEvent::serialize<freeorion_xml_iarchive>(freeorion_xml_iarchive& ar, const unsigned int version);
+
+
 
 //////////////////////////////////////////
 /////////// FighterLaunchEvent ///////////
@@ -908,6 +933,110 @@ void FighterLaunchEvent::serialize<freeorion_xml_iarchive>(freeorion_xml_iarchiv
 
 
 
+//////////////////////////////////////////
+///////// FightersDestroyedEvent ////
+//////////////////////////////////////////
+
+FightersDestroyedEvent::FightersDestroyedEvent() :
+    bout(-1),
+    events()
+{}
+
+FightersDestroyedEvent::FightersDestroyedEvent(int bout_) :
+    bout(bout_),
+    events()
+{}
+
+void FightersDestroyedEvent::AddEvent(int target_empire_)
+{ events[target_empire_] += 1; }
+
+std::string FightersDestroyedEvent::DebugString() const {
+    std::stringstream ss;
+    ss << "FightersDestroyedEvent: ";
+    for (const auto& index_and_event : events) {
+        ss << index_and_event.second << " repeated fighters from empire "
+           << index_and_event.first << " destroyed.";
+    }
+    return ss.str();
+}
+
+std::string FightersDestroyedEvent::CombatLogDescription(int viewing_empire_id) const {
+    if (events.empty())
+        return "";
+
+    const auto& events_to_show = events;
+    auto num_events_remaining = events.size();
+    std::stringstream ss;
+
+    // Use show_events_for_empire to show events in this order: viewing empire,
+    // ALL_EMPIRES and then the remainder.
+    auto show_events_for_empire =
+        [&ss, &num_events_remaining, &events_to_show, &viewing_empire_id]
+        (boost::optional<int> show_empire_id) {
+            int count;
+            int target_empire_id;
+            for (const auto& index_and_event : events_to_show) {
+                std::tie(target_empire_id, count) = index_and_event;
+
+                // Skip if this is not the particular attacker requested
+                if (show_empire_id && *show_empire_id != target_empire_id)
+                    continue;
+
+                // Skip if no particular empire was requested and this empire is the viewing
+                // empire or ALL_EMPIRES
+                if (!show_empire_id && (target_empire_id == viewing_empire_id || target_empire_id == ALL_EMPIRES))
+                    continue;
+
+                auto count_str = std::to_string(index_and_event.second);
+                auto target_empire_link = EmpireLink(target_empire_id);
+                const auto&& target_link = FighterOrPublicNameLink(
+                    viewing_empire_id, INVALID_OBJECT_ID, target_empire_id);
+
+                if (count == 1) {
+                    const std::string& template_str = UserString("ENC_COMBAT_FIGHTER_INCAPACITATED_STR");
+                    ss << str(FlexibleFormat(template_str) % target_empire_link % target_link);
+                }else {
+                    const std::string& template_str = UserString("ENC_COMBAT_FIGHTER_INCAPACITATED_REPEATED_STR");
+                    ss << str(FlexibleFormat(template_str) % count_str % target_empire_link % target_link);
+                }
+                if (--num_events_remaining > 0)
+                    ss << "\n";
+            }
+        };
+
+    // Sort the events by viewing empire, then ALL_EMPIRES and then other empires.
+    show_events_for_empire(viewing_empire_id);
+    show_events_for_empire(ALL_EMPIRES);
+    show_events_for_empire(boost::none);
+
+    return ss.str();
+}
+
+
+template <class Archive>
+void FightersDestroyedEvent::serialize(Archive& ar, const unsigned int version) {
+    ar & BOOST_SERIALIZATION_BASE_OBJECT_NVP(CombatEvent);
+    ar & BOOST_SERIALIZATION_NVP(bout)
+       & BOOST_SERIALIZATION_NVP(events);
+}
+
+BOOST_CLASS_VERSION(FightersDestroyedEvent, 4)
+BOOST_CLASS_EXPORT(FightersDestroyedEvent)
+
+template
+void FightersDestroyedEvent::serialize<freeorion_bin_oarchive>(freeorion_bin_oarchive& ar, const unsigned int version);
+
+template
+void FightersDestroyedEvent::serialize<freeorion_bin_iarchive>(freeorion_bin_iarchive& ar, const unsigned int version);
+
+template
+void FightersDestroyedEvent::serialize<freeorion_xml_oarchive>(freeorion_xml_oarchive& ar, const unsigned int version);
+
+template
+void FightersDestroyedEvent::serialize<freeorion_xml_iarchive>(freeorion_xml_iarchive& ar, const unsigned int version);
+
+
+
 //////////////////////////////////////////
 ///////// WeaponsPlatformEvent /////////////
 //////////////////////////////////////////
@@ -927,24 +1056,22 @@ WeaponsPlatformEvent::WeaponsPlatformEvent(int bout_, int attacker_id_, int atta
 {}
 
 void WeaponsPlatformEvent::AddEvent(
-    int round_, int target_id_, std::string const & weapon_name_,
+    int round_, int target_id_, int target_owner_id_, std::string const & weapon_name_,
     float power_, float shield_, float damage_) {
     events[target_id_].push_back(
-        boost::make_shared<WeaponFireEvent>(
-            bout, round_, attacker_id, target_id_, weapon_name_, power_, shield_, damage_, attacker_owner_id));
+        std::make_shared<WeaponFireEvent>(
+            bout, round_, attacker_id, target_id_, weapon_name_,
+            std::tie(power_, shield_, damage_),
+            attacker_owner_id, target_owner_id_));
 }
 
 std::string WeaponsPlatformEvent::DebugString() const {
     std::stringstream desc;
     desc << "WeaponsPlatformEvent bout = " << bout << " attacker_id = "
         << attacker_id << " attacker_owner = "<< attacker_owner_id;
-    for (std::map<int, std::vector<WeaponFireEvent::WeaponFireEventPtr> >::const_iterator target_it = events.begin()
-             ; target_it != events.end(); ++target_it)
-    {
-        for (std::vector<WeaponFireEvent::WeaponFireEventPtr>::const_iterator attack_it = target_it->second.begin();
-             attack_it != target_it->second.end(); ++attack_it)
-        {
-            desc << std::endl << (*attack_it)->DebugString();
+    for (const std::map<int, std::vector<WeaponFireEvent::WeaponFireEventPtr>>::value_type& target : events) {
+        for (const WeaponFireEvent::WeaponFireEventPtr attack : target.second) {
+            desc << std::endl << attack->DebugString();
         }
     }
     return desc.str();
@@ -954,22 +1081,29 @@ std::string WeaponsPlatformEvent::CombatLogDescription(int viewing_empire_id) co
     if (events.empty())
         return "";
 
-    std::map<int, double> damaged;
-    std::set<int> did_not_damage;
+    std::vector<std::string> damaged_target_links;
+    std::vector<std::string> undamaged_target_links;
+
+    for (const std::map<int, std::vector<WeaponFireEvent::WeaponFireEventPtr>>::value_type& target : events) {
+        if (target.second.empty())
+            continue;
+
+        const WeaponFireEvent::WeaponFireEventPtr& fire_event(*target.second.begin());
+        std::string target_public_name(
+            FighterOrPublicNameLink(viewing_empire_id, target.first, fire_event->target_owner_id));
 
-    for (std::map<int, std::vector<WeaponFireEvent::WeaponFireEventPtr> >::const_iterator target_it = events.begin()
-             ; target_it != events.end(); ++target_it)
-    {
         double damage = 0.0f;
-        for (std::vector<WeaponFireEvent::WeaponFireEventPtr>::const_iterator attack_it = target_it->second.begin();
-             attack_it != target_it->second.end(); ++attack_it)
-        {
-            damage += (*attack_it)->damage;
+        for (WeaponFireEvent::WeaponFireEventPtr attack_it : target.second) {
+            damage += attack_it->damage;
         }
+
         if (damage <= 0.0f) {
-            did_not_damage.insert(target_it->first);
+            undamaged_target_links.push_back(target_public_name);
+
         } else {
-            damaged[target_it->first] = damage;
+            damaged_target_links.push_back(
+                str(FlexibleFormat(UserString("ENC_COMBAT_PLATFORM_TARGET_AND_DAMAGE"))
+                    % target_public_name % damage));
         }
     }
 
@@ -978,32 +1112,18 @@ std::string WeaponsPlatformEvent::CombatLogDescription(int viewing_empire_id) co
     const std::vector<std::string> attacker_link(
         1, FighterOrPublicNameLink(viewing_empire_id, attacker_id, attacker_owner_id));
 
-    if (!damaged.empty() ) {
-        std::vector<std::string> target_links;
-        for (std::map<int, double>::iterator target_it = damaged.begin();
-             target_it != damaged.end(); ++target_it) {
-            target_links.push_back(
-                str(FlexibleFormat(UserString("ENC_COMBAT_PLATFORM_TARGET_AND_DAMAGE"))
-                    % PublicNameLink(viewing_empire_id, target_it->first) % target_it->second));
-        }
+    if (!damaged_target_links.empty() ) {
+        desc += FlexibleFormatList(attacker_link, damaged_target_links,
+                                   UserString("ENC_COMBAT_PLATFORM_DAMAGE_MANY_EVENTS"),
+                                   UserString("ENC_COMBAT_PLATFORM_DAMAGE_1_EVENTS")).str();
 
-        desc += FlexibleFormatList(attacker_link, target_links
-                                   , UserString("ENC_COMBAT_PLATFORM_DAMAGE_MANY_EVENTS")
-                                   , UserString("ENC_COMBAT_PLATFORM_DAMAGE_1_EVENTS")).str();
-
-        if (!did_not_damage.empty())
+        if (!undamaged_target_links.empty())
             desc += "\n";
     }
-    if (!did_not_damage.empty()) {
-        std::vector<std::string> target_links;
-        for (std::set<int>::iterator target_it = did_not_damage.begin();
-             target_it != did_not_damage.end(); ++target_it) {
-            target_links.push_back( PublicNameLink(viewing_empire_id, *target_it));
-        }
-
-        desc += FlexibleFormatList(attacker_link, target_links
-                                   , UserString("ENC_COMBAT_PLATFORM_NO_DAMAGE_MANY_EVENTS")
-                                   , UserString("ENC_COMBAT_PLATFORM_NO_DAMAGE_1_EVENTS")).str();
+    if (!undamaged_target_links.empty()) {
+        desc += FlexibleFormatList(attacker_link, undamaged_target_links,
+                                   UserString("ENC_COMBAT_PLATFORM_NO_DAMAGE_MANY_EVENTS"),
+                                   UserString("ENC_COMBAT_PLATFORM_NO_DAMAGE_1_EVENTS")).str();
     }
     return desc;
 }
@@ -1014,12 +1134,9 @@ bool WeaponsPlatformEvent::AreSubEventsEmpty(int viewing_empire_id) const {
 
 std::vector<ConstCombatEventPtr> WeaponsPlatformEvent::SubEvents(int viewing_empire_id) const {
     std::vector<ConstCombatEventPtr> all_events;
-    for (std::map<int, std::vector<WeaponFireEvent::WeaponFireEventPtr> >::const_iterator target_it = events.begin();
-         target_it != events.end(); ++target_it) {
-
-        for (std::vector<WeaponFireEvent::WeaponFireEventPtr>::const_iterator event_it = target_it->second.begin();
-             event_it != target_it->second.end(); ++event_it){
-            all_events.push_back(boost::dynamic_pointer_cast<CombatEvent>(*event_it));
+    for (const std::map<int, std::vector<WeaponFireEvent::WeaponFireEventPtr>>::value_type& target : events) {
+        for (WeaponFireEvent::WeaponFireEventPtr event : target.second) {
+            all_events.push_back(std::dynamic_pointer_cast<CombatEvent>(event));
         }
     }
     return all_events;
