@@ -1,16 +1,21 @@
 #ifndef COMBATEVENTS_H
 #define COMBATEVENTS_H
 
+
+#include <map>
+#include <memory>
 #include <set>
-#include <boost/shared_ptr.hpp>
+
+#include <tuple>
 #include <boost/serialization/access.hpp>
 #include <boost/serialization/export.hpp>
 
 #include "../util/Export.h"
-#include "../universe/Enums.h"
+#include "../universe/EnumsFwd.h"
 
 #include "CombatEvent.h"
 
+
 /** \file
  * Contains all combat event implementation declarations.
  */
@@ -18,11 +23,15 @@
 /// Generated when a new bout begins in the battle
 struct FO_COMMON_API BoutBeginEvent : public CombatEvent {
     BoutBeginEvent();
-    BoutBeginEvent(int bout);
-    virtual ~BoutBeginEvent() {}
 
-    virtual std::string DebugString() const;
-    virtual std::string CombatLogDescription(int viewing_empire_id) const;
+    BoutBeginEvent(int bout);
+
+    virtual ~BoutBeginEvent()
+    {}
+
+    std::string DebugString() const override;
+
+    std::string CombatLogDescription(int viewing_empire_id) const override;
 
     int bout;
 
@@ -35,23 +44,30 @@ private:
 /** BoutEvent describes all the events that happen in one bout of combat.
    It may have some sub-events and the sub-events will be ordered.*/
 struct FO_COMMON_API BoutEvent : public CombatEvent {
-    typedef boost::shared_ptr<BoutEvent> BoutEventPtr;
+    typedef std::shared_ptr<BoutEvent> BoutEventPtr;
 
     BoutEvent();
+
     BoutEvent(int bout);
 
-    virtual ~BoutEvent() {}
+    virtual ~BoutEvent()
+    {}
 
-    void AddEvent(CombatEventPtr & event);
+    void AddEvent(const CombatEventPtr& event);
+
+    std::string DebugString() const override;
+
+    std::string CombatLogDescription(int viewing_empire_id) const override;
+
+    std::vector<ConstCombatEventPtr> SubEvents(int viewing_empire_id) const override;
+
+    bool AreSubEventsEmpty(int viewing_empire_id) const override
+    { return events.empty(); }
 
-    virtual std::string DebugString() const;
-    virtual std::string CombatLogDescription(int viewing_empire_id) const;
-    virtual std::vector<ConstCombatEventPtr> SubEvents(int viewing_empire_id) const;
-    virtual bool AreSubEventsEmpty(int viewing_empire_id) const
-    { return events.empty();}
     virtual bool AreSubEventsOrdered() const
     { return true; }
-    virtual bool FlattenSubEvents() const
+
+    bool FlattenSubEvents() const override
     { return true; }
 
 private:
@@ -68,20 +84,25 @@ private:
    It may have some sub-events and the sub-events will be shown in
    viewing empire first followed by ALL_EMPIRES and then other empires.*/
 struct FO_COMMON_API SimultaneousEvents : public CombatEvent {
-    typedef boost::shared_ptr<SimultaneousEvents> SimultaneousEventsPtr;
+    typedef std::shared_ptr<SimultaneousEvents> SimultaneousEventsPtr;
 
     SimultaneousEvents();
 
-    virtual ~SimultaneousEvents() {}
+    virtual ~SimultaneousEvents()
+    {}
 
-    void AddEvent(CombatEventPtr &event);
+    void AddEvent(const CombatEventPtr& event);
 
-    virtual std::string DebugString() const;
-    virtual std::string CombatLogDescription(int viewing_empire_id) const;
-    virtual std::vector<ConstCombatEventPtr> SubEvents(int viewing_empire_id) const;
-    virtual bool AreSubEventsEmpty(int viewing_empire_id) const
-    { return events.empty();}
-    virtual bool FlattenSubEvents() const
+    std::string DebugString() const override;
+
+    std::string CombatLogDescription(int viewing_empire_id) const override;
+
+    std::vector<ConstCombatEventPtr> SubEvents(int viewing_empire_id) const override;
+
+    bool AreSubEventsEmpty(int viewing_empire_id) const override
+    { return events.empty(); }
+
+    virtual bool FlattenSubEvents() const override
     { return true; }
 
 protected:
@@ -94,13 +115,13 @@ protected:
 
 
 typedef SimultaneousEvents AttacksEvent;
-typedef boost::shared_ptr<AttacksEvent> AttacksEventPtr;
+typedef std::shared_ptr<AttacksEvent> AttacksEventPtr;
 
 typedef SimultaneousEvents IncapacitationsEvent;
-typedef boost::shared_ptr<IncapacitationsEvent> IncapacitationsEventPtr;
+typedef std::shared_ptr<IncapacitationsEvent> IncapacitationsEventPtr;
 
 typedef SimultaneousEvents FighterLaunchesEvent;
-typedef boost::shared_ptr<FighterLaunchesEvent> FighterLaunchesEventPtr;
+typedef std::shared_ptr<FighterLaunchesEvent> FighterLaunchesEventPtr;
 
 
 /** InitialStealthEvent describes the initially stealthy combatants.
@@ -109,15 +130,18 @@ typedef boost::shared_ptr<FighterLaunchesEvent> FighterLaunchesEventPtr;
 struct FO_COMMON_API InitialStealthEvent : public CombatEvent {
 
     /// a map of attacker empire id -> defender empire id -> set of (attacker ids, visibility)
-    typedef std::map<int, std::map< int, std::set< std::pair<int, Visibility> > > > StealthInvisbleMap;
+    typedef std::map<int, std::map<int, std::set<std::pair<int, Visibility>>>> StealthInvisbleMap;
 
     InitialStealthEvent();
+
     InitialStealthEvent(const StealthInvisbleMap &);
 
-    virtual ~InitialStealthEvent() {}
+    virtual ~InitialStealthEvent()
+    {}
 
-    virtual std::string DebugString() const;
-    virtual std::string CombatLogDescription(int viewing_empire_id) const;
+    std::string DebugString() const override;
+
+    std::string CombatLogDescription(int viewing_empire_id) const override;
 
 private:
 
@@ -132,31 +156,39 @@ private:
  At this time always decloaking.*/
 struct FO_COMMON_API StealthChangeEvent : public CombatEvent {
     StealthChangeEvent();
+
     StealthChangeEvent(int bout);
 
-    virtual ~StealthChangeEvent() {}
+    virtual ~StealthChangeEvent()
+    {}
+
+    std::string DebugString() const override;
+
+    std::string CombatLogDescription(int viewing_empire_id) const override;
+
+    std::vector<ConstCombatEventPtr> SubEvents(int viewing_empire_id) const override;
+
+    bool AreSubEventsEmpty(int viewing_empire_id) const override;
 
     void AddEvent(int attacker_id_, int target_id_, int attacker_empire_, int target_empire_, Visibility new_visibility_);
 
-    virtual std::string DebugString() const;
-    virtual std::string CombatLogDescription(int viewing_empire_id) const;
-    virtual std::vector<ConstCombatEventPtr> SubEvents(int viewing_empire_id) const;
-    virtual bool AreSubEventsEmpty(int viewing_empire_id) const;
-
     struct StealthChangeEventDetail;
-    typedef boost::shared_ptr<StealthChangeEventDetail> StealthChangeEventDetailPtr;
-    typedef boost::shared_ptr<const StealthChangeEventDetail> ConstStealthChangeEventDetailPtr;
+    typedef std::shared_ptr<StealthChangeEventDetail> StealthChangeEventDetailPtr;
+    typedef std::shared_ptr<const StealthChangeEventDetail> ConstStealthChangeEventDetailPtr;
     struct StealthChangeEventDetail : public CombatEvent {
+        StealthChangeEventDetail();
+
+        StealthChangeEventDetail(int attacker_id_, int target_id_, int attacker_empire_, int target_empire_, Visibility new_visibility_);
+
+        std::string DebugString() const override;
+
+        std::string CombatLogDescription(int viewing_empire_id) const override;
+
         int attacker_id;
         int target_id;
         int attacker_empire_id;
         int target_empire_id;
         Visibility visibility;
-        StealthChangeEventDetail();
-        StealthChangeEventDetail(int attacker_id_, int target_id_, int attacker_empire_, int target_empire_, Visibility new_visibility_);
-
-        virtual std::string DebugString() const;
-        virtual std::string CombatLogDescription(int viewing_empire_id) const;
 
         friend class boost::serialization::access;
         template <class Archive>
@@ -164,9 +196,9 @@ struct FO_COMMON_API StealthChangeEvent : public CombatEvent {
     };
 
 private:
-    int     bout;
+    int bout;
 
-    std::map<int, std::vector<StealthChangeEventDetailPtr> > events;
+    std::map<int, std::vector<StealthChangeEventDetailPtr>> events;
 
     friend class boost::serialization::access;
     template <class Archive>
@@ -175,31 +207,48 @@ private:
 
 /// An event that describes a single attack by one object or fighter against another object or fighter
 struct FO_COMMON_API WeaponFireEvent : public CombatEvent {
-    typedef boost::shared_ptr<WeaponFireEvent> WeaponFireEventPtr;
-    typedef boost::shared_ptr<const WeaponFireEvent> ConstWeaponFireEventPtr;
+    typedef std::shared_ptr<WeaponFireEvent> WeaponFireEventPtr;
+    typedef std::shared_ptr<const WeaponFireEvent> ConstWeaponFireEventPtr;
 
     WeaponFireEvent();
-    WeaponFireEvent(int bout, int round, int attacker_id, int target_id, const std::string &weapon_name
-                    , float power_, float shield_, float damage_, int attacker_owner_id_);
 
-    virtual ~WeaponFireEvent() {}
+    /** WeaponFireEvent encodes a single attack in \p bout, \p round by \p
+        attacker_id owned by \p attacker_owner_id on \p target_id with \p
+        weapon_name of \p power against \p shield doing \p damage.
 
-    virtual std::string DebugString() const;
-    virtual std::string CombatLogDescription(int viewing_empire_id) const;
-    virtual std::string CombatLogDetails(int viewing_empire_id) const;
-    virtual bool AreDetailsEmpty(int viewing_empire_id) const
+        If \p shield is negative that implies the weapon is shield piercing.
+
+        The use of tuple in the constructor is to keep the number of parameters below 10 which
+        is the maximum that some compilers that emulate variadic templates support.
+     */
+    WeaponFireEvent(int bout, int round, int attacker_id, int target_id, const std::string &weapon_name,
+                    const std::tuple<float, float, float> &power_shield_damage,
+                    int attacker_owner_id_, int target_owner_id_);
+
+    virtual ~WeaponFireEvent()
+    {}
+
+    std::string DebugString() const override;
+
+    std::string CombatLogDescription(int viewing_empire_id) const override;
+
+    std::string CombatLogDetails(int viewing_empire_id) const override;
+
+    bool AreDetailsEmpty(int viewing_empire_id) const override
     { return false; }
-    virtual boost::optional<int> PrincipalFaction(int viewing_empire_id) const;
 
-    int     bout;
-    int     round;
-    int     attacker_id;
-    int     target_id;
+    boost::optional<int> PrincipalFaction(int viewing_empire_id) const override;
+
+    int bout;
+    int round;
+    int attacker_id;
+    int target_id;
     std::string weapon_name;
-    float   power;
-    float   shield;
-    float   damage;
-    int     attacker_owner_id;
+    float power;
+    float shield;
+    float damage;
+    int attacker_owner_id;
+    int target_owner_id;
 
 private:
     friend class boost::serialization::access;
@@ -211,12 +260,17 @@ private:
 /// eg. a ship is destroyed or a planet loses all defence
 struct FO_COMMON_API IncapacitationEvent : public CombatEvent {
     IncapacitationEvent();
-    IncapacitationEvent(int bout_, int object_id_, int object_owner_id_);
-    virtual ~IncapacitationEvent() {}
 
-    virtual std::string DebugString() const;
-    virtual std::string CombatLogDescription(int viewing_empire_id) const;
-    virtual boost::optional<int> PrincipalFaction(int viewing_empire_id) const;
+    IncapacitationEvent(int bout_, int object_id_, int object_owner_id_);
+
+    virtual ~IncapacitationEvent()
+    {}
+
+    std::string DebugString() const override;
+
+    std::string CombatLogDescription(int viewing_empire_id) const override;
+
+    boost::optional<int> PrincipalFaction(int viewing_empire_id) const override;
 
     int bout;
     int object_id;
@@ -228,25 +282,28 @@ private:
     void serialize(Archive& ar, const unsigned int version);
 };
 
-/// Created when an fighter is destroyed
-struct FO_COMMON_API FighterAttackedEvent : public CombatEvent {
-    typedef boost::shared_ptr<FighterAttackedEvent> FighterAttackedEventPtr;
 
-    FighterAttackedEvent();
-    FighterAttackedEvent(int bout_, int round_, int attacked_by_object_id_, int attacker_owner_empire_id_, int attacked_owner_id_);
-    virtual ~FighterAttackedEvent() {}
+/** FightersAttackFightersEvent aggregates all the fighter on fighter combat for one bout.*/
+struct FO_COMMON_API FightersAttackFightersEvent : public CombatEvent {
+    FightersAttackFightersEvent();
 
-    virtual std::string DebugString() const;
-    virtual std::string CombatLogDescription(int viewing_empire_id) const;
-    virtual boost::optional<int> PrincipalFaction(int viewing_empire_id) const;
+    FightersAttackFightersEvent(int bout);
 
-    int bout;
-    int round;
-    int attacker_owner_empire_id;   // may be ALL_EMPIRES if attacking fighter was owned by no empire
-    int attacked_by_object_id;     // may be INVALID_OBJECT_ID if destroyed by another fighter
-    int attacked_owner_id;         // may be ALL_EMPIRES if destroyed fighter was owned by no empire
+    virtual ~FightersAttackFightersEvent()
+    {}
+
+    std::string DebugString() const override;
+
+    std::string CombatLogDescription(int viewing_empire_id) const override;
+
+    void AddEvent(int attacker_empire_, int target_empire_);
 
 private:
+    int bout;
+
+    // Store the number of each of the identical fighter combat events.
+    std::map<std::pair<int, int>, unsigned int> events;
+
     friend class boost::serialization::access;
     template <class Archive>
     void serialize(Archive& ar, const unsigned int version);
@@ -254,15 +311,20 @@ private:
 
 /// Created when an fighter is launched
 struct FO_COMMON_API FighterLaunchEvent : public CombatEvent {
-    typedef boost::shared_ptr<FighterLaunchEvent> FighterLaunchEventPtr;
+    typedef std::shared_ptr<FighterLaunchEvent> FighterLaunchEventPtr;
 
     FighterLaunchEvent();
-    FighterLaunchEvent(int bout_, int launched_from_id_, int fighter_owner_empire_id_, int number_launched_);
-    virtual ~FighterLaunchEvent() {}
 
-    virtual std::string DebugString() const;
-    virtual std::string CombatLogDescription(int viewing_empire_id) const;
-    virtual boost::optional<int> PrincipalFaction(int viewing_empire_id) const;
+    FighterLaunchEvent(int bout_, int launched_from_id_, int fighter_owner_empire_id_, int number_launched_);
+
+    virtual ~FighterLaunchEvent()
+    {}
+
+    std::string DebugString() const override;
+
+    std::string CombatLogDescription(int viewing_empire_id) const override;
+
+    boost::optional<int> PrincipalFaction(int viewing_empire_id) const override;
 
     int bout;
     int fighter_owner_empire_id;    // may be ALL_EMPIRE if fighter was owned by no empire
@@ -275,32 +337,64 @@ private:
     void serialize(Archive& ar, const unsigned int version);
 };
 
+/** FightersDestroyedEvent aggregates all the fighters destroyed during one combat bout.*/
+struct FO_COMMON_API FightersDestroyedEvent : public CombatEvent {
+    FightersDestroyedEvent();
+
+    FightersDestroyedEvent(int bout);
+
+    virtual ~FightersDestroyedEvent()
+    {}
+
+    std::string DebugString() const override;
+
+    std::string CombatLogDescription(int viewing_empire_id) const override;
+
+    void AddEvent(int target_empire_);
+
+private:
+    int bout;
+
+    // Store the number of each of the identical fighter combat events.
+    std::map<int, unsigned int> events;
+
+    friend class boost::serialization::access;
+    template <class Archive>
+    void serialize(Archive& ar, const unsigned int version);
+};
+
 /** WeaponsPlatformEvent describes a ship or planet with zero or more weapons firing its weapons in combat.
    It may have some WeaponFire sub-events.*/
 struct FO_COMMON_API WeaponsPlatformEvent : public CombatEvent {
-    typedef boost::shared_ptr<WeaponsPlatformEvent> WeaponsPlatformEventPtr;
-    typedef boost::shared_ptr<const WeaponsPlatformEvent> ConstWeaponsPlatformEventPtr;
+    typedef std::shared_ptr<WeaponsPlatformEvent> WeaponsPlatformEventPtr;
+    typedef std::shared_ptr<const WeaponsPlatformEvent> ConstWeaponsPlatformEventPtr;
 
     WeaponsPlatformEvent();
+
     WeaponsPlatformEvent(int bout, int attacker_id, int attacker_owner_id_);
 
-    virtual ~WeaponsPlatformEvent() {}
+    virtual ~WeaponsPlatformEvent()
+    {}
 
-    void AddEvent(int round, int target_id, std::string const & weapon_name_,
+    void AddEvent(int round, int target_id, int target_owner_id_, std::string const & weapon_name_,
                   float power_, float shield_, float damage_);
 
-    virtual std::string DebugString() const;
-    virtual std::string CombatLogDescription(int viewing_empire_id) const;
-    virtual std::vector<ConstCombatEventPtr> SubEvents(int viewing_empire_id) const;
-    virtual bool AreSubEventsEmpty(int viewing_empire_id) const;
-    virtual boost::optional<int> PrincipalFaction(int viewing_empire_id) const;
+    std::string DebugString() const override;
+
+    std::string CombatLogDescription(int viewing_empire_id) const override;
+
+    std::vector<ConstCombatEventPtr> SubEvents(int viewing_empire_id) const override;
+
+    bool AreSubEventsEmpty(int viewing_empire_id) const override;
+
+    boost::optional<int> PrincipalFaction(int viewing_empire_id) const override;
 
     int bout;
     int attacker_id;
     int attacker_owner_id;
 
 private:
-    std::map<int, std::vector<WeaponFireEvent::WeaponFireEventPtr> > events;
+    std::map<int, std::vector<WeaponFireEvent::WeaponFireEventPtr>> events;
 
     friend class boost::serialization::access;
     template <class Archive>
