@@ -1,3 +1,5 @@
+import sys
+
 import freeOrionAIInterface as fo  # pylint: disable=import-error
 
 from fleet_orders import OrderMove, OrderOutpost, OrderColonize, OrderMilitary, OrderInvade, OrderDefend
@@ -7,9 +9,10 @@ import FreeOrionAI as foAI
 import MoveUtilsAI
 import MilitaryAI
 import InvasionAI
-import PlanetUtilsAI
-from universe_object import System, Fleet, Planet
+import CombatRatingsAI
+from universe_object import UniverseObject, System, Fleet, Planet
 from EnumsAI import MissionType
+from AIDependencies import INVALID_ID
 
 ORDERS_FOR_MISSION = {
     MissionType.EXPLORATION: OrderMove,
@@ -17,8 +20,9 @@ ORDERS_FOR_MISSION = {
     MissionType.COLONISATION: OrderColonize,
     MissionType.INVASION: OrderInvade,
     MissionType.MILITARY: OrderMilitary,
+    # SECURE is mostly same as MILITARY, but waits for system removal from all targeted system lists
+    # (invasion, colonization, outpost, blockade) before clearing
     MissionType.SECURE: OrderMilitary,
-    # mostly same as MILITARY, but waits for system removal from all targeted system lists (invasion, colonization, outpost, blockade) before clearing
     MissionType.ORBITAL_INVASION: OrderInvade,
     MissionType.ORBITAL_OUTPOST: OrderOutpost,
     MissionType.ORBITAL_DEFENSE: OrderDefend
@@ -29,10 +33,26 @@ COMBAT_MISSION_TYPES = (
     MissionType.SECURE
 )
 
+MERGEABLE_MISSION_TYPES = (
+    MissionType.MILITARY,
+    MissionType.INVASION,
+    MissionType.ORBITAL_INVASION,
+    MissionType.SECURE,
+    MissionType.ORBITAL_DEFENSE,
+)
+
+COMPATIBLE_ROLES_MAP = {
+    MissionType.ORBITAL_DEFENSE: [MissionType.ORBITAL_DEFENSE],
+    MissionType.MILITARY: [MissionType.MILITARY],
+    MissionType.ORBITAL_INVASION: [MissionType.ORBITAL_INVASION],
+    MissionType.INVASION: [MissionType.INVASION],
+}
+
 
 class AIFleetMission(object):
     """
     Stores information about AI mission. Every mission has fleetID and AI targets depending upon AI fleet mission type.
+    :type target: UniverseObject | None
     """
 
     def __init__(self, fleet_id):
@@ -41,75 +61,82 @@ class AIFleetMission(object):
         self.type = None
         self.target = None
 
-    def add_target(self, mission_type, target):
+    def set_target(self, mission_type, target):
+        """
+        Set mission and target for this fleet.
+
+        :type mission_type: MissionType
+        :type target: UniverseObject
+        """
         if self.type == mission_type and self.target == target:
             return
         if self.type or self.target:
-            print "Change mission assignment from %s:%s to %s:%s" % (self.type,
-                                                                     self.target,
-                                                                     mission_type,
-                                                                     target)
+            print "Change mission assignment from %s:%s to %s:%s" % (self.type, self.target, mission_type, target)
         self.type = mission_type
         self.target = target
 
     def clear_target(self):
+        """Clear target and mission for this fleet."""
         self.target = None
         self.type = None
 
     def has_target(self, mission_type, target):
+        """
+        Check if fleet has specified mission_type and target.
+
+        :type mission_type: MissionType
+        :type target: UniverseObject
+        :rtype: bool
+        """
         return self.type == mission_type and self.target == target
 
     def clear_fleet_orders(self):
+        """Clear this fleets orders but do not clear mission and target."""
         self.orders = []
 
     def _get_fleet_order_from_target(self, mission_type, target):
+        """
+        Get a fleet order according to mission type and target.
+
+        :type mission_type: MissionType
+        :type target: UniverseObject
+        :rtype: AIFleetOrder
+        """
         fleet_target = Fleet(self.fleet.id)
         return ORDERS_FOR_MISSION[mission_type](fleet_target, target)
 
-    def check_mergers(self, fleet_id=None, context=""):
-        if fleet_id is None:
-            fleet_id = self.fleet.id
+    def check_mergers(self, context=""):
+        """
+        If possible and reasonable, merge this fleet with others.
 
-        if self.type not in (MissionType.MILITARY,
-                             MissionType.INVASION,
-                             MissionType.ORBITAL_INVASION,
-                             MissionType.SECURE,
-                             MissionType.ORBITAL_DEFENSE,
-                             ):
+        :param context: Context of the function call for logging purposes
+        :type context: str
+        """
+        if self.type not in MERGEABLE_MISSION_TYPES:
             return
         universe = fo.getUniverse()
         empire_id = fo.empireID()
+        fleet_id = self.fleet.id
         main_fleet = universe.getFleet(fleet_id)
         system_id = main_fleet.systemID
-        if system_id == -1:
+        if system_id == INVALID_ID:
             return  # can't merge fleets in middle of starlane
         system_status = foAI.foAIstate.systemStatus[system_id]
         destroyed_list = list(universe.destroyedObjectIDs(empire_id))
-        other_fleets_here = [fid for fid in system_status.get('myFleetsAccessible', []) if fid != fleet_id and fid not in destroyed_list and universe.getFleet(fid).ownedBy(empire_id)]
+        other_fleets_here = [fid for fid in system_status.get('myFleetsAccessible', []) if fid != fleet_id and
+                             fid not in destroyed_list and universe.getFleet(fid).ownedBy(empire_id)]
         if not other_fleets_here:
-            return  # nothing of record to merge with
-
-        if not self.target:
-            m_mt0_id = None
-        else:
-            m_mt0_id = self.target.id
-
-        # TODO consider establishing an AI strategy & tactics planning document for discussing & planning
-        # high level considerations for issues like fleet merger
-        compatible_roles_map = {
-            MissionType.ORBITAL_DEFENSE: [MissionType.ORBITAL_DEFENSE],
-            MissionType.MILITARY: [MissionType.MILITARY],
-            MissionType.ORBITAL_INVASION: [MissionType.ORBITAL_INVASION],
-            MissionType.INVASION: [MissionType.INVASION],
-        }
+            return
 
+        target_id = self.target.id if self.target else None
         main_fleet_role = foAI.foAIstate.get_fleet_role(fleet_id)
         for fid in other_fleets_here:
             fleet_role = foAI.foAIstate.get_fleet_role(fid)
-            if fleet_role not in compatible_roles_map[main_fleet_role]:  # TODO: if fleetRoles such as LongRange start being used, adjust this
-                continue  # will only considering subsuming fleets that have a compatible role
+            if fleet_role not in COMPATIBLE_ROLES_MAP[main_fleet_role]:
+                continue
             fleet = universe.getFleet(fid)
-            if not (fleet and (fleet.systemID == system_id)):
+
+            if not fleet or fleet.systemID != system_id or len(fleet.shipIDs) == 0:
                 continue
             if not (fleet.speed > 0 or main_fleet.speed == 0):  # TODO(Cjkjvfnby) Check this condition
                 continue
@@ -126,26 +153,21 @@ class AIFleetMission(object):
                 do_merge = True
             else:
                 if not self.target and (main_fleet.speed > 0 or fleet.speed == 0):
-                    # print "\t\t\t ** Considering merging fleetA (id: %4d) into fleet (id %d) and former has no targets, will take it. FleetA mission was %s "%(fid, fleetID, fleet_mission)
                     do_merge = True
                 else:
                     target = fleet_mission.target.id if fleet_mission.target else None
-                    if target == m_mt0_id:
-                        print "Military fleet %d has same target as %s fleet %d and will (at least temporarily) be merged into the latter" % (fid, fleet_role, fleet_id)
+                    if target == target_id:
+                        print "Military fleet %d has same target as %s fleet %d. Merging." % (fid, fleet_role, fleet_id)
                         do_merge = True  # TODO: should probably ensure that fleetA has aggression on now
                     elif main_fleet.speed > 0:
                         neighbors = foAI.foAIstate.systemStatus.get(system_id, {}).get('neighbors', [])
-                        if (target == system_id) and m_mt0_id in neighbors:  # consider 'borrowing' for work in neighbor system  # TODO check condition
-                            if self.type in (MissionType.MILITARY,
-                                             MissionType.SECURE):
-                                # continue
-                                if self.type == MissionType.SECURE:  # actually, currently this is probably the only one of all four that should really be possible in this situation
-                                    need_left = 1.5 * sum([sysStat.get('fleetThreat', 0) for sysStat in
-                                                           [foAI.foAIstate.systemStatus.get(neighbor, {}) for neighbor in
-                                                                                                          [nid for nid in foAI.foAIstate.systemStatus.get(system_id, {}).get('neighbors', []) if nid != m_mt0_id]]])
-                                    fb_rating = foAI.foAIstate.get_rating(fid)
-                                    if (need_left < fb_rating.get('overall', 0)) and fb_rating.get('nships', 0) > 1:
-                                        do_merge = True
+                        if target == system_id and target_id in neighbors and self.type == MissionType.SECURE:
+                            # consider 'borrowing' for work in neighbor system  # TODO check condition
+                            need_left = 1.5 * sum(foAI.foAIstate.systemStatus.get(nid, {}).get('fleetThreat', 0)
+                                                  for nid in neighbors if nid != target_id)
+                            fleet_rating = CombatRatingsAI.get_fleet_rating(fid)
+                            if need_left < fleet_rating:
+                                do_merge = True
             if do_merge:
                 FleetUtilsAI.merge_fleet_a_into_b(fid, fleet_id, need_left,
                                                   context="Order %s of mission %s" % (context, self))
@@ -222,18 +244,17 @@ class AIFleetMission(object):
     def _check_retarget_invasion(self):
         """checks if an invasion mission should be retargeted"""
         universe = fo.getUniverse()
-        empire = fo.getEmpire()
         empire_id = fo.empireID()
         fleet_id = self.fleet.id
         fleet = universe.getFleet(fleet_id)
-        if fleet.systemID == -1:
+        if fleet.systemID == INVALID_ID:
             # next_loc = fleet.nextSystemID
             return  # TODO: still check
         system = universe.getSystem(fleet.systemID)
         if not system:
             return
         orders = self.orders
-        last_sys_target = -1
+        last_sys_target = INVALID_ID
         if orders:
             last_sys_target = orders[-1].target.id
         if last_sys_target == fleet.systemID:
@@ -251,11 +272,11 @@ class AIFleetMission(object):
         if not open_targets:
             return
         troops_in_fleet = FleetUtilsAI.count_troops_in_fleet(fleet_id)
-        target_id = -1
+        target_id = INVALID_ID
         best_score = -1
         target_troops = 0
         #
-        for pid, rating in InvasionAI.assign_invasion_values(open_targets, empire).items():
+        for pid, rating in InvasionAI.assign_invasion_values(open_targets).items():
             p_score, p_troops = rating
             if p_score > best_score:
                 if p_troops >= troops_in_fleet:
@@ -263,30 +284,57 @@ class AIFleetMission(object):
                 best_score = p_score
                 target_id = pid
                 target_troops = p_troops
-        if target_id == -1:
+        if target_id == INVALID_ID:
             return
 
         print "\t AIFleetMission._check_retarget_invasion: splitting and retargetting fleet %d" % fleet_id
         new_fleets = FleetUtilsAI.split_fleet(fleet_id)
         self.clear_target()  # TODO: clear from foAIstate
         self.clear_fleet_orders()
-        # pods_needed = max(0, math.ceil((target_troops - 2 * (FleetUtilsAI.count_parts_fleetwide(fleet_id, ["GT_TROOP_POD"])) + 0.05) / 2.0))
         troops_needed = max(0, target_troops - FleetUtilsAI.count_troops_in_fleet(fleet_id))
-        found_stats = {}
         min_stats = {'rating': 0, 'troopCapacity': troops_needed}
         target_stats = {'rating': 10, 'troopCapacity': troops_needed}
         found_fleets = []
         # TODO check if next statement does not mutate any global states and can be removed
-        _ = FleetUtilsAI.get_fleets_for_mission(1, target_stats, min_stats, found_stats, "",
-                                                systems_to_check=[fleet.systemID], systems_checked=[],
-                                                fleet_pool_set=set(new_fleets), fleet_list=found_fleets,
-                                                verbose=False)
+        _ = FleetUtilsAI.get_fleets_for_mission(target_stats, min_stats, {}, starting_system=fleet.systemID,
+                                                fleet_pool_set=set(new_fleets), fleet_list=found_fleets)
         for fid in found_fleets:
             FleetUtilsAI.merge_fleet_a_into_b(fid, fleet_id)
         target = Planet(target_id)
-        self.add_target(MissionType.INVASION, target)
+        self.set_target(MissionType.INVASION, target)
         self.generate_fleet_orders()
 
+    def need_to_pause_movement(self, last_move_target_id, new_move_order):
+        """
+        When a fleet has consecutive move orders, assesses whether something about the interim destination warrants
+        forcing a stop (such as a military fleet choosing to engage with an enemy fleet about to enter the same system,
+        or it may provide a good vantage point to view current status of next system in path). Assessments about whether
+        the new destination is suitable to move to are (currently) separately made by OrderMove.can_issue_order()
+        :param last_move_target_id:
+        :type last_move_target_id: int
+        :param new_move_order:
+        :type new_move_order: OrderMove
+        :rtype: bool
+        """
+        fleet = self.fleet.get_object()
+        # don't try skipping over more than one System
+        if fleet.nextSystemID != last_move_target_id:
+            return True
+        universe = fo.getUniverse()
+        current_dest_system = universe.getSystem(fleet.nextSystemID)
+        if not current_dest_system:
+            # shouldn't really happen, but just to be safe
+            return True
+        distance_to_next_system = ((fleet.x - current_dest_system.x)**2 + (fleet.y - current_dest_system.y)**2)**0.5
+        surplus_travel_distance = fleet.speed - distance_to_next_system
+        # if need more than one turn to reach current destination, then don't add another jump yet
+        if surplus_travel_distance < 0:
+            return True
+        # TODO: add assessments for other situations we'd prefer to pause, such as cited above re military fleets, and
+        # for situations where high value fleets like colony fleets might deem it safest to stop and look around
+        # before proceeding
+        return False
+
     def issue_fleet_orders(self):
         """issues AIFleetOrders which can be issued in system and moves to next one if is possible"""
         # TODO: priority
@@ -295,7 +343,14 @@ class AIFleetMission(object):
         print "Checking orders for fleet %s (on turn %d), with mission type %s" % (self.fleet.get_object(), fo.currentTurn(), self.type or 'No mission')
         if MissionType.INVASION == self.type:
             self._check_retarget_invasion()
+        just_issued_move_order = False
+        last_move_target_id = INVALID_ID
         for fleet_order in self.orders:
+            if just_issued_move_order and self.fleet.get_object().systemID != last_move_target_id:
+                # having just issued a move order, we will normally stop issuing orders this turn, except that if there
+                # are consecutive move orders we will consider moving through the first destination rather than stopping
+                if not isinstance(fleet_order, OrderMove) or self.need_to_pause_movement(last_move_target_id, fleet_order):
+                    break
             print "Checking order: %s" % fleet_order
             if isinstance(fleet_order, (OrderColonize, OrderOutpost, OrderInvade)):  # TODO: invasion?
                 if self._check_abort_mission(fleet_order):
@@ -306,6 +361,8 @@ class AIFleetMission(object):
                 if isinstance(fleet_order, OrderMove) and order_completed:  # only move if all other orders completed
                     print "Issuing fleet order %s" % fleet_order
                     fleet_order.issue_order()
+                    just_issued_move_order = True
+                    last_move_target_id = fleet_order.target.id
                 elif not isinstance(fleet_order, OrderMove):
                     print "Issuing fleet order %s" % fleet_order
                     fleet_order.issue_order()
@@ -336,12 +393,7 @@ class AIFleetMission(object):
                             self.clear_fleet_orders()
                             self.clear_target()
                             return
-            # moving to another system stops issuing all orders in system where fleet is
-            # move order is also the last order in system
-            if isinstance(fleet_order, OrderMove):
-                fleet = self.fleet.get_object()
-                if fleet.systemID != fleet_order.target.id:
-                    break
+                break  # do not order the next order until this one is finished.
         else:  # went through entire order list
             if order_completed:
                 print "Final order is completed"
@@ -354,7 +406,7 @@ class AIFleetMission(object):
                     sys_partial_vis_turn = universe.getVisibilityTurnsMap(planet.systemID, fo.empireID()).get(fo.visibility.partial, -9999)
                     planet_partial_vis_turn = universe.getVisibilityTurnsMap(planet.id, fo.empireID()).get(fo.visibility.partial, -9999)
                     if planet_partial_vis_turn == sys_partial_vis_turn and not planet.currentMeterValue(fo.meterType.population):
-                        print "Potential Error: Fleet %d has tentatively completed its colonize mission but will wait to confirm population." % self.fleet.id
+                        print >> sys.stderr, "Fleet %d has tentatively completed its colonize mission but will wait to confirm population." % self.fleet.id
                         print "    Order details are %s" % last_order
                         print "    Order is valid: %s; issued: %s; executed: %s" % (last_order.is_valid(), last_order.order_issued, last_order.executed)
                         if not last_order.is_valid():
@@ -363,7 +415,7 @@ class AIFleetMission(object):
                             print "        source target validity: %s; target target validity: %s " % (bool(source_target), bool(target_target))
                         return  # colonize order must not have completed yet
                 clear_all = True
-                last_sys_target = -1
+                last_sys_target = INVALID_ID
                 if last_order and isinstance(last_order, OrderMilitary):
                     last_sys_target = last_order.target.id
                     # if (MissionType.SECURE == self.type) or # not doing this until decide a way to release from a SECURE mission
@@ -447,30 +499,25 @@ class AIFleetMission(object):
                 repair_fleet_order = MoveUtilsAI.get_repair_fleet_order(self.fleet, start_sys_id)
                 if repair_fleet_order and repair_fleet_order.is_valid():
                     self.orders.append(repair_fleet_order)
-            if fleet.fuel < fleet.maxFuel and self.get_location_target().id not in fleet_supplyable_system_ids:
+            cur_fighter_capacity, max_fighter_capacity = FleetUtilsAI.get_fighter_capacity_of_fleet(fleet_id)
+            if (fleet.fuel < fleet.maxFuel or cur_fighter_capacity < max_fighter_capacity
+                    and self.get_location_target().id not in fleet_supplyable_system_ids):
                 resupply_fleet_order = MoveUtilsAI.get_resupply_fleet_order(self.fleet, self.get_location_target())
                 if resupply_fleet_order.is_valid():
                     self.orders.append(resupply_fleet_order)
             return  # no targets
 
-        # for some targets fleet has to visit systems and therefore fleet visit them
         if self.target:
-            system_targets_required_to_visit = [self.target.get_system()]
-            orders_to_visit_systems = MoveUtilsAI.get_fleet_orders_from_system_targets(self.fleet, system_targets_required_to_visit)
+            # for some targets fleet has to visit systems and therefore fleet visit them
+            system_to_visit = self.target.get_system()
+            orders_to_visit_systems = MoveUtilsAI.create_move_orders_to_system(self.fleet, system_to_visit)
             # TODO: if fleet doesn't have enough fuel to get to final target, consider resetting Mission
             for fleet_order in orders_to_visit_systems:
                 self.orders.append(fleet_order)
 
-        # if fleet is in some system = fleet.system_id >=0, then also generate system AIFleetOrders
-        if system_id >= 0 and self.target:
-            # system in where fleet is
-            system_target = System(system_id)
-            # if mission aiTarget has required system where fleet is, then generate fleet_order from this aiTarget
-            # for all targets in all mission types get required systems to visit
-            if system_target == self.target.get_system():
-                # from target required to visit get fleet orders to accomplish target
-                fleet_order = self._get_fleet_order_from_target(self.type, self.target)
-                self.orders.append(fleet_order)
+            # also generate appropriate final orders
+            fleet_order = self._get_fleet_order_from_target(self.type, self.target)
+            self.orders.append(fleet_order)
 
     def _need_repair(self, repair_limit=0.70):
         """Check if fleet needs to be repaired.
@@ -527,6 +574,5 @@ class AIFleetMission(object):
         return "%-25s [%-11s] ships: %2d; total rating: %4d; target: %s" % (fleet,
                                                                             "NONE" if self.type is None else self.type,
                                                                             (fleet and len(fleet.shipIDs)) or 0,
-                                                                            foAI.foAIstate.get_rating(fleet_id).get(
-                                                                                'overall', 0),
+                                                                            CombatRatingsAI.get_fleet_rating(fleet_id),
                                                                             self.target or 'no target')
