@@ -14,36 +14,41 @@ import ResearchAI
 import AIDependencies
 from turn_state import state
 from EnumsAI import PriorityType, MissionType, EmpireProductionTypes, get_priority_production_types, ShipRoleType
-from freeorion_tools import Timer, tech_is_complete
+from freeorion_tools import AITimer, tech_is_complete
+from AIDependencies import INVALID_ID
 
-prioritiees_timer = Timer('calculate_priorities()')
+prioritiees_timer = AITimer('calculate_priorities()')
 
 allottedInvasionTargets = 0
 allottedColonyTargets = 0
 allotted_outpost_targets = 0
-colony_growth_barrier = 2
-scoutsNeeded = 0
 unmetThreat = 0
 
 
 def calculate_priorities():
     """Calculates the priorities of the AI player."""
-    print("checking statuses")
-    # Industry, Research, Colony, Invasion, Military
-
+    print "\n", 10 * "=", "Preparing to Calculate Priorities", 10 * "="
     prioritiees_timer.start('setting Production Priority')
     foAI.foAIstate.set_priority(PriorityType.RESOURCE_PRODUCTION, 50)  # let this one stay fixed & just adjust Research
+
+    print "\n*** Calculating Research Priority ***\n"
     prioritiees_timer.start('setting Research Priority')
     foAI.foAIstate.set_priority(PriorityType.RESOURCE_RESEARCH, _calculate_research_priority())  # TODO: do univ _survey before this
-    prioritiees_timer.start('Evaluating Colonization Status')
 
+    print "\n*** Updating Colonization Status ***\n"
+    prioritiees_timer.start('Evaluating Colonization Status')
     ColonisationAI.get_colony_fleets()  # sets foAI.foAIstate.colonisablePlanetIDs and foAI.foAIstate.outpostPlanetIDs and many other values used by other modules
+
+    print "\n*** Updating Invasion Status ***\n"
     prioritiees_timer.start('Evaluating Invasion Status')
     InvasionAI.get_invasion_fleets()  # sets AIstate.invasionFleetIDs, AIstate.opponentPlanetIDs, and AIstate.invasionTargetedPlanetIDs
+
+    print "\n*** Updating Military Status ***\n"
     prioritiees_timer.start('Evaluating Military Status')
     MilitaryAI.get_military_fleets()
+
+    print("\n** Calculating Production Priorities ***\n")
     prioritiees_timer.start('reporting Production Priority')
-    print("Calculating priorities")
     _calculate_industry_priority()  # purely for reporting purposes
     prioritiees_timer.start('setting Exploration Priority')
 
@@ -69,7 +74,7 @@ def calculate_priorities():
     foAI.foAIstate.set_priority(PriorityType.RESEARCH_ECONOMICS, 0)
     foAI.foAIstate.set_priority(PriorityType.RESEARCH_SHIPS, _calculate_ships_priority())
     foAI.foAIstate.set_priority(PriorityType.RESEARCH_DEFENSE, 0)
-    prioritiees_timer.end()
+    prioritiees_timer.stop_print_and_clear()
 
 
 def _calculate_industry_priority():  # currently only used to print status
@@ -117,11 +122,10 @@ def _calculate_research_priority():
 
     total_pp = empire.productionPoints
     total_rp = empire.resourceProduction(fo.resourceType.research)
-    industry_surge = ((foAI.foAIstate.aggression > fo.aggression.cautious) and
-                     ((total_pp + 1.6 * total_rp) < (50 * foAI.foAIstate.aggression)) and
-                     (((orb_gen_tech in research_queue_list[:2] or got_orb_gen) and state.have_gas_giant) or
-                      ((mgrav_prod_tech in research_queue_list[:2] or got_mgrav_prod) and state.have_asteroids)) and
-                     (not (len(AIstate.popCtrIDs) >= 12)))
+    industry_surge = (foAI.foAIstate.character.may_surge_industry(total_pp, total_rp) and
+                      (((orb_gen_tech in research_queue_list[:2] or got_orb_gen) and state.have_gas_giant) or
+                       ((mgrav_prod_tech in research_queue_list[:2] or got_mgrav_prod) and state.have_asteroids)) and
+                      (not (len(AIstate.popCtrIDs) >= 12)))
     # get current industry production & Target
     owned_planet_ids = PlanetUtilsAI.get_owned_planets_by_empire(universe.planetIDs)
     planets = map(universe.getPlanet, owned_planet_ids)
@@ -129,8 +133,8 @@ def _calculate_research_priority():
     galaxy_is_sparse = ColonisationAI.galaxy_is_sparse()
     enemies_sighted = foAI.foAIstate.misc.get('enemies_sighted', {})
 
-    style_index = empire_id % 2
-    if foAI.foAIstate.aggression >= fo.aggression.maniacal:
+    style_index = foAI.foAIstate.character.preferred_research_cutoff([0, 1])
+    if foAI.foAIstate.character.may_maximize_research():
         style_index += 1
 
     cutoff_sets = [[25, 45, 70, 110], [30, 45, 70, 150], [25, 40, 80, 160]]
@@ -165,13 +169,12 @@ def _calculate_research_priority():
     if ((tech_is_complete("SHP_WEAPON_2_4") or
          tech_is_complete("SHP_WEAPON_4_1")) and
             tech_is_complete(AIDependencies.PROD_AUTO_NAME)):
-        # industry_factor = [ [0.25, 0.2], [0.3, 0.25], [0.3, 0.25] ][styleIndex ]
+        # industry_factor = [ [0.25, 0.2], [0.3, 0.25], [0.3, 0.25] ][style_index ]
         # researchPriority = min(researchPriority, industry_factor[got_solar_gen]*industryPriority)
         research_priority *= 0.9
     if got_quant:
         research_priority = min(research_priority + 0.1 * industry_priority, research_priority * 1.3)
     research_priority = int(research_priority)
-    print ""
     print "Research Production (current/target) : ( %.1f / %.1f )" % (total_rp, target_rp)
     print "Priority for Research: %d (new target ~ %d RP)" % (research_priority, total_pp * research_priority / industry_priority)
 
@@ -184,11 +187,10 @@ def _calculate_research_priority():
 
 def _calculate_exploration_priority():
     """Calculates the demand for scouts by unexplored systems."""
-    global scoutsNeeded
     empire = fo.getEmpire()
-    num_unexplored_systems = len(ExplorationAI.borderUnexploredSystemIDs)  # len(foAI.foAIstate.get_explorable_systems(ExplorableSystemType.UNEXPLORED))
+    num_unexplored_systems = len(ExplorationAI.borderUnexploredSystemIDs)
     num_scouts = sum([foAI.foAIstate.fleetStatus.get(fid, {}).get('nships', 0) for fid in FleetUtilsAI.get_empire_fleet_ids_by_role(
-        MissionType.EXPLORATION)])  # FleetUtilsAI.get_empire_fleet_ids_by_role(MissionType.EXPLORATION)
+        MissionType.EXPLORATION)])
     production_queue = empire.productionQueue
     queued_scout_ships = 0
     for queue_index in range(0, len(production_queue)):
@@ -197,15 +199,16 @@ def _calculate_exploration_priority():
             if foAI.foAIstate.get_ship_role(element.designID) == ShipRoleType.CIVILIAN_EXPLORATION:
                 queued_scout_ships += element.remaining * element.blocksize
 
-    mil_ships = MilitaryAI.num_milships
+    mil_ships = MilitaryAI.get_num_military_ships()
     # intent of the following calc is essentially
     # new_scouts_needed = min(need_cap_A, need_cap_B, base_need) - already_got_or_queued
     # where need_cap_A is to help prevent scouting needs from swamping military needs, and
     # need_cap_B is to help regulate investment into scouting while the empire is small.
     # These caps could perhaps instead be tied more directly to military priority and
     # total empire production.
-    scoutsNeeded = max(0, min(4 + int(mil_ships / 5), 4 + int(fo.currentTurn() / 50), 2 + num_unexplored_systems**0.5) - num_scouts - queued_scout_ships)
-    exploration_priority = int(40 * scoutsNeeded)
+    desired_number_of_scouts = int(min(4 + mil_ships/5, 4 + fo.currentTurn()/50.0, 2 + num_unexplored_systems**0.5))
+    scouts_needed = max(0, desired_number_of_scouts - (num_scouts + queued_scout_ships))
+    exploration_priority = int(40 * scouts_needed)
 
     print
     print "Number of Scouts: %s" % num_scouts
@@ -218,18 +221,18 @@ def _calculate_exploration_priority():
 
 def _calculate_colonisation_priority():
     """Calculates the demand for colony ships by colonisable planets."""
-    global allottedColonyTargets, colony_growth_barrier
+    global allottedColonyTargets
     enemies_sighted = foAI.foAIstate.misc.get('enemies_sighted', {})
     galaxy_is_sparse = ColonisationAI.galaxy_is_sparse()
     total_pp = fo.getEmpire().productionPoints
     num_colonies = len(list(AIstate.popCtrIDs))
-    # significant growth barrier for low aggression, negligible for high aggression
-    colony_growth_barrier = 2 + ((0.5+foAI.foAIstate.aggression)**2)*fo.currentTurn()/50.0
+    colony_growth_barrier = foAI.foAIstate.character.max_number_colonies()
     colony_cost = AIDependencies.COLONY_POD_COST * (1 + AIDependencies.COLONY_POD_UPKEEP * num_colonies)
     turns_to_build = 8  # TODO: check for susp anim pods, build time 10
     mil_prio = foAI.foAIstate.get_priority(PriorityType.PRODUCTION_MILITARY)
-    allotted_portion = ([[[0.6, 0.8], [0.3, 0.4]], [[0.8, 0.9], [0.3, 0.4]]][galaxy_is_sparse]
-                       [any(enemies_sighted)][fo.empireID() % 2])
+    allotted_portion = ([[[0.6, 0.8], [0.3, 0.4]], [[0.8, 0.9], [0.4, 0.6]]][galaxy_is_sparse]
+                       [any(enemies_sighted)])
+    allotted_portion = foAI.foAIstate.character.preferred_colonization_portion(allotted_portion)
     # if ( foAI.foAIstate.get_priority(AIPriorityType.PRIORITY_PRODUCTION_COLONISATION)
     # > 2 * foAI.foAIstate.get_priority(AIPriorityType.PRIORITY_PRODUCTION_MILITARY)):
     # allotted_portion *= 1.5
@@ -276,7 +279,7 @@ def _calculate_outpost_priority():
     galaxy_is_sparse = ColonisationAI.galaxy_is_sparse()
     total_pp = fo.getEmpire().productionPoints
     num_colonies = len(list(AIstate.popCtrIDs))
-    # significant growth barrier for low aggression, negligible for high aggression
+    colony_growth_barrier = foAI.foAIstate.character.max_number_colonies()
     if num_colonies > colony_growth_barrier:
         return 0.0
     mil_prio = foAI.foAIstate.get_priority(PriorityType.PRODUCTION_MILITARY)
@@ -288,7 +291,8 @@ def _calculate_outpost_priority():
         (not_sparse, enemy_seen): (0.3, 0.4),
         (is_sparse, enemy_unseen): (0.8, 0.9),
         (is_sparse, enemy_seen): (0.3, 0.4),
-    }[(galaxy_is_sparse, any(enemies_sighted))][fo.empireID() % 2]
+    }[(galaxy_is_sparse, any(enemies_sighted))]
+    allotted_portion = foAI.foAIstate.character.preferred_outpost_portion(allotted_portion)
     if mil_prio < 100:
         allotted_portion *= 2
     elif mil_prio < 200:
@@ -318,13 +322,14 @@ def _calculate_invasion_priority():
     """Calculates the demand for troop ships by opponent planets."""
     
     global allottedInvasionTargets
-    if foAI.foAIstate.aggression <= fo.aggression.turtle:
+    if not foAI.foAIstate.character.may_invade():
         return 0
     
     empire = fo.getEmpire()
     enemies_sighted = foAI.foAIstate.misc.get('enemies_sighted', {})
     multiplier = 1
     num_colonies = len(list(AIstate.popCtrIDs))
+    colony_growth_barrier = foAI.foAIstate.character.max_number_colonies()
     if num_colonies > colony_growth_barrier:
         return 0.0
     
@@ -333,9 +338,6 @@ def _calculate_invasion_priority():
     else:
         best_colony_score = 2
 
-    if foAI.foAIstate.aggression == fo.aggression.beginner and fo.currentTurn() < 150:
-        return 0
-
     allottedInvasionTargets = 1 + int(fo.currentTurn() / 25)
     total_val = 0
     troops_needed = 0
@@ -385,10 +387,8 @@ def _calculate_invasion_priority():
         
     if invasion_priority < 0:
         return 0
-    if foAI.foAIstate.aggression == fo.aggression.beginner:
-        return 0.5 * invasion_priority
-    else:
-        return invasion_priority
+
+    return invasion_priority * foAI.foAIstate.character.invasion_priority_scaling()
 
 
 def _calculate_military_priority():
@@ -397,7 +397,7 @@ def _calculate_military_priority():
 
     universe = fo.getUniverse()
     capital_id = PlanetUtilsAI.get_capital()
-    if capital_id is None or capital_id == -1:
+    if capital_id is None or capital_id == INVALID_ID:
         return 0  # no capitol (not even a capitol-in-the-making), means can't produce any ships
         
     have_l1_weaps = (tech_is_complete("SHP_WEAPON_1_4") or
@@ -458,8 +458,7 @@ def _calculate_military_priority():
     print fmt_string % (military_priority, ships_needed, defense_ships_needed, cur_ship_rating, have_l1_weaps, enemies_sighted)
     print "Source of milship demand: ", ships_needed_allocation
     
-    if foAI.foAIstate.aggression < fo.aggression.typical:
-        military_priority *= (1.0 + foAI.foAIstate.aggression) / (1.0 + fo.aggression.typical)
+    military_priority *= foAI.foAIstate.character.military_priority_scaling()
     return max(military_priority, 0)
 
 
