@@ -1,6 +1,7 @@
 from functools import partial
 import math
 import random
+import sys
 import traceback
 
 import freeOrionAIInterface as fo  # pylint: disable=import-error
@@ -8,6 +9,7 @@ import FreeOrionAI as foAI
 import AIDependencies as Dep
 import AIstate
 import ColonisationAI
+import ProductionAI
 import ShipDesignAI
 import TechsListsAI
 from turn_state import state
@@ -56,17 +58,11 @@ HIGH = 42
 
 # TODO research AI no longer use this method, rename and move this method elsewhere
 def get_research_index():
-    empire_id = fo.empireID()
-    research_index = empire_id % 2
-    if foAI.foAIstate.aggression >= fo.aggression.aggressive:  # maniacal
-        research_index = 2 + (empire_id % 3)  # so indices [2,3,4]
-    elif foAI.foAIstate.aggression >= fo.aggression.typical:
-        research_index += 1
-    return research_index
+    return foAI.foAIstate.character.get_research_index()
 
 
 def has_low_aggression():
-    return foAI.foAIstate.aggression < fo.aggression.typical
+    return foAI.foAIstate.character.prefer_research_low_aggression()
 
 
 def conditional_priority(func_if_true, func_if_false, cond_func):
@@ -198,9 +194,9 @@ def get_ship_tech_usefulness(tech, ship_designer):
     if not (old_designs and new_designs):
         # AI is likely defeated; don't bother with logging error message
         return 0
-    old_rating, old_pid, old_design_id, old_cost = old_designs[0]
+    old_rating, old_pid, old_design_id, old_cost, old_stats = old_designs[0]
     old_rating = old_rating
-    new_rating, new_pid, new_design_id, new_cost = new_designs[0]
+    new_rating, new_pid, new_design_id, new_cost, new_stats = new_designs[0]
     new_rating = new_rating
     if new_rating > old_rating:
         ratio = (new_rating - old_rating) / (new_rating + old_rating)
@@ -223,7 +219,7 @@ def get_stealth_priority(tech_name=""):
 
 
 def get_xeno_genetics_priority(tech_name=""):
-    if foAI.foAIstate.aggression < fo.aggression.cautious:
+    if not foAI.foAIstate.character.may_research_xeno_genetics_variances():
         return get_population_boost_priority()
     if has_only_bad_colonizers():
         # Empire only have lousy colonisers, xeno-genetics are really important for them
@@ -352,7 +348,7 @@ def init():
     """
     choices.init()
     # prefixes for tech search. Check for prefix will be applied in same order as they defined
-    defensive = foAI.foAIstate.aggression <= fo.aggression.cautious
+    defensive = foAI.foAIstate.character.prefer_research_defensive()
     prefixes = [
         (Dep.DEFENSE_TECHS_PREFIX, 2.0 if defensive else if_enemies(1.0, 0.2)),
         (Dep.WEAPON_PREFIX, ship_usefulness(if_enemies(1.0, 0.2), MIL_IDX))
@@ -454,7 +450,7 @@ def generate_research_orders():
             tech_turns_left[element.tech] = element.turnsLeft
             this_tech = fo.getTech(element.tech)
             if not this_tech:
-                print "Error: can't retrieve tech ", element.tech
+                print >> sys.stderr, "Can't retrieve tech ", element.tech
                 continue
             missing_prereqs = [preReq for preReq in this_tech.recursivePrerequisites(empire_id) if preReq not in completed_techs]
             # unlocked_items = [(uli.name, uli.type) for uli in this_tech.unlocked_items]
@@ -609,9 +605,9 @@ def get_research_queue_techs():
 
 
 def exclude_tech(tech_name):
-    return ((foAI.foAIstate.aggression < Dep.TECH_EXCLUSION_MAP_1.get(tech_name, fo.aggression.invalid)) or
-            (foAI.foAIstate.aggression > Dep.TECH_EXCLUSION_MAP_2.get(tech_name, fo.aggression.maniacal)) or
-            tech_name in TechsListsAI.unusable_techs())
+    return ((not foAI.foAIstate.character.may_research_tech(tech_name))
+            or tech_name in TechsListsAI.unusable_techs()
+            or tech_name in Dep.UNRESEARCHABLE_TECHS)
 
 
 def generate_classic_research_orders():
@@ -648,7 +644,7 @@ def generate_classic_research_orders():
                 inProgressTechs[element.tech] = True
             this_tech = fo.getTech(element.tech)
             if not this_tech:
-                print "Error: can't retrieve tech ", element.tech
+                print >> sys.stderr, "Can't retrieve tech ", element.tech
                 continue
             missing_prereqs = [preReq for preReq in this_tech.recursivePrerequisites(empire_id) if preReq not in completed_techs]
             # unlocked_items = [(uli.name, uli.type) for uli in this_tech.unlocked_items]
@@ -678,7 +674,7 @@ def generate_classic_research_orders():
             if tech not in tech_base:
                 this_tech = fo.getTech(tech)
                 if this_tech is None:
-                    print "Error: desired tech '%s' appears to not exist" % tech
+                    print >> sys.stderr, "Desired tech '%s' appears to not exist" % tech
                     continue
                 missing_prereqs = [preReq for preReq in this_tech.recursivePrerequisites(empire_id) if preReq not in tech_base]
                 techs_to_add.extend(missing_prereqs + [tech])
@@ -696,10 +692,10 @@ def generate_classic_research_orders():
                         cum_cost += this_cost
                     print "    Enqueued Tech: %20s \t\t %8.0f \t %8.0f" % (name, this_cost, cum_cost)
                 else:
-                    print "    Error: failed attempt to enqueued Tech: " + name
+                    print >> sys.stderr, "    Failed attempt to enqueued Tech: " + name
             except:
-                print "    Error: failed attempt to enqueued Tech: " + name
-                print "    Error: exception triggered and caught: ", traceback.format_exc()
+                print >> sys.stderr, "    Failed attempt to enqueued Tech: " + name
+                print >> sys.stderr, "    Exception triggered and caught: ", traceback.format_exc()
 
         print '\n\nAll techs:'
         print '=' * 20
@@ -714,23 +710,25 @@ def generate_classic_research_orders():
 
         if fo.currentTurn() == 1:
             return
-        if foAI.foAIstate.aggression <= fo.aggression.cautious:
+        if True:
             research_queue_list = get_research_queue_techs()
             def_techs = TechsListsAI.defense_techs_1()
             for def_tech in def_techs:
-                if def_tech not in research_queue_list[:5] and not tech_is_complete(def_tech):
+                if (foAI.foAIstate.character.may_research_tech_classic(def_tech)
+                    and def_tech not in research_queue_list[:5] and not tech_is_complete(def_tech)):
                     res = fo.issueEnqueueTechOrder(def_tech, min(3, len(research_queue_list)))
                     print "Empire is very defensive, so attempted to fast-track %s, got result %d" % (def_tech, res)
-        if False and foAI.foAIstate.aggression >= fo.aggression.aggressive:  # with current stats of Conc Camps, disabling this fast-track
+        if False:  # with current stats of Conc Camps, disabling this fast-track
             research_queue_list = get_research_queue_techs()
-            if "CON_CONC_CAMP" in research_queue_list:
+            if "CON_CONC_CAMP" in research_queue_list and foAI.foAIstate.character.may_research_tech_classic("CON_CONC_CAMP"):
                 insert_idx = min(40, research_queue_list.index("CON_CONC_CAMP"))
             else:
                 insert_idx = max(0, min(40, len(research_queue_list) - 10))
-            if "SHP_DEFLECTOR_SHIELD" in research_queue_list:
+            if "SHP_DEFLECTOR_SHIELD" in research_queue_list and foAI.foAIstate.character.may_research_tech_classic("SHP_DEFLECTOR_SHIELD"):
                 insert_idx = min(insert_idx, research_queue_list.index("SHP_DEFLECTOR_SHIELD"))
             for cc_tech in ["CON_ARCH_PSYCH", "CON_CONC_CAMP"]:
-                if cc_tech not in research_queue_list[:insert_idx + 1] and not tech_is_complete(cc_tech):
+                if (cc_tech not in research_queue_list[:insert_idx + 1] and not tech_is_complete(cc_tech)
+                    and foAI.foAIstate.character.may_research_tech_classic(cc_tech)):
                     res = fo.issueEnqueueTechOrder(cc_tech, insert_idx)
                     msg = "Empire is very aggressive, so attempted to fast-track %s, got result %d" % (cc_tech, res)
                     if report_adjustments:
@@ -759,6 +757,23 @@ def generate_classic_research_orders():
 
     nest_tech = Dep.NEST_DOMESTICATION_TECH
     artif_minds = Dep.ART_MINDS
+
+    # fast-track LRN_PHYS_BRAIN if we want to build the Automatic History Analyzer
+    # The decision-making is made by the relevant ProductionAI.py scripts which also enqueue the building.
+    # Both LRN_ART_MINDS and GRO_SUBTER_HAB are prioritized over this.
+    if not tech_is_complete(Dep.LRN_PHYS_BRAIN):
+        print "Considering whether to fast-track %s to build Automatic History Analyzer" % Dep.LRN_PHYS_BRAIN
+        if ProductionAI.find_automatic_historic_analyzer_candidates():
+            artif_mind_idx = artif_minds in research_queue_list and research_queue_list.index(artif_minds) or 0
+            subter_hab_idx = Dep.GRO_SUBTER_HAB in research_queue_list and research_queue_list.index(Dep.GRO_SUBTER_HAB) or 0
+            insert_idx = 1 + max(artif_mind_idx, subter_hab_idx)
+            res = fo.issueEnqueueTechOrder(Dep.LRN_PHYS_BRAIN, insert_idx)
+            num_techs_accelerated += 1
+            print "Got possible locations to build the Automatic History Analyzer. Enqueuing, got result %d" % res
+            research_queue_list = get_research_queue_techs()
+        else:
+            print "Currently, no interest in building Automatic History Analyzer. Do not fast-track."
+
     if state.have_nest and not tech_is_complete(nest_tech):
         if artif_minds in research_queue_list:
             insert_idx = 1 + research_queue_list.index(artif_minds)
@@ -841,8 +856,8 @@ def generate_classic_research_orders():
     #
     # check to accelerate xeno_arch
     if True:  # just to help with cold-folding /  organization
-        if (state.have_ruins and not tech_is_complete("LRN_XENOARCH") and
-                foAI.foAIstate.aggression >= fo.aggression.typical):
+        if (state.have_ruins and not tech_is_complete("LRN_XENOARCH")
+            and foAI.foAIstate.character.may_research_tech_classic("LRN_XENOARCH")):
             if artif_minds in research_queue_list:
                 insert_idx = 7 + research_queue_list.index(artif_minds)
             elif "GRO_SYMBIOTIC_BIO" in research_queue_list:
@@ -934,7 +949,7 @@ def generate_classic_research_orders():
     #
     # assess if our empire has any non-lousy colonizers, & boost gro_xeno_gen if we don't
     if True:  # just to help with cold-folding / organization
-        if got_ggg_tech and got_sym_bio and (not got_xeno_gen) and foAI.foAIstate.aggression >= fo.aggression.cautious:
+        if got_ggg_tech and got_sym_bio and (not got_xeno_gen):
             most_adequate = 0
             for specName in ColonisationAI.empire_colonizers:
                 environs = {}
@@ -948,7 +963,8 @@ def generate_classic_research_orders():
             if most_adequate == 0:
                 insert_idx = num_techs_accelerated
                 for xg_tech in ["GRO_XENO_GENETICS", "GRO_GENETIC_ENG"]:
-                    if xg_tech not in research_queue_list[:1+num_techs_accelerated] and not tech_is_complete(xg_tech):
+                    if (xg_tech not in research_queue_list[:1+num_techs_accelerated] and not tech_is_complete(xg_tech)
+                        and foAI.foAIstate.character.may_research_tech_classic(xg_tech)):
                         res = fo.issueEnqueueTechOrder(xg_tech, insert_idx)
                         num_techs_accelerated += 1
                         msg = "Empire has poor colonizers, so attempted to fast-track %s, got result %d" % (xg_tech, res)
@@ -961,15 +977,16 @@ def generate_classic_research_orders():
     if True:  # just to help with cold-folding / organization
         if not tech_is_complete("LRN_DISTRIB_THOUGHT"):
             got_telepathy = False
-            for specName in ColonisationAI.empire_species:
+            for specName in state.get_empire_planets_by_species():
                 this_spec = fo.getSpecies(specName)
                 if this_spec and ("TELEPATHIC" in list(this_spec.tags)):
                     got_telepathy = True
                     break
-            if (foAI.foAIstate.aggression > fo.aggression.cautious) and (empire.population() > ([300, 100][got_telepathy])):
+            if empire.population() > ([300, 100][got_telepathy]):
                 insert_idx = num_techs_accelerated
                 for dt_ech in ["LRN_PHYS_BRAIN", "LRN_TRANSLING_THT", "LRN_PSIONICS", "LRN_DISTRIB_THOUGHT"]:
-                    if dt_ech not in research_queue_list[:insert_idx + 2] and not tech_is_complete(dt_ech):
+                    if (dt_ech not in research_queue_list[:insert_idx + 2] and not tech_is_complete(dt_ech)
+                        and foAI.foAIstate.character.may_research_tech_classic(dt_ech)):
                         res = fo.issueEnqueueTechOrder(dt_ech, insert_idx)
                         num_techs_accelerated += 1
                         insert_idx += 1
@@ -983,7 +1000,7 @@ def generate_classic_research_orders():
     #
     # check to accelerate quant net
     if False:  # disabled for now, otherwise just to help with cold-folding / organization
-        if (foAI.foAIstate.aggression > fo.aggression.cautious) and (ColonisationAI.empire_status.get('researchers', 0) >= 40):
+        if foAI.foAIstate.character.may_research_tech_classic("LRN_QUANT_NET") and (ColonisationAI.empire_status.get('researchers', 0) >= 40):
             if not tech_is_complete("LRN_QUANT_NET"):
                 insert_idx = num_techs_accelerated  # TODO determine min target slot if reenabling
                 for qnTech in ["LRN_NDIM_SUBSPACE", "LRN_QUANT_NET"]:
@@ -1001,7 +1018,7 @@ def generate_classic_research_orders():
     # if we own a blackhole, accelerate sing_gen and conc camp
     if True:  # just to help with cold-folding / organization
         if (fo.currentTurn() > 50 and len(AIstate.empireStars.get(fo.starType.blackHole, [])) != 0 and
-                foAI.foAIstate.aggression > fo.aggression.cautious and not tech_is_complete(Dep.PRO_SINGULAR_GEN) and
+                foAI.foAIstate.character.may_research_tech_classic("PRO_SINGULAR_GEN") and not tech_is_complete(Dep.PRO_SINGULAR_GEN) and
                 tech_is_complete(Dep.PRO_SOL_ORB_GEN)):
             # sing_tech_list = [ "LRN_GRAVITONICS" , "PRO_SINGULAR_GEN"]  # formerly also "CON_ARCH_PSYCH", "CON_CONC_CAMP",
             sing_gen_tech = fo.getTech(Dep.PRO_SINGULAR_GEN)
@@ -1055,13 +1072,15 @@ def generate_classic_research_orders():
             print "No new ship parts/hulls unlocked by tech %s" % tech
             continue
         old_designs = ShipDesignAI.MilitaryShipDesigner().optimize_design(consider_fleet_count=False)
-        new_designs = ShipDesignAI.MilitaryShipDesigner().optimize_design(additional_hulls=unlocked_hulls, additional_parts=unlocked_parts, consider_fleet_count=False)
+        new_designs = ShipDesignAI.MilitaryShipDesigner().optimize_design(additional_hulls=unlocked_hulls,
+                                                                          additional_parts=unlocked_parts,
+                                                                          consider_fleet_count=False)
         if not (old_designs and new_designs):
             # AI is likely defeated; don't bother with logging error message
             continue
-        old_rating, old_pid, old_design_id, old_cost = old_designs[0]
+        old_rating, old_pid, old_design_id, old_cost, old_stats = old_designs[0]
         old_design = fo.getShipDesign(old_design_id)
-        new_rating, new_pid, new_design_id, new_cost = new_designs[0]
+        new_rating, new_pid, new_design_id, new_cost, new_stats = new_designs[0]
         new_design = fo.getShipDesign(new_design_id)
         if new_rating > old_rating:
             print "Tech %s gives access to a better design!" % tech
