@@ -9,6 +9,7 @@ have their future focus decided.
 """
 # Note: The algorithm is not stable with respect to pid order.  i.e. Two empire with
 #       exactly the same colonies, but different pids may make different choices.
+import sys
 
 import freeOrionAIInterface as fo  # pylint: disable=import-error
 import FreeOrionAI as foAI
@@ -17,10 +18,11 @@ import PlanetUtilsAI
 import random
 import ColonisationAI
 import AIDependencies
-import FleetUtilsAI
-from freeorion_tools import tech_is_complete, Timer
+import CombatRatingsAI
+from common.print_utils import Table, Text, Float
+from freeorion_tools import tech_is_complete, AITimer
 
-resource_timer = Timer('timer_bucket')
+resource_timer = AITimer('timer_bucket')
 
 # Local Constants
 INDUSTRY = FocusType.FOCUS_INDUSTRY
@@ -30,7 +32,7 @@ PRODUCTION = FocusType.FOCUS_PROTECTION
 _focus_names = {INDUSTRY: "Industry", RESEARCH: "Research", GROWTH: "Growth", PRODUCTION: "Defense"}
 
 # TODO use the priorityRatio to weight
-RESEARCH_WEIGHTING = 2.0
+RESEARCH_WEIGHTING = 2.3
 
 useGrowth = True
 limitAssessments = False
@@ -175,6 +177,8 @@ class Reporter(object):
     def print_table_footer(self, priority_ratio):
         current_industry_target = 0
         current_research_target = 0
+        new_industry_target = 0
+        new_research_target = 0
         all_industry_industry_target = 0
         all_industry_research_target = 0
         all_research_industry_target = 0
@@ -184,9 +188,13 @@ class Reporter(object):
             if info.current_focus != info.future_focus:
                 total_changed += 1
 
+            old_pp, old_rp = info.possible_output[info.current_focus]
+            current_industry_target += old_pp
+            current_research_target += old_rp
+
             future_pp, future_rp = info.possible_output[info.future_focus]
-            current_industry_target += future_pp
-            current_research_target += future_rp
+            new_industry_target += future_pp
+            new_research_target += future_rp
 
             industry_pp, industry_rp = info.possible_output[INDUSTRY] if INDUSTRY in info.possible_output else (future_pp, future_rp)
             all_industry_industry_target += industry_pp
@@ -197,7 +205,7 @@ class Reporter(object):
             all_research_research_target += research_rp
 
         print "-----------------------------------"
-        print "Planet Focus Assignments to achieve target RP/PP ratio of %.2f from current ratio of %.2f ( %.1f / %.1f )" \
+        print "Planet Focus Assignments to achieve target RP/PP ratio of %.2f from current target ratio of %.2f ( %.1f / %.1f )" \
             % (priority_ratio, current_research_target / (current_industry_target + 0.0001),
                current_research_target, current_industry_target)
         print "Max Industry assignments would result in target RP/PP ratio of %.2f ( %.1f / %.1f )" \
@@ -208,8 +216,8 @@ class Reporter(object):
                all_research_research_target, all_research_industry_target)
         print "-----------------------------------"
         print "Final Ratio Target (turn %4d) RP/PP : %.2f ( %.1f / %.1f ) after %d Focus changes" \
-            % (fo.currentTurn(), current_research_target / (current_industry_target + 0.0001),
-               current_research_target, current_industry_target, total_changed)
+            % (fo.currentTurn(), new_research_target / (new_industry_target + 0.0001),
+               new_research_target, new_industry_target, total_changed)
 
     def print_table(self, priority_ratio):
         """Prints a table of all of the captured sections of assignments."""
@@ -249,8 +257,6 @@ class Reporter(object):
         universe = fo.getUniverse()
         empire = fo.getEmpire()
         empire_planet_ids = PlanetUtilsAI.get_owned_planets_by_empire(universe.planetIDs)
-        print "Resource Management:"
-        print
         print "Resource Priorities:"
         resource_priorities = {}
         for priority_type in get_priority_resource_types():
@@ -262,27 +268,38 @@ class Reporter(object):
         for evaluation_priority, evaluation_score in sorted_priorities:
             if top_priority < 0:
                 top_priority = evaluation_priority
-            print "    ResourcePriority |Score: %s | %s " % (evaluation_priority, evaluation_score)
+            print "  %s: %.2f" % (evaluation_priority, evaluation_score)
 
         # what is the focus of available resource centers?
         print
         warnings = {}
-        # TODO combine this with previous table to reduce report duplication?
-        print "Planet Resources Foci:"
+        foci_table = Table([
+                Text('Planet'),
+                Text('Size'),
+                Text('Type'),
+                Text('Focus'),
+                Text('Species'),
+                Text('Pop')
+            ], table_name="Planetary Foci Overview Turn %d" % fo.currentTurn())
         for pid in empire_planet_ids:
             planet = universe.getPlanet(pid)
             population = planet.currentMeterValue(fo.meterType.population)
             max_population = planet.currentMeterValue(fo.meterType.targetPopulation)
             if max_population < 1 and population > 0:
                 warnings[planet.name] = (population, max_population)
-            print "  ID: %d Name: % 18s -- % 6s % 8s  Focus: % 8s Species: %s Pop: %2d/%2d" % (pid, planet.name, planet.size, planet.type, "_".join(str(planet.focus).split("_")[1:])[:8], planet.speciesName, population, max_population)
-        print "\n\nEmpire Totals:\nPopulation: %5d \nProduction: %5d\nResearch: %5d\n" % (empire.population(), empire.productionPoints, empire.resourceProduction(fo.resourceType.research))
-        if warnings != {}:
-            for name in warnings:
-                mp, cp = warnings[name]
-                print "Population Warning! -- %s has unsustainable current pop %d -- target %d" % (name, cp, mp)
-            print
-        warnings.clear()
+            foci_table.add_row([
+                planet,
+                planet.size,
+                planet.type,
+                "_".join(str(planet.focus).split("_")[1:])[:8],
+                planet.speciesName,
+                "%.1f/%.1f" % (population, max_population)
+            ])
+        foci_table.print_table()
+        print "Empire Totals:\nPopulation: %5d \nProduction: %5d\nResearch: %5d\n" % (
+            empire.population(), empire.productionPoints, empire.resourceProduction(fo.resourceType.research))
+        for name, (cp, mp) in warnings.iteritems():
+            print "Population Warning! -- %s has unsustainable current pop %d -- target %d" % (name, cp, mp)
 
 
 def weighted_sum_output(outputs):
@@ -367,9 +384,9 @@ def assess_protection_focus(pid, info):
             local_production_diff > 5))):
         use_protection = False
         reason = "F"
-    elif (regional_threat <= FleetUtilsAI.combine_ratings(local_fleet_rating, adjusted_p_defenses) and
+    elif (regional_threat <= CombatRatingsAI.combine_ratings(local_fleet_rating, adjusted_p_defenses) and
           safety_factor * regional_threat <=
-          FleetUtilsAI.combine_ratings_list([my_neighbor_rating, local_fleet_rating, adjusted_p_defenses]) and
+          CombatRatingsAI.combine_ratings_list([my_neighbor_rating, local_fleet_rating, adjusted_p_defenses]) and
           local_production_diff > 5):
         use_protection = False
         reason = "G"
@@ -449,8 +466,8 @@ def set_planet_production_and_research_specials(focus_manager):
                 continue
             else:
                 new_planet = universe.getPlanet(pid)
-                print ("Error: Failed setting %s for Concentration Camp planet %s (%d) with species %s and current focus %s, but new planet copy shows %s" %
-                       (info.future_focus, planet.name, pid, planet.speciesName, planet.focus, new_planet.focus))
+                print >> sys.stderr, ("Failed setting %s for Concentration Camp planet %s (%d) with species %s and current focus %s, but new planet copy shows %s" %
+                                      (info.future_focus, planet.name, pid, planet.speciesName, planet.focus, new_planet.focus))
 
 
 def set_planet_protection_foci(focus_manager):
@@ -468,8 +485,8 @@ def set_planet_protection_foci(focus_manager):
                 continue
             else:
                 newplanet = universe.getPlanet(pid)
-                print ("Error: Failed setting %s for planet %s (%d) with species %s and current focus %s, but new planet copy shows %s" %
-                       (focus_manager.new_foci[pid], planet.name, pid, planet.speciesName, planet.focus, newplanet.focus))
+                print >> sys.stderr, ("Error: Failed setting %s for planet %s (%d) with species %s and current focus %s, but new planet copy shows %s" %
+                                      (focus_manager.new_foci[pid], planet.name, pid, planet.speciesName, planet.focus, newplanet.focus))
 
 
 def set_planet_happiness_foci(focus_manager):
@@ -530,7 +547,7 @@ def set_planet_industry_and_research_foci(focus_manager, priority_ratio):
                     focus_manager.bake_future_focus(pid, RESEARCH, False)
                 continue
             if adj_round == 3:  # take research at planets where can do reasonable balance
-                if has_force or (foAI.foAIstate.aggression < fo.aggression.aggressive) or (target_rp >= priority_ratio * cumulative_pp):
+                if has_force or foAI.foAIstate.character.may_dither_focus_to_gain_research() or (target_rp >= priority_ratio * cumulative_pp):
                     continue
                 pop = planet.currentMeterValue(fo.meterType.population)
                 t_pop = planet.currentMeterValue(fo.meterType.targetPopulation)
@@ -552,7 +569,7 @@ def set_planet_industry_and_research_foci(focus_manager, priority_ratio):
     got_algo = tech_is_complete("LRN_ALGO_ELEGANCE")
     for ratio, pid, info in ratios:
         if priority_ratio < (target_rp / (target_pp + 0.0001)):  # we have enough RP
-            if ratio < 1.1 and foAI.foAIstate.aggression > fo.aggression.cautious:  # but wait, RP is still super cheap relative to PP, maybe will take more RP
+            if ratio < 1.1 and foAI.foAIstate.character.may_research_heavily():  # but wait, RP is still super cheap relative to PP, maybe will take more RP
                 if priority_ratio < 1.5 * (target_rp / (target_pp + 0.0001)):  # yeah, really a glut of RP, stop taking RP
                     break
             else:  # RP not super cheap & we have enough, stop taking it
@@ -628,7 +645,7 @@ def set_planet_resource_foci():
 
         reporter.print_table(priority_ratio)
 
-        resource_timer.end()
+        resource_timer.stop_print_and_clear()
 
     Reporter.print_resource_ai_footer()
 
