@@ -3,10 +3,10 @@ The TechsListAI module provides functions that describes dependencies between
 various technologies to help the AI decide which technologies should be
 researched next.
 """
+import sys
 
 from freeorion_tools import print_error
 import freeOrionAIInterface as fo  # pylint: disable=import-error
-import sys
 
 EXOBOT_TECH_NAME = "PRO_EXOBOTS"
 
@@ -112,12 +112,14 @@ class TechGroup1(TechGroup):
             "GRO_PLANET_ECOL",
             "GRO_SUBTER_HAB",
             "LRN_ALGO_ELEGANCE",
+            "LRN_PHYS_BRAIN",
             "LRN_ARTIF_MINDS",
             "CON_ORBITAL_CON",  # not a economy tech in the strictest sense but bonus supply often equals more planets
             "PRO_ROBOTIC_PROD",
         ])
         self.weapon.extend([
             "SHP_WEAPON_1_2",
+            "SHP_FIGHTERS_1",
             "SHP_WEAPON_1_3",
             "SHP_WEAPON_1_4",
         ])
@@ -127,12 +129,13 @@ class TechGroup1(TechGroup):
         self.hull.extend([
             "SHP_MIL_ROBO_CONT",
         ])
-        # always start with the same first 6 techs
+        # always start with the same first 7 techs; leaves 2 econ, 3 weap, 1 hull
         self.enqueue(
             self.economy,
             self.economy,
             self.economy,
             self.economy,
+            self.economy,
             self.weapon,
             self.defense,
         )
@@ -142,11 +145,12 @@ class TechGroup1a(TechGroup1):
     def __init__(self):
         super(TechGroup1a, self).__init__()
         self.enqueue(
+            self.weapon,
             self.weapon,
             self.weapon,
             self.economy,
             self.economy,
-            self.hull
+            self.hull,
         )
 
 
@@ -157,7 +161,9 @@ class TechGroup1b(TechGroup1):
             self.weapon,
             self.hull,
             self.economy,
-            self.weapon
+            self.weapon,
+            self.weapon,
+            self.economy,
         )
 
 
@@ -170,6 +176,8 @@ class TechGroup1SparseA(TechGroup1):
             self.hull,
             self.weapon,
             self.weapon,
+            self.weapon,
+
             "SHP_SPACE_FLUX_DRIVE"
         )
 
@@ -184,12 +192,43 @@ class TechGroup1SparseB(TechGroup1):
             "GRO_SYMBIOTIC_BIO",
             self.weapon,
             self.weapon,
+            self.weapon,
             "PRO_ORBITAL_GEN",
             self.hull,
             "SHP_ZORTRIUM_PLATE",
             "SHP_SPACE_FLUX_DRIVE"
         )
 
+class TechGroup1SparseC(TechGroup1):
+    def __init__(self):
+        super(TechGroup1SparseC, self).__init__()
+        self.enqueue(
+            self.economy,
+            self.economy,
+            self.weapon,
+            "SHP_ORG_HULL",
+            self.weapon,
+            "PRO_NANOTECH_PROD",
+            "GRO_GENETIC_ENG",
+            "PRO_SENTIENT_AUTOMATION",
+            "GRO_GENETIC_MED",
+            "GRO_SYMBIOTIC_BIO",
+            "PRO_MICROGRAV_MAN",
+            "PRO_EXOBOTS",
+            "GRO_XENO_GENETICS",
+            "SHP_ASTEROID_HULLS",
+            "PRO_FUSION_GEN",
+            "SHP_WEAPON_2_1",
+            "SHP_ZORTRIUM_PLATE",
+            self.hull,
+            "LRN_FORCE_FIELD",
+            "SHP_WEAPON_2_2",
+            "PRO_ORBITAL_GEN",
+            "SPY_DETECT_2",
+            "SHP_SPACE_FLUX_DRIVE",
+        )
+        self.weapon = []
+
 
 class TechGroup2(TechGroup):
     def __init__(self):
@@ -220,6 +259,7 @@ class TechGroup2(TechGroup):
         self.weapon.extend([
             "SHP_WEAPON_2_1",
             "SHP_WEAPON_2_2",  # enables the building of laser based ships in ShipDesigner algorithm
+            "SHP_FIGHTERS_2",
             "SHP_WEAPON_2_3",
             "SHP_WEAPON_2_4",
         ])
@@ -242,6 +282,7 @@ class TechGroup2A(TechGroup2):
             self.economy,
             self.weapon,
             self.weapon,
+            self.weapon,
             self.economy
         )
 
@@ -262,6 +303,7 @@ class TechGroup2B(TechGroup2):
             self.defense,
             self.weapon,
             self.weapon,
+            self.weapon,
             self.economy,
             self.economy
         )
@@ -284,6 +326,7 @@ class TechGroup2SparseA(TechGroup2):
             self.weapon,
             self.defense,
             self.weapon,
+            self.weapon,
             self.weapon
         )
 
@@ -305,6 +348,7 @@ class TechGroup2SparseB(TechGroup2):
             self.defense,
             self.armor,
             self.weapon,
+            self.weapon,
             self.weapon
         )
 
@@ -320,12 +364,12 @@ class TechGroup3(TechGroup):
         ])
         self.economy.extend([
             "PRO_INDUSTRY_CENTER_I",
-            "PRO_SOL_ORB_GEN",
             "GRO_GENETIC_ENG",
             "GRO_GENETIC_MED",
             "GRO_XENO_GENETICS",
-            "PRO_INDUSTRY_CENTER_II",
             "LRN_QUANT_NET",
+            "PRO_SOL_ORB_GEN",
+            "PRO_INDUSTRY_CENTER_II",
             "GRO_XENO_HYBRIDS",
             "CON_ORBITAL_HAB",
             "CON_NDIM_STRC",
@@ -360,6 +404,7 @@ class TechGroup3(TechGroup):
         self.weapon.extend([
             "SHP_WEAPON_3_1",
             "SHP_WEAPON_3_2",
+            "SHP_FIGHTERS_3",
             "SHP_WEAPON_3_3",
             "SHP_WEAPON_3_4",
         ])
@@ -407,6 +452,7 @@ class TechGroup3A(TechGroup3):
             self.weapon,
             self.weapon,
             self.weapon,
+            self.weapon,
             self.weapon
         )
 
@@ -434,6 +480,7 @@ class TechGroup3B(TechGroup3):
             self.economy,
             self.misc,
             self.weapon,
+            self.weapon,
             self.hull,
             self.hull,
             self.misc,
@@ -462,12 +509,13 @@ class TechGroup3Sparse(TechGroup3):
         super(TechGroup3Sparse, self).__init__()
         self.enqueue(
             self.hull,
-            self.economy,
-            self.economy,
-            self.economy,
-            self.economy,
             self.defense,
             self.defense,
+            self.economy,
+            self.economy,
+            self.economy,
+            self.economy,
+            self.economy,
             self.misc,
             self.economy,
             self.weapon,
@@ -480,6 +528,7 @@ class TechGroup3Sparse(TechGroup3):
             self.economy,
             self.misc,
             self.weapon,
+            self.weapon,
             self.hull,
             self.hull,
             self.misc,
@@ -499,7 +548,6 @@ class TechGroup3Sparse(TechGroup3):
             self.defense,
             self.economy,
             self.misc,
-            self.economy
         )
 
 
@@ -521,15 +569,14 @@ class TechGroup5(TechGroup):
     def __init__(self):
         super(TechGroup5, self).__init__()
         self._tech_queue.extend([
-            "LRN_DISTRIB_THOUGHT",
             "DEF_GARRISON_4",
             "DEF_PLAN_BARRIER_SHLD_4",
             "LRN_XENOARCH",
             "LRN_GRAVITONICS",
+            "LRN_STELLAR_TOMOGRAPHY",
             "LRN_ENCLAVE_VOID",
             "SHP_MONOMOLEC_LATTICE",
             "SHP_ADV_DAM_CONT",
-            "LRN_STELLAR_TOMOGRAPHY",
             "LRN_TIME_MECH",
             "SPY_DETECT_4",
             "SHP_CONT_SYMB",
@@ -537,15 +584,17 @@ class TechGroup5(TechGroup):
             "GRO_CYBORG",
             "GRO_TERRAFORM",
             "GRO_ENERGY_META",
+            "LRN_DISTRIB_THOUGHT",
             "SHP_WEAPON_4_1",
             "SHP_WEAPON_4_2",
+            "SHP_FIGHTERS_4",
             "SHP_WEAPON_4_3",
-            "SHP_WEAPON_4_4",
             "PRO_INDUSTRY_CENTER_III",
             "PRO_SINGULAR_GEN",
             "SHP_QUANT_ENRG_MAG",
             "SHP_PLASMA_SHIELD",
             "SHP_ENRG_BOUND_MAN",
+            "SHP_WEAPON_4_4",
             "PRO_NEUTRONIUM_EXTRACTION",
             "SHP_SOLAR_CONT",
             "CON_CONC_CAMP",
@@ -575,6 +624,7 @@ def test_tech_integrity():
         TechGroup1b,
         TechGroup1SparseA,
         TechGroup1SparseB,
+        TechGroup1SparseC,
         TechGroup2,
         TechGroup2A,
         TechGroup2B,
@@ -627,13 +677,13 @@ def sparse_galaxy_techs(index):
         result += TechGroup4().get_techs()
         result += TechGroup5().get_techs()  #
     elif index == 3:
-        result = TechGroup1SparseB().get_techs()  # early org_hull
+        result = TechGroup1SparseC().get_techs()  # early org_hull
         result += TechGroup2SparseB().get_techs()
         result += TechGroup3A().get_techs()
         result += TechGroup4().get_techs()
         result += TechGroup5().get_techs()  #
     elif index == 4:
-        result = TechGroup1SparseB().get_techs()  # early _lrn_artif_minds
+        result = TechGroup1SparseC().get_techs()  # early _lrn_artif_minds
         result += TechGroup2SparseB().get_techs()
         result += TechGroup3B().get_techs()  # faster plasma weaps
         result += TechGroup4().get_techs()
