@@ -1,16 +1,59 @@
+import sys
+
 from EnumsAI import ShipRoleType, MissionType
+import AIstate
 import FleetUtilsAI
 import freeOrionAIInterface as fo  # pylint: disable=import-error
 import FreeOrionAI as foAI
-import MilitaryAI
 import MoveUtilsAI
 import PlanetUtilsAI
+import CombatRatingsAI
 from freeorion_tools import print_error
 from universe_object import Fleet, System, Planet
-
+from AIDependencies import INVALID_ID
 dumpTurn = 0
 
 
+def trooper_move_reqs_met(main_fleet_mission, order, verbose):
+    """
+    Indicates whether or not move requirements specific to invasion troopers are met for the provided mission and order.
+    :type main_fleet_mission: AIFleetMission.AIFleetMission
+    :type order: OrderMove
+    :param verbose: whether to print verbose decision details
+    :type verbose: bool
+    :rtype: bool
+    """
+    # Don't advance outside of our fleet-supply zone unless the target either has no shields at all or there
+    # is already a military fleet assigned to secure the target, and don't take final jump unless the planet is
+    # (to the AI's knowledge) down to zero shields.  Additional checks will also be done by the later
+    # generic movement code
+    system_id = order.fleet.get_system().id
+    invasion_target = main_fleet_mission.target
+    invasion_planet = invasion_target.get_object()
+    invasion_system = invasion_target.get_system()
+    supplied_systems = fo.getEmpire().fleetSupplyableSystemIDs
+    # if about to leave supply lines
+    if (order.target.id not in supplied_systems and system_id in supplied_systems):
+        if (invasion_planet.currentMeterValue(fo.meterType.maxShield) and
+                    invasion_system.id not in AIstate.militarySystemIDs):
+            if verbose:
+                print ("trooper_move_reqs_met() holding Invasion fleet %d before leaving supply "
+                       "because target (%s) has nonzero max shields and there is not yet a military fleet "
+                       "assigned to secure the target system.") % (order.fleet.id, invasion_planet)
+            return False
+        if verbose:
+            print ("trooper_move_reqs_met() allowing Invasion fleet %d to leave supply "
+                   "because target (%s) has zero max shields or there is a military fleet assigned to secure "
+                   "the target system.") % (order.fleet.id, invasion_planet)
+    # if on final leg of journey
+    if (order.target.id == invasion_system.id) and invasion_planet.currentMeterValue(fo.meterType.shield):
+        if verbose:
+            print ("trooper_move_reqs_met() holding Invasion fleet %d before final leg to target system because "
+                   "target (%s) has nonzero shields") % (order.fleet.id, invasion_planet)
+        return False
+    return True
+
+
 class AIFleetOrder(object):
     """Stores information about orders which can be executed."""
     TARGET_TYPE = None
@@ -106,32 +149,38 @@ class OrderMove(AIFleetOrder):
         if system_id == self.target.get_system().id:
             return True  # TODO: already there, but could consider retreating
 
-        fleet_rating = foAI.foAIstate.get_rating(self.fleet.id).get('overall', 0)
+        main_fleet_mission = foAI.foAIstate.get_fleet_mission(self.fleet.id)
+
+        fleet_rating = CombatRatingsAI.get_fleet_rating(self.fleet.id)
+        fleet_rating_vs_planets = CombatRatingsAI.get_fleet_rating_against_planets(self.fleet.id)
         target_sys_status = foAI.foAIstate.systemStatus.get(self.target.id, {})
         f_threat = target_sys_status.get('fleetThreat', 0)
         m_threat = target_sys_status.get('monsterThreat', 0)
         p_threat = target_sys_status.get('planetThreat', 0)
         threat = f_threat + m_threat + p_threat
-        safety_factor = MilitaryAI.get_safety_factor()
+        safety_factor = foAI.foAIstate.character.military_safety_factor()
         universe = fo.getUniverse()
-        if fleet_rating >= safety_factor * threat:
+        if main_fleet_mission.type == MissionType.INVASION and not trooper_move_reqs_met(main_fleet_mission, self, verbose):
+            return False
+        if fleet_rating >= safety_factor * threat and fleet_rating_vs_planets >= p_threat:
             return True
         elif not p_threat and self.target.id in fo.getEmpire().supplyUnobstructedSystems:
-                    return True
+            return True
         else:
             sys1 = universe.getSystem(system_id)
             sys1_name = sys1 and sys1.name or "unknown"
             target_system = self.target.get_system()
             target_system_name = (target_system and target_system.get_object().name) or "unknown"
-            my_other_fleet_rating = foAI.foAIstate.systemStatus.get(self.target.id, {}).get('myFleetRating', 0)  # TODO: adjust calc for any departing fleets
+            # TODO: adjust calc for any departing fleets
+            my_other_fleet_rating = foAI.foAIstate.systemStatus.get(self.target.id, {}).get('myFleetRating', 0)
+            my_other_fleet_rating_vs_planets = foAI.foAIstate.systemStatus.get(self.target.id, {}).get('myFleetRatingVsPlanets', 0)
             is_military = foAI.foAIstate.get_fleet_role(self.fleet.id) == MissionType.MILITARY
 
+            # TODO(Morlic): Is there any reason to add this linearly instead of using CombineRatings?
             total_rating = my_other_fleet_rating + fleet_rating
-            if any((
-                (my_other_fleet_rating > 3 * safety_factor * threat),
-                (is_military and total_rating > safety_factor * threat),
-                (is_military and total_rating > 0.8 * safety_factor * threat and fleet_rating > 0.2 * threat)
-            )):
+            total_rating_vs_planets = my_other_fleet_rating_vs_planets + fleet_rating_vs_planets
+            if (my_other_fleet_rating > 3 * safety_factor * threat or
+                    (is_military and total_rating_vs_planets > 2.5*p_threat and total_rating > safety_factor * threat)):
                 if verbose:
                     print "\tAdvancing fleet %d (rating %d) at system %d (%s) into system %d (%s) with threat %d because of sufficient empire fleet strength already at destination" % (
                         self.fleet.id, fleet_rating, system_id, sys1_name, self.target.id, target_system_name, threat)
@@ -165,7 +214,7 @@ class OrderMove(AIFleetOrder):
             if foAI.foAIstate.get_fleet_role(fleet_id) == MissionType.EXPLORATION:
                 if system_id in foAI.foAIstate.needsEmergencyExploration:
                     del foAI.foAIstate.needsEmergencyExploration[foAI.foAIstate.needsEmergencyExploration.index(system_id)]
-            self.order_issued = True
+        self.order_issued = True
 
 
 class OrderResupply(AIFleetOrder):
@@ -189,7 +238,7 @@ class OrderResupply(AIFleetOrder):
             # else:
             # if self.order_type == AIFleetOrderType.ORDER_REPAIR:
             #     fo.issueAggressionOrder(fleet_id, False)
-            start_id = [fleet.systemID, fleet.nextSystemID][fleet.systemID == -1]
+            start_id = [fleet.systemID, fleet.nextSystemID][fleet.systemID == INVALID_ID]
             dest_id = MoveUtilsAI.get_safe_path_leg_to_dest(fleet_id, start_id, system_id)
             print "fleet %d with order type(%s) sent to safe leg dest %s and ultimate dest %s" % (fleet_id, self.ORDER_NAME,
                                                                                                   PlanetUtilsAI.sys_name_ids([dest_id]),
@@ -255,7 +304,7 @@ class OrderOutpost(AIFleetOrder):
         if ship_id is None:
             ship_id = FleetUtilsAI.get_ship_id_with_role(self.fleet.id, ShipRoleType.BASE_OUTPOST)
         ship = universe.getShip(ship_id)
-        return ship is not None and self.fleet.get_system() == self.target.get_system() and ship.canColonize
+        return ship is not None and self.fleet.get_object().systemID == self.target.get_system().id and ship.canColonize
 
     def issue_order(self):
         if not super(OrderOutpost, self).issue_order():
@@ -319,8 +368,8 @@ class OrderColonize(AIFleetOrder):
         universe = fo.getUniverse()
         ship = universe.getShip(ship_id)
         if ship and not ship.canColonize:
-            print "Error: colonization fleet %d has no colony ship" % self.fleet.id
-        return ship is not None and self.fleet.get_system() == self.target.get_system() and ship.canColonize
+            print >> sys.stderr, "colonization fleet %d has no colony ship" % self.fleet.id
+        return ship is not None and self.fleet.get_object().systemID == self.target.get_system().id and ship.canColonize
 
 
 class OrderAttack(AIFleetOrder):
@@ -371,7 +420,7 @@ class OrderInvade(AIFleetOrder):
         planet = self.target.get_object()
         return all((
             ship is not None,
-            self.fleet.get_system().id == planet.systemID,
+            self.fleet.get_object().systemID == planet.systemID,
             ship.canInvade,
             not planet.currentMeterValue(fo.meterType.shield)
         ))
@@ -423,13 +472,17 @@ class OrderMilitary(AIFleetOrder):
     def is_valid(self):
         if not super(OrderMilitary, self).is_valid():
             return False
-        return self.fleet.get_object().hasArmedShips
+        thisFleet = self.fleet.get_object()
+        # TODO: consider bombardment-only fleets/orders
+        return thisFleet is not None and (thisFleet.hasArmedShips or thisFleet.hasFighterShips)
 
     def can_issue_order(self, verbose=False):
+        # TODO: consider bombardment
+        # TODO: consider simmply looking at fleet characteristics, as is done for is_valid()
         ship_id = FleetUtilsAI.get_ship_id_with_role(self.fleet.id, ShipRoleType.MILITARY)
         universe = fo.getUniverse()
         ship = universe.getShip(ship_id)
-        return ship is not None and self.fleet.get_system() == self.target and ship.isArmed
+        return ship is not None and self.fleet.get_object().systemID == self.target.id and (ship.isArmed or ship.hasFighters)
 
     def issue_order(self):
         if not super(OrderMilitary, self).issue_order():
@@ -437,7 +490,9 @@ class OrderMilitary(AIFleetOrder):
         target_sys_id = self.target.id
         fleet = self.target.get_object()
         system_status = foAI.foAIstate.systemStatus.get(target_sys_id, {})
-        if fleet and fleet.systemID == target_sys_id and not (system_status.get('fleetThreat', 0) + system_status.get('planetThreat', 0) + system_status.get('monsterThreat', 0)):
+        if all((fleet, fleet.systemID == target_sys_id, system_status.get('currently_visible', False),
+               not (system_status.get('fleetThreat', 0) + system_status.get('planetThreat', 0) +
+                        system_status.get('monsterThreat', 0)))):
             self.order_issued = True
 
 
@@ -458,7 +513,7 @@ class OrderRepair(AIFleetOrder):
         fleet = self.fleet.get_object()
         if system_id not in [fleet.systemID, fleet.nextSystemID]:
             fo.issueAggressionOrder(fleet_id, False)
-            start_id = [fleet.systemID, fleet.nextSystemID][fleet.systemID == -1]
+            start_id = [fleet.systemID, fleet.nextSystemID][fleet.systemID == INVALID_ID]
             dest_id = MoveUtilsAI.get_safe_path_leg_to_dest(fleet_id, start_id, system_id)
             print "fleet %d with order type(%s) sent to safe leg dest %s and ultimate dest %s" % (fleet_id, self.ORDER_NAME,
                                                                                                   PlanetUtilsAI.sys_name_ids([dest_id]),
