@@ -1,12 +1,16 @@
 #ifndef _ClientNetworking_h_
 #define _ClientNetworking_h_
 
-#include "Message.h"
-#include "MessageQueue.h"
+#include <boost/optional/optional_fwd.hpp>
 
-#include <boost/array.hpp>
-#include <boost/asio.hpp>
+#include <string>
+#include <vector>
+#include <memory>
+#include <chrono>
 
+#include <memory>
+
+class Message;
 
 /** Encapsulates the networking facilities of the client.  The client must
     execute its networking code in a separate thread from its main processing
@@ -21,7 +25,7 @@
     periodically request the next incoming message; the arrival of incoming
     messages is never explicitly signalled to the main thread.  The same
     applies to unintentional disconnects from the server.  The client must
-    periodically check Connected().
+    periodically check IsConnected().
 
     The ClientNetworking has three modes of operation.  First, it can discover
     FreeOrion servers on the local network; this is a blocking operation with
@@ -39,22 +43,25 @@
     even if it is not at the front of the queue at the time.  This implies
     that some response messages may be handled out of order with respect to
     regular messages, but these are in fact the desired semantics. */
-class ClientNetworking {
+class ClientNetworking : public std::enable_shared_from_this<ClientNetworking> {
 public:
     /** The type of list returned by a call to DiscoverLANServers(). */
-    typedef std::vector<std::pair<boost::asio::ip::address, std::string> >  ServerList;
-    typedef boost::array<int, 5>                                            MessageHeaderBuffer;
+    using ServerNames =  std::vector<std::string>;
 
     /** \name Structors */ //@{
-    ClientNetworking(); ///< Basic ctor.
+    ClientNetworking();
+    ~ClientNetworking();
     //@}
 
     /** \name Accessors */ //@{
-    /** Returns true iff the client is connected to the server. */
-    bool Connected() const;
+    /** Returns true iff the client is full duplex connected to the server. */
+    bool IsConnected() const;
 
-    /** Returns true iff there is at least one incoming message available. */
-    bool MessageAvailable() const;
+    /** Returns true iff the client is connected to receive from the server. */
+    bool IsRxConnected() const;
+
+    /** Returns true iff the client is connected to send to the server. */
+    bool IsTxConnected() const;
 
     /** Returns the ID of the player on this client. */
     int PlayerID() const;
@@ -69,34 +76,31 @@ public:
     /** \name Mutators */ //@{
     /** Returns a list of the addresses and names of all servers on the Local
         Area Network. */
-    ServerList DiscoverLANServers();
+    ServerNames DiscoverLANServerNames();
 
     /** Connects to the server at \a ip_address.  On failure, repeated
         attempts will be made until \a timeout seconds has elapsed. */
     bool ConnectToServer(const std::string& ip_address,
-                         boost::posix_time::seconds timeout = boost::posix_time::seconds(5));
+                         const std::chrono::milliseconds& timeout = std::chrono::seconds(10));
 
     /** Connects to the server on the client's host.  On failure, repeated
         attempts will be made until \a timeout seconds has elapsed. */
-    bool ConnectToLocalHostServer(boost::posix_time::seconds timeout =
-                                  boost::posix_time::seconds(5));
+    bool ConnectToLocalHostServer(const std::chrono::milliseconds& timeout =
+                                  std::chrono::seconds(10));
 
     /** Sends \a message to the server.  This function actually just enqueues
         the message for sending and returns immediately. */
-    void SendMessage(Message message);
+    void SendMessage(const Message& message);
 
-    /** Gets the next incoming message from the server, places it into \a
-        message, and removes it from the incoming message queue.  The function
-        assumes that there is at least one message in the incoming queue.
-        Users must call MessageAvailable() first to make sure this is the
-        case. */
-    void GetMessage(Message& message);
+    /** Return the next incoming message from the server if available or boost::none.
+        Remove the message from the incoming message queue. */
+    boost::optional<Message> GetMessage();
 
     /** Sends \a message to the server, then blocks until it sees the first
         synchronous response from the server. */
-    void SendSynchronousMessage(Message message, Message& response_message);
+    boost::optional<Message> SendSynchronousMessage(const Message& message);
 
-    /** Disconnects the client from the server. */
+    /** Disconnects the client from the server. First tries to send any pending transmit messages. */
     void DisconnectFromServer();
 
     /** Sets player ID for this client. */
@@ -107,34 +111,8 @@ public:
     //@}
 
 private:
-    void HandleException(const boost::system::system_error& error);
-    void HandleConnection(boost::asio::ip::tcp::resolver::iterator* it,
-                          boost::asio::deadline_timer* timer,
-                          const boost::system::error_code& error);
-    void CancelRetries();
-    void NetworkingThread();
-    void HandleMessageBodyRead(     boost::system::error_code error, std::size_t bytes_transferred);
-    void HandleMessageHeaderRead(   boost::system::error_code error, std::size_t bytes_transferred);
-    void AsyncReadMessage();
-    void HandleMessageWrite(        boost::system::error_code error, std::size_t bytes_transferred);
-    void AsyncWriteMessage();
-    void SendMessageImpl(Message message);
-    void DisconnectFromServerImpl();
-
-    int                             m_player_id;
-    int                             m_host_player_id;
-
-    boost::asio::io_service         m_io_service;
-    boost::asio::ip::tcp::socket    m_socket;
-    mutable boost::mutex            m_mutex;
-    MessageQueue                    m_incoming_messages; // accessed from multiple threads, but its interface is threadsafe
-    std::list<Message>              m_outgoing_messages;
-    bool                            m_connected;         // accessed from multiple threads
-    bool                            m_cancel_retries;
-
-    MessageHeaderBuffer             m_incoming_header;
-    Message                         m_incoming_message;
-    MessageHeaderBuffer             m_outgoing_header;
+    class Impl;
+    std::unique_ptr<Impl> const m_impl;
 };
 
 #endif
