@@ -17,7 +17,6 @@
 #include "../util/ScopedTimer.h"
 #include "../util/i18n.h"
 #include "../util/Version.h"
-#include <boost/lexical_cast.hpp>
 #include <boost/algorithm/string/erase.hpp>
 #include <boost/algorithm/string/predicate.hpp>
 #include <boost/serialization/deque.hpp>
@@ -139,20 +138,20 @@ bool operator!=(const Message& lhs, const Message& rhs)
 void swap(Message& lhs, Message& rhs)
 { lhs.Swap(rhs); }
 
-void BufferToHeader(const int* header_buf, Message& message) {
-    message.m_type = static_cast<Message::MessageType>(header_buf[0]);
-    message.m_sending_player = header_buf[1];
-    message.m_receiving_player = header_buf[2];
-    message.m_synchronous_response = (header_buf[3] != 0);
-    message.m_message_size = header_buf[4];
+void BufferToHeader(const Message::HeaderBuffer& buffer, Message& message) {
+    message.m_type = static_cast<Message::MessageType>(buffer[0]);
+    message.m_sending_player = buffer[1];
+    message.m_receiving_player = buffer[2];
+    message.m_synchronous_response = (buffer[3] != 0);
+    message.m_message_size = buffer[4];
 }
 
-void HeaderToBuffer(const Message& message, int* header_buf) {
-    header_buf[0] = message.Type();
-    header_buf[1] = message.SendingPlayer();
-    header_buf[2] = message.ReceivingPlayer();
-    header_buf[3] = message.SynchronousResponse();
-    header_buf[4] = message.Size();
+void HeaderToBuffer(const Message& message, Message::HeaderBuffer& buffer) {
+    buffer[0] = message.Type();
+    buffer[1] = message.SendingPlayer();
+    buffer[2] = message.ReceivingPlayer();
+    buffer[3] = message.SynchronousResponse();
+    buffer[4] = message.Size();
 }
 
 ////////////////////////////////////////////////
@@ -215,13 +214,13 @@ Message JoinGameMessage(const std::string& player_name, Networking::ClientType c
 
 Message HostIDMessage(int host_player_id) {
     return Message(Message::HOST_ID, Networking::INVALID_PLAYER_ID, Networking::INVALID_PLAYER_ID,
-                   boost::lexical_cast<std::string>(host_player_id));
+                   std::to_string(host_player_id));
 }
 
 Message GameStartMessage(int player_id, bool single_player_game, int empire_id,
                          int current_turn, const EmpireManager& empires,
                          const Universe& universe, const SpeciesManager& species,
-                         const CombatLogManager& combat_logs, const SupplyManager& supply,
+                         CombatLogManager& combat_logs, const SupplyManager& supply,
                          const std::map<int, PlayerInfo>& players,
                          const GalaxySetupData& galaxy_setup_data,
                          bool use_binary_serialization)
@@ -235,9 +234,9 @@ Message GameStartMessage(int player_id, bool single_player_game, int empire_id,
                << BOOST_SERIALIZATION_NVP(current_turn);
             GetUniverse().EncodingEmpire() = empire_id;
             oa << BOOST_SERIALIZATION_NVP(empires)
-               << BOOST_SERIALIZATION_NVP(species)
-               << BOOST_SERIALIZATION_NVP(combat_logs)
-               << BOOST_SERIALIZATION_NVP(supply);
+               << BOOST_SERIALIZATION_NVP(species);
+            combat_logs.SerializeIncompleteLogs(oa, 1);
+            oa << BOOST_SERIALIZATION_NVP(supply);
             Serialize(oa, universe);
             bool loaded_game_data = false;
             oa << BOOST_SERIALIZATION_NVP(players)
@@ -250,9 +249,9 @@ Message GameStartMessage(int player_id, bool single_player_game, int empire_id,
                << BOOST_SERIALIZATION_NVP(current_turn);
             GetUniverse().EncodingEmpire() = empire_id;
             oa << BOOST_SERIALIZATION_NVP(empires)
-               << BOOST_SERIALIZATION_NVP(species)
-               << BOOST_SERIALIZATION_NVP(combat_logs)
-               << BOOST_SERIALIZATION_NVP(supply);
+               << BOOST_SERIALIZATION_NVP(species);
+            combat_logs.SerializeIncompleteLogs(oa, 1);
+            oa << BOOST_SERIALIZATION_NVP(supply);
             Serialize(oa, universe);
             bool loaded_game_data = false;
             oa << BOOST_SERIALIZATION_NVP(players)
@@ -266,7 +265,7 @@ Message GameStartMessage(int player_id, bool single_player_game, int empire_id,
 Message GameStartMessage(int player_id, bool single_player_game, int empire_id,
                          int current_turn, const EmpireManager& empires,
                          const Universe& universe, const SpeciesManager& species,
-                         const CombatLogManager& combat_logs, const SupplyManager& supply,
+                         CombatLogManager& combat_logs, const SupplyManager& supply,
                          const std::map<int, PlayerInfo>& players,
                          const OrderSet& orders, const SaveGameUIData* ui_data,
                          const GalaxySetupData& galaxy_setup_data,
@@ -281,15 +280,15 @@ Message GameStartMessage(int player_id, bool single_player_game, int empire_id,
                << BOOST_SERIALIZATION_NVP(current_turn);
             GetUniverse().EncodingEmpire() = empire_id;
             oa << BOOST_SERIALIZATION_NVP(empires)
-               << BOOST_SERIALIZATION_NVP(species)
-               << BOOST_SERIALIZATION_NVP(combat_logs)
-               << BOOST_SERIALIZATION_NVP(supply);
+               << BOOST_SERIALIZATION_NVP(species);
+            combat_logs.SerializeIncompleteLogs(oa, 1);
+            oa << BOOST_SERIALIZATION_NVP(supply);
             Serialize(oa, universe);
             bool loaded_game_data = true;
             oa << BOOST_SERIALIZATION_NVP(players)
                << BOOST_SERIALIZATION_NVP(loaded_game_data);
             Serialize(oa, orders);
-            bool ui_data_available = (ui_data != 0);
+            bool ui_data_available = (ui_data != nullptr);
             oa << BOOST_SERIALIZATION_NVP(ui_data_available);
             if (ui_data_available)
                 oa << boost::serialization::make_nvp("ui_data", *ui_data);
@@ -303,15 +302,15 @@ Message GameStartMessage(int player_id, bool single_player_game, int empire_id,
                << BOOST_SERIALIZATION_NVP(current_turn);
             GetUniverse().EncodingEmpire() = empire_id;
             oa << BOOST_SERIALIZATION_NVP(empires)
-               << BOOST_SERIALIZATION_NVP(species)
-               << BOOST_SERIALIZATION_NVP(combat_logs)
-               << BOOST_SERIALIZATION_NVP(supply);
+               << BOOST_SERIALIZATION_NVP(species);
+            combat_logs.SerializeIncompleteLogs(oa, 1);
+            oa << BOOST_SERIALIZATION_NVP(supply);
             Serialize(oa, universe);
             bool loaded_game_data = true;
             oa << BOOST_SERIALIZATION_NVP(players)
                << BOOST_SERIALIZATION_NVP(loaded_game_data);
             Serialize(oa, orders);
-            bool ui_data_available = (ui_data != 0);
+            bool ui_data_available = (ui_data != nullptr);
             oa << BOOST_SERIALIZATION_NVP(ui_data_available);
             if (ui_data_available)
                 oa << boost::serialization::make_nvp("ui_data", *ui_data);
@@ -326,7 +325,7 @@ Message GameStartMessage(int player_id, bool single_player_game, int empire_id,
 Message GameStartMessage(int player_id, bool single_player_game, int empire_id,
                          int current_turn, const EmpireManager& empires,
                          const Universe& universe, const SpeciesManager& species,
-                         const CombatLogManager& combat_logs, const SupplyManager& supply,
+                         CombatLogManager& combat_logs, const SupplyManager& supply,
                          const std::map<int, PlayerInfo>& players,
                          const OrderSet& orders, const std::string* save_state_string,
                          const GalaxySetupData& galaxy_setup_data,
@@ -341,9 +340,9 @@ Message GameStartMessage(int player_id, bool single_player_game, int empire_id,
                << BOOST_SERIALIZATION_NVP(current_turn);
             GetUniverse().EncodingEmpire() = empire_id;
             oa << BOOST_SERIALIZATION_NVP(empires)
-               << BOOST_SERIALIZATION_NVP(species)
-               << BOOST_SERIALIZATION_NVP(combat_logs)
-               << BOOST_SERIALIZATION_NVP(supply);
+               << BOOST_SERIALIZATION_NVP(species);
+            combat_logs.SerializeIncompleteLogs(oa, 1);
+            oa << BOOST_SERIALIZATION_NVP(supply);
             Serialize(oa, universe);
             bool loaded_game_data = true;
             oa << BOOST_SERIALIZATION_NVP(players)
@@ -351,7 +350,7 @@ Message GameStartMessage(int player_id, bool single_player_game, int empire_id,
             Serialize(oa, orders);
             bool ui_data_available = false;
             oa << BOOST_SERIALIZATION_NVP(ui_data_available);
-            bool save_state_string_available = (save_state_string != 0);
+            bool save_state_string_available = (save_state_string != nullptr);
             oa << BOOST_SERIALIZATION_NVP(save_state_string_available);
             if (save_state_string_available)
                 oa << boost::serialization::make_nvp("save_state_string", *save_state_string);
@@ -363,9 +362,9 @@ Message GameStartMessage(int player_id, bool single_player_game, int empire_id,
                << BOOST_SERIALIZATION_NVP(current_turn);
             GetUniverse().EncodingEmpire() = empire_id;
             oa << BOOST_SERIALIZATION_NVP(empires)
-               << BOOST_SERIALIZATION_NVP(species)
-               << BOOST_SERIALIZATION_NVP(combat_logs)
-               << BOOST_SERIALIZATION_NVP(supply);
+               << BOOST_SERIALIZATION_NVP(species);
+            combat_logs.SerializeIncompleteLogs(oa, 1);
+            oa << BOOST_SERIALIZATION_NVP(supply);
             Serialize(oa, universe);
             bool loaded_game_data = true;
             oa << BOOST_SERIALIZATION_NVP(players)
@@ -373,7 +372,7 @@ Message GameStartMessage(int player_id, bool single_player_game, int empire_id,
             Serialize(oa, orders);
             bool ui_data_available = false;
             oa << BOOST_SERIALIZATION_NVP(ui_data_available);
-            bool save_state_string_available = (save_state_string != 0);
+            bool save_state_string_available = (save_state_string != nullptr);
             oa << BOOST_SERIALIZATION_NVP(save_state_string_available);
             if (save_state_string_available)
                 oa << boost::serialization::make_nvp("save_state_string", *save_state_string);
@@ -422,7 +421,7 @@ Message PlayerStatusMessage(int player_id, int about_player_id, Message::PlayerS
 
 Message TurnUpdateMessage(int player_id, int empire_id, int current_turn,
                           const EmpireManager& empires, const Universe& universe,
-                          const SpeciesManager& species, const CombatLogManager& combat_logs,
+                          const SpeciesManager& species, CombatLogManager& combat_logs,
                           const SupplyManager& supply,
                           const std::map<int, PlayerInfo>& players,
                           bool use_binary_serialization)
@@ -432,11 +431,11 @@ Message TurnUpdateMessage(int player_id, int empire_id, int current_turn,
         if (use_binary_serialization) {
             freeorion_bin_oarchive oa(os);
             GetUniverse().EncodingEmpire() = empire_id;
-            oa << BOOST_SERIALIZATION_NVP(current_turn)
-               << BOOST_SERIALIZATION_NVP(empires)
-               << BOOST_SERIALIZATION_NVP(species)
-               << BOOST_SERIALIZATION_NVP(combat_logs)
-               << BOOST_SERIALIZATION_NVP(supply);
+            oa << BOOST_SERIALIZATION_NVP(current_turn);
+            oa << BOOST_SERIALIZATION_NVP(empires);
+            oa << BOOST_SERIALIZATION_NVP(species);
+            combat_logs.SerializeIncompleteLogs(oa, 1);
+            oa << BOOST_SERIALIZATION_NVP(supply);
             Serialize(oa, universe);
             oa << BOOST_SERIALIZATION_NVP(players);
         } else {
@@ -444,9 +443,9 @@ Message TurnUpdateMessage(int player_id, int empire_id, int current_turn,
             GetUniverse().EncodingEmpire() = empire_id;
             oa << BOOST_SERIALIZATION_NVP(current_turn)
                << BOOST_SERIALIZATION_NVP(empires)
-               << BOOST_SERIALIZATION_NVP(species)
-               << BOOST_SERIALIZATION_NVP(combat_logs)
-               << BOOST_SERIALIZATION_NVP(supply);
+               << BOOST_SERIALIZATION_NVP(species);
+            combat_logs.SerializeIncompleteLogs(oa, 1);
+            oa << BOOST_SERIALIZATION_NVP(supply);
             Serialize(oa, universe);
             oa << BOOST_SERIALIZATION_NVP(players);
         }
@@ -517,7 +516,7 @@ Message RequestNewObjectIDMessage(int sender)
 
 Message DispatchObjectIDMessage(int player_id, int new_id) {
     return Message(Message::DISPATCH_NEW_OBJECT_ID, Networking::INVALID_PLAYER_ID, player_id,
-                   boost::lexical_cast<std::string>(new_id), true);
+                   std::to_string(new_id), true);
 }
 
 Message RequestNewDesignIDMessage(int sender)
@@ -525,7 +524,7 @@ Message RequestNewDesignIDMessage(int sender)
 
 Message DispatchDesignIDMessage(int player_id, int new_id) {
     return Message(Message::DISPATCH_NEW_DESIGN_ID, Networking::INVALID_PLAYER_ID, player_id,
-                   boost::lexical_cast<std::string>(new_id), true);
+                   std::to_string(new_id), true);
 }
 
 Message HostSaveGameInitiateMessage(int sender, const std::string& filename)
@@ -584,6 +583,9 @@ Message EndGameMessage(int receiver, Message::EndGameReason reason,
     return Message(Message::END_GAME, Networking::INVALID_PLAYER_ID, receiver, os.str());
 }
 
+Message AIEndGameAcknowledgeMessage(int sender)
+{ return Message(Message::AI_END_GAME_ACK, sender, Networking::INVALID_PLAYER_ID, DUMMY_EMPTY_MESSAGE); }
+
 Message ModeratorActionMessage(int sender, const Moderator::ModeratorAction& action) {
     std::ostringstream os;
     {
@@ -611,6 +613,20 @@ Message DispatchSavePreviewsMessage(int receiver, const PreviewInformation& prev
     return Message(Message::DISPATCH_SAVE_PREVIEWS, Networking::INVALID_PLAYER_ID, receiver, os.str(), true);
 }
 
+Message RequestCombatLogsMessage(int sender, const std::vector<int>& ids) {
+    std::ostringstream os;
+    freeorion_xml_oarchive oa(os);
+    oa << BOOST_SERIALIZATION_NVP(ids);
+    return Message(Message::REQUEST_COMBAT_LOGS, sender, Networking::INVALID_PLAYER_ID, os.str());
+}
+
+Message DispatchCombatLogsMessage(int receiver, const std::vector<std::pair<int, const CombatLog>>& logs) {
+    std::ostringstream os;
+    freeorion_xml_oarchive oa(os);
+    oa << BOOST_SERIALIZATION_NVP(logs);
+    return Message(Message::DISPATCH_COMBAT_LOGS, Networking::INVALID_PLAYER_ID, receiver, os.str(), true);
+}
+
 ////////////////////////////////////////////////
 // Multiplayer Lobby Message named ctors
 ////////////////////////////////////////////////
@@ -645,14 +661,14 @@ Message StartMPGameMessage(int player_id)
 ////////////////////////////////////////////////
 // Message data extractors
 ////////////////////////////////////////////////
-void ExtractMessageData(const Message& msg, std::string& problem, bool& fatal) {
+void ExtractErrorMessageData(const Message& msg, std::string& problem, bool& fatal) {
     try {
         std::istringstream is(msg.Text());
         freeorion_xml_iarchive ia(is);
         ia >> BOOST_SERIALIZATION_NVP(problem)
            >> BOOST_SERIALIZATION_NVP(fatal);
     } catch (const std::exception& err) {
-        ErrorLogger() << "ExtractMessageData(const Message& msg, std::string& problem, bool& fatal) failed!  Message:\n"
+        ErrorLogger() << "ExtractErrorMessageData(const Message& msg, std::string& problem, bool& fatal) failed!  Message:\n"
                       << msg.Text() << "\n"
                       << "Error: " << err.what();
         problem = UserStringNop("SERVER_MESSAGE_NOT_UNDERSTOOD");
@@ -660,7 +676,7 @@ void ExtractMessageData(const Message& msg, std::string& problem, bool& fatal) {
     }
 }
 
-void ExtractMessageData(const Message& msg, std::string& host_player_name, std::string& client_version_string) {
+void ExtractHostMPGameMessageData(const Message& msg, std::string& host_player_name, std::string& client_version_string) {
     try {
         std::istringstream is(msg.Text());
         freeorion_xml_iarchive ia(is);
@@ -668,28 +684,28 @@ void ExtractMessageData(const Message& msg, std::string& host_player_name, std::
            >> BOOST_SERIALIZATION_NVP(client_version_string);
 
     } catch (const std::exception& err) {
-        ErrorLogger() << "ExtractMessageData(const Message& msg, std::string& host_player_name, std::string& client_version_string) failed!  Message:\n"
+        ErrorLogger() << "ExtractHostMPGameMessageData(const Message& msg, std::string& host_player_name, std::string& client_version_string) failed!  Message:\n"
                       << msg.Text() << "\n"
                       << "Error: " << err.what();
         throw err;
     }
 }
 
-void ExtractMessageData(const Message& msg, MultiplayerLobbyData& lobby_data) {
+void ExtractLobbyUpdateMessageData(const Message& msg, MultiplayerLobbyData& lobby_data) {
     try {
         std::istringstream is(msg.Text());
         freeorion_xml_iarchive ia(is);
         ia >> BOOST_SERIALIZATION_NVP(lobby_data);
 
     } catch (const std::exception& err) {
-        ErrorLogger() << "ExtractMessageData(const Message& msg, MultiplayerLobbyData& lobby_data) failed!  Message:\n"
+        ErrorLogger() << "ExtractLobbyUpdateMessageData(const Message& msg, MultiplayerLobbyData& lobby_data) failed!  Message:\n"
                       << msg.Text() << "\n"
                       << "Error: " << err.what();
         throw err;
     }
 }
 
-void ExtractMessageData(const Message& msg, bool& single_player_game, int& empire_id, int& current_turn,
+void ExtractGameStartMessageData(const Message& msg, bool& single_player_game, int& empire_id, int& current_turn,
                         EmpireManager& empires, Universe& universe, SpeciesManager& species, CombatLogManager& combat_logs,
                         SupplyManager& supply,
                         std::map<int, PlayerInfo>& players, OrderSet& orders, bool& loaded_game_data, bool& ui_data_available,
@@ -709,15 +725,15 @@ void ExtractMessageData(const Message& msg, bool& single_player_game, int& empir
 
             boost::timer deserialize_timer;
             ia >> BOOST_SERIALIZATION_NVP(empires);
-            DebugLogger() << "ExtractMessage empire deserialization time " << (deserialize_timer.elapsed() * 1000.0);
+            DebugLogger() << "ExtractGameStartMessage empire deserialization time " << (deserialize_timer.elapsed() * 1000.0);
 
-            ia >> BOOST_SERIALIZATION_NVP(species)
-               >> BOOST_SERIALIZATION_NVP(combat_logs)
-               >> BOOST_SERIALIZATION_NVP(supply);
+            ia >> BOOST_SERIALIZATION_NVP(species);
+            combat_logs.SerializeIncompleteLogs(ia, 1);
+            ia >> BOOST_SERIALIZATION_NVP(supply);
 
             deserialize_timer.restart();
             Deserialize(ia, universe);
-            DebugLogger() << "ExtractMessage universe deserialization time " << (deserialize_timer.elapsed() * 1000.0);
+            DebugLogger() << "ExtractGameStartMessage universe deserialization time " << (deserialize_timer.elapsed() * 1000.0);
 
 
             ia >> BOOST_SERIALIZATION_NVP(players)
@@ -748,15 +764,15 @@ void ExtractMessageData(const Message& msg, bool& single_player_game, int& empir
 
             boost::timer deserialize_timer;
             ia >> BOOST_SERIALIZATION_NVP(empires);
-            DebugLogger() << "ExtractMessage empire deserialization time " << (deserialize_timer.elapsed() * 1000.0);
+            DebugLogger() << "ExtractGameStartMessage empire deserialization time " << (deserialize_timer.elapsed() * 1000.0);
 
-            ia >> BOOST_SERIALIZATION_NVP(species)
-               >> BOOST_SERIALIZATION_NVP(combat_logs)
-               >> BOOST_SERIALIZATION_NVP(supply);
+            ia >> BOOST_SERIALIZATION_NVP(species);
+            combat_logs.SerializeIncompleteLogs(ia, 1);
+            ia >> BOOST_SERIALIZATION_NVP(supply);
 
             deserialize_timer.restart();
             Deserialize(ia, universe);
-            DebugLogger() << "ExtractMessage universe deserialization time " << (deserialize_timer.elapsed() * 1000.0);
+            DebugLogger() << "ExtractGameStartMessage universe deserialization time " << (deserialize_timer.elapsed() * 1000.0);
 
 
             ia >> BOOST_SERIALIZATION_NVP(players)
@@ -777,16 +793,16 @@ void ExtractMessageData(const Message& msg, bool& single_player_game, int& empir
         }
 
     } catch (const std::exception& err) {
-        ErrorLogger() << "ExtractMessageData(...) failed!  Message probably long, so not outputting to log.\n"
+        ErrorLogger() << "ExtractGameStartMessageData(...) failed!  Message probably long, so not outputting to log.\n"
                       << "Error: " << err.what();
         throw err;
     }
 }
 
-void ExtractMessageData(const Message& msg, std::string& player_name, Networking::ClientType& client_type,
+void ExtractJoinGameMessageData(const Message& msg, std::string& player_name, Networking::ClientType& client_type,
                         std::string& version_string)
 {
-    DebugLogger() << "ExtractMessageData() from " << player_name << " client type " << client_type;
+    DebugLogger() << "ExtractJoinGameMessageData() from " << player_name << " client type " << client_type;
     try {
         std::istringstream is(msg.Text());
         freeorion_xml_iarchive ia(is);
@@ -795,7 +811,7 @@ void ExtractMessageData(const Message& msg, std::string& player_name, Networking
            >> BOOST_SERIALIZATION_NVP(version_string);
 
     } catch (const std::exception& err) {
-        ErrorLogger() << "ExtractMessageData(const Message& msg, std::string& player_name, "
+        ErrorLogger() << "ExtractJoinGameMessageData(const Message& msg, std::string& player_name, "
                       << "Networking::ClientType client_type, std::string& version_string) failed!  Message:\n"
                       << msg.Text() << "\n"
                       << "Error: " << err.what();
@@ -803,23 +819,23 @@ void ExtractMessageData(const Message& msg, std::string& player_name, Networking
     }
 }
 
-void ExtractMessageData(const Message& msg, OrderSet& orders) {
+void ExtractTurnOrdersMessageData(const Message& msg, OrderSet& orders) {
     try {
         std::istringstream is(msg.Text());
         freeorion_xml_iarchive ia(is);
         Deserialize(ia, orders);
 
     } catch (const std::exception& err) {
-        ErrorLogger() << "ExtractMessageData(const Message& msg, OrderSet& orders) failed!  Message:\n"
+        ErrorLogger() << "ExtractTurnOrdersMessageData(const Message& msg, OrderSet& orders) failed!  Message:\n"
                       << msg.Text() << "\n"
                       << "Error: " << err.what();
         throw err;
     }
 }
 
-void ExtractMessageData(const Message& msg, int empire_id, int& current_turn, EmpireManager& empires, Universe& universe,
-                        SpeciesManager& species, CombatLogManager& combat_logs, SupplyManager& supply,
-                        std::map<int, PlayerInfo>& players)
+void ExtractTurnUpdateMessageData(const Message& msg, int empire_id, int& current_turn, EmpireManager& empires,
+                                  Universe& universe, SpeciesManager& species, CombatLogManager& combat_logs,
+                                  SupplyManager& supply, std::map<int, PlayerInfo>& players)
 {
     try {
         ScopedTimer timer("Turn Update Unpacking", true);
@@ -831,9 +847,9 @@ void ExtractMessageData(const Message& msg, int empire_id, int& current_turn, Em
             GetUniverse().EncodingEmpire() = empire_id;
             ia >> BOOST_SERIALIZATION_NVP(current_turn)
                >> BOOST_SERIALIZATION_NVP(empires)
-               >> BOOST_SERIALIZATION_NVP(species)
-               >> BOOST_SERIALIZATION_NVP(combat_logs)
-               >> BOOST_SERIALIZATION_NVP(supply);
+               >> BOOST_SERIALIZATION_NVP(species);
+            combat_logs.SerializeIncompleteLogs(ia, 1);
+            ia >> BOOST_SERIALIZATION_NVP(supply);
             Deserialize(ia, universe);
             ia >> BOOST_SERIALIZATION_NVP(players);
 
@@ -844,21 +860,21 @@ void ExtractMessageData(const Message& msg, int empire_id, int& current_turn, Em
             GetUniverse().EncodingEmpire() = empire_id;
             ia >> BOOST_SERIALIZATION_NVP(current_turn)
                >> BOOST_SERIALIZATION_NVP(empires)
-               >> BOOST_SERIALIZATION_NVP(species)
-               >> BOOST_SERIALIZATION_NVP(combat_logs)
-               >> BOOST_SERIALIZATION_NVP(supply);
+               >> BOOST_SERIALIZATION_NVP(species);
+            combat_logs.SerializeIncompleteLogs(ia, 1);
+            ia >> BOOST_SERIALIZATION_NVP(supply);
             Deserialize(ia, universe);
             ia >> BOOST_SERIALIZATION_NVP(players);
         }
 
     } catch (const std::exception& err) {
-        ErrorLogger() << "ExtractMessageData(...) failed!  Message probably long, so not outputting to log.\n"
+        ErrorLogger() << "ExtractTurnUpdateMessageData(...) failed!  Message probably long, so not outputting to log.\n"
                       << "Error: " << err.what();
         throw err;
     }
 }
 
-void ExtractMessageData(const Message& msg, int empire_id, Universe& universe) {
+void ExtractTurnPartialUpdateMessageData(const Message& msg, int empire_id, Universe& universe) {
     try {
         ScopedTimer timer("Mid Turn Update Unpacking", true);
 
@@ -878,14 +894,15 @@ void ExtractMessageData(const Message& msg, int empire_id, Universe& universe) {
         }
 
     } catch (const std::exception& err) {
-        ErrorLogger() << "ExtractMessageData(...) failed!  Message probably long, so not outputting to log.\n"
+        ErrorLogger() << "ExtracturnPartialUpdateMessageData(...) failed!  Message probably long, so not outputting to log.\n"
                       << "Error: " << err.what();
         throw err;
     }
 }
 
-void ExtractMessageData(const Message& msg, OrderSet& orders, bool& ui_data_available, SaveGameUIData& ui_data,
-                        bool& save_state_string_available, std::string& save_state_string)
+void ExtractClientSaveDataMessageData(const Message& msg, OrderSet& orders, bool& ui_data_available,
+                                      SaveGameUIData& ui_data, bool& save_state_string_available,
+                                      std::string& save_state_string)
 {
     try {
         std::istringstream is(msg.Text());
@@ -906,27 +923,27 @@ void ExtractMessageData(const Message& msg, OrderSet& orders, bool& ui_data_avai
         }
 
     } catch (const std::exception& err) {
-        ErrorLogger() << "ExtractMessageData(...) failed!  Message probably long, so not outputting to log.\n"
+        ErrorLogger() << "ExtractClientSaveDataMessageData(...) failed!  Message probably long, so not outputting to log.\n"
                       << "Error: " << err.what();
         throw err;
     }
 }
 
-void ExtractMessageData(const Message& msg, Message::TurnProgressPhase& phase_id) {
+void ExtractTurnProgressMessageData(const Message& msg, Message::TurnProgressPhase& phase_id) {
     try {
         std::istringstream is(msg.Text());
         freeorion_xml_iarchive ia(is);
         ia >> BOOST_SERIALIZATION_NVP(phase_id);
 
     } catch (const std::exception& err) {
-        ErrorLogger() << "ExtractMessageData(const Message& msg, Message::TurnProgressPhase& phase_id) failed!  Message:\n"
+        ErrorLogger() << "ExtractTurnProgressMessageData(const Message& msg, Message::TurnProgressPhase& phase_id) failed!  Message:\n"
                       << msg.Text() << "\n"
                       << "Error: " << err.what();
         throw err;
     }
 }
 
-void ExtractMessageData(const Message& msg, int& about_player_id, Message::PlayerStatus& status) {
+void ExtractPlayerStatusMessageData(const Message& msg, int& about_player_id, Message::PlayerStatus& status) {
     try {
         std::istringstream is(msg.Text());
         freeorion_xml_iarchive ia(is);
@@ -934,14 +951,14 @@ void ExtractMessageData(const Message& msg, int& about_player_id, Message::Playe
            >> BOOST_SERIALIZATION_NVP(status);
  
     } catch (const std::exception& err) {
-        ErrorLogger() << "ExtractMessageData(const Message& msg, int& about_player_id, Message::PlayerStatus&) failed!  Message:\n"
+        ErrorLogger() << "ExtractPlayerStatusMessageData(const Message& msg, int& about_player_id, Message::PlayerStatus&) failed!  Message:\n"
                       << msg.Text() << "\n"
                       << "Error: " << err.what();
         throw err;
     }
 }
 
-void ExtractMessageData(const Message& msg, SinglePlayerSetupData& setup_data, std::string& client_version_string) {
+void ExtractHostSPGameMessageData(const Message& msg, SinglePlayerSetupData& setup_data, std::string& client_version_string) {
     try {
         std::istringstream is(msg.Text());
         freeorion_xml_iarchive ia(is);
@@ -949,14 +966,14 @@ void ExtractMessageData(const Message& msg, SinglePlayerSetupData& setup_data, s
            >> BOOST_SERIALIZATION_NVP(client_version_string);
 
     } catch (const std::exception& err) {
-        ErrorLogger() << "ExtractMessageData(const Message& msg, SinglePlayerSetupData& setup_data, std::string& client_version_string) failed!  Message:\n"
+        ErrorLogger() << "ExtractHostSPGameMessageData(const Message& msg, SinglePlayerSetupData& setup_data, std::string& client_version_string) failed!  Message:\n"
                       << msg.Text() << "\n"
                       << "Error: " << err.what();
         throw err;
     }
 }
 
-void ExtractMessageData(const Message& msg, Message::EndGameReason& reason, std::string& reason_player_name) {
+void ExtractEndGameMessageData(const Message& msg, Message::EndGameReason& reason, std::string& reason_player_name) {
     try {
         std::istringstream is(msg.Text());
         freeorion_xml_iarchive ia(is);
@@ -964,7 +981,7 @@ void ExtractMessageData(const Message& msg, Message::EndGameReason& reason, std:
            >> BOOST_SERIALIZATION_NVP(reason_player_name);
 
     } catch (const std::exception& err) {
-        ErrorLogger() << "ExtractMessageData(const Message& msg, Message::EndGameReason& reason, "
+        ErrorLogger() << "ExtractEndGameMessageData(const Message& msg, Message::EndGameReason& reason, "
                       << "std::string& reason_player_name) failed!  Message:\n"
                       << msg.Text() << "\n"
                       << "Error: " << err.what();
@@ -972,44 +989,28 @@ void ExtractMessageData(const Message& msg, Message::EndGameReason& reason, std:
     }
 }
 
-void ExtractMessageData(const Message& msg, Moderator::ModeratorAction*& mod_action) {
+void ExtractModeratorActionMessageData(const Message& msg, Moderator::ModeratorAction*& mod_action) {
     try {
         std::istringstream is(msg.Text());
         freeorion_xml_iarchive ia(is);
         ia >> BOOST_SERIALIZATION_NVP(mod_action);
 
     } catch (const std::exception& err) {
-        ErrorLogger() << "ExtractMessageData(const Message& msg, Moderator::ModeratorAction& mod_act) "
+        ErrorLogger() << "ExtractModeratorActionMessageData(const Message& msg, Moderator::ModeratorAction& mod_act) "
                       << "failed!  Message:\n"
                       << msg.Text() << "\n"
                       << "Error: " << err.what();
     }
 }
 
-void ExtractMessageData(const Message& msg, int& empire_id, std::string& empire_name) {
-    try {
-        std::istringstream is(msg.Text());
-        freeorion_xml_iarchive ia(is);
-        ia >> BOOST_SERIALIZATION_NVP(empire_id)
-           >> BOOST_SERIALIZATION_NVP(empire_name);
-
-    } catch (const std::exception& err) {
-        ErrorLogger() << "ExtractMessageData(const Message& msg, int empire_id, std::string& "
-                      << "empire_name) failed!  Message:\n"
-                      << msg.Text() << "\n"
-                      << "Error: " << err.what();
-        throw err;
-    }
-}
-
-void ExtractMessageData(const Message& msg, DiplomaticMessage& diplo_message) {
+void ExtractDiplomacyMessageData(const Message& msg, DiplomaticMessage& diplo_message) {
     try {
         std::istringstream is(msg.Text());
         freeorion_xml_iarchive ia(is);
         ia >> BOOST_SERIALIZATION_NVP(diplo_message);
 
     } catch (const std::exception& err) {
-        ErrorLogger() << "ExtractMessageData(const Message& msg, DiplomaticMessage& "
+        ErrorLogger() << "ExtractDiplomacyMessageData(const Message& msg, DiplomaticMessage& "
                       << "diplo_message) failed!  Message:\n"
                       << msg.Text() << "\n"
                       << "Error: " << err.what();
@@ -1017,7 +1018,7 @@ void ExtractMessageData(const Message& msg, DiplomaticMessage& diplo_message) {
     }
 }
 
-void ExtractMessageData(const Message& msg, DiplomaticStatusUpdateInfo& diplo_update) {
+void ExtractDiplomaticStatusMessageData(const Message& msg, DiplomaticStatusUpdateInfo& diplo_update) {
     try {
         std::istringstream is(msg.Text());
         freeorion_xml_iarchive ia(is);
@@ -1026,31 +1027,31 @@ void ExtractMessageData(const Message& msg, DiplomaticStatusUpdateInfo& diplo_up
            >> BOOST_SERIALIZATION_NVP(diplo_update.diplo_status);
 
     } catch (const std::exception& err) {
-        ErrorLogger() << "ExtractMessageData(const Message& msg, DiplomaticStatusUpdate& diplo_update) failed!  Message:\n"
+        ErrorLogger() << "ExtractDiplomaticStatusMessageData(const Message& msg, DiplomaticStatusUpdate& diplo_update) failed!  Message:\n"
                       << msg.Text() << "\n"
                       << "Error: " << err.what();
         throw err;
     }
 }
 
-void ExtractMessageData(const Message& msg, std::string& directory)
+void ExtractRequestSavePreviewsMessageData(const Message& msg, std::string& directory)
 { directory = msg.Text(); }
 
-void ExtractMessageData(const Message& msg, PreviewInformation& previews) {
+void ExtractDispatchSavePreviewsMessageData(const Message& msg, PreviewInformation& previews) {
     try {
         std::istringstream is(msg.Text());
         freeorion_xml_iarchive ia(is);
         ia >> BOOST_SERIALIZATION_NVP(previews);
 
     } catch(const std::exception& err) {
-        ErrorLogger() << "ExtractMessageData(const Message& msg, PreviewInformation& previews) failed!  Message:\n"
+        ErrorLogger() << "ExtractDispatchSavePreviewsMessageData(const Message& msg, PreviewInformation& previews) failed!  Message:\n"
                       << msg.Text() << "\n"
                       << "Error: " << err.what();
         throw err;
     }
 }
 
-FO_COMMON_API void ExtractMessageData(const Message& msg, std::string& save_filename, int& bytes_written) {
+FO_COMMON_API void ExtractServerSaveGameCompleteMessageData(const Message& msg, std::string& save_filename, int& bytes_written) {
     try {
         std::istringstream is(msg.Text());
         freeorion_xml_iarchive ia(is);
@@ -1058,9 +1059,38 @@ FO_COMMON_API void ExtractMessageData(const Message& msg, std::string& save_file
            >> BOOST_SERIALIZATION_NVP(bytes_written);
 
     } catch(const std::exception& err) {
-        ErrorLogger() << "ExtractMessageData(const Message& msg, std::string& save_filename, int& bytes_written) failed!  Message:\n"
+        ErrorLogger() << "ExtractServerSaveGameCompleteServerSaveGameCompleteMessageData(const Message& msg, std::string& save_filename, int& bytes_written) failed!  Message:\n"
                       << msg.Text() << "\n"
                       << "Error: " << err.what();
         throw err;
     }
 }
+
+FO_COMMON_API void ExtractRequestCombatLogsMessageData(const Message& msg, std::vector<int>& ids) {
+    try {
+        std::istringstream is(msg.Text());
+        freeorion_xml_iarchive ia(is);
+        ia >> BOOST_SERIALIZATION_NVP(ids);
+    } catch(const std::exception& err) {
+        ErrorLogger() << "ExtractRequestCombatLogMessageData(const Message& msg, std::vector<int>& ids) failed!  Message:\n"
+                      << msg.Text() << "\n"
+                      << "Error: " << err.what();
+        throw err;
+    }
+}
+
+FO_COMMON_API void ExtractDispatchCombatLogsMessageData(
+    const Message& msg, std::vector<std::pair<int, CombatLog>>& logs)
+{
+    try {
+        std::istringstream is(msg.Text());
+        freeorion_xml_iarchive ia(is);
+        ia >> BOOST_SERIALIZATION_NVP(logs);
+    } catch(const std::exception& err) {
+        ErrorLogger() << "ExtractDispatchCombatLogMessageData(const Message& msg, std::vector<std::pair<int, const CombatLog&>>& logs) failed!  Message:\n"
+                      << msg.Text() << "\n"
+                      << "Error: " << err.what();
+        throw err;
+    }
+
+}
