@@ -3,6 +3,7 @@
 
 #include <boost/thread/condition.hpp>
 #include <boost/thread/mutex.hpp>
+#include <boost/optional/optional_fwd.hpp>
 
 #include <list>
 
@@ -15,7 +16,7 @@ class Message;
 class FO_COMMON_API MessageQueue
 {
 public:
-    MessageQueue(boost::mutex& monitor);
+    MessageQueue(boost::mutex& monitor, const bool& rx_connected);
 
     /** Returns true iff the queue is empty. */
     bool Empty() const;
@@ -29,17 +30,22 @@ public:
     /** Adds \a message to the end of the queue. */
     void PushBack(Message& message);
 
-    /** Returns the front message in the queue. */
-    void PopFront(Message& message);
+    /** Return and remove the first message in the queue. */
+    boost::optional<Message> PopFront();
 
-    /** Returns the first synchronous repsonse message in the queue.  If no such message is found, this function blocks
-        the calling thread until a synchronous response element is added. */
-    void EraseFirstSynchronousResponse(Message& message);
+    /** Indicates that no more new message will be added to the queue. This causes any pending
+        GetFirstSynchronousMessage() to return boost::none.*/
+    void RxDisconnected();
+
+    /** Return and remove the first synchronous repsonse message from the queue or return
+        boost::none if there is no synchronous message and the queue has stopped growwing. */
+    boost::optional<Message> GetFirstSynchronousMessage();
 
 private:
     std::list<Message> m_queue;
     boost::condition   m_have_synchronous_response;
     boost::mutex&      m_monitor;
+    const bool&        m_rx_connected;
 };
 
 
