@@ -3,13 +3,12 @@
 
 #include "Message.h"
 
-#include <boost/array.hpp>
 #include <boost/asio.hpp>
-#include <boost/enable_shared_from_this.hpp>
 #include <boost/function.hpp>
 #include <boost/iterator/filter_iterator.hpp>
 #include <boost/signals2/signal.hpp>
 
+#include <memory>
 #include <queue>
 #include <set>
 
@@ -17,7 +16,7 @@
 class DiscoveryServer;
 class PlayerConnection;
 
-typedef boost::shared_ptr<PlayerConnection> PlayerConnectionPtr;
+typedef std::shared_ptr<PlayerConnection> PlayerConnectionPtr;
 typedef boost::function<void (Message, PlayerConnectionPtr)> MessageAndConnectionFn;
 typedef boost::function<void (PlayerConnectionPtr)> ConnectionFn;
 typedef boost::function<void ()> NullaryFn;
@@ -39,13 +38,12 @@ public:
     typedef boost::filter_iterator<EstablishedPlayer, PlayerConnections::const_iterator>    const_established_iterator;
 
     /** \name Structors */ //@{
-    /** Basic ctor. */
     ServerNetworking(boost::asio::io_service& io_service,
                      MessageAndConnectionFn nonplayer_message_callback,
                      MessageAndConnectionFn player_message_callback,
                      ConnectionFn disconnected_callback);
 
-    ~ServerNetworking(); //< Dtor.
+    ~ServerNetworking();
     //@}
 
     /** \name Accessors */ //@{
@@ -90,11 +88,11 @@ public:
     //@}
 
     /** \name Mutators */ //@{
-    /** Sends message \a message to \a player_connection. */
-    void SendMessage(const Message& message, PlayerConnectionPtr player_connection);
+    /** Sends a synchronous message \a message to \a player_connection and returns true on success. */
+    bool SendMessage(const Message& message, PlayerConnectionPtr player_connection);
 
-    /** Sends message \a message to the player indicated in the message. */
-    void SendMessage(const Message& message);
+    /** Sends a synchronous message \a message to the player indicated in the message and returns true on success. */
+    bool SendMessage(const Message& message);
 
     /** Disconnects the server from player \a id. */
     void Disconnect(int id);
@@ -159,7 +157,7 @@ private:
     server as an actual player in a game, EstablishPlayer() should be called.
     This establishes the aforementioned properties. */
 class PlayerConnection :
-    public boost::enable_shared_from_this<PlayerConnection>
+    public std::enable_shared_from_this<PlayerConnection>
 {
 public:
     /** \name Structors */ //@{
@@ -167,7 +165,7 @@ public:
     //@}
 
     /** \name Accessors */ //@{
-    /** Returns true iff EstablishPlayer() has been called on this
+    /** Returns true if EstablishPlayer() successfully has been called on this
         connection. */
     bool EstablishedPlayer() const;
 
@@ -199,8 +197,8 @@ public:
     /** Starts the connection reading incoming messages on its socket. */
     void Start();
 
-    /** Sends \a message to out on the connection. */
-    void SendMessage(const Message& message);
+    /** Sends \a synchronous message to out on the connection and return true on success. */
+    bool SendMessage(const Message& message);
 
     /** Establishes a connection as a player with a specific name and id.
         This function must only be called once. */
@@ -220,16 +218,18 @@ public:
                   MessageAndConnectionFn player_message_callback, ConnectionFn disconnected_callback);
 
 private:
-    typedef boost::array<int, 5> MessageHeaderBuffer;
 
     PlayerConnection(boost::asio::io_service& io_service, MessageAndConnectionFn nonplayer_message_callback,
                      MessageAndConnectionFn player_message_callback, ConnectionFn disconnected_callback);
     void HandleMessageBodyRead(boost::system::error_code error, std::size_t bytes_transferred);
     void HandleMessageHeaderRead(boost::system::error_code error, std::size_t bytes_transferred);
     void AsyncReadMessage();
+    bool SyncWriteMessage(const Message& message);
+    void AsyncErrorHandler(boost::system::error_code handled_error, boost::system::error_code error);
 
+    boost::asio::io_service&        m_service;
     boost::asio::ip::tcp::socket    m_socket;
-    MessageHeaderBuffer             m_incoming_header_buffer;
+    Message::HeaderBuffer           m_incoming_header_buffer;
     Message                         m_incoming_message;
     int                             m_ID;
     std::string                     m_player_name;
@@ -241,11 +241,6 @@ private:
     MessageAndConnectionFn          m_player_message_callback;
     ConnectionFn                    m_disconnected_callback;
 
-    enum {
-        HEADER_SIZE =
-        MessageHeaderBuffer::static_size * sizeof(MessageHeaderBuffer::value_type)
-    };
-
     friend class ServerNetworking;
 };
 
