@@ -1,11 +1,12 @@
-#include "Label.h"
 #include "Parse.h"
+
 #include "ParseImpl.h"
 
 #include "../Empire/Empire.h"
 
 #include <boost/spirit/include/phoenix.hpp>
 
+
 #define DEBUG_PARSERS 0
 
 #if DEBUG_PARSERS
@@ -16,11 +17,15 @@ namespace std {
 #endif
 
 namespace {
-    std::vector<boost::shared_ptr<Effect::EffectsGroup> >* g_effects_groups = 0;
+    std::vector<std::shared_ptr<Effect::EffectsGroup>>* g_effects_groups = nullptr;
 
     struct rules {
         rules() {
-            const parse::lexer& tok = parse::lexer::instance();
+            namespace phoenix = boost::phoenix;
+            namespace qi = boost::spirit::qi;
+
+            using phoenix::construct;
+            using phoenix::push_back;
 
             qi::_1_type _1;
             qi::_2_type _2;
@@ -29,14 +34,14 @@ namespace {
             qi::_a_type _a;
             qi::_b_type _b;
             qi::_r1_type _r1;
-            using phoenix::construct;
-            using phoenix::push_back;
+
+            const parse::lexer& tok = parse::lexer::instance();
 
             alignment
                 =    tok.Alignment_
-                >    parse::label(Name_token)        > tok.string [ _a = _1 ]
-                >    parse::label(Description_token) > tok.string [ _b = _1 ]
-                >    parse::label(Graphic_token)     > tok.string [ push_back(_r1, construct<Alignment>(_a, _b, _1)) ]
+                >    parse::detail::label(Name_token)        > tok.string [ _a = _1 ]
+                >    parse::detail::label(Description_token) > tok.string [ _b = _1 ]
+                >    parse::detail::label(Graphic_token)     > tok.string [ push_back(_r1, construct<Alignment>(_a, _b, _1)) ]
                 ;
 
             start
@@ -44,7 +49,7 @@ namespace {
                             alignment(_r1)
                         >  -(
                                 tok.AlignmentEffects_
-                            >   parse::label(EffectsGroups_token) > parse::detail::effects_group_parser() [ phoenix::ref(*g_effects_groups) = _1 ]
+                            >   parse::detail::label(EffectsGroups_token) > parse::detail::effects_group_parser() [ phoenix::ref(*g_effects_groups) = _1 ]
                             )
                      )
                 ;
@@ -58,20 +63,16 @@ namespace {
             qi::on_error<qi::fail>(start, parse::report_error(_1, _2, _3, _4));
         }
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             void (std::vector<Alignment>&),
-            qi::locals<
+            boost::spirit::qi::locals<
                 std::string,
                 std::string
-            >,
-            parse::skipper_type
+            >
         > alignment_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            void (std::vector<Alignment>&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            void (std::vector<Alignment>&)
         > start_rule;
 
         alignment_rule alignment;
@@ -81,18 +82,14 @@ namespace {
 
 namespace parse {
     bool alignments(std::vector<Alignment>& alignments_,
-                    std::vector<boost::shared_ptr<Effect::EffectsGroup> >& effects_groups)
+                    std::vector<std::shared_ptr<Effect::EffectsGroup>>& effects_groups)
     {
         bool result = true;
 
-        std::vector<boost::filesystem::path> file_list = ListScripts("scripting/alignments");
-
         g_effects_groups = &effects_groups;
 
-        for (std::vector<boost::filesystem::path>::iterator file_it = file_list.begin();
-             file_it != file_list.end(); ++file_it)
-        {
-            result &= detail::parse_file<rules, std::vector<Alignment> >(*file_it, alignments_);
+        for (const boost::filesystem::path& file : ListScripts("scripting/alignments")) {
+            result &= detail::parse_file<rules, std::vector<Alignment>>(file, alignments_);
         }
 
         return result;
