@@ -1,20 +1,20 @@
-#include "ConditionParserImpl.h"
-#include "Double.h"
-#include "EnumParser.h"
-#include "Int.h"
-#include "Label.h"
 #include "Parse.h"
+
 #include "ParseImpl.h"
+#include "EnumParser.h"
+#include "ConditionParserImpl.h"
 #include "ValueRefParser.h"
 #include "CommonParams.h"
 
 #include "../universe/Building.h"
+#include "../universe/Enums.h"
+
 
 #define DEBUG_PARSERS 0
 
 #if DEBUG_PARSERS
 namespace std {
-    inline ostream& operator<<(ostream& os, const std::vector<boost::shared_ptr<Effect::EffectsGroup> >&) { return os; }
+    inline ostream& operator<<(ostream& os, const std::vector<std::shared_ptr<Effect::EffectsGroup>>&) { return os; }
     inline ostream& operator<<(ostream& os, const std::map<std::string, BuildingType*>&) { return os; }
     inline ostream& operator<<(ostream& os, const std::pair<const std::string, BuildingType*>&) { return os; }
 }
@@ -22,13 +22,7 @@ namespace std {
 
 namespace {
     struct insert_ {
-#if BOOST_VERSION < 105600
-        template <typename Arg1, typename Arg2> // Phoenix v2
-        struct result
-        { typedef void type; };
-#else
         typedef void result_type;
-#endif
 
         void operator()(std::map<std::string, BuildingType*>& building_types, BuildingType* building_type) const {
             if (!building_types.insert(std::make_pair(building_type->Name(), building_type)).second) {
@@ -41,7 +35,10 @@ namespace {
 
     struct rules {
         rules() {
-            const parse::lexer& tok = parse::lexer::instance();
+            namespace phoenix = boost::phoenix;
+            namespace qi = boost::spirit::qi;
+
+            using phoenix::new_;
 
             qi::_1_type _1;
             qi::_2_type _2;
@@ -53,17 +50,18 @@ namespace {
             qi::_d_type _d;
             qi::_r1_type _r1;
             qi::eps_type eps;
-            using phoenix::new_;
+
+            const parse::lexer& tok = parse::lexer::instance();
 
             building_type
                 =   tok.BuildingType_
-                >   parse::label(Name_token)                > tok.string        [ _a = _1 ]
-                >   parse::label(Description_token)         > tok.string        [ _b = _1 ]
-                >   (   parse::label(CaptureResult_token)   >> parse::enum_parser<CaptureResult>() [ _d = _1 ]
+                >   parse::detail::label(Name_token)                > tok.string        [ _a = _1 ]
+                >   parse::detail::label(Description_token)         > tok.string        [ _b = _1 ]
+                >   (   parse::detail::label(CaptureResult_token)   >> parse::capture_result_enum() [ _d = _1 ]
                     |   eps [ _d = CR_CAPTURE ]
                     )
                 >   parse::detail::common_params_parser() [ _c = _1 ]
-                >   parse::label(Icon_token)               > tok.string
+                >   parse::detail::label(Icon_token)      > tok.string
                 [ insert(_r1, new_<BuildingType>(_a, _b, _c, _d, _1)) ]
                 ;
 
@@ -80,22 +78,18 @@ namespace {
             qi::on_error<qi::fail>(start, parse::report_error(_1, _2, _3, _4));
         }
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             void (std::map<std::string, BuildingType*>&),
-            qi::locals<
+            boost::spirit::qi::locals<
                 std::string,
                 std::string,
                 CommonParams,
                 CaptureResult
-            >,
-            parse::skipper_type
+            >
         > building_type_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            void (std::map<std::string, BuildingType*>&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            void (std::map<std::string, BuildingType*>&)
         > start_rule;
 
         building_type_rule          building_type;
@@ -107,12 +101,8 @@ namespace parse {
     bool buildings(std::map<std::string, BuildingType*>& building_types) {
         bool result = true;
 
-        std::vector<boost::filesystem::path> file_list = ListScripts("scripting/buildings");
-
-        for (std::vector<boost::filesystem::path>::iterator file_it = file_list.begin();
-             file_it != file_list.end(); ++file_it)
-        {
-            result &= detail::parse_file<rules, std::map<std::string, BuildingType*> >(*file_it, building_types);
+        for (const boost::filesystem::path& file : ListScripts("scripting/buildings")) {
+            result &= detail::parse_file<rules, std::map<std::string, BuildingType*>>(file, building_types);
         }
 
         return result;
