@@ -1,10 +1,11 @@
 #include "ConditionParserImpl.h"
 
+#include "ParseImpl.h"
 #include "EnumParser.h"
-#include "Label.h"
 #include "ValueRefParser.h"
 #include "../universe/Condition.h"
 #include "../universe/ValueRef.h"
+#include "../universe/Enums.h"
 
 #include <boost/spirit/include/phoenix.hpp>
 
@@ -23,9 +24,6 @@ namespace {
         condition_parser_rules_1() {
             const parse::lexer& tok = parse::lexer::instance();
 
-            const parse::value_ref_parser_rule<int>::type& int_value_ref =
-                parse::value_ref_parser<int>();
-
             qi::_1_type _1;
             qi::_a_type _a;
             qi::_val_type _val;
@@ -86,13 +84,13 @@ namespace {
 
             owned_by_1
                 =   tok.OwnedBy_
-                >>  parse::label(Empire_token) > int_value_ref
+                >>  parse::detail::label(Empire_token) > parse::int_value_ref()
                     [ _val = new_<Condition::EmpireAffiliation>(_1) ]
                 ;
 
             owned_by_2
                 =   tok.OwnedBy_
-                >>  parse::label(Affiliation_token) >> tok.AnyEmpire_
+                >>  parse::detail::label(Affiliation_token) >> tok.AnyEmpire_
                 [ _val = new_<Condition::EmpireAffiliation>( AFFIL_ANY ) ]
                 ;
 
@@ -102,9 +100,14 @@ namespace {
                 ;
 
             owned_by_4
+                =   tok.Human_
+                    [ _val = new_<Condition::EmpireAffiliation>( AFFIL_HUMAN ) ]
+                ;
+
+            owned_by_5
                 =   tok.OwnedBy_
-                >>  parse::label(Affiliation_token) >> parse::enum_parser<EmpireAffiliationType>() [ _a = _1 ]
-                >>  parse::label(Empire_token)      >  int_value_ref
+                >>  parse::detail::label(Affiliation_token) >> parse::empire_affiliation_type_enum() [ _a = _1 ]
+                >>  parse::detail::label(Empire_token)      >  parse::int_value_ref()
                 [ _val = new_<Condition::EmpireAffiliation>(_1, _a) ]
                 ;
 
@@ -113,6 +116,7 @@ namespace {
                 |   owned_by_2
                 |   owned_by_3
                 |   owned_by_4
+                |   owned_by_5
                 ;
 
             and_
@@ -134,8 +138,8 @@ namespace {
 
             described
                 =   tok.Described_
-                >   parse::label(Description_token) > tok.string [ _a = _1 ]
-                >   parse::label(Condition_token) > parse::detail::condition_parser
+                >   parse::detail::label(Description_token) > tok.string [ _a = _1 ]
+                >   parse::detail::label(Condition_token) > parse::detail::condition_parser
                 [ _val = new_<Condition::Described>(_1, _a) ]
                 ;
 
@@ -196,25 +200,19 @@ namespace {
 #endif
         }
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             Condition::ConditionBase* (),
-            qi::locals<EmpireAffiliationType>,
-            parse::skipper_type
+            qi::locals<EmpireAffiliationType>
         > owned_by_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             Condition::ConditionBase* (),
-            qi::locals<std::vector<Condition::ConditionBase*> >,
-            parse::skipper_type
+            qi::locals<std::vector<Condition::ConditionBase*>>
         > and_or_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             Condition::ConditionBase* (),
-            qi::locals<std::string>,
-            parse::skipper_type
+            qi::locals<std::string>
         > described_rule;
 
         parse::condition_parser_rule    all;
@@ -232,7 +230,8 @@ namespace {
         parse::condition_parser_rule    owned_by_1;
         parse::condition_parser_rule    owned_by_2;
         parse::condition_parser_rule    owned_by_3;
-        owned_by_rule                   owned_by_4;
+        parse::condition_parser_rule    owned_by_4;
+        owned_by_rule                   owned_by_5;
         parse::condition_parser_rule    owned_by;
         and_or_rule                     and_;
         and_or_rule                     or_;
