@@ -1,9 +1,10 @@
 #include "ConditionParserImpl.h"
 
+#include "ParseImpl.h"
 #include "EnumParser.h"
-#include "Label.h"
 #include "ValueRefParser.h"
 #include "../universe/Condition.h"
+#include "../universe/Enums.h"
 
 #include <boost/spirit/include/phoenix.hpp>
 
@@ -16,10 +17,6 @@ namespace {
         condition_parser_rules_2() {
             const parse::lexer& tok = parse::lexer::instance();
 
-            const parse::value_ref_parser_rule<int>::type& int_value_ref = parse::value_ref_parser<int>();
-            const parse::value_ref_parser_rule<int>::type& flexible_int_ref = parse::value_ref_parser_flexible_int();
-            const parse::value_ref_parser_rule<std::string>::type& string_value_ref = parse::value_ref_parser<std::string>();
-
             qi::_1_type _1;
             qi::_a_type _a; // intref
             qi::_b_type _b; // intref
@@ -32,9 +29,9 @@ namespace {
 
             has_special_since_turn
                 =   (       tok.HasSpecialSinceTurn_
-                        >   parse::label(Name_token) >  string_value_ref [ _e = _1 ]
-                        > -(parse::label(Low_token)  >  flexible_int_ref [ _a = _1 ] )
-                        > -(parse::label(High_token) >  flexible_int_ref [ _b = _1 ] )
+                        >   parse::detail::label(Name_token) >  parse::string_value_ref() [ _e = _1 ]
+                        > -(parse::detail::label(Low_token)  >  parse::flexible_int_value_ref() [ _a = _1 ] )
+                        > -(parse::detail::label(High_token) >  parse::flexible_int_value_ref() [ _b = _1 ] )
                     ) [ _val = new_<Condition::HasSpecial>(_e, _a, _b) ]
                 ;
 
@@ -47,61 +44,61 @@ namespace {
 
             enqueued1
                 =   (   tok.Enqueued_
-                        >>  parse::label(Type_token)   >>   tok.Building_
-                        > -(parse::label(Name_token)   >    string_value_ref[ _e = _1 ])
-                        > -(parse::label(Empire_token) >    int_value_ref   [ _a = _1 ])
-                        > -(parse::label(Low_token)    >    flexible_int_ref[ _b = _1 ])
-                        > -(parse::label(High_token)   >    flexible_int_ref[ _c = _1 ])
+                        >>  parse::detail::label(Type_token)   >>   tok.Building_
+                        > -(parse::detail::label(Name_token)   >    parse::string_value_ref() [ _e = _1 ])
+                        > -(parse::detail::label(Empire_token) >    parse::int_value_ref() [ _a = _1 ])
+                        > -(parse::detail::label(Low_token)    >    parse::flexible_int_value_ref() [ _b = _1 ])
+                        > -(parse::detail::label(High_token)   >    parse::flexible_int_value_ref() [ _c = _1 ])
                     ) [ _val = new_<Condition::Enqueued>(BT_BUILDING, _e, _a, _b, _c) ]
                 ;
 
             enqueued2
                 =   (   tok.Enqueued_
-                        >>  parse::label(Type_token)   >>   tok.Ship_
-                        > -(parse::label(Design_token) >    int_value_ref   [ _d = _1 ])
-                        > -(parse::label(Empire_token) >    int_value_ref   [ _a = _1 ])
-                        > -(parse::label(Low_token)    >    flexible_int_ref[ _b = _1 ])
-                        > -(parse::label(High_token)   >    flexible_int_ref[ _c = _1 ])
+                        >>  parse::detail::label(Type_token)   >>   tok.Ship_
+                        > -(parse::detail::label(Design_token) >    parse::int_value_ref() [ _d = _1 ])
+                        > -(parse::detail::label(Empire_token) >    parse::int_value_ref() [ _a = _1 ])
+                        > -(parse::detail::label(Low_token)    >    parse::flexible_int_value_ref() [ _b = _1 ])
+                        > -(parse::detail::label(High_token)   >    parse::flexible_int_value_ref() [ _c = _1 ])
                     ) [ _val = new_<Condition::Enqueued>(_d, _a, _b, _c) ]
                 ;
 
             enqueued3
                 =   (   tok.Enqueued_
-                        >>  parse::label(Type_token)   >>   tok.Ship_
-                        >>  parse::label(Name_token)   >    string_value_ref[ _e = _1 ]
-                        > -(parse::label(Empire_token) >    int_value_ref   [ _a = _1 ])
-                        > -(parse::label(Low_token)    >    flexible_int_ref[ _b = _1 ])
-                        > -(parse::label(High_token)   >    flexible_int_ref[ _c = _1 ])
+                        >>  parse::detail::label(Type_token)   >>   tok.Ship_
+                        >>  parse::detail::label(Name_token)   >    parse::string_value_ref() [ _e = _1 ]
+                        > -(parse::detail::label(Empire_token) >    parse::int_value_ref() [ _a = _1 ])
+                        > -(parse::detail::label(Low_token)    >    parse::flexible_int_value_ref() [ _b = _1 ])
+                        > -(parse::detail::label(High_token)   >    parse::flexible_int_value_ref() [ _c = _1 ])
                     ) [ _val = new_<Condition::Enqueued>(BT_SHIP, _e, _a, _b, _c) ]
                 ;
 
             enqueued4
                 =   (   tok.Enqueued_
-                        > -(parse::label(Empire_token) >    int_value_ref   [ _a = _1 ])
-                        > -(parse::label(Low_token)    >    flexible_int_ref[ _b = _1 ])
-                        > -(parse::label(High_token)   >    flexible_int_ref[ _c = _1 ])
+                        > -(parse::detail::label(Empire_token) >    parse::int_value_ref() [ _a = _1 ])
+                        > -(parse::detail::label(Low_token)    >    parse::flexible_int_value_ref() [ _b = _1 ])
+                        > -(parse::detail::label(High_token)   >    parse::flexible_int_value_ref() [ _c = _1 ])
                     ) [ _val = new_<Condition::Enqueued>(INVALID_BUILD_TYPE, _e, _a, _b, _c) ]
                 ;
 
             design_has_part
                 =   (   tok.DesignHasPart_
-                        > -(parse::label(Low_token)   > flexible_int_ref [ _a = _1 ])
-                        > -(parse::label(High_token)  > flexible_int_ref [ _b = _1 ])
-                    )   >   parse::label(Name_token)  > string_value_ref
+                        > -(parse::detail::label(Low_token)   > parse::flexible_int_value_ref() [ _a = _1 ])
+                        > -(parse::detail::label(High_token)  > parse::flexible_int_value_ref() [ _b = _1 ])
+                    )   >   parse::detail::label(Name_token)  > parse::string_value_ref()
                     [ _val = new_<Condition::DesignHasPart>(_1, _a, _b) ]
                 ;
 
             design_has_part_class
                 =   (   tok.DesignHasPartClass_
-                        > -(parse::label(Low_token)   > flexible_int_ref [ _a = _1 ])
-                        > -(parse::label(High_token)  > flexible_int_ref [ _b = _1 ])
-                    )   >   parse::label(Class_token) > parse::enum_parser<ShipPartClass>()
+                        > -(parse::detail::label(Low_token)   > parse::flexible_int_value_ref() [ _a = _1 ])
+                        > -(parse::detail::label(High_token)  > parse::flexible_int_value_ref() [ _b = _1 ])
+                    )   >   parse::detail::label(Class_token) > parse::ship_part_class_enum()
                     [ _val = new_<Condition::DesignHasPartClass>(_1, _a, _b) ]
                 ;
 
             in_system
                 =   (   tok.InSystem_
-                    >  -(parse::label(ID_token)  > int_value_ref [ _a = _1 ])
+                    >  -(parse::detail::label(ID_token)  > parse::int_value_ref() [ _a = _1 ])
                     )
                     [ _val = new_<Condition::InSystem>(_a) ]
                 ;
@@ -129,8 +126,7 @@ namespace {
 #endif
         }
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             Condition::ConditionBase* (),
             qi::locals<
                 ValueRef::ValueRefBase<int>*,
@@ -138,8 +134,7 @@ namespace {
                 ValueRef::ValueRefBase<int>*,
                 ValueRef::ValueRefBase<int>*,
                 ValueRef::ValueRefBase<std::string>*
-            >,
-            parse::skipper_type
+            >
         > common_rule;
 
         common_rule                     has_special_since_turn;
