@@ -1,8 +1,9 @@
 #include "ConditionParserImpl.h"
 
+#include "ParseImpl.h"
 #include "EnumParser.h"
-#include "Label.h"
 #include "ValueRefParser.h"
+#include "ValueRefParserImpl.h"
 #include "../universe/Condition.h"
 #include "../universe/ValueRef.h"
 
@@ -28,21 +29,6 @@ namespace {
         condition_parser_rules_6() {
             const parse::lexer& tok = parse::lexer::instance();
 
-            const parse::value_ref_parser_rule<std::string>::type& string_value_ref =
-                parse::value_ref_parser<std::string>();
-
-            const parse::value_ref_parser_rule<PlanetType>::type& planet_type_value_ref =
-                parse::value_ref_parser<PlanetType>();
-
-            const parse::value_ref_parser_rule<PlanetSize>::type& planet_size_value_ref =
-                parse::value_ref_parser<PlanetSize>();
-
-            const parse::value_ref_parser_rule<PlanetEnvironment>::type& planet_environment_value_ref =
-                parse::value_ref_parser<PlanetEnvironment>();
-
-            const parse::value_ref_parser_rule<UniverseObjectType>::type& universe_object_type_value_ref =
-                parse::value_ref_parser<UniverseObjectType>();
-
             qi::_1_type _1;
             qi::_a_type _a;
             qi::_b_type _b;
@@ -52,22 +38,22 @@ namespace {
             using phoenix::push_back;
 
             string_ref_vec
-                =   '[' > +string_value_ref [ push_back(_val, _1) ] > ']'
-                |   string_value_ref [ push_back(_val, _1) ]
+                =   ('[' > +parse::string_value_ref() [ push_back(_val, _1) ] > ']')
+                |    parse::string_value_ref() [ push_back(_val, _1) ]
                 ;
 
             homeworld
                 =   tok.Homeworld_
                 >   (
-                        parse::label(Name_token) > string_ref_vec [ _val = new_<Condition::Homeworld>(_1) ]
-                    |   eps [ _val = new_<Condition::Homeworld>() ]
+                        (parse::detail::label(Name_token) > string_ref_vec [ _val = new_<Condition::Homeworld>(_1) ])
+                    |    eps [ _val = new_<Condition::Homeworld>() ]
                     )
                 ;
 
             building
                 =   (
                         tok.Building_
-                    >  -(parse::label(Name_token) > string_ref_vec [ _a = _1 ])
+                    >  -(parse::detail::label(Name_token) > string_ref_vec [ _a = _1 ])
                     )
                     [ _val = new_<Condition::Building>(_a) ]
                 ;
@@ -75,55 +61,55 @@ namespace {
             species
                 =   tok.Species_
                 >   (
-                        parse::label(Name_token) > string_ref_vec [ _val = new_<Condition::Species>(_1) ]
-                    |   eps [ _val = new_<Condition::Species>() ]
+                        (parse::detail::label(Name_token) > string_ref_vec [ _val = new_<Condition::Species>(_1) ])
+                    |    eps [ _val = new_<Condition::Species>() ]
                     )
                 ;
 
             focus_type
                 =   tok.Focus_
                 >   (
-                        parse::label(Type_token) > string_ref_vec [ _val = new_<Condition::FocusType>(_1) ]
-                |   eps [ _val = new_<Condition::FocusType>(std::vector<ValueRef::ValueRefBase<std::string>*>()) ]
+                        (parse::detail::label(Type_token) > string_ref_vec [ _val = new_<Condition::FocusType>(_1) ])
+                |        eps [ _val = new_<Condition::FocusType>(std::vector<ValueRef::ValueRefBase<std::string>*>()) ]
                     )
                 ;
 
             planet_type
                 =   tok.Planet_
-                >>  parse::label(Type_token)
+                >>  parse::detail::label(Type_token)
                 >   (
-                        '[' > +planet_type_value_ref [ push_back(_a, _1) ] > ']'
-                    |   planet_type_value_ref [ push_back(_a, _1) ]
+                        ('[' > +parse::detail::planet_type_rules().expr [ push_back(_a, _1) ] > ']')
+                    |    parse::detail::planet_type_rules().expr [ push_back(_a, _1) ]
                     )
                     [ _val = new_<Condition::PlanetType>(_a) ]
                 ;
 
             planet_size
                 =   tok.Planet_
-                >>  parse::label(Size_token)
+                >>  parse::detail::label(Size_token)
                 >   (
-                        '[' > +planet_size_value_ref [ push_back(_a, _1) ] > ']'
-                    |   planet_size_value_ref [ push_back(_a, _1) ]
+                        ('[' > +parse::detail::planet_size_rules().expr [ push_back(_a, _1) ] > ']')
+                    |    parse::detail::planet_size_rules().expr [ push_back(_a, _1) ]
                     )
                     [ _val = new_<Condition::PlanetSize>(_a) ]
                 ;
 
             planet_environment
                 =   (tok.Planet_
-                >>  parse::label(Environment_token)
+                >>  parse::detail::label(Environment_token)
                 >   (
-                        '[' > +planet_environment_value_ref [ push_back(_a, _1) ] > ']'
-                    |   planet_environment_value_ref [ push_back(_a, _1) ]
+                        ('[' > +parse::detail::planet_environment_rules().expr [ push_back(_a, _1) ] > ']')
+                    |    parse::detail::planet_environment_rules().expr [ push_back(_a, _1) ]
                     )
-                >  -(parse::label(Species_token)        >  string_value_ref [_b = _1]))
+                >  -(parse::detail::label(Species_token)        >  parse::string_value_ref() [_b = _1]))
                     [ _val = new_<Condition::PlanetEnvironment>(_a, _b) ]
                 ;
 
             object_type
-                =   parse::enum_parser<UniverseObjectType>() [ _val = new_<Condition::Type>(new_<ValueRef::Constant<UniverseObjectType> >(_1)) ]
+                =   parse::detail::universe_object_type_rules().enum_expr [ _val = new_<Condition::Type>(new_<ValueRef::Constant<UniverseObjectType>>(_1)) ]
                 |   (
                         tok.ObjectType_
-                    >   parse::label(Type_token) > universe_object_type_value_ref [ _val = new_<Condition::Type>(_1) ]
+                    >   parse::detail::label(Type_token) > parse::detail::universe_object_type_rules().expr [ _val = new_<Condition::Type>(_1) ]
                     )
                 ;
 
@@ -162,41 +148,31 @@ namespace {
 #endif
         }
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            std::vector<ValueRef::ValueRefBase<std::string>*> (),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            std::vector<ValueRef::ValueRefBase<std::string>*> ()
         > string_ref_vec_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             Condition::ConditionBase* (),
-            qi::locals<std::vector<ValueRef::ValueRefBase<std::string>*> >,
-            parse::skipper_type
+            qi::locals<std::vector<ValueRef::ValueRefBase<std::string>*>>
         > building_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             Condition::ConditionBase* (),
-            qi::locals<std::vector<ValueRef::ValueRefBase<PlanetType>*> >,
-            parse::skipper_type
+            qi::locals<std::vector<ValueRef::ValueRefBase<PlanetType>*>>
         > planet_type_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             Condition::ConditionBase* (),
-            qi::locals<std::vector<ValueRef::ValueRefBase<PlanetSize>*> >,
-            parse::skipper_type
+            qi::locals<std::vector<ValueRef::ValueRefBase<PlanetSize>*>>
         > planet_size_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             Condition::ConditionBase* (),
             qi::locals<
                 std::vector<ValueRef::ValueRefBase<PlanetEnvironment>*>,
                 ValueRef::ValueRefBase<std::string>*
-            >,
-            parse::skipper_type
+            >
         > planet_environment_rule;
 
         string_ref_vec_rule             string_ref_vec;
