@@ -1,10 +1,12 @@
 #include "EffectParserImpl.h"
 
+#include "ParseImpl.h"
 #include "EnumParser.h"
-#include "Label.h"
 #include "ValueRefParser.h"
+#include "ValueRefParserImpl.h"
 #include "ConditionParserImpl.h"
 #include "../universe/Effect.h"
+#include "../universe/Enums.h"
 
 #include <boost/spirit/include/phoenix.hpp>
 
@@ -26,75 +28,69 @@ namespace {
 
             const parse::lexer& tok =                                                       parse::lexer::instance();
 
-            const parse::value_ref_parser_rule<int>::type& int_value_ref =                  parse::value_ref_parser<int>();
-            const parse::value_ref_parser_rule<double>::type& double_value_ref =            parse::value_ref_parser<double>();
-            const parse::value_ref_parser_rule<std::string>::type& string_value_ref =       parse::value_ref_parser<std::string>();
-            const parse::value_ref_parser_rule<PlanetType>::type& planet_type_value_ref =   parse::value_ref_parser<PlanetType>();
-            const parse::value_ref_parser_rule<PlanetSize>::type& planet_size_value_ref =   parse::value_ref_parser<PlanetSize>();
-
             set_meter
                 =   parse::set_non_ship_part_meter_type_enum() [ _a = _1 ] /* has some overlap with parse::set_ship_part_meter_type_enum() so can't use '>' */
-                >>  parse::label(Value_token)               > double_value_ref [ _c = _1 ]
+                >>  parse::detail::label(Value_token)               > parse::double_value_ref() [ _c = _1 ]
                 >   (
-                        (parse::label(AccountingLabel_token) > tok.string [ _val = new_<Effect::SetMeter>(_a, _c, _1) ] )
+                        (parse::detail::label(AccountingLabel_token) > tok.string [ _val = new_<Effect::SetMeter>(_a, _c, _1) ] )
                     |    eps [ _val = new_<Effect::SetMeter>(_a, _c) ]
                     )
                 ;
 
             set_ship_part_meter
                 =    parse::set_ship_part_meter_type_enum() [ _a = _1 ]
-                >>   parse::label(PartName_token)   > string_value_ref [ _b = _1 ]
-                >    parse::label(Value_token)      > double_value_ref [ _val = new_<Effect::SetShipPartMeter>(_a, _b, _1) ]
+                >>   parse::detail::label(PartName_token)   > parse::string_value_ref() [ _b = _1 ]
+                >    parse::detail::label(Value_token)      > parse::double_value_ref() [ _val = new_<Effect::SetShipPartMeter>(_a, _b, _1) ]
                 ;
 
             set_empire_stockpile
                 =   tok.SetEmpireTradeStockpile_ [ _a = RE_TRADE ]
                 >   (
-                        (   parse::label(Empire_token) > int_value_ref [ _b = _1 ]
-                        >   parse::label(Value_token)  > double_value_ref [ _val = new_<Effect::SetEmpireStockpile>(_b, _a, _1) ]
+                        (   parse::detail::label(Empire_token) > parse::int_value_ref() [ _b = _1 ]
+                        >   parse::detail::label(Value_token)  > parse::double_value_ref() [ _val = new_<Effect::SetEmpireStockpile>(_b, _a, _1) ]
                         )
-                        |  (parse::label(Value_token)  > double_value_ref [ _val = new_<Effect::SetEmpireStockpile>(_a, _1) ])
+                        |  (parse::detail::label(Value_token)  > parse::double_value_ref() [ _val = new_<Effect::SetEmpireStockpile>(_a, _1) ])
                     )
                 ;
 
             set_empire_capital
                 =    tok.SetEmpireCapital_
                 >   (
-                        (parse::label(Empire_token) > int_value_ref [ _val = new_<Effect::SetEmpireCapital>(_1) ])
+                        (parse::detail::label(Empire_token) > parse::int_value_ref() [ _val = new_<Effect::SetEmpireCapital>(_1) ])
                     |    eps [ _val = new_<Effect::SetEmpireCapital>() ]
                     )
                 ;
 
             set_planet_type
                 =    tok.SetPlanetType_
-                >    parse::label(Type_token) > planet_type_value_ref [ _val = new_<Effect::SetPlanetType>(_1) ]
+                >    parse::detail::label(Type_token) > parse::detail::planet_type_rules().expr [ _val = new_<Effect::SetPlanetType>(_1) ]
                 ;
 
             set_planet_size
                 =    tok.SetPlanetSize_
-                >    parse::label(PlanetSize_token) > planet_size_value_ref [ _val = new_<Effect::SetPlanetSize>(_1) ]
+                >    parse::detail::label(PlanetSize_token) > parse::detail::planet_size_rules().expr [ _val = new_<Effect::SetPlanetSize>(_1) ]
                 ;
 
             set_species
                 =    tok.SetSpecies_
-                >    parse::label(Name_token) > string_value_ref [ _val = new_<Effect::SetSpecies>(_1) ]
+                >    parse::detail::label(Name_token) > parse::string_value_ref() [ _val = new_<Effect::SetSpecies>(_1) ]
                 ;
 
             set_owner
                 =    tok.SetOwner_
-                >    parse::label(Empire_token) > int_value_ref [ _val = new_<Effect::SetOwner>(_1) ]
+                >    parse::detail::label(Empire_token) > parse::int_value_ref() [ _val = new_<Effect::SetOwner>(_1) ]
                 ;
 
             set_species_opinion
                 =    tok.SetSpeciesOpinion_
-                >    parse::label(Species_token) >   string_value_ref [ _a = _1 ]
+                >    parse::detail::label(Species_token) >   parse::string_value_ref() [ _a = _1 ]
                 > (
-                    (   parse::label(Empire_token) >  int_value_ref [ _c = _1 ]
-                     >  parse::label(Opinion_token) > double_value_ref
+                    (   parse::detail::label(Empire_token) >  parse::int_value_ref() [ _c = _1 ]
+                     >  parse::detail::label(Opinion_token) > parse::double_value_ref()
                         [ _val = new_<Effect::SetSpeciesEmpireOpinion>(_a, _c, _1) ])
                    |
-                    (   parse::label(Species_token) > string_value_ref [ _b = _1 ]
-                    >   parse::label(Opinion_token) > double_value_ref
+                    (   parse::detail::label(Species_token) > parse::string_value_ref() [ _b = _1 ]
+                    >   parse::detail::label(Opinion_token) > parse::double_value_ref()
                         [ _val = new_<Effect::SetSpeciesSpeciesOpinion>(_a, _b, _1) ])
                    )
                 ;
@@ -102,7 +98,7 @@ namespace {
             set_visibility
                 =    tok.SetVisibility_
                 >   (
-                        parse::label(Visibility_token)
+                        parse::detail::label(Visibility_token)
                     > ( tok.Invisible_  [ _c = VIS_NO_VISIBILITY ]
                     |   tok.Basic_      [ _c = VIS_BASIC_VISIBILITY ]
                     |   tok.Partial_    [ _c = VIS_PARTIAL_VISIBILITY ]
@@ -112,19 +108,19 @@ namespace {
                         (   // empire id specified, optionally with an affiliation type:
                             // useful to specify a single recipient empire, or the allies
                             // or enemies of a single empire
-                            (   parse::label(Affiliation_token) > parse::enum_parser<EmpireAffiliationType>() [ _d = _1 ]
-                            |   eps [ _d = AFFIL_SELF ]
+                            (   (parse::detail::label(Affiliation_token) > parse::empire_affiliation_type_enum() [ _d = _1 ])
+                            |    eps [ _d = AFFIL_SELF ]
                             )
-                        >>  parse::label(Empire_token) > int_value_ref [ _b = _1 ]
+                        >>  parse::detail::label(Empire_token) > parse::int_value_ref() [ _b = _1 ]
                         )
                      |  (   // no empire id or condition specified, with or without an
                             // affiliation type: useful to specify no or all empires
-                            (   parse::label(Affiliation_token) > parse::enum_parser<EmpireAffiliationType>() [ _d = _1 ]
-                            |   eps [ _d = AFFIL_ANY ]
+                            (   (parse::detail::label(Affiliation_token) > parse::empire_affiliation_type_enum() [ _d = _1 ])
+                            |    eps [ _d = AFFIL_ANY ]
                             )
                         )
                      )
-                    >-(parse::label(Condition_token) > parse::detail::condition_parser [ _e = _1 ])
+                    >-(parse::detail::label(Condition_token) > parse::detail::condition_parser [ _e = _1 ])
                     ) [ _val = new_<Effect::SetVisibility>(_c, _d, _b, _e) ]
                 ;
 
@@ -167,20 +163,17 @@ namespace {
 #endif
         }
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             Effect::EffectBase* (),
             qi::locals<
                 MeterType,
                 ValueRef::ValueRefBase<std::string>*,
                 ValueRef::ValueRefBase<double>*,
                 std::string
-            >,
-            parse::skipper_type
+            >
         > set_meter_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             Effect::EffectBase* (),
             qi::locals<
                 ResourceType,
@@ -188,19 +181,16 @@ namespace {
                 Visibility,
                 EmpireAffiliationType,
                 Condition::ConditionBase*
-            >,
-            parse::skipper_type
+            >
         > set_stockpile_or_vis_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             Effect::EffectBase* (),
             qi::locals<
                 ValueRef::ValueRefBase<std::string>*,
                 ValueRef::ValueRefBase<std::string>*,
                 ValueRef::ValueRefBase<int>*
-            >,
-            parse::skipper_type
+            >
         > string_string_int_rule;
 
         set_meter_rule                      set_meter;
