@@ -1,8 +1,9 @@
 #include "EffectParserImpl.h"
 
+#include "ParseImpl.h"
 #include "EnumParser.h"
-#include "Label.h"
 #include "ValueRefParser.h"
+#include "ValueRefParserImpl.h"
 #include "../universe/Effect.h"
 
 #include <boost/spirit/include/phoenix.hpp>
@@ -28,22 +29,15 @@ namespace {
 
             const parse::lexer& tok =                                                       parse::lexer::instance();
 
-            const parse::value_ref_parser_rule<int>::type& int_value_ref =                  parse::value_ref_parser<int>();
-            const parse::value_ref_parser_rule<double>::type& double_value_ref =            parse::value_ref_parser<double>();
-            const parse::value_ref_parser_rule<std::string>::type& string_value_ref =       parse::value_ref_parser<std::string>();
-            const parse::value_ref_parser_rule< ::PlanetType>::type& planet_type_value_ref= parse::value_ref_parser< ::PlanetType>();
-            const parse::value_ref_parser_rule< ::PlanetSize>::type& planet_size_value_ref= parse::value_ref_parser< ::PlanetSize>();
-            const parse::value_ref_parser_rule< ::StarType>::type& star_type_value_ref =    parse::value_ref_parser< ::StarType>();
-
             create_planet
                 =   (       tok.CreatePlanet_
-                        >   parse::label(Type_token)        >   planet_type_value_ref [ _a = _1 ]
-                        >   parse::label(PlanetSize_token)  >   planet_size_value_ref [ _b = _1 ]
-                        > -(parse::label(Name_token)        >   string_value_ref      [ _c = _1 ])
-                        > -(parse::label(Effects_token)
+                        >   parse::detail::label(Type_token)        >   parse::detail::planet_type_rules().expr [ _a = _1 ]
+                        >   parse::detail::label(PlanetSize_token)  >   parse::detail::planet_size_rules().expr [ _b = _1 ]
+                        > -(parse::detail::label(Name_token)        >   parse::string_value_ref()      [ _c = _1 ])
+                        > -(parse::detail::label(Effects_token)
                         >   (
-                                '[' > +parse::effect_parser() [ push_back(_d, _1) ] > ']'
-                            |   parse::effect_parser() [ push_back(_d, _1) ]
+                                ('[' > +parse::effect_parser() [ push_back(_d, _1) ] > ']')
+                            |    parse::effect_parser() [ push_back(_d, _1) ]
                             )
                            )
                     ) [ _val = new_<Effect::CreatePlanet>(_a, _b, _c, _d) ]
@@ -51,12 +45,12 @@ namespace {
 
             create_building
                 =   (       tok.CreateBuilding_
-                        >   parse::label(Type_token)        >   string_value_ref [ _a = _1 ]
-                        > -(parse::label(Name_token)        >   string_value_ref [ _b = _1 ])
-                        > -(parse::label(Effects_token)
+                        >   parse::detail::label(Type_token)        >   parse::string_value_ref() [ _a = _1 ]
+                        > -(parse::detail::label(Name_token)        >   parse::string_value_ref() [ _b = _1 ])
+                        > -(parse::detail::label(Effects_token)
                         >   (
-                                '[' > +parse::effect_parser() [ push_back(_c, _1) ] > ']'
-                            |   parse::effect_parser() [ push_back(_c, _1) ]
+                                ('[' > +parse::effect_parser() [ push_back(_c, _1) ] > ']')
+                            |    parse::effect_parser() [ push_back(_c, _1) ]
                             )
                            )
                     ) [ _val = new_<Effect::CreateBuilding>(_a, _b, _c) ]
@@ -64,14 +58,14 @@ namespace {
 
             create_ship_1
                 =   (       tok.CreateShip_
-                        >>  parse::label(DesignID_token)    >   int_value_ref    [ _b = _1 ]
-                        > -(parse::label(Empire_token)      >   int_value_ref    [ _c = _1 ])
-                        > -(parse::label(Species_token)     >   string_value_ref [ _d = _1 ])
-                        > -(parse::label(Name_token)        >   string_value_ref [ _e = _1 ])
-                        > -(parse::label(Effects_token)
+                        >>  parse::detail::label(DesignID_token)    >   parse::int_value_ref()    [ _b = _1 ]
+                        > -(parse::detail::label(Empire_token)      >   parse::int_value_ref()    [ _c = _1 ])
+                        > -(parse::detail::label(Species_token)     >   parse::string_value_ref() [ _d = _1 ])
+                        > -(parse::detail::label(Name_token)        >   parse::string_value_ref() [ _e = _1 ])
+                        > -(parse::detail::label(Effects_token)
                         >   (
-                                '[' > +parse::effect_parser() [ push_back(_f, _1) ] > ']'
-                            |   parse::effect_parser() [ push_back(_f, _1) ]
+                                ('[' > +parse::effect_parser() [ push_back(_f, _1) ] > ']')
+                            |    parse::effect_parser() [ push_back(_f, _1) ]
                             )
                            )
                     ) [ _val = new_<Effect::CreateShip>(_b, _c, _d, _e, _f) ]
@@ -79,14 +73,14 @@ namespace {
 
             create_ship_2
                 =   (       tok.CreateShip_
-                        >>  parse::label(DesignName_token)  >>  string_value_ref [ _a = _1 ]
-                        > -(parse::label(Empire_token)      >   int_value_ref    [ _c = _1 ])
-                        > -(parse::label(Species_token)     >   string_value_ref [ _d = _1 ])
-                        > -(parse::label(Name_token)        >   string_value_ref [ _e = _1 ])
-                        > -(parse::label(Effects_token)
+                        >>  parse::detail::label(DesignName_token)  >>  parse::string_value_ref() [ _a = _1 ]
+                        > -(parse::detail::label(Empire_token)      >   parse::int_value_ref()    [ _c = _1 ])
+                        > -(parse::detail::label(Species_token)     >   parse::string_value_ref() [ _d = _1 ])
+                        > -(parse::detail::label(Name_token)        >   parse::string_value_ref() [ _e = _1 ])
+                        > -(parse::detail::label(Effects_token)
                         >   (
-                                '[' > +parse::effect_parser() [ push_back(_f, _1) ] > ']'
-                            |   parse::effect_parser() [ push_back(_f, _1) ]
+                                ('[' > +parse::effect_parser() [ push_back(_f, _1) ] > ']')
+                            |    parse::effect_parser() [ push_back(_f, _1) ]
                             )
                            )
                     ) [ _val = new_<Effect::CreateShip>(_a, _c, _d, _e, _f) ]
@@ -94,13 +88,13 @@ namespace {
 
             create_field_1
                 =   (       tok.CreateField_
-                        >>  parse::label(Type_token)    >>  string_value_ref [ _a = _1 ]
-                        >>  parse::label(Size_token)    >   double_value_ref [ _b = _1 ]
-                        > -(parse::label(Name_token)    >   string_value_ref [ _d = _1 ])
-                        > -(parse::label(Effects_token)
+                        >>  parse::detail::label(Type_token)    >>  parse::string_value_ref() [ _a = _1 ]
+                        >>  parse::detail::label(Size_token)    >   parse::double_value_ref() [ _b = _1 ]
+                        > -(parse::detail::label(Name_token)    >   parse::string_value_ref() [ _d = _1 ])
+                        > -(parse::detail::label(Effects_token)
                         >   (
-                                '[' > +parse::effect_parser() [ push_back(_f, _1) ] > ']'
-                            |   parse::effect_parser() [ push_back(_f, _1) ]
+                                ('[' > +parse::effect_parser() [ push_back(_f, _1) ] > ']')
+                            |    parse::effect_parser() [ push_back(_f, _1) ]
                             )
                            )
                     ) [ _val = new_<Effect::CreateField>(_a, _b, _d, _f) ]
@@ -108,15 +102,15 @@ namespace {
 
             create_field_2
                 =   (       tok.CreateField_
-                        >>  parse::label(Type_token)    >>  string_value_ref [ _a = _1 ]
-                        >>  parse::label(X_token)       >   double_value_ref [ _b = _1 ]
-                        >   parse::label(Y_token)       >   double_value_ref [ _c = _1 ]
-                        >   parse::label(Size_token)    >   double_value_ref [ _e = _1 ]
-                        > -(parse::label(Name_token)    >   string_value_ref [ _d = _1 ])
-                        > -(parse::label(Effects_token)
+                        >>  parse::detail::label(Type_token)    >>  parse::string_value_ref() [ _a = _1 ]
+                        >>  parse::detail::label(X_token)       >   parse::double_value_ref() [ _b = _1 ]
+                        >   parse::detail::label(Y_token)       >   parse::double_value_ref() [ _c = _1 ]
+                        >   parse::detail::label(Size_token)    >   parse::double_value_ref() [ _e = _1 ]
+                        > -(parse::detail::label(Name_token)    >   parse::string_value_ref() [ _d = _1 ])
+                        > -(parse::detail::label(Effects_token)
                         >   (
-                                '[' > +parse::effect_parser() [ push_back(_f, _1) ] > ']'
-                            |   parse::effect_parser() [ push_back(_f, _1) ]
+                                ('[' > +parse::effect_parser() [ push_back(_f, _1) ] > ']')
+                            |    parse::effect_parser() [ push_back(_f, _1) ]
                             )
                            )
                     ) [ _val = new_<Effect::CreateField>(_a, _b, _c, _e, _d, _f) ]
@@ -124,14 +118,14 @@ namespace {
 
             create_system_1
                 =   (       tok.CreateSystem_
-                        >>  parse::label(Type_token)    >   star_type_value_ref [ _a = _1 ]
-                        >   parse::label(X_token)       >   double_value_ref    [ _b = _1 ]
-                        >   parse::label(Y_token)       >   double_value_ref    [ _c = _1 ]
-                        > -(parse::label(Name_token)    >   string_value_ref    [ _d = _1 ])
-                        > -(parse::label(Effects_token)
+                        >>  parse::detail::label(Type_token)    >   parse::detail::star_type_rules().expr [ _a = _1 ]
+                        >   parse::detail::label(X_token)       >   parse::double_value_ref()    [ _b = _1 ]
+                        >   parse::detail::label(Y_token)       >   parse::double_value_ref()    [ _c = _1 ]
+                        > -(parse::detail::label(Name_token)    >   parse::string_value_ref()    [ _d = _1 ])
+                        > -(parse::detail::label(Effects_token)
                         >   (
-                                '[' > +parse::effect_parser() [ push_back(_e, _1) ] > ']'
-                            |   parse::effect_parser() [ push_back(_e, _1) ]
+                                ('[' > +parse::effect_parser() [ push_back(_e, _1) ] > ']')
+                            |    parse::effect_parser() [ push_back(_e, _1) ]
                             )
                            )
                     ) [ _val = new_<Effect::CreateSystem>(_a, _b, _c, _d, _e) ]
@@ -139,13 +133,13 @@ namespace {
 
             create_system_2
                 =   (       tok.CreateSystem_
-                        >>  parse::label(X_token)       >   double_value_ref [ _b = _1 ]
-                        >   parse::label(Y_token)       >   double_value_ref [ _c = _1 ]
-                        > -(parse::label(Name_token)    >   string_value_ref [ _d = _1 ])
-                        > -(parse::label(Effects_token)
+                        >>  parse::detail::label(X_token)       >   parse::double_value_ref() [ _b = _1 ]
+                        >   parse::detail::label(Y_token)       >   parse::double_value_ref() [ _c = _1 ]
+                        > -(parse::detail::label(Name_token)    >   parse::string_value_ref() [ _d = _1 ])
+                        > -(parse::detail::label(Effects_token)
                         >   (
-                                '[' > +parse::effect_parser() [ push_back(_e, _1) ] > ']'
-                            |   parse::effect_parser() [ push_back(_e, _1) ]
+                                ('[' > +parse::effect_parser() [ push_back(_e, _1) ] > ']')
+                            |    parse::effect_parser() [ push_back(_e, _1) ]
                             )
                            )
                     ) [ _val = new_<Effect::CreateSystem>(_b, _c, _d, _e) ]
@@ -183,31 +177,26 @@ namespace {
 #endif
         }
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             Effect::EffectBase* (),
             qi::locals<
                 ValueRef::ValueRefBase< ::PlanetType>*,
                 ValueRef::ValueRefBase< ::PlanetSize>*,
                 ValueRef::ValueRefBase<std::string>*,
                 std::vector<Effect::EffectBase*>
-            >,
-            parse::skipper_type
+            >
         > create_planet_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             Effect::EffectBase* (),
             qi::locals<
                 ValueRef::ValueRefBase<std::string>*,
                 ValueRef::ValueRefBase<std::string>*,
                 std::vector<Effect::EffectBase*>
-            >,
-            parse::skipper_type
+            >
         > create_building_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             Effect::EffectBase* (),
             qi::locals<
                 ValueRef::ValueRefBase< ::StarType>*,
@@ -215,12 +204,10 @@ namespace {
                 ValueRef::ValueRefBase<double>*,
                 ValueRef::ValueRefBase<std::string>*,
                 std::vector<Effect::EffectBase*>
-            >,
-            parse::skipper_type
+            >
         > create_system_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             Effect::EffectBase* (),
             qi::locals<
                 ValueRef::ValueRefBase<std::string>*,
@@ -229,12 +216,10 @@ namespace {
                 ValueRef::ValueRefBase<std::string>*,
                 ValueRef::ValueRefBase<std::string>*,
                 std::vector<Effect::EffectBase*>
-            >,
-            parse::skipper_type
+            >
         > create_ship_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             Effect::EffectBase* (),
             qi::locals<
                 ValueRef::ValueRefBase<std::string>*,
@@ -243,8 +228,7 @@ namespace {
                 ValueRef::ValueRefBase<std::string>*,
                 ValueRef::ValueRefBase<double>*,
                 std::vector<Effect::EffectBase*>
-            >,
-            parse::skipper_type
+            >
         > create_field_rule;
 
         create_planet_rule              create_planet;
