@@ -1,10 +1,11 @@
-#include "ValueRefParser.h"
-#include "Label.h"
 #include "Parse.h"
+
 #include "ParseImpl.h"
+#include "ValueRefParser.h"
 
 #include <boost/spirit/include/phoenix.hpp>
 
+
 #define DEBUG_PARSERS 0
 #if DEBUG_PARSERS
 namespace std {
@@ -16,8 +17,11 @@ namespace std {
 namespace {
     struct rules {
         rules() {
-            const parse::lexer& tok =                                               parse::lexer::instance();
-            const parse::value_ref_parser_rule<double>::type& double_value_ref =    parse::value_ref_parser<double>();
+            namespace phoenix = boost::phoenix;
+            namespace qi = boost::spirit::qi;
+
+            using phoenix::insert;
+            using phoenix::construct;
 
             qi::_1_type _1;
             qi::_2_type _2;
@@ -26,15 +30,14 @@ namespace {
             qi::_a_type _a;
             qi::_r1_type _r1;
             qi::_val_type _val;
-            using phoenix::insert;
-            using phoenix::construct;
 
+            const parse::lexer& tok = parse::lexer::instance();
 
             stat
                 =   tok.Statistic_
-                >   parse::label(Name_token)    > tok.string [ _a = _1 ]
-                >   parse::label(Value_token)   > double_value_ref
-                [ _val = construct<std::pair<std::string, ValueRef::ValueRefBase<double>*> >(_a, _1) ]
+                >   parse::detail::label(Name_token)    > tok.string [ _a = _1 ]
+                >   parse::detail::label(Value_token)   > parse::double_value_ref()
+                [ _val = construct<std::pair<std::string, ValueRef::ValueRefBase<double>*>>(_a, _1) ]
                 ;
 
             start
@@ -50,17 +53,13 @@ namespace {
             qi::on_error<qi::fail>(start, parse::report_error(_1, _2, _3, _4));
         }
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             std::pair<std::string, ValueRef::ValueRefBase<double>*> (),
-            qi::locals<std::string>,
-            parse::skipper_type
+            boost::spirit::qi::locals<std::string>
         > stat_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            void (std::map<std::string, ValueRef::ValueRefBase<double>*>&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            void (std::map<std::string, ValueRef::ValueRefBase<double>*>&)
         > start_rule;
 
         stat_rule   stat;
@@ -72,12 +71,8 @@ namespace parse {
     bool statistics(std::map<std::string, ValueRef::ValueRefBase<double>*>& stats_) {
         bool result = true;
 
-        std::vector<boost::filesystem::path> file_list = ListScripts("scripting/empire_statistics");
-
-        for (std::vector<boost::filesystem::path>::iterator file_it = file_list.begin();
-             file_it != file_list.end(); ++file_it)
-        {
-            result &= detail::parse_file<rules, std::map<std::string, ValueRef::ValueRefBase<double>*> >(*file_it, stats_);
+        for (const boost::filesystem::path& file : ListScripts("scripting/empire_statistics")) {
+            result &= detail::parse_file<rules, std::map<std::string, ValueRef::ValueRefBase<double>*>>(file, stats_);
         }
 
         return result;
