@@ -1,11 +1,12 @@
-#include "Label.h"
 #include "Parse.h"
+
 #include "ParseImpl.h"
 
 #include "../universe/Encyclopedia.h"
 
 #include <boost/spirit/include/phoenix.hpp>
 
+
 #define DEBUG_PARSERS 0
 
 #if DEBUG_PARSERS
@@ -17,13 +18,7 @@ namespace std {
 
 namespace {
     struct insert_ {
-#if BOOST_VERSION < 105600
-        template <typename Arg1, typename Arg2> // Phoenix v2
-        struct result
-        { typedef void type; };
-#else
         typedef void result_type;
-#endif
 
         void operator()(Encyclopedia& enc, const EncyclopediaArticle& article) const
         { enc.articles[article.category].push_back(article); }
@@ -32,7 +27,10 @@ namespace {
 
     struct rules {
         rules() {
-            const parse::lexer& tok = parse::lexer::instance();
+            namespace phoenix = boost::phoenix;
+            namespace qi = boost::spirit::qi;
+
+            using phoenix::construct;
 
             qi::_1_type _1;
             qi::_2_type _2;
@@ -43,15 +41,16 @@ namespace {
             qi::_c_type _c;
             qi::_d_type _d;
             qi::_r1_type _r1;
-            using phoenix::construct;
+
+            const parse::lexer& tok = parse::lexer::instance();
 
             article
                 =    tok.Article_
-                >    parse::label(Name_token)                > tok.string [ _a = _1 ]
-                >    parse::label(Category_token)            > tok.string [ _b = _1 ]
-                >    parse::label(Short_Description_token)   > tok.string [ _c = _1 ]
-                >    parse::label(Description_token)         > tok.string [ _d = _1 ]
-                >    parse::label(Icon_token)                > tok.string
+                >    parse::detail::label(Name_token)                > tok.string [ _a = _1 ]
+                >    parse::detail::label(Category_token)            > tok.string [ _b = _1 ]
+                >    parse::detail::label(Short_Description_token)   > tok.string [ _c = _1 ]
+                >    parse::detail::label(Description_token)         > tok.string [ _d = _1 ]
+                >    parse::detail::label(Icon_token)                > tok.string
                     [ insert(_r1, construct<EncyclopediaArticle>(_a, _b, _c, _d, _1)) ]
                 ;
 
@@ -68,22 +67,18 @@ namespace {
             qi::on_error<qi::fail>(start, parse::report_error(_1, _2, _3, _4));
         }
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             void (Encyclopedia&),
-            qi::locals<
+            boost::spirit::qi::locals<
                 std::string,
                 std::string,
                 std::string,
                 std::string
-            >,
-            parse::skipper_type
+            >
         > strings_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            void (Encyclopedia&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            void (Encyclopedia&)
         > start_rule;
 
 
@@ -98,10 +93,8 @@ namespace parse {
 
         std::vector<boost::filesystem::path> file_list = ListScripts("scripting/encyclopedia");
 
-        for (std::vector<boost::filesystem::path>::iterator file_it = file_list.begin();
-             file_it != file_list.end(); ++file_it) 
-        {
-            result &= detail::parse_file<rules, Encyclopedia>(*file_it, enc);
+        for (const boost::filesystem::path& file : file_list) {
+            result &= detail::parse_file<rules, Encyclopedia>(file, enc);
         }
 
         return result;
