@@ -1,5 +1,7 @@
 #include "EnumParser.h"
 
+#include "../universe/Enums.h"
+
 
 namespace qi = boost::spirit::qi;
 
@@ -10,135 +12,10 @@ namespace {
 }
 
 namespace parse {
-    template <>
-    enum_parser_rule<PlanetSize>::type& enum_parser<PlanetSize>()
+    enum_rule<EmpireAffiliationType>& empire_affiliation_type_enum()
     {
         const parse::lexer& tok = parse::lexer::instance();
-        static enum_parser_rule<PlanetSize>::type retval
-            =   tok.Tiny_       [ _val = SZ_TINY ]
-            |   tok.Small_      [ _val = SZ_SMALL ]
-            |   tok.Medium_     [ _val = SZ_MEDIUM ]
-            |   tok.Large_      [ _val = SZ_LARGE ]
-            |   tok.Huge_       [ _val = SZ_HUGE ]
-            |   tok.Asteroids_  [ _val = SZ_ASTEROIDS ]
-            |   tok.GasGiant_   [ _val = SZ_GASGIANT ]
-            ;
-        static bool once = true;
-        if (once) {
-            retval.name("PlanetSize");
-#if DEBUG_PARSERS
-            debug(retval);
-#endif
-            once = false;
-        }
-        return retval;
-    }
-
-    template <>
-    enum_parser_rule<PlanetType>::type& enum_parser<PlanetType>()
-    {
-        const parse::lexer& tok = parse::lexer::instance();
-        static enum_parser_rule<PlanetType>::type retval
-            =   tok.Swamp_      [ _val = PT_SWAMP ]
-            |   tok.Toxic_      [ _val = PT_TOXIC ]
-            |   tok.Inferno_    [ _val = PT_INFERNO ]
-            |   tok.Radiated_   [ _val = PT_RADIATED ]
-            |   tok.Barren_     [ _val = PT_BARREN ]
-            |   tok.Tundra_     [ _val = PT_TUNDRA ]
-            |   tok.Desert_     [ _val = PT_DESERT ]
-            |   tok.Terran_     [ _val = PT_TERRAN ]
-            |   tok.Ocean_      [ _val = PT_OCEAN ]
-            |   tok.Asteroids_  [ _val = PT_ASTEROIDS ]
-            |   tok.GasGiant_   [ _val = PT_GASGIANT ]
-            ;
-        static bool once = true;
-        if (once) {
-            retval.name("PlanetType");
-#if DEBUG_PARSERS
-            debug(retval);
-#endif
-            once = false;
-        }
-        return retval;
-    }
-
-    template <>
-    enum_parser_rule<PlanetEnvironment>::type& enum_parser<PlanetEnvironment>()
-    {
-        const parse::lexer& tok = parse::lexer::instance();
-        static enum_parser_rule<PlanetEnvironment>::type retval
-            =   tok.Uninhabitable_  [ _val = PE_UNINHABITABLE ]
-            |   tok.Hostile_        [ _val = PE_HOSTILE ]
-            |   tok.Poor_           [ _val = PE_POOR ]
-            |   tok.Adequate_       [ _val = PE_ADEQUATE ]
-            |   tok.Good_           [ _val = PE_GOOD ]
-            ;
-        static bool once = true;
-        if (once) {
-            retval.name("PlanetEnvironment");
-#if DEBUG_PARSERS
-            debug(retval);
-#endif
-            once = false;
-        }
-        return retval;
-    }
-
-    template <>
-    enum_parser_rule<UniverseObjectType>::type& enum_parser<UniverseObjectType>()
-    {
-        const parse::lexer& tok = parse::lexer::instance();
-        static enum_parser_rule<UniverseObjectType>::type retval
-            =   tok.Building_           [ _val = OBJ_BUILDING ]
-            |   tok.Ship_               [ _val = OBJ_SHIP ]
-            |   tok.Fleet_              [ _val = OBJ_FLEET ]
-            |   tok.Planet_             [ _val = OBJ_PLANET ]
-            |   tok.PopulationCenter_   [ _val = OBJ_POP_CENTER ]
-            |   tok.ProductionCenter_   [ _val = OBJ_PROD_CENTER ]
-            |   tok.System_             [ _val = OBJ_SYSTEM ]
-            |   tok.Field_              [ _val = OBJ_FIELD ]
-            ;
-        static bool once = true;
-        if (once) {
-            retval.name("UniverseObjectType");
-#if DEBUG_PARSERS
-            debug(retval);
-#endif
-            once = false;
-        }
-        return retval;
-    }
-
-    template <>
-    enum_parser_rule<StarType>::type& enum_parser<StarType>()
-    {
-        const parse::lexer& tok = parse::lexer::instance();
-        static enum_parser_rule<StarType>::type retval
-            =   tok.Blue_           [ _val = STAR_BLUE ]
-            |   tok.White_          [ _val = STAR_WHITE ]
-            |   tok.Yellow_         [ _val = STAR_YELLOW ]
-            |   tok.Orange_         [ _val = STAR_ORANGE ]
-            |   tok.Red_            [ _val = STAR_RED ]
-            |   tok.Neutron_        [ _val = STAR_NEUTRON ]
-            |   tok.BlackHole_      [ _val = STAR_BLACK ]
-            |   tok.NoStar_         [ _val = STAR_NONE ]
-            ;
-        static bool once = true;
-        if (once) {
-            retval.name("StarType");
-#if DEBUG_PARSERS
-            debug(retval);
-#endif
-            once = false;
-        }
-        return retval;
-    }
-
-    template <>
-    enum_parser_rule<EmpireAffiliationType>::type& enum_parser<EmpireAffiliationType>()
-    {
-        const parse::lexer& tok = parse::lexer::instance();
-        static enum_parser_rule<EmpireAffiliationType>::type retval
+        static enum_rule<EmpireAffiliationType> retval
             =   tok.TheEmpire_  [ _val = AFFIL_SELF ]
             |   tok.EnemyOf_    [ _val = AFFIL_ENEMY ]
             |   tok.AllyOf_     [ _val = AFFIL_ALLY ]
@@ -158,11 +35,10 @@ namespace parse {
         return retval;
     }
 
-    template <>
-    enum_parser_rule<UnlockableItemType>::type& enum_parser<UnlockableItemType>()
+    enum_rule<UnlockableItemType>& unlockable_item_type_enum()
     {
         const parse::lexer& tok = parse::lexer::instance();
-        static enum_parser_rule<UnlockableItemType>::type retval
+        static enum_rule<UnlockableItemType> retval
             =   tok.Building_       [ _val = UIT_BUILDING ]
             |   tok.ShipPart_       [ _val = UIT_SHIP_PART ]
             |   tok.ShipHull_       [ _val = UIT_SHIP_HULL ]
@@ -180,11 +56,10 @@ namespace parse {
         return retval;
     }
 
-    template <>
-    enum_parser_rule<ShipSlotType>::type& enum_parser<ShipSlotType>()
+    enum_rule<ShipSlotType>& ship_slot_type_enum()
     {
         const parse::lexer& tok = parse::lexer::instance();
-        static enum_parser_rule<ShipSlotType>::type retval
+        static enum_rule<ShipSlotType> retval
             =   tok.External_   [ _val = SL_EXTERNAL ]
             |   tok.Internal_   [ _val = SL_INTERNAL ]
             |   tok.Core_       [ _val = SL_CORE ]
@@ -200,11 +75,10 @@ namespace parse {
         return retval;
     }
 
-    template <>
-    enum_parser_rule<ShipPartClass>::type& enum_parser<ShipPartClass>()
+    enum_rule<ShipPartClass>& ship_part_class_enum()
     {
         const parse::lexer& tok = parse::lexer::instance();
-        static enum_parser_rule<ShipPartClass>::type retval
+        static enum_rule<ShipPartClass> retval
             =   tok.ShortRange_         [ _val = PC_DIRECT_WEAPON ]
             |   tok.FighterBay_         [ _val = PC_FIGHTER_BAY ]
             |   tok.FighterHangar_      [ _val = PC_FIGHTER_HANGAR ]
@@ -221,7 +95,7 @@ namespace parse {
             |   tok.Research_           [ _val = PC_RESEARCH ]
             |   tok.Industry_           [ _val = PC_INDUSTRY ]
             |   tok.Trade_              [ _val = PC_TRADE ]
-            |   tok.ProductionLocation_ [ _val = PC_PRODICTION_LOCATION ]
+            |   tok.ProductionLocation_ [ _val = PC_PRODUCTION_LOCATION ]
             ;
         static bool once = true;
         if (once) {
@@ -234,11 +108,10 @@ namespace parse {
         return retval;
     }
 
-    template <>
-    enum_parser_rule<CaptureResult>::type& enum_parser<CaptureResult>()
+    enum_rule<CaptureResult>& capture_result_enum()
     {
         const parse::lexer& tok = parse::lexer::instance();
-        static enum_parser_rule<CaptureResult>::type retval
+        static enum_rule<CaptureResult> retval
             =   tok.Capture_            [ _val = CR_CAPTURE ]
             |   tok.Retain_             [ _val = CR_RETAIN ]
             |   tok.Destroy_            [ _val = CR_DESTROY ]
@@ -254,11 +127,10 @@ namespace parse {
         return retval;
     }
 
-    template <>
-    enum_parser_rule<ValueRef::StatisticType>::type& enum_parser<ValueRef::StatisticType>()
+    enum_rule<ValueRef::StatisticType>& statistic_type_enum()
     {
         const parse::lexer& tok = parse::lexer::instance();
-        static enum_parser_rule<ValueRef::StatisticType>::type retval
+        static enum_rule<ValueRef::StatisticType> retval
             =   tok.CountUnique_    [ _val = ValueRef::UNIQUE_COUNT ]
             |   tok.Count_          [ _val = ValueRef::COUNT ]
             |   tok.If_             [ _val = ValueRef::IF ]
@@ -283,9 +155,9 @@ namespace parse {
         return retval;
     }
 
-    enum_parser_rule<MeterType>::type& non_ship_part_meter_type_enum() {
+    enum_rule<MeterType>& non_ship_part_meter_type_enum() {
         const parse::lexer& tok = parse::lexer::instance();
-        static enum_parser_rule<MeterType>::type retval
+        static enum_rule<MeterType> retval
             =   tok.TargetConstruction_     [ _val = METER_TARGET_CONSTRUCTION ]
             |   tok.TargetIndustry_         [ _val = METER_TARGET_INDUSTRY ]
             |   tok.TargetPopulation_       [ _val = METER_TARGET_POPULATION ]
@@ -332,9 +204,9 @@ namespace parse {
         return retval;
     }
 
-    enum_parser_rule<MeterType>::type& ship_part_meter_type_enum() {
+    enum_rule<MeterType>& ship_part_meter_type_enum() {
         const parse::lexer& tok = parse::lexer::instance();
-        static enum_parser_rule<MeterType>::type retval
+        static enum_rule<MeterType> retval
             =   tok.MaxCapacity_            [ _val = METER_MAX_CAPACITY ]
             |   tok.MaxDamage_              [ _val = METER_MAX_CAPACITY ]
             |   tok.Capacity_               [ _val = METER_CAPACITY ]
@@ -351,9 +223,9 @@ namespace parse {
         return retval;
     }
 
-    enum_parser_rule<MeterType>::type& set_non_ship_part_meter_type_enum() {
+    enum_rule<MeterType>& set_non_ship_part_meter_type_enum() {
         const parse::lexer& tok = parse::lexer::instance();
-        static enum_parser_rule<MeterType>::type retval
+        static enum_rule<MeterType> retval
             =   tok.SetTargetConstruction_  [ _val = METER_TARGET_CONSTRUCTION ]
             |   tok.SetTargetIndustry_      [ _val = METER_TARGET_INDUSTRY ]
             |   tok.SetTargetPopulation_    [ _val = METER_TARGET_POPULATION ]
@@ -400,10 +272,10 @@ namespace parse {
         return retval;
     }
 
-    enum_parser_rule<MeterType>::type& set_ship_part_meter_type_enum()
+    enum_rule<MeterType>& set_ship_part_meter_type_enum()
     {
         const parse::lexer& tok = parse::lexer::instance();
-        static enum_parser_rule<MeterType>::type retval
+        static enum_rule<MeterType> retval
             =   tok.SetMaxCapacity_         [ _val = METER_MAX_CAPACITY ]
             |   tok.SetMaxDamage_           [ _val = METER_MAX_CAPACITY ]
             |   tok.SetMaxSecondaryStat_    [ _val = METER_MAX_SECONDARY_STAT ]
