@@ -1,6 +1,5 @@
-#include "Double.h"
-#include "Label.h"
 #include "Parse.h"
+
 #include "ParseImpl.h"
 
 #include "../universe/Condition.h"
@@ -8,11 +7,12 @@
 
 #include <boost/spirit/include/phoenix.hpp>
 
+
 #define DEBUG_PARSERS 0
 
 #if DEBUG_PARSERS
 namespace std {
-    inline ostream& operator<<(ostream& os, const std::vector<boost::shared_ptr<Effect::EffectsGroup> >&) { return os; }
+    inline ostream& operator<<(ostream& os, const std::vector<std::shared_ptr<Effect::EffectsGroup>>&) { return os; }
     inline ostream& operator<<(ostream& os, const std::map<std::string, FieldType*>&) { return os; }
     inline ostream& operator<<(ostream& os, const std::pair<const std::string, FieldType*>&) { return os; }
 }
@@ -20,13 +20,7 @@ namespace std {
 
 namespace {
     struct insert_ {
-#if BOOST_VERSION < 105600
-        template <typename Arg1, typename Arg2> // Phoenix v2
-        struct result
-        { typedef void type; };
-#else
         typedef void result_type;
-#endif
 
         void operator()(std::map<std::string, FieldType*>& fields, FieldType* field_type) const {
             if (!fields.insert(std::make_pair(field_type->Name(), field_type)).second) {
@@ -39,7 +33,10 @@ namespace {
 
     struct rules {
         rules() {
-            const parse::lexer& tok = parse::lexer::instance();
+            namespace phoenix = boost::phoenix;
+            namespace qi = boost::spirit::qi;
+
+            using phoenix::new_;
 
             qi::_1_type _1;
             qi::_2_type _2;
@@ -51,16 +48,17 @@ namespace {
             qi::_d_type _d;
             qi::_e_type _e;
             qi::_r1_type _r1;
-            using phoenix::new_;
+
+            const parse::lexer& tok = parse::lexer::instance();
 
             field
                 =   tok.FieldType_
-                >   parse::label(Name_token)                > tok.string [ _a = _1 ]
-                >   parse::label(Description_token)         > tok.string [ _b = _1 ]
-                >   parse::label(Stealth_token)             > parse::double_ [ _c = _1]
+                >   parse::detail::label(Name_token)                > tok.string [ _a = _1 ]
+                >   parse::detail::label(Description_token)         > tok.string [ _b = _1 ]
+                >   parse::detail::label(Stealth_token)             > parse::detail::double_ [ _c = _1]
                 >   parse::detail::tags_parser()(_d)
-                > -(parse::label(EffectsGroups_token)       > parse::detail::effects_group_parser() [ _e = _1 ])
-                >   parse::label(Graphic_token)             > tok.string
+                > -(parse::detail::label(EffectsGroups_token)       > parse::detail::effects_group_parser() [ _e = _1 ])
+                >   parse::detail::label(Graphic_token)             > tok.string
                 [ insert(_r1, new_<FieldType>(_a, _b, _c, _d, _e, _1)) ]
                 ;
 
@@ -77,23 +75,19 @@ namespace {
             qi::on_error<qi::fail>(start, parse::report_error(_1, _2, _3, _4));
         }
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             void (std::map<std::string, FieldType*>&),
-            qi::locals<
+            boost::spirit::qi::locals<
                 std::string,
                 std::string,
                 float,
                 std::set<std::string>,
-                std::vector<boost::shared_ptr<Effect::EffectsGroup> >
-            >,
-            parse::skipper_type
+                std::vector<std::shared_ptr<Effect::EffectsGroup>>
+            >
         > field_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            void (std::map<std::string, FieldType*>&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            void (std::map<std::string, FieldType*>&)
         > start_rule;
 
         field_rule          field;
@@ -105,12 +99,8 @@ namespace parse {
     bool fields(std::map<std::string, FieldType*>& field_types) {
         bool result = true;
 
-        std::vector<boost::filesystem::path> file_list = ListScripts("scripting/fields");
-
-        for (std::vector<boost::filesystem::path>::iterator file_it = file_list.begin();
-             file_it != file_list.end(); ++file_it)
-        {
-            result &= detail::parse_file<rules, std::map<std::string, FieldType*> >(*file_it, field_types);
+        for (const boost::filesystem::path& file : ListScripts("scripting/fields")) {
+            result &= detail::parse_file<rules, std::map<std::string, FieldType*>>(file, field_types);
         }
 
         return result;
