@@ -1,13 +1,13 @@
-#include "Label.h"
 #include "Parse.h"
+
 #include "ParseImpl.h"
 
-#include "../universe/Universe.h"
 #include "../universe/UniverseGenerator.h"
 #include "../util/Directories.h"
 
 #include <boost/spirit/include/phoenix.hpp>
 
+
 #define DEBUG_PARSERS 0
 
 #if DEBUG_PARSERS
@@ -20,7 +20,11 @@ namespace std {
 namespace {
     struct rules {
         rules() {
-            const parse::lexer& tok = parse::lexer::instance();
+            namespace phoenix = boost::phoenix;
+            namespace qi = boost::spirit::qi;
+
+            using phoenix::new_;
+            using phoenix::push_back;
 
             qi::_1_type _1;
             qi::_2_type _2;
@@ -29,16 +33,16 @@ namespace {
             qi::_a_type _a;
             qi::_b_type _b;
             qi::_r1_type _r1;
-            using phoenix::new_;
-            using phoenix::push_back;
+
+            const parse::lexer& tok = parse::lexer::instance();
 
             fleet_plan
                 =    tok.Fleet_
-                >    parse::label(Name_token) > tok.string [ _a = _1 ]
-                >    parse::label(Ships_token)
+                >    parse::detail::label(Name_token) > tok.string [ _a = _1 ]
+                >    parse::detail::label(Ships_token)
                 >    (
-                            '[' > +tok.string [ push_back(_b, _1) ] > ']'
-                        |   tok.string [ push_back(_b, _1) ]
+                            ('[' > +tok.string [ push_back(_b, _1) ] > ']')
+                        |    tok.string [ push_back(_b, _1) ]
                      )
                 [ push_back(_r1, new_<FleetPlan>(_a, _b, phoenix::val(true))) ]
                 ;
@@ -56,20 +60,16 @@ namespace {
             qi::on_error<qi::fail>(start, parse::report_error(_1, _2, _3, _4));
         }
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             void (std::vector<FleetPlan*>&),
-            qi::locals<
+            boost::spirit::qi::locals<
                 std::string,
                 std::vector<std::string>
-            >,
-            parse::skipper_type
+            >
         > fleet_plan_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            void (std::vector<FleetPlan*>&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            void (std::vector<FleetPlan*>&)
         > start_rule;
 
         fleet_plan_rule fleet_plan;
@@ -80,6 +80,6 @@ namespace {
 namespace parse {
     bool fleet_plans(std::vector<FleetPlan*>& fleet_plans_) {
         const boost::filesystem::path& path = GetResourceDir() / "scripting/starting_unlocks/fleets.inf";
-        return detail::parse_file<rules, std::vector<FleetPlan*> >(path, fleet_plans_);
+        return detail::parse_file<rules, std::vector<FleetPlan*>>(path, fleet_plans_);
     }
 }
