@@ -1,17 +1,18 @@
-#include "Label.h"
 #include "Parse.h"
+
 #include "ParseImpl.h"
 
 #include "../util/Directories.h"
 
 #include <boost/spirit/include/phoenix.hpp>
 
+
 #define DEBUG_PARSERS 0
 
 #if DEBUG_PARSERS
 namespace std {
     inline ostream& operator<<(ostream& os, const std::map<int, int>&) { return os; 
-    inline ostream& operator<<(ostream& os, const std::map<std::string, std::map<int, int> >&) { return os; }
+    inline ostream& operator<<(ostream& os, const std::map<std::string, std::map<int, int>>&) { return os; }
 }
 #endif
 
@@ -20,13 +21,7 @@ namespace {
     typedef std::map<std::string, Keymap>   NamedKeymaps;
 
     struct insert_key_pair_ {
-#if BOOST_VERSION < 105600
-        template <typename Arg1, typename Arg2> // Phoenix v2
-        struct result
-        { typedef void type; };
-#else
         typedef void result_type;
-#endif
 
         void operator()(Keymap& keymap, const Keymap::value_type& key_id_pair) const {
             keymap[key_id_pair.first] = key_id_pair.second;
@@ -36,13 +31,7 @@ namespace {
     const boost::phoenix::function<insert_key_pair_> insert_key_pair;
 
     struct insert_key_map_ {
-#if BOOST_VERSION < 105600
-        template <typename Arg1, typename Arg2> // Phoenix v2
-        struct result
-        { typedef void type; };
-#else
         typedef void result_type;
-#endif
 
         void operator()(NamedKeymaps& named_keymaps, const NamedKeymaps::value_type& name_keymap) const {
             named_keymaps[name_keymap.first] = name_keymap.second;
@@ -53,7 +42,10 @@ namespace {
 
     struct rules {
         rules() {
-            const parse::lexer& tok = parse::lexer::instance();
+            namespace phoenix = boost::phoenix;
+            namespace qi = boost::spirit::qi;
+
+            using phoenix::construct;
 
             qi::_1_type _1;
             qi::_2_type _2;
@@ -62,7 +54,8 @@ namespace {
             qi::_a_type _a;
             qi::_b_type _b;
             qi::_r1_type _r1;
-            using phoenix::construct;
+
+            const parse::lexer& tok = parse::lexer::instance();
 
             int_pair
                 =   tok.int_ [ _a = _1 ] >> tok.int_ [ _b = _1 ]
@@ -71,8 +64,8 @@ namespace {
 
             keymap
                 =   tok.Keymap_
-                >   parse::label(Name_token) > tok.string [ _a = _1 ]
-                >   parse::label(Keys_token)
+                >   parse::detail::label(Name_token) > tok.string [ _a = _1 ]
+                >   parse::detail::label(Keys_token)
                 >   ( '[' > *(int_pair(_b)) > ']' )
                     [ insert_key_map(_r1, construct<NamedKeymaps::value_type>(_a, _b)) ]
                 ;
@@ -92,24 +85,18 @@ namespace {
             qi::on_error<qi::fail>(start, parse::report_error(_1, _2, _3, _4));
         }
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             void (Keymap&),
-            qi::locals<int, int>,
-            parse::skipper_type
+            boost::spirit::qi::locals<int, int>
         > int_pair_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             void (NamedKeymaps&),
-            qi::locals<std::string, Keymap>,
-            parse::skipper_type
+            boost::spirit::qi::locals<std::string, Keymap>
         > keymap_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            void (NamedKeymaps&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            void (NamedKeymaps&)
         > start_rule;
 
         int_pair_rule   int_pair;
