@@ -1,9 +1,7 @@
-#include "ConditionParserImpl.h"
-#include "Double.h"
-#include "Int.h"
-#include "Label.h"
 #include "Parse.h"
+
 #include "ParseImpl.h"
+#include "ConditionParserImpl.h"
 
 #include "../universe/Universe.h"
 #include "../universe/UniverseGenerator.h"
@@ -11,6 +9,7 @@
 
 #include <boost/spirit/include/phoenix.hpp>
 
+
 #define DEBUG_PARSERS 0
 
 #if DEBUG_PARSERS
@@ -22,13 +21,7 @@ namespace std {
 
 namespace {
     struct new_monster_fleet_plan_ {
-#if BOOST_VERSION < 105600
-        template <typename Arg1, typename Arg2, typename Arg3, typename Arg4, typename Arg5> // Phoenix v2
-        struct result
-        { typedef MonsterFleetPlan* type; };
-#else
         typedef MonsterFleetPlan* result_type;
-#endif
 
         MonsterFleetPlan* operator()(const std::string& fleet_name, const std::vector<std::string>& ship_design_names,
                                      double spawn_rate, int spawn_limit, Condition::ConditionBase* location) const
@@ -38,7 +31,11 @@ namespace {
 
     struct rules {
         rules() {
-            const parse::lexer& tok = parse::lexer::instance();
+            namespace phoenix = boost::phoenix;
+            namespace qi = boost::spirit::qi;
+
+            using phoenix::clear;
+            using phoenix::push_back;
 
             qi::_1_type _1;
             qi::_2_type _2;
@@ -47,31 +44,31 @@ namespace {
             qi::_r1_type _r1;
             qi::_val_type _val;
             qi::eps_type eps;
-            using phoenix::clear;
-            using phoenix::push_back;
+
+            const parse::lexer& tok = parse::lexer::instance();
 
             monster_fleet_plan_prefix
                 =    tok.MonsterFleet_
-                >    parse::label(Name_token) > tok.string [ phoenix::ref(_a) = _1 ]
+                >    parse::detail::label(Name_token) > tok.string [ phoenix::ref(_a) = _1 ]
                 ;
 
             ships
-                =    parse::label(Ships_token)
+                =    parse::detail::label(Ships_token)
                 >    eps [ clear(phoenix::ref(_b)) ]
                 >    (
-                            '[' > +tok.string [ push_back(phoenix::ref(_b), _1) ] > ']'
-                        |   tok.string [ push_back(phoenix::ref(_b), _1) ]
+                            ('[' > +tok.string [ push_back(phoenix::ref(_b), _1) ] > ']')
+                        |    tok.string [ push_back(phoenix::ref(_b), _1) ]
                      )
                 ;
 
             spawns
                 =    (
-                            parse::label(SpawnRate_token) > parse::double_ [ phoenix::ref(_c) = _1 ]
-                        |   eps [ phoenix::ref(_c) = 1.0 ]
+                            (parse::detail::label(SpawnRate_token) > parse::detail::double_ [ phoenix::ref(_c) = _1 ])
+                        |    eps [ phoenix::ref(_c) = 1.0 ]
                      )
                 >    (
-                            parse::label(SpawnLimit_token) > parse::int_ [ phoenix::ref(_d) = _1 ]
-                        |   eps [ phoenix::ref(_d) = 9999 ]
+                            (parse::detail::label(SpawnLimit_token) > parse::detail::int_ [ phoenix::ref(_d) = _1 ])
+                        |    eps [ phoenix::ref(_d) = 9999 ]
                      )
                 ;
 
@@ -80,7 +77,7 @@ namespace {
                             monster_fleet_plan_prefix
                         >   ships
                         >   spawns
-                        > -(parse::label(Location_token) > parse::detail::condition_parser [ phoenix::ref(_e) = _1 ])
+                        > -(parse::detail::label(Location_token) > parse::detail::condition_parser [ phoenix::ref(_e) = _1 ])
                      )
                 [ _val = new_monster_fleet_plan(phoenix::ref(_a), phoenix::ref(_b), phoenix::ref(_c), phoenix::ref(_d), phoenix::ref(_e)) ]
                 ;
@@ -101,21 +98,14 @@ namespace {
             qi::on_error<qi::fail>(start, parse::report_error(_1, _2, _3, _4));
         }
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            parse::skipper_type
-        > generic_rule;
+        typedef parse::detail::rule<> generic_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            MonsterFleetPlan* (),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            MonsterFleetPlan* ()
         > monster_fleet_plan_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            void (std::vector<MonsterFleetPlan*>&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            void (std::vector<MonsterFleetPlan*>&)
         > start_rule;
 
         generic_rule            monster_fleet_plan_prefix;
@@ -137,6 +127,6 @@ namespace {
 namespace parse {
     bool monster_fleet_plans(std::vector<MonsterFleetPlan*>& monster_fleet_plans_) {
         boost::filesystem::path path = GetResourceDir() / "scripting/monster_fleets.inf";
-        return detail::parse_file<rules, std::vector<MonsterFleetPlan*> >(path, monster_fleet_plans_);
+        return detail::parse_file<rules, std::vector<MonsterFleetPlan*>>(path, monster_fleet_plans_);
     }
 }
