@@ -1,11 +1,9 @@
-#include "ConditionParserImpl.h"
-#include "Double.h"
-#include "EffectParser.h"
-#include "EnumParser.h"
-#include "Int.h"
-#include "Label.h"
 #include "Parse.h"
 #include "ParseImpl.h"
+
+#include "ConditionParserImpl.h"
+#include "EffectParser.h"
+#include "EnumParser.h"
 #include "ValueRefParser.h"
 
 #include "../universe/Effect.h"
@@ -32,15 +30,19 @@ namespace std {
 namespace {
     struct tags_rules {
         tags_rules() {
-            const parse::lexer& tok = parse::lexer::instance();
+            namespace phoenix = boost::phoenix;
+            namespace qi = boost::spirit::qi;
+
+            using phoenix::insert;
 
             qi::_1_type _1;
             qi::_r1_type _r1;
-            using phoenix::insert;
+
+            const parse::lexer& tok = parse::lexer::instance();
 
             start
                 =  -(
-                        parse::label(Tags_token)
+                        parse::detail::label(Tags_token)
                     >>  (
                             ('[' > +tok.string [ insert(_r1, _1) ] > ']')
                             |   tok.string [ insert(_r1, _1) ]
@@ -60,7 +62,12 @@ namespace {
 
     struct effects_group_rules {
         effects_group_rules() {
-            const parse::lexer& tok = parse::lexer::instance();
+            namespace phoenix = boost::phoenix;
+            namespace qi = boost::spirit::qi;
+
+            using phoenix::construct;
+            using phoenix::new_;
+            using phoenix::push_back;
 
             qi::_1_type _1;
             qi::_a_type _a;
@@ -73,29 +80,28 @@ namespace {
             qi::_val_type _val;
             qi::lit_type lit;
             qi::eps_type eps;
-            using phoenix::construct;
-            using phoenix::new_;
-            using phoenix::push_back;
+
+            const parse::lexer& tok = parse::lexer::instance();
 
             effects_group
                 =   tok.EffectsGroup_
-                > -(parse::label(Description_token)      > tok.string [ _g = _1 ])
-                >   parse::label(Scope_token)            > parse::detail::condition_parser [ _a = _1 ]
-                > -(parse::label(Activation_token)       > parse::detail::condition_parser [ _b = _1 ])
-                > -(parse::label(StackingGroup_token)    > tok.string [ _c = _1 ])
-                > -(parse::label(AccountingLabel_token)  > tok.string [ _e = _1 ])
-                > ((parse::label(Priority_token)         > tok.int_ [ _f = _1 ]) | eps [ _f = 100 ])
-                >   parse::label(Effects_token)
+                > -(parse::detail::label(Description_token)      > tok.string [ _g = _1 ])
+                >   parse::detail::label(Scope_token)            > parse::detail::condition_parser [ _a = _1 ]
+                > -(parse::detail::label(Activation_token)       > parse::detail::condition_parser [ _b = _1 ])
+                > -(parse::detail::label(StackingGroup_token)    > tok.string [ _c = _1 ])
+                > -(parse::detail::label(AccountingLabel_token)  > tok.string [ _e = _1 ])
+                > ((parse::detail::label(Priority_token)         > tok.int_ [ _f = _1 ]) | eps [ _f = 100 ])
+                >   parse::detail::label(Effects_token)
                 >   (
-                            '[' > +parse::effect_parser() [ push_back(_d, _1) ] > ']'
-                        |   parse::effect_parser() [ push_back(_d, _1) ]
+                            ('[' > +parse::effect_parser() [ push_back(_d, _1) ] > ']')
+                        |    parse::effect_parser() [ push_back(_d, _1) ]
                     )
                     [ _val = new_<Effect::EffectsGroup>(_a, _b, _d, _e, _c, _f, _g) ]
                 ;
 
             start
-                =    '[' > +effects_group [ push_back(_val, construct<boost::shared_ptr<Effect::EffectsGroup> >(_1)) ] > ']'
-                |    effects_group [ push_back(_val, construct<boost::shared_ptr<Effect::EffectsGroup> >(_1)) ]
+                =    ('[' > +effects_group [ push_back(_val, construct<std::shared_ptr<Effect::EffectsGroup>>(_1)) ] > ']')
+                |     effects_group [ push_back(_val, construct<std::shared_ptr<Effect::EffectsGroup>>(_1)) ]
                 ;
 
             effects_group.name("EffectsGroup");
@@ -107,10 +113,9 @@ namespace {
 #endif
         }
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             Effect::EffectsGroup* (),
-            qi::locals<
+            boost::spirit::qi::locals<
                 Condition::ConditionBase*,
                 Condition::ConditionBase*,
                 std::string,
@@ -118,8 +123,7 @@ namespace {
                 std::string,
                 int,
                 std::string
-            >,
-            parse::skipper_type
+            >
         > effects_group_rule;
 
         effects_group_rule effects_group;
@@ -128,7 +132,11 @@ namespace {
 
     struct color_parser_rules {
         color_parser_rules() {
-            const parse::lexer& tok = parse::lexer::instance();
+            namespace phoenix = boost::phoenix;
+            namespace qi = boost::spirit::qi;
+
+            using phoenix::construct;
+            using phoenix::if_;
 
             qi::_1_type _1;
             qi::_a_type _a;
@@ -138,8 +146,8 @@ namespace {
             qi::_val_type _val;
             qi::eps_type eps;
             qi::uint_type uint_;
-            using phoenix::construct;
-            using phoenix::if_;
+
+            const parse::lexer& tok = parse::lexer::instance();
 
             channel
                 =    tok.int_ [ _val = _1, _pass = 0 <= _1 && _1 <= 255 ]
@@ -167,10 +175,8 @@ namespace {
 #endif
         }
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            unsigned int (),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            unsigned int ()
         > rule;
 
         rule channel;
@@ -179,17 +185,21 @@ namespace {
 
     struct item_spec_parser_rules {
         item_spec_parser_rules() {
-            const parse::lexer& tok = parse::lexer::instance();
+            namespace phoenix = boost::phoenix;
+            namespace qi = boost::spirit::qi;
+
+            using phoenix::construct;
 
             qi::_1_type _1;
             qi::_a_type _a;
             qi::_val_type _val;
-            using phoenix::construct;
+
+            const parse::lexer& tok = parse::lexer::instance();
 
             start
                 =    tok.Item_
-                >    parse::label(Type_token) > parse::enum_parser<UnlockableItemType>() [ _a = _1 ]
-                >    parse::label(Name_token) > tok.string [ _val = construct<ItemSpec>(_a, _1) ]
+                >    parse::detail::label(Type_token) > parse::unlockable_item_type_enum() [ _a = _1 ]
+                >    parse::detail::label(Name_token) > tok.string [ _val = construct<ItemSpec>(_a, _1) ]
                 ;
 
             start.name("ItemSpec");
@@ -205,32 +215,37 @@ namespace {
 
 namespace parse {
     void init() {
-        const lexer& tok = lexer::instance();
-        qi::_1_type _1;
-        qi::_val_type _val;
+        namespace phoenix = boost::phoenix;
+        namespace qi = boost::spirit::qi;
+
         using phoenix::static_cast_;
 
-        int_
+        qi::_1_type _1;
+        qi::_val_type _val;
+
+        const lexer& tok = lexer::instance();
+
+        detail::int_
             =    '-' >> tok.int_ [ _val = -_1 ]
             |    tok.int_ [ _val = _1 ]
             ;
 
-        double_
+        detail::double_
             =    '-' >> tok.int_ [ _val = -static_cast_<double>(_1) ]
             |    tok.int_ [ _val = static_cast_<double>(_1) ]
             |    '-' >> tok.double_ [ _val = -_1 ]
             |    tok.double_ [ _val = _1 ]
             ;
 
-        int_.name("integer");
-        double_.name("real number");
+        detail::int_.name("integer");
+        detail::double_.name("real number");
 
 #if DEBUG_PARSERS
-        debug(int_);
-        debug(double_);
+        debug(detail::int_);
+        debug(detail::double_);
 #endif
 
-        value_ref_parser<int>();
+        int_value_ref();
 
         condition_parser();
     }
@@ -309,11 +324,8 @@ namespace parse {
         if (macros_directly_referenced_in_input_text.find(macro_to_find) != macros_directly_referenced_in_input_text.end())
             return true;
         // check if macros referenced in text reference macro_to_find
-        for (std::set<std::string>::const_iterator direct_refs_it = macros_directly_referenced_in_input_text.begin();
-             direct_refs_it != macros_directly_referenced_in_input_text.end(); ++direct_refs_it)
-        {
+        for (const std::string& direct_referenced_macro_key : macros_directly_referenced_in_input_text) {
             // get text of directly referenced macro
-            const std::string& direct_referenced_macro_key = *direct_refs_it;
             std::map<std::string, std::string>::const_iterator macro_it = macros.find(direct_referenced_macro_key);
             if (macro_it == macros.end()) {
                 ErrorLogger() << "macro_deep_referenced_in_text couldn't find referenced macro: " << direct_referenced_macro_key;
@@ -329,11 +341,9 @@ namespace parse {
     }
 
     void check_for_cyclic_macro_references(const std::map<std::string, std::string>& macros) {
-        for (std::map<std::string, std::string>::const_iterator macro_it = macros.begin();
-             macro_it != macros.end(); ++macro_it)
-        {
-            if (macro_deep_referenced_in_text(macro_it->first, macro_it->second, macros))
-                ErrorLogger() << "Cyclic macro found: " << macro_it->first << " references itself (eventually)";
+        for (const std::map<std::string, std::string>::value_type& macro : macros) {
+            if (macro_deep_referenced_in_text(macro.first, macro.second, macros))
+                ErrorLogger() << "Cyclic macro found: " << macro.first << " references itself (eventually)";
         }
     }
 
@@ -364,7 +374,7 @@ namespace parse {
                                 ++it, ++replace_number)
                             {
                                 // not using %1% (and boost::fmt) because the replaced text may itself have %s inside it that will get eaten
-                                boost::replace_all(replacement, "@" + boost::lexical_cast<std::string>(replace_number) + "@", boost::copy_range<std::string>(*it));
+                                boost::replace_all(replacement, "@" + std::to_string(replace_number) + "@", boost::copy_range<std::string>(*it));
                             }
                         }
                         text.replace(position, matched_text.length(), replacement);
@@ -395,9 +405,8 @@ namespace parse {
 
         // recursively expand macro keys: replace [[MACRO_KEY]] in other macro
         // text with the macro text corresponding to MACRO_KEY.
-        for (std::map<std::string, std::string>::iterator macro_it = macros.begin();
-             macro_it != macros.end(); ++macro_it)
-        { replace_macro_references(macro_it->second, macros); }
+        for (std::map<std::string, std::string>::value_type& macro : macros)
+        { replace_macro_references(macro.second, macros); }
 
         // substitute macro keys - replace [[MACRO_KEY]] in the input text with
         // the macro text corresponding to MACRO_KEY
@@ -412,9 +421,8 @@ namespace parse {
             return false;
 
         // skip byte order mark (BOM)
-        static const int UTF8_BOM[3] = {0x00EF, 0x00BB, 0x00BF};
-        for (int i = 0; i < 3; i++) {
-            if (UTF8_BOM[i] != ifs.get()) {
+        for (int BOM : {0xEF, 0xBB, 0xBF}) {
+            if (BOM != ifs.get()) {
                 // no header set stream back to start of file
                 ifs.seekg(0, std::ios::beg);
                 // and continue
@@ -435,18 +443,15 @@ namespace parse {
      */
     std::vector<boost::filesystem::path> ListScripts(const boost::filesystem::path& path) {
         std::vector<boost::filesystem::path> retval;
-        std::vector<boost::filesystem::path> fn_list = ListDir(path);
 
         try {
-            for (std::vector<boost::filesystem::path>::iterator fn_it = fn_list.begin();
-                 fn_it != fn_list.end(); ++fn_it)
-            {
-                std::string fn_ext = fn_it->extension().string();
-                std::string fn_stem_ext = fn_it->stem().extension().string();
+            for (const boost::filesystem::path& file : ListDir(path)) {
+                std::string fn_ext = file.extension().string();
+                std::string fn_stem_ext = file.stem().extension().string();
                 if (fn_ext == ".txt" && fn_stem_ext == ".focs") {
-                    retval.push_back(*fn_it);
+                    retval.push_back(file);
                 } else {
-                    TraceLogger() << "Parse: Skipping file " << fn_it->string() 
+                    TraceLogger() << "Parse: Skipping file " << file.string()
                                   << " due to extension (" << fn_stem_ext << fn_ext << ")";
                 }
             }
@@ -522,33 +527,28 @@ namespace parse {
                     continue;
                 }
                 fn_str = fn_str.substr(1, fn_str.size() - 1);
-                std::vector<boost::filesystem::path> fn_list = ListDir(match_path.parent_path());
                 std::set<boost::filesystem::path> match_list;
                 // filter results
-                for (std::vector<boost::filesystem::path>::iterator fn_it = fn_list.begin();
-                    fn_it != fn_list.end(); ++fn_it)
-                {
-                    std::string it_str = fn_it->filename().string();
+                for (const boost::filesystem::path& file : ListDir(match_path.parent_path())) {
+                    std::string it_str = file.filename().string();
                     std::size_t it_len = it_str.length();
                     std::size_t match_len = fn_str.length();
                     if (it_len > match_len) {
                         if (it_str.substr(it_len - match_len, match_len) == fn_str) {
-                            match_list.insert(*fn_it);
+                            match_list.insert(file);
                         }
                     }
                 }
                 // read in results
                 std::string dir_text;
-                for (std::set<boost::filesystem::path>::iterator list_it = match_list.begin();
-                     list_it != match_list.end(); ++list_it)
-                {
-                    if (files_included.insert(boost::filesystem::canonical(*list_it)).second) {
+                for (const boost::filesystem::path& file : match_list) {
+                    if (files_included.insert(boost::filesystem::canonical(file)).second) {
                         std::string new_text;
-                        if (read_file(*list_it, new_text)) {
+                        if (read_file(file, new_text)) {
                             new_text.append("\n");
                             dir_text.append(new_text);
                         } else {
-                            ErrorLogger() << "Parse: Unable to read file " << list_it->string();
+                            ErrorLogger() << "Parse: Unable to read file " << file.string();
                         }
                     }
                 }
@@ -571,6 +571,29 @@ namespace parse {
     }
 
     namespace detail {
+        double_rule double_;
+
+        int_rule int_;
+
+        label_rule& label(const char* name) {
+            static const bool PARSING_LABELS_OPTIONAL = false;
+            static std::map<const char*, label_rule> rules;
+            std::map<const char*, label_rule>::iterator it = rules.find(name);
+            if (it == rules.end()) {
+                const lexer& l = lexer::instance();
+                label_rule& retval = rules[name];
+                if (PARSING_LABELS_OPTIONAL) {
+                    retval = -(l.name_token(name) >> '=');
+                } else {
+                    retval =  (l.name_token(name) >> '=');
+                }
+                retval.name(std::string(name) + " =");
+                return retval;
+            } else {
+                return it->second;
+            }
+        }
+
         tags_rule& tags_parser() {
             static tags_rules rules;
             return rules.start;
