@@ -42,6 +42,8 @@ namespace parse {
 
     FO_PARSE_API bool items(std::vector<ItemSpec>& items_);
 
+    FO_PARSE_API bool starting_buildings(std::vector<ItemSpec>& starting_buildings_);
+
     FO_PARSE_API bool ship_parts(std::map<std::string, PartType*>& parts);
 
     FO_PARSE_API bool ship_hulls(std::map<std::string, HullType*>& hulls);
@@ -58,13 +60,13 @@ namespace parse {
     FO_PARSE_API bool monster_fleet_plans(std::vector<MonsterFleetPlan*>& monster_fleet_plans_);
 
     FO_PARSE_API bool alignments(std::vector<Alignment>& alignments_,
-                                 std::vector<boost::shared_ptr<Effect::EffectsGroup> >& effects_groups);
+                                 std::vector<std::shared_ptr<Effect::EffectsGroup>>& effects_groups);
 
     FO_PARSE_API bool statistics(std::map<std::string, ValueRef::ValueRefBase<double>*>& stats_);
 
     FO_PARSE_API bool encyclopedia_articles(Encyclopedia& enc);
 
-    FO_PARSE_API bool keymaps(std::map<std::string, std::map<int, int> >& nkm);
+    FO_PARSE_API bool keymaps(std::map<std::string, std::map<int, int>>& nkm);
 
     FO_PARSE_API bool read_file(const boost::filesystem::path& path, std::string& file_contents);
 
