@@ -11,45 +11,62 @@
 
 #include <GG/Clr.h>
 
-namespace qi = boost::spirit::qi;
-namespace phoenix = boost::phoenix;
 
 namespace parse { namespace detail {
+    template <
+        typename signature = boost::spirit::qi::unused_type,
+        typename locals = boost::spirit::qi::unused_type
+    >
+    using rule = boost::spirit::qi::rule<
+        parse::token_iterator,
+        parse::skipper_type,
+        signature,
+        locals
+    >;
 
-    typedef qi::rule<
-        token_iterator,
-        void (std::set<std::string>&),
-        parse::skipper_type
+
+    using double_rule = detail::rule<
+        double ()
+    >;
+    extern double_rule double_;
+
+
+    using int_rule = detail::rule<
+        int ()
+    >;
+    extern int_rule int_;
+
+
+    typedef detail::rule<> label_rule;
+    label_rule& label(const char* name);
+
+
+    typedef rule<
+        void (std::set<std::string>&)
     > tags_rule;
     tags_rule& tags_parser();
 
 
-    typedef qi::rule<
-        token_iterator,
-        std::vector<boost::shared_ptr<Effect::EffectsGroup> > (),
-        skipper_type
+    typedef rule<
+        std::vector<std::shared_ptr<Effect::EffectsGroup>> ()
     > effects_group_rule;
     effects_group_rule& effects_group_parser();
 
 
-    typedef qi::rule<
-        token_iterator,
+    typedef rule<
         GG::Clr (),
-        qi::locals<
+        boost::spirit::qi::locals<
             unsigned int,
             unsigned int,
             unsigned int
-        >,
-        skipper_type
+        >
     > color_parser_rule;
     color_parser_rule& color_parser();
 
 
-    typedef qi::rule<
-        token_iterator,
+    typedef rule<
         ItemSpec (),
-        qi::locals<UnlockableItemType>,
-        skipper_type
+        boost::spirit::qi::locals<UnlockableItemType>
     > item_spec_parser_rule;
     item_spec_parser_rule& item_spec_parser();
 
@@ -87,7 +104,7 @@ namespace parse { namespace detail {
 
         std::ptrdiff_t distance = std::distance(first, parse::detail::s_end);
 
-        return success && (!distance || distance == 1 && *first == '\n');
+        return success && (!distance || (distance == 1 && *first == '\n'));
     }
 } }
 
