@@ -1,11 +1,14 @@
-#include "Label.h"
 #include "Parse.h"
+
 #include "ParseImpl.h"
 
 #include "../universe/ShipDesign.h"
 
 #include <boost/spirit/include/phoenix.hpp>
 
+
+FO_COMMON_API extern const int ALL_EMPIRES;
+
 #define DEBUG_PARSERS 0
 
 #if DEBUG_PARSERS
@@ -18,13 +21,7 @@ namespace std {
 
 namespace {
     struct insert_ {
-#if BOOST_VERSION < 105600
-        template <typename Arg1, typename Arg2> // Phoenix v2
-        struct result
-        { typedef void type; };
-#else
         typedef void result_type;
-#endif
 
         void operator()(std::map<std::string, ShipDesign*>& designs, ShipDesign* design) const {
             if (!designs.insert(std::make_pair(design->Name(false), design)).second) {
@@ -37,7 +34,11 @@ namespace {
 
     struct rules {
         rules() {
-            const parse::lexer& tok = parse::lexer::instance();
+            namespace phoenix = boost::phoenix;
+            namespace qi = boost::spirit::qi;
+
+            using phoenix::new_;
+            using phoenix::push_back;
 
             qi::_1_type _1;
             qi::_2_type _2;
@@ -54,31 +55,31 @@ namespace {
             qi::_r3_type _r3;
             qi::_r4_type _r4;
             qi::eps_type eps;
-            using phoenix::new_;
-            using phoenix::push_back;
+
+            const parse::lexer& tok = parse::lexer::instance();
 
             design_prefix
                 =    tok.ShipDesign_
-                >    parse::label(Name_token)        > tok.string [ _r1 = _1 ]
-                >    parse::label(Description_token) > tok.string [ _r2 = _1 ]
+                >    parse::detail::label(Name_token)        > tok.string [ _r1 = _1 ]
+                >    parse::detail::label(Description_token) > tok.string [ _r2 = _1 ]
                 > (
                      tok.NoStringtableLookup_ [ _r4 = false ]
                     | eps [ _r4 = true ]
                   )
-                >    parse::label(Hull_token)        > tok.string [ _r3 = _1 ]
+                >    parse::detail::label(Hull_token)        > tok.string [ _r3 = _1 ]
                 ;
 
             design
                 =    design_prefix(_a, _b, _c, _f)
-                >    parse::label(Parts_token)
+                >    parse::detail::label(Parts_token)
                 >    (
-                            '[' > +tok.string [ push_back(_d, _1) ] > ']'
-                        |   tok.string [ push_back(_d, _1) ]
+                            ('[' > +tok.string [ push_back(_d, _1) ] > ']')
+                        |    tok.string [ push_back(_d, _1) ]
                      )
                 >   -(
-                        parse::label(Icon_token)     > tok.string [ _e = _1 ]
+                        parse::detail::label(Icon_token)     > tok.string [ _e = _1 ]
                      )
-                >    parse::label(Model_token)       > tok.string
+                >    parse::detail::label(Model_token)       > tok.string
                 [ insert(_r1, new_<ShipDesign>(_a, _b, 0, ALL_EMPIRES, _c, _d, _e, _1, _f)) ]
                 ;
 
@@ -97,30 +98,24 @@ namespace {
             qi::on_error<qi::fail>(start, parse::report_error(_1, _2, _3, _4));
         }
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            void (std::string&, std::string&, std::string&, bool&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            void (std::string&, std::string&, std::string&, bool&)
         > design_prefix_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             void (std::map<std::string, ShipDesign*>&),
-            qi::locals<
+            boost::spirit::qi::locals<
                 std::string,
                 std::string,
                 std::string,
                 std::vector<std::string>,
                 std::string,
                 bool
-            >,
-            parse::skipper_type
+            >
         > design_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            void (std::map<std::string, ShipDesign*>&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            void (std::map<std::string, ShipDesign*>&)
         > start_rule;
 
         design_prefix_rule design_prefix;
@@ -131,16 +126,13 @@ namespace {
 
 namespace parse {
     bool ship_designs(const boost::filesystem::path& path, std::map<std::string, ShipDesign*>& designs)
-    { return detail::parse_file<rules, std::map<std::string, ShipDesign*> >(path, designs); }
+    { return detail::parse_file<rules, std::map<std::string, ShipDesign*>>(path, designs); }
 
     bool ship_designs(std::map<std::string, ShipDesign*>& designs) {
         bool result = true;
 
-        std::vector<boost::filesystem::path> file_list = ListScripts("scripting/ship_designs");
-
-        for(std::vector<boost::filesystem::path>::iterator file_it = file_list.begin(); file_it != file_list.end(); ++file_it)
-        {
-            result &= detail::parse_file<rules, std::map<std::string, ShipDesign*> >(*file_it, designs);
+        for(const boost::filesystem::path& file : ListScripts("scripting/ship_designs")) {
+            result &= detail::parse_file<rules, std::map<std::string, ShipDesign*>>(file, designs);
         }
 
         return result;
@@ -149,12 +141,8 @@ namespace parse {
     bool monster_designs(std::map<std::string, ShipDesign*>& designs) {
         bool result = true;
 
-        std::vector<boost::filesystem::path> file_list = ListScripts("scripting/monster_designs");
-
-        for (std::vector<boost::filesystem::path>::iterator file_it = file_list.begin();
-             file_it != file_list.end(); ++file_it)
-        {
-            result &= detail::parse_file<rules, std::map<std::string, ShipDesign*> >(*file_it, designs);
+        for (const boost::filesystem::path& file : ListScripts("scripting/monster_designs")) {
+            result &= detail::parse_file<rules, std::map<std::string, ShipDesign*>>(file, designs);
         }
 
         return result;
