@@ -1,12 +1,11 @@
 #define PHOENIX_LIMIT 11
 #define BOOST_RESULT_OF_NUM_ARGS PHOENIX_LIMIT
 
-#include "ConditionParserImpl.h"
-#include "Double.h"
-#include "EnumParser.h"
-#include "Label.h"
 #include "Parse.h"
+
 #include "ParseImpl.h"
+#include "EnumParser.h"
+#include "ConditionParserImpl.h"
 #include "ValueRefParser.h"
 #include "CommonParams.h"
 
@@ -15,12 +14,13 @@
 
 #include <boost/spirit/include/phoenix.hpp>
 
+
 #define DEBUG_PARSERS 0
 
 #if DEBUG_PARSERS
 namespace std {
     inline ostream& operator<<(ostream& os, const std::vector<HullType::Slot>&) { return os; }
-    inline ostream& operator<<(ostream& os, const std::vector<boost::shared_ptr<Effect::EffectsGroup> >&) { return os; }
+    inline ostream& operator<<(ostream& os, const std::vector<std::shared_ptr<Effect::EffectsGroup>>&) { return os; }
     inline ostream& operator<<(ostream& os, const std::map<std::string, HullType*>&) { return os; }
     inline ostream& operator<<(ostream& os, const std::pair<const std::string, HullType*>&) { return os; }
     inline ostream& operator<<(ostream& os, const HullType::Slot&) { return os; }
@@ -30,13 +30,7 @@ namespace std {
 
 namespace {
     struct insert_hull_ {
-#if BOOST_VERSION < 105600
-        template <typename Arg1, typename Arg2> // Phoenix v2
-        struct result
-        { typedef void type; };
-#else
         typedef void result_type;
-#endif
 
         void operator()(std::map<std::string, HullType*>& hulls, HullType* hull) const {
             if (!hulls.insert(std::make_pair(hull->Name(), hull)).second) {
@@ -49,7 +43,13 @@ namespace {
 
     struct rules {
         rules() {
-            const parse::lexer& tok = parse::lexer::instance();
+            namespace phoenix = boost::phoenix;
+            namespace qi = boost::spirit::qi;
+
+            using phoenix::construct;
+            using phoenix::new_;
+            using phoenix::push_back;
+            using phoenix::insert;
 
             qi::_1_type _1;
             qi::_2_type _2;
@@ -65,33 +65,31 @@ namespace {
             qi::_val_type _val;
             qi::eps_type eps;
             qi::lit_type lit;
-            using phoenix::construct;
-            using phoenix::new_;
-            using phoenix::push_back;
-            using phoenix::insert;
+
+            const parse::lexer& tok = parse::lexer::instance();
 
             hull_stats
-                =   parse::label(Speed_token)       >   parse::double_ [ _a = _1 ]
-                >   parse::label(Fuel_token)        >   parse::double_ [ _c = _1 ]
-                >   parse::label(Stealth_token)     >   parse::double_ [ _d = _1 ]
-                >   parse::label(Structure_token)   >   parse::double_
+                =   parse::detail::label(Speed_token)       >   parse::detail::double_ [ _a = _1 ]
+                >   parse::detail::label(Fuel_token)        >   parse::detail::double_ [ _c = _1 ]
+                >   parse::detail::label(Stealth_token)     >   parse::detail::double_ [ _d = _1 ]
+                >   parse::detail::label(Structure_token)   >   parse::detail::double_
                     [ _val = construct<HullTypeStats>(_c, _a, _d, _1) ]
                 ;
 
             slot
                 =   tok.Slot_
-                >   parse::label(Type_token) > parse::enum_parser<ShipSlotType>() [ _a = _1 ]
-                >   parse::label(Position_token)
-                >   '(' > parse::double_ [ _b = _1 ] > ',' > parse::double_ [ _c = _1 ] > lit(')')
+                >   parse::detail::label(Type_token) > parse::ship_slot_type_enum() [ _a = _1 ]
+                >   parse::detail::label(Position_token)
+                >   '(' > parse::detail::double_ [ _b = _1 ] > ',' > parse::detail::double_ [ _c = _1 ] > lit(')')
                     [ _val = construct<HullType::Slot>(_a, _b, _c) ]
                 ;
 
             slots
                 =  -(
-                        parse::label(Slots_token)
+                        parse::detail::label(Slots_token)
                     >   (
-                            '[' > +slot [ push_back(_r1, _1) ] > ']'
-                            |   slot [ push_back(_r1, _1) ]
+                                ('[' > +slot [ push_back(_r1, _1) ] > ']')
+                            |    slot [ push_back(_r1, _1) ]
                         )
                      )
                 ;
@@ -102,8 +100,8 @@ namespace {
                 >   hull_stats                                  [ _c = _1 ]
                 >  -slots(_e)
                 >   parse::detail::common_params_parser()       [ _d = _1 ]
-                >   parse::label(Icon_token)    > tok.string    [ _f = _1 ]
-                >   parse::label(Graphic_token) > tok.string
+                >   parse::detail::label(Icon_token)    > tok.string    [ _f = _1 ]
+                >   parse::detail::label(Graphic_token) > tok.string
                     [ insert_hull(_r1, new_<HullType>(_c, _d, _a, _e, _f, _1)) ]
                 ;
 
@@ -127,53 +125,43 @@ namespace {
             qi::on_error<qi::fail>(start, parse::report_error(_1, _2, _3, _4));
         }
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             HullTypeStats (),
-            qi::locals<
+            boost::spirit::qi::locals<
                 double,
                 double,
                 double,
                 double
-            >,
-            parse::skipper_type
+            >
         > hull_stats_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             HullType::Slot (),
-            qi::locals<
+            boost::spirit::qi::locals<
                 ShipSlotType,
                 double,
                 double
-            >,
-            parse::skipper_type
+            >
         > slot_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            void (std::vector<HullType::Slot>&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            void (std::vector<HullType::Slot>&)
         > slots_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             void (std::map<std::string, HullType*>&),
-            qi::locals<
+            boost::spirit::qi::locals<
                 MoreCommonParams,
                 std::string,    // dummy
                 HullTypeStats,
                 CommonParams,
                 std::vector<HullType::Slot>,
                 std::string
-            >,
-            parse::skipper_type
+            >
         > hull_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            void (std::map<std::string, HullType*>&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            void (std::map<std::string, HullType*>&)
         > start_rule;
 
         hull_stats_rule                             hull_stats;
@@ -188,12 +176,8 @@ namespace parse {
     bool ship_hulls(std::map<std::string, HullType*>& hulls) {
         bool result = true;
 
-        std::vector<boost::filesystem::path> file_list = ListScripts("scripting/ship_hulls");
-
-        for (std::vector<boost::filesystem::path>::iterator file_it = file_list.begin();
-             file_it != file_list.end(); ++file_it)
-        {
-            result &= detail::parse_file<rules, std::map<std::string, HullType*> >(*file_it, hulls);
+        for (const boost::filesystem::path& file : ListScripts("scripting/ship_hulls")) {
+            result &= detail::parse_file<rules, std::map<std::string, HullType*>>(file, hulls);
         }
 
         return result;
