@@ -1,25 +1,26 @@
 #define PHOENIX_LIMIT 11
 #define BOOST_RESULT_OF_NUM_ARGS PHOENIX_LIMIT
 
-#include "ConditionParserImpl.h"
-#include "EnumParser.h"
-#include "Double.h"
-#include "Label.h"
 #include "Parse.h"
+
 #include "ParseImpl.h"
+#include "EnumParser.h"
 #include "ValueRefParser.h"
+#include "ConditionParserImpl.h"
 #include "CommonParams.h"
+
 #include "../universe/ShipDesign.h"
 #include "../universe/Condition.h"
 
 #include <boost/spirit/include/phoenix.hpp>
 
+
 #define DEBUG_PARSERS 0
 
 #if DEBUG_PARSERS
 namespace std {
     inline ostream& operator<<(ostream& os, const std::vector<ShipSlotType>&) { return os; }
-    inline ostream& operator<<(ostream& os, const std::vector<boost::shared_ptr<Effect::EffectsGroup> >&) { return os; }
+    inline ostream& operator<<(ostream& os, const std::vector<std::shared_ptr<Effect::EffectsGroup>>&) { return os; }
     inline ostream& operator<<(ostream& os, const std::map<std::string, PartType*>&) { return os; }
     inline ostream& operator<<(ostream& os, const std::pair<const std::string, PartType*>&) { return os; }
 }
@@ -27,13 +28,7 @@ namespace std {
 
 namespace {
     struct insert_part_type_ {
-#if BOOST_VERSION < 105600
-        template <typename Arg1, typename Arg2> // Phoenix v2
-        struct result
-        { typedef void type; };
-#else
         typedef void result_type;
-#endif
 
         void operator()(std::map<std::string, PartType*>& part_types, PartType* part_type) const {
             if (!part_types.insert(std::make_pair(part_type->Name(), part_type)).second) {
@@ -47,7 +42,12 @@ namespace {
 
     struct rules {
         rules() {
-            const parse::lexer& tok = parse::lexer::instance();
+            namespace phoenix = boost::phoenix;
+            namespace qi = boost::spirit::qi;
+
+            using phoenix::construct;
+            using phoenix::new_;
+            using phoenix::push_back;
 
             qi::_1_type _1;
             qi::_2_type _2;
@@ -63,16 +63,15 @@ namespace {
             qi::_h_type _h;
             qi::_r1_type _r1;
             qi::eps_type eps;
-            using phoenix::construct;
-            using phoenix::new_;
-            using phoenix::push_back;
+
+            const parse::lexer& tok = parse::lexer::instance();
 
             slots
                 =  -(
-                        parse::label(MountableSlotTypes_token)
+                        parse::detail::label(MountableSlotTypes_token)
                     >   (
-                            '[' > +parse::enum_parser<ShipSlotType>() [ push_back(_r1, _1) ] > ']'
-                        |   parse::enum_parser<ShipSlotType>() [ push_back(_r1, _1) ]
+                            ('[' > +parse::ship_slot_type_enum() [ push_back(_r1, _1) ] > ']')
+                        |    parse::ship_slot_type_enum() [ push_back(_r1, _1) ]
                         )
                      )
                 ;
@@ -80,13 +79,13 @@ namespace {
             part_type
                 = ( tok.Part_
                 >   parse::detail::more_common_params_parser()       [ _a = _1 ]
-                >   parse::label(Class_token)       > parse::enum_parser<ShipPartClass>() [ _c = _1 ]
-                > (  (parse::label(Capacity_token)  > parse::double_ [ _d = _1 ])
-                   | (parse::label(Damage_token)    > parse::double_ [ _d = _1 ])
+                >   parse::detail::label(Class_token)       > parse::ship_part_class_enum() [ _c = _1 ]
+                > (  (parse::detail::label(Capacity_token)  > parse::detail::double_ [ _d = _1 ])
+                   | (parse::detail::label(Damage_token)    > parse::detail::double_ [ _d = _1 ])
                    |  eps [ _d = 0.0 ]
                   )
-                > (  (parse::label(Damage_token)    > parse::double_ [ _h = _1 ])   // damage is secondary for fighters
-                   | (parse::label(Shots_token)     > parse::double_ [ _h = _1 ])   // shots is secondary for direct fire weapons
+                > (  (parse::detail::label(Damage_token)    > parse::detail::double_ [ _h = _1 ])   // damage is secondary for fighters
+                   | (parse::detail::label(Shots_token)     > parse::detail::double_ [ _h = _1 ])   // shots is secondary for direct fire weapons
                    |  eps [ _h = 1.0 ]
                   )
                 > (   tok.NoDefaultCapacityEffect_ [ _g = false ]
@@ -94,7 +93,7 @@ namespace {
                   )
                 >   slots(_f)
                 >   parse::detail::common_params_parser()           [ _e = _1 ]
-                >   parse::label(Icon_token)        > tok.string    [ _b = _1 ]
+                >   parse::detail::label(Icon_token)        > tok.string    [ _b = _1 ]
                   ) [ insert_part_type(_r1, new_<PartType>(_c, _d, _h, _e, _a, _f, _b, _g)) ]
                 ;
 
@@ -113,16 +112,13 @@ namespace {
             qi::on_error<qi::fail>(start, parse::report_error(_1, _2, _3, _4));
         }
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            void (std::vector<ShipSlotType>&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            void (std::vector<ShipSlotType>&)
         > slots_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             void (std::map<std::string, PartType*>&),
-            qi::locals<
+            boost::spirit::qi::locals<
                 MoreCommonParams,
                 std::string,
                 ShipPartClass,
@@ -131,14 +127,11 @@ namespace {
                 std::vector<ShipSlotType>,
                 bool,
                 double
-            >,
-            parse::skipper_type
+            >
         > part_type_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            void (std::map<std::string, PartType*>&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            void (std::map<std::string, PartType*>&)
         > start_rule;
 
         slots_rule                                  slots;
@@ -152,12 +145,8 @@ namespace parse {
     bool ship_parts(std::map<std::string, PartType*>& parts) {
         bool result = true;
 
-        std::vector<boost::filesystem::path> file_list = ListScripts("scripting/ship_parts");
-
-        for(std::vector<boost::filesystem::path>::iterator file_it = file_list.begin();
-            file_it != file_list.end(); ++file_it)
-        {
-            result &= detail::parse_file<rules, std::map<std::string, PartType*> >(*file_it, parts);
+        for(const boost::filesystem::path& file : ListScripts("scripting/ship_parts")) {
+            result &= detail::parse_file<rules, std::map<std::string, PartType*>>(file, parts);
         }
 
         return result;
