@@ -1,19 +1,19 @@
-#include "ConditionParserImpl.h"
-#include "Double.h"
-#include "Int.h"
-#include "Label.h"
-#include "ValueRefParser.h"
 #include "Parse.h"
+
 #include "ParseImpl.h"
+#include "ConditionParserImpl.h"
+#include "ValueRefParser.h"
+
 #include "../universe/Special.h"
 
 #include <boost/spirit/include/phoenix.hpp>
 
+
 #define DEBUG_PARSERS 0
 
 #if DEBUG_PARSERS
 namespace std {
-    inline ostream& operator<<(ostream& os, const std::vector<boost::shared_ptr<Effect::EffectsGroup> >&) { return os; }
+    inline ostream& operator<<(ostream& os, const std::vector<std::shared_ptr<Effect::EffectsGroup>>&) { return os; }
     inline ostream& operator<<(ostream& os, const std::map<std::string, Special*>&) { return os; }
     inline ostream& operator<<(ostream& os, const std::pair<const std::string, Special*>&) { return os; }
 }
@@ -21,13 +21,7 @@ namespace std {
 
 namespace {
     struct insert_ {
-#if BOOST_VERSION < 105600
-        template <typename Arg1, typename Arg2> // Phoenix v2
-        struct result
-        { typedef void type; };
-#else
         typedef void result_type;
-#endif
 
         void operator()(std::map<std::string, Special*>& specials, Special* special) const {
             if (!specials.insert(std::make_pair(special->Name(), special)).second) {
@@ -40,8 +34,10 @@ namespace {
 
     struct rules {
         rules() {
-            const parse::lexer& tok =                                               parse::lexer::instance();
-            const parse::value_ref_parser_rule<double>::type& double_value_ref =    parse::value_ref_parser<double>();
+            namespace phoenix = boost::phoenix;
+            namespace qi = boost::spirit::qi;
+
+            using phoenix::new_;
 
             qi::_1_type _1;
             qi::_2_type _2;
@@ -58,31 +54,32 @@ namespace {
             qi::_r1_type _r1;
             qi::_r2_type _r2;
             qi::eps_type eps;
-            using phoenix::new_;
+
+            const parse::lexer& tok = parse::lexer::instance();
 
             special_prefix
                 =    tok.Special_
-                >    parse::label(Name_token)               > tok.string [ _r1 = _1 ]
-                >    parse::label(Description_token)        > tok.string [ _r2 = _1 ]
+                >    parse::detail::label(Name_token)               > tok.string [ _r1 = _1 ]
+                >    parse::detail::label(Description_token)        > tok.string [ _r2 = _1 ]
                 ;
 
             spawn
-                =    (      parse::label(SpawnRate_token)   > parse::double_ [ _r1 = _1 ]
-                        |   eps [ _r1 = 1.0 ]
+                =    (      (parse::detail::label(SpawnRate_token)   > parse::detail::double_ [ _r1 = _1 ])
+                        |    eps [ _r1 = 1.0 ]
                      )
-                >    (      parse::label(SpawnLimit_token)  > parse::int_ [ _r2 = _1 ]
-                        |   eps [ _r2 = 9999 ]
+                >    (      (parse::detail::label(SpawnLimit_token)  > parse::detail::int_ [ _r2 = _1 ])
+                        |    eps [ _r2 = 9999 ]
                      )
                 ;
 
             special
                 =    special_prefix(_a, _b)
-                >  -(parse::label(Stealth_token)            > double_value_ref [ _g = _1 ])
+                >  -(parse::detail::label(Stealth_token)            > parse::double_value_ref() [ _g = _1 ])
                 >    spawn(_c, _d)
-                >  -(parse::label(Capacity_token)           > double_value_ref [ _h = _1 ])
-                >  -(parse::label(Location_token)           > parse::detail::condition_parser [ _e = _1 ])
-                >  -(parse::label(EffectsGroups_token)      > parse::detail::effects_group_parser() [ _f = _1 ])
-                >    parse::label(Graphic_token)            > tok.string
+                >  -(parse::detail::label(Capacity_token)           > parse::double_value_ref() [ _h = _1 ])
+                >  -(parse::detail::label(Location_token)           > parse::detail::condition_parser [ _e = _1 ])
+                >  -(parse::detail::label(EffectsGroups_token)      > parse::detail::effects_group_parser() [ _f = _1 ])
+                >    parse::detail::label(Graphic_token)            > tok.string
                 [ insert(_r1, new_<Special>(_a, _b, _g, _f, _c, _d, _h, _e, _1)) ]
                 ;
 
@@ -103,38 +100,30 @@ namespace {
             qi::on_error<qi::fail>(start, parse::report_error(_1, _2, _3, _4));
         }
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            void (std::string&, std::string&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            void (std::string&, std::string&)
         > special_prefix_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            void (double&, int&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            void (double&, int&)
         > spawn_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             void (std::map<std::string, Special*>&),
-            qi::locals<
+            boost::spirit::qi::locals<
                 std::string,
                 std::string,
                 double,
                 int,
                 Condition::ConditionBase*,
-                std::vector<boost::shared_ptr<Effect::EffectsGroup> >,
+                std::vector<std::shared_ptr<Effect::EffectsGroup>>,
                 ValueRef::ValueRefBase<double>*,
                 ValueRef::ValueRefBase<double>*
-            >,
-            parse::skipper_type
+            >
         > special_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            void (std::map<std::string, Special*>&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            void (std::map<std::string, Special*>&)
         > start_rule;
 
 
@@ -149,12 +138,8 @@ namespace parse {
     bool specials(std::map<std::string, Special*>& specials_) {
         bool result = true;
 
-        std::vector<boost::filesystem::path> file_list = ListScripts("scripting/specials");
-
-        for (std::vector<boost::filesystem::path>::iterator file_it = file_list.begin();
-             file_it != file_list.end(); ++file_it)
-        {
-            result &= detail::parse_file<rules, std::map<std::string, Special*> >(*file_it, specials_);
+        for (const boost::filesystem::path& file : ListScripts("scripting/specials")) {
+            result &= detail::parse_file<rules, std::map<std::string, Special*>>(file, specials_);
         }
 
         return result;
