@@ -1,19 +1,22 @@
-#include "ConditionParserImpl.h"
-#include "EnumParser.h"
-#include "Label.h"
 #include "Parse.h"
+
 #include "ParseImpl.h"
+#include "EnumParser.h"
+#include "ValueRefParserImpl.h"
+#include "ConditionParserImpl.h"
+
 #include "../universe/Species.h"
 
 #include <boost/spirit/include/phoenix.hpp>
 
+
 #define DEBUG_PARSERS 0
 
 #if DEBUG_PARSERS
 namespace std {
     inline ostream& operator<<(ostream& os, const FocusType&) { return os; }
     inline ostream& operator<<(ostream& os, const std::vector<FocusType>&) { return os; }
-    inline ostream& operator<<(ostream& os, const std::vector<boost::shared_ptr<Effect::EffectsGroup> >&) { return os; }
+    inline ostream& operator<<(ostream& os, const std::vector<std::shared_ptr<Effect::EffectsGroup>>&) { return os; }
     inline ostream& operator<<(ostream& os, const std::pair<PlanetType, PlanetEnvironment>&) { return os; }
     inline ostream& operator<<(ostream& os, const std::pair<const PlanetType, PlanetEnvironment>&) { return os; }
     inline ostream& operator<<(ostream& os, const std::map<PlanetType, PlanetEnvironment>&) { return os; }
@@ -24,13 +27,7 @@ namespace std {
 
 namespace {
     struct insert_species_ {
-#if BOOST_VERSION < 105600
-        template <typename Arg1, typename Arg2> // Phoenix v2
-        struct result
-        { typedef void type; };
-#else
         typedef void result_type;
-#endif
 
         void operator()(std::map<std::string, Species*>& species, Species* specie) const {
             if (!species.insert(std::make_pair(specie->Name(), specie)).second) {
@@ -43,7 +40,13 @@ namespace {
 
     struct rules {
         rules() {
-            const parse::lexer& tok = parse::lexer::instance();
+            namespace phoenix = boost::phoenix;
+            namespace qi = boost::spirit::qi;
+
+            using phoenix::construct;
+            using phoenix::insert;
+            using phoenix::new_;
+            using phoenix::push_back;
 
             qi::_1_type _1;
             qi::_2_type _2;
@@ -59,45 +62,43 @@ namespace {
             qi::_r1_type _r1;
             qi::_val_type _val;
             qi::eps_type eps;
-            using phoenix::construct;
-            using phoenix::insert;
-            using phoenix::new_;
-            using phoenix::push_back;
+
+            const parse::lexer& tok = parse::lexer::instance();
 
             focus_type
                 =    tok.Focus_
-                >    parse::label(Name_token)        > tok.string [ _a = _1 ]
-                >    parse::label(Description_token) > tok.string [ _b = _1 ]
-                >    parse::label(Location_token)    > parse::detail::condition_parser [ _c = _1 ]
-                >    parse::label(Graphic_token)     > tok.string
+                >    parse::detail::label(Name_token)        > tok.string [ _a = _1 ]
+                >    parse::detail::label(Description_token) > tok.string [ _b = _1 ]
+                >    parse::detail::label(Location_token)    > parse::detail::condition_parser [ _c = _1 ]
+                >    parse::detail::label(Graphic_token)     > tok.string
                      [ _val = construct<FocusType>(_a, _b, _c, _1) ]
                 ;
 
             foci
-                =    parse::label(Foci_token)
+                =    parse::detail::label(Foci_token)
                 >    (
-                            '[' > +focus_type [ push_back(_r1, _1) ] > ']'
-                        |   focus_type [ push_back(_r1, _1) ]
+                            ('[' > +focus_type [ push_back(_r1, _1) ] > ']')
+                        |    focus_type [ push_back(_r1, _1) ]
                      )
                 ;
 
             effects
-                =    parse::label(EffectsGroups_token) > parse::detail::effects_group_parser() [ _r1 = _1 ]
+                =    parse::detail::label(EffectsGroups_token) > parse::detail::effects_group_parser() [ _r1 = _1 ]
                 ;
 
             environment_map_element
-                =    parse::label(Type_token)        > parse::enum_parser<PlanetType>() [ _a = _1 ]
-                >    parse::label(Environment_token) >  parse::enum_parser<PlanetEnvironment>()
-                     [ _val = construct<std::pair<PlanetType, PlanetEnvironment> >(_a, _1) ]
+                =    parse::detail::label(Type_token)        > parse::detail::planet_type_rules().enum_expr [ _a = _1 ]
+                >    parse::detail::label(Environment_token) > parse::detail::planet_environment_rules().enum_expr
+                     [ _val = construct<std::pair<PlanetType, PlanetEnvironment>>(_a, _1) ]
                 ;
 
             environment_map
-                =    '[' > +environment_map_element [ insert(_val, _1) ] > ']'
-                |    environment_map_element [ insert(_val, _1) ]
+                =    ('[' > +environment_map_element [ insert(_val, _1) ] > ']')
+                |     environment_map_element [ insert(_val, _1) ]
                 ;
 
             environments
-                =    parse::label(Environments_token) > environment_map [ _r1 = _1 ]
+                =    parse::detail::label(Environments_token) > environment_map [ _r1 = _1 ]
                 ;
 
             species_params
@@ -109,9 +110,9 @@ namespace {
                 ;
 
             species_strings
-                =    parse::label(Name_token)                   > tok.string [ _a = _1 ]
-                >    parse::label(Description_token)            > tok.string [ _b = _1 ]
-                >    parse::label(Gameplay_Description_token)   > tok.string [ _c = _1 ]
+                =    parse::detail::label(Name_token)                   > tok.string [ _a = _1 ]
+                >    parse::detail::label(Description_token)            > tok.string [ _b = _1 ]
+                >    parse::detail::label(Gameplay_Description_token)   > tok.string [ _c = _1 ]
                     [ _val = construct<SpeciesStrings>(_a, _b, _c) ]
                 ;
 
@@ -121,10 +122,10 @@ namespace {
                 >    species_params [ _b = _1]
                 >    parse::detail::tags_parser()(_c)
                 >   -foci(_d)
-                >   -(parse::label(PreferredFocus_token)        >> tok.string [ _g = _1 ])
+                >   -(parse::detail::label(PreferredFocus_token)        >> tok.string [ _g = _1 ])
                 >   -effects(_e)
                 >   -environments(_f)
-                >    parse::label(Graphic_token) > tok.string
+                >    parse::detail::label(Graphic_token) > tok.string
                      [ insert_species(_r1, new_<Species>(_a, _d, _g, _f, _e, _b, _c, _1)) ]
                 ;
 
@@ -159,90 +160,70 @@ namespace {
             qi::on_error<qi::fail>(start, parse::report_error(_1, _2, _3, _4));
         }
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             FocusType (),
-            qi::locals<
+            boost::spirit::qi::locals<
                 std::string,
                 std::string,
                 Condition::ConditionBase*
-            >,
-            parse::skipper_type
+            >
         > focus_type_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            void (std::vector<FocusType>&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            void (std::vector<FocusType>&)
         > foci_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            void (std::vector<boost::shared_ptr<Effect::EffectsGroup> >&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            void (std::vector<std::shared_ptr<Effect::EffectsGroup>>&)
         > effects_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             std::pair<PlanetType, PlanetEnvironment> (),
-            qi::locals<PlanetType>,
-            parse::skipper_type
+            boost::spirit::qi::locals<PlanetType>
         > environment_map_element_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            std::map<PlanetType, PlanetEnvironment> (),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            std::map<PlanetType, PlanetEnvironment> ()
         > environment_map_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            void (std::map<PlanetType, PlanetEnvironment>&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            void (std::map<PlanetType, PlanetEnvironment>&)
         > environments_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             SpeciesParams (),
-            qi::locals<
+            boost::spirit::qi::locals<
                 bool,
                 bool,
                 bool,
                 bool
-            >,
-            parse::skipper_type
+            >
         > species_params_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             SpeciesStrings (),
-            qi::locals<
+            boost::spirit::qi::locals<
                 std::string,
                 std::string,
                 std::string
-            >,
-            parse::skipper_type
+            >
         > species_strings_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             void (std::map<std::string, Species*>&),
-            qi::locals<
+            boost::spirit::qi::locals<
                 SpeciesStrings,
                 SpeciesParams,
                 std::set<std::string>,  // tags
                 std::vector<FocusType>,
-                std::vector<boost::shared_ptr<Effect::EffectsGroup> >,
+                std::vector<std::shared_ptr<Effect::EffectsGroup>>,
                 std::map<PlanetType, PlanetEnvironment>,
                 std::string             // graphic
-            >,
-            parse::skipper_type
+            >
         > species_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            void (std::map<std::string, Species*>&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            void (std::map<std::string, Species*>&)
         > start_rule;
 
         foci_rule                       foci;
@@ -262,12 +243,8 @@ namespace parse {
     bool species(std::map<std::string, Species*>& species_) {
         bool result = true;
 
-        std::vector<boost::filesystem::path> file_list = ListScripts("scripting/species");
-
-        for (std::vector<boost::filesystem::path>::iterator file_it = file_list.begin();
-             file_it != file_list.end(); ++file_it)
-        {
-            result &= detail::parse_file<rules, std::map<std::string, Species*> >(*file_it, species_);
+        for (const boost::filesystem::path& file : ListScripts("scripting/species")) {
+            result &= detail::parse_file<rules, std::map<std::string, Species*>>(file, species_);
         }
 
         return result;
