@@ -1,37 +1,33 @@
-#include "Label.h"
+#include "Parse.h"
+
+#include "ParseImpl.h"
 #include "EnumParser.h"
 #include "ValueRefParser.h"
-#include "ParseImpl.h"
-#include "Parse.h"
+
 #include "../universe/Species.h"
 #include "../util/Directories.h"
 
 #include <boost/spirit/include/phoenix.hpp>
 
+
 #define DEBUG_PARSERS 0
 
 #if DEBUG_PARSERS
 namespace std {
     inline ostream& operator<<(ostream& os, const std::vector<ItemSpec>&) { return os; }
     inline ostream& operator<<(ostream& os, const std::set<std::string>&) { return os; }
-    inline ostream& operator<<(ostream& os, const std::vector<boost::shared_ptr<Effect::EffectsGroup> >&) { return os; }
+    inline ostream& operator<<(ostream& os, const std::vector<std::shared_ptr<Effect::EffectsGroup>>&) { return os; }
     inline ostream& operator<<(ostream& os, const Tech::TechInfo&) { return os; }
     inline ostream& operator<<(ostream& os, const std::pair<const std::string, TechCategory*>&) { return os; }
 }
 #endif
 
 namespace {
-    std::set<std::string>* g_categories_seen = 0;
-    std::map<std::string, TechCategory*>* g_categories = 0;
+    std::set<std::string>* g_categories_seen = nullptr;
+    std::map<std::string, TechCategory*>* g_categories = nullptr;
 
     struct insert_tech_ {
-#if BOOST_VERSION < 105600
-        template <typename Arg1, typename Arg2> // Phoenix v2
-        struct result
-        { typedef void type; };
-#else
         typedef void result_type;
-#endif
 
         void operator()(TechManager::TechContainer& techs, Tech* tech) const {
             g_categories_seen->insert(tech->Category());
@@ -49,13 +45,7 @@ namespace {
     const boost::phoenix::function<insert_tech_> insert_tech;
 
     struct insert_category_ {
-#if BOOST_VERSION < 105600
-        template <typename Arg1, typename Arg2> // Phoenix v2
-        struct result
-        { typedef void type; };
-#else
         typedef void result_type;
-#endif
 
         void operator()(std::map<std::string, TechCategory*>& categories, TechCategory* category) const {
             if (!categories.insert(std::make_pair(category->name, category)).second) {
@@ -68,10 +58,13 @@ namespace {
 
     struct rules {
         rules() {
-            const parse::lexer& tok = parse::lexer::instance();
+            namespace phoenix = boost::phoenix;
+            namespace qi = boost::spirit::qi;
 
-            const parse::value_ref_parser_rule<double>::type& double_value_ref =    parse::value_ref_parser<double>();
-            const parse::value_ref_parser_rule< int >::type& flexible_int_ref =     parse::value_ref_parser_flexible_int();
+            using phoenix::construct;
+            using phoenix::insert;
+            using phoenix::new_;
+            using phoenix::push_back;
 
             qi::_1_type _1;
             qi::_2_type _2;
@@ -90,22 +83,20 @@ namespace {
             qi::_r3_type _r3;
             qi::_val_type _val;
             qi::eps_type eps;
-            using phoenix::construct;
-            using phoenix::insert;
-            using phoenix::new_;
-            using phoenix::push_back;
+
+            const parse::lexer& tok = parse::lexer::instance();
 
             tech_info_name_desc
-                =   parse::label(Name_token)              > tok.string [ _r1 = _1 ]
-                >   parse::label(Description_token)       > tok.string [ _r2 = _1 ]
-                >   parse::label(Short_Description_token) > tok.string [ _r3 = _1 ] // TODO: Get rid of underscore.
+                =   parse::detail::label(Name_token)              > tok.string [ _r1 = _1 ]
+                >   parse::detail::label(Description_token)       > tok.string [ _r2 = _1 ]
+                >   parse::detail::label(Short_Description_token) > tok.string [ _r3 = _1 ] // TODO: Get rid of underscore.
                 ;
 
             tech_info
                 =   tech_info_name_desc(_a, _b, _c)
-                >   parse::label(Category_token)      > tok.string      [ _e = _1 ]
-                >   parse::label(ResearchCost_token)  > double_value_ref[ _f = _1 ]
-                >   parse::label(ResearchTurns_token) > flexible_int_ref[ _g = _1 ]
+                >   parse::detail::label(Category_token)      > tok.string      [ _e = _1 ]
+                >   parse::detail::label(ResearchCost_token)  > parse::double_value_ref() [ _f = _1 ]
+                >   parse::detail::label(ResearchTurns_token) > parse::flexible_int_value_ref() [ _g = _1 ]
                 >  (    tok.Unresearchable_ [ _h = false ]
                     |   tok.Researchable_ [ _h = true ]
                     |   eps [ _h = true ]
@@ -115,16 +106,16 @@ namespace {
                 ;
 
             prerequisites
-                =   parse::label(Prerequisites_token)
-                >  (    '[' > +tok.string [ insert(_r1, _1) ] > ']'
-                    |   tok.string [ insert(_r1, _1) ]
+                =   parse::detail::label(Prerequisites_token)
+                >  (    ('[' > +tok.string [ insert(_r1, _1) ] > ']')
+                    |    tok.string [ insert(_r1, _1) ]
                    )
                 ;
 
             unlocks
-                =   parse::label(Unlock_token)
-                >  (    '[' > +parse::detail::item_spec_parser() [ push_back(_r1, _1) ] > ']'
-                    |   parse::detail::item_spec_parser() [ push_back(_r1, _1) ]
+                =   parse::detail::label(Unlock_token)
+                >  (    ('[' > +parse::detail::item_spec_parser() [ push_back(_r1, _1) ] > ']')
+                    |    parse::detail::item_spec_parser() [ push_back(_r1, _1) ]
                    )
                 ;
 
@@ -133,17 +124,17 @@ namespace {
                 >   tech_info [ _a = _1 ]
                 >  -prerequisites(_b)
                 >  -unlocks(_c)
-                > -(parse::label(EffectsGroups_token) > parse::detail::effects_group_parser() [ _d = _1 ])
-                > -(parse::label(Graphic_token) > tok.string [ _e = _1 ])
+                > -(parse::detail::label(EffectsGroups_token) > parse::detail::effects_group_parser() [ _d = _1 ])
+                > -(parse::detail::label(Graphic_token) > tok.string [ _e = _1 ])
                    )
                 [ insert_tech(_r1, new_<Tech>(_a, _d, _b, _c, _e)) ]
                 ;
 
             category
                 =   tok.Category_
-                >   parse::label(Name_token)    > tok.string [ _a = _1 ]
-                >   parse::label(Graphic_token) > tok.string [ _b = _1 ]
-                >   parse::label(Colour_token)  > parse::detail::color_parser() [ insert_category(_r1, new_<TechCategory>(_a, _b, _1)) ]
+                >   parse::detail::label(Name_token)    > tok.string [ _a = _1 ]
+                >   parse::detail::label(Graphic_token) > tok.string [ _b = _1 ]
+                >   parse::detail::label(Colour_token)  > parse::detail::color_parser() [ insert_category(_r1, new_<TechCategory>(_a, _b, _1)) ]
                 ;
 
             start
@@ -172,16 +163,13 @@ namespace {
             qi::on_error<qi::fail>(start, parse::report_error(_1, _2, _3, _4));
         }
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            Tech::TechInfo (std::string&, std::string&, std::string&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            Tech::TechInfo (std::string&, std::string&, std::string&)
         > tech_info_name_desc_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             Tech::TechInfo (),
-            qi::locals<
+            boost::spirit::qi::locals<
                 std::string,
                 std::string,
                 std::string,
@@ -190,49 +178,38 @@ namespace {
                 ValueRef::ValueRefBase<double>*,
                 ValueRef::ValueRefBase<int>*,
                 bool
-            >,
-            parse::skipper_type
+            >
         > tech_info_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            Tech::TechInfo (std::set<std::string>&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            Tech::TechInfo (std::set<std::string>&)
         > prerequisites_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            Tech::TechInfo (std::vector<ItemSpec>&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            Tech::TechInfo (std::vector<ItemSpec>&)
         > unlocks_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             void (TechManager::TechContainer&),
-            qi::locals<
+            boost::spirit::qi::locals<
                 Tech::TechInfo,
                 std::set<std::string>,
                 std::vector<ItemSpec>,
-                std::vector<boost::shared_ptr<Effect::EffectsGroup> >,
+                std::vector<std::shared_ptr<Effect::EffectsGroup>>,
                 std::string
-            >,
-            parse::skipper_type
+            >
         > tech_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
+        typedef parse::detail::rule<
             void (std::map<std::string, TechCategory*>&),
-            qi::locals<
+            boost::spirit::qi::locals<
                 std::string,
                 std::string
-            >,
-            parse::skipper_type
+            >
         > category_rule;
 
-        typedef boost::spirit::qi::rule<
-            parse::token_iterator,
-            void (TechManager::TechContainer&),
-            parse::skipper_type
+        typedef parse::detail::rule<
+            void (TechManager::TechContainer&)
         > start_rule;
 
         tech_info_name_desc_rule    tech_info_name_desc;
@@ -252,19 +229,13 @@ namespace parse {
     {
         bool result = true;
 
-        std::vector<boost::filesystem::path> file_list = ListScripts("scripting/techs");
-
         g_categories_seen = &categories_seen;
         g_categories = &categories;
 
         result &= detail::parse_file<rules, TechManager::TechContainer>(GetResourceDir() / "scripting/techs/Categories.inf", techs_);
 
-        for (std::vector<boost::filesystem::path>::iterator file_it = file_list.begin();
-             file_it != file_list.end(); ++file_it)
-        {
-            boost::filesystem::path path = *file_it;
-
-            result &= detail::parse_file<rules, TechManager::TechContainer>(path, techs_);
+        for (const boost::filesystem::path& file : ListScripts("scripting/techs")) {
+            result &= detail::parse_file<rules, TechManager::TechContainer>(file, techs_);
         }
 
         return result;
