@@ -102,6 +102,7 @@
     (Else)                                      \
     (Empire)                                    \
     (EmpireMeter)                               \
+    (EmpireMeterValue)                          \
     (EmpireShipsDestroyed)                      \
     (Endpoint)                                  \
     (EnemyOf)                                   \
@@ -241,6 +242,7 @@
     (NextYoungerStarType)                       \
     (NoDefaultCapacityEffect)                   \
     (None)                                      \
+    (NoOp)                                      \
     (NoStar)                                    \
     (NoStringtableLookup)                       \
     (Not)                                       \
@@ -263,19 +265,21 @@
     (OutpostsOwned)                             \
     (OwnedBy)                                   \
     (Owner)                                     \
+    (OwnerHasShipPartAvailable)                 \
     (OwnerHasTech)                              \
     (OwnerTradeStockpile)                       \
     (Parameters)                                \
     (Part)                                      \
     (PartCapacity)                              \
-    (PartDamage)                                \
     (PartClass)                                 \
+    (PartDamage)                                \
     (Partial)                                   \
-    (PartName)
+    (PartName)                                  \
+    (PartSecondaryStat)
 
 #define TOKEN_SEQ_9                             \
     (Parts)                                     \
-    (PartClassInShipDesign)                     \
+    (PartOfClassInShipDesign)                   \
     (PartsInShipDesign)                         \
     (Passive)                                   \
     (Planet)                                    \
@@ -401,6 +405,7 @@
     (ShipHull)                                  \
     (ShipPart)                                  \
     (ShipPartMeter)                             \
+    (ShipPartsOwned)                            \
     (Ships)                                     \
     (Short_Description)                         \
     (ShortestPath)                              \
@@ -425,6 +430,7 @@
     (SpawnRate)                                 \
     (Special)                                   \
     (Species)                                   \
+    (SpeciesID)                                 \
     (SpeciesOpinion)                            \
     (SpeciesPlanetsBombed)                      \
     (SpeciesColoniesOwned)                      \
