@@ -6,6 +6,7 @@
 #include "../universe/UniverseObject.h"
 #include "../universe/Planet.h"
 #include "../universe/Tech.h"
+#include "../universe/Enums.h"
 #include "../util/SitRepEntry.h"
 #include "../util/AppInterface.h"
 #include "../util/Logger.h"
@@ -20,7 +21,10 @@
 #include <boost/python/suite/indexing/vector_indexing_suite.hpp>
 #include <boost/python/tuple.hpp>
 #include <boost/python/to_python_converter.hpp>
-#include <boost/shared_ptr.hpp>
+
+#include <iterator>
+#include <memory>
+
 
 namespace {
     // Research queue tests whether it contains a Tech by name, but Python needs
@@ -63,9 +67,7 @@ namespace {
         static SitRepEntry EMPTY_ENTRY;
         if (index < 0 || index >= empire.NumSitRepEntries())
             return EMPTY_ENTRY;
-        Empire::SitRepItr it = empire.SitRepBegin();
-        std::advance(it, index);
-        return *it;
+        return *std::next(empire.SitRepBegin(), index);
     }
     boost::function<const SitRepEntry&(const Empire&, int)> GetEmpireSitRepFunc =                       GetSitRep;
 
@@ -88,8 +90,8 @@ namespace {
         const std::set<IntPair>& laneset = GetSupplyManager().SupplyObstructedStarlaneTraversals(empire.EmpireID());
         std::vector<IntPair> retval;
         try {
-            for (std::set<std::pair<int, int> >::const_iterator it = laneset.begin(); it != laneset.end(); ++it)
-            {retval.push_back(*it); }
+            for (const std::pair<int, int>& lane : laneset)
+            { retval.push_back(lane); }
         } catch (...) {
         }
         return retval;
@@ -98,15 +100,12 @@ namespace {
 
     std::map<int, int> jumpsToSuppliedSystemP(const Empire& empire) {
         std::map<int, int> retval;
-        const std::set<int>& empire_supplyable_system_ids = GetSupplyManager().FleetSupplyableSystemIDs(empire.EmpireID());
-        const std::map<int, std::set<int> >& empire_starlanes = empire.KnownStarlanes();
+        const std::map<int, std::set<int>>& empire_starlanes = empire.KnownStarlanes();
         std::list<int> propagating_list;
 
-        for (std::set<int>::const_iterator sys_it = empire_supplyable_system_ids.begin();
-             sys_it != empire_supplyable_system_ids.end(); ++sys_it)
-        {
-            retval[*sys_it] = 0;
-            propagating_list.push_back(*sys_it);
+        for (int system_id : GetSupplyManager().FleetSupplyableSystemIDs(empire.EmpireID(), true)) {
+            retval[system_id] = 0;
+            propagating_list.push_back(system_id);
         }
 
         // iteratively propagate supply out from supplied systems, to determine
@@ -118,25 +117,25 @@ namespace {
             int from_sys_dist = retval[from_sys_id];
 
             // get lanes connected to this system
-            std::map<int, std::set<int> >::const_iterator lane_set_it = empire_starlanes.find(from_sys_id);
+            std::map<int, std::set<int>>::const_iterator lane_set_it = empire_starlanes.find(from_sys_id);
             if (lane_set_it == empire_starlanes.end())
                 continue;   // no lanes to propagate from for this supply source
             const std::set<int>& lane_ends = lane_set_it->second;
 
             // propagate to any not-already-counted adjacent system
-            for (std::set<int>::const_iterator lane_it = lane_ends.begin(); lane_it != lane_ends.end(); ++lane_it) {
-                if (retval.find(*lane_it) != retval.end())
+            for (int lane_end_system_id : lane_ends) {
+                if (retval.find(lane_end_system_id) != retval.end())
                     continue;   // system already processed
                 // system not yet processed; add it to list to propagate from, and set its range to one more than this system
-                propagating_list.push_back(*lane_it);
-                retval[*lane_it] = from_sys_dist - 1;   // negative values used to indicate jumps to nearest supply for historical compatibility reasons
+                propagating_list.push_back(lane_end_system_id);
+                retval[lane_end_system_id] = from_sys_dist - 1;   // negative values used to indicate jumps to nearest supply for historical compatibility reasons
             }
         }
 
         //// DEBUG
         //DebugLogger() << "jumpsToSuppliedSystemP results for empire, " << empire.Name() << " (" << empire.EmpireID() << ") :";
-        //for (std::map<int, int>::const_iterator it = retval.begin(); it != retval.end(); ++it) {
-        //    DebugLogger() << "sys " << it->first << "  range: " << it->second;
+        //for (const std::map<int, int>::value_type& system_jumps : retval) {
+        //    DebugLogger() << "sys " << system_jumps.first << "  range: " << system_jumps.second;
         //}
         //// END DEBUG
 
@@ -163,23 +162,17 @@ namespace {
     //.def("objectsWithWastedPP",             make_function(&ProductionQueue::ObjectsWithWastedPP,  return_value_policy<return_by_value>()))
 
     std::map<std::set<int>, float> PlanetsWithAvailablePP_P(const Empire& empire) {
-        const boost::shared_ptr<ResourcePool>& industry_pool = empire.GetResourcePool(RE_INDUSTRY);
+        const std::shared_ptr<ResourcePool>& industry_pool = empire.GetResourcePool(RE_INDUSTRY);
         const ProductionQueue& prodQueue = empire.GetProductionQueue();
         std::map<std::set<int>, float> planetsWithAvailablePP;
-        std::map<std::set<int>, float> objectsWithAvailablePP = prodQueue.AvailablePP(industry_pool);
-        for (std::map<std::set<int>, float>::iterator map_it = objectsWithAvailablePP.begin(); 
-             map_it != objectsWithAvailablePP.end(); ++map_it)
-        {
+        for (const std::map<std::set<int>, float>::value_type& objects_pp : prodQueue.AvailablePP(industry_pool)) {
             std::set<int> planetSet;
-            std::set<int> objSet = map_it->first;
-            for (std::set<int>::iterator obj_it = objSet.begin(); obj_it != objSet.end(); ++obj_it)
-            {
-                TemporaryPtr<UniverseObject> location = GetUniverseObject(*obj_it);
-                if (/* TemporaryPtr<const Planet> planet = */ boost::dynamic_pointer_cast<const Planet>(location))
-                    planetSet.insert(*obj_it);
+            for (int object_id : objects_pp.first) {
+                if (/* std::shared_ptr<const Planet> planet = */ GetPlanet(object_id))
+                    planetSet.insert(object_id);
             }
             if (!planetSet.empty())
-                planetsWithAvailablePP[planetSet] = map_it->second;
+                planetsWithAvailablePP[planetSet] = objects_pp.second;
         }
         return planetsWithAvailablePP;
     }
@@ -189,46 +182,36 @@ namespace {
         const ProductionQueue& prodQueue = empire.GetProductionQueue();
         std::map<std::set<int>, float> planetsWithAllocatedPP;
         std::map<std::set<int>, float> objectsWithAllocatedPP = prodQueue.AllocatedPP();
-        for (std::map<std::set<int>, float>::iterator map_it = objectsWithAllocatedPP.begin(); 
-             map_it != objectsWithAllocatedPP.end(); ++map_it)
-        {
+        for (const std::map<std::set<int>, float>::value_type& objects_pp : objectsWithAllocatedPP) {
             std::set<int> planetSet;
-            std::set<int> objSet = map_it->first;
-            for (std::set<int>::iterator obj_it = objSet.begin(); obj_it != objSet.end(); ++obj_it)
-            {
-                TemporaryPtr<UniverseObject> location = GetUniverseObject(*obj_it);
-                if (/* TemporaryPtr<const Planet> planet = */ boost::dynamic_pointer_cast<const Planet>(location))
-                    planetSet.insert(*obj_it);
+            for (int object_id : objects_pp.first) {
+                if (/* std::shared_ptr<const Planet> planet = */ GetPlanet(object_id))
+                    planetSet.insert(object_id);
             }
             if (!planetSet.empty())
-                planetsWithAllocatedPP[planetSet] = map_it->second;
+                planetsWithAllocatedPP[planetSet] = objects_pp.second;
         }
         return planetsWithAllocatedPP;
     }
     boost::function<std::map<std::set<int>, float>(const Empire& )> PlanetsWithAllocatedPP_Func =   &PlanetsWithAllocatedPP_P;
 
-    std::set<std::set<int> > PlanetsWithWastedPP_P(const Empire& empire) {
-        const boost::shared_ptr<ResourcePool>& industry_pool = empire.GetResourcePool(RE_INDUSTRY);
+    std::set<std::set<int>> PlanetsWithWastedPP_P(const Empire& empire) {
+        const std::shared_ptr<ResourcePool>& industry_pool = empire.GetResourcePool(RE_INDUSTRY);
         const ProductionQueue& prodQueue = empire.GetProductionQueue();
-        std::set<std::set<int> > planetsWithWastedPP;
-        std::set<std::set<int> > objectsWithWastedPP = prodQueue.ObjectsWithWastedPP(industry_pool);
-        for (std::set<std::set<int> >::iterator sets_it = objectsWithWastedPP.begin(); 
-             sets_it != objectsWithWastedPP.end(); ++sets_it)
-             {
+        std::set<std::set<int>> planetsWithWastedPP;
+        std::set<std::set<int>> objectsWithWastedPP = prodQueue.ObjectsWithWastedPP(industry_pool);
+        for (const std::set<int>&  objects : objectsWithWastedPP) {
                  std::set<int> planetSet;
-                 std::set<int> objSet = *sets_it;
-                 for (std::set<int>::iterator obj_it = objSet.begin(); obj_it != objSet.end(); ++obj_it)
-                 {
-                     TemporaryPtr<UniverseObject> location = GetUniverseObject(*obj_it);
-                     if (/* TemporaryPtr<const Planet> planet = */ boost::dynamic_pointer_cast<const Planet>(location))
-                         planetSet.insert(*obj_it);
+                 for (int object_id : objects) {
+                     if (/* std::shared_ptr<const Planet> planet = */ GetPlanet(object_id))
+                         planetSet.insert(object_id);
                  }
                  if (!planetSet.empty())
                      planetsWithWastedPP.insert(planetSet);
              }
              return planetsWithWastedPP;
     }
-    boost::function<std::set<std::set<int> >(const Empire&)>        PlanetsWithWastedPP_Func =      &PlanetsWithWastedPP_P;
+    boost::function<std::set<std::set<int>>(const Empire&)>         PlanetsWithWastedPP_Func =      &PlanetsWithWastedPP_P;
 }
 
 namespace FreeOrionPython {
@@ -267,18 +250,18 @@ namespace FreeOrionPython {
 
         boost::python::to_python_converter<IntPair, myIntIntPairConverter>();
 
-        class_<std::vector<IntPair> >("IntPairVec")
+        class_<std::vector<IntPair>>("IntPairVec")
             .def(boost::python::vector_indexing_suite<std::vector<IntPair>, true>())
         ;
-        class_<std::vector<ItemSpec> >("ItemSpecVec")
+        class_<std::vector<ItemSpec>>("ItemSpecVec")
             .def(boost::python::vector_indexing_suite<std::vector<ItemSpec>, true>())
         ;
         boost::python::to_python_converter<FloatIntPair, FloatIntPairConverter>();
 
-        class_<ResourcePool, boost::shared_ptr<ResourcePool>, boost::noncopyable >("resPool", boost::python::no_init);
+        class_<ResourcePool, std::shared_ptr<ResourcePool>, boost::noncopyable>("resPool", boost::python::no_init);
         //FreeOrionPython::SetWrapper<int>::Wrap("IntSet");
         FreeOrionPython::SetWrapper<IntSet>::Wrap("IntSetSet");
-        class_<std::map<std::set<int>, float> > ("resPoolMap")
+        class_<std::map<std::set<int>, float>> ("resPoolMap")
             .def(boost::python::map_indexing_suite<std::map<std::set<int>, float>, true>())
         ;
 
@@ -320,7 +303,7 @@ namespace FreeOrionPython {
             .add_property("planetsWithWastedPP",    make_function(
                                                         PlanetsWithWastedPP_Func,
                                                         return_value_policy<return_by_value>(),
-                                                        boost::mpl::vector<std::set<std::set<int> >, const Empire& >()
+                                                        boost::mpl::vector<std::set<std::set<int>>, const Empire& >()
                                                     ))
 
             .def("techResearched",                  &Empire::TechResearched)
@@ -463,7 +446,7 @@ namespace FreeOrionPython {
 
         boost::python::object techsFunc = make_function(boost::bind(TechNamesMemberFunc, &(GetTechManager())),
                                                         return_value_policy<boost::python::return_by_value>(),
-                                                        boost::mpl::vector<std::vector<std::string> >());
+                                                        boost::mpl::vector<std::vector<std::string>>());
         boost::python::setattr(techsFunc, "__doc__", boost::python::str("Returns the names of all techs (StringVec)."));
         def("techs", techsFunc);
 
