@@ -1,3 +1,4 @@
+#include "../universe/Pathfinder.h"
 #include "../universe/Universe.h"
 #include "../universe/UniverseObject.h"
 #include "../universe/Fleet.h"
@@ -11,6 +12,7 @@
 #include "../universe/Field.h"
 #include "../universe/Special.h"
 #include "../universe/Species.h"
+#include "../universe/Enums.h"
 #include "../util/Logger.h"
 #include "../util/MultiplayerCommon.h"
 
@@ -19,12 +21,36 @@
 #include <boost/python/suite/indexing/map_indexing_suite.hpp>
 #include <boost/python/suite/indexing/vector_indexing_suite.hpp>
 
+
+#if defined(_MSC_VER)
+#  if (_MSC_VER == 1900)
+namespace boost {
+    template<>
+    const volatile UniverseObject*  get_pointer(const volatile UniverseObject* p) { return p; }
+    template<>
+    const volatile Fleet*           get_pointer(const volatile Fleet* p) { return p; }
+    template<>
+    const volatile Ship*            get_pointer(const volatile Ship* p) { return p; }
+    template<>
+    const volatile Planet*          get_pointer(const volatile Planet* p) { return p; }
+    template<>
+    const volatile System*          get_pointer(const volatile System* p) { return p; }
+    template<>
+    const volatile Field*           get_pointer(const volatile Field* p) { return p; }
+    template<>
+    const volatile Building*        get_pointer(const volatile Building* p) { return p; }
+    template<>
+    const volatile Universe*        get_pointer<const volatile Universe>(const volatile Universe* p) { return p; }
+}
+#  endif
+#endif
+
 namespace {
     void                    DumpObjects(const Universe& universe)
     { DebugLogger() << universe.Objects().Dump(); }
 
     // We're returning the result of operator-> here so that python doesn't
-    // need to deal with our TemporaryPtr class.
+    // need to deal with std::shared_ptr class.
     // Please don't use this trick elsewhere to grab a raw UniverseObject*!
     const UniverseObject*   GetUniverseObjectP(const Universe& universe, int id)
     { return ::GetUniverseObject(id).operator->(); }
@@ -58,9 +84,8 @@ namespace {
 
     std::vector<std::string>SpeciesFoci(const Species& species) {
         std::vector<std::string> retval;
-        const std::vector<FocusType>& foci = species.Foci();
-        for (std::vector<FocusType>::const_iterator f_it = foci.begin(); f_it != foci.end(); ++f_it)
-            retval.push_back(f_it->Name());
+        for (const FocusType& focus : species.Foci())
+            retval.push_back(focus.Name());
         return retval;
     }
 
@@ -76,19 +101,19 @@ namespace {
 
     double                  LinearDistance(const Universe& universe, int system1_id, int system2_id) {
         double retval = 9999999.9;  // arbitrary large value
-        retval = universe.LinearDistance(system1_id, system2_id);
+        retval = universe.GetPathfinder()->LinearDistance(system1_id, system2_id);
         return retval;
     }
     boost::function<double(const Universe&, int, int)> LinearDistanceFunc =                     &LinearDistance;
 
     int                     JumpDistanceBetweenSystems(const Universe& universe, int system1_id, int system2_id) {
-        return universe.JumpDistanceBetweenSystems(system1_id, system2_id);
+        return universe.GetPathfinder()->JumpDistanceBetweenSystems(system1_id, system2_id);
     }
     boost::function<int(const Universe&, int, int)> JumpDistanceFunc =                          &JumpDistanceBetweenSystems;
 
     std::vector<int>        ShortestPath(const Universe& universe, int start_sys, int end_sys, int empire_id) {
         std::vector<int> retval;
-        std::pair<std::list<int>, int> path = universe.ShortestPath(start_sys, end_sys, empire_id);
+        std::pair<std::list<int>, int> path = universe.GetPathfinder()->ShortestPath(start_sys, end_sys, empire_id);
         std::copy(path.first.begin(), path.first.end(), std::back_inserter(retval));
         return retval;
     }
@@ -96,7 +121,7 @@ namespace {
 
     std::vector<int>        LeastJumpsPath(const Universe& universe, int start_sys, int end_sys, int empire_id) {
         std::vector<int> retval;
-        std::pair<std::list<int>, int> path = universe.LeastJumpsPath(start_sys, end_sys, empire_id);
+        std::pair<std::list<int>, int> path = universe.GetPathfinder()->LeastJumpsPath(start_sys, end_sys, empire_id);
         std::copy(path.first.begin(), path.first.end(), std::back_inserter(retval));
         return retval;
     }
@@ -104,28 +129,29 @@ namespace {
 
     bool                    SystemsConnectedP(const Universe& universe, int system1_id, int system2_id, int empire_id=ALL_EMPIRES) {
         //DebugLogger() << "SystemsConnected!(" << system1_id << ", " << system2_id << ")";
-        bool retval = universe.SystemsConnected(system1_id, system2_id, empire_id);
+        bool retval = universe.GetPathfinder()->SystemsConnected(system1_id, system2_id, empire_id);
         //DebugLogger() << "SystemsConnected! retval: " << retval;
         return retval;
     }
     boost::function<bool(const Universe&, int, int, int)> SystemsConnectedFunc =                &SystemsConnectedP;
 
+    bool                    SystemHasVisibleStarlanesP(const Universe& universe, int system_id, int empire_id = ALL_EMPIRES) {
+        return universe.GetPathfinder()->SystemHasVisibleStarlanes(system_id, empire_id);
+    }
+    boost::function<bool(const Universe&, int, int)> SystemHasVisibleStarlanesFunc =            &SystemHasVisibleStarlanesP;
+
     std::vector<int>        ImmediateNeighborsP(const Universe& universe, int system1_id, int empire_id = ALL_EMPIRES) {
-        std::multimap<double, int> lanemap;
         std::vector<int> retval;
-        lanemap = universe.ImmediateNeighbors(system1_id, empire_id);
-        for (std::multimap<double, int>::const_iterator it = lanemap.begin(); it != lanemap.end(); ++it)
-        { retval.push_back(it->second); }
+        for (const std::multimap<double, int>::value_type& entry : universe.GetPathfinder()->ImmediateNeighbors(system1_id, empire_id))
+        { retval.push_back(entry.second); }
         return retval;
     }
     boost::function<std::vector<int> (const Universe&, int, int)> ImmediateNeighborsFunc =      &ImmediateNeighborsP;
 
     std::map<int,double>    SystemNeighborsMapP(const Universe& universe, int system1_id, int empire_id = ALL_EMPIRES) {
-        std::multimap<double, int> lanemap;
         std::map<int,double> retval;
-        lanemap = universe.ImmediateNeighbors(system1_id, empire_id);
-        for (std::multimap<double, int>::const_iterator it = lanemap.begin(); it != lanemap.end(); ++it)
-        { retval[it->second] = it->first; }
+        for (const std::multimap<double, int>::value_type& entry : universe.GetPathfinder()->ImmediateNeighbors(system1_id, empire_id))
+        { retval[entry.second] = entry.first; }
         return retval;
     }
     boost::function<std::map<int, double> (const Universe&, int, int)> SystemNeighborsMapFunc = &SystemNeighborsMapP;
@@ -136,9 +162,8 @@ namespace {
 
     std::vector<std::string> ObjectSpecials(const UniverseObject& object) {
         std::vector<std::string> retval;
-        for (std::map<std::string, std::pair<int, float> >::const_iterator it = object.Specials().begin();
-             it != object.Specials().end(); ++it)
-        { retval.push_back(it->first); }
+        for (const std::map<std::string, std::pair<int, float>>::value_type& special : object.Specials())
+        { retval.push_back(special.first); }
         return retval;
     }
 
@@ -163,11 +188,9 @@ namespace {
     //std::vector<std::string>        (ShipDesign::*PartsSlotType)(ShipSlotType) const =          &ShipDesign::Parts;
 
     std::vector<int>        AttackStatsP(const ShipDesign& ship_design) {
-        const std::vector<std::string>& partslist = ship_design.Parts();
         std::vector<int> results;
-        for (std::vector<std::string>::const_iterator part_it = partslist.begin(); part_it!=partslist.end(); ++part_it)
-        {
-            const PartType* part = GetPartType(*part_it);
+        for (const std::string& part_name : ship_design.Parts()) {
+            const PartType* part = GetPartType(part_name);
             if (part && part->Class() == PC_DIRECT_WEAPON) { // TODO: handle other weapon classes when they are implemented
                 results.push_back(part->Capacity());
             }
@@ -178,9 +201,8 @@ namespace {
 
     std::vector<ShipSlotType> HullSlots(const HullType& hull) {
         std::vector<ShipSlotType> retval;
-        std::vector< HullType::Slot > slots = hull.Slots();
-        for (std::vector<HullType::Slot>::iterator s_it = slots.begin(); s_it != slots.end(); ++s_it)
-            retval.push_back(s_it->type);
+        for (const HullType::Slot& slot : hull.Slots())
+            retval.push_back(slot.type);
         return retval;
     }
     boost::function<std::vector<ShipSlotType> (const HullType&)> HullSlotsFunc =                &HullSlots;
@@ -232,26 +254,26 @@ namespace FreeOrionPython {
         ////////////////////////
         // Container Wrappers //
         ////////////////////////
-        class_<std::map<int, int> >("IntIntMap")
+        class_<std::map<int, int>>("IntIntMap")
             .def(boost::python::map_indexing_suite<std::map<int, int>, true>())
         ;
-        class_<std::map<int, double> >("IntDblMap")
+        class_<std::map<int, double>>("IntDblMap")
             .def(boost::python::map_indexing_suite<std::map<int, double>, true>())
         ;
-        class_<std::map<int, float> >("IntFltMap")
+        class_<std::map<int, float>>("IntFltMap")
             .def(boost::python::map_indexing_suite<std::map<int, float>, true>())
         ;
-        class_<std::map<Visibility,int> >("VisibilityIntMap")
+        class_<std::map<Visibility,int>>("VisibilityIntMap")
             .def(boost::python::map_indexing_suite<std::map<Visibility, int>, true>())
         ;
-        class_<std::vector<ShipSlotType> >("ShipSlotVec")
+        class_<std::vector<ShipSlotType>>("ShipSlotVec")
             .def(boost::python::vector_indexing_suite<std::vector<ShipSlotType>, true>())
         ;
-        class_<std::map<MeterType, Meter> >("MeterTypeMeterMap")
+        class_<std::map<MeterType, Meter>>("MeterTypeMeterMap")
             .def(boost::python::map_indexing_suite<std::map<MeterType, Meter>, true>())
         ;
         // typedef std::map<std::pair<MeterType, std::string>, Meter>          PartMeterMap;
-        class_<std::pair<MeterType, std::string> >("MeterTypeStringPair")
+        class_<std::pair<MeterType, std::string>>("MeterTypeStringPair")
             .add_property("meterType",  &std::pair<MeterType, std::string>::first)
             .add_property("string",     &std::pair<MeterType, std::string>::second)
         ;
@@ -291,7 +313,11 @@ namespace FreeOrionPython {
             .def("destroyedObjectIDs",          make_function(&Universe::EmpireKnownDestroyedObjectIDs,
                                                                                     return_value_policy<return_by_value>()))
 
-            .def("systemHasStarlane",           &Universe::SystemHasVisibleStarlanes)
+            .def("systemHasStarlane",           make_function(
+                                                    SystemHasVisibleStarlanesFunc,
+                                                    return_value_policy<return_by_value>(),
+                                                    boost::mpl::vector<bool, const Universe&, int, int>()
+                                                ))
 
             .def("updateMeterEstimates",        &UpdateMetersWrapper)
 
@@ -395,6 +421,7 @@ namespace FreeOrionPython {
             .add_property("canChangeDirectionEnRoute",  &Fleet::CanChangeDirectionEnRoute)
             .add_property("hasMonsters",                &Fleet::HasMonsters)
             .add_property("hasArmedShips",              &Fleet::HasArmedShips)
+            .add_property("hasFighterShips",            &Fleet::HasFighterShips)
             .add_property("hasColonyShips",             &Fleet::HasColonyShips)
             .add_property("hasOutpostShips",            &Fleet::HasOutpostShips)
             .add_property("hasTroopShips",              &Fleet::HasTroopShips)
@@ -413,6 +440,7 @@ namespace FreeOrionPython {
             .add_property("producedByEmpireID",     &Ship::ProducedByEmpireID)
             .add_property("isMonster",              &Ship::IsMonster)
             .add_property("isArmed",                &Ship::IsArmed)
+            .add_property("hasFighters",            &Ship::HasFighters)
             .add_property("canColonize",            &Ship::CanColonize)
             .add_property("canInvade",              &Ship::HasTroops)
             .add_property("canBombard",             &Ship::CanBombard)
@@ -461,6 +489,7 @@ namespace FreeOrionPython {
             .add_property("canColonize",        make_function(&ShipDesign::CanColonize,     return_value_policy<return_by_value>()))
             .add_property("canInvade",          make_function(&ShipDesign::HasTroops,       return_value_policy<return_by_value>()))
             .add_property("isArmed",            make_function(&ShipDesign::IsArmed,         return_value_policy<return_by_value>()))
+            .add_property("hasFighters",        make_function(&ShipDesign::HasFighters,     return_value_policy<return_by_value>()))
             .add_property("isMonster",          make_function(&ShipDesign::IsMonster,       return_value_policy<return_by_value>()))
             .def("productionCost",              &ShipDesign::ProductionCost)
             .def("productionTime",              &ShipDesign::ProductionTime)
