@@ -11,8 +11,11 @@
 #include "../../universe/Ship.h"
 #include "../../universe/Field.h"
 #include "../../universe/Tech.h"
+#include "../../universe/Pathfinder.h"
 #include "../../universe/Universe.h"
 #include "../../universe/UniverseGenerator.h"
+#include "../../universe/Enums.h"
+#include "../../universe/ValueRef.h"
 
 #include "../../server/ServerApp.h"
 #include "../../util/Directories.h"
@@ -34,11 +37,11 @@
 #include <string>
 #include <utility>
 
-#include <boost/lexical_cast.hpp>
 #include <boost/python.hpp>
 #include <boost/python/list.hpp>
 #include <boost/python/tuple.hpp>
 #include <boost/python/extract.hpp>
+#include <boost/python/stl_iterator.hpp>
 #include <boost/date_time/posix_time/time_formatters.hpp>
 
 #ifdef FREEORION_MACOSX
@@ -91,8 +94,8 @@ namespace {
     // Wrapper for getting empire objects
     list GetAllEmpires() {
         list empire_list;
-        for (EmpireManager::const_iterator it = Empires().begin(); it != Empires().end(); ++it)
-            empire_list.append(it->second->EmpireID());
+        for (const std::map<int, Empire*>::value_type& entry : Empires())
+            empire_list.append(entry.second->EmpireID());
         return empire_list;
     }
 
@@ -102,7 +105,7 @@ namespace {
     {
         int sitrep_turn = CurrentTurn() + 1;
 
-        std::vector<std::pair<std::string, std::string> > params;
+        std::vector<std::pair<std::string, std::string>> params;
 
         if (py_params) {
             for (int i = 0; i < len(py_params); i++) {
@@ -113,8 +116,8 @@ namespace {
         }
 
         if (empire_id == ALL_EMPIRES) {
-            for (EmpireManager::const_iterator it = Empires().begin(); it != Empires().end(); ++it) {
-                it->second->AddSitRepEntry(CreateSitRep(template_string, sitrep_turn, icon, params));
+            for (const std::map<int, Empire*>::value_type& entry : Empires()) {
+                entry.second->AddSitRepEntry(CreateSitRep(template_string, sitrep_turn, icon, params));
             }
         } else {
             Empire* empire = GetEmpire(empire_id);
@@ -179,9 +182,8 @@ namespace {
 
     list GetAllSpecies() {
         list            species_list;
-        SpeciesManager& species_manager = GetSpeciesManager();
-        for (SpeciesManager::iterator it = species_manager.begin(); it != species_manager.end(); ++it) {
-            species_list.append(object(it->first));
+        for (const std::map<std::string, Species*>::value_type& entry : GetSpeciesManager()) {
+            species_list.append(object(entry.first));
         }
         return species_list;
     }
@@ -204,6 +206,41 @@ namespace {
         return species_list;
     }
 
+    //Checks the condition against many objects at once.
+    //Checking many systems is more efficient because for example monster fleet plans
+    //typically uses WithinStarLaneJumps to exclude placement near empires.
+    list FilterIDsWithCondition(const Condition::ConditionBase* cond, const list &obj_ids) {
+        list permitted_ids;
+
+        Condition::ObjectSet objs;
+        boost::python::stl_input_iterator<int> end;
+        for (boost::python::stl_input_iterator<int> id(obj_ids); id != end; ++id) {
+            if (std::shared_ptr<const UniverseObject> obj = GetUniverseObject(*id))
+                objs.push_back(obj);
+            else
+                ErrorLogger() << "FilterIDsWithCondition:: Passed an invalid universe object id " << *id;
+        }
+        if (objs.empty()) {
+            ErrorLogger() << "FilterIDsWithCondition:: Couldn't get any valid objects";
+            return permitted_ids;
+        }
+
+        Condition::ObjectSet permitted_objs;
+
+        // get location condition and evaluate it with the specified universe object
+        // if no location condition has been defined, all objects matches
+        if (cond)
+            cond->Eval(ScriptingContext(), permitted_objs, objs);
+        else
+            permitted_objs = objs;
+
+        for (auto &obj : permitted_objs) {
+            permitted_ids.append(obj->ID());
+        }
+
+        return permitted_ids;
+    }
+
     // Wrappers for Specials / SpecialManager functions
     double SpecialSpawnRate(const std::string special_name) {
         const Special* special = GetSpecial(special_name);
@@ -223,25 +260,15 @@ namespace {
         return special->SpawnLimit();
     }
 
-    bool SpecialLocation(const std::string special_name, int object_id) {
+    list SpecialLocations(const std::string special_name, list object_ids) {
         // get special and check if it exists
         const Special* special = GetSpecial(special_name);
         if (!special) {
             ErrorLogger() << "SpecialLocation: couldn't get special " << special_name;
-            return false;
+            return list();
         }
 
-        // get the universe object to test and check if it exists
-        TemporaryPtr<UniverseObject> obj = GetUniverseObject(object_id);
-        if (!obj) {
-            ErrorLogger() << "SpecialLocation: Couldn't get object with ID " << object_id;
-            return false;
-        }
-
-        // get special location condition and evaluate it with the specified universe object
-        // if no location condition has been defined, no object matches
-        const Condition::ConditionBase* location_test = special->Location();
-        return (location_test && location_test->Eval(obj));
+        return FilterIDsWithCondition(special->Location(), object_ids);
     }
 
     bool SpecialHasLocation(const std::string special_name) {
@@ -256,9 +283,8 @@ namespace {
 
     list GetAllSpecials() {
         list py_specials;
-        const std::vector<std::string> special_names = SpecialNames();
-        for (std::vector<std::string>::const_iterator it = special_names.begin(); it != special_names.end(); ++it) {
-            py_specials.append(object(*it));
+        for (const std::string& special_name : SpecialNames()) {
+            py_specials.append(object(special_name));
         }
         return py_specials;
     }
@@ -317,8 +343,22 @@ namespace {
         list py_items;
         std::vector<ItemSpec> items;
         parse::items(items);
-        for (unsigned int i = 0; i < items.size(); i++) {
-            py_items.append(object(items[i]));
+        for (const ItemSpec& item : items) {
+            py_items.append(object(item));
+        }
+        return py_items;
+    }
+
+    // Wrapper for starting buildings
+    list LoadStartingBuildings() {
+        list py_items;
+        std::vector<ItemSpec> buildings;
+        parse::starting_buildings(buildings);
+        for (auto building : buildings) {
+            if (GetBuildingType(building.name))
+                py_items.append(object(building));
+            else
+                ErrorLogger() << "The item " << building.name << " in the starting building list is not a building.";
         }
         return py_items;
     }
@@ -371,9 +411,8 @@ namespace {
 
     list ShipDesignGetPremadeList() {
         list py_ship_designs;
-        const PredefinedShipDesignManager& manager = GetPredefinedShipDesignManager();
-        for (PredefinedShipDesignManager::iterator it = manager.begin(); it != manager.end(); ++it) {
-            py_ship_designs.append(object(it->first));
+        for (const std::map<std::string, ShipDesign*>::value_type& entry : GetPredefinedShipDesignManager()) {
+            py_ship_designs.append(object(entry.first));
         }
         return list(py_ship_designs);
     }
@@ -403,7 +442,6 @@ namespace {
             m_fleet_plan = new FleetPlan(fleet_name, designs, false);
         }
 
-        // dtor
         virtual ~FleetPlanWrapper()
         { delete m_fleet_plan; }
 
@@ -413,9 +451,8 @@ namespace {
 
         list ShipDesigns() {
             list py_designs;
-            const std::vector<std::string>& designs = m_fleet_plan->ShipDesigns();
-            for (unsigned int i = 0; i < designs.size(); i++) {
-                py_designs.append(object(designs[i]));
+            for (const std::string& design_name : m_fleet_plan->ShipDesigns()) {
+                py_designs.append(object(design_name));
             }
             return list(py_designs);
         }
@@ -431,8 +468,8 @@ namespace {
         list py_fleet_plans;
         std::vector<FleetPlan*> fleet_plans;
         parse::fleet_plans(fleet_plans);
-        for (unsigned int i = 0; i < fleet_plans.size(); i++) {
-            py_fleet_plans.append(new FleetPlanWrapper(fleet_plans[i]));
+        for (FleetPlan* fleet_plan : fleet_plans) {
+            py_fleet_plans.append(new FleetPlanWrapper(fleet_plan));
         }
         return list(py_fleet_plans);
     }
@@ -457,7 +494,6 @@ namespace {
                                  spawn_limit, 0, false);
         }
 
-        // dtor
         virtual ~MonsterFleetPlanWrapper()
         { delete m_monster_fleet_plan; }
 
@@ -467,9 +503,8 @@ namespace {
 
         list ShipDesigns() {
             list py_designs;
-            const std::vector<std::string>& designs = m_monster_fleet_plan->ShipDesigns();
-            for (unsigned int i = 0; i < designs.size(); i++) {
-                py_designs.append(object(designs[i]));
+            for (const std::string& design_name : m_monster_fleet_plan->ShipDesigns()) {
+                py_designs.append(object(design_name));
             }
             return list(py_designs);
         }
@@ -480,18 +515,8 @@ namespace {
         int SpawnLimit()
         { return m_monster_fleet_plan->SpawnLimit(); }
 
-        bool Location(int object_id) {
-            // get the universe object to test and check if it exists
-            TemporaryPtr<UniverseObject> obj = GetUniverseObject(object_id);
-            if (!obj) {
-                ErrorLogger() << "MonsterFleetPlanWrapper::Location: Couldn't get object with ID " << object_id;
-                return false;
-            }
-
-            // get location condition and evaluate it with the specified universe object
-            // if no location condition has been defined, any object matches
-            const Condition::ConditionBase* location_test = m_monster_fleet_plan->Location();
-            return (!location_test || location_test->Eval(obj));
+        list Locations(list systems) {
+            return FilterIDsWithCondition(m_monster_fleet_plan->Location(), systems);
         }
 
         const MonsterFleetPlan& GetMonsterFleetPlan()
@@ -505,8 +530,8 @@ namespace {
         list py_monster_fleet_plans;
         std::vector<MonsterFleetPlan*> monster_fleet_plans;
         parse::monster_fleet_plans(monster_fleet_plans);
-        for (unsigned int i = 0; i < monster_fleet_plans.size(); i++) {
-            py_monster_fleet_plans.append(new MonsterFleetPlanWrapper(monster_fleet_plans[i]));
+        for (MonsterFleetPlan* fleet_plan : monster_fleet_plans) {
+            py_monster_fleet_plans.append(new MonsterFleetPlanWrapper(fleet_plan));
         }
         return list(py_monster_fleet_plans);
     }
@@ -519,7 +544,7 @@ namespace {
     //
     // Wrappers for common UniverseObject class member funtions
     object GetName(int object_id) {
-        TemporaryPtr<UniverseObject> obj = GetUniverseObject(object_id);
+        std::shared_ptr<UniverseObject> obj = GetUniverseObject(object_id);
         if (!obj) {
             ErrorLogger() << "GetName: Couldn't get object with ID " << object_id;
             return object("");
@@ -528,7 +553,7 @@ namespace {
     }
 
     void SetName(int object_id, const std::string& name) {
-        TemporaryPtr<UniverseObject> obj = GetUniverseObject(object_id);
+        std::shared_ptr<UniverseObject> obj = GetUniverseObject(object_id);
         if (!obj) {
             ErrorLogger() << "RenameUniverseObject: Couldn't get object with ID " << object_id;
             return;
@@ -537,7 +562,7 @@ namespace {
     }
 
     double GetX(int object_id) {
-        TemporaryPtr<UniverseObject> obj = GetUniverseObject(object_id);
+        std::shared_ptr<UniverseObject> obj = GetUniverseObject(object_id);
         if (!obj) {
             ErrorLogger() << "GetX: Couldn't get object with ID " << object_id;
             return UniverseObject::INVALID_POSITION;
@@ -546,7 +571,7 @@ namespace {
     }
 
     double GetY(int object_id) {
-        TemporaryPtr<UniverseObject> obj = GetUniverseObject(object_id);
+        std::shared_ptr<UniverseObject> obj = GetUniverseObject(object_id);
         if (!obj) {
             ErrorLogger() << "GetY: Couldn't get object with ID " << object_id;
             return UniverseObject::INVALID_POSITION;
@@ -555,7 +580,7 @@ namespace {
     }
 
     tuple GetPos(int object_id) {
-        TemporaryPtr<UniverseObject> obj = GetUniverseObject(object_id);
+        std::shared_ptr<UniverseObject> obj = GetUniverseObject(object_id);
         if (!obj) {
             ErrorLogger() << "GetPos: Couldn't get object with ID " << object_id;
             return make_tuple(UniverseObject::INVALID_POSITION, UniverseObject::INVALID_POSITION);
@@ -564,7 +589,7 @@ namespace {
     }
 
     int GetOwner(int object_id) {
-        TemporaryPtr<UniverseObject> obj = GetUniverseObject(object_id);
+        std::shared_ptr<UniverseObject> obj = GetUniverseObject(object_id);
         if (!obj) {
             ErrorLogger() << "GetOwner: Couldn't get object with ID " << object_id;
             return ALL_EMPIRES;
@@ -574,7 +599,7 @@ namespace {
 
     void AddSpecial(int object_id, const std::string special_name) {
         // get the universe object and check if it exists
-        TemporaryPtr<UniverseObject> obj = GetUniverseObject(object_id);
+        std::shared_ptr<UniverseObject> obj = GetUniverseObject(object_id);
         if (!obj) {
             ErrorLogger() << "AddSpecial: Couldn't get object with ID " << object_id;
             return;
@@ -593,7 +618,7 @@ namespace {
 
     void RemoveSpecial(int object_id, const std::string special_name) {
         // get the universe object and check if it exists
-        TemporaryPtr<UniverseObject> obj = GetUniverseObject(object_id);
+        std::shared_ptr<UniverseObject> obj = GetUniverseObject(object_id);
         if (!obj) {
             ErrorLogger() << "RemoveSpecial: Couldn't get object with ID " << object_id;
             return;
@@ -614,25 +639,23 @@ namespace {
     { GetUniverse().SetUniverseWidth(width); }
 
     double LinearDistance(int system1_id, int system2_id)
-    { return GetUniverse().LinearDistance(system1_id, system2_id); }
+    { return GetPathfinder()->LinearDistance(system1_id, system2_id); }
 
     int JumpDistanceBetweenSystems(int system1_id, int system2_id)
-    { return GetUniverse().JumpDistanceBetweenSystems(system1_id, system2_id); }
+    { return GetPathfinder()->JumpDistanceBetweenSystems(system1_id, system2_id); }
 
     list GetAllObjects() {
         list py_all_objects;
-        std::vector<int> all_objects = Objects().FindObjectIDs();
-        for (std::vector<int>::iterator it = all_objects.begin(); it != all_objects.end(); ++it) {
-            py_all_objects.append(*it);
+        for (int object_id : Objects().FindObjectIDs()) {
+            py_all_objects.append(object_id);
         }
         return py_all_objects;
     }
 
     list GetSystems() {
         list py_systems;
-        std::vector<int> systems = Objects().FindObjectIDs<System>();
-        for (std::vector<int>::iterator it = systems.begin(); it != systems.end(); ++it) {
-            py_systems.append(*it);
+        for (int system_id : Objects().FindObjectIDs<System>()) {
+            py_systems.append(system_id);
         }
         return py_systems;
     }
@@ -645,7 +668,7 @@ namespace {
         }
 
         // Create system and insert it into the object map
-        TemporaryPtr<System> system = GetUniverse().CreateSystem(star_type, star_name, x, y);
+        std::shared_ptr<System> system = GetUniverse().CreateSystem(star_type, star_name, x, y);
         if (!system) {
             ErrorLogger() << "CreateSystem : Attempt to insert system into the object map failed";
             return INVALID_OBJECT_ID;
@@ -655,7 +678,7 @@ namespace {
     }
 
     int CreatePlanet(PlanetSize size, PlanetType planet_type, int system_id, int orbit, const std::string& name) {
-        TemporaryPtr<System> system = Objects().Object<System>(system_id);
+        std::shared_ptr<System> system = Objects().Object<System>(system_id);
 
         // Perform some validity checks
         // Check if system with id system_id exists
@@ -696,14 +719,14 @@ namespace {
         }
 
         // Create planet and insert it into the object map
-        TemporaryPtr<Planet> planet = GetUniverse().CreatePlanet(planet_type, size);
+        std::shared_ptr<Planet> planet = GetUniverse().CreatePlanet(planet_type, size);
         if (!planet) {
             ErrorLogger() << "CreateSystem : Attempt to insert planet into the object map failed";
             return INVALID_OBJECT_ID;
         }
 
         // Add planet to system map
-        system->Insert(TemporaryPtr<UniverseObject>(planet), orbit);
+        system->Insert(std::shared_ptr<UniverseObject>(planet), orbit);
 
         // If a name has been specified, set planet name
         if (!(name.empty()))
@@ -713,13 +736,13 @@ namespace {
     }
 
     int CreateBuilding(const std::string& building_type, int planet_id, int empire_id) {
-        TemporaryPtr<Planet> planet = Objects().Object<Planet>(planet_id);
+        std::shared_ptr<Planet> planet = Objects().Object<Planet>(planet_id);
         if (!planet) {
             ErrorLogger() << "CreateBuilding: couldn't get planet with ID " << planet_id;
             return INVALID_OBJECT_ID;
         }
 
-        TemporaryPtr<System> system = GetSystem(planet->SystemID());
+        std::shared_ptr<System> system = GetSystem(planet->SystemID());
         if (!system) {
             ErrorLogger() << "CreateBuilding: couldn't get system for planet";
             return INVALID_OBJECT_ID;
@@ -731,7 +754,7 @@ namespace {
             return INVALID_OBJECT_ID;
         }
 
-        TemporaryPtr<Building> building = GetUniverse().CreateBuilding(empire_id, building_type, empire_id);
+        std::shared_ptr<Building> building = GetUniverse().CreateBuilding(empire_id, building_type, empire_id);
         if (!building) {
             ErrorLogger() << "CreateBuilding: couldn't create building";
             return INVALID_OBJECT_ID;
@@ -745,14 +768,14 @@ namespace {
 
     int CreateFleet(const std::string& name, int system_id, int empire_id) {
         // Get system and check if it exists
-        TemporaryPtr<System> system = Objects().Object<System>(system_id);
+        std::shared_ptr<System> system = Objects().Object<System>(system_id);
         if (!system) {
             ErrorLogger() << "CreateFleet: couldn't get system with ID " << system_id;
             return INVALID_OBJECT_ID;
         }
 
         // Create new fleet at the position of the specified system
-        TemporaryPtr<Fleet> fleet = GetUniverse().CreateFleet(name, system->X(), system->Y(), empire_id);
+        std::shared_ptr<Fleet> fleet = GetUniverse().CreateFleet(name, system->X(), system->Y(), empire_id);
         if (!fleet) {
             ErrorLogger() << "CreateFleet: couldn't create new fleet";
             return INVALID_OBJECT_ID;
@@ -764,7 +787,7 @@ namespace {
         // check if we got a fleet name...
         if (name.empty()) {
             // ...no name has been specified, so we have to generate one using the new fleet id
-            fleet->Rename(UserString("OBJ_FLEET") + " " + boost::lexical_cast<std::string>(fleet->ID()));
+            fleet->Rename(UserString("OBJ_FLEET") + " " + std::to_string(fleet->ID()));
         }
 
         // return fleet ID
@@ -788,13 +811,13 @@ namespace {
         }
 
         // get fleet and check if it exists
-        TemporaryPtr<Fleet> fleet = GetFleet(fleet_id);
+        std::shared_ptr<Fleet> fleet = GetFleet(fleet_id);
         if (!fleet) {
             ErrorLogger() << "CreateShip: couldn't get fleet with ID " << fleet_id;
             return INVALID_OBJECT_ID;
         }
 
-        TemporaryPtr<System> system = GetSystem(fleet->SystemID());
+        std::shared_ptr<System> system = GetSystem(fleet->SystemID());
         if (!system) {
             ErrorLogger() << "CreateShip: couldn't get system for fleet";
             return INVALID_OBJECT_ID;
@@ -803,7 +826,7 @@ namespace {
         // get owner empire of specified fleet
         int empire_id = fleet->Owner();
         // if we got the id of an actual empire, get the empire object and check if it exists
-        Empire* empire = 0;
+        Empire* empire = nullptr;
         if (empire_id != ALL_EMPIRES) {
             empire = GetEmpire(empire_id);
             if (!empire) {
@@ -813,7 +836,7 @@ namespace {
         }
 
         // create new ship
-        TemporaryPtr<Ship> ship = universe.CreateShip(empire_id, ship_design->ID(), species, empire_id);
+        std::shared_ptr<Ship> ship = universe.CreateShip(empire_id, ship_design->ID(), species, empire_id);
         if (!ship) {
             ErrorLogger() << "CreateShip: couldn't create new ship";
             return INVALID_OBJECT_ID;
@@ -830,7 +853,7 @@ namespace {
                 ship->Rename(empire->NewShipName());
             } else {
                 // ...no, so construct a name using the new ships id
-                ship->Rename(UserString("OBJ_SHIP") + " " + boost::lexical_cast<std::string>(ship->ID()));
+                ship->Rename(UserString("OBJ_SHIP") + " " + std::to_string(ship->ID()));
             }
         } else {
             // ...yes, name has been specified, so use it
@@ -840,7 +863,7 @@ namespace {
         // add ship to fleet, this also moves the ship to the
         // fleets location and inserts it into the system
         fleet->AddShip(ship->ID());
-        fleet->SetAggressive(fleet->HasArmedShips());
+        fleet->SetAggressive(fleet->HasArmedShips() || fleet->HasFighterShips());
         ship->SetFleetID(fleet->ID());
 
         // set the meters of the ship to max values
@@ -860,12 +883,12 @@ namespace {
     int CreateMonster(const std::string& design_name, int fleet_id)
     { return CreateShip(NewMonsterName(), design_name, "", fleet_id); }
 
-    TemporaryPtr<Field> CreateFieldImpl(const std::string& field_type_name, double x, double y, double size) {
+    std::shared_ptr<Field> CreateFieldImpl(const std::string& field_type_name, double x, double y, double size) {
         // check if a field type with the specified field type name exists and get the field type
         const FieldType* field_type = GetFieldType(field_type_name);
         if (!field_type) {
             ErrorLogger() << "CreateFieldImpl: couldn't get field type with name: " << field_type_name;
-            return TemporaryPtr<Field>();
+            return nullptr;
         }
 
         // check if the specified size is within sane limits, and reset its value if not
@@ -879,10 +902,10 @@ namespace {
         }
 
         // create the new field
-        TemporaryPtr<Field> field = GetUniverse().CreateField(field_type->Name(), x, y, size);
+        std::shared_ptr<Field> field = GetUniverse().CreateField(field_type->Name(), x, y, size);
         if (!field) {
             ErrorLogger() << "CreateFieldImpl: couldn't create field";
-            return TemporaryPtr<Field>();
+            return nullptr;
         }
 
         // get the localized version of the field type name and set that as the fields name
@@ -891,7 +914,7 @@ namespace {
     }
 
     int CreateField(const std::string& field_type_name, double x, double y, double size) {
-        TemporaryPtr<Field> field = CreateFieldImpl(field_type_name, x, y, size);
+        std::shared_ptr<Field> field = CreateFieldImpl(field_type_name, x, y, size);
         if (field)
             return field->ID();
         else
@@ -900,22 +923,57 @@ namespace {
 
     int CreateFieldInSystem(const std::string& field_type_name, double size, int system_id) {
         // check if system exists and get system
-        TemporaryPtr<System> system = GetSystem(system_id);
+        std::shared_ptr<System> system = GetSystem(system_id);
         if (!system) {
             ErrorLogger() << "CreateFieldInSystem: couldn't get system with ID" << system_id;
             return INVALID_OBJECT_ID;
         }
         // create the field with the coordinates of the system
-        TemporaryPtr<Field> field = CreateFieldImpl(field_type_name, system->X(), system->Y(), size);
+        std::shared_ptr<Field> field = CreateFieldImpl(field_type_name, system->X(), system->Y(), size);
         if (!field)
             return INVALID_OBJECT_ID;
         system->Insert(field); // insert the field into the system
         return field->ID();
     }
 
+    // Return a list of system ids of universe objects with @p obj_ids.
+    list ObjectsGetSystems(list obj_ids) {
+        list py_systems;
+        boost::python::stl_input_iterator<int> end;
+        for (boost::python::stl_input_iterator<int> id(obj_ids);
+             id != end; ++id) {
+            if (std::shared_ptr<const UniverseObject> obj = GetUniverseObject(*id)) {
+                py_systems.append(obj->SystemID());
+            } else {
+                ErrorLogger() << "Passed an invalid universe object id " << *id;
+                py_systems.append(INVALID_OBJECT_ID);
+            }
+        }
+        return py_systems;
+    }
+
+    // Return all systems within \p jumps of \p sys_ids
+    list SystemsWithinJumps(size_t jumps, list sys_ids) {
+        list py_systems;
+        boost::python::stl_input_iterator<int> end;
+
+        std::vector<int> systems;
+
+        for (boost::python::stl_input_iterator<int> id(sys_ids); id != end; ++id) {
+            systems.push_back(*id);
+        }
+
+        auto systems_in_vicinity = GetPathfinder()->WithinJumps(jumps, systems);
+
+        for (auto system_id : systems_in_vicinity) {
+            py_systems.append(system_id);
+        }
+        return py_systems;
+    }
+
     // Wrappers for System class member functions
     StarType SystemGetStarType(int system_id) {
-        TemporaryPtr<System> system = GetSystem(system_id);
+        std::shared_ptr<System> system = GetSystem(system_id);
         if (!system) {
             ErrorLogger() << "SystemGetStarType: couldn't get system with ID " << system_id;
             return INVALID_STAR_TYPE;
@@ -930,7 +988,7 @@ namespace {
             return;
         }
 
-        TemporaryPtr<System> system = GetSystem(system_id);
+        std::shared_ptr<System> system = GetSystem(system_id);
         if (!system) {
             ErrorLogger() << "SystemSetStarType : Couldn't get system with ID " << system_id;
             return;
@@ -940,7 +998,7 @@ namespace {
     }
 
     int SystemGetNumOrbits(int system_id) {
-        TemporaryPtr<System> system = GetSystem(system_id);
+        std::shared_ptr<System> system = GetSystem(system_id);
         if (!system) {
             ErrorLogger() << "SystemGetNumOrbits : Couldn't get system with ID " << system_id;
             return 0;
@@ -950,20 +1008,18 @@ namespace {
 
     list SystemFreeOrbits(int system_id) {
         list py_orbits;
-        TemporaryPtr<System> system = GetSystem(system_id);
+        std::shared_ptr<System> system = GetSystem(system_id);
         if (!system) {
             ErrorLogger() << "SystemFreeOrbits : Couldn't get system with ID " << system_id;
             return py_orbits;
         }
-        const std::set<int>& orbits = system->FreeOrbits();
-        for (std::set<int>::const_iterator it = orbits.begin();
-             it != orbits.end(); ++it)
-        { py_orbits.append(*it); }
+        for (int orbit_idx : system->FreeOrbits())
+        { py_orbits.append(orbit_idx); }
         return py_orbits;
     }
 
     bool SystemOrbitOccupied(int system_id, int orbit) {
-        TemporaryPtr<System> system = GetSystem(system_id);
+        std::shared_ptr<System> system = GetSystem(system_id);
         if (!system) {
             ErrorLogger() << "SystemOrbitOccupied : Couldn't get system with ID " << system_id;
             return 0;
@@ -972,7 +1028,7 @@ namespace {
     }
 
     int SystemOrbitOfPlanet(int system_id, int planet_id) {
-        TemporaryPtr<System> system = GetSystem(system_id);
+        std::shared_ptr<System> system = GetSystem(system_id);
         if (!system) {
             ErrorLogger() << "SystemOrbitOfPlanet : Couldn't get system with ID " << system_id;
             return 0;
@@ -982,50 +1038,44 @@ namespace {
 
     list SystemGetPlanets(int system_id) {
         list py_planets;
-        TemporaryPtr<System> system = GetSystem(system_id);
+        std::shared_ptr<System> system = GetSystem(system_id);
         if (!system) {
             ErrorLogger() << "SystemGetPlanets : Couldn't get system with ID " << system_id;
             return py_planets;
         }
-        const std::set<int>& planets = system->PlanetIDs();
-        for (std::set<int>::const_iterator it = planets.begin();
-             it != planets.end(); ++it)
-        { py_planets.append(*it); }
+        for (int planet_id : system->PlanetIDs())
+        { py_planets.append(planet_id); }
         return py_planets;
     }
 
     list SystemGetFleets(int system_id) {
         list py_fleets;
-        TemporaryPtr<System> system = GetSystem(system_id);
+        std::shared_ptr<System> system = GetSystem(system_id);
         if (!system) {
             ErrorLogger() << "SystemGetFleets : Couldn't get system with ID " << system_id;
             return py_fleets;
         }
-        const std::set<int>& fleets = system->FleetIDs();
-        for (std::set<int>::const_iterator it = fleets.begin();
-             it != fleets.end(); ++it)
-        { py_fleets.append(*it); }
+        for (int fleet_id : system->FleetIDs())
+        { py_fleets.append(fleet_id); }
         return py_fleets;
     }
 
     list SystemGetStarlanes(int system_id) {
         list py_starlanes;
         // get source system
-        TemporaryPtr<System> system = GetSystem(system_id);
+        std::shared_ptr<System> system = GetSystem(system_id);
         if (!system) {
             ErrorLogger() << "SystemGetStarlanes : Couldn't get system with ID " << system_id;
             return py_starlanes;
         }
         // get list of systems the source system has starlanes to
         // we actually get a map of ids and a bool indicating if the entry is a starlane (false) or wormhole (true)
-        const std::map<int, bool>& starlanes = system->StarlanesWormholes();
         // iterate over the map we got, only copy starlanes to the python list object we are going to return
-        for (std::map<int, bool>::const_iterator it = starlanes.begin();
-             it != starlanes.end(); ++it) {
+        for (const std::map<int, bool>::value_type& lane : system->StarlanesWormholes()) {
             // if the bool value is false, we have a starlane
             // in this case copy the destination system id to our starlane list
-            if (!(it->second)) {
-                py_starlanes.append(it->first);
+            if (!(lane.second)) {
+                py_starlanes.append(lane.first);
             }
         }
         return py_starlanes;
@@ -1033,12 +1083,12 @@ namespace {
 
     void SystemAddStarlane(int from_sys_id, int to_sys_id) {
         // get source and destination system, check that both exist
-        TemporaryPtr<System> from_sys = GetSystem(from_sys_id);
+        std::shared_ptr<System> from_sys = GetSystem(from_sys_id);
         if (!from_sys) {
             ErrorLogger() << "SystemAddStarlane : Couldn't find system with ID " << from_sys_id;
             return;
         }
-        TemporaryPtr<System> to_sys = GetSystem(to_sys_id);
+        std::shared_ptr<System> to_sys = GetSystem(to_sys_id);
         if (!to_sys) {
             ErrorLogger() << "SystemAddStarlane : Couldn't find system with ID " << to_sys_id;
             return;
@@ -1050,12 +1100,12 @@ namespace {
 
     void SystemRemoveStarlane(int from_sys_id, int to_sys_id) {
         // get source and destination system, check that both exist
-        TemporaryPtr<System> from_sys = GetSystem(from_sys_id);
+        std::shared_ptr<System> from_sys = GetSystem(from_sys_id);
         if (!from_sys) {
             ErrorLogger() << "SystemRemoveStarlane : Couldn't find system with ID " << from_sys_id;
             return;
         }
-        TemporaryPtr<System> to_sys = GetSystem(to_sys_id);
+        std::shared_ptr<System> to_sys = GetSystem(to_sys_id);
         if (!to_sys) {
             ErrorLogger() << "SystemRemoveStarlane : Couldn't find system with ID " << to_sys_id;
             return;
@@ -1067,7 +1117,7 @@ namespace {
 
     // Wrapper for Planet class member functions
     PlanetType PlanetGetType(int planet_id) {
-        TemporaryPtr<Planet> planet = GetPlanet(planet_id);
+        std::shared_ptr<Planet> planet = GetPlanet(planet_id);
         if (!planet) {
             ErrorLogger() << "PlanetGetType: Couldn't get planet with ID " << planet_id;
             return INVALID_PLANET_TYPE;
@@ -1076,7 +1126,7 @@ namespace {
     }
 
     void PlanetSetType(int planet_id, PlanetType planet_type) {
-        TemporaryPtr<Planet> planet = GetPlanet(planet_id);
+        std::shared_ptr<Planet> planet = GetPlanet(planet_id);
         if (!planet) {
             ErrorLogger() << "PlanetSetType: Couldn't get planet with ID " << planet_id;
             return;
@@ -1094,7 +1144,7 @@ namespace {
     }
 
     PlanetSize PlanetGetSize(int planet_id) {
-        TemporaryPtr<Planet> planet = GetPlanet(planet_id);
+        std::shared_ptr<Planet> planet = GetPlanet(planet_id);
         if (!planet) {
             ErrorLogger() << "PlanetGetSize: Couldn't get planet with ID " << planet_id;
             return INVALID_PLANET_SIZE;
@@ -1103,7 +1153,7 @@ namespace {
     }
 
     void PlanetSetSize(int planet_id, PlanetSize planet_size) {
-        TemporaryPtr<Planet> planet = GetPlanet(planet_id);
+        std::shared_ptr<Planet> planet = GetPlanet(planet_id);
         if (!planet) {
             ErrorLogger() << "PlanetSetSize: Couldn't get planet with ID " << planet_id;
             return;
@@ -1119,7 +1169,7 @@ namespace {
     }
 
     object PlanetGetSpecies(int planet_id) {
-        TemporaryPtr<Planet> planet = GetPlanet(planet_id);
+        std::shared_ptr<Planet> planet = GetPlanet(planet_id);
         if (!planet) {
             ErrorLogger() << "PlanetGetSpecies: Couldn't get planet with ID " << planet_id;
             return object("");
@@ -1128,7 +1178,7 @@ namespace {
     }
 
     void PlanetSetSpecies(int planet_id, const std::string& species_name) {
-        TemporaryPtr<Planet> planet = GetPlanet(planet_id);
+        std::shared_ptr<Planet> planet = GetPlanet(planet_id);
         if (!planet) {
             ErrorLogger() << "PlanetSetSpecies: Couldn't get planet with ID " << planet_id;
             return;
@@ -1137,7 +1187,7 @@ namespace {
     }
 
     object PlanetGetFocus(int planet_id) {
-        TemporaryPtr<Planet> planet = GetPlanet(planet_id);
+        std::shared_ptr<Planet> planet = GetPlanet(planet_id);
         if (!planet) {
             ErrorLogger() << "PlanetGetFocus: Couldn't get planet with ID " << planet_id;
             return object("");
@@ -1146,7 +1196,7 @@ namespace {
     }
 
     void PlanetSetFocus(int planet_id, const std::string& focus) {
-        TemporaryPtr<Planet> planet = GetPlanet(planet_id);
+        std::shared_ptr<Planet> planet = GetPlanet(planet_id);
         if (!planet) {
             ErrorLogger() << "PlanetSetSpecies: Couldn't get planet with ID " << planet_id;
             return;
@@ -1156,20 +1206,19 @@ namespace {
 
     list PlanetAvailableFoci(int planet_id) {
         list py_foci;
-        TemporaryPtr<Planet> planet = GetPlanet(planet_id);
+        std::shared_ptr<Planet> planet = GetPlanet(planet_id);
         if (!planet) {
             ErrorLogger() << "PlanetAvailableFoci: Couldn't get planet with ID " << planet_id;
             return py_foci;
         }
-        std::vector<std::string> foci = planet->AvailableFoci();
-        for (unsigned int i = 0; i < foci.size(); i++) {
-            py_foci.append(object(foci[i]));
+        for (const std::string& focus : planet->AvailableFoci()) {
+            py_foci.append(object(focus));
         }
         return py_foci;
     }
 
     bool PlanetMakeOutpost(int planet_id, int empire_id) {
-        TemporaryPtr<Planet> planet = GetPlanet(planet_id);
+        std::shared_ptr<Planet> planet = GetPlanet(planet_id);
         if (!planet) {
             ErrorLogger() << "PlanetMakeOutpost: couldn't get planet with ID:" << planet_id;
             return false;
@@ -1184,7 +1233,7 @@ namespace {
     }
 
     bool PlanetMakeColony(int planet_id, int empire_id, const std::string& species, double population) {
-        TemporaryPtr<Planet> planet = GetPlanet(planet_id);
+        std::shared_ptr<Planet> planet = GetPlanet(planet_id);
         if (!planet) {
             ErrorLogger() << "PlanetMakeColony: couldn't get planet with ID:" << planet_id;
             return false;
@@ -1207,7 +1256,7 @@ namespace {
     }
 
     object PlanetCardinalSuffix(int planet_id) {
-        TemporaryPtr<Planet> planet = GetPlanet(planet_id);
+        std::shared_ptr<Planet> planet = GetPlanet(planet_id);
         if (!planet) {
             ErrorLogger() << "PlanetCardinalSuffix: couldn't get planet with ID:" << planet_id;
             return object(UserString("ERROR"));
@@ -1234,7 +1283,7 @@ namespace FreeOrionPython {
             .def("ship_designs",        &MonsterFleetPlanWrapper::ShipDesigns)
             .def("spawn_rate",          &MonsterFleetPlanWrapper::SpawnRate)
             .def("spawn_limit",         &MonsterFleetPlanWrapper::SpawnLimit)
-            .def("location",            &MonsterFleetPlanWrapper::Location);
+            .def("locations",           &MonsterFleetPlanWrapper::Locations);
 
         def("get_universe",                         GetUniverse,                    return_value_policy<reference_existing_object>());
         def("get_all_empires",                      GetAllEmpires);
@@ -1266,7 +1315,7 @@ namespace FreeOrionPython {
 
         def("special_spawn_rate",                   SpecialSpawnRate);
         def("special_spawn_limit",                  SpecialSpawnLimit);
-        def("special_location",                     SpecialLocation);
+        def("special_locations",                    SpecialLocations);
         def("special_has_location",                 SpecialHasLocation);
         def("get_all_specials",                     GetAllSpecials);
 
@@ -1280,6 +1329,7 @@ namespace FreeOrionPython {
         def("design_get_monster_list",              ShipDesignGetMonsterList);
 
         def("load_item_spec_list",                  LoadItemSpecList);
+        def("load_starting_buildings",              LoadStartingBuildings);
         def("load_fleet_plan_list",                 LoadFleetPlanList);
         def("load_monster_fleet_plan_list",         LoadMonsterFleetPlanList);
 
@@ -1308,6 +1358,10 @@ namespace FreeOrionPython {
         def("create_field",                         CreateField);
         def("create_field_in_system",               CreateFieldInSystem);
 
+        def("objs_get_systems",                     ObjectsGetSystems);
+
+        def("systems_within_jumps_unordered",       SystemsWithinJumps, "Return all systems within ''jumps'' of the systems with ids ''sys_ids''");
+
         def("sys_get_star_type",                    SystemGetStarType);
         def("sys_set_star_type",                    SystemSetStarType);
         def("sys_get_num_orbits",                   SystemGetNumOrbits);
