@@ -22,9 +22,6 @@
 #include "../util/ScopedTimer.h"
 #include "../combat/CombatLogManager.h"
 
-#include <GG/utf8/checked.h>
-
-#include <boost/filesystem/operations.hpp>
 #include <boost/filesystem/fstream.hpp>
 #include <boost/serialization/deque.hpp>
 #include <boost/serialization/list.hpp>
@@ -39,14 +36,34 @@
 #include <boost/iostreams/stream.hpp>
 #include <boost/thread.hpp>
 
+// HACK: The following two includes work around a bug in boost 1.56,
+// which uses them without including.
+#include <boost/version.hpp>
+#if BOOST_VERSION == 105600
+#include <boost/serialization/singleton.hpp> // This
+#include <boost/serialization/extended_type_info.hpp> //This
+#endif
+// HACK: For a similar boost 1.57 bug
+#if BOOST_VERSION == 105700
+#include <boost/serialization/type_info_implementation.hpp> // This
+#endif
+
+#if BOOST_VERSION == 105800
+// HACK: The following two includes work around a bug in boost 1.58
+#include <boost/serialization/type_info_implementation.hpp>
+#include <boost/archive/basic_archive.hpp>
+#endif
+
+#include <boost/serialization/shared_ptr.hpp>
+
+
 namespace fs = boost::filesystem;
 
 namespace {
     std::map<int, SaveGameEmpireData> CompileSaveGameEmpireData(const EmpireManager& empire_manager) {
         std::map<int, SaveGameEmpireData> retval;
-        const EmpireManager& empires = Empires();
-        for (EmpireManager::const_iterator it = empires.begin(); it != empires.end(); ++it)
-            retval[it->first] = SaveGameEmpireData(it->first, it->second->Name(), it->second->PlayerName(), it->second->Color());
+        for (const std::map<int, Empire*>::value_type& entry : Empires())
+            retval[entry.first] = SaveGameEmpireData(entry.first, entry.second->Name(), entry.second->PlayerName(), entry.second->Color());
         return retval;
     }
 
@@ -55,8 +72,6 @@ namespace {
                                     const std::map<int, SaveGameEmpireData>& empire_save_game_data,
                                     SaveGamePreviewData& preview)
     {
-        typedef std::vector<PlayerSaveGameData>::const_iterator player_iterator;
-
         // First compile the non-player related data
         preview.current_turn = server_save_game_data.m_current_turn;
         preview.number_of_empires = empire_save_game_data.size();
@@ -72,13 +87,13 @@ namespace {
 
             // If there are human players, the first of them should be the main player
             short humans = 0;
-            for (player_iterator it = player_save_game_data.begin(); it != player_save_game_data.end(); ++it) {
-                if (it->m_client_type == Networking::CLIENT_TYPE_HUMAN_PLAYER) {
+            for (const PlayerSaveGameData& psgd : player_save_game_data) {
+                if (psgd.m_client_type == Networking::CLIENT_TYPE_HUMAN_PLAYER) {
                     if (player->m_client_type != Networking::CLIENT_TYPE_HUMAN_PLAYER &&
                        player->m_client_type != Networking::CLIENT_TYPE_HUMAN_OBSERVER &&
                        player->m_client_type != Networking::CLIENT_TYPE_HUMAN_MODERATOR)
                     {
-                        player = &(*it);
+                        player = &psgd;
                     }
                     ++humans;
                 }
@@ -114,14 +129,14 @@ int SaveGame(const std::string& filename, const ServerSaveGameData& server_save_
     DebugLogger() << "Compiling save empire and preview data";
     std::map<int, SaveGameEmpireData> empire_save_game_data = CompileSaveGameEmpireData(empire_manager);
     SaveGamePreviewData save_preview_data;
-    CompileSaveGamePreviewData(server_save_game_data, player_save_game_data, empire_save_game_data, save_preview_data);
+    CompileSaveGamePreviewData(server_save_game_data, player_save_game_data,
+                               empire_save_game_data, save_preview_data);
 
 
     // reinterpret save game data as header data for uncompressed header
     std::vector<PlayerSaveHeaderData> player_save_header_data;
-    for (std::vector<PlayerSaveGameData>::const_iterator it = player_save_game_data.begin();
-         it != player_save_game_data.end(); ++it)
-    { player_save_header_data.push_back(*it); }
+    for (const PlayerSaveGameData& psgd : player_save_game_data)
+    { player_save_header_data.push_back(psgd); }
 
 
     int bytes_written = 0;
@@ -149,8 +164,85 @@ int SaveGame(const std::string& filename, const ServerSaveGameData& server_save_
             throw std::runtime_error(UNABLE_TO_OPEN_FILE);
         pos_before_writing = ofs.tellp();
 
-        if (use_binary) {
+        bool save_completed_as_xml = false;
+
+        if (!use_binary) {
+            // Attempt compressed XML serialization
+
+            try {
+                // Two-tier serialization:
+                // main archive is uncompressed serialized header data first
+                // then contains a string for compressed second archive
+                // that contains the main gamestate info
+                save_preview_data.SetBinary(false);
+
+                // allocate buffers for serialized gamestate
+                DebugLogger() << "Allocating buffers for XML serialization...";
+                std::string serial_str, compressed_str;
+                try {
+                    DebugLogger() << "String Max Size: " << serial_str.max_size();
+                    std::string::size_type capacity = std::min(serial_str.max_size(),
+                        static_cast<std::string::size_type>(std::pow(2.0, 29.0)));
+                    DebugLogger() << "Reserving Capacity:: " << capacity;
+                    serial_str.reserve(capacity);
+                    compressed_str.reserve(std::pow(2.0, 26.0));
+                }
+                catch (...) {
+                    DebugLogger() << "Unable to preallocate full serialization buffers. Attempting serialization with dynamic buffer allocation.";
+                }
+
+                // wrap buffer string in iostream::stream to receive serialized data
+                typedef boost::iostreams::back_insert_device<std::string> InsertDevice;
+                InsertDevice serial_inserter(serial_str);
+                boost::iostreams::stream<InsertDevice> s_sink(serial_inserter);
+
+                // create archive with (preallocated) buffer...
+                freeorion_xml_oarchive xoa(s_sink);
+                // serialize main gamestate info
+                xoa << BOOST_SERIALIZATION_NVP(player_save_game_data);
+                xoa << BOOST_SERIALIZATION_NVP(empire_manager);
+                xoa << BOOST_SERIALIZATION_NVP(species_manager);
+                xoa << BOOST_SERIALIZATION_NVP(combat_log_manager);
+                Serialize(xoa, universe);
+                s_sink.flush();
+
+                // wrap gamestate string in iostream::stream to extract serialized data
+                typedef boost::iostreams::basic_array_source<char> SourceDevice;
+                SourceDevice source(serial_str.data(), serial_str.size());
+                boost::iostreams::stream<SourceDevice> s_source(source);
+
+                // wrap compresed buffer string in iostream::streams to receive compressed string
+                InsertDevice compressed_inserter(compressed_str);
+                boost::iostreams::stream<InsertDevice> c_sink(compressed_inserter);
+
+                // compression-filter gamestate into compressed string
+                boost::iostreams::filtering_ostreambuf o;
+                o.push(boost::iostreams::zlib_compressor());
+                o.push(c_sink);
+                boost::iostreams::copy(s_source, o);
+                c_sink.flush();
+
+                // write to save file: uncompressed header serialized data, with compressed main archive string at end...
+                freeorion_xml_oarchive xoa2(ofs);
+                // serialize uncompressed save header info
+                xoa2 << BOOST_SERIALIZATION_NVP(save_preview_data);
+                xoa2 << BOOST_SERIALIZATION_NVP(galaxy_setup_data);
+                xoa2 << BOOST_SERIALIZATION_NVP(server_save_game_data);
+                xoa2 << BOOST_SERIALIZATION_NVP(player_save_header_data);
+                xoa2 << BOOST_SERIALIZATION_NVP(empire_save_game_data);
+                // append compressed gamestate info
+                xoa2 << BOOST_SERIALIZATION_NVP(compressed_str);
+
+                save_completed_as_xml = true;
+            } catch (...) {
+                save_completed_as_xml = false;  // redundant, but here for clarity
+            }
+        }
+
+        if (!save_completed_as_xml) {
             DebugLogger() << "Creating binary oarchive";
+            save_preview_data.SetBinary(true);
+
             freeorion_bin_oarchive boa(ofs);
             boa << BOOST_SERIALIZATION_NVP(save_preview_data);
             boa << BOOST_SERIALIZATION_NVP(galaxy_setup_data);
@@ -164,65 +256,6 @@ int SaveGame(const std::string& filename, const ServerSaveGameData& server_save_
             boa << BOOST_SERIALIZATION_NVP(combat_log_manager);
             Serialize(boa, universe);
             DebugLogger() << "Done serializing";
-
-        } else {
-            // Two-tier serialization:
-            // main archive is uncompressed serialized header data first
-            // then contains a string for compressed second archive
-            // that contains the main gamestate info
-
-
-            // allocate buffers for serialized gamestate
-            DebugLogger() << "Allocating buffers for XML serialization...";
-            std::string serial_str, compressed_str;
-            try {
-                serial_str.reserve(    std::pow(2.0, 29.0));
-                compressed_str.reserve(std::pow(2.0, 26.0));
-            } catch (...) {
-                DebugLogger() << "Unable to preallocate full serialization buffers. Attempting serialization with dynamic buffer allocation.";
-            }
-
-            // wrap buffer string in iostream::stream to receive serialized data
-            typedef boost::iostreams::back_insert_device<std::string> InsertDevice;
-            InsertDevice serial_inserter(serial_str);
-            boost::iostreams::stream<InsertDevice> s_sink(serial_inserter);
-
-            // create archive with (preallocated) buffer...
-            freeorion_xml_oarchive xoa(s_sink);
-            // serialize main gamestate info
-            xoa << BOOST_SERIALIZATION_NVP(player_save_game_data);
-            xoa << BOOST_SERIALIZATION_NVP(empire_manager);
-            xoa << BOOST_SERIALIZATION_NVP(species_manager);
-            xoa << BOOST_SERIALIZATION_NVP(combat_log_manager);
-            Serialize(xoa, universe);
-            s_sink.flush();
-
-            // wrap gamestate string in iostream::stream to extract serialized data
-            typedef boost::iostreams::basic_array_source<char> SourceDevice;
-            SourceDevice source(serial_str.data(), serial_str.size());
-            boost::iostreams::stream<SourceDevice> s_source(source);
-
-            // wrap compresed buffer string in iostream::streams to receive compressed string
-            InsertDevice compressed_inserter(compressed_str);
-            boost::iostreams::stream<InsertDevice> c_sink(compressed_inserter);
-
-            // compression-filter gamestate into compressed string
-            boost::iostreams::filtering_ostreambuf o;
-            o.push(boost::iostreams::zlib_compressor());
-            o.push(c_sink);
-            boost::iostreams::copy(s_source, o);
-            c_sink.flush();
-
-            // write to save file: uncompressed header serialized data, with compressed main archive string at end...
-            freeorion_xml_oarchive xoa2(ofs);
-            // serialize uncompressed save header info
-            xoa2 << BOOST_SERIALIZATION_NVP(save_preview_data);
-            xoa2 << BOOST_SERIALIZATION_NVP(galaxy_setup_data);
-            xoa2 << BOOST_SERIALIZATION_NVP(server_save_game_data);
-            xoa2 << BOOST_SERIALIZATION_NVP(player_save_header_data);
-            xoa2 << BOOST_SERIALIZATION_NVP(empire_save_game_data);
-            // append compressed gamestate info
-            xoa2 << BOOST_SERIALIZATION_NVP(compressed_str);
         }
 
         ofs.flush();
@@ -242,8 +275,6 @@ void LoadGame(const std::string& filename, ServerSaveGameData& server_save_game_
               EmpireManager& empire_manager, SpeciesManager& species_manager,
               CombatLogManager& combat_log_manager, GalaxySetupData& galaxy_setup_data)
 {
-    //boost::this_thread::sleep_for(boost::chrono::seconds(1));
-
     ScopedTimer timer("LoadGame: " + filename, true);
 
     // player notifications
