@@ -21,7 +21,8 @@ struct ServerFSM;
 /** Contains basic data about a player in a game. */
 struct PlayerSaveHeaderData {
     PlayerSaveHeaderData();
-    PlayerSaveHeaderData(const std::string& name, int empire_id, Networking::ClientType client_type); ///< ctor
+
+    PlayerSaveHeaderData(const std::string& name, int empire_id, Networking::ClientType client_type);
 
     std::string                         m_name;
     int                                 m_empire_id;
@@ -35,13 +36,14 @@ private:
 
 /** Contains data that must be saved for a single player. */
 struct PlayerSaveGameData : public PlayerSaveHeaderData {
-    PlayerSaveGameData(); ///< default ctor
-    PlayerSaveGameData(const std::string& name, int empire_id, const boost::shared_ptr<OrderSet>& orders,
-                       const boost::shared_ptr<SaveGameUIData>& ui_data, const std::string& save_state_string,
-                       Networking::ClientType client_type); ///< ctor
+    PlayerSaveGameData();
 
-    boost::shared_ptr<OrderSet>         m_orders;
-    boost::shared_ptr<SaveGameUIData>   m_ui_data;
+    PlayerSaveGameData(const std::string& name, int empire_id, const std::shared_ptr<OrderSet>& orders,
+                       const std::shared_ptr<SaveGameUIData>& ui_data, const std::string& save_state_string,
+                       Networking::ClientType client_type);
+
+    std::shared_ptr<OrderSet> m_orders;
+    std::shared_ptr<SaveGameUIData> m_ui_data;
     std::string                         m_save_state_string;
 
 private:
@@ -53,7 +55,8 @@ private:
 /** contains data that must be retained by the server when saving and loading a
   * game that isn't player data or the universe */
 struct ServerSaveGameData {
-    ServerSaveGameData();                               ///< default ctor
+    ServerSaveGameData();
+
     ServerSaveGameData(int current_turn);
 
     int                                     m_current_turn;
@@ -69,35 +72,68 @@ class ServerApp : public IApp {
 public:
     /** \name Structors */ //@{
     ServerApp();
+
     ~ServerApp();
     //@}
 
     /** \name Accessors */ //@{
-    int     CurrentTurn() const {return m_current_turn;}   ///< returns current turn of the server
 
-    /** Returns the galaxy setup data used for the current game */
-    const GalaxySetupData&  GetGalaxySetupData() const { return m_galaxy_setup_data; }
+    /** Returns a ClientApp pointer to the singleton instance of the app. */
+    static ServerApp* GetApp();
 
-    /** Returns the empire ID for the player with ID \a player_id */
-    int     PlayerEmpireID(int player_id) const;
+    Universe& GetUniverse() override;
 
-    /** Returns the player ID for the player controlling the empire with id \a empire_id */
-    int     EmpirePlayerID(int empire_id) const;
+    EmpireManager& Empires() override;
+
+    Empire* GetEmpire(int id) override;
+
+    SupplyManager& GetSupplyManager() override;
+
+    std::shared_ptr<UniverseObject> GetUniverseObject(int object_id) override;
+
+    /** Returns the server's map for known objects of specified empire. */
+    ObjectMap& EmpireKnownObjects(int empire_id) override;
+
+    std::shared_ptr<UniverseObject> EmpireKnownObject(int object_id, int empire_id) override;
+
+    std::string GetVisibleObjectName(std::shared_ptr<const UniverseObject> object) override;
+
+    int GetNewObjectID() override;
+
+    int GetNewDesignID() override;
+
+    int CurrentTurn() const override
+    { return m_current_turn; }
+
+    const GalaxySetupData&  GetGalaxySetupData() const override
+    { return m_galaxy_setup_data; }
 
     /** Checks if player with ID \a player_id is a human player
         who's client runs on the same machine as the server */
-    bool    IsLocalHumanPlayer(int player_id);
+    bool IsLocalHumanPlayer(int player_id);
 
-    Networking::ClientType  GetEmpireClientType(int empire_id) const;   ///< returns the networking client type for the given empire_id
-    Networking::ClientType  GetPlayerClientType(int player_id) const;   ///< returns the networking client type for the given player_id
+    /** Returns the networking client type for the given empire_id. */
+    Networking::ClientType GetEmpireClientType(int empire_id) const override;
 
-    virtual int             EffectsProcessingThreads() const;
+    /** Returns the networking client type for the given player_id. */
+    Networking::ClientType GetPlayerClientType(int player_id) const override;
+
+    int EffectsProcessingThreads() const override;
+
+    /** Returns the empire ID for the player with ID \a player_id */
+    int PlayerEmpireID(int player_id) const;
+
+    /** Returns the player ID for the player controlling the empire with id \a
+        empire_id */
+    int EmpirePlayerID(int empire_id) const;
+
+    /** Checks if \a player_name are not used by other players. */
+    bool IsAvailableName(const std::string& player_name) const;
     //@}
 
 
     /** \name Mutators */ //@{
     void    operator()();               ///< external interface to Run()
-    void    Exit(int code);             ///< does basic clean-up, then calls exit(); callable from anywhere in user code via GetApp()
 
     /** Returns the galaxy setup data used for the current game */
     GalaxySetupData&    GetGalaxySetupData() { return m_galaxy_setup_data; }
@@ -149,10 +185,13 @@ public:
       * are thereby victorious. */
     void    CheckForEmpireElimination();
 
-    /** Intializes single player game universe, sends out initial game state to
-      * clients, and signals clients to start first turn */
+    /** Intializes single player game universe.*/
     void    NewSPGameInit(const SinglePlayerSetupData& single_player_setup_data);
 
+    /** Return true if single player game AIs are compatible with created
+      * universe and are ready to start a new game. */
+    bool    VerifySPGameAIs(const SinglePlayerSetupData& single_player_setup_data);
+
     /** Intializes multi player game universe, sends out initial game state to
       * clients, and signals clients to start first turn */
     void    NewMPGameInit(const MultiplayerLobbyData& multiplayer_lobby_data);
@@ -160,55 +199,47 @@ public:
     /** Restores saved single player gamestate and human and AI client state
       * information. */
     void    LoadSPGameInit(const std::vector<PlayerSaveGameData>& player_save_game_data,
-                                       boost::shared_ptr<ServerSaveGameData> server_save_game_data);
+                           std::shared_ptr<ServerSaveGameData> server_save_game_data);
 
     /** Restores saved multiplayer gamestate and human and AI client state
       * information. */
     void    LoadMPGameInit(const MultiplayerLobbyData& lobby_data,
                            const std::vector<PlayerSaveGameData>& player_save_game_data,
-                           boost::shared_ptr<ServerSaveGameData> server_save_game_data);
+                           std::shared_ptr<ServerSaveGameData> server_save_game_data);
     //@}
 
     void UpdateSavePreviews(const Message& msg, PlayerConnectionPtr player_connection);
 
-    static ServerApp*           GetApp();         ///< returns a ClientApp pointer to the singleton instance of the app
-    Universe&                   GetUniverse();    ///< returns server's copy of Universe
-    EmpireManager&              Empires();        ///< returns the server's copy of the Empires
-    Empire*                     GetEmpire(int id);
-    SupplyManager&              GetSupplyManager();
-    TemporaryPtr<UniverseObject>GetUniverseObject(int object_id);
-    ObjectMap&                  EmpireKnownObjects(int empire_id); ///< returns the server's map for known objects of specified empire. */
-    TemporaryPtr<UniverseObject>EmpireKnownObject(int object_id, int empire_id);
+    /** Send the requested combat logs to the client.*/
+    void UpdateCombatLogs(const Message& msg, PlayerConnectionPtr player_connection);
 
     ServerNetworking&           Networking();     ///< returns the networking object for the server
 
-    std::string                 GetVisibleObjectName(TemporaryPtr<const UniverseObject> object);
-
-    /** returns a universe object ID which can be used for new objects.
-        Can return INVALID_OBJECT_ID if an ID cannot be created. */
-    int                         GetNewObjectID();
-
-    /** returns a design ID which can be used for a new design to uniquely identify it.
-        Can return INVALID_OBJECT_ID if an ID cannot be created. */
-    int                         GetNewDesignID();
-
 private:
     const ServerApp& operator=(const ServerApp&); // disabled
     ServerApp(const ServerApp&); // disabled
 
     void    Run();          ///< initializes app state, then executes main event handler/render loop (Poll())
 
+    /** Initialize the python engine or exit. */
+    void InitializePython();
+
     /** Called when server process receive termination signal */
     void    SignalHandler(const boost::system::error_code& error, int signal_number);
 
 
     /** Clears any old game stored orders, victors or eliminated players, ads
       * empires to turn processing list, does start-of-turn empire supply and
-      * resource pool determination, and compiles and sends out basic
-      * information about players in game for all other players as part of the
-      * game start messages sent to players. */
-    void    NewGameInit(const GalaxySetupData& galaxy_setup_data,
-                        const std::map<int, PlayerSetupData>& player_id_setup_data);
+      * resource pool determination. */
+    void    NewGameInitConcurrentWithJoiners(const GalaxySetupData& galaxy_setup_data,
+                                             const std::vector<PlayerSetupData>& player_setup_data);
+
+    /** Return true if player data is consistent with starting a new game. */
+    bool    NewGameInitVerifyJoiners(const GalaxySetupData& galaxy_setup_data,
+                                     const std::vector<PlayerSetupData>& player_setup_data);
+
+    /** Sends out initial new game state to clients, and signals clients to start first turn. */
+    void SendNewGameStartMessages();
 
     /** Clears any old game stored orders, victors or eliminated players, ads
       * empires to turn processing list, does start-of-turn empire supply and
@@ -217,8 +248,8 @@ private:
       * about all players to send out to all other players are part of game
       * start messages. */
     void    LoadGameInit(const std::vector<PlayerSaveGameData>& player_save_game_data,
-                         const std::vector<std::pair<int, int> >& player_id_to_save_game_data_index,
-                         boost::shared_ptr<ServerSaveGameData> server_save_game_data);
+                         const std::vector<std::pair<int, int>>& player_id_to_save_game_data_index,
+                         std::shared_ptr<ServerSaveGameData> server_save_game_data);
 
     /** Calls Python universe generator script.
       * Supposed to be called to create a new universe so it can be used by content
@@ -253,6 +284,9 @@ private:
     /** Called by ServerNetworking when a player's TCP connection is closed*/
     void    PlayerDisconnected(PlayerConnectionPtr player_connection);
 
+    /** Handle shutdown timeout by killing all ais. */
+    void ShutdownTimedoutHandler(boost::system::error_code error);
+
     /** Called when the host player has disconnected.  Select a new host player*/
     void    SelectNewHost();
 
@@ -296,6 +330,7 @@ private:
     friend struct WaitingForTurnEndIdle;
     friend struct WaitingForSaveData;
     friend struct ProcessingTurn;
+    friend struct ShuttingDownServer;
 };
 
 // template implementations
