@@ -1,10 +1,9 @@
 #ifndef _ServerFSM_h_
 #define _ServerFSM_h_
 
-#include "../network/Message.h"
-#include "../universe/TemporaryPtr.h"
 
-#include <boost/shared_ptr.hpp>
+#include "../network/Message.h"
+
 #include <boost/mpl/list.hpp>
 #include <boost/preprocessor/seq/for_each.hpp>
 #include <boost/statechart/custom_reaction.hpp>
@@ -13,9 +12,13 @@
 #include <boost/statechart/in_state_reaction.hpp>
 #include <boost/statechart/state.hpp>
 #include <boost/statechart/state_machine.hpp>
+#include <boost/asio/high_resolution_timer.hpp>
 
+#include <memory>
 #include <set>
 #include <vector>
+#include <unordered_set>
+#include <chrono>
 
 
 struct MultiplayerLobbyData;
@@ -25,7 +28,7 @@ class PlayerConnection;
 struct PlayerSaveGameData;
 struct ServerSaveGameData;
 class System;
-typedef boost::shared_ptr<PlayerConnection> PlayerConnectionPtr;
+typedef std::shared_ptr<PlayerConnection> PlayerConnectionPtr;
 
 namespace sc = boost::statechart;
 
@@ -37,14 +40,17 @@ struct Disconnection : sc::event<Disconnection> {
 
 struct LoadSaveFileFailed : sc::event<LoadSaveFileFailed>               {};
 struct CheckStartConditions : sc::event<CheckStartConditions>           {};
+struct CheckEndConditions : sc::event<CheckEndConditions>               {};
 struct CheckTurnEndConditions : sc::event<CheckTurnEndConditions>       {};
 struct ProcessTurn : sc::event<ProcessTurn>                             {};
+struct DisconnectClients : sc::event<DisconnectClients>                 {};
+struct ShutdownServer : sc::event<ShutdownServer>                       {};
 
 
 //  Message events
 /** The base class for all state machine events that are based on Messages. */
 struct MessageEventBase {
-    MessageEventBase(const Message& message, PlayerConnectionPtr& player_connection); ///< Basic ctor.
+    MessageEventBase(const Message& message, PlayerConnectionPtr& player_connection);
 
     Message              m_message;
     PlayerConnectionPtr  m_player_connection;
@@ -58,12 +64,14 @@ struct MessageEventBase {
     (LobbyUpdate)                           \
     (LobbyChat)                             \
     (JoinGame)                              \
+    (LeaveGame)                             \
     (SaveGameRequest)                       \
     (TurnOrders)                            \
     (CombatTurnOrders)                      \
     (ClientSaveData)                        \
     (RequestObjectID)                       \
     (RequestDesignID)                       \
+    (RequestCombatLogs)                     \
     (PlayerChat)                            \
     (Diplomacy)                             \
     (ModeratorAct)                          \
@@ -91,6 +99,7 @@ struct MPLobby;
 struct WaitingForSPGameJoiners;
 struct WaitingForMPGameJoiners;
 struct PlayingGame;
+struct ShuttingDownServer;
 
 // Substates of PlayingGame
 struct WaitingForTurnEnd;
@@ -111,10 +120,10 @@ struct ServerFSM : sc::state_machine<ServerFSM, Idle> {
     ServerApp& Server();
     void HandleNonLobbyDisconnection(const Disconnection& d);
 
-    boost::shared_ptr<MultiplayerLobbyData>     m_lobby_data;
-    boost::shared_ptr<SinglePlayerSetupData>    m_single_player_setup_data;
+    std::shared_ptr<MultiplayerLobbyData> m_lobby_data;
+    std::shared_ptr<SinglePlayerSetupData> m_single_player_setup_data;
     std::vector<PlayerSaveGameData>             m_player_save_game_data;
-    boost::shared_ptr<ServerSaveGameData>       m_server_save_game_data;
+    std::shared_ptr<ServerSaveGameData> m_server_save_game_data;
 
 private:
     ServerApp& m_server;
@@ -126,6 +135,7 @@ struct Idle : sc::state<Idle, ServerFSM> {
     typedef boost::mpl::list<
         sc::custom_reaction<HostMPGame>,
         sc::custom_reaction<HostSPGame>,
+        sc::custom_reaction<ShutdownServer>,
         sc::custom_reaction<Error>
     > reactions;
 
@@ -134,6 +144,7 @@ struct Idle : sc::state<Idle, ServerFSM> {
 
     sc::result react(const HostMPGame& msg);
     sc::result react(const HostSPGame& msg);
+    sc::result react(const ShutdownServer& u);
     sc::result react(const Error& msg);
 
     SERVER_ACCESSOR
@@ -150,6 +161,7 @@ struct MPLobby : sc::state<MPLobby, ServerFSM> {
         sc::custom_reaction<StartMPGame>,
         sc::custom_reaction<HostMPGame>,
         sc::custom_reaction<HostSPGame>,
+        sc::custom_reaction<ShutdownServer>,
         sc::custom_reaction<Error>
     > reactions;
 
@@ -163,11 +175,12 @@ struct MPLobby : sc::state<MPLobby, ServerFSM> {
     sc::result react(const StartMPGame& msg);
     sc::result react(const HostMPGame& msg);
     sc::result react(const HostSPGame& msg);
+    sc::result react(const ShutdownServer& u);
     sc::result react(const Error& msg);
 
-    boost::shared_ptr<MultiplayerLobbyData> m_lobby_data;
+    std::shared_ptr<MultiplayerLobbyData> m_lobby_data;
     std::vector<PlayerSaveGameData>         m_player_save_game_data;
-    boost::shared_ptr<ServerSaveGameData>   m_server_save_game_data;
+    std::shared_ptr<ServerSaveGameData> m_server_save_game_data;
 
     SERVER_ACCESSOR
 };
@@ -194,10 +207,10 @@ struct WaitingForSPGameJoiners : sc::state<WaitingForSPGameJoiners, ServerFSM> {
     sc::result react(const LoadSaveFileFailed& u);
     sc::result react(const Error& msg);
 
-    boost::shared_ptr<SinglePlayerSetupData> m_single_player_setup_data;
+    std::shared_ptr<SinglePlayerSetupData>   m_single_player_setup_data;
     std::vector<PlayerSaveGameData>          m_player_save_game_data;
-    boost::shared_ptr<ServerSaveGameData>    m_server_save_game_data;
-    std::set<std::string>                    m_expected_ai_player_names;
+    std::shared_ptr<ServerSaveGameData>      m_server_save_game_data;
+    std::map<std::string, int>               m_expected_ai_names_and_ids;
     int                                      m_num_expected_players;
 
     SERVER_ACCESSOR
@@ -224,9 +237,9 @@ struct WaitingForMPGameJoiners : sc::state<WaitingForMPGameJoiners, ServerFSM> {
     // as such, no file load error handling reaction is needed in this state.
     sc::result react(const Error& msg);
 
-    boost::shared_ptr<MultiplayerLobbyData> m_lobby_data;
+    std::shared_ptr<MultiplayerLobbyData> m_lobby_data;
     std::vector<PlayerSaveGameData>         m_player_save_game_data;
-    boost::shared_ptr<ServerSaveGameData>   m_server_save_game_data;
+    std::shared_ptr<ServerSaveGameData> m_server_save_game_data;
     std::set<std::string>                   m_expected_ai_player_names;
     int                                     m_num_expected_players;
 
@@ -242,6 +255,8 @@ struct PlayingGame : sc::state<PlayingGame, ServerFSM, WaitingForTurnEnd> {
         sc::custom_reaction<PlayerChat>,
         sc::custom_reaction<Diplomacy>,
         sc::custom_reaction<ModeratorAct>,
+        sc::custom_reaction<RequestCombatLogs>,
+        sc::custom_reaction<ShutdownServer>,
         sc::custom_reaction<Error>
     > reactions;
 
@@ -251,6 +266,8 @@ struct PlayingGame : sc::state<PlayingGame, ServerFSM, WaitingForTurnEnd> {
     sc::result react(const PlayerChat& msg);
     sc::result react(const Diplomacy& msg);
     sc::result react(const ModeratorAct& msg);
+    sc::result react(const ShutdownServer& u);
+    sc::result react(const RequestCombatLogs& msg);
     sc::result react(const Error& msg);
 
     SERVER_ACCESSOR
@@ -345,6 +362,32 @@ struct ProcessingTurn : sc::state<ProcessingTurn, PlayingGame> {
     SERVER_ACCESSOR
 };
 
+/** The server state in which the server is shutting down and waiting for AIs to exit. */
+struct ShuttingDownServer : sc::state<ShuttingDownServer, ServerFSM> {
+    using Clock = std::chrono::high_resolution_clock;
+    typedef boost::mpl::list<
+        sc::in_state_reaction<Disconnection>,
+        sc::custom_reaction<LeaveGame>,
+        sc::custom_reaction<CheckEndConditions>,
+        sc::custom_reaction<DisconnectClients>,
+        sc::custom_reaction<Error>
+    > reactions;
+
+    ShuttingDownServer(my_context c);
+    ~ShuttingDownServer();
+
+    sc::result react(const LeaveGame& msg);
+    sc::result react(const CheckEndConditions& u);
+    sc::result react(const DisconnectClients& u);
+    sc::result react(const Disconnection& d);
+    sc::result react(const Error& msg);
+
+    std::unordered_set<int>            m_player_id_ack_expected;
+    boost::asio::high_resolution_timer m_timeout;
+
+    SERVER_ACCESSOR
+};
+
 #undef SERVER_ACCESSOR
 
 #endif
