@@ -45,17 +45,17 @@ Building* Building::Clone(int empire_id) const {
     Visibility vis = GetUniverse().GetObjectVisibilityByEmpire(this->ID(), empire_id);
 
     if (!(vis >= VIS_BASIC_VISIBILITY && vis <= VIS_FULL_VISIBILITY))
-        return 0;
+        return nullptr;
 
     Building* retval = new Building();
-    retval->Copy(TemporaryFromThis(), empire_id);
+    retval->Copy(shared_from_this(), empire_id);
     return retval;
 }
 
-void Building::Copy(TemporaryPtr<const UniverseObject> copied_object, int empire_id) {
-    if (copied_object == this)
+void Building::Copy(std::shared_ptr<const UniverseObject> copied_object, int empire_id) {
+    if (copied_object.get() == this)
         return;
-    TemporaryPtr<const Building> copied_building = boost::dynamic_pointer_cast<const Building>(copied_object);
+    std::shared_ptr<const Building> copied_building = std::dynamic_pointer_cast<const Building>(copied_object);
     if (!copied_building) {
         ErrorLogger() << "Building::Copy passed an object that wasn't a Building";
         return;
@@ -114,8 +114,8 @@ std::string Building::Dump() const {
     return os.str();
 }
 
-TemporaryPtr<UniverseObject> Building::Accept(const UniverseObjectVisitor& visitor) const
-{ return visitor.Visit(boost::const_pointer_cast<Building>(boost::static_pointer_cast<const Building>(TemporaryFromThis()))); }
+std::shared_ptr<UniverseObject> Building::Accept(const UniverseObjectVisitor& visitor) const
+{ return visitor.Visit(std::const_pointer_cast<Building>(std::static_pointer_cast<const Building>(shared_from_this()))); }
 
 void Building::SetPlanetID(int planet_id) {
     if (planet_id != m_planet_id) {
@@ -159,14 +159,12 @@ void BuildingType::Init() {
         m_location->SetTopLevelContent(m_name);
     if (m_enqueue_location)
         m_enqueue_location->SetTopLevelContent(m_name);
-    for (std::vector<boost::shared_ptr<Effect::EffectsGroup> >::iterator it = m_effects.begin();
-         it != m_effects.end(); ++it)
-    { (*it)->SetTopLevelContent(m_name); }
+    for (std::shared_ptr<Effect::EffectsGroup> effect : m_effects) {
+        effect->SetTopLevelContent(m_name);
+    }
 }
 
 std::string BuildingType::Dump() const {
-    using boost::lexical_cast;
-
     std::string retval = DumpIndent() + "BuildingType\n";
     ++g_indent;
     retval += DumpIndent() + "name = \"" + m_name + "\"\n";
@@ -176,16 +174,17 @@ std::string BuildingType::Dump() const {
     if (m_production_time)
         retval += DumpIndent() + "buildtime = " + m_production_time->Dump() + "\n";
     retval += DumpIndent() + (m_producible ? "Producible" : "Unproducible") + "\n";
-    retval += DumpIndent() + "captureresult = " + lexical_cast<std::string>(m_capture_result) + "\n";
+    retval += DumpIndent() + "captureresult = " +
+        boost::lexical_cast<std::string>(m_capture_result) + "\n";
 
     if (!m_tags.empty()) {
         if (m_tags.size() == 1) {
             retval += DumpIndent() + "tags = \"" + *m_tags.begin() + "\"\n";
         } else {
             retval += DumpIndent() + "tags = [ ";
-            for (std::set<std::string>::const_iterator tag_it = m_tags.begin();
-                 tag_it != m_tags.end(); ++tag_it)
-            { retval += "\"" + *tag_it + "\" "; }
+            for (const std::string& tag : m_tags) {
+               retval += "\"" + tag + "\" ";
+            }
             retval += " ]\n";
         }
     }
@@ -211,8 +210,8 @@ std::string BuildingType::Dump() const {
     } else {
         retval += DumpIndent() + "effectsgroups = [\n";
         ++g_indent;
-        for (unsigned int i = 0; i < m_effects.size(); ++i) {
-            retval += m_effects[i]->Dump();
+        for (std::shared_ptr<Effect::EffectsGroup> effect : m_effects) {
+            retval += effect->Dump();
         }
         --g_indent;
         retval += DumpIndent() + "]\n";
@@ -222,31 +221,6 @@ std::string BuildingType::Dump() const {
     return retval;
 }
 
-namespace {
-    TemporaryPtr<const UniverseObject> SourceForEmpire(int empire_id) {
-        const Empire* empire = GetEmpire(empire_id);
-        if (!empire) {
-            DebugLogger() << "SourceForEmpire: Unable to get empire with ID: " << empire_id;
-            return TemporaryPtr<const UniverseObject>();
-        }
-        // get a source object, which is owned by the empire with the passed-in
-        // empire id.  this is used in conditions to reference which empire is
-        // doing the building.  Ideally this will be the capital, but any object
-        // owned by the empire will work.
-        TemporaryPtr<const UniverseObject> source = GetUniverseObject(empire->CapitalID());
-        // no capital?  scan through all objects to find one owned by this empire
-        if (!source) {
-            for (ObjectMap::const_iterator<> obj_it = Objects().const_begin(); obj_it != Objects().const_end(); ++obj_it) {
-                if (obj_it->OwnedBy(empire_id)) {
-                    source = *obj_it;
-                    break;
-                }
-            }
-        }
-        return source;
-    }
-}
-
 bool BuildingType::ProductionCostTimeLocationInvariant() const {
     if (CHEAP_AND_FAST_BUILDING_PRODUCTION)
         return true;
@@ -259,18 +233,20 @@ bool BuildingType::ProductionCostTimeLocationInvariant() const {
 
 float BuildingType::ProductionCost(int empire_id, int location_id) const {
     if (CHEAP_AND_FAST_BUILDING_PRODUCTION || !m_production_cost) {
-        return 1.0;
+        return 1.0f;
     } else {
-        if (ValueRef::ConstantExpr(m_production_cost))
+        if (m_production_cost && m_production_cost->ConstantExpr())
             return m_production_cost->Eval();
 
-        TemporaryPtr<UniverseObject> location = GetUniverseObject(location_id);
-        if (!location)
-            return 999999.9f;    // arbitrary large number
+        const auto arbitrary_large_number = 999999.9f;
 
-        TemporaryPtr<const UniverseObject> source = SourceForEmpire(empire_id);
+        std::shared_ptr<UniverseObject> location = GetUniverseObject(location_id);
+        if (!location)
+            return arbitrary_large_number;
+
+        std::shared_ptr<const UniverseObject> source = Empires().GetSource(empire_id);
         if (!source && !m_production_cost->SourceInvariant())
-            return 999999.9f;
+            return arbitrary_large_number;
 
         ScriptingContext context(source, location);
 
@@ -285,16 +261,18 @@ int BuildingType::ProductionTime(int empire_id, int location_id) const {
     if (CHEAP_AND_FAST_BUILDING_PRODUCTION || !m_production_time) {
         return 1;
     } else {
-        if (ValueRef::ConstantExpr(m_production_time))
+        if (m_production_time && m_production_time->ConstantExpr())
             return m_production_time->Eval();
 
-        TemporaryPtr<UniverseObject> location = GetUniverseObject(location_id);
-        if (!location)
-            return 9999;    // arbitrary large number
+        const auto arbitrary_large_number = 9999;
 
-        TemporaryPtr<const UniverseObject> source = SourceForEmpire(empire_id);
+        std::shared_ptr<UniverseObject> location = GetUniverseObject(location_id);
+        if (!location)
+            return arbitrary_large_number;
+
+        std::shared_ptr<const UniverseObject> source = Empires().GetSource(empire_id);
         if (!source && !m_production_time->SourceInvariant())
-            return 9999;
+            return arbitrary_large_number;
 
         ScriptingContext context(source, location);
 
@@ -306,11 +284,11 @@ bool BuildingType::ProductionLocation(int empire_id, int location_id) const {
     if (!m_location)
         return true;
 
-    TemporaryPtr<const UniverseObject> location = GetUniverseObject(location_id);
+    std::shared_ptr<const UniverseObject> location = GetUniverseObject(location_id);
     if (!location)
         return false;
 
-    TemporaryPtr<const UniverseObject> source = SourceForEmpire(empire_id);
+    std::shared_ptr<const UniverseObject> source = Empires().GetSource(empire_id);
     if (!source)
         return false;
 
@@ -321,11 +299,11 @@ bool BuildingType::EnqueueLocation(int empire_id, int location_id) const {
     if (!m_enqueue_location)
         return ProductionLocation(empire_id, location_id);
 
-    TemporaryPtr<const UniverseObject> location = GetUniverseObject(location_id);
+    std::shared_ptr<const UniverseObject> location = GetUniverseObject(location_id);
     if (!location)
         return false;
 
-    TemporaryPtr<const UniverseObject> source = SourceForEmpire(empire_id);
+    std::shared_ptr<const UniverseObject> source = Empires().GetSource(empire_id);
     if (!source)
         return false;
 
@@ -336,7 +314,7 @@ bool BuildingType::EnqueueLocation(int empire_id, int location_id) const {
 // BuildingTypeManager                         //
 /////////////////////////////////////////////////
 // static(s)
-BuildingTypeManager* BuildingTypeManager::s_instance = 0;
+BuildingTypeManager* BuildingTypeManager::s_instance = nullptr;
 
 BuildingTypeManager::BuildingTypeManager() {
     if (s_instance)
@@ -357,21 +335,21 @@ BuildingTypeManager::BuildingTypeManager() {
 
     if (GetOptionsDB().Get<bool>("verbose-logging")) {
         DebugLogger() << "Building Types:";
-        for (iterator it = begin(); it != end(); ++it) {
-            DebugLogger() << " ... " << it->first;
+        for (const std::map<const std::string, BuildingType*>::value_type& entry : m_building_types) {
+            DebugLogger() << " ... " << entry.first;
         }
     }
 }
 
 BuildingTypeManager::~BuildingTypeManager() {
-    for (std::map<std::string, BuildingType*>::iterator it = m_building_types.begin();
-         it != m_building_types.end(); ++it)
-    { delete it->second; }
+    for (std::map<std::string, BuildingType*>::value_type& entry : m_building_types) {
+        delete entry.second;
+    }
 }
 
 const BuildingType* BuildingTypeManager::GetBuildingType(const std::string& name) const {
     std::map<std::string, BuildingType*>::const_iterator it = m_building_types.find(name);
-    return it != m_building_types.end() ? it->second : 0;
+    return it != m_building_types.end() ? it->second : nullptr;
 }
 
 BuildingTypeManager& BuildingTypeManager::GetBuildingTypeManager() {
