@@ -21,36 +21,41 @@ namespace Condition {
 class FO_COMMON_API Building : public UniverseObject {
 public:
     /** \name Accessors */ //@{
-    virtual std::set<std::string>
-                                Tags() const;                                       ///< returns all tags this object has
-    virtual bool                HasTag(const std::string& name) const;              ///< returns true iff this object has the tag with the indicated \a name
+    std::set<std::string> Tags() const override;
 
-    virtual UniverseObjectType  ObjectType() const;
-    virtual std::string         Dump() const;
+    bool HasTag(const std::string& name) const override;
 
-    const std::string&      BuildingTypeName() const    { return m_building_type; };        ///< returns the name of the BuildingType object for this building
+    UniverseObjectType ObjectType() const override;
+
+    std::string Dump() const override;
+
+    int ContainerObjectID() const override
+    { return m_planet_id; }
+
+    bool ContainedBy(int object_id) const override;
+
+    std::shared_ptr<UniverseObject> Accept(const UniverseObjectVisitor& visitor) const override;
+
+    /** Returns the name of the BuildingType object for this building. */
+    const std::string& BuildingTypeName() const
+    { return m_building_type; };
 
-    virtual int             ContainerObjectID() const   { return m_planet_id; }             ///< returns id of the object that directly contains this object, if any, or INVALID_OBJECT_ID if this object is not contained by any other
-    virtual bool            ContainedBy(int object_id) const;                               ///< returns true if there is an object with id \a object_id that contains this UniverseObject
     int                     PlanetID() const            { return m_planet_id; }             ///< returns the ID number of the planet this building is on
 
     int                     ProducedByEmpireID() const  { return m_produced_by_empire_id; } ///< returns the empire ID of the empire that produced this building
 
-    virtual TemporaryPtr<UniverseObject>
-                            Accept(const UniverseObjectVisitor& visitor) const;
-
     bool                    OrderedScrapped() const     { return m_ordered_scrapped; }
     //@}
 
     /** \name Mutators */ //@{
-    virtual void    Copy(TemporaryPtr<const UniverseObject> copied_object, int empire_id = ALL_EMPIRES);
+    void Copy(std::shared_ptr<const UniverseObject> copied_object, int empire_id = ALL_EMPIRES) override;
 
     void            SetPlanetID(int planet_id);         ///< sets the planet on which the building is located
 
     void            Reset();                            ///< resets any building state, and removes owners
     void            SetOrderedScrapped(bool b = true);  ///< flags building for scrapping
 
-    virtual void    ResetTargetMaxUnpairedMeters();
+    void ResetTargetMaxUnpairedMeters() override;
     //@}
 
 protected:
@@ -63,21 +68,19 @@ protected:
         m_ordered_scrapped(false),
         m_produced_by_empire_id(ALL_EMPIRES)
     {}
+
     Building(int empire_id, const std::string& building_type,
-             int produced_by_empire_id = ALL_EMPIRES);                  ///< basic ctor
+             int produced_by_empire_id = ALL_EMPIRES);
 
+    template <typename T> friend void UniverseObjectDeleter(T*);
     template <class T> friend void boost::python::detail::value_destroyer<false>::execute(T const volatile* p);
-    template <class T> friend void boost::checked_delete(T* x);
 
-#if BOOST_VERSION == 106100
 public:
-#endif
     ~Building() {}
-#if BOOST_VERSION == 106100
-protected:
-#endif
 
-    virtual Building*       Clone(int empire_id = ALL_EMPIRES) const;   ///< returns new copy of this Building
+protected:
+    /** Returns new copy of this Building. */
+    Building* Clone(int empire_id = ALL_EMPIRES) const override;
     //@}
 
 private:
@@ -117,8 +120,8 @@ public:
         m_icon(icon)
     {
         Init();
-        for (std::set< std::string >::iterator tag_it = common_params.tags.begin(); tag_it != common_params.tags.end(); ++tag_it)
-            m_tags.insert(boost::to_upper_copy<std::string>(*tag_it));
+        for (const std::string& tag : common_params.tags)
+            m_tags.insert(boost::to_upper_copy<std::string>(tag));
     }
 
     ~BuildingType();
@@ -139,17 +142,20 @@ public:
 
     bool                            Producible() const      { return m_producible; }        ///< returns whether this building type is producible by players and appears on the production screen
 
-    const std::map<MeterType, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >&
+    const std::map<MeterType, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*>>&
                                     ProductionMeterConsumption() const  { return m_production_meter_consumption; }
-    const std::map<std::string, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >&
+    const std::map<std::string, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*>>&
                                     ProductionSpecialConsumption() const{ return m_production_special_consumption; }
 
     const std::set<std::string>&    Tags() const            { return m_tags; }
     const Condition::ConditionBase* Location() const        { return m_location; }          ///< returns the condition that determines the locations where this building can be produced
     const Condition::ConditionBase* EnqueueLocation() const { return m_enqueue_location; }  ///< returns the condition that determines the locations where this building can be enqueued (ie. put onto the production queue)
 
-    const std::vector<boost::shared_ptr<Effect::EffectsGroup> >&
-                                    Effects() const         { return m_effects; }           ///< returns the EffectsGroups that encapsulate the effects that buildings of this type have when operational
+    /** Returns the EffectsGroups that encapsulate the effects that buildings ofi
+        this type have when operational. */
+    const std::vector<std::shared_ptr<Effect::EffectsGroup>>& Effects() const
+    { return m_effects; }
+
     const std::string&              Icon() const            { return m_icon; }              ///< returns the name of the grapic file for this building type
 
     bool ProductionLocation(int empire_id, int location_id) const;  ///< returns true iff the empire with ID empire_id can produce this building at the location with location_id
@@ -175,13 +181,13 @@ private:
     bool                                                    m_producible;
     CaptureResult                                           m_capture_result;
     std::set<std::string>                                   m_tags;
-    std::map<MeterType, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >
+    std::map<MeterType, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*>>
                                                             m_production_meter_consumption;
-    std::map<std::string, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >
+    std::map<std::string, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*>>
                                                             m_production_special_consumption;
     Condition::ConditionBase*                               m_location;
     Condition::ConditionBase*                               m_enqueue_location;
-    std::vector<boost::shared_ptr<Effect::EffectsGroup> >   m_effects;
+    std::vector<std::shared_ptr<Effect::EffectsGroup>> m_effects;
     std::string                                             m_icon;
 };
 
