@@ -1,20 +1,20 @@
 #ifndef _Effect_Accounting_h_
 #define _Effect_Accounting_h_
 
-#include "Enums.h"
 
-#include "TemporaryPtr.h"
-#include <boost/shared_ptr.hpp>
+#include "EnumsFwd.h"
 
+#include <memory>
 #include <map>
 #include <string>
 #include <vector>
 
+
 class UniverseObject;
 
 namespace Effect {
     class EffectsGroup;
-    typedef std::vector<TemporaryPtr<UniverseObject> > TargetSet;
+    typedef std::vector<std::shared_ptr<UniverseObject>> TargetSet;
 
     /** Description of cause of an effect: the general cause type, and the
       * specific cause.  eg. Building and a particular BuildingType. */
@@ -30,7 +30,8 @@ namespace Effect {
     /** Accounting information about what the causes are and changes produced
       * by effects groups acting on meters of objects. */
     struct AccountingInfo : public EffectCause {
-        AccountingInfo();           ///< default ctor
+        AccountingInfo();
+
         int     source_id;          ///< source object of effect
         float   meter_change;       ///< net change on meter due to this effect, as best known by client's empire
         float   running_meter_total;///< meter total as of this effect.
@@ -38,7 +39,7 @@ namespace Effect {
 
     /** Effect accounting information for all meters of all objects that are
       * acted on by effects. */
-    typedef std::map<int, std::map<MeterType, std::vector<AccountingInfo> > > AccountingMap;
+    typedef std::map<int, std::map<MeterType, std::vector<AccountingInfo>>> AccountingMap;
 
     /** Combination of targets and cause for an effects group. */
     struct TargetsAndCause {
@@ -51,12 +52,12 @@ namespace Effect {
     /** Combination of an EffectsGroup and the id of a source object. */
     struct SourcedEffectsGroup {
         SourcedEffectsGroup();
-        SourcedEffectsGroup(int source_object_id_, const boost::shared_ptr<EffectsGroup>& effects_group_);
+        SourcedEffectsGroup(int source_object_id_, const std::shared_ptr<EffectsGroup>& effects_group_);
         bool    operator<(const SourcedEffectsGroup& right) const;
         void    Execute(const TargetSet& targets) const;
         void    Execute(const TargetsAndCause& targets_and_cause, AccountingMap& accounting_map) const;
         int                             source_object_id;
-        boost::shared_ptr<EffectsGroup> effects_group;
+        std::shared_ptr<EffectsGroup> effects_group;
     };
 
     /** Discrepancy between meter's value at start of turn, and the value that
@@ -64,7 +65,7 @@ namespace Effect {
       * available -> the unknown factor affecting the meter.  This is used
       * when generating effect accounting, in the case where the expected
       * and actual meter values don't match. */
-    typedef std::map<int, std::map<MeterType, double> > DiscrepancyMap;
+    typedef std::map<int, std::map<MeterType, double>> DiscrepancyMap;
 
     /** Map from (effects group and source object) to target set of for
       * that effects group with that source object.  A multimap is used
@@ -72,7 +73,7 @@ namespace Effect {
       * same effectsgroup.  This is useful when a Ship has multiple copies
       * of the same effects group due to having multiple copies of the same
       * ship part in its design. */
-    typedef std::vector<std::pair<SourcedEffectsGroup, TargetsAndCause> > TargetsCauses;
+    typedef std::vector<std::pair<SourcedEffectsGroup, TargetsAndCause>> TargetsCauses;
 }
 
 #endif
