@@ -8,6 +8,13 @@
 #include "../util/Export.h"
 
 struct FO_COMMON_API EncyclopediaArticle {
+    EncyclopediaArticle() :
+        name(""),
+        category(""),
+        short_description(""),
+        description(""),
+        icon("")
+    {}
     EncyclopediaArticle(const std::string& name_, const std::string& category_,
                         const std::string& short_description_, const std::string& description_,
                         const std::string& icon_) :
@@ -26,7 +33,8 @@ struct FO_COMMON_API EncyclopediaArticle {
 
 struct FO_COMMON_API Encyclopedia {
     Encyclopedia();
-    std::map<std::string, std::vector<EncyclopediaArticle> >   articles;
+    std::map<std::string, std::vector<EncyclopediaArticle>> articles;
+    const EncyclopediaArticle                               empty_article;
 };
 
 FO_COMMON_API const Encyclopedia& GetEncyclopedia();
