@@ -184,6 +184,7 @@ GG_ENUM(DiplomaticStatus,
     INVALID_DIPLOMATIC_STATUS = -1,
     DIPLO_WAR,
     DIPLO_PEACE,
+    DIPLO_ALLIED,
     NUM_DIPLO_STATUSES
 )
 
@@ -247,7 +248,7 @@ GG_ENUM(ShipPartClass,
     PC_INDUSTRY,            ///< generates production points for owner at its location
     PC_RESEARCH,            ///< generates research points for owner
     PC_TRADE,               ///< generates trade points for owner
-    PC_PRODICTION_LOCATION, ///< allows production items to be produced at its location
+    PC_PRODUCTION_LOCATION, ///< allows production items to be produced at its location
     NUM_SHIP_PART_CLASSES
 )
 
@@ -276,9 +277,6 @@ FO_COMMON_API ResourceType MeterToResource(MeterType type);
 FO_COMMON_API MeterType AssociatedMeterType(MeterType meter_type);
 
 
-extern const int ALL_EMPIRES;
-
-
 /** degrees of visibility an Empire or UniverseObject can have for an
   * UniverseObject.  determines how much information the empire
   * gets about the (non)visible object. */
