@@ -16,12 +16,12 @@
 #include <boost/filesystem/fstream.hpp>
 
 namespace {
-    boost::shared_ptr<Effect::EffectsGroup>
+    std::shared_ptr<Effect::EffectsGroup>
     IncreaseMeter(MeterType meter_type, double increase) {
-        typedef boost::shared_ptr<Effect::EffectsGroup> EffectsGroupPtr;
+        typedef std::shared_ptr<Effect::EffectsGroup> EffectsGroupPtr;
         typedef std::vector<Effect::EffectBase*> Effects;
         Condition::Source* scope = new Condition::Source;
-        Condition::Source* activation = 0;
+        Condition::Source* activation = nullptr;
         ValueRef::ValueRefBase<double>* vr =
             new ValueRef::Operation<double>(
                 ValueRef::PLUS,
@@ -42,6 +42,9 @@ Field::Field() :
     m_type_name("")
 {}
 
+Field::~Field()
+{}
+
 Field::Field(const std::string& field_type, double x, double y, double radius) :
     UniverseObject("", x, y),
     m_type_name(field_type)
@@ -64,17 +67,17 @@ Field* Field::Clone(int empire_id) const {
     Visibility vis = GetUniverse().GetObjectVisibilityByEmpire(this->ID(), empire_id);
 
     if (!(vis >= VIS_BASIC_VISIBILITY && vis <= VIS_FULL_VISIBILITY))
-        return 0;
+        return nullptr;
 
     Field* retval = new Field();
-    retval->Copy(TemporaryFromThis(), empire_id);
+    retval->Copy(shared_from_this(), empire_id);
     return retval;
 }
 
-void Field::Copy(TemporaryPtr<const UniverseObject> copied_object, int empire_id) {
-    if (copied_object == this)
+void Field::Copy(std::shared_ptr<const UniverseObject> copied_object, int empire_id) {
+    if (copied_object.get() == this)
         return;
-    TemporaryPtr<const Field> copied_field = boost::dynamic_pointer_cast<const Field>(copied_object);
+    std::shared_ptr<const Field> copied_field = std::dynamic_pointer_cast<const Field>(copied_object);
     if (!copied_field) {
         ErrorLogger() << "Field::Copy passed an object that wasn't a Field";
         return;
@@ -120,8 +123,8 @@ const std::string& Field::PublicName(int empire_id) const {
     return UserString(m_type_name);
 }
 
-TemporaryPtr<UniverseObject> Field::Accept(const UniverseObjectVisitor& visitor) const
-{ return visitor.Visit(boost::const_pointer_cast<Field>(boost::static_pointer_cast<const Field>(TemporaryFromThis()))); }
+std::shared_ptr<UniverseObject> Field::Accept(const UniverseObjectVisitor& visitor) const
+{ return visitor.Visit(std::const_pointer_cast<Field>(std::static_pointer_cast<const Field>(shared_from_this()))); }
 
 int Field::ContainerObjectID() const
 { return this->SystemID(); }
@@ -131,7 +134,7 @@ bool Field::ContainedBy(int object_id) const {
         && object_id == this->SystemID();
 }
 
-bool Field::InField(TemporaryPtr<const UniverseObject> obj) const
+bool Field::InField(std::shared_ptr<const UniverseObject> obj) const
 { return obj && InField(obj->X(), obj->Y()); }
 
 bool Field::InField(double x, double y) const {
@@ -163,7 +166,7 @@ void Field::ClampMeters() {
 /////////////////////////////////////////////////
 FieldType::FieldType(const std::string& name, const std::string& description,
                      float stealth, const std::set<std::string>& tags,
-                     const std::vector<boost::shared_ptr<Effect::EffectsGroup> >& effects,
+                     const std::vector<std::shared_ptr<Effect::EffectsGroup>>& effects,
                      const std::string& graphic) :
     m_name(name),
     m_description(description),
@@ -172,23 +175,21 @@ FieldType::FieldType(const std::string& name, const std::string& description,
     m_effects(effects),
     m_graphic(graphic)
 {
-    for (std::set< std::string >::iterator tag_it = tags.begin(); tag_it != tags.end(); ++tag_it)
-        m_tags.insert(boost::to_upper_copy<std::string>(*tag_it));
+    for (const std::string& tag : tags)
+        m_tags.insert(boost::to_upper_copy<std::string>(tag));
 
     if (m_stealth != 0.0f)
         m_effects.push_back(IncreaseMeter(METER_STEALTH,    m_stealth));
 
-    for (std::vector<boost::shared_ptr<Effect::EffectsGroup> >::iterator it = m_effects.begin();
-         it != m_effects.end(); ++it)
-    { (*it)->SetTopLevelContent(m_name); }
+    for (std::shared_ptr<Effect::EffectsGroup> effect : m_effects) {
+        effect->SetTopLevelContent(m_name);
+    }
 }
 
 FieldType::~FieldType()
 {}
 
 std::string FieldType::Dump() const {
-    using boost::lexical_cast;
-
     std::string retval = DumpIndent() + "FieldType\n";
     ++g_indent;
     retval += DumpIndent() + "name = \"" + m_name + "\"\n";
@@ -205,8 +206,8 @@ std::string FieldType::Dump() const {
     } else {
         retval += DumpIndent() + "effectsgroups = [\n";
         ++g_indent;
-        for (unsigned int i = 0; i < m_effects.size(); ++i) {
-            retval += m_effects[i]->Dump();
+        for (std::shared_ptr<Effect::EffectsGroup> effect : m_effects) {
+            retval += effect->Dump();
         }
         --g_indent;
         retval += DumpIndent() + "]\n";
@@ -221,7 +222,7 @@ std::string FieldType::Dump() const {
 // FieldTypeManager                         //
 /////////////////////////////////////////////////
 // static(s)
-FieldTypeManager* FieldTypeManager::s_instance = 0;
+FieldTypeManager* FieldTypeManager::s_instance = nullptr;
 
 FieldTypeManager::FieldTypeManager() {
     if (s_instance)
@@ -238,21 +239,21 @@ FieldTypeManager::FieldTypeManager() {
 
     if (GetOptionsDB().Get<bool>("verbose-logging")) {
         DebugLogger() << "Field Types:";
-        for (iterator it = begin(); it != end(); ++it) {
-            DebugLogger() << " ... " << it->first;
+        for (const std::map<std::string, FieldType*>::value_type& entry : *this) {
+            DebugLogger() << " ... " << entry.first;
         }
     }
 }
 
 FieldTypeManager::~FieldTypeManager() {
-    for (std::map<std::string, FieldType*>::iterator it = m_field_types.begin();
-         it != m_field_types.end(); ++it)
-    { delete it->second; }
+    for (const std::map<std::string, FieldType*>::value_type& entry : m_field_types) {
+        delete entry.second;
+    }
 }
 
 const FieldType* FieldTypeManager::GetFieldType(const std::string& name) const {
     std::map<std::string, FieldType*>::const_iterator it = m_field_types.find(name);
-    return it != m_field_types.end() ? it->second : 0;
+    return it != m_field_types.end() ? it->second : nullptr;
 }
 
 FieldTypeManager& FieldTypeManager::GetFieldTypeManager() {
