@@ -13,55 +13,56 @@ namespace Effect {
 class FO_COMMON_API Field : public UniverseObject {
 public:
     /** \name Accessors */ //@{
-    virtual std::set<std::string>
-                                Tags() const;                                       ///< returns all tags this object has
-    virtual bool                HasTag(const std::string& name) const;              ///< returns true iff this object has the tag with the indicated \a name
+    std::set<std::string> Tags() const override;
 
-    virtual UniverseObjectType  ObjectType() const;
-    virtual std::string         Dump() const;
+    bool HasTag(const std::string& name) const override;
+
+    UniverseObjectType ObjectType() const override;
+
+    std::string Dump() const override;
+
+    int ContainerObjectID() const override;
+
+    bool ContainedBy(int object_id) const override;
+
+    std::shared_ptr<UniverseObject> Accept(const UniverseObjectVisitor& visitor) const override;
+
+    const std::string& PublicName(int empire_id) const override;
+
+    void ClampMeters() override;
 
     const std::string&          FieldTypeName() const { return m_type_name; }
-    virtual TemporaryPtr<UniverseObject>
-                                Accept(const UniverseObjectVisitor& visitor) const;
 
-    virtual int                 ContainerObjectID() const;                          ///< returns id of the object that directly contains this object, if any, or INVALID_OBJECT_ID if this object is not contained by any other
-    virtual bool                ContainedBy(int object_id) const;                   ///< returns true if there is an object with id \a object_id that contains this UniverseObject
+    bool InField(std::shared_ptr<const UniverseObject> obj) const;
 
-    bool                        InField(TemporaryPtr<const UniverseObject> obj) const;
     bool                        InField(double x, double y) const;
-
-    virtual const std::string&  PublicName(int empire_id) const;
     //@}
 
     /** \name Mutators */ //@{
-    virtual void                Copy(TemporaryPtr<const UniverseObject> copied_object, int empire_id = ALL_EMPIRES);
+    void Copy(std::shared_ptr<const UniverseObject> copied_object, int empire_id = ALL_EMPIRES) override;
 
-    virtual void                ResetTargetMaxUnpairedMeters();
+    void ResetTargetMaxUnpairedMeters() override;
     //@}
 
 protected:
     friend class Universe;
     /** \name Structors */ //@{
-    Field();                                        ///< default ctor
+    Field();
+
     Field(const std::string& field_type, double x, double y, double radius);
-    
+
+    template <typename T> friend void UniverseObjectDeleter(T*);
     template <class T> friend void boost::python::detail::value_destroyer<false>::execute(T const volatile* p);
-    template <class T> friend void boost::checked_delete(T* x);
 
-#if BOOST_VERSION == 106100
 public:
-#endif
-    ~Field() {}
-#if BOOST_VERSION == 106100
-protected:
-#endif
+    ~Field();
 
-    virtual Field*              Clone(int empire_id = ALL_EMPIRES) const;   ///< returns new copy of this Field
+protected:
+    /** Returns new copy of this Field. */
+    Field* Clone(int empire_id = ALL_EMPIRES) const override;
     //@}
 
 private:
-    virtual void                ClampMeters();
-
     std::string     m_type_name;
 
     friend class boost::serialization::access;
@@ -76,7 +77,7 @@ public:
     /** \name Structors */ //@{
     FieldType(const std::string& name, const std::string& description,
               float stealth, const std::set<std::string>& tags,
-              const std::vector<boost::shared_ptr<Effect::EffectsGroup> >& effects,
+              const std::vector<std::shared_ptr<Effect::EffectsGroup>>& effects,
               const std::string& graphic);
     ~FieldType();
     //@}
@@ -87,8 +88,12 @@ public:
     std::string                     Dump() const;                                       ///< returns a data file format representation of this object
     float                           Stealth() const         { return m_stealth; }       ///< returns stealth of field type
     const std::set<std::string>&    Tags() const            { return m_tags; }
-    const std::vector<boost::shared_ptr<Effect::EffectsGroup> >&
-                                    Effects() const         { return m_effects; }       ///< returns the EffectsGroups that encapsulate the effects of this FieldType
+
+    /** Returns the EffectsGroups that encapsulate the effects of thisi
+        FieldType. */
+    const std::vector<std::shared_ptr<Effect::EffectsGroup>>& Effects() const
+    { return m_effects; }
+
     const std::string&              Graphic() const         { return m_graphic; }       ///< returns the name of the grapic file for this field type
     //@}
 
@@ -97,7 +102,7 @@ private:
     std::string                                             m_description;
     float                                                   m_stealth;
     std::set<std::string>                                   m_tags;
-    std::vector<boost::shared_ptr<Effect::EffectsGroup> >   m_effects;
+    std::vector<std::shared_ptr<Effect::EffectsGroup>> m_effects;
     std::string                                             m_graphic;
 };
 
