@@ -13,21 +13,24 @@ class FO_COMMON_API Fighter : public UniverseObject {
 public:
     Fighter(int empire_id, int launched_from_id, const std::string& species_name, float damage);
     Fighter();
-    ~Fighter() {}
+    ~Fighter();
+
+    UniverseObjectType ObjectType() const override;
+
+    std::string Dump() const override;
+
+    std::shared_ptr<UniverseObject> Accept(const UniverseObjectVisitor& visitor) const override;
+
+    void Copy(std::shared_ptr<const UniverseObject> copied_object, int empire_id = ALL_EMPIRES) override;
 
-    virtual UniverseObjectType  ObjectType() const;
-    virtual std::string         Dump() const;
     float                       Damage() const;
     bool                        Destroyed() const;
     int                         LaunchedFrom() const;
     const std::string&          SpeciesName() const;
     void                        SetDestroyed(bool destroyed = true);
 
-    virtual TemporaryPtr<UniverseObject>    Accept(const UniverseObjectVisitor& visitor) const;
-    virtual void                            Copy(TemporaryPtr<const UniverseObject> copied_object, int empire_id = ALL_EMPIRES);
-
 protected:
-    virtual Fighter*    Clone(int empire_id = ALL_EMPIRES) const;
+    Fighter* Clone(int empire_id = ALL_EMPIRES) const override;
 
 private:
     float       m_damage;           // strength of fighter's attack
