@@ -30,18 +30,24 @@ struct MovePathNode {
 class FO_COMMON_API Fleet : public UniverseObject {
 public:
     /** \name Accessors */ //@{
-    virtual UniverseObjectType          ObjectType() const;
-    virtual std::string                 Dump() const;
+    UniverseObjectType ObjectType() const override;
+
+    std::string Dump() const override;
+
+    int ContainerObjectID() const override;
+
+    const std::set<int>& ContainedObjectIDs() const override;
+
+    bool Contains(int object_id) const override;
+
+    bool ContainedBy(int object_id) const override;
+
+    const std::string& PublicName(int empire_id) const override;
+
+    std::shared_ptr<UniverseObject>Accept(const UniverseObjectVisitor& visitor) const override;
 
     const std::set<int>&                ShipIDs() const     { return m_ships; }         ///< returns set of IDs of ships in fleet.
 
-    virtual int                         ContainerObjectID() const;          ///< returns id of the object that directly contains this object, if any, or INVALID_OBJECT_ID if this object is not contained by any other
-    virtual const std::set<int>&        ContainedObjectIDs() const;         ///< returns ids of objects directly contained within this object
-    virtual bool                        Contains(int object_id) const;      ///< returns true if there is an object with id \a object_id is contained within this UniverseObject
-    virtual bool                        ContainedBy(int object_id) const;   ///< returns true if there is an object with id \a object_id that contains this UniverseObject
-
-    virtual const std::string&          PublicName(int empire_id) const;
-
     /** Returns the list of systems that this fleet will move through en route
       * to its destination (may be empty).  If this fleet is currently at a
       * system, that system will be the first one in the list. */
@@ -71,6 +77,7 @@ public:
     bool                                CanChangeDirectionEnRoute() const   { return false; }           ///< Returns true iff this fleet can change its direction while in interstellar space.
     bool                                HasMonsters() const;                ///< returns true iff this fleet contains monster ships.
     bool                                HasArmedShips() const;              ///< Returns true if there is at least one armed ship in the fleet.
+    bool                                HasFighterShips() const;            ///< Returns true if there is at least one ship with fighters in the fleet.
     bool                                HasColonyShips() const;             ///< Returns true if there is at least one colony ship with nonzero capacity in the fleet.
     bool                                HasOutpostShips() const;            ///< Returns true if there is at least one colony ship with zero capacity in the fleet
     bool                                HasTroopShips() const;              ///< Returns true if there is at least one troop ship in the fleet.
@@ -78,6 +85,7 @@ public:
     bool                                HasShipsWithoutScrapOrders() const; ///< Returns true if there is at least one ship without any scrap orders in the fleet.
     int                                 NumShips() const                    { return m_ships.size(); }  ///< Returns number of ships in fleet.
     bool                                Empty() const                       { return m_ships.empty(); } ///< Returns true if fleet contains no ships, false otherwise.
+    float                               ResourceOutput(ResourceType type) const;
 
     /** Returns true iff this fleet is moving, but the route is unknown.  This
       * is usually the case when a foreign player A's fleet is represented on
@@ -93,20 +101,20 @@ public:
      * If in a system and not blockaded, the value is the current system ID. The blockade intent is that you can't
      * break a blockade unless you beat the blockaders (via combat or they retreat).**/
     int                                 ArrivalStarlane() const             { return m_arrival_starlane; }
-
-    virtual TemporaryPtr<UniverseObject>Accept(const UniverseObjectVisitor& visitor) const;
     //@}
 
     /** \name Mutators */ //@{
-    virtual void            Copy(TemporaryPtr<const UniverseObject> copied_object, int empire_id = ALL_EMPIRES);
+    void Copy(std::shared_ptr<const UniverseObject> copied_object, int empire_id = ALL_EMPIRES) override;
+
+    void MovementPhase() override;
+
+    void ResetTargetMaxUnpairedMeters() override;
 
     void                    SetRoute(const std::list<int>& route);          ///< sets this fleet to move through the series of systems in the list, in order
     void                    CalculateRouteTo(int target_system_id);         ///< sets this fleet to move through the series of systems that makes the shortest path from its current location to target_system_id
 
     void                    SetAggressive(bool aggressive = true);          ///< sets this fleet to be agressive (true) or passive (false)
 
-    virtual void            MovementPhase();
-
     void                    AddShip(int ship_id);                           ///< adds the ship to the fleet
     void                    AddShips(const std::vector<int>& ship_ids);     ///< adds the ships to the fleet
     void                    RemoveShip(int ship_id);                        ///< removes the ship from the fleet.
@@ -118,8 +126,6 @@ public:
 
     void                    SetGiveToEmpire(int empire_id);                 ///< marks fleet to be given to empire
     void                    ClearGiveToEmpire();                            ///< marks fleet not to be given to any empire
-
-    virtual void            ResetTargetMaxUnpairedMeters();
     //@}
 
     /* returns a name for a fleet based on its ships*/
@@ -147,18 +153,15 @@ protected:
     {}
     Fleet(const std::string& name, double x, double y, int owner);      ///< general ctor taking name, position and owner id
 
+    template <typename T> friend void UniverseObjectDeleter(T*);
     template <class T> friend void boost::python::detail::value_destroyer<false>::execute(T const volatile* p);
-    template <class T> friend void boost::checked_delete(T* x);
 
-#if BOOST_VERSION == 106100
 public:
-#endif
     ~Fleet() {}
-#if BOOST_VERSION == 106100
-protected:
-#endif
 
-    virtual Fleet*          Clone(int empire_id = ALL_EMPIRES) const;  ///< returns new copy of this Fleet
+protected:
+    /** Returns new copy of this Fleet. */
+    Fleet* Clone(int empire_id = ALL_EMPIRES) const override;
     //@}
 
 private:
