@@ -9,6 +9,7 @@
 #include "Condition.h"
 #include "Universe.h"
 #include "ValueRef.h"
+#include "Enums.h"
 #include "../util/Logger.h"
 #include "../util/OptionsDB.h"
 #include "../util/Random.h"
@@ -17,7 +18,6 @@
 #include "../Empire/Empire.h"
 #include "../Empire/EmpireManager.h"
 
-#include <boost/assign/list_of.hpp>
 
 namespace {
     // high tilt is arbitrarily taken to mean 45 degrees or more
@@ -33,21 +33,15 @@ namespace {
         case SZ_GASGIANT: return 0.25f;
         default:          return 1.0f;
         }
-        return 1.0f;
     }
 
     static const std::string EMPTY_STRING;
+
+    /** @content_tag{CTRL_STAT_SKIP_DEPOP} Do not count PopCenter%s with this tag for SpeciesPlanetsDepoped stat */
+    const std::string TAG_STAT_SKIP_DEPOP = "CTRL_STAT_SKIP_DEPOP";
 }
 
 
-////////////////////////////////////////////////////////////
-// Year
-////////////////////////////////////////////////////////////
-Year::Year(Day d) :
-    TypesafeFloat(d / 360.0f)
-{}
-
-
 ////////////////////////////////////////////////////////////
 // Planet
 ////////////////////////////////////////////////////////////
@@ -111,17 +105,17 @@ Planet* Planet::Clone(int empire_id) const {
     Visibility vis = GetUniverse().GetObjectVisibilityByEmpire(this->ID(), empire_id);
 
     if (!(vis >= VIS_BASIC_VISIBILITY && vis <= VIS_FULL_VISIBILITY))
-        return 0;
+        return nullptr;
 
     Planet* retval = new Planet();
-    retval->Copy(TemporaryFromThis(), empire_id);
+    retval->Copy(UniverseObject::shared_from_this(), empire_id);
     return retval;
 }
 
-void Planet::Copy(TemporaryPtr<const UniverseObject> copied_object, int empire_id) {
-    if (copied_object == this)
+void Planet::Copy(std::shared_ptr<const UniverseObject> copied_object, int empire_id) {
+    if (copied_object.get() == this)
         return;
-    TemporaryPtr<const Planet> copied_planet = boost::dynamic_pointer_cast<const Planet>(copied_object);
+    std::shared_ptr<const Planet> copied_planet = std::dynamic_pointer_cast<const Planet>(copied_object);
     if (!copied_planet) {
         ErrorLogger() << "Planet::Copy passed an object that wasn't a Planet";
         return;
@@ -188,9 +182,9 @@ std::string Planet::Dump() const {
     os << UniverseObject::Dump();
     os << PopCenter::Dump();
     os << ResourceCenter::Dump();
-    os << " type: " << UserString(EnumToString(m_type))
-       << " original type: " << UserString(EnumToString(m_original_type))
-       << " size: " << UserString(EnumToString(m_size))
+    os << " type: " << m_type
+       << " original type: " << m_original_type
+       << " size: " << m_size
        << " rot period: " << m_rotational_period
        << " axis tilt: " << m_axial_tilt
        << " buildings: ";
@@ -241,7 +235,7 @@ void Planet::Init() {
 }
 
 PlanetEnvironment Planet::EnvironmentForSpecies(const std::string& species_name/* = ""*/) const {
-    const Species* species = 0;
+    const Species* species = nullptr;
     if (species_name.empty()) {
         const std::string& this_planet_species_name = this->SpeciesName();
         if (this_planet_species_name.empty())
@@ -258,7 +252,7 @@ PlanetEnvironment Planet::EnvironmentForSpecies(const std::string& species_name/
 }
 
 PlanetType Planet::NextBetterPlanetTypeForSpecies(const std::string& species_name/* = ""*/) const {
-    const Species* species = 0;
+    const Species* species = nullptr;
     if (species_name.empty()) {
         const std::string& this_planet_species_name = this->SpeciesName();
         if (this_planet_species_name.empty())
@@ -407,23 +401,23 @@ PlanetSize Planet::NextLargerPlanetSize() const
 PlanetSize Planet::NextSmallerPlanetSize() const
 { return PlanetSizeIncrement(m_size, -1); }
 
-Year Planet::OrbitalPeriod() const
+float Planet::OrbitalPeriod() const
 { return m_orbital_period; }
 
-Radian Planet::InitialOrbitalPosition() const
+float Planet::InitialOrbitalPosition() const
 { return m_initial_orbital_position; }
 
-Radian Planet::OrbitalPositionOnTurn(int turn) const
+float Planet::OrbitalPositionOnTurn(int turn) const
 { return m_initial_orbital_position + OrbitalPeriod() * 2.0 * 3.1415926 / 4 * turn; }
 
-Day Planet::RotationalPeriod() const
+float Planet::RotationalPeriod() const
 { return m_rotational_period; }
 
-Degree Planet::AxialTilt() const
+float Planet::AxialTilt() const
 { return m_axial_tilt; }
 
-TemporaryPtr<UniverseObject> Planet::Accept(const UniverseObjectVisitor& visitor) const
-{ return visitor.Visit(boost::const_pointer_cast<Planet>(boost::static_pointer_cast<const Planet>(TemporaryFromThis()))); }
+std::shared_ptr<UniverseObject> Planet::Accept(const UniverseObjectVisitor& visitor) const
+{ return visitor.Visit(std::const_pointer_cast<Planet>(std::static_pointer_cast<const Planet>(UniverseObject::shared_from_this()))); }
 
 Meter* Planet::GetMeter(MeterType type)
 { return UniverseObject::GetMeter(type); }
@@ -492,7 +486,7 @@ std::string Planet::CardinalSuffix() const {
     if (Type() == PT_ASTEROIDS)
         retval.append(UserString("NEW_ASTEROIDS_SUFFIX") + " ");
 
-    TemporaryPtr<System> cur_system = GetSystem(SystemID());
+    std::shared_ptr<System> cur_system = GetSystem(SystemID());
     if (cur_system) {
         if (cur_system->OrbitOfPlanet(ID()) < 0) {
             ErrorLogger() << "Planet " << Name() << "(" << ID() << ") "
@@ -503,14 +497,11 @@ std::string Planet::CardinalSuffix() const {
         int num_planets_lteq = 0;  // number of planets at this orbit or smaller
         int num_planets_total = 0;
         bool prior_current_planet = true;
-        const std::vector<int>& sys_orbits = cur_system->PlanetIDsByOrbit();
-        for (std::vector<int>::const_iterator it = sys_orbits.begin();
-             it != sys_orbits.end(); ++it)
-        {
-            if (*it == INVALID_OBJECT_ID)
+        for (int sys_orbit : cur_system->PlanetIDsByOrbit()) {
+            if (sys_orbit == INVALID_OBJECT_ID)
                 continue;
 
-            PlanetType other_planet_type = GetPlanet(*it)->Type();
+            PlanetType other_planet_type = GetPlanet(sys_orbit)->Type();
             if (other_planet_type == INVALID_PLANET_TYPE)
                 continue;
 
@@ -528,7 +519,7 @@ std::string Planet::CardinalSuffix() const {
                 }
             }
 
-            if (*it == ID())
+            if (sys_orbit == ID())
                 prior_current_planet =false;
         }
         // For asteroids: If no other asteroids in this system, suffix does not receive a number
@@ -554,14 +545,12 @@ bool Planet::ContainedBy(int object_id) const
 
 std::vector<std::string> Planet::AvailableFoci() const {
     std::vector<std::string> retval;
-    TemporaryPtr<const Planet> this_planet = boost::dynamic_pointer_cast<const Planet>(TemporaryFromThis());
+    std::shared_ptr<const Planet> this_planet = std::dynamic_pointer_cast<const Planet>(UniverseObject::shared_from_this());
     if (!this_planet)
         return retval;
     ScriptingContext context(this_planet);
     if (const Species* species = GetSpecies(this_planet->SpeciesName())) {
-        const std::vector<FocusType>& foci = species->Foci();
-        for (std::vector<FocusType>::const_iterator it = foci.begin(); it != foci.end(); ++it) {
-            const FocusType& focus_type = *it;
+        for (const FocusType& focus_type : species->Foci()) {
             if (const Condition::ConditionBase* location = focus_type.Location()) {
                 if (location->Eval(context, this_planet))
                     retval.push_back(focus_type.Name());
@@ -574,9 +563,7 @@ std::vector<std::string> Planet::AvailableFoci() const {
 
 const std::string& Planet::FocusIcon(const std::string& focus_name) const {
     if (const Species* species = GetSpecies(this->SpeciesName())) {
-        const std::vector<FocusType>& foci = species->Foci();
-        for (std::vector<FocusType>::const_iterator it = foci.begin(); it != foci.end(); ++it) {
-            const FocusType& focus_type = *it;
+        for (const FocusType& focus_type : species->Foci()) {
             if (focus_type.Name() == focus_name)
                 return focus_type.Graphic();
         }
@@ -611,7 +598,7 @@ void Planet::SetSize(PlanetSize size) {
     StateChangedSignal();
 }
 
-void Planet::SetRotationalPeriod(Day days)
+void Planet::SetRotationalPeriod(float days)
 { m_rotational_period = days; }
 
 void Planet::SetHighAxialTilt() {
@@ -650,8 +637,8 @@ void Planet::Reset() {
     GetMeter(METER_REBEL_TROOPS)->Reset();
 
     if (m_is_about_to_be_colonized && !OwnedBy(ALL_EMPIRES)) {
-        for (std::set<int>::const_iterator it = m_buildings.begin(); it != m_buildings.end(); ++it)
-            if (TemporaryPtr<Building> building = GetBuilding(*it))
+        for (int building_id : m_buildings)
+            if (std::shared_ptr<Building> building = GetBuilding(building_id))
                 building->Reset();
     }
 
@@ -680,11 +667,7 @@ void Planet::Conquer(int conquerer) {
     Empire::ConquerProductionQueueItemsAtLocation(ID(), conquerer);
 
     // deal with UniverseObjects (eg. buildings) located on this planet
-    std::vector<TemporaryPtr<Building> > buildings = Objects().FindObjects<Building>(m_buildings);
-    for (std::vector<TemporaryPtr<Building> >::iterator it = buildings.begin();
-         it != buildings.end(); ++it)
-    {
-        TemporaryPtr<Building> building = *it;
+    for (std::shared_ptr<Building> building : Objects().FindObjects<Building>(m_buildings)) {
         const BuildingType* type = GetBuildingType(building->BuildingTypeName());
 
         // determine what to do with building of this type...
@@ -697,7 +680,7 @@ void Planet::Conquer(int conquerer) {
             // destroy object
             //DebugLogger() << "Planet::Conquer destroying object: " << building->Name();
             this->RemoveBuilding(building->ID());
-            if (TemporaryPtr<System> system = GetSystem(this->SystemID()))
+            if (std::shared_ptr<System> system = GetSystem(this->SystemID()))
                 system->Remove(building->ID());
             GetUniverse().Destroy(building->ID());
         } else if (cap_result == CR_RETAIN) {
@@ -724,10 +707,12 @@ void Planet::Conquer(int conquerer) {
     GetMeter(METER_SHIELD)->BackPropagate();
     GetMeter(METER_HAPPINESS)->SetCurrent(0.0f);
     GetMeter(METER_HAPPINESS)->BackPropagate();
+    GetMeter(METER_DETECTION)->SetCurrent(0.0f);
+    GetMeter(METER_DETECTION)->BackPropagate();
 }
 
 bool Planet::Colonize(int empire_id, const std::string& species_name, double population) {
-    const Species* species = 0;
+    const Species* species = nullptr;
 
     // if desired pop > 0, we want a colony, not an outpost, so we need to do some checks
     if (population > 0.0) {
@@ -749,8 +734,8 @@ bool Planet::Colonize(int empire_id, const std::string& species_name, double pop
         Reset();
     } else {
         PopCenter::Reset();
-        for (std::set<int>::const_iterator it = m_buildings.begin(); it != m_buildings.end(); ++it)
-            if (TemporaryPtr<Building> building = GetBuilding(*it))
+        for (int building_id : m_buildings)
+            if (std::shared_ptr<Building> building = GetBuilding(building_id))
                 building->Reset();
         m_just_conquered = false;
         m_is_about_to_be_colonized = false;
@@ -769,11 +754,9 @@ bool Planet::Colonize(int empire_id, const std::string& species_name, double pop
     std::vector<std::string> available_foci = AvailableFoci();
     if (species && !available_foci.empty()) {
         bool found_preference = false;
-        for (std::vector<std::string>::const_iterator it = available_foci.begin();
-             it != available_foci.end(); ++it)
-        {
-            if (!it->empty() && *it == species->PreferredFocus()) {
-                SetFocus(*it);
+        for (const std::string& focus : available_foci) {
+            if (!focus.empty() && focus == species->PreferredFocus()) {
+                SetFocus(focus);
                 found_preference = true;
                 break;
             }
@@ -795,10 +778,8 @@ bool Planet::Colonize(int empire_id, const std::string& species_name, double pop
     SetOwner(empire_id);
 
     // if there are buildings on the planet, set the specified empire as their owner too
-    std::vector<TemporaryPtr<Building> > buildings = Objects().FindObjects<Building>(BuildingIDs());
-    for (std::vector<TemporaryPtr<Building> >::iterator building_it = buildings.begin();
-         building_it != buildings.end(); ++building_it)
-    { (*building_it)->SetOwner(empire_id); }
+    for (std::shared_ptr<Building> building : Objects().FindObjects<Building>(BuildingIDs()))
+    { building->SetOwner(empire_id); }
 
     return true;
 }
@@ -867,15 +848,17 @@ void Planet::PopGrowthProductionResearchPhase() {
         if (Empire* empire = GetEmpire(this->Owner())) {
             empire->AddSitRepEntry(CreatePlanetDepopulatedSitRep(this->ID()));
 
-            // record depopulation of planet with species while owned by this empire
-            std::map<std::string, int>::iterator species_it = empire->SpeciesPlanetsDepoped().find(SpeciesName());
-            if (species_it == empire->SpeciesPlanetsDepoped().end())
-                empire->SpeciesPlanetsDepoped()[SpeciesName()] = 1;
-            else
-                species_it->second++;
+            if (!HasTag(TAG_STAT_SKIP_DEPOP)) {
+                // record depopulation of planet with species while owned by this empire
+                std::map<std::string, int>::iterator species_it = empire->SpeciesPlanetsDepoped().find(SpeciesName());
+                if (species_it == empire->SpeciesPlanetsDepoped().end())
+                    empire->SpeciesPlanetsDepoped()[SpeciesName()] = 1;
+                else
+                    species_it->second++;
+            }
         }
         // remove species
-        SetSpecies("");
+        PopCenter::Reset();
     }
 
     if (!just_conquered) {
