@@ -4,6 +4,7 @@
 #include "../util/Directories.h"
 #include "../util/Logger.h"
 #include "Meter.h"
+#include "Enums.h"
 
 #include <algorithm>
 #include <stdexcept>
@@ -26,8 +27,8 @@ PopCenter::PopCenter() :
 PopCenter::~PopCenter()
 {}
 
-void PopCenter::Copy(TemporaryPtr<const PopCenter> copied_object, Visibility vis) {
-    if (copied_object == this)
+void PopCenter::Copy(std::shared_ptr<const PopCenter> copied_object, Visibility vis) {
+    if (copied_object.get() == this)
         return;
     if (!copied_object) {
         ErrorLogger() << "PopCenter::Copy passed a null object";
@@ -39,6 +40,9 @@ void PopCenter::Copy(TemporaryPtr<const PopCenter> copied_object, Visibility vis
     }
 }
 
+void PopCenter::Copy(std::shared_ptr<const PopCenter> copied_object)
+{ Copy(copied_object, VIS_FULL_VISIBILITY); }
+
 void PopCenter::Init() {
     //DebugLogger() << "PopCenter::Init";
     AddMeter(METER_POPULATION);
@@ -117,6 +121,11 @@ void PopCenter::PopCenterResetTargetMaxUnpairedMeters() {
 }
 
 void PopCenter::PopCenterPopGrowthProductionResearchPhase() {
+    if (m_species_name.empty()) {
+        // No changes to population or happiness
+        return;
+    }
+
     float cur_pop = CurrentMeterValue(METER_POPULATION);
     float pop_growth = NextTurnPopGrowth(); // may be negative
     float new_pop = cur_pop + pop_growth;
