@@ -1,9 +1,7 @@
 #ifndef _ResourceCenter_h_
 #define _ResourceCenter_h_
 
-#include "Enums.h"
-#include "EnableTemporaryFromThis.h"
-#include "TemporaryPtr.h"
+#include "EnumsFwd.h"
 #include <boost/signals2/signal.hpp>
 #include <boost/serialization/nvp.hpp>
 
@@ -21,12 +19,14 @@ class UniverseObject;
   * classes could be made from it as well (e.g., a trade-ship or mining vessel,
   * or a non-Planet UniverseObject- and PopCenter- derived object of some
   * sort. */
-class FO_COMMON_API ResourceCenter : virtual public EnableTemporaryFromThis<UniverseObject> {
+class FO_COMMON_API ResourceCenter : virtual public std::enable_shared_from_this<UniverseObject> {
 public:
     /** \name Structors */ //@{
-    ResourceCenter();                               ///< default ctor
-    virtual ~ResourceCenter();                      ///< dtor
-    ResourceCenter(const ResourceCenter& rhs);      ///< copy ctor
+    ResourceCenter();
+
+    ResourceCenter(const ResourceCenter& rhs);
+
+    virtual ~ResourceCenter();
     //@}
 
     /** \name Accessors */ //@{
@@ -46,13 +46,17 @@ public:
     //@}
 
     /** \name Mutators */ //@{
-    void            Copy(TemporaryPtr<const ResourceCenter> copied_object, Visibility vis = VIS_FULL_VISIBILITY);
+    void Copy(std::shared_ptr<const ResourceCenter> copied_object, Visibility vis);
+
+    void Copy(std::shared_ptr<const ResourceCenter> copied_object);
 
     void            SetFocus(const std::string& focus);
     void            ClearFocus();
     void            UpdateFocusHistory();
 
-    virtual void    Reset();                                                        ///< Resets the meters, etc.  This should be called when a ResourceCenter is wiped out due to starvation, etc.
+    /** Resets the meters, etc. This should be called when a ResourceCenter is
+        wiped out due to starvation, etc. */
+    virtual void Reset();
     //@}
 
 protected:
