@@ -9,6 +9,7 @@
 #include "ShipDesign.h"
 #include "Species.h"
 #include "Universe.h"
+#include "Enums.h"
 #include "../Empire/Empire.h"
 #include "../Empire/EmpireManager.h"
 
@@ -70,11 +71,11 @@ Ship::Ship(int empire_id, int design_id, const std::string& species_name,
     AddMeter(METER_TRADE);
 
     const std::vector<std::string>& part_names = Design()->Parts();
-    for (std::size_t i = 0; i < part_names.size(); ++i) {
-        if (part_names[i] != "") {
-            const PartType* part = GetPartType(part_names[i]);
+    for (const std::string& part_name : part_names) {
+        if (part_name != "") {
+            const PartType* part = GetPartType(part_name);
             if (!part) {
-                ErrorLogger() << "Ship::Ship couldn't get part with name " << part_names[i];
+                ErrorLogger() << "Ship::Ship couldn't get part with name " << part_name;
                 continue;
             }
 
@@ -106,17 +107,17 @@ Ship* Ship::Clone(int empire_id) const {
     Visibility vis = GetUniverse().GetObjectVisibilityByEmpire(this->ID(), empire_id);
 
     if (!(vis >= VIS_BASIC_VISIBILITY && vis <= VIS_FULL_VISIBILITY))
-        return 0;
+        return nullptr;
 
     Ship* retval = new Ship();
-    retval->Copy(TemporaryFromThis(), empire_id);
+    retval->Copy(shared_from_this(), empire_id);
     return retval;
 }
 
-void Ship::Copy(TemporaryPtr<const UniverseObject> copied_object, int empire_id) {
-    if (copied_object == this)
+void Ship::Copy(std::shared_ptr<const UniverseObject> copied_object, int empire_id) {
+    if (copied_object.get() == this)
         return;
-    TemporaryPtr<const Ship> copied_ship = boost::dynamic_pointer_cast<const Ship>(copied_object);
+    std::shared_ptr<const Ship> copied_ship = std::dynamic_pointer_cast<const Ship>(copied_object);
     if (!copied_ship) {
         ErrorLogger() << "Ship::Copy passed an object that wasn't a Ship";
         return;
@@ -131,7 +132,7 @@ void Ship::Copy(TemporaryPtr<const UniverseObject> copied_object, int empire_id)
     if (vis >= VIS_BASIC_VISIBILITY) {
         if (this->m_fleet_id != copied_ship->m_fleet_id) {
             // as with other containers, removal from the old container is triggered by the contained Object; removal from System is handled by UniverseObject::Copy
-            if (TemporaryPtr<Fleet> oldFleet = GetFleet(this->m_fleet_id)) 
+            if (std::shared_ptr<Fleet> oldFleet = GetFleet(this->m_fleet_id))
                 oldFleet->RemoveShip(this->ID());
             this->m_fleet_id =              copied_ship->m_fleet_id; // as with other containers (Systems), actual insertion into fleet ships set is handled by the fleet
         }
@@ -141,9 +142,7 @@ void Ship::Copy(TemporaryPtr<const UniverseObject> copied_object, int empire_id)
                 this->m_name =                  copied_ship->m_name;
 
             this->m_design_id =                 copied_ship->m_design_id;
-            for (PartMeterMap::const_iterator it = copied_ship->m_part_meters.begin();
-                 it != copied_ship->m_part_meters.end(); ++it)
-            { this->m_part_meters = copied_ship->m_part_meters; }
+            this->m_part_meters =               copied_ship->m_part_meters;
             this->m_species_name =              copied_ship->m_species_name;
 
             this->m_last_turn_active_in_combat= copied_ship->m_last_turn_active_in_combat;
@@ -176,8 +175,8 @@ std::set<std::string> Ship::Tags() const {
     if (parts.empty())
         return retval;
 
-    for (std::vector<std::string>::const_iterator part_it = parts.begin(); part_it != parts.end(); ++part_it) {
-        if (const PartType* part = GetPartType(*part_it)) {
+    for (const std::string& part_name : parts) {
+        if (const PartType* part = GetPartType(part_name)) {
             retval.insert(part->Tags().begin(), part->Tags().end());
         }
     }
@@ -194,9 +193,8 @@ bool Ship::HasTag(const std::string& name) const {
             return true;
 
         // check parts for tag
-        const std::vector<std::string>& parts = design->Parts();
-        for (std::vector<std::string>::const_iterator part_it = parts.begin(); part_it != parts.end(); ++part_it) {
-            const PartType* part = GetPartType(*part_it);
+        for (const std::string& part_name : design->Parts()) {
+            const PartType* part = GetPartType(part_name);
             if (part && part->Tags().count(name))
                 return true;
         }
@@ -228,13 +226,12 @@ std::string Ship::Dump() const {
        << " arrived on turn: " << m_arrived_on_turn;
     if (!m_part_meters.empty()) {
         os << " part meters: ";
-        for (PartMeterMap::const_iterator it = m_part_meters.begin(); it != m_part_meters.end();) {
-        const std::string part_name = it->first.second;
-        MeterType meter_type = it->first.first;
-        const Meter& meter = it->second;
-        ++it;
-        os << UserString(part_name) << " "
-           << UserString(EnumToString(meter_type))
+        for (const PartMeterMap::value_type& entry : m_part_meters) {
+        const std::string part_name = entry.first.second;
+        MeterType meter_type = entry.first.first;
+        const Meter& meter = entry.second;
+        os << part_name << " "
+           << meter_type
            << ": " << meter.Current() << "  ";
     }
     }
@@ -260,6 +257,14 @@ bool Ship::IsArmed() const {
         return false;
 }
 
+bool Ship::HasFighters() const {
+    const ShipDesign* design = Design();
+    if (design)
+        return design->HasFighters();
+    else
+        return false;
+}
+
 bool Ship::CanColonize() const {
     if (m_species_name.empty())
         return false;
@@ -290,11 +295,8 @@ float Ship::ColonyCapacity() const {
     if (!design)
         return retval;
 
-    const std::vector<std::string>& parts = design->Parts();
-    for (std::vector<std::string>::const_iterator part_it = parts.begin();
-         part_it != parts.end(); ++part_it)
+    for (const std::string& part_name : design->Parts())
     {
-        const std::string& part_name = *part_it;
         if (part_name.empty())
             continue;
         const PartType* part_type = GetPartType(part_name);
@@ -317,11 +319,7 @@ float Ship::TroopCapacity() const {
     if (!design)
         return retval;
 
-    const std::vector<std::string>& parts = design->Parts();
-    for (std::vector<std::string>::const_iterator part_it = parts.begin();
-         part_it != parts.end(); ++part_it)
-    {
-        const std::string& part_name = *part_it;
+    for (const std::string& part_name : design->Parts()) {
         if (part_name.empty())
             continue;
         const PartType* part_type = GetPartType(part_name);
@@ -362,8 +360,8 @@ const std::string& Ship::PublicName(int empire_id) const {
         return UserString("OBJ_SHIP");
 }
 
-TemporaryPtr<UniverseObject> Ship::Accept(const UniverseObjectVisitor& visitor) const
-{ return visitor.Visit(boost::const_pointer_cast<Ship>(boost::static_pointer_cast<const Ship>(TemporaryFromThis()))); }
+std::shared_ptr<UniverseObject> Ship::Accept(const UniverseObjectVisitor& visitor) const
+{ return visitor.Visit(std::const_pointer_cast<Ship>(std::static_pointer_cast<const Ship>(shared_from_this()))); }
 
 float Ship::NextTurnCurrentMeterValue(MeterType type) const {
     const Meter* meter = UniverseObject::GetMeter(type);
@@ -371,11 +369,6 @@ float Ship::NextTurnCurrentMeterValue(MeterType type) const {
         throw std::invalid_argument("Ship::NextTurnCurrentMeterValue passed meter type that the Ship does not have: " + boost::lexical_cast<std::string>(type));
     float current_meter_value = meter->Current();
 
-    //if (type == METER_FUEL) {
-    //    // todo: consider fleet movement or being stationary, which may partly replenish fuel
-    //    // todo: consider fleet passing through or being in a supplied system, which replenishes fuel
-    //}
-
     if (type == METER_SHIELD) {
         if (m_last_turn_active_in_combat >= CurrentTurn())
             return std::max(0.0f,   // battle just happened. shields limited to max shield, but don't regen
@@ -420,7 +413,7 @@ const Meter* Ship::GetPartMeter(MeterType type, const std::string& part_name) co
 { return const_cast<Ship*>(this)->GetPartMeter(type, part_name); }
 
 Meter* Ship::GetPartMeter(MeterType type, const std::string& part_name) {
-    Meter* retval = 0;
+    Meter* retval = nullptr;
     PartMeterMap::iterator it = m_part_meters.find(std::make_pair(type, part_name));
     if (it != m_part_meters.end())
         retval = &it->second;
@@ -439,12 +432,73 @@ float Ship::InitialPartMeterValue(MeterType type, const std::string& part_name)
     return 0.0f;
 }
 
+float Ship::SumCurrentPartMeterValuesForPartClass(MeterType type, ShipPartClass part_class) const {
+    float retval = 0.0f;
+
+    const ShipDesign* design = GetShipDesign(m_design_id);
+    if (!design)
+        return retval;
+
+    const std::vector<std::string>& parts = design->Parts();
+    if (parts.empty())
+        return retval;
+
+    std::map<std::string, int> part_counts;
+    for (const std::string& part : parts)
+        part_counts[part]++;
+
+    for (const PartMeterMap::value_type& part_meter : m_part_meters) {
+        if (part_meter.first.first != type)
+            continue;
+        const std::string& part_name = part_meter.first.second;
+        if (part_counts[part_name] < 1)
+            continue;
+        const PartType* part_type = GetPartType(part_name);
+        if (!part_type)
+            continue;
+        if (part_class == part_type->Class())
+            retval += part_meter.second.Current() * part_counts[part_name];
+    }
+
+    return retval;
+}
+
+float Ship::FighterCount() const {
+    float retval = 0.0f;
+    for (const PartMeterMap::value_type& entry : m_part_meters) {
+        //std::map<std::pair<MeterType, std::string>, Meter>
+        if (entry.first.first != METER_CAPACITY)
+            continue;
+        const PartType* part_type = GetPartType(entry.first.second);
+        if (!part_type || part_type->Class() != PC_FIGHTER_HANGAR)
+            continue;
+        retval += entry.second.Current();
+    }
+
+    return retval;
+}
+
+float Ship::FighterMax() const {
+    float retval = 0.0f;
+    for (const PartMeterMap::value_type& entry : m_part_meters) {
+        //std::map<std::pair<MeterType, std::string>, Meter>
+        if (entry.first.first != METER_MAX_CAPACITY)
+            continue;
+        const PartType* part_type = GetPartType(entry.first.second);
+        if (!part_type || part_type->Class() != PC_FIGHTER_HANGAR)
+            continue;
+        retval += entry.second.Current();
+    }
+
+    return retval;
+}
+
 float Ship::TotalWeaponsDamage(float shield_DR, bool include_fighters) const {
     // sum up all individual weapons' attack strengths
     float total_attack = 0.0f;
     std::vector<float> all_weapons_damage = AllWeaponsDamage(shield_DR, include_fighters);
-    for (std::vector<float>::iterator it = all_weapons_damage.begin(); it != all_weapons_damage.end(); ++it)
-        total_attack += *it;
+    for (float attack : all_weapons_damage)
+        total_attack += attack;
     return total_attack;
 }
 
@@ -468,10 +522,7 @@ namespace {
         int available_fighters = 0;
 
         // for each weapon part, get its damage meter value
-        for (std::vector<std::string>::const_iterator part_it = parts.begin();
-             part_it != parts.end(); ++part_it)
-        {
-            const std::string& part_name = *part_it;
+        for (const std::string& part_name : parts) {
             const PartType* part = GetPartType(part_name);
             if (!part)
                 continue;
@@ -490,24 +541,31 @@ namespace {
 
             } else if (part_class == PC_FIGHTER_HANGAR && include_fighters) {
                 // attack strength of a ship's fighters determined by the hangar...
-                fighter_damage = ship->CurrentPartMeterValue(SECONDARY_METER, part_name) - DR;  // assuming all hangars have the same damage...
-                available_fighters += std::max(0, static_cast<int>(ship->CurrentPartMeterValue(METER, part_name)));
+                fighter_damage = ship->CurrentPartMeterValue(SECONDARY_METER, part_name);  // assuming all hangars have the same damage...
+                available_fighters = std::max(0, static_cast<int>(ship->CurrentPartMeterValue(METER, part_name)));  // stacked meter
             }
         }
 
         if (!include_fighters || fighter_damage <= 0.0f || available_fighters <= 0 || fighter_launch_capacity <= 0)
             return retval;
 
-        // assuming 3 combat "bouts"
         int fighter_shots = std::min(available_fighters, fighter_launch_capacity);  // how many fighters launched in bout 1
         available_fighters -= fighter_shots;
-        fighter_shots *= 2;                                                         // first-bout-launched fighters attack in bouts 2 and 3
-        fighter_shots += std::min(available_fighters, fighter_launch_capacity);     // second-bout-launched fighters attack only in bout 3
+        int launched_fighters = fighter_shots;
+        int num_bouts = GetUniverse().GetNumCombatRounds();
+        int remaining_bouts = num_bouts - 2;  // no attack for first round, second round already added
+        while (remaining_bouts > 0) {
+            int fighters_launched_this_bout = std::min(available_fighters, fighter_launch_capacity);
+            available_fighters -= fighters_launched_this_bout;
+            launched_fighters += fighters_launched_this_bout;
+            fighter_shots += launched_fighters;
+            --remaining_bouts;
+        }
 
         // how much damage does a fighter shot do?
         fighter_damage = std::max(0.0f, fighter_damage);
 
-        retval.push_back(fighter_damage * fighter_shots / 3.0f);    // divide by 3 because fighter calculation is for a full combat, but direct firefor one attack
+        retval.push_back(fighter_damage * fighter_shots / num_bouts);    // divide by bouts because fighter calculation is for a full combat, but direct firefor one attack
 
         return retval;
     }
@@ -551,8 +609,8 @@ void Ship::BackPropagateMeters() {
     UniverseObject::BackPropagateMeters();
 
     // ship part meter back propagation, since base class function doesn't do this...
-    for (PartMeterMap::iterator it = m_part_meters.begin(); it != m_part_meters.end(); ++it)
-        it->second.BackPropagate();
+    for (PartMeterMap::value_type& entry : m_part_meters)
+        entry.second.BackPropagate();
 }
 
 void Ship::Resupply() {
@@ -568,20 +626,20 @@ void Ship::Resupply() {
     // set all part capacities equal to any associated max capacity
     // this "upgrades" any direct-fire weapon parts to their latest-allowed
     // strengths, and replaces any lost fighters
-    for (PartMeterMap::iterator it = m_part_meters.begin(); it != m_part_meters.end(); ++it) {
+    for (PartMeterMap::value_type& entry : m_part_meters) {
         PartMeterMap::iterator max_it = m_part_meters.end();
-        if (it->first.first == METER_CAPACITY) {
-            max_it = m_part_meters.find(std::make_pair(METER_MAX_CAPACITY, it->first.second));
-        } else if (it->first.first == METER_SECONDARY_STAT) {
-            max_it = m_part_meters.find(std::make_pair(METER_MAX_SECONDARY_STAT, it->first.second));
+        if (entry.first.first == METER_CAPACITY) {
+            max_it = m_part_meters.find(std::make_pair(METER_MAX_CAPACITY, entry.first.second));
+        } else if (entry.first.first == METER_SECONDARY_STAT) {
+            max_it = m_part_meters.find(std::make_pair(METER_MAX_SECONDARY_STAT, entry.first.second));
         }
         if (max_it == m_part_meters.end())
             continue;
 
         const Meter& max_meter = max_it->second;
 
-        it->second.SetCurrent(max_meter.Current());
-        it->second.BackPropagate();
+        entry.second.SetCurrent(max_meter.Current());
+        entry.second.BackPropagate();
     }
 }
 
@@ -638,31 +696,31 @@ void Ship::ResetTargetMaxUnpairedMeters() {
     UniverseObject::GetMeter(METER_SPEED)->ResetCurrent();
     //UniverseObject::GetMeter(METER_STEALTH)->ResetCurrent(); redundant with base class function
 
-    for (PartMeterMap::iterator it = m_part_meters.begin(); it != m_part_meters.end(); ++it) {
+    for (PartMeterMap::value_type& entry : m_part_meters) {
         PartMeterMap::iterator max_it = m_part_meters.end();
-        if (it->first.first == METER_CAPACITY) {
-            max_it = m_part_meters.find(std::make_pair(METER_MAX_CAPACITY, it->first.second));
-        } else if (it->first.first == METER_SECONDARY_STAT) {
-            max_it = m_part_meters.find(std::make_pair(METER_MAX_SECONDARY_STAT, it->first.second));
+        if (entry.first.first == METER_CAPACITY) {
+            max_it = m_part_meters.find(std::make_pair(METER_MAX_CAPACITY, entry.first.second));
+        } else if (entry.first.first == METER_SECONDARY_STAT) {
+            max_it = m_part_meters.find(std::make_pair(METER_MAX_SECONDARY_STAT, entry.first.second));
         }
         if (max_it != m_part_meters.end())
             continue;
-        it->second.ResetCurrent();
+        entry.second.ResetCurrent();
     }
 }
 
 void Ship::ResetPairedActiveMeters() {
     UniverseObject::ResetPairedActiveMeters();
 
-    for (PartMeterMap::iterator it = m_part_meters.begin(); it != m_part_meters.end(); ++it) {
+    for (PartMeterMap::value_type& entry : m_part_meters) {
         PartMeterMap::iterator max_it = m_part_meters.end();
-        if (it->first.first == METER_CAPACITY) {
-            max_it = m_part_meters.find(std::make_pair(METER_MAX_CAPACITY, it->first.second));
-        } else if (it->first.first == METER_SECONDARY_STAT) {
-            max_it = m_part_meters.find(std::make_pair(METER_MAX_SECONDARY_STAT, it->first.second));
+        if (entry.first.first == METER_CAPACITY) {
+            max_it = m_part_meters.find(std::make_pair(METER_MAX_CAPACITY, entry.first.second));
+        } else if (entry.first.first == METER_SECONDARY_STAT) {
+            max_it = m_part_meters.find(std::make_pair(METER_MAX_SECONDARY_STAT, entry.first.second));
         }
         if (max_it != m_part_meters.end())
-            it->second.SetCurrent(it->second.Initial());
+            entry.second.SetCurrent(entry.second.Initial());
     }
 }
 
@@ -675,17 +733,17 @@ void Ship::SetShipMetersToMax() {
     UniverseObject::GetMeter(METER_STRUCTURE)->SetCurrent(Meter::LARGE_VALUE);
 
     // some part capacity meters may have an associated max capacity...
-    for (PartMeterMap::iterator it = m_part_meters.begin(); it != m_part_meters.end(); ++it) {
+    for (PartMeterMap::value_type& entry : m_part_meters) {
         PartMeterMap::iterator max_it = m_part_meters.end();
-        if (it->first.first == METER_CAPACITY) {
-            max_it = m_part_meters.find(std::make_pair(METER_MAX_CAPACITY, it->first.second));
-        } else if (it->first.first == METER_SECONDARY_STAT) {
-            max_it = m_part_meters.find(std::make_pair(METER_MAX_SECONDARY_STAT, it->first.second));
+        if (entry.first.first == METER_CAPACITY) {
+            max_it = m_part_meters.find(std::make_pair(METER_MAX_CAPACITY, entry.first.second));
+        } else if (entry.first.first == METER_SECONDARY_STAT) {
+            max_it = m_part_meters.find(std::make_pair(METER_MAX_SECONDARY_STAT, entry.first.second));
         }
         if (max_it == m_part_meters.end())
             continue;
         max_it->second.SetCurrent(Meter::LARGE_VALUE);
-        it->second.SetCurrent(Meter::LARGE_VALUE);
+        entry.second.SetCurrent(Meter::LARGE_VALUE);
     }
 }
 
@@ -722,29 +780,29 @@ void Ship::ClampMeters() {
     UniverseObject::GetMeter(METER_SPEED)->ClampCurrentToRange();
 
     // clamp most part meters to basic range limits
-    for (PartMeterMap::iterator it = m_part_meters.begin(); it != m_part_meters.end(); ++it) {
-        if (it->first.first == METER_CAPACITY || it->first.first == METER_SECONDARY_STAT)
+    for (PartMeterMap::value_type& entry : m_part_meters) {
+        if (entry.first.first == METER_CAPACITY || entry.first.first == METER_SECONDARY_STAT)
             continue;
-        it->second.ClampCurrentToRange();
+        entry.second.ClampCurrentToRange();
     }
 
     // special case extra clamping for paired CAPACTY meters dependent on their associated MAX_CAPACITY meter...
-    for (PartMeterMap::iterator it = m_part_meters.begin(); it != m_part_meters.end(); ++it) {
+    for (PartMeterMap::value_type& entry : m_part_meters) {
         PartMeterMap::iterator max_it = m_part_meters.end();
-        if (it->first.first == METER_CAPACITY) {
-            max_it = m_part_meters.find(std::make_pair(METER_MAX_CAPACITY, it->first.second));
-        } else if (it->first.first == METER_SECONDARY_STAT) {
-            max_it = m_part_meters.find(std::make_pair(METER_MAX_SECONDARY_STAT, it->first.second));
+        if (entry.first.first == METER_CAPACITY) {
+            max_it = m_part_meters.find(std::make_pair(METER_MAX_CAPACITY, entry.first.second));
+        } else if (entry.first.first == METER_SECONDARY_STAT) {
+            max_it = m_part_meters.find(std::make_pair(METER_MAX_SECONDARY_STAT, entry.first.second));
         }
 
         if (max_it == m_part_meters.end()) {
             // no found max meter, revert to normal clamping for this meter
-            it->second.ClampCurrentToRange();
+            entry.second.ClampCurrentToRange();
             continue;
         }
 
         const Meter& max_meter = max_it->second;
-        it->second.ClampCurrentToRange(Meter::DEFAULT_VALUE, max_meter.Current());
+        entry.second.ClampCurrentToRange(Meter::DEFAULT_VALUE, max_meter.Current());
     }
 }
 
@@ -752,18 +810,11 @@ void Ship::ClampMeters() {
 // Free Functions //
 ////////////////////
 std::string NewMonsterName() {
-    static std::vector<std::string> monster_names;
+    static std::vector<std::string> monster_names = UserStringList("MONSTER_NAMES");
     static std::map<std::string, int> monster_names_used;
-    if (monster_names.empty()) {
-        // load monster names from stringtable
-        std::list<std::string> monster_names_list;
-        UserStringList("MONSTER_NAMES", monster_names_list);
 
-        monster_names.reserve(monster_names_list.size());
-        std::copy(monster_names_list.begin(), monster_names_list.end(), std::back_inserter(monster_names));
-        if (monster_names.empty()) // safety check to ensure not leaving list empty in case of stringtable failure
-            monster_names.push_back(UserString("MONSTER"));
-    }
+    if (monster_names.empty())
+        monster_names.push_back(UserString("MONSTER"));
 
     // select name randomly from list
     int monster_name_index = RandSmallInt(0, static_cast<int>(monster_names.size()) - 1);
