@@ -9,10 +9,12 @@
 #include "Condition.h"
 #include "Effect.h"
 #include "Planet.h"
+#include "Ship.h"
 #include "Predicates.h"
 #include "Species.h"
 #include "Universe.h"
 #include "ValueRef.h"
+#include "Enums.h"
 
 #include <cfloat>
 #include <boost/filesystem/fstream.hpp>
@@ -21,12 +23,11 @@ using boost::io::str;
 
 namespace {
     const bool CHEAP_AND_FAST_SHIP_PRODUCTION = false;    // makes all ships cost 1 PP and take 1 turn to build
-}
+    const std::string EMPTY_STRING;
 
-namespace {
-    boost::shared_ptr<Effect::EffectsGroup>
+    std::shared_ptr<Effect::EffectsGroup>
     IncreaseMeter(MeterType meter_type, float increase) {
-        typedef boost::shared_ptr<Effect::EffectsGroup> EffectsGroupPtr;
+        typedef std::shared_ptr<Effect::EffectsGroup> EffectsGroupPtr;
         typedef std::vector<Effect::EffectBase*> Effects;
         Condition::Source* scope = new Condition::Source;
         Condition::Source* activation = new Condition::Source;
@@ -41,9 +42,9 @@ namespace {
                 scope, activation, Effects(1, new Effect::SetMeter(meter_type, vr))));
     }
 
-    boost::shared_ptr<Effect::EffectsGroup>
+    std::shared_ptr<Effect::EffectsGroup>
     IncreaseMeter(MeterType meter_type, const std::string& part_name, float increase, bool allow_stacking = true) {
-        typedef boost::shared_ptr<Effect::EffectsGroup> EffectsGroupPtr;
+        typedef std::shared_ptr<Effect::EffectsGroup> EffectsGroupPtr;
         typedef std::vector<Effect::EffectBase*> Effects;
         Condition::Source* scope = new Condition::Source;
         Condition::Source* activation = new Condition::Source;
@@ -108,7 +109,7 @@ const ShipDesign* GetShipDesign(int ship_design_id)
 // PartTypeManager                 //
 /////////////////////////////////////
 // static
-PartTypeManager* PartTypeManager::s_instance = 0;
+PartTypeManager* PartTypeManager::s_instance = nullptr;
 
 PartTypeManager::PartTypeManager() {
     if (s_instance)
@@ -125,22 +126,22 @@ PartTypeManager::PartTypeManager() {
 
     if (GetOptionsDB().Get<bool>("verbose-logging")) {
         DebugLogger() << "Part Types:";
-        for (iterator it = begin(); it != end(); ++it) {
-            const PartType* p = it->second;
+        for (const std::map<std::string, PartType*>::value_type& entry : m_parts) {
+            const PartType* p = entry.second;
             DebugLogger() << " ... " << p->Name() << " class: " << p->Class();
         }
     }
 }
 
 PartTypeManager::~PartTypeManager() {
-    for (std::map<std::string, PartType*>::iterator it = m_parts.begin(); it != m_parts.end(); ++it) {
-        delete it->second;
+    for (std::map<std::string, PartType*>::value_type& entry : m_parts) {
+        delete entry.second;
     }
 }
 
 const PartType* PartTypeManager::GetPartType(const std::string& name) const {
     std::map<std::string, PartType*>::const_iterator it = m_parts.find(name);
-    return it != m_parts.end() ? it->second : 0;
+    return it != m_parts.end() ? it->second : nullptr;
 }
 
 const PartTypeManager& PartTypeManager::GetPartTypeManager() {
@@ -155,37 +156,59 @@ PartTypeManager::iterator PartTypeManager::end() const
 { return m_parts.end(); }
 
 
-namespace {
-    // Looks like there are at least 3 SourceForEmpire functions lying around - one in ShipDesign, one in Tech, and one in Building
-    // TODO: Eliminate duplication
-    TemporaryPtr<const UniverseObject> SourceForEmpire(int empire_id) {
-        const Empire* empire = GetEmpire(empire_id);
-        if (!empire) {
-            DebugLogger() << "SourceForEmpire: Unable to get empire with ID: " << empire_id;
-            return TemporaryPtr<const UniverseObject>();
-        }
-        // get a source object, which is owned by the empire with the passed-in
-        // empire id.  this is used in conditions to reference which empire is
-        // doing the building.  Ideally this will be the capital, but any object
-        // owned by the empire will work.
-        TemporaryPtr<const UniverseObject> source = GetUniverseObject(empire->CapitalID());
-        // no capital?  scan through all objects to find one owned by this empire
-        if (!source) {
-            for (ObjectMap::const_iterator<> obj_it = Objects().const_begin(); obj_it != Objects().const_end(); ++obj_it) {
-                if (obj_it->OwnedBy(empire_id)) {
-                    source = *obj_it;
-                    break;
-                }
-            }
-        }
-        return source;
-    }
-}
-
 ////////////////////////////////////////////////
 // PartType
 ////////////////////////////////////////////////
-void PartType::Init(const std::vector<boost::shared_ptr<Effect::EffectsGroup> >& effects) {
+
+PartType::PartType() :
+    m_name("invalid part type"),
+    m_description("indescribable"),
+    m_class(INVALID_SHIP_PART_CLASS),
+    m_capacity(0.0f),
+    m_secondary_stat(1.0f),
+    m_production_cost(0),
+    m_production_time(0),
+    m_producible(false),
+    m_mountable_slot_types(),
+    m_tags(),
+    m_production_meter_consumption(),
+    m_production_special_consumption(),
+    m_location(0),
+    m_exclusions(),
+    m_effects(),
+    m_icon(),
+    m_add_standard_capacity_effect(false)
+{}
+
+PartType::PartType(ShipPartClass part_class, double capacity, double stat2,
+                   const CommonParams& common_params, const MoreCommonParams& more_common_params,
+                   std::vector<ShipSlotType> mountable_slot_types,
+                   const std::string& icon, bool add_standard_capacity_effect) :
+    m_name(more_common_params.name),
+    m_description(more_common_params.description),
+    m_class(part_class),
+    m_capacity(capacity),
+    m_secondary_stat(stat2),
+    m_production_cost(common_params.production_cost),
+    m_production_time(common_params.production_time),
+    m_producible(common_params.producible),
+    m_mountable_slot_types(mountable_slot_types),
+    m_tags(),
+    m_production_meter_consumption(common_params.production_meter_consumption),
+    m_production_special_consumption(common_params.production_special_consumption),
+    m_location(common_params.location),
+    m_exclusions(more_common_params.exclusions),
+    m_effects(),
+    m_icon(icon),
+    m_add_standard_capacity_effect(add_standard_capacity_effect)
+{
+    //std::cout << "part type: " << m_name << " producible: " << m_producible << std::endl;
+    Init(common_params.effects);
+    for (const std::string& tag : common_params.tags)
+        m_tags.insert(boost::to_upper_copy<std::string>(tag));
+}
+
+void PartType::Init(const std::vector<std::shared_ptr<Effect::EffectsGroup>>& effects) {
     if ((m_capacity != 0 || m_secondary_stat != 0) && m_add_standard_capacity_effect) {
         switch (m_class) {
         case PC_COLONY:
@@ -235,11 +258,9 @@ void PartType::Init(const std::vector<boost::shared_ptr<Effect::EffectsGroup> >&
         }
     }
 
-    for (std::vector<boost::shared_ptr<Effect::EffectsGroup> >::const_iterator
-         it = effects.begin(); it != effects.end(); ++it)
-    {
-        (*it)->SetTopLevelContent(m_name);
-        m_effects.push_back(*it);
+    for (std::shared_ptr<Effect::EffectsGroup> effect : effects) {
+        effect->SetTopLevelContent(m_name);
+        m_effects.push_back(effect);
     }
 }
 
@@ -286,8 +307,8 @@ std::string PartType::CapacityDescription() const {
 bool PartType::CanMountInSlotType(ShipSlotType slot_type) const {
     if (INVALID_SHIP_SLOT_TYPE == slot_type)
         return false;
-    for (std::vector<ShipSlotType>::const_iterator it = m_mountable_slot_types.begin(); it != m_mountable_slot_types.end(); ++it)
-        if (*it == slot_type)
+    for (ShipSlotType mountable_slot_type : m_mountable_slot_types)
+        if (mountable_slot_type == slot_type)
             return true;
     return false;
 }
@@ -306,16 +327,18 @@ float PartType::ProductionCost(int empire_id, int location_id) const {
     if (CHEAP_AND_FAST_SHIP_PRODUCTION || !m_production_cost) {
         return 1.0f;
     } else {
-        if (ValueRef::ConstantExpr(m_production_cost))
+        if (m_production_cost->ConstantExpr())
             return static_cast<float>(m_production_cost->Eval());
 
-        TemporaryPtr<UniverseObject> location = GetUniverseObject(location_id);
-        if (!location)
-            return 999999.9f;    // arbitrary large number
+        const auto arbitrary_large_number = 999999.9f;
 
-        TemporaryPtr<const UniverseObject> source = SourceForEmpire(empire_id);
+        std::shared_ptr<UniverseObject> location = GetUniverseObject(location_id);
+        if (!location)
+            return arbitrary_large_number;
+
+        std::shared_ptr<const UniverseObject> source = Empires().GetSource(empire_id);
         if (!source && !m_production_cost->SourceInvariant())
-            return 999999.9f;
+            return arbitrary_large_number;
 
         ScriptingContext context(source, location);
 
@@ -324,19 +347,21 @@ float PartType::ProductionCost(int empire_id, int location_id) const {
 }
 
 int PartType::ProductionTime(int empire_id, int location_id) const {
+    const auto arbitrary_large_number = 9999;
+
     if (CHEAP_AND_FAST_SHIP_PRODUCTION || !m_production_time) {
         return 1;
     } else {
-        if (ValueRef::ConstantExpr(m_production_time))
+        if (m_production_time->ConstantExpr())
             return m_production_time->Eval();
 
-        TemporaryPtr<UniverseObject> location = GetUniverseObject(location_id);
+        std::shared_ptr<UniverseObject> location = GetUniverseObject(location_id);
         if (!location)
-            return 9999;    // arbitrary large number
+            return arbitrary_large_number;
 
-        TemporaryPtr<const UniverseObject> source = SourceForEmpire(empire_id);
+        std::shared_ptr<const UniverseObject> source = Empires().GetSource(empire_id);
         if (!source && !m_production_time->SourceInvariant())
-            return 9999;
+            return arbitrary_large_number;
 
         ScriptingContext context(source, location);
 
@@ -348,7 +373,11 @@ int PartType::ProductionTime(int empire_id, int location_id) const {
 ////////////////////////////////////////////////
 // HullType
 ////////////////////////////////////////////////
-void HullType::Init(const std::vector<boost::shared_ptr<Effect::EffectsGroup> >& effects) {
+HullType::Slot::Slot() :
+    type(INVALID_SHIP_SLOT_TYPE), x(0.5), y(0.5)
+{}
+
+void HullType::Init(const std::vector<std::shared_ptr<Effect::EffectsGroup>>& effects) {
     if (m_fuel != 0)
         m_effects.push_back(IncreaseMeter(METER_MAX_FUEL,       m_fuel));
     if (m_stealth != 0)
@@ -358,11 +387,9 @@ void HullType::Init(const std::vector<boost::shared_ptr<Effect::EffectsGroup> >&
     if (m_speed != 0)
         m_effects.push_back(IncreaseMeter(METER_SPEED,          m_speed));
 
-    for (std::vector<boost::shared_ptr<Effect::EffectsGroup> >::const_iterator it = effects.begin();
-         it != effects.end(); ++it)
-    {
-        (*it)->SetTopLevelContent(m_name);
-        m_effects.push_back(*it);
+    for (std::shared_ptr<Effect::EffectsGroup> effect : effects) {
+        effect->SetTopLevelContent(m_name);
+        m_effects.push_back(effect);
     }
 }
 
@@ -371,8 +398,8 @@ HullType::~HullType()
 
 unsigned int HullType::NumSlots(ShipSlotType slot_type) const {
     unsigned int count = 0;
-    for (std::vector<Slot>::const_iterator it = m_slots.begin(); it != m_slots.end(); ++it)
-        if (it->type == slot_type)
+    for (const Slot& slot : m_slots)
+        if (slot.type == slot_type)
             ++count;
     return count;
 }
@@ -395,16 +422,18 @@ float HullType::ProductionCost(int empire_id, int location_id) const {
     if (CHEAP_AND_FAST_SHIP_PRODUCTION || !m_production_cost) {
         return 1.0f;
     } else {
-        if (ValueRef::ConstantExpr(m_production_cost))
+        if (m_production_cost->ConstantExpr())
             return static_cast<float>(m_production_cost->Eval());
 
-        TemporaryPtr<UniverseObject> location = GetUniverseObject(location_id);
-        if (!location)
-            return 999999.9f;    // arbitrary large number
+        const auto arbitrary_large_number = 999999.9f;
 
-        TemporaryPtr<const UniverseObject> source = SourceForEmpire(empire_id);
+        std::shared_ptr<UniverseObject> location = GetUniverseObject(location_id);
+        if (!location)
+            return arbitrary_large_number;
+
+        std::shared_ptr<const UniverseObject> source = Empires().GetSource(empire_id);
         if (!source && !m_production_cost->SourceInvariant())
-            return 999999.9f;
+            return arbitrary_large_number;
 
         ScriptingContext context(source, location);
 
@@ -416,16 +445,18 @@ int HullType::ProductionTime(int empire_id, int location_id) const {
     if (CHEAP_AND_FAST_SHIP_PRODUCTION || !m_production_time) {
         return 1;
     } else {
-        if (ValueRef::ConstantExpr(m_production_time))
+        if (m_production_time->ConstantExpr())
             return m_production_time->Eval();
 
-        TemporaryPtr<UniverseObject> location = GetUniverseObject(location_id);
-        if (!location)
-            return 9999;    // arbitrary large number
+        const auto arbitrary_large_number = 999999;
 
-        TemporaryPtr<const UniverseObject> source = SourceForEmpire(empire_id);
+        std::shared_ptr<UniverseObject> location = GetUniverseObject(location_id);
+        if (!location)
+            return arbitrary_large_number;
+
+        std::shared_ptr<const UniverseObject> source = Empires().GetSource(empire_id);
         if (!source && !m_production_time->SourceInvariant())
-            return 999999;
+            return arbitrary_large_number;
 
         ScriptingContext context(source, location);
 
@@ -438,7 +469,7 @@ int HullType::ProductionTime(int empire_id, int location_id) const {
 // HullTypeManager                 //
 /////////////////////////////////////
 // static
-HullTypeManager* HullTypeManager::s_instance = 0;
+HullTypeManager* HullTypeManager::s_instance = nullptr;
 
 HullTypeManager::HullTypeManager() {
     if (s_instance)
@@ -455,22 +486,22 @@ HullTypeManager::HullTypeManager() {
 
     if (GetOptionsDB().Get<bool>("verbose-logging")) {
         DebugLogger() << "Hull Types:";
-        for (iterator it = begin(); it != end(); ++it) {
-            const HullType* h = it->second;
+        for (const std::map<std::string, HullType*>::value_type& entry : m_hulls) {
+            const HullType* h = entry.second;
             DebugLogger() << " ... " << h->Name();
         }
     }
 }
 
 HullTypeManager::~HullTypeManager() {
-    for (std::map<std::string, HullType*>::iterator it = m_hulls.begin(); it != m_hulls.end(); ++it) {
-        delete it->second;
+    for (std::map<std::string, HullType*>::value_type& entry : m_hulls) {
+        delete entry.second;
     }
 }
 
 const HullType* HullTypeManager::GetHullType(const std::string& name) const {
     std::map<std::string, HullType*>::const_iterator it = m_hulls.find(name);
-    return it != m_hulls.end() ? it->second : 0;
+    return it != m_hulls.end() ? it->second : nullptr;
 }
 
 const HullTypeManager& HullTypeManager::GetHullTypeManager() {
@@ -505,6 +536,7 @@ ShipDesign::ShipDesign() :
     m_3D_model(),
     m_name_desc_in_stringtable(false),
     m_is_armed(false),
+    m_has_fighters(false),
     m_can_bombard(false),
     m_detection(0.0),
     m_colony_capacity(0.0),
@@ -538,6 +570,7 @@ ShipDesign::ShipDesign(const std::string& name, const std::string& description,
     m_3D_model(model),
     m_name_desc_in_stringtable(name_desc_in_stringtable),
     m_is_armed(false),
+    m_has_fighters(false),
     m_can_bombard(false),
     m_detection(0.0),
     m_colony_capacity(0.0),
@@ -602,8 +635,8 @@ bool ShipDesign::ProductionCostTimeLocationInvariant() const {
     if (const HullType* hull = GetHullType(m_hull))
         if (!hull->ProductionCostTimeLocationInvariant())
             return false;
-    for (std::vector<std::string>::const_iterator it = m_parts.begin(); it != m_parts.end(); ++it)
-        if (const PartType* part = GetPartType(*it))
+    for (const std::string& part_name : m_parts)
+        if (const PartType* part = GetPartType(part_name))
             if (!part->ProductionCostTimeLocationInvariant())
                 return false;
 
@@ -618,8 +651,8 @@ float ShipDesign::ProductionCost(int empire_id, int location_id) const {
         float cost_accumulator = 0.0f;
         if (const HullType* hull = GetHullType(m_hull))
             cost_accumulator += hull->ProductionCost(empire_id, location_id);
-        for (std::vector<std::string>::const_iterator it = m_parts.begin(); it != m_parts.end(); ++it)
-            if (const PartType* part = GetPartType(*it))
+        for (const std::string& part_name : m_parts)
+            if (const PartType* part = GetPartType(part_name))
                 cost_accumulator += part->ProductionCost(empire_id, location_id);
         return std::max(0.0f, cost_accumulator);
     }
@@ -635,16 +668,15 @@ int ShipDesign::ProductionTime(int empire_id, int location_id) const {
         int time_accumulator = 1;
         if (const HullType* hull = GetHullType(m_hull))
             time_accumulator = std::max(time_accumulator, hull->ProductionTime(empire_id, location_id));
-        for (std::vector<std::string>::const_iterator it = m_parts.begin(); it != m_parts.end(); ++it)
-            if (const PartType* part = GetPartType(*it))
+        for (const std::string& part_name : m_parts)
+            if (const PartType* part = GetPartType(part_name))
                 time_accumulator = std::max(time_accumulator, part->ProductionTime(empire_id, location_id));
         return std::max(1, time_accumulator);
     }
 }
 
 bool ShipDesign::CanColonize() const {
-    for (std::vector<std::string>::const_iterator it = m_parts.begin(); it != m_parts.end(); ++it) {
-        const std::string& part_name = *it;
+    for (const std::string& part_name : m_parts) {
         if (part_name.empty())
             continue;
         if (const PartType* part = GetPartType(part_name))
@@ -658,9 +690,8 @@ float ShipDesign::Defense() const {
     // accumulate defense from defensive parts in design.
     float total_defense = 0.0f;
     const PartTypeManager& part_manager = GetPartTypeManager();
-    std::vector<std::string> all_parts = Parts();
-    for (std::vector<std::string>::const_iterator it = all_parts.begin(); it != all_parts.end(); ++it) {
-        const PartType* part = part_manager.GetPartType(*it);
+    for (const std::string& part_name : Parts()) {
+        const PartType* part = part_manager.GetPartType(part_name);
         if (part && (part->Class() == PC_SHIELD || part->Class() == PC_ARMOUR))
             total_defense += part->Capacity();
     }
@@ -680,8 +711,8 @@ float ShipDesign::AdjustedAttack(float shield) const {
     float fighter_damage = 0.0f;
     float direct_attack = 0.0f;
 
-    for (std::vector<std::string>::const_iterator it = m_parts.begin(); it != m_parts.end(); ++it) {
-        const PartType* part = GetPartType(*it);
+    for (const std::string& part_name : m_parts) {
+        const PartType* part = GetPartType(part_name);
         if (!part)
             continue;
         ShipPartClass part_class = part->Class();
@@ -692,23 +723,30 @@ float ShipDesign::AdjustedAttack(float shield) const {
             if (part_attack > shield)
                 direct_attack += (part_attack - shield)*part->SecondaryStat();  // here, secondary stat is number of shots per round
         } else if (part_class == PC_FIGHTER_HANGAR) {
-            available_fighters += part->Capacity();
+            available_fighters = part->Capacity();                              // stacked meter
         } else if (part_class == PC_FIGHTER_BAY) {
             fighter_launch_capacity += part->Capacity();
             fighter_damage = part->SecondaryStat();                             // here, secondary stat is fighter damage per shot
         }
     }
 
-    // assuming 3 combat "bouts"
     int fighter_shots = std::min(available_fighters, fighter_launch_capacity);  // how many fighters launched in bout 1
     available_fighters -= fighter_shots;
-    fighter_shots *= 2;                                                         // first-bout-launched fighters attack in bouts 2 and 3
-    fighter_shots += std::min(available_fighters, fighter_launch_capacity);     // second-bout-launched fighters attack only in bout 3
+    int launched_fighters = fighter_shots;
+    int num_bouts = GetUniverse().GetNumCombatRounds();
+    int remaining_bouts = num_bouts - 2;  // no attack for first round, second round already added
+    while (remaining_bouts > 0) {
+        int fighters_launched_this_bout = std::min(available_fighters, fighter_launch_capacity);
+        available_fighters -= fighters_launched_this_bout;
+        launched_fighters += fighters_launched_this_bout;
+        fighter_shots += launched_fighters;
+        --remaining_bouts;
+    }
 
     // how much damage does a fighter shot do?
-    fighter_damage = std::max(0.0f, fighter_damage - shield);
+    fighter_damage = std::max(0.0f, fighter_damage);
 
-    return direct_attack + fighter_shots*fighter_damage/3.0f;   // divide by 3 because fighter calculation is for a full combat, but direct firefor one attack
+    return direct_attack + fighter_shots*fighter_damage/num_bouts;   // divide by bouts because fighter calculation is for a full combat, but direct firefor one attack
 }
 
 std::vector<std::string> ShipDesign::Parts(ShipSlotType slot_type) const {
@@ -735,8 +773,7 @@ std::vector<std::string> ShipDesign::Parts(ShipSlotType slot_type) const {
 std::vector<std::string> ShipDesign::Weapons() const {
     std::vector<std::string> retval;
     retval.reserve(m_parts.size());
-    for (unsigned int i = 0; i < m_parts.size(); ++i) {
-        const std::string& part_name = m_parts[i];
+    for (const std::string& part_name : m_parts) {
         const PartType* part = GetPartType(part_name);
         if (!part)
             continue;
@@ -748,40 +785,36 @@ std::vector<std::string> ShipDesign::Weapons() const {
 }
 
 bool ShipDesign::ProductionLocation(int empire_id, int location_id) const {
-    TemporaryPtr<const UniverseObject> location = GetUniverseObject(location_id);
-    if (!location)
-        return false;
-
-    // currently ships can only be built at planets, and by species that are
-    // not planetbound
-    TemporaryPtr<const Planet> planet = boost::dynamic_pointer_cast<const Planet>(location);
-    if (!planet)
-        return false;
-    const std::string& species_name = planet->SpeciesName();
-    if (species_name.empty())
-        return false;
-    const Species* species = GetSpecies(species_name);
-    if (!species)
-        return false;
-    if (!species->CanProduceShips())
-        return false;
-    // also, species that can't colonize can't produce colony ships
-    if (this->CanColonize() && !species->CanColonize())
-        return false;
-
     Empire* empire = GetEmpire(empire_id);
     if (!empire) {
         DebugLogger() << "ShipDesign::ProductionLocation: Unable to get pointer to empire " << empire_id;
         return false;
     }
 
-    // get a source object, which is owned by the empire with the passed-in
-    // empire id.  this is used in conditions to reference which empire is
-    // doing the producing.  Ideally this will be the capital, but any object
-    // owned by the empire will work.
-    TemporaryPtr<const UniverseObject> source = SourceForEmpire(empire_id);
-    // if this empire doesn't own ANYTHING, then how is it producing anyway?
-    if (!source)
+    // must own the production location...
+    std::shared_ptr<const UniverseObject> location = GetUniverseObject(location_id);
+    if (!location->OwnedBy(empire_id))
+        return false;
+
+    std::shared_ptr<const Planet> planet = std::dynamic_pointer_cast<const Planet>(location);
+    std::shared_ptr<const Ship> ship;
+    if (!planet)
+        ship = std::dynamic_pointer_cast<const Ship>(location);
+    if (!planet && !ship)
+        return false;
+
+    // ships can only be produced by species that are not planetbound
+    const std::string& species_name = planet ? planet->SpeciesName() : (ship ? ship->SpeciesName() : EMPTY_STRING);
+    if (species_name.empty())
+        return false;
+    const Species* species = GetSpecies(species_name);
+    if (!species)
+        return false;
+
+    if (!species->CanProduceShips())
+        return false;
+    // also, species that can't colonize can't produce colony ships
+    if (this->CanColonize() && !species->CanColonize())
         return false;
 
     // apply hull location conditions to potential location
@@ -790,12 +823,13 @@ bool ShipDesign::ProductionLocation(int empire_id, int location_id) const {
         ErrorLogger() << "ShipDesign::ProductionLocation  ShipDesign couldn't get its own hull with name " << m_hull;
         return false;
     }
-    if (!hull->Location()->Eval(ScriptingContext(source), location))
+    // evaluate using location as the source, as it should be an object owned by this empire.
+    ScriptingContext location_as_source_context(location);
+    if (!hull->Location()->Eval(location_as_source_context, location))
         return false;
 
     // apply external and internal parts' location conditions to potential location
-    for (std::vector<std::string>::const_iterator part_it = m_parts.begin(); part_it != m_parts.end(); ++part_it) {
-        std::string part_name = *part_it;
+    for (const std::string& part_name : m_parts) {
         if (part_name.empty())
             continue;       // empty slots don't limit build location
 
@@ -804,7 +838,7 @@ bool ShipDesign::ProductionLocation(int empire_id, int location_id) const {
             ErrorLogger() << "ShipDesign::ProductionLocation  ShipDesign couldn't get part with name " << part_name;
             return false;
         }
-        if (!part->Location()->Eval(ScriptingContext(source), location))
+        if (!part->Location()->Eval(location_as_source_context, location))
             return false;
     }
     // location matched all hull and part conditions, so is a valid build location
@@ -832,26 +866,25 @@ bool ShipDesign::ValidDesign(const std::string& hull, const std::vector<std::str
 
     // check hull exclusions against all parts...
     const std::set<std::string>& hull_exclusions = hull_type->Exclusions();
-    for (std::vector<std::string>::const_iterator part_it = parts.begin(); part_it != parts.end(); ++part_it) {
-        if (part_it->empty())
+    for (const std::string& part_name : parts) {
+        if (part_name.empty())
             continue;
-        if (hull_exclusions.find(*part_it) != hull_exclusions.end())
+        if (hull_exclusions.find(part_name) != hull_exclusions.end())
             return false;
     }
 
     // check part exclusions against other parts and hull
     std::set<std::string> already_seen_component_names;
     already_seen_component_names.insert(hull);
-    for (std::vector<std::string>::const_iterator part_it = parts.begin(); part_it != parts.end(); ++part_it) {
-        const PartType* part_type = GetPartType(*part_it);
+    for (const std::string& part_name : parts) {
+        const PartType* part_type = GetPartType(part_name);
         if (!part_type)
             continue;
-        const std::set<std::string>& part_exclusions = part_type->Exclusions();
-        for (std::set<std::string>::const_iterator ex_it = part_exclusions.begin(); ex_it != part_exclusions.end(); ++ex_it) {
-            if (already_seen_component_names.find(*ex_it) != already_seen_component_names.end())
+        for (const std::string& excluded : part_type->Exclusions()) {
+            if (already_seen_component_names.find(excluded) != already_seen_component_names.end())
                 return false;
         }
-        already_seen_component_names.insert(*part_it);
+        already_seen_component_names.insert(part_name);
     }
 
 
@@ -895,25 +928,29 @@ void ShipDesign::BuildStatCaches() {
     m_structure =       hull->Structure();
     m_speed =           hull->Speed();
 
-    for (std::vector<std::string>::const_iterator it = m_parts.begin(); it != m_parts.end(); ++it) {
-        if (it->empty())
+    for (const std::string& part_name : m_parts) {
+        if (part_name.empty())
             continue;
 
-        const PartType* part = GetPartType(*it);
+        const PartType* part = GetPartType(part_name);
         if (!part) {
-            ErrorLogger() << "ShipDesign::BuildStatCaches couldn't get part with name " << *it;
+            ErrorLogger() << "ShipDesign::BuildStatCaches couldn't get part with name " << part_name;
             continue;
         }
 
         if (!part->Producible())
             m_producible = false;
 
-        switch (part->Class()) {
+        ShipPartClass part_class = part->Class();
+
+        switch (part_class) {
         case PC_DIRECT_WEAPON:
-        case PC_FIGHTER_BAY: {
             m_is_armed = true;
             break;
-        }
+        case PC_FIGHTER_BAY:
+        case PC_FIGHTER_HANGAR:
+            m_has_fighters = true;
+            break;
         case PC_COLONY:
             m_colony_capacity += part->Capacity();
             break;
@@ -950,12 +987,16 @@ void ShipDesign::BuildStatCaches() {
         case PC_TRADE:
             m_trade_generation += part->Capacity();
             break;
-        case PC_PRODICTION_LOCATION:
+        case PC_PRODUCTION_LOCATION:
             m_is_production_location = true;
             break;
         default:
             break;
         }
+
+        m_num_part_types[part_name]++;
+        if (part_class > INVALID_SHIP_PART_CLASS && part_class < NUM_SHIP_PART_CLASSES)
+            m_num_part_classes[part_class]++;
     }
 }
 
@@ -976,8 +1017,8 @@ std::string ShipDesign::Dump() const {
     } else {
         retval += "[\n";
         ++g_indent;
-        for (std::vector<std::string>::const_iterator it = m_parts.begin(); it != m_parts.end(); ++it) {
-            retval += DumpIndent() + "\"" + *it + "\"\n";
+        for (const std::string& part_name : m_parts) {
+            retval += DumpIndent() + "\"" + part_name + "\"\n";
         }
         --g_indent;
         retval += DumpIndent() + "]\n";
@@ -996,13 +1037,11 @@ bool operator ==(const ShipDesign& first, const ShipDesign& second) {
     std::map<std::string, int> first_parts;
     std::map<std::string, int> second_parts;
 
-    for (std::vector<std::string>::const_iterator it = first.Parts().begin();
-         it != first.Parts().end(); ++it)
-    { ++first_parts[*it]; }
+    for (const std::string& part_name : first.Parts())
+    { ++first_parts[part_name]; }
 
-    for (std::vector<std::string>::const_iterator it = second.Parts().begin();
-         it != second.Parts().end(); ++it)
-    { ++second_parts[*it]; }
+    for (const std::string& part_name : second.Parts())
+    { ++second_parts[part_name]; }
 
     return first_parts == second_parts;
 }
@@ -1012,7 +1051,7 @@ bool operator ==(const ShipDesign& first, const ShipDesign& second) {
 // PredefinedShipDesignManager     //
 /////////////////////////////////////
 // static(s)
-PredefinedShipDesignManager* PredefinedShipDesignManager::s_instance = 0;
+PredefinedShipDesignManager* PredefinedShipDesignManager::s_instance = nullptr;
 
 PredefinedShipDesignManager::PredefinedShipDesignManager() {
     if (s_instance)
@@ -1038,21 +1077,21 @@ PredefinedShipDesignManager::PredefinedShipDesignManager() {
 
     if (GetOptionsDB().Get<bool>("verbose-logging")) {
         DebugLogger() << "Predefined Ship Designs:";
-        for (iterator it = begin(); it != end(); ++it) {
-            const ShipDesign* d = it->second;
+        for (const std::map<std::string, ShipDesign*>::value_type& entry : m_ship_designs) {
+            const ShipDesign* d = entry.second;
             DebugLogger() << " ... " << d->Name();
         }
         DebugLogger() << "Monster Ship Designs:";
-        for (iterator it = begin_monsters(); it != end_monsters(); ++it) {
-            const ShipDesign* d = it->second;
+        for (const std::map<std::string, ShipDesign*>::value_type& entry : m_monster_designs) {
+            const ShipDesign* d = entry.second;
             DebugLogger() << " ... " << d->Name();
         }
     }
 }
 
 PredefinedShipDesignManager::~PredefinedShipDesignManager() {
-    for (std::map<std::string, ShipDesign*>::iterator it = m_ship_designs.begin(); it != m_ship_designs.end(); ++it)
-        delete it->second;
+    for (std::map<std::string, ShipDesign*>::value_type& entry : m_ship_designs)
+        delete entry.second;
 }
 
 void PredefinedShipDesignManager::AddShipDesignsToEmpire(Empire* empire,
@@ -1063,8 +1102,7 @@ void PredefinedShipDesignManager::AddShipDesignsToEmpire(Empire* empire,
     int empire_id = empire->EmpireID();
     Universe& universe = GetUniverse();
 
-    for (std::vector<std::string>::const_iterator it = design_names.begin(); it != design_names.end(); ++it) {
-        const std::string& design_name = *it;
+    for (const std::string& design_name : design_names) {
         std::map<std::string, ShipDesign*>::const_iterator design_it = m_ship_designs.find(design_name);
         if (design_it == m_ship_designs.end()) {
             ErrorLogger() << "Couldn't find predefined ship design with name " << design_name << " to add to empire";
@@ -1150,13 +1188,13 @@ namespace {
 const std::map<std::string, int>& PredefinedShipDesignManager::AddShipDesignsToUniverse() const {
     m_design_generic_ids.clear();   // std::map<std::string, int>
 
-    for (iterator it = begin(); it != end(); ++it) {
-        ShipDesign* d = it->second;
+    for (const std::map<std::string, ShipDesign*>::value_type& entry : m_ship_designs) {
+        ShipDesign* d = entry.second;
         AddDesignToUniverse(m_design_generic_ids, d, false);
     }
 
-    for (iterator it = begin_monsters(); it != end_monsters(); ++it) {
-        ShipDesign* d = it->second;
+    for (const std::map<std::string, ShipDesign*>::value_type& entry : m_monster_designs) {
+        ShipDesign* d = entry.second;
         AddDesignToUniverse(m_design_generic_ids, d, true);
     }
 
