@@ -1,23 +1,25 @@
 #ifndef _ShipDesign_h_
 #define _ShipDesign_h_
 
+
 #include "ValueRefFwd.h"
 
+#include <map>
+#include <memory>
+#include <set>
 #include <string>
 #include <vector>
-#include <map>
-#include <set>
 
 #include <boost/algorithm/string/case_conv.hpp>
-#include <boost/shared_ptr.hpp>
 #include <boost/variant.hpp>
 #include <boost/serialization/access.hpp>
 #include <boost/serialization/nvp.hpp>
 
-#include "Enums.h"
+#include "EnumsFwd.h"
 
 #include "../util/Export.h"
 
+
 namespace Condition {
     struct ConditionBase;
 }
@@ -30,14 +32,14 @@ class Empire;
   * storage for parsing to reduce number of sub-items parsed per item. */
 struct CommonParams {
     CommonParams() :
-        production_cost(0),
-        production_time(0),
+        production_cost(nullptr),
+        production_time(nullptr),
         producible(false),
         tags(),
         production_meter_consumption(),
         production_special_consumption(),
-        location(0),
-        enqueue_location(0),
+        location(nullptr),
+        enqueue_location(nullptr),
         effects()
     {}
     CommonParams(ValueRef::ValueRefBase<double>* production_cost_,
@@ -45,9 +47,9 @@ struct CommonParams {
                  bool producible_,
                  const std::set<std::string>& tags_,
                  Condition::ConditionBase* location_,
-                 const std::vector<boost::shared_ptr<Effect::EffectsGroup> >& effects_,
-                 std::map<MeterType, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> > production_meter_consumption_,
-                 std::map<std::string, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> > production_special_consumption_,
+                 const std::vector<std::shared_ptr<Effect::EffectsGroup>>& effects_,
+                 std::map<MeterType, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*>> production_meter_consumption_,
+                 std::map<std::string, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*>> production_special_consumption_,
                  Condition::ConditionBase* enqueue_location_) :
         production_cost(production_cost_),
         production_time(production_time_),
@@ -59,21 +61,21 @@ struct CommonParams {
         enqueue_location(enqueue_location_),
         effects(effects_)
     {
-        for (std::set< std::string >::iterator tag_it = tags_.begin(); tag_it != tags_.end(); ++tag_it)
-            tags.insert(boost::to_upper_copy<std::string>(*tag_it));
+        for (const std::string& tag : tags_)
+            tags.insert(boost::to_upper_copy<std::string>(tag));
     }
 
     ValueRef::ValueRefBase<double>*                         production_cost;
     ValueRef::ValueRefBase<int>*                            production_time;
     bool                                                    producible;
     std::set<std::string>                                   tags;
-    std::map<MeterType, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >
+    std::map<MeterType, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*>>
                                                             production_meter_consumption;
-    std::map<std::string, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >
+    std::map<std::string, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*>>
                                                             production_special_consumption;
     Condition::ConditionBase*                               location;
     Condition::ConditionBase*                               enqueue_location;
-    std::vector<boost::shared_ptr<Effect::EffectsGroup> >   effects;
+    std::vector<std::shared_ptr<Effect::EffectsGroup>> effects;
 };
 
 struct MoreCommonParams {
@@ -97,55 +99,11 @@ struct MoreCommonParams {
 class FO_COMMON_API PartType {
 public:
     /** \name Structors */ //@{
-    PartType() :
-        m_name("invalid part type"),
-        m_description("indescribable"),
-        m_class(INVALID_SHIP_PART_CLASS),
-        m_capacity(0.0f),
-        m_secondary_stat(1.0f),
-        m_production_cost(0),
-        m_production_time(0),
-        m_producible(false),
-        m_mountable_slot_types(),
-        m_tags(),
-        m_production_meter_consumption(),
-        m_production_special_consumption(),
-        m_location(0),
-        m_exclusions(),
-        m_effects(),
-        m_icon(),
-        m_add_standard_capacity_effect(false)
-    {}
-
+    PartType();
     PartType(ShipPartClass part_class, double capacity, double stat2,
-             const CommonParams& common_params,
-             const MoreCommonParams& more_common_params,
+             const CommonParams& common_params, const MoreCommonParams& more_common_params,
              std::vector<ShipSlotType> mountable_slot_types,
-             const std::string& icon,
-             bool add_standard_capacity_effect = true) :
-        m_name(more_common_params.name),
-        m_description(more_common_params.description),
-        m_class(part_class),
-        m_capacity(capacity),
-        m_secondary_stat(stat2),
-        m_production_cost(common_params.production_cost),
-        m_production_time(common_params.production_time),
-        m_producible(common_params.producible),
-        m_mountable_slot_types(mountable_slot_types),
-        m_tags(),
-        m_production_meter_consumption(common_params.production_meter_consumption),
-        m_production_special_consumption(common_params.production_special_consumption),
-        m_location(common_params.location),
-        m_exclusions(more_common_params.exclusions),
-        m_effects(),
-        m_icon(icon),
-        m_add_standard_capacity_effect(add_standard_capacity_effect)
-    {
-        //std::cout << "part type: " << m_name << " producible: " << m_producible << std::endl;
-        Init(common_params.effects);
-        for (std::set<std::string>::iterator tag_it = common_params.tags.begin(); tag_it != common_params.tags.end(); ++tag_it)
-            m_tags.insert(boost::to_upper_copy<std::string>(*tag_it));
-    }
+             const std::string& icon, bool add_standard_capacity_effect = true);
 
     ~PartType();
     //@}
@@ -167,21 +125,24 @@ public:
     int                     ProductionTime(int empire_id, int location_id) const;   ///< returns the number of turns required to produce this part
     bool                    Producible() const      { return m_producible; }        ///< returns whether this part type is producible by players and appears on the design screen
 
-    const std::map<MeterType, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >&
+    const std::map<MeterType, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*>>&
                             ProductionMeterConsumption() const  { return m_production_meter_consumption; }
-    const std::map<std::string, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >&
+    const std::map<std::string, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*>>&
                             ProductionSpecialConsumption() const{ return m_production_special_consumption; }
 
     const std::set<std::string>& Tags() const       { return m_tags; }
     const Condition::ConditionBase* Location() const{ return m_location; }          ///< returns the condition that determines the locations where ShipDesign containing part can be produced
     const std::set<std::string>& Exclusions() const { return m_exclusions; }        ///< returns the names of other content that cannot be used in the same ship design as this part
-    const std::vector<boost::shared_ptr<Effect::EffectsGroup> >& Effects() const
-                                                    { return m_effects; }           ///< returns the EffectsGroups that encapsulate the effects this part has
+
+    /** Returns the EffectsGroups that encapsulate the effects this part has. */
+    const std::vector<std::shared_ptr<Effect::EffectsGroup>>& Effects() const
+    { return m_effects; }
+
     const std::string&      Icon() const            { return m_icon; }              ///< returns icon graphic that represents part in UI
     //@}
 
 private:
-    void                    Init(const std::vector<boost::shared_ptr<Effect::EffectsGroup> >& effects);
+    void Init(const std::vector<std::shared_ptr<Effect::EffectsGroup>>& effects);
 
     std::string                                             m_name;
     std::string                                             m_description;
@@ -193,13 +154,13 @@ private:
     bool                                                    m_producible;
     std::vector<ShipSlotType>                               m_mountable_slot_types;
     std::set<std::string>                                   m_tags;
-    std::map<MeterType, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >
+    std::map<MeterType, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*>>
                                                             m_production_meter_consumption;
-    std::map<std::string, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >
+    std::map<std::string, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*>>
                                                             m_production_special_consumption;
     Condition::ConditionBase*                               m_location;
     std::set<std::string>                                   m_exclusions;
-    std::vector<boost::shared_ptr<Effect::EffectsGroup> >   m_effects;
+    std::vector<std::shared_ptr<Effect::EffectsGroup>> m_effects;
     std::string                                             m_icon;
     bool                                                    m_add_standard_capacity_effect;
 
@@ -283,9 +244,7 @@ struct HullTypeStats {
 class FO_COMMON_API HullType {
 public:
     struct Slot {
-        Slot() :
-            type(INVALID_SHIP_SLOT_TYPE), x(0.5), y(0.5)
-        {}
+        Slot();
         Slot(ShipSlotType slot_type, double x_, double y_) :
             type(slot_type), x(x_), y(y_)
         {}
@@ -301,14 +260,14 @@ public:
         m_fuel(0.0f),
         m_stealth(0.0f),
         m_structure(0.0f),
-        m_production_cost(0),
-        m_production_time(0),
+        m_production_cost(nullptr),
+        m_production_time(nullptr),
         m_producible(false),
         m_slots(),
         m_tags(),
         m_production_meter_consumption(),
         m_production_special_consumption(),
-        m_location(0),
+        m_location(nullptr),
         m_effects(),
         m_graphic(),
         m_icon()
@@ -339,8 +298,8 @@ public:
     {
         //std::cout << "hull type: " << m_name << " producible: " << m_producible << std::endl;
         Init(common_params.effects);
-        for (std::set< std::string >::iterator tag_it = common_params.tags.begin(); tag_it != common_params.tags.end(); ++tag_it)
-            m_tags.insert(boost::to_upper_copy<std::string>(*tag_it));
+        for (const std::string& tag : common_params.tags)
+            m_tags.insert(boost::to_upper_copy<std::string>(tag));
     }
 
     ~HullType();
@@ -364,9 +323,9 @@ public:
     int                 ProductionTime(int empire_id, int location_id) const;   ///< returns the number of turns required to produce this hull
     bool                Producible() const      { return m_producible; }        ///< returns whether this hull type is producible by players and appears on the design screen
 
-    const std::map<MeterType, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >&
+    const std::map<MeterType, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*>>&
                             ProductionMeterConsumption() const  { return m_production_meter_consumption; }
-    const std::map<std::string, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >&
+    const std::map<std::string, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*>>&
                             ProductionSpecialConsumption() const{ return m_production_special_consumption; }
 
     unsigned int        NumSlots() const        { return m_slots.size(); }      ///< returns total number of of slots in hull
@@ -380,15 +339,18 @@ public:
     const Condition::ConditionBase* Location() const
     { return m_location; }                                                      ///< returns the condition that determines the locations where ShipDesign containing hull can be produced
     const std::set<std::string>& Exclusions() const { return m_exclusions; }    ///< returns the names of other content that cannot be used in the same ship design as this part
-    const std::vector<boost::shared_ptr<Effect::EffectsGroup> >& Effects() const
-    { return m_effects; }                                                       ///< returns the EffectsGroups that encapsulate the effects this part hull has
+
+    /** Returns the EffectsGroups that encapsulate the effects this part hull
+        has. */
+    const std::vector<std::shared_ptr<Effect::EffectsGroup>>& Effects() const
+    { return m_effects; }
 
     const std::string&  Graphic() const         { return m_graphic; }           ///< returns the image that represents the hull on the design screen
     const std::string&  Icon() const            { return m_icon; }              ///< returns the small icon to represent hull
     //@}
 
 private:
-    void                Init(const std::vector<boost::shared_ptr<Effect::EffectsGroup> >& effects);
+    void Init(const std::vector<std::shared_ptr<Effect::EffectsGroup>>& effects);
 
     std::string                                             m_name;
     std::string                                             m_description;
@@ -401,13 +363,13 @@ private:
     bool                                                    m_producible;
     std::vector<Slot>                                       m_slots;
     std::set<std::string>                                   m_tags;
-    std::map<MeterType, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >
+    std::map<MeterType, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*>>
                                                             m_production_meter_consumption;
-    std::map<std::string, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*> >
+    std::map<std::string, std::pair<ValueRef::ValueRefBase<double>*, Condition::ConditionBase*>>
                                                             m_production_special_consumption;
     Condition::ConditionBase*                               m_location;
     std::set<std::string>                                   m_exclusions;
-    std::vector<boost::shared_ptr<Effect::EffectsGroup> >   m_effects;
+    std::vector<std::shared_ptr<Effect::EffectsGroup>> m_effects;
     std::string                                             m_graphic;
     std::string                                             m_icon;
 
@@ -496,15 +458,16 @@ public:
     float                           TroopCapacity() const   { return m_troop_capacity; }        ///< returns the troop capacity of this design
     float                           Stealth() const         { return m_stealth; }               ///< returns the stealth of this design
 
-    float                           IndustryGeneration() const { return m_industry_generation; }///< returns the industry output from this ship design
-    float                           ResearchGeneration() const { return m_research_generation; }///< returns the research output from this ship design
-    float                           TradeGeneration() const    { return m_trade_generation; }   ///< returns the trade output from this ship design
-    bool                            IsProductionLocation() const { return m_is_production_location;}///< returns true if this ship design can be a production location
+    float                           IndustryGeneration() const  { return m_industry_generation; }   ///< returns the industry output from this ship design
+    float                           ResearchGeneration() const  { return m_research_generation; }   ///< returns the research output from this ship design
+    float                           TradeGeneration() const     { return m_trade_generation; }      ///< returns the trade output from this ship design
+    bool                            IsProductionLocation() const{ return m_is_production_location;} ///< returns true if this ship design can be a production location
 
     bool                            CanColonize() const;
     bool                            HasTroops() const       { return (m_troop_capacity > 0.0f); }
     bool                            CanBombard() const      { return m_can_bombard; }
     bool                            IsArmed() const         { return m_is_armed; }
+    bool                            HasFighters() const     { return m_has_fighters; }
     bool                            IsMonster() const       { return m_is_monster; }
 
     float                           Attack() const;
@@ -519,15 +482,19 @@ public:
     std::vector<std::string>        Parts(ShipSlotType slot_type) const;            ///< returns vector of names of parts in slots of indicated type in this design, unrelated to slot positions
     std::vector<std::string>        Weapons() const;                                ///< returns vector of names of weapon parts in, unrelated to slot positions
 
-    std::vector<std::string>        Tags() const;                                   ///< returns tags that would apply to ships of this design, from parts or the hull (but not from species or any other source of tags)
-
     const std::string&              Icon() const            { return m_icon; }      ///< returns filename for small-size icon graphic for design
     const std::string&              Model() const           { return m_3D_model; }  ///< returns filename of 3D model that represents ships of design
     bool                            LookupInStringtable() const { return m_name_desc_in_stringtable; }
 
+    /** returns number of parts in this ship design, indexed by PartType name */
+    const std::map<std::string, int>&     PartTypeCount() const { return m_num_part_types; }
+
+    /** returns number of parts in this ship design, indexed by ShipPartClass */
+    const std::map<ShipPartClass, int>&   PartClassCount() const { return m_num_part_classes; }
+
     std::string                     Dump() const;                                   ///< returns a data file format representation of this object
 
-    friend bool operator ==(const ShipDesign& first, const ShipDesign& second);
+    friend FO_COMMON_API bool operator ==(const ShipDesign& first, const ShipDesign& second);
     //@}
 
     bool                            ProductionLocation(int empire_id, int location_id) const;   ///< returns true iff the empire with ID empire_id can produce this design at the location with location_id
@@ -571,6 +538,7 @@ private:
     // Note that these are fine to compute on demand and cache here -- it is
     // not necessary to serialize them.
     bool    m_is_armed;
+    bool    m_has_fighters;
     bool    m_can_bombard;
     float   m_detection;
     float   m_colony_capacity;
@@ -584,6 +552,8 @@ private:
     float   m_industry_generation;
     float   m_trade_generation;
     bool    m_is_production_location;
+    std::map<std::string, int>      m_num_part_types;
+    std::map<ShipPartClass, int>    m_num_part_classes;
 
     bool    m_producible;
 
