@@ -6,6 +6,7 @@
 #include "Ship.h"
 #include "UniverseObject.h"
 #include "ValueRef.h"
+#include "Enums.h"
 #include "../parse/Parse.h"
 #include "../util/OptionsDB.h"
 #include "../util/Logger.h"
@@ -14,6 +15,9 @@
 
 #include <boost/filesystem/fstream.hpp>
 
+#include <iterator>
+
+
 /////////////////////////////////////////////////
 // FocusType                                   //
 /////////////////////////////////////////////////
@@ -77,9 +81,9 @@ Species::~Species()
 void Species::Init() {
     if (m_location)
         m_location->SetTopLevelContent(this->m_name);
-    for (std::vector<boost::shared_ptr<Effect::EffectsGroup> >::iterator it = m_effects.begin();
-         it != m_effects.end(); ++it)
-    { (*it)->SetTopLevelContent(m_name); }
+    for (std::shared_ptr<Effect::EffectsGroup> effect : m_effects) {
+        effect->SetTopLevelContent(m_name);
+    }
 }
 
 std::string Species::Dump() const {
@@ -102,8 +106,8 @@ std::string Species::Dump() const {
     } else {
         retval += DumpIndent() + "foci = [\n";
         ++g_indent;
-        for (std::vector<FocusType>::const_iterator it = m_foci.begin(); it != m_foci.end(); ++it) {
-            retval += it->Dump();
+        for (const FocusType& focus : m_foci) {
+            retval += focus.Dump();
         }
         --g_indent;
         retval += DumpIndent() + "]\n";
@@ -116,8 +120,8 @@ std::string Species::Dump() const {
     } else {
         retval += DumpIndent() + "effectsgroups = [\n";
         ++g_indent;
-        for (unsigned int i = 0; i < m_effects.size(); ++i) {
-            retval += m_effects[i]->Dump();
+        for (std::shared_ptr<Effect::EffectsGroup> effect : m_effects) {
+            retval += effect->Dump();
         }
         --g_indent;
         retval += DumpIndent() + "]\n";
@@ -132,9 +136,9 @@ std::string Species::Dump() const {
     } else {
         retval += DumpIndent() + "environments = [\n";
         ++g_indent;
-        for (std::map<PlanetType, PlanetEnvironment>::const_iterator it = m_planet_environments.begin(); it != m_planet_environments.end(); ++it) {
-            retval += DumpIndent() + "type = " + PlanetTypeToString(it->first)
-                                   + " environment = " + PlanetEnvironmentToString(it->second)
+        for (const std::map<PlanetType, PlanetEnvironment>::value_type& entry : m_planet_environments) {
+            retval += DumpIndent() + "type = " + PlanetTypeToString(entry.first)
+                                   + " environment = " + PlanetEnvironmentToString(entry.second)
                                    + "\n";
         }
         --g_indent;
@@ -152,10 +156,8 @@ std::string Species::GameplayDescription() const {
 
     bool requires_separator = true;
 
-    for (std::vector<boost::shared_ptr<Effect::EffectsGroup> >::const_iterator it = m_effects.begin();
-         m_effects.end() != it; ++it)
-    {
-        const std::string& description = (*it)->GetDescription();
+    for (std::shared_ptr<Effect::EffectsGroup> effect : m_effects) {
+        const std::string& description = effect->GetDescription();
         if (description.empty())
             continue;
 
@@ -230,15 +232,13 @@ PlanetType Species::NextBetterPlanetType(PlanetType initial_planet_type) const
     // determine which environment rating is the best available for this species
     PlanetEnvironment best_environment = PE_UNINHABITABLE;
     //std::set<PlanetType> best_types;
-    for (std::map<PlanetType, PlanetEnvironment>::const_iterator it = m_planet_environments.begin();
-         it != m_planet_environments.end(); ++it)
-    {
-        if (it->second == best_environment) {
-            //best_types.insert(it->first);
-        } else if (it->second > best_environment) {
-            best_environment = it->second;
+    for (const std::map<PlanetType, PlanetEnvironment>::value_type& entry : m_planet_environments) {
+        if (entry.second == best_environment) {
+            //best_types.insert(entry.first);
+        } else if (entry.second > best_environment) {
+            best_environment = entry.second;
             //best_types.clear();
-            //best_types.insert(it->first);
+            //best_types.insert(entry.first);
         }
     }
 
@@ -308,7 +308,7 @@ void Species::SetOtherSpeciesOpinion(const std::string& species_name, double opi
 // SpeciesManager                              //
 /////////////////////////////////////////////////
 // static(s)
-SpeciesManager* SpeciesManager::s_instance = 0;
+SpeciesManager* SpeciesManager::s_instance = nullptr;
 
 bool SpeciesManager::PlayableSpecies::operator()(
     const std::map<std::string, Species*>::value_type& species_map_iterator) const
@@ -332,8 +332,8 @@ SpeciesManager::SpeciesManager() {
 
     if (GetOptionsDB().Get<bool>("verbose-logging")) {
         DebugLogger() << "Species:";
-        for (iterator it = begin(); it != end(); ++it) {
-            const Species* s = it->second;
+        for (const std::map<std::string, Species*>::value_type& entry : m_species) {
+            const Species* s = entry.second;
             DebugLogger() << " ... " << s->Name() << "  \t" <<
                 (s->Playable() ?        "Playable " : "         ") <<
                 (s->Native() ?          "Native " : "       ") <<
@@ -344,18 +344,18 @@ SpeciesManager::SpeciesManager() {
 }
 
 SpeciesManager::~SpeciesManager() {
-    for (std::map<std::string, Species*>::iterator it = m_species.begin(); it != m_species.end(); ++it)
-        delete it->second;
+    for (std::map<std::string, Species*>::value_type& entry : m_species)
+        delete entry.second;
 }
 
 const Species* SpeciesManager::GetSpecies(const std::string& name) const {
     std::map<std::string, Species*>::const_iterator it = m_species.find(name);
-    return it != m_species.end() ? it->second : 0;
+    return it != m_species.end() ? it->second : nullptr;
 }
 
 Species* SpeciesManager::GetSpecies(const std::string& name) {
     std::map<std::string, Species*>::iterator it = m_species.find(name);
-    return it != m_species.end() ? it->second : 0;
+    return it != m_species.end() ? it->second : nullptr;
 }
 
 int SpeciesManager::GetSpeciesID(const std::string& name) const {
@@ -409,9 +409,7 @@ const std::string& SpeciesManager::RandomSpeciesName() const {
         return EMPTY_STRING;
 
     int species_idx = RandSmallInt(0, static_cast<int>(m_species.size()) - 1);
-    std::map<std::string, Species*>::const_iterator it = m_species.begin();
-    std::advance(it, species_idx);
-    return it->first;
+    return std::next(m_species.begin(), species_idx)->first;
 }
 
 const std::string& SpeciesManager::RandomPlayableSpeciesName() const {
@@ -419,9 +417,7 @@ const std::string& SpeciesManager::RandomPlayableSpeciesName() const {
         return EMPTY_STRING;
 
     int species_idx = RandSmallInt(0, NumPlayableSpecies() - 1);
-    playable_iterator it = playable_begin();
-    std::advance(it, species_idx);
-    return it->first;
+    return std::next(playable_begin(), species_idx)->first;
 }
 
 const std::string& SpeciesManager::SequentialPlayableSpeciesName(int id) const {
@@ -430,23 +426,21 @@ const std::string& SpeciesManager::SequentialPlayableSpeciesName(int id) const {
 
     int species_idx = id % NumPlayableSpecies();
     DebugLogger() << "SpeciesManager::SequentialPlayableSpeciesName has " << NumPlayableSpecies() << " and is given id " << id << " yielding index " << species_idx;
-    playable_iterator it = playable_begin();
-    std::advance(it, species_idx);
-    return it->first;
+    return std::next(playable_begin(), species_idx)->first;
 }
 
 void SpeciesManager::ClearSpeciesHomeworlds() {
-    for (std::map<std::string, Species*>::iterator it = m_species.begin(); it != m_species.end(); ++it)
-        it->second->SetHomeworlds(std::set<int>());
+    for (std::map<std::string, Species*>::value_type& entry : m_species)
+        entry.second->SetHomeworlds(std::set<int>());
 }
 
-void SpeciesManager::SetSpeciesHomeworlds(const std::map<std::string, std::set<int> >& species_homeworld_ids) {
+void SpeciesManager::SetSpeciesHomeworlds(const std::map<std::string, std::set<int>>& species_homeworld_ids) {
     ClearSpeciesHomeworlds();
-    for (std::map<std::string, std::set<int> >::const_iterator it = species_homeworld_ids.begin(); it != species_homeworld_ids.end(); ++it) {
-        const std::string& species_name = it->first;
-        const std::set<int>& homeworlds = it->second;
+    for (const std::map<std::string, std::set<int>>::value_type& entry : species_homeworld_ids) {
+        const std::string& species_name = entry.first;
+        const std::set<int>& homeworlds = entry.second;
 
-        Species* species = 0;
+        Species* species = nullptr;
         std::map<std::string, Species*>::iterator species_it = m_species.find(species_name);
         if (species_it != m_species.end())
             species = species_it->second;
@@ -459,42 +453,41 @@ void SpeciesManager::SetSpeciesHomeworlds(const std::map<std::string, std::set<i
     }
 }
 
-void SpeciesManager::SetSpeciesEmpireOpinions(const std::map<std::string, std::map<int, float> >& species_empire_opinions)
+void SpeciesManager::SetSpeciesEmpireOpinions(const std::map<std::string, std::map<int, float>>& species_empire_opinions)
 { m_species_empire_opinions = species_empire_opinions; }
 
 void SpeciesManager::SetSpeciesEmpireOpinion(const std::string& species_name, int empire_id, float opinion)
 { m_species_empire_opinions[species_name][empire_id] = opinion; }
 
-void SpeciesManager::SetSpeciesSpeciesOpinions(const std::map<std::string, std::map<std::string, float> >& species_species_opinions)
+void SpeciesManager::SetSpeciesSpeciesOpinions(const std::map<std::string, std::map<std::string, float>>& species_species_opinions)
 { m_species_species_opinions = species_species_opinions; }
 
 void SpeciesManager::SetSpeciesSpeciesOpinion(const std::string& opinionated_species, const std::string& rated_species, float opinion)
 { m_species_species_opinions[opinionated_species][rated_species] = opinion; }
 
-std::map<std::string, std::set<int> > SpeciesManager::GetSpeciesHomeworldsMap(int encoding_empire/* = ALL_EMPIRES*/) const {
-    std::map<std::string, std::set<int> > retval;
-    for (iterator it = begin(); it != end(); ++it) {
-        const std::string species_name = it->first;
-        const Species* species = it->second;
+std::map<std::string, std::set<int>> SpeciesManager::GetSpeciesHomeworldsMap(int encoding_empire/* = ALL_EMPIRES*/) const {
+    std::map<std::string, std::set<int>> retval;
+    for (const std::map<std::string, Species*>::value_type& entry : m_species) {
+        const std::string species_name = entry.first;
+        const Species* species = entry.second;
         if (!species) {
             ErrorLogger() << "SpeciesManager::GetSpeciesHomeworldsMap found a null species pointer in SpeciesManager?!";
             continue;
         }
-        const std::set<int>& homeworld_ids = species->Homeworlds();
-        for (std::set<int>::const_iterator homeworlds_it = homeworld_ids.begin(); homeworlds_it != homeworld_ids.end(); ++homeworlds_it)
-            retval[species_name].insert(*homeworlds_it);
+        for (int homeworld_id : species->Homeworlds())
+            retval[species_name].insert(homeworld_id);
     }
     return retval;
 }
 
-const std::map<std::string, std::map<int, float> >& SpeciesManager::GetSpeciesEmpireOpinionsMap(int encoding_empire/* = ALL_EMPIRES*/) const
+const std::map<std::string, std::map<int, float>>& SpeciesManager::GetSpeciesEmpireOpinionsMap(int encoding_empire/* = ALL_EMPIRES*/) const
 { return m_species_empire_opinions; }
 
-const std::map<std::string, std::map<std::string, float> >& SpeciesManager::GetSpeciesSpeciesOpinionsMap(int encoding_empire/* = ALL_EMPIRES*/) const
+const std::map<std::string, std::map<std::string, float>>& SpeciesManager::GetSpeciesSpeciesOpinionsMap(int encoding_empire/* = ALL_EMPIRES*/) const
 { return m_species_species_opinions; }
 
 float SpeciesManager::SpeciesEmpireOpinion(const std::string& species_name, int empire_id) const {
-    std::map<std::string, std::map<int, float> >::const_iterator sp_it = m_species_empire_opinions.find(species_name);
+    std::map<std::string, std::map<int, float>>::const_iterator sp_it = m_species_empire_opinions.find(species_name);
     if (sp_it == m_species_empire_opinions.end())
         return 0.0f;
     const std::map<int, float>& emp_map = sp_it->second;
@@ -505,7 +498,7 @@ float SpeciesManager::SpeciesEmpireOpinion(const std::string& species_name, int
 }
 
 float SpeciesManager::SpeciesSpeciesOpinion(const std::string& opinionated_species_name, const std::string& rated_species_name) const {
-    std::map<std::string, std::map<std::string, float> >::const_iterator sp_it = m_species_species_opinions.find(opinionated_species_name);
+    std::map<std::string, std::map<std::string, float>>::const_iterator sp_it = m_species_species_opinions.find(opinionated_species_name);
     if (sp_it == m_species_species_opinions.end())
         return 0.0f;
     const std::map<std::string, float>& ra_sp_map = sp_it->second;
@@ -523,14 +516,12 @@ void SpeciesManager::ClearSpeciesOpinions() {
 void SpeciesManager::UpdatePopulationCounter() {
     // ships of each species and design
     m_species_object_populations.clear();
-    for (std::map<int, TemporaryPtr<UniverseObject> >::iterator obj_it = Objects().ExistingObjectsBegin();
-         obj_it != Objects().ExistingObjectsEnd(); ++obj_it)
-    {
-        TemporaryPtr<UniverseObject> obj = obj_it->second;
+    for (const std::map<int, std::shared_ptr<UniverseObject>>::value_type& entry : Objects().ExistingObjects()) {
+        std::shared_ptr<UniverseObject> obj = entry.second;
         if (obj->ObjectType() != OBJ_PLANET && obj->ObjectType() != OBJ_POP_CENTER)
             continue;
 
-        TemporaryPtr<PopCenter> pop_center = boost::dynamic_pointer_cast<PopCenter>(obj);
+        std::shared_ptr<PopCenter> pop_center = std::dynamic_pointer_cast<PopCenter>(obj);
         if (!pop_center)
             continue;
 
@@ -546,10 +537,10 @@ void SpeciesManager::UpdatePopulationCounter() {
     }
 }
 
-std::map<std::string, std::map<int, float> >& SpeciesManager::SpeciesObjectPopulations(int encoding_empire)
+std::map<std::string, std::map<int, float>>& SpeciesManager::SpeciesObjectPopulations(int encoding_empire)
 { return m_species_object_populations; }
 
-std::map<std::string, std::map<std::string, int> >& SpeciesManager::SpeciesShipsDestroyed(int encoding_empire)
+std::map<std::string, std::map<std::string, int>>& SpeciesManager::SpeciesShipsDestroyed(int encoding_empire)
 { return m_species_species_ships_destroyed; }
 
 
