@@ -6,6 +6,7 @@
 #include "Building.h"
 #include "Predicates.h"
 #include "Universe.h"
+#include "Enums.h"
 
 #include "../Empire/Empire.h"
 #include "../Empire/EmpireManager.h"
@@ -77,17 +78,17 @@ System* System::Clone(int empire_id) const {
     Visibility vis = GetUniverse().GetObjectVisibilityByEmpire(this->ID(), empire_id);
 
     if (!(vis >= VIS_BASIC_VISIBILITY && vis <= VIS_FULL_VISIBILITY))
-        return 0;
+        return nullptr;
 
     System* retval = new System();
-    retval->Copy(TemporaryFromThis(), empire_id);
+    retval->Copy(shared_from_this(), empire_id);
     return retval;
 }
 
-void System::Copy(TemporaryPtr<const UniverseObject> copied_object, int empire_id) {
-    if (copied_object == this)
+void System::Copy(std::shared_ptr<const UniverseObject> copied_object, int empire_id) {
+    if (copied_object.get() == this)
         return;
-    TemporaryPtr<const System> copied_system = boost::dynamic_pointer_cast<const System>(copied_object);
+    std::shared_ptr<const System> copied_system = std::dynamic_pointer_cast<const System>(copied_object);
     if (!copied_system) {
         ErrorLogger() << "System::Copy passed an object that wasn't a System";
         return;
@@ -102,9 +103,8 @@ void System::Copy(TemporaryPtr<const UniverseObject> copied_object, int empire_i
     if (vis >= VIS_BASIC_VISIBILITY) {
         // add any visible lanes, without removing existing entries
         std::map<int, bool> visible_lanes_holes = copied_system->VisibleStarlanesWormholes(empire_id);
-        for (std::map<int, bool>::const_iterator it = visible_lanes_holes.begin();
-             it != visible_lanes_holes.end(); ++it)
-        { this->m_starlanes_wormholes[it->first] = it->second; }
+        for (const std::map<int, bool>::value_type& entry : visible_lanes_holes)
+        { this->m_starlanes_wormholes[entry.first] = entry.second; }
 
         // copy visible info of visible contained objects
         this->m_objects = copied_system->VisibleContainedObjectIDs(empire_id);
@@ -121,44 +121,35 @@ void System::Copy(TemporaryPtr<const UniverseObject> copied_object, int empire_i
 
         // copy visible contained object per-type info
         m_planets.clear();
-        for (std::set<int>::const_iterator it = copied_system->m_planets.begin();
-             it != copied_system->m_planets.end(); ++it)
+        for (int planet_id : copied_system->m_planets)
         {
-            if (m_objects.find(*it) != m_objects.end())
-                m_planets.insert(*it);
+            if (m_objects.find(planet_id) != m_objects.end())
+                m_planets.insert(planet_id);
         }
 
         m_buildings.clear();
-        for (std::set<int>::const_iterator it = copied_system->m_buildings.begin();
-             it != copied_system->m_buildings.end(); ++it)
-        {
-            if (m_objects.find(*it) != m_objects.end())
-                m_buildings.insert(*it);
+        for (int building_id : copied_system->m_buildings) {
+            if (m_objects.find(building_id) != m_objects.end())
+                m_buildings.insert(building_id);
         }
 
         m_fleets.clear();
-        for (std::set<int>::const_iterator it = copied_system->m_fleets.begin();
-             it != copied_system->m_fleets.end(); ++it)
-        {
-            if (m_objects.find(*it) != m_objects.end())
-                m_fleets.insert(*it);
+        for (int fleet_id : copied_system->m_fleets) {
+            if (m_objects.find(fleet_id) != m_objects.end())
+                m_fleets.insert(fleet_id);
         }
 
         m_ships.clear();
-        for (std::set<int>::const_iterator it = copied_system->m_ships.begin();
-             it != copied_system->m_ships.end(); ++it)
-        {
-            if (m_objects.find(*it) != m_objects.end())
-                m_ships.insert(*it);
+        for (int ship_id : copied_system->m_ships) {
+            if (m_objects.find(ship_id) != m_objects.end())
+                m_ships.insert(ship_id);
         }
 
         m_fields.clear();
-        for (std::set<int>::const_iterator it = copied_system->m_fields.begin();
-             it != copied_system->m_fields.end(); ++it)
-            {
-                if (m_objects.find(*it) != m_objects.end())
-                    m_fields.insert(*it);
-            }
+        for (int field_id : copied_system->m_fields) {
+            if (m_objects.find(field_id) != m_objects.end())
+                m_fields.insert(field_id);
+        }
 
 
         if (vis >= VIS_PARTIAL_VISIBILITY) {
@@ -170,17 +161,16 @@ void System::Copy(TemporaryPtr<const UniverseObject> copied_object, int empire_i
             // partial vis, they should be seen, but aren't, so are known not
             // to exist any more
 
-            // assemble all previously known lanes
-            std::vector<int> initial_known_lanes;
-            for (std::map<int, bool>::const_iterator it = this->m_starlanes_wormholes.begin();
-                 it != this->m_starlanes_wormholes.end(); ++it)
-            { initial_known_lanes.push_back(it->first); }
-
             // remove previously known lanes that aren't currently visible
-            for (std::vector<int>::const_iterator it = initial_known_lanes.begin(); it != initial_known_lanes.end(); ++it) {
-                int lane_end_sys_id = *it;
-                if (visible_lanes_holes.find(lane_end_sys_id) == visible_lanes_holes.end())
-                    this->m_starlanes_wormholes.erase(lane_end_sys_id);
+            for (auto entry_it = m_starlanes_wormholes.begin(); entry_it != m_starlanes_wormholes.end();
+                 /* conditional increment in deleting loop */)
+            {
+                int lane_end_sys_id = entry_it->first;
+                if (visible_lanes_holes.find(lane_end_sys_id) == visible_lanes_holes.end()) {
+                    entry_it = m_starlanes_wormholes.erase(entry_it);
+                } else {
+                    ++entry_it;
+                }
             }
         }
     }
@@ -192,7 +182,7 @@ UniverseObjectType System::ObjectType() const
 std::string System::Dump() const {
     std::stringstream os;
     os << UniverseObject::Dump();
-    os << " star type: " << UserString(EnumToString(m_star))
+    os << " star type: " << m_star
        << "  last combat on turn: " << m_last_turn_battle_here
        << "  total orbits: " << m_orbits.size();
 
@@ -251,10 +241,8 @@ const std::string& System::ApparentName(int empire_id, bool blank_unexplored_and
 
     if (m_star == STAR_NONE) {
         // determine if there are any planets in the system
-        for (std::map<int, TemporaryPtr<UniverseObject> >::iterator it = Objects().ExistingPlanetsBegin();
-             it != Objects().ExistingPlanetsEnd(); ++it)
-        {
-            if (it->second->SystemID() == this->ID())
+        for (const std::map<int, std::shared_ptr<UniverseObject>>::value_type& entry : Objects().ExistingPlanets()) {
+            if (entry.second->SystemID() == this->ID())
                 return this->PublicName(empire_id);
         }
         if (blank_unexplored_and_none) {
@@ -290,10 +278,8 @@ StarType System::NextYoungerStarType() const {
 
 int System::NumStarlanes() const {
     int retval = 0;
-    for (std::map<int, bool>::const_iterator it = m_starlanes_wormholes.begin();
-         it != m_starlanes_wormholes.end(); ++it)
-    {
-        if (!it->second)
+    for (const std::map<int, bool>::value_type& entry : m_starlanes_wormholes) {
+        if (!entry.second)
             ++retval;
     }
     return retval;
@@ -301,10 +287,8 @@ int System::NumStarlanes() const {
 
 int System::NumWormholes() const {
     int retval = 0;
-    for (std::map<int, bool>::const_iterator it = m_starlanes_wormholes.begin();
-         it != m_starlanes_wormholes.end(); ++it)
-    {
-        if (it->second)
+    for (const std::map<int, bool>::value_type& entry : m_starlanes_wormholes) {
+        if (entry.second)
             ++retval;
     }
     return retval;
@@ -320,6 +304,23 @@ bool System::HasWormholeTo(int id) const {
     return (it == m_starlanes_wormholes.end() ? false : it->second == true);
 }
 
+int  System::Owner() const {
+    // Check if all of the owners are the same empire.
+    int first_owner_found(ALL_EMPIRES);
+    for (int planet_id : m_planets) {
+        if (std::shared_ptr<const Planet> planet = GetPlanet(planet_id)) {
+            const int owner(planet->Owner());
+            if (owner == ALL_EMPIRES)
+                continue;
+            if (first_owner_found == ALL_EMPIRES)
+                first_owner_found = owner;
+            if (first_owner_found != owner)
+                return ALL_EMPIRES;
+        }
+    }
+    return first_owner_found;
+}
+
 const std::set<int>& System::ContainedObjectIDs() const
 { return m_objects; }
 
@@ -329,10 +330,10 @@ bool System::Contains(int object_id) const {
     return m_objects.find(object_id) != m_objects.end();
 }
 
-TemporaryPtr<UniverseObject> System::Accept(const UniverseObjectVisitor& visitor) const
-{ return visitor.Visit(boost::const_pointer_cast<System>(boost::static_pointer_cast<const System>(TemporaryFromThis()))); }
+std::shared_ptr<UniverseObject> System::Accept(const UniverseObjectVisitor& visitor) const
+{ return visitor.Visit(std::const_pointer_cast<System>(std::static_pointer_cast<const System>(shared_from_this()))); }
 
-void System::Insert(TemporaryPtr<UniverseObject> obj, int orbit/* = -1*/) {
+void System::Insert(std::shared_ptr<UniverseObject> obj, int orbit/* = -1*/) {
     if (!obj) {
         ErrorLogger() << "System::Insert() : Attempted to place a null object in a System";
         return;
@@ -348,8 +349,8 @@ void System::Insert(TemporaryPtr<UniverseObject> obj, int orbit/* = -1*/) {
     if (obj->ObjectType() == OBJ_PLANET) {
         if (orbit == -1) {
             bool already_in_orbit = false;
-            for (int o = 0; o < static_cast<int>(m_orbits.size()); ++o) {
-                if (m_orbits[o] == obj->ID()) {
+            for (int planet_id : m_orbits) {
+                if (planet_id == obj->ID()) {
                     already_in_orbit = true;
                     break;
                 }
@@ -401,14 +402,14 @@ void System::Insert(TemporaryPtr<UniverseObject> obj, int orbit/* = -1*/) {
     switch (obj->ObjectType()) {
     case OBJ_SHIP: {
         m_ships.insert(obj->ID());
-        if (TemporaryPtr<Ship> ship = boost::dynamic_pointer_cast<Ship>(obj))
+        if (std::shared_ptr<Ship> ship = std::dynamic_pointer_cast<Ship>(obj))
             ship->SetArrivedOnTurn(CurrentTurn());
         break;
     }
     case OBJ_FLEET: {
         m_fleets.insert(obj->ID());
-        std::vector<TemporaryPtr<Fleet> > fleets;
-        fleets.push_back(boost::dynamic_pointer_cast<Fleet>(obj));
+        std::vector<std::shared_ptr<Fleet>> fleets;
+        fleets.push_back(std::dynamic_pointer_cast<Fleet>(obj));
         FleetsInsertedSignal(fleets);
         break;
     }
@@ -447,9 +448,9 @@ void System::Remove(int id) {
     it = m_planets.find(id);
     if (it != m_planets.end()) {
         m_planets.erase(it);
-        for (int i = 0; i < static_cast<int>(m_orbits.size()); ++i)
-            if (m_orbits[i] == id)
-                m_orbits[i] = INVALID_OBJECT_ID;
+        for (int& planet_id : m_orbits)
+            if (planet_id == id)
+                planet_id = INVALID_OBJECT_ID;
     }
 
     m_ships.erase(id);
@@ -458,8 +459,8 @@ void System::Remove(int id) {
     m_objects.erase(id);
 
     if (removed_fleet) {
-        if (TemporaryPtr<Fleet> fleet = GetFleet(id)) {
-            std::vector<TemporaryPtr<Fleet> > fleets;
+        if (std::shared_ptr<Fleet> fleet = GetFleet(id)) {
+            std::vector<std::shared_ptr<Fleet>> fleets;
             fleets.push_back(fleet);
             FleetsRemovedSignal(fleets);
         }
@@ -492,7 +493,7 @@ void System::AddWormhole(int id) {
 
 bool System::RemoveStarlane(int id) {
     bool retval = false;
-    if (retval = HasStarlaneTo(id)) {
+    if ((retval = HasStarlaneTo(id))) {
         m_starlanes_wormholes.erase(id);
         StateChangedSignal();
     }
@@ -501,7 +502,7 @@ bool System::RemoveStarlane(int id) {
 
 bool System::RemoveWormhole(int id) {
     bool retval = false;
-    if (retval = HasWormholeTo(id)) {
+    if ((retval = HasWormholeTo(id))) {
         m_starlanes_wormholes.erase(id);
         StateChangedSignal();
     }
@@ -579,12 +580,10 @@ std::map<int, bool> System::VisibleStarlanesWormholes(int empire_id) const {
 
 
     // check if any of the adjacent systems are partial or better visible
-    for (std::map<int, bool>::const_iterator it = m_starlanes_wormholes.begin();
-         it != m_starlanes_wormholes.end(); ++it)
-    {
-        int lane_end_sys_id = it->first;
+    for (const std::map<int, bool>::value_type& entry : m_starlanes_wormholes) {
+        int lane_end_sys_id = entry.first;
         if (universe.GetObjectVisibilityByEmpire(lane_end_sys_id, empire_id) >= VIS_PARTIAL_VISIBILITY)
-            retval[lane_end_sys_id] = it->second;
+            retval[lane_end_sys_id] = entry.second;
     }
 
 
@@ -596,21 +595,15 @@ std::map<int, bool> System::VisibleStarlanesWormholes(int empire_id) const {
     // check if any fleets owned by empire are moving along a starlane connected to this system...
 
     // get moving fleets owned by empire
-    std::vector<TemporaryPtr<const Fleet> > moving_empire_fleets;
-    std::vector<TemporaryPtr<const UniverseObject> > moving_fleet_objects = objects.FindObjects(MovingFleetVisitor());
-    for (std::vector<TemporaryPtr<const UniverseObject> >::const_iterator
-         it = moving_fleet_objects.begin(); it != moving_fleet_objects.end(); ++it) 
-    {
-        if (TemporaryPtr<const Fleet> fleet = boost::dynamic_pointer_cast<const Fleet>(*it))
+    std::vector<std::shared_ptr<const Fleet>> moving_empire_fleets;
+    for (std::shared_ptr<const UniverseObject> object : objects.FindObjects(MovingFleetVisitor())) {
+        if (std::shared_ptr<const Fleet> fleet = std::dynamic_pointer_cast<const Fleet>(object))
             if (fleet->OwnedBy(empire_id))
                 moving_empire_fleets.push_back(fleet);
     }
 
     // add any lanes an owned fleet is moving along that connect to this system
-    for (std::vector<TemporaryPtr<const Fleet> >::const_iterator it = moving_empire_fleets.begin();
-         it != moving_empire_fleets.end(); ++it)
-    {
-        TemporaryPtr<const Fleet> fleet = *it;
+    for (std::shared_ptr<const Fleet> fleet : moving_empire_fleets) {
         if (fleet->SystemID() != INVALID_OBJECT_ID) {
             ErrorLogger() << "System::VisibleStarlanesWormholes somehow got a moving fleet that had a valid system id?";
             continue;
