@@ -23,9 +23,26 @@ struct UniverseObjectVisitor;
    orbit), and all objects in a paricular orbit.*/
 class FO_COMMON_API System : public UniverseObject {
 public:
+    /** Systems are unowned unless at least one planet is owned by an empire and
+        no other empire owns any planets in the system.*/
+    int Owner() const override;
+
+    int SystemID() const override
+    { return this->ID(); }
+
     /** \name Accessors */ //@{
-    virtual UniverseObjectType  ObjectType() const;
-    virtual std::string         Dump() const;
+    UniverseObjectType ObjectType() const override;
+
+    std::string Dump() const override;
+
+    const std::set<int>& ContainedObjectIDs() const override;
+
+    bool Contains(int object_id) const override;
+
+    bool ContainedBy(int object_id) const override
+    { return false; }
+
+    std::shared_ptr<UniverseObject> Accept(const UniverseObjectVisitor& visitor) const override;
 
     /** returns the name to display for players for this system.  While all
       * systems may have a proper name assigned, if they contain no planets or
@@ -46,10 +63,6 @@ public:
     bool                    HasStarlaneTo(int id) const;                ///< returns true if there is a starlane from this system to the system with ID number \a id
     bool                    HasWormholeTo(int id) const;                ///< returns true if there is a wormhole from this system to the system with ID number \a id
 
-    virtual int             SystemID() const                { return this->ID(); }
-
-    virtual const std::set<int>&    ContainedObjectIDs() const;                     ///< returns ids of objects in this system
-
     const std::set<int>&    ObjectIDs() const               { return m_objects; }
     const std::set<int>&    PlanetIDs() const               { return m_planets; }
     const std::set<int>&    BuildingIDs() const             { return m_buildings; }
@@ -58,9 +71,6 @@ public:
     const std::set<int>&    FieldIDs() const                { return m_fields; }
     const std::vector<int>& PlanetIDsByOrbit() const        { return m_orbits; }
 
-    virtual bool            Contains(int object_id) const;                                  ///< returns true if object with id \a object_id is in this System
-    virtual bool            ContainedBy(int object_id) const{ return false; }               ///< returns true if there is an object with id \a object_id that contains this UniverseObject
-
     int                     PlanetInOrbit(int orbit) const;             ///< returns the ID of the planet in the specified \a orbit, or INVALID_OBJECT_ID if there is no planet in that orbit or it is an invalid orbit
     int                     OrbitOfPlanet(int object_id) const;         ///< returns the orbit ID in which the planet with \a object_id is located, or -1 the specified ID is not a planet in an orbit of this system
     bool                    OrbitOccupied(int orbit) const;             ///< returns true if there is an object in \a orbit
@@ -76,23 +86,26 @@ public:
 
     int                     LastTurnBattleHere() const  { return m_last_turn_battle_here; }
 
-    virtual TemporaryPtr<UniverseObject>
-                            Accept(const UniverseObjectVisitor& visitor) const;
-
     const std::string&      OverlayTexture() const      { return m_overlay_texture; }
     double                  OverlaySize() const         { return m_overlay_size; }  ///< size in universe units
 
     /** fleets are inserted into system */
-    mutable boost::signals2::signal<void (const std::vector<TemporaryPtr<Fleet> >& fleets)> FleetsInsertedSignal;
+    mutable boost::signals2::signal<void (const std::vector<std::shared_ptr<Fleet>>&)> FleetsInsertedSignal;
     /** fleets are removed from system */
-    mutable boost::signals2::signal<void (const std::vector<TemporaryPtr<Fleet> >& fleets)> FleetsRemovedSignal;
+    mutable boost::signals2::signal<void (const std::vector<std::shared_ptr<Fleet>>&)> FleetsRemovedSignal;
     //@}
 
     /** \name Mutators */ //@{
-    virtual void            Copy(TemporaryPtr<const UniverseObject> copied_object, int empire_id = ALL_EMPIRES);
+    void Copy(std::shared_ptr<const UniverseObject> copied_object, int empire_id = ALL_EMPIRES) override;
+
+    /** Adding owner to system objects is a no-op. */
+    void SetOwner(int id) override
+    {}
+
+    void ResetTargetMaxUnpairedMeters() override;
 
     /** adds an object to this system. */
-    void                    Insert(TemporaryPtr<UniverseObject> obj, int orbit = -1);
+    void Insert(std::shared_ptr<UniverseObject> obj, int orbit = -1);
 
     /** removes the object with ID number \a id from this system. */
     void                    Remove(int id);
@@ -103,12 +116,9 @@ public:
     bool                    RemoveStarlane(int id);         ///< removes a starlane between this system and the system with ID number \a id.  Returns false if there was no starlane from this system to system \a id.
     bool                    RemoveWormhole(int id);         ///< removes a wormhole between this system and the system with ID number \a id.  Returns false if there was no wormhole from this system to system \a id.
 
-    virtual void            SetOwner(int id) {};            ///< adding owner to system objects is a no-op
     void                    SetLastTurnBattleHere(int turn);///< Sets the last turn there was a battle at this system
 
     void                    SetOverlayTexture(const std::string& texture, double size);
-
-    virtual void            ResetTargetMaxUnpairedMeters();
     //@}
 
 protected:
@@ -116,7 +126,7 @@ protected:
     friend class ObjectMap;
 
     /** \name Structors */ //@{
-    System();   ///< default ctor
+    System();
 
     /** general ctor.  \throw std::invalid_arugment May throw
       * std::invalid_arugment if \a star is out of the range of StarType,
@@ -131,18 +141,15 @@ protected:
     System(StarType star, const std::map<int, bool>& lanes_and_holes,
            const std::string& name, double x, double y);
 
+    template <typename T> friend void UniverseObjectDeleter(T*);
     template <class T> friend void boost::python::detail::value_destroyer<false>::execute(T const volatile* p);
-    template <class T> friend void boost::checked_delete(T* x);
 
-#if BOOST_VERSION == 106100
 public:
-#endif
     ~System() {}
-#if BOOST_VERSION == 106100
-protected:
-#endif
 
-    virtual System*         Clone(int empire_id = ALL_EMPIRES) const;   ///< returns new copy of this System
+protected:
+    /** Returns new copy of this System. */
+    System* Clone(int empire_id = ALL_EMPIRES) const override;
     //@}
 
 private:
