@@ -10,8 +10,8 @@
 #include "../Empire/Empire.h"
 #include "../Empire/EmpireManager.h"
 #include "ValueRef.h"
+#include "Enums.h"
 
-#include <boost/lexical_cast.hpp>
 #include <boost/filesystem/fstream.hpp>
 
 namespace {
@@ -32,10 +32,9 @@ namespace {
             while (!stack.empty()) {
                 const Tech* current_tech = stack.back();
                 unsigned int starting_stack_size = stack.size();
-                const std::set<std::string>& prereqs = current_tech->Prerequisites();
                 bool all_prereqs_known = true;
-                for (std::set<std::string>::const_iterator prereq_it = prereqs.begin(); prereq_it != prereqs.end(); ++prereq_it) {
-                    const Tech* prereq_tech = GetTech(*prereq_it);
+                for (const std::string& prereq_name : current_tech->Prerequisites()) {
+                    const Tech* prereq_tech = GetTech(prereq_name);
                     bool prereq_unknown = known_techs.find(prereq_tech->Name()) == known_techs.end();
                     if (prereq_unknown)
                         all_prereqs_known = false;
@@ -54,7 +53,7 @@ namespace {
 
     const Tech* Cheapest(const std::vector<const Tech*>& next_techs, int empire_id) {
         if (next_techs.empty())
-            return 0;
+            return nullptr;
 
         float min_price = next_techs[0]->ResearchCost(empire_id);
         int min_index = 0;
@@ -80,14 +79,11 @@ void Tech::Init() {
     if (m_research_turns)
         m_research_turns->SetTopLevelContent(m_name);
 
-    for (std::vector<boost::shared_ptr<Effect::EffectsGroup> >::iterator it = m_effects.begin();
-         it != m_effects.end(); ++it)
-    { (*it)->SetTopLevelContent(m_name); }
+    for (std::shared_ptr<Effect::EffectsGroup> effect : m_effects)
+    { effect->SetTopLevelContent(m_name); }
 }
 
 std::string Tech::Dump() const {
-    using boost::lexical_cast;
-
     std::string retval = DumpIndent() + "Tech\n";
     ++g_indent;
     retval += DumpIndent() + "name = \"" + m_name + "\"\n";
@@ -104,8 +100,8 @@ std::string Tech::Dump() const {
     } else {
         retval += "[\n";
         ++g_indent;
-        for (std::set<std::string>::const_iterator it = m_prerequisites.begin(); it != m_prerequisites.end(); ++it) {
-            retval += DumpIndent() + "\"" + *it + "\"\n";
+        for (const std::string& prerequisite : m_prerequisites) {
+            retval += DumpIndent() + "\"" + prerequisite + "\"\n";
         }
         --g_indent;
         retval += DumpIndent() + "]\n";
@@ -118,8 +114,8 @@ std::string Tech::Dump() const {
     } else {
         retval += "[\n";
         ++g_indent;
-        for (unsigned int i = 0; i < m_unlocked_items.size(); ++i) {
-            retval += DumpIndent() + m_unlocked_items[i].Dump();
+        for (const ItemSpec& unlocked_item : m_unlocked_items) {
+            retval += DumpIndent() + unlocked_item.Dump();
         }
         --g_indent;
         retval += DumpIndent() + "]\n";
@@ -133,8 +129,8 @@ std::string Tech::Dump() const {
         } else {
             retval += DumpIndent() + "effectsgroups = [\n";
             ++g_indent;
-            for (unsigned int i = 0; i < m_effects.size(); ++i) {
-                retval += m_effects[i]->Dump();
+            for (std::shared_ptr<Effect::EffectsGroup> effect : m_effects) {
+                retval += effect->Dump();
             }
             --g_indent;
             retval += DumpIndent() + "]\n";
@@ -145,41 +141,22 @@ std::string Tech::Dump() const {
     return retval;
 }
 
-namespace {
-    TemporaryPtr<const UniverseObject> SourceForEmpire(int empire_id) {
-        const Empire* empire = GetEmpire(empire_id);
-        if (!empire) {
-            DebugLogger() << "SourceForEmpire: Unable to get empire with ID: " << empire_id;
-            return TemporaryPtr<const UniverseObject>();
-        }
-        // get a source object, which is owned by the empire with the passed-in
-        // empire id.  this is used in conditions to reference which empire is
-        // doing the building.  Ideally this will be the capital, but any object
-        // owned by the empire will work.
-        TemporaryPtr<const UniverseObject> source = GetUniverseObject(empire->CapitalID());
-        // no capital?  scan through all objects to find one owned by this empire
-        if (!source) {
-            for (ObjectMap::const_iterator<> obj_it = Objects().const_begin(); obj_it != Objects().const_end(); ++obj_it) {
-                if (obj_it->OwnedBy(empire_id)) {
-                    source = *obj_it;
-                    break;
-                }
-            }
-        }
-        return source;
-    }
-}
-
 float Tech::ResearchCost(int empire_id) const {
+    const auto arbitrary_large_number = 999999.9f;
+
     if (CHEAP_AND_FAST_TECH_RESEARCH || !m_research_cost) {
         return 1.0;
-    } else {
-        if (ValueRef::ConstantExpr(m_research_cost))
-            return m_research_cost->Eval();
 
-        TemporaryPtr<const UniverseObject> source = SourceForEmpire(empire_id);
+    } else if (m_research_cost->ConstantExpr()) {
+        return m_research_cost->Eval();
+
+    } else if (empire_id == ALL_EMPIRES) {
+        return arbitrary_large_number;
+
+    } else {
+        std::shared_ptr<const UniverseObject> source = Empires().GetSource(empire_id);
         if (!source && !m_research_cost->SourceInvariant())
-            return 999999.9f;
+            return arbitrary_large_number;
 
         ScriptingContext context(source);
         return m_research_cost->Eval(context);
@@ -190,15 +167,21 @@ float Tech::PerTurnCost(int empire_id) const
 { return ResearchCost(empire_id) / std::max(1, ResearchTime(empire_id)); }
 
 int Tech::ResearchTime(int empire_id) const {
+    const auto arbitrary_large_number = 9999;
+
     if (CHEAP_AND_FAST_TECH_RESEARCH || !m_research_turns) {
         return 1;
-    } else {
-        if (ValueRef::ConstantExpr(m_research_turns))
+
+    } else if (m_research_turns->ConstantExpr()) {
             return m_research_turns->Eval();
 
-        TemporaryPtr<const UniverseObject> source = SourceForEmpire(empire_id);
+    } else if (empire_id == ALL_EMPIRES) {
+        return arbitrary_large_number;
+
+    } else {
+        std::shared_ptr<const UniverseObject> source = Empires().GetSource(empire_id);
         if (!source && !m_research_turns->SourceInvariant())
-            return 9999;
+            return arbitrary_large_number;
 
         ScriptingContext context(source);
 
@@ -208,8 +191,13 @@ int Tech::ResearchTime(int empire_id) const {
 
 
 ///////////////////////////////////////////////////////////
-// ItemSpec                                        //
+// ItemSpec                                              //
 ///////////////////////////////////////////////////////////
+ItemSpec::ItemSpec() :
+    type(INVALID_UNLOCKABLE_ITEM_TYPE),
+    name()
+{}
+
 std::string ItemSpec::Dump() const {
     std::string retval = "Item type = ";
     switch (type) {
@@ -236,29 +224,29 @@ bool operator!=(const ItemSpec& lhs, const ItemSpec& rhs)
 // TechManager                                           //
 ///////////////////////////////////////////////////////////
 // static(s)
-TechManager* TechManager::s_instance = 0;
+TechManager* TechManager::s_instance = nullptr;
 
 const Tech* TechManager::GetTech(const std::string& name) const {
     iterator it = m_techs.get<NameIndex>().find(name);
-    return it == m_techs.get<NameIndex>().end() ? 0 : *it;
+    return it == m_techs.get<NameIndex>().end() ? nullptr : *it;
 }
 
 const TechCategory* TechManager::GetTechCategory(const std::string& name) const {
     std::map<std::string, TechCategory*>::const_iterator it = m_categories.find(name);
-    return it == m_categories.end() ? 0 : it->second;
+    return it == m_categories.end() ? nullptr : it->second;
 }
 
 std::vector<std::string> TechManager::CategoryNames() const {
     std::vector<std::string> retval;
-    for (std::map<std::string, TechCategory*>::const_iterator it = m_categories.begin(); it != m_categories.end(); ++it)
-        retval.push_back(it->first);
+    for (const std::map<std::string, TechCategory*>::value_type& entry : m_categories)
+        retval.push_back(entry.first);
     return retval;
 }
 
 std::vector<std::string> TechManager::TechNames() const {
     std::vector<std::string> retval;
-    for (TechManager::iterator it = begin(); it != end(); ++it)
-        retval.push_back((*it)->Name());
+    for (const TechContainer::value_type& tech : m_techs.get<NameIndex>())
+        retval.push_back(tech->Name());
     return retval;
 }
 
@@ -327,10 +315,10 @@ TechManager::TechManager() {
     }
 
     std::set<std::string> empty_defined_categories;
-    for (std::map<std::string, TechCategory*>::iterator map_it = m_categories.begin(); map_it != m_categories.end(); ++map_it) {
-        std::set<std::string>::iterator set_it = categories_seen_in_techs.find(map_it->first);
+    for (const std::map<std::string, TechCategory*>::value_type& map : m_categories) {
+        std::set<std::string>::iterator set_it = categories_seen_in_techs.find(map.first);
         if (set_it == categories_seen_in_techs.end()) {
-            empty_defined_categories.insert(map_it->first);
+            empty_defined_categories.insert(map.first);
         } else {
             categories_seen_in_techs.erase(set_it);
         }
@@ -338,8 +326,8 @@ TechManager::TechManager() {
 
     if (!empty_defined_categories.empty()) {
         std::stringstream stream;
-        for (std::set<std::string>::iterator it = empty_defined_categories.begin(); it != empty_defined_categories.end(); ++it) {
-            stream << " \"" << *it << "\"";
+        for (const std::string& empty_defined_category : empty_defined_categories) {
+            stream << " \"" << empty_defined_category << "\"";
         }
         std::string error_str = "ERROR: The following categories were defined in techs.txt, but no "
             "techs were defined that fell within them:" + stream.str();
@@ -349,8 +337,8 @@ TechManager::TechManager() {
 
     if (!categories_seen_in_techs.empty()) {
         std::stringstream stream;
-        for (std::set<std::string>::iterator it = categories_seen_in_techs.begin(); it != categories_seen_in_techs.end(); ++it) {
-            stream << " \"" << *it << "\"";
+        for (const std::string& category_seen_in_techs : categories_seen_in_techs) {
+            stream << " \"" << category_seen_in_techs << "\"";
         }
         std::string error_str = "ERROR: The following categories were never defined in techs.txt, but some "
             "techs were defined that fell within them:" + stream.str();
@@ -371,11 +359,10 @@ TechManager::TechManager() {
     }
 
     // fill in the unlocked techs data for each loaded tech
-    for (iterator it = begin(); it != end(); ++it) {
-        const std::set<std::string>& prereqs = (*it)->Prerequisites();
-        for (std::set<std::string>::const_iterator prereq_it = prereqs.begin(); prereq_it != prereqs.end(); ++prereq_it) {
-            if (Tech* tech = const_cast<Tech*>(GetTech(*prereq_it)))
-                tech->m_unlocked_techs.insert((*it)->Name());
+    for (const Tech* tech : m_techs) {
+        for (const std::string& prereq : tech->Prerequisites()) {
+            if (Tech* prereq_tech = const_cast<Tech*>(GetTech(prereq)))
+                prereq_tech->m_unlocked_techs.insert(tech->Name());
         }
     }
 
@@ -384,39 +371,36 @@ TechManager::TechManager() {
         ErrorLogger() << redundant_dependency;
 
 #ifdef OUTPUT_TECH_LIST
-    for (iterator it = begin(); it != end(); ++it) {
-        const Tech* tech = *it;
+    for (const Tech* tech : m_techs.get<NameIndex>()) {
         std::cerr << UserString(tech->Name()) << " (" 
-                  << UserString(tech->Category()) << " "
-                  << UserString(boost::lexical_cast<std::string>(tech->Type())) << ") - "
+                  << UserString(tech->Category()) << ") - "
                   << tech->Graphic() << std::endl;
     }
 #endif
 }
 
 TechManager::~TechManager() {
-    for (std::map<std::string, TechCategory*>::iterator it = m_categories.begin(); it != m_categories.end(); ++it)
-        delete it->second;
-    for (TechContainer::iterator it = m_techs.begin(); it != m_techs.end(); ++it)
-        delete *it;
+    for (std::map<std::string, TechCategory*>::value_type& entry : m_categories)
+        delete entry.second;
+    for (const Tech* tech : m_techs)
+        delete tech;
 }
 
 std::string TechManager::FindIllegalDependencies() {
     assert(!m_techs.empty());
     std::string retval;
-    for (iterator it = begin(); it != end(); ++it) {
-        const Tech* tech = *it;
+    for (const Tech* tech : m_techs) {
         if (!tech) {
             std::stringstream stream;
             stream << "ERROR: Missing tech referenced in techs.txt for unknown reasons...";
             return stream.str();
         }
-        const std::set<std::string>& prereqs = tech->Prerequisites();
-        for (std::set<std::string>::const_iterator prereq_it = prereqs.begin(); prereq_it != prereqs.end(); ++prereq_it) {
-            const Tech* prereq_tech = GetTech(*prereq_it);
+
+        for (const std::string& prereq : tech->Prerequisites()) {
+            const Tech* prereq_tech = GetTech(prereq);
             if (!prereq_tech) {
                 std::stringstream stream;
-                stream << "ERROR: Tech \"" << tech->Name() << "\" requires a missing or malformed tech \"" << *prereq_it << "\" as its prerequisite.";
+                stream << "ERROR: Tech \"" << tech->Name() << "\" requires a missing or malformed tech \"" << prereq << "\" as its prerequisite.";
                 return stream.str();
             }
         }
@@ -429,12 +413,12 @@ std::string TechManager::FindFirstDependencyCycle() {
     static const std::set<std::string> EMPTY_STRING_SET;    // used in case an invalid tech is processed
 
     std::set<const Tech*> checked_techs; // the list of techs that are not part of any cycle
-    for (iterator it = begin(); it != end(); ++it) {
-        if (checked_techs.find(*it) != checked_techs.end())
+    for (const Tech* tech : *this) {
+        if (checked_techs.find(tech) != checked_techs.end())
             continue;
 
         std::vector<const Tech*> stack;
-        stack.push_back(*it);
+        stack.push_back(tech);
         while (!stack.empty()) {
             // Examine the tech on top of the stack.  If the tech has no prerequisite techs, or if all
             // of its prerequisite techs have already been checked, pop it off the stack and mark it as
@@ -443,10 +427,8 @@ std::string TechManager::FindFirstDependencyCycle() {
             unsigned int starting_stack_size = stack.size();
 
             const std::set<std::string>& prereqs = (current_tech ? current_tech->Prerequisites() : EMPTY_STRING_SET);
-            for (std::set<std::string>::const_iterator prereq_it = prereqs.begin();
-                 prereq_it != prereqs.end(); ++prereq_it)
-            {
-                const Tech* prereq_tech = GetTech(*prereq_it);
+            for (const std::string& prereq_name : prereqs) {
+                const Tech* prereq_tech = GetTech(prereq_name);
                 if (!prereq_tech || checked_techs.find(prereq_tech) != checked_techs.end())
                     continue;
 
@@ -485,8 +467,7 @@ std::string TechManager::FindFirstDependencyCycle() {
 std::string TechManager::FindRedundantDependency() {
     assert(!m_techs.empty());
 
-    for (iterator it = begin(); it != end(); ++it) {
-        const Tech* tech = *it;
+    for (const Tech* tech : m_techs) {
         if (!tech) {
             std::stringstream stream;
             stream << "ERROR: Missing referenced tech for unknown reasons...";
@@ -494,23 +475,23 @@ std::string TechManager::FindRedundantDependency() {
         }
         std::set<std::string> prereqs = tech->Prerequisites();
         std::map<std::string, std::string> techs_unlocked_by_prereqs;
-        for (std::set<std::string>::const_iterator prereq_it = prereqs.begin(); prereq_it != prereqs.end(); ++prereq_it) {
-            const Tech* prereq_tech = GetTech(*prereq_it);
+        for (const std::string& prereq_name : prereqs) {
+            const Tech* prereq_tech = GetTech(prereq_name);
             if (!prereq_tech) {
                 std::stringstream stream;
-                stream << "ERROR: Tech \"" << tech->Name() << "\" requires a missing or malformed tech \"" << *prereq_it << "\" as its prerequisite.";
+                stream << "ERROR: Tech \"" << tech->Name() << "\" requires a missing or malformed tech \"" << prereq_name << "\" as its prerequisite.";
                 return stream.str();
             }
             AllChildren(prereq_tech, techs_unlocked_by_prereqs);
         }
-        for (std::set<std::string>::const_iterator prereq_it = prereqs.begin(); prereq_it != prereqs.end(); ++prereq_it) {
-            std::map<std::string, std::string>::const_iterator map_it = techs_unlocked_by_prereqs.find(*prereq_it);
+        for (const std::string& prereq_name : prereqs) {
+            std::map<std::string, std::string>::const_iterator map_it = techs_unlocked_by_prereqs.find(prereq_name);
             if (map_it != techs_unlocked_by_prereqs.end()) {
                 std::stringstream stream;
                 stream << "ERROR: Redundant dependency found in techs.txt (A <-- B means A is a prerequisite of B): "
                        << map_it->second << " <-- " << map_it->first << ", "
-                       << map_it->first << " <-- " << (*it)->Name() << ", "
-                       << map_it->second << " <-- " << (*it)->Name() << "; remove the " << map_it->second << " <-- " << (*it)->Name()
+                       << map_it->first << " <-- " << tech->Name() << ", "
+                       << map_it->second << " <-- " << tech->Name() << "; remove the " << map_it->second << " <-- " << tech->Name()
                        << " dependency.";
                 return stream.str();
             }
@@ -520,10 +501,14 @@ std::string TechManager::FindRedundantDependency() {
 }
 
 void TechManager::AllChildren(const Tech* tech, std::map<std::string, std::string>& children) {
-    const std::set<std::string>& unlocked_techs = tech->UnlockedTechs();
-    for (std::set<std::string>::const_iterator it = unlocked_techs.begin(); it != unlocked_techs.end(); ++it) {
-        children[*it] = tech->Name();
-        AllChildren(GetTech(*it), children);
+    for (const std::string& unlocked_tech : tech->UnlockedTechs()) {
+        if (unlocked_tech == tech->Name()) {
+            // infinite loop
+            FatalLogger() << "Tech " << unlocked_tech << " unlocks itself";
+            continue;
+        }
+        children[unlocked_tech] = tech->Name();
+        AllChildren(GetTech(unlocked_tech), children);
     }
 }
 
@@ -548,8 +533,7 @@ std::vector<std::string> TechManager::RecursivePrereqs(const std::string& tech_n
     const Empire* empire = GetEmpire(empire_id);
 
     // traverse list, appending new prereqs to it, and putting unique prereqs into set
-    for (std::list<std::string>::iterator it = prereqs_list.begin(); it != prereqs_list.end(); ++it) {
-        std::string cur_name = *it;
+    for (const std::string& cur_name : prereqs_list) {
         const Tech* cur_tech = this->GetTech(cur_name);
 
         // check if this tech is already in the map of prereqs.  If so, it has already been processed, and can be skipped.
@@ -571,9 +555,8 @@ std::vector<std::string> TechManager::RecursivePrereqs(const std::string& tech_n
 
     // extract sorted techs into vector, to be passed to signal...
     std::vector<std::string> retval;
-    for (std::multimap<float, std::string>::const_iterator it = techs_to_add_map.begin();
-         it != techs_to_add_map.end(); ++it)
-    { retval.push_back(it->second); }
+    for (const std::multimap<float, std::string>::value_type& tech_to_add : techs_to_add_map)
+    { retval.push_back(tech_to_add.second); }
 
     return retval;
 }
