@@ -1,39 +1,42 @@
 #ifndef _Universe_h_
 #define _Universe_h_
 
-#include "Enums.h"
+
+#include "EnumsFwd.h"
 #include "ObjectMap.h"
-#include "TemporaryPtr.h"
 #include "UniverseObject.h"
 
 #include <boost/signals2/signal.hpp>
 #include <boost/unordered_map.hpp>
+// Fix for issue #1513 (boost ticket #12978)
+// Starting with v1.64, boost::numeric::ublas was not updated for changes to
+// boost::serialization, which moved array_wrapper to separate header
+#if BOOST_VERSION >= 106400
+    #include <boost/serialization/array_wrapper.hpp>
+#endif
 #include <boost/numeric/ublas/symmetric.hpp>
 #include <boost/serialization/access.hpp>
 #include <boost/thread/shared_mutex.hpp>
 
-#include <vector>
 #include <list>
 #include <map>
-#include <string>
+#include <memory>
 #include <set>
+#include <string>
+#include <vector>
 
 #include "../util/Export.h"
 
-#ifdef FREEORION_WIN32
-// because the linker gets confused about Win32 API functions...
-#  undef GetObject
-#  undef GetObjectA
-#endif
 
 class Empire;
 struct UniverseObjectVisitor;
 class XMLElement;
 class ShipDesign;
 class System;
+class Pathfinder;
 namespace Condition {
     struct ConditionBase;
-    typedef std::vector<TemporaryPtr<const UniverseObject> > ObjectSet;
+    typedef std::vector<std::shared_ptr<const UniverseObject>> ObjectSet;
 }
 
 namespace Effect {
@@ -41,12 +44,21 @@ namespace Effect {
     struct TargetsAndCause;
     struct SourcedEffectsGroup;
     class EffectsGroup;
-    typedef std::vector<TemporaryPtr<UniverseObject> > TargetSet;
-    typedef std::map<int, std::map<MeterType, std::vector<AccountingInfo> > > AccountingMap;
-    typedef std::vector<std::pair<SourcedEffectsGroup, TargetsAndCause> > TargetsCauses;
-    typedef std::map<int, std::map<MeterType, double> > DiscrepancyMap;
+    typedef std::vector<std::shared_ptr<UniverseObject>> TargetSet;
+    typedef std::map<int, std::map<MeterType, std::vector<AccountingInfo>>> AccountingMap;
+    typedef std::vector<std::pair<SourcedEffectsGroup, TargetsAndCause>> TargetsCauses;
+    typedef std::map<int, std::map<MeterType, double>> DiscrepancyMap;
 }
 
+#if defined(_MSC_VER)
+#  if (_MSC_VER == 1900)
+namespace boost {
+    FO_COMMON_API const volatile Universe* get_pointer(const volatile Universe* p);
+}
+#  endif
+#endif
+
+
 /** The Universe class contains the majority of FreeOrion gamestate: All the
   * UniverseObjects in a game, and (of less importance) all ShipDesigns in a
   * game.  (Other gamestate is contained in the Empire class.)
@@ -65,13 +77,13 @@ private:
     typedef std::map<int, VisibilityTurnMap>        ObjectVisibilityTurnMap;        ///< Most recent turn number on which the objects were observed at various Visibility ratings; keyed by object id
     typedef std::map<int, ObjectVisibilityTurnMap>  EmpireObjectVisibilityTurnMap;  ///< Each empire's most recent turns on which object information was known; keyed by empire id
 
-    typedef std::map<int, std::set<int> >           ObjectKnowledgeMap;             ///< IDs of Empires which know information about an object (or deleted object); keyed by object id
+    typedef std::map<int, std::set<int>>            ObjectKnowledgeMap;             ///< IDs of Empires which know information about an object (or deleted object); keyed by object id
 
 public:
     typedef std::map<int, Visibility>               ObjectVisibilityMap;            ///< map from object id to Visibility level for a particular empire
     typedef std::map<int, ObjectVisibilityMap>      EmpireObjectVisibilityMap;      ///< map from empire id to ObjectVisibilityMap for that empire
 
-    typedef std::map<int, std::set<std::string> >   ObjectSpecialsMap;              ///< map from object id to names of specials on an object
+    typedef std::map<int, std::set<std::string>>    ObjectSpecialsMap;              ///< map from object id to names of specials on an object
     typedef std::map<int, ObjectSpecialsMap>        EmpireObjectSpecialsMap;        ///< map from empire id to ObjectSpecialsMap of known specials for objects for that empire
 
     typedef std::map<int, ShipDesign*>              ShipDesignMap;                  ///< ShipDesigns in universe; keyed by design id
@@ -79,12 +91,13 @@ public:
 
     /** \name Signal Types */ //@{
     /** emitted just before the UniverseObject is deleted */
-    typedef boost::signals2::signal<void (TemporaryPtr<const UniverseObject>)> UniverseObjectDeleteSignalType;
+    typedef boost::signals2::signal<void (std::shared_ptr<const UniverseObject>)> UniverseObjectDeleteSignalType;
     //@}
 
     /** \name Structors */ //@{
-    Universe();                                     ///< default ctor
-    virtual ~Universe();                            ///< dtor
+    Universe();
+
+    virtual ~Universe();
     //@}
 
     /** \name Accessors */ //@{
@@ -148,92 +161,22 @@ public:
       * that the empire with id \a empire_id can see this turn. */
     std::set<std::string>   GetObjectVisibleSpecialsByEmpire(int object_id, int empire_id) const;
 
-    /** Returns the straight-line distance between the objects with the given
-      * IDs. \throw std::out_of_range This function will throw if either object
-      * ID is out of range. */
-    double                  LinearDistance(int object1_id, int object2_id) const;
-
-    /** Returns the number of starlane jumps between the systems with the given
-      * IDs. If there is no path between the systems, -1 is returned.
-      * \throw std::out_of_range This function will throw if either system
-      * ID is not a valid system id. */
-    short                   JumpDistanceBetweenSystems(int system1_id, int system2_id) const;
-
-    /** Returns the number of starlane jumps between any two objects, accounting
-      * for cases where one or the other are fleets / ships on starlanes between
-      * systems. Returns INT_MAX when no path exists, or either object does not
-      * exist. */
-    int                     JumpDistanceBetweenObjects(int object1_id, int object2_id) const;
-
-
-    /** Returns the sequence of systems, including \a system1_id and
-      * \a system2_id, that defines the shortest path from \a system1 to
-      * \a system2, and the distance travelled to get there.  If no such path
-      * exists, the list will be empty.  Note that the path returned may be via
-      * one or more starlane, or may be "offroad".  The path is calculated
-      * using the visibility for empire \a empire_id, or without regard to
-      * visibility if \a empire_id == ALL_EMPIRES.
-      * \throw std::out_of_range This function will throw if either system ID
-      * is out of range, or if the empire ID is not known. */
-    std::pair<std::list<int>, double>
-                            ShortestPath(int system1_id, int system2_id, int empire_id = ALL_EMPIRES) const;
-
-    /** Returns the shortest starlane path distance between any two objects, accounting
-      * for cases where one or the other are fleets / ships on starlanes between
-      * systems. Returns -1 when no path exists, or either object does not
-      * exist. */
-    double                  ShortestPathDistance(int object1_id, int object2_id) const;
-
-    /** Returns the sequence of systems, including \a system1 and \a system2,
-      * that defines the path with the fewest jumps from \a system1 to
-      * \a system2, and the number of jumps to get there.  If no such path
-      * exists, the list will be empty.  The path is calculated using the
-      * visibility for empire \a empire_id, or without regard to visibility if
-      * \a empire_id == ALL_EMPIRES.  \throw std::out_of_range This function
-      * will throw if either system ID is out of range or if the empire ID is
-      * not known. */
-    std::pair<std::list<int>, int>
-                            LeastJumpsPath(int system1_id, int system2_id, int empire_id = ALL_EMPIRES,
-                                           int max_jumps = INT_MAX) const;
-
-    /** Returns whether there is a path known to empire \a empire_id between
-      * system \a system1 and system \a system2.  The path is calculated using
-      * the visibility for empire \a empire_id, or without regard to visibility
-      * if \a empire_id == ALL_EMPIRES.  \throw std::out_of_range This function
-      * will throw if either system ID is out of range or if the empire ID is
-      * not known. */
-    bool                    SystemsConnected(int system1_id, int system2_id, int empire_id = ALL_EMPIRES) const;
-
-    /** Returns true iff \a system is reachable from another system (i.e. it
-      * has at least one known starlane to it).   This does not guarantee that
-      * the system is reachable from any specific other system, as two separate
-      * groups of locally but not globally interconnected systems may exist.
-      * The starlanes considered depend on their visibility for empire
-      * \a empire_id, or without regard to visibility if
-      * \a empire_id == ALL_EMPIRES. */
-    bool                    SystemHasVisibleStarlanes(int system_id, int empire_id = ALL_EMPIRES) const;
-
-    /** Returns the systems that are one starlane hop away from system
-      * \a system.  The returned systems are indexed by distance from
-      * \a system.  The neighborhood is calculated using the visibility
-      * for empire \a empire_id, or without regard to visibility if
-      * \a empire_id == ALL_EMPIRES.
-      * \throw std::out_of_range This function will throw if the  system
-      * ID is out of range. */
-    std::multimap<double, int>              ImmediateNeighbors(int system_id, int empire_id = ALL_EMPIRES) const;
-
-    /** Returns the id of the System object that is closest to the specified
-      * (\a x, \a y) location on the map, by direct-line distance. */
-    int                                     NearestSystemTo(double x, double y) const;
+    /** Return the Pathfinder */
+    std::shared_ptr<const Pathfinder> GetPathfinder() const {return m_pathfinder;}
 
     /** Returns map, indexed by object id, to map, indexed by MeterType,
       * to vector of EffectAccountInfo for the meter, in order effects
       * were applied to the meter. */
     const Effect::AccountingMap&            GetEffectAccountingMap() const {return m_effect_accounting_map;}
 
-    const std::map<std::string, std::map<int, std::map<int, double> > >&
+    const std::map<std::string, std::map<int, std::map<int, double>>>&
                                             GetStatRecords() const { return m_stat_records; }
 
+    /** Number of combat rounds
+     * TODO make into a configurable game rule
+     */
+    const int                               GetNumCombatRounds() const;
+
     mutable UniverseObjectDeleteSignalType UniverseObjectDeleteSignal; ///< the state changed signal object for this UniverseObject
     //@}
 
@@ -426,70 +369,45 @@ public:
     bool            AllObjectsVisible() const { return m_all_objects_visible; }
 
     /** \name Generators */ //@{
-    TemporaryPtr<Ship> CreateShip(int id = INVALID_OBJECT_ID);
-    TemporaryPtr<Ship> CreateShip(int empire_id, int design_id, const std::string& species_name,
-                                  int produced_by_empire_id = ALL_EMPIRES, int id = INVALID_OBJECT_ID);
+    std::shared_ptr<Ship> CreateShip(int id = INVALID_OBJECT_ID);
+    std::shared_ptr<Ship> CreateShip(int empire_id, int design_id, const std::string& species_name,
+                                     int produced_by_empire_id = ALL_EMPIRES, int id = INVALID_OBJECT_ID);
 
-    TemporaryPtr<Fleet> CreateFleet(int id = INVALID_OBJECT_ID);
-    TemporaryPtr<Fleet> CreateFleet(const std::string& name, double x, double y, int owner, int id = INVALID_OBJECT_ID);
+    std::shared_ptr<Fleet> CreateFleet(int id = INVALID_OBJECT_ID);
+    std::shared_ptr<Fleet> CreateFleet(const std::string& name, double x, double y, int owner, int id = INVALID_OBJECT_ID);
 
-    TemporaryPtr<Planet> CreatePlanet(int id = INVALID_OBJECT_ID);
-    TemporaryPtr<Planet> CreatePlanet(PlanetType type, PlanetSize size, int id = INVALID_OBJECT_ID);
+    std::shared_ptr<Planet> CreatePlanet(int id = INVALID_OBJECT_ID);
+    std::shared_ptr<Planet> CreatePlanet(PlanetType type, PlanetSize size, int id = INVALID_OBJECT_ID);
 
-    TemporaryPtr<System> CreateSystem(int id = INVALID_OBJECT_ID);
-    TemporaryPtr<System> CreateSystem(StarType star, const std::string& name, double x, double y, int id = INVALID_OBJECT_ID);
-    TemporaryPtr<System> CreateSystem(StarType star, const std::map<int, bool>& lanes_and_holes,
-                                      const std::string& name, double x, double y, int id = INVALID_OBJECT_ID);
+    std::shared_ptr<System> CreateSystem(int id = INVALID_OBJECT_ID);
+    std::shared_ptr<System> CreateSystem(StarType star, const std::string& name, double x, double y, int id = INVALID_OBJECT_ID);
+    std::shared_ptr<System> CreateSystem(StarType star, const std::map<int, bool>& lanes_and_holes,
+                                         const std::string& name, double x, double y, int id = INVALID_OBJECT_ID);
 
-    TemporaryPtr<Building> CreateBuilding(int id = INVALID_OBJECT_ID);
-    TemporaryPtr<Building> CreateBuilding(int empire_id, const std::string& building_type,
-                                          int produced_by_empire_id = ALL_EMPIRES, int id = INVALID_OBJECT_ID);
+    std::shared_ptr<Building> CreateBuilding(int id = INVALID_OBJECT_ID);
+    std::shared_ptr<Building> CreateBuilding(int empire_id, const std::string& building_type,
+                                             int produced_by_empire_id = ALL_EMPIRES, int id = INVALID_OBJECT_ID);
 
-    TemporaryPtr<Field> CreateField(int id = INVALID_OBJECT_ID);
-    TemporaryPtr<Field> CreateField(const std::string& field_type, double x, double y, double radius, int id = INVALID_OBJECT_ID);
+    std::shared_ptr<Field> CreateField(int id = INVALID_OBJECT_ID);
+    std::shared_ptr<Field> CreateField(const std::string& field_type, double x, double y, double radius, int id = INVALID_OBJECT_ID);
     //@}
 
 private:
-    /// minimal public interface for distance caches
-    template <class T> struct distance_matrix_storage {
-        typedef T value_type;
-        typedef std::vector<T>& row_ref;
+    /* Pathfinder setup for the viewing empire
+     */
+    std::shared_ptr<Pathfinder> const m_pathfinder;
 
-        distance_matrix_storage() {};
-        distance_matrix_storage(const distance_matrix_storage<T>& src)
-        { resize(src.m_data.size()); };
-
-        size_t size() 
-        { return m_data.size(); }
-
-        void resize(size_t a_size) {
-            const size_t old_size = size();
-
-            m_data.clear();
-            m_data.resize(a_size);
-            m_row_mutexes.resize(a_size);
-            for (size_t i = old_size; i < a_size; ++i)
-                m_row_mutexes[i] = boost::shared_ptr<boost::shared_mutex>(new boost::shared_mutex());
-        }
-
-        std::vector< std::vector<T> > m_data;
-        std::vector< boost::shared_ptr<boost::shared_mutex> > m_row_mutexes;
-        boost::shared_mutex m_mutex;
-    };
-
-    /** Inserts object \a obj into the universe; returns a TemporaryPtr
+    /** Inserts object \a obj into the universe; returns a std::shared_ptr
       * to the inserted object. */
     template <class T>
-    TemporaryPtr<T>             Insert(T* obj);
+    std::shared_ptr<T> Insert(T* obj);
 
     /** Inserts object \a obj of given ID into the universe; returns a
-      * TemporaryPtr to the inserted object on proper insert, or a null
-      * TemporaryPtr on failure.  Useful mostly for times when ID needs
+      * std::shared_ptr to the inserted object on proper insert, or a null
+      * std::shared_ptr on failure.  Useful mostly for times when ID needs
       * to be consistent on client and server */
     template <class T>
-    TemporaryPtr<T>             InsertID(T* obj, int id);
-
-    struct GraphImpl;
+    std::shared_ptr<T> InsertID(T* obj, int id);
 
     /** Clears \a targets_causes, and then populates with all
       * EffectsGroups and their targets in the known universe. */
@@ -536,11 +454,7 @@ private:
     ObjectKnowledgeMap              m_empire_stale_knowledge_object_ids;///< map from empire id to (set of object ids that the empire has previously observed but has subsequently been unable to detect at its last known location despite expecting to be able to detect it based on stealth of the object and having detectors in range)
 
     ShipDesignMap                   m_ship_designs;                     ///< ship designs in the universe
-    std::map<int, std::set<int> >   m_empire_known_ship_design_ids;     ///< ship designs known to each empire
-
-    mutable distance_matrix_storage<short>  m_system_jumps;             ///< indexed by system graph index (not system id), caches the smallest number of jumps to travel between all the systems
-    boost::shared_ptr<GraphImpl>            m_graph_impl;               ///< a graph in which the systems are vertices and the starlanes are edges
-    boost::unordered_map<int, size_t>       m_system_id_to_graph_index;
+    std::map<int, std::set<int>>   m_empire_known_ship_design_ids;     ///< ship designs known to each empire
 
     Effect::AccountingMap           m_effect_accounting_map;            ///< map from target object id, to map from target meter, to orderered list of structs with details of an effect and what it does to the meter
     Effect::DiscrepancyMap          m_effect_discrepancy_map;           ///< map from target object id, to map from target meter, to discrepancy between meter's actual initial value, and the initial value that this meter should have as far as the client can tell: the unknown factor affecting the meter
@@ -548,14 +462,14 @@ private:
     int                             m_last_allocated_object_id;
     int                             m_last_allocated_design_id;
 
-    std::map<int, std::set<int> >   m_marked_destroyed;                 ///< used while applying effects to cache objects that have been destroyed.  this allows to-be-destroyed objects to remain undestroyed until all effects have been processed, which ensures that to-be-destroyed objects still exist when other effects need to access them as a source object. key is destroyed object, and value set are the ids of objects that caused the destruction (may be multiples destroying a single target on a given turn)
+    std::map<int, std::set<int>>    m_marked_destroyed;                 ///< used while applying effects to cache objects that have been destroyed.  this allows to-be-destroyed objects to remain undestroyed until all effects have been processed, which ensures that to-be-destroyed objects still exist when other effects need to access them as a source object. key is destroyed object, and value set are the ids of objects that caused the destruction (may be multiples destroying a single target on a given turn)
 
     double                          m_universe_width;
     bool                            m_inhibit_universe_object_signals;
     int                             m_encoding_empire;                  ///< used during serialization to globally set what empire knowledge to use
     bool                            m_all_objects_visible;              ///< flag set to skip visibility tests and make everything visible to all players
 
-    std::map<std::string, std::map<int, std::map<int, double> > >
+    std::map<std::string, std::map<int, std::map<int, double>>>
                                     m_stat_records;                     ///< storage for statistics calculated for empires. Indexed by stat name (string), contains a map indexed by empire id, contains a map from turn number (int) to stat value (double).
 
     /** Fills \a designs_to_serialize with ShipDesigns known to the empire with
@@ -591,7 +505,7 @@ private:
     void    GetEmpireStaleKnowledgeObjects(ObjectKnowledgeMap& empire_stale_knowledge_object_ids, int encoding_empire) const;
 
     template <class T>
-    TemporaryPtr<T> InsertNewObject(T* object);
+    std::shared_ptr<T> InsertNewObject(T* object);
 
     friend class boost::serialization::access;
     template <class Archive>
