@@ -12,6 +12,7 @@
 #include "System.h"
 #include "Species.h"
 #include "ValueRef.h"
+#include "Enums.h"
 
 //////////////////////////////////////////
 //  Universe Setup Functions            //
@@ -136,7 +137,7 @@ namespace Delauney {
 
     /** runs a Delauney Triangulation routine on a set of 2D points extracted
       * from an array of systems returns the list of triangles produced */
-    std::list<Delauney::DTTriangle>* DelauneyTriangulate(std::vector<TemporaryPtr<System> > &systems) {
+    std::list<Delauney::DTTriangle>* DelauneyTriangulate(std::vector<std::shared_ptr<System>> &systems) {
 
         int n, c, theSize, num, num2; // loop counters, storage for retreived size of a vector, temp storage
         std::list<Delauney::DTTriangle>::iterator itCur, itEnd;
@@ -155,9 +156,8 @@ namespace Delauney {
 
         // extract systems positions, and store in vector.  Can't use actual systems data since
         // systems have position limitations which would interfere with algorithm
-        theSize = static_cast<int>(systems.size());
-        for (n = 0; n < theSize; n++) {
-            points.push_back(Delauney::DTPoint(systems[n]->X(), systems[n]->Y()));
+        for (std::shared_ptr<System> system : systems) {
+            points.push_back(Delauney::DTPoint(system->X(), system->Y()));
         }
 
         // add points for covering triangle.  the point positions should be big enough to form a triangle
@@ -311,14 +311,14 @@ namespace {
 
 #ifdef OUTPUT_PLANS_LIST
         DebugLogger() << "Starting Fleet Plans:";
-        for (iterator it = begin(); it != end(); ++it)
-            DebugLogger() << " ... " << (*it)->Name();
+        for (FleetPlan* plan : m_plans)
+            DebugLogger() << " ... " << plan->Name();
 #endif
     }
 
     FleetPlanManager::~FleetPlanManager() {
-        for (std::vector<FleetPlan*>::iterator it = m_plans.begin(); it != m_plans.end(); ++it)
-            delete *it;
+        for (FleetPlan* plan : m_plans)
+            delete plan;
         m_plans.clear();
     }
 };
@@ -362,15 +362,15 @@ namespace {
 
 //#ifdef OUTPUT_PLANS_LIST
         DebugLogger() << "Starting Monster Fleet Plans:";
-        for (iterator it = begin(); it != end(); ++it)
-            DebugLogger() << " ... " << (*it)->Name() << " spawn rate: " << (*it)->SpawnRate()
-                          << " spawn limit: " << (*it)->SpawnLimit();
+        for (const MonsterFleetPlan* plan : m_plans)
+            DebugLogger() << " ... " << plan->Name() << " spawn rate: " << plan->SpawnRate()
+                          << " spawn limit: " << plan->SpawnLimit();
 //#endif
     }
 
     MonsterFleetPlanManager::~MonsterFleetPlanManager() {
-        for (std::vector<MonsterFleetPlan*>::iterator it = m_plans.begin(); it != m_plans.end(); ++it)
-            delete *it;
+        for (MonsterFleetPlan* plan : m_plans)
+            delete plan;
         m_plans.clear();
     }
 
@@ -382,7 +382,7 @@ namespace {
 namespace {
     /** Used by GenerateStarlanes.  Determines if two systems are connected by
       * maxLaneJumps or less edges on graph. */
-    bool ConnectedWithin(int system1, int system2, int maxLaneJumps, std::vector<std::set<int> >& laneSetArray) {
+    bool ConnectedWithin(int system1, int system2, int maxLaneJumps, std::vector<std::set<int>>& laneSetArray) {
         // list of indices of systems that are accessible from previously visited systems.
         // when a new system is found to be accessible, it is added to the back of the
         // list.  the list is iterated through from front to back to find systems
@@ -462,8 +462,8 @@ namespace {
 
     /** Removes lanes from passed graph that are angularly too close to
       * each other. */
-    void CullAngularlyTooCloseLanes(double maxLaneUVectDotProd, std::vector<std::set<int> >& laneSetArray,
-                                    std::vector<TemporaryPtr<System> > &systems)
+    void CullAngularlyTooCloseLanes(double maxLaneUVectDotProd, std::vector<std::set<int>>& laneSetArray,
+                                    std::vector<std::shared_ptr<System>> &systems)
     {
         // start and end systems of a new lane being considered, and end points of lanes that already exist with that
         // start at the start or destination of the new lane
@@ -486,8 +486,8 @@ namespace {
         // iterators to go through sets of lanes in array
         std::set<int>::iterator laneSetIter1, laneSetIter2, laneSetEnd;
 
-        std::set<std::pair<int, int> > lanesToRemoveSet;  // start and end stars of lanes to be removed in final step...
-        std::set<std::pair<int, int> >::iterator lanesToRemoveIter, lanesToRemoveEnd;
+        std::set<std::pair<int, int>> lanesToRemoveSet;  // start and end stars of lanes to be removed in final step...
+        std::set<std::pair<int, int>>::iterator lanesToRemoveIter, lanesToRemoveEnd;
         std::pair<int, int> lane1, lane2;
 
         int curNumLanes;
@@ -634,8 +634,8 @@ namespace {
 
     /** Removes lanes from passed graph that are angularly too close to
       * each other. */
-    void CullTooLongLanes(double maxLaneLength, std::vector<std::set<int> >& laneSetArray,
-                          std::vector<TemporaryPtr<System> > &systems)
+    void CullTooLongLanes(double maxLaneLength, std::vector<std::set<int>>& laneSetArray,
+                          std::vector<std::shared_ptr<System>> &systems)
     {
         // start and end systems of a new lane being considered, and end points of lanes that already exist with that start
         // at the start or destination of the new lane
@@ -648,10 +648,10 @@ namespace {
         std::set<int>::iterator laneSetIter, laneSetEnd;
 
         // map, indexed by lane length, of start and end stars of lanes to be removed
-        std::multimap<double, std::pair<int, int>, std::greater<double> > lanesToRemoveMap;
-        std::multimap<double, std::pair<int, int>, std::greater<double> >::iterator lanesToRemoveIter, lanesToRemoveEnd;
+        std::multimap<double, std::pair<int, int>, std::greater<double>> lanesToRemoveMap;
+        std::multimap<double, std::pair<int, int>, std::greater<double>>::iterator lanesToRemoveIter, lanesToRemoveEnd;
         std::pair<int, int> lane;
-        typedef std::pair<double, std::pair<int, int> > MapInsertableTypeQQ;
+        typedef std::pair<double, std::pair<int, int>> MapInsertableTypeQQ;
 
         int num_systems = systems.size();
         // make sure data is consistent
@@ -731,16 +731,16 @@ void GenerateStarlanes(int max_jumps_between_systems, int max_starlane_length) {
     std::vector<int> triVerts;  // indices of stars that form vertices of a triangle
 
     // array of set to store final, included starlanes for each star
-    std::vector<std::set<int> > laneSetArray;
+    std::vector<std::set<int>> laneSetArray;
 
     // array of set to store possible starlanes for each star, as extracted form triangulation
-    std::vector<std::set<int> > potential_lane_set_array;
+    std::vector<std::set<int>> potential_lane_set_array;
 
     // iterators for traversing lists of starlanes
     std::set<int>::iterator laneSetIter, laneSetEnd, laneSetIter2, laneSetEnd2;
 
     // get systems
-    std::vector<TemporaryPtr<System> > sys_vec = Objects().FindObjects<System>();
+    std::vector<std::shared_ptr<System>> sys_vec = Objects().FindObjects<System>();
     num_systems = sys_vec.size();  // (actually = number of systems + 1)
 
     // pass systems to Delauney Triangulation routine, getting array of triangles back
@@ -753,13 +753,7 @@ void GenerateStarlanes(int max_jumps_between_systems, int max_starlane_length) {
     Delauney::DTTriangle tri;
     // initialize arrays...
     potential_lane_set_array.resize(num_systems);
-    for (n = 0; n < num_systems; n++) {
-        potential_lane_set_array[n].clear();
-    }
     laneSetArray.resize(num_systems);
-    for (n = 0; n < num_systems; n++) {
-        laneSetArray[n].clear();
-    }
 
     // extract triangles from list, add edges to sets of potential starlanes for each star (in array)
     while (!triangle_list->empty()) {
@@ -828,9 +822,8 @@ void GenerateStarlanes(int max_jumps_between_systems, int max_starlane_length) {
 
     // add the starlane to the stars
     for (n = 0; n < num_systems; ++n) {
-        const std::set<int>& lanes = laneSetArray[n];
-        for (std::set<int>::const_iterator it = lanes.begin(); it != lanes.end(); ++it)
-            sys_vec[n]->AddStarlane(sys_vec[*it]->ID()); // System::AddStarlane() expects a system ID
+        for (int system_idx : laneSetArray[n])
+            sys_vec[n]->AddStarlane(sys_vec[system_idx]->ID()); // System::AddStarlane() expects a system ID
     }
 
     DebugLogger() << "Initializing System Graph";
@@ -854,20 +847,20 @@ void SetActiveMetersToTargetMaxCurrentValues(ObjectMap& object_map) {
 
     // check for each pair of meter types.  if both exist, set active
     // meter current value equal to target meter current value.
-    for (ObjectMap::iterator<> it = object_map.begin(); it != object_map.end(); ++it) {
-        for (std::map<MeterType, MeterType>::const_iterator meter_it = meters.begin(); meter_it != meters.end(); ++meter_it)
-            if (Meter* meter = it->GetMeter(meter_it->first))
-                if (Meter* targetmax_meter = it->GetMeter(meter_it->second))
+    for (std::shared_ptr<UniverseObject> object : object_map) {
+        for (std::map<MeterType, MeterType>::value_type& entry : meters)
+            if (Meter* meter = object->GetMeter(entry.first))
+                if (Meter* targetmax_meter = object->GetMeter(entry.second))
                     meter->SetCurrent(targetmax_meter->Current());
     }
 }
 
 void SetNativePopulationValues(ObjectMap& object_map) {
-    for (ObjectMap::iterator<> it = object_map.begin(); it != object_map.end(); ++it) {
-        Meter* meter = it->GetMeter(METER_POPULATION);
-        Meter* targetmax_meter = it->GetMeter(METER_TARGET_POPULATION);
+    for (std::shared_ptr<UniverseObject> object : object_map) {
+        Meter* meter = object->GetMeter(METER_POPULATION);
+        Meter* targetmax_meter = object->GetMeter(METER_TARGET_POPULATION);
         // only applies to unowned planets
-        if (meter && targetmax_meter && it->Unowned()) {
+        if (meter && targetmax_meter && object->Unowned()) {
             double r = RandZeroToOne();
             double factor = (0.1 < r) ? r : 0.1;
             meter->SetCurrent(targetmax_meter->Current() * factor);
@@ -877,8 +870,8 @@ void SetNativePopulationValues(ObjectMap& object_map) {
 
 bool SetEmpireHomeworld(Empire* empire, int planet_id, std::string species_name) {
     // get home planet and system, check if they exist
-    TemporaryPtr<Planet> home_planet = GetPlanet(planet_id);
-    TemporaryPtr<System> home_system;
+    std::shared_ptr<Planet> home_planet = GetPlanet(planet_id);
+    std::shared_ptr<System> home_system;
     if (home_planet)
         home_system = GetSystem(home_planet->SystemID());
     if (!home_planet || !home_system) {
@@ -903,8 +896,8 @@ bool SetEmpireHomeworld(Empire* empire, int planet_id, std::string species_name)
         // invert map from planet type to environments to map from
         // environments to type, sorted by environment
         std::map<PlanetEnvironment, PlanetType> sept;
-        for (std::map<PlanetType, PlanetEnvironment>::const_iterator it = spte.begin(); it != spte.end(); ++it)
-            sept[it->second] = it->first;
+        for (const std::map<PlanetType, PlanetEnvironment>::value_type& entry : spte)
+            sept[entry.second] = entry.first;
         // assuming enum values are ordered in increasing goodness...
         PlanetType preferred_planet_type = sept.rbegin()->second;
 
@@ -936,19 +929,17 @@ void InitEmpires(const std::map<int, PlayerSetupData>& player_setup_data)
     std::vector<GG::Clr> colors = EmpireColors();
 
     // create empire objects and do some basic initilization for each player
-    int player_i = 0;
-    for (std::map<int, PlayerSetupData>::const_iterator setup_data_it = player_setup_data.begin();
-         setup_data_it != player_setup_data.end(); ++setup_data_it, ++player_i)
+    for (const std::map<int, PlayerSetupData>::value_type& entry : player_setup_data)
     {
-        int         player_id =     setup_data_it->first;
+        int         player_id =     entry.first;
         if (player_id == Networking::INVALID_PLAYER_ID)
             ErrorLogger() << "InitEmpires player id (" << player_id << ") is invalid";
 
         // use player ID for empire ID so that the calling code can get the
         // correct empire for each player ID  in player_setup_data
         int         empire_id =     player_id;
-        std::string player_name =   setup_data_it->second.m_player_name;
-        GG::Clr     empire_colour = setup_data_it->second.m_empire_color;
+        std::string player_name =   entry.second.m_player_name;
+        GG::Clr     empire_colour = entry.second.m_empire_color;
 
         // validate or generate empire colour
         // ensure no other empire gets auto-assigned this colour automatically
@@ -970,7 +961,7 @@ void InitEmpires(const std::map<int, PlayerSetupData>& player_setup_data)
         }
 
         // set generic default empire name
-        std::string empire_name = UserString("EMPIRE") + boost::lexical_cast<std::string>(empire_id);
+        std::string empire_name = UserString("EMPIRE") + std::to_string(empire_id);
 
         DebugLogger() << "Universe::InitEmpires creating new empire" << " with ID: " << empire_id
                       << " for player: " << player_name << " (with player id: " << player_id << ")";
