@@ -41,7 +41,7 @@ protected:
 class FO_COMMON_API MonsterFleetPlan : public FleetPlan {
 public:
     MonsterFleetPlan(const std::string& fleet_name, const std::vector<std::string>& ship_design_names,
-                     double spawn_rate = 1.0, int spawn_limit = 9999, const Condition::ConditionBase* location = 0,
+                     double spawn_rate = 1.0, int spawn_limit = 9999, const Condition::ConditionBase* location = nullptr,
                      bool lookup_name_userstring = false) :
         FleetPlan(fleet_name, ship_design_names, lookup_name_userstring),
         m_spawn_rate(spawn_rate),
@@ -52,7 +52,7 @@ public:
         FleetPlan(),
         m_spawn_rate(1.0),
         m_spawn_limit(9999),
-        m_location(0)
+        m_location(nullptr)
     {}
     virtual ~MonsterFleetPlan()
     { delete m_location; }
