@@ -6,8 +6,10 @@
 #include "Meter.h"
 #include "System.h"
 #include "Special.h"
+#include "Pathfinder.h"
 #include "Universe.h"
 #include "Predicates.h"
+#include "Enums.h"
 
 #include <stdexcept>
 #include <boost/lexical_cast.hpp>
@@ -22,7 +24,7 @@ const int       UniverseObject::INVALID_OBJECT_AGE = -(1 << 30) - 1;  // using b
 const int       UniverseObject::SINCE_BEFORE_TIME_AGE = (1 << 30) + 1;
 
 UniverseObject::UniverseObject() :
-    StateChangedSignal(blocking_combiner<boost::signals2::optional_last_value<void> >(GetUniverse().UniverseObjectSignalsInhibited())),
+    StateChangedSignal(blocking_combiner<boost::signals2::optional_last_value<void>>(GetUniverse().UniverseObjectSignalsInhibited())),
     m_name(""),
     m_id(INVALID_OBJECT_ID),
     m_x(INVALID_POSITION),
@@ -36,7 +38,7 @@ UniverseObject::UniverseObject() :
 }
 
 UniverseObject::UniverseObject(const std::string name, double x, double y) :
-    StateChangedSignal(blocking_combiner<boost::signals2::optional_last_value<void> >(GetUniverse().UniverseObjectSignalsInhibited())),
+    StateChangedSignal(blocking_combiner<boost::signals2::optional_last_value<void>>(GetUniverse().UniverseObjectSignalsInhibited())),
     m_name(name),
     m_id(INVALID_OBJECT_ID),
     m_x(x),
@@ -52,10 +54,10 @@ UniverseObject::UniverseObject(const std::string name, double x, double y) :
 UniverseObject::~UniverseObject()
 {}
 
-void UniverseObject::Copy(TemporaryPtr<const UniverseObject> copied_object, Visibility vis,
+void UniverseObject::Copy(std::shared_ptr<const UniverseObject> copied_object, Visibility vis,
                           const std::set<std::string>& visible_specials)
 {
-    if (copied_object == this)
+    if (copied_object.get() == this)
         return;
     if (!copied_object) {
         ErrorLogger() << "UniverseObject::Copy passed a null object";
@@ -63,10 +65,9 @@ void UniverseObject::Copy(TemporaryPtr<const UniverseObject> copied_object, Visi
     }
 
     std::map<MeterType, Meter> censored_meters = copied_object->CensoredMeters(vis);
-    for (std::map<MeterType, Meter>::const_iterator it = copied_object->m_meters.begin();
-         it != copied_object->m_meters.end(); ++it)
+    for (const std::map<MeterType, Meter>::value_type& entry : copied_object->m_meters)
     {
-        MeterType type = it->first;
+        MeterType type = entry.first;
 
         // get existing meter in this object, or create a default one
         std::map<MeterType, Meter>::iterator m_meter_it = m_meters.find(type);
@@ -103,11 +104,9 @@ void UniverseObject::Copy(TemporaryPtr<const UniverseObject> copied_object, Visi
         this->m_y =                     copied_object->m_y;
 
         this->m_specials.clear();
-        for (std::map<std::string, std::pair<int, float> >::const_iterator copied_special_it = copied_object->m_specials.begin();
-             copied_special_it != copied_object->m_specials.end(); ++copied_special_it)
-        {
-            if (visible_specials.find(copied_special_it->first) != visible_specials.end())
-            { this->m_specials[copied_special_it->first] = copied_special_it->second; }
+        for (const std::map<std::string, std::pair<int, float>>::value_type& entry_special : copied_object->m_specials) {
+            if (visible_specials.find(entry_special.first) != visible_specials.end())
+            { this->m_specials[entry_special.first] = entry_special.second; }
         }
 
         if (vis >= VIS_PARTIAL_VISIBILITY) {
@@ -153,21 +152,21 @@ int UniverseObject::Owner() const
 int UniverseObject::SystemID() const
 { return m_system_id; }
 
-const std::map<std::string, std::pair<int, float> >& UniverseObject::Specials() const
+const std::map<std::string, std::pair<int, float>>& UniverseObject::Specials() const
 { return m_specials; }
 
 bool UniverseObject::HasSpecial(const std::string& name) const
 { return m_specials.find(name) != m_specials.end(); }
 
 int UniverseObject::SpecialAddedOnTurn(const std::string& name) const {
-    std::map<std::string, std::pair<int, float> >::const_iterator it = m_specials.find(name);
+    std::map<std::string, std::pair<int, float>>::const_iterator it = m_specials.find(name);
     if (it == m_specials.end())
         return INVALID_GAME_TURN;
     return it->second.first;
 }
 
 float UniverseObject::SpecialCapacity(const std::string& name) const {
-    std::map<std::string, std::pair<int, float> >::const_iterator it = m_specials.find(name);
+    std::map<std::string, std::pair<int, float>>::const_iterator it = m_specials.find(name);
     if (it == m_specials.end())
         return 0.0f;
     return it->second.second;
@@ -183,7 +182,7 @@ UniverseObjectType UniverseObject::ObjectType() const
 { return INVALID_UNIVERSE_OBJECT_TYPE; }
 
 std::string UniverseObject::Dump() const {
-    TemporaryPtr<const System> system = GetSystem(this->SystemID());
+    std::shared_ptr<const System> system = GetSystem(this->SystemID());
 
     std::stringstream os;
 
@@ -198,8 +197,8 @@ std::string UniverseObject::Dump() const {
             os << "  at: " << sys_name;
     } else {
         os << "  at: (" << this->X() << ", " << this->Y() << ")";
-        int near_id = GetUniverse().NearestSystemTo(this->X(), this->Y());
-        TemporaryPtr<const System> system = GetSystem(near_id);
+        int near_id = GetPathfinder()->NearestSystemTo(this->X(), this->Y());
+        std::shared_ptr<const System> system = GetSystem(near_id);
         if (system) {
             const std::string& sys_name = system->Name();
             if (sys_name.empty())
@@ -219,12 +218,12 @@ std::string UniverseObject::Dump() const {
     }
     os << " created on turn: " << m_created_on_turn
        << " specials: ";
-    for (std::map<std::string, std::pair<int, float> >::const_iterator it = m_specials.begin(); it != m_specials.end(); ++it)
-        os << "(" << it->first << ", " << it->second.first << ", " << it->second.second << ") ";
+    for (const std::map<std::string, std::pair<int, float>>::value_type& entry : m_specials)
+        os << "(" << entry.first << ", " << entry.second.first << ", " << entry.second.second << ") ";
     os << "  Meters: ";
-    for (std::map<MeterType, Meter>::const_iterator it = m_meters.begin(); it != m_meters.end(); ++it)
-        os << UserString(EnumToString(it->first))
-           << ": " << it->second.Dump() << "  ";
+    for (const std::map<MeterType, Meter>::value_type& entry : m_meters)
+        os << entry.first
+           << ": " << entry.second.Dump() << "  ";
     return os.str();
 }
 
@@ -238,9 +237,7 @@ const std::set<int>& UniverseObject::ContainedObjectIDs() const
 std::set<int> UniverseObject::VisibleContainedObjectIDs(int empire_id) const {
     std::set<int> retval;
     const Universe& universe = GetUniverse();
-    const std::set<int>& contained_obj_ids = ContainedObjectIDs();
-    for (std::set<int>::const_iterator it = contained_obj_ids.begin(); it != contained_obj_ids.end(); ++it) {
-        int object_id = *it;
+    for (int object_id : ContainedObjectIDs()) {
         if (universe.GetObjectVisibilityByEmpire(object_id, empire_id) >= VIS_BASIC_VISIBILITY)
             retval.insert(object_id);
     }
@@ -260,7 +257,7 @@ const Meter* UniverseObject::GetMeter(MeterType type) const {
     std::map<MeterType, Meter>::const_iterator it = m_meters.find(type);
     if (it != m_meters.end())
         return &(it->second);
-    return 0;
+    return nullptr;
 }
 
 float UniverseObject::CurrentMeterValue(MeterType type) const {
@@ -290,10 +287,10 @@ void UniverseObject::AddMeter(MeterType meter_type) {
 }
 
 bool UniverseObject::Unowned() const
-{ return m_owner_empire_id == ALL_EMPIRES; }
+{ return Owner() == ALL_EMPIRES; }
 
 bool UniverseObject::OwnedBy(int empire) const
-{ return empire != ALL_EMPIRES && empire == m_owner_empire_id; }
+{ return empire != ALL_EMPIRES && empire == Owner(); }
 
 Visibility UniverseObject::GetVisibility(int empire_id) const
 { return GetUniverse().GetObjectVisibilityByEmpire(this->ID(), empire_id); }
@@ -301,8 +298,8 @@ Visibility UniverseObject::GetVisibility(int empire_id) const
 const std::string& UniverseObject::PublicName(int empire_id) const
 { return m_name; }
 
-TemporaryPtr<UniverseObject> UniverseObject::Accept(const UniverseObjectVisitor& visitor) const
-{ return visitor.Visit(boost::const_pointer_cast<UniverseObject>(TemporaryFromThis()));}
+std::shared_ptr<UniverseObject> UniverseObject::Accept(const UniverseObjectVisitor& visitor) const
+{ return visitor.Visit(std::const_pointer_cast<UniverseObject>(shared_from_this())); }
 
 void UniverseObject::SetID(int id) {
     m_id = id;
@@ -320,7 +317,7 @@ void UniverseObject::Move(double x, double y)
 void UniverseObject::MoveTo(int object_id)
 { MoveTo(GetUniverseObject(object_id)); }
 
-void UniverseObject::MoveTo(TemporaryPtr<UniverseObject> object) {
+void UniverseObject::MoveTo(std::shared_ptr<UniverseObject> object) {
     if (!object) {
         ErrorLogger() << "UniverseObject::MoveTo : attempted to move to a null object.";
         return;
@@ -329,8 +326,12 @@ void UniverseObject::MoveTo(TemporaryPtr<UniverseObject> object) {
 }
 
 void UniverseObject::MoveTo(double x, double y) {
-    if (x < 0.0 || GetUniverse().UniverseWidth() < x || y < 0.0 || GetUniverse().UniverseWidth() < y)
-        DebugLogger() << "UniverseObject::MoveTo : Placing object \"" + m_name + "\" off the map area.";
+    if ((x < 0.0 || GetUniverse().UniverseWidth() < x || y < 0.0 || GetUniverse().UniverseWidth() < y)
+        && (x != INVALID_POSITION || y != INVALID_POSITION))
+    {
+        DebugLogger() << "UniverseObject::MoveTo : Placing object \"" << m_name << "\" ("
+                      << m_id << ") outside the map area at (" << x << ", " << y << ").";
+    }
 
     if (m_x == x && m_y == y)
         return;
@@ -345,7 +346,7 @@ Meter* UniverseObject::GetMeter(MeterType type) {
     std::map<MeterType, Meter>::iterator it = m_meters.find(type);
     if (it != m_meters.end())
         return &(it->second);
-    return 0;
+    return nullptr;
 }
 
 void UniverseObject::BackPropagateMeters() {
