@@ -2,9 +2,7 @@
 #define _UniverseObject_h_
 
 
-#include "Enums.h"
-#include "TemporaryPtr.h"
-#include "EnableTemporaryFromThis.h"
+#include "EnumsFwd.h"
 #include "../util/Export.h"
 #include "../util/blocking_combiner.h"
 
@@ -21,6 +19,7 @@ class Meter;
 class System;
 class SitRepEntry;
 struct UniverseObjectVisitor;
+FO_COMMON_API extern const int ALL_EMPIRES;
 
 // The ID number assigned to a UniverseObject upon construction;
 // It is assigned an ID later when it is placed in the universe
@@ -44,10 +43,10 @@ FO_COMMON_API extern const int TEMPORARY_OBJECT_ID;
   * Signal.  This means that all mutators on UniverseObject and its subclasses
   * need to emit this signal.  This is how the UI becomes aware that an object
   * that is being displayed has changed.*/
-class FO_COMMON_API UniverseObject : virtual public EnableTemporaryFromThis<UniverseObject> {
+class FO_COMMON_API UniverseObject : virtual public std::enable_shared_from_this<UniverseObject> {
 public:
     /** \name Signal Types */ //@{
-    typedef boost::signals2::signal<void (), blocking_combiner<boost::signals2::optional_last_value<void> > > StateChangedSignalType;
+    typedef boost::signals2::signal<void (), blocking_combiner<boost::signals2::optional_last_value<void>>> StateChangedSignalType;
     //@}
 
     /** \name Slot Types */ //@{
@@ -60,27 +59,42 @@ public:
     virtual double              X() const;                          ///< the X-coordinate of this object
     virtual double              Y() const;                          ///< the Y-coordinate of this object
 
-    int                         Owner() const;                      ///< returns the ID of the empire that owns this object, or ALL_EMPIRES if there is no owner
+    virtual int                 Owner() const;                      ///< returns the ID of the empire that owns this object, or ALL_EMPIRES if there is no owner
     bool                        Unowned() const;                    ///< returns true iff there are no owners of this object
     bool                        OwnedBy(int empire) const;          ///< returns true iff the empire with id \a empire owns this object; unowned objects always return false;
 
     virtual int                 SystemID() const;                   ///< returns the ID number of the system in which this object can be found, or INVALID_OBJECT_ID if the object is not within any system
 
-    const std::map<std::string, std::pair<int, float> >&   Specials() const;        ///< returns the Specials attached to this object
+    const std::map<std::string, std::pair<int, float>>&   Specials() const;         ///< returns the Specials attached to this object
     bool                        HasSpecial(const std::string& name) const;          ///< returns true iff this object has a special with the indicated \a name
     int                         SpecialAddedOnTurn(const std::string& name) const;  ///< returns the turn on which the special with name \a name was added to this object, or INVALID_GAME_TURN if that special is not present
     float                       SpecialCapacity(const std::string& name) const;     ///> returns the capacity of the special with name \a name or 0 if that special is not present
 
-    virtual std::set<std::string>   Tags() const;                                   ///< returns all tags this object has
-    virtual bool                    HasTag(const std::string& name) const;          ///< returns true iff this object has the tag with the indicated \a name
+    /** Returns all tags this object has. */
+    virtual std::set<std::string>   Tags() const;
+
+    /** Returns true iff this object has the tag with the indicated \a name. */
+    virtual bool                HasTag(const std::string& name) const;
 
     virtual UniverseObjectType  ObjectType() const;
-    virtual std::string         Dump() const;                       ///< outputs textual description of object to logger
 
-    virtual int                 ContainerObjectID() const;          ///< returns id of the object that directly contains this object, if any, or INVALID_OBJECT_ID if this object is not contained by any other
-    virtual const std::set<int>&ContainedObjectIDs() const;         ///< returns ids of objects contained within this object
-    virtual bool                Contains(int object_id) const;      ///< returns true if there is an object with id \a object_id is contained within this UniverseObject
-    virtual bool                ContainedBy(int object_id) const;   ///< returns true if there is an object with id \a object_id that contains this UniverseObject
+    /** Outputs textual description of object to logger. */
+    virtual std::string         Dump() const;
+
+    /** Returns id of the object that directly contains this object, if any, or
+        INVALID_OBJECT_ID if this object is not contained by any other. */
+    virtual int                 ContainerObjectID() const;
+
+    /** Returns ids of objects contained within this object. */
+    virtual const std::set<int>&ContainedObjectIDs() const;
+
+    /** Returns true if there is an object with id \a object_id is contained
+        within this UniverseObject. */
+    virtual bool                Contains(int object_id) const;
+
+    /* Returns true if there is an object with id \a object_id that contains
+       this UniverseObject. */
+    virtual bool                ContainedBy(int object_id) const;
 
     std::set<int>               VisibleContainedObjectIDs(int empire_id) const; ///< returns the subset of contained object IDs that is visible to empire with id \a empire_id
 
@@ -88,13 +102,18 @@ public:
     const Meter*                GetMeter(MeterType type) const;                 ///< returns the requested Meter, or 0 if no such Meter of that type is found in this object
     float                       CurrentMeterValue(MeterType type) const;        ///< returns current value of the specified meter \a type
     float                       InitialMeterValue(MeterType type) const;        ///< returns this turn's initial value for the speicified meter \a type
-    virtual float               NextTurnCurrentMeterValue(MeterType type) const;///< returns an estimate of the next turn's current value of the specified meter \a type
+
+    /** Returns an estimate of the next turn's current value of the specified
+        meter \a type. */
+    virtual float               NextTurnCurrentMeterValue(MeterType type) const;
 
     Visibility                  GetVisibility(int empire_id) const; ///< returns the visibility status of this universe object relative to the input empire.
-    virtual const std::string&  PublicName(int empire_id) const;    ///< returns the name of this objectas it appears to empire \a empire_id
 
-    /** accepts a visitor object \see UniverseObjectVisitor */
-    virtual TemporaryPtr<UniverseObject>    Accept(const UniverseObjectVisitor& visitor) const;
+    /** Returns the name of this objectas it appears to empire \a empire_id .*/
+    virtual const std::string&  PublicName(int empire_id) const;
+
+    /** Accepts a visitor object \see UniverseObjectVisitor */
+    virtual std::shared_ptr<UniverseObject> Accept(const UniverseObjectVisitor& visitor) const;
 
     int                         CreationTurn() const;               ///< returns game turn on which object was created
     int                         AgeInTurns() const;                 ///< returns elapsed number of turns between turn object was created and current game turn
@@ -106,7 +125,7 @@ public:
     /** copies data from \a copied_object to this object, limited to only copy
       * data about the copied object that is known to the empire with id
       * \a empire_id (or all data if empire_id is ALL_EMPIRES) */
-    virtual void            Copy(TemporaryPtr<const UniverseObject> copied_object, int empire_id) = 0;
+    virtual void Copy(std::shared_ptr<const UniverseObject> copied_object, int empire_id) = 0;
 
     void                    SetID(int id);                      ///< sets the ID number of the object to \a id
     void                    Rename(const std::string& name);    ///< renames this object to \a name
@@ -114,11 +133,12 @@ public:
     /** moves this object by relative displacements x and y. */
     void                    Move(double x, double y);
 
-    /** calls MoveTo(TemporaryPtr<const UniverseObject>) with the object pointed to by \a object_id. */
+    /** calls MoveTo(std::shared_ptr<const UniverseObject>) with the object
+        pointed to by \a object_id. */
     void                    MoveTo(int object_id);
 
     /** moves this object to exact map coordinates of specified \a object. */
-    void                    MoveTo(TemporaryPtr<UniverseObject> object);
+    void MoveTo(std::shared_ptr<UniverseObject> object);
 
     /** moves this object to map coordinates (x, y). */
     void                    MoveTo(double x, double y);
@@ -127,9 +147,13 @@ public:
     std::map<MeterType, Meter>&
                             Meters() { return m_meters; }           ///< returns this UniverseObject's meters
     Meter*                  GetMeter(MeterType type);               ///< returns the requested Meter, or 0 if no such Meter of that type is found in this object
-    virtual void            BackPropagateMeters();                  ///< sets all this UniverseObject's meters' initial values equal to their current values
 
-    virtual void            SetOwner(int id);                       ///< sets the empire that owns this object
+    /** Sets all this UniverseObject's meters' initial values equal to their
+        current values. */
+    virtual void BackPropagateMeters();
+
+    /** Sets the empire that owns this object. */
+    virtual void SetOwner(int id);
 
     void                    SetSystem(int sys);                     ///< assigns this object to a System.  does not actually move object in universe
     virtual void            AddSpecial(const std::string& name, float capacity = 0.0f); ///< adds the Special \a name to this object, if it is not already present
@@ -138,26 +162,28 @@ public:
 
     /** Performs the movement that this object is responsible for this object's
       * actions during the movement phase of a turn. */
-    virtual void            MovementPhase() {};
+    virtual void MovementPhase()
+    {};
 
     /** Sets current value of max, target and unpaired meters in in this
       * UniverseObject to Meter::DEFAULT_VALUE.  This should be done before any
       * Effects that alter these meter(s) act on the object. */
-    virtual void            ResetTargetMaxUnpairedMeters();
+    virtual void ResetTargetMaxUnpairedMeters();
 
     /** Sets current value of active paired meters (the non-max non-target
       * meters that have a max or target meter associated with them) back to
       * the initial value the meter had at the start of this turn. */
-    virtual void            ResetPairedActiveMeters();
+    virtual void ResetPairedActiveMeters();
 
     /** calls Clamp(min, max) on meters each meter in this UniverseObject, to
       * ensure that meter current values aren't outside the valid range for
       * each meter. */
-    virtual void            ClampMeters();
+    virtual void ClampMeters();
 
-    /** performs the movement that this object is responsible for this object's actions during the pop growth/production/research
-        phase of a turn. */
-    virtual void            PopGrowthProductionResearchPhase() {};
+    /** performs the movement that this object is responsible for this object's
+        actions during the pop growth/production/research phase of a turn. */
+    virtual void PopGrowthProductionResearchPhase()
+    {};
     //@}
 
     static const double         INVALID_POSITION;       ///< the position in x and y at which default-constructed objects are placed
@@ -165,16 +191,20 @@ public:
     static const int            SINCE_BEFORE_TIME_AGE;  ///< the age returned by UniverseObject::AgeInTurns() if an object was created on turn BEFORE_FIRST_TURN
 
 protected:
-    template <class T> friend void boost::python::detail::value_destroyer<false>::execute(T const volatile* p);
     friend class Universe;
     friend class ObjectMap;
-    template <class T> friend void boost::checked_delete(T* x);
 
     /** \name Structors */ //@{
     UniverseObject();
     UniverseObject(const std::string name, double x, double y);
+
+    template <typename T> friend void UniverseObjectDeleter(T*);
+    template <class T> friend void boost::python::detail::value_destroyer<false>::execute(T const volatile* p);
+
+public:
     virtual ~UniverseObject();
 
+protected:
     /** returns new copy of this UniverseObject, limited to only copy data that
       * is visible to the empire with the specified \a empire_id as determined
       * by the detection and visibility system.  Caller takes ownership of
@@ -185,8 +215,10 @@ protected:
     void                    AddMeter(MeterType meter_type); ///< inserts a meter into object as the \a meter_type meter.  Should be used by derived classes to add their specialized meters to objects
     void                    Init();                         ///< adds stealth meter
 
-    void                    Copy(TemporaryPtr<const UniverseObject> copied_object, Visibility vis,
-                                 const std::set<std::string>& visible_specials);///< used by public UniverseObject::Copy and derived classes' ::Copy methods
+    /** Used by public UniverseObject::Copy and derived classes' ::Copy methods.
+      */
+    void Copy(std::shared_ptr<const UniverseObject> copied_object, Visibility vis,
+              const std::set<std::string>& visible_specials);
 
     std::string             m_name;
 
@@ -198,7 +230,7 @@ private:
     double                                          m_y;
     int                                             m_owner_empire_id;
     int                                             m_system_id;
-    std::map<std::string, std::pair<int, float> >   m_specials; // map from special name to pair of (turn added, capacity)
+    std::map<std::string, std::pair<int, float>>    m_specials; // map from special name to pair of (turn added, capacity)
     std::map<MeterType, Meter>                      m_meters;
     int                                             m_created_on_turn;
 
