@@ -36,7 +36,7 @@ namespace ValueRef {
     };
     template <class FromType, class ToType> struct StaticCast;
     template <class FromType> struct StringCast;
-    struct UserStringLookup;
+    template <class FromType> struct UserStringLookup;
     struct NameLookup;
     template <class T> struct Operation;
     enum OpType {
@@ -56,7 +56,6 @@ namespace ValueRef {
         RANDOM_PICK,
         SUBSTITUTION
     };
-    template <class T> bool ConstantExpr(const ValueRefBase<T>* expr);
 }
 
 #endif // _ValueRefFwd_h_
