@@ -25,39 +25,50 @@ class FO_COMMON_API IApp {
 public:
     virtual ~IApp();
 
-    static IApp* GetApp(); ///< returns a IApp pointer to the singleton instance of the app
+    /** Returns a IApp pointer to the singleton instance of the app. */
+    static IApp* GetApp();
 
-    virtual Universe& GetUniverse() = 0;  ///< returns applications copy of Universe
+    /** Returns applications copy of Universe. */
+    virtual Universe& GetUniverse() = 0;
 
-    virtual EmpireManager& Empires() = 0; ///< returns the set of known Empires for this application
+    /** Returns the set of known Empires for this application. */
+    virtual EmpireManager& Empires() = 0;
 
     virtual Empire* GetEmpire(int id) = 0;
 
     virtual SupplyManager& GetSupplyManager() = 0;
 
-    virtual TemporaryPtr<UniverseObject> GetUniverseObject(int object_id) = 0;
+    virtual std::shared_ptr<UniverseObject> GetUniverseObject(int object_id) = 0;
 
     /** Accessor for known objects of specified empire. */
     virtual ObjectMap& EmpireKnownObjects(int empire_id) = 0;
 
-    virtual TemporaryPtr<UniverseObject> EmpireKnownObject(int object_id, int empire_id) = 0;
+    virtual std::shared_ptr<UniverseObject> EmpireKnownObject(int object_id, int empire_id) = 0;
 
-    virtual std::string GetVisibleObjectName(TemporaryPtr<const UniverseObject> object) = 0;
+    virtual std::string GetVisibleObjectName(std::shared_ptr<const UniverseObject> object) = 0;
 
     /** returns a universe object ID which can be used for new objects.
         Can return INVALID_OBJECT_ID if an ID cannot be created. */
     virtual int GetNewObjectID() = 0;
 
-    /** returns a design ID which can be used for a new design to uniquely identify it.
+    /** returns a design ID which can be used for a new design to uniquely
+        identify it.
         Can return INVALID_OBJECT_ID if an ID cannot be created. */
     virtual int GetNewDesignID() = 0;
 
-    virtual int CurrentTurn() const = 0;        ///< returns the current game turn
+    /** Returns the current game turn. */
+    virtual int CurrentTurn() const = 0;
 
+    static int MAX_AI_PLAYERS(); ///<Maximum number of AIs
+
+    /** Returns the galaxy setup data used for the current game */
     virtual const GalaxySetupData& GetGalaxySetupData() const = 0;
 
-    virtual Networking::ClientType GetEmpireClientType(int empire_id) const = 0;///< returns the networking client type for the given empire_id
-    virtual Networking::ClientType GetPlayerClientType(int player_id) const = 0;///< returns the networking client type for the given player_id
+    /** Returns the networking client type for the given empire_id. */
+    virtual Networking::ClientType GetEmpireClientType(int empire_id) const = 0;
+
+    /** Returns the networking client type for the given player_id. */
+    virtual Networking::ClientType GetPlayerClientType(int player_id) const = 0;
 
     virtual int EffectsProcessingThreads() const = 0;
 
@@ -66,6 +77,10 @@ protected:
 
     static IApp* s_app; ///< a IApp pointer to the singleton instance of the app
 
+    // NormalExitException is used to break out of the run loop, without calling
+    // terminate and failing to unroll the stack.
+    class NormalExitException {};
+
 private:
     const IApp& operator=(const IApp&); // disabled
     IApp(const IApp&); // disabled
@@ -87,6 +102,10 @@ inline SupplyManager& GetSupplyManager()
 inline Universe& GetUniverse()
 { return IApp::GetApp()->GetUniverse(); }
 
+/** Accessor for the App's universe object */
+inline std::shared_ptr<const Pathfinder> GetPathfinder()
+{ return IApp::GetApp()->GetUniverse().GetPathfinder(); }
+
 /** Accessor for all (on server) or all known (on client) objects ObjectMap */
 inline ObjectMap& Objects()
 { return IApp::GetApp()->GetUniverse().Objects(); }
@@ -96,63 +115,63 @@ inline ObjectMap& EmpireKnownObjects(int empire_id)
 { return IApp::GetApp()->EmpireKnownObjects(empire_id); }
 
 /** Accessor for individual objects. */
-inline TemporaryPtr<UniverseObject> GetUniverseObject(int object_id)
+inline std::shared_ptr<UniverseObject> GetUniverseObject(int object_id)
 { return IApp::GetApp()->GetUniverseObject(object_id); }
 
-inline TemporaryPtr<UniverseObject> GetEmpireKnownObject(int object_id, int empire_id)
+inline std::shared_ptr<UniverseObject> GetEmpireKnownObject(int object_id, int empire_id)
 { return IApp::GetApp()->EmpireKnownObject(object_id, empire_id); }
 
-inline TemporaryPtr<ResourceCenter> GetResourceCenter(int object_id)
+inline std::shared_ptr<ResourceCenter> GetResourceCenter(int object_id)
 { return IApp::GetApp()->GetUniverse().Objects().Object<ResourceCenter>(object_id); }
 
-inline TemporaryPtr<ResourceCenter> GetEmpireKnownResourceCenter(int object_id, int empire_id)
+inline std::shared_ptr<ResourceCenter> GetEmpireKnownResourceCenter(int object_id, int empire_id)
 { return IApp::GetApp()->EmpireKnownObjects(empire_id).Object<ResourceCenter>(object_id); }
 
-inline TemporaryPtr<PopCenter> GetPopCenter(int object_id)
+inline std::shared_ptr<PopCenter> GetPopCenter(int object_id)
 { return IApp::GetApp()->GetUniverse().Objects().Object<PopCenter>(object_id); }
 
-inline TemporaryPtr<PopCenter> GetEmpireKnownPopCenter(int object_id, int empire_id)
+inline std::shared_ptr<PopCenter> GetEmpireKnownPopCenter(int object_id, int empire_id)
 { return IApp::GetApp()->EmpireKnownObjects(empire_id).Object<PopCenter>(object_id); }
 
-inline TemporaryPtr<Planet> GetPlanet(int object_id)
+inline std::shared_ptr<Planet> GetPlanet(int object_id)
 { return IApp::GetApp()->GetUniverse().Objects().Object<Planet>(object_id); }
 
-inline TemporaryPtr<Planet> GetEmpireKnownPlanet(int object_id, int empire_id)
+inline std::shared_ptr<Planet> GetEmpireKnownPlanet(int object_id, int empire_id)
 { return IApp::GetApp()->EmpireKnownObjects(empire_id).Object<Planet>(object_id); }
 
-inline TemporaryPtr<System> GetSystem(int object_id)
+inline std::shared_ptr<System> GetSystem(int object_id)
 { return IApp::GetApp()->GetUniverse().Objects().Object<System>(object_id); }
 
-inline TemporaryPtr<System> GetEmpireKnownSystem(int object_id, int empire_id)
+inline std::shared_ptr<System> GetEmpireKnownSystem(int object_id, int empire_id)
 { return IApp::GetApp()->EmpireKnownObjects(empire_id).Object<System>(object_id); }
 
-inline TemporaryPtr<Field> GetField(int object_id)
+inline std::shared_ptr<Field> GetField(int object_id)
 { return IApp::GetApp()->GetUniverse().Objects().Object<Field>(object_id); }
 
-inline TemporaryPtr<Field> GetEmpireKnownField(int object_id, int empire_id)
+inline std::shared_ptr<Field> GetEmpireKnownField(int object_id, int empire_id)
 { return IApp::GetApp()->EmpireKnownObjects(empire_id).Object<Field>(object_id); }
 
-inline TemporaryPtr<Ship> GetShip(int object_id)
+inline std::shared_ptr<Ship> GetShip(int object_id)
 { return IApp::GetApp()->GetUniverse().Objects().Object<Ship>(object_id); }
 
-inline TemporaryPtr<Ship> GetEmpireKnownShip(int object_id, int empire_id)
+inline std::shared_ptr<Ship> GetEmpireKnownShip(int object_id, int empire_id)
 { return IApp::GetApp()->EmpireKnownObjects(empire_id).Object<Ship>(object_id); }
 
-inline TemporaryPtr<Fleet> GetFleet(int object_id)
+inline std::shared_ptr<Fleet> GetFleet(int object_id)
 { return IApp::GetApp()->GetUniverse().Objects().Object<Fleet>(object_id); }
 
-inline TemporaryPtr<Fleet> GetEmpireKnownFleet(int object_id, int empire_id)
+inline std::shared_ptr<Fleet> GetEmpireKnownFleet(int object_id, int empire_id)
 { return IApp::GetApp()->EmpireKnownObjects(empire_id).Object<Fleet>(object_id); }
 
-inline TemporaryPtr<Building> GetBuilding(int object_id)
+inline std::shared_ptr<Building> GetBuilding(int object_id)
 { return IApp::GetApp()->GetUniverse().Objects().Object<Building>(object_id); }
 
-inline TemporaryPtr<Building> GetEmpireKnownBuilding(int object_id, int empire_id)
+inline std::shared_ptr<Building> GetEmpireKnownBuilding(int object_id, int empire_id)
 { return IApp::GetApp()->EmpireKnownObjects(empire_id).Object<Building>(object_id); }
 
 /** Returns the object name of the universe object. This can be apperant object
  * name, if the application isn't supposed to see the real object name. */
-inline std::string GetVisibleObjectName(TemporaryPtr<const UniverseObject> object)
+inline std::string GetVisibleObjectName(std::shared_ptr<const UniverseObject> object)
 { return IApp::GetApp()->GetVisibleObjectName(object); }
 
 /** Returns a new object ID from the server */
