@@ -1,46 +1,31 @@
 #ifndef _MultiplayerCommon_h_
 #define _MultiplayerCommon_h_
 
-#include "../universe/Enums.h"
+#include "../universe/EnumsFwd.h"
 #include "../network/Networking.h"
 #include "Export.h"
+#include "Serialize.h"
 
 #include <GG/Clr.h>
 
 #include <list>
 #include <set>
 #include <vector>
+#include <map>
 #include <boost/serialization/access.hpp>
 
 
-
-class XMLElement;
-
 FO_COMMON_API extern const std::string MP_SAVE_FILE_EXTENSION;
 FO_COMMON_API extern const std::string SP_SAVE_FILE_EXTENSION;
 
-/** Returns an XML representation of a GG::Clr object. */
-XMLElement ClrToXML(const GG::Clr& clr);
-
-/** Returns a GG::Clr object constructed from its XML representation. */
-FO_COMMON_API GG::Clr XMLToClr(const XMLElement& clr);
+FO_COMMON_API extern const int ALL_EMPIRES;
 
 /** The data that represent the galaxy setup for a new game. */
 struct FO_COMMON_API GalaxySetupData {
     /** \name Structors */ //@{
-    GalaxySetupData() :
-        m_seed(),
-        m_size(100),
-        m_shape(SPIRAL_2),
-        m_age(GALAXY_SETUP_MEDIUM),
-        m_starlane_freq(GALAXY_SETUP_MEDIUM),
-        m_planet_density(GALAXY_SETUP_MEDIUM),
-        m_specials_freq(GALAXY_SETUP_MEDIUM),
-        m_monster_freq(GALAXY_SETUP_MEDIUM),
-        m_native_freq(GALAXY_SETUP_MEDIUM),
-        m_ai_aggr(MANIACAL)
-    {}
+    GalaxySetupData();
     //@}
+
     /** \name Accessors */ //@{
     const std::string&  GetSeed() const;
     int                 GetSize() const;
@@ -71,6 +56,11 @@ private:
     void serialize(Archive& ar, const unsigned int version);
 };
 
+extern template FO_COMMON_API void GalaxySetupData::serialize<freeorion_bin_oarchive>(freeorion_bin_oarchive&, const unsigned int);
+extern template FO_COMMON_API void GalaxySetupData::serialize<freeorion_bin_iarchive>(freeorion_bin_iarchive&, const unsigned int);
+extern template FO_COMMON_API void GalaxySetupData::serialize<freeorion_xml_oarchive>(freeorion_xml_oarchive&, const unsigned int);
+extern template FO_COMMON_API void GalaxySetupData::serialize<freeorion_xml_iarchive>(freeorion_xml_iarchive&, const unsigned int);
+
 /** Contains the UI data that must be saved in save game files in order to
   * restore games to the users' last views. */
 struct FO_COMMON_API SaveGameUIData {
@@ -85,6 +75,11 @@ private:
     void serialize(Archive& ar, const unsigned int version);
 };
 
+extern template FO_COMMON_API void SaveGameUIData::serialize<freeorion_bin_oarchive>(freeorion_bin_oarchive&, const unsigned int);
+extern template FO_COMMON_API void SaveGameUIData::serialize<freeorion_bin_iarchive>(freeorion_bin_iarchive&, const unsigned int);
+extern template FO_COMMON_API void SaveGameUIData::serialize<freeorion_xml_oarchive>(freeorion_xml_oarchive&, const unsigned int);
+extern template FO_COMMON_API void SaveGameUIData::serialize<freeorion_xml_iarchive>(freeorion_xml_iarchive&, const unsigned int);
+
 /** The data for one empire necessary for game-setup during multiplayer loading. */
 struct FO_COMMON_API SaveGameEmpireData {
     /** \name Structors */ //@{
@@ -114,21 +109,29 @@ private:
     void serialize(Archive& ar, const unsigned int version);
 };
 
+extern template FO_COMMON_API void SaveGameEmpireData::serialize<freeorion_bin_oarchive>(freeorion_bin_oarchive&, const unsigned int);
+extern template FO_COMMON_API void SaveGameEmpireData::serialize<freeorion_bin_iarchive>(freeorion_bin_iarchive&, const unsigned int);
+extern template FO_COMMON_API void SaveGameEmpireData::serialize<freeorion_xml_oarchive>(freeorion_xml_oarchive&, const unsigned int);
+extern template FO_COMMON_API void SaveGameEmpireData::serialize<freeorion_xml_iarchive>(freeorion_xml_iarchive&, const unsigned int);
+
 /** The data structure used to represent a single player's setup options for a
   * multiplayer game (in the multiplayer lobby screen). */
 struct PlayerSetupData {
     /** \name Structors */ //@{
     PlayerSetupData() :
         m_player_name(),
+        m_player_id(Networking::INVALID_PLAYER_ID),
         m_empire_name(),
         m_empire_color(GG::Clr(0, 0, 0, 0)),
         m_starting_species_name(),
         m_save_game_empire_id(ALL_EMPIRES),
-        m_client_type(Networking::INVALID_CLIENT_TYPE)
+        m_client_type(Networking::INVALID_CLIENT_TYPE),
+        m_player_ready(false)
     {}
     //@}
 
     std::string             m_player_name;          ///< the player's name
+    int                     m_player_id;            ///< player id
 
     std::string             m_empire_name;          ///< the name of the player's empire when starting a new game
     GG::Clr                 m_empire_color;         ///< the color used to represent this player's empire when starting a new game
@@ -137,6 +140,7 @@ struct PlayerSetupData {
     int                     m_save_game_empire_id;  ///< when loading a game, the ID of the empire that this player will control
 
     Networking::ClientType  m_client_type;          ///< is this player an AI, human player or...?
+    bool                    m_player_ready;         ///< if player ready to play.
 
 private:
     friend class boost::serialization::access;
@@ -184,7 +188,9 @@ struct FO_COMMON_API MultiplayerLobbyData : public GalaxySetupData {
     std::string Dump() const;
 
     bool                                        m_new_game;
-    std::list<std::pair<int, PlayerSetupData> > m_players;              // <player_id, PlayerSetupData>
+    // TODO: Change from a list<(player_id, PlayerSetupData)> where
+    // PlayerSetupData contain player_id to a vector of PlayerSetupData
+    std::list<std::pair<int, PlayerSetupData>>  m_players;              // <player_id, PlayerSetupData>
 
     std::string                                 m_save_game;            //< File name of a save file
     std::map<int, SaveGameEmpireData>           m_save_game_empire_data;// indexed by empire_id
