@@ -3,15 +3,18 @@
 
 #include <boost/any.hpp>
 #include <boost/lexical_cast.hpp>
-#include <boost/scoped_ptr.hpp>
+
 #include <cmath>
-#include <string>
+#include <memory>
 #include <set>
+#include <string>
+#include <vector>
+
 
 // these are needed by the StepValidator
 namespace details {
     template <class T> inline T mod (T dividend, T divisor)
-    {  return (dividend % divisor); }
+    { return (dividend % divisor); }
 
     template <> inline float mod<float>(float dividend, float divisor)
     { return std::fmod(dividend, divisor); }
@@ -24,8 +27,11 @@ namespace details {
 }
 
 /** base class for all OptionsDB validators. Simply provides the basic interface. */
-struct ValidatorBase {
-    virtual ~ValidatorBase() {}
+struct ValidatorBase
+{
+    virtual ~ValidatorBase()
+    {}
+
     /** returns normally if \a str is a valid value, or throws otherwise */
     virtual boost::any Validate(const std::string& str) const = 0;
 
@@ -40,30 +46,46 @@ struct ValidatorBase {
 template <class T>
 struct Validator : public ValidatorBase
 {
-    virtual boost::any Validate(const std::string& str) const
+    boost::any Validate(const std::string& str) const override
     { return boost::any(boost::lexical_cast<T>(str)); }
 
-    virtual std::string String(const boost::any& value) const
+    std::string String(const boost::any& value) const override
     { return boost::lexical_cast<std::string>(boost::any_cast<T>(value)); }
 
-    virtual Validator *Clone() const
+    Validator *Clone() const override
     { return new Validator<T>(); }
 };
 
+FO_COMMON_API std::string ListToString(const std::vector<std::string>& input_list);
+FO_COMMON_API std::vector<std::string> StringToList(const std::string& input_string);
+
+template <>
+struct Validator<std::vector<std::string>> : public ValidatorBase
+{
+    boost::any Validate(const std::string& str) const override
+    { return boost::any(StringToList(str)); }
+
+    std::string String(const boost::any& value) const override
+    { return ListToString(boost::any_cast<std::vector<std::string>>(value)); }
+
+    Validator *Clone() const override
+    { return new Validator<std::vector<std::string>>(); }
+};
+
 /** a Validator that constrains the range of valid values */
 template <class T>
 struct RangedValidator : public Validator<T>
 {
     RangedValidator(const T& min, const T& max) : m_min(min), m_max(max) {}
 
-    virtual boost::any Validate(const std::string& str) const {
+    boost::any Validate(const std::string& str) const override {
         T val = boost::lexical_cast<T>(str);
         if (val < m_min || val > m_max)
             throw boost::bad_lexical_cast();
         return boost::any(val);
     }
 
-    virtual RangedValidator *Clone() const
+    RangedValidator *Clone() const override
     { return new RangedValidator<T>(m_min, m_max); }
 
     const T m_min;
@@ -77,14 +99,14 @@ struct StepValidator : public Validator<T>
 {
     StepValidator(const T& step, const T& origin = T()) : m_step_size(step), m_origin(origin) {}
 
-    virtual boost::any Validate(const std::string& str) const {
+    boost::any Validate(const std::string& str) const override {
         T val = boost::lexical_cast<T>(str);
         if (std::abs(details::mod((val - m_origin), m_step_size)) > std::numeric_limits<T>::epsilon())
             throw boost::bad_lexical_cast();
         return boost::any(val);
     }
 
-    virtual StepValidator *Clone() const
+    StepValidator *Clone() const override
     { return new StepValidator<T>(m_step_size, m_origin); }
 
     const T m_step_size;
@@ -99,16 +121,16 @@ public:
     RangedStepValidator(const T& step, const T& min, const T& max) : m_step_size(step), m_origin(T()), m_min(min), m_max(max) {}
     RangedStepValidator(const T& step, const T& origin, const T& min, const T& max) : m_step_size (step), m_origin (origin), m_min (min), m_max (max) {}
 
-    virtual boost::any Validate(const std::string& str) const {
+    boost::any Validate(const std::string& str) const override {
         T val = boost::lexical_cast<T>(str);
-        if (val < m_min || val > m_max ||
-            std::abs(details::mod<T>(val - m_origin, m_step_size)) > std::numeric_limits<T>::epsilon() &&
-            std::abs(m_step_size - details::mod<T>(val - m_origin, m_step_size)) > std::numeric_limits<T>::epsilon())
+        if ((val < m_min) || (val > m_max) ||
+            ((std::abs(details::mod<T>(val - m_origin, m_step_size)) > std::numeric_limits<T>::epsilon()) &&
+             (std::abs(m_step_size - details::mod<T>(val - m_origin, m_step_size)) > std::numeric_limits<T>::epsilon())))
             throw boost::bad_lexical_cast();
         return boost::any(val);
     }
 
-    virtual RangedStepValidator *Clone() const
+    RangedStepValidator *Clone() const override
     { return new RangedStepValidator<T>(m_step_size, m_origin, m_min, m_max); }
 
     const T m_step_size;
@@ -140,7 +162,7 @@ struct DiscreteValidator : public Validator<T>
         m_values(in, in + N)
     { }
 
-    virtual boost::any Validate(const std::string& str) const {
+    boost::any Validate(const std::string& str) const override {
         T val = boost::lexical_cast<T>(str);
 
         if (m_values.find(val) == m_values.end())
@@ -149,7 +171,7 @@ struct DiscreteValidator : public Validator<T>
         return boost::any(val);
     }
 
-    virtual DiscreteValidator* Clone() const
+    DiscreteValidator* Clone() const override
     { return new DiscreteValidator<T>(m_values); }
 
     /// Stores the list of vaild values.
@@ -157,7 +179,7 @@ struct DiscreteValidator : public Validator<T>
 };
 
 /// a Validator that performs a logical OR of two validators.
-/** Stores and owns clones of the provided validators in boost::scoped_ptrs.
+/** Stores and owns clones of the provided validators in std::unique_ptr.
  *  Always calls m_validator_a->Validate(). Only calls m_validator_b->Validate()
  *  if the first one throws. */
 template <class T>
@@ -169,7 +191,7 @@ struct OrValidator : public Validator<T>
         m_validator_b(validator_b.Clone())
     { }
 
-    virtual boost::any Validate(const std::string& str) const {
+    boost::any Validate(const std::string& str) const override {
         boost::any result;
 
         try {
@@ -181,11 +203,11 @@ struct OrValidator : public Validator<T>
         return result;
     }
 
-    virtual OrValidator* Clone() const
+    OrValidator* Clone() const override
     { return new OrValidator<T>(*m_validator_a.get(), *m_validator_b.get()); }
 
-    const boost::scoped_ptr<Validator<T> > m_validator_a;
-    const boost::scoped_ptr<Validator<T> > m_validator_b;
+    const std::unique_ptr<Validator<T>> m_validator_a;
+    const std::unique_ptr<Validator<T>> m_validator_b;
 };
 
 #endif // _OptionValidators_h_
