@@ -3,6 +3,7 @@
 #include "i18n.h"
 #include "Logger.h"
 #include "OptionValidators.h"
+#include "XMLDoc.h"
 
 #include "util/Directories.h"
 
@@ -10,10 +11,12 @@
 #include <stdexcept>
 #include <string>
 
-#include <boost/spirit/include/classic.hpp>
+#include <boost/algorithm/string/classification.hpp>
 #include <boost/algorithm/string/erase.hpp>
 #include <boost/algorithm/string/predicate.hpp>
 #include <boost/filesystem/fstream.hpp>
+#include <boost/range/algorithm_ext/erase.hpp>
+#include <boost/tokenizer.hpp>
 
 namespace {
     std::vector<OptionsDBFn>& OptionsRegistry() {
@@ -31,12 +34,6 @@ namespace {
         return retval;
     }
 
-    struct PushBack {
-        PushBack(std::vector<std::string>& string_vec) : m_string_vec(string_vec) {}
-        void operator()(const char* first, const char* last) const {m_string_vec.push_back(std::string(first, last));}
-        std::vector<std::string>& m_string_vec;
-    };
-
     void StripQuotation(std::string& str) {
         using namespace boost::algorithm;
         if (starts_with(str, "\"") && ends_with(str, "\"")) {
@@ -56,9 +53,9 @@ bool RegisterOptions(OptionsDBFn function) {
 
 OptionsDB& GetOptionsDB() {
     static OptionsDB options_db;
-    if (unsigned int registry_size = OptionsRegistry().size()) {
-        for (unsigned int i = 0; i < registry_size; ++i)
-            OptionsRegistry()[i](options_db);
+    if (!OptionsRegistry().empty()) {
+        for (OptionsDBFn fn : OptionsRegistry())
+            fn(options_db);
         OptionsRegistry().clear();
     }
     return options_db;
@@ -131,7 +128,7 @@ std::string OptionsDB::Option::DefaultValueToString() const {
 // OptionsDB
 /////////////////////////////////////////////
 // static(s)
-OptionsDB* OptionsDB::s_options_db = 0;
+OptionsDB* OptionsDB::s_options_db = nullptr;
 
 OptionsDB::OptionsDB() {
     if (s_options_db)
@@ -146,7 +143,9 @@ void OptionsDB::Commit()
         return;
     boost::filesystem::ofstream ofs(GetConfigPath());
     if (ofs) {
-        GetOptionsDB().GetXML().WriteDoc(ofs);
+        XMLDoc doc;
+        GetOptionsDB().GetXML(doc);
+        doc.WriteDoc(ofs);
         m_dirty = false;
     } else {
         std::cerr << UserString("UNABLE_TO_WRITE_CONFIG_XML") << std::endl;
@@ -188,7 +187,7 @@ const std::string& OptionsDB::GetDescription(const std::string& option_name) con
     return it->second.description;
 }
 
-boost::shared_ptr<const ValidatorBase> OptionsDB::GetValidator(const std::string& option_name) const {
+std::shared_ptr<const ValidatorBase> OptionsDB::GetValidator(const std::string& option_name) const {
     std::map<std::string, Option>::const_iterator it = m_options.find(option_name);
     if (!OptionExists(it))
         throw std::runtime_error(("OptionsDB::GetValidator(): No option called \"" + option_name + "\" could be found.").c_str());
@@ -199,9 +198,9 @@ void OptionsDB::GetUsage(std::ostream& os, const std::string& command_line/* = "
     os << UserString("COMMAND_LINE_USAGE") << command_line << "\n";
 
     int longest_param_name = 0;
-    for (std::map<std::string, Option>::const_iterator it = m_options.begin(); it != m_options.end(); ++it) {
-        if (longest_param_name < static_cast<int>(it->first.size()))
-            longest_param_name = it->first.size();
+    for (const std::map<std::string, Option>::value_type& option : m_options) {
+        if (longest_param_name < static_cast<int>(option.first.size()))
+            longest_param_name = option.first.size();
     }
 
     int description_column = 5;
@@ -210,38 +209,36 @@ void OptionsDB::GetUsage(std::ostream& os, const std::string& command_line/* = "
     if (description_width <= 0)
         throw std::runtime_error("The longest parameter name leaves no room for a description.");
 
-    for (std::map<std::string, Option>::const_iterator it = m_options.begin(); it != m_options.end(); ++it) {
+    for (const std::map<std::string, Option>::value_type& option : m_options) {
         // Ignore unrecognized options that have not been formally registered
         // with Add().
-        if (!it->second.recognized)
+        if (!option.second.recognized)
             continue;
 
-        if (it->second.short_name)
-            os << "-" << it->second.short_name << ", --" << it->second.name << "\n";
+        if (option.second.short_name)
+            os << "-" << option.second.short_name << ", --" << option.second.name << "\n";
         else
-            os << "--" << it->second.name << "\n";
+            os << "--" << option.second.name << "\n";
 
         os << std::string(description_column - 1, ' ');
 
-        std::vector<std::string> tokenized_strings;
-        using boost::spirit::classic::anychar_p;
-        using boost::spirit::classic::rule;
-        using boost::spirit::classic::space_p;
-        rule<> tokenizer = +(*space_p >> (+(anychar_p - space_p))[PushBack(tokenized_strings)]);
-        parse(UserString(it->second.description).c_str(), tokenizer);
+        typedef boost::tokenizer<boost::char_separator<char>> Tokenizer;
+        boost::char_separator<char> separator(" \t");
+        Tokenizer tokens(UserString(option.second.description), separator);
         int curr_column = description_column;
-        for (unsigned int i = 0; i < tokenized_strings.size(); ++i) {
-            if (80 < curr_column + tokenized_strings[i].size() + (i ? 1 : 0)) {
-                os << "\n" << std::string(description_column, ' ') << tokenized_strings[i];
-                curr_column = description_column + tokenized_strings[i].size();
+        for (const Tokenizer::value_type& token : tokens) {
+            if (80 < curr_column + token.size()) {
+                os << "\n" << std::string(description_column, ' ') << token;
+                curr_column = description_column + token.size();
             } else {
-                os << " " << tokenized_strings[i];
-                curr_column += tokenized_strings[i].size() + 1;
+                os << " " << token;
+                curr_column += token.size() + 1;
             }
         }
-        if (it->second.validator) {
+
+        if (option.second.validator) {
             std::stringstream stream;
-            stream << UserString("COMMAND_LINE_DEFAULT") << it->second.DefaultValueToString();
+            stream << UserString("COMMAND_LINE_DEFAULT") << option.second.DefaultValueToString();
             if (80 < curr_column + stream.str().size() + 3) {
                 os << "\n" << std::string(description_column, ' ') << stream.str() << "\n";
             } else {
@@ -254,18 +251,18 @@ void OptionsDB::GetUsage(std::ostream& os, const std::string& command_line/* = "
     }
 }
 
-XMLDoc OptionsDB::GetXML() const {
-    XMLDoc doc;
+void OptionsDB::GetXML(XMLDoc& doc) const {
+    doc = XMLDoc();
 
     std::vector<XMLElement*> elem_stack;
     elem_stack.push_back(&doc.root_node);
 
-    for (std::map<std::string, Option>::const_iterator it = m_options.begin(); it != m_options.end(); ++it) {
-        if (!it->second.storable)
+    for (const std::map<std::string, Option>::value_type& option : m_options) {
+        if (!option.second.storable)
             continue;
-        std::string::size_type last_dot = it->first.find_last_of('.');
-        std::string section_name = last_dot == std::string::npos ? "" : it->first.substr(0, last_dot);
-        std::string name = it->first.substr(last_dot == std::string::npos ? 0 : last_dot + 1);
+        std::string::size_type last_dot = option.first.find_last_of('.');
+        std::string section_name = last_dot == std::string::npos ? "" : option.first.substr(0, last_dot);
+        std::string name = option.first.substr(last_dot == std::string::npos ? 0 : last_dot + 1);
         while (1 < elem_stack.size()) {
             std::string prev_section = PreviousSectionName(elem_stack);
             if (prev_section == section_name) {
@@ -282,27 +279,25 @@ XMLDoc OptionsDB::GetXML() const {
             std::string::size_type pos = 0;
             while ((pos = section_name.find('.', last_pos)) != std::string::npos) {
                 XMLElement temp(section_name.substr(last_pos, pos - last_pos));
-                elem_stack.back()->AppendChild(temp);
+                elem_stack.back()->children.push_back(temp);
                 elem_stack.push_back(&elem_stack.back()->Child(temp.Tag()));
                 last_pos = pos + 1;
             }
             XMLElement temp(section_name.substr(last_pos));
-            elem_stack.back()->AppendChild(temp);
+            elem_stack.back()->children.push_back(temp);
             elem_stack.push_back(&elem_stack.back()->Child(temp.Tag()));
         }
 
         XMLElement temp(name);
-        if (it->second.validator) {
-            temp.SetText(it->second.ValueToString());
-        } else if (it->second.flag) {
-            if (!boost::any_cast<bool>(it->second.value))
+        if (option.second.validator) {
+            temp.SetText(option.second.ValueToString());
+        } else if (option.second.flag) {
+            if (!boost::any_cast<bool>(option.second.value))
                 continue;
         }
-        elem_stack.back()->AppendChild(temp);
+        elem_stack.back()->children.push_back(temp);
         elem_stack.push_back(&elem_stack.back()->Child(temp.Tag()));
     }
-
-    return doc;
 }
 
 OptionsDB::OptionChangedSignalType& OptionsDB::OptionChangedSignal(const std::string& option) {
@@ -334,9 +329,9 @@ void OptionsDB::RemoveUnrecognized(const std::string& prefix) {
 
 void OptionsDB::FindOptions(std::set<std::string>& ret, const std::string& prefix) const {
     ret.clear();
-    for (std::map<std::string, Option>::const_iterator it = m_options.begin(); it != m_options.end(); ++it)
-        if (it->second.recognized && it->first.find(prefix) == 0)
-            ret.insert(it->first);
+    for (const std::map<std::string, Option>::value_type& option : m_options)
+        if (option.second.recognized && option.first.find(prefix) == 0)
+            ret.insert(option.first);
 }
 
 void OptionsDB::SetFromCommandLine(const std::vector<std::string>& args) {
@@ -437,16 +432,16 @@ void OptionsDB::SetFromCommandLine(const std::vector<std::string>& args) {
 }
 
 void OptionsDB::SetFromXML(const XMLDoc& doc) {
-    for (int i = 0; i < doc.root_node.NumChildren(); ++i)
-    { SetFromXMLRecursive(doc.root_node.Child(i), ""); }
+    for (const XMLElement& child : doc.root_node.children)
+    { SetFromXMLRecursive(child, ""); }
 }
 
 void OptionsDB::SetFromXMLRecursive(const XMLElement& elem, const std::string& section_name) {
     std::string option_name = section_name + (section_name == "" ? "" : ".") + elem.Tag();
 
-    if (elem.NumChildren()) {
-        for (int i = 0; i < elem.NumChildren(); ++i)
-            SetFromXMLRecursive(elem.Child(i), option_name);
+    if (!elem.children.empty()) {
+        for (const XMLElement& child : elem.children)
+            SetFromXMLRecursive(child, option_name);
 
     } else {
         std::map<std::string, Option>::iterator it = m_options.find(option_name);
@@ -483,3 +478,28 @@ void OptionsDB::SetFromXMLRecursive(const XMLElement& elem, const std::string& s
         }
     }
 }
+
+std::string ListToString(const std::vector<std::string>& input_list) {
+    // list input strings in comma-separated-value format
+    std::string retval;
+    for (std::vector<std::string>::const_iterator it = input_list.begin(); it != input_list.end(); ++it) {
+        if (it != input_list.begin())
+            retval += ",";
+        std::string str(*it);
+        boost::remove_erase_if(str, boost::is_any_of("<&>'\",[]|\a\b\f\n\r\t\b"));  // remove XML protected characters and a few other semi-randomly chosen characters to avoid corrupting enclosing XML document structure
+        retval += str;
+    }
+    return retval;
+}
+
+std::vector<std::string> StringToList(const std::string& input_string) {
+    std::vector<std::string> retval;
+    typedef boost::tokenizer<boost::char_separator<char>> Tokenizer;
+    boost::char_separator<char> separator(",");
+    Tokenizer tokens(input_string, separator);
+    for (const Tokenizer::value_type& token : tokens) {
+        retval.push_back(token);
+    }
+    return retval;
+}
+
