@@ -4,7 +4,6 @@
 #include "Export.h"
 #include "Logger.h"
 #include "OptionValidators.h"
-#include "XMLDoc.h"
 
 #include <boost/any.hpp>
 #include <boost/signals2/signal.hpp>
@@ -13,6 +12,8 @@
 
 
 class OptionsDB;
+class XMLDoc;
+class XMLElement;
 
 /////////////////////////////////////////////
 // Free Functions
@@ -146,7 +147,7 @@ public:
         return boost::any_cast<T>(it->second.default_value);
     }
 
-    bool IsDefaultValue(const std::string& name) const {
+    bool        IsDefaultValue(const std::string& name) const {
         std::map<std::string, Option>::const_iterator it = m_options.find(name);
         if (!OptionExists(it))
             throw std::runtime_error("OptionsDB::IsDefaultValue<>() : Attempted to get nonexistent option \"" + name + "\".");
@@ -166,13 +167,17 @@ public:
 
     /** returns the validator for option \a option_name, or throws
       * std::runtime_error if no such Option exists. */
-    boost::shared_ptr<const ValidatorBase>  GetValidator(const std::string& option_name) const;
+    std::shared_ptr<const ValidatorBase> GetValidator(const std::string& option_name) const;
 
     /** writes a usage message to \a os */
     void        GetUsage(std::ostream& os, const std::string& command_line = "") const;
 
-    /** returns the contents of the DB as an XMLDoc. */
-    XMLDoc      GetXML() const;
+    /** @brief  Saves the contents of the options DB to the @p doc XMLDoc.
+     *
+     * @param[in,out] doc  The document this OptionsDB should be written to.
+     *      This resets the given @p doc.
+     */
+    void GetXML(XMLDoc& doc) const;
 
     /** find all registered Options that begin with \a prefix and store them in
       * \a ret. */
@@ -256,7 +261,7 @@ public:
             value = true; // if the flag is present at all its value is true
         }
         m_options[name] = Option(static_cast<char>(0), name, value, boost::lexical_cast<std::string>(false),
-                                 description, 0, storable, true, true);
+                                 description, nullptr, storable, true, true);
         m_dirty = true;
         OptionAddedSignal(name);
     }
@@ -278,7 +283,7 @@ public:
             value = true; // if the flag is present at all its value is true
         }
         m_options[name] = Option(short_name, name, value, boost::lexical_cast<std::string>(false),
-                                 description, 0, storable, true, true);
+                                 description, nullptr, storable, true, true);
         m_dirty = true;
         OptionAddedSignal(name);
     }
@@ -309,7 +314,7 @@ public:
     void        SetFromXML(const XMLDoc& doc);
 
 private:
-    struct Option {
+    struct FO_COMMON_API Option {
         Option();
         Option(char short_name_, const std::string& name_, const boost::any& value_,
                const boost::any& default_value_, const std::string& description_,
@@ -328,13 +333,16 @@ private:
         boost::any      value;          ///< the value of the option
         boost::any      default_value;  ///< the default value of the option
         std::string     description;    ///< a desription of the option
-        boost::shared_ptr<const ValidatorBase>
-                        validator;      ///< a validator for the option. Flags have no validators; lexical_cast boolean conversions are done for them.
+
+        /** A validator for the option.  Flags have no validators; lexical_cast
+            boolean conversions are done for them. */
+        std::shared_ptr<const ValidatorBase> validator;
+
         bool            storable;       ///< whether this option can be stored in an XML config file for use across multiple runs
         bool            flag;
         bool            recognized;     ///< whether this option has been registered before being specified via an XML input, unrecognized options can't be parsed (don't know their type) but are stored in case they are later registered with Add()
 
-        mutable boost::shared_ptr<boost::signals2::signal<void ()> > option_changed_sig_ptr;
+        mutable std::shared_ptr<boost::signals2::signal<void ()>> option_changed_sig_ptr;
 
         static std::map<char, std::string> short_names;   ///< the master list of abbreviated option names, and their corresponding long-form names
     };
@@ -358,7 +366,7 @@ private:
     static OptionsDB*               s_options_db;
     bool                            m_dirty; //< has OptionsDB changed since last Commit()
 
-    friend OptionsDB& GetOptionsDB();
+    friend FO_COMMON_API OptionsDB& GetOptionsDB();
 };
 
 template <typename T>
