@@ -10,13 +10,13 @@
 #include "../universe/Ship.h"
 #include "../universe/ShipDesign.h"
 #include "../universe/System.h"
+#include "../universe/Pathfinder.h"
 #include "../universe/Universe.h"
 #include "../universe/UniverseObject.h"
+#include "../universe/Enums.h"
 #include "../Empire/EmpireManager.h"
 #include "../Empire/Empire.h"
 
-#include <boost/lexical_cast.hpp>
-
 #include <fstream>
 #include <vector>
 
@@ -61,7 +61,7 @@ RenameOrder::RenameOrder(int empire, int object, const std::string& name) :
     m_object(object),
     m_name(name)
 {
-    TemporaryPtr<const UniverseObject> obj = GetUniverseObject(object);
+    std::shared_ptr<const UniverseObject> obj = GetUniverseObject(object);
     if (!obj) {
         ErrorLogger() << "RenameOrder::RenameOrder() : Attempted to rename nonexistant object with id " << object;
         return;
@@ -78,7 +78,7 @@ RenameOrder::RenameOrder(int empire, int object, const std::string& name) :
 void RenameOrder::ExecuteImpl() const {
     ValidateEmpireID();
 
-    TemporaryPtr<UniverseObject> obj = GetUniverseObject(m_object);
+    std::shared_ptr<UniverseObject> obj = GetUniverseObject(m_object);
 
     if (!obj) {
         ErrorLogger() << "Attempted to rename nonexistant object with id " << m_object;
@@ -87,7 +87,9 @@ void RenameOrder::ExecuteImpl() const {
 
     // verify that empire specified in order owns specified object
     if (!obj->OwnedBy(EmpireID())) {
-        ErrorLogger() << "Empire specified in rename order does not own specified object.";
+        ErrorLogger() << "Empire (" << EmpireID()
+                      << ") specified in rename order does not own specified object which is owned by "
+                      << obj->Owner() << ".";
         return;
     }
 
@@ -128,7 +130,7 @@ NewFleetOrder::NewFleetOrder(int empire, const std::string& fleet_name, int flee
 
 NewFleetOrder::NewFleetOrder(int empire, const std::vector<std::string>& fleet_names,
                              const std::vector<int>& fleet_ids, int system_id,
-                             const std::vector<std::vector<int> >& ship_id_groups,
+                             const std::vector<std::vector<int>>& ship_id_groups,
                              const std::vector<bool>& aggressives) :
     Order(empire),
     m_fleet_names(fleet_names),
@@ -145,7 +147,7 @@ void NewFleetOrder::ExecuteImpl() const {
         ErrorLogger() << "Empire attempted to create a new fleet outside a system";
         return;
     }
-    TemporaryPtr<System> system = GetSystem(m_system_id);
+    std::shared_ptr<System> system = GetSystem(m_system_id);
     if (!system) {
         ErrorLogger() << "Empire attempted to create a new fleet in a nonexistant system";
         return;
@@ -162,7 +164,7 @@ void NewFleetOrder::ExecuteImpl() const {
     }
 
     GetUniverse().InhibitUniverseObjectSignals(true);
-    std::vector<TemporaryPtr<Fleet> > created_fleets;
+    std::vector<std::shared_ptr<Fleet>> created_fleets;
     created_fleets.reserve(m_ship_id_groups.size());
 
 
@@ -177,11 +179,11 @@ void NewFleetOrder::ExecuteImpl() const {
             continue;   // nothing to do...
 
         // validate specified ships
-        std::vector<TemporaryPtr<Ship> >    validated_ships;
+        std::vector<std::shared_ptr<Ship>> validated_ships;
         std::vector<int>                    validated_ships_ids;
-        for (unsigned int i = 0; i < ship_ids.size(); ++i) {
+        for (int ship_id : ship_ids) {
             // verify that empire is not trying to take ships from somebody else's fleet
-            TemporaryPtr<Ship> ship = GetShip(ship_ids[i]);
+            std::shared_ptr<Ship> ship = GetShip(ship_id);
             if (!ship) {
                 ErrorLogger() << "Empire attempted to create a new fleet with an invalid ship";
                 continue;
@@ -201,8 +203,8 @@ void NewFleetOrder::ExecuteImpl() const {
             continue;
 
         // create fleet
-        TemporaryPtr<Fleet> fleet = GetUniverse().CreateFleet(fleet_name, system->X(), system->Y(),
-                                                              EmpireID(), fleet_id);
+        std::shared_ptr<Fleet> fleet = GetUniverse().CreateFleet(fleet_name, system->X(), system->Y(),
+                                                                 EmpireID(), fleet_id);
         fleet->GetMeter(METER_STEALTH)->SetCurrent(Meter::LARGE_VALUE);
         fleet->SetAggressive(aggressive);
 
@@ -212,17 +214,14 @@ void NewFleetOrder::ExecuteImpl() const {
         system->Insert(fleet);
 
         // new fleet will get same m_arrival_starlane as fleet of the first ship in the list.
-        TemporaryPtr<Ship> firstShip = validated_ships[0];
-        TemporaryPtr<Fleet> firstFleet = GetFleet(firstShip->FleetID());
+        std::shared_ptr<Ship> firstShip = validated_ships[0];
+        std::shared_ptr<Fleet> firstFleet = GetFleet(firstShip->FleetID());
         if (firstFleet)
             fleet->SetArrivalStarlane(firstFleet->ArrivalStarlane());
 
         // remove ships from old fleet(s) and add to new
-        for (std::vector<TemporaryPtr<Ship> >::iterator ship_it = validated_ships.begin();
-             ship_it != validated_ships.end(); ++ship_it)
-        {
-            TemporaryPtr<Ship> ship = *ship_it;
-            if (TemporaryPtr<Fleet> old_fleet = GetFleet(ship->FleetID()))
+        for (std::shared_ptr<Ship> ship : validated_ships) {
+            if (std::shared_ptr<Fleet> old_fleet = GetFleet(ship->FleetID()))
                 old_fleet->RemoveShip(ship->ID());
             ship->SetFleetID(fleet->ID());
         }
@@ -259,13 +258,13 @@ FleetMoveOrder::FleetMoveOrder(int empire, int fleet_id, int start_system_id, in
     m_append(append)
 {
     // perform sanity checks
-    TemporaryPtr<const Fleet> fleet = GetFleet(FleetID());
+    std::shared_ptr<const Fleet> fleet = GetFleet(FleetID());
     if (!fleet) {
         ErrorLogger() << "Empire with id " << EmpireID() << " ordered fleet with id " << FleetID() << " to move, but no such fleet exists";
         return;
     }
 
-    TemporaryPtr<const System> destination_system = GetSystem(DestinationSystemID());
+    std::shared_ptr<const System> destination_system = GetSystem(DestinationSystemID());
     if (!destination_system) {
         ErrorLogger() << "Empire with id " << EmpireID() << " ordered fleet to move to system with id " << DestinationSystemID() << " but no such system exists / is known to exist";
         return;
@@ -277,7 +276,7 @@ FleetMoveOrder::FleetMoveOrder(int empire, int fleet_id, int start_system_id, in
         return;
     }
 
-    std::pair<std::list<int>, double> short_path = GetUniverse().ShortestPath(m_start_system, m_dest_system, empire);
+    std::pair<std::list<int>, double> short_path = GetPathfinder()->ShortestPath(m_start_system, m_dest_system, empire);
 
     m_route.clear();
     std::copy(short_path.first.begin(), short_path.first.end(), std::back_inserter(m_route));
@@ -290,13 +289,13 @@ FleetMoveOrder::FleetMoveOrder(int empire, int fleet_id, int start_system_id, in
 void FleetMoveOrder::ExecuteImpl() const {
     ValidateEmpireID();
 
-    TemporaryPtr<Fleet> fleet = GetFleet(FleetID());
+    std::shared_ptr<Fleet> fleet = GetFleet(FleetID());
     if (!fleet) {
         ErrorLogger() << "Empire with id " << EmpireID() << " ordered fleet with id " << FleetID() << " to move, but no such fleet exists";
         return;
     }
 
-    TemporaryPtr<const System> destination_system = GetEmpireKnownSystem(DestinationSystemID(), EmpireID());
+    std::shared_ptr<const System> destination_system = GetEmpireKnownSystem(DestinationSystemID(), EmpireID());
     if (!destination_system) {
         ErrorLogger() << "Empire with id " << EmpireID() << " ordered fleet to move to system with id " << DestinationSystemID() << " but no such system is known to that empire";
         return;
@@ -369,8 +368,8 @@ void FleetMoveOrder::ExecuteImpl() const {
     }
 
     std::string waypoints;
-    for (std::list<int>::iterator it = route_list.begin(); it != route_list.end(); ++it) {
-        waypoints += std::string(" ") + boost::lexical_cast<std::string>(*it);
+    for (int waypoint : route_list) {
+        waypoints += std::string(" ") + std::to_string(waypoint);
     }
     DebugLogger() << "FleetMoveOrder::ExecuteImpl Setting route of fleet " << fleet->ID() << " to " << waypoints;
 
@@ -395,7 +394,7 @@ void FleetTransferOrder::ExecuteImpl() const {
     ValidateEmpireID();
 
     // look up the destination fleet
-    TemporaryPtr<Fleet> target_fleet = GetFleet(DestinationFleet());
+    std::shared_ptr<Fleet> target_fleet = GetFleet(DestinationFleet());
     if (!target_fleet) {
         ErrorLogger() << "Empire attempted to move ships to a nonexistant fleet";
         return;
@@ -412,17 +411,14 @@ void FleetTransferOrder::ExecuteImpl() const {
     }
 
     // check that all ships are in the same system
-    std::vector<TemporaryPtr<Ship> > ships = Objects().FindObjects<Ship>(m_add_ships);
+    std::vector<std::shared_ptr<Ship>> ships = Objects().FindObjects<Ship>(m_add_ships);
 
-    std::vector<TemporaryPtr<Ship> > validated_ships;
+    std::vector<std::shared_ptr<Ship>> validated_ships;
     validated_ships.reserve(m_add_ships.size());
     std::vector<int>                 validated_ship_ids;
     validated_ship_ids.reserve(m_add_ships.size());
 
-    for (std::vector<TemporaryPtr<Ship> >::const_iterator it = ships.begin();
-         it != ships.end(); ++it)
-    {
-        TemporaryPtr<Ship> ship = *it;
+    for (std::shared_ptr<Ship> ship : ships) {
         if (!ship->OwnedBy(EmpireID()))
             continue;
         if (ship->SystemID() != target_fleet->SystemID())
@@ -438,12 +434,9 @@ void FleetTransferOrder::ExecuteImpl() const {
     GetUniverse().InhibitUniverseObjectSignals(true);
 
     // remove from old fleet(s)
-    std::set<TemporaryPtr<Fleet> > modified_fleets;
-    for (std::vector<TemporaryPtr<Ship> >::iterator it = validated_ships.begin();
-         it != validated_ships.end(); ++it)
-    {
-        TemporaryPtr<Ship> ship = *it;
-        if (TemporaryPtr<Fleet> source_fleet = GetFleet(ship->FleetID())) {
+    std::set<std::shared_ptr<Fleet>> modified_fleets;
+    for (std::shared_ptr<Ship> ship : validated_ships) {
+        if (std::shared_ptr<Fleet> source_fleet = GetFleet(ship->FleetID())) {
             source_fleet->RemoveShip(ship->ID());
             modified_fleets.insert(source_fleet);
         }
@@ -458,10 +451,7 @@ void FleetTransferOrder::ExecuteImpl() const {
     // signal change to fleet states
     modified_fleets.insert(target_fleet);
 
-    for (std::set<TemporaryPtr<Fleet> >::iterator it = modified_fleets.begin();
-         it != modified_fleets.end(); ++it)
-    {
-        TemporaryPtr<Fleet> modified_fleet = *it;
+    for (std::shared_ptr<Fleet> modified_fleet : modified_fleets) {
         if (!modified_fleet->Empty())
             modified_fleet->StateChangedSignal();
         // if modified fleet is empty, it should be immently destroyed, so that updating it now is redundant
@@ -487,7 +477,7 @@ void ColonizeOrder::ExecuteImpl() const {
     ValidateEmpireID();
     int empire_id = EmpireID();
 
-    TemporaryPtr<Ship> ship = GetShip(m_ship);
+    std::shared_ptr<Ship> ship = GetShip(m_ship);
     if (!ship) {
         ErrorLogger() << "ColonizeOrder::ExecuteImpl couldn't get ship with id " << m_ship;
         return;
@@ -503,7 +493,7 @@ void ColonizeOrder::ExecuteImpl() const {
 
     float colonist_capacity = ship->ColonyCapacity();
 
-    TemporaryPtr<Planet> planet = GetPlanet(m_planet);
+    std::shared_ptr<Planet> planet = GetPlanet(m_planet);
     if (!planet) {
         ErrorLogger() << "ColonizeOrder::ExecuteImpl couldn't get planet with id " << m_planet;
         return;
@@ -547,12 +537,12 @@ void ColonizeOrder::ExecuteImpl() const {
     planet->SetIsAboutToBeColonized(true);
     ship->SetColonizePlanet(m_planet);
 
-    if (TemporaryPtr<Fleet> fleet = GetFleet(ship->FleetID()))
+    if (std::shared_ptr<Fleet> fleet = GetFleet(ship->FleetID()))
         fleet->StateChangedSignal();
 }
 
 bool ColonizeOrder::UndoImpl() const {
-    TemporaryPtr<Planet> planet = GetPlanet(m_planet);
+    std::shared_ptr<Planet> planet = GetPlanet(m_planet);
     if (!planet) {
         ErrorLogger() << "ColonizeOrder::UndoImpl couldn't get planet with id " << m_planet;
         return false;
@@ -562,7 +552,7 @@ bool ColonizeOrder::UndoImpl() const {
         return false;
     }
 
-    TemporaryPtr<Ship> ship = GetShip(m_ship);
+    std::shared_ptr<Ship> ship = GetShip(m_ship);
     if (!ship) {
         ErrorLogger() << "ColonizeOrder::UndoImpl couldn't get ship with id " << m_ship;
         return false;
@@ -575,7 +565,7 @@ bool ColonizeOrder::UndoImpl() const {
     planet->SetIsAboutToBeColonized(false);
     ship->ClearColonizePlanet();
 
-    if (TemporaryPtr<Fleet> fleet = GetFleet(ship->FleetID()))
+    if (std::shared_ptr<Fleet> fleet = GetFleet(ship->FleetID()))
         fleet->StateChangedSignal();
 
     return true;
@@ -600,7 +590,7 @@ void InvadeOrder::ExecuteImpl() const {
     ValidateEmpireID();
     int empire_id = EmpireID();
 
-    TemporaryPtr<Ship> ship = GetShip(m_ship);
+    std::shared_ptr<Ship> ship = GetShip(m_ship);
     if (!ship) {
         ErrorLogger() << "InvadeOrder::ExecuteImpl couldn't get ship with id " << m_ship;
         return;
@@ -614,7 +604,7 @@ void InvadeOrder::ExecuteImpl() const {
         return;
     }
 
-    TemporaryPtr<Planet> planet = GetPlanet(m_planet);
+    std::shared_ptr<Planet> planet = GetPlanet(m_planet);
     if (!planet) {
         ErrorLogger() << "InvadeOrder::ExecuteImpl couldn't get planet with id " << m_planet;
         return;
@@ -657,18 +647,18 @@ void InvadeOrder::ExecuteImpl() const {
     planet->SetIsAboutToBeInvaded(true);
     ship->SetInvadePlanet(m_planet);
 
-    if (TemporaryPtr<Fleet> fleet = GetFleet(ship->FleetID()))
+    if (std::shared_ptr<Fleet> fleet = GetFleet(ship->FleetID()))
         fleet->StateChangedSignal();
 }
 
 bool InvadeOrder::UndoImpl() const {
-    TemporaryPtr<Planet> planet = GetPlanet(m_planet);
+    std::shared_ptr<Planet> planet = GetPlanet(m_planet);
     if (!planet) {
         ErrorLogger() << "InvadeOrder::UndoImpl couldn't get planet with id " << m_planet;
         return false;
     }
 
-    TemporaryPtr<Ship> ship = GetShip(m_ship);
+    std::shared_ptr<Ship> ship = GetShip(m_ship);
     if (!ship) {
         ErrorLogger() << "InvadeOrder::UndoImpl couldn't get ship with id " << m_ship;
         return false;
@@ -681,7 +671,7 @@ bool InvadeOrder::UndoImpl() const {
     planet->SetIsAboutToBeInvaded(false);
     ship->ClearInvadePlanet();
 
-    if (TemporaryPtr<Fleet> fleet = GetFleet(ship->FleetID()))
+    if (std::shared_ptr<Fleet> fleet = GetFleet(ship->FleetID()))
         fleet->StateChangedSignal();
 
     return true;
@@ -706,7 +696,7 @@ void BombardOrder::ExecuteImpl() const {
     ValidateEmpireID();
     int empire_id = EmpireID();
 
-    TemporaryPtr<Ship> ship = GetShip(m_ship);
+    std::shared_ptr<Ship> ship = GetShip(m_ship);
     if (!ship) {
         ErrorLogger() << "BombardOrder::ExecuteImpl couldn't get ship with id " << m_ship;
         return;
@@ -720,7 +710,7 @@ void BombardOrder::ExecuteImpl() const {
         return;
     }
 
-    TemporaryPtr<Planet> planet = GetPlanet(m_planet);
+    std::shared_ptr<Planet> planet = GetPlanet(m_planet);
     if (!planet) {
         ErrorLogger() << "BombardOrder::ExecuteImpl couldn't get planet with id " << m_planet;
         return;
@@ -755,18 +745,18 @@ void BombardOrder::ExecuteImpl() const {
     planet->SetIsAboutToBeBombarded(true);
     ship->SetBombardPlanet(m_planet);
 
-    if (TemporaryPtr<Fleet> fleet = GetFleet(ship->FleetID()))
+    if (std::shared_ptr<Fleet> fleet = GetFleet(ship->FleetID()))
         fleet->StateChangedSignal();
 }
 
 bool BombardOrder::UndoImpl() const {
-    TemporaryPtr<Planet> planet = GetPlanet(m_planet);
+    std::shared_ptr<Planet> planet = GetPlanet(m_planet);
     if (!planet) {
         ErrorLogger() << "BombardOrder::UndoImpl couldn't get planet with id " << m_planet;
         return false;
     }
 
-    TemporaryPtr<Ship> ship = GetShip(m_ship);
+    std::shared_ptr<Ship> ship = GetShip(m_ship);
     if (!ship) {
         ErrorLogger() << "BombardOrder::UndoImpl couldn't get ship with id " << m_ship;
         return false;
@@ -779,7 +769,7 @@ bool BombardOrder::UndoImpl() const {
     planet->SetIsAboutToBeBombarded(false);
     ship->ClearBombardPlanet();
 
-    if (TemporaryPtr<Fleet> fleet = GetFleet(ship->FleetID()))
+    if (std::shared_ptr<Fleet> fleet = GetFleet(ship->FleetID()))
         fleet->StateChangedSignal();
 
     return true;
@@ -801,7 +791,7 @@ DeleteFleetOrder::DeleteFleetOrder(int empire, int fleet) :
 void DeleteFleetOrder::ExecuteImpl() const {
     ValidateEmpireID();
 
-    TemporaryPtr<Fleet> fleet = GetFleet(FleetID());
+    std::shared_ptr<Fleet> fleet = GetFleet(FleetID());
 
     if (!fleet) {
         ErrorLogger() << "Illegal fleet id specified in fleet delete order: " << FleetID();
@@ -816,7 +806,7 @@ void DeleteFleetOrder::ExecuteImpl() const {
     if (!fleet->Empty())
         return; // should be no ships to delete
 
-    TemporaryPtr<System> system = GetSystem(fleet->SystemID());
+    std::shared_ptr<System> system = GetSystem(fleet->SystemID());
     if (system)
         system->Remove(fleet->ID());
 
@@ -841,7 +831,7 @@ ChangeFocusOrder::ChangeFocusOrder(int empire, int planet, const std::string& fo
 void ChangeFocusOrder::ExecuteImpl() const {
     ValidateEmpireID();
 
-    TemporaryPtr<Planet> planet = GetPlanet(PlanetID());
+    std::shared_ptr<Planet> planet = GetPlanet(PlanetID());
 
     if (!planet) {
         ErrorLogger() << "Illegal planet id specified in change planet focus order.";
@@ -861,22 +851,33 @@ void ChangeFocusOrder::ExecuteImpl() const {
 ////////////////////////////////////////////////
 ResearchQueueOrder::ResearchQueueOrder() :
     Order(),
-    m_position(-1),
-    m_remove(false)
+    m_position(INVALID_INDEX),
+    m_remove(false),
+    m_pause(INVALID_PAUSE_RESUME)
 {}
 
 ResearchQueueOrder::ResearchQueueOrder(int empire, const std::string& tech_name) :
     Order(empire),
     m_tech_name(tech_name),
-    m_position(-1),
-    m_remove(true)
+    m_position(INVALID_INDEX),
+    m_remove(true),
+    m_pause(INVALID_PAUSE_RESUME)
 {}
 
 ResearchQueueOrder::ResearchQueueOrder(int empire, const std::string& tech_name, int position) :
     Order(empire),
     m_tech_name(tech_name),
     m_position(position),
-    m_remove(false)
+    m_remove(false),
+    m_pause(INVALID_PAUSE_RESUME)
+{}
+
+ResearchQueueOrder::ResearchQueueOrder(int empire, const std::string& tech_name, bool pause, float dummy) :
+    Order(empire),
+    m_tech_name(tech_name),
+    m_position(INVALID_INDEX),
+    m_remove(false),
+    m_pause(pause ? PAUSE : RESUME)
 {}
 
 void ResearchQueueOrder::ExecuteImpl() const {
@@ -887,9 +888,18 @@ void ResearchQueueOrder::ExecuteImpl() const {
     if (m_remove) {
         DebugLogger() << "ResearchQueueOrder::ExecuteImpl: removing from queue tech: " << m_tech_name;
         empire->RemoveTechFromQueue(m_tech_name);
-    }
-    else
+    } else if (m_pause == PAUSE) {
+        DebugLogger() << "ResearchQueueOrder::ExecuteImpl: pausing tech: " << m_tech_name;
+        empire->PauseResearch(m_tech_name);
+    } else if (m_pause == RESUME) {
+        DebugLogger() << "ResearchQueueOrder::ExecuteImpl: unpausing tech: " << m_tech_name;
+        empire->ResumeResearch(m_tech_name);
+    } else if (m_position != INVALID_INDEX) {
+        DebugLogger() << "ResearchQueueOrder::ExecuteImpl: adding tech to queue: " << m_tech_name;
         empire->PlaceTechInQueue(m_tech_name, m_position);
+    } else {
+        ErrorLogger() << "ResearchQueueOrder::ExecuteImpl: Malformed";
+    }
 }
 
 ////////////////////////////////////////////////
@@ -904,10 +914,12 @@ ProductionQueueOrder::ProductionQueueOrder() :
     m_new_quantity(INVALID_QUANTITY),
     m_new_blocksize(INVALID_QUANTITY),
     m_new_index(INVALID_INDEX),
-    m_rally_point_id(INVALID_OBJECT_ID)
+    m_rally_point_id(INVALID_OBJECT_ID),
+    m_pause(INVALID_PAUSE_RESUME)
 {}
 
-ProductionQueueOrder::ProductionQueueOrder(int empire, const ProductionQueue::ProductionItem& item, int number, int location, int pos) :
+ProductionQueueOrder::ProductionQueueOrder(int empire, const ProductionQueue::ProductionItem& item,
+                                           int number, int location, int pos) :
     Order(empire),
     m_item(item),
     m_number(number),
@@ -916,7 +928,8 @@ ProductionQueueOrder::ProductionQueueOrder(int empire, const ProductionQueue::Pr
     m_new_quantity(INVALID_QUANTITY),
     m_new_blocksize(INVALID_QUANTITY),
     m_new_index(pos),
-    m_rally_point_id(INVALID_OBJECT_ID)
+    m_rally_point_id(INVALID_OBJECT_ID),
+    m_pause(INVALID_PAUSE_RESUME)
 {}
 
 ProductionQueueOrder::ProductionQueueOrder(int empire, int index, int new_quantity, int new_blocksize) :
@@ -928,7 +941,8 @@ ProductionQueueOrder::ProductionQueueOrder(int empire, int index, int new_quanti
     m_new_quantity(new_quantity),
     m_new_blocksize(new_blocksize),
     m_new_index(INVALID_INDEX),
-    m_rally_point_id(INVALID_OBJECT_ID)
+    m_rally_point_id(INVALID_OBJECT_ID),
+    m_pause(INVALID_PAUSE_RESUME)
 {}
 
 ProductionQueueOrder::ProductionQueueOrder(int empire, int index, int new_quantity, bool dummy) :
@@ -940,7 +954,8 @@ ProductionQueueOrder::ProductionQueueOrder(int empire, int index, int new_quanti
     m_new_quantity(new_quantity),
     m_new_blocksize(INVALID_QUANTITY),
     m_new_index(INVALID_INDEX),
-    m_rally_point_id(INVALID_OBJECT_ID)
+    m_rally_point_id(INVALID_OBJECT_ID),
+    m_pause(INVALID_PAUSE_RESUME)
 {}
 
 ProductionQueueOrder::ProductionQueueOrder(int empire, int index, int rally_point_id, bool dummy1, bool dummy2) :
@@ -952,7 +967,8 @@ ProductionQueueOrder::ProductionQueueOrder(int empire, int index, int rally_poin
     m_new_quantity(INVALID_QUANTITY),
     m_new_blocksize(INVALID_QUANTITY),
     m_new_index(INVALID_INDEX),
-    m_rally_point_id(rally_point_id)
+    m_rally_point_id(rally_point_id),
+    m_pause(INVALID_PAUSE_RESUME)
 {}
 
 ProductionQueueOrder::ProductionQueueOrder(int empire, int index, int new_index) :
@@ -964,7 +980,8 @@ ProductionQueueOrder::ProductionQueueOrder(int empire, int index, int new_index)
     m_new_quantity(INVALID_QUANTITY),
     m_new_blocksize(INVALID_QUANTITY),
     m_new_index(new_index),
-    m_rally_point_id(INVALID_OBJECT_ID)
+    m_rally_point_id(INVALID_OBJECT_ID),
+    m_pause(INVALID_PAUSE_RESUME)
 {}
 
 ProductionQueueOrder::ProductionQueueOrder(int empire, int index) :
@@ -976,7 +993,21 @@ ProductionQueueOrder::ProductionQueueOrder(int empire, int index) :
     m_new_quantity(INVALID_QUANTITY),
     m_new_blocksize(INVALID_QUANTITY),
     m_new_index(INVALID_INDEX),
-    m_rally_point_id(INVALID_OBJECT_ID)
+    m_rally_point_id(INVALID_OBJECT_ID),
+    m_pause(INVALID_PAUSE_RESUME)
+{}
+
+ProductionQueueOrder::ProductionQueueOrder(int empire, int index, bool pause, float dummy) :
+    Order(empire),
+    m_item(),
+    m_number(0),
+    m_location(INVALID_OBJECT_ID),
+    m_index(index),
+    m_new_quantity(INVALID_QUANTITY),
+    m_new_blocksize(INVALID_QUANTITY),
+    m_new_index(INVALID_INDEX),
+    m_rally_point_id(INVALID_OBJECT_ID),
+    m_pause(pause ? PAUSE : RESUME)
 {}
 
 void ProductionQueueOrder::ExecuteImpl() const {
@@ -985,28 +1016,39 @@ void ProductionQueueOrder::ExecuteImpl() const {
     Empire* empire = GetEmpire(EmpireID());
     try {
         if (m_item.build_type == BT_BUILDING || m_item.build_type == BT_SHIP) {
-            empire->PlaceBuildInQueue(m_item, m_number, m_location, m_new_index);
+            empire->PlaceProductionOnQueue(m_item, m_number, m_location, m_new_index);
 
         } else if (m_new_blocksize != INVALID_QUANTITY) {
             DebugLogger() << "ProductionQueueOrder quantity " << m_new_quantity << " Blocksize " << m_new_blocksize;
-            empire->SetBuildQuantityAndBlocksize(m_index, m_new_quantity, m_new_blocksize);
+            empire->SetProductionQuantityAndBlocksize(m_index, m_new_quantity, m_new_blocksize);
 
         } else if (m_new_quantity != INVALID_QUANTITY) {
-            empire->SetBuildQuantity(m_index, m_new_quantity);
+            DebugLogger() << "ProductionQueueOrder quantity " << m_new_quantity;
+            empire->SetProductionQuantity(m_index, m_new_quantity);
 
         } else if (m_new_index != INVALID_INDEX) {
-            empire->MoveBuildWithinQueue(m_index, m_new_index);
+            DebugLogger() << "ProductionQueueOrder moving item in queue";
+            empire->MoveProductionWithinQueue(m_index, m_new_index);
 
         } else if (m_rally_point_id != INVALID_OBJECT_ID) {
             DebugLogger() << "ProductionQueueOrder setting rally point to id: " << m_rally_point_id;
-            empire->SetBuildRallyPoint(m_index, m_rally_point_id);
+            empire->SetProductionRallyPoint(m_index, m_rally_point_id);
 
         } else if (m_index != INVALID_INDEX) {
-            DebugLogger() << "ProductionQueueOrder removing build from index " << m_index;
-            empire->RemoveBuildFromQueue(m_index);
+            if (m_pause == PAUSE) {
+                DebugLogger() << "ProductionQueueOrder: pausing production";
+                empire->PauseProduction(m_index);
 
+            } else if (m_pause == RESUME) {
+                DebugLogger() << "ProductionQueueOrder: unpausing production";
+                empire->ResumeProduction(m_index);
+
+            } else /*if (m_pause == INVALID_PAUSE_RESUME)*/ {
+                DebugLogger() << "ProductionQueueOrder: removing item from index " << m_index;
+                empire->RemoveProductionFromQueue(m_index);
+            }
         } else {
-            ErrorLogger() << "Malformed ProductionQueueOrder.";
+            ErrorLogger() << "ProductionQueueOrder: Malformed";
         }
     } catch (const std::exception& e) {
         ErrorLogger() << "Build order execution threw exception: " << e.what();
@@ -1205,16 +1247,16 @@ void ScrapOrder::ExecuteImpl() const {
     ValidateEmpireID();
     int empire_id = EmpireID();
 
-    if (TemporaryPtr<Ship> ship = GetShip(m_object_id)) {
+    if (std::shared_ptr<Ship> ship = GetShip(m_object_id)) {
         if (ship->SystemID() != INVALID_OBJECT_ID && ship->OwnedBy(empire_id)) {
             ship->SetOrderedScrapped(true);
             //DebugLogger() << "ScrapOrder::ExecuteImpl empire: " << empire_id
             //                       << " on ship: " << ship->ID() << " at system: " << ship->SystemID()
             //                       << " ... ordered scrapped?: " << ship->OrderedScrapped();
         }
-    } else if (TemporaryPtr<Building> building = GetBuilding(m_object_id)) {
+    } else if (std::shared_ptr<Building> building = GetBuilding(m_object_id)) {
         int planet_id = building->PlanetID();
-        if (TemporaryPtr<const Planet> planet = GetPlanet(planet_id)) {
+        if (std::shared_ptr<const Planet> planet = GetPlanet(planet_id)) {
             if (building->OwnedBy(empire_id) && planet->OwnedBy(empire_id))
                 building->SetOrderedScrapped(true);
         }
@@ -1225,10 +1267,10 @@ bool ScrapOrder::UndoImpl() const {
     ValidateEmpireID();
     int empire_id = EmpireID();
 
-    if (TemporaryPtr<Ship> ship = GetShip(m_object_id)) {
+    if (std::shared_ptr<Ship> ship = GetShip(m_object_id)) {
         if (ship->OwnedBy(empire_id))
             ship->SetOrderedScrapped(false);
-    } else if (TemporaryPtr<Building> building = GetBuilding(m_object_id)) {
+    } else if (std::shared_ptr<Building> building = GetBuilding(m_object_id)) {
         if (building->OwnedBy(empire_id))
             building->SetOrderedScrapped(false);
     } else {
@@ -1255,7 +1297,7 @@ AggressiveOrder::AggressiveOrder(int empire, int object_id, bool aggression/* =
 void AggressiveOrder::ExecuteImpl() const {
     ValidateEmpireID();
     int empire_id = EmpireID();
-    if (TemporaryPtr<Fleet> fleet = GetFleet(m_object_id)) {
+    if (std::shared_ptr<Fleet> fleet = GetFleet(m_object_id)) {
         if (fleet->OwnedBy(empire_id))
             fleet->SetAggressive(m_aggression);
     }
@@ -1280,11 +1322,11 @@ void GiveObjectToEmpireOrder::ExecuteImpl() const {
     ValidateEmpireID();
     int empire_id = EmpireID();
 
-    if (TemporaryPtr<Fleet> fleet = GetFleet(m_object_id)) {
+    if (std::shared_ptr<Fleet> fleet = GetFleet(m_object_id)) {
         if (fleet->OwnedBy(empire_id))
             fleet->SetGiveToEmpire(m_recipient_empire_id);
 
-    } else if (TemporaryPtr<Planet> planet = GetPlanet(m_object_id)) {
+    } else if (std::shared_ptr<Planet> planet = GetPlanet(m_object_id)) {
         if (planet->OwnedBy(empire_id))
             planet->SetGiveToEmpire(m_recipient_empire_id);
     }
@@ -1294,12 +1336,12 @@ bool GiveObjectToEmpireOrder::UndoImpl() const {
     ValidateEmpireID();
     int empire_id = EmpireID();
 
-    if (TemporaryPtr<Fleet> fleet = GetFleet(m_object_id)) {
+    if (std::shared_ptr<Fleet> fleet = GetFleet(m_object_id)) {
         if (fleet->OwnedBy(empire_id)) {
             fleet->ClearGiveToEmpire();
             return true;
         }
-    } else if (TemporaryPtr<Planet> planet = GetPlanet(m_object_id)) {
+    } else if (std::shared_ptr<Planet> planet = GetPlanet(m_object_id)) {
         if (planet->OwnedBy(empire_id)) {
             planet->ClearGiveToEmpire();
             return true;
