@@ -17,7 +17,8 @@ void ResearchQueue::Element::serialize(Archive& ar, const unsigned int version)
     ar  & BOOST_SERIALIZATION_NVP(name)
         & BOOST_SERIALIZATION_NVP(empire_id)
         & BOOST_SERIALIZATION_NVP(allocated_rp)
-        & BOOST_SERIALIZATION_NVP(turns_left);
+        & BOOST_SERIALIZATION_NVP(turns_left)
+        & BOOST_SERIALIZATION_NVP(paused);
 }
 
 template void ResearchQueue::Element::serialize<freeorion_bin_oarchive>(freeorion_bin_oarchive&, const unsigned int);
@@ -67,7 +68,8 @@ void ProductionQueue::Element::serialize(Archive& ar, const unsigned int version
         & BOOST_SERIALIZATION_NVP(blocksize_memory)
         & BOOST_SERIALIZATION_NVP(turns_left_to_next_item)
         & BOOST_SERIALIZATION_NVP(turns_left_to_completion)
-        & BOOST_SERIALIZATION_NVP(rally_point_id);
+        & BOOST_SERIALIZATION_NVP(rally_point_id)
+        & BOOST_SERIALIZATION_NVP(paused);
 }
 
 template void ProductionQueue::Element::serialize<freeorion_bin_oarchive>(freeorion_bin_oarchive&, const unsigned int);
@@ -97,6 +99,7 @@ void Empire::serialize(Archive& ar, const unsigned int version)
         & BOOST_SERIALIZATION_NVP(m_player_name)
         & BOOST_SERIALIZATION_NVP(m_color)
         & BOOST_SERIALIZATION_NVP(m_capital_id)
+        & BOOST_SERIALIZATION_NVP(m_source_id)
         & BOOST_SERIALIZATION_NVP(m_eliminated)
         & BOOST_SERIALIZATION_NVP(m_victories)
         & BOOST_SERIALIZATION_NVP(m_techs)
@@ -126,6 +129,8 @@ void Empire::serialize(Archive& ar, const unsigned int version)
 
             & BOOST_SERIALIZATION_NVP(m_species_ships_owned)
             & BOOST_SERIALIZATION_NVP(m_ship_designs_owned)
+            & BOOST_SERIALIZATION_NVP(m_ship_part_types_owned)
+            & BOOST_SERIALIZATION_NVP(m_ship_part_class_owned)
             & BOOST_SERIALIZATION_NVP(m_species_colonies_owned)
             & BOOST_SERIALIZATION_NVP(m_outposts_owned)
             & BOOST_SERIALIZATION_NVP(m_building_types_owned)
@@ -199,7 +204,10 @@ void SupplyManager::serialize(Archive& ar, const unsigned int version)
         & BOOST_SERIALIZATION_NVP(m_supply_starlane_obstructed_traversals)
         & BOOST_SERIALIZATION_NVP(m_fleet_supplyable_system_ids)
         & BOOST_SERIALIZATION_NVP(m_resource_supply_groups)
-        & BOOST_SERIALIZATION_NVP(m_propagated_supply_ranges);
+        & BOOST_SERIALIZATION_NVP(m_propagated_supply_ranges)
+        & BOOST_SERIALIZATION_NVP(m_empire_propagated_supply_ranges)
+        & BOOST_SERIALIZATION_NVP(m_propagated_supply_distances)
+        & BOOST_SERIALIZATION_NVP(m_empire_propagated_supply_distances);
 }
 
 template void SupplyManager::serialize<freeorion_bin_oarchive>(freeorion_bin_oarchive&, const unsigned int);
