@@ -123,21 +123,18 @@ void Universe::serialize(Archive& ar, const unsigned int version)
         DebugLogger() << "Universe::serialize : Cleaning up temporary data";
         // clean up temporary objects in temporary ObjectMaps
         objects.Clear();
-        for (EmpireObjectMap::iterator it = empire_latest_known_objects.begin();
-             it != empire_latest_known_objects.end(); ++it)
-        { it->second.Clear(); }
+        for (std::map<int, ObjectMap>::value_type& elko : empire_latest_known_objects)
+        { elko.second.Clear(); }
     }
 
     if (Archive::is_loading::value) {
         DebugLogger() << "Universe::serialize : updating empires' latest known object destruction states";
         // update known destroyed objects state in each empire's latest known objects
-        for (EmpireObjectMap::iterator it = m_empire_latest_known_objects.begin();
-             it != m_empire_latest_known_objects.end(); ++it)
-        {
-            std::map< int, std::set< int > >::iterator destroyed_ids_it =
-                m_empire_known_destroyed_object_ids.find(it->first);
+        for (std::map<int, ObjectMap>::value_type& elko : m_empire_latest_known_objects) {
+            std::map<int, std::set<int>>::iterator destroyed_ids_it =
+                m_empire_known_destroyed_object_ids.find(elko.first);
             if (destroyed_ids_it != m_empire_known_destroyed_object_ids.end())
-                it->second.UpdateCurrentDestroyedObjects(destroyed_ids_it->second);
+                elko.second.UpdateCurrentDestroyedObjects(destroyed_ids_it->second);
         }
     }
     DebugLogger() << "Universe::serialize done";
@@ -263,13 +260,10 @@ void ShipDesign::serialize(Archive& ar, const unsigned int version)
         BuildStatCaches();
 }
 
-// explicit template initialization of System::serialize needed to avoid bug with GCC 4.5.2.
 template
 void SpeciesManager::serialize<freeorion_bin_oarchive>(freeorion_bin_oarchive& ar, const unsigned int version);
 template
 void SpeciesManager::serialize<freeorion_xml_oarchive>(freeorion_xml_oarchive& ar, const unsigned int version);
-
-// explicit template initialization of System::serialize needed to avoid bug with GCC 4.5.2.
 template
 void SpeciesManager::serialize<freeorion_bin_iarchive>(freeorion_bin_iarchive& ar, const unsigned int version);
 template
@@ -284,11 +278,11 @@ void SpeciesManager::serialize(Archive& ar, const unsigned int version)
     // species: their homeworlds in the current game, and their opinions of
     // empires and eachother
 
-    std::map<std::string, std::set<int> >                   species_homeworlds;
-    std::map<std::string, std::map<int, float> >            empire_opinions;
-    std::map<std::string, std::map<std::string, float> >    other_species_opinions;
-    std::map<std::string, std::map<int, float> >            species_object_populations;
-    std::map<std::string, std::map<std::string, int> >      species_ships_destroyed;
+    std::map<std::string, std::set<int>>                species_homeworlds;
+    std::map<std::string, std::map<int, float>>         empire_opinions;
+    std::map<std::string, std::map<std::string, float>> other_species_opinions;
+    std::map<std::string, std::map<int, float>>         species_object_populations;
+    std::map<std::string, std::map<std::string, int>>   species_ships_destroyed;
 
     if (Archive::is_saving::value) {
         species_homeworlds =        GetSpeciesHomeworldsMap(GetUniverse().EncodingEmpire());
@@ -333,10 +327,10 @@ template FO_COMMON_API void Serialize<freeorion_bin_oarchive>(freeorion_bin_oarc
 template FO_COMMON_API void Serialize<freeorion_xml_oarchive>(freeorion_xml_oarchive& oa, const Universe& universe);
 
 template <class Archive>
-void Serialize(Archive& oa, const std::map<int, TemporaryPtr<UniverseObject> >& objects)
+void Serialize(Archive& oa, const std::map<int, std::shared_ptr<UniverseObject>>& objects)
 { oa << BOOST_SERIALIZATION_NVP(objects); }
-template void Serialize<freeorion_bin_oarchive>(freeorion_bin_oarchive& oa, const std::map<int, TemporaryPtr<UniverseObject> >& objects);
-template void Serialize<freeorion_xml_oarchive>(freeorion_xml_oarchive& oa, const std::map<int, TemporaryPtr<UniverseObject> >& objects);
+template void Serialize<freeorion_bin_oarchive>(freeorion_bin_oarchive& oa, const std::map<int, std::shared_ptr<UniverseObject>>& objects);
+template void Serialize<freeorion_xml_oarchive>(freeorion_xml_oarchive& oa, const std::map<int, std::shared_ptr<UniverseObject>>& objects);
 
 template <class Archive>
 void Deserialize(Archive& ia, Universe& universe)
@@ -345,7 +339,7 @@ template FO_COMMON_API void Deserialize<freeorion_bin_iarchive>(freeorion_bin_ia
 template FO_COMMON_API void Deserialize<freeorion_xml_iarchive>(freeorion_xml_iarchive& ia, Universe& universe);
 
 template <class Archive>
-void Deserialize(Archive& ia, std::map<int, TemporaryPtr<UniverseObject> >& objects)
+void Deserialize(Archive& ia, std::map<int, std::shared_ptr<UniverseObject>>& objects)
 { ia >> BOOST_SERIALIZATION_NVP(objects); }
-template void Deserialize<freeorion_bin_iarchive>(freeorion_bin_iarchive& ia, std::map<int, TemporaryPtr<UniverseObject> >& objects);
-template void Deserialize<freeorion_xml_iarchive>(freeorion_xml_iarchive& ia, std::map<int, TemporaryPtr<UniverseObject> >& objects);
+template void Deserialize<freeorion_bin_iarchive>(freeorion_bin_iarchive& ia, std::map<int, std::shared_ptr<UniverseObject>>& objects);
+template void Deserialize<freeorion_xml_iarchive>(freeorion_xml_iarchive& ia, std::map<int, std::shared_ptr<UniverseObject>>& objects);
