@@ -25,12 +25,10 @@ SitRepEntry::SitRepEntry(const std::string& template_string, int turn, const std
 {}
 
 int SitRepEntry::GetDataIDNumber(const std::string& tag) const {
-    if (!m_variables.ContainsChild(tag))
-        return -1;
-    const XMLElement& token_elem = m_variables.Child(tag);
+    std::map<std::string, std::string>::const_iterator elem = m_variables.find(tag);
     try {
-        const std::string& text = token_elem.Attribute("value");
-        return boost::lexical_cast<int>(text);
+        if (elem != m_variables.end())
+            return boost::lexical_cast<int>(elem->second);
     } catch (...) {
         return -1;
     }
@@ -39,20 +37,17 @@ int SitRepEntry::GetDataIDNumber(const std::string& tag) const {
 
 const std::string& SitRepEntry::GetDataString(const std::string& tag) const {
     static const std::string EMPTY_STRING;
-    if (!m_variables.ContainsChild(tag))
+    std::map<std::string, std::string>::const_iterator elem = m_variables.find(tag);
+    if (elem == m_variables.end())
         return EMPTY_STRING;
-    const XMLElement& token_elem = m_variables.Child(tag);
-    return token_elem.Attribute("value");
-    return EMPTY_STRING;
+    return elem->second;
 }
 
 std::string SitRepEntry::Dump() const {
     std::string retval = "SitRep template_string = \"" + m_template_string + "\"";
-    if (m_variables.NumChildren() > 0) {
-        for (XMLElement::const_child_iterator it = m_variables.child_begin(); it != m_variables.child_end(); ++it)
-            retval += " " + it->Tag() + " = " + it->Attribute("value");
-    }
-    retval += " turn = " + boost::lexical_cast<std::string>(m_turn);
+    for (const std::map<std::string, std::string>::value_type& variable : m_variables)
+        retval += " " + variable.first + " = " + variable.second;
+    retval += " turn = " + std::to_string(m_turn);
     retval += " icon = " + m_icon;
     retval += " label = " + m_label;
     return retval;
@@ -74,9 +69,9 @@ SitRepEntry CreateShipBuiltSitRep(int ship_id, int system_id, int shipdesign_id)
         CurrentTurn() + 1,
         "icons/sitrep/ship_produced.png",
         UserStringNop("SITREP_SHIP_BUILT_LABEL"), true);
-    sitrep.AddVariable(VarText::SYSTEM_ID_TAG,     boost::lexical_cast<std::string>(system_id));
-    sitrep.AddVariable(VarText::SHIP_ID_TAG,       boost::lexical_cast<std::string>(ship_id));
-    sitrep.AddVariable(VarText::DESIGN_ID_TAG,     boost::lexical_cast<std::string>(shipdesign_id));
+    sitrep.AddVariable(VarText::SYSTEM_ID_TAG,     std::to_string(system_id));
+    sitrep.AddVariable(VarText::SHIP_ID_TAG,       std::to_string(ship_id));
+    sitrep.AddVariable(VarText::DESIGN_ID_TAG,     std::to_string(shipdesign_id));
     return sitrep;
 }
 
@@ -86,9 +81,9 @@ SitRepEntry CreateShipBlockBuiltSitRep(int system_id, int shipdesign_id, int num
         CurrentTurn() + 1,
         "icons/sitrep/ship_produced.png",
         UserStringNop("SITREP_SHIP_BATCH_BUILT_LABEL"), true);
-    sitrep.AddVariable(VarText::SYSTEM_ID_TAG,     boost::lexical_cast<std::string>(system_id));
-    sitrep.AddVariable(VarText::DESIGN_ID_TAG,     boost::lexical_cast<std::string>(shipdesign_id));
-    sitrep.AddVariable(VarText::RAW_TEXT_TAG,      boost::lexical_cast<std::string>(number));
+    sitrep.AddVariable(VarText::SYSTEM_ID_TAG,     std::to_string(system_id));
+    sitrep.AddVariable(VarText::DESIGN_ID_TAG,     std::to_string(shipdesign_id));
+    sitrep.AddVariable(VarText::RAW_TEXT_TAG,      std::to_string(number));
     return sitrep;
 }
 
@@ -98,8 +93,8 @@ SitRepEntry CreateBuildingBuiltSitRep(int building_id, int planet_id) {
         CurrentTurn() + 1,
         "icons/sitrep/building_produced.png",
         UserStringNop("SITREP_BUILDING_BUILT_LABEL"), true);
-    sitrep.AddVariable(VarText::PLANET_ID_TAG,     boost::lexical_cast<std::string>(planet_id));
-    sitrep.AddVariable(VarText::BUILDING_ID_TAG,   boost::lexical_cast<std::string>(building_id));
+    sitrep.AddVariable(VarText::PLANET_ID_TAG,     std::to_string(planet_id));
+    sitrep.AddVariable(VarText::BUILDING_ID_TAG,   std::to_string(building_id));
     return sitrep;
 }
 
@@ -153,9 +148,9 @@ SitRepEntry CreateCombatSitRep(int system_id, int log_id, int enemy_id) {
     SitRepEntry sitrep(
         template_string,
         CurrentTurn() + 1, "icons/sitrep/combat.png", label_string, true);
-    sitrep.AddVariable(VarText::SYSTEM_ID_TAG,  boost::lexical_cast<std::string>(system_id));
-    sitrep.AddVariable(VarText::COMBAT_ID_TAG,  boost::lexical_cast<std::string>(log_id));
-    sitrep.AddVariable(VarText::EMPIRE_ID_TAG,  boost::lexical_cast<std::string>(enemy_id));
+    sitrep.AddVariable(VarText::SYSTEM_ID_TAG,  std::to_string(system_id));
+    sitrep.AddVariable(VarText::COMBAT_ID_TAG,  std::to_string(log_id));
+    sitrep.AddVariable(VarText::EMPIRE_ID_TAG,  std::to_string(enemy_id));
     return sitrep;
 }
 
@@ -169,8 +164,8 @@ SitRepEntry CreateGroundCombatSitRep(int planet_id, int enemy_id) {
     SitRepEntry sitrep(
         template_string,
         CurrentTurn() + 1, "icons/sitrep/ground_combat.png", label_string, true);
-    sitrep.AddVariable(VarText::PLANET_ID_TAG,     boost::lexical_cast<std::string>(planet_id));
-    sitrep.AddVariable(VarText::EMPIRE_ID_TAG,     boost::lexical_cast<std::string>(enemy_id));
+    sitrep.AddVariable(VarText::PLANET_ID_TAG,     std::to_string(planet_id));
+    sitrep.AddVariable(VarText::EMPIRE_ID_TAG,     std::to_string(enemy_id));
     return sitrep;
 }
 
@@ -180,8 +175,8 @@ SitRepEntry CreatePlanetCapturedSitRep(int planet_id, int empire_id) {
         CurrentTurn() + 1,
         "icons/sitrep/planet_captured.png",
         UserStringNop("SITREP_PLANET_CAPTURED_LABEL"), true);
-    sitrep.AddVariable(VarText::PLANET_ID_TAG,     boost::lexical_cast<std::string>(planet_id));
-    sitrep.AddVariable(VarText::EMPIRE_ID_TAG,     boost::lexical_cast<std::string>(empire_id));
+    sitrep.AddVariable(VarText::PLANET_ID_TAG,     std::to_string(planet_id));
+    sitrep.AddVariable(VarText::EMPIRE_ID_TAG,     std::to_string(empire_id));
     return sitrep;
 }
 
@@ -192,7 +187,7 @@ namespace {
             CurrentTurn() + 1,
             "icons/sitrep/combat_damage.png",
             UserStringNop("SITREP_OBJECT_DAMAGED_AT_SYSTEM_LABEL"), true);
-        sitrep.AddVariable(VarText::SYSTEM_ID_TAG,     boost::lexical_cast<std::string>(combat_system_id));
+        sitrep.AddVariable(VarText::SYSTEM_ID_TAG,     std::to_string(combat_system_id));
         return sitrep;
     }
 
@@ -202,19 +197,19 @@ namespace {
             CurrentTurn() + 1,
             "icons/sitrep/combat_destroyed.png",
             UserStringNop("SITREP_OBJECT_DESTROYED_AT_SYSTEM_LABEL"), true);
-        sitrep.AddVariable(VarText::SYSTEM_ID_TAG,     boost::lexical_cast<std::string>(combat_system_id));
+        sitrep.AddVariable(VarText::SYSTEM_ID_TAG,     std::to_string(combat_system_id));
         return sitrep;
     }
 }
 
 SitRepEntry CreateCombatDamagedObjectSitRep(int object_id, int combat_system_id, int empire_id) {
-    TemporaryPtr<const UniverseObject> obj = GetUniverseObject(object_id);
+    std::shared_ptr<const UniverseObject> obj = GetUniverseObject(object_id);
     if (!obj)
         return GenericCombatDamagedObjectSitrep(combat_system_id);
 
     SitRepEntry sitrep;
 
-    if (TemporaryPtr<const Ship> ship = boost::dynamic_pointer_cast<const Ship>(obj)) {
+    if (std::shared_ptr<const Ship> ship = std::dynamic_pointer_cast<const Ship>(obj)) {
         if (ship->Unowned())
             sitrep = SitRepEntry(
                 UserStringNop("SITREP_UNOWNED_SHIP_DAMAGED_AT_SYSTEM"),
@@ -227,10 +222,10 @@ SitRepEntry CreateCombatDamagedObjectSitRep(int object_id, int combat_system_id,
                 CurrentTurn() + 1,
                 "icons/sitrep/combat_damage.png",
                 UserStringNop("SITREP_SHIP_DAMAGED_AT_SYSTEM_LABEL"), true);
-        sitrep.AddVariable(VarText::SHIP_ID_TAG,       boost::lexical_cast<std::string>(object_id));
-        sitrep.AddVariable(VarText::DESIGN_ID_TAG,     boost::lexical_cast<std::string>(ship->DesignID()));
+        sitrep.AddVariable(VarText::SHIP_ID_TAG,       std::to_string(object_id));
+        sitrep.AddVariable(VarText::DESIGN_ID_TAG,     std::to_string(ship->DesignID()));
 
-    } else if (TemporaryPtr<const Planet> planet = boost::dynamic_pointer_cast<const Planet>(obj)) {
+    } else if (std::shared_ptr<const Planet> planet = std::dynamic_pointer_cast<const Planet>(obj)) {
         if (planet->Unowned())
             sitrep = SitRepEntry(
                 UserStringNop("SITREP_UNOWNED_PLANET_BOMBARDED_AT_SYSTEM"),
@@ -243,20 +238,20 @@ SitRepEntry CreateCombatDamagedObjectSitRep(int object_id, int combat_system_id,
                 CurrentTurn() + 1,
                 "icons/sitrep/colony_bombarded.png",
                 UserStringNop("SITREP_PLANET_BOMBARDED_AT_SYSTEM_LABEL"), true);
-        sitrep.AddVariable(VarText::PLANET_ID_TAG,     boost::lexical_cast<std::string>(object_id));
+        sitrep.AddVariable(VarText::PLANET_ID_TAG,     std::to_string(object_id));
 
     } else {
         sitrep = GenericCombatDestroyedObjectSitrep(combat_system_id);
     }
 
-    sitrep.AddVariable(VarText::EMPIRE_ID_TAG,     boost::lexical_cast<std::string>(obj->Owner()));
-    sitrep.AddVariable(VarText::SYSTEM_ID_TAG,     boost::lexical_cast<std::string>(combat_system_id));
+    sitrep.AddVariable(VarText::EMPIRE_ID_TAG,     std::to_string(obj->Owner()));
+    sitrep.AddVariable(VarText::SYSTEM_ID_TAG,     std::to_string(combat_system_id));
 
     return sitrep;
 }
 
 SitRepEntry CreateCombatDestroyedObjectSitRep(int object_id, int combat_system_id, int empire_id) {
-    TemporaryPtr<const UniverseObject> obj = GetEmpireKnownObject(object_id, empire_id);
+    std::shared_ptr<const UniverseObject> obj = GetEmpireKnownObject(object_id, empire_id);
     if (!obj) {
         DebugLogger() << "Object " << object_id << " does not exist!!!";
         return GenericCombatDestroyedObjectSitrep(combat_system_id);
@@ -264,7 +259,7 @@ SitRepEntry CreateCombatDestroyedObjectSitRep(int object_id, int combat_system_i
 
     SitRepEntry sitrep;
 
-    if (TemporaryPtr<const Ship> ship = boost::dynamic_pointer_cast<const Ship>(obj)) {
+    if (std::shared_ptr<const Ship> ship = std::dynamic_pointer_cast<const Ship>(obj)) {
         if (ship->Unowned())
             sitrep = SitRepEntry(
                 UserStringNop("SITREP_UNOWNED_SHIP_DESTROYED_AT_SYSTEM"),
@@ -283,10 +278,10 @@ SitRepEntry CreateCombatDestroyedObjectSitRep(int object_id, int combat_system_i
                 CurrentTurn() + 1,
                 "icons/sitrep/combat_destroyed.png",
                 UserStringNop("SITREP_SHIP_DESTROYED_AT_SYSTEM_LABEL"), true);
-        sitrep.AddVariable(VarText::SHIP_ID_TAG,       boost::lexical_cast<std::string>(object_id));
-        sitrep.AddVariable(VarText::DESIGN_ID_TAG,     boost::lexical_cast<std::string>(ship->DesignID()));
+        sitrep.AddVariable(VarText::SHIP_ID_TAG,       std::to_string(object_id));
+        sitrep.AddVariable(VarText::DESIGN_ID_TAG,     std::to_string(ship->DesignID()));
 
-    } else if (TemporaryPtr<const Fleet> fleet = boost::dynamic_pointer_cast<const Fleet>(obj)) {
+    } else if (std::shared_ptr<const Fleet> fleet = std::dynamic_pointer_cast<const Fleet>(obj)) {
         if (fleet->Unowned())
             sitrep = SitRepEntry(
                 UserStringNop("SITREP_UNOWNED_FLEET_DESTROYED_AT_SYSTEM"),
@@ -299,9 +294,9 @@ SitRepEntry CreateCombatDestroyedObjectSitRep(int object_id, int combat_system_i
                 CurrentTurn() + 1,
                 "icons/sitrep/combat_destroyed.png",
                 UserStringNop("SITREP_FLEET_DESTROYED_AT_SYSTEM_LABEL"), true);
-        sitrep.AddVariable(VarText::FLEET_ID_TAG,      boost::lexical_cast<std::string>(object_id));
+        sitrep.AddVariable(VarText::FLEET_ID_TAG,      std::to_string(object_id));
 
-    } else if (TemporaryPtr<const Planet> planet = boost::dynamic_pointer_cast<const Planet>(obj)) {
+    } else if (std::shared_ptr<const Planet> planet = std::dynamic_pointer_cast<const Planet>(obj)) {
         if (planet->Unowned())
             sitrep = SitRepEntry(
                 UserStringNop("SITREP_UNOWNED_PLANET_DESTROYED_AT_SYSTEM"),
@@ -314,9 +309,9 @@ SitRepEntry CreateCombatDestroyedObjectSitRep(int object_id, int combat_system_i
                 CurrentTurn() + 1,
                 "icons/sitrep/combat_destroyed.png",
                 UserStringNop("SITREP_PLANET_DESTROYED_AT_SYSTEM_LABEL"), true);
-        sitrep.AddVariable(VarText::PLANET_ID_TAG,     boost::lexical_cast<std::string>(object_id));
+        sitrep.AddVariable(VarText::PLANET_ID_TAG,     std::to_string(object_id));
 
-    } else if (TemporaryPtr<const Building> building = boost::dynamic_pointer_cast<const Building>(obj)) {
+    } else if (std::shared_ptr<const Building> building = std::dynamic_pointer_cast<const Building>(obj)) {
         if (building->Unowned())
             sitrep = SitRepEntry(
                 UserStringNop("SITREP_UNOWNED_BUILDING_DESTROYED_ON_PLANET_AT_SYSTEM"),
@@ -329,14 +324,14 @@ SitRepEntry CreateCombatDestroyedObjectSitRep(int object_id, int combat_system_i
                 CurrentTurn() + 1,
                 "icons/sitrep/combat_destroyed.png",
                 UserStringNop("SITREP_BUILDING_DESTROYED_ON_PLANET_AT_SYSTEM_LABEL"), true);
-        sitrep.AddVariable(VarText::BUILDING_ID_TAG,   boost::lexical_cast<std::string>(object_id));
-        sitrep.AddVariable(VarText::PLANET_ID_TAG,     boost::lexical_cast<std::string>(building->PlanetID()));
+        sitrep.AddVariable(VarText::BUILDING_ID_TAG,   std::to_string(object_id));
+        sitrep.AddVariable(VarText::PLANET_ID_TAG,     std::to_string(building->PlanetID()));
     } else {
         sitrep = GenericCombatDestroyedObjectSitrep(combat_system_id);
     }
 
-    sitrep.AddVariable(VarText::EMPIRE_ID_TAG,     boost::lexical_cast<std::string>(obj->Owner()));
-    sitrep.AddVariable(VarText::SYSTEM_ID_TAG,     boost::lexical_cast<std::string>(combat_system_id));
+    sitrep.AddVariable(VarText::EMPIRE_ID_TAG,     std::to_string(obj->Owner()));
+    sitrep.AddVariable(VarText::SYSTEM_ID_TAG,     std::to_string(combat_system_id));
 
     return sitrep;
 }
@@ -347,7 +342,7 @@ SitRepEntry CreatePlanetDepopulatedSitRep(int planet_id) {
         CurrentTurn() + 1,
         "icons/sitrep/colony_destroyed.png",
         UserStringNop("SITREP_PLANET_DEPOPULATED_LABEL"), true);
-    sitrep.AddVariable(VarText::PLANET_ID_TAG,     boost::lexical_cast<std::string>(planet_id));
+    sitrep.AddVariable(VarText::PLANET_ID_TAG,     std::to_string(planet_id));
     return sitrep;
 }
 
@@ -357,7 +352,7 @@ SitRepEntry CreatePlanetColonizedSitRep(int planet_id, const std::string& specie
         CurrentTurn() + 1,
         "icons/sitrep/planet_colonized.png",
         UserStringNop("SITREP_PLANET_COLONIZED_LABEL"), true);
-    sitrep.AddVariable(VarText::PLANET_ID_TAG,  boost::lexical_cast<std::string>(planet_id));
+    sitrep.AddVariable(VarText::PLANET_ID_TAG,  std::to_string(planet_id));
     sitrep.AddVariable(VarText::SPECIES_TAG,    species);
     return sitrep;
 }
@@ -368,20 +363,17 @@ SitRepEntry CreatePlanetOutpostedSitRep(int planet_id) {
         CurrentTurn() + 1,
         "icons/sitrep/planet_colonized.png",
         UserStringNop("SITREP_PLANET_OUTPOSTED_LABEL"), true);
-    sitrep.AddVariable(VarText::PLANET_ID_TAG,     boost::lexical_cast<std::string>(planet_id));
+    sitrep.AddVariable(VarText::PLANET_ID_TAG,     std::to_string(planet_id));
     return sitrep;
 }
 
 SitRepEntry CreateFleetArrivedAtDestinationSitRep(int system_id, int fleet_id, int recipient_empire_id) {
-    TemporaryPtr<const Fleet> fleet = GetFleet(fleet_id);
+    std::shared_ptr<const Fleet> fleet = GetFleet(fleet_id);
 
     //bool system_contains_recipient_empire_planets = false;
     //if (const System* system = GetSystem(system_id)) {
-    //    std::vector<int> system_planets = system->FindObjectIDs<Planet>();
-    //    for (std::vector<int>::const_iterator planet_it = system_planets.begin();
-    //         planet_it != system_planets.end(); ++planet_it)
-    //    {
-    //        const Planet* planet = GetPlanet(*planet_it);
+    //    for (int planet_id : system->FindObjectIDs<Planet>()) {
+    //        const Planet* planet = GetPlanet(planet_id);
     //        if (!planet || planet->Unowned())
     //            continue;
     //        if (planet->OwnedBy(recipient_empire_id)) {
@@ -401,8 +393,8 @@ SitRepEntry CreateFleetArrivedAtDestinationSitRep(int system_id, int fleet_id, i
             CurrentTurn() + 1,
             "icons/sitrep/fleet_arrived.png",
             UserStringNop("SITREP_FLEET_ARRIVED_AT_SYSTEM_LABEL"), true);
-        sitrep.AddVariable(VarText::SYSTEM_ID_TAG,  boost::lexical_cast<std::string>(system_id));
-        sitrep.AddVariable(VarText::FLEET_ID_TAG,   boost::lexical_cast<std::string>(fleet_id));
+        sitrep.AddVariable(VarText::SYSTEM_ID_TAG,  std::to_string(system_id));
+        sitrep.AddVariable(VarText::FLEET_ID_TAG,   std::to_string(fleet_id));
         return sitrep;
     } else if (fleet->Unowned() && fleet->HasMonsters()) {
         if (fleet->NumShips() == 1) {
@@ -411,12 +403,12 @@ SitRepEntry CreateFleetArrivedAtDestinationSitRep(int system_id, int fleet_id, i
                 CurrentTurn() + 1,
                 "icons/sitrep/fleet_arrived.png",
                 UserStringNop("SITREP_MONSTER_SHIP_ARRIVED_AT_DESTINATION_LABEL"), true);
-            sitrep.AddVariable(VarText::SYSTEM_ID_TAG,     boost::lexical_cast<std::string>(system_id));
-            sitrep.AddVariable(VarText::FLEET_ID_TAG,      boost::lexical_cast<std::string>(fleet_id));
+            sitrep.AddVariable(VarText::SYSTEM_ID_TAG,     std::to_string(system_id));
+            sitrep.AddVariable(VarText::FLEET_ID_TAG,      std::to_string(fleet_id));
             int ship_id = *fleet->ShipIDs().begin();
-            sitrep.AddVariable(VarText::SHIP_ID_TAG,       boost::lexical_cast<std::string>(ship_id));
-            if (TemporaryPtr<const Ship> ship = GetShip(ship_id))
-                sitrep.AddVariable(VarText::DESIGN_ID_TAG, boost::lexical_cast<std::string>(ship->DesignID()));
+            sitrep.AddVariable(VarText::SHIP_ID_TAG,       std::to_string(ship_id));
+            if (std::shared_ptr<const Ship> ship = GetShip(ship_id))
+                sitrep.AddVariable(VarText::DESIGN_ID_TAG, std::to_string(ship->DesignID()));
             return sitrep;
         } else {
             SitRepEntry sitrep(
@@ -424,9 +416,9 @@ SitRepEntry CreateFleetArrivedAtDestinationSitRep(int system_id, int fleet_id, i
                 CurrentTurn() + 1,
                 "icons/sitrep/fleet_arrived.png",
                 UserStringNop("SITREP_MONSTER_FLEET_ARRIVED_AT_DESTINATION_LABEL"), true);
-            sitrep.AddVariable(VarText::SYSTEM_ID_TAG,  boost::lexical_cast<std::string>(system_id));
-            sitrep.AddVariable(VarText::FLEET_ID_TAG,   boost::lexical_cast<std::string>(fleet_id));
-            sitrep.AddVariable(VarText::RAW_TEXT_TAG,   boost::lexical_cast<std::string>(fleet->NumShips()));
+            sitrep.AddVariable(VarText::SYSTEM_ID_TAG,  std::to_string(system_id));
+            sitrep.AddVariable(VarText::FLEET_ID_TAG,   std::to_string(fleet_id));
+            sitrep.AddVariable(VarText::RAW_TEXT_TAG,   std::to_string(fleet->NumShips()));
             return sitrep;
         }
     } else if (fleet->Unowned()) {
@@ -435,9 +427,9 @@ SitRepEntry CreateFleetArrivedAtDestinationSitRep(int system_id, int fleet_id, i
             CurrentTurn() + 1,
             "icons/sitrep/fleet_arrived.png",
             UserStringNop("SITREP_FLEET_ARRIVED_AT_DESTINATION_LABEL"), true);
-        sitrep.AddVariable(VarText::SYSTEM_ID_TAG,  boost::lexical_cast<std::string>(system_id));
-        sitrep.AddVariable(VarText::FLEET_ID_TAG,   boost::lexical_cast<std::string>(fleet_id));
-        sitrep.AddVariable(VarText::RAW_TEXT_TAG,   boost::lexical_cast<std::string>(fleet->NumShips()));
+        sitrep.AddVariable(VarText::SYSTEM_ID_TAG,  std::to_string(system_id));
+        sitrep.AddVariable(VarText::FLEET_ID_TAG,   std::to_string(fleet_id));
+        sitrep.AddVariable(VarText::RAW_TEXT_TAG,   std::to_string(fleet->NumShips()));
         return sitrep;
     } else if (fleet->OwnedBy(recipient_empire_id)) {
         if (fleet->NumShips() == 1) {
@@ -446,13 +438,13 @@ SitRepEntry CreateFleetArrivedAtDestinationSitRep(int system_id, int fleet_id, i
                 CurrentTurn() + 1,
                 "icons/sitrep/fleet_arrived.png",
                 UserStringNop("SITREP_OWN_SHIP_ARRIVED_AT_DESTINATION_LABEL"), true);
-            sitrep.AddVariable(VarText::SYSTEM_ID_TAG,     boost::lexical_cast<std::string>(system_id));
-            sitrep.AddVariable(VarText::FLEET_ID_TAG,      boost::lexical_cast<std::string>(fleet_id));
-            sitrep.AddVariable(VarText::EMPIRE_ID_TAG,     boost::lexical_cast<std::string>(fleet->Owner()));
+            sitrep.AddVariable(VarText::SYSTEM_ID_TAG,     std::to_string(system_id));
+            sitrep.AddVariable(VarText::FLEET_ID_TAG,      std::to_string(fleet_id));
+            sitrep.AddVariable(VarText::EMPIRE_ID_TAG,     std::to_string(fleet->Owner()));
             int ship_id = *fleet->ShipIDs().begin();
-            sitrep.AddVariable(VarText::SHIP_ID_TAG,       boost::lexical_cast<std::string>(ship_id));
-            if (TemporaryPtr<const Ship> ship = GetShip(ship_id))
-                sitrep.AddVariable(VarText::DESIGN_ID_TAG, boost::lexical_cast<std::string>(ship->DesignID()));
+            sitrep.AddVariable(VarText::SHIP_ID_TAG,       std::to_string(ship_id));
+            if (std::shared_ptr<const Ship> ship = GetShip(ship_id))
+                sitrep.AddVariable(VarText::DESIGN_ID_TAG, std::to_string(ship->DesignID()));
             return sitrep;
         } else {
             SitRepEntry sitrep(
@@ -460,10 +452,10 @@ SitRepEntry CreateFleetArrivedAtDestinationSitRep(int system_id, int fleet_id, i
                 CurrentTurn() + 1,
                 "icons/sitrep/fleet_arrived.png",
                 UserStringNop("SITREP_OWN_FLEET_ARRIVED_AT_DESTINATION_LABEL"), true);
-            sitrep.AddVariable(VarText::SYSTEM_ID_TAG,  boost::lexical_cast<std::string>(system_id));
-            sitrep.AddVariable(VarText::FLEET_ID_TAG,   boost::lexical_cast<std::string>(fleet_id));
-            sitrep.AddVariable(VarText::EMPIRE_ID_TAG,  boost::lexical_cast<std::string>(fleet->Owner()));
-            sitrep.AddVariable(VarText::RAW_TEXT_TAG,   boost::lexical_cast<std::string>(fleet->NumShips()));
+            sitrep.AddVariable(VarText::SYSTEM_ID_TAG,  std::to_string(system_id));
+            sitrep.AddVariable(VarText::FLEET_ID_TAG,   std::to_string(fleet_id));
+            sitrep.AddVariable(VarText::EMPIRE_ID_TAG,  std::to_string(fleet->Owner()));
+            sitrep.AddVariable(VarText::RAW_TEXT_TAG,   std::to_string(fleet->NumShips()));
             return sitrep;
         }
     } else {
@@ -473,13 +465,13 @@ SitRepEntry CreateFleetArrivedAtDestinationSitRep(int system_id, int fleet_id, i
                 CurrentTurn() + 1,
                 "icons/sitrep/fleet_arrived.png",
                 UserStringNop("SITREP_FOREIGN_SHIP_ARRIVED_AT_DESTINATION_LABEL"), true);
-            sitrep.AddVariable(VarText::SYSTEM_ID_TAG,     boost::lexical_cast<std::string>(system_id));
-            sitrep.AddVariable(VarText::FLEET_ID_TAG,      boost::lexical_cast<std::string>(fleet_id));
-            sitrep.AddVariable(VarText::EMPIRE_ID_TAG,     boost::lexical_cast<std::string>(fleet->Owner()));
+            sitrep.AddVariable(VarText::SYSTEM_ID_TAG,     std::to_string(system_id));
+            sitrep.AddVariable(VarText::FLEET_ID_TAG,      std::to_string(fleet_id));
+            sitrep.AddVariable(VarText::EMPIRE_ID_TAG,     std::to_string(fleet->Owner()));
             int ship_id = *fleet->ShipIDs().begin();
-            sitrep.AddVariable(VarText::SHIP_ID_TAG,       boost::lexical_cast<std::string>(ship_id));
-            if (TemporaryPtr<const Ship> ship = GetShip(ship_id))
-                sitrep.AddVariable(VarText::DESIGN_ID_TAG, boost::lexical_cast<std::string>(ship->DesignID()));
+            sitrep.AddVariable(VarText::SHIP_ID_TAG,       std::to_string(ship_id));
+            if (std::shared_ptr<const Ship> ship = GetShip(ship_id))
+                sitrep.AddVariable(VarText::DESIGN_ID_TAG, std::to_string(ship->DesignID()));
             return sitrep;
         } else {
             SitRepEntry sitrep(
@@ -487,10 +479,10 @@ SitRepEntry CreateFleetArrivedAtDestinationSitRep(int system_id, int fleet_id, i
                 CurrentTurn() + 1,
                 "icons/sitrep/fleet_arrived.png",
                 UserStringNop("SITREP_FOREIGN_FLEET_ARRIVED_AT_DESTINATION_LABEL"), true);
-            sitrep.AddVariable(VarText::SYSTEM_ID_TAG,  boost::lexical_cast<std::string>(system_id));
-            sitrep.AddVariable(VarText::FLEET_ID_TAG,   boost::lexical_cast<std::string>(fleet_id));
-            sitrep.AddVariable(VarText::EMPIRE_ID_TAG,  boost::lexical_cast<std::string>(fleet->Owner()));
-            sitrep.AddVariable(VarText::RAW_TEXT_TAG,   boost::lexical_cast<std::string>(fleet->NumShips()));
+            sitrep.AddVariable(VarText::SYSTEM_ID_TAG,  std::to_string(system_id));
+            sitrep.AddVariable(VarText::FLEET_ID_TAG,   std::to_string(fleet_id));
+            sitrep.AddVariable(VarText::EMPIRE_ID_TAG,  std::to_string(fleet->Owner()));
+            sitrep.AddVariable(VarText::RAW_TEXT_TAG,   std::to_string(fleet->NumShips()));
             return sitrep;
         }
     }
@@ -502,23 +494,22 @@ SitRepEntry CreateEmpireEliminatedSitRep(int empire_id) {
         CurrentTurn() + 1,
         "icons/sitrep/empire_eliminated.png",
         UserStringNop("SITREP_EMPIRE_ELIMINATED_LABEL"), true);
-    sitrep.AddVariable(VarText::EMPIRE_ID_TAG, boost::lexical_cast<std::string>(empire_id));
+    sitrep.AddVariable(VarText::EMPIRE_ID_TAG, std::to_string(empire_id));
     return sitrep;
 }
 
 SitRepEntry CreateVictorySitRep(const std::string& reason_string, int empire_id) {
     SitRepEntry sitrep(reason_string, CurrentTurn() + 1, "icons/sitrep/victory.png", UserStringNop("SITREP_VICTORY_LABEL"), true);
-    sitrep.AddVariable(VarText::EMPIRE_ID_TAG, boost::lexical_cast<std::string>(empire_id));
+    sitrep.AddVariable(VarText::EMPIRE_ID_TAG, std::to_string(empire_id));
     return sitrep;
 }
 
 SitRepEntry CreateSitRep(const std::string& template_string, int turn, const std::string& icon,
-                         const std::vector<std::pair<std::string, std::string> >& parameters,
+                         const std::vector<std::pair<std::string, std::string>>& parameters,
                          const std::string label, bool stringtable_lookup)
 {
     SitRepEntry sitrep(template_string, turn, icon, label, stringtable_lookup);
-    for (std::vector<std::pair<std::string, std::string> >::const_iterator it = parameters.begin();
-         it != parameters.end(); ++it)
-    { sitrep.AddVariable(it->first, it->second); }
+    for (const std::pair<std::string, std::string>& parameter : parameters)
+    { sitrep.AddVariable(parameter.first, parameter.second); }
     return sitrep;
 }
