@@ -28,8 +28,8 @@
 //! EXIT<br>
 //! Exit the program<br>
 //! <br>
-//! #These are comments<br>
-//! #these are more comments<br>
+//! # These are comments<br>
+//! # these are more comments<br>
 //! TEST_ONE<br>
 //! test one<br>
 //! TEST TWO<br>
@@ -49,9 +49,10 @@ public:
     //!@{
     StringTable_();  //!< default construction, uses S_DEFAULT_FILENAME
 
+    //! construct a StringTable_ from the given filename
     //! @param filename A file containing the data for this StringTable_
     //! @param lookups_fallback_table A StringTable_ to be used as fallback expansions lookup
-    StringTable_(const std::string& filename, const StringTable_* lookups_fallback_table = 0);   //!< construct a StringTable_ from the given filename
+    StringTable_(const std::string& filename, const StringTable_* lookups_fallback_table = nullptr);
 
     ~StringTable_();                             //!< default destructor
     //!@}
@@ -85,8 +86,9 @@ public:
 private:
     //! \name Internal Functions
     //!@{
+    //! Loads the String table file from m_filename
     //! @param lookups_fallback_table A StringTable_ to be used as fallback expansions lookup
-    void Load(const StringTable_* lookups_fallback_table = 0);    //!< Loads the String table file from m_filename
+    void Load(const StringTable_* lookups_fallback_table = nullptr);
 
     const std::map<std::string, std::string>& GetStrings() const {return m_strings;}    //!< returns a const reference to the strings in this table
 
