@@ -2,7 +2,7 @@
 #define _I18N_h_
 
 #include <string>
-#include <list>
+#include <vector>
 
 #include <boost/format.hpp>
 #include <boost/lexical_cast.hpp>
@@ -12,8 +12,8 @@
 /** Returns a language-specific string for the key-string \a str */
 FO_COMMON_API const std::string& UserString(const std::string& str);
 
-/** Returns a language-specific list of strings for the key-string \a str_list */
-FO_COMMON_API void UserStringList(const std::string& str_list, std::list<std::string>& strings);
+/** Returns a language-specific vector of strings for given @a key. */
+FO_COMMON_API std::vector<std::string> UserStringList(const std::string& key);
 
 /** Returns true iff a user-string exists for the key string \a str */
 FO_COMMON_API bool UserStringExists(const std::string& str);
@@ -52,7 +52,7 @@ FO_COMMON_API int EffectiveSign(double val);
  TODO: Adopt an out of house i18n framework and handle of empty, singular, pairs, triplets etc. correctly. */
 
 template<typename HeaderContainer, typename ListContainer>
-FO_COMMON_API boost::format FlexibleFormatList(
+boost::format FlexibleFormatList(
     const HeaderContainer& header_words,
     const ListContainer& words,
     const std::string& plural_header_template,
@@ -77,8 +77,8 @@ FO_COMMON_API boost::format FlexibleFormatList(
 
     boost::format header_fmt = FlexibleFormat(header_template) % boost::lexical_cast<std::string>(words.size());
 
-    for (typename HeaderContainer::const_iterator it = header_words.begin(); it != header_words.end(); ++it) {
-        header_fmt % *it;
+    for (const typename HeaderContainer::value_type& word : header_words) {
+        header_fmt % word;
     }
 
     std::string template_str;
@@ -124,15 +124,15 @@ FO_COMMON_API boost::format FlexibleFormatList(
 
     boost::format fmt = FlexibleFormat(template_str) % header_fmt.str();
 
-    for (typename ListContainer::const_iterator it = words.begin(); it != words.end(); ++it) {
-        fmt % *it;
+    for (const typename ListContainer::value_type& word : words) {
+        fmt % word;
     }
 
     return fmt;
 }
 
 template<typename Container>
-FO_COMMON_API boost::format FlexibleFormatList(const Container& words) {
+boost::format FlexibleFormatList(const Container& words) {
     return FlexibleFormatList(std::vector<std::string>(), words,
                               UserString("FORMAT_LIST_DEFAULT_PLURAL_HEADER"),
                               UserString("FORMAT_LIST_DEFAULT_SINGLE_HEADER"),
@@ -141,24 +141,24 @@ FO_COMMON_API boost::format FlexibleFormatList(const Container& words) {
 }
 
 template<typename Container>
-FO_COMMON_API boost::format FlexibleFormatList(
+boost::format FlexibleFormatList(
     const Container& words, const std::string& all_header) {
     return FlexibleFormatList(std::vector<std::string>(), words, all_header, all_header, all_header, all_header);
 }
 template<typename Container>
-FO_COMMON_API boost::format FlexibleFormatList(
+boost::format FlexibleFormatList(
     const Container& words, const std::string& plural_header, const std::string& single_header) {
     return FlexibleFormatList(std::vector<std::string>(), words, plural_header, single_header, plural_header, plural_header);
 }
 template<typename Container>
-FO_COMMON_API boost::format FlexibleFormatList(
+boost::format FlexibleFormatList(
     const Container& words, const std::string& plural_header
     , const std::string& single_header, const std::string& empty_header) {
     return FlexibleFormatList(std::vector<std::string>(), words, plural_header, single_header, empty_header, plural_header);
 }
 
 template<typename T1, typename T2>
-FO_COMMON_API boost::format FlexibleFormatList(const T2& header_words, const T1& words) {
+boost::format FlexibleFormatList(const T2& header_words, const T1& words) {
     return FlexibleFormatList(header_words, words
                               , UserString("FORMAT_LIST_DEFAULT_PLURAL_HEADER")
                               , UserString("FORMAT_LIST_DEFAULT_SINGLE_HEADER")
@@ -167,18 +167,18 @@ FO_COMMON_API boost::format FlexibleFormatList(const T2& header_words, const T1&
 }
 
 template<typename T1, typename T2>
-FO_COMMON_API boost::format FlexibleFormatList(
+boost::format FlexibleFormatList(
     const T2& header_words, const T1& words, const std::string& all_header) {
     return FlexibleFormatList(header_words, words, all_header, all_header, all_header, all_header);
 }
 template<typename T1, typename T2>
-FO_COMMON_API boost::format FlexibleFormatList(
+boost::format FlexibleFormatList(
     const T2& header_words, const T1& words, const std::string& plural_header, const std::string& single_header) {
     return FlexibleFormatList(header_words, words, plural_header, single_header, plural_header, plural_header);
 }
 
 template<typename T1, typename T2>
-FO_COMMON_API boost::format FlexibleFormatList(
+boost::format FlexibleFormatList(
     const T2& header_words, const T1& words, const std::string& plural_header
     , const std::string& single_header, const std::string& empty_header) {
     return FlexibleFormatList(header_words, words, plural_header, single_header, empty_header, plural_header);
